/*!
\file
\brief The hffix_fields.hpp file is generated by the fixspec/spec-parse-fields Haskell program from the FIX Protocol specification documents in the fixspec/ directory.

Do not edit this file, instead modify the fixspec/spec-parse-fields program and run it from the fixspec/spec-parse-fields directory as shown here. You must have The Haskell Tool Stack https://haskellstack.org installed.

    cd fixspec/spec-parse-fields && stack run > hffix_fields.hpp && mv hffix_fields.hpp ../../include/hffix_fields.hpp

Line comments for each field indicate the data type of the field for each version of the FIX spec in which the field appears.
*/

#ifndef HFFIX_FIELDS_HEADER
#define HFFIX_FIELDS_HEADER
namespace hffix {

/*!
\brief Namespace for all field tag name enums.
*/
namespace tag {
enum {
Account                                            = 1, /*!< 1 (String FIX.2.7) Account mnemonic as agreed between buy and sell sides, e.g. broker and institution or investor/intermediary and fund manager.*/
AdvId                                              = 2, /*!< 2 (String FIX.2.7) Unique identifier of advertisement message.

(Prior to FIX 4.1 this field was of type int)*/
AdvRefID                                           = 3, /*!< 3 (String FIX.2.7) Reference identifier used with CANCEL and REPLACE transaction types.

(Prior to FIX 4.1 this field was of type int)*/
AdvSide                                            = 4, /*!< 4 (char FIX.2.7) Broker's side of advertised trade*/
AdvTransType                                       = 5, /*!< 5 (String FIX.2.7) Identifies advertisement message transaction type*/
AvgPx                                              = 6, /*!< 6 (Price FIX.2.7) Calculated average price of all fills on this order.

For Fixed Income trades AvgPx is always expressed as percent-of-par, regardless of the PriceType (423) of LastPx (31). I.e., AvgPx will contain an average of percent-of-par values (see LastParPx (669)) for issues traded in Yield, Spread or Discount.*/
BeginSeqNo                                         = 7, /*!< 7 (SeqNum FIX.2.7) Message sequence number of first message in range to be resent*/
BeginString                                        = 8, /*!< 8 (String FIX.2.7) Identifies beginning of new message and protocol version. ALWAYS FIRST FIELD IN MESSAGE. (Always unencrypted)

Valid values:

FIXT.1.1

*/
BodyLength                                         = 9, /*!< 9 (Length FIX.2.7) Message length, in bytes, forward to the CheckSum field. ALWAYS SECOND FIELD IN MESSAGE. (Always unencrypted)*/
CheckSum                                           = 10, /*!< 10 (String FIX.2.7) Three byte, simple checksum (see Volume 2: "Checksum Calculation" for description). ALWAYS LAST FIELD IN MESSAGE; i.e. serves, with the trailing &lt;SOH&gt;, as the end-of-message delimiter. Always defined as three characters. (Always unencrypted)*/
ClOrdID                                            = 11, /*!< 11 (String FIX.2.7) Unique identifier for Order as assigned by the buy-side (institution, broker, intermediary etc.) (identified by SenderCompID (49) or OnBehalfOfCompID (5) as appropriate). Uniqueness must be guaranteed within a single trading day. Firms, particularly those which electronically submit multi-day orders, trade globally or throughout market close periods, should ensure uniqueness across days, for example by embedding a date within the ClOrdID field.*/
Commission                                         = 12, /*!< 12 (Amt FIX.2.7) Commission. Note if CommType (13) is percentage, Commission of 5% should be represented as .05.*/
CommType                                           = 13, /*!< 13 (char FIX.2.7) Specifies the basis or unit used to calculate the total commission based on the rate.*/
CumQty                                             = 14, /*!< 14 (Qty FIX.2.7) Total quantity (e.g. number of shares) filled.

(Prior to FIX 4.2 this field was of type int)*/
Currency                                           = 15, /*!< 15 (Currency FIX.2.7) Identifies currency used for price. Absence of this field is interpreted as the default for the security. It is recommended that systems provide the currency value whenever possible. See "Appendix 6-A: Valid Currency Codes" for information on obtaining valid values.*/
EndSeqNo                                           = 16, /*!< 16 (SeqNum FIX.2.7) Message sequence number of last message in range to be resent. If request is for a single message BeginSeqNo (7) = EndSeqNo. If request is for all messages subsequent to a particular message, EndSeqNo = "0" (representing infinity).*/
ExecID                                             = 17, /*!< 17 (String FIX.2.7) Unique identifier of execution message as assigned by sell-side (broker, exchange, ECN) (will be 0 (zero) for ExecType (150)=I (Order Status)).

Uniqueness must be guaranteed within a single trading day or the life of a multi-day order. Firms which accept multi-day orders should consider embedding a date within the ExecID field to assure uniqueness across days.

(Prior to FIX 4.1 this field was of type int).*/
ExecInst                                           = 18, /*!< 18 (MultipleCharValue FIX.2.7) Instructions for order handling on exchange trading floor. If more than one instruction is applicable to an order, this field can contain multiple instructions separated by space. *** SOME VALUES HAVE BEEN REPLACED - See "Replaced Features and Supported Approach" *** (see Volume : "Glossary" for value definitions)*/
ExecRefID                                          = 19, /*!< 19 (String FIX.2.7) Reference identifier used with Trade, Trade Cancel and Trade Correct execution types.

(Prior to FIX 4.1 this field was of type int)*/
ExecTransType                                      = 20, /*!< 20 (char FIX.4.2) Identifies transaction type



Valid values:

   0 = New

   1 = Cancel

   2 = Correct

   3 = Status

*/
HandlInst                                          = 21, /*!< 21 (char FIX.2.7) Instructions for order handling on Broker trading floor*/
SecurityIDSource                                   = 22, /*!< 22 (String FIX.2.7) Identifies class or source of the SecurityID(48) value. */
IOIID                                              = 23, /*!< 23 (String FIX.2.7) Unique identifier of IOI message.

(Prior to FIX 4.1 this field was of type int)*/
IOIQltyInd                                         = 25, /*!< 25 (char FIX.2.7) Relative quality of indication*/
IOIRefID                                           = 26, /*!< 26 (String FIX.2.7) Reference identifier used with CANCEL and REPLACE, transaction types.

(Prior to FIX 4.1 this field was of type int)*/
IOIQty                                             = 27, /*!< 27 (String FIX.2.7) Quantity (e.g. number of shares) in numeric form or relative size.*/
IOITransType                                       = 28, /*!< 28 (char FIX.2.7) Identifies IOI message transaction type*/
LastCapacity                                       = 29, /*!< 29 (char FIX.2.7) Broker capacity in order execution*/
LastMkt                                            = 30, /*!< 30 (Exchange FIX.2.7) Market of execution for last fill, or an indication of the market where an order was routed

Valid values:

See "Appendix 6-C"*/
LastPx                                             = 31, /*!< 31 (Price FIX.2.7) Price of this (last) fill.*/
LastQty                                            = 32, /*!< 32 (Qty FIX.2.7) Quantity (e.g. shares) bought/sold on this (last) fill.

(Prior to FIX 4.2 this field was of type int)*/
NoLinesOfText                                      = 33, /*!< 33 (NumInGroup FIX.2.7) Identifies number of lines of text body*/
MsgSeqNum                                          = 34, /*!< 34 (SeqNum FIX.2.7) Integer message sequence number.*/
MsgType                                            = 35, /*!< 35 (String FIX.2.7) Defines message type ALWAYS THIRD FIELD IN MESSAGE. (Always unencrypted)

Note: A "U" as the first character in the MsgType field (i.e. U, U2, etc) indicates that the message format is privately defined between the sender and receiver.

*** Note the use of lower case letters ****/
NewSeqNo                                           = 36, /*!< 36 (SeqNum FIX.2.7) New sequence number*/
OrderID                                            = 37, /*!< 37 (String FIX.2.7) Unique identifier for Order as assigned by sell-side (broker, exchange, ECN). Uniqueness must be guaranteed within a single trading day. Firms which accept multi-day orders should consider embedding a date within the OrderID field to assure uniqueness across days.*/
OrderQty                                           = 38, /*!< 38 (Qty FIX.2.7) Quantity ordered. This represents the number of shares for equities or par, face or nominal value for FI instruments.

(Prior to FIX 4.2 this field was of type int)*/
OrdStatus                                          = 39, /*!< 39 (char FIX.2.7) Identifies current status of order. *** SOME VALUES HAVE BEEN REPLACED - See "Replaced Features and Supported Approach" *** (see Volume : "Glossary" for value definitions)*/
OrdType                                            = 40, /*!< 40 (char FIX.2.7) Order type. *** SOME VALUES ARE NO LONGER USED - See "Deprecated (Phased-out) Features and Supported Approach" *** (see Volume : "Glossary" for value definitions)*/
OrigClOrdID                                        = 41, /*!< 41 (String FIX.2.7) ClOrdID (11) of the previous order (NOT the initial order of the day) as assigned by the institution, used to identify the previous order in cancel and cancel/replace requests.*/
OrigTime                                           = 42, /*!< 42 (UTCTimestamp FIX.2.7) Time of message origination (always expressed in UTC (Universal Time Coordinated, also known as "GMT"))*/
PossDupFlag                                        = 43, /*!< 43 (Boolean FIX.2.7) Indicates possible retransmission of message with this sequence number*/
Price                                              = 44, /*!< 44 (Price FIX.2.7) Price per unit of quantity (e.g. per share)*/
RefSeqNum                                          = 45, /*!< 45 (SeqNum FIX.2.7) Reference message sequence number*/
SecurityID                                         = 48, /*!< 48 (String FIX.2.7) Security identifier value of SecurityIDSource (22) type (e.g. CUSIP, SEDOL, ISIN, etc). Requires SecurityIDSource.*/
SenderCompID                                       = 49, /*!< 49 (String FIX.2.7) Assigned value used to identify firm sending message.*/
SenderSubID                                        = 50, /*!< 50 (String FIX.2.7) Assigned value used to identify specific message originator (desk, trader, etc.)*/
SendingTime                                        = 52, /*!< 52 (UTCTimestamp FIX.2.7) Time of message transmission (always expressed in UTC (Universal Time Coordinated, also known as "GMT")*/
Quantity                                           = 53, /*!< 53 (Qty FIX.2.7) Overall/total quantity (e.g. number of shares)

(Prior to FIX 4.2 this field was of type int)*/
Side                                               = 54, /*!< 54 (char FIX.2.7) Side of order (see Volume : "Glossary" for value definitions)*/
Symbol                                             = 55, /*!< 55 (String FIX.2.7) Ticker symbol. Common, "human understood" representation of the security. SecurityID (48) value can be specified if no symbol exists (e.g. non-exchange traded Collective Investment Vehicles)

Use "[N/A]" for products which do not have a symbol.*/
TargetCompID                                       = 56, /*!< 56 (String FIX.2.7) Assigned value used to identify receiving firm.*/
TargetSubID                                        = 57, /*!< 57 (String FIX.2.7) Assigned value used to identify specific individual or unit intended to receive message. "ADMIN" reserved for administrative messages not intended for a specific user.*/
Text                                               = 58, /*!< 58 (String FIX.2.7) Free format text string

(Note: this field does not have a specified maximum length)*/
TimeInForce                                        = 59, /*!< 59 (char FIX.2.7) Specifies how long the order remains in effect. Absence of this field is interpreted as DAY. NOTE not applicable to CIV Orders. (see Volume : "Glossary" for value definitions)*/
TransactTime                                       = 60, /*!< 60 (UTCTimestamp FIX.2.7) Timestamp when the business transaction represented by the message occurred.*/
Urgency                                            = 61, /*!< 61 (char FIX.2.7) Urgency flag*/
ValidUntilTime                                     = 62, /*!< 62 (UTCTimestamp FIX.2.7) Indicates expiration time of indication message (always expressed in UTC (Universal Time Coordinated, also known as "GMT")*/
SettlType                                          = 63, /*!< 63 (String FIX.2.7) Indicates order settlement period. If present, SettlDate (64) overrides this field. If both SettlType (63) and SettDate (64) are omitted, the default for SettlType (63) is 0 (Regular)

Regular is defined as the default settlement period for the particular security on the exchange of execution.

In Fixed Income the contents of this field may influence the instrument definition if the SecurityID (48) is ambiguous. In the US an active Treasury offering may be re-opened, and for a time one CUSIP will apply to both the current and "when-issued" securities. Supplying a value of "7" clarifies the instrument description; any other value or the absence of this field should cause the respondent to default to the active issue.

Additionally the following patterns may be uses as well as enum values

Dx = FX tenor expression for "days", e.g. "D5", where "x" is any integer &gt; 0

Mx = FX tenor expression for "months", e.g. "M3", where "x" is any integer &gt; 0

Wx = FX tenor expression for "weeks", e.g. "W13", where "x" is any integer &gt; 0

Yx = FX tenor expression for "years", e.g. "Y1", where "x" is any integer &gt; 0

Noted that for FX the tenors expressed using Dx, Mx, Wx, and Yx values do not denote business days, but calendar days.*/
SettlDate                                          = 64, /*!< 64 (LocalMktDate FIX.2.7) Specific date of trade settlement (SettlementDate) in YYYYMMDD format.

If present, this field overrides SettlType (63). This field is required if the value of SettlType (63) is 6 (Future) or 8 (Sellers Option). This field must be omitted if the value of SettlType (63) is 7 (When and If Issued)

(expressed in local time at place of settlement)*/
SymbolSfx                                          = 65, /*!< 65 (String FIX.2.7) Additional information about the security (e.g. preferred, warrants, etc.). Note also see SecurityType (167).

As defined in the NYSE Stock and bond Symbol Directory and in the AMEX Fitch Directory.*/
ListID                                             = 66, /*!< 66 (String FIX.2.7) Unique identifier for list as assigned by institution, used to associate multiple individual orders. Uniqueness must be guaranteed within a single trading day. Firms which generate multi-day orders should consider embedding a date within the ListID field to assure uniqueness across days.*/
ListSeqNo                                          = 67, /*!< 67 (int FIX.2.7) Sequence of individual order within list (i.e. ListSeqNo of TotNoOrders (68), 2 of 25, 3 of 25, . . . )*/
TotNoOrders                                        = 68, /*!< 68 (int FIX.2.7) Total number of list order entries across all messages. Should be the sum of all NoOrders (73) in each message that has repeating list order entries related to the same ListID (66). Used to support fragmentation.

(Prior to FIX 4.2 this field was named "ListNoOrds")*/
ListExecInst                                       = 69, /*!< 69 (String FIX.2.7) Free format text message containing list handling and execution instructions.*/
AllocID                                            = 70, /*!< 70 (String FIX.2.7) Unique identifier for allocation message.

(Prior to FIX 4.1 this field was of type int)*/
AllocTransType                                     = 71, /*!< 71 (char FIX.2.7) Identifies allocation transaction type *** SOME VALUES HAVE BEEN REPLACED - See "Replaced Features and Supported Approach" ****/
RefAllocID                                         = 72, /*!< 72 (String FIX.2.7) Reference identifier to be used with AllocTransType (71) = Replace or Cancel.

(Prior to FIX 4.1 this field was of type int)*/
NoOrders                                           = 73, /*!< 73 (NumInGroup FIX.2.7) Indicates number of orders to be combined for average pricing and allocation.*/
AvgPxPrecision                                     = 74, /*!< 74 (int FIX.2.7) Indicates number of decimal places to be used for average pricing. Absence of this field indicates that default precision arranged by the broker/institution is to be used.*/
TradeDate                                          = 75, /*!< 75 (LocalMktDate FIX.2.7) Indicates date of trading day. Absence of this field indicates current day (expressed in local time at place of trade).*/
PositionEffect                                     = 77, /*!< 77 (char FIX.2.7) Indicates whether the resulting position after a trade should be an opening position or closing position. Used for omnibus accounting - where accounts are held on a gross basis instead of being netted together.*/
NoAllocs                                           = 78, /*!< 78 (NumInGroup FIX.2.7) Number of repeating AllocAccount (79)/AllocPrice (366) entries.*/
AllocAccount                                       = 79, /*!< 79 (String FIX.2.7) Sub-account mnemonic*/
AllocQty                                           = 80, /*!< 80 (Qty FIX.2.7) Quantity to be allocated to specific sub-account

(Prior to FIX 4.2 this field was of type int)*/
ProcessCode                                        = 81, /*!< 81 (char FIX.2.7) Processing code for sub-account. Absence of this field in AllocAccount (79) / AllocPrice (366) /AllocQty (80) / ProcessCode instance indicates regular trade.*/
NoRpts                                             = 82, /*!< 82 (int FIX.2.7) Total number of reports within series.*/
RptSeq                                             = 83, /*!< 83 (int FIX.2.7) Sequence number of message within report series. Used to carry reporting sequence number of the fill as represented on the Trade Report Side.*/
CxlQty                                             = 84, /*!< 84 (Qty FIX.2.7) Total quantity canceled for this order.

(Prior to FIX 4.2 this field was of type int)*/
NoDlvyInst                                         = 85, /*!< 85 (NumInGroup FIX.2.7) Number of delivery instruction fields in repeating group.

Note this field was removed in FIX 4.1 and reinstated in FIX 4.4.*/
AllocStatus                                        = 87, /*!< 87 (int FIX.2.7) Identifies status of allocation.*/
AllocRejCode                                       = 88, /*!< 88 (int FIX.2.7) Identifies reason for rejection.*/
Signature                                          = 89, /*!< 89 (data FIX.2.7) Electronic signature*/
SecureDataLen                                      = 90, /*!< 90 (Length FIX.2.7) Length of encrypted message*/
SecureData                                         = 91, /*!< 91 (data FIX.2.7) Actual encrypted data stream*/
SignatureLength                                    = 93, /*!< 93 (Length FIX.2.7) Number of bytes in signature field*/
EmailType                                          = 94, /*!< 94 (char FIX.2.7) Email message type.*/
RawDataLength                                      = 95, /*!< 95 (Length FIX.2.7) Number of bytes in raw data field.*/
RawData                                            = 96, /*!< 96 (data FIX.2.7) Unformatted raw data, can include bitmaps, word processor documents, etc.*/
PossResend                                         = 97, /*!< 97 (Boolean FIX.2.7) Indicates that message may contain information that has been sent under another sequence number.*/
EncryptMethod                                      = 98, /*!< 98 (int FIX.2.7) Method of encryption.*/
StopPx                                             = 99, /*!< 99 (Price FIX.2.7) Price per unit of quantity (e.g. per share)*/
ExDestination                                      = 100, /*!< 100 (Exchange FIX.2.7) Execution destination as defined by institution when order is entered.

Valid values:

See "Appendix 6-C"*/
CxlRejReason                                       = 102, /*!< 102 (int FIX.2.7) Code to identify reason for cancel rejection.*/
OrdRejReason                                       = 103, /*!< 103 (int FIX.2.7) Code to identify reason for order rejection. Note: Values 3, 4, and 5 will be used when rejecting an order due to pre-allocation information errors.*/
IOIQualifier                                       = 104, /*!< 104 (char FIX.3.0) Code to qualify IOI use. (see Volume : "Glossary" for value definitions)*/
Issuer                                             = 106, /*!< 106 (String FIX.3.0) Name of security issuer (e.g. International Business Machines, GNMA).

see also Volume 7: "PRODUCT: FIXED INCOME - Euro Issuer Values"*/
SecurityDesc                                       = 107, /*!< 107 (String FIX.3.0) Can be used by the venue or one of the trading parties to provide a non-normative textual description for the financial instrument.*/
HeartBtInt                                         = 108, /*!< 108 (int FIX.3.0) Heartbeat interval (seconds)*/
MinQty                                             = 110, /*!< 110 (Qty FIX.3.0) Minimum quantity of an order to be executed.

(Prior to FIX 4.2 this field was of type int)*/
MaxFloor                                           = 111, /*!< 111 (Qty FIX.3.0) The quantity to be displayed . Required for reserve orders. On orders specifies the qty to be displayed, on execution reports the currently displayed quantity.*/
TestReqID                                          = 112, /*!< 112 (String FIX.3.0) Identifier included in Test Request message to be returned in resulting Heartbeat*/
ReportToExch                                       = 113, /*!< 113 (Boolean FIX.3.0) Identifies party of trade responsible for exchange reporting.*/
LocateReqd                                         = 114, /*!< 114 (Boolean FIX.4.0) Indicates whether the broker is to locate the stock in conjunction with a short sell order.*/
OnBehalfOfCompID                                   = 115, /*!< 115 (String FIX.4.0) Assigned value used to identify firm originating message if the message was delivered by a third party i.e. the third party firm identifier would be delivered in the SenderCompID field and the firm originating the message in this field.*/
OnBehalfOfSubID                                    = 116, /*!< 116 (String FIX.4.0) Assigned value used to identify specific message originator (i.e. trader) if the message was delivered by a third party*/
QuoteID                                            = 117, /*!< 117 (String FIX.4.0) Unique identifier for quote*/
NetMoney                                           = 118, /*!< 118 (Amt FIX.4.0) Total amount due as the result of the transaction (e.g. for Buy order - principal + commission + fees) reported in currency of execution.*/
SettlCurrAmt                                       = 119, /*!< 119 (Amt FIX.4.0) Total amount due expressed in settlement currency (includes the effect of the forex transaction)*/
SettlCurrency                                      = 120, /*!< 120 (Currency FIX.4.0) Currency code of settlement denomination.*/
ForexReq                                           = 121, /*!< 121 (Boolean FIX.4.0) Indicates request for forex accommodation trade to be executed along with security transaction.*/
OrigSendingTime                                    = 122, /*!< 122 (UTCTimestamp FIX.4.0) Original time of message transmission (always expressed in UTC (Universal Time Coordinated, also known as "GMT") when transmitting orders as the result of a resend request.*/
GapFillFlag                                        = 123, /*!< 123 (Boolean FIX.4.0) Indicates that the Sequence Reset message is replacing administrative or application messages which will not be resent.*/
NoExecs                                            = 124, /*!< 124 (NumInGroup FIX.4.0) Number of executions or trades.*/
ExpireTime                                         = 126, /*!< 126 (UTCTimestamp FIX.4.0) Time/Date of order expiration (always expressed in UTC (Universal Time Coordinated, also known as "GMT")



The meaning of expiration is specific to the context where the field is used.



For orders, this is the expiration time of a Good Til Date TimeInForce.



For Quotes - this is the expiration of the quote.



Expiration time is provided across the quote message dialog to control the length of time of the overall quoting process.



For collateral requests, this is the time by which collateral must be assigned.



For collateral assignments, this is the time by which a response to the assignment is expected.



For credit/risk limit checks, this is the time when the reserved credit limit will expire for the requested transaction.*/
DKReason                                           = 127, /*!< 127 (char FIX.4.0) Reason for execution rejection.*/
DeliverToCompID                                    = 128, /*!< 128 (String FIX.4.0) Assigned value used to identify the firm targeted to receive the message if the message is delivered by a third party i.e. the third party firm identifier would be delivered in the TargetCompID (56) field and the ultimate receiver firm ID in this field.*/
DeliverToSubID                                     = 129, /*!< 129 (String FIX.4.0) Assigned value used to identify specific message recipient (i.e. trader) if the message is delivered by a third party*/
IOINaturalFlag                                     = 130, /*!< 130 (Boolean FIX.4.0) Indicates that IOI is the result of an existing agency order or a facilitation position resulting from an agency order, not from principal trading or order solicitation activity.*/
QuoteReqID                                         = 131, /*!< 131 (String FIX.4.0) Unique identifier for a QuoteRequest(35=R).*/
BidPx                                              = 132, /*!< 132 (Price FIX.4.0) Bid price/rate*/
OfferPx                                            = 133, /*!< 133 (Price FIX.4.0) Offer price/rate*/
BidSize                                            = 134, /*!< 134 (Qty FIX.4.0) Quantity of bid

(Prior to FIX 4.2 this field was of type int)*/
OfferSize                                          = 135, /*!< 135 (Qty FIX.4.0) Quantity of offer

(Prior to FIX 4.2 this field was of type int)*/
NoMiscFees                                         = 136, /*!< 136 (NumInGroup FIX.4.0) Number of repeating groups of miscellaneous fees*/
MiscFeeAmt                                         = 137, /*!< 137 (Amt FIX.4.0) Miscellaneous fee value*/
MiscFeeCurr                                        = 138, /*!< 138 (Currency FIX.4.0) Currency of miscellaneous fee*/
MiscFeeType                                        = 139, /*!< 139 (String FIX.4.0) Indicates type of miscellaneous fee.*/
PrevClosePx                                        = 140, /*!< 140 (Price FIX.4.0) Previous closing price of security.*/
ResetSeqNumFlag                                    = 141, /*!< 141 (Boolean FIX.4.1) Indicates that both sides of the FIX session should reset sequence numbers.*/
SenderLocationID                                   = 142, /*!< 142 (String FIX.4.1) Assigned value used to identify specific message originator's location (i.e. geographic location and/or desk, trader)*/
TargetLocationID                                   = 143, /*!< 143 (String FIX.4.1) Assigned value used to identify specific message destination's location (i.e. geographic location and/or desk, trader)*/
OnBehalfOfLocationID                               = 144, /*!< 144 (String FIX.4.1) Assigned value used to identify specific message originator's location (i.e. geographic location and/or desk, trader) if the message was delivered by a third party*/
DeliverToLocationID                                = 145, /*!< 145 (String FIX.4.1) Assigned value used to identify specific message recipient's location (i.e. geographic location and/or desk, trader) if the message was delivered by a third party*/
NoRelatedSym                                       = 146, /*!< 146 (NumInGroup FIX.4.1) Specifies the number of repeating symbols specified.*/
Subject                                            = 147, /*!< 147 (String FIX.4.1) The subject of an Email message*/
Headline                                           = 148, /*!< 148 (String FIX.4.1) The headline of a News message*/
URLLink                                            = 149, /*!< 149 (String FIX.4.1) A URI (Uniform Resource Identifier) or URL (Uniform Resource Locator) link to additional information (i.e. http://www.XYZ.com/research.html)

See "Appendix 6-B FIX Fields Based Upon Other Standards"*/
ExecType                                           = 150, /*!< 150 (char FIX.4.1) Describes the specific ExecutionRpt (e.g. Pending Cancel) while OrdStatus(39) will always identify the current order status (e.g. Partially Filled).*/
LeavesQty                                          = 151, /*!< 151 (Qty FIX.4.1) Quantity open for further execution. If the OrdStatus (39) is Canceled, DoneForTheDay, Expired, Calculated, or Rejected (in which case the order is no longer active) then LeavesQty could be 0, otherwise LeavesQty = OrderQty (38) - CumQty (14).

(Prior to FIX 4.2 this field was of type int)*/
CashOrderQty                                       = 152, /*!< 152 (Qty FIX.4.1) Specifies the approximate order quantity desired in total monetary units vs. as tradeable units (e.g. number of shares). The broker or fund manager (for CIV orders) would be responsible for converting and calculating a tradeable unit (e.g. share) quantity (OrderQty (38)) based upon this amount to be used for the actual order and subsequent messages.*/
AllocAvgPx                                         = 153, /*!< 153 (Price FIX.4.1) AvgPx (6) for a specific AllocAccount (79)

For Fixed Income this is always expressed as "percent of par" price type.*/
AllocNetMoney                                      = 154, /*!< 154 (Amt FIX.4.1) NetMoney (8) for a specific AllocAccount (79)*/
SettlCurrFxRate                                    = 155, /*!< 155 (float FIX.4.1) Foreign exchange rate used to compute SettlCurrAmt (9) from Currency (5) to SettlCurrency (20)*/
SettlCurrFxRateCalc                                = 156, /*!< 156 (char FIX.4.1) Specifies whether or not SettlCurrFxRate (155) should be multiplied or divided.*/
NumDaysInterest                                    = 157, /*!< 157 (int FIX.4.1) Number of Days of Interest for convertible bonds and fixed income. Note value may be negative.*/
AccruedInterestRate                                = 158, /*!< 158 (Percentage FIX.4.1) The amount the buyer compensates the seller for the portion of the next coupon interest payment the seller has earned but will not receive from the issuer because the issuer will send the next coupon payment to the buyer. Accrued Interest Rate is the annualized Accrued Interest amount divided by the purchase price of the bond.*/
AccruedInterestAmt                                 = 159, /*!< 159 (Amt FIX.4.1) Amount of Accrued Interest for convertible bonds and fixed income*/
SettlInstMode                                      = 160, /*!< 160 (char FIX.4.1) Indicates mode used for Settlement Instructions message. *** SOME VALUES HAVE BEEN REPLACED - See "Replaced Features and Supported Approach" ****/
AllocText                                          = 161, /*!< 161 (String FIX.4.1) Free format text related to a specific AllocAccount (79).*/
SettlInstID                                        = 162, /*!< 162 (String FIX.4.1) Unique identifier for Settlement Instruction.*/
SettlInstTransType                                 = 163, /*!< 163 (char FIX.4.1) Settlement Instructions message transaction type*/
EmailThreadID                                      = 164, /*!< 164 (String FIX.4.1) Unique identifier for an email thread (new and chain of replies)*/
SettlInstSource                                    = 165, /*!< 165 (char FIX.4.1) Indicates source of Settlement Instructions*/
SecurityType                                       = 167, /*!< 167 (String FIX.4.1) Indicates type of security. Security type enumerations are grouped by Product(460) field value. NOTE: Additional values may be used by mutual agreement of the counterparties.*/
EffectiveTime                                      = 168, /*!< 168 (UTCTimestamp FIX.4.1) Time the details within the message should take effect (always expressed in UTC (Universal Time Coordinated, also known as "GMT")*/
StandInstDbType                                    = 169, /*!< 169 (int FIX.4.1) Identifies the Standing Instruction database used*/
StandInstDbName                                    = 170, /*!< 170 (String FIX.4.1) Name of the Standing Instruction database represented with StandInstDbType (169) (i.e. the Global Custodian's name).*/
StandInstDbID                                      = 171, /*!< 171 (String FIX.4.1) Unique identifier used on the Standing Instructions database for the Standing Instructions to be referenced.*/
SettlDeliveryType                                  = 172, /*!< 172 (int FIX.4.1) Identifies type of settlement*/
BidSpotRate                                        = 188, /*!< 188 (Price FIX.4.1) Bid F/X spot rate.*/
BidForwardPoints                                   = 189, /*!< 189 (PriceOffset FIX.4.1) Bid F/X forward points added to spot rate. May be a negative value.*/
OfferSpotRate                                      = 190, /*!< 190 (Price FIX.4.1) Offer F/X spot rate.*/
OfferForwardPoints                                 = 191, /*!< 191 (PriceOffset FIX.4.1) Offer F/X forward points added to spot rate. May be a negative value.*/
OrderQty2                                          = 192, /*!< 192 (Qty FIX.4.1) OrderQty (38) of the future part of a F/X swap order.*/
SettlDate2                                         = 193, /*!< 193 (LocalMktDate FIX.4.1) SettDate (64) of the future part of a F/X swap order.*/
LastSpotRate                                       = 194, /*!< 194 (Price FIX.4.1) F/X spot rate.*/
LastForwardPoints                                  = 195, /*!< 195 (PriceOffset FIX.4.1) F/X forward points added to LastSpotRate (94). May be a negative value. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
AllocLinkID                                        = 196, /*!< 196 (String FIX.4.1) Can be used to link two different Allocation messages (each with unique AllocID (70)) together, i.e. for F/X "Netting" or "Swaps". Should be unique.*/
AllocLinkType                                      = 197, /*!< 197 (int FIX.4.1) Identifies the type of Allocation linkage when AllocLinkID (96) is used.*/
SecondaryOrderID                                   = 198, /*!< 198 (String FIX.4.1) Assigned by the party which accepts the order. Can be used to provide the OrderID (37) used by an exchange or executing system.*/
NoIOIQualifiers                                    = 199, /*!< 199 (NumInGroup FIX.4.1) Number of repeating groups of IOIQualifiers (04).*/
MaturityMonthYear                                  = 200, /*!< 200 (MonthYear FIX.4.1) Can be used with standardized derivatives vs. the MaturityDate (54) field.  Month and Year of the maturity (used for standardized futures and options).

Format:

YYYYMM (e.g. 199903)

YYYYMMDD (e.g. 20030323)

YYYYMMwN (e.g. 200303w) for week

A specific date or can be appended to the MaturityMonthYear. For instance, if multiple standard products exist that mature in the same Year and Month, but actually mature at a different time, a value can be appended, such as "w" or "w2" to indicate week  as opposed to week 2 expiration. Likewise, the date (0-3) can be appended to indicate a specific expiration (maturity date).*/
PutOrCall                                          = 201, /*!< 201 (int FIX.4.1) Indicates whether an option contract is a put, call, chooser or undetermined.*/
StrikePrice                                        = 202, /*!< 202 (Price FIX.4.1) Strike Price for an Option.*/
CoveredOrUncovered                                 = 203, /*!< 203 (int FIX.4.1) Used for derivative products, such as options*/
OptAttribute                                       = 206, /*!< 206 (char FIX.4.1) Provided to support versioning of option contracts as a result of corporate actions or events. Use of this field is defined by counterparty agreement or market conventions.*/
SecurityExchange                                   = 207, /*!< 207 (Exchange FIX.4.1) Market used to help identify a security.

Valid values:

See "Appendix 6-C"*/
NotifyBrokerOfCredit                               = 208, /*!< 208 (Boolean FIX.4.1) Indicates whether or not details should be communicated to BrokerOfCredit (i.e. step-in broker).*/
AllocHandlInst                                     = 209, /*!< 209 (int FIX.4.1) Indicates how the receiver (i.e. third party) of Allocation message should handle/process the account details.*/
MaxShow                                            = 210, /*!< 210 (Qty FIX.4.1) Maximum quantity (e.g. number of shares) within an order to be shown to other customers (i.e. sent via an IOI).

(Prior to FIX 4.2 this field was of type int)*/
PegOffsetValue                                     = 211, /*!< 211 (float FIX.4.1) Amount (signed) added to the peg for a pegged order in the context of the PegOffsetType (836)

(Prior to FIX 4.4 this field was of type PriceOffset)*/
XmlDataLen                                         = 212, /*!< 212 (Length FIX.4.2) Length of the XmlData data block.*/
XmlData                                            = 213, /*!< 213 (data FIX.4.2) Actual XML data stream (e.g. FIXML). See approriate XML reference (e.g. FIXML). Note: may contain embedded SOH characters.*/
SettlInstRefID                                     = 214, /*!< 214 (String FIX.4.2) Reference identifier for the SettlInstID (162) with Cancel and Replace SettlInstTransType (163) transaction types.*/
NoRoutingIDs                                       = 215, /*!< 215 (NumInGroup FIX.4.2) Number of repeating groups of RoutingID (217) and RoutingType (216) values.

See Volume 3: "Pre-Trade Message Targeting/Routing"*/
RoutingType                                        = 216, /*!< 216 (int FIX.4.2) Indicates the type of RoutingID (217) specified.*/
RoutingID                                          = 217, /*!< 217 (String FIX.4.2) Assigned value used to identify a specific routing destination.*/
Spread                                             = 218, /*!< 218 (PriceOffset FIX.4.2) For Fixed Income. Either Swap Spread or Spread to Benchmark depending upon the order type.

Spread to Benchmark: Basis points relative to a benchmark. To be expressed as "count of basis points" (vs. an absolute value). E.g. High Grade Corporate Bonds may express price as basis points relative to benchmark (the BenchmarkCurveName (22) field). Note: Basis points can be negative.

Swap Spread: Target spread for a swap.*/
BenchmarkCurveCurrency                             = 220, /*!< 220 (Currency FIX.4.2) Identifies currency used for benchmark curve. See "Appendix 6-A: Valid Currency Codes" for information on obtaining valid values.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
BenchmarkCurveName                                 = 221, /*!< 221 (String FIX.4.2) Name of benchmark curve.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
BenchmarkCurvePoint                                = 222, /*!< 222 (String FIX.4.2) Point on benchmark curve. Free form values: e.g. "Y", "7Y", "INTERPOLATED". 

Sample values: 

M = combination of a number between 1-12 and a "M" for month 

Y = combination of number between 1-100 and a "Y" for year} 

10Y-OLD = see above, then add "-OLD" when appropriate 

INTERPOLATED = the point is mathematically derived 

2/2031 5 3/8 = the point is stated via a combination of maturity month / year and coupon 

See Fixed Income-specific documentation at http://www.fixtradingcommunity.org for additional values. 

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)	*/
CouponRate                                         = 223, /*!< 223 (Percentage FIX.4.2) The rate of interest that, when multiplied by the principal, par value, or face value of a bond, provides the currency amount of the periodic interest payment. The coupon is always cited, along with maturity, in any quotation of a bond's price.*/
CouponPaymentDate                                  = 224, /*!< 224 (LocalMktDate FIX.4.2) Date interest is to be paid. Used in identifying Corporate Bond issues.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
IssueDate                                          = 225, /*!< 225 (LocalMktDate FIX.4.2) The date on which a bond or stock offering is issued. It may or may not be the same as the effective date ("Dated Date") or the date on which interest begins to accrue ("Interest Accrual Date")

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
RepurchaseTerm                                     = 226, /*!< 226 (int FIX.4.2) Number of business days before repurchase of a repo. (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
RepurchaseRate                                     = 227, /*!< 227 (Percentage FIX.4.2) Percent of par at which a Repo will be repaid. Represented as a percent, e.g. .9525 represents 95-/4 percent of par. (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
Factor                                             = 228, /*!< 228 (float FIX.4.2) For Fixed Income: Amorization Factor for deriving Current face from Original face for ABS or MBS securities, note the fraction may be greater than, equal to or less than . In TIPS securities this is the Inflation index.

Qty * Factor * Price = Gross Trade Amount

For Derivatives: Contract Value Factor by which price must be adjusted to determine the true nominal value of one futures/options contract.

(Qty * Price) * Factor = Nominal Value

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
TradeOriginationDate                               = 229, /*!< 229 (LocalMktDate FIX.4.2) Used with Fixed Income for Muncipal New Issue Market. Agreement in principal between counter-parties prior to actual trade date.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
ExDate                                             = 230, /*!< 230 (LocalMktDate FIX.4.2) The date when a distribution of interest is deducted from a securities assets or set aside for payment to bondholders. On the ex-date, the securities price drops by the amount of the distribution (plus or minus any market activity).

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
ContractMultiplier                                 = 231, /*!< 231 (float FIX.4.2) Specifies the ratio or multiply factor to convert from "nominal" units (e.g. contracts) to total units (e.g. shares) (e.g. 1.0, 100, 1000, etc). Applicable For Fixed Income, Convertible Bonds, Derivatives, etc.*/
NoStipulations                                     = 232, /*!< 232 (NumInGroup FIX.4.2) Number of stipulation entries

(Note tag # was reserved in FIX 4.1, added in FIX 4.3).*/
StipulationType                                    = 233, /*!< 233 (String FIX.4.2) For Fixed Income.

Type of Stipulation.

Other types may be used by mutual agreement of the counterparties.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
StipulationValue                                   = 234, /*!< 234 (String FIX.4.2) For Fixed Income. Value of stipulation.

The expression can be an absolute single value or a combination of values and logical operators:

&lt; value

&gt; value

&lt;= value

&gt;= value

value

value - value2

value OR value2

value AND value2

YES

NO

Bargain conditions recognized by the London Stock Exchange - to be used when StipulationType is "BGNCON".

CD = Special cum Dividend

XD = Special ex Dividend

CC = Special cum Coupon

XC = Special ex Coupon

CB = Special cum Bonus

XB = Special ex Bonus

CR = Special cum Rights

XR = Special ex Rights

CP = Special cum Capital Repayments

XP = Special ex Capital Repayments

CS = Cash Settlement

SP = Special Price

TR = Report for European Equity Market Securities in accordance with Chapter 8 of the Rules.

GD = Guaranteed Delivery

Values for StipulationType = "PXSOURCE":

BB GENERIC

BB FAIRVALUE

BROKERTEC

ESPEED

GOVPX

HILLIARD FARBER

ICAP

TRADEWEB

TULLETT LIBERTY

If a particular side of the market is wanted append /BID /OFFER or /MID.

plus appropriate combinations of the above and other expressions by mutual agreement of the counterparties.

Examples: "&gt;=60", ".25", "ORANGE OR CONTRACOSTA", etc.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
YieldType                                          = 235, /*!< 235 (String FIX.4.2) Type of yield. (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
Yield                                              = 236, /*!< 236 (Percentage FIX.4.2) Yield percentage.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
TotalTakedown                                      = 237, /*!< 237 (Amt FIX.4.2) The price at which the securities are distributed to the different members of an underwriting group for the primary market in Municipals, total gross underwriter's spread.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
Concession                                         = 238, /*!< 238 (Amt FIX.4.2) Provides the reduction in price for the secondary market in Muncipals.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
RepoCollateralSecurityType                         = 239, /*!< 239 (String FIX.4.3) Identifies the collateral used in the transaction.

Valid values: see SecurityType (167) field (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
RedemptionDate                                     = 240, /*!< 240 (LocalMktDate FIX.4.2) Return of investor's principal in a security. Bond redemption can occur before maturity date.(Note tag # was reserved in FIX 4.1, added in FIX 4.3) (prior to FIX 4.4 field was of type UTCDate)*/
UnderlyingCouponPaymentDate                        = 241, /*!< 241 (LocalMktDate FIX.4.2) Underlying security's CouponPaymentDate.

See CouponPaymentDate (224) field for description

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
UnderlyingIssueDate                                = 242, /*!< 242 (LocalMktDate FIX.4.2) Underlying security's IssueDate.

See IssueDate (225) field for description

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
UnderlyingRepoCollateralSecurityType               = 243, /*!< 243 (String FIX.4.3) Underlying security's RepoCollateralSecurityType. See RepoCollateralSecurityType (239) field for description.(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
UnderlyingRepurchaseTerm                           = 244, /*!< 244 (int FIX.4.2) Underlying security's RepurchaseTerm. See RepurchaseTerm (226) field for description (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
UnderlyingRepurchaseRate                           = 245, /*!< 245 (Percentage FIX.4.2) Underlying security's RepurchaseRate. See RepurchaseRate (227) field for description (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
UnderlyingFactor                                   = 246, /*!< 246 (float FIX.4.2) Underlying security's Factor.

See Factor (228) field for description

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
UnderlyingRedemptionDate                           = 247, /*!< 247 (LocalMktDate FIX.4.2) Underlying security's RedemptionDate. See RedemptionDate (240) field for description (Note tag # was reserved in FIX 4.1, added in FIX 4.3) (prior to FIX 4.4 field was of type UTCDate)*/
LegCouponPaymentDate                               = 248, /*!< 248 (LocalMktDate FIX.4.2) Multileg instrument's individual leg security's CouponPaymentDate.

See CouponPaymentDate (224) field for description

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
LegIssueDate                                       = 249, /*!< 249 (LocalMktDate FIX.4.2) Multileg instrument's individual leg security's IssueDate.

See IssueDate (225) field for description

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
LegRepoCollateralSecurityType                      = 250, /*!< 250 (String FIX.4.3) Multileg instrument's individual leg security's RepoCollateralSecurityType. See RepoCollateralSecurityType (239) field for description (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
LegRepurchaseTerm                                  = 251, /*!< 251 (int FIX.4.2) Multileg instrument's individual leg security's RepurchaseTerm. See RepurchaseTerm (226) field for description (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
LegRepurchaseRate                                  = 252, /*!< 252 (Percentage FIX.4.2) Multileg instrument's individual leg security's RepurchaseRate. See RepurchaseRate (227) field for description (Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
LegFactor                                          = 253, /*!< 253 (float FIX.4.2) Multileg instrument's individual leg security's Factor.

See Factor (228) field for description

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
LegRedemptionDate                                  = 254, /*!< 254 (LocalMktDate FIX.4.2) Multileg instrument's individual leg security's RedemptionDate. See RedemptionDate (240) field for description (Note tag # was reserved in FIX 4.1, added in FIX 4.3) (prior to FIX 4.4 field was of type UTCDate)*/
CreditRating                                       = 255, /*!< 255 (String FIX.4.2) An evaluation of a company's ability to repay obligations or its likelihood of not defaulting. These evaluation are provided by Credit Rating Agencies, i.e. S&P, Moody's.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
UnderlyingCreditRating                             = 256, /*!< 256 (String FIX.4.2) Underlying security's CreditRating.

See CreditRating (255) field for description

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
LegCreditRating                                    = 257, /*!< 257 (String FIX.4.2) Multileg instrument's individual leg security's CreditRating.

See CreditRating (255) field for description

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
TradedFlatSwitch                                   = 258, /*!< 258 (Boolean FIX.4.2) Driver and part of trade in the event that the Security Master file was wrong at the point of entry(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
BasisFeatureDate                                   = 259, /*!< 259 (LocalMktDate FIX.4.2) BasisFeatureDate allows requesting firms within fixed income the ability to request an alternative yield-to-worst, -maturity, -extended or other call. This flows through the confirm process.

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)

(prior to FIX 4.4 field was of type UTCDate)*/
BasisFeaturePrice                                  = 260, /*!< 260 (Price FIX.4.2) Price for BasisFeatureDate.

See BasisFeatureDate (259)

(Note tag # was reserved in FIX 4.1, added in FIX 4.3)*/
MDReqID                                            = 262, /*!< 262 (String FIX.4.2) Unique identifier for Market Data Request*/
SubscriptionRequestType                            = 263, /*!< 263 (char FIX.4.2) Subscription Request Type*/
MarketDepth                                        = 264, /*!< 264 (int FIX.4.2) Depth of market for Book Snapshot / Incremental updates

0 - full book depth

1 - top of book

2 and above - book depth (number of levels)*/
MDUpdateType                                       = 265, /*!< 265 (int FIX.4.2) Specifies the type of Market Data update.*/
AggregatedBook                                     = 266, /*!< 266 (Boolean FIX.4.2) Specifies whether or not book entries should be aggregated. (Not specified) = broker option*/
NoMDEntryTypes                                     = 267, /*!< 267 (NumInGroup FIX.4.2) Number of MDEntryType (269) fields requested.*/
NoMDEntries                                        = 268, /*!< 268 (NumInGroup FIX.4.2) Number of entries in Market Data message.*/
MDEntryType                                        = 269, /*!< 269 (char FIX.4.2) Type of market data entry.*/
MDEntryPx                                          = 270, /*!< 270 (Price FIX.4.2) Price of the Market Data Entry.*/
MDEntrySize                                        = 271, /*!< 271 (Qty FIX.4.2) Quantity or volume represented by the Market Data Entry.*/
MDEntryDate                                        = 272, /*!< 272 (UTCDateOnly FIX.4.2) Date of Market Data Entry.

(prior to FIX 4.4 field was of type UTCDate)*/
MDEntryTime                                        = 273, /*!< 273 (UTCTimeOnly FIX.4.2) Time of Market Data Entry.*/
TickDirection                                      = 274, /*!< 274 (char FIX.4.2) Direction of the "tick".*/
MDMkt                                              = 275, /*!< 275 (Exchange FIX.4.2) Market posting quote / trade.

Valid values:

See "Appendix 6-C"*/
QuoteCondition                                     = 276, /*!< 276 (MultipleStringValue FIX.4.2) Space-delimited list of conditions describing a quote.*/
TradeCondition                                     = 277, /*!< 277 (MultipleStringValue FIX.4.2) Type of market data entry.*/
MDEntryID                                          = 278, /*!< 278 (String FIX.4.2) Unique Market Data Entry identifier.*/
MDUpdateAction                                     = 279, /*!< 279 (char FIX.4.2) Type of Market Data update action.*/
MDEntryRefID                                       = 280, /*!< 280 (String FIX.4.2) Refers to a previous MDEntryID (278).*/
MDReqRejReason                                     = 281, /*!< 281 (char FIX.4.2) Reason for the rejection of a Market Data request.*/
MDEntryOriginator                                  = 282, /*!< 282 (String FIX.4.2) Originator of a Market Data Entry*/
LocationID                                         = 283, /*!< 283 (String FIX.4.2) Identification of a Market Maker's location*/
DeskID                                             = 284, /*!< 284 (String FIX.4.2) Identification of a Market Maker's desk*/
DeleteReason                                       = 285, /*!< 285 (char FIX.4.2) Reason for deletion.*/
OpenCloseSettlFlag                                 = 286, /*!< 286 (MultipleCharValue FIX.4.2) Flag that identifies a market data entry. (Prior to FIX 4.3 this field was of type char)*/
SellerDays                                         = 287, /*!< 287 (int FIX.4.2) Specifies the number of days that may elapse before delivery of the security*/
MDEntryBuyer                                       = 288, /*!< 288 (String FIX.4.2) Buying party in a trade*/
MDEntrySeller                                      = 289, /*!< 289 (String FIX.4.2) Selling party in a trade*/
MDEntryPositionNo                                  = 290, /*!< 290 (int FIX.4.2) Display position of a bid or offer, numbered from most competitive to least competitive, per market side, beginning with .*/
FinancialStatus                                    = 291, /*!< 291 (MultipleCharValue FIX.4.2) Identifies a firm's or a security's financial status*/
CorporateAction                                    = 292, /*!< 292 (MultipleCharValue FIX.4.2) Identifies the type of Corporate Action.*/
DefBidSize                                         = 293, /*!< 293 (Qty FIX.4.2) Default Bid Size.*/
DefOfferSize                                       = 294, /*!< 294 (Qty FIX.4.2) Default Offer Size.*/
NoQuoteEntries                                     = 295, /*!< 295 (NumInGroup FIX.4.2) The number of quote entries for a QuoteSet.*/
NoQuoteSets                                        = 296, /*!< 296 (NumInGroup FIX.4.2) The number of sets of quotes in the message.*/
QuoteStatus                                        = 297, /*!< 297 (int FIX.4.2) Identifies the status of the quote acknowledgement.*/
QuoteCancelType                                    = 298, /*!< 298 (int FIX.4.2) Identifies the type of quote cancel.*/
QuoteEntryID                                       = 299, /*!< 299 (String FIX.4.2) Unique identifier for a quote. The QuoteEntryID stays with the quote as a static identifier even if the quote is updated.*/
QuoteRejectReason                                  = 300, /*!< 300 (int FIX.4.2) Reason Quote was rejected:*/
QuoteResponseLevel                                 = 301, /*!< 301 (int FIX.4.2) Level of Response requested from receiver of quote messages. A default value should be bilaterally agreed.*/
QuoteSetID                                         = 302, /*!< 302 (String FIX.4.2) Unique id for the Quote Set.*/
QuoteRequestType                                   = 303, /*!< 303 (int FIX.4.2) Indicates the type of Quote Request being generated*/
TotNoQuoteEntries                                  = 304, /*!< 304 (int FIX.4.2) Total number of quotes for the quote set.*/
UnderlyingSecurityIDSource                         = 305, /*!< 305 (String FIX.4.2) Underlying security's SecurityIDSource.

Valid values: see SecurityIDSource (22) field*/
UnderlyingIssuer                                   = 306, /*!< 306 (String FIX.4.2) Underlying security's Issuer.

See Issuer (06) field for description*/
UnderlyingSecurityDesc                             = 307, /*!< 307 (String FIX.4.2) Description of the underlying security.

Can be used by the venue or one of the trading parties to provide an optional non-normative textual description of the financial instrument.

*/
UnderlyingSecurityExchange                         = 308, /*!< 308 (Exchange FIX.4.2) Underlying security's SecurityExchange. Can be used to identify the underlying security.

Valid values: see SecurityExchange (207)*/
UnderlyingSecurityID                               = 309, /*!< 309 (String FIX.4.2) Underlying security's SecurityID.

See SecurityID (48) field for description*/
UnderlyingSecurityType                             = 310, /*!< 310 (String FIX.4.2) Underlying security's SecurityType.

Valid values: see SecurityType (167) field

(see below for details concerning this fields use in conjunction with SecurityType=REPO)

The following applies when used in conjunction with SecurityType=REPO

Represents the general or specific type of security that underlies a financing agreement

Valid values for SecurityType=REPO:

If bonds of a particular issuer or country are wanted in an Order or are in the basket of an Execution and the SecurityType is not granular enough, include the UnderlyingIssuer (306), UnderlyingCountryOfIssue (592), UnderlyingProgram, UnderlyingRegType and/or &lt; UnderlyingStipulations &gt; block e.g.:*/
UnderlyingSymbol                                   = 311, /*!< 311 (String FIX.4.2) Underlying security's Symbol.

See Symbol (55) field for description*/
UnderlyingSymbolSfx                                = 312, /*!< 312 (String FIX.4.2) Underlying security's SymbolSfx.

See SymbolSfx (65) field for description*/
UnderlyingMaturityMonthYear                        = 313, /*!< 313 (MonthYear FIX.4.2) Underlying security's MaturityMonthYear. Can be used with standardized derivatives vs. the UnderlyingMaturityDate (542) field.

See MaturityMonthYear (200) field for description*/
UnderlyingPutOrCall                                = 315, /*!< 315 (int FIX.4.2) Indicates whether an underlying option contract is a put, call, chooser or undetermined.*/
UnderlyingStrikePrice                              = 316, /*!< 316 (Price FIX.4.2) Underlying security's StrikePrice.

See StrikePrice (202) field for description*/
UnderlyingOptAttribute                             = 317, /*!< 317 (char FIX.4.2) Underlying security's OptAttribute.

See OptAttribute (206) field for description*/
UnderlyingCurrency                                 = 318, /*!< 318 (Currency FIX.4.2) Underlying security's Currency.

See Currency (5) field for description and valid values*/
SecurityReqID                                      = 320, /*!< 320 (String FIX.4.2) Unique ID of a Security Definition Request.*/
SecurityRequestType                                = 321, /*!< 321 (int FIX.4.2) Type of Security Definition Request.*/
SecurityResponseID                                 = 322, /*!< 322 (String FIX.4.2) Unique ID of a Security Definition message.*/
SecurityResponseType                               = 323, /*!< 323 (int FIX.4.2) Type of Security Definition message response.*/
SecurityStatusReqID                                = 324, /*!< 324 (String FIX.4.2) Unique ID of a Security Status Request or a Security Mass Status Request message.*/
UnsolicitedIndicator                               = 325, /*!< 325 (Boolean FIX.4.2) Indicates whether or not message is being sent as a result of a subscription request or not.*/
SecurityTradingStatus                              = 326, /*!< 326 (int FIX.4.2) Identifies the trading status applicable to the transaction.*/
HaltReason                                         = 327, /*!< 327 (int FIX.4.2) Denotes the reason for the Opening Delay or Trading Halt.*/
InViewOfCommon                                     = 328, /*!< 328 (Boolean FIX.4.2) Indicates whether or not the halt was due to Common Stock trading being halted.*/
DueToRelated                                       = 329, /*!< 329 (Boolean FIX.4.2) Indicates whether or not the halt was due to the Related Security being halted.*/
BuyVolume                                          = 330, /*!< 330 (Qty FIX.4.2) Quantity bought.*/
SellVolume                                         = 331, /*!< 331 (Qty FIX.4.2) Quantity sold.*/
HighPx                                             = 332, /*!< 332 (Price FIX.4.2) Represents an indication of the high end of the price range for a security prior to the open or reopen*/
LowPx                                              = 333, /*!< 333 (Price FIX.4.2) Represents an indication of the low end of the price range for a security prior to the open or reopen*/
Adjustment                                         = 334, /*!< 334 (int FIX.4.2) Identifies the type of adjustment.*/
TradSesReqID                                       = 335, /*!< 335 (String FIX.4.2) Unique ID of a Trading Session Status message.*/
TradingSessionID                                   = 336, /*!< 336 (String FIX.4.2) Identifier for a trading session.

A trading session spans an extended period of time that can also be expressed informally in terms of the trading day. Usage is determined by market or counterparties.

To specify good for session where session spans more than one calendar day, use TimeInForce = 0 (Day) in conjunction with TradingSessionID(336).

Bilaterally agreed values of data type "String" that start with a character can be used for backward compatibility.*/
ContraTrader                                       = 337, /*!< 337 (String FIX.4.2) Identifies the trader (e.g. "badge number") of the ContraBroker.*/
TradSesMethod                                      = 338, /*!< 338 (int FIX.4.2) Method of trading*/
TradSesMode                                        = 339, /*!< 339 (int FIX.4.2) Trading Session Mode*/
TradSesStatus                                      = 340, /*!< 340 (int FIX.4.2) State of the trading session.*/
TradSesStartTime                                   = 341, /*!< 341 (UTCTimestamp FIX.4.2) Starting time of the trading session*/
TradSesOpenTime                                    = 342, /*!< 342 (UTCTimestamp FIX.4.2) Time of the opening of the trading session*/
TradSesPreCloseTime                                = 343, /*!< 343 (UTCTimestamp FIX.4.2) Time of the pre-closed of the trading session*/
TradSesCloseTime                                   = 344, /*!< 344 (UTCTimestamp FIX.4.2) Closing time of the trading session*/
TradSesEndTime                                     = 345, /*!< 345 (UTCTimestamp FIX.4.2) End time of the trading session*/
NumberOfOrders                                     = 346, /*!< 346 (int FIX.4.2) Number of orders in the market.*/
MessageEncoding                                    = 347, /*!< 347 (String FIX.4.2) Type of message encoding (non-ASCII (non-English) characters) used in a message's "Encoded" fields.*/
EncodedIssuerLen                                   = 348, /*!< 348 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedIssuer (349) field.*/
EncodedIssuer                                      = 349, /*!< 349 (data FIX.4.2) Encoded (non-ASCII characters) representation of the Issuer field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the Issuer field.*/
EncodedSecurityDescLen                             = 350, /*!< 350 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedSecurityDesc (351) field.*/
EncodedSecurityDesc                                = 351, /*!< 351 (data FIX.4.2) Encoded (non-ASCII characters) representation of the SecurityDesc (107) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the SecurityDesc field.*/
EncodedListExecInstLen                             = 352, /*!< 352 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedListExecInst (353) field.*/
EncodedListExecInst                                = 353, /*!< 353 (data FIX.4.2) Encoded (non-ASCII characters) representation of the ListExecInst (69) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the ListExecInst field.*/
EncodedTextLen                                     = 354, /*!< 354 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedText (355) field.*/
EncodedText                                        = 355, /*!< 355 (data FIX.4.2) Encoded (non-ASCII characters) representation of the Text (58) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the Text(58) field.*/
EncodedSubjectLen                                  = 356, /*!< 356 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedSubject (357) field.*/
EncodedSubject                                     = 357, /*!< 357 (data FIX.4.2) Encoded (non-ASCII characters) representation of the Subject (147) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the Subject field.*/
EncodedHeadlineLen                                 = 358, /*!< 358 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedHeadline (359) field.*/
EncodedHeadline                                    = 359, /*!< 359 (data FIX.4.2) Encoded (non-ASCII characters) representation of the Headline (148) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the Headline field.*/
EncodedAllocTextLen                                = 360, /*!< 360 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedAllocText (361) field.*/
EncodedAllocText                                   = 361, /*!< 361 (data FIX.4.2) Encoded (non-ASCII characters) representation of the AllocText (161) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the AllocText field.*/
EncodedUnderlyingIssuerLen                         = 362, /*!< 362 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingIssuer (363) field.*/
EncodedUnderlyingIssuer                            = 363, /*!< 363 (data FIX.4.2) Encoded (non-ASCII characters) representation of the UnderlyingIssuer (306) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingIssuer field.*/
EncodedUnderlyingSecurityDescLen                   = 364, /*!< 364 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingSecurityDesc (365) field.*/
EncodedUnderlyingSecurityDesc                      = 365, /*!< 365 (data FIX.4.2) Encoded (non-ASCII characters) representation of the UnderlyingSecurityDesc (307) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingSecurityeDesc field.*/
AllocPrice                                         = 366, /*!< 366 (Price FIX.4.2) Executed price for an AllocAccount (79) entry used when using "executed price" vs. "average price" allocations (e.g. Japan).*/
QuoteSetValidUntilTime                             = 367, /*!< 367 (UTCTimestamp FIX.4.2) Indicates expiration time of this particular QuoteSet (always expressed in UTC (Universal Time Coordinated, also known as "GMT")*/
QuoteEntryRejectReason                             = 368, /*!< 368 (int FIX.4.2) Reason Quote Entry was rejected:*/
LastMsgSeqNumProcessed                             = 369, /*!< 369 (SeqNum FIX.4.2) The last MsgSeqNum (34) value received by the FIX engine and processed by downstream application, such as trading engine or order routing system. Can be specified on every message sent. Useful for detecting a backlog with a counterparty.*/
RefTagID                                           = 371, /*!< 371 (int FIX.4.2) The tag number of the FIX field being referenced.*/
RefMsgType                                         = 372, /*!< 372 (String FIX.4.2) The MsgType (35) of the FIX message being referenced.*/
SessionRejectReason                                = 373, /*!< 373 (int FIX.4.2) Code to identify reason for a session-level Reject message.*/
BidRequestTransType                                = 374, /*!< 374 (char FIX.4.2) Identifies the Bid Request message type.*/
ContraBroker                                       = 375, /*!< 375 (String FIX.4.2) Identifies contra broker. Standard NASD market-maker mnemonic is preferred.*/
ComplianceID                                       = 376, /*!< 376 (String FIX.4.2) ID used to represent this transaction for compliance purposes (e.g. OATS reporting).*/
SolicitedFlag                                      = 377, /*!< 377 (Boolean FIX.4.2) Indicates whether or not the order was solicited.*/
ExecRestatementReason                              = 378, /*!< 378 (int FIX.4.2) The reason for restatement when an ExecutionReport(35=8) or TradeCaptureReport(35=AE) message is sent with ExecType(150) = D (Restated) or used when communicating an unsolicited cancel.*/
BusinessRejectRefID                                = 379, /*!< 379 (String FIX.4.2) The value of the business-level "ID" field on the message being referenced.*/
BusinessRejectReason                               = 380, /*!< 380 (int FIX.4.2) Code to identify reason for a Business Message Reject message.*/
GrossTradeAmt                                      = 381, /*!< 381 (Amt FIX.4.2) Total amount traded (i.e. quantity * price) expressed in units of currency. For FX Futures this is used to express the notional value of a fill when quantity fields are expressed in terms of contract size (i.e. quantity * price * contract size).*/
NoContraBrokers                                    = 382, /*!< 382 (NumInGroup FIX.4.2) The number of ContraBroker (375) entries.*/
MaxMessageSize                                     = 383, /*!< 383 (Length FIX.4.2) Maximum number of bytes supported for a single message.*/
NoMsgTypes                                         = 384, /*!< 384 (NumInGroup FIX.4.2) Number of MsgTypes (35) in repeating group.*/
MsgDirection                                       = 385, /*!< 385 (char FIX.4.2) Specifies the direction of the messsage.*/
NoTradingSessions                                  = 386, /*!< 386 (NumInGroup FIX.4.2) Number of TradingSessionIDs (336) in repeating group.*/
TotalVolumeTraded                                  = 387, /*!< 387 (Qty FIX.4.2) Total volume (quantity) traded.*/
DiscretionInst                                     = 388, /*!< 388 (char FIX.4.2) Code to identify the price a DiscretionOffsetValue (389) is related to and should be mathematically added to.*/
DiscretionOffsetValue                              = 389, /*!< 389 (float FIX.4.2) Amount (signed) added to the "related to" price specified via DiscretionInst (388), in the context of DiscretionOffsetType (842)

(Prior to FIX 4.4 this field was of type PriceOffset)*/
BidID                                              = 390, /*!< 390 (String FIX.4.2) For bid lists, unique identifier for BidResponse(35=I) as assigned by sell-side (broker, exchange, ECN). Uniqueness must be guaranteed within a single trading day.



For quotes, unique identifier for the bid side of the quote assigned by the quote issuer.*/
ClientBidID                                        = 391, /*!< 391 (String FIX.4.2) Unique identifier for a Bid Request as assigned by institution. Uniqueness must be guaranteed within a single trading day.*/
ListName                                           = 392, /*!< 392 (String FIX.4.2) Descriptive name for list order.*/
TotNoRelatedSym                                    = 393, /*!< 393 (int FIX.4.2) Total number of securities.

(Prior to FIX 4.4 this field was named TotalNumSecurities)*/
BidType                                            = 394, /*!< 394 (int FIX.4.2) Code to identify the type of Bid Request.*/
NumTickets                                         = 395, /*!< 395 (int FIX.4.2) Total number of tickets.*/
SideValue1                                         = 396, /*!< 396 (Amt FIX.4.2) Amounts in currency*/
SideValue2                                         = 397, /*!< 397 (Amt FIX.4.2) Amounts in currency*/
NoBidDescriptors                                   = 398, /*!< 398 (NumInGroup FIX.4.2) Number of BidDescriptor (400) entries.*/
BidDescriptorType                                  = 399, /*!< 399 (int FIX.4.2) Code to identify the type of BidDescriptor (400).*/
BidDescriptor                                      = 400, /*!< 400 (String FIX.4.2) BidDescriptor value. Usage depends upon BidDescriptorTyp (399).

If BidDescriptorType = 1

Industrials etc - Free text

If BidDescriptorType = 2

"FR" etc - ISO Country Codes

If BidDescriptorType = 3

FT00, FT250, STOX - Free text*/
SideValueInd                                       = 401, /*!< 401 (int FIX.4.2) Code to identify which "SideValue" the value refers to. SideValue1 and SideValue2 are used as opposed to Buy or Sell so that the basket can be quoted either way as Buy or Sell.*/
LiquidityPctLow                                    = 402, /*!< 402 (Percentage FIX.4.2) Liquidity indicator or lower limit if TotalNumSecurities (393) &gt; 1. Represented as a percentage.*/
LiquidityPctHigh                                   = 403, /*!< 403 (Percentage FIX.4.2) Upper liquidity indicator if TotalNumSecurities (393) &gt; 1. Represented as a percentage.*/
LiquidityValue                                     = 404, /*!< 404 (Amt FIX.4.2) Value between LiquidityPctLow (402) and LiquidityPctHigh (403) in Currency*/
EFPTrackingError                                   = 405, /*!< 405 (Percentage FIX.4.2) Eg Used in EFP trades 2% (EFP - Exchange for Physical ). Represented as a percentage.*/
FairValue                                          = 406, /*!< 406 (Amt FIX.4.2) Used in EFP trades*/
OutsideIndexPct                                    = 407, /*!< 407 (Percentage FIX.4.2) Used in EFP trades. Represented as a percentage.*/
ValueOfFutures                                     = 408, /*!< 408 (Amt FIX.4.2) Used in EFP trades*/
LiquidityIndType                                   = 409, /*!< 409 (int FIX.4.2) Code to identify the type of liquidity indicator.*/
WtAverageLiquidity                                 = 410, /*!< 410 (Percentage FIX.4.2) Overall weighted average liquidity expressed as a % of average daily volume. Represented as a percentage.*/
ExchangeForPhysical                                = 411, /*!< 411 (Boolean FIX.4.2) Indicates whether or not to exchange for phsyical.*/
OutMainCntryUIndex                                 = 412, /*!< 412 (Amt FIX.4.2) Value of stocks in Currency*/
CrossPercent                                       = 413, /*!< 413 (Percentage FIX.4.2) Percentage of program that crosses in Currency. Represented as a percentage.*/
ProgRptReqs                                        = 414, /*!< 414 (int FIX.4.2) Code to identify the desired frequency of progress reports.*/
ProgPeriodInterval                                 = 415, /*!< 415 (int FIX.4.2) Time in minutes between each ListStatus report sent by SellSide. Zero means don't send status.*/
IncTaxInd                                          = 416, /*!< 416 (int FIX.4.2) Code to represent whether value is net (inclusive of tax) or gross.*/
NumBidders                                         = 417, /*!< 417 (int FIX.4.2) Indicates the total number of bidders on the list*/
BidTradeType                                       = 418, /*!< 418 (char FIX.4.2) Code to represent the type of trade.

(Prior to FIX 4.4 this field was named "TradeType")*/
BasisPxType                                        = 419, /*!< 419 (char FIX.4.2) Code to represent the basis price type.*/
NoBidComponents                                    = 420, /*!< 420 (NumInGroup FIX.4.2) Indicates the number of list entries.*/
Country                                            = 421, /*!< 421 (Country FIX.4.2) ISO Country Code in field*/
TotNoStrikes                                       = 422, /*!< 422 (int FIX.4.2) Total number of strike price entries across all messages. Should be the sum of all NoStrikes (428) in each message that has repeating strike price entries related to the same ListID (66). Used to support fragmentation.*/
PriceType                                          = 423, /*!< 423 (int FIX.4.2) Code to represent the price type.*/
DayOrderQty                                        = 424, /*!< 424 (Qty FIX.4.2) For GT orders, the OrderQty (38) less all quantity (adjusted for stock splits) that traded on previous days. DayOrderQty (424) = OrderQty - (CumQty (14) - DayCumQty (425))*/
DayCumQty                                          = 425, /*!< 425 (Qty FIX.4.2) Quantity on a GT order that has traded today.*/
DayAvgPx                                           = 426, /*!< 426 (Price FIX.4.2) The average price for quantity on a GT order that has traded today.*/
GTBookingInst                                      = 427, /*!< 427 (int FIX.4.2) Code to identify whether to book out executions on a part-filled GT order on the day of execution or to accumulate.*/
NoStrikes                                          = 428, /*!< 428 (NumInGroup FIX.4.2) Number of list strike price entries.*/
ListStatusType                                     = 429, /*!< 429 (int FIX.4.2) Code to represent the status type.*/
NetGrossInd                                        = 430, /*!< 430 (int FIX.4.2) Code to represent whether value is net (inclusive of tax) or gross.*/
ListOrderStatus                                    = 431, /*!< 431 (int FIX.4.2) Code to represent the status of a list order.*/
ExpireDate                                         = 432, /*!< 432 (LocalMktDate FIX.4.2) Date of order expiration (last day the order can trade), always expressed in terms of the local market date. The time at which the order expires is determined by the local market's business practices*/
ListExecInstType                                   = 433, /*!< 433 (char FIX.4.2) Identifies the type of ListExecInst (69).*/
CxlRejResponseTo                                   = 434, /*!< 434 (char FIX.4.2) Identifies the type of request that a Cancel Reject is in response to.*/
UnderlyingCouponRate                               = 435, /*!< 435 (Percentage FIX.4.2) Underlying security's CouponRate.

See CouponRate (223) field for description*/
UnderlyingContractMultiplier                       = 436, /*!< 436 (float FIX.4.2) Underlying security's ContractMultiplier.

See ContractMultiplier (231) field for description*/
ContraTradeQty                                     = 437, /*!< 437 (Qty FIX.4.2) Quantity traded with the ContraBroker (375).*/
ContraTradeTime                                    = 438, /*!< 438 (UTCTimestamp FIX.4.2) Identifes the time of the trade with the ContraBroker (375). (always expressed in UTC (Universal Time Coordinated, also known as "GMT")*/
LiquidityNumSecurities                             = 441, /*!< 441 (int FIX.4.2) Number of Securites between LiquidityPctLow (402) and LiquidityPctHigh (403) in Currency.*/
MultiLegReportingType                              = 442, /*!< 442 (char FIX.4.2) Used to indicate how the multi-legged security (e.g. option strategies, spreads, etc.) is being reported. */
StrikeTime                                         = 443, /*!< 443 (UTCTimestamp FIX.4.2) The time at which current market prices are used to determine the value of a basket. 

In negotiation workflows where a spread-to-benchmark price is negotiated, this is the pre-determined time at which the benchmark is to be spotted. */
ListStatusText                                     = 444, /*!< 444 (String FIX.4.2) Free format text string related to List Status.*/
EncodedListStatusTextLen                           = 445, /*!< 445 (Length FIX.4.2) Byte length of encoded (non-ASCII characters) EncodedListStatusText (446) field.*/
EncodedListStatusText                              = 446, /*!< 446 (data FIX.4.2) Encoded (non-ASCII characters) representation of the ListStatusText (444) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the ListStatusText field.*/
PartyIDSource                                      = 447, /*!< 447 (char FIX.4.3) Identifies class or source of the PartyID (448) value. Required if PartyID is specified. Note: applicable values depend upon PartyRole (452) specified.

See "Appendix 6-G - Use of &lt;Parties&gt; Component Block"*/
PartyID                                            = 448, /*!< 448 (String FIX.4.3) Party identifier/code. See PartyIDSource (447) and PartyRole (452).

See "Appendix 6-G - Use of &lt;Parties&gt; Component Block"*/
NetChgPrevDay                                      = 451, /*!< 451 (PriceOffset FIX.4.3) Net change from previous day's closing price vs. last traded price.*/
PartyRole                                          = 452, /*!< 452 (int FIX.4.3) Identifies the type or role of the PartyID (448) specified.

See "Appendix 6-G - Use of &lt;Parties&gt; Component Block"

(see Volume : "Glossary" for value definitions)*/
NoPartyIDs                                         = 453, /*!< 453 (NumInGroup FIX.4.3) Number of PartyID (448), PartyIDSource (447), and PartyRole (452) entries*/
NoSecurityAltID                                    = 454, /*!< 454 (NumInGroup FIX.4.3) Number of SecurityAltID (455) entries.*/
SecurityAltID                                      = 455, /*!< 455 (String FIX.4.3) Alternate Security identifier value for this security of SecurityAltIDSource (456) type (e.g. CUSIP, SEDOL, ISIN, etc). Requires SecurityAltIDSource.*/
SecurityAltIDSource                                = 456, /*!< 456 (String FIX.4.3) Identifies class or source of the SecurityAltID (455) value. Required if SecurityAltID is specified.

Valid values:

Same valid values as the SecurityIDSource (22) field*/
NoUnderlyingSecurityAltID                          = 457, /*!< 457 (NumInGroup FIX.4.3) Number of UnderlyingSecurityAltID (458) entries.*/
UnderlyingSecurityAltID                            = 458, /*!< 458 (String FIX.4.3) Alternate Security identifier value for this underlying security of UnderlyingSecurityAltIDSource (459) type (e.g. CUSIP, SEDOL, ISIN, etc). Requires UnderlyingSecurityAltIDSource.*/
UnderlyingSecurityAltIDSource                      = 459, /*!< 459 (String FIX.4.3) Identifies class or source of the UnderlyingSecurityAltID (458) value. Required if UnderlyingSecurityAltID is specified.

Valid values:

Same valid values as the SecurityIDSource (22) field*/
Product                                            = 460, /*!< 460 (int FIX.4.3) Indicates the type of product the security is associated with. See also the CFICode (461) and SecurityType (167) fields.*/
CFICode                                            = 461, /*!< 461 (String FIX.4.3) Indicates the type of security using ISO 10962 standard, Classification of Financial Instruments (CFI code) values. ISO 10962 is maintained by ANNA (Association of National Numbering Agencies) acting as Registration Authority. See "Appendix 6-B FIX Fields Based Upon Other Standards". See also the Product (460) and SecurityType (167) fields. It is recommended that CFICode be used instead of SecurityType (167) for non-Fixed Income instruments.

A subset of possible values applicable to FIX usage are identified in "Appendix 6-D CFICode Usage - ISO 10962 Classification of Financial Instruments (CFI code)"*/
UnderlyingProduct                                  = 462, /*!< 462 (int FIX.4.3) Underlying security's Product.

Valid values: see Product(460) field*/
UnderlyingCFICode                                  = 463, /*!< 463 (String FIX.4.3) Underlying security's CFICode.

Valid values: see CFICode (461) field*/
TestMessageIndicator                               = 464, /*!< 464 (Boolean FIX.4.3) Indicates whether or not this FIX Session is a "test" vs. "production" connection. Useful for preventing "accidents".*/
BookingRefID                                       = 466, /*!< 466 (String FIX.4.3) Common reference passed to a post-trade booking process (e.g. industry matching utility).*/
IndividualAllocID                                  = 467, /*!< 467 (String FIX.4.3) Unique identifier for a specific NoAllocs (78) repeating group instance (e.g. for an AllocAccount).*/
RoundingDirection                                  = 468, /*!< 468 (char FIX.4.3) Specifies which direction to round For CIV - indicates whether or not the quantity of shares/units is to be rounded and in which direction where CashOrdQty (152) or (for CIV only) OrderPercent (516) are specified on an order.

The default is for rounding to be at the discretion of the executing broker or fund manager.

e.g. for an order specifying CashOrdQty or OrderPercent if the calculated number of shares/units was 325.76 and RoundingModulus (469) was 0 - "round down" would give 320 units, 1 - "round up" would give 330 units and "round to nearest" would give 320 units.*/
RoundingModulus                                    = 469, /*!< 469 (float FIX.4.3) For CIV - a float value indicating the value to which rounding is required.

i.e. 0 means round to a multiple of 0 units/shares; 0.5 means round to a multiple of 0.5 units/shares.

The default, if RoundingDirection (468) is specified without RoundingModulus, is to round to a whole unit/share.*/
CountryOfIssue                                     = 470, /*!< 470 (Country FIX.4.3) ISO Country code of instrument issue (e.g. the country portion typically used in ISIN). Can be used in conjunction with non-ISIN SecurityID (48) (e.g. CUSIP for Municipal Bonds without ISIN) to provide uniqueness.*/
StateOrProvinceOfIssue                             = 471, /*!< 471 (String FIX.4.3) A two-character state or province abbreviation.*/
LocaleOfIssue                                      = 472, /*!< 472 (String FIX.4.3) Identifies the locale or region of issue. */
NoRegistDtls                                       = 473, /*!< 473 (NumInGroup FIX.4.3) The number of registration details on a Registration Instructions message*/
MailingDtls                                        = 474, /*!< 474 (String FIX.4.3) Set of Correspondence address details, possibly including phone, fax, etc.*/
InvestorCountryOfResidence                         = 475, /*!< 475 (Country FIX.4.3) The ISO 366 Country code (2 character) identifying which country the beneficial investor is resident for tax purposes.*/
PaymentRef                                         = 476, /*!< 476 (String FIX.4.3) "Settlement Payment Reference" - A free format Payment reference to assist with reconciliation, e.g. a Client and/or Order ID number.*/
DistribPaymentMethod                               = 477, /*!< 477 (int FIX.4.3) A code identifying the payment method for a (fractional) distribution.

13 through 998 are reserved for future use

Values above 1000 are available for use by private agreement among counterparties*/
CashDistribCurr                                    = 478, /*!< 478 (Currency FIX.4.3) Specifies currency to be used for Cash Distributions see "Appendix 6-A Valid Currency Codes".*/
CommCurrency                                       = 479, /*!< 479 (Currency FIX.4.3) Specifies currency to be use for Commission (12) if the Commission currency is different from the Deal Currency - see "Appendix 6-A; Valid Currency Codes".*/
CancellationRights                                 = 480, /*!< 480 (char FIX.4.3) For CIV - A one character code identifying whether Cancellation rights/Cooling off period applies.*/
MoneyLaunderingStatus                              = 481, /*!< 481 (char FIX.4.3) A one character code identifying Money laundering status.*/
MailingInst                                        = 482, /*!< 482 (String FIX.4.3) Free format text to specify mailing instruction requirements, e.g. "no third party mailings".*/
TransBkdTime                                       = 483, /*!< 483 (UTCTimestamp FIX.4.3) For CIV A date and time stamp to indicate the time a CIV order was booked by the fund manager.

For derivatives a date and time stamp to indicate when this order was booked with the agent prior to submission to the VMU. Indicates the time at which the order was finalized between the buyer and seller prior to submission.*/
ExecPriceType                                      = 484, /*!< 484 (char FIX.4.3) For CIV - Identifies how the execution price LastPx (31) was calculated from the fund unit/share price(s) calculated at the fund valuation point.*/
ExecPriceAdjustment                                = 485, /*!< 485 (float FIX.4.3) For CIV the amount or percentage by which the fund unit/share price was adjusted, as indicated by ExecPriceType (484)*/
DateOfBirth                                        = 486, /*!< 486 (LocalMktDate FIX.4.3) The date of birth applicable to the individual, e.g. required to open some types of tax-exempt account.*/
TradeReportTransType                               = 487, /*!< 487 (int FIX.4.3) Identifies Trade Report message transaction type

(Prior to FIX 4.4 this field was of type char)*/
CardHolderName                                     = 488, /*!< 488 (String FIX.4.3) The name of the payment card holder as specified on the card being used for payment.*/
CardNumber                                         = 489, /*!< 489 (String FIX.4.3) The number of the payment card as specified on the card being used for payment.*/
CardExpDate                                        = 490, /*!< 490 (LocalMktDate FIX.4.3) The expiry date of the payment card as specified on the card being used for payment.*/
CardIssNum                                         = 491, /*!< 491 (String FIX.4.3) The issue number of the payment card as specified on the card being used for payment. This is only applicable to certain types of card.*/
PaymentMethod                                      = 492, /*!< 492 (int FIX.4.3) A code identifying the Settlement payment method. 16 through 998 are reserved for future use

Values above 1000 are available for use by private agreement among counterparties*/
RegistAcctType                                     = 493, /*!< 493 (String FIX.4.3) For CIV - a fund manager-defined code identifying which of the fund manager's account types is required.*/
Designation                                        = 494, /*!< 494 (String FIX.4.3) Free format text defining the designation to be associated with a holding on the register. Used to identify assets of a specific underlying investor using a common registration, e.g. a broker's nominee or street name.*/
TaxAdvantageType                                   = 495, /*!< 495 (int FIX.4.3) For CIV - a code identifying the type of tax exempt account in which purchased shares/units are to be held.

30 - 998 are reserved for future use by recognized taxation authorities

999=Other

values above 1000 are available for use by private agreement among counterparties*/
RegistRejReasonText                                = 496, /*!< 496 (String FIX.4.3) Text indicating reason(s) why a Registration Instruction has been rejected.*/
FundRenewWaiv                                      = 497, /*!< 497 (char FIX.4.3) A one character code identifying whether the Fund based renewal commission is to be waived.*/
CashDistribAgentName                               = 498, /*!< 498 (String FIX.4.3) Name of local agent bank if for cash distributions*/
CashDistribAgentCode                               = 499, /*!< 499 (String FIX.4.3) BIC (Bank Identification Code--Swift managed) code of agent bank for cash distributions*/
CashDistribAgentAcctNumber                         = 500, /*!< 500 (String FIX.4.3) Account number at agent bank for distributions.*/
CashDistribPayRef                                  = 501, /*!< 501 (String FIX.4.3) Free format Payment reference to assist with reconciliation of distributions.*/
CashDistribAgentAcctName                           = 502, /*!< 502 (String FIX.4.3) Name of account at agent bank for distributions.*/
CardStartDate                                      = 503, /*!< 503 (LocalMktDate FIX.4.3) The start date of the card as specified on the card being used for payment.*/
PaymentDate                                        = 504, /*!< 504 (LocalMktDate FIX.4.3) The date written on a cheque or date payment should be submitted to the relevant clearing system.*/
PaymentRemitterID                                  = 505, /*!< 505 (String FIX.4.3) Identifies sender of a payment, e.g. the payment remitter or a customer reference number.*/
RegistStatus                                       = 506, /*!< 506 (char FIX.4.3) Registration status as returned by the broker or (for CIV) the fund manager:*/
RegistRejReasonCode                                = 507, /*!< 507 (int FIX.4.3) Reason(s) why Registration Instructions has been rejected.

The reason may be further amplified in the RegistRejReasonCode field.

Possible values of reason code include:*/
RegistRefID                                        = 508, /*!< 508 (String FIX.4.3) Reference identifier for the RegistID (53) with Cancel and Replace RegistTransType (54) transaction types.*/
RegistDtls                                         = 509, /*!< 509 (String FIX.4.3) Set of Registration name and address details, possibly including phone, fax etc.*/
NoDistribInsts                                     = 510, /*!< 510 (NumInGroup FIX.4.3) The number of Distribution Instructions on a Registration Instructions message*/
RegistEmail                                        = 511, /*!< 511 (String FIX.4.3) Email address relating to Registration name and address details*/
DistribPercentage                                  = 512, /*!< 512 (Percentage FIX.4.3) The amount of each distribution to go to this beneficiary, expressed as a percentage*/
RegistID                                           = 513, /*!< 513 (String FIX.4.3) Unique identifier of the registration details as assigned by institution or intermediary.*/
RegistTransType                                    = 514, /*!< 514 (char FIX.4.3) Identifies Registration Instructions transaction type*/
ExecValuationPoint                                 = 515, /*!< 515 (UTCTimestamp FIX.4.3) For CIV - a date and time stamp to indicate the fund valuation point with respect to which a order was priced by the fund manager.*/
OrderPercent                                       = 516, /*!< 516 (Percentage FIX.4.3) For CIV specifies the approximate order quantity desired. For a CIV Sale it specifies percentage of investor's total holding to be sold. For a CIV switch/exchange it specifies percentage of investor's cash realised from sales to be re-invested. The executing broker, intermediary or fund manager is responsible for converting and calculating OrderQty (38) in shares/units for subsequent messages.*/
OwnershipType                                      = 517, /*!< 517 (char FIX.4.3) The relationship between Registration parties.*/
NoContAmts                                         = 518, /*!< 518 (NumInGroup FIX.4.3) The number of Contract Amount details on an Execution Report message*/
ContAmtType                                        = 519, /*!< 519 (int FIX.4.3) Type of ContAmtValue (520).

NOTE That Commission Amount / % in Contract Amounts is the commission actually charged, rather than the commission instructions given in Fields 2/3.*/
ContAmtValue                                       = 520, /*!< 520 (float FIX.4.3) Value of Contract Amount, e.g. a financial amount or percentage as indicated by ContAmtType (519).*/
ContAmtCurr                                        = 521, /*!< 521 (Currency FIX.4.3) Specifies currency for the Contract amount if different from the Deal Currency - see "Appendix 6-A; Valid Currency Codes".*/
OwnerType                                          = 522, /*!< 522 (int FIX.4.3) Identifies the type of owner.*/
PartySubID                                         = 523, /*!< 523 (String FIX.4.3) Sub-identifier (e.g. Clearing Account for PartyRole (452)=Clearing Firm, Locate ID # for PartyRole=Locate/Lending Firm, etc). Not required when using PartyID (448), PartyIDSource (447), and PartyRole.*/
NestedPartyID                                      = 524, /*!< 524 (String FIX.4.3) PartyID value within a nested repeating group.

Same values as PartyID (448)*/
NestedPartyIDSource                                = 525, /*!< 525 (char FIX.4.3) PartyIDSource value within a nested repeating group.

Same values as PartyIDSource (447)*/
SecondaryClOrdID                                   = 526, /*!< 526 (String FIX.4.3) Assigned by the party which originates the order. Can be used to provide the ClOrdID (11) used by an exchange or executing system.*/
SecondaryExecID                                    = 527, /*!< 527 (String FIX.4.3) Assigned by the party which accepts the order. Can be used to provide the ExecID (17) used by an exchange or executing system.*/
OrderCapacity                                      = 528, /*!< 528 (char FIX.4.3) Designates the capacity of the firm placing the order.

(as of FIX 4.3, this field replaced Rule80A (tag 47) --used in conjunction with OrderRestrictions (529) field)

(see Volume : "Glossary" for value definitions)*/
OrderRestrictions                                  = 529, /*!< 529 (MultipleCharValue FIX.4.3) Restrictions associated with an order. If more than one restriction is applicable to an order, this field can contain multiple instructions separated by space.*/
MassCancelRequestType                              = 530, /*!< 530 (char FIX.4.3) Specifies scope of Order Mass Cancel Request.*/
MassCancelResponse                                 = 531, /*!< 531 (char FIX.4.3) Specifies the action taken by counterparty order handling system as a result of the Order Mass Cancel Request*/
MassCancelRejectReason                             = 532, /*!< 532 (int FIX.4.3) Reason Order Mass Cancel Request was rejected*/
TotalAffectedOrders                                = 533, /*!< 533 (int FIX.4.3) Total number of orders affected by either the OrderMassActionRequest(MsgType=CA) or OrderMassCancelRequest(MsgType=Q).*/
NoAffectedOrders                                   = 534, /*!< 534 (NumInGroup FIX.4.3) Number of affected orders in the repeating group of order ids.*/
AffectedOrderID                                    = 535, /*!< 535 (String FIX.4.3) OrderID(37) of an order affected by a mass cancel or mass action request.*/
AffectedSecondaryOrderID                           = 536, /*!< 536 (String FIX.4.3) SecondaryOrderID(198) of an order affected by a mass cancel or mass action request.*/
QuoteType                                          = 537, /*!< 537 (int FIX.4.3) Identifies the type of quote.

An indicative quote is used to inform a counterparty of a market. An indicative quote does not result directly in a trade.

A tradeable quote is submitted to a market and will result directly in a trade against other orders and quotes in a market.

A restricted tradeable quote is submitted to a market and within a certain restriction (possibly based upon price or quantity) will automatically trade against orders. Order that do not comply with restrictions are sent to the quote issuer who can choose to accept or decline the order.

A counter quote is used in the negotiation model. See Volume 7 - Product: Fixed Income for example usage.*/
NestedPartyRole                                    = 538, /*!< 538 (int FIX.4.3) PartyRole value within a nested repeating group.

Same values as PartyRole (452)*/
NoNestedPartyIDs                                   = 539, /*!< 539 (NumInGroup FIX.4.3) Number of NestedPartyID (524), NestedPartyIDSource (525), and NestedPartyRole (538) entries*/
TotalAccruedInterestAmt                            = 540, /*!< 540 (Amt FIX.4.3) Total Amount of Accrued Interest for convertible bonds and fixed income*/
MaturityDate                                       = 541, /*!< 541 (LocalMktDate FIX.4.3) Date of maturity.*/
UnderlyingMaturityDate                             = 542, /*!< 542 (LocalMktDate FIX.4.3) Underlying security's maturity date.

See MaturityDate (541) field for description*/
InstrRegistry                                      = 543, /*!< 543 (String FIX.4.3) Values may include BIC for the depository or custodian who maintain ownership records, the ISO country code for the location of the record, or the value "ZZ" to specify physical ownership of the security (e.g. stock certificate).*/
CashMargin                                         = 544, /*!< 544 (char FIX.4.3) Identifies whether an order is a margin order or a non-margin order. This is primarily used when sending orders to Japanese exchanges to indicate sell margin or buy to cover. The same tag could be assigned also by buy-side to indicate the intent to sell or buy margin and the sell-side to accept or reject (base on some validation criteria) the margin request.*/
NestedPartySubID                                   = 545, /*!< 545 (String FIX.4.3) PartySubID value within a nested repeating group.

Same values as PartySubID (523)*/
Scope                                              = 546, /*!< 546 (MultipleCharValue FIX.4.3) Specifies the market scope of the market data.*/
MDImplicitDelete                                   = 547, /*!< 547 (Boolean FIX.4.3) Defines how a server handles distribution of a truncated book. Defaults to broker option.*/
CrossID                                            = 548, /*!< 548 (String FIX.4.3) Identifier for a cross order. Must be unique during a given trading day. Recommend that firms use the order date as part of the CrossID for Good Till Cancel (GT) orders.*/
CrossType                                          = 549, /*!< 549 (int FIX.4.3) Type of cross being submitted to a market*/
CrossPrioritization                                = 550, /*!< 550 (int FIX.4.3) Indicates if one side or the other of a cross order should be prioritized.

The definition of prioritization is left to the market. In some markets prioritization means which side of the cross order is applied to the market first. In other markets - prioritization may mean that the prioritized side is fully executed (sometimes referred to as the side being protected).*/
OrigCrossID                                        = 551, /*!< 551 (String FIX.4.3) CrossID of the previous cross order (NOT the initial cross order of the day) as assigned by the institution, used to identify the previous cross order in Cross Cancel and Cross Cancel/Replace Requests.*/
NoSides                                            = 552, /*!< 552 (NumInGroup FIX.4.3) Number of Side repeating group instances.*/
Username                                           = 553, /*!< 553 (String FIX.4.3) Userid or username.*/
Password                                           = 554, /*!< 554 (String FIX.4.3) Password or passphrase.*/
NoLegs                                             = 555, /*!< 555 (NumInGroup FIX.4.3) Number of InstrumentLeg repeating group instances.*/
LegCurrency                                        = 556, /*!< 556 (Currency FIX.4.3) Currency associated with a particular Leg's quantity*/
TotNoSecurityTypes                                 = 557, /*!< 557 (int FIX.4.3) Used to support fragmentation. Indicates total number of security types when multiple Security Type messages are used to return results.*/
NoSecurityTypes                                    = 558, /*!< 558 (NumInGroup FIX.4.3) Number of Security Type repeating group instances.*/
SecurityListRequestType                            = 559, /*!< 559 (int FIX.4.3) Identifies the type/criteria of Security List Request*/
SecurityRequestResult                              = 560, /*!< 560 (int FIX.4.3) The results returned to a Security Request message*/
RoundLot                                           = 561, /*!< 561 (Qty FIX.4.3) The trading lot size of a security*/
MinTradeVol                                        = 562, /*!< 562 (Qty FIX.4.3) The minimum order quantity (as expressed by TradeVolType(1786)) that can be submitted for a security.*/
MultiLegRptTypeReq                                 = 563, /*!< 563 (int FIX.4.3) Indicates the method of execution reporting requested by issuer of the order.*/
LegPositionEffect                                  = 564, /*!< 564 (char FIX.4.3) PositionEffect for leg of a multileg

See PositionEffect (77) field for description*/
LegCoveredOrUncovered                              = 565, /*!< 565 (int FIX.4.3) CoveredOrUncovered for leg of a multileg

See CoveredOrUncovered (203) field for description*/
LegPrice                                           = 566, /*!< 566 (Price FIX.4.3) Price for leg of a multileg

See Price (44) field for description*/
TradSesStatusRejReason                             = 567, /*!< 567 (int FIX.4.3) Indicates the reason a Trading Session Status Request was rejected.*/
TradeRequestID                                     = 568, /*!< 568 (String FIX.4.3) Trade Capture Report Request ID*/
TradeRequestType                                   = 569, /*!< 569 (int FIX.4.3) Type of Trade Capture Report.*/
PreviouslyReported                                 = 570, /*!< 570 (Boolean FIX.4.3) Indicates if the transaction was previously reported to the counterparty or market.*/
TradeReportID                                      = 571, /*!< 571 (String FIX.4.3) Unique identifier of trade capture report*/
TradeReportRefID                                   = 572, /*!< 572 (String FIX.4.3) Reference identifier used with CANCEL and REPLACE transaction types.*/
MatchStatus                                        = 573, /*!< 573 (char FIX.4.3) The status of this trade with respect to matching or comparison.*/
MatchType                                          = 574, /*!< 574 (String FIX.4.3) The point in the matching process at which this trade was matched.*/
OddLot                                             = 575, /*!< 575 (Boolean FIX.4.3) This trade is to be treated as an odd lot

If this field is not specified, the default will be "N"*/
NoClearingInstructions                             = 576, /*!< 576 (NumInGroup FIX.4.3) Number of clearing instructions*/
ClearingInstruction                                = 577, /*!< 577 (int FIX.4.3) Eligibility of this trade for clearing and central counterparty processing.*/
TradeInputSource                                   = 578, /*!< 578 (String FIX.4.3) Type of input device or system from which the trade was entered.*/
TradeInputDevice                                   = 579, /*!< 579 (String FIX.4.3) Specific device number, terminal number or station where trade was entered*/
NoDates                                            = 580, /*!< 580 (NumInGroup FIX.4.3) Number of Date fields provided in date range*/
AccountType                                        = 581, /*!< 581 (int FIX.4.3) Type of account associated with an order*/
CustOrderCapacity                                  = 582, /*!< 582 (int FIX.4.3) Capacity of customer placing the order.*/
ClOrdLinkID                                        = 583, /*!< 583 (String FIX.4.3) Permits order originators to tie together groups of orders in which trades resulting from orders are associated for a specific purpose, for example the calculation of average execution price for a customer or to associate lists submitted to a broker as waves of a larger program trade.*/
MassStatusReqID                                    = 584, /*!< 584 (String FIX.4.3) Value assigned by issuer of Mass Status Request to uniquely identify the request*/
MassStatusReqType                                  = 585, /*!< 585 (int FIX.4.3) Mass Status Request Type*/
OrigOrdModTime                                     = 586, /*!< 586 (UTCTimestamp FIX.4.3) The most recent (or current) modification TransactTime (tag 60) reported on an Execution Report for the order. The OrigOrdModTime is provided as an optional field on Order Cancel Request and Order Cancel Replace Requests to identify that the state of the order has not changed since the request was issued. The use of this approach is not recommended.*/
LegSettlType                                       = 587, /*!< 587 (String FIX.4.3) Indicates order settlement period. If present, LegSettlDate (588) overrides this field. If both LegSettlType (587) and LegSettDate (588) are omitted, the default for LegSettlType (587) is 0 (Regular)

Regular is defined as the default settlement period for the particular security on the exchange of execution.

In Fixed Income the contents of this field may influence the instrument definition if the LegSecurityID (602) is ambiguous. In the US an active Treasury offering may be re-opened, and for a time one CUSIP will apply to both the current and "when-issued" securities. Supplying a value of "7" clarifies the instrument description; any other value or the absence of this field should cause the respondent to default to the active issue.

Additionally the following patterns may be uses as well as enum values

Dx = FX tenor expression for "days", e.g. "D5", where "x" is any integer &gt; 0

Mx = FX tenor expression for "months", e.g. "M3", where "x" is any integer &gt; 0

Wx = FX tenor expression for "weeks", e.g. "W13", where "x" is any integer &gt; 0

Yx = FX tenor expression for "years", e.g. "Y1", where "x" is any integer &gt; 0.

Note that for FX the tenors expressed using Dx, Mx, Wx, and Yx values do not denote business days, but calendar days.*/
LegSettlDate                                       = 588, /*!< 588 (LocalMktDate FIX.4.3) Refer to description for SettlDate[64]*/
DayBookingInst                                     = 589, /*!< 589 (char FIX.4.3) Indicates whether or not automatic booking can occur.*/
BookingUnit                                        = 590, /*!< 590 (char FIX.4.3) Indicates what constitutes a bookable unit.*/
PreallocMethod                                     = 591, /*!< 591 (char FIX.4.3) Indicates the method of preallocation.*/
UnderlyingCountryOfIssue                           = 592, /*!< 592 (Country FIX.4.3) Underlying security's CountryOfIssue.

See CountryOfIssue (470) field for description*/
UnderlyingStateOrProvinceOfIssue                   = 593, /*!< 593 (String FIX.4.3) Underlying security's StateOrProvinceOfIssue.

See StateOrProvinceOfIssue (471) field for description*/
UnderlyingLocaleOfIssue                            = 594, /*!< 594 (String FIX.4.3) Underlying security's LocaleOfIssue.

See LocaleOfIssue (472) field for description*/
UnderlyingInstrRegistry                            = 595, /*!< 595 (String FIX.4.3) Underlying security's InstrRegistry.

See InstrRegistry (543) field for description*/
LegCountryOfIssue                                  = 596, /*!< 596 (Country FIX.4.3) Multileg instrument's individual leg security's CountryOfIssue.

See CountryOfIssue (470) field for description*/
LegStateOrProvinceOfIssue                          = 597, /*!< 597 (String FIX.4.3) Multileg instrument's individual leg security's StateOrProvinceOfIssue.

See StateOrProvinceOfIssue (471) field for description*/
LegLocaleOfIssue                                   = 598, /*!< 598 (String FIX.4.3) Multileg instrument's individual leg security's LocaleOfIssue.

See LocaleOfIssue (472) field for description*/
LegInstrRegistry                                   = 599, /*!< 599 (String FIX.4.3) Multileg instrument's individual leg security's InstrRegistry.

See InstrRegistry (543) field for description*/
LegSymbol                                          = 600, /*!< 600 (String FIX.4.3) Multileg instrument's individual security's Symbol.

See Symbol (55) field for description*/
LegSymbolSfx                                       = 601, /*!< 601 (String FIX.4.3) Multileg instrument's individual security's SymbolSfx.

See SymbolSfx (65) field for description*/
LegSecurityID                                      = 602, /*!< 602 (String FIX.4.3) Multileg instrument's individual security's SecurityID.

See SecurityID (48) field for description*/
LegSecurityIDSource                                = 603, /*!< 603 (String FIX.4.3) Multileg instrument's individual security's SecurityIDSource.

See SecurityIDSource (22) field for description*/
NoLegSecurityAltID                                 = 604, /*!< 604 (NumInGroup FIX.4.3) Multileg instrument's individual security's NoSecurityAltID.

See NoSecurityAltID (454) field for description*/
LegSecurityAltID                                   = 605, /*!< 605 (String FIX.4.3) Multileg instrument's individual security's SecurityAltID.

See SecurityAltID (455) field for description*/
LegSecurityAltIDSource                             = 606, /*!< 606 (String FIX.4.3) Multileg instrument's individual security's SecurityAltIDSource.

See SecurityAltIDSource (456) field for description*/
LegProduct                                         = 607, /*!< 607 (int FIX.4.3) Multileg instrument's individual security's Product.

See Product (460) field for description*/
LegCFICode                                         = 608, /*!< 608 (String FIX.4.3) Multileg instrument's individual security's CFICode.

See CFICode (461) field for description*/
LegSecurityType                                    = 609, /*!< 609 (String FIX.4.3) Refer to definition of SecurityType(167)*/
LegMaturityMonthYear                               = 610, /*!< 610 (MonthYear FIX.4.3) Multileg instrument's individual security's MaturityMonthYear.

See MaturityMonthYear (200) field for description*/
LegMaturityDate                                    = 611, /*!< 611 (LocalMktDate FIX.4.3) Multileg instrument's individual security's MaturityDate.

See MaturityDate (54) field for description*/
LegStrikePrice                                     = 612, /*!< 612 (Price FIX.4.3) Multileg instrument's individual security's StrikePrice.

See StrikePrice (202) field for description*/
LegOptAttribute                                    = 613, /*!< 613 (char FIX.4.3) Multileg instrument's individual security's OptAttribute.

See OptAttribute (206) field for description*/
LegContractMultiplier                              = 614, /*!< 614 (float FIX.4.3) Multileg instrument's individual security's ContractMultiplier.

See ContractMultiplier (23) field for description*/
LegCouponRate                                      = 615, /*!< 615 (Percentage FIX.4.3) Multileg instrument's individual security's CouponRate.

See CouponRate (223) field for description*/
LegSecurityExchange                                = 616, /*!< 616 (Exchange FIX.4.3) Multileg instrument's individual security's SecurityExchange.

See SecurityExchange (207) field for description*/
LegIssuer                                          = 617, /*!< 617 (String FIX.4.3) Multileg instrument's individual security's Issuer.

See Issuer (106) field for description*/
EncodedLegIssuerLen                                = 618, /*!< 618 (Length FIX.4.3) Multileg instrument's individual security's EncodedIssuerLen.

See EncodedIssuerLen (348) field for description*/
EncodedLegIssuer                                   = 619, /*!< 619 (data FIX.4.3) Multileg instrument's individual security's EncodedIssuer.

See EncodedIssuer (349) field for description*/
LegSecurityDesc                                    = 620, /*!< 620 (String FIX.4.3) Description of a multileg instrument.

Can be used by the venue or one of the trading parties to provide an optional non-normative textual description of the financial instrument.

*/
EncodedLegSecurityDescLen                          = 621, /*!< 621 (Length FIX.4.3) Multileg instrument's individual security's EncodedSecurityDescLen.

See EncodedSecurityDescLen (350) field for description*/
EncodedLegSecurityDesc                             = 622, /*!< 622 (data FIX.4.3) Multileg instrument's individual security's EncodedSecurityDesc.

See EncodedSecurityDesc (35) field for description*/
LegRatioQty                                        = 623, /*!< 623 (float FIX.4.3) The ratio of quantity for this individual leg relative to the entire multileg security.*/
LegSide                                            = 624, /*!< 624 (char FIX.4.3) The side of this individual leg (multileg security).

See Side (54) field for description and values*/
TradingSessionSubID                                = 625, /*!< 625 (String FIX.4.3) Optional market assigned sub identifier for a trading phase within a trading session. Usage is determined by market or counterparties. Used by US based futures markets to identify exchange specific execution time bracket codes as required by US market regulations. Bilaterally agreed values of data type "String" that start with a character can be used for backward compatibility*/
AllocType                                          = 626, /*!< 626 (int FIX.4.3) Describes the specific type or purpose of an Allocation message (i.e. "Buyside Calculated")

(see Volume : "Glossary" for value definitions)

*** SOME VALUES HAVE BEEN REPLACED - See "Replaced Features and Supported Approach" ****/
NoHops                                             = 627, /*!< 627 (NumInGroup FIX.4.3) Number of HopCompID entries in repeating group.*/
HopCompID                                          = 628, /*!< 628 (String FIX.4.3) Assigned value used to identify the third party firm which delivered a specific message either from the firm which originated the message or from another third party (if multiple "hops" are performed). It is recommended that this value be the SenderCompID (49) of the third party.

Applicable when messages are communicated/re-distributed via third parties which function as service bureaus or "hubs". Only applicable if OnBehalfOfCompID (115) is being used.*/
HopSendingTime                                     = 629, /*!< 629 (UTCTimestamp FIX.4.3) Time that HopCompID (628) sent the message. It is recommended that this value be the SendingTime (52) of the message sent by the third party.

Applicable when messages are communicated/re-distributed via third parties which function as service bureaus or "hubs". Only applicable if OnBehalfOfCompID (115) is being used.*/
HopRefID                                           = 630, /*!< 630 (SeqNum FIX.4.3) Reference identifier assigned by HopCompID (628) associated with the message sent. It is recommended that this value be the MsgSeqNum (34) of the message sent by the third party.

Applicable when messages are communicated/re-distributed via third parties which function as service bureaus or "hubs". Only applicable if OnBehalfOfCompID (115) is being used.*/
MidPx                                              = 631, /*!< 631 (Price FIX.4.3) Mid price/rate. 

For OTC swaps this is the mid-market mark (for example, as defined by CFTC).

For uncleared OTC swaps, LegMidPx(2346) and the MidPx(631) fields are mutually exclusive.*/
BidYield                                           = 632, /*!< 632 (Percentage FIX.4.3) Bid yield*/
MidYield                                           = 633, /*!< 633 (Percentage FIX.4.3) Mid yield*/
OfferYield                                         = 634, /*!< 634 (Percentage FIX.4.3) Offer yield*/
ClearingFeeIndicator                               = 635, /*!< 635 (String FIX.4.3) Indicates type of fee being assessed of the customer for trade executions at an exchange. Applicable for futures markets only at this time.

(Values source CBOT, CME, NYBOT, and NYMEX):*/
WorkingIndicator                                   = 636, /*!< 636 (Boolean FIX.4.3) Indicates if the order is currently being worked. Applicable only for OrdStatus = "New". For open outcry markets this indicates that the order is being worked in the crowd. For electronic markets it indicates that the order has transitioned from a contingent order to a market order.*/
LegLastPx                                          = 637, /*!< 637 (Price FIX.4.3) Execution price assigned to a leg of a multileg instrument.

See LastPx (31) field for description and values*/
PriorityIndicator                                  = 638, /*!< 638 (int FIX.4.3) Indicates if a Cancel/Replace has caused an order to lose book priority.*/
PriceImprovement                                   = 639, /*!< 639 (PriceOffset FIX.4.3) Amount of price improvement.*/
Price2                                             = 640, /*!< 640 (Price FIX.4.3) Price of the future part of a F/X swap order.

See Price (44) for description.*/
LastForwardPoints2                                 = 641, /*!< 641 (PriceOffset FIX.4.3) F/X forward points of the future part of a F/X swap order added to LastSpotRate (94). May be a negative value.*/
BidForwardPoints2                                  = 642, /*!< 642 (PriceOffset FIX.4.3) Bid F/X forward points of the future portion of a F/X swap quote added to spot rate. May be a negative value.*/
OfferForwardPoints2                                = 643, /*!< 643 (PriceOffset FIX.4.3) Offer F/X forward points of the future portion of a F/X swap quote added to spot rate. May be a negative value.*/
RFQReqID                                           = 644, /*!< 644 (String FIX.4.3) RFQ Request ID - used to identify an RFQ Request.*/
MktBidPx                                           = 645, /*!< 645 (Price FIX.4.3) Used to indicate the best bid in a market*/
MktOfferPx                                         = 646, /*!< 646 (Price FIX.4.3) Used to indicate the best offer in a market*/
MinBidSize                                         = 647, /*!< 647 (Qty FIX.4.3) Used to indicate a minimum quantity for a bid.*/
MinOfferSize                                       = 648, /*!< 648 (Qty FIX.4.3) Used to indicate a minimum quantity for an offer. If this field is used the OfferSize (135) field is interpreted as the maximum offer size.*/
QuoteStatusReqID                                   = 649, /*!< 649 (String FIX.4.3) Unique identifier for Quote Status Request.*/
LegalConfirm                                       = 650, /*!< 650 (Boolean FIX.4.3) Indicates that this message is to serve as the final and legal confirmation.*/
UnderlyingLastPx                                   = 651, /*!< 651 (Price FIX.4.3) The calculated or traded price for the underlying instrument that corresponds to a derivative. Used for transactions that include the cash instrument and the derivative.*/
UnderlyingLastQty                                  = 652, /*!< 652 (Qty FIX.4.3) The calculated or traded quantity for the underlying instrument that corresponds to a derivative. Used for transactions that include the cash instrument and the derivative.*/
LegRefID                                           = 654, /*!< 654 (String FIX.4.3) Unique identifier for a specific leg (uniqueness not defined as part of the FIX specification). LegRefID(654) be used to reference the value from LegID(1788).*/
ContraLegRefID                                     = 655, /*!< 655 (String FIX.4.3) Unique indicator for a specific leg for the ContraBroker (375).*/
SettlCurrBidFxRate                                 = 656, /*!< 656 (float FIX.4.3) Foreign exchange rate used to compute the bid "SettlCurrAmt" (119) from Currency (15) to SettlCurrency (120)*/
SettlCurrOfferFxRate                               = 657, /*!< 657 (float FIX.4.3) Foreign exchange rate used to compute the offer "SettlCurrAmt" (119) from Currency (15) to SettlCurrency (120)*/
QuoteRequestRejectReason                           = 658, /*!< 658 (int FIX.4.3) Reason Quote was rejected:*/
SideComplianceID                                   = 659, /*!< 659 (String FIX.4.3) ID within repeating group of sides which is used to represent this transaction for compliance purposes (e.g. OATS reporting).*/
AcctIDSource                                       = 660, /*!< 660 (int FIX.4.4) Used to identify the source of the Account (1) code. This is especially useful if the account is a new account that the Respondent may not have setup yet in their system.*/
AllocAcctIDSource                                  = 661, /*!< 661 (int FIX.4.4) Used to identify the source of the AllocAccount (79) code.

See AcctIDSource (660) for valid values.*/
BenchmarkPrice                                     = 662, /*!< 662 (Price FIX.4.4) Specifies the price of the benchmark.*/
BenchmarkPriceType                                 = 663, /*!< 663 (int FIX.4.4) Identifies type of BenchmarkPrice (662).

See PriceType (423) for valid values.*/
ConfirmID                                          = 664, /*!< 664 (String FIX.4.4) Message reference for Confirmation*/
ConfirmStatus                                      = 665, /*!< 665 (int FIX.4.4) Identifies the status of the Confirmation.*/
ConfirmTransType                                   = 666, /*!< 666 (int FIX.4.4) Identifies the Confirmation transaction type.*/
ContractSettlMonth                                 = 667, /*!< 667 (MonthYear FIX.4.4) Specifies when the contract (i.e. MBS/TBA) will settle.*/
DeliveryForm                                       = 668, /*!< 668 (int FIX.4.4) Identifies the form of delivery.*/
LastParPx                                          = 669, /*!< 669 (Price FIX.4.4) Last price expressed in percent-of-par. Conditionally required for Fixed Income trades when LastPx (31) is expressed in Yield, Spread, Discount or any other type.

Usage: Execution Report and Allocation Report repeating executions block (from sellside).*/
NoLegAllocs                                        = 670, /*!< 670 (NumInGroup FIX.4.4) Number of Allocations for the leg*/
LegAllocAccount                                    = 671, /*!< 671 (String FIX.4.4) Allocation Account for the leg

See AllocAccount (79) for description and valid values.*/
LegIndividualAllocID                               = 672, /*!< 672 (String FIX.4.4) Reference for the individual allocation ticket

See IndividualAllocID (467) for description and valid values.*/
LegAllocQty                                        = 673, /*!< 673 (Qty FIX.4.4) Leg allocation quantity.

See AllocQty (80) for description and valid values.*/
LegAllocAcctIDSource                               = 674, /*!< 674 (String FIX.4.4) The source of the LegAllocAccount (671)

See AllocAcctIDSource (661) for description and valid values.*/
LegSettlCurrency                                   = 675, /*!< 675 (Currency FIX.4.4) Identifies settlement currency for the Leg.

See SettlCurrency (20) for description and valid values*/
LegBenchmarkCurveCurrency                          = 676, /*!< 676 (Currency FIX.4.4) LegBenchmarkPrice (679) currency

See BenchmarkCurveCurrency (220) for description and valid values.*/
LegBenchmarkCurveName                              = 677, /*!< 677 (String FIX.4.4) Name of the Leg Benchmark Curve.

See BenchmarkCurveName (22) for description and valid values.*/
LegBenchmarkCurvePoint                             = 678, /*!< 678 (String FIX.4.4) Identifies the point on the Leg Benchmark Curve.

See BenchmarkCurvePoint (222) for description and valid values.*/
LegBenchmarkPrice                                  = 679, /*!< 679 (Price FIX.4.4) Used to identify the price of the benchmark security.

See BenchmarkPrice (662) for description and valid values.*/
LegBenchmarkPriceType                              = 680, /*!< 680 (int FIX.4.4) The price type of the LegBenchmarkPrice(679).*/
LegBidPx                                           = 681, /*!< 681 (Price FIX.4.4) Bid price of this leg.

See BidPx (32) for description and valid values.*/
LegIOIQty                                          = 682, /*!< 682 (String FIX.4.4) Leg-specific IOI quantity.

See IOIQty (27) for description and valid values*/
NoLegStipulations                                  = 683, /*!< 683 (NumInGroup FIX.4.4) Number of leg stipulation entries*/
LegOfferPx                                         = 684, /*!< 684 (Price FIX.4.4) Offer price of this leg.

See OfferPx (133) for description and valid values*/
LegOrderQty                                        = 685, /*!< 685 (Qty FIX.4.4) Quantity ordered of this leg.

See OrderQty (38) for description and valid values*/
LegPriceType                                       = 686, /*!< 686 (int FIX.4.4) The price type of the LegBidPx (681) and/or LegOfferPx (684).

See PriceType (423) for description and valid values*/
LegQty                                             = 687, /*!< 687 (Qty FIX.4.4) This field is deprecated and has been replaced by LegOrderQty(865). This field will likely be removed from the FIX standard in a future version.*/
LegStipulationType                                 = 688, /*!< 688 (String FIX.4.4) For Fixed Income, type of Stipulation for this leg.

See StipulationType (233) for description and valid values*/
LegStipulationValue                                = 689, /*!< 689 (String FIX.4.4) For Fixed Income, value of stipulation.

See StipulationValue (234) for description and valid values*/
LegSwapType                                        = 690, /*!< 690 (int FIX.4.4) For Fixed Income, used instead of LegOrderQty(685) to requests the respondent to calculate the quantity based on the quantity on the opposite side of the swap.*/
Pool                                               = 691, /*!< 691 (String FIX.4.4) For Fixed Income, identifies MBS / ABS pool.*/
QuotePriceType                                     = 692, /*!< 692 (int FIX.4.4) Code to represent price type requested in Quote.

If the Quote Request is for a Swap, values 1-8 apply to all legs.*/
QuoteRespID                                        = 693, /*!< 693 (String FIX.4.4) Message reference for Quote Response*/
QuoteRespType                                      = 694, /*!< 694 (int FIX.4.4) Identifies the type of Quote Response.*/
QuoteQualifier                                     = 695, /*!< 695 (char FIX.4.4) Code to qualify Quote use and other aspects of price negotiation.*/
YieldRedemptionDate                                = 696, /*!< 696 (LocalMktDate FIX.4.4) Date to which the yield has been calculated (i.e. maturity, par call or current call, pre-refunded date).*/
YieldRedemptionPrice                               = 697, /*!< 697 (Price FIX.4.4) Price to which the yield has been calculated.*/
YieldRedemptionPriceType                           = 698, /*!< 698 (int FIX.4.4) The price type of the YieldRedemptionPrice (697)

See PriceType (423) for description and valid values.*/
BenchmarkSecurityID                                = 699, /*!< 699 (String FIX.4.4) The identifier of the benchmark security, e.g. Treasury against Corporate bond.

See SecurityID (tag 48) for description and valid values.*/
ReversalIndicator                                  = 700, /*!< 700 (Boolean FIX.4.4) Indicates a trade that reverses a previous trade.*/
YieldCalcDate                                      = 701, /*!< 701 (LocalMktDate FIX.4.4) Include as needed to clarify yield irregularities associated with date, e.g. when it falls on a non-business day.*/
NoPositions                                        = 702, /*!< 702 (NumInGroup FIX.4.4) Number of position entries.*/
PosType                                            = 703, /*!< 703 (String FIX.4.4) Used to identify the type of quantity that is being returned.*/
LongQty                                            = 704, /*!< 704 (Qty FIX.4.4) Long quantity.*/
ShortQty                                           = 705, /*!< 705 (Qty FIX.4.4) Short quantity.*/
PosQtyStatus                                       = 706, /*!< 706 (int FIX.4.4) Status of this position.*/
PosAmtType                                         = 707, /*!< 707 (String FIX.4.4) Type of Position amount*/
PosAmt                                             = 708, /*!< 708 (Amt FIX.4.4) Position amount*/
PosTransType                                       = 709, /*!< 709 (int FIX.4.4) Identifies the type of position transaction.*/
PosReqID                                           = 710, /*!< 710 (String FIX.4.4) Unique identifier for the position maintenance request as assigned by the submitter*/
NoUnderlyings                                      = 711, /*!< 711 (NumInGroup FIX.4.4) Number of underlying legs that make up the security.*/
PosMaintAction                                     = 712, /*!< 712 (int FIX.4.4) Maintenance Action to be performed.*/
OrigPosReqRefID                                    = 713, /*!< 713 (String FIX.4.4) Reference to the PosReqID (710) of a previous maintenance request that is being replaced or canceled.*/
PosMaintRptRefID                                   = 714, /*!< 714 (String FIX.4.4) Reference to a PosMaintRptID (721) from a previous Position Maintenance Report that is being replaced or canceled.*/
ClearingBusinessDate                               = 715, /*!< 715 (LocalMktDate FIX.4.4) The business date for which the trade is expected to be cleared.*/
SettlSessID                                        = 716, /*!< 716 (String FIX.4.4) Identifies a specific settlement session*/
SettlSessSubID                                     = 717, /*!< 717 (String FIX.4.4) SubID value associated with SettlSessID(716)*/
AdjustmentType                                     = 718, /*!< 718 (int FIX.4.4) Type of adjustment to be applied. Used for Position Change Submission (PCS), Position Adjustment (PAJ), and Customer Gross Margin (CGM).*/
ContraryInstructionIndicator                       = 719, /*!< 719 (Boolean FIX.4.4) Used to indicate when a contrary instruction for exercise or abandonment is being submitted*/
PriorSpreadIndicator                               = 720, /*!< 720 (Boolean FIX.4.4) Indicates if requesting a rollover of prior day's spread submissions.*/
PosMaintRptID                                      = 721, /*!< 721 (String FIX.4.4) Unique identifier for this position report*/
PosMaintStatus                                     = 722, /*!< 722 (int FIX.4.4) Status of Position Maintenance Request*/
PosMaintResult                                     = 723, /*!< 723 (int FIX.4.4) Result of Position Maintenance Request.*/
PosReqType                                         = 724, /*!< 724 (int FIX.4.4) Used to specify the type of position request being made.*/
ResponseTransportType                              = 725, /*!< 725 (int FIX.4.4) Identifies how the response to the request should be transmitted.

Details specified via ResponseDestination (726).*/
ResponseDestination                                = 726, /*!< 726 (String FIX.4.4) URI (Uniform Resource Identifier) for details) or other pre-arranged value. Used in conjunction with ResponseTransportType (725) value of Out-of-Band to identify the out-of-band destination.

See "Appendix 6-B FIX Fields Based Upon Other Standards"*/
TotalNumPosReports                                 = 727, /*!< 727 (int FIX.4.4) Total number of Position Reports being returned.*/
PosReqResult                                       = 728, /*!< 728 (int FIX.4.4) Result of Request for Positions.*/
PosReqStatus                                       = 729, /*!< 729 (int FIX.4.4) Status of Request for Positions*/
SettlPrice                                         = 730, /*!< 730 (Price FIX.4.4) Settlement price*/
SettlPriceType                                     = 731, /*!< 731 (int FIX.4.4) Type of settlement price*/
UnderlyingSettlPrice                               = 732, /*!< 732 (Price FIX.4.4) Underlying security's SettlPrice.

See SettlPrice (730) field for description*/
UnderlyingSettlPriceType                           = 733, /*!< 733 (int FIX.4.4) Underlying security's SettlPriceType.

See SettlPriceType (731) field for description*/
PriorSettlPrice                                    = 734, /*!< 734 (Price FIX.4.4) Previous settlement price*/
NoQuoteQualifiers                                  = 735, /*!< 735 (NumInGroup FIX.4.4) Number of repeating groups of QuoteQualifiers (695).*/
AllocSettlCurrency                                 = 736, /*!< 736 (Currency FIX.4.4) Currency code of settlement denomination for a specific AllocAccount (79).*/
AllocSettlCurrAmt                                  = 737, /*!< 737 (Amt FIX.4.4) Total amount due expressed in settlement currency (includes the effect of the forex transaction) for a specific AllocAccount (79).*/
InterestAtMaturity                                 = 738, /*!< 738 (Amt FIX.4.4) Amount of interest (i.e. lump-sum) at maturity.*/
LegDatedDate                                       = 739, /*!< 739 (LocalMktDate FIX.4.4) The effective date of a new securities issue determined by its underwriters. Often but not always the same as the Issue Date and the Interest Accrual Date*/
LegPool                                            = 740, /*!< 740 (String FIX.4.4) For Fixed Income, identifies MBS / ABS pool for a specific leg of a multi-leg instrument.

See Pool (691) for description and valid values.*/
AllocInterestAtMaturity                            = 741, /*!< 741 (Amt FIX.4.4) Amount of interest (i.e. lump-sum) at maturity at the account-level.*/
AllocAccruedInterestAmt                            = 742, /*!< 742 (Amt FIX.4.4) Amount of Accrued Interest for convertible bonds and fixed income at the allocation-level.*/
DeliveryDate                                       = 743, /*!< 743 (LocalMktDate FIX.4.4) Date of delivery.*/
AssignmentMethod                                   = 744, /*!< 744 (char FIX.4.4) Method by which short positions are assigned to an exercise notice during exercise and assignment processing*/
AssignmentUnit                                     = 745, /*!< 745 (Qty FIX.4.4) Quantity Increment used in performing assignment.*/
OpenInterest                                       = 746, /*!< 746 (Amt FIX.4.4) Open interest that was eligible for assignment.*/
ExerciseMethod                                     = 747, /*!< 747 (char FIX.4.4) Exercise Method used to in performing assignment.*/
TotNumTradeReports                                 = 748, /*!< 748 (int FIX.4.4) Total number of trade reports returned.*/
TradeRequestResult                                 = 749, /*!< 749 (int FIX.4.4) Result of Trade Request*/
TradeRequestStatus                                 = 750, /*!< 750 (int FIX.4.4) Status of Trade Request.*/
TradeReportRejectReason                            = 751, /*!< 751 (int FIX.4.4) Reason Trade Capture Request was rejected.

100+ Reserved and available for bi-laterally agreed upon user-defined values.*/
SideMultiLegReportingType                          = 752, /*!< 752 (int FIX.4.4) Used to indicate if the side being reported on Trade Capture Report represents a leg of a multileg instrument or a single security.*/
NoPosAmt                                           = 753, /*!< 753 (NumInGroup FIX.4.4) Number of position amount entries.*/
AutoAcceptIndicator                                = 754, /*!< 754 (Boolean FIX.4.4) Identifies whether or not an allocation has been automatically accepted on behalf of the Carry Firm by the Clearing House.*/
AllocReportID                                      = 755, /*!< 755 (String FIX.4.4) Unique identifier for Allocation Report message.*/
NoNested2PartyIDs                                  = 756, /*!< 756 (NumInGroup FIX.4.4) Number of Nested2PartyID (757), Nested2PartyIDSource (758), and Nested2PartyRole (759) entries*/
Nested2PartyID                                     = 757, /*!< 757 (String FIX.4.4) PartyID value within a "second instance" Nested repeating group.

Same values as PartyID (448)*/
Nested2PartyIDSource                               = 758, /*!< 758 (char FIX.4.4) PartyIDSource value within a "second instance" Nested repeating group.

Same values as PartyIDSource (447)*/
Nested2PartyRole                                   = 759, /*!< 759 (int FIX.4.4) PartyRole value within a "second instance" Nested repeating group.

Same values as PartyRole (452)*/
Nested2PartySubID                                  = 760, /*!< 760 (String FIX.4.4) PartySubID value within a "second instance" Nested repeating group.

Same values as PartySubID (523)*/
BenchmarkSecurityIDSource                          = 761, /*!< 761 (String FIX.4.4) Identifies class or source of the BenchmarkSecurityID (699) value. Required if BenchmarkSecurityID is specified.

Same values as the SecurityIDSource (22) field*/
SecuritySubType                                    = 762, /*!< 762 (String FIX.4.4) Sub-type qualification/identification of the SecurityType. As an example for SecurityType(167)="REPO", the SecuritySubType="General Collateral" can be used to further specify the type of REPO.

If SecuritySubType is used, then SecurityType is required.

For SecurityType="MLEG" a name of the option or futures strategy name can be specified, such as "Calendar", "Vertical", "Butterfly".

For SecurityType(167)="OPT" the subclassification can be specified, such as "Asian".

For SecurityType(167)="SWAPTION" a value of "Straddle" is used to identify a straddle swaption.*/
UnderlyingSecuritySubType                          = 763, /*!< 763 (String FIX.4.4) Underlying security's SecuritySubType.

See SecuritySubType (762) field for description*/
LegSecuritySubType                                 = 764, /*!< 764 (String FIX.4.4) SecuritySubType of the leg instrument.

See SecuritySubType (762) field for description*/
AllowableOneSidednessPct                           = 765, /*!< 765 (Percentage FIX.4.4) The maximum percentage that execution of one side of a program trade can exceed execution of the other.*/
AllowableOneSidednessValue                         = 766, /*!< 766 (Amt FIX.4.4) The maximum amount that execution of one side of a program trade can exceed execution of the other.*/
AllowableOneSidednessCurr                          = 767, /*!< 767 (Currency FIX.4.4) The currency that AllowableOneSidednessValue (766) is expressed in if AllowableOneSidednessValue is used.*/
NoTrdRegTimestamps                                 = 768, /*!< 768 (NumInGroup FIX.4.4) Number of TrdRegTimestamp (769) entries*/
TrdRegTimestamp                                    = 769, /*!< 769 (UTCTimestamp FIX.4.4) Traded / Regulatory timestamp value. Use to store time information required by government regulators or self regulatory organizations (such as an exchange or clearing house).*/
TrdRegTimestampType                                = 770, /*!< 770 (int FIX.4.4) Trading / Regulatory timestamp type.

Note of applicability: Values are required in US futures markets by the CFTC to support computerized trade reconstruction, and required by MiFID II / MiFIR for transaction reporting and publication.

(see Volume : "Glossary" for value definitions)

*/
TrdRegTimestampOrigin                              = 771, /*!< 771 (String FIX.4.4) Text which identifies the "origin" (i.e. system which was used to generate the time stamp) for the Traded / Regulatory timestamp value.*/
ConfirmRefID                                       = 772, /*!< 772 (String FIX.4.4) Reference identifier to be used with ConfirmTransType (666) = Replace or Cancel*/
ConfirmType                                        = 773, /*!< 773 (int FIX.4.4) Identifies the type of Confirmation message being sent.*/
ConfirmRejReason                                   = 774, /*!< 774 (int FIX.4.4) Identifies the reason for rejecting a Confirmation.*/
BookingType                                        = 775, /*!< 775 (int FIX.4.4) Method for booking out this order. Used when notifying a broker that an order to be settled by that broker is to be booked out as an OTC derivative (e.g. CFD or similar).*/
IndividualAllocRejCode                             = 776, /*!< 776 (int FIX.4.4) Identified reason for rejecting an individual AllocAccount (79) detail.

Same values as AllocRejCode (88)*/
SettlInstMsgID                                     = 777, /*!< 777 (String FIX.4.4) Unique identifier for Settlement Instruction message.*/
NoSettlInst                                        = 778, /*!< 778 (NumInGroup FIX.4.4) Number of settlement instructions within repeating group.*/
LastUpdateTime                                     = 779, /*!< 779 (UTCTimestamp FIX.4.4) Timestamp of last update to data item (or creation if no updates made since creation).*/
AllocSettlInstType                                 = 780, /*!< 780 (int FIX.4.4) Used to indicate whether settlement instructions are provided on an allocation instruction message, and if not, how they are to be derived.*/
NoSettlPartyIDs                                    = 781, /*!< 781 (NumInGroup FIX.4.4) Number of SettlPartyID (782), SettlPartyIDSource (783), and SettlPartyRole (784) entries*/
SettlPartyID                                       = 782, /*!< 782 (String FIX.4.4) PartyID value within a settlement parties component. Nested repeating group.

Same values as PartyID (448)*/
SettlPartyIDSource                                 = 783, /*!< 783 (char FIX.4.4) PartyIDSource value within a settlement parties component.

Same values as PartyIDSource (447)*/
SettlPartyRole                                     = 784, /*!< 784 (int FIX.4.4) PartyRole value within a settlement parties component.

Same values as PartyRole (452)*/
SettlPartySubID                                    = 785, /*!< 785 (String FIX.4.4) PartySubID value within a settlement parties component.

Same values as PartySubID (523)*/
SettlPartySubIDType                                = 786, /*!< 786 (int FIX.4.4) Type of SettlPartySubID (785) value.

Same values as PartySubIDType (803)*/
DlvyInstType                                       = 787, /*!< 787 (char FIX.4.4) Used to indicate whether a delivery instruction is used for securities or cash settlement.*/
TerminationType                                    = 788, /*!< 788 (int FIX.4.4) Type of financing termination.*/
NextExpectedMsgSeqNum                              = 789, /*!< 789 (SeqNum FIX.4.4) Next expected MsgSeqNum value to be received.*/
OrdStatusReqID                                     = 790, /*!< 790 (String FIX.4.4) Can be used to uniquely identify a specific Order Status Request message.*/
SettlInstReqID                                     = 791, /*!< 791 (String FIX.4.4) Unique ID of settlement instruction request message*/
SettlInstReqRejCode                                = 792, /*!< 792 (int FIX.4.4) Identifies reason for rejection (of a settlement instruction request message).*/
SecondaryAllocID                                   = 793, /*!< 793 (String FIX.4.4) Secondary allocation identifier. Unlike the AllocID (70), this can be shared across a number of allocation instruction or allocation report messages, thereby making it possible to pass an identifier for an original allocation message on multiple messages (e.g. from one party to a second to a third, across cancel and replace messages etc.).*/
AllocReportType                                    = 794, /*!< 794 (int FIX.4.4) Describes the specific type or purpose of an Allocation Report message*/
AllocReportRefID                                   = 795, /*!< 795 (String FIX.4.4) Reference identifier to be used with AllocTransType (7) = Replace or Cancel*/
AllocCancReplaceReason                             = 796, /*!< 796 (int FIX.4.4) Reason for cancelling or replacing an Allocation Instruction or Allocation Report message*/
CopyMsgIndicator                                   = 797, /*!< 797 (Boolean FIX.4.4) Indicates whether or not this message is a drop copy of another message.*/
AllocAccountType                                   = 798, /*!< 798 (int FIX.4.4) Type of account associated with a confirmation or other trade-level message*/
OrderAvgPx                                         = 799, /*!< 799 (Price FIX.4.4) Average price for a specific order*/
OrderBookingQty                                    = 800, /*!< 800 (Qty FIX.4.4) Quantity of the order that is being booked out as part of an Allocation Instruction or Allocation Report message*/
NoSettlPartySubIDs                                 = 801, /*!< 801 (NumInGroup FIX.4.4) Number of SettlPartySubID (785) and SettlPartySubIDType (786) entries*/
NoPartySubIDs                                      = 802, /*!< 802 (NumInGroup FIX.4.4) Number of PartySubID (523)and PartySubIDType (803) entries*/
PartySubIDType                                     = 803, /*!< 803 (int FIX.4.4) Type of PartySubID(523) value.*/
NoNestedPartySubIDs                                = 804, /*!< 804 (NumInGroup FIX.4.4) Number of NestedPartySubID (545) and NestedPartySubIDType (805) entries*/
NestedPartySubIDType                               = 805, /*!< 805 (int FIX.4.4) Type of NestedPartySubID (545) value.

Same values as PartySubIDType (803)*/
NoNested2PartySubIDs                               = 806, /*!< 806 (NumInGroup FIX.4.4) Number of Nested2PartySubID (760) and Nested2PartySubIDType (807) entries. Second instance of &lt;NestedParties&gt;.*/
Nested2PartySubIDType                              = 807, /*!< 807 (int FIX.4.4) Type of Nested2PartySubID (760) value. Second instance of &lt;NestedParties&gt;.

Same values as PartySubIDType (803)*/
AllocIntermedReqType                               = 808, /*!< 808 (int FIX.4.4) Response to allocation to be communicated to a counterparty through an intermediary, i.e. clearing house. Used in conjunction with AllocType = "Request to Intermediary" and AllocReportType = "Request to Intermediary"*/
NoUsernames                                        = 809, /*!< 809 (NumInGroup FIX.4.4) Number of Usernames to which this this response is directed*/
UnderlyingPx                                       = 810, /*!< 810 (Price FIX.4.4) Underlying price associate with a derivative instrument.*/
PriceDelta                                         = 811, /*!< 811 (float FIX.4.4) The rate of change in the price of a derivative with respect to the movement in the price of the underlying instrument(s) upon which the derivative instrument price is based.

This value is normally between -1.0 and 1.0.*/
ApplQueueMax                                       = 812, /*!< 812 (int FIX.4.4) Used to specify the maximum number of application messages that can be queued bedore a corrective action needs to take place to resolve the queuing issue.*/
ApplQueueDepth                                     = 813, /*!< 813 (int FIX.4.4) Current number of application messages that were queued at the time that the message was created by the counterparty.*/
ApplQueueResolution                                = 814, /*!< 814 (int FIX.4.4) Resolution taken when ApplQueueDepth (813) exceeds ApplQueueMax (812) or system specified maximum queue size.*/
ApplQueueAction                                    = 815, /*!< 815 (int FIX.4.4) Action to take to resolve an application message queue (backlog).*/
NoAltMDSource                                      = 816, /*!< 816 (NumInGroup FIX.4.4) Number of alternative market data sources*/
AltMDSourceID                                      = 817, /*!< 817 (String FIX.4.4) Session layer source for market data

(For the standard FIX session layer, this would be the TargetCompID (56) where market data can be obtained).*/
SecondaryTradeReportID                             = 818, /*!< 818 (String FIX.4.4) Secondary trade report identifier - can be used to associate an additional identifier with a trade.*/
AvgPxIndicator                                     = 819, /*!< 819 (int FIX.4.4) Average pricing indicator.*/
TradeLinkID                                        = 820, /*!< 820 (String FIX.4.4) Used to link a group of trades together.*/
OrderInputDevice                                   = 821, /*!< 821 (String FIX.4.4) Specific device number, terminal number or station where order was entered*/
UnderlyingTradingSessionID                         = 822, /*!< 822 (String FIX.4.4) Trading Session in which the underlying instrument trades*/
UnderlyingTradingSessionSubID                      = 823, /*!< 823 (String FIX.4.4) Trading Session sub identifier in which the underlying instrument trades*/
TradeLegRefID                                      = 824, /*!< 824 (String FIX.4.4) Reference to the leg of a multileg instrument to which this trade refers*/
ExchangeRule                                       = 825, /*!< 825 (String FIX.4.4) Used to report any exchange rules that apply to this trade.

Primarily intended for US futures markets. Certain trading practices are permitted by the CFTC, such as large lot trading, block trading, all or none trades. If the rules are used, the exchanges are required to indicate these rules on the trade.*/
TradeAllocIndicator                                = 826, /*!< 826 (int FIX.4.4) Identifies if, and how, the trade is to be allocated or split.*/
ExpirationCycle                                    = 827, /*!< 827 (int FIX.4.4) Part of trading cycle when an instrument expires. Field is applicable for derivatives.*/
TrdType                                            = 828, /*!< 828 (int FIX.4.4) Type of trade.*/
TrdSubType                                         = 829, /*!< 829 (int FIX.4.4) Further qualification to the trade type*/
TransferReason                                     = 830, /*!< 830 (String FIX.4.4) Reason trade is being transferred*/
TotNumAssignmentReports                            = 832, /*!< 832 (int FIX.4.4) Total Number of Assignment Reports being returned to a firm*/
AsgnRptID                                          = 833, /*!< 833 (String FIX.4.4) Unique identifier for the Assignment Report*/
ThresholdAmount                                    = 834, /*!< 834 (PriceOffset FIX.4.4) Amount that a position has to be in the money before it is exercised.*/
PegMoveType                                        = 835, /*!< 835 (int FIX.4.4) Describes whether peg is static or floats*/
PegOffsetType                                      = 836, /*!< 836 (int FIX.4.4) Type of Peg Offset value*/
PegLimitType                                       = 837, /*!< 837 (int FIX.4.4) Type of Peg Limit*/
PegRoundDirection                                  = 838, /*!< 838 (int FIX.4.4) If the calculated peg price is not a valid tick price, specifies whether to round the price to be more or less aggressive*/
PeggedPrice                                        = 839, /*!< 839 (Price FIX.4.4) The price the order is currently pegged at*/
PegScope                                           = 840, /*!< 840 (int FIX.4.4) The scope of the peg*/
DiscretionMoveType                                 = 841, /*!< 841 (int FIX.4.4) Describes whether discretionay price is static or floats*/
DiscretionOffsetType                               = 842, /*!< 842 (int FIX.4.4) Type of Discretion Offset value*/
DiscretionLimitType                                = 843, /*!< 843 (int FIX.4.4) Type of Discretion Limit*/
DiscretionRoundDirection                           = 844, /*!< 844 (int FIX.4.4) If the calculated discretionary price is not a valid tick price, specifies whether to round the price to be more or less aggressive*/
DiscretionPrice                                    = 845, /*!< 845 (Price FIX.4.4) The current discretionary price of the order*/
DiscretionScope                                    = 846, /*!< 846 (int FIX.4.4) The scope of the discretion*/
TargetStrategy                                     = 847, /*!< 847 (int FIX.4.4) The target strategy of the order

1000+ = Reserved and available for bi-laterally agreed upon user defined values*/
TargetStrategyParameters                           = 848, /*!< 848 (String FIX.4.4) Field to allow further specification of the TargetStrategy - usage to be agreed between counterparties*/
ParticipationRate                                  = 849, /*!< 849 (Percentage FIX.4.4) For a TargetStrategy=Participate order specifies the target particpation rate. For other order types this is a volume limit (i.e. do not be more than this percent of the market volume)*/
TargetStrategyPerformance                          = 850, /*!< 850 (float FIX.4.4) For communication of the performance of the order versus the target strategy*/
LastLiquidityInd                                   = 851, /*!< 851 (int FIX.4.4) Indicator to identify whether this fill was a result of a liquidity provider providing or liquidity taker taking the liquidity. */
PublishTrdIndicator                                = 852, /*!< 852 (Boolean FIX.4.4) Indicates if a trade should be reported via a market reporting service.*/
ShortSaleReason                                    = 853, /*!< 853 (int FIX.4.4) Reason for short sale.*/
QtyType                                            = 854, /*!< 854 (int FIX.4.4) Type of quantity specified in quantity field. ContractMultiplier (tag 231) is required when QtyType = 1 (Contracts). UnitOfMeasure (tag 996) and TimeUnit (tag 997) are required when QtyType =  2 (Units of Measure per Time Unit).*/
SecondaryTrdType                                   = 855, /*!< 855 (int FIX.4.4) Additional TrdType(828) assigned to a trade by trade match system.*/
TradeReportType                                    = 856, /*!< 856 (int FIX.4.4) Type of Trade Report*/
AllocNoOrdersType                                  = 857, /*!< 857 (int FIX.4.4) Indicates how the orders being booked and allocated by an AllocationInstruction or AllocationReport message are identified, e.g. by explicit definition in the OrdAllocGrp or ExecAllocGrp components, or not identified explicitly.*/
SharedCommission                                   = 858, /*!< 858 (Amt FIX.4.4) Commission to be shared with a third party, e.g. as part of a directed brokerage commission sharing arrangement.*/
ConfirmReqID                                       = 859, /*!< 859 (String FIX.4.4) Unique identifier for a Confirmation Request message*/
AvgParPx                                           = 860, /*!< 860 (Price FIX.4.4) Used to express average price as percent of par (used where AvgPx field is expressed in some other way)*/
ReportedPx                                         = 861, /*!< 861 (Price FIX.4.4) Reported price (used to differentiate from AvgPx on a confirmation of a marked-up or marked-down principal trade)*/
NoCapacities                                       = 862, /*!< 862 (NumInGroup FIX.4.4) Number of repeating OrderCapacity entries.*/
OrderCapacityQty                                   = 863, /*!< 863 (Qty FIX.4.4) Quantity executed under a specific OrderCapacity (e.g. quantity executed as agent, quantity executed as principal)*/
NoEvents                                           = 864, /*!< 864 (NumInGroup FIX.4.4) Number of repeating EventType entries.*/
EventType                                          = 865, /*!< 865 (int FIX.4.4) Code to represent the type of event*/
EventDate                                          = 866, /*!< 866 (LocalMktDate FIX.4.4) Date of event*/
EventPx                                            = 867, /*!< 867 (Price FIX.4.4) Predetermined price of issue at event, if applicable*/
EventText                                          = 868, /*!< 868 (String FIX.4.4) Comments related to the event.*/
PctAtRisk                                          = 869, /*!< 869 (Percentage FIX.4.4) Percent at risk due to lowest possible call.*/
NoInstrAttrib                                      = 870, /*!< 870 (NumInGroup FIX.4.4) Number of repeating InstrAttribType entries.*/
InstrAttribType                                    = 871, /*!< 871 (int FIX.4.4) Code to represent the type of instrument attribute*/
InstrAttribValue                                   = 872, /*!< 872 (String FIX.4.4) Attribute value appropriate to the InstrAttribType (87) field.*/
DatedDate                                          = 873, /*!< 873 (LocalMktDate FIX.4.4) The effective date of a new securities issue determined by its underwriters. Often but not always the same as the Issue Date and the Interest Accrual Date*/
InterestAccrualDate                                = 874, /*!< 874 (LocalMktDate FIX.4.4) The start date used for calculating accrued interest on debt instruments which are being sold between interest payment dates. Often but not always the same as the Issue Date and the Dated Date*/
CPProgram                                          = 875, /*!< 875 (int FIX.4.4) The program under which a commercial paper offering is exempt from SEC registration identified by the paragraph number(s) within the US Securities Act of 1933 or as identified below.*/
CPRegType                                          = 876, /*!< 876 (String FIX.4.4) The description of commercial paper registration or rule under which exempt commercial paper is offered. For example "144a", "Tax Exempt" or "REG. S".*/
UnderlyingCPProgram                                = 877, /*!< 877 (int FIX.4.4) The program under which the underlying commercial paper is issued*/
UnderlyingCPRegType                                = 878, /*!< 878 (String FIX.4.4) The registration type of the underlying commercial paper issuance*/
UnderlyingQty                                      = 879, /*!< 879 (Qty FIX.4.4) Unit amount of the underlying security (par, shares, currency, etc.)*/
TrdMatchID                                         = 880, /*!< 880 (String FIX.4.4) Identifier assigned to a trade by a matching system.*/
SecondaryTradeReportRefID                          = 881, /*!< 881 (String FIX.4.4) Used to refer to a previous SecondaryTradeReportRefID when amending the transaction (cancel, replace, release, or reversal).*/
UnderlyingDirtyPrice                               = 882, /*!< 882 (Price FIX.4.4) Price (percent-of-par or per unit) of the underlying security or basket. "Dirty" means it includes accrued interest*/
UnderlyingEndPrice                                 = 883, /*!< 883 (Price FIX.4.4) Price (percent-of-par or per unit) of the underlying security or basket at the end of the agreement.*/
UnderlyingStartValue                               = 884, /*!< 884 (Amt FIX.4.4) Currency value attributed to this collateral at the start of the agreement*/
UnderlyingCurrentValue                             = 885, /*!< 885 (Amt FIX.4.4) Currency value currently attributed to this collateral*/
UnderlyingEndValue                                 = 886, /*!< 886 (Amt FIX.4.4) Currency value attributed to this collateral at the end of the agreement*/
NoUnderlyingStips                                  = 887, /*!< 887 (NumInGroup FIX.4.4) Number of underlying stipulation entries*/
UnderlyingStipType                                 = 888, /*!< 888 (String FIX.4.4) Type of stipulation.

Same values as StipulationType (233)*/
UnderlyingStipValue                                = 889, /*!< 889 (String FIX.4.4) Value of stipulation.

Same values as StipulationValue (234)*/
MaturityNetMoney                                   = 890, /*!< 890 (Amt FIX.4.4) Net Money at maturity if Zero Coupon and maturity value is different from par value*/
MiscFeeBasis                                       = 891, /*!< 891 (int FIX.4.4) Defines the unit for a miscellaneous fee.*/
TotNoAllocs                                        = 892, /*!< 892 (int FIX.4.4) Total number of NoAlloc entries across all messages. Should be the sum of all NoAllocs in each message that has repeating NoAlloc entries related to the same AllocID or AllocReportID. Used to support fragmentation.*/
LastFragment                                       = 893, /*!< 893 (Boolean FIX.4.4) Indicates whether this message is the last in a sequence of messages for those messages that support fragmentation, such as Allocation Instruction, Mass Quote, Security List, Derivative Security List*/
CollReqID                                          = 894, /*!< 894 (String FIX.4.4) Collateral Request Identifier*/
CollAsgnReason                                     = 895, /*!< 895 (int FIX.4.4) Reason for Collateral Assignment*/
CollInquiryQualifier                               = 896, /*!< 896 (int FIX.4.4) Collateral inquiry qualifiers:*/
NoTrades                                           = 897, /*!< 897 (NumInGroup FIX.4.4) Number of trades in repeating group.*/
MarginRatio                                        = 898, /*!< 898 (Percentage FIX.4.4) The fraction of the cash consideration that must be collateralized, expressed as a percent. A MarginRatio of 02% indicates that the value of the collateral (after deducting for "haircut") must exceed the cash consideration by 2%.*/
MarginExcess                                       = 899, /*!< 899 (Amt FIX.4.4) Excess margin amount (deficit if value is negative)*/
TotalNetValue                                      = 900, /*!< 900 (Amt FIX.4.4) TotalNetValue is determined as follows:

At the initial collateral assignment TotalNetValue is the sum of (UnderlyingStartValue * (1-haircut)).

In a collateral substitution TotalNetValue is the sum of (UnderlyingCurrentValue * (1-haircut)).

For listed derivatives clearing margin management, this is the collateral value which equals (Market value * haircut)*/
CashOutstanding                                    = 901, /*!< 901 (Amt FIX.4.4) Starting consideration less repayments*/
CollAsgnID                                         = 902, /*!< 902 (String FIX.4.4) Collateral Assignment Identifier*/
CollAsgnTransType                                  = 903, /*!< 903 (int FIX.4.4) Collateral Assignment Transaction Type*/
CollRespID                                         = 904, /*!< 904 (String FIX.4.4) Collateral Response Identifier*/
CollAsgnRespType                                   = 905, /*!< 905 (int FIX.4.4) Type of collateral assignment response.*/
CollAsgnRejectReason                               = 906, /*!< 906 (int FIX.4.4) Collateral Assignment Reject Reason*/
CollAsgnRefID                                      = 907, /*!< 907 (String FIX.4.4) Collateral Assignment Identifier to which a transaction refers*/
CollRptID                                          = 908, /*!< 908 (String FIX.4.4) Collateral Report Identifier*/
CollInquiryID                                      = 909, /*!< 909 (String FIX.4.4) Collateral Inquiry Identifier*/
CollStatus                                         = 910, /*!< 910 (int FIX.4.4) Collateral Status*/
TotNumReports                                      = 911, /*!< 911 (int FIX.4.4) Total number of reports returned in response to a request.*/
LastRptRequested                                   = 912, /*!< 912 (Boolean FIX.4.4) Indicates whether this message is the last report message in response to a request message, e.g. OrderMassStatusRequest(35=AF), TradeCaptureReportRequest(35=AD).*/
AgreementDesc                                      = 913, /*!< 913 (String FIX.4.4) The full name of the base standard agreement, annexes and amendments in place between the principals applicable to a financing transaction.  See http://www.fpml.org/coding-scheme/master-agreement-type for derivative values.*/
AgreementID                                        = 914, /*!< 914 (String FIX.4.4) A common reference to the applicable standing agreement between the counterparties to a financing transaction.*/
AgreementDate                                      = 915, /*!< 915 (LocalMktDate FIX.4.4) A reference to the date the underlying agreement specified by AgreementID and AgreementDesc was executed.*/
StartDate                                          = 916, /*!< 916 (LocalMktDate FIX.4.4) Start date of a financing deal, i.e. the date the buyer pays the seller cash and takes control of the collateral*/
EndDate                                            = 917, /*!< 917 (LocalMktDate FIX.4.4) End date of a financing deal, i.e. the date the seller reimburses the buyer and takes back control of the collateral*/
AgreementCurrency                                  = 918, /*!< 918 (Currency FIX.4.4) Contractual currency forming the basis of a financing agreement and associated transactions. Usually, but not always, the same as the trade currency.*/
DeliveryType                                       = 919, /*!< 919 (int FIX.4.4) Identifies type of settlement*/
EndAccruedInterestAmt                              = 920, /*!< 920 (Amt FIX.4.4) Accrued Interest Amount applicable to a financing transaction on the End Date.*/
StartCash                                          = 921, /*!< 921 (Amt FIX.4.4) Starting dirty cash consideration of a financing deal, i.e. paid to the seller on the Start Date.*/
EndCash                                            = 922, /*!< 922 (Amt FIX.4.4) Ending dirty cash consideration of a financing deal. i.e. reimbursed to the buyer on the End Date.*/
UserRequestID                                      = 923, /*!< 923 (String FIX.4.4) Unique identifier for a User Request.*/
UserRequestType                                    = 924, /*!< 924 (int FIX.4.4) Indicates the action required by a User Request Message*/
NewPassword                                        = 925, /*!< 925 (String FIX.4.4) New Password or passphrase*/
UserStatus                                         = 926, /*!< 926 (int FIX.4.4) Indicates the status of a user*/
UserStatusText                                     = 927, /*!< 927 (String FIX.4.4) A text description associated with a user status.*/
StatusValue                                        = 928, /*!< 928 (int FIX.4.4) Indicates the status of a network connection*/
StatusText                                         = 929, /*!< 929 (String FIX.4.4) A text description associated with a network status.*/
RefCompID                                          = 930, /*!< 930 (String FIX.4.4) Assigned value used to identify a firm.*/
RefSubID                                           = 931, /*!< 931 (String FIX.4.4) Assigned value used to identify specific elements within a firm.*/
NetworkResponseID                                  = 932, /*!< 932 (String FIX.4.4) Unique identifier for a network response.*/
NetworkRequestID                                   = 933, /*!< 933 (String FIX.4.4) Unique identifier for a network resquest.*/
LastNetworkResponseID                              = 934, /*!< 934 (String FIX.4.4) Identifier of the previous Network Response message sent to a counterparty, used to allow incremental updates.*/
NetworkRequestType                                 = 935, /*!< 935 (int FIX.4.4) Indicates the type and level of details required for a Network Status Request Message

Boolean logic applies EG If you want to subscribe for changes to certain id's then UserRequestType =0 (8+2), Snapshot for certain ID's = 9 (8+1)*/
NoCompIDs                                          = 936, /*!< 936 (NumInGroup FIX.4.4) Number of CompID entries in a repeating group.*/
NetworkStatusResponseType                          = 937, /*!< 937 (int FIX.4.4) Indicates the type of Network Response Message.*/
NoCollInquiryQualifier                             = 938, /*!< 938 (NumInGroup FIX.4.4) Number of CollInquiryQualifier entries in a repeating group.*/
TrdRptStatus                                       = 939, /*!< 939 (int FIX.4.4) Trade Report Status*/
AffirmStatus                                       = 940, /*!< 940 (int FIX.4.4) Specifies the affirmation status of the confirmation.*/
UnderlyingStrikeCurrency                           = 941, /*!< 941 (Currency FIX.4.4) Currency in which the strike price of an underlying instrument is denominated*/
LegStrikeCurrency                                  = 942, /*!< 942 (Currency FIX.4.4) Currency in which the strike price of a instrument leg of a multileg instrument is denominated*/
TimeBracket                                        = 943, /*!< 943 (String FIX.4.4) A code that represents a time interval in which a fill or trade occurred.

Required for US futures markets.*/
CollAction                                         = 944, /*!< 944 (int FIX.4.4) Action proposed for an Underlying Instrument instance.*/
CollInquiryStatus                                  = 945, /*!< 945 (int FIX.4.4) Status of Collateral Inquiry*/
CollInquiryResult                                  = 946, /*!< 946 (int FIX.4.4) Result returned in response to Collateral Inquiry

4000+ Reserved and available for bi-laterally agreed upon user-defined values*/
StrikeCurrency                                     = 947, /*!< 947 (Currency FIX.4.4) Currency in which the StrikePrice is denominated.*/
NoNested3PartyIDs                                  = 948, /*!< 948 (NumInGroup FIX.4.4) Number of Nested3PartyID (949), Nested3PartyIDSource (950), and Nested3PartyRole (95) entries*/
Nested3PartyID                                     = 949, /*!< 949 (String FIX.4.4) PartyID value within a "third instance" Nested repeating group.

Same values as PartyID (448)*/
Nested3PartyIDSource                               = 950, /*!< 950 (char FIX.4.4) PartyIDSource value within a "third instance" Nested repeating group.

Same values as PartyIDSource (447)*/
Nested3PartyRole                                   = 951, /*!< 951 (int FIX.4.4) PartyRole value within a "third instance" Nested repeating group.

Same values as PartyRole (452)*/
NoNested3PartySubIDs                               = 952, /*!< 952 (NumInGroup FIX.4.4) Number of Nested3PartySubIDs (953) entries*/
Nested3PartySubID                                  = 953, /*!< 953 (String FIX.4.4) PartySubID value within a "third instance" Nested repeating group.

Same values as PartySubID (523)*/
Nested3PartySubIDType                              = 954, /*!< 954 (int FIX.4.4) PartySubIDType value within a "third instance" Nested repeating group.

Same values as PartySubIDType (803)*/
LegContractSettlMonth                              = 955, /*!< 955 (MonthYear FIX.4.4) Specifies when the contract (i.e. MBS/TBA) will settle.*/
LegInterestAccrualDate                             = 956, /*!< 956 (LocalMktDate FIX.4.4) The start date used for calculating accrued interest on debt instruments which are being sold between interest payment dates. Often but not always the same as the Issue Date and the Dated Date*/
NoStrategyParameters                               = 957, /*!< 957 (NumInGroup FIX.4.4) Indicates number of strategy parameters*/
StrategyParameterName                              = 958, /*!< 958 (String FIX.4.4) Name of parameter*/
StrategyParameterType                              = 959, /*!< 959 (int FIX.4.4) Datatype of the parameter*/
StrategyParameterValue                             = 960, /*!< 960 (String FIX.4.4) Value of the parameter*/
HostCrossID                                        = 961, /*!< 961 (String FIX.4.4) Host assigned entity ID that can be used to reference all components of a cross; sides + strategy + legs. Used as the primary key with which to refer to the Cross Order for cancellation and replace. The HostCrossID will also be used to link together components of the Cross Order. For example, each individual Execution Report associated with the order will carry HostCrossID in order to tie back to the original cross order.*/
SideTimeInForce                                    = 962, /*!< 962 (UTCTimestamp FIX.4.4) Indicates how long the order as specified in the side stays in effect. SideTimeInForce allows a two-sided cross order to specify order behavior separately for each side. Absence of this field indicates that TimeInForce should be referenced. SideTimeInForce will override TimeInForce if both are provided.*/
MDReportID                                         = 963, /*!< 963 (int FIX.4.4) Unique identifier for the Market Data Report.*/
SecurityReportID                                   = 964, /*!< 964 (int FIX.4.4) Identifies a Security List message.*/
SecurityStatus                                     = 965, /*!< 965 (String FIX.4.4) Used for derivatives. Denotes the current state of the Instrument.*/
SettleOnOpenFlag                                   = 966, /*!< 966 (String FIX.4.4) Indicator to determine if instrument is settle on open*/
StrikeMultiplier                                   = 967, /*!< 967 (float FIX.4.4) Used for derivatives. Multiplier applied to the strike price for the purpose of calculating the settlement value.*/
StrikeValue                                        = 968, /*!< 968 (float FIX.4.4) Used for derivatives. The number of shares/units for the financial instrument involved in the option trade.*/
MinPriceIncrement                                  = 969, /*!< 969 (float FIX.4.4) Minimum price increase for a given exchange-traded Instrument*/
PositionLimit                                      = 970, /*!< 970 (int FIX.4.4) Position Limit for a given exchange-traded product.*/
NTPositionLimit                                    = 971, /*!< 971 (int FIX.4.4) Position Limit in the near-term contract for a given exchange-traded product.*/
UnderlyingAllocationPercent                        = 972, /*!< 972 (Percentage FIX.4.4) Percent of the Strike Price that this underlying represents.*/
UnderlyingCashAmount                               = 973, /*!< 973 (Amt FIX.4.4) Cash amount associated with the underlying component.*/
UnderlyingCashType                                 = 974, /*!< 974 (String FIX.4.4) Used for derivatives that deliver into cash underlying.*/
UnderlyingSettlementType                           = 975, /*!< 975 (int FIX.4.4) Indicates order settlement period for the underlying instrument.*/
QuantityDate                                       = 976, /*!< 976 (LocalMktDate FIX.4.4) Date associated to the quantity that is being reported for the position.*/
ContIntRptID                                       = 977, /*!< 977 (String FIX.4.4) Unique identifier for the Contrary Intention report*/
LateIndicator                                      = 978, /*!< 978 (Boolean FIX.4.4) Indicates if the contrary intention was received after the exchange imposed cutoff time*/
InputSource                                        = 979, /*!< 979 (String FIX.4.4) Originating source of the request.*/
NoExpiration                                       = 981, /*!< 981 (NumInGroup FIX.4.4) Number of Expiration Qty entries*/
ExpirationQtyType                                  = 982, /*!< 982 (int FIX.4.4) Expiration Quantity type*/
ExpQty                                             = 983, /*!< 983 (Qty FIX.4.4) Expiration Quantity associated with the Expiration Type*/
NoUnderlyingAmounts                                = 984, /*!< 984 (NumInGroup FIX.4.4) Total number of occurrences of Amount to pay in order to receive the underlying instrument*/
UnderlyingPayAmount                                = 985, /*!< 985 (Amt FIX.4.4) Amount to pay in order to receive the underlying instrument*/
UnderlyingCollectAmount                            = 986, /*!< 986 (Amt FIX.4.4) Amount to collect in order to deliver the underlying instrument*/
UnderlyingSettlementDate                           = 987, /*!< 987 (LocalMktDate FIX.4.4) Date the underlying instrument will settle. Used for derivatives that deliver into more than one underlying instrument. Settlement dates can vary across underlying instruments.*/
UnderlyingSettlementStatus                         = 988, /*!< 988 (String FIX.4.4) Settlement status of the underlying instrument. Used for derivatives that deliver into more than one underlying instrument. Settlement can be delayed for an underlying instrument.*/
SecondaryIndividualAllocID                         = 989, /*!< 989 (String FIX.4.4) Will allow the intermediary to specify an allocation ID generated by their system.*/
LegReportID                                        = 990, /*!< 990 (String FIX.4.4) Additional attribute to store the Trade ID of the Leg.*/
RndPx                                              = 991, /*!< 991 (Price FIX.4.4) Specifies average price rounded to quoted precision.*/
IndividualAllocType                                = 992, /*!< 992 (int FIX.4.4) Identifies whether the allocation is to be sub-allocated or allocated to a third party*/
AllocCustomerCapacity                              = 993, /*!< 993 (String FIX.4.4) Capacity of customer in the allocation block.*/
TierCode                                           = 994, /*!< 994 (String FIX.4.4) The Tier the trade was matched by the clearing system.*/
UnitOfMeasure                                      = 996, /*!< 996 (String FIX.4.4) The unit of measure of the underlying commodity upon which the contract is based. Two groups of units of measure enumerations are supported.

Fixed Magnitude UOMs are primarily used in energy derivatives and specify a magnitude (such as, MM, Kilo, M, etc.) and the dimension (such as, watt hours, BTU's) to produce standard fixed measures (such as MWh - Megawatt-hours, MMBtu - One million BTUs).

The second group, Variable Quantity UOMs, specifies the dimension as a single unit without a magnitude (or more accurately a magnitude of one) and uses the UnitOfMeasureQty(1147) field to define the quantity of units per contract. Variable Quantity UOMs are used for both commodities (such as lbs of lean cattle, bushels of corn, ounces of gold) and financial futures.

Examples:

For lean cattle futures contracts, a UnitOfMeasure of 'lbs' with a UnitOfMeasureQty(1147) of 40,000, means each lean cattle futures contract represents 40,000 lbs of lean cattle.

For Eurodollars futures contracts, a UnitOfMeasure of Ccy with a UnitOfMeasureCurrency(1716)  of USD and a UnitOfMeasureQty(1147) of 1,000,000, means a Eurodollar futures contract represents 1,000,000 USD.

For gold futures contracts, a UnitOfMeasure is oz_tr (Troy ounce) with a UnitOfMeasureQty(1147) of 1,000, means each gold futures contract represents 1,000 troy ounces of gold.*/
TimeUnit                                           = 997, /*!< 997 (String FIX.4.4) Unit of time associated with the contract.

NOTE: Additional values may be used by mutual agreement of the counterparties*/
UnderlyingUnitOfMeasure                            = 998, /*!< 998 (String FIX.4.4) Refer to defintion of UnitOfMeasure(996)*/
LegUnitOfMeasure                                   = 999, /*!< 999 (String FIX.4.4) Refer to defintion of UnitOfMeasure(996)*/
UnderlyingTimeUnit                                 = 1000, /*!< 1000 (String FIX.4.4) Same as TimeUnit.*/
LegTimeUnit                                        = 1001, /*!< 1001 (String FIX.4.4) Same as TimeUnit.*/
AllocMethod                                        = 1002, /*!< 1002 (int FIX.4.4) Specifies the method under which a trade quantity was allocated.*/
TradeID                                            = 1003, /*!< 1003 (String FIX.4.4) The unique ID assigned to the trade entity once it is received or matched by the exchange or central counterparty.*/
SideTradeReportID                                  = 1005, /*!< 1005 (String FIX.4.4) Used on a multi-sided trade to designate the ReportID*/
SideFillStationCd                                  = 1006, /*!< 1006 (String FIX.4.4) Used on a multi-sided trade to convey order routing information*/
SideReasonCd                                       = 1007, /*!< 1007 (String FIX.4.4) Used on a multi-sided trade to convey reason for execution*/
SideTrdSubTyp                                      = 1008, /*!< 1008 (int FIX.4.4) Used on a multi-sided trade to specify the type of trade for a given side. Same values as TrdSubType (829).*/
SideLastQty                                        = 1009, /*!< 1009 (Qty FIX.4.4) Used to indicate the quantity on one side of a multi-sided trade.*/
MessageEventSource                                 = 1011, /*!< 1011 (String FIX.4.4) Used to identify the event or source which gave rise to a message.

Valid values will be based on an exchange's implementation.

Example values are:

"MQM" (originated at Firm Back Office)

"Clear" (originated in Clearing System)

"Reg" (static data generated via Register request)*/
SideTrdRegTimestamp                                = 1012, /*!< 1012 (UTCTimestamp FIX.4.4) Will be used in a multi-sided message.

Traded Regulatory timestamp value Use to store time information required by government regulators or self regulatory organizations such as an exchange or clearing house*/
SideTrdRegTimestampType                            = 1013, /*!< 1013 (int FIX.4.4) Same as TrdRegTimeStampType*/
SideTrdRegTimestampSrc                             = 1014, /*!< 1014 (String FIX.4.4) Same as TrdRegTimestampOrigin

Text which identifies the origin i.e. system which was used to generate the time stamp for the Traded Regulatory timestamp value*/
AsOfIndicator                                      = 1015, /*!< 1015 (char FIX.4.4) A trade that is being submitted for a trade date prior to the current trade or clearing date, e.g. in an open outcry market an out trade being submitted for the previous trading session or trading day.*/
NoSideTrdRegTS                                     = 1016, /*!< 1016 (NumInGroup FIX.4.4) Indicates number of SideTimestamps contained in group*/
LegOptionRatio                                     = 1017, /*!< 1017 (float FIX.4.4) Expresses the risk of an option leg

Value must be between -1 and 1.

A Call Option will require a ratio value between 0 and 1

A Put Option will require a ratio value between -1 and 0*/
NoInstrumentParties                                = 1018, /*!< 1018 (NumInGroup FIX.4.4) Identifies the number of parties identified with an instrument*/
InstrumentPartyID                                  = 1019, /*!< 1019 (String FIX.4.4) PartyID value within an instrument party repeating group. Same values as PartyID (448)*/
TradeVolume                                        = 1020, /*!< 1020 (Qty FIX.4.4) Used to report volume with a trade*/
MDBookType                                         = 1021, /*!< 1021 (int FIX.4.4) Describes the type of book for which the feed is intended. Used when multiple feeds are provided over the same connection*/
MDFeedType                                         = 1022, /*!< 1022 (String FIX.4.4) Describes a class of service for a given data feed, ie Regular and Market Maker, Bandwidth Intensive or Bandwidth Conservative*/
MDPriceLevel                                       = 1023, /*!< 1023 (int FIX.4.4) Integer to convey the level of a bid or offer at a given price level. This is in contrast to MDEntryPositionNo which is used to convey the position of an order within a Price level*/
MDOriginType                                       = 1024, /*!< 1024 (int FIX.4.4) Used to describe the origin of the market data entry.*/
FirstPx                                            = 1025, /*!< 1025 (Price FIX.4.4) Indicates the first trade price of the day/session*/
MDEntrySpotRate                                    = 1026, /*!< 1026 (float FIX.4.4) The spot rate for an FX entry*/
MDEntryForwardPoints                               = 1027, /*!< 1027 (PriceOffset FIX.4.4) Used for an F/X entry. The forward points to be added to or subtracted from the spot rate to get the "all-in" rate in MDEntryPx. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
ManualOrderIndicator                               = 1028, /*!< 1028 (Boolean FIX.4.4) Indicates if the order was initially received manually (as opposed to electronically) or if it was entered manually (as opposed to entered by automated trading software).*/
CustDirectedOrder                                  = 1029, /*!< 1029 (Boolean FIX.4.4) Indicates if the customer directed this order to a specific execution venue "Y" or not "N".

A default of "N" customer did not direct this order should be used in the case where the information is both missing and essential.*/
ReceivedDeptID                                     = 1030, /*!< 1030 (String FIX.4.4) Identifies the broker-dealer department that first took the order.*/
CustOrderHandlingInst                              = 1031, /*!< 1031 (MultipleStringValue FIX.4.4) Codes that apply special information that the Broker / Dealer needs to report, as specified by the customer.

NOTE: This field and its values have no bearing on the ExecInst and TimeInForce fields. These values should not be used instead of ExecInst or TimeInForce. This field and its values are intended for compliance reporting and/or billing purposes only.



For OrderHandlingInstSrc(1032) = 1 (FINRA OATS), valid values are (as of OATS Phase 3 as provided by FINRA. See also http://www.finra.org/Industry/Compliance/MarketTransparency/OATS/PhaseIII/index.htm for a complete list.



For OrderHandlingInstSrc(1032) = 2 (FIA Execution Source Code), only one enumeration value may be specified.*/
OrderHandlingInstSource                            = 1032, /*!< 1032 (int FIX.4.4) Identifies the class or source of the order handling instruction values.  Scope of this will apply to both CustOrderHandlingInst(1031) and DeskOrderHandlingInst(1035).

Conditionally required when CustOrderHandlingInst(1031) or DeskOrderHandlingInst(1035) is specified.*/
DeskType                                           = 1033, /*!< 1033 (String FIX.4.4) Identifies the type of Trading Desk.

Conditionally required when InformationBarrierID(1727) is specified for OATS.*/
DeskTypeSource                                     = 1034, /*!< 1034 (int FIX.4.4) Identifies the class or source of DeskType(1033) values. Conditionally required when DeskType(1033) is specified.*/
DeskOrderHandlingInst                              = 1035, /*!< 1035 (MultipleStringValue FIX.4.4) Codes that apply special information that the broker-dealer needs to report.*/
ExecAckStatus                                      = 1036, /*!< 1036 (char FIX.4.4) The status of this execution acknowledgement message.*/
UnderlyingDeliveryAmount                           = 1037, /*!< 1037 (Amt FIX.4.4) Indicates the underlying position amount to be delivered*/
UnderlyingCapValue                                 = 1038, /*!< 1038 (Amt FIX.4.4) Maximum notional value for a capped financial instrument*/
UnderlyingSettlMethod                              = 1039, /*!< 1039 (String FIX.4.4) Settlement method for a contract or instrument. Additional values may be used with bilateral agreement.*/
SecondaryTradeID                                   = 1040, /*!< 1040 (String FIX.4.4) Used to carry an internal trade entity ID which may or may not be reported to the firm*/
FirmTradeID                                        = 1041, /*!< 1041 (String FIX.4.4) The ID assigned to a trade by the Firm to track a trade within the Firm system. This ID can be assigned either before or after submission to the exchange or central counterpary*/
SecondaryFirmTradeID                               = 1042, /*!< 1042 (String FIX.4.4) Used to carry an internal firm assigned ID which may or may not be reported to the exchange or central counterpary*/
CollApplType                                       = 1043, /*!< 1043 (int FIX.4.4) conveys how the collateral should be/has been applied*/
UnderlyingAdjustedQuantity                         = 1044, /*!< 1044 (Qty FIX.4.4) Unit amount of the underlying security (shares) adjusted for pending corporate action not yet allocated.*/
UnderlyingFXRate                                   = 1045, /*!< 1045 (float FIX.4.4) Foreign exchange rate used to compute UnderlyingCurrentValue(885) (or market value) from UnderlyingCurrency(318) to Currency(15).*/
UnderlyingFXRateCalc                               = 1046, /*!< 1046 (char FIX.4.4) Specifies whether the UnderlyingFxRate(1045) should be multiplied or divided.*/
AllocPositionEffect                                = 1047, /*!< 1047 (char FIX.4.4) Indicates whether the resulting position after a trade should be an opening position or closing position. Used for omnibus accounting - where accounts are held on a gross basis instead of being netted together.*/
DealingCapacity                                    = 1048, /*!< 1048 (char FIX.4.4) Identifies role of dealer; Agent, Principal, RisklessPrincipal*/
InstrmtAssignmentMethod                            = 1049, /*!< 1049 (char FIX.4.4) Method under which assignment was conducted*/
InstrumentPartyIDSource                            = 1050, /*!< 1050 (char FIX.4.4) PartyIDSource value within an instrument partyrepeating group.

Same values as PartyIDSource (447)*/
InstrumentPartyRole                                = 1051, /*!< 1051 (int FIX.4.4) PartyRole value within an instrument partyepeating group.

Same values as PartyRole (452)*/
NoInstrumentPartySubIDs                            = 1052, /*!< 1052 (NumInGroup FIX.4.4) Number of InstrumentPartySubID (1053) and InstrumentPartySubIDType (1054) entries*/
InstrumentPartySubID                               = 1053, /*!< 1053 (String FIX.4.4) PartySubID value within an instrument party repeating group.

Same values as PartySubID (523)*/
InstrumentPartySubIDType                           = 1054, /*!< 1054 (int FIX.4.4) Type of InstrumentPartySubID (1053) value.

Same values as PartySubIDType (803)*/
PositionCurrency                                   = 1055, /*!< 1055 (String FIX.4.4) The Currency in which the position Amount is denominated*/
CalculatedCcyLastQty                               = 1056, /*!< 1056 (Qty FIX.4.4) Used for the calculated quantity of the other side of the currency trade. Can be derived from LastQty and LastPx.*/
AggressorIndicator                                 = 1057, /*!< 1057 (Boolean FIX.4.4) Used to identify whether the order initiator is an aggressor or not in the trade.*/
NoUndlyInstrumentParties                           = 1058, /*!< 1058 (NumInGroup FIX.4.4) Identifies the number of parties identified with an underlying instrument*/
UnderlyingInstrumentPartyID                        = 1059, /*!< 1059 (String FIX.4.4) PartyID value within an underlying instrument party repeating group.

Same values as PartyID (448)*/
UnderlyingInstrumentPartyIDSource                  = 1060, /*!< 1060 (char FIX.4.4) PartyIDSource value within an underlying instrument partyrepeating group.

Same values as PartyIDSource (447)*/
UnderlyingInstrumentPartyRole                      = 1061, /*!< 1061 (int FIX.4.4) PartyRole value within an underlying instrument partyepeating group.

Same values as PartyRole (452)*/
NoUndlyInstrumentPartySubIDs                       = 1062, /*!< 1062 (NumInGroup FIX.4.4) Number of Underlying InstrumentPartySubID (1053) and InstrumentPartySubIDType (1054) entries*/
UnderlyingInstrumentPartySubID                     = 1063, /*!< 1063 (String FIX.4.4) PartySubID value within an underlying instrument party repeating group.

Same values as PartySubID (523)*/
UnderlyingInstrumentPartySubIDType                 = 1064, /*!< 1064 (int FIX.4.4) Type of underlying InstrumentPartySubID (1053) value.

Same values as PartySubIDType (803)*/
BidSwapPoints                                      = 1065, /*!< 1065 (PriceOffset FIX.4.4) The bid FX Swap points for an FX Swap. It is the "far bid forward points - near offer forward point". Value can be negative. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
OfferSwapPoints                                    = 1066, /*!< 1066 (PriceOffset FIX.4.4) The offer FX Swap points for an FX Swap. It is the "far offer forward points - near bid forward points". Value can be negative. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
LegBidForwardPoints                                = 1067, /*!< 1067 (PriceOffset FIX.4.4) The bid FX forward points for the leg of an FX Swap. Value can be negative. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
LegOfferForwardPoints                              = 1068, /*!< 1068 (PriceOffset FIX.4.4) The offer FX forward points for the leg of an FX Swap. Value can be negative. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
SwapPoints                                         = 1069, /*!< 1069 (PriceOffset FIX.4.4) For FX Swap, this is used to express the differential between the far leg's bid/offer and the near leg's bid/offer. Value can be negative. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
MDQuoteType                                        = 1070, /*!< 1070 (int FIX.4.4) Identifies market data quote type.*/
LastSwapPoints                                     = 1071, /*!< 1071 (PriceOffset FIX.4.4) For FX Swap, this is used to express the last market event for the differential between the far leg's bid/offer and the near leg's bid/offer in a fill or partial fill. Value can be negative. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
SideGrossTradeAmt                                  = 1072, /*!< 1072 (Amt FIX.4.4) The gross trade amount for this side of the trade. See also GrossTradeAmt (381) for additional definition.*/
LegLastForwardPoints                               = 1073, /*!< 1073 (PriceOffset FIX.4.4) The forward points for this leg's fill event. Value can be negative. Expressed in decimal form. For example, 61.99 points is expressed and sent as 0.006199*/
LegCalculatedCcyLastQty                            = 1074, /*!< 1074 (Qty FIX.4.4) Used for the calculated quantity of the other side of the currency for this leg. Can be derived from LegQty and LegLastPx.*/
LegGrossTradeAmt                                   = 1075, /*!< 1075 (Amt FIX.4.4) The gross trade amount of the leg. For FX Futures this is used to express the notional value of a fill when LegLastQty and other quantity fields are express in terms of contract size.*/
MaturityTime                                       = 1079, /*!< 1079 (TZTimeOnly FIX.4.4) Time of security's maturity expressed in local time with offset to UTC specified*/
RefOrderID                                         = 1080, /*!< 1080 (String FIX.4.4) The ID reference to the order being hit or taken.



For pre-trade credit/risk limit check process, this is the reference to the placed order, quote request or quote for the credit/risk limit check.*/
RefOrderIDSource                                   = 1081, /*!< 1081 (char FIX.4.4) Used to specify what identifier, provided in order depth market data, to use when hitting (taking) a specific order or to identify what type of order or quote reference is being provided when seeking credit limit check.*/
SecondaryDisplayQty                                = 1082, /*!< 1082 (Qty FIX.4.4) Used for reserve orders when DisplayQty applies to the primary execution market (e.g.an ECN) and another quantity is to be shown at other markets (e.g. the exchange). On orders specifies the qty to be displayed, on execution reports the currently displayed quantity.*/
DisplayWhen                                        = 1083, /*!< 1083 (char FIX.4.4) Instructs when to refresh DisplayQty (1138).*/
DisplayMethod                                      = 1084, /*!< 1084 (char FIX.4.4) Defines what value to use in DisplayQty (1138). If not specified the default DisplayMethod is "1"*/
DisplayLowQty                                      = 1085, /*!< 1085 (Qty FIX.4.4) Defines the lower quantity limit to a randomized refresh of DisplayQty.*/
DisplayHighQty                                     = 1086, /*!< 1086 (Qty FIX.4.4) Defines the upper quantity limit to a randomized refresh of DisplayQty.*/
DisplayMinIncr                                     = 1087, /*!< 1087 (Qty FIX.4.4) Defines the minimum increment to be used when calculating a random refresh of DisplayQty. A user specifies this when he wants a larger increment than the standard provided by the market (e.g. the round lot size).*/
RefreshQty                                         = 1088, /*!< 1088 (Qty FIX.4.4) Defines the quantity used to refresh DisplayQty.*/
MatchIncrement                                     = 1089, /*!< 1089 (Qty FIX.4.4) Allows orders to specify a minimum quantity that applies to every execution (one execution could be for multiple counter-orders). The order may still fill against smaller orders, but the cumulative quantity of the execution must be in multiples of the MatchIncrement.*/
MaxPriceLevels                                     = 1090, /*!< 1090 (int FIX.4.4) Allows an order to specify a maximum number of price levels to trade through. Only valid for aggressive orders and during continuous (autoexecution) trading sessions. Property lost when order is put on book. A partially filled order is assigned last trade price as limit price. Non-filled order behaves as ordinary Market or Limit.*/
PreTradeAnonymity                                  = 1091, /*!< 1091 (Boolean FIX.4.4) Allows trader to explicitly request anonymity or disclosure in pre-trade market data feeds. Anonymity is relevant in markets where counterparties are regularly disclosed in order depth feeds. Disclosure is relevant when counterparties are not normally visible.*/
PriceProtectionScope                               = 1092, /*!< 1092 (char FIX.4.4) Defines the type of price protection the customer requires on their order.*/
LotType                                            = 1093, /*!< 1093 (char FIX.4.4) Defines the lot type assigned to the order.*/
PegPriceType                                       = 1094, /*!< 1094 (int FIX.4.4) Defines the type of peg.*/
PeggedRefPrice                                     = 1095, /*!< 1095 (Price FIX.4.4) The value of the reference price that the order is pegged to. PeggedRefPrice + PegOffsetValue (211) = PeggedPrice (839) unless the limit price (44, Price) is breached. The values may not be exact due to rounding.*/
PegSecurityIDSource                                = 1096, /*!< 1096 (String FIX.4.4) Defines the identity of the security off whose prices the order will peg. Same values as SecurityIDSource (22)*/
PegSecurityID                                      = 1097, /*!< 1097 (String FIX.4.4) Defines the identity of the security off whose prices the order will peg.*/
PegSymbol                                          = 1098, /*!< 1098 (String FIX.4.4) Defines the common, 'human understood' representation of the security off whose prices the order will Peg.*/
PegSecurityDesc                                    = 1099, /*!< 1099 (String FIX.4.4) Security description of the security off whose prices the order will Peg.*/
TriggerType                                        = 1100, /*!< 1100 (char FIX.5.0) Defines when the trigger will hit, i.e. the action specified by the trigger instructions will come into effect.*/
TriggerAction                                      = 1101, /*!< 1101 (char FIX.5.0) Defines the type of action to take when the trigger hits.*/
TriggerPrice                                       = 1102, /*!< 1102 (Price FIX.5.0) The price at which the trigger should hit.*/
TriggerSymbol                                      = 1103, /*!< 1103 (String FIX.5.0) Defines the common, 'human understood' representation of the security whose prices will be tracked by the trigger logic.*/
TriggerSecurityID                                  = 1104, /*!< 1104 (String FIX.5.0) Defines the identity of the security whose prices will be tracked by the trigger logic.*/
TriggerSecurityIDSource                            = 1105, /*!< 1105 (String FIX.5.0) Defines the identity of the security whose prices will be tracked by the trigger logic. Same values as SecurityIDSource (22).*/
TriggerSecurityDesc                                = 1106, /*!< 1106 (String FIX.5.0) Defines the security description of the security whose prices will be tracked by the trigger logic.*/
TriggerPriceType                                   = 1107, /*!< 1107 (char FIX.5.0) The type of price that the trigger is compared to.*/
TriggerPriceTypeScope                              = 1108, /*!< 1108 (char FIX.5.0) Defines the type of price protection the customer requires on their order.*/
TriggerPriceDirection                              = 1109, /*!< 1109 (char FIX.5.0) The side from which the trigger price is reached.*/
TriggerNewPrice                                    = 1110, /*!< 1110 (Price FIX.5.0) The Price that the order should have after the trigger has hit. Could be applicable for any trigger type, but must be specified for Trigger Type 1.*/
TriggerOrderType                                   = 1111, /*!< 1111 (char FIX.5.0) The OrdType the order should have after the trigger has hit. Required to express orders that change from Limit to Market. Other values from OrdType (40) may be used if appropriate and bilaterally agreed upon.*/
TriggerNewQty                                      = 1112, /*!< 1112 (Qty FIX.5.0) The Quantity the order should have after the trigger has hit.*/
TriggerTradingSessionID                            = 1113, /*!< 1113 (String FIX.5.0) Defines the trading session at which the order will be activated.*/
TriggerTradingSessionSubID                         = 1114, /*!< 1114 (String FIX.5.0) Defines the subordinate trading session at which the order will be activated.*/
OrderCategory                                      = 1115, /*!< 1115 (char FIX.4.4) Defines the type of interest behind a trade (fill or partial fill).*/
NoRootPartyIDs                                     = 1116, /*!< 1116 (NumInGroup FIX.4.4) Number of RootPartyID (1117), RootPartyIDSource (1118), and RootPartyRole (1119) entries*/
RootPartyID                                        = 1117, /*!< 1117 (String FIX.4.4) PartyID value within a root parties component. Same values as PartyID (448)*/
RootPartyIDSource                                  = 1118, /*!< 1118 (char FIX.4.4) PartyIDSource value within a root parties component. Same values as PartyIDSource (447)*/
RootPartyRole                                      = 1119, /*!< 1119 (int FIX.4.4) PartyRole value within a root parties component. Same values as PartyRole (452)*/
NoRootPartySubIDs                                  = 1120, /*!< 1120 (NumInGroup FIX.4.4) Number of RootPartySubID (1121) and RootPartySubIDType (1122) entries*/
RootPartySubID                                     = 1121, /*!< 1121 (String FIX.4.4) PartySubID value within a root parties component. Same values as PartySubID (523)*/
RootPartySubIDType                                 = 1122, /*!< 1122 (int FIX.4.4) Type of RootPartySubID (1121) value. Same values as PartySubIDType (803)*/
TradeHandlingInstr                                 = 1123, /*!< 1123 (char FIX.4.4) Specified how the TradeCaptureReport(35=AE) should be handled by the respondent.*/
OrigTradeHandlingInstr                             = 1124, /*!< 1124 (char FIX.4.4) Optionally used with TradeHandlingInstr = 0 to relay the trade handling instruction used when reporting the trade to the marketplace. Same values as TradeHandlingInstr (1123)*/
OrigTradeDate                                      = 1125, /*!< 1125 (LocalMktDate FIX.4.4) Used to preserve original trade date when original trade is being referenced in a subsequent trade transaction such as a transfer*/
OrigTradeID                                        = 1126, /*!< 1126 (String FIX.4.4) Used to preserve original trade id when original trade is being referenced in a subsequent trade transaction such as a transfer*/
OrigSecondaryTradeID                               = 1127, /*!< 1127 (String FIX.4.4) Used to preserve original secondary trade id when original trade is being referenced in a subsequent trade transaction such as a transfer*/
ApplVerID                                          = 1128, /*!< 1128 (String FIX.4.4) Specifies the service pack release being applied at message level. Enumerated field with values assigned at time of service pack release*/
CstmApplVerID                                      = 1129, /*!< 1129 (String FIX.4.4) Specifies a custom extension to a message being applied at the message level. Enumerated field*/
RefApplVerID                                       = 1130, /*!< 1130 (String FIX.4.4) Specifies the service pack release being applied to a message at the session level. Enumerated field with values assigned at time of service pack release. Uses same values as ApplVerID*/
RefCstmApplVerID                                   = 1131, /*!< 1131 (String FIX.4.4) Specifies a custom extension to a message being applied at the session level.*/
TZTransactTime                                     = 1132, /*!< 1132 (TZTimestamp FIX.4.4) Transact time in the local date-time stamp with a TZ offset to UTC identified*/
ExDestinationIDSource                              = 1133, /*!< 1133 (char FIX.4.4) The ID source of ExDestination*/
ReportedPxDiff                                     = 1134, /*!< 1134 (Boolean FIX.4.4) Indicates that the reported price that is different from the market price. The price difference should be stated by using field 828 TrdType and, if required, field 829 TrdSubType*/
RptSys                                             = 1135, /*!< 1135 (String FIX.4.4) Indicates the system or medium on which the report has been published*/
AllocClearingFeeIndicator                          = 1136, /*!< 1136 (String FIX.4.4) ClearingFeeIndicator(635) for Allocation, see ClearingFeeIndicator(635) for permitted values.*/
DefaultApplVerID                                   = 1137, /*!< 1137 (String FIX.4.4) Specifies the service pack release being applied, by default, to message at the session level. Enumerated field with values assigned at time of service pack release. Uses same values as ApplVerID*/
DisplayQty                                         = 1138, /*!< 1138 (Qty FIX.4.4) The quantity to be displayed . Required for reserve orders. On orders specifies the qty to be displayed, on execution reports the currently displayed quantity.*/
ExchangeSpecialInstructions                        = 1139, /*!< 1139 (String FIX.4.4) Free format text string related to exchange.*/
MaxTradeVol                                        = 1140, /*!< 1140 (Qty FIX.5.0) The maximum order quantity (as expressed by TradeVolType(1786)) that can be submitted for a security.*/
NoMDFeedTypes                                      = 1141, /*!< 1141 (NumInGroup FIX.5.0) The number of feed types and corresponding book depths associated with a security*/
MatchAlgorithm                                     = 1142, /*!< 1142 (String FIX.5.0) The types of algorithm used to match orders in a specific security. Possilbe value types are FIFO, Allocation, Pro-rata, Lead Market Maker, Currency Calender.*/
MaxPriceVariation                                  = 1143, /*!< 1143 (float FIX.5.0) The maximum price variation of an execution from one event to the next for a given security. Expressed in absolute price terms.*/
ImpliedMarketIndicator                             = 1144, /*!< 1144 (int FIX.5.0) Indicates that an implied market should be created for either the legs of a multi-leg instrument (Implied-in) or for the multi-leg instrument based on the existence of the legs (Implied-out). Determination as to whether implied markets should be created is generally done at the level of the multi-leg instrument. Commonly used in listed derivatives.*/
EventTime                                          = 1145, /*!< 1145 (UTCTimestamp FIX.5.0) Specific time of event. To be used in combination with EventDate [866]*/
MinPriceIncrementAmount                            = 1146, /*!< 1146 (Amt FIX.5.0) Minimum price increment amount associated with the MinPriceIncrement ( tag 969). For listed derivatives, the value can be calculated by multiplying MinPriceIncrement by ContractValueFactor(231).*/
UnitOfMeasureQty                                   = 1147, /*!< 1147 (Qty FIX.5.0) Used to indicate the quantity of the underlying commodity unit of measure on which the contract is based, such as, 2500 lbs of lean cattle, 1000 barrels of crude oil, 1000 bushels of corn, etc. UnitOfMeasureQty is required for UnitOfMeasure(996) Variable Quantity UOMs enumerations. Refer to the definition of UnitOfMeasure(996) for more information on the use of UnitOfMeasureQty.*/
LowLimitPrice                                      = 1148, /*!< 1148 (Price FIX.5.0) Allowable low limit price for the trading day. A key parameter in validating order price. Used as the lower band for validating order prices. Orders submitted with prices below the lower limit will be rejected*/
HighLimitPrice                                     = 1149, /*!< 1149 (Price FIX.5.0) Allowable high limit price for the trading day. A key parameter in validating order price. Used as the upper band for validating order prices. Orders submitted with prices above the upper limit will be rejected*/
TradingReferencePrice                              = 1150, /*!< 1150 (Price FIX.5.0) Reference price for the current trading price range usually representing the mid price between the HighLimitPrice and LowLimitPrice. The value may be the settlement price or closing price of the prior trading day.*/
SecurityGroup                                      = 1151, /*!< 1151 (String FIX.5.0) An exchange specific name assigned to a group of related securities which may be concurrently affected by market events and actions.*/
LegNumber                                          = 1152, /*!< 1152 (int FIX.5.0) Allow sequencing of Legs for a Strategy to be captured*/
SettlementCycleNo                                  = 1153, /*!< 1153 (int FIX.5.0) Settlement cycle in which the settlement obligation was generated*/
SideCurrency                                       = 1154, /*!< 1154 (Currency FIX.5.0) Used to identify the trading currency on the Trade Capture Report Side*/
SideSettlCurrency                                  = 1155, /*!< 1155 (Currency FIX.5.0) Used to identify the settlement currency on the Trade Capture Report Side*/
ApplExtID                                          = 1156, /*!< 1156 (int FIX.5.0) The extension pack number associated with an application message.*/
CcyAmt                                             = 1157, /*!< 1157 (Amt FIX.5.0) Net flow of Currency 1*/
NoSettlDetails                                     = 1158, /*!< 1158 (NumInGroup FIX.5.0) Used to group Each Settlement Party*/
SettlObligMode                                     = 1159, /*!< 1159 (int FIX.5.0) Used to identify the reporting mode of the settlement obligation which is either preliminary or final*/
SettlObligMsgID                                    = 1160, /*!< 1160 (String FIX.5.0) Message identifier for Settlement Obligation Report*/
SettlObligID                                       = 1161, /*!< 1161 (String FIX.5.0) Unique ID for this settlement instruction.*/
SettlObligTransType                                = 1162, /*!< 1162 (char FIX.5.0) Transaction Type - required except where SettlInstMode is 5=Reject SSI request*/
SettlObligRefID                                    = 1163, /*!< 1163 (String FIX.5.0) Required where SettlInstTransType is Cancel or Replace*/
SettlObligSource                                   = 1164, /*!< 1164 (char FIX.5.0) Used to identify whether these delivery instructions are for the buyside or the sellside.*/
NoSettlOblig                                       = 1165, /*!< 1165 (NumInGroup FIX.5.0) Number of settlement obligations*/
QuoteMsgID                                         = 1166, /*!< 1166 (String FIX.5.0) Unique identifier for a quote message.*/
QuoteEntryStatus                                   = 1167, /*!< 1167 (int FIX.5.0) Identifies the status of an individual quote. See also QuoteStatus(297) which is used for single Quotes.*/
TotNoCxldQuotes                                    = 1168, /*!< 1168 (int FIX.5.0) Specifies the number of canceled quotes*/
TotNoAccQuotes                                     = 1169, /*!< 1169 (int FIX.5.0) Specifies the number of accepted quotes*/
TotNoRejQuotes                                     = 1170, /*!< 1170 (int FIX.5.0) Specifies the number of rejected quotes*/
PrivateQuote                                       = 1171, /*!< 1171 (Boolean FIX.5.0) Specifies whether a quote is public, i.e. available to the market, or private, i.e. available to a specified counterparty only.*/
RespondentType                                     = 1172, /*!< 1172 (int FIX.5.0) Specifies the type of respondents requested.*/
MDSubBookType                                      = 1173, /*!< 1173 (int FIX.5.0) Describes a class of sub book, e.g. for the separation of various lot types. The Sub Book Type indicates that the following Market Data Entries belong to a non-integrated Sub Book. Whenever provided the Sub Book must be used together with MDPriceLevel and MDEntryPositionNo in order to sort the order properly.

Values are bilaterally agreed.*/
SecurityTradingEvent                               = 1174, /*!< 1174 (int FIX.5.0) Identifies an event related to a SecurityTradingStatus(326). An event occurs and is gone, it is not a state that applies for a period of time.*/
NoStatsIndicators                                  = 1175, /*!< 1175 (NumInGroup FIX.5.0) Number of statistics indicator repeating group entries*/
StatsType                                          = 1176, /*!< 1176 (int FIX.5.0) Type of statistics*/
NoOfSecSizes                                       = 1177, /*!< 1177 (NumInGroup FIX.5.0) The number of secondary sizes specifies in this entry*/
MDSecSizeType                                      = 1178, /*!< 1178 (int FIX.5.0) Specifies the type of secondary size.*/
MDSecSize                                          = 1179, /*!< 1179 (Qty FIX.5.0) A part of the MDEntrySize(271) that represents secondary interest as specified by MDSecSizeType(1178).*/
ApplID                                             = 1180, /*!< 1180 (String FIX.5.0) Identifies the application with which a message is associated. Used only if application sequencing is in effect.*/
ApplSeqNum                                         = 1181, /*!< 1181 (SeqNum FIX.5.0) Data sequence number to be used when FIX session is not in effect*/
ApplBegSeqNum                                      = 1182, /*!< 1182 (SeqNum FIX.5.0) Beginning range of application sequence numbers*/
ApplEndSeqNum                                      = 1183, /*!< 1183 (SeqNum FIX.5.0) Ending range of application sequence numbers*/
SecurityXMLLen                                     = 1184, /*!< 1184 (Length FIX.5.0) The length of the SecurityXML(1185) data block.*/
SecurityXML                                        = 1185, /*!< 1185 (XMLData FIX.5.0) XML definition for the security.*/
SecurityXMLSchema                                  = 1186, /*!< 1186 (String FIX.5.0) The schema used to validate the contents of SecurityXML(1185).*/
RefreshIndicator                                   = 1187, /*!< 1187 (Boolean FIX.5.0) Set by the sender to tell the receiver to perform an immediate refresh of the book due to disruptions in the accompanying real-time feed

'Y' - Mandatory refresh by all participants

'N' - Process as required*/
Volatility                                         = 1188, /*!< 1188 (float FIX.5.0) Annualized volatility for option model calculations*/
TimeToExpiration                                   = 1189, /*!< 1189 (float FIX.5.0) Time to expiration in years calculated as the number of days remaining to expiration divided by 365 days per year.*/
RiskFreeRate                                       = 1190, /*!< 1190 (float FIX.5.0) Interest rate. Usually some form of short term rate.*/
PriceUnitOfMeasure                                 = 1191, /*!< 1191 (String FIX.5.0) Used to express the UOM of the price if different from the contract. In futures, this can be different for cross-rate products in which the price is quoted in units differently from the contract*/
PriceUnitOfMeasureQty                              = 1192, /*!< 1192 (Qty FIX.5.0) Used to express the UOM Quantity of the price if different from the contract. In futures, this can be different for physically delivered products in which price is quoted in a unit size different from the contract, i.e. a Cattle Future contract has a UOMQty of 40,000 and a PriceUOMQty of 100.*/
SettlMethod                                        = 1193, /*!< 1193 (String FIX.5.0) Settlement method for a contract or instrument. Additional values may be used with bilateral agreement.*/
ExerciseStyle                                      = 1194, /*!< 1194 (int FIX.5.0) Type of exercise of a derivatives security*/
OptPayoutAmount                                    = 1195, /*!< 1195 (Amt FIX.5.0) Cash amount indicating the pay out associated with an option. For binary options this is a fixed amount. */
PriceQuoteMethod                                   = 1196, /*!< 1196 (String FIX.5.0) Method for price quotation*/
ValuationMethod                                    = 1197, /*!< 1197 (String FIX.5.0) Specifies the type of valuation method applied.*/
ListMethod                                         = 1198, /*!< 1198 (int FIX.5.0) Indicates whether instruments are pre-listed only or can also be defined via user request*/
CapPrice                                           = 1199, /*!< 1199 (Price FIX.5.0) Used to express the ceiling price of a capped call*/
FloorPrice                                         = 1200, /*!< 1200 (Price FIX.5.0) Used to express the floor price of a capped put*/
NoStrikeRules                                      = 1201, /*!< 1201 (NumInGroup FIX.5.0) Number of strike rule entries. This block specifies the rules for determining how new strikes should be listed within the stated price range of the underlying instrument*/
StartStrikePxRange                                 = 1202, /*!< 1202 (Price FIX.5.0) Starting price for the range to which the StrikeIncrement applies. Price refers to the price of the underlying*/
EndStrikePxRange                                   = 1203, /*!< 1203 (Price FIX.5.0) Ending price of the range to which the StrikeIncrement applies. Price refers to the price of the underlying*/
StrikeIncrement                                    = 1204, /*!< 1204 (float FIX.5.0) Value by which strike price should be incremented within the specified price range.*/
NoTickRules                                        = 1205, /*!< 1205 (NumInGroup FIX.5.0) Number of tick rules. This block specifies the rules for determining how a security ticks, i.e. the price increments at which it can be quoted and traded, depending on the current price of the security*/
StartTickPriceRange                                = 1206, /*!< 1206 (Price FIX.5.0) Starting price range for specified tick increment*/
EndTickPriceRange                                  = 1207, /*!< 1207 (Price FIX.5.0) Ending price range for the specified tick increment*/
TickIncrement                                      = 1208, /*!< 1208 (Price FIX.5.0) Tick increment for stated price range. Specifies the valid price increments at which a security can be quoted and traded*/
TickRuleType                                       = 1209, /*!< 1209 (int FIX.5.0) Specifies the type of tick rule which is being described*/
NestedInstrAttribType                              = 1210, /*!< 1210 (int FIX.5.0) Code to represent the type of instrument attribute*/
NestedInstrAttribValue                             = 1211, /*!< 1211 (String FIX.5.0) Attribute value appropriate to the NestedInstrAttribType field*/
LegMaturityTime                                    = 1212, /*!< 1212 (TZTimeOnly FIX.5.0) Time of security's maturity expressed in local time with offset to UTC specified*/
UnderlyingMaturityTime                             = 1213, /*!< 1213 (TZTimeOnly FIX.5.0) Time of security's maturity expressed in local time with offset to UTC specified*/
DerivativeSymbol                                   = 1214, /*!< 1214 (String FIX.5.0) Refer to definition for Symbol(55)*/
DerivativeSymbolSfx                                = 1215, /*!< 1215 (String FIX.5.0) Refer to definition for SymbolSfx(65)*/
DerivativeSecurityID                               = 1216, /*!< 1216 (String FIX.5.0) Refer to definition for SecurityID(48)*/
DerivativeSecurityIDSource                         = 1217, /*!< 1217 (String FIX.5.0) Refer to definition for SecurityIDSoruce(22)*/
NoDerivativeSecurityAltID                          = 1218, /*!< 1218 (NumInGroup FIX.5.0) Refer to definition for NoSecurityAltID(454)*/
DerivativeSecurityAltID                            = 1219, /*!< 1219 (String FIX.5.0) Refer to definition for SecurityAltID(455)*/
DerivativeSecurityAltIDSource                      = 1220, /*!< 1220 (String FIX.5.0) Refer to definition for SecurityAltIDSource(456)*/
SecondaryLowLimitPrice                             = 1221, /*!< 1221 (Price FIX.5.0) Refer to definition of LowLimitPrice(1148)*/
MaturityRuleID                                     = 1222, /*!< 1222 (String FIX.5.0) Allows maturity rule to be referenced via an identifier so that rules do not need to be explicitly enumerated*/
StrikeRuleID                                       = 1223, /*!< 1223 (String FIX.5.0) Allows strike rule to be referenced via an identifier so that rules do not need to be explicitly enumerated*/
LegUnitOfMeasureQty                                = 1224, /*!< 1224 (Qty FIX.5.0) Refer to definition of UnitOfMeasureQty(1147)*/
DerivativeOptPayAmount                             = 1225, /*!< 1225 (Amt FIX.5.0) Cash amount indicating the pay out associated with an option. For binary options this is a fixed amount*/
EndMaturityMonthYear                               = 1226, /*!< 1226 (MonthYear FIX.5.0) Ending maturity month year for an option class*/
ProductComplex                                     = 1227, /*!< 1227 (String FIX.5.0) Identifies an entire suite of products for a given market. In Futures this may be "interest rates", "agricultural", "equity indexes", etc.*/
DerivativeProductComplex                           = 1228, /*!< 1228 (String FIX.5.0) Refer to ProductComplex(1227)*/
MaturityMonthYearIncrement                         = 1229, /*!< 1229 (int FIX.5.0) Increment between successive maturities for an option class*/
SecondaryHighLimitPrice                            = 1230, /*!< 1230 (Price FIX.5.0) Refer to definition of HighLimitPrice(1149)*/
MinLotSize                                         = 1231, /*!< 1231 (Qty FIX.5.0) Minimum lot size allowed based on lot type specified in LotType(1093)*/
NoExecInstRules                                    = 1232, /*!< 1232 (NumInGroup FIX.5.0) Number of execution instructions*/
CommRate                                           = 1233, /*!< 1233 (float FIX.5.0SP2) The commission rate when Commission(12) is based on a percentage of quantity, amount per unit or a factor of "unit of measure". If the rate is a percentage, use the decimalized form, e.g. "0.05" for a 5% commission or "0.005" for 50 basis points.*/
NoLotTypeRules                                     = 1234, /*!< 1234 (NumInGroup FIX.5.0) Number of Lot Type Rules*/
NoMatchRules                                       = 1235, /*!< 1235 (NumInGroup FIX.5.0) Number of Match Rules*/
NoMaturityRules                                    = 1236, /*!< 1236 (NumInGroup FIX.5.0) Number of maturity rules in MarurityRules component block*/
NoOrdTypeRules                                     = 1237, /*!< 1237 (NumInGroup FIX.5.0) Number of order types*/
CommUnitOfMeasure                                  = 1238, /*!< 1238 (String FIX.5.0SP2) The commission rate unit of measure.*/
NoTimeInForceRules                                 = 1239, /*!< 1239 (NumInGroup FIX.5.0) Number of time in force techniques*/
SecondaryTradingReferencePrice                     = 1240, /*!< 1240 (Price FIX.5.0) Refer to definition for TradingReferencePrice(1150)*/
StartMaturityMonthYear                             = 1241, /*!< 1241 (MonthYear FIX.5.0) Starting maturity month year for an option class*/
FlexProductEligibilityIndicator                    = 1242, /*!< 1242 (Boolean FIX.5.0) Used to indicate if a product or group of product supports the creation of flexible securities*/
DerivFlexProductEligibilityIndicator               = 1243, /*!< 1243 (Boolean FIX.5.0) Refer to FlexProductEligibilityIndicator(1242)*/
FlexibleIndicator                                  = 1244, /*!< 1244 (Boolean FIX.5.0) Used to indicate a derivatives security that can be defined using flexible terms. The terms commonly permitted to be defined by market participants are expiration date and strike price. FlexibleIndicator is an alternative CFICode(461) Standard/Non-standard attribute.*/
TradingCurrency                                    = 1245, /*!< 1245 (Currency FIX.5.0) Used when the trading currency can differ from the price currency*/
DerivativeSecurityXMLLen                           = 1282, /*!< 1282 (Length FIX.5.0) Refer to definition SecurityXMLLen(1184)*/
DerivativeSecurityXML                              = 1283, /*!< 1283 (data FIX.5.0) Refer to definition of SecurityXML(1185)*/
DerivativeSecurityXMLSchema                        = 1284, /*!< 1284 (String FIX.5.0) Refer to definition of SecurityXMLSchema(1186)*/
NoDerivativeInstrumentParties                      = 1292, /*!< 1292 (NumInGroup FIX.5.0) Refer to definition of NoParties(453)*/
DerivativeInstrumentPartyID                        = 1293, /*!< 1293 (String FIX.5.0) Refer to definition of PartyID(448)*/
DerivativeInstrumentPartyIDSource                  = 1294, /*!< 1294 (char FIX.5.0) Refer to definition of PartyIDSource(447)*/
DerivativeInstrumentPartyRole                      = 1295, /*!< 1295 (int FIX.5.0) REfer to definition of PartyRole(452)*/
NoDerivativeInstrumentPartySubIDs                  = 1296, /*!< 1296 (NumInGroup FIX.5.0) Refer to definition for NoPartySubIDs(802)*/
DerivativeInstrumentPartySubID                     = 1297, /*!< 1297 (String FIX.5.0) Refer to definition for PartySubID(523)*/
DerivativeInstrumentPartySubIDType                 = 1298, /*!< 1298 (int FIX.5.0) Refer to definition for PartySubIDType(803)*/
DerivativeExerciseStyle                            = 1299, /*!< 1299 (int FIX.5.0) Type of exercise of a derivatives security*/
MarketSegmentID                                    = 1300, /*!< 1300 (String FIX.5.0) Identifies the market segment*/
MarketID                                           = 1301, /*!< 1301 (Exchange FIX.5.0) Identifies the market*/
MaturityMonthYearIncrementUnits                    = 1302, /*!< 1302 (int FIX.5.0) Unit of measure for the Maturity Month Year Increment*/
MaturityMonthYearFormat                            = 1303, /*!< 1303 (int FIX.5.0) Format used to generate the MaturityMonthYear for each option*/
StrikeExerciseStyle                                = 1304, /*!< 1304 (int FIX.5.0) Expiration Style for an option class:*/
SecondaryPriceLimitType                            = 1305, /*!< 1305 (int FIX.5.0) Describes the how the price limits are expressed*/
PriceLimitType                                     = 1306, /*!< 1306 (int FIX.5.0) Describes the how the price limits are expressed.*/
ExecInstValue                                      = 1308, /*!< 1308 (MultipleCharValue FIX.5.0) Indicates execution instructions that are valid for the specified market segment*/
NoTradingSessionRules                              = 1309, /*!< 1309 (NumInGroup FIX.5.0) Allows trading rules to be expressed by trading session*/
NoMarketSegments                                   = 1310, /*!< 1310 (NumInGroup FIX.5.0) Number of Market Segments on which a security may trade.*/
DerivativeInstrAttribType                          = 1313, /*!< 1313 (int FIX.5.0) Refer to definition of InstrAttribType(871)*/
DerivativeInstrAttribValue                         = 1314, /*!< 1314 (String FIX.5.0) Refer to definition of InstrAttribValue(872)*/
DerivativePriceUnitOfMeasure                       = 1315, /*!< 1315 (String FIX.5.0) Refer to definition for PriceUnitOfMeasure(1191)*/
DerivativePriceUnitOfMeasureQty                    = 1316, /*!< 1316 (Qty FIX.5.0) Refer to definition of PriceUnitOfMeasureQty(1192)*/
DerivativeSettlMethod                              = 1317, /*!< 1317 (String FIX.5.0) Settlement method for a contract or instrument. Additional values may be used with bilateral agreement.*/
DerivativePriceQuoteMethod                         = 1318, /*!< 1318 (String FIX.5.0) Refer to definition of PriceQuoteMethod(1196)*/
DerivativeValuationMethod                          = 1319, /*!< 1319 (String FIX.5.0) Refer to definition of ValuationMethod(1197).*/
DerivativeListMethod                               = 1320, /*!< 1320 (int FIX.5.0) Indicates whether instruments are pre-listed only or can also be defined via user request*/
DerivativeCapPrice                                 = 1321, /*!< 1321 (Price FIX.5.0) Refer to definition of CapPrice(1199)*/
DerivativeFloorPrice                               = 1322, /*!< 1322 (Price FIX.5.0) Refer to definition of FloorPrice(1200)*/
DerivativePutOrCall                                = 1323, /*!< 1323 (int FIX.5.0) Indicates whether an option contract is a put, call, chooser or undetermined.*/
ListUpdateAction                                   = 1324, /*!< 1324 (char FIX.5.0) If provided, then Instrument occurrence has explicitly changed*/
ParentMktSegmID                                    = 1325, /*!< 1325 (String FIX.5.0) Reference to a parent Market Segment. See MarketSegmentID(1300)*/
TradingSessionDesc                                 = 1326, /*!< 1326 (String FIX.5.0) Trading Session description*/
TradSesUpdateAction                                = 1327, /*!< 1327 (char FIX.5.0) Specifies the action taken for the specified trading sessions.*/
RejectText                                         = 1328, /*!< 1328 (String FIX.5.0) Identifies the reason for rejection.*/
FeeMultiplier                                      = 1329, /*!< 1329 (float FIX.5.0) This is a multiplier that Clearing (Fee system) will use to calculate fees and will be sent to the firms on their confirms.*/
UnderlyingLegSymbol                                = 1330, /*!< 1330 (String FIX.5.0) Refer to definition for Symbol(55)*/
UnderlyingLegSymbolSfx                             = 1331, /*!< 1331 (String FIX.5.0) Refer to definition for SymbolSfx(65)*/
UnderlyingLegSecurityID                            = 1332, /*!< 1332 (String FIX.5.0) Refer to definition for SecurityID(48)*/
UnderlyingLegSecurityIDSource                      = 1333, /*!< 1333 (String FIX.5.0) Refer to definition for SecurityIDSource(22)*/
NoUnderlyingLegSecurityAltID                       = 1334, /*!< 1334 (NumInGroup FIX.5.0) Refer to definition for NoSecurityAltID(454)*/
UnderlyingLegSecurityAltID                         = 1335, /*!< 1335 (String FIX.5.0) Refer to definition for SecurityAltID(455)*/
UnderlyingLegSecurityAltIDSource                   = 1336, /*!< 1336 (String FIX.5.0) Refer to definition for SecurityAltIDSource(456)*/
UnderlyingLegSecurityType                          = 1337, /*!< 1337 (String FIX.5.0) Refer to definition for SecurityType(167)*/
UnderlyingLegSecuritySubType                       = 1338, /*!< 1338 (String FIX.5.0) Refer to definition for SecuritySubType(762)*/
UnderlyingLegMaturityMonthYear                     = 1339, /*!< 1339 (MonthYear FIX.5.0) Refer to definition for MaturityMonthYear(200)*/
UnderlyingLegStrikePrice                           = 1340, /*!< 1340 (Price FIX.5.0) Refer to definition for StrikePrice(202)*/
UnderlyingLegSecurityExchange                      = 1341, /*!< 1341 (String FIX.5.0) Refer to definition for SecurityExchange(207)*/
NoOfLegUnderlyings                                 = 1342, /*!< 1342 (NumInGroup FIX.5.0) Number of Underlyings, Identifies the Underlying of the Leg*/
UnderlyingLegPutOrCall                             = 1343, /*!< 1343 (int FIX.5.0) Refer to definition for PutOrCall(201)*/
UnderlyingLegCFICode                               = 1344, /*!< 1344 (String FIX.5.0) Refer to definition for CFICode(461)*/
UnderlyingLegMaturityDate                          = 1345, /*!< 1345 (LocalMktDate FIX.5.0) Date of maturity.*/
ApplReqID                                          = 1346, /*!< 1346 (String FIX.5.0) Unique identifier for request*/
ApplReqType                                        = 1347, /*!< 1347 (int FIX.5.0) Type of Application Message Request being made.*/
ApplResponseType                                   = 1348, /*!< 1348 (int FIX.5.0) Used to indicate the type of acknowledgement being sent.*/
ApplTotalMessageCount                              = 1349, /*!< 1349 (int FIX.5.0) Total number of messages included in transmission.*/
ApplLastSeqNum                                     = 1350, /*!< 1350 (SeqNum FIX.5.0) Application sequence number of last message in transmission*/
NoApplIDs                                          = 1351, /*!< 1351 (NumInGroup FIX.5.0) Specifies number of application id occurrences*/
ApplResendFlag                                     = 1352, /*!< 1352 (Boolean FIX.5.0) Used to indicate that a message is being sent in response to an Application Message Request. It is possible for both ApplResendFlag and PossDupFlag to be set on the same message if the Sender's cache size is greater than zero and the message is being resent due to a session level resend request*/
ApplResponseID                                     = 1353, /*!< 1353 (String FIX.5.0) Identifier for the Applicaton Message Request Ack*/
ApplResponseError                                  = 1354, /*!< 1354 (int FIX.5.0) Used to return an error code or text associated with a response to an Application Request.*/
RefApplID                                          = 1355, /*!< 1355 (String FIX.5.0) Reference to the unique application identifier which corresponds to ApplID(1180) from the Application Sequence Group component*/
ApplReportID                                       = 1356, /*!< 1356 (String FIX.5.0) Identifier for the Application Sequence Reset*/
RefApplLastSeqNum                                  = 1357, /*!< 1357 (SeqNum FIX.5.0) Application sequence number of last message in transmission.*/
LegPutOrCall                                       = 1358, /*!< 1358 (int FIX.5.0) Indicates whether a leg option contract is a put, call, chooser or undetermined.*/
TotNoFills                                         = 1361, /*!< 1361 (int FIX.5.0) Total number of fill entries across all messages. Should be the sum of all NoFills(1362) in each message that has repeating list of fill entries related to the same ExecID(17). Used to support fragmentation.*/
FillExecID                                         = 1363, /*!< 1363 (String FIX.5.0) Refer to ExecID(17). Used when multiple partial fills are reported in single Execution Report. ExecID and FillExecID should not overlap,*/
FillPx                                             = 1364, /*!< 1364 (Price FIX.5.0) Price of Fill. Refer to LastPx(31).*/
FillQty                                            = 1365, /*!< 1365 (Qty FIX.5.0) Quantity of Fill. Refer to LastQty(32).*/
LegAllocID                                         = 1366, /*!< 1366 (String FIX.5.0) The AllocID(70) of an individual leg of a multileg order.*/
LegAllocSettlCurrency                              = 1367, /*!< 1367 (Currency FIX.5.0) Identifies settlement currency for the leg level allocation.*/
TradSesEvent                                       = 1368, /*!< 1368 (int FIX.5.0) Identifies an event related to a TradSesStatus(340). An event occurs and is gone, it is not a state that applies for a period of time.*/
MassActionReportID                                 = 1369, /*!< 1369 (String FIX.5.0) Unique identifier of Order Mass Cancel Report or Order Mass Action Report message as assigned by sell-side (broker, exchange, ECN)*/
NoNotAffectedOrders                                = 1370, /*!< 1370 (NumInGroup FIX.5.0) Number of not affected orders in the repeating group of order ids.*/
NotAffectedOrderID                                 = 1371, /*!< 1371 (String FIX.5.0) OrderID(37) of an order not affected by a mass cancel or mass action request.*/
NotAffOrigClOrdID                                  = 1372, /*!< 1372 (String FIX.5.0) ClOrdID(11) of an order not affected by a mass cancel or mass action request.*/
MassActionType                                     = 1373, /*!< 1373 (int FIX.5.0) Specifies the type of action requested*/
MassActionScope                                    = 1374, /*!< 1374 (int FIX.5.0) Specifies scope of Order Mass Action Request.*/
MassActionResponse                                 = 1375, /*!< 1375 (int FIX.5.0) Specifies the action taken by counterparty order handling system as a result of the action type indicated in MassActionType of the Order Mass Action Request.*/
MassActionRejectReason                             = 1376, /*!< 1376 (int FIX.5.0) Reason Order Mass Action Request was rejected*/
MultilegModel                                      = 1377, /*!< 1377 (int FIX.5.0) Specifies the type of multileg order. Defines whether the security is pre-defined or user-defined. Note that MultilegModel(1377)=2(User-defined, Non-Securitized, Multileg) does not apply for Securities.*/
MultilegPriceMethod                                = 1378, /*!< 1378 (int FIX.5.0) Code to represent how the multileg price is to be interpreted when applied to the legs.

(See Volume : "Glossary" for further value definitions)*/
LegVolatility                                      = 1379, /*!< 1379 (float FIX.5.0) Specifies the volatility of an instrument leg.*/
DividendYield                                      = 1380, /*!< 1380 (Percentage FIX.5.0) The continuously-compounded annualized dividend yield of the underlying(s) of an option. Used as a parameter to theoretical option pricing models.*/
LegDividendYield                                   = 1381, /*!< 1381 (Percentage FIX.5.0) Refer to definition for DividendYield(1380).*/
CurrencyRatio                                      = 1382, /*!< 1382 (float FIX.5.0) Specifies the currency ratio between the currency used for a multileg price and the currency used by the outright book defined by the leg. Example: Multileg quoted in EUR, outright leg in USD and 1 EUR = 0,7 USD then CurrencyRatio = 0.7*/
LegCurrencyRatio                                   = 1383, /*!< 1383 (float FIX.5.0) Specifies the currency ratio between the currency used for a multileg price and the currency used by the outright book defined by the leg. Example: Multileg quoted in EUR, outright leg in USD and 1 EUR = 0,7 USD then LegCurrencyRatio = 0.7*/
LegExecInst                                        = 1384, /*!< 1384 (MultipleCharValue FIX.5.0) Refer to ExecInst(18)

Same values as ExecInst(18)*/
ContingencyType                                    = 1385, /*!< 1385 (int FIX.5.0) Defines the type of contingency.*/
ListRejectReason                                   = 1386, /*!< 1386 (int FIX.5.0) Identifies the reason for rejection of a New Order List message. Note that OrdRejReason(103) is used if the rejection is based on properties of an individual order part of the List.*/
NoTrdRepIndicators                                 = 1387, /*!< 1387 (NumInGroup FIX.5.0) Number of trade reporting indicators*/
TrdRepPartyRole                                    = 1388, /*!< 1388 (int FIX.5.0) Identifies the type of party for trade reporting. Same values as PartyRole(452).*/
TrdRepIndicator                                    = 1389, /*!< 1389 (Boolean FIX.5.0) Specifies whether the trade should be reported (or not) to parties of the provided TrdRepPartyRole(1388). Used to override standard reporting behavior by the receiver of the trade report and thereby complements the PublTrdIndicator( tag1390).*/
TradePublishIndicator                              = 1390, /*!< 1390 (int FIX.5.0) Indicates if a trade should be or has been published via a market publication service. The indicator governs all publication services of the recipient. Replaces PublishTrdIndicator(852).*/
UnderlyingLegOptAttribute                          = 1391, /*!< 1391 (char FIX.5.0) Refer to definition of OptAttribute(206)*/
UnderlyingLegSecurityDesc                          = 1392, /*!< 1392 (String FIX.5.0) Refer to definition of SecurityDesc(107)*/
MarketReqID                                        = 1393, /*!< 1393 (String FIX.5.0) Unique ID of a Market Definition Request message.*/
MarketReportID                                     = 1394, /*!< 1394 (String FIX.5.0) Market Definition message identifier.*/
MarketUpdateAction                                 = 1395, /*!< 1395 (char FIX.5.0) Specifies the action taken for the specified MarketID(1301) + MarketSegmentID(1300).*/
MarketSegmentDesc                                  = 1396, /*!< 1396 (String FIX.5.0) Description or name of Market Segment*/
EncodedMktSegmDescLen                              = 1397, /*!< 1397 (Length FIX.5.0) Byte length of encoded (non-ASCII characters) EncodedMktSegmDesc(1324) field.*/
EncodedMktSegmDesc                                 = 1398, /*!< 1398 (data FIX.5.0) Encoded (non-ASCII characters) representation of the MarketSegmDesc(1396) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) representation should also be specified in the MarketSegmDesc field.*/
ApplNewSeqNum                                      = 1399, /*!< 1399 (SeqNum FIX.5.0) Used to specify a new application sequence number.*/
EncryptedPasswordMethod                            = 1400, /*!< 1400 (int FIX.5.0) Enumeration defining the encryption method used to encrypt password fields.

At this time there are no encryption methods defined by FPL.*/
EncryptedPasswordLen                               = 1401, /*!< 1401 (Length FIX.5.0) Length of the EncryptedPassword(1402) field*/
EncryptedPassword                                  = 1402, /*!< 1402 (data FIX.5.0) Encrypted password - encrypted via the method specified in the field EncryptedPasswordMethod(1400)*/
EncryptedNewPasswordLen                            = 1403, /*!< 1403 (Length FIX.5.0) Length of the EncryptedNewPassword(1404) field*/
EncryptedNewPassword                               = 1404, /*!< 1404 (data FIX.5.0) Encrypted new password - encrypted via the method specified in the field EncryptedPasswordMethod(1400)*/
UnderlyingLegMaturityTime                          = 1405, /*!< 1405 (TZTimeOnly FIX.5.0) Time of security's maturity expressed in local time with offset to UTC specified*/
RefApplExtID                                       = 1406, /*!< 1406 (int FIX.5.0) The extension pack number associated with an application message.*/
DefaultApplExtID                                   = 1407, /*!< 1407 (int FIX.5.0) The extension pack number that is the default for a FIX session.*/
DefaultCstmApplVerID                               = 1408, /*!< 1408 (String FIX.5.0) The default custom application version ID that is the default for a session.*/
SessionStatus                                      = 1409, /*!< 1409 (int FIX.5.0) Status of a FIX session*/
Nested4PartySubIDType                              = 1411, /*!< 1411 (int FIX.5.0) Refer to definition of PartySubIDType(803)*/
Nested4PartySubID                                  = 1412, /*!< 1412 (String FIX.5.0) Refer to definition of PartySubID(523)*/
NoNested4PartySubIDs                               = 1413, /*!< 1413 (NumInGroup FIX.5.0) Refer to definition of NoPartySubIDs(802)*/
NoNested4PartyIDs                                  = 1414, /*!< 1414 (NumInGroup FIX.5.0) Refer to definition of NoPartyIDs(453)*/
Nested4PartyID                                     = 1415, /*!< 1415 (String FIX.5.0) Refer to definition of PartyID(448)*/
Nested4PartyIDSource                               = 1416, /*!< 1416 (char FIX.5.0) Refer to definition of PartyIDSource(447)*/
Nested4PartyRole                                   = 1417, /*!< 1417 (int FIX.5.0) Refer to definition of PartyRole(452)*/
LegLastQty                                         = 1418, /*!< 1418 (Qty FIX.5.0) Fill quantity for the leg instrument*/
UnderlyingExerciseStyle                            = 1419, /*!< 1419 (int FIX.5.0) Type of exercise of a derivatives security*/
LegExerciseStyle                                   = 1420, /*!< 1420 (int FIX.5.0) Type of exercise of a derivatives security*/
LegPriceUnitOfMeasure                              = 1421, /*!< 1421 (String FIX.5.0) Refer to definition for PriceUnitOfMeasure(1191)*/
LegPriceUnitOfMeasureQty                           = 1422, /*!< 1422 (Qty FIX.5.0) Refer to definition of PriceUnitOfMeasureQty(1192)*/
UnderlyingUnitOfMeasureQty                         = 1423, /*!< 1423 (Qty FIX.5.0) Refer to definition of UnitOfMeasureQty(1147)*/
UnderlyingPriceUnitOfMeasure                       = 1424, /*!< 1424 (String FIX.5.0) Refer to definition for PriceUnitOfMeasure(1191)*/
UnderlyingPriceUnitOfMeasureQty                    = 1425, /*!< 1425 (Qty FIX.5.0) Refer to definition of PriceUnitOfMeasureQty(1192)*/
ApplReportType                                     = 1426, /*!< 1426 (int FIX.5.0SP2) Type of report*/
SideExecID                                         = 1427, /*!< 1427 (String FIX.5.0SP1) When reporting trades, used to reference the identifier of the execution (ExecID) being reported if different ExecIDs were assigned to each side of the trade.*/
OrderDelay                                         = 1428, /*!< 1428 (int FIX.5.0SP1) Time lapsed from order entry until match, based on the unit of time specified in OrderDelayUnit.  Default is seconds if OrderDelayUnit is not specified. Value = 0, indicates the aggressor (the initiating side of the trade).*/
OrderDelayUnit                                     = 1429, /*!< 1429 (int FIX.5.0SP1) Time unit in which the OrderDelay(1428) is expressed*/
VenueType                                          = 1430, /*!< 1430 (char FIX.5.0SP1) Identifies the type of venue where a trade was executed*/
RefOrdIDReason                                     = 1431, /*!< 1431 (int FIX.5.0SP1) The reason for updating the RefOrdID*/
OrigCustOrderCapacity                              = 1432, /*!< 1432 (int FIX.5.0SP1) The customer capacity for this trade at the time of the order/execution.

Primarily used by futures exchanges to indicate the CTICode (customer type indicator) as required by the US CFTC (Commodity Futures Trading Commission).*/
RefApplReqID                                       = 1433, /*!< 1433 (String FIX.5.0SP1) Used to reference a previously submitted ApplReqID (1346) from within a subsequent ApplicationMessageRequest(MsgType=BW)*/
ModelType                                          = 1434, /*!< 1434 (int FIX.5.0SP1) Type of pricing model used*/
ContractMultiplierUnit                             = 1435, /*!< 1435 (int FIX.5.0SP1) Indicates the type of multiplier being applied to the contract. Can be optionally used to further define what unit ContractMultiplier(tag 231) is expressed in.*/
LegContractMultiplierUnit                          = 1436, /*!< 1436 (int FIX.5.0SP1) "Indicates the type of multiplier being applied to the contract. Can be optionally used to further define what unit LegContractMultiplier(tag 614) is expressed in.

*/
UnderlyingContractMultiplierUnit                   = 1437, /*!< 1437 (int FIX.5.0SP1) Indicates the type of multiplier being applied to the contract. */
DerivativeContractMultiplierUnit                   = 1438, /*!< 1438 (int FIX.5.0SP1) Indicates the type of multiplier being applied to the contract. Can be optionally used to further define what unit DerivativeContractMultiplier(tag 1266)is expressed in.*/
FlowScheduleType                                   = 1439, /*!< 1439 (int FIX.5.0SP1) The industry standard flow schedule by which electricity or natural gas is traded. Schedules may exist by regions and on-peak and off-peak status, such as "Western Peak".*/
LegFlowScheduleType                                = 1440, /*!< 1440 (int FIX.5.0SP1) The industry standard flow schedule by which electricity or natural gas is traded. Schedules exist by regions and on-peak and off-peak status, such as "Western Peak".*/
UnderlyingFlowScheduleType                         = 1441, /*!< 1441 (int FIX.5.0SP1) The industry standard flow schedule by which electricity or natural gas is traded. Schedules exist by regions and on-peak and off-peak status, such as "Western Peak".*/
DerivativeFlowScheduleType                         = 1442, /*!< 1442 (int FIX.5.0SP1) The industry standard flow schedule by which electricity or natural gas is traded. Schedules exist by regions and on-peak and off-peak status, such as "Western Peak".*/
FillLiquidityInd                                   = 1443, /*!< 1443 (int FIX.5.0SP1) Indicator to identify whether this fill was a result of a liquidity provider providing or liquidity taker taking the liquidity. Applicable only for OrdStatus of Partial or Filled*/
SideLiquidityInd                                   = 1444, /*!< 1444 (int FIX.5.0SP1) Indicator to identify whether this fill was a result of a liquidity provider providing or liquidity taker taking the liquidity. Applicable only for OrdStatus of Partial or Filled.*/
NoRateSources                                      = 1445, /*!< 1445 (NumInGroup FIX.5.0SP1) Number of rate sources being specified.*/
RateSource                                         = 1446, /*!< 1446 (int FIX.5.0SP1) Identifies the source of rate information.

For FX, the reference source to be used for the FX spot rate.*/
RateSourceType                                     = 1447, /*!< 1447 (int FIX.5.0SP1) Indicates whether the rate source specified is a primary or secondary source.*/
ReferencePage                                      = 1448, /*!< 1448 (String FIX.5.0SP1) Identifies the reference "page" from the rate source.

For FX, the reference page to the spot rate to be used for the reference FX spot rate.



When RateSource(1446) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
RestructuringType                                  = 1449, /*!< 1449 (String FIX.5.0SP1) A category of CDS credit event in which the underlying bond experiences a restructuring.

Used to define a CDS instrument.*/
Seniority                                          = 1450, /*!< 1450 (String FIX.5.0SP1) Specifies which issue (underlying bond) will receive payment priority in the event of a default.

Used to define a CDS instrument. */
NotionalPercentageOutstanding                      = 1451, /*!< 1451 (Percentage FIX.5.0SP1) Indicates the notional percentage of the deal that is still outstanding based on the remaining components of the index.

Used to calculate the true value of a CDS trade or position.*/
OriginalNotionalPercentageOutstanding              = 1452, /*!< 1452 (Percentage FIX.5.0SP1) Used to reflect the Original value prior to the application of a credit event. See NotionalPercentageOutstanding(1451).*/
UnderlyingRestructuringType                        = 1453, /*!< 1453 (String FIX.5.0SP1) See RestructuringType(1449)*/
UnderlyingSeniority                                = 1454, /*!< 1454 (String FIX.5.0SP1) See Seniority(1450)*/
UnderlyingNotionalPercentageOutstanding            = 1455, /*!< 1455 (Percentage FIX.5.0SP1) See NotionalPercentageOutstanding(1451)*/
UnderlyingOriginalNotionalPercentageOutstanding    = 1456, /*!< 1456 (Percentage FIX.5.0SP1) See OriginalNotionalPercentageOutstanding(1452)*/
AttachmentPoint                                    = 1457, /*!< 1457 (Percentage FIX.5.0SP1) Lower bound percentage of the loss that the tranche can endure.*/
DetachmentPoint                                    = 1458, /*!< 1458 (Percentage FIX.5.0SP1) Upper bound percentage of the loss the tranche can endure.*/
UnderlyingAttachmentPoint                          = 1459, /*!< 1459 (Percentage FIX.5.0SP1) See AttachmentPoint(1457).*/
UnderlyingDetachmentPoint                          = 1460, /*!< 1460 (Percentage FIX.5.0SP1) See DetachmentPoint(1458).*/
NoTargetPartyIDs                                   = 1461, /*!< 1461 (NumInGroup FIX.5.0SP1) Identifies the number of target parties identified in a mass action.*/
TargetPartyID                                      = 1462, /*!< 1462 (String FIX.5.0SP1) PartyID value within an target party repeating group.*/
TargetPartyIDSource                                = 1463, /*!< 1463 (char FIX.5.0SP1) PartyIDSource value within an target party repeating group.

Same values as PartyIDSource (447)*/
TargetPartyRole                                    = 1464, /*!< 1464 (int FIX.5.0SP1) PartyRole value within an target party repeating group.

Same values as PartyRole (452)*/
SecurityListID                                     = 1465, /*!< 1465 (String FIX.5.0SP1) Specifies an identifier for a Security List*/
SecurityListRefID                                  = 1466, /*!< 1466 (String FIX.5.0SP1) Specifies a reference from one Security List to another. Used to support a hierarchy of Security Lists.*/
SecurityListDesc                                   = 1467, /*!< 1467 (String FIX.5.0SP1) Specifies a description or name of a Security List.*/
EncodedSecurityListDescLen                         = 1468, /*!< 1468 (Length FIX.5.0SP1) Byte length of encoded (non-ASCII characters) EncodedSecurityListDesc (tbd) field.*/
EncodedSecurityListDesc                            = 1469, /*!< 1469 (data FIX.5.0SP1) Encoded (non-ASCII characters) representation of the SecurityListDesc (1467) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the SecurityListDesc field.*/
SecurityListType                                   = 1470, /*!< 1470 (int FIX.5.0SP1) Specifies a type of Security List.*/
SecurityListTypeSource                             = 1471, /*!< 1471 (int FIX.5.0SP1) Specifies a specific source for a SecurityListType. Relevant when a certain type can be provided from various sources.*/
NewsID                                             = 1472, /*!< 1472 (String FIX.5.0SP1) Unique identifier for a News message*/
NewsCategory                                       = 1473, /*!< 1473 (int FIX.5.0SP1) Category of news mesage.*/
LanguageCode                                       = 1474, /*!< 1474 (Language FIX.5.0SP1) The national language in which the news item is provided.*/
NoNewsRefIDs                                       = 1475, /*!< 1475 (NumInGroup FIX.5.0SP1) Number of News reference items*/
NewsRefID                                          = 1476, /*!< 1476 (String FIX.5.0SP1) Reference to another News message identified by NewsID(1474).*/
NewsRefType                                        = 1477, /*!< 1477 (int FIX.5.0SP1) Type of reference to another News(35=B) message item.*/
StrikePriceDeterminationMethod                     = 1478, /*!< 1478 (int FIX.5.0SP1) Specifies how the strike price is determined at the point of option exercise. The strike may be fixed throughout the life of the option, set at expiration to the value of the underlying, set to the average value of the underlying , or set to the optimal value of the underlying. */
StrikePriceBoundaryMethod                          = 1479, /*!< 1479 (int FIX.5.0SP1) Specifies the boundary condition to be used for the strike price relative to the underlying price at the point of option exercise.*/
StrikePriceBoundaryPrecision                       = 1480, /*!< 1480 (Percentage FIX.5.0SP1) Used in combination with StrikePriceBoundaryMethod to specify the percentage of the strike price in relation to the  underlying price. The percentage is generally 100 or greater for puts and 100 or less for calls.*/
UnderlyingPriceDeterminationMethod                 = 1481, /*!< 1481 (int FIX.5.0SP1) Specifies how the underlying price is determined at the point of option exercise. The underlying price may be set to the current settlement price, set to a special reference, set to the optimal value of the underlying during the defined period ("Look-back") or set to the average value of the underlying during the defined period ("Asian option").*/
OptPayoutType                                      = 1482, /*!< 1482 (int FIX.5.0SP1) Indicates the type of valuation method or payout trigger for an in-the-money option.*/
NoComplexEvents                                    = 1483, /*!< 1483 (NumInGroup FIX.5.0SP1) Number of complex event occurrences.*/
ComplexEventType                                   = 1484, /*!< 1484 (int FIX.5.0SP1) Identifies the type of complex event.*/
ComplexOptPayoutAmount                             = 1485, /*!< 1485 (Amt FIX.5.0SP1) Cash amount indicating the pay out associated with an event. For binary options this is a fixed amount.*/
ComplexEventPrice                                  = 1486, /*!< 1486 (Price FIX.5.0SP1) Specifies the price at which the complex event takes effect. Impact of the event price is determined by the ComplexEventType(1484).*/
ComplexEventPriceBoundaryMethod                    = 1487, /*!< 1487 (int FIX.5.0SP1) Specifies the boundary condition to be used for the event price relative to the underlying price at the point the complex event outcome takes effect as determined by the ComplexEventPriceTimeType.*/
ComplexEventPriceBoundaryPrecision                 = 1488, /*!< 1488 (Percentage FIX.5.0SP1) Used in combination with ComplexEventPriceBoundaryMethod to specify the percentage of the strike price in relation to the  underlying price. The percentage is generally 100 or greater for puts and 100 or less for calls.*/
ComplexEventPriceTimeType                          = 1489, /*!< 1489 (int FIX.5.0SP1) Specifies when the complex event outcome takes effect. The outcome of a complex event is a payout or barrier action as specified by the ComplexEventType(1484).*/
ComplexEventCondition                              = 1490, /*!< 1490 (int FIX.5.0SP1) Specifies the condition between complex events when more than one event is specified.  

Multiple barrier events would use an "or" condition since only one can be effective at a given time. A set of digital range events would use an "and" condition since both conditions must be in effect for a payout to result.*/
NoComplexEventDates                                = 1491, /*!< 1491 (NumInGroup FIX.5.0SP1) Number of complex event date occurrences for a given complex event.*/
ComplexEventStartDate                              = 1492, /*!< 1492 (UTCDateOnly FIX.5.0SP1) Specifies the start date of the date range on which a complex event is effective. The start date will be set equal to the end date for single day events such as Bermuda options

ComplexEventStartDate must always be less than or equal to ComplexEventEndDate.*/
ComplexEventEndDate                                = 1493, /*!< 1493 (UTCDateOnly FIX.5.0SP1) Specifies the end date of the date range on which a complex event is effective. The start date will be set equal to the end date for single day events such as Bermuda options

ComplexEventEndDate must always be greater than or equal to ComplexEventStartDate.*/
NoComplexEventTimes                                = 1494, /*!< 1494 (NumInGroup FIX.5.0SP1) Number of complex event time occurrences for a given complex event date

The default in case of an absence of time fields is 00:00:00-23:59:59.*/
ComplexEventStartTime                              = 1495, /*!< 1495 (UTCTimeOnly FIX.5.0SP1) Specifies the start time of the time range on which a complex event date is effective. 

ComplexEventStartTime must always be less than or equal to ComplexEventEndTime.*/
ComplexEventEndTime                                = 1496, /*!< 1496 (UTCTimeOnly FIX.5.0SP1) Specifies the end time of the time range on which a complex event date is effective.

ComplexEventEndTime must always be greater than or equal to ComplexEventStartTime.*/
StreamAsgnReqID                                    = 1497, /*!< 1497 (String FIX.5.0SP1) Unique identifier for the stream assignment request provided by the requester.*/
StreamAsgnReqType                                  = 1498, /*!< 1498 (int FIX.5.0SP1) Type of stream assignment request.*/
NoAsgnReqs                                         = 1499, /*!< 1499 (NumInGroup FIX.5.0SP1) Number of assignment requests.*/
MDStreamID                                         = 1500, /*!< 1500 (String FIX.5.0SP1) The identifier or name of the price stream.*/
StreamAsgnRptID                                    = 1501, /*!< 1501 (String FIX.5.0SP1) Unique identifier of the stream assignment report provided by the respondent.*/
StreamAsgnRejReason                                = 1502, /*!< 1502 (int FIX.5.0SP1) Reason code for stream assignment request reject.*/
StreamAsgnAckType                                  = 1503, /*!< 1503 (int FIX.5.0SP1) Type of acknowledgement.*/
RelSymTransactTime                                 = 1504, /*!< 1504 (UTCTimestamp FIX.5.0SP1) See TransactTime(60)*/
PartyDetailsListRequestID                          = 1505, /*!< 1505 (String FIX.5.0SP2) Unique identifier for PartyDetailsListRequest.*/
SideTradeID                                        = 1506, /*!< 1506 (String FIX.5.0SP2) Used to represent the trade ID for each side of the trade assigned by an intermediary.*/
SideOrigTradeID                                    = 1507, /*!< 1507 (String FIX.5.0SP2) Used to capture the original trade id for each side of a trade undergoing novation to a standardized model.*/
NoRequestedPartyRoles                              = 1508, /*!< 1508 (NumInGroup FIX.5.0SP2) Number of requested party roles.*/
RequestedPartyRole                                 = 1509, /*!< 1509 (int FIX.5.0SP2) Identifies the type or role of party that has been requested.*/
PartyDetailsListReportID                           = 1510, /*!< 1510 (String FIX.5.0SP2) Identifier for the PartyDetailsListReport and the PartyDetailsListUpdateReport.*/
RequestResult                                      = 1511, /*!< 1511 (int FIX.5.0SP2) Result of a request as identified by the appropriate request ID field*/
TotNoParties                                       = 1512, /*!< 1512 (int FIX.5.0SP2) Total number of PartyListGrp returned.*/
DocumentationText                                  = 1513, /*!< 1513 (String FIX.5.0SP2) A sentence or phrase pertenant to the trade, not a reference to an external document. E.g. "To be registered with the U.S. Environmental Protection Agency, Acid Rain Division, SO2 Allowance Tracking System"*/
NoPartyRelationships                               = 1514, /*!< 1514 (NumInGroup FIX.5.0SP2) Number of party relationships.*/
PartyRelationship                                  = 1515, /*!< 1515 (int FIX.5.0SP2) Used to specify the type of the party relationship.*/
NoPartyDetailAltID                                 = 1516, /*!< 1516 (NumInGroup FIX.5.0SP2) Number of party alternative identifiers.*/
PartyDetailAltID                                   = 1517, /*!< 1517 (String FIX.5.0SP2) An alternate party identifier for the party specified in PartyDetailID(1691)*/
PartyDetailAltIDSource                             = 1518, /*!< 1518 (char FIX.5.0SP2) Identifies the source of the PartyDetailAltID(1517) value.*/
NoPartyDetailAltSubIDs                             = 1519, /*!< 1519 (NumInGroup FIX.5.0SP2) Number of party detail alternate sub-identifiers.*/
PartyDetailAltSubID                                = 1520, /*!< 1520 (String FIX.5.0SP2) Sub-identifier for the party specified in PartyDetailAltID(1517).*/
PartyDetailAltSubIDType                            = 1521, /*!< 1521 (int FIX.5.0SP2) Type of PartyDetailAltSubID(1520) value.*/
DifferentialPrice                                  = 1522, /*!< 1522 (PriceOffset FIX.5.0SP2) Used to specify the differential price when reporting the individual leg of a spread trade. Both leg price and differential price may be provided on such a report. Note that MultiLegReportingType(442) will be set to 2 (Individual leg of a multi-leg security) in this case.

Also used in pricing Trade at Settlement (TAS) and Trade At Marker (TAM) contracts for which the value is the negotiated currency offset either at settlement (TAS) or at time specified in the product definition (TAM). The final contract price is specified in LastPx(31).*/
TrdAckStatus                                       = 1523, /*!< 1523 (int FIX.5.0SP2) Used to indicate the status of the trade submission (not the trade report)*/
PriceQuoteCurrency                                 = 1524, /*!< 1524 (Currency FIX.5.0SP2) Default currency in which the price is quoted. Defined at the instrument level. Used in place of Currency (tag 15) to express the currency of a product when the former is implemented as the FX dealt currency.*/
EncodedDocumentationTextLen                        = 1525, /*!< 1525 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedDocumentationText(1527) field.*/
UnderlyingPriceQuoteCurrency                       = 1526, /*!< 1526 (Currency FIX.5.0SP2) Default currency in which the price is quoted. Defined at the instrument level. Used in place of Currency (tag 15) to express the currency of a product when the former is implemented as the FX dealt currency.*/
EncodedDocumentationText                           = 1527, /*!< 1527 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the DocumentationText(1513) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the DocumentationText(1513) field.*/
LegPriceQuoteCurrency                              = 1528, /*!< 1528 (Currency FIX.5.0SP2) Default currency in which the price is quoted. Defined at the instrument level. Used in place of Currency (tag 15) to express the currency of a product when the former is implemented as the FX dealt currency.*/
NoRiskLimitTypes                                   = 1529, /*!< 1529 (NumInGroup FIX.5.0SP2) Number of risk limits with associated warning levels.*/
RiskLimitType                                      = 1530, /*!< 1530 (int FIX.5.0SP2) Used to specify the type of risk limit amount or position limit quantity or margin requirement amounts.*/
RiskLimitAmount                                    = 1531, /*!< 1531 (Amt FIX.5.0SP2) Specifies the risk limit amount.*/
RiskLimitCurrency                                  = 1532, /*!< 1532 (Currency FIX.5.0SP2) Used to specify the currency of the risk limit amount.*/
RiskLimitPlatform                                  = 1533, /*!< 1533 (String FIX.5.0SP2) The area to which risk limit is applicable. This can be a trading platform or an offering.*/
NoRiskInstrumentScopes                             = 1534, /*!< 1534 (NumInGroup FIX.5.0SP2) Number of risk instrument scopes.*/
InstrumentScopeOperator                            = 1535, /*!< 1535 (int FIX.5.0SP2) Operator to perform on the instrument(s) specified*/
InstrumentScopeSymbol                              = 1536, /*!< 1536 (String FIX.5.0SP2) Used to limit instrument scope to specified symbol.

See Symbol(55) field for description.*/
InstrumentScopeSymbolSfx                           = 1537, /*!< 1537 (String FIX.5.0SP2) Used to limit instrument scope to specified symbol suffix.

See SymbolSfx(65) field for description.*/
InstrumentScopeSecurityID                          = 1538, /*!< 1538 (String FIX.5.0SP2) Used to limit instrument scope to specified security identifier.

See SecurityID(48) field for description.*/
InstrumentScopeSecurityIDSource                    = 1539, /*!< 1539 (String FIX.5.0SP2) Used to limit instrument scope to specified security identifier source.

See SecurityIDSource(22) field for description.*/
NoInstrumentScopeSecurityAltID                     = 1540, /*!< 1540 (NumInGroup FIX.5.0SP2) Number of alternate security identifier for the specified InstrumentScopeSecurityID(1538).*/
InstrumentScopeSecurityAltID                       = 1541, /*!< 1541 (String FIX.5.0SP2) Used to limit instrument scope to specified security alternate identifier.

See SecurityAltID(455) field for description.*/
InstrumentScopeSecurityAltIDSource                 = 1542, /*!< 1542 (String FIX.5.0SP2) Used to limit instrument scope to specified security alternate identifier source.

See SecurityAltIDSource(456) field for description.*/
InstrumentScopeProduct                             = 1543, /*!< 1543 (int FIX.5.0SP2) Used to limit instrument scope to specified instrument product category.

See Product (460) field for description.*/
InstrumentScopeProductComplex                      = 1544, /*!< 1544 (String FIX.5.0SP2) Used to limit instrument scope to specified product complex.

See ProductComplex(1227) field for description.*/
InstrumentScopeSecurityGroup                       = 1545, /*!< 1545 (String FIX.5.0SP2) Used to limit instrument scope to specified security group.

See SecurityGroup(1151) field for description.*/
InstrumentScopeCFICode                             = 1546, /*!< 1546 (String FIX.5.0SP2) Used to limit instrument scope to specified CFICode.

See CFICode(461) field for description.*/
InstrumentScopeSecurityType                        = 1547, /*!< 1547 (String FIX.5.0SP2) Used to limit instrument scope to specified security type.

See SecurityType(167) field for description).*/
InstrumentScopeSecuritySubType                     = 1548, /*!< 1548 (String FIX.5.0SP2) Used to limit instrument scope to specified security sub-type.

See SecuritySubType(762) field for description.*/
InstrumentScopeMaturityMonthYear                   = 1549, /*!< 1549 (MonthYear FIX.5.0SP2) Used to limit instrument scope to specified maturity month and year.

See MaturityMonthYear(200) field for description.*/
InstrumentScopeMaturityTime                        = 1550, /*!< 1550 (TZTimeOnly FIX.5.0SP2) Used to limit instrument scope to specified maturity time.

See MaturityTime(1079) field for description.*/
InstrumentScopeRestructuringType                   = 1551, /*!< 1551 (String FIX.5.0SP2) Used to limit instrument scope to specified restructuring type.

See RestructuringType(1449) field for description.*/
InstrumentScopeSeniority                           = 1552, /*!< 1552 (String FIX.5.0SP2) Used to limit instrument scope to specified seniority type.

See Seniority(1450) field for description.*/
InstrumentScopePutOrCall                           = 1553, /*!< 1553 (int FIX.5.0SP2) Used to limit instrument scope to puts or calls.

See PutOrCall(201) field for description.*/
InstrumentScopeFlexibleIndicator                   = 1554, /*!< 1554 (Boolean FIX.5.0SP2) Used to limit instrument scope to securities that can be defined using flexible terms or not.

See FlexibleIndicator(1244) field for description.*/
InstrumentScopeCouponRate                          = 1555, /*!< 1555 (Percentage FIX.5.0SP2) Used to limit instrument scope to specified coupon rate.

See CouponRate(223) field for description.*/
InstrumentScopeSecurityDesc                        = 1556, /*!< 1556 (String FIX.5.0SP2) Used to limit instrument scope to specified security description.

See SecurityDesc(107) field for description.*/
InstrumentScopeSettlType                           = 1557, /*!< 1557 (String FIX.5.0SP2) Used to limit instrument scope to specified settlement type.

See SettlType(63) field for description.*/
RiskInstrumentMultiplier                           = 1558, /*!< 1558 (float FIX.5.0SP2) Multiplier applied to the transaction amount for comparison with risk limits. Default if not specified is 1.0.*/
NoRiskWarningLevels                                = 1559, /*!< 1559 (NumInGroup FIX.5.0SP2) Number of risk warning levels.*/
RiskWarningLevelPercent                            = 1560, /*!< 1560 (Percentage FIX.5.0SP2) Percent of risk limit at which a warning is issued.*/
RiskWarningLevelName                               = 1561, /*!< 1561 (String FIX.5.0SP2) Name or error message associated with the risk warning level.*/
NoRelatedPartyDetailID                             = 1562, /*!< 1562 (NumInGroup FIX.5.0SP2) Number of related party detail identifiers.*/
RelatedPartyDetailID                               = 1563, /*!< 1563 (String FIX.5.0SP2) Party identifier for the party related to the party specified in PartyDetailID(1691).*/
RelatedPartyDetailIDSource                         = 1564, /*!< 1564 (char FIX.5.0SP2) Identifies the source of the RelatedPartyDetailID(1563).*/
RelatedPartyDetailRole                             = 1565, /*!< 1565 (int FIX.5.0SP2) Identifies the type or role of the RelatedPartyDetailID(1563) specified.*/
NoRelatedPartyDetailSubIDs                         = 1566, /*!< 1566 (NumInGroup FIX.5.0SP2) Number of related party detail sub-identifiers.*/
RelatedPartyDetailSubID                            = 1567, /*!< 1567 (String FIX.5.0SP2) Sub-identifier for the party specified in RelatedPartyID(1563).*/
RelatedPartyDetailSubIDType                        = 1568, /*!< 1568 (int FIX.5.0SP2) Type of RelatedPartyDetailSubID(1567) value.*/
NoRelatedPartyDetailAltID                          = 1569, /*!< 1569 (NumInGroup FIX.5.0SP2) Number of related party detail alternate identifiers.*/
RelatedPartyDetailAltID                            = 1570, /*!< 1570 (String FIX.5.0SP2) An alternate party identifier for the party specified in RelatedPartyID(1563).*/
RelatedPartyDetailAltIDSource                      = 1571, /*!< 1571 (char FIX.5.0SP2) Identifies the source of the RelatedPartyDetailAltID(1570) value.*/
NoRelatedPartyDetailAltSubIDs                      = 1572, /*!< 1572 (NumInGroup FIX.5.0SP2) Number of related party detail alternate sub-identifiers.*/
RelatedPartyDetailAltSubID                         = 1573, /*!< 1573 (String FIX.5.0SP2) Sub-identifier for the party specified in RelatedPartyDetailAltID(1570).*/
RelatedPartyDetailAltSubIDType                     = 1574, /*!< 1574 (int FIX.5.0SP2) Type of RelatedPartyDetailAltSubID(1573) value.*/
SwapSubClass                                       = 1575, /*!< 1575 (String FIX.5.0SP2) The sub-classification or notional schedule type of the swap.*/
DerivativePriceQuoteCurrency                       = 1576, /*!< 1576 (Currency FIX.5.0SP2) Default currency in which the price is quoted. Defined at the instrument level. Used in place of Currency (tag 15) to express the currency of a product when the former is implemented as the FX dealt currency.*/
SettlRateIndex                                     = 1577, /*!< 1577 (String FIX.5.0SP2) In an outright or forward commodity trade that is cash settled this is the index used to determine the cash payment.*/
EncodedEventTextLen                                = 1578, /*!< 1578 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedEventText(868) fied.*/
EncodedEventText                                   = 1579, /*!< 1579 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the EventText(868) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the EventText(868) field.*/
SettlRateIndexLocation                             = 1580, /*!< 1580 (String FIX.5.0SP2) This is an optional qualifying attribute of SettlRateIndex(1577) such as the delivery zone for an electricity contract.*/
OptionExpirationDesc                               = 1581, /*!< 1581 (String FIX.5.0SP2) Description of the option expiration.*/
NoSecurityClassifications                          = 1582, /*!< 1582 (NumInGroup FIX.5.0SP2) Number of Security Classifications.*/
SecurityClassificationReason                       = 1583, /*!< 1583 (int FIX.5.0SP2) Allows classification of instruments according to a set of high level reasons. Classification reasons describe the classes in which the instrument participates.*/
SecurityClassificationValue                        = 1584, /*!< 1584 (String FIX.5.0SP2) Specifies the product classification value which further details the manner in which the instrument participates in the class.*/
PosAmtReason                                       = 1585, /*!< 1585 (int FIX.5.0SP2) Specifies the reason for an amount type when reported on a position. Useful when multiple instances of the same amount type are reported.*/
NoLegPosAmt                                        = 1586, /*!< 1586 (NumInGroup FIX.5.0SP2) Number of TrdInstrmtLegPosAmt values.*/
LegPosAmt                                          = 1587, /*!< 1587 (Amt FIX.5.0SP2) Leg position amount.*/
LegPosAmtType                                      = 1588, /*!< 1588 (String FIX.5.0SP2) Type of leg position amount.*/
LegPosCurrency                                     = 1589, /*!< 1589 (Currency FIX.5.0SP2) Leg position currency.*/
LegPosAmtReason                                    = 1590, /*!< 1590 (int FIX.5.0SP2) Specifies the reason for an amount type when reported on a position. Useful when multiple instances of the same amount type are reported.*/
LegQtyType                                         = 1591, /*!< 1591 (int FIX.5.0SP2) Type of quantity specified in LegQty field. LegContractMultiplier (614) is required when LegQtyType = 1 (Contracts). LegUnitOfMeasure (tag 999) and LegTimeUnit (tag 1001) are required when LegQtyType = 2 (Units of Measure per Time Unit). LegQtyType can be different for each leg. */
DiscountFactor                                     = 1592, /*!< 1592 (float FIX.5.0SP2) Used to calculate the present value of an amount to be paid in the future.*/
ParentAllocID                                      = 1593, /*!< 1593 (String FIX.5.0SP2) Contains the IndividualAllocId (tag 467) value of the allocation that is being offset as a result of a new allocation. This would be an optional field that would only be populated in the case of an allocation of an allocation (as well as any subsequent allocations). This wouldn’t be populated for an initial allocation since an allocation id is not supplied on default (initial) allocations.*/
LegSecurityGroup                                   = 1594, /*!< 1594 (String FIX.5.0SP2) Represents the product group of a leg.This is useful in conveying multi-leg instruments where the legs may participate in separate security groups.*/
PositionContingentPrice                            = 1595, /*!< 1595 (Price FIX.5.0SP2) Risk adjusted price used to calculate variation margin on a position.*/
ClearingTradePrice                                 = 1596, /*!< 1596 (Price FIX.5.0SP2) Alternate clearing price*/
SideClearingTradePrice                             = 1597, /*!< 1597 (Price FIX.5.0SP2) Alternate clearing price for the side being reported.*/
SideClearingTradePriceType                         = 1598, /*!< 1598 (int FIX.5.0SP2) Indicates to recipient whether trade is clearing at execution prices LastPx(tag 31) or alternate clearing prices SideClearingTradePrice(tag 1597).*/
SidePriceDifferential                              = 1599, /*!< 1599 (Price FIX.5.0SP2) Price Differential between the front and back leg of a spread or complex instrument.*/
FIXEngineName                                      = 1600, /*!< 1600 (String FIX.5.0SP2) Provides the name of the infrastructure component being used for session level communication. Normally this would be the FIX Engine or FIX Gateway product name.*/
FIXEngineVersion                                   = 1601, /*!< 1601 (String FIX.5.0SP2) Provides the version of the infrastructure component.*/
FIXEngineVendor                                    = 1602, /*!< 1602 (String FIX.5.0SP2) Provides the name of the vendor providing the infrastructure component.*/
ApplicationSystemName                              = 1603, /*!< 1603 (String FIX.5.0SP2) Provides the name of the application system being used to generate FIX application messages. This will normally be a trading system, OMS, or EMS.*/
ApplicationSystemVersion                           = 1604, /*!< 1604 (String FIX.5.0SP2) Provides the version of the application system being used to initiate FIX application messages.*/
ApplicationSystemVendor                            = 1605, /*!< 1605 (String FIX.5.0SP2) Provides the vendor of the application system.*/
NumOfSimpleInstruments                             = 1606, /*!< 1606 (int FIX.5.0SP2) Represents the total number of simple instruments that make up a multi-legged security. Complex spread instruments may be constructed of legs which themselves are multi-leg instruments.*/
SecurityRejectReason                               = 1607, /*!< 1607 (int FIX.5.0SP2) Identifies the reason  a security definition request is being rejected.*/
InitialDisplayQty                                  = 1608, /*!< 1608 (Qty FIX.5.0SP2) Used to convey the initially requested display quantity specified in DisplayQty(1138) on order entry and modification messages in ExecutionReport message. Applicable only in ExecutionReport message where DisplayQty(1138) is the currently displayed quantity and the requested display quantity of the order also needs to be conveyed. The values of the two fields are different as soon as the order is partially filled and also after a refresh of the order whenever DisplayMethod(1084) is not 1=Initial.*/
ThrottleStatus                                     = 1609, /*!< 1609 (int FIX.5.0SP2) Indicates whether a message was queued as a result of throttling.*/
NoThrottles                                        = 1610, /*!< 1610 (NumInGroup FIX.5.0SP2) Indicates number of repeating groups to follow.*/
ThrottleAction                                     = 1611, /*!< 1611 (int FIX.5.0SP2) Action to take should throttle limit be exceeded.*/
ThrottleType                                       = 1612, /*!< 1612 (int FIX.5.0SP2) Type of throttle.*/
ThrottleNoMsgs                                     = 1613, /*!< 1613 (int FIX.5.0SP2) Maximum number of messages allowed by the throttle. May be a rate limit or a limit on the number of outstanding requests.*/
ThrottleTimeInterval                               = 1614, /*!< 1614 (int FIX.5.0SP2) Value of the time interval in which the rate throttle is applied.*/
ThrottleTimeUnit                                   = 1615, /*!< 1615 (int FIX.5.0SP2) Units in which ThrottleTimeInterval is expressed. Uses same enumerations as OrderDelayUnit(1429).*/
InstrumentScopeSecurityExchange                    = 1616, /*!< 1616 (Exchange FIX.5.0SP2) Used to limit instrument scope to specified security exchange.

See SecurityExchange(207) field for description.*/
StreamAsgnType                                     = 1617, /*!< 1617 (int FIX.5.0SP1) The type of assignment being affected in the Stream Assignment Report.*/
NoThrottleMsgType                                  = 1618, /*!< 1618 (NumInGroup FIX.5.0SP2) Number of ThrottleMsgType fields.*/
ThrottleMsgType                                    = 1619, /*!< 1619 (String FIX.5.0SP2) The MsgType (35) of the FIX message being referenced.*/
InstrumentScopeEncodedSecurityDescLen              = 1620, /*!< 1620 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) InstrumentScopeEncodedSecurityDesc (1621) field*/
InstrumentScopeEncodedSecurityDesc                 = 1621, /*!< 1621 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the InstrumentScopeSecurityDesc (1556) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the InstrumentScopeSecurityDesc field.*/
FillYieldType                                      = 1622, /*!< 1622 (String FIX.5.0SP2) Yield Type, using same values as YieldType (235)*/
FillYield                                          = 1623, /*!< 1623 (Percentage FIX.5.0SP2) Yield Percentage, using same values as Yield (236)*/
NoMatchInst                                        = 1624, /*!< 1624 (NumInGroup FIX.5.0SP2) Number of Instructions in the &lt;MatchingInstructions&gt; repeating group.*/
MatchInst                                          = 1625, /*!< 1625 (int FIX.5.0SP2) Matching Instruction for the order.*/
MatchAttribTagID                                   = 1626, /*!< 1626 (TagNum FIX.5.0SP2) Existing FIX field to be applied as a matching criteria to the instruction, bilaterally agreed between parties.*/
MatchAttribValue                                   = 1627, /*!< 1627 (String FIX.5.0SP2) Value of MatchAttribTagID(1626) on which to apply the matching instruction.*/
TriggerScope                                       = 1628, /*!< 1628 (int FIX.5.0SP2) Defines the scope of TriggerAction(1101) when it is set to "cancel" (3).*/
ExposureDuration                                   = 1629, /*!< 1629 (int FIX.5.0SP2) This is the time in seconds of a "Good for Time" (GFT) TimeInForce.

Positive integer value which represents the time is seconds in which the new order remains active in the market before it is automatically cancelled (e.g. expired).

Bi-lateral agreements will dictate the maximum value of this field. It is assumed that most systems will impose a max limit of 86,400 seconds (i.e. 24 hours).

For Quotes: The period of time a quoted price is tradable(i.e. on-the-wire) before it becomes indicative (i.e. off-the-wire).*/
NoLimitAmts                                        = 1630, /*!< 1630 (NumInGroup FIX.5.0SP2) The number of limit amount entries.*/
LimitAmtType                                       = 1631, /*!< 1631 (int FIX.5.0SP2) Identifies the type of limit amount expressed in LastLimitAmt(1632) and LimitAmtRemaining(1633).*/
LastLimitAmt                                       = 1632, /*!< 1632 (Amt FIX.5.0SP2) The amount that has been drawn down against the counterparty for a given trade.  The type of limit is specified in LimitAmtType(1631).

						Bilateral agreements dictate the units and maximum value of this field.*/
LimitAmtRemaining                                  = 1633, /*!< 1633 (Amt FIX.5.0SP2) The remaining limit amount available between the counterparties.  The type of limit is specified in LimitAmtType(1631).

						Bilateral agreements dictate the units and maximum value of this field.*/
LimitAmtCurrency                                   = 1634, /*!< 1634 (Currency FIX.5.0SP2) Indicates the currency that the limit amount is specified in.  See Currency(15) for additional description and valid values.*/
MarginReqmtInqID                                   = 1635, /*!< 1635 (String FIX.5.0SP2) Unique identifier of the MarginRequirementInquiry.*/
NoMarginReqmtInqQualifier                          = 1636, /*!< 1636 (NumInGroup FIX.5.0SP2) Number of margin requirement inquiry qualifiers.*/
MarginReqmtInqQualifier                            = 1637, /*!< 1637 (int FIX.5.0SP2) Qualifier for MarginRequirementInquiry to identify a specific report.*/
MarginReqmtRptType                                 = 1638, /*!< 1638 (int FIX.5.0SP2) Type of MarginRequirementReport.*/
MarginClass                                        = 1639, /*!< 1639 (String FIX.5.0SP2) Identifier for group of instruments with similar risk profile.*/
MarginReqmtInqStatus                               = 1640, /*!< 1640 (int FIX.5.0SP2) Status of MarginRequirementInquiry.*/
MarginReqmtInqResult                               = 1641, /*!< 1641 (int FIX.5.0SP2) Result returned in response to MarginRequirementInquiry.*/
MarginReqmtRptID                                   = 1642, /*!< 1642 (String FIX.5.0SP2) Identifier for the MarginRequirementReport message.*/
NoMarginAmt                                        = 1643, /*!< 1643 (NumInGroup FIX.5.0SP2) Number of margin requirement amounts.*/
MarginAmtType                                      = 1644, /*!< 1644 (int FIX.5.0SP2) Type of margin requirement amount being specified.*/
MarginAmt                                          = 1645, /*!< 1645 (Amt FIX.5.0SP2) Amount of margin requirement.*/
MarginAmtCcy                                       = 1646, /*!< 1646 (Currency FIX.5.0SP2) Currency of the MarginAmt(1645).*/
NoRelatedInstruments                               = 1647, /*!< 1647 (NumInGroup FIX.5.0SP2) Number of related instruments*/
RelatedInstrumentType                              = 1648, /*!< 1648 (int FIX.5.0SP2) The type of instrument relationship*/
RelatedSymbol                                      = 1649, /*!< 1649 (String FIX.5.0SP2) Ticker symbol of the related security. Common "human understood" representation of the security. */
RelatedSecurityID                                  = 1650, /*!< 1650 (String FIX.5.0SP2) Related security identifier value of RelatedSecurityIDSource(1651) type.*/
RelatedSecurityIDSource                            = 1651, /*!< 1651 (String FIX.5.0SP2) Identifies class or source of the RelatedSecurityID (1650) value. */
RelatedSecurityType                                = 1652, /*!< 1652 (String FIX.5.0SP2) Security type of the related instrument. */
RelatedMaturityMonthYear                           = 1653, /*!< 1653 (MonthYear FIX.5.0SP2) Expiration date for the related instrument contract.*/
CoveredQty                                         = 1654, /*!< 1654 (Qty FIX.5.0SP2) Used to specify the portion of the short contract quantity that is considered covered (e.g. used for short option position).*/
MarketMakerActivity                                = 1655, /*!< 1655 (int FIX.5.0SP2) Indicates market maker participation in security.*/
NoInstrumentScopes                                 = 1656, /*!< 1656 (NumInGroup FIX.5.0SP2) Number of instrument scopes.*/
NoRequestingPartyIDs                               = 1657, /*!< 1657 (NumInGroup FIX.5.0SP2) Number of requesting party identifiers.*/
RequestingPartyID                                  = 1658, /*!< 1658 (String FIX.5.0SP2) Party identifier for the requesting party.*/
RequestingPartyIDSource                            = 1659, /*!< 1659 (char FIX.5.0SP2) Identifies the source of the RequestingPartyID(1658) value.*/
RequestingPartyRole                                = 1660, /*!< 1660 (int FIX.5.0SP2) Identifies the type or role of the RequestingPartyID(1658) specified.*/
NoRequestingPartySubIDs                            = 1661, /*!< 1661 (NumInGroup FIX.5.0SP2) Number of requesting party sub-identifiers.*/
RequestingPartySubID                               = 1662, /*!< 1662 (String FIX.5.0SP2) Sub-identifier for the party specified in RequestingPartyID(1658).*/
RequestingPartySubIDType                           = 1663, /*!< 1663 (int FIX.5.0SP2) Type of RequestingPartySubID(1662) value.*/
EncodedRejectTextLen                               = 1664, /*!< 1664 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedRejectText(1665) field.*/
EncodedRejectText                                  = 1665, /*!< 1665 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the RejectText(1328) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) representation should also be specified in the RejectText(1328) field.*/
RiskLimitRequestID                                 = 1666, /*!< 1666 (String FIX.5.0SP2) Unique identifier for the PartyRiskLimitsRequest*/
RiskLimitReportID                                  = 1667, /*!< 1667 (String FIX.5.0SP2) Identifier for the PartyRiskLimitsReport*/
NoRequestedRiskLimitType                           = 1668, /*!< 1668 (NumInGroup FIX.5.0SP2) Number of  risk limit types requested.*/
NoRiskLimits                                       = 1669, /*!< 1669 (NumInGroup FIX.5.0SP2) Number of risk limits for different instrument scopes.*/
RiskLimitID                                        = 1670, /*!< 1670 (String FIX.5.0SP2) Unique reference identifier for a specific risk limit defined for the specified party.*/
NoPartyDetails                                     = 1671, /*!< 1671 (NumInGroup FIX.5.0SP2) Number of party details.*/
PartyDetailStatus                                  = 1672, /*!< 1672 (int FIX.5.0SP2) Indicates the status of the party identified with PartyDetailID(1691).*/
MatchInstMarketID                                  = 1673, /*!< 1673 (Exchange FIX.5.0SP2) Identifies the market to which the matching instruction applies.*/
PartyDetailRoleQualifier                           = 1674, /*!< 1674 (int FIX.5.0SP2) Qualifies the value of PartyDetailRole(1693).*/
RelatedPartyDetailRoleQualifier                    = 1675, /*!< 1675 (int FIX.5.0SP2) Qualifies the value of RelatedPartyRole(1565)*/
NoPartyUpdates                                     = 1676, /*!< 1676 (NumInGroup FIX.5.0SP2) Number of party updates.*/
NoPartyRiskLimits                                  = 1677, /*!< 1677 (NumInGroup FIX.5.0SP2) Number of party risk limits.*/
EncodedOptionExpirationDescLen                     = 1678, /*!< 1678 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedOptionExpirationDesc(1697) field.*/
SecurityMassTradingStatus                          = 1679, /*!< 1679 (int FIX.5.0SP2) Identifies the trading status applicable to a group of instruments.*/
SecurityMassTradingEvent                           = 1680, /*!< 1680 (int FIX.5.0SP2) Identifies an event related to the mass trading status.*/
MassHaltReason                                     = 1681, /*!< 1681 (int FIX.5.0SP2) Denotes the reason for the Opening Delay or Trading halt of a group of securities.*/
MDSecurityTradingStatus                            = 1682, /*!< 1682 (int FIX.5.0SP2) Identifies the trading status applicable to the instrument in the market data message.*/
MDSubFeedType                                      = 1683, /*!< 1683 (String FIX.5.0SP2) Describes a sub-class for a given class of service defined by MDFeedType (1022)*/
MDHaltReason                                       = 1684, /*!< 1684 (int FIX.5.0SP2) Denotes the reason for the Opening Delay or Trading Halt.*/
ThrottleInst                                       = 1685, /*!< 1685 (int FIX.5.0SP2) Describes action recipient should take if a throttle limit were exceeded.*/
ThrottleCountIndicator                             = 1686, /*!< 1686 (int FIX.5.0SP2) Indicates whether a message decrements the number of outstanding requests, e.g. one where ThrottleType = Outstanding Requests.*/
ShortSaleRestriction                               = 1687, /*!< 1687 (int FIX.5.0SP2) Indicates whether a restriction applies to short selling a security.*/
ShortSaleExemptionReason                           = 1688, /*!< 1688 (int FIX.5.0SP2) Indicates the reason a short sale order is exempted from applicable regulation (e.g. Reg SHO addendum (b)(1) in the U.S.).*/
LegShortSaleExemptionReason                        = 1689, /*!< 1689 (int FIX.5.0SP2) Indicates the reason a short sale is exempted from applicable regulation (e.g. Reg SHO addendum (b)(1) in the U.S.)*/
SideShortSaleExemptionReason                       = 1690, /*!< 1690 (int FIX.5.0SP2) Indicates the reason a short sale is exempted from applicable regulation (e.g. Reg SHO addendum (b)(1) in the U.S.)*/
PartyDetailID                                      = 1691, /*!< 1691 (String FIX.5.0SP2) Party identifier within Parties Reference Data messages.*/
PartyDetailIDSource                                = 1692, /*!< 1692 (char FIX.5.0SP2) Source of the identifier of the PartyDetailID(1691) specified.*/
PartyDetailRole                                    = 1693, /*!< 1693 (int FIX.5.0SP2) Identifies the type or role of PartyDetailID(1691) specified.*/
NoPartyDetailSubIDs                                = 1694, /*!< 1694 (NumInGroup FIX.5.0SP2) Number of party detail sub-identifiers.*/
PartyDetailSubID                                   = 1695, /*!< 1695 (String FIX.5.0SP2) Sub-identifier for the party specified in PartyDetailID(1691).*/
PartyDetailSubIDType                               = 1696, /*!< 1696 (int FIX.5.0SP2) Type of PartyDetailSubID(1695) value.*/
EncodedOptionExpirationDesc                        = 1697, /*!< 1697 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the OptionExpirationDesc(1581) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the OptionExpirationDesc(1581).*/
StrikeUnitOfMeasure                                = 1698, /*!< 1698 (String FIX.5.0SP2) Used to express the unit of measure (UOM) of the price if different from the contract.*/
AccountSummaryReportID                             = 1699, /*!< 1699 (String FIX.5.0SP2) Unique identifier for the AccountSummaryReport(35=CQ).*/
NoSettlementAmounts                                = 1700, /*!< 1700 (NumInGroup FIX.5.0SP2) Number of settlement amount entries.*/
SettlementAmount                                   = 1701, /*!< 1701 (Amt FIX.5.0SP2) The amount of settlement.*/
SettlementAmountCurrency                           = 1702, /*!< 1702 (Currency FIX.5.0SP2) The currency of the reported settlement amount.*/
NoCollateralAmounts                                = 1703, /*!< 1703 (NumInGroup FIX.5.0SP2) Number of collateral amount entries.*/
CurrentCollateralAmount                            = 1704, /*!< 1704 (Amt FIX.5.0SP2) Currency value currently attributed to the collateral.*/
CollateralCurrency                                 = 1705, /*!< 1705 (Currency FIX.5.0SP2) Currency of the collateral; optional, defaults to the Settlement Currency if not specified.*/
CollateralType                                     = 1706, /*!< 1706 (String FIX.5.0SP2) Type of collateral on deposit being reported.*/
NoPayCollects                                      = 1707, /*!< 1707 (NumInGroup FIX.5.0SP2) Number of pay collect entries.*/
PayCollectType                                     = 1708, /*!< 1708 (String FIX.5.0SP2) Category describing the reason for funds paid to, or the funds collected from the clearing firm.*/
PayCollectCurrency                                 = 1709, /*!< 1709 (Currency FIX.5.0SP2) Currency denomination of value in PayAmount(1710) and CollectAmount(1711).  If not specified, default to currency specified in SettlementAmountCurrency(1702).*/
PayAmount                                          = 1710, /*!< 1710 (Amt FIX.5.0SP2) Amount to be paid by the clearinghouse to the clearing firm.*/
CollectAmount                                      = 1711, /*!< 1711 (Amt FIX.5.0SP2) Amount to be collected by the clearinghouse from the clearing firm.*/
PayCollectMarketSegmentID                          = 1712, /*!< 1712 (String FIX.5.0SP2) Market segment associated with the pay collect amount.*/
PayCollectMarketID                                 = 1713, /*!< 1713 (String FIX.5.0SP2) Market associated with the pay collect amount.*/
MarginAmountMarketSegmentID                        = 1714, /*!< 1714 (String FIX.5.0SP2) Market segment associated with the margin amount.*/
MarginAmountMarketID                               = 1715, /*!< 1715 (String FIX.5.0SP2) Market associated with the margin amount*/
UnitOfMeasureCurrency                              = 1716, /*!< 1716 (Currency FIX.5.0SP2) Indicates the currency of the unit of measure. Conditionally required when UnitOfMeasure(996) = Ccy*/
PriceUnitOfMeasureCurrency                         = 1717, /*!< 1717 (Currency FIX.5.0SP2) Indicates the currency of the price unit of measure. Conditionally required when PriceUnitOfMeasure(1191) = Ccy*/
UnderlyingUnitOfMeasureCurrency                    = 1718, /*!< 1718 (Currency FIX.5.0SP2) Indicates the currency of the underlying unit of measure. Conditionally required when UnderlyingUnitOfMeasure(998) = Ccy*/
UnderlyingPriceUnitOfMeasureCurrency               = 1719, /*!< 1719 (Currency FIX.5.0SP2) Indicates the currency of the underlying price unit of measure. Conditionally required when UnderlyingPriceUnitOfMeasure(1424) = Ccy*/
LegUnitOfMeasureCurrency                           = 1720, /*!< 1720 (Currency FIX.5.0SP2) Indicates the currency of the unit of measure. Conditionally required when LegUnitOfMeasure(999) = Ccy*/
LegPriceUnitOfMeasureCurrency                      = 1721, /*!< 1721 (Currency FIX.5.0SP2) Indicates the currency of the price unit of measure. Conditionally required when LegPriceUnitOfMeasure(1421) = Ccy*/
DerivativeUnitOfMeasureCurrency                    = 1722, /*!< 1722 (Currency FIX.5.0SP2) Indicates the currency of the unit of measure. Conditionally required when DerivativeUnitOfMeasure(1269) = Ccy*/
DerivativePriceUnitOfMeasureCurrency               = 1723, /*!< 1723 (Currency FIX.5.0SP2) Indicates the currency of the price unit of measure. Conditionally required when DerivativePriceUnitOfMeasure(1315) = Ccy*/
OrderOrigination                                   = 1724, /*!< 1724 (int FIX.5.0SP2) Identifies the origin of the order.*/
OriginatingDeptID                                  = 1725, /*!< 1725 (String FIX.5.0SP2) An identifier representing the department or desk within the firm that originated the order.*/
ReceivingDeptID                                    = 1726, /*!< 1726 (String FIX.5.0SP2) An identifier representing the department or desk within the firm that received the order.*/
InformationBarrierID                               = 1727, /*!< 1727 (String FIX.5.0SP2) The identifier of the information barrier in place for a trading unit that will meet the criteria of the "no-knowledge" exception in FINRA Rule 5320.02.*/
FirmGroupID                                        = 1728, /*!< 1728 (String FIX.5.0SP2) Firm assigned group allocation entity identifier.*/
FirmMnemonic                                       = 1729, /*!< 1729 (String FIX.5.0SP2) Allocation identifier assigned by the Firm submitting the allocation for an individual allocation instruction (as opposed to the overall message level identifier).*/
AllocGroupID                                       = 1730, /*!< 1730 (String FIX.5.0SP2) Intended to be used by a central counterparty to assign an identifier to allocations of trades for the same instrument traded at the same price.*/
AvgPxGroupID                                       = 1731, /*!< 1731 (String FIX.5.0SP2) Used by submitting firm to group trades being allocated into an average price group. The trades in average price group will be used to calculate an average price for the group.*/
FirmAllocText                                      = 1732, /*!< 1732 (String FIX.5.0SP2) Firm reference information, usually internal information, that is part of the initial message. The information would not be carried forward (e.g to Take-up Firm) and preserved with the transaction.*/
EncodedFirmAllocTextLen                            = 1733, /*!< 1733 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedFirmAllocText(1734) field.*/
EncodedFirmAllocText                               = 1734, /*!< 1734 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the FirmAllocText(1732) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) represention should also be specified in FirmAllocText(1732) field.*/
AllocationRollupInstruction                        = 1735, /*!< 1735 (int FIX.5.0SP2) An indicator to override the normal procedure to roll up allocations for the same take-up firm.*/
AllocGroupQuantity                                 = 1736, /*!< 1736 (Qty FIX.5.0SP2) Indicates the total quantity of an allocation group. Includes any allocated quantity.*/
AllocGroupRemainingQuantity                        = 1737, /*!< 1737 (Qty FIX.5.0SP2) Indicates the remaining quantity of an allocation group that has not yet been allocated.*/
AllocReversalStatus                                = 1738, /*!< 1738 (int FIX.5.0SP2) Identifies the status of a reversal transaction.*/
ObligationType                                     = 1739, /*!< 1739 (String FIX.5.0SP2) Type of reference obligation for credit derivatives contracts.*/
TradePriceNegotiationMethod                        = 1740, /*!< 1740 (int FIX.5.0SP2) Method used for negotiation of contract price.*/
UpfrontPriceType                                   = 1741, /*!< 1741 (int FIX.5.0SP2) Type of price used to determine upfront payment for swaps contracts.*/
UpfrontPrice                                       = 1742, /*!< 1742 (Price FIX.5.0SP2) Price used to determine upfront payment for swaps contracts.*/
LastUpfrontPrice                                   = 1743, /*!< 1743 (Price FIX.5.0SP2) Price used to determine upfront payment for swaps contracts reported for a deal (trade).*/
ApplLevelRecoveryIndicator                         = 1744, /*!< 1744 (int FIX.5.0SP2) Indicates whether application level recovery is needed.*/
BidMDEntryID                                       = 1745, /*!< 1745 (String FIX.5.0SP2) The market data entry identifier of the bid side of a quote*/
OfferMDEntryID                                     = 1746, /*!< 1746 (String FIX.5.0SP2) The market data entry identifier of the offer side of a quote.*/
BidQuoteID                                         = 1747, /*!< 1747 (String FIX.5.0SP2) Marketplace assigned quote identifier for the bid side. Can be used to indicate priority.*/
OfferQuoteID                                       = 1748, /*!< 1748 (String FIX.5.0SP2) Marketplace assigned quote identifier for the offer side. Can be used to indicate priority.*/
TotalBidSize                                       = 1749, /*!< 1749 (Qty FIX.5.0SP2) Specifies the total bid size.*/
TotalOfferSize                                     = 1750, /*!< 1750 (Qty FIX.5.0SP2) Specifies the total offer size.*/
SecondaryQuoteID                                   = 1751, /*!< 1751 (String FIX.5.0SP2) Assigned by the party which accepts the quote. Can be used to provide the quote identifier assigned by an exchange, marketplace or executing system.*/
CustodialLotID                                     = 1752, /*!< 1752 (String FIX.5.0SP2) An opaque identifier used to communicate the custodian’s identifier for the lot.  It is expected that this information would be provided by the custodian as part of a reconciliation process that occurs before trading.*/
VersusPurchaseDate                                 = 1753, /*!< 1753 (LocalMktDate FIX.5.0SP2) The effective acquisition date of the lot that would be used for gain-loss trade lot reporting. The versus purchase date used to identify the lot in situations where a custodial lot identifier is not available.*/
VersusPurchasePrice                                = 1754, /*!< 1754 (Price FIX.5.0SP2) The versus purchase price used to identify the lot in situations where a custodial lot identifier is not available. The value should be calculated based on current cost basis / quantity held.*/
CurrentCostBasis                                   = 1755, /*!< 1755 (Amt FIX.5.0SP2) The amount that the current shares are worth.  If this lot was liquidated, the total gain/loss for a trade is equal to the trade amount minus the current cost basis.*/
LegCustodialLotID                                  = 1756, /*!< 1756 (String FIX.5.0SP2) An opaque identifier used to communicate the custodian’s identifier for the lot.  It is expected that this information would be provided by the custodian as part of a reconciliation process that occurs before trading.*/
LegVersusPurchaseDate                              = 1757, /*!< 1757 (LocalMktDate FIX.5.0SP2) The effective acquisition date of the lot that would be used for gain-loss trade lot reporting. The versus purchase date used to identify the lot in situations where a custodial lot identifier is not available.*/
LegVersusPurchasePrice                             = 1758, /*!< 1758 (Price FIX.5.0SP2) The versus purchase price used to identify the lot in situations where a custodial lot identifier is not available.The value should be calculated based on current cost basis / quantity held.*/
LegCurrentCostBasis                                = 1759, /*!< 1759 (Amt FIX.5.0SP2) The amount that the current shares are worth.  If this lot was liquidated, the total gain/loss for a trade is equal to the trade amount minus the current cost basis.*/
RiskLimitRequestType                               = 1760, /*!< 1760 (int FIX.5.0SP2) Type of risk limit information.*/
RiskLimitRequestResult                             = 1761, /*!< 1761 (int FIX.5.0SP2) Result of risk limit definition request.*/
RiskLimitRequestStatus                             = 1762, /*!< 1762 (int FIX.5.0SP2) Status of risk limit definition request.*/
RiskLimitStatus                                    = 1763, /*!< 1763 (int FIX.5.0SP2) Status of risk limit definition for one party.*/
RiskLimitResult                                    = 1764, /*!< 1764 (int FIX.5.0SP2) Result of risk limit definition for one party.*/
RiskLimitUtilizationPercent                        = 1765, /*!< 1765 (Percentage FIX.5.0SP2) Percentage of utilization of a party's set risk limit.*/
RiskLimitUtilizationAmount                         = 1766, /*!< 1766 (Amt FIX.5.0SP2) Absolute amount of utilization of a party's set risk limit.*/
RiskLimitAction                                    = 1767, /*!< 1767 (int FIX.5.0SP2) Identifies the action to take or risk model to assume should risk limit be exceeded or breached for the specified party.*/
RiskWarningLevelAmount                             = 1768, /*!< 1768 (int FIX.5.0SP2) Amount at which a warning is issued.*/
RiskWarningLevelAction                             = 1769, /*!< 1769 (int FIX.5.0SP2) Action to take should warning level be exceeded.*/
EntitlementRequestID                               = 1770, /*!< 1770 (String FIX.5.0SP2) Unique identifier for PartyEntitlementsRequest(35=CU).*/
EntitlementReportID                                = 1771, /*!< 1771 (String FIX.5.0SP2) Identifier for the PartyEntitlementsReport(35=CV).*/
NoPartyEntitlements                                = 1772, /*!< 1772 (NumInGroup FIX.5.0SP2) Number of party entitlement values.*/
NoEntitlements                                     = 1773, /*!< 1773 (NumInGroup FIX.5.0SP2) Number of entitlement values.*/
EntitlementIndicator                               = 1774, /*!< 1774 (Boolean FIX.5.0SP2) Used to indicate if a party is entitled to an entitlement type specified in the EntitlementType(1775) field.*/
EntitlementType                                    = 1775, /*!< 1775 (int FIX.5.0SP2) Type of entitlement.*/
EntitlementID                                      = 1776, /*!< 1776 (String FIX.5.0SP2) Unique identifier for a specific NoEntitlements(1773) repeating group instance.*/
NoEntitlementAttrib                                = 1777, /*!< 1777 (int FIX.5.0SP2) Number of entitlement attributes.*/
EntitlementAttribType                              = 1778, /*!< 1778 (int FIX.5.0SP2) Name of the entitlement attribute type. A code list of allowed values will be maintained on the FIX Protocol website.

Values "4000" and above are reserved for bilaterally agreed upon user defined enumerations.

*/
EntitlementAttribDatatype                          = 1779, /*!< 1779 (int FIX.5.0SP2) Datatype of the entitlement attribute.*/
EntitlementAttribValue                             = 1780, /*!< 1780 (String FIX.5.0SP2) Value of the entitlement attribute.*/
EntitlementAttribCurrency                          = 1781, /*!< 1781 (Currency FIX.5.0SP2) Currency for EntitlementAttribValue(1780). Can be used if these fields represent a price, price offset, or amount.*/
EntitlementStartDate                               = 1782, /*!< 1782 (LocalMktDate FIX.5.0SP2) Indicates the starting date of the entitlement.*/
EntitlementEndDate                                 = 1783, /*!< 1783 (LocalMktDate FIX.5.0SP2) Indicates the ending date of the entitlement.*/
EntitlementPlatform                                = 1784, /*!< 1784 (String FIX.5.0SP2) The area to which the entitlement is applicable. This can be a trading platform or an offering.*/
TradSesControl                                     = 1785, /*!< 1785 (int FIX.5.0SP2) Indicates how control of trading session and subsession transitions are performed.*/
TradeVolType                                       = 1786, /*!< 1786 (int FIX.5.0SP2) Define the type of trade volume applicable for the MinTradeVol(562) and MaxTradeVol(1140)*/
RefTickTableID                                     = 1787, /*!< 1787 (int FIX.5.0SP2) Spread table code referred by the security or symbol.*/
LegID                                              = 1788, /*!< 1788 (String FIX.5.0SP2) Unique identifier for the leg within the context of a message (the scope of uniqueness to be defined by counterparty agreement). The LegID(1788) can be referenced using LegRefID(654).*/
NoTargetMarketSegments                             = 1789, /*!< 1789 (NumInGroup FIX.5.0SP2) Number of market segments upon which a mass action is to be taken.*/
TargetMarketSegmentID                              = 1790, /*!< 1790 (String FIX.5.0SP2) Market segment within a target market segment repeating group.*/
NoAffectedMarketSegments                           = 1791, /*!< 1791 (NumInGroup FIX.5.0SP2) Number of market segments affected by a mass action.*/
AffectedMarketSegmentID                            = 1792, /*!< 1792 (String FIX.5.0SP2) Market segment within an affected market repeating segment group.*/
NoNotAffectedMarketSegments                        = 1793, /*!< 1793 (NumInGroup FIX.5.0SP2) Number of market segments left unaffected by a mass action.*/
NotAffectedMarketSegmentID                         = 1794, /*!< 1794 (String FIX.5.0SP2) Market segment within an unaffected market repeating segment group.*/
NoOrderEvents                                      = 1795, /*!< 1795 (NumInGroup FIX.5.0SP2) Number of order events.*/
OrderEventType                                     = 1796, /*!< 1796 (int FIX.5.0SP2) The type of event affecting an order. The last event type within the OrderEventGrp component indicates the ExecType(150) value resulting from the series of events (ExecType(150) values are shown in brackets).*/
OrderEventExecID                                   = 1797, /*!< 1797 (String FIX.5.0SP2) Refer to ExecID(17). Used when multiple different events are reported in single Execution Report. ExecID(17) and OrderEventExecID(1797) values should not overlap.*/
OrderEventReason                                   = 1798, /*!< 1798 (int FIX.5.0SP2) Action that caused the event to occur.*/
OrderEventPx                                       = 1799, /*!< 1799 (Price FIX.5.0SP2) Price associated with the event.*/
OrderEventQty                                      = 1800, /*!< 1800 (Qty FIX.5.0SP2) Quantity associated with the event.*/
OrderEventLiquidityIndicator                       = 1801, /*!< 1801 (int FIX.5.0SP2) Indicator to identify whether this fill was a result of a liquidity provider providing or liquidity taker taking the liquidity. Applicable only for OrderEventType(1796) values of 4(Partially Filled) or 5(Filled).*/
OrderEventText                                     = 1802, /*!< 1802 (String FIX.5.0SP2) Additional information about the event.*/
AuctionType                                        = 1803, /*!< 1803 (int FIX.5.0SP2) Type of auction order.*/
AuctionAllocationPct                               = 1804, /*!< 1804 (Percentage FIX.5.0SP2) Percentage of matched quantity to be allocated to the submitter of the response to an auction order.*/
AuctionInstruction                                 = 1805, /*!< 1805 (int FIX.5.0SP2) Instruction related to system generated auctions, e.g. flash order auctions.*/
RefClOrdID                                         = 1806, /*!< 1806 (String FIX.5.0SP2) Used to reference an order via ClOrdID(11).*/
LockType                                           = 1807, /*!< 1807 (int FIX.5.0SP2) Indicates whether an order is locked and for what reason.*/
LockedQty                                          = 1808, /*!< 1808 (Qty FIX.5.0SP2) Locked order quantity.*/
SecondaryLockedQty                                 = 1809, /*!< 1809 (Qty FIX.5.0SP2) Locked order quantity in addition to LockedQty (1808), e.g. to distinguish total locked quantity from currently locked quantity.*/
ReleaseInstruction                                 = 1810, /*!< 1810 (int FIX.5.0SP2) Instruction to define conditions under which to release a locked order or parts of it.*/
ReleaseQty                                         = 1811, /*!< 1811 (Qty FIX.5.0SP2) Quantity to be made available, i.e. released from a lock.*/
NoDisclosureInstructions                           = 1812, /*!< 1812 (NumInGroup FIX.5.0SP2) Number of disclosure instructions.*/
DisclosureType                                     = 1813, /*!< 1813 (int FIX.5.0SP2) Information subject to disclosure.*/
DisclosureInstruction                              = 1814, /*!< 1814 (int FIX.5.0SP2) Instruction to disclose information or to use default value of the receiver.*/
TradingCapacity                                    = 1815, /*!< 1815 (int FIX.5.0SP2) Designates the capacity in which the order is submitted for trading by the market participant.*/
ClearingAccountType                                = 1816, /*!< 1816 (int FIX.5.0SP2) Designates the account type to be used for the order when submitted to clearing.*/
LegClearingAccountType                             = 1817, /*!< 1817 (int FIX.5.0SP2) Designates the capacity in which the order will be submitted to clearing.*/
TargetPartyRoleQualifier                           = 1818, /*!< 1818 (int FIX.5.0SP2) Qualifies the value of TargetPartyRole (1464).*/
RelatedHighPrice                                   = 1819, /*!< 1819 (Price FIX.5.0SP2) Upper boundary for the price of a related entity, e.g. price of the underlying instrument in an Underlying Price Contingency (UPC) order.*/
RelatedLowPrice                                    = 1820, /*!< 1820 (Price FIX.5.0SP2) Lower boundary for the price of a related entity, e.g. price of the underlying instrument in an Underlying Price Contingency (UPC) order.*/
RelatedPriceSource                                 = 1821, /*!< 1821 (int FIX.5.0SP2) Source for the price of a related entity, e.g. price of the underlying instrument in an Underlying Price Contingency (UPC) order. Can be used together with RelatedHighPrice (1819) and/or RelatedLowPrice (1820).*/
MinQtyMethod                                       = 1822, /*!< 1822 (int FIX.5.0SP2) Indicates how the minimum quantity should be applied when executing the order.*/
Triggered                                          = 1823, /*!< 1823 (int FIX.5.0SP2) Indicates whether order has been triggered during its lifetime. Applies to cases where original information, e.g. OrdType(40), is modified when the order is triggered.*/
AffectedOrigClOrdID                                = 1824, /*!< 1824 (String FIX.5.0SP2) OrigClOrdID(41) of an order affected by a mass cancel or mass action request.*/
NotAffSecondaryOrderID                             = 1825, /*!< 1825 (String FIX.5.0SP2) SecondaryOrderID (198) of an order not affected by a mass cancel or mass action request.*/
EventTimePeriod                                    = 1826, /*!< 1826 (int FIX.5.0SP2) Time unit multiplier for the event.*/
EventTimeUnit                                      = 1827, /*!< 1827 (String FIX.5.0SP2) Time unit associated with the event.*/
LastQtyVariance                                    = 1828, /*!< 1828 (Qty FIX.5.0SP2) When LastQty is an estimated value, e.g. for a Repo “circled” trade, LastQtyVariance specifies the absolute amount that the size may vary up or down when finalized. Omitted when LastQty(32) is already final.*/
NoCrossLegs                                        = 1829, /*!< 1829 (NumInGroup FIX.5.0SP2) Number of legs in the side of a cross order.*/
SettlPriceIncrement                                = 1830, /*!< 1830 (Price FIX.5.0SP2) Settlement price increment for stated price range.*/
SettlPriceSecondaryIncrement                       = 1831, /*!< 1831 (Price FIX.5.0SP2) Secondary settlement price increment for stated price range. The meaning of secondary is left to bilateral agreement, e.g. it may refer to final settlement for a contract.*/
ClearedIndicator                                   = 1832, /*!< 1832 (int FIX.5.0SP2) Indicates whether the trade or position being reported was cleared through a clearing organization.*/
ContractRefPosType                                 = 1833, /*!< 1833 (int FIX.5.0SP2) Additional information related to the pricing of a commodity swaps position, specifically an indicator referring to the position type.*/
PositionCapacity                                   = 1834, /*!< 1834 (int FIX.5.0SP2) Used to describe the ownership of the position.*/
PosQtyUnitOfMeasureCurrency                        = 1835, /*!< 1835 (Currency FIX.5.0SP2) Indicates the currency of the unit of measure if position quantity is expressed in valuation rather than contracts.  Conditionally required when PosQtyUnitOfMeasure(1836)=Ccy.*/
PosQtyUnitOfMeasure                                = 1836, /*!< 1836 (String FIX.5.0SP2) Indicates the unit of measure of the position quantity when not expressed in contracts. */
UnderlyingContractPriceRefMonth                    = 1837, /*!< 1837 (MonthYear FIX.5.0SP2) Reference month if there is no applicable UnderlyingMaturityMonth(313) value for the contract or security.*/
NoTradePriceConditions                             = 1838, /*!< 1838 (NumInGroup FIX.5.0SP2) Number of trade price conditions.*/
TradePriceCondition                                = 1839, /*!< 1839 (int FIX.5.0SP2) Price conditions in effect at the time of the trade. Multiple price conditions can be in effect at the same time. Price conditions are usually required to be reported in markets that have regulations on price execution at a market or national best bid or offer, and the trade price differs from the best bid or offer.*/
TradeAllocStatus                                   = 1840, /*!< 1840 (int FIX.5.0SP2) Identifies the status of an allocation when using a pre-clear workflow. */
NoTradeQtys                                        = 1841, /*!< 1841 (NumInGroup FIX.5.0SP2) Number of trade quantities.*/
TradeQtyType                                       = 1842, /*!< 1842 (int FIX.5.0SP2) Indicates the type of trade quantity in TradeQty(1843).*/
TradeQty                                           = 1843, /*!< 1843 (Qty FIX.5.0SP2) Trade quantity.*/
NoTradeAllocAmts                                   = 1844, /*!< 1844 (NumInGroup FIX.5.0SP2) Number of trade allocation amount entries.*/
TradeAllocAmtType                                  = 1845, /*!< 1845 (String FIX.5.0SP2) Type of the amount associated with a trade allocation.*/
TradeAllocAmt                                      = 1846, /*!< 1846 (Amt FIX.5.0SP2) The amount associated with a trade allocation.*/
TradeAllocCurrency                                 = 1847, /*!< 1847 (Currency FIX.5.0SP2) Currency denomination of the trade allocation amount.*/
TradeAllocGroupInstruction                         = 1848, /*!< 1848 (int FIX.5.0SP2) Instruction on how to add a trade to an allocation group when it is being given-up. */
OffsetInstruction                                  = 1849, /*!< 1849 (int FIX.5.0SP2) Indicates the trade is a result of an offset or onset.*/
TradeAllocAmtReason                                = 1850, /*!< 1850 (int FIX.5.0SP2) Specifies the reason for an amount type when reported on an allocation. Useful when multiple instances of the same amount type are reported.*/
StrategyLinkID                                     = 1851, /*!< 1851 (String FIX.5.0SP2) Identifies the multileg strategy (e.g. spread) to which the trade belongs. This links together trade legs executed as part of a strategy during a single match event.*/
SideAvgPx                                          = 1852, /*!< 1852 (Price FIX.5.0SP2) Calculated average price for this side of the trade.*/
SideAvgPxIndicator                                 = 1853, /*!< 1853 (int FIX.5.0SP2) Used to indicate whether a trade or a sub-allocation should be allocated at the trade price (e.g. no average pricing), or whether it should be grouped with other trades/sub-allocations and allocated at the average price of the group. */
SideAvgPxGroupID                                   = 1854, /*!< 1854 (String FIX.5.0SP2) The identifier for the average price group for the trade side.  See also AvgPxGroupID(1731).*/
NoRelatedTrades                                    = 1855, /*!< 1855 (NumInGroup FIX.5.0SP2) Number of related trades.*/
RelatedTradeID                                     = 1856, /*!< 1856 (String FIX.5.0SP2) Identifier of a related trade.*/
RelatedTradeIDSource                               = 1857, /*!< 1857 (int FIX.5.0SP2) Describes the source of the identifier that RelatedTradeID(1856) represents.*/
RelatedTradeDate                                   = 1858, /*!< 1858 (LocalMktDate FIX.5.0SP2) Date of a related trade.*/
RelatedTradeMarketID                               = 1859, /*!< 1859 (Exchange FIX.5.0SP2) Market of execution of related trade.*/
RelatedTradeQuantity                               = 1860, /*!< 1860 (Qty FIX.5.0SP2) Quantity of the related trade which can be less than or equal to the actual quantity of the related trade. For example, when one trade offsets another across asset classes.*/
NoRelatedPositions                                 = 1861, /*!< 1861 (NumInGroup FIX.5.0SP2) Number of related positions.*/
RelatedPositionID                                  = 1862, /*!< 1862 (String FIX.5.0SP2) Identifier of a related position.*/
RelatedPositionIDSource                            = 1863, /*!< 1863 (int FIX.5.0SP2) Describes the source of the identifier that RelatedPositionID(1862) represents.*/
RelatedPositionDate                                = 1864, /*!< 1864 (LocalMktDate FIX.5.0SP2) Used to help identify the position when RelatedPositionID(1862) is not unique across multiple days. This date is generally the creation date of the identifier.*/
QuoteAckStatus                                     = 1865, /*!< 1865 (int FIX.5.0SP2) Acknowledgement status of a Quote(35=S) or QuoteCancel(35=Z) message submission.*/
StrikeIndex                                        = 1866, /*!< 1866 (String FIX.5.0SP2) Specifies the index used to calculate the strike price.*/
OfferID                                            = 1867, /*!< 1867 (String FIX.5.0SP2) Unique identifier for the ask side of the quote assigned by the quote issuer.*/
NoValueChecks                                      = 1868, /*!< 1868 (NumInGroup FIX.5.0SP2) Number of value check entries.*/
ValueCheckType                                     = 1869, /*!< 1869 (int FIX.5.0SP2) Type of value to be checked.*/
ValueCheckAction                                   = 1870, /*!< 1870 (int FIX.5.0SP2) Action to be taken for the ValueCheckType(1869). */
LegSecurityXMLLen                                  = 1871, /*!< 1871 (Length FIX.5.0SP2) The length of the LegSecurityXML(1872) data block.*/
LegSecurityXML                                     = 1872, /*!< 1872 (XMLData FIX.5.0SP2) XML definition for the leg security.*/
LegSecurityXMLSchema                               = 1873, /*!< 1873 (String FIX.5.0SP2) The schema used to validate the contents of LegSecurityXML(1872).*/
UnderlyingSecurityXMLLen                           = 1874, /*!< 1874 (Length FIX.5.0SP2) The length of the UnderlyingSecurityXML(1875) data block. */
UnderlyingSecurityXML                              = 1875, /*!< 1875 (XMLData FIX.5.0SP2) XML definition for the underlying security. */
UnderlyingSecurityXMLSchema                        = 1876, /*!< 1876 (String FIX.5.0SP2) The schema used to validate the contents of UnderlyingSecurityXML(1875).*/
PartyDetailRequestResult                           = 1877, /*!< 1877 (int FIX.5.0SP2) Result party detail definition request.*/
PartyDetailRequestStatus                           = 1878, /*!< 1878 (int FIX.5.0SP2) Status of party details definition request.*/
PartyDetailDefinitionStatus                        = 1879, /*!< 1879 (int FIX.5.0SP2) Status of party detail definition for one party.*/
PartyDetailDefinitionResult                        = 1880, /*!< 1880 (int FIX.5.0SP2) Result of party detail definition for one party.*/
EntitlementRequestResult                           = 1881, /*!< 1881 (int FIX.5.0SP2) Result of risk limit definition request.*/
EntitlementRequestStatus                           = 1882, /*!< 1882 (int FIX.5.0SP2) Status of party entitlements definition request.*/
EntitlementStatus                                  = 1883, /*!< 1883 (int FIX.5.0SP2) Status of entitlement definition for one party.*/
EntitlementResult                                  = 1884, /*!< 1884 (int FIX.5.0SP2) Result of entitlement definition for one party.*/
EntitlementRefID                                   = 1885, /*!< 1885 (String FIX.5.0SP2) Reference to an EntitlementID(1776). Used for modification or deletion of an entitlement.*/
SettlPriceUnitOfMeasure                            = 1886, /*!< 1886 (String FIX.5.0SP2) Used to express the unit of measure of the settlement price if different from the contract.*/
SettlPriceUnitOfMeasureCurrency                    = 1887, /*!< 1887 (Currency FIX.5.0SP2) Indicates the currency of the settlement price unit of measure if expressed in another currency than the base currency.

Conditionally required when SettlPriceUnitOfMeasure(1886)=Ccy.*/
TradeMatchTimestamp                                = 1888, /*!< 1888 (UTCTimestamp FIX.5.0SP2) Timestamp of the match event. For off-exchange trades the time at which the deal was matched by the exchange.

This timestamp will be the same on all the trades and will not change when a trade is modified.*/
NoInstrmtMatchSides                                = 1889, /*!< 1889 (NumInGroup FIX.5.0SP2) Number of instrument match sides.*/
NoTrdMatchSides                                    = 1890, /*!< 1890 (NumInGroup FIX.5.0SP2) Number of trade match sides.*/
TrdMatchSubID                                      = 1891, /*!< 1891 (String FIX.5.0SP2) Used to identify each price level, step or clip within a match event.*/
NoLegExecs                                         = 1892, /*!< 1892 (NumInGroup FIX.5.0SP2) Number of instrument leg executions.*/
LegExecID                                          = 1893, /*!< 1893 (String FIX.5.0SP2) The ExecID(17) value corresponding to a trade leg.*/
LegTradeID                                         = 1894, /*!< 1894 (String FIX.5.0SP2) The TradeID(1003) value corresponding to a trade leg.*/
LegTradeReportID                                   = 1895, /*!< 1895 (String FIX.5.0SP2) The TradeReportID(571) value corresponding to a trade leg.*/
TradeMatchAckStatus                                = 1896, /*!< 1896 (int FIX.5.0SP2) Used to indicate the status of the trade match report submission.*/
TradeMatchRejectReason                             = 1897, /*!< 1897 (int FIX.5.0SP2) Reason the trade match report submission was rejected.*/
SideMarketSegmentID                                = 1898, /*!< 1898 (String FIX.5.0SP2) Identifies the market segment of the side.*/
SideVenueType                                      = 1899, /*!< 1899 (char FIX.5.0SP2) Identifies the type of venue where the trade was executed for the side.*/
SideExecRefID                                      = 1900, /*!< 1900 (String FIX.5.0SP2) Used to reference the value from SideExecID(1427).*/
LegExecRefID                                       = 1901, /*!< 1901 (String FIX.5.0SP2) Used to reference the value from LegExecID(1893).*/
HaircutIndicator                                   = 1902, /*!< 1902 (Boolean FIX.5.0SP2) Indicates, if "Y", that a stated valuation includes a haircut, e.g. that the stated value reflects the subtraction of the haircut. Note that a value of "N" does not imply a haircut is not applicable, only that the haircut (if any) is not reflected in the stated valuation.*/
RegulatoryTradeID                                  = 1903, /*!< 1903 (String FIX.5.0SP2) Trade identifier required by government regulators or other regulatory organizations for regulatory reporting purposes.  For example, unique swap identifer (USI) as required by the U.S. Commodity Futures Trading Commission.*/
RegulatoryTradeIDEvent                             = 1904, /*!< 1904 (int FIX.5.0SP2) Identifies the event which caused origination of the identifier in RegulatoryTradeID(1903). When more than one event is the cause, use the higher enumeration value.  For example, if the identifier is originated due to an allocated trade which was cleared and reported, use the enumeration value 2 (Clearing).*/
RegulatoryTradeIDSource                            = 1905, /*!< 1905 (String FIX.5.0SP2) Identifies the reporting entity that originated the value in RegulatoryTradeID(1903). The reporting entitiy identifier may be assigned by a regulator.*/
RegulatoryTradeIDType                              = 1906, /*!< 1906 (int FIX.5.0SP2) Specifies the type of trade identifier provided in RegulatoryTradeID(1903).  

Contextual hierarchy of events for the same trade or transaction maybe captured through use of the different RegulatoryTradeIDType(1906) values using multiple instances of the repeating group as needed for regulatory reporting.*/
NoRegulatoryTradeIDs                               = 1907, /*!< 1907 (NumInGroup FIX.5.0SP2) Number of regulatory IDs in the repeating group.*/
NoAllocRegulatoryTradeIDs                          = 1908, /*!< 1908 (NumInGroup FIX.5.0SP2) Number of regulatory IDs in the repeating group.*/
AllocRegulatoryTradeID                             = 1909, /*!< 1909 (String FIX.5.0SP2) Trade identifier required by government regulators or other regulatory organizations for regulatory reporting purposes.  For example, unique swap identifer (USI) as required by the U.S. Commodity Futures Trading Commission.*/
AllocRegulatoryTradeIDSource                       = 1910, /*!< 1910 (String FIX.5.0SP2) Identifies the reporting entity that originated the value in AllocRegulatoryTradeID(1909).  The reporting entity identifier may be assigned by a regulator.*/
AllocRegulatoryTradeIDEvent                        = 1911, /*!< 1911 (int FIX.5.0SP2) Identifies the event which caused the origination of the identifier in AllocRegulatoryTradeID(1909).  When more than one event is the cause, use the higher enumeration value.  For example, if the identifier is originated due to an allocated trade which was cleared and reported, use the enumeration value 2(Clearing). */
AllocRegulatoryTradeIDType                         = 1912, /*!< 1912 (int FIX.5.0SP2) Specifies the type of trade identifier provided in AllocRegulatoryTradeID(1909), within the context of the hierarchy of trade events.*/
NumOfCompetitors                                   = 1913, /*!< 1913 (int FIX.5.0SP2) The number of competing Respondents (e.g. dealers) to receive a quote request (either via the QuoteRequest(35=R) or via other means).*/
ResponseTime                                       = 1914, /*!< 1914 (UTCTimestamp FIX.5.0SP2) The time by which a meaningful response should arrive back (always expressed in UTC (Universal Time Coordinated, also known as "GMT").*/
QuoteDisplayTime                                   = 1915, /*!< 1915 (UTCTimestamp FIX.5.0SP2) Time by which the quote will be displayed.*/
ExposureDurationUnit                               = 1916, /*!< 1916 (int FIX.5.0SP2) Time unit in which the ExposureDuration(1629) is expressed.*/
CoverPrice                                         = 1917, /*!< 1917 (Price FIX.5.0SP2) The best quoted price received among those not traded.*/
NoClearingAccountTypes                             = 1918, /*!< 1918 (NumInGroup FIX.5.0SP2) Number of clearing account type entries.*/
NoPriceMovements                                   = 1919, /*!< 1919 (NumInGroup FIX.5.0SP2) Number of price movement entries.*/
NoPriceMovementValues                              = 1920, /*!< 1920 (NumInGroup FIX.5.0SP2) Number of price movement value entries.*/
PriceMovementValue                                 = 1921, /*!< 1921 (float FIX.5.0SP2) Value at specific price movement point.*/
PriceMovementPoint                                 = 1922, /*!< 1922 (int FIX.5.0SP2) Price movement point up (positive integer) or down (negative integer) relative to the underlying price of the instrument.*/
PriceMovementType                                  = 1923, /*!< 1923 (int FIX.5.0SP2) Describes the format of the PriceMovementValue(1921).*/
ClearingIntention                                  = 1924, /*!< 1924 (int FIX.5.0SP2) Specifies the party's or parties' intention to clear the trade.*/
TradeClearingInstruction                           = 1925, /*!< 1925 (int FIX.5.0SP2) Specifies the eligibility of this trade for clearing and central counterparty processing.*/
BackloadedTradeIndicator                           = 1926, /*!< 1926 (Boolean FIX.5.0SP2) Indicates that the trade being reported occurred in the past and is still in effect or active.*/
ConfirmationMethod                                 = 1927, /*!< 1927 (int FIX.5.0SP2) Specifies how a trade was confirmed.*/
MandatoryClearingIndicator                         = 1928, /*!< 1928 (Boolean FIX.5.0SP2) An indication that the trade is flagged for mandatory clearing.*/
MixedSwapIndicator                                 = 1929, /*!< 1929 (Boolean FIX.5.0SP2) An indication that the trade is a mixed swap.*/
OffMarketPriceIndicator                            = 1930, /*!< 1930 (Boolean FIX.5.0SP2) An indication that the price is off-market.*/
VerificationMethod                                 = 1931, /*!< 1931 (int FIX.5.0SP2) Indication of how a trade was verified.*/
ClearingRequirementException                       = 1932, /*!< 1932 (int FIX.5.0SP2) Specifies whether a party to a swap is using an exception to a clearing requirement. In the US, one such clearing requirement is CFTC's rule pursuant to CEA Section 2(h)(1).*/
IRSDirection                                       = 1933, /*!< 1933 (String FIX.5.0SP2) Used to specify whether the principal is paying or receiving the fixed rate in an interest rate swap.*/
RegulatoryReportType                               = 1934, /*!< 1934 (int FIX.5.0SP2) Type of regulatory report.*/
VoluntaryRegulatoryReport                          = 1935, /*!< 1935 (Boolean FIX.5.0SP2) Used in conjunction with RegulatoryReportType(1934) to indicate whether the trade report is a voluntary regulatory report. If not specified, the default for a regulatory report is "N".

When VoluntaryRegulatoryReport(1935)=Y it is recommended that one of the parties to the trade be identified as the voluntary reporting party through PartySubIDType(803) = 63 (Voluntary reporting entity).*/
TradeCollateralization                             = 1936, /*!< 1936 (int FIX.5.0SP2) Specifies how the trade is collateralized.*/
TradeContinuation                                  = 1937, /*!< 1937 (int FIX.5.0SP2) Specifies the post-execution trade continuation or lifecycle event. Additional values may be used by mutual agreement of the counterparties.*/
AssetClass                                         = 1938, /*!< 1938 (int FIX.5.0SP2) The broad asset category for assessing risk exposure.*/
AssetSubClass                                      = 1939, /*!< 1939 (int FIX.5.0SP2) The subcategory description of the asset class. */
AssetType                                          = 1940, /*!< 1940 (String FIX.5.0SP2) Used to provide more specific description of the asset specified in AssetSubClass(1939).

See https://www.fixtrading.org/codelists/AssetType for code list of applicable values.  ISO 4721 Currency Code values are to be used when specific currency as an asset type is to be expressed.

Other values may be used by mutual agreement of the counterparties.*/
SwapClass                                          = 1941, /*!< 1941 (String FIX.5.0SP2) The classification or type of swap. Additional values may be used by mutual agreement of the counterparties.*/
NthToDefault                                       = 1942, /*!< 1942 (int FIX.5.0SP2) The Nth reference obligation to default in a CDS reference basket. If specified without MthToDefault(1943) the default will trigger a CDS payout.  If MthToDefault(1943) is also present then payout occurs between the Nth and Mth obligations to default.*/
MthToDefault                                       = 1943, /*!< 1943 (int FIX.5.0SP2) The Mth reference obligation to default in a CDS reference basket. When NthToDefault(1942) and MthToDefault(1943) are represented then the CDS payout occurs between the Nth and Mth obligations to default.*/
SettledEntityMatrixSource                          = 1944, /*!< 1944 (String FIX.5.0SP2) Relevant settled entity matrix source.*/
SettledEntityMatrixPublicationDate                 = 1945, /*!< 1945 (LocalMktDate FIX.5.0SP2) The publication date of the applicable version of the matrix. If not specified, the Standard Terms Supplement defines rules for which version of the matrix is applicable.*/
CouponType                                         = 1946, /*!< 1946 (int FIX.5.0SP2) Coupon type of the bond. */
TotalIssuedAmount                                  = 1947, /*!< 1947 (Amt FIX.5.0SP2) Specifies the total amount of the issue. Corresponds to the par value multiplied by the number of issued securities.*/
CouponFrequencyPeriod                              = 1948, /*!< 1948 (int FIX.5.0SP2) Time unit multiplier for the frequency of the bond's coupon payment. */
CouponFrequencyUnit                                = 1949, /*!< 1949 (String FIX.5.0SP2) Time unit associated with the frequency of the bond's coupon payment. */
CouponDayCount                                     = 1950, /*!< 1950 (int FIX.5.0SP2) The day count convention used in interest calculations for a bond or an interest bearing security.  Absence of this field for a bond or an interest bearing security transaction implies a "flat" trade, i.e. no accrued interest determined at time of the transaction.*/
ConvertibleBondEquityID                            = 1951, /*!< 1951 (String FIX.5.0SP2) Identifies the equity in which a convertible bond can be converted to.*/
ConvertibleBondEquityIDSource                      = 1952, /*!< 1952 (String FIX.5.0SP2) Identifies class or source of the ConvertibleBondEquityID(1951) value. 

100+ are reserved for private security.*/
ContractPriceRefMonth                              = 1953, /*!< 1953 (MonthYear FIX.5.0SP2) Reference month if there is no applicable MaturityMonthYear(200) value for the contract or security.*/
LienSeniority                                      = 1954, /*!< 1954 (int FIX.5.0SP2) Indicates the seniority level of the lien in a loan. */
LoanFacility                                       = 1955, /*!< 1955 (int FIX.5.0SP2) Specifies the type of loan when the credit default swap's reference obligation is a loan.*/
ReferenceEntityType                                = 1956, /*!< 1956 (int FIX.5.0SP2) Specifies the type of reference entity for first-to-default CDS basket contracts.*/
IndexSeries                                        = 1957, /*!< 1957 (int FIX.5.0SP2) The series identifier of a credit default swap index.*/
IndexAnnexVersion                                  = 1958, /*!< 1958 (int FIX.5.0SP2) The version of a credit default swap index annex.*/
IndexAnnexDate                                     = 1959, /*!< 1959 (LocalMktDate FIX.5.0SP2) The date of a credit default swap index series annex.*/
IndexAnnexSource                                   = 1960, /*!< 1960 (String FIX.5.0SP2) The source of a credit default swap series annex.*/
AgreementVersion                                   = 1961, /*!< 1961 (String FIX.5.0SP2) The version of the master agreement*/
MasterConfirmationDesc                             = 1962, /*!< 1962 (String FIX.5.0SP2) The type of master confirmation executed between the parties. 

See http://www.fpml.org/coding-scheme/master-confirmation-type for values.*/
MasterConfirmationDate                             = 1963, /*!< 1963 (LocalMktDate FIX.5.0SP2) Alternative to broker confirmation. The date of the confirmation executed between the parties and intended to govern all relevant transactions between those parties.*/
MasterConfirmationAnnexDesc                        = 1964, /*!< 1964 (String FIX.5.0SP2) The type of master confirmation annex executed between the parties. 

See http://www.fpml.org/coding-scheme/master-confirmation-annex-type for values.*/
MasterConfirmationAnnexDate                        = 1965, /*!< 1965 (LocalMktDate FIX.5.0SP2) The date that an annex to the master confirmation was executed between the parties.*/
BrokerConfirmationDesc                             = 1966, /*!< 1966 (String FIX.5.0SP2) Describes the type of broker confirmation executed between the parites.  Can be used as an alterative to MasterConfirmationDesc(1962).  See http://www.fpml.org/coding-scheme/broker-confirmation-type for values.*/
CreditSupportAgreementDesc                         = 1967, /*!< 1967 (String FIX.5.0SP2) The type of ISDA Credit Support Agreement. See http://www.fpml.org/coding-scheme/credit-support-agreement-type for values.*/
CreditSupportAgreementDate                         = 1968, /*!< 1968 (LocalMktDate FIX.5.0SP2) The date of the ISDA Credit Support Agreement executed between the parties and intended to govern collateral arrangements for all OTC derivatives transactions between those parties.*/
CreditSupportAgreementID                           = 1969, /*!< 1969 (String FIX.5.0SP2) A common reference or unique identifier to identify the ISDA Credit Support Agreement executed between the parties.*/
GoverningLaw                                       = 1970, /*!< 1970 (String FIX.5.0SP2) Identification of the law governing the transaction. See http://www.fpml.org/coding-scheme/governing-law for values.*/
NoSideRegulatoryTradeIDs                           = 1971, /*!< 1971 (NumInGroup FIX.5.0SP2) Number of regulatory IDs in the repeating group.*/
SideRegulatoryTradeID                              = 1972, /*!< 1972 (String FIX.5.0SP2) Trade identifier required by government regulators or other regulatory organziations for regulatory reporting purposes.  For example, unique swap identifier (USI) as required by the U.S. Commodity Futures Trading Commission.  */
SideRegulatoryTradeIDSource                        = 1973, /*!< 1973 (String FIX.5.0SP2) Identifies the  reporting entity that originated the value in SideRegulatoryTradeID(1972).  The reporting entity identifier may be assigned by a regulator.*/
SideRegulatoryTradeIDEvent                         = 1974, /*!< 1974 (int FIX.5.0SP2) Identifies the event which caused origination of the identifier in SideRegulatoryTradeID(1972). When more than one event is the cause, use the higher enumeration value. For example, if the identifier is originated due to an allocated trade which was cleared and reported, use the enumeration value 2 (Clearing).*/
SideRegulatoryTradeIDType                          = 1975, /*!< 1975 (int FIX.5.0SP2) Specifies the type of trade identifier provided in SideRegulatoryTradeID(1972), within the context of the hierarchy of trade events.*/
NoSecondaryAssetClasses                            = 1976, /*!< 1976 (NumInGroup FIX.5.0SP2) Number of secondary asset classes in the repeating group.*/
SecondaryAssetClass                                = 1977, /*!< 1977 (int FIX.5.0SP2) The broad asset category for assessing risk exposure for a multi-asset trade.*/
SecondaryAssetSubClass                             = 1978, /*!< 1978 (int FIX.5.0SP2) An indication of the general description of the asset class. */
SecondaryAssetType                                 = 1979, /*!< 1979 (String FIX.5.0SP2) Used to provide more specific description of the asset specified in SecondaryAssetSubClass(1978).

See https://www.fixtrading.org/codelists/AssetType for code list of applicable values.  ISO 4721 Currency Code values are to be used when specific currency as an asset type is to be expressed.

Other values may be used by mutual agreement of the counterparties.*/
BlockTrdAllocIndicator                             = 1980, /*!< 1980 (int FIX.5.0SP2) Indication that a block trade will be allocated.*/
NoUnderlyingEvents                                 = 1981, /*!< 1981 (NumInGroup FIX.5.0SP2) Number of events in the repeating group.*/
UnderlyingEventType                                = 1982, /*!< 1982 (int FIX.5.0SP2) Code to represent the type of event.*/
UnderlyingEventDate                                = 1983, /*!< 1983 (LocalMktDate FIX.5.0SP2) The date of the event.*/
UnderlyingEventTime                                = 1984, /*!< 1984 (UTCTimestamp FIX.5.0SP2) The time of the event. To be used in combination with UnderlyingEventDate(1983).*/
UnderlyingEventTimeUnit                            = 1985, /*!< 1985 (String FIX.5.0SP2) Time unit associated with the event. */
UnderlyingEventTimePeriod                          = 1986, /*!< 1986 (int FIX.5.0SP2) Time unit multiplier for the event. */
UnderlyingEventPx                                  = 1987, /*!< 1987 (Price FIX.5.0SP2) Predetermined price of issue at event, if applicable.*/
UnderlyingConstituentWeight                        = 1988, /*!< 1988 (float FIX.5.0SP2) For a basket, or pool, describes the weight of each of the constituents within the basket. If not provided, it is assumed to be equal weighted.*/
UnderlyingCouponType                               = 1989, /*!< 1989 (int FIX.5.0SP2) Specifies the coupon type of the underlying bond. */
UnderlyingTotalIssuedAmount                        = 1990, /*!< 1990 (Amt FIX.5.0SP2) Specifies the total amount of the issue. Corresponds to the par value multiplied by the number of issued security.*/
UnderlyingCouponFrequencyPeriod                    = 1991, /*!< 1991 (int FIX.5.0SP2) Time unit multiplier for the frequency of the bond's coupon payment.*/
UnderlyingCouponFrequencyUnit                      = 1992, /*!< 1992 (String FIX.5.0SP2) Time unit associated with the frequency of the bond's coupon payment.*/
UnderlyingCouponDayCount                           = 1993, /*!< 1993 (int FIX.5.0SP2) The day count convention used in interest calculations for a bond or an interest bearing security.*/
UnderlyingObligationID                             = 1994, /*!< 1994 (String FIX.5.0SP2) For a CDS basket or pool identifies the reference obligation.*/
UnderlyingObligationIDSource                       = 1995, /*!< 1995 (String FIX.5.0SP2) Identifies the source scheme of the UnderlyingObligationID(1994).*/
UnderlyingEquityID                                 = 1996, /*!< 1996 (String FIX.5.0SP2) Specifies the equity in which a convertible bond can be converted.*/
UnderlyingEquityIDSource                           = 1997, /*!< 1997 (String FIX.5.0SP2) Identifies the source of the UnderlyingEquityID(1996).*/
UnderlyingLienSeniority                            = 1998, /*!< 1998 (int FIX.5.0SP2) Indicates the seniority level of the lien in a loan. */
UnderlyingLoanFacility                             = 1999, /*!< 1999 (int FIX.5.0SP2) Specifies the type of loan when the credit default swap's reference obligation is a loan.*/
UnderlyingReferenceEntityType                      = 2000, /*!< 2000 (int FIX.5.0SP2) Specifies the type of reference entity for first-to-default CDS basket contracts.*/
StrikeIndexSpread                                  = 2001, /*!< 2001 (PriceOffset FIX.5.0SP2) Specifies the strike price offset from the named index.*/
ValuationSource                                    = 2002, /*!< 2002 (String FIX.5.0SP2) Specifies the source of trade valuation data.*/
UnderlyingIndexSeries                              = 2003, /*!< 2003 (int FIX.5.0SP2) The series identifier of a credit default swap index. */
UnderlyingIndexAnnexVersion                        = 2004, /*!< 2004 (int FIX.5.0SP2) The version identifier of a credit default swap index annex.*/
UnderlyingIndexAnnexDate                           = 2005, /*!< 2005 (LocalMktDate FIX.5.0SP2) The date of a credit default swap index series annex.*/
UnderlyingIndexAnnexSource                         = 2006, /*!< 2006 (String FIX.5.0SP2) The source of a credit default swap index series annex.*/
UnderlyingProductComplex                           = 2007, /*!< 2007 (String FIX.5.0SP2) Identifies an entire suite of products for a given market. In Futures this may be "interest rates", "agricultural", "equity indexes", etc*/
UnderlyingSecurityGroup                            = 2008, /*!< 2008 (String FIX.5.0SP2) An exchange specific name assigned to a group of related securities which may be concurrently affected by market events and actions.*/
UnderlyingSettleOnOpenFlag                         = 2009, /*!< 2009 (String FIX.5.0SP2) Indicator to determine if Instrument is Settle on Open.*/
UnderlyingAssignmentMethod                         = 2010, /*!< 2010 (char FIX.5.0SP2) Method under which assignment was conducted*/
UnderlyingSecurityStatus                           = 2011, /*!< 2011 (String FIX.5.0SP2) Gives the current state of the instrument*/
UnderlyingObligationType                           = 2012, /*!< 2012 (String FIX.5.0SP2) Type of reference obligation for credit derivatives contracts.*/
UnderlyingAssetClass                               = 2013, /*!< 2013 (int FIX.5.0SP2) The broad asset category for assessing risk exposure.*/
UnderlyingAssetSubClass                            = 2014, /*!< 2014 (int FIX.5.0SP2) An indication of the general description of the asset class. */
UnderlyingAssetType                                = 2015, /*!< 2015 (String FIX.5.0SP2) Used to provide more specific description of the asset specified in UnderlyingAssetSubClass(2082).

See https://www.fixtrading.org/codelists/AssetType for code list of applicable values.  ISO 4721 Currency Code values are to be used when specific currency as an asset type is to be expressed.

Other values may be used by mutual agreement of the counterparties.*/
UnderlyingSwapClass                                = 2016, /*!< 2016 (String FIX.5.0SP2) The type or classification of swap. Additional values may be used by mutual agreement of the counterparties.*/
UnderlyingNthToDefault                             = 2017, /*!< 2017 (int FIX.5.0SP2) The Nth reference obligation to default in a CDS reference basket. If specified without UnderlyingMthToDefault(2018) the default will trigger a CDS payout.  If UnderlyingMthToDefault(2018) is also present then payout occurs between the Nth and Mth obligations to default.*/
UnderlyingMthToDefault                             = 2018, /*!< 2018 (int FIX.5.0SP2) The Mth reference obligation to default in a CDS reference basket. When UnderlyingNthToDefault(2017) and UnderlyingMthToDefault(2018) are represented then the CDS payout occurs between the Nth and Mth obligations to default.*/
UnderlyingSettledEntityMatrixSource                = 2019, /*!< 2019 (String FIX.5.0SP2) Relevant settled entity matrix source.*/
UnderlyingSettledEntityMatrixPublicationDate       = 2020, /*!< 2020 (LocalMktDate FIX.5.0SP2) Specifies the publication date of the applicable version of the matrix. If not specified, the Standard Terms Supplement defines rules for which version of the matrix is applicable.*/
UnderlyingStrikeMultiplier                         = 2021, /*!< 2021 (float FIX.5.0SP2) Used for derivatives. Multiplier applied to the strike price for the purpose of calculating the settlement value.*/
UnderlyingStrikeValue                              = 2022, /*!< 2022 (float FIX.5.0SP2) Used for derivatives. The number of shares/units for the financial instrument involved in the option trade.*/
UnderlyingStrikePriceDeterminationMethod           = 2023, /*!< 2023 (int FIX.5.0SP2) Specifies how the strike price is determined at the point of option exercise. The strike may be fixed throughout the life of the option, set at expiration to the value of the underlying, set to the average value of the underlying , or set to the optimal value of the underlying.*/
UnderlyingStrikePriceBoundaryMethod                = 2024, /*!< 2024 (int FIX.5.0SP2) Specifies the boundary condition to be used for the strike price relative to the underlying price at the point of option exercise.*/
UnderlyingStrikePriceBoundaryPrecision             = 2025, /*!< 2025 (Percentage FIX.5.0SP2) Used in combination with StrikePriceBoundaryMethod(1479) to specify the percentage of the strike price in relation to the underlying price. The percentage is generally 100 or greater for puts and 100 or less for calls.*/
UnderlyingMinPriceIncrement                        = 2026, /*!< 2026 (float FIX.5.0SP2) Minimum price increment for the instrument. Could also be used to represent tick value.*/
UnderlyingMinPriceIncrementAmount                  = 2027, /*!< 2027 (Amt FIX.5.0SP2) Minimum price increment amount associated with the UnderlyingMinPriceIncrement(2026). For listed derivatives, the value can be calculated by multiplying UnderlyingMinPriceIncrement(2026) by UnderlyingContractMultiplier(436).*/
UnderlyingOptPayoutType                            = 2028, /*!< 2028 (int FIX.5.0SP2) Indicates the type of valuation method or payout trigger for an in-the-money option.*/
UnderlyingOptPayoutAmount                          = 2029, /*!< 2029 (Amt FIX.5.0SP2) Cash amount indicating the pay out associated with an option. For binary options this is a fixed amount.*/
UnderlyingPriceQuoteMethod                         = 2030, /*!< 2030 (String FIX.5.0SP2) Method for price quotation.*/
UnderlyingValuationMethod                          = 2031, /*!< 2031 (String FIX.5.0SP2) Indicates type of valuation method used.*/
UnderlyingListMethod                               = 2032, /*!< 2032 (int FIX.5.0SP2) Indicates whether the instruments are pre-listed only or can also be defined via user request.*/
UnderlyingCapPrice                                 = 2033, /*!< 2033 (Price FIX.5.0SP2) Used to express the ceiling price of a capped call.*/
UnderlyingFloorPrice                               = 2034, /*!< 2034 (Price FIX.5.0SP2) Used to express the floor price of a capped put.*/
UnderlyingFlexibleIndicator                        = 2035, /*!< 2035 (Boolean FIX.5.0SP2) Used to indicate if a security has been defined as flexible according to "non-standard" means. Analog to CFICode Standard/Non-standard indicator.*/
UnderlyingFlexProductEligibilityIndicator          = 2036, /*!< 2036 (Boolean FIX.5.0SP2) Used to indicate if a product or group of product supports the creation of flexible securities.*/
UnderlyingPositionLimit                            = 2037, /*!< 2037 (int FIX.5.0SP2) Position limit for the instrument.*/
UnderlyingNTPositionLimit                          = 2038, /*!< 2038 (int FIX.5.0SP2) Position Limit in the near-term contract for a given exchange-traded product.*/
UnderlyingPool                                     = 2039, /*!< 2039 (String FIX.5.0SP2) Identifies the mortgage backed security (MBS) / asset backed security (ABS) pool.*/
UnderlyingContractSettlMonth                       = 2040, /*!< 2040 (MonthYear FIX.5.0SP2) Specifies when the contract (i.e. MBS/TBA) will settle.  Must be present for MBS/TBA.*/
UnderlyingDatedDate                                = 2041, /*!< 2041 (LocalMktDate FIX.5.0SP2) If different from IssueDate()*/
UnderlyingInterestAccrualDate                      = 2042, /*!< 2042 (LocalMktDate FIX.5.0SP2) If different from IssueDate and DatedDate*/
UnderlyingShortSaleRestriction                     = 2043, /*!< 2043 (int FIX.5.0SP2) Indicates whether a restriction applies to short selling a security.*/
UnderlyingRefTickTableID                           = 2044, /*!< 2044 (int FIX.5.0SP2) Spread table code referred by the security or symbol.*/
NoUnderlyingComplexEvents                          = 2045, /*!< 2045 (NumInGroup FIX.5.0SP2) Number of complex events in the repeating group.*/
UnderlyingComplexEventType                         = 2046, /*!< 2046 (int FIX.5.0SP2) Identifies the type of complex event.*/
UnderlyingComplexOptPayoutAmount                   = 2047, /*!< 2047 (Amt FIX.5.0SP2) Cash amount indicating the pay out associated with an event. For binary options this is a fixed amount.*/
UnderlyingComplexEventPrice                        = 2048, /*!< 2048 (Price FIX.5.0SP2) Specifies the price at which the complex event takes effect. Impact of the event price is determined by the UnderlyingComplexEventType(2046).*/
UnderlyingComplexEventPriceBoundaryMethod          = 2049, /*!< 2049 (int FIX.5.0SP2) Specifies the boundary condition to be used for the event price relative to the UnderlyingComplexEventPrice(2048) at the point the complex event outcome takes effect as determined by the UnderlyingComplexEventPriceTimeType(2051).*/
UnderlyingComplexEventPriceBoundaryPrecision       = 2050, /*!< 2050 (Percentage FIX.5.0SP2) Used in combination with UnderlyingComplexEventPriceBoundaryMethod(2049) to specify the percentage of the strike price in relation to the  underlying price. The percentage is generally 100 or greater for puts and 100 or less for calls.*/
UnderlyingComplexEventPriceTimeType                = 2051, /*!< 2051 (int FIX.5.0SP2) Specifies when the complex event outcome takes effect. The outcome of a complex event is a payout or barrier action as specified by the UnderlyingComplexEventType(2046).*/
UnderlyingComplexEventCondition                    = 2052, /*!< 2052 (int FIX.5.0SP2) 	



Specifies the condition between complex events when more than one event is specified.



Multiple barrier events would use an "or" condition since only one can be effective at a given time. A set of digital range events would use an "and" condition since both conditions must be in effect for a payout to result.*/
NoUnderlyingComplexEventDates                      = 2053, /*!< 2053 (NumInGroup FIX.5.0SP2) Number of underlying complex event dates in the repeating group.*/
UnderlyingComplexEventStartDate                    = 2054, /*!< 2054 (UTCDateOnly FIX.5.0SP2) The start date of the date range on which a complex event is effective. The start date will be set equal to the end date for single day events such as Bermuda options.

The start date must always be less than or equal to end date.*/
UnderlyingComplexEventEndDate                      = 2055, /*!< 2055 (UTCDateOnly FIX.5.0SP2) The end date of the date range on which a complex event is effective. The start date will be set equal to the end date for single day events such as Bermuda options.

UnderlyingComplexEventEndDate(2056) must always be greater than or equal to UnderlyingComplexEventStartDate(2055).*/
NoUnderlyingComplexEventTimes                      = 2056, /*!< 2056 (NumInGroup FIX.5.0SP2) Number of complex event times in the repeating group.*/
UnderlyingComplexEventStartTime                    = 2057, /*!< 2057 (UTCTimeOnly FIX.5.0SP2) The start time of the time range on which a complex event date is effective.

UnderlyingComplexEventStartTime(2057) must always be less than or equal to UndelryingComplexEventEndTime(2058).*/
UnderlyingComplexEventEndTime                      = 2058, /*!< 2058 (UTCTimeOnly FIX.5.0SP2) The end time of the time range on which a complex event date is effective.

UnderlyingComplexEventEndTime(2058) must always be greater than or equal to UnderlyingComplexEventStartTime(2057).*/
NoLegEvents                                        = 2059, /*!< 2059 (NumInGroup FIX.5.0SP2) Number of events in the repeating group*/
LegEventType                                       = 2060, /*!< 2060 (int FIX.5.0SP2) Code to represent the type of event.*/
LegEventDate                                       = 2061, /*!< 2061 (LocalMktDate FIX.5.0SP2) The date of the event.*/
LegEventTime                                       = 2062, /*!< 2062 (UTCTimestamp FIX.5.0SP2) Specific time of event. To be used in combination with LegEventDate(2061).*/
LegEventTimeUnit                                   = 2063, /*!< 2063 (String FIX.5.0SP2) Time unit associated with the event. */
LegEventTimePeriod                                 = 2064, /*!< 2064 (int FIX.5.0SP2) Time unit multiplier for the event. */
LegEventPx                                         = 2065, /*!< 2065 (Price FIX.5.0SP2) Predetermined price of issue at event, if applicable.*/
LegEventText                                       = 2066, /*!< 2066 (String FIX.5.0SP2) Free form text to specify additional information or enumeration description when a standard value does not apply.*/
LegAssetClass                                      = 2067, /*!< 2067 (int FIX.5.0SP2) The broad asset category for assessing risk exposure.*/
LegAssetSubClass                                   = 2068, /*!< 2068 (int FIX.5.0SP2) The general subcategory description of the asset class. */
LegAssetType                                       = 2069, /*!< 2069 (String FIX.5.0SP2) Used to provide more specific description of the asset specified in LegAssetSubClass(2068).

See https://www.fixtrading.org/codelists/AssetType for code list of applicable values.  ISO 4721 Currency Code values are to be used when specific currency as an asset type is to be expressed.

Other values may be used by mutual agreement of the counterparties.*/
LegSwapClass                                       = 2070, /*!< 2070 (String FIX.5.0SP2) Swap type.*/
UnderlyingEventText                                = 2071, /*!< 2071 (String FIX.5.0SP2) Free form text to specify comments related to the event.*/
EncodedUnderlyingEventTextLen                      = 2072, /*!< 2072 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingEventText(2073) field.*/
EncodedUnderlyingEventText                         = 2073, /*!< 2073 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingEventText(2071) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingEventText(2071) field.*/
EncodedLegEventTextLen                             = 2074, /*!< 2074 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegEventText(2075) field.*/
EncodedLegEventText                                = 2075, /*!< 2075 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegEventText(2066) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegEventText(2066) field.*/
NoLegSecondaryAssetClasses                         = 2076, /*!< 2076 (NumInGroup FIX.5.0SP2) Number of secondary asset classes in the repeating group.*/
LegSecondaryAssetClass                             = 2077, /*!< 2077 (int FIX.5.0SP2) The broad asset category for assessing risk exposure for a multi-asset trade.*/
LegSecondaryAssetSubClass                          = 2078, /*!< 2078 (int FIX.5.0SP2) An indication of the general description of the asset class. */
LegSecondaryAssetType                              = 2079, /*!< 2079 (String FIX.5.0SP2) Used to provide more specific description of the asset specified in LegSecondaryAssetSubClass(2078).

See https://www.fixtrading.org/codelists/AssetType for code list of applicable values.  ISO 4721 Currency Code values are to be used when specific currency as an asset type is to be expressed.

Other values may be used by mutual agreement of the counterparties.*/
NoUnderlyingSecondaryAssetClasses                  = 2080, /*!< 2080 (NumInGroup FIX.5.0SP2) Number of secondary asset classes in the repeating group.*/
UnderlyingSecondaryAssetClass                      = 2081, /*!< 2081 (int FIX.5.0SP2) The broad asset category for assessing risk exposure for a multi-asset trade.*/
UnderlyingSecondaryAssetSubClass                   = 2082, /*!< 2082 (int FIX.5.0SP2) An indication of the general description of the asset class.*/
UnderlyingSecondaryAssetType                       = 2083, /*!< 2083 (String FIX.5.0SP2) Used to provide more specific description of the asset specified in UnderlyingSecondaryAssetSubClass(2082).

See https://www.fixtrading.org/codelists/AssetType for code list of applicable values.  ISO 4721 Currency Code values are to be used when specific currency as an asset type is to be expressed.

Other values may be used by mutual agreement of the counterparties.*/
PreviousClearingBusinessDate                       = 2084, /*!< 2084 (LocalMktDate FIX.5.0SP2) The date of the previous clearing business day.*/
ValuationDate                                      = 2085, /*!< 2085 (LocalMktDate FIX.5.0SP2) The valuation date of the trade.*/
ValuationTime                                      = 2086, /*!< 2086 (LocalMktTime FIX.5.0SP2) The valuation time of the trade.*/
ValuationBusinessCenter                            = 2087, /*!< 2087 (String FIX.5.0SP2) Identifies the business center whose calendar is used for valuation, e.g. "GLOB". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
MarginAmtFXRate                                    = 2088, /*!< 2088 (float FIX.5.0SP2) Foreign exchange rate used to compute the MarginAmt(1645) from the MarginAmtCcy(1646) and the Currency(15).*/
MarginAmtFXRateCalc                                = 2089, /*!< 2089 (char FIX.5.0SP2) Specifies whether or not MarginAmtFXRate(2088) should be multipled or divided.*/
CollateralFXRate                                   = 2090, /*!< 2090 (float FIX.5.0SP2) Foreign exchange rate used to compute the CurrentCollateralAmount(1704) from the CollateralCurrency(1646) and the Currency(15).*/
CollateralFXRateCalc                               = 2091, /*!< 2091 (char FIX.5.0SP2) Specifies whether or not CollateralFXRate(2090) should be multipled or divided.*/
CollateralAmountMarketSegmentID                    = 2092, /*!< 2092 (String FIX.5.0SP2) Market segment associated with the collateral amount.*/
CollateralAmountMarketID                           = 2093, /*!< 2093 (String FIX.5.0SP2) Market associated with the collateral amount.*/
PayCollectFXRate                                   = 2094, /*!< 2094 (float FIX.5.0SP2) Foreign exchange rate used to compute the PayAmount(1710) or CollectAmount(1711) from the PayCollectCurrency(1709) and the Currency(15).*/
PayCollectFXRateCalc                               = 2095, /*!< 2095 (char FIX.5.0SP2) Specifies whether or not PayCollectFXRate(2094) should be multipled or divided.*/
PosAmtStreamDesc                                   = 2096, /*!< 2096 (String FIX.5.0SP2) Corresponds to the value in StreamDesc(40051) in the StreamGrp component.*/
PositionFXRate                                     = 2097, /*!< 2097 (float FIX.5.0SP2) Foreign exchange rate used to compute the PosAmt(708) from the PositionCurrency(1055) and the Currency (15).*/
PositionFXRateCalc                                 = 2098, /*!< 2098 (char FIX.5.0SP2) Specifies whether or not PositionFXRate(2097) should be multipled or divided.*/
PosAmtMarketSegmentID                              = 2099, /*!< 2099 (String FIX.5.0SP2) Market segment associated with the position amount.*/
PosAmtMarketID                                     = 2100, /*!< 2100 (String FIX.5.0SP2) Market associated with the position amount.*/
TerminatedIndicator                                = 2101, /*!< 2101 (Boolean FIX.5.0SP2) Indicates if the position has been terminated.*/
ShortMarkingExemptIndicator                        = 2102, /*!< 2102 (Boolean FIX.5.0SP2) Indicates whether the originating account is exempt (Y) from marking orders as short or not (N). This designation may be used on both buy and sell orders.*/
RelatedRegulatoryTradeIDSource                     = 2103, /*!< 2103 (String FIX.5.0SP2) Specifies the identifier of the reporting entity as assigned by regulatory agency.*/
NoAttachments                                      = 2104, /*!< 2104 (NumInGroup FIX.5.0SP2) The number of attached files.*/
AttachmentName                                     = 2105, /*!< 2105 (String FIX.5.0SP2) Specifies the file name of the attachment.*/
AttachmentMediaType                                = 2106, /*!< 2106 (String FIX.5.0SP2) The MIME media type (and optional subtype) of the attachment.  The values used are those assigned, listed and maintained by IANA (www.iana.org) [RFC2046]. See http://www.iana.org/assignments/media-types/index.html for available types.





Examples values (RFC number provided for reference here only):

"application/pdf" (see [RFC3778])

"application/msword" (for .doc files)

"multipart/signed" (see [RFC1847])

"application/vnd.openxmlformats-officedocument.wordprocessingml.document" (for .docx files)

*/
AttachmentClassification                           = 2107, /*!< 2107 (String FIX.5.0SP2) Specifies  semantically the type of the attached document from a business perspective. The default classification scheme reuses the FIX standard classification scheme of a high level section (pretrade, trade, posttrade, etc.) and a category, then a specific application or document type. The expression follows {"section/category/application type"}.



The goal here is to map the attachment into the sections and categories of the FIX business messages if possible. The classification scheme can be expanded or replaced by counterparty agreement. This approach permits the introduction and reference to other business ontologies.



Example:

posttrade/confirmation/confirm

pretrade//termsheet*/
AttachmentExternalURL                              = 2108, /*!< 2108 (String FIX.5.0SP2) Used to specify an external URL where the attachment can be obtained.*/
AttachmentEncodingType                             = 2109, /*!< 2109 (int FIX.5.0SP2) The encoding type of the content provided in EncodedAttachment(2112).*/
UnencodedAttachmentLen                             = 2110, /*!< 2110 (int FIX.5.0SP2) Unencoded content length in bytes. Can be used to validate successful unencoding.*/
EncodedAttachmentLen                               = 2111, /*!< 2111 (Length FIX.5.0SP2) Byte length of encoded the EncodedAttachment(2112) field.*/
EncodedAttachment                                  = 2112, /*!< 2112 (data FIX.5.0SP2) The content of the attachment in the encoding format specified in the AttachmentEncodingType(2109) field.*/
NoAttachmentKeywords                               = 2113, /*!< 2113 (NumInGroup FIX.5.0SP2) The number of attachment keywords.*/
AttachmentKeyword                                  = 2114, /*!< 2114 (String FIX.5.0SP2) Can be used to provide data or keyword tagging of the content of the attachment.*/
NegotiationMethod                                  = 2115, /*!< 2115 (int FIX.5.0SP2) Specifies the negotiation method to be used.*/
NextAuctionTime                                    = 2116, /*!< 2116 (UTCTimestamp FIX.5.0SP2) The time of the next auction.*/
ComplexOptPayoutPaySide                            = 2117, /*!< 2117 (int FIX.5.0SP2) Trade side of payout payer.*/
ComplexOptPayoutReceiveSide                        = 2118, /*!< 2118 (int FIX.5.0SP2) Trade side of payout receiver.*/
ComplexOptPayoutUnderlier                          = 2119, /*!< 2119 (String FIX.5.0SP2) Reference to the underlier whose payments are being passed through.*/
ComplexOptPayoutPercentage                         = 2120, /*!< 2120 (Percentage FIX.5.0SP2) Percentage of observed price for calculating the payout associated with the event.*/
ComplexOptPayoutTime                               = 2121, /*!< 2121 (int FIX.5.0SP2) Specifies when the payout is to occur.*/
ComplexOptPayoutCurrency                           = 2122, /*!< 2122 (Currency FIX.5.0SP2) Specifies the currency of the payout amount. Uses ISO 4217 currency codes.*/
ComplexEventPricePercentage                        = 2123, /*!< 2123 (Percentage FIX.5.0SP2) Specifies the price percentage at which the complex event takes effect. Impact of the event price is determined by the ComplexEventType(1484).*/
ComplexEventCurrencyOne                            = 2124, /*!< 2124 (Currency FIX.5.0SP2) Specifies the first or only reference currency of the trade. Uses ISO 4217 currency codes.*/
ComplexEventCurrencyTwo                            = 2125, /*!< 2125 (Currency FIX.5.0SP2) Specifies the second reference currencyof the trade. Uses ISO 4217 currency codes.*/
ComplexEventQuoteBasis                             = 2126, /*!< 2126 (int FIX.5.0SP2) For foreign exchange Quanto option feature.*/
ComplexEventFixedFXRate                            = 2127, /*!< 2127 (float FIX.5.0SP2) Specifies the fixed FX rate alternative for FX Quantro options.*/
ComplexEventDeterminationMethod                    = 2128, /*!< 2128 (String FIX.5.0SP2) Specifies the method according to which an amount or a date is determined.  

See http://www.fpml.org/coding-scheme/determination-method for values.*/
ComplexEventCalculationAgent                       = 2129, /*!< 2129 (int FIX.5.0SP2) Used to identify the calculation agent.*/
ComplexEventStrikePrice                            = 2130, /*!< 2130 (Price FIX.5.0SP2) Upper strike price for Asian option feature. Strike percentage for a Strike Spread.*/
ComplexEventStrikeFactor                           = 2131, /*!< 2131 (float FIX.5.0SP2) Strike factor for Asian option feature. Upper strike percentage for a Strike Spread.*/
ComplexEventStrikeNumberOfOptions                  = 2132, /*!< 2132 (int FIX.5.0SP2) Upper string number of options for a Strike Spread.*/
ComplexEventCreditEventsXIDRef                     = 2133, /*!< 2133 (XIDREF FIX.5.0SP2) Reference to credit event table elsewhere in the message.*/
ComplexEventCreditEventNotifyingParty              = 2134, /*!< 2134 (int FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring. */
ComplexEventCreditEventBusinessCenter              = 2135, /*!< 2135 (String FIX.5.0SP2) The local business center for which the credit event is to be determined. The inclusion of this business center implies that Greenwich Mean Time in Section 3.3 of the 2003 ISDA Credit Derivatives Definitions is replaced by the local time of the specified business center. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ComplexEventCreditEventStandardSources             = 2136, /*!< 2136 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', indicates that ISDA defined Standard Public Sources are applicable.*/
ComplexEventCreditEventMinimumSources              = 2137, /*!< 2137 (int FIX.5.0SP2) The minimum number of the specified public information sources that must publish information that reasonably confirms that a credit event has occurred. The market convention is two.*/
ComplexEventXID                                    = 2138, /*!< 2138 (XID FIX.5.0SP2) Identifier of this complex event for cross referencing elsewhere in the message.*/
ComplexEventXIDRef                                 = 2139, /*!< 2139 (XIDREF FIX.5.0SP2) Reference to a complex event elsewhere in the message.*/
ValuationReferenceModel                            = 2140, /*!< 2140 (String FIX.5.0SP2) Specifies the methodology and/or assumptions used to generate the  trade value.*/
StrategyType                                       = 2141, /*!< 2141 (String FIX.5.0SP2) Specifies the type of trade strategy.*/
CommonPricingIndicator                             = 2142, /*!< 2142 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', it indicates that common pricing applies. Common pricing may be relevant for a transaction that references more than one commodity reference price.*/
SettlDisruptionProvision                           = 2143, /*!< 2143 (int FIX.5.0SP2) Specifies the consequences of bullion settlement disruption events. */
InstrumentRoundingDirection                        = 2144, /*!< 2144 (char FIX.5.0SP2) Specifies the rounding direction if not overridden elsewhere.*/
InstrumentRoundingPrecision                        = 2145, /*!< 2145 (int FIX.5.0SP2) Specifies the rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
LegSettleOnOpenFlag                                = 2146, /*!< 2146 (String FIX.5.0SP2) Indicator to determine if the instrument is to settle on open.*/
LegInstrmtAssignmentMethod                         = 2147, /*!< 2147 (char FIX.5.0SP2) Specifies the method under which assignment was conducted.*/
LegSecurityStatus                                  = 2148, /*!< 2148 (String FIX.5.0SP2) Used for derivatives. Denotes the current state of the InstrumentLeg.*/
LegRestructuringType                               = 2149, /*!< 2149 (String FIX.5.0SP2) A category of CDS credit event in which the underlying bond experiences a restructuring.

Used to define a CDS instrument.*/
LegSeniority                                       = 2150, /*!< 2150 (String FIX.5.0SP2) Specifies which issue (underlying bond) will receive payment priority in the event of a default.

Used to define a CDS instrument.*/
LegNotionalPercentageOutstanding                   = 2151, /*!< 2151 (Percentage FIX.5.0SP2) Indicates the notional percentage of the deal that is still outstanding based on the remaining components of the index.

Used to calculate the true value of a CDS trade or position.*/
LegOriginalNotionalPercentageOutstanding           = 2152, /*!< 2152 (Percentage FIX.5.0SP2) Used to reflect the Original value prior to the application of a credit event. See LegNotionalPercentageOutstanding(2151).*/
LegAttachmentPoint                                 = 2153, /*!< 2153 (Percentage FIX.5.0SP2) Lower bound percentage of the loss that the tranche can endure.*/
LegDetachmentPoint                                 = 2154, /*!< 2154 (Percentage FIX.5.0SP2) Upper bound percentage of the loss the tranche can endure.*/
LegObligationType                                  = 2155, /*!< 2155 (String FIX.5.0SP2) Type of reference obligation for credit derivatives contracts.*/
LegSwapSubClass                                    = 2156, /*!< 2156 (String FIX.5.0SP2) The sub-classification or notional schedule type of the swap.*/
LegNthToDefault                                    = 2157, /*!< 2157 (int FIX.5.0SP2) The Nth reference obligation in a CDS reference basket. If specified without LegMthToDefault(2158) the default will trigger a CDS payout. If LegMthToDefault(2158) is also present then payout occurs between the Nth and Mth obligations to default.*/
LegMthToDefault                                    = 2158, /*!< 2158 (int FIX.5.0SP2) The Mth reference obligation to default in a CDS reference basket. When an NthToDefault(2157) to MthToDefault(2158) are represented then the CDS payout occurs between the Nth and Mth obligations to default.*/
LegSettledEntityMatrixSource                       = 2159, /*!< 2159 (String FIX.5.0SP2) Relevant settled entity matrix source.*/
LegSettledEntityMatrixPublicationDate              = 2160, /*!< 2160 (LocalMktDate FIX.5.0SP2) The publication date of the applicable version of the matrix. When this element is omitted, the Standard Terms Supplement defines rules for which version of the matrix is applicable.*/
LegCouponType                                      = 2161, /*!< 2161 (int FIX.5.0SP2) Specifies the coupon type of the bond.*/
LegTotalIssuedAmount                               = 2162, /*!< 2162 (Amt FIX.5.0SP2) Specifies the total amount of the issue. Corresponds to the par value multiplied by the number of issued security.*/
LegCouponFrequencyPeriod                           = 2163, /*!< 2163 (int FIX.5.0SP2) Time unit multiplier for the frequency of the bond's coupon payment. */
LegCouponFrequencyUnit                             = 2164, /*!< 2164 (String FIX.5.0SP2) Time unit associated with the frequency of the bond's coupon payment.*/
LegCouponDayCount                                  = 2165, /*!< 2165 (int FIX.5.0SP2) The day count convention used in interest calculations for a bond or an interest bearing security.*/
LegConvertibleBondEquityID                         = 2166, /*!< 2166 (String FIX.5.0SP2) Identifies the equity in which a convertible bond can be converted to.*/
LegConvertibleBondEquityIDSource                   = 2167, /*!< 2167 (String FIX.5.0SP2) Identifies class or source of the LegConvertibleBondEquitySecurityID(2166) value.*/
LegContractPriceRefMonth                           = 2168, /*!< 2168 (MonthYear FIX.5.0SP2) Reference month if there is no applicable LegMaturityMonthYear(610) value for the contract or security.*/
LegLienSeniority                                   = 2169, /*!< 2169 (int FIX.5.0SP2) Indicates the seniority level of the lien in a loan.*/
LegLoanFacility                                    = 2170, /*!< 2170 (int FIX.5.0SP2) Specifies the type of loan when the credit default swap's reference obligation is a loan.*/
LegReferenceEntityType                             = 2171, /*!< 2171 (int FIX.5.0SP2) Specifies the type of reference entity for first-to-default CDS basket contracts.*/
LegIndexSeries                                     = 2172, /*!< 2172 (int FIX.5.0SP2) The series identifier of a credit default swap index.*/
LegIndexAnnexVersion                               = 2173, /*!< 2173 (int FIX.5.0SP2) The version of a credit default swap index annex.*/
LegIndexAnnexDate                                  = 2174, /*!< 2174 (LocalMktDate FIX.5.0SP2) The date of a credit default swap index series annex.*/
LegIndexAnnexSource                                = 2175, /*!< 2175 (String FIX.5.0SP2) The source of a credit default swap series annex.*/
LegSettlRateIndex                                  = 2176, /*!< 2176 (String FIX.5.0SP2) In an outright or forward commodity trade that is cash settled this is the index used to determine the cash payment.*/
LegSettlRateIndexLocation                          = 2177, /*!< 2177 (String FIX.5.0SP2) This is an optional qualifying attribute of LegSettlementRateIndex(2176) such as the delivery zone for an electricity contract.*/
LegOptionExpirationDesc                            = 2178, /*!< 2178 (String FIX.5.0SP2) Description of the option expiration.*/
EncodedLegOptionExpirationDescLen                  = 2179, /*!< 2179 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegOptionExpirationDesc(2180) field.*/
EncodedLegOptionExpirationDesc                     = 2180, /*!< 2180 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegOptionExpirationDesc(2178) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegOptionExpirationDesc(2178).*/
LegStrikeMultiplier                                = 2181, /*!< 2181 (float FIX.5.0SP2) Used for derivatives. Multiplier applied to the strike price for the purpose of calculating the settlement value.*/
LegStrikeValue                                     = 2182, /*!< 2182 (float FIX.5.0SP2) The number of shares/units for the financial instrument involved in the option trade. Used for derivatives.*/
LegStrikeUnitOfMeasure                             = 2183, /*!< 2183 (String FIX.5.0SP2) Used to express the unit of measure (UOM) of the price if different from the contract. */
LegStrikeIndex                                     = 2184, /*!< 2184 (String FIX.5.0SP2) Specifies the index used to calculate the strike price.*/
LegStrikeIndexSpread                               = 2185, /*!< 2185 (PriceOffset FIX.5.0SP2) Specifies the strike price offset from the named index.*/
LegStrikePriceDeterminationMethod                  = 2186, /*!< 2186 (int FIX.5.0SP2) Specifies how the strike price is determined at the point of option exercise. The strike may be fixed throughout the life of the option, set at expiration to the value of the underlying, set to the average value of the underlying , or set to the optimal value of the underlying.*/
LegStrikePriceBoundaryMethod                       = 2187, /*!< 2187 (int FIX.5.0SP2) Specifies the boundary condition to be used for the strike price relative to the underlying price at the point of option exercise.*/
LegStrikePriceBoundaryPrecision                    = 2188, /*!< 2188 (Percentage FIX.5.0SP2) Used in combination with StrikePriceBoundaryMethod(2187) to specify the percentage of the strike price in relation to the underlying price. The percentage is generally 100 or greater for puts and 100 or less for calls.*/
LegUnderlyingPriceDeterminationMethod              = 2189, /*!< 2189 (int FIX.5.0SP2) Specifies how the underlying price is determined at the point of option exercise. The underlying price may be set to the current settlement price, set to a special reference, set to the optimal value of the underlying during the defined period ("Look-back") or set to the average value of the underlying during the defined period ("Asian option").*/
LegMinPriceIncrement                               = 2190, /*!< 2190 (float FIX.5.0SP2) Minimum price increment for a given exchange-traded instrument.  Could also be used to represent tick value.*/
LegMinPriceIncrementAmount                         = 2191, /*!< 2191 (Amt FIX.5.0SP2) Minimum price increment amount associated with the LegMinPriceIncrement(2190). For listed derivatives, the value can be calculated by multiplying LegMinPriceIncrement(2190) by LegContractMultiplier(614).*/
LegSettlMethod                                     = 2192, /*!< 2192 (String FIX.5.0SP2) Settlement method for a contract or instrument. Additional values may be used with bilateral agreement. */
LegOptPayoutType                                   = 2193, /*!< 2193 (int FIX.5.0SP2) Indicates the type of valuation method or trigger payout for an in-the-money option.*/
LegOptPayoutAmount                                 = 2194, /*!< 2194 (Amt FIX.5.0SP2) Cash amount indicating the pay out associated with an option. For binary options this is a fixed amount.*/
LegPriceQuoteMethod                                = 2195, /*!< 2195 (String FIX.5.0SP2) Specifies the method for price quotation.*/
LegValuationMethod                                 = 2196, /*!< 2196 (String FIX.5.0SP2) Specifies the type of valuation method applied.*/
LegValuationSource                                 = 2197, /*!< 2197 (String FIX.5.0SP2) Specifies the source of trade valuation data.*/
LegValuationReferenceModel                         = 2198, /*!< 2198 (String FIX.5.0SP2) Specifies the methodology and/or assumptions used to generate the trade value.*/
LegListMethod                                      = 2199, /*!< 2199 (int FIX.5.0SP2) Indicates whether instruments are pre-listed only or can also be defined via user request.*/
LegCapPrice                                        = 2200, /*!< 2200 (Price FIX.5.0SP2) Used to express the ceiling price of a capped call.*/
LegFloorPrice                                      = 2201, /*!< 2201 (Price FIX.5.0SP2) Used to express the floor price of a capped put.*/
LegFlexibleIndicator                               = 2202, /*!< 2202 (Boolean FIX.5.0SP2) Used to indicate a derivatives security that can be defined using flexible terms. The terms commonly permitted to be defined by market participants are expiration date and strike price. FlexibleIndicator is an alternative to LegCFICode(608) Standard/Non-standard attribute.*/
LegFlexProductEligibilityIndicator                 = 2203, /*!< 2203 (Boolean FIX.5.0SP2) Used to indicate if a product or group of product supports the creation of flexible securities.*/
LegComplexEventStartTime                           = 2204, /*!< 2204 (UTCTimeOnly FIX.5.0SP2) The start time of the time range on which a complex event date is effective.

The start time must always be less than or equal to the end time.*/
LegPositionLimit                                   = 2205, /*!< 2205 (int FIX.5.0SP2) Position Limit for a given exchange-traded product.*/
LegNTPositionLimit                                 = 2206, /*!< 2206 (int FIX.5.0SP2) Position limit in the near-term contract for a given exchange-traded product.*/
LegCPProgram                                       = 2207, /*!< 2207 (int FIX.5.0SP2) The program under which a commercial paper is issued.*/
LegCPRegType                                       = 2208, /*!< 2208 (String FIX.5.0SP2) The registration type of a commercial paper issuance.*/
LegShortSaleRestriction                            = 2209, /*!< 2209 (int FIX.5.0SP2) Indicates whether a restriction applies to short selling a security.*/
AssetGroup                                         = 2210, /*!< 2210 (int FIX.5.0SP2) Indicates the broad product or asset classification.  May be used to provide grouping for the product taxonomy (Product(460), SecurityType(167), etc.) and/or the risk taxonomy (AssetClass(1938), AssetSubClass(1939), AssetType(1940), etc.).*/
LegStrategyType                                    = 2211, /*!< 2211 (String FIX.5.0SP2) Specifies the type of trade strategy.*/
LegCommonPricingIndicator                          = 2212, /*!< 2212 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', it indicates that common pricing applies. Common pricing may be relevant for a transaction that references more than one commodity reference price.*/
LegSettlDisruptionProvision                        = 2213, /*!< 2213 (int FIX.5.0SP2) Specifies the consequences of bullion settlement disruption events. */
LegInstrumentRoundingDirection                     = 2214, /*!< 2214 (char FIX.5.0SP2) Specifies the rounding direction if not overridden elsewhere.*/
LegInstrumentRoundingPrecision                     = 2215, /*!< 2215 (int FIX.5.0SP2) Specifies the rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
MiscFeeRate                                        = 2216, /*!< 2216 (Percentage FIX.5.0SP2) The fee rate when MiscFeeAmt(137) is a percentage of trade quantity.*/
MiscFeeAmountDue                                   = 2217, /*!< 2217 (Amt FIX.5.0SP2) The fee amount due if different from MiscFeeAmt(137).*/
NoLegComplexEvents                                 = 2218, /*!< 2218 (NumInGroup FIX.5.0SP2) Number of complex events in the repeating group.*/
LegComplexEventType                                = 2219, /*!< 2219 (int FIX.5.0SP2) Identifies the type of complex event.*/
LegComplexOptPayoutPaySide                         = 2220, /*!< 2220 (int FIX.5.0SP2) Trade side of payout payer.*/
LegComplexOptPayoutReceiveSide                     = 2221, /*!< 2221 (int FIX.5.0SP2) Trade side of payout receiver.*/
LegComplexOptPayoutUnderlier                       = 2222, /*!< 2222 (String FIX.5.0SP2) Reference to the underlier whose payments are being passed through.*/
LegComplexOptPayoutAmount                          = 2223, /*!< 2223 (Amt FIX.5.0SP2) Cash amount indicating the pay out associated with an event. For binary options this is a fixed amount.*/
LegComplexOptPayoutPercentage                      = 2224, /*!< 2224 (Percentage FIX.5.0SP2) Percentage of observed price for calculating the payout associated with the event.*/
LegComplexOptPayoutTime                            = 2225, /*!< 2225 (int FIX.5.0SP2) Specifies when the payout is to occur.*/
LegComplexOptPayoutCurrency                        = 2226, /*!< 2226 (Currency FIX.5.0SP2) Specifies the currency of the payout amount. Uses ISO 4217 currency codes.*/
LegComplexEventPrice                               = 2227, /*!< 2227 (Price FIX.5.0SP2) Specifies the price at which the complex event takes effect. Impact of the event price is determined by the LegComplexEventType(2219).*/
LegComplexEventPricePercentage                     = 2228, /*!< 2228 (Percentage FIX.5.0SP2) Specifies the price percentage at which the complex event takes effect. Impact of the event price is determined by the LegComplexEventType(2219).*/
LegComplexEventPriceBoundaryMethod                 = 2229, /*!< 2229 (int FIX.5.0SP2) Specifies the boundary condition to be used for the event price relative to the complex event price at the point the complex event outcome takes effect as determined by the LegComplexEventPriceTimeType(2231).*/
LegComplexEventPriceBoundaryPrecision              = 2230, /*!< 2230 (Percentage FIX.5.0SP2) Used in combination with LegComplexEventPriceBoundaryMethod(2229) to specify the percentage of the strike price in relation to the  underlying price. The percentage is generally 100 or greater for puts and 100 or less for calls.*/
LegComplexEventPriceTimeType                       = 2231, /*!< 2231 (int FIX.5.0SP2) Specifies when the complex event outcome takes effect. The outcome of a complex event is a payout or barrier action as specified by the LegComplexEventType(2219).*/
LegComplexEventCondition                           = 2232, /*!< 2232 (int FIX.5.0SP2) Specifies the condition between complex events when more than one event is specified.

Multiple barrier events would use an "or" condition since only one can be effective at a given time. A set of digital range events would use an "and" condition since both conditions must be in effect for a payout to result.*/
LegComplexEventCurrencyOne                         = 2233, /*!< 2233 (Currency FIX.5.0SP2) Specifies the first or only reference currency of the trade. Uses ISO 4217 currency codes.*/
LegComplexEventCurrencyTwo                         = 2234, /*!< 2234 (Currency FIX.5.0SP2) Specifies the second reference currency of the trade.  Uses ISO 4217 currency codes.*/
LegComplexEventQuoteBasis                          = 2235, /*!< 2235 (int FIX.5.0SP2) For foreign exchange Quanto option feature.*/
LegComplexEventFixedFXRate                         = 2236, /*!< 2236 (float FIX.5.0SP2) Specifies the fixed FX rate alternative for FX Quantro options.*/
LegComplexEventDeterminationMethod                 = 2237, /*!< 2237 (String FIX.5.0SP2) Specifies the method according to which an amount or a date is determined.  

See http://www.fpml.org/coding-scheme/determination-method for values.*/
LegComplexEventCalculationAgent                    = 2238, /*!< 2238 (int FIX.5.0SP2) Used to identify the calculation agent.*/
LegComplexEventStrikePrice                         = 2239, /*!< 2239 (Price FIX.5.0SP2) Upper strike price for Asian option feature. Strike percentage for a Strike Spread.*/
LegComplexEventStrikeFactor                        = 2240, /*!< 2240 (float FIX.5.0SP2) Strike factor for Asian option feature. Upper strike percentage for a Strike Spread.*/
LegComplexEventStrikeNumberOfOptions               = 2241, /*!< 2241 (int FIX.5.0SP2) Upper string number of options for a Strike Spread.*/
LegComplexEventCreditEventsXIDRef                  = 2242, /*!< 2242 (XIDREF FIX.5.0SP2) Reference to credit event table elsewhere in the message.*/
LegComplexEventCreditEventNotifyingParty           = 2243, /*!< 2243 (int FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring.*/
LegComplexEventCreditEventBusinessCenter           = 2244, /*!< 2244 (String FIX.5.0SP2) Specifies the local business center for which the credit event is to be determined. The inclusion of this business center implies that Greenwich Mean Time in Section 3.3 of the 2003 ISDA Credit Derivatives Definitions is replaced by the local time of the specified business center. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegComplexEventCreditEventStandardSources          = 2245, /*!< 2245 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', indicates that ISDA defined Standard Public Sources are applicable.*/
LegComplexEventCreditEventMinimumSources           = 2246, /*!< 2246 (int FIX.5.0SP2) The minimum number of the specified public information sources that must publish information that reasonably confirms that a credit event has occurred. The market convention is two.*/
LegComplexEventEndTime                             = 2247, /*!< 2247 (UTCTimeOnly FIX.5.0SP2) The end time of the time range on which a complex event date is effective.

The end time must always be greater than or equal to the start time.*/
LegComplexEventXID                                 = 2248, /*!< 2248 (XID FIX.5.0SP2) Identifier of this complex event for cross referencing elsewhere in the message.*/
LegComplexEventXIDRef                              = 2249, /*!< 2249 (XIDREF FIX.5.0SP2) Reference to a complex event elsewhere in the message.*/
NoLegComplexEventDates                             = 2250, /*!< 2250 (NumInGroup FIX.5.0SP2) Number of complex event dates in the repeating group.*/
LegComplexEventStartDate                           = 2251, /*!< 2251 (UTCDateOnly FIX.5.0SP2) The start date of the date range on which a complex event is effective. The start date will be set equal to the end date for single day events such as Bermuda options.

The start date must always be less than or equal to end date.*/
LegComplexEventEndDate                             = 2252, /*!< 2252 (UTCDateOnly FIX.5.0SP2) The end date of the date range on which a complex event is effective. The start date will be set equal to the end date for single day events such as Bermuda options.

The end date must always be greater than or equal to start date.*/
NoLegComplexEventTimes                             = 2253, /*!< 2253 (NumInGroup FIX.5.0SP2) Number of complex event times in the repeating group.*/
NoLegInstrumentParties                             = 2254, /*!< 2254 (NumInGroup FIX.5.0SP2) Number of parties in the repeating group.*/
LegInstrumentPartyID                               = 2255, /*!< 2255 (String FIX.5.0SP2) Used to identify party id related to instrument.*/
LegInstrumentPartyIDSource                         = 2256, /*!< 2256 (char FIX.5.0SP2) Used to identify source of instrument party id.*/
LegInstrumentPartyRole                             = 2257, /*!< 2257 (int FIX.5.0SP2) Used to identify the role of instrument party id.*/
NoLegInstrumentPartySubIDs                         = 2258, /*!< 2258 (NumInGroup FIX.5.0SP2) Number of parties sub-IDs in the repeating group.*/
LegInstrumentPartySubID                            = 2259, /*!< 2259 (String FIX.5.0SP2) PartySubID value within an instrument party repeating group.*/
LegInstrumentPartySubIDType                        = 2260, /*!< 2260 (int FIX.5.0SP2) Type of LegInstrumentPartySubID (2259) value.*/
UnderlyingComplexOptPayoutPaySide                  = 2261, /*!< 2261 (int FIX.5.0SP2) Trade side of payout payer.*/
UnderlyingComplexOptPayoutReceiveSide              = 2262, /*!< 2262 (int FIX.5.0SP2) Trade side of payout receiver.*/
UnderlyingComplexOptPayoutUnderlier                = 2263, /*!< 2263 (String FIX.5.0SP2) Reference to the underlier whose payments are being passed through.*/
UnderlyingComplexOptPayoutPercentage               = 2264, /*!< 2264 (Percentage FIX.5.0SP2) Percentage of observed price for calculating the payout associated with the event.*/
UnderlyingComplexOptPayoutTime                     = 2265, /*!< 2265 (int FIX.5.0SP2) The time when the payout is to occur.*/
UnderlyingComplexOptPayoutCurrency                 = 2266, /*!< 2266 (Currency FIX.5.0SP2) Specifies the currency of the payout amount. Uses ISO 4217 currency codes.*/
UnderlyingComplexEventPricePercentage              = 2267, /*!< 2267 (Percentage FIX.5.0SP2) Specifies the price percentage at which the complex event takes effect. Impact of the event price is determined by the UnderlyingComplexEventType(2046).*/
UnderlyingComplexEventCurrencyOne                  = 2268, /*!< 2268 (Currency FIX.5.0SP2) Specifies the first or only reference currency of the trade. Uses ISO 4217 currency codes.*/
UnderlyingComplexEventCurrencyTwo                  = 2269, /*!< 2269 (Currency FIX.5.0SP2) Specifies the second reference currency of the trade. Uses ISO 4217 currency codes.*/
UnderlyingComplexEventQuoteBasis                   = 2270, /*!< 2270 (int FIX.5.0SP2) Specifies the currency pairing for the quote.  */
UnderlyingComplexEventFixedFXRate                  = 2271, /*!< 2271 (float FIX.5.0SP2) Specifies the fixed FX rate alternative for FX Quantro options.*/
UnderlyingComplexEventDeterminationMethod          = 2272, /*!< 2272 (String FIX.5.0SP2) Specifies the method according to which an amount or a date is determined.  

See http://www.fpml.org/coding-scheme/determination-method for values.*/
UnderlyingComplexEventCalculationAgent             = 2273, /*!< 2273 (int FIX.5.0SP2) Used to identify the calculation agent.*/
UnderlyingComplexEventStrikePrice                  = 2274, /*!< 2274 (Price FIX.5.0SP2) Upper strike price for Asian option feature. Strike percentage for a Strike Spread.*/
UnderlyingComplexEventStrikeFactor                 = 2275, /*!< 2275 (float FIX.5.0SP2) Strike factor for Asian option feature. Upper strike percentage for a Strike Spread.*/
UnderlyingComplexEventStrikeNumberOfOptions        = 2276, /*!< 2276 (int FIX.5.0SP2) Upper string number of options for a Strike Spread.*/
UnderlyingComplexEventCreditEventsXIDRef           = 2277, /*!< 2277 (XIDREF FIX.5.0SP2) Reference to credit event table elsewhere in the message.*/
UnderlyingComplexEventCreditEventNotifyingParty    = 2278, /*!< 2278 (int FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring.*/
UnderlyingComplexEventCreditEventBusinessCenter    = 2279, /*!< 2279 (String FIX.5.0SP2) Specifies the local business center for which the credit event is to be determined. The inclusion of this business center implies that Greenwich Mean Time in Section 3.3 of the 2003 ISDA Credit Derivatives Definitions is replaced by the local time of the specified business center. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingComplexEventCreditEventStandardSources   = 2280, /*!< 2280 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', indicates that ISDA defined Standard Public Sources are applicable.*/
UnderlyingComplexEventCreditEventMinimumSources    = 2281, /*!< 2281 (int FIX.5.0SP2) The minimum number of the specified public information sources that must publish information that reasonably confirms that a credit event has occurred. The market convention is two. */
UnderlyingComplexEventXID                          = 2282, /*!< 2282 (XID FIX.5.0SP2) Identifier of this complex event for cross referencing elsewhere in the message.*/
UnderlyingComplexEventXIDRef                       = 2283, /*!< 2283 (XIDREF FIX.5.0SP2) Reference to a complex event elsewhere in the message.*/
UnderlyingSettlRateIndex                           = 2284, /*!< 2284 (String FIX.5.0SP2) In an outright or forward commodity trade that is cash settled this is the index used to determine the cash payment.*/
UnderlyingSettlRateIndexLocation                   = 2285, /*!< 2285 (String FIX.5.0SP2) This is an optional qualifying attribute of UnderlyingSettlementRateIndex(2284) such as the delivery zone for an electricity contract.*/
UnderlyingOptionExpirationDesc                     = 2286, /*!< 2286 (String FIX.5.0SP2) Description of the option expiration.*/
EncodedUnderlyingOptionExpirationDescLen           = 2287, /*!< 2287 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingOptionExpirationDesc(2288) field.*/
EncodedUnderlyingOptionExpirationDesc              = 2288, /*!< 2288 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingOptionExpirationDesc(2286) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingOptionExpirationDesc(2286).*/
UnderlyingSwapSubClass                             = 2289, /*!< 2289 (String FIX.5.0SP2) The sub-classification or notional schedule type of the swap.*/
UnderlyingStrikeUnitOfMeasure                      = 2290, /*!< 2290 (String FIX.5.0SP2) Used to express the unit of measure (UOM) of the price if different from the contract.*/
UnderlyingStrikeIndex                              = 2291, /*!< 2291 (String FIX.5.0SP2) Specifies the index used to calculate the strike price.*/
UnderlyingStrikeIndexSpread                        = 2292, /*!< 2292 (PriceOffset FIX.5.0SP2) Specifies the strike price offset from the named index.*/
UnderlyingValuationSource                          = 2293, /*!< 2293 (String FIX.5.0SP2) Specifies the source of trade valuation data.*/
UnderlyingValuationReferenceModel                  = 2294, /*!< 2294 (String FIX.5.0SP2) Specifies the methodology and/or assumptions used to generate the  trade value.*/
UnderlyingStrategyType                             = 2295, /*!< 2295 (String FIX.5.0SP2) Specifies the type of trade strategy.*/
UnderlyingCommonPricingIndicator                   = 2296, /*!< 2296 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', it indicates that common pricing applies. Common pricing may be relevant for a transaction that references more than one commodity reference price.*/
UnderlyingSettlDisruptionProvision                 = 2297, /*!< 2297 (int FIX.5.0SP2) Specifies the consequences of settlement disruption events.*/
UnderlyingInstrumentRoundingDirection              = 2298, /*!< 2298 (char FIX.5.0SP2) Specifies the rounding direction if not overridden elsewhere.*/
UnderlyingInstrumentRoundingPrecision              = 2299, /*!< 2299 (int FIX.5.0SP2) Specifies the rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
AllocGrossTradeAmt                                 = 2300, /*!< 2300 (Amt FIX.5.0SP2) Total amount traded for this account (i.e. quantity * price) expressed in units of currency. */
LastQtyChanged                                     = 2301, /*!< 2301 (Qty FIX.5.0SP2) The positive or negative change in quantity when this report is a trade correction or continuation. */
TradeVersion                                       = 2302, /*!< 2302 (String FIX.5.0SP2) Specifies the version of a trade or contract.  This is used by systems or trading platforms in conjunction with TradeID(1003) to uniquely identify the version of a trade or contract. If used the conditions for a change of version are subject to bilateral agreement. It is recommended to change the version only for significant updates to the business entity rather than for minor changes to trade details or systematic distribution of reports. Examples where the version would change are trade quantity modification, customer account assignment or trade novation.*/
HistoricalReportIndicator                          = 2303, /*!< 2303 (Boolean FIX.5.0SP2) Indicates that the trade or event being reported occurred in the past and the trade is terminated or no longer active.*/
NoAssetAttributes                                  = 2304, /*!< 2304 (NumInGroup FIX.5.0SP2) The number of asset attribute entries in the group.*/
AssetAttributeType                                 = 2305, /*!< 2305 (String FIX.5.0SP2) Specifies the name of the attribute.  

See http://www.fixtradingcommunity.org/codelists#Asset_Attribute_Types for code list of applicable asset attribute types.*/
AssetAttributeValue                                = 2306, /*!< 2306 (String FIX.5.0SP2) Specifies the value of the asset attribute.*/
AssetAttributeLimit                                = 2307, /*!< 2307 (String FIX.5.0SP2) Limit or lower acceptable value of the attribute.*/
NoLegAssetAttributes                               = 2308, /*!< 2308 (NumInGroup FIX.5.0SP2) Number of asset attribute entries in the group.*/
LegAssetAttributeType                              = 2309, /*!< 2309 (String FIX.5.0SP2) Specifies the name of the attribute.  

See http://www.fixtradingcommunity.org/codelists#Asset_Attribute_Types for code list of applicable asset attribute types.*/
LegAssetAttributeValue                             = 2310, /*!< 2310 (String FIX.5.0SP2) Specifies the value of the attribute.*/
LegAssetAttributeLimit                             = 2311, /*!< 2311 (String FIX.5.0SP2) Limit or lower acceptable value of the attribute.*/
NoUnderlyingAssetAttributes                        = 2312, /*!< 2312 (NumInGroup FIX.5.0SP2) Number of asset attribute entries in the group.*/
UnderlyingAssetAttributeType                       = 2313, /*!< 2313 (String FIX.5.0SP2) Specifies the name of the attribute.  

See http://www.fixtradingcommunity.org/codelists#Asset_Attribute_Types for code list of applicable asset attribute types.*/
UnderlyingAssetAttributeValue                      = 2314, /*!< 2314 (String FIX.5.0SP2) Specifies the value of the attribute.*/
UnderlyingAssetAttributeLimit                      = 2315, /*!< 2315 (String FIX.5.0SP2) Limit or lower acceptable value of the attribute.*/
RiskLimitReportStatus                              = 2316, /*!< 2316 (int FIX.5.0SP2) Status of risk limit report.*/
RiskLimitReportRejectReason                        = 2317, /*!< 2317 (int FIX.5.0SP2) The reason for rejecting the PartyRiskLimitsReport(35=CM) or PartyRiskLimitsUpdateReport(35=CR).*/
RiskLimitCheckRequestID                            = 2318, /*!< 2318 (String FIX.5.0SP2) The unique identifier of the PartyRiskLimitCheckRequest(35=DF) message.*/
RiskLimitCheckID                                   = 2319, /*!< 2319 (String FIX.5.0SP2) The unique and static identifier, at the business entity level, of a risk limit check request.*/
RiskLimitCheckTransType                            = 2320, /*!< 2320 (int FIX.5.0SP2) Specifies the transaction type of the risk limit check request.*/
RiskLimitCheckType                                 = 2321, /*!< 2321 (int FIX.5.0SP2) Specifies the type of limit check message.*/
RiskLimitCheckRequestRefID                         = 2322, /*!< 2322 (int FIX.5.0SP2) Specifies the message reference identifier of the risk limit check request message.*/
RiskLimitCheckRequestType                          = 2323, /*!< 2323 (int FIX.5.0SP2) Specifies the type of limit amount check being requested.*/
RiskLimitCheckAmount                               = 2324, /*!< 2324 (Amt FIX.5.0SP2) Specifies the amount being requested for approval.*/
RiskLimitCheckRequestStatus                        = 2325, /*!< 2325 (int FIX.5.0SP2) Indicates the status of the risk limit check request.*/
RiskLimitCheckRequestResult                        = 2326, /*!< 2326 (int FIX.5.0SP2) Result of the credit limit check request.*/
RiskLimitApprovedAmount                            = 2327, /*!< 2327 (Amt FIX.5.0SP2) The credit/risk limit amount approved.*/
PartyActionRequestID                               = 2328, /*!< 2328 (String FIX.5.0SP2) The unique identifier of the PartyActionRequest(35=DH) message.*/
PartyActionType                                    = 2329, /*!< 2329 (int FIX.5.0SP2) Specifies the type of action to take or was taken for a given party.*/
ApplTestMessageIndicator                           = 2330, /*!< 2330 (Boolean FIX.5.0SP2) Used to indicate whether the message being sent is to test the receiving application's availability to process the message. When set to "Y" the message is a test message. If not specified, the message is by default not a test message.*/
PartyActionReportID                                = 2331, /*!< 2331 (String FIX.5.0SP2) The unique identifier of the PartyActionReport(35=DI) message as assigned by the message sender.*/
PartyActionResponse                                = 2332, /*!< 2332 (int FIX.5.0SP2) Specifies the action taken as a result of the PartyActionType(2239) of the PartyActionRequest(35=DH) message.*/
PartyActionRejectReason                            = 2333, /*!< 2333 (int FIX.5.0SP2) Specifies the reason the PartyActionRequest(35=DH) was rejected.*/
RefRiskLimitCheckID                                = 2334, /*!< 2334 (String FIX.5.0SP2) The reference identifier of the PartyRiskLimitCheckRequest(35=DF) message, or a similar out of band message, that contained the approval for the risk/credit limit check request.*/
RefRiskLimitCheckIDType                            = 2335, /*!< 2335 (int FIX.5.0SP2) Specifies which type of identifier is specified in RefRiskLimitCheckID(2334) field.*/
RiskLimitVelocityPeriod                            = 2336, /*!< 2336 (int FIX.5.0SP2) The time interval for which the clip size limit applies.  The velocity time unit is expressed in RiskLimitVelocityUnit(2337).*/
RiskLimitVelocityUnit                              = 2337, /*!< 2337 (String FIX.5.0SP2) Unit of time in which RiskLimitVelocityPeriod(2336) is expressed.*/
RequestingPartyRoleQualifier                       = 2338, /*!< 2338 (int FIX.5.0SP2) Qualifies the value of RequestingPartyRole(1660).*/
RiskLimitCheckModelType                            = 2339, /*!< 2339 (int FIX.5.0SP2) Specifies the type of credit limit check model workflow to apply for the specified party*/
EventMonthYear                                     = 2340, /*!< 2340 (MonthYear FIX.5.0SP2) Used with derivatives when an event is express as a month-year with optional day or month or week of month.

Format:

YYYYMM (e.g. 199903)

YYYYMMDD (e.g. 20030323)

YYYYMMwN (e.g. 200303w2) for week

A specific date can be appended to the month-year. For instance, if multiple event types exist in the same Year and Month, but actually at a different time, a value can be appended, such as "w" or "w2" to indicate week. Likewise, the day of monty (0-31) can be appended to indicate a specific event date.*/
LegEventMonthYear                                  = 2341, /*!< 2341 (MonthYear FIX.5.0SP2) Used with derivatives when an event is express as a month-year with optional day or month or week of month.

Format:

YYYYMM (e.g. 199903)

YYYYMMDD (e.g. 20030323)

YYYYMMwN (e.g. 200303w2) for week

A specific date can be appended to the month-year. For instance, if multiple event types exist in the same Year and Month, but actually at a different time, a value can be appended, such as "w" or "w2" to indicate week. Likewise, the day of monty (0-31) can be appended to indicate a specific event date. */
UnderlyingEventMonthYear                           = 2342, /*!< 2342 (MonthYear FIX.5.0SP2) Used with derivatives when an event is express as a month-year with optional day or month or week of month.

Format:

YYYYMM (e.g. 199903)

YYYYMMDD (e.g. 20030323)

YYYYMMwN (e.g. 200303w2) for week

A specific date can be appended to the month-year. For instance, if multiple event types exist in the same Year and Month, but actually at a different time, a value can be appended, such as "w" or "w2" to indicate week. Likewise, the day of monty (0-31) can be appended to indicate a specific event date.*/
RiskLimitCheckStatus                               = 2343, /*!< 2343 (int FIX.5.0SP2) Indicates the status of the risk limit check performed on a trade.*/
SideRiskLimitCheckStatus                           = 2344, /*!< 2344 (int FIX.5.0SP2) Indicates the status of the risk limit check performed on the side of a trade.*/
NoEntitlementTypes                                 = 2345, /*!< 2345 (NumInGroup FIX.5.0SP2) Number of entitlement types in the repeating group.*/
LegMidPx                                           = 2346, /*!< 2346 (Price FIX.5.0SP2) Leg Mid price/rate.

For OTC swaps, this is the mid-market mark (for example, as defined by CFTC).

For uncleared OTC swaps, LegMidPx(2346) and the MidPx(631) fields are mutually exclusive.*/
RegulatoryTransactionType                          = 2347, /*!< 2347 (int FIX.5.0SP2) Specifies the regulatory mandate or rule that the transaction complies with.*/
LegAssetGroup                                      = 2348, /*!< 2348 (int FIX.5.0SP2) Indicates the broad product or asset classification.  May be used to provide grouping for the product taxonomy (Product(460), SecurityType(167), etc.) and/or the risk taxonomy (AssetClass(1938), AssetSubClass(1939), AssetType(1940), etc.).*/
PricePrecision                                     = 2349, /*!< 2349 (int FIX.5.0SP2) Specifies the price decimal precision of the instrument.*/
CollateralPortfolioID                              = 2350, /*!< 2350 (String FIX.5.0SP2) Identifier of the collateral portfolio when reporting on a portfolio basis.*/
EncodedComplianceTextLen                           = 2351, /*!< 2351 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedComplianceText(2352) field.*/
EncodedComplianceText                              = 2352, /*!< 2352 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the ComplianceText(2404) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the ComplianceText(2404) field.*/
TradingUnitPeriodMultiplier                        = 2353, /*!< 2353 (int FIX.5.0SP2) Indicates the number of contract periods associated with the minimum trading unit for a given contract duration resulting in the number of total traded contracts.*/
LegTradingUnitPeriodMultiplier                     = 2354, /*!< 2354 (int FIX.5.0SP2) Indicates the number of contract periods associated with the minimum trading unit for a given contract duration resulting in the number of total traded contracts.*/
PartyRiskLimitStatus                               = 2355, /*!< 2355 (int FIX.5.0SP2) The status of risk limits for a party.*/
RemunerationIndicator                              = 2356, /*!< 2356 (int FIX.5.0SP2) Indicates whether the trade price was adjusted for compensation (i.e. includes a mark-up, mark-down or commission) in the price paid.*/
LegTotalTradeQty                                   = 2357, /*!< 2357 (Qty FIX.5.0SP2) Expresses the total quantity traded over the life of the contract when LegLastQty(1418) is to be repeated periodically over the term of the contract. The value is the product of LegLastQty(1418) and LegTradingUnitPeriodMultiplier(2353).*/
LegLastMultipliedQty                               = 2358, /*!< 2358 (Qty FIX.5.0SP2) Expresses the quantity bought/sold when LastQty is expressed in contracts. Used in addition to LegLastQty(1418), it is the product of LegLastQty(1418) and LegContractMultiplier(614).*/
LegTotalGrossTradeAmt                              = 2359, /*!< 2359 (Amt FIX.5.0SP2) Expresses the full total monetary value of the traded contract. The value is the product of LegLastPx(637) and LegTotalTradeQty(2357) or LegTotalTradeMultipliedQty(2360), if priced in units instead of contracts.*/
LegTotalTradeMultipliedQty                         = 2360, /*!< 2360 (Qty FIX.5.0SP2) Expresses the total trade quantity in units where LegContractMultiplier(614) is not 1. The value is the product of LegTotalTradeQty(2357) and LegContractMultiplier(614).*/
CompressionGroupID                                 = 2361, /*!< 2361 (String FIX.5.0SP2) Use to identify a netting or compression group where trades in the group were netted or compressed.  This includes both terminating trades and any remnant trades that result from the operation.*/
SelfMatchPreventionID                              = 2362, /*!< 2362 (String FIX.5.0SP2) Identifies an order or trade that should not be matched to an opposite order or trade if both buy and sell orders for the same asset contain the same SelfMatchPreventionID(2362) and submitted by the same firm.*/
UnderlyingTradingUnitPeriodMultiplier              = 2363, /*!< 2363 (int FIX.5.0SP2) Indicates the number of contract periods associated with the minimum trading unit for a given contract duration resulting in the number of total traded contracts.*/
PosReportAction                                    = 2364, /*!< 2364 (int FIX.5.0SP2) Indicates action that triggered the Position Report.*/
SettlForwardPoints                                 = 2365, /*!< 2365 (PriceOffset FIX.5.0SP2) FX forward points added to SettlPrice(730). The value is expressed in decimal form and may be a negative.*/
SettlPriceFxRateCalc                               = 2366, /*!< 2366 (char FIX.5.0SP2) Specifies whether LastPx(31) [TradeCaptureReport] or SettlPrice(730) [PositionReport] should be multiplied or divided.*/
TotalTradeQty                                      = 2367, /*!< 2367 (Qty FIX.5.0SP2) Expresses the total quantity traded over the life of the contract when LastQty(32) is repeated periodically over the term of the contract. The value is the product of LastQty(32) and TradingUnitPeriodMultiplier(2353).*/
LastMultipliedQty                                  = 2368, /*!< 2368 (Qty FIX.5.0SP2) Expresses the quantity bought or sold when LastQty(32) is expressed in number of contracts. Used in addition to LastQty(32). It is the product of LastQty(32) and ContractMultiplier(231).*/
TotalGrossTradeAmt                                 = 2369, /*!< 2369 (Amt FIX.5.0SP2) Expresses the full total monetary value of the traded contract. The value is the product of LastPx(31) and TotalTradeQty(2367) or TotalTradeMultipliedQty(2370), if priced in units instead of contracts.*/
TotalTradeMultipliedQty                            = 2370, /*!< 2370 (Qty FIX.5.0SP2) Expresses the total trade quantity in units where ContractMultiplier(231) is not 1. The value is the product of TotalTradeQty(2367) and ContractMultiplier(231).*/
EncodedTradeContinuationText                       = 2371, /*!< 2371 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the TradeContinuationText(2374) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) representation should also be specified in the TradeContinuationText(2374) field.*/
EncodedTradeContinuationTextLen                    = 2372, /*!< 2372 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedTradeContinuationText(2371) field.*/
IntraFirmTradeIndicator                            = 2373, /*!< 2373 (Boolean FIX.5.0SP2) Indicates whether the trade or position was entered into as an intra-group transaction, i.e. between two units of the same parent entity having majority ownership interest in both counterparties.*/
TradeContinuationText                              = 2374, /*!< 2374 (String FIX.5.0SP2) Elaboration of the purpose or action of the regulatory report when TradeContinuation(1937)=99 (Other).*/
TaxonomyType                                       = 2375, /*!< 2375 (char FIX.5.0SP2) The type of identification taxonomy used to identify the security.*/
PartyRoleQualifier                                 = 2376, /*!< 2376 (int FIX.5.0SP2) Used to further qualify the value of PartyRole(452).*/
DerivativeInstrumentPartyRoleQualifier             = 2377, /*!< 2377 (int FIX.5.0SP2) Used to further qualify the value of DerivativeInstrumentPartyRole(1295).*/
InstrumentPartyRoleQualifier                       = 2378, /*!< 2378 (int FIX.5.0SP2) Used to further qualify the value of InstrumentPartyRole(1051).*/
LegInstrumentPartyRoleQualifier                    = 2379, /*!< 2379 (int FIX.5.0SP2) Used to further qualify the value of LegInstrumentPartyRole(2257).*/
LegProvisionPartyRoleQualifier                     = 2380, /*!< 2380 (int FIX.5.0SP2) Used to further qualify the value of LegProvisionPartyRole(40536).*/
Nested2PartyRoleQualifier                          = 2381, /*!< 2381 (int FIX.5.0SP2) Used to further qualify the value of Nested2PartyRole(759).*/
Nested3PartyRoleQualifier                          = 2382, /*!< 2382 (int FIX.5.0SP2) Used to further qualify the value of Nested3PartyRole(951).*/
Nested4PartyRoleQualifier                          = 2383, /*!< 2383 (int FIX.5.0SP2) Used to further qualify the value of Nested4PartyRole(1417).*/
NestedPartyRoleQualifier                           = 2384, /*!< 2384 (int FIX.5.0SP2) Used to further qualify the value of NestedPartyRole(538).*/
ProvisionPartyRoleQualifier                        = 2385, /*!< 2385 (int FIX.5.0SP2) Used to further qualify the value of ProvisionPartyRole(40177).*/
RequestedPartyRoleQualifier                        = 2386, /*!< 2386 (int FIX.5.0SP2) Used to further qualify the value of RequestedPartyRole(1509).*/
TradeContingency                                   = 2387, /*!< 2387 (int FIX.5.0SP2) Indicates the contingency attribute for a trade in an asset class that may be contingent on the clearing of a corresponding paired trade (for example Exchange for Physical (EFP), Exchange for Swap (EFS), Exchange for Related (EFR) or Exchange for Option (EFO), collectively called EFRPs). Once the paired trade clears or fails to clear, the related trade (the trade which carries this attribute) ceases to exist.*/
RootPartyRoleQualifier                             = 2388, /*!< 2388 (int FIX.5.0SP2) Used to further qualify the value of RootPartyRole(1119).*/
SettlPartyRoleQualifier                            = 2389, /*!< 2389 (int FIX.5.0SP2) Used to further qualify the value of SettlPartyRole(784).*/
TradeConfirmationReferenceID                       = 2390, /*!< 2390 (String FIX.5.0SP2) A reference or control identifier or number used as a trade confirmation key.*/
UnderlyingInstrumentPartyRoleQualifier             = 2391, /*!< 2391 (int FIX.5.0SP2) Used to further qualify the value of UnderlyingInstrumentPartyRole(1061).*/
AllocRefRiskLimitCheckID                           = 2392, /*!< 2392 (String FIX.5.0SP2) The reference identifier to the PartyRiskLimitCheckRequest(35=DF), or a similar out of band message, message that contained the approval or rejection for risk/credit limit check for this allocation.*/
AllocRefRiskLimitCheckIDType                       = 2393, /*!< 2393 (int FIX.5.0SP2) Specifies which type of identifier is specified in AllocRefRiskLimitCheckID(2392) field.*/
LimitUtilizationAmt                                = 2394, /*!< 2394 (Amt FIX.5.0SP2) The total amount of the limit that has been drawn down against the counterparty. This includes the amount for prior trades. It may or may not include the amount for the given trade, specified in LastLimitAmt(1632), depending upon whether the given trade is considered pending. */
LimitAmt                                           = 2395, /*!< 2395 (Amt FIX.5.0SP2) The limit for the counterparty. This represents the total limit amount, independent of any amount already utilized.*/
LimitRole                                          = 2396, /*!< 2396 (int FIX.5.0SP2) Indicates the scope of the limit by role.*/
RegulatoryTradeIDScope                             = 2397, /*!< 2397 (int FIX.5.0SP2) Specifies the scope to which the RegulatoryTradeID(1903) applies.  Used when a trade must be assigned more than one identifier, e.g. one for the clearing member and another for the client on a cleared trade as with the principal model in Europe.*/
SideRegulatoryTradeIDScope                         = 2398, /*!< 2398 (int FIX.5.0SP2) Specifies the scope to which the SideRegulatoryTradeID(1972) applies.  Used when a trade must be assigned more than one identifier, e.g. one for the clearing member and another for the client on a cleared trade as with the principal model in Europe.*/
AllocRegulatoryTradeIDScope                        = 2399, /*!< 2399 (int FIX.5.0SP2) Specifies the scope to which the AllocRegulatoryTradeID(1909) applies.  Used when a trade must be assigned more than one identifier, e.g. one for the clearing member and another for the client on a cleared trade as with the principal model in Europe.*/
EffectiveBusinessDate                              = 2400, /*!< 2400 (LocalMktDate FIX.5.0SP2) Specifies an explicit business date for associated reference data or transaction. Used when an implicit date is not sufficiently specific.*/
ListManualOrderIndicator                           = 2401, /*!< 2401 (Boolean FIX.5.0SP2) Indicates if the list of orders was initially received manually (as opposed to electronically) or if it was entered manually (as opposed to entered by automated trading software).*/
EntitlementSubType                                 = 2402, /*!< 2402 (int FIX.5.0SP2) Subtype of an entitlement specified in EntitlementType(1775).*/
QuoteModelType                                     = 2403, /*!< 2403 (int FIX.5.0SP2) Quote model type*/
ComplianceText                                     = 2404, /*!< 2404 (String FIX.5.0SP2) Free text for compliance information required for regulatory reporting.*/
ExecMethod                                         = 2405, /*!< 2405 (int FIX.5.0SP2) Specifies how the transaction was executed, e.g. via an automated execution platform or other method.*/
AllocRegulatoryLegRefID                            = 2406, /*!< 2406 (String FIX.5.0SP2) Identifies the leg of the trade the entry applies to by referencing the leg's LegID(1788).*/
ComplexEventSpotRate                               = 2407, /*!< 2407 (Price FIX.5.0SP2) FX spot rate.*/
ComplexEventForwardPoints                          = 2408, /*!< 2408 (PriceOffset FIX.5.0SP2) FX forward points added to spot rate. May be a negative value.*/
LegComplexEventSpotRate                            = 2409, /*!< 2409 (Price FIX.5.0SP2) FX spot rate.*/
LegComplexEventForwardPoints                       = 2410, /*!< 2410 (PriceOffset FIX.5.0SP2) FX forward points added to spot rate. May be a negative value.*/
RegulatoryLegRefID                                 = 2411, /*!< 2411 (String FIX.5.0SP2) Identifies the leg of the trade the entry applies to by referencing the leg's LegID(1788).*/
RateSourceReferemcePageHeading                     = 2412, /*!< 2412 (String FIX.5.0SP2) Identifies the page heading from the rate source.*/
RelatedToSecurityID                                = 2413, /*!< 2413 (String FIX.5.0SP2) The security identifier of the instrument, instrument leg or underlying instrument with which the related instrument has correlation. */
RelatedToSecurityIDSource                          = 2414, /*!< 2414 (String FIX.5.0SP2) Identifies class or source of the RelatedToSecurityID(2413) value. */
RelatedToStreamXIDRef                              = 2415, /*!< 2415 (XIDREF FIX.5.0SP2) StreamXID(41303), LegStreamXID(41700) or UnderlyingStreamXID(42016) of the stream with which the related instrument has correlation.*/
SideRegulatoryLegRefID                             = 2416, /*!< 2416 (String FIX.5.0SP2) Identifies the leg of the trade the entry applies to by referencing the leg's LegID(1788).*/
RelatedToDividendPeriodXIDRef                      = 2417, /*!< 2417 (XIDREF FIX.5.0SP2) The DividendPeriodXID(42293) of the stream dividend period with which the related instrument has correlation.*/
FirmTradeEventID                                   = 2418, /*!< 2418 (String FIX.5.0SP2) An identifier created by the trading party for the life cycle event associated with this report.*/
UnderlyingComplexEventSpotRate                     = 2419, /*!< 2419 (Price FIX.5.0SP2) FX spot rate.*/
UnderlyingComplexEventForwardPoints                = 2420, /*!< 2420 (PriceOffset FIX.5.0SP2) FX forward points added to spot rate. May be a negative value.*/
FillRefID                                          = 2421, /*!< 2421 (String FIX.5.0SP2) A reference to either the value of the FillExecID(1363) or an implicit position of a fills instance in the FillsGrp component.*/
OrderRequestID                                     = 2422, /*!< 2422 (int FIX.5.0SP2) Unique message identifier for an order request as assigned by the submitter of the request.*/
MassOrderRequestID                                 = 2423, /*!< 2423 (String FIX.5.0SP2) Unique message identifier for a mass order request as assigned by the submitter of the orders.*/
MassOrderReportID                                  = 2424, /*!< 2424 (String FIX.5.0SP2) Unique message identifier for a mass order request as assigned by the receiver of the orders.*/
MassOrderRequestStatus                             = 2425, /*!< 2425 (int FIX.5.0SP2) Status of mass order request.*/
MassOrderRequestResult                             = 2426, /*!< 2426 (int FIX.5.0SP2) Request result of mass order request.*/
OrderResponseLevel                                 = 2427, /*!< 2427 (int FIX.5.0SP2) The level of response requested from receiver of mass order messages. A default value should be bilaterally agreed.*/
NoOrderEntries                                     = 2428, /*!< 2428 (NumInGroup FIX.5.0SP2) Number of order entries.*/
OrderEntryAction                                   = 2429, /*!< 2429 (char FIX.5.0SP2) Specifies the action to be taken for the given order.*/
OrderEntryID                                       = 2430, /*!< 2430 (int FIX.5.0SP2) Unique identifier for an order within a single MassOrder(35=DJ) message that can be used as a reference in the MassOrderAck(35=DK) message.*/
ExecTypeReason                                     = 2431, /*!< 2431 (int FIX.5.0SP2) The initiating event when an ExecutionReport(35=8) is sent.*/
TotNoOrderEntries                                  = 2432, /*!< 2432 (int FIX.5.0SP2) Totals number of orders for a mass order or its acknowledgment being fragmented across multiple messages.*/
NoTargetPartySubIDs                                = 2433, /*!< 2433 (NumInGroup FIX.5.0SP2) Number of target party sub IDs in the repeating group.*/
TargetPartySubID                                   = 2434, /*!< 2434 (String FIX.5.0SP2) Party sub-identifier value within a target party repeating group.*/
TargetPartySubIDType                               = 2435, /*!< 2435 (int FIX.5.0SP2) Type of TargetPartySubID(2434) value.*/
TransferInstructionID                              = 2436, /*!< 2436 (String FIX.5.0SP2) Unique identifier for the transfer instruction assigned by the submitter. */
TransferID                                         = 2437, /*!< 2437 (String FIX.5.0SP2) The unique identifier assigned to the transfer entity once it is received, for example, by the CCP or the party governing the transfer process.  Generally this same identifier for the transfer is used by all parties involved.*/
TransferReportID                                   = 2438, /*!< 2438 (String FIX.5.0SP2) Unique identifier for the transfer report message.*/
TransferTransType                                  = 2439, /*!< 2439 (int FIX.5.0SP2) Indicates the type of transfer transaction.*/
TransferType                                       = 2440, /*!< 2440 (int FIX.5.0SP2) Indicates the type of transfer request.*/
TransferScope                                      = 2441, /*!< 2441 (int FIX.5.0SP2) Indicates the type of transfer.*/
TransferStatus                                     = 2442, /*!< 2442 (int FIX.5.0SP2) Status of the transfer.*/
TransferRejectReason                               = 2443, /*!< 2443 (int FIX.5.0SP2) Reason the transfer instruction was rejected.*/
TransferReportType                                 = 2444, /*!< 2444 (int FIX.5.0SP2) Indicates the type of transfer report.*/
AggressorTime                                      = 2445, /*!< 2445 (UTCTimestamp FIX.5.0SP2) Timestamp of aggressive order or quote resulting in match event.*/
AggressorSide                                      = 2446, /*!< 2446 (char FIX.5.0SP2) Side of aggressive order or quote resulting in match event.*/
FastMarketIndicator                                = 2447, /*!< 2447 (Boolean FIX.5.0SP2) Indicates if the instrument is in "fast market" state.*/
LinkageHandlingIndicator                           = 2448, /*!< 2448 (Boolean FIX.5.0SP2) Indicate whether linkage handling is in effect for an instrument or not.*/
NumberOfBuyOrders                                  = 2449, /*!< 2449 (int FIX.5.0SP2) Number of buy orders involved in a trade.*/
NumberOfSellOrders                                 = 2450, /*!< 2450 (int FIX.5.0SP2) Number of sell orders involved in a trade.*/
SettlPriceDeterminationMethod                      = 2451, /*!< 2451 (int FIX.5.0SP2) Calculation method used to determine settlement price.*/
MDStatisticReqID                                   = 2452, /*!< 2452 (String FIX.5.0SP2) Message identifier for a statistics request.*/
MDStatisticRptID                                   = 2453, /*!< 2453 (String FIX.5.0SP2) Message identifier for a statistics report.*/
MDStatisticName                                    = 2454, /*!< 2454 (String FIX.5.0SP2) The short name or acronym for a set of statistic parameters.*/
MDStatisticDesc                                    = 2455, /*!< 2455 (String FIX.5.0SP2) Can be used to provide an optional textual description for a statistic.*/
MDStatisticType                                    = 2456, /*!< 2456 (int FIX.5.0SP2) Type of statistic value.*/
MDStatisticScope                                   = 2457, /*!< 2457 (int FIX.5.0SP2) Entities used as basis for the statistics.*/
MDStatisticSubScope                                = 2458, /*!< 2458 (int FIX.5.0SP2) Sub-scope of the statistics to further reduce the entities used as basis for the statistics.*/
MDStatisticScopeType                               = 2459, /*!< 2459 (int FIX.5.0SP2) Scope details of the statistics to reduce the number of events being used as basis for the statistics.*/
MDStatisticFrequencyPeriod                         = 2460, /*!< 2460 (int FIX.5.0SP2) Dissemination frequency of statistics.

Special meaning for a value of zero which represents  an event-driven dissemination in real time (e.g. as soon as a new trade occurs).*/
MDStatisticFrequencyUnit                           = 2461, /*!< 2461 (int FIX.5.0SP2) Time unit for MDStatisticFrequencyPeriod(2460).*/
MDStatisticDelayPeriod                             = 2462, /*!< 2462 (int FIX.5.0SP2) Number of time units between the calculation of the statistic and its dissemination. Can be used to defer or delay publication.*/
MDStatisticDelayUnit                               = 2463, /*!< 2463 (int FIX.5.0SP2) Time unit for MDStatisticDelayPeriod(2462).*/
MDStatisticIntervalType                            = 2464, /*!< 2464 (int FIX.5.0SP2) Type of interval over which statistic is calculated.*/
MDStatisticIntervalTypeUnit                        = 2465, /*!< 2465 (String FIX.5.0SP2) Time unit for MDStatisticIntervalType(2464).*/
MDStatisticIntervalPeriod                          = 2466, /*!< 2466 (int FIX.5.0SP2) Length of time over which the statistic is calculated. Special meaning for a value of zero to express that there is no aggregation over time. Can be used with other interval types expressing relative date and time ranges to combine them with sliding window peaks, e.g. highest volume across 1 minute intervals of the previous day.*/
MDStatisticIntervalUnit                            = 2467, /*!< 2467 (int FIX.5.0SP2) Time unit for MDStatisticIntervalPeriod(2466).*/
MDStatisticStartDate                               = 2468, /*!< 2468 (UTCTimestamp FIX.5.0SP2) First day of range for which statistical data is collected.*/
MDStatisticEndDate                                 = 2469, /*!< 2469 (UTCTimestamp FIX.5.0SP2) Last day of range for which statistical data is collected.*/
MDStatisticStartTime                               = 2470, /*!< 2470 (UTCTimeOnly FIX.5.0SP2) Start time of the time range for which statistical data is collected.*/
MDStatisticEndTime                                 = 2471, /*!< 2471 (UTCTimeOnly FIX.5.0SP2) End time of the time range for which statistical data is collected.*/
MDStatisticRatioType                               = 2472, /*!< 2472 (int FIX.5.0SP2) Ratios between various entities.*/
MDStatisticRequestResult                           = 2473, /*!< 2473 (int FIX.5.0SP2) Result returned in response to MarketDataStatisticsRequest (35=DO).*/
NoMDStatistics                                     = 2474, /*!< 2474 (NumInGroup FIX.5.0SP2) Number of market data statistics.*/
MDStatisticID                                      = 2475, /*!< 2475 (String FIX.5.0SP2) Unique identifier for a statistic.*/
MDStatisticTime                                    = 2476, /*!< 2476 (UTCTimestamp FIX.5.0SP2) Time of calculation of a statistic.*/
MDStatisticStatus                                  = 2477, /*!< 2477 (int FIX.5.0SP2) Status for a statistic to indicate its availability.*/
MDStatisticValue                                   = 2478, /*!< 2478 (float FIX.5.0SP2) Statistical value.*/
MDStatisticValueType                               = 2479, /*!< 2479 (int FIX.5.0SP2) Type of statistical value.*/
MDStatisticValueUnit                               = 2480, /*!< 2480 (int FIX.5.0SP2) Unit of time for statistical value.*/
EncodedMDStatisticDescLen                          = 2481, /*!< 2481 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedMDStatisticDesc(2482) field.*/
EncodedMDStatisticDesc                             = 2482, /*!< 2482 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the MDStatisticDesc(2455) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the MDStatisticDesc(2455) field.*/
AllocRiskLimitCheckStatus                          = 2483, /*!< 2483 (int FIX.5.0SP2) Indicates the status of the risk limit check performed on a trade for this allocation instance.*/
FirmTransactionID                                  = 2484, /*!< 2484 (String FIX.5.0SP2) The unique transaction entity identifier assigned by the firm.*/
TransactionID                                      = 2485, /*!< 2485 (String FIX.5.0SP2) The unique transaction entity identifier.*/
WireReference                                      = 2486, /*!< 2486 (String FIX.5.0SP2) The reference to a wire transfer associated with the transaction. Wire references done via wire services such as Fedwire Output Message Accountabilitty Data "OMAD" or SWIFT Output Sequence Number "OSN".*/
CollRptRejectReason                                = 2487, /*!< 2487 (int FIX.5.0SP2) Reject reason code for rejecting the collateral report.*/
CollRptStatus                                      = 2488, /*!< 2488 (int FIX.5.0SP2) The status of the collateral report.*/
PackageID                                          = 2489, /*!< 2489 (String FIX.5.0SP2) Identifier assigned to a collection of trades so that they can be analyzed as one atomic unit for risk assessment and clearing.*/
TradeNumber                                        = 2490, /*!< 2490 (int FIX.5.0SP2) Ordinal number of the trade within a series of related trades.*/
UnderlyingAssetGroup                               = 2491, /*!< 2491 (int FIX.5.0SP2) Indicates the broad product or asset classification.  May be used to provide grouping for the product taxonomy (Product(460), SecurityType(167), etc.) and/or the risk taxonomy (AssetClass(1938), AssetSubClass(1939), AssetType(1940), etc.).*/
LegDifferentialPrice                               = 2492, /*!< 2492 (PriceOffset FIX.5.0SP2) Used in pricing a group of individual Trade at Settlement (TAS) and Trade At Marker (TAM) contracts as an atomic unit. The value is the negotiated currency offset either at settlement (TAS) or at the time specified in the product definition (TAM). The final contract price is reported in LegLastPx(637).*/
EncodedLegDocumentationText                        = 2493, /*!< 2493 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegDocumentationText(2505) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) representation should also be specified in the LegDocumentationText(2505) field.*/
EncodedLegDocumentationTextLen                     = 2494, /*!< 2494 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegDocumentationText(2493) field.*/
LegAgreementCurrency                               = 2495, /*!< 2495 (Currency FIX.5.0SP2) Contractual currency forming the basis of a financing agreement and associated transactions. Usually, but not always, the same as the trade currency.*/
LegAgreementDate                                   = 2496, /*!< 2496 (LocalMktDate FIX.5.0SP2) A reference to the date the underlying agreement specified by LegAgreementID(2498) and LegAgreementDesc(2497) was executed.*/
LegAgreementDesc                                   = 2497, /*!< 2497 (String FIX.5.0SP2) The full name of the base standard agreement, annexes and amendments in place between the principals applicable to a financing transaction. See http://www.fpml.org/coding-scheme/master-agreement-type for derivative values.*/
LegAgreementID                                     = 2498, /*!< 2498 (String FIX.5.0SP2) A common reference to the applicable standing agreement between the counterparties to a financing transaction.*/
LegAgreementVersion                                = 2499, /*!< 2499 (String FIX.5.0SP2) The version of the master agreement.*/
LegBrokerConfirmationDesc                          = 2500, /*!< 2500 (String FIX.5.0SP2) Describes the type of broker confirmation executed between the parties. Can be used as an alternative to MasterConfirmationDesc(1962). See http://www.fpml.org/coding-scheme/broker-confirmation-type for values.*/
LegCreditSupportAgreementDate                      = 2501, /*!< 2501 (LocalMktDate FIX.5.0SP2) The date of the ISDA Credit Support Agreement executed between the parties and intended to govern collateral arrangements for all OTC derivatives transactions between those parties.*/
LegCreditSupportAgreementDesc                      = 2502, /*!< 2502 (String FIX.5.0SP2) The type of ISDA Credit Support Agreement. See http://www.fpml.org/coding-scheme/credit-support-agreement-type for values.*/
LegCreditSupportAgreementID                        = 2503, /*!< 2503 (String FIX.5.0SP2) A common reference or unique identifier to identify the ISDA Credit Support Agreement executed between the parties.*/
LegDeliveryType                                    = 2504, /*!< 2504 (int FIX.5.0SP2) Identifies type of settlement.*/
LegDocumentationText                               = 2505, /*!< 2505 (String FIX.5.0SP2) A sentence or phrase pertinent to the trade, not a reference to an external document. E.g. "To be registered with the U.S. Environmental Protection Agency, Acid Rain Division, SO2 Allowance Tracking System".*/
LegEndDate                                         = 2506, /*!< 2506 (LocalMktDate FIX.5.0SP2) End date of a financing deal, i.e. the date the seller reimburses the buyer and takes back control of the collateral.*/
LegGoverningLaw                                    = 2507, /*!< 2507 (String FIX.5.0SP2) Identification of the law governing the transaction. See http://www.fpml.org/coding-scheme/governing-law for values.*/
LegMarginRatio                                     = 2508, /*!< 2508 (Percentage FIX.5.0SP2) The fraction of the cash consideration that must be collateralized, expressed as a percent. A MarginRatio of 2% indicates that the value of the collateral (after deducting for "haircut") must exceed the cash consideration by 2%.*/
LegMasterConfirmationAnnexDate                     = 2509, /*!< 2509 (LocalMktDate FIX.5.0SP2) The date that an annexation to the master confirmation was executed between the parties.*/
LegMasterConfirmationDate                          = 2510, /*!< 2510 (LocalMktDate FIX.5.0SP2) Alternative to broker confirmation. The date of the confirmation executed between the parties and intended to govern all relevant transactions between those parties.*/
LegMasterConfirmationDesc                          = 2511, /*!< 2511 (String FIX.5.0SP2) The type of master confirmation executed between the parties. See http://www.fpml.org/coding-scheme/master-confirmation-type for values.*/
LegMasterConfirmationAnnexDesc                     = 2512, /*!< 2512 (String FIX.5.0SP2) The type of master confirmation annexation executed between the parties. See http://www.fpml.org/coding-scheme/master-confirmation-annex-type for values.*/
LegStartDate                                       = 2513, /*!< 2513 (LocalMktDate FIX.5.0SP2) Start date of a financing deal, i.e. the date the buyer pays the seller cash and takes control of the collateral.*/
LegTerminationType                                 = 2514, /*!< 2514 (int FIX.5.0SP2) Type of financing termination.*/
AllocCalculatedCcyQty                              = 2515, /*!< 2515 (Qty FIX.5.0SP2) Used for the calculated quantity of the other side of the currency trade applicable to the allocation instance.*/
CollateralRequestInstruction                       = 2516, /*!< 2516 (String FIX.5.0SP2) An encoded collateral request processing instruction to the receiver. */
CollateralRequestLinkID                            = 2517, /*!< 2517 (String FIX.5.0SP2) A unique identifier to link together a set or group of requests.*/
CollateralRequestNumber                            = 2518, /*!< 2518 (int FIX.5.0SP2) Ordinal number of the request within a set or group of requests.*/
TotNumCollateralRequests                           = 2519, /*!< 2519 (int FIX.5.0SP2) Total number of request messages within a set or group of requests.*/
WarningText                                        = 2520, /*!< 2520 (String FIX.5.0SP2) Communicates the underlying condition when the request response indicates "warning".*/
EncodedWarningText                                 = 2521, /*!< 2521 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the WarningText(2520) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) representation should also be specified in the WarningText(2520) field.*/
EncodedWarningTextLen                              = 2522, /*!< 2522 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedWarningtText(2521) field.*/
CrossedIndicator                                   = 2523, /*!< 2523 (int FIX.5.0SP2) Indicates whether the order or quote was crossed with another order or quote having the same context, e.g. having accounts with a common ownership.*/
TradeReportingIndicator                            = 2524, /*!< 2524 (int FIX.5.0SP2) Used between parties to convey trade reporting status.*/
AffiliatedFirmsTradeIndicator                      = 2525, /*!< 2525 (Boolean FIX.5.0SP2) Indicates whether the transaction or position was entered into between two affiliated firms. I.e. one counterparty has an ownership interest in the other counterparty but less than the majority interest. */
InternationalSwapIndicator                         = 2526, /*!< 2526 (Boolean FIX.5.0SP2) Identifies the swap trade as an "international" transaction.*/
MultiAssetSwapIndicator                            = 2527, /*!< 2527 (Boolean FIX.5.0SP2) Indicates a swap that does not have one easily identifiable primary underlying asset, but instead involves multiple underlying assets within one trade repository's jurisdiction that belong to different asset classes.*/
ClearingSettlPrice                                 = 2528, /*!< 2528 (Price FIX.5.0SP2) Clearing settlement price.*/
NoRelativeValues                                   = 2529, /*!< 2529 (NumInGroup FIX.5.0SP2) Number of relative value metrics entries in the repeating group.*/
RelativeValueType                                  = 2530, /*!< 2530 (int FIX.5.0SP2) Indicates the type of relative value measurement being specified.*/
RelativeValue                                      = 2531, /*!< 2531 (float FIX.5.0SP2) The valuation of an instrument relative to a base measurement specified in RelativeValueType(2530). This value can be negative.*/
RelativeValueSide                                  = 2532, /*!< 2532 (int FIX.5.0SP2) Specifies the side of the relative value.*/
BidSpread                                          = 2533, /*!< 2533 (float FIX.5.0SP2) Basis points relative to a benchmark curve on the bid side, such as LIBOR, or a known security, such as 10Y US Treasury bond. The benchmark security or curve name is specified in the SpreadOrBenchmarkCurveData component.*/
OfferSpread                                        = 2534, /*!< 2534 (float FIX.5.0SP2) Basis points relative to a benchmark curve on the offer side, such as LIBOR, or a known security, such as 10Y US Treasury bond. The benchmark security or curve name is specified in the SpreadOrBenchmarkCurveData component.*/
MDReportEvent                                      = 2535, /*!< 2535 (int FIX.5.0SP2) Technical event within market data feed.*/
MDReportCount                                      = 2536, /*!< 2536 (int FIX.5.0SP2) Number of reference and market data messages in-between two MarketDataReport(35=DR) messages.*/
TotNoMarketSegmentReports                          = 2537, /*!< 2537 (int FIX.5.0SP2) Total number of reports related to market segments.*/
TotNoInstrumentReports                             = 2538, /*!< 2538 (int FIX.5.0SP2) Total number of reports related to instruments.*/
TotNoPartyDetailReports                            = 2539, /*!< 2539 (int FIX.5.0SP2) Total number of reports related to party detail information.*/
TotNoEntitlementReports                            = 2540, /*!< 2540 (int FIX.5.0SP2) Total number of reports related to party entitlement information.*/
TotNoRiskLimitReports                              = 2541, /*!< 2541 (int FIX.5.0SP2) Total number of reports related to party risk limit information.*/
MarketSegmentStatus                                = 2542, /*!< 2542 (int FIX.5.0SP2) Status of market segment.*/
MarketSegmentType                                  = 2543, /*!< 2543 (int FIX.5.0SP2) Used to classify the type of market segment.*/
MarketSegmentSubType                               = 2544, /*!< 2544 (int FIX.5.0SP2) Used to further categorize market segments within a MarketSegmentType(2543).*/
NoRelatedMarketSegments                            = 2545, /*!< 2545 (NumInGroup FIX.5.0SP2) Number of related market segments.*/
RelatedMarketSegmentID                             = 2546, /*!< 2546 (String FIX.5.0SP2) Identifies a related market segment.*/
MarketSegmentRelationship                          = 2547, /*!< 2547 (int FIX.5.0SP2) Type of relationship between two or more market segments.*/
NoAuctionTypeRules                                 = 2548, /*!< 2548 (NumInGroup FIX.5.0SP2) Number of auction order types.*/
AuctionTypeProductComplex                          = 2549, /*!< 2549 (String FIX.5.0SP2) Identifies an entire suite of products for which the auction order type rule applies. */
NoPriceRangeRules                                  = 2550, /*!< 2550 (NumInGroup FIX.5.0SP2) Number of rules related to price ranges.*/
StartPriceRange                                    = 2551, /*!< 2551 (Price FIX.5.0SP2) Lower boundary for price range.*/
EndPriceRange                                      = 2552, /*!< 2552 (Price FIX.5.0SP2) Upper boundary for price range.*/
PriceRangeValue                                    = 2553, /*!< 2553 (Price FIX.5.0SP2) Maximum range expressed as absolute value.*/
PriceRangePercentage                               = 2554, /*!< 2554 (Percentage FIX.5.0SP2) Maximum range expressed as percentage.*/
PriceRangeProductComplex                           = 2555, /*!< 2555 (String FIX.5.0SP2) Identifies an entire suite of products in the context of trading rules related to price ranges.*/
PriceRangeRuleID                                   = 2556, /*!< 2556 (String FIX.5.0SP2) Identifier for a price range rule.*/
FastMarketPercentage                               = 2557, /*!< 2557 (Percentage FIX.5.0SP2) The percentage factor to be applied to trading rule parameters (e.g. price ranges, size ranges, etc.) when fast market conditions are applicable.*/
NoQuoteSizeRules                                   = 2558, /*!< 2558 (NumInGroup FIX.5.0SP2) Number of rules related to quote sizes.*/
QuoteSideIndicator                                 = 2559, /*!< 2559 (Boolean FIX.5.0SP2) Indicates whether single sided quotes are allowed.*/
NoFlexProductEligibilities                         = 2560, /*!< 2560 (NumInGroup FIX.5.0SP2) Number of eligibility indicators for the creation of flexible securities.*/
FlexProductEligibilityComplex                      = 2561, /*!< 2561 (String FIX.5.0SP2) Identifies an entire suite of products which are eligible for the creation of flexible securities.*/
NumOfComplexInstruments                            = 2562, /*!< 2562 (int FIX.5.0SP2) Represents the total number of multileg securities or user defined securities that make up the security.*/
MarketDepthTimeInterval                            = 2563, /*!< 2563 (int FIX.5.0SP2) Specifies the time interval used for netting market data in a price depth feed.*/
MarketDepthTimeIntervalUnit                        = 2564, /*!< 2564 (int FIX.5.0SP2) The time unit associated with the time interval of the netting of market data in a price depth feed.*/
MDRecoveryTimeInterval                             = 2565, /*!< 2565 (int FIX.5.0SP2) Specifies the time interval between two repetitions of the same market data for cyclic recovery feeds.*/
MDRecoveryTimeIntervalUnit                         = 2566, /*!< 2566 (int FIX.5.0SP2) The time unit associated with the time interval between two cycles of the same market data in cyclic data recovery feeds.*/
PrimaryServiceLocationID                           = 2567, /*!< 2567 (String FIX.5.0SP2) Primary service location identifier.*/
SecondaryServiceLocationID                         = 2568, /*!< 2568 (String FIX.5.0SP2) Secondary or alternate service location identifier.*/
MatchRuleProductComplex                            = 2569, /*!< 2569 (String FIX.5.0SP2) Identifies an entire suite of products for which the matching rule applies.*/
CustomerPriority                                   = 2570, /*!< 2570 (int FIX.5.0SP2) Specifies the kind of priority given to customers.*/
TickRuleProductComplex                             = 2571, /*!< 2571 (String FIX.5.0SP2) Identifies an entire suite of products for which the price tick rule applies.*/
PreviousAdjustedOpenInterest                       = 2572, /*!< 2572 (Amt FIX.5.0SP2) Previous day's adjusted open interest.*/
PreviousUnadjustedOpenInterest                     = 2573, /*!< 2573 (Amt FIX.5.0SP2) Previous day's unadjusted open interest.*/
LowExercisePriceOptionIndicator                    = 2574, /*!< 2574 (Boolean FIX.5.0SP2) Indicates if a given option instrument permits low exercise prices (LEPO).*/
BlockTradeEligibilityIndicator                     = 2575, /*!< 2575 (Boolean FIX.5.0SP2) Indicates if a given instrument is eligible for block trading.*/
InstrumentPricePrecision                           = 2576, /*!< 2576 (int FIX.5.0SP2) Specifies the number of decimal places for instrument prices.*/
StrikePricePrecision                               = 2577, /*!< 2577 (int FIX.5.0SP2) Specifies the number of decimal places for exercise price.*/
OrigStrikePrice                                    = 2578, /*!< 2578 (Price FIX.5.0SP2) Original exercise price, e.g. after corporate action requiring changes.*/
SettlSubMethod                                     = 2579, /*!< 2579 (int FIX.5.0SP2) Specifies a suitable settlement sub-method for a given settlement method.*/
NoClearingPriceParameters                          = 2580, /*!< 2580 (NumInGroup FIX.5.0SP2) Number of parameter sets for clearing prices.*/
BusinessDayType                                    = 2581, /*!< 2581 (int FIX.5.0SP2) Relative identification of a business day.*/
ClearingPriceOffset                                = 2582, /*!< 2582 (PriceOffset FIX.5.0SP2) Constant value required for the calculation of the clearing price, e.g. for variance futures.*/
VegaMultiplier                                     = 2583, /*!< 2583 (float FIX.5.0SP2) Constant value required for the calculation of the clearing quantity, e.g. for variance futures.*/
AnnualTradingBusinessDays                          = 2584, /*!< 2584 (int FIX.5.0SP2) Number of trading business days in a year.*/
TotalTradingBusinessDays                           = 2585, /*!< 2585 (int FIX.5.0SP2) Number of trading business days over the lifetime of an instrument.*/
TradingBusinessDays                                = 2586, /*!< 2586 (int FIX.5.0SP2) Number of actual trading business days of an instrument.*/
RealizedVariance                                   = 2587, /*!< 2587 (float FIX.5.0SP2) Actual or realized variance of an instrument used to calculate settlement prices, e.g. for variance futures.*/
StandardVariance                                   = 2588, /*!< 2588 (float FIX.5.0SP2) Standard variance (over the lifetime of an instrument) or initial variance used to calculate settlement prices, e.g. for variance futures.*/
RelatedClosePrice                                  = 2589, /*!< 2589 (Price FIX.5.0SP2) Closing price of the underlying required to calculate the RealizedVariance(2587).*/
OvernightInterestRate                              = 2590, /*!< 2590 (float FIX.5.0SP2) Overnight interest rate.*/
AccumulatedReturnModifiedVariationMargin           = 2591, /*!< 2591 (float FIX.5.0SP2) The economic cost of the variation margin from one trading day to the next.*/
CalculationMethod                                  = 2592, /*!< 2592 (int FIX.5.0SP2) Specifies how the calculation will be made.*/
NoOrderAttributes                                  = 2593, /*!< 2593 (NumInGroup FIX.5.0SP2) Number of order attribute entries.*/
OrderAttributeType                                 = 2594, /*!< 2594 (int FIX.5.0SP2) The type of order attribute.*/
OrderAttributeValue                                = 2595, /*!< 2595 (String FIX.5.0SP2) The value associated with the order attribute type specified in OrderAttributeType(2594).*/
DeltaCrossed                                       = 2596, /*!< 2596 (Boolean FIX.5.0SP2) Indicates that the party has taken a position on both a put and a call on the same underlying asset.*/
ComplexEventFuturesPriceValuation                  = 2597, /*!< 2597 (Boolean FIX.5.0SP2) Indicates whether the official settlement price as announced by the related exchange is applicable, in accordance with the ISDA 2002 definitions. Applicable only to futures contracts.*/
ComplexEventOptionsPriceValuation                  = 2598, /*!< 2598 (Boolean FIX.5.0SP2) Indicates whether the official settlement price as announced by the related exchange is applicable, in accordance with the ISDA 2002 definitions. Applicable only to options contracts.*/
ComplexEventPVFinalPriceElectionFallback           = 2599, /*!< 2599 (int FIX.5.0SP2) Specifies the fallback provisions for the hedging party in the determination of the final settlement price.*/
StrikeIndexCurvePoint                              = 2600, /*!< 2600 (String FIX.5.0SP2) The point on the floating rate index curve. Sample values:

M = combination of a number between 1-12 and an "M" for month, e.g. 3M

Y = combination of number between 1-100 and a "Y" for year, e.g. 10Y

10Y-OLD = see above, then add "-OLD" when appropriate

INTERPOLATED = the point is mathematically derived

2/2031 5 3/8 = the point is stated via a combination of maturity month / year and coupon.*/
StrikeIndexQuote                                   = 2601, /*!< 2601 (int FIX.5.0SP2) The quote side from which the index price is to be determined.*/
ExtraordinaryEventAdjustmentMethod                 = 2602, /*!< 2602 (int FIX.5.0SP2) Defines how adjustments will be made to the contract should one or more of the extraordinary events occur.*/
ExchangeLookAlike                                  = 2603, /*!< 2603 (Boolean FIX.5.0SP2) For a share option trade, indicates whether the instrument is to be treated as an 'exchange look-alike'. */
LegStrikeIndexCurvePoint                           = 2604, /*!< 2604 (String FIX.5.0SP2) The point on the floating rate index curve. Sample values:

M = combination of a number between 1-12 and an "M" for month, e.g. 3M

Y = combination of number between 1-100 and a "Y" for year, e.g. 10Y

10Y-OLD = see above, then add "-OLD" when appropriate

INTERPOLATED = the point is mathematically derived

2/2031 5 3/8 = the point is stated via a combination of maturity month / year and coupon.*/
LegStrikeIndexQuote                                = 2605, /*!< 2605 (int FIX.5.0SP2) The quote side from which the index price is to be determined.*/
LegExtraordinaryEventAdjustmentMethod              = 2606, /*!< 2606 (int FIX.5.0SP2) Defines how adjustments will be made to the contract should one or more of the extraordinary events occur.*/
LegExchangeLookAlike                               = 2607, /*!< 2607 (Boolean FIX.5.0SP2) For a share option trade, indicates whether the instrument is to be treated as an 'exchange look-alike'. */
LegComplexEventFuturesPriceValuation               = 2608, /*!< 2608 (Boolean FIX.5.0SP2) Indicates whether the official settlement price as announced by the related exchange is applicable, in accordance with the ISDA 2002 definitions. Applicable only to futures contracts.*/
LegComplexEventOptionsPriceValuation               = 2609, /*!< 2609 (Boolean FIX.5.0SP2) Indicates whether the official settlement price as announced by the related exchange is applicable, in accordance with the ISDA 2002 definitions. Applicable only to options contracts.*/
LegComplexEventPVFinalPriceElectionFallback        = 2610, /*!< 2610 (int FIX.5.0SP2) Specifies the fallback provisions for the hedging party in the determination of the final settlement price*/
UnderlyingComplexEventFuturesPriceValuation        = 2611, /*!< 2611 (Boolean FIX.5.0SP2) Indicates whether the official settlement price as announced by the related exchange is applicable, in accordance with the ISDA 2002 definitions. Applicable only to futures contracts.*/
UnderlyingComplexEventOptionsPriceValuation        = 2612, /*!< 2612 (Boolean FIX.5.0SP2) Indicates whether the official settlement price as announced by the related exchange is applicable, in accordance with the ISDA 2002 definitions. Applicable only to options contracts.*/
UnderlyingComplexEventPVFinalPriceElectionFallback = 2613, /*!< 2613 (int FIX.5.0SP2) Specifies the fallback provisions for the hedging party in the determination of the final settlement price*/
UnderlyingNotional                                 = 2614, /*!< 2614 (Amt FIX.5.0SP2) Notional value for the equity or bond underlier.*/
UnderlyingNotionalCurrency                         = 2615, /*!< 2615 (Currency FIX.5.0SP2) Specifies the currency denomination of the notional value. Uses ISO 4217 currency codes.*/
UnderlyingNotionalDeterminationMethod              = 2616, /*!< 2616 (String FIX.5.0SP2) Specifies the method of determining the notional amount. 

See: http://www.fpml.org/coding-scheme/determination-method for values.*/
UnderlyingNotionalAdjustments                      = 2617, /*!< 2617 (int FIX.5.0SP2) Specifies the conditions that govern the adjustment to the number of units of the return swap.*/
PositionID                                         = 2618, /*!< 2618 (String FIX.5.0SP2) Unique identifier for a position entity. Refer to PosMaintRptID(721) for a unique identifier of a position report message.*/
UnderlyingNotionalXIDRef                           = 2619, /*!< 2619 (XIDREF FIX.5.0SP2) Cross reference to another notional amount for duplicating its properties.*/
UnderlyingFutureID                                 = 2620, /*!< 2620 (String FIX.5.0SP2) In the case of an index underlier specifies the unique identifier for the referenced futures contract.*/
UnderlyingFutureIDSource                           = 2621, /*!< 2621 (String FIX.5.0SP2) Identifies the source of the UnderlyingFutureID(2620).*/
UnderlyingStrikeIndexCurvePoint                    = 2622, /*!< 2622 (String FIX.5.0SP2) The point on the floating rate index curve. Sample values:

M = combination of a number between 1-12 and an "M" for month, e.g. 3M

Y = combination of number between 1-100 and a "Y" for year, e.g. 10Y

10Y-OLD = see above, then add "-OLD" when appropriate

INTERPOLATED = the point is mathematically derived

2/2031 5 3/8 = the point is stated via a combination of maturity month / year and coupon.*/
UnderlyingStrikeIndexQuote                         = 2623, /*!< 2623 (int FIX.5.0SP2) The quote side from which the index price is to be determined.*/
UnderlyingExtraordinaryEventAdjustmentMethod       = 2624, /*!< 2624 (int FIX.5.0SP2) Defines how adjustments will be made to the contract should one or more of the extraordinary events occur.*/
UnderlyingExchangeLookAlike                        = 2625, /*!< 2625 (Boolean FIX.5.0SP2) For a share option trade, indicates whether the instrument is to be treated as an 'exchange look-alike'. */
UnderlyingAverageVolumeLimitationPercentage        = 2626, /*!< 2626 (Amt FIX.5.0SP2) The limit of average percentage of individual securities traded in a day or a number of days.*/
UnderlyingAverageVolumeLimitationPeriodDays        = 2627, /*!< 2627 (int FIX.5.0SP2) Specifies the limitation period for average daily trading volume in number of days.*/
UnderlyingDepositoryReceiptIndicator               = 2628, /*!< 2628 (Boolean FIX.5.0SP2) Indicates whether the underlier is a depository receipt.*/
UnderlyingOpenUnits                                = 2629, /*!< 2629 (Qty FIX.5.0SP2) The number of units (units of the index or number of securities, par amount of a bond) that constitute the underlier. In the case of a basket swap, this is used to reference both the number of basket units, and the number of each asset components of the basket when these are expressed in absolute terms.*/
UnderlyingBasketDivisor                            = 2630, /*!< 2630 (float FIX.5.0SP2) Specifies the basket divisor amount. This value is normally used to adjust the constituent weight for pricing or to adjust for dividends, or other corporate actions.*/
UnderlyingInstrumentXID                            = 2631, /*!< 2631 (XID FIX.5.0SP2) Identifier for referencing this UnderlyingInstrument from a parent instrument or a convertible instrument.*/
CollateralAmountType                               = 2632, /*!< 2632 (int FIX.5.0SP2) The type of value in CurrentCollateralAmount(1704).*/
NoMiscFeeSubTypes                                  = 2633, /*!< 2633 (NumInGroup FIX.5.0SP2) Specifies the number of miscellaneous fee sub-types.*/
MiscFeeSubType                                     = 2634, /*!< 2634 (String FIX.5.0SP2) Used to provide more granular fee types related to a value of MiscFeeType(139).

See http://www.fixtradingcommunity.org/codelists#Misc_Fee_Sub_Types for code list of applicable fees. Other fee sub-types may be used by mutual agreement of the counterparties.*/
MiscFeeSubTypeAmt                                  = 2635, /*!< 2635 (Amt FIX.5.0SP2) The amount of the specified MiscFeeSubType(2634).*/
MiscFeeSubTypeDesc                                 = 2636, /*!< 2636 (String FIX.5.0SP2) Can be used to provide an optional textual description of the fee sub-type.*/
EncodedMiscFeeSubTypeDescLen                       = 2637, /*!< 2637 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedMiscFeeSubTypeDesc(2638) field.*/
EncodedMiscFeeSubTypeDesc                          = 2638, /*!< 2638 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the MiscFeeSubTypeDesc(2636) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the MiscFeeSubTypeDesc(2636) field.*/
NoCommissions                                      = 2639, /*!< 2639 (NumInGroup FIX.5.0SP2) Number of commissions in the repeating group.*/
CommissionAmount                                   = 2640, /*!< 2640 (Amt FIX.5.0SP2) The commission amount.*/
CommissionAmountType                               = 2641, /*!< 2641 (int FIX.5.0SP2) Indicates what type of commission is being expressed in CommissionAmount(2640).*/
CommissionBasis                                    = 2642, /*!< 2642 (char FIX.5.0SP2) Specifies the basis or unit used to calculate the commission.*/
CommissionCurrency                                 = 2643, /*!< 2643 (Currency FIX.5.0SP2) Specifies the currency denomination of the commission amount if different from the trade's currency. Uses ISO 4217 currency codes.*/
CommissionUnitOfMeasure                            = 2644, /*!< 2644 (String FIX.5.0SP2) The commission rate unit of measure.*/
CommissionUnitOfMeasureCurrency                    = 2645, /*!< 2645 (Currency FIX.5.0SP2) Indicates the currency of the unit of measure. Conditionally required when CommissionUnitOfMeasure(2644) = Ccy (Amount of currency).*/
CommissionRate                                     = 2646, /*!< 2646 (float FIX.5.0SP2) The commission rate when CommissionAmount(2640) is based on a percentage of quantity, amount per unit or a factor of "unit of measure". If the rate is a percentage or expressed in basis points, use the decimalized form, e.g. "0.05" for a 5% commission or "0.005" for 50 basis points.*/
CommissionSharedIndicator                          = 2647, /*!< 2647 (Boolean FIX.5.0SP2) Indicates whether the amount in CommissionAmount(2640) is to be shared with a third party, e.g. as part of a directed brokerage commission sharing arrangement.*/
CommissionAmountShared                             = 2648, /*!< 2648 (Amt FIX.5.0SP2) Commission amount to be shared with a third party, e.g. as part of a directed brokerage commission sharing arrangement. If specified, this amount should not exceed the amount in CommissionAmount(2640).*/
CommissionLegRefID                                 = 2649, /*!< 2649 (String FIX.5.0SP2) Identifies the leg of the trade the entry applies to by referencing the leg's LegID(1788).*/
CommissionDesc                                     = 2650, /*!< 2650 (String FIX.5.0SP2) Description of the commission.*/
EncodedCommissionDescLen                           = 2651, /*!< 2651 (Length FIX.5.0SP2) Byte length of the encoded (non-ASCII characters) EncodedCommissionDesc(2652) field.*/
EncodedCommissionDesc                              = 2652, /*!< 2652 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the CommissionDesc(2650) field in the encoded format specified via the MessageEncoding(347) field.  If used, the ASCII (English) representation should also be specified in the CommissionDesc(2650) field.*/
NoAllocCommissions                                 = 2653, /*!< 2653 (NumInGroup FIX.5.0SP2) Number of commissions in the repeating group.*/
AllocCommissionAmount                              = 2654, /*!< 2654 (Amt FIX.5.0SP2) The commission amount.*/
AllocCommissionAmountType                          = 2655, /*!< 2655 (int FIX.5.0SP2) Indicates what type of commission is being expressed in AllocCommissionAmount(2654).*/
AllocCommissionBasis                               = 2656, /*!< 2656 (char FIX.5.0SP2) Specifies the basis or unit used to calculate the commission.*/
AllocCommissionCurrency                            = 2657, /*!< 2657 (Currency FIX.5.0SP2) Specifies the currency denomination of the commission amount if different from the trade's currency. Uses ISO 4217 currency codes.*/
AllocCommissionUnitOfMeasure                       = 2658, /*!< 2658 (String FIX.5.0SP2) The commission rate unit of measure.*/
AllocCommissionUnitOfMeasureCurrency               = 2659, /*!< 2659 (Currency FIX.5.0SP2) Indicates the currency of the unit of measure. Conditionally required when AllocCommissionUnitOfMeasure(2658) = Ccy (Currency).*/
AllocCommissionRate                                = 2660, /*!< 2660 (float FIX.5.0SP2) The commission rate when AllocCommissionAmount(2654) is based on a percentage of quantity, amount per unit or a factor of "unit of measure". If the rate is a percentage or expressed in basis points, use the decimalized form, e.g. "0.05" for a 5% commission or "0.005" for 50 basis points.*/
AllocCommissionSharedIndicator                     = 2661, /*!< 2661 (Boolean FIX.5.0SP2) Indicates whether the amount in AllocCommissionAmount(2654) is to be shared with a third party, e.g. as part of a directed brokerage commission sharing arrangement.*/
AllocCommissionAmountShared                        = 2662, /*!< 2662 (Amt FIX.5.0SP2) Commission amount to be shared with a third party, e.g. as part of a directed brokerage commission sharing arrangement. If specified, this amount should not exceed the amount in AllocCommissionAmount(2654).*/
AllocCommissionLegRefID                            = 2663, /*!< 2663 (String FIX.5.0SP2) Identifies the leg of the trade the entry applies to by referencing the leg's LegID(1788).*/
AllocCommissionDesc                                = 2664, /*!< 2664 (String FIX.5.0SP2) Description of the commission.*/
EncodedAllocCommissionDescLen                      = 2665, /*!< 2665 (Length FIX.5.0SP2) Byte length of the encoded (non-ASCII characters) EncodedAllocCommissionDesc(2666) field.*/
EncodedAllocCommissionDesc                         = 2666, /*!< 2666 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the AllocCommissionDesc(2664) field in the encoded format specified via the MessageEncoding(347) field.  If used, the ASCII (English) representation should also be specified in the AllocCommissionDesc(2664) field.*/
AlgorithmicTradeIndicator                          = 2667, /*!< 2667 (int FIX.5.0SP2) Indicates that the order or trade originates from a computer program or algorithm requiring little-to-no human intervention. */
NoTrdRegPublications                               = 2668, /*!< 2668 (NumInGroup FIX.5.0SP2) Number of regulatory publication rules in repeating group.*/
TrdRegPublicationType                              = 2669, /*!< 2669 (int FIX.5.0SP2) Specifies the type of regulatory trade publication.

Additional reasons for the publication type may be specified in TrdRegPublicationReason(2670).*/
TrdRegPublicationReason                            = 2670, /*!< 2670 (int FIX.5.0SP2) Additional reason for trade publication type specified in TrdRegPublicationType(2669).

Reasons may be specific to regulatory trade publication rules.

*/
SideTradeReportingIndicator                        = 2671, /*!< 2671 (int FIX.5.0SP2) Used between parties to convey trade reporting status.*/
CrossRequestID                                     = 2672, /*!< 2672 (String FIX.5.0SP2) Unique message identifier for a cross request as assigned by the submitter of the request.*/
FillMatchID                                        = 2673, /*!< 2673 (String FIX.5.0SP2) Identifier assigned by a matching system to a match event containing multiple executions.*/
FillMatchSubID                                     = 2674, /*!< 2674 (String FIX.5.0SP2) Identifier assigned by a matching system to a price level (e.g. match step, clip) within a match event containing multiple executions.*/
MassActionReason                                   = 2675, /*!< 2675 (int FIX.5.0SP2) Reason for submission of mass action.*/
MaximumPricePercentage                             = 2676, /*!< 2676 (Percentage FIX.5.0SP2) Maximum deviation, in percentage terms, of an execution price from a reference price, e.g. the initial price of a match event.*/
NotAffectedReason                                  = 2677, /*!< 2677 (int FIX.5.0SP2) Reason for order being unaffected by mass action even though it belongs to the orders covered by MassActionScope(1374).*/
TotalNotAffectedOrders                             = 2678, /*!< 2678 (int FIX.5.0SP2) Total number of orders unaffected by either the OrderMassActionRequest(35=CA) or OrderMassCancelRequest(35=Q).*/
OrderOwnershipIndicator                            = 2679, /*!< 2679 (int FIX.5.0SP2) Change of ownership of an order to a specific party.*/
LegAccount                                         = 2680, /*!< 2680 (String FIX.5.0SP2) Account mnemonic as agreed between buy and sell sides, e.g. broker and institution or investor/intermediary and fund manager.*/
InTheMoneyCondition                                = 2681, /*!< 2681 (int FIX.5.0SP2) Specifies an option instrument's "in the money" condition.*/
LegInTheMoneyCondition                             = 2682, /*!< 2682 (int FIX.5.0SP2) Specifies an option instrument's "in the money" condition in general terms.*/
UnderlyingInTheMoneyCondition                      = 2683, /*!< 2683 (int FIX.5.0SP2) Specifies an option instrument's "in the money" condition in general terms.*/
DerivativeInTheMoneyCondition                      = 2684, /*!< 2684 (int FIX.5.0SP2) Specifies an option instrument's "in the money" condition in general terms.*/
ContraryInstructionEligibilityIndicator            = 2685, /*!< 2685 (Boolean FIX.5.0SP2) Identifies whether the option instrument is eligible for contrary instructions at the time of exercise.  The contrariness of an instruction will be determined in the context of InTheMoneyCondition(2681).  When not specified, the eligibility is undefined or not applicable.*/
LegContraryInstructionEligibilityIndicator         = 2686, /*!< 2686 (Boolean FIX.5.0SP2) Identifies whether the option instrument is eligible for contrary instructions at the time of exercise.  The contrariness of an instruction will be determined in the context of LegInTheMoneyCondition(2682).  When not specified, the eligibility is undefined or not applicable.*/
UnderlyingContraryInstructionEligibilityIndicator  = 2687, /*!< 2687 (Boolean FIX.5.0SP2) 	Identifies whether the option instrument is eligible for contrary instructions at the time of exercise.  The contrariness of an instruction will be determined in the context of UnderlyingInTheMoneyCondition(2683).  When not specified, the eligibility is undefined or not applicable.*/
DerivativeContraryInstructionEligibilityIndicator  = 2688, /*!< 2688 (Boolean FIX.5.0SP2) Identifies whether the option instrument is eligible for contrary instructions at the time of exercise.  The contrariness of an instruction will be determined in the context of DerivativeInTheMoneyCondition(2684).  When not specified, the eligibility is undefined or not applicable.*/
CollateralMarketPrice                              = 2689, /*!< 2689 (Price FIX.5.0SP2) Market price of the collateral, either from market sources or pre-agreed by the counterparties.*/
CollateralPercentOverage                           = 2690, /*!< 2690 (Percentage FIX.5.0SP2) Percentage of over-collateralization particularly when CollateralAmountType(2632) = 4 (Additional collateral value)*/
NoSideCollateralAmounts                            = 2691, /*!< 2691 (NumInGroup FIX.5.0SP2) Number of side collateral amount entries.*/
SideCollateralAmountMarketID                       = 2692, /*!< 2692 (String FIX.5.0SP2) Market associated with the collateral amount.*/
SideCollateralAmountMarketSegmentID                = 2693, /*!< 2693 (String FIX.5.0SP2) Market segment associated with the collateral amount.*/
SideCollateralAmountType                           = 2694, /*!< 2694 (int FIX.5.0SP2) The type of value in CurrentCollateralAmount(1704).*/
SideCollateralCurrency                             = 2695, /*!< 2695 (Currency FIX.5.0SP2) Specifies the currency of the collateral; optional, defaults to the settlement currency if not specified. Uses ISO 4217 Currency Code.*/
SideCollateralFXRate                               = 2696, /*!< 2696 (float FIX.5.0SP2) Foreign exchange rate used to compute the SideCurrentCollateralAmount(2702) from the SideCollateralCurrency(2695) and the Currency(15).*/
SideCollateralFXRateCalc                           = 2697, /*!< 2697 (char FIX.5.0SP2) Specifies whether or not SideCollateralFXRate(2696) should be multiplied or divided.*/
SideCollateralMarketPrice                          = 2698, /*!< 2698 (Price FIX.5.0SP2) Market price of the collateral, either from market sources or pre-agreed by the counterparties.*/
SideCollateralPercentOverage                       = 2699, /*!< 2699 (Percentage FIX.5.0SP2) Percentage of over-collateralization particularly when SideCollateralAmountType(2694) = 4 (Additional collateral value).*/
SideCollateralPortfolioID                          = 2700, /*!< 2700 (String FIX.5.0SP2) Identifier of the collateral portfolio when reporting on a portfolio basis.*/
SideCollateralType                                 = 2701, /*!< 2701 (String FIX.5.0SP2) Type of collateral on deposit being reported.*/
SideCurrentCollateralAmount                        = 2702, /*!< 2702 (Amt FIX.5.0SP2) Currency value currently attributed to the collateral.*/
SideHaircutIndicator                               = 2703, /*!< 2703 (Boolean FIX.5.0SP2) Indicates, if "Y", that a stated valuation includes a haircut, e.g. that the stated value reflects the subtraction of the haircut. Note that a value of "N" does not imply a haircut is not applicable, only that the haircut (if any) is not reflected in the stated valuation.*/
ExDestinationType                                  = 2704, /*!< 2704 (int FIX.5.0SP2) Identifies the type of execution destination for the order.*/
MarketCondition                                    = 2705, /*!< 2705 (int FIX.5.0SP2) Market condition. In the context of ESMA RTS 8 it is important that trading venues communicate the condition of the market, particularly "stressed" and "exceptional", in order to provide incentives for firms contributing to liquidity.*/
NoQuoteAttributes                                  = 2706, /*!< 2706 (NumInGroup FIX.5.0SP2) Number of quote attributes entries.*/
QuoteAttributeType                                 = 2707, /*!< 2707 (int FIX.5.0SP2) The type of attribute for the quote.*/
QuoteAttributeValue                                = 2708, /*!< 2708 (String FIX.5.0SP2) The value associated with the quote attribute type specified in QuoteAttributeType(2707).*/
NoPriceQualifiers                                  = 2709, /*!< 2709 (NumInGroup FIX.5.0SP2) Number of price qualifiers in the repeating group.*/
PriceQualifier                                     = 2710, /*!< 2710 (int FIX.5.0SP2) Qualifier for price.  May be used when the price needs to be explicitly qualified. */
MDValueTier                                        = 2711, /*!< 2711 (int FIX.5.0SP2) Describes the reporting ranges for executed transactions.*/
MiscFeeQualifier                                   = 2712, /*!< 2712 (int FIX.5.0SP2) Identifies whether the current entry contributes to the trade or transaction economics, i.e. affects NetMoney(118).	*/
MiscFeeDesc                                        = 2713, /*!< 2713 (String FIX.5.0SP2) Can be used to provide a textual description of the fee type.*/
FinancialInstrumentFullName                        = 2714, /*!< 2714 (String FIX.5.0SP2) The full normative name of the financial instrument.*/
EncodedFinancialInstrumentFullNameLen              = 2715, /*!< 2715 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedFinancialInstrumentFullName(2716) field.*/
EncodedFinancialInstrumentFullName                 = 2716, /*!< 2716 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the FinancialInstrumentFullName(2714) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) representation should also be specified in the FinancialInstrumentFullName(2714) field.*/
LegFinancialInstrumentFullName                     = 2717, /*!< 2717 (String FIX.5.0SP2) The full normative name of the multileg's financial instrument.*/
EncodedLegFinancialInstrumentFullNameLen           = 2718, /*!< 2718 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) individual multileg instrument's EncodedLegFinancialInstrumentFullName(2719).*/
EncodedLegFinancialInstrumentFullName              = 2719, /*!< 2719 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegFinancialInstrumentFullName(2717) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) representation should also be specified in the LegFinancialInstrumentFullName(2717) field.*/
UnderlyingFinancialInstrumentFullName              = 2720, /*!< 2720 (String FIX.5.0SP2) The full normative name of the underlying financial instrument.*/
EncodedUnderlyingFinancialInstrumentFullNameLen    = 2721, /*!< 2721 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) underlying instrument's EncodedUnderlyingFinancialInstrumentFullName(2722).*/
EncodedUnderlyingFinancialInstrumentFullName       = 2722, /*!< 2722 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingFinancialInstrumentFullName(2720) field in the encoded format specified via the MessageEncoding(347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingFinancialInstrumentFullName(2720) field.*/
UnderlyingIndexCurveUnit                           = 2723, /*!< 2723 (String FIX.5.0SP2) Curve time unit associated with the underlying index.





*/
UnderlyingIndexCurvePeriod                         = 2724, /*!< 2724 (int FIX.5.0SP2) Curve time multiplier for the underlying index.*/
CommissionAmountSubType                            = 2725, /*!< 2725 (int FIX.5.0SP2) Further sub classification of the CommissionAmountType(2641).*/
AllocCommissionAmountSubType                       = 2726, /*!< 2726 (int FIX.5.0SP2) Further sub classification of the AllocCommissionAmountType(2655).*/
AllocLegRefID                                      = 2727, /*!< 2727 (String FIX.5.0SP2) Unique identifier for a specific leg (uniqueness not defined as part of the FIX specification). AllocLegRefID(2727tbd) references the value from LegID(1788) in the current multileg order or trade message specifying to which leg the allocation instance applies.*/
FloatingRateIndexCurvePeriod                       = 2728, /*!< 2728 (int FIX.5.0SP2) Time unit multiplier for the floating rate index identified in FloatingRateIndexID(2731).*/
FloatingRateIndexCurveSpread                       = 2729, /*!< 2729 (PriceOffset FIX.5.0SP2) Spread from the floating rate index.*/
FloatingRateIndexCurveUnit                         = 2730, /*!< 2730 (String FIX.5.0SP2) Time unit associated with the floating rate index identified in FloatingRateIndexID(2731).*/
FloatingRateIndexID                                = 2731, /*!< 2731 (String FIX.5.0SP2) Security identifier of the floating rate index.*/
FloatingRateIndexIDSource                          = 2732, /*!< 2732 (String FIX.5.0SP2) Source for the floating rate index identified in FloatingRateIndexID(2731).*/
IndexRollMonth                                     = 2733, /*!< 2733 (String FIX.5.0SP2) Month identified in the index roll.*/
NoIndexRollMonths                                  = 2734, /*!< 2734 (NumInGroup FIX.5.0SP2) Number of instances of the index roll month.*/
AssetSubType                                       = 2735, /*!< 2735 (String FIX.5.0SP2) Used to provide a more specific description of the asset specified in AssetType(1940). 

See https://www.fixtrading.org/codelists/AssetSubType for code list of applicable values.

*/
CommodityFinalPriceType                            = 2736, /*!< 2736 (int FIX.5.0SP2) Final price type of the commodity as specified by the trading venue.

*/
FinancialInstrumentShortName                       = 2737, /*!< 2737 (String FIX.5.0SP2) Short name of the financial instrument. Uses ISO 18774 (FINS) values. */
NextIndexRollDate                                  = 2738, /*!< 2738 (LocalMktDate FIX.5.0SP2) Next index roll date.*/
LegAssetSubType                                    = 2739, /*!< 2739 (String FIX.5.0SP2) Used to provide a more specific description of the asset specified in LegAssetType(2069). 

See https://www.fixtrading.org/codelists/AssetSubType for code list of applicable values. 

*/
LegFinancialInstrumentShortName                    = 2740, /*!< 2740 (String FIX.5.0SP2) Short name of the financial instrument. Uses ISO 18774 (FISN) values.*/
SecondaryAssetSubType                              = 2741, /*!< 2741 (String FIX.5.0SP2) Used to provide a more specific description of the asset specified in SecondaryAssetType(1979). 

See https://www.fixtrading.org/codelists/AssetSubType for code list of applicable values.*/
UnderlyingFinancialInstrumentShortName             = 2742, /*!< 2742 (String FIX.5.0SP2) Short name of the financial instrument. Uses ISO 18774 (FINS) values.*/
LegSecondaryAssetSubType                           = 2743, /*!< 2743 (String FIX.5.0SP2) Used to provide a more specific description of the asset specified in LegSecondaryAssetType(2079). 

See https://www.fixtrading.org/codelists/AssetSubType for code list of applicable values.

*/
UnderlyingAssetSubType                             = 2744, /*!< 2744 (String FIX.5.0SP2) Used to provide a more specific description of the asset specified in UnderlyingAssetType(2015). 

See https://www.fixtrading.org/codelists/AssetSubType for code list of applicable values.*/
UnderlyingSecondaryAssetSubType                    = 2745, /*!< 2745 (String FIX.5.0SP2) May be used to provide a more specific description of the asset specified in UnderlyingSecondaryAssetType(2083). 

See https://www.fixtrading.org/codelists/AssetSubType for code list of applicable values.*/
NoReferenceDataDates                               = 2746, /*!< 2746 (NumInGroup FIX.5.0SP2) Number of instances of reference data dates.*/
ReferenceDataDate                                  = 2747, /*!< 2747 (UTCTimestamp FIX.5.0SP2) Reference data entry's date-time of the type specified in ReferenceDataDateType(2748).*/
ReferenceDataDateType                              = 2748, /*!< 2748 (int FIX.5.0SP2) Reference data entry's date-time type.*/
ExecutionTimestamp                                 = 2749, /*!< 2749 (UTCTimestamp FIX.5.0SP2) Time of the individual execution.*/
ReportingPx                                        = 2750, /*!< 2750 (Price FIX.5.0SP2) Represents the reportable price on fill when an instance of the Parties component with PartyRole(452) = 73 (Execution Venue) is present to prevent having to compute running totals.*/
ReportingQty                                       = 2751, /*!< 2751 (Qty FIX.5.0SP2) Represents the reportable quantity on fill when an instance of the Parties component with PartyRole(452) = 73 (Execution Venue) is present to prevent having to compute running totals.*/
DeliveryRouteOrCharter                             = 2752, /*!< 2752 (String FIX.5.0SP2) Specific delivery route or time charter average. Applicable to commodity freight contracts.*/
ReturnTrigger                                      = 2753, /*!< 2753 (int FIX.5.0SP2) Indicates the type of return or payout trigger for the swap or forward.*/
LegDeliveryRouteOrCharter                          = 2754, /*!< 2754 (String FIX.5.0SP2) Specific delivery route or time charter average. Applicable to commodity freight contracts.*/
LegReturnTrigger                                   = 2755, /*!< 2755 (int FIX.5.0SP2) Indicates the type of return or payout trigger for the swap or forward.*/
UnderlyingDeliveryRouteOrCharter                   = 2756, /*!< 2756 (String FIX.5.0SP2) Specific delivery route or time charter average. Applicable to commodity freight contracts.*/
UnderlyingReturnTrigger                            = 2757, /*!< 2757 (int FIX.5.0SP2) Indicates the type of return or payout trigger for the swap or forward.*/
AllocRequestID                                     = 2758, /*!< 2758 (String FIX.5.0SP2) Unique identifier for the request message.*/
GroupAmount                                        = 2759, /*!< 2759 (Amt FIX.5.0SP2) Indicates the total notional units or amount of an allocation group.  Includes any allocated units or amount.*/
GroupRemainingAmount                               = 2760, /*!< 2760 (Amt FIX.5.0SP2) Indicates the remaining notional units or amount of an allocation group that has not yet been allocated.*/
AllocGroupAmount                                   = 2761, /*!< 2761 (Amt FIX.5.0SP2) Indicates the notional units or amount being allocated.*/
PriceMarkup                                        = 2762, /*!< 2762 (PriceOffset FIX.5.0SP2) Price offset of the markup denominated in the price type of the trade.*/
AveragePriceType                                   = 2763, /*!< 2763 (int FIX.5.0SP2) The average pricing model used for block trades.*/
AveragePriceStartTime                              = 2764, /*!< 2764 (UTCTimestamp FIX.5.0SP2) Start of the time period during which price averaging occurred.*/
AveragePriceEndTime                                = 2765, /*!< 2765 (UTCTimestamp FIX.5.0SP2) End of the time period during which price averaging occurred.*/
OrderPercentOfTotalVolume                          = 2766, /*!< 2766 (Percentage FIX.5.0SP2) For Percent-of-volume (POV) average pricing this is the target percentage this order quantity represents of the total trading volume of an instrument during the specified time period. This provides the data needed to ensure that the average price is fair based on the total sum of grouped POV trades. */
AllocGroupStatus                                   = 2767, /*!< 2767 (int FIX.5.0SP2) Status of the trade give-up relative to the group identified in AllocGroupID(1730).*/
NoAdditionalTermBondRefs                           = 40000, /*!< 40000 (NumInGroup FIX.5.0SP2) Number of bonds in the repeating group.*/
AdditionalTermBondSecurityID                       = 40001, /*!< 40001 (String FIX.5.0SP2) Security identifier of the bond.  */
AdditionalTermBondSecurityIDSource                 = 40002, /*!< 40002 (String FIX.5.0SP2) Identifies the source scheme of the AdditionalTermBondSecurityID(40001) value.  */
AdditionalTermBondDesc                             = 40003, /*!< 40003 (String FIX.5.0SP2) Description of the bond.*/
EncodedAdditionalTermBondDescLen                   = 40004, /*!< 40004 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedAdditionalTermBondDesc(40005) field.*/
EncodedAdditionalTermBondDesc                      = 40005, /*!< 40005 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the AdditionalTermBondDesc(40003) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the AdditionalTermBondDesc(40003) field.*/
AdditionalTermBondCurrency                         = 40006, /*!< 40006 (Currency FIX.5.0SP2) Specifies the currency the bond value is denominated in.  Uses ISO 4217 currency codes.*/
AdditionalTermBondIssuer                           = 40007, /*!< 40007 (String FIX.5.0SP2) Issuer of the bond.*/
EncodedAdditionalTermBondIssuerLen                 = 40008, /*!< 40008 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedAdditionalTermBondIssuer(40009) field.*/
EncodedAdditionalTermBondIssuer                    = 40009, /*!< 40009 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the AdditionalTermBondIssuer(40007) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the AdditionalTermBondIssuer(40007) field.*/
AdditionalTermBondSeniority                        = 40010, /*!< 40010 (String FIX.5.0SP2) Specifies the bond's payment priority in the event of a default.*/
AdditionalTermBondCouponType                       = 40011, /*!< 40011 (int FIX.5.0SP2) Coupon type of the bond.*/
AdditionalTermBondCouponRate                       = 40012, /*!< 40012 (Percentage FIX.5.0SP2) Coupon rate of the bond.  See also CouponRate(223).*/
AdditionalTermBondMaturityDate                     = 40013, /*!< 40013 (LocalMktDate FIX.5.0SP2) The maturity date of the bond.*/
AdditionalTermBondParValue                         = 40014, /*!< 40014 (Amt FIX.5.0SP2) The par value of the bond.*/
AdditionalTermBondCurrentTotalIssuedAmount         = 40015, /*!< 40015 (Amt FIX.5.0SP2) Total issued amount of the bond.*/
AdditionalTermBondCouponFrequencyPeriod            = 40016, /*!< 40016 (int FIX.5.0SP2) Time unit multiplier for the frequency of the bond's coupon payment. 			*/
AdditionalTermBondCouponFrequencyUnit              = 40017, /*!< 40017 (String FIX.5.0SP2) Time unit associated with the frequency of the bond's coupon payment. */
AdditionalTermBondDayCount                         = 40018, /*!< 40018 (int FIX.5.0SP2) The day count convention used in interest calculations for a bond or an interest bearing security.*/
NoAdditionalTerms                                  = 40019, /*!< 40019 (NumInGroup FIX.5.0SP2) Number of additional terms in the repeating group.*/
AdditionalTermConditionPrecedentBondIndicator      = 40020, /*!< 40020 (Boolean FIX.5.0SP2) Indicates whether the condition precedent bond is applicable. The swap contract is only valid if the bond is issued and if there is any dispute over the terms of fixed stream then the bond terms would be used.*/
AdditionalTermDiscrepancyClauseIndicator           = 40021, /*!< 40021 (Boolean FIX.5.0SP2) Indicates whether the discrepancy clause is applicable.*/
NoCashSettlTerms                                   = 40022, /*!< 40022 (NumInGroup FIX.5.0SP2) Number of elements in the repeating group.*/
CashSettlCurrency                                  = 40023, /*!< 40023 (Currency FIX.5.0SP2) Specifies the currency the CashSettlAmount(40034) is denominated in.  Uses ISO 4217 currency codes.*/
CashSettlValuationFirstBusinessDayOffset           = 40024, /*!< 40024 (int FIX.5.0SP2) The number of business days after settlement conditions have been satisfied, when the calculation agent is to obtain a price quotation on the reference obligation for the purpose of cash settlement.*/
CashSettlValuationTime                             = 40025, /*!< 40025 (LocalMktTime FIX.5.0SP2) The time of valuation.*/
CashSettlBusinessCenter                            = 40026, /*!< 40026 (String FIX.5.0SP2) Identifies the business center calendar used at valuation time for cash settlement purposes e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
CashSettlQuoteMethod                               = 40027, /*!< 40027 (int FIX.5.0SP2) The type of quote used to determine the cash settlement price.*/
CashSettlQuoteAmount                               = 40028, /*!< 40028 (Amt FIX.5.0SP2) When determining the cash settlement amount, if weighted average price quotes are to be obtained for the reference obligation, this is the upper limit to the outstanding principal balance of the reference obligation for which the quote should be obtained. If not specifed, the ISDA definitions provide for a fallback amount equal to floating rate payer calculation amount.*/
CashSettlQuoteCurrency                             = 40029, /*!< 40029 (Currency FIX.5.0SP2) Specifies the currency the CashSettlQuoteAmount(40028) is denominated in.  Uses ISO 4217 Currency Code.*/
CashSettlMinimumQuoteAmount                        = 40030, /*!< 40030 (Amt FIX.5.0SP2) When determining the cash settlement amount, if weighted average price quotes are to be obtained for the reference obligation, this is the minimum intended threshold amount of outstanding principal balance of the reference obligation for which the quote should be obtained.  If not specified, the ISDA definitions provide for a fallback amount of the lower of either USD1,000,000 (or its equivalent in the relevent obligation currency) or the (minimum) quoted amount.*/
CashSettlMinimumQuoteCurrency                      = 40031, /*!< 40031 (Currency FIX.5.0SP2) Specifies the currency the CashSettlMinimumQuoteAmount(40030) is denominated in.  Uses ISO 4217 Currency Code.*/
CashSettlDealer                                    = 40032, /*!< 40032 (String FIX.5.0SP2) Identifies the dealer from whom price quotations for the reference obligation are obtained for the purpose of cash settlement valuation calculation.*/
CashSettlBusinessDays                              = 40033, /*!< 40033 (int FIX.5.0SP2) The number of business days used in the determination of the cash settlement payment date. */
CashSettlAmount                                    = 40034, /*!< 40034 (Amt FIX.5.0SP2) The amount paid between the trade parties, seller to the buyer, for cash settlement on the cash settlement date.*/
CashSettlRecoveryFactor                            = 40035, /*!< 40035 (float FIX.5.0SP2) Used for fixed recovery, this specifies the recovery level as determined at contract inception, to be applied in the event of a default. The factor is used to calculate the amount paid by the seller to the buyer for cash settlement on the cash settlement date. The amount calculated is (1 - CashSettlRecoveryFactor(40035)) x floating rate payer calculation amount. The currency is derived from the floating rate payer calculation amount.*/
CashSettlFixedTermIndicator                        = 40036, /*!< 40036 (Boolean FIX.5.0SP2) Indicates whether fixed settlement is applicable or not applicable in a recovery lock.*/
CashSettlAccruedInterestIndicator                  = 40037, /*!< 40037 (Boolean FIX.5.0SP2) Indicates whether accrued interest is included or not in the value provided in CashSettlAmount(40034).  For cash settlement this specifies whether quotations should be obtained inclusive or not of accrued interest. 

For physical settlement this specifies whether the buyer should deliver the obligation with an outstanding principal balance that includes or excludes accrued interest. */
CashSettlValuationMethod                           = 40038, /*!< 40038 (int FIX.5.0SP2) The ISDA defined methodology for determining the final price of the reference obligation for purposes of cash settlement.*/
CashSettlTermXID                                   = 40039, /*!< 40039 (XID FIX.5.0SP2) A named string value referenced by UnderlyingSettlTermXIDRef(41315). */
NoContractualDefinitions                           = 40040, /*!< 40040 (NumInGroup FIX.5.0SP2) Number of financing definitions in the repeating group.*/
ContractualDefinition                              = 40041, /*!< 40041 (String FIX.5.0SP2) Specifies which contract definition, such as those published by ISDA, will apply for the terms of the trade. See http://www.fpml.org/coding-scheme/contractual-definitions for values.*/
NoContractualMatrices                              = 40042, /*!< 40042 (NumInGroup FIX.5.0SP2) Number of contractual matrices in the repeating group.*/
ContractualMatrixSource                            = 40043, /*!< 40043 (String FIX.5.0SP2) Identifies the applicable contract matrix. See http://www.fpml.org/coding-scheme/matrix-type-1-0.xml for values.*/
ContractualMatrixDate                              = 40044, /*!< 40044 (LocalMktDate FIX.5.0SP2) The publication date of the applicable version of the contract matrix. If not specified, the ISDA Standard Terms Supplement defines rules for which version of the matrix is applicable.*/
ContractualMatrixTerm                              = 40045, /*!< 40045 (String FIX.5.0SP2) Specifies the applicable key into the relevent contract matrix.  In the case of 2000 ISDA Definitions Settlement Matrix for Early Termination and Swaptions, the ContractualMatrixTerm(40045) is not applicable and is to be omitted. See http://www.fpml.org/coding-scheme/credit-matrix-transaction-type for values.*/
NoFinancingTermSupplements                         = 40046, /*!< 40046 (NumInGroup FIX.5.0SP2) Number of financing terms supplements in the repeating group.*/
FinancingTermSupplementDesc                        = 40047, /*!< 40047 (String FIX.5.0SP2) Identifies the applicable contractual supplement. See http://www.fpml.org/coding-scheme/contractual-supplement for values.*/
FinancingTermSupplementDate                        = 40048, /*!< 40048 (LocalMktDate FIX.5.0SP2) The publication date of the applicable version of the contractual supplement.*/
NoStreams                                          = 40049, /*!< 40049 (NumInGroup FIX.5.0SP2) Number of swap streams in the repeating group.*/
StreamType                                         = 40050, /*!< 40050 (int FIX.5.0SP2) Type of swap stream.*/
StreamDesc                                         = 40051, /*!< 40051 (String FIX.5.0SP2) A short descriptive name given to the payment stream. Eg. CDS, Fixed, Float, Float2, GBP. The description has no intrinsic meaning but should be arbitrarily chosen by the remitter as reference.*/
StreamPaySide                                      = 40052, /*!< 40052 (int FIX.5.0SP2) The side of the party paying the stream.*/
StreamReceiveSide                                  = 40053, /*!< 40053 (int FIX.5.0SP2) The side of the party receiving the stream.*/
StreamNotional                                     = 40054, /*!< 40054 (Amt FIX.5.0SP2) Notional, or initial notional value for the payment stream. Use the PaymentScheduleGrp component to specify the rate steps. */
StreamCurrency                                     = 40055, /*!< 40055 (Currency FIX.5.0SP2) Specifies the currency the StreamNotional(40054) is denominated in.  Uses ISO 4217 currency codes.*/
StreamText                                         = 40056, /*!< 40056 (String FIX.5.0SP2) Free form text to specify additional information or enumeration description when a standard value does not apply.*/
UnderlyingStreamEffectiveDateUnadjusted            = 40057, /*!< 40057 (LocalMktDate FIX.5.0SP2) The unadjusted effective date. */
UnderlyingStreamEffectiveDateBusinessDayConvention = 40058, /*!< 40058 (int FIX.5.0SP2) The business day convention used to adjust the underlying instrument's stream's effective, or relative effective, date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingStreamEffectiveDateBusinessCenter        = 40059, /*!< 40059 (String FIX.5.0SP2) The business center calendar used to adjust the underlying instrument's stream's effective, or relative effective, date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
UnderlyingStreamEffectiveDateRelativeTo            = 40060, /*!< 40060 (int FIX.5.0SP2) Specifies the anchor date when the effective date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingStreamEffectiveDateOffsetPeriod          = 40061, /*!< 40061 (int FIX.5.0SP2) Time unit multiplier for the relative effective date offset.*/
UnderlyingStreamEffectiveDateOffsetUnit            = 40062, /*!< 40062 (String FIX.5.0SP2) Time unit associated with the relative effective date offset.*/
UnderlyingStreamEffectiveDateOffsetDayType         = 40063, /*!< 40063 (int FIX.5.0SP2) Specifies the day type of the relative effective date offset.*/
UnderlyingStreamEffectiveDateAdjusted              = 40064, /*!< 40064 (LocalMktDate FIX.5.0SP2) The adjusted effective date. */
StreamTerminationDateUnadjusted                    = 40065, /*!< 40065 (LocalMktDate FIX.5.0SP2) The unadjusted termination date.*/
StreamTerminationDateBusinessDayConvention         = 40066, /*!< 40066 (int FIX.5.0SP2) The business day convention used to adjust the instrument's stream's termination, or relative termination, date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
StreamTerminationDateBusinessCenter                = 40067, /*!< 40067 (String FIX.5.0SP2) The business center calendar used to adjust the instrument's stream's termination, or relative termination, date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
StreamTerminationDateRelativeTo                    = 40068, /*!< 40068 (int FIX.5.0SP2) Specifies the anchor date when the termination date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
StreamTerminationDateOffsetPeriod                  = 40069, /*!< 40069 (int FIX.5.0SP2) Time unit multiplier for the relative termination date offset.*/
StreamTerminationDateOffsetUnit                    = 40070, /*!< 40070 (String FIX.5.0SP2) Time unit associated with the relative termination date offset.*/
StreamTerminationDateOffsetDayType                 = 40071, /*!< 40071 (int FIX.5.0SP2) Specifies the day type of the relative termination date offset.*/
StreamTerminationDateAdjusted                      = 40072, /*!< 40072 (LocalMktDate FIX.5.0SP2) The adjusted termination date.*/
StreamCalculationPeriodBusinessDayConvention       = 40073, /*!< 40073 (int FIX.5.0SP2) The business day convention used to adjust calculation periods. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
StreamCalculationPeriodBusinessCenter              = 40074, /*!< 40074 (String FIX.5.0SP2) The business center calendar used to adjust calculation periods, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
StreamFirstPeriodStartDateUnadjusted               = 40075, /*!< 40075 (LocalMktDate FIX.5.0SP2) The unadjusted first calculation period start date if before the effective date.*/
StreamFirstPeriodStartDateBusinessDayConvention    = 40076, /*!< 40076 (int FIX.5.0SP2) The business day convention used to adjust the instrument's stream's first calculation period start date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
StreamFirstPeriodStartDateBusinessCenter           = 40077, /*!< 40077 (String FIX.5.0SP2) The business center calendar used to adjust the instrument's stream's first calculation period start date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
StreamFirstPeriodStartDateAdjusted                 = 40078, /*!< 40078 (LocalMktDate FIX.5.0SP2) The adjusted first calculation period start date, if it is before the effective date.*/
StreamFirstRegularPeriodStartDateUnadjusted        = 40079, /*!< 40079 (LocalMktDate FIX.5.0SP2) The unadjusted first start date of the regular calculation period, if there is an initial stub period.*/
StreamFirstCompoundingPeriodEndDateUnadjusted      = 40080, /*!< 40080 (LocalMktDate FIX.5.0SP2) The unadjusted end date of the initial compounding period.*/
StreamLastRegularPeriodEndDateUnadjusted           = 40081, /*!< 40081 (LocalMktDate FIX.5.0SP2) The unadjusted last regular period end date if there is a final stub period.*/
StreamCalculationFrequencyPeriod                   = 40082, /*!< 40082 (int FIX.5.0SP2) Time unit multiplier for the frequency at which calculation period end dates occur.*/
StreamCalculationFrequencyUnit                     = 40083, /*!< 40083 (String FIX.5.0SP2) Time unit associated with the frequency at which calculation period end dates occur.*/
StreamCalculationRollConvention                    = 40084, /*!< 40084 (String FIX.5.0SP2) The convention for determining the sequence of end dates. It is used in conjunction with a specified frequency. Used only to override the roll convention specified in the DateAdjustment component within the Instrument component.*/
NoSettlRateFallbacks                               = 40085, /*!< 40085 (NumInGroup FIX.5.0SP2) Number of settlement rate fallbacks in the repeating group*/
SettlRatePostponementMaximumDays                   = 40086, /*!< 40086 (int FIX.5.0SP2) The maximum number of days to wait for a quote from the disrupted settlement rate option before proceding to this method.*/
LegPaymentStreamNonDeliverableSettlRateSource      = 40087, /*!< 40087 (int FIX.5.0SP2) Identifies the source of the rate information.*/
SettlRatePostponementSurvey                        = 40088, /*!< 40088 (Boolean FIX.5.0SP2) Indicates whether to request a settlement rate quote from the market.*/
SettlRatePostponementCalculationAgent              = 40089, /*!< 40089 (int FIX.5.0SP2) Used to identify the settlement rate postponement calculation agent.*/
NoProvisions                                       = 40090, /*!< 40090 (NumInGroup FIX.5.0SP2) Number of provisions in the repeating group.*/
ProvisionType                                      = 40091, /*!< 40091 (int FIX.5.0SP2) Type of provisions.*/
ProvisionDateUnadjusted                            = 40092, /*!< 40092 (LocalMktDate FIX.5.0SP2) The unadjusted date of the provision.*/
ProvisionDateBusinessDayConvention                 = 40093, /*!< 40093 (int FIX.5.0SP2) The business day convention used to adjust the instrument's provision's dates. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
ProvisionDateBusinessCenter                        = 40094, /*!< 40094 (String FIX.5.0SP2) The business center calendar used to adjust the instrument's provision's dates, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionDateAdjusted                              = 40095, /*!< 40095 (LocalMktDate FIX.5.0SP2) The adjusted date of the provision.*/
ProvisionDateTenorPeriod                           = 40096, /*!< 40096 (int FIX.5.0SP2) Time unit multiplier for the provision's tenor period.*/
ProvisionDateTenorUnit                             = 40097, /*!< 40097 (String FIX.5.0SP2) Time unit associated with the provision's tenor period.*/
ProvisionCalculationAgent                          = 40098, /*!< 40098 (int FIX.5.0SP2) Used to identify the calculation agent.  The calculation agent may be identified in ProvisionCalculationAgent(40098) or in the ProvisionParties component.*/
ProvisionOptionSinglePartyBuyerSide                = 40099, /*!< 40099 (int FIX.5.0SP2) If optional early termination is not available to both parties then this component identifies the buyer of the option through its side of the trade.*/
ProvisionOptionSinglePartySellerSide               = 40100, /*!< 40100 (int FIX.5.0SP2) If optional early termination is not available to both parties then this component identifies the seller of the option through its side of the trade.*/
ProvisionOptionExerciseStyle                       = 40101, /*!< 40101 (int FIX.5.0SP2) The instrument provision option’s exercise style.*/
ProvisionOptionExerciseMultipleNotional            = 40102, /*!< 40102 (Amt FIX.5.0SP2) A notional amount which restricts the amount of notional that can be exercised when partial exercise or multiple exercise is applicable. The integral multiple amount defines a lower limit of notional that can be exercised and also defines a unit multiple of notional that can be exercised, i.e. only integer multiples of this amount can be exercised.*/
ProvisionOptionExerciseMinimumNotional             = 40103, /*!< 40103 (Amt FIX.5.0SP2) The minimum notional amount that can be exercised on a given exercise date.*/
ProvisionOptionExerciseMaximumNotional             = 40104, /*!< 40104 (Amt FIX.5.0SP2) The maximum notional amount that can be exercised on a given exercise date.*/
ProvisionOptionMinimumNumber                       = 40105, /*!< 40105 (int FIX.5.0SP2) The minimum number of options that can be exercised on a given exercise date.*/
ProvisionOptionMaximumNumber                       = 40106, /*!< 40106 (int FIX.5.0SP2) The maximum number of options that can be exercised on a given exercise date. If the number is not specified, it means that the maximum number of options corresponds to the remaining unexercised options.*/
ProvisionOptionExerciseConfirmation                = 40107, /*!< 40107 (Boolean FIX.5.0SP2) Used to indicate whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller's agent.*/
ProvisionCashSettlMethod                           = 40108, /*!< 40108 (int FIX.5.0SP2) An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (e).*/
ProvisionCashSettlCurrency                         = 40109, /*!< 40109 (Currency FIX.5.0SP2) Specifies the currency of settlement.  Uses ISO 4217 currency codes.*/
ProvisionCashSettlCurrency2                        = 40110, /*!< 40110 (Currency FIX.5.0SP2) Specifies the currency of settlement for a cross-currency provision.  Uses ISO 4217 currency codes.*/
ProvisionCashSettlQuoteType                        = 40111, /*!< 40111 (int FIX.5.0SP2) Identifies the type of quote to be used. */
ProvisionCashSettlQuoteSource                      = 40112, /*!< 40112 (int FIX.5.0SP2) Identifies the source of quote information. */
ProvisionText                                      = 40113, /*!< 40113 (String FIX.5.0SP2) Free form text to specify additional information or  enumeration description when a standard value does not apply.*/
ProvisionCashSettlValueTime                        = 40114, /*!< 40114 (LocalMktTime FIX.5.0SP2) A time specified in 24-hour format, e.g. 11am would be represented as 11:00:00. The time of the cash settlement valuation date when the cash settlement amount will be determined according to the cash settlement method if the parties have not otherwise been able to agree to the cash settlement amount.*/
ProvisionCashSettlValueTimeBusinessCenter          = 40115, /*!< 40115 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's cash settlement valuation time. See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionCashSettlValueDateBusinessDayConvention   = 40116, /*!< 40116 (int FIX.5.0SP2) The cash settlement valuation date adjustment business day convention. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
ProvisionCashSettlValueDateBusinessCenter          = 40117, /*!< 40117 (String FIX.5.0SP2) The business center calendar used to adjust the provision's cash settlement valuation date, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionCashSettlValueDateRelativeTo              = 40118, /*!< 40118 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement value date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values*/
ProvisionCashSettlValueDateOffsetPeriod            = 40119, /*!< 40119 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement value date offset.*/
ProvisionCashSettlValueDateOffsetUnit              = 40120, /*!< 40120 (String FIX.5.0SP2) Time unit associated with the relative cash settlement value date offset.*/
ProvisionCashSettlValueDateOffsetDayType           = 40121, /*!< 40121 (int FIX.5.0SP2) Specifies the day type of the provision's relative cash settlement value date offset.*/
ProvisionCashSettlValueDateAdjusted                = 40122, /*!< 40122 (LocalMktDate FIX.5.0SP2) The adjusted cash settlement value date.*/
ProvisionOptionExerciseBusinessDayConvention       = 40123, /*!< 40123 (int FIX.5.0SP2) The business day convention used to adjust the instrument's provision's option exercise date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
ProvisionOptionExerciseBusinessCenter              = 40124, /*!< 40124 (String FIX.5.0SP2) The business center calendar used to adjust the instrument's provision's option exercise date, e.g. "GBLO".

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionOptionExerciseEarliestDateOffsetPeriod    = 40125, /*!< 40125 (int FIX.5.0SP2) Time unit multiplier for the interval to the first (and possibly only) exercise date in the exercise period.*/
ProvisionOptionExerciseEarliestDateOffsetUnit      = 40126, /*!< 40126 (String FIX.5.0SP2) Time unit associated with the interval to the first (and possibly only) exercise date in the exercise period.*/
ProvisionOptionExerciseFrequencyPeriod             = 40127, /*!< 40127 (int FIX.5.0SP2) Time unit multiplier for the frequency of subsequent exercise dates in the exercise period following the earliest exercise date. An interval of 1 day should be used to indicate an American style exercise period.*/
ProvisionOptionExerciseFrequencyUnit               = 40128, /*!< 40128 (String FIX.5.0SP2) Time unit associated with the frequency of subsequent exercise dates in the exercise period following the earliest exercise date.*/
ProvisionOptionExerciseStartDateUnadjusted         = 40129, /*!< 40129 (LocalMktDate FIX.5.0SP2) The unadjusted first day of the exercise period for an American style option.*/
ProvisionOptionExerciseStartDateRelativeTo         = 40130, /*!< 40130 (int FIX.5.0SP2) Specifies the anchor date when the option exercise start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
ProvisionOptionExerciseStartDateOffsetPeriod       = 40131, /*!< 40131 (int FIX.5.0SP2) Time unit multiplier for the relative option exercise start date offset.*/
ProvisionOptionExerciseStartDateOffsetUnit         = 40132, /*!< 40132 (String FIX.5.0SP2) Time unit associated with the relative option exercise start date offset.*/
ProvisionOptionExerciseStartDateOffsetDayType      = 40133, /*!< 40133 (int FIX.5.0SP2) Specifies the day type of the provision's relative option exercise start date offset.*/
ProvisionOptionExerciseStartDateAdjusted           = 40134, /*!< 40134 (LocalMktDate FIX.5.0SP2) The adjusted first day of the exercise period for an American style option.*/
ProvisionOptionExercisePeriodSkip                  = 40135, /*!< 40135 (int FIX.5.0SP2) The number of periods in the referenced date schedule that are between each date in the relative date schedule. Thus a skip of 2 would mean that dates are relative to every second date in the referenced schedule. If present this should have a value greater than 1.*/
ProvisionOptionExerciseBoundsFirstDateUnadjusted   = 40136, /*!< 40136 (LocalMktDate FIX.5.0SP2) The unadjusted first date of a schedule. This can be used to restrict the range of exercise dates when they are relative.*/
ProvisionOptionExerciseBoundsLastDateUnadjusted    = 40137, /*!< 40137 (LocalMktDate FIX.5.0SP2) The unadjusted last date of a schedule. This can be used to restrict the range of exercise dates when they are relative.*/
ProvisionOptionExerciseEarliestTime                = 40138, /*!< 40138 (LocalMktTime FIX.5.0SP2) The earliest time at which notice of exercise can be given by the buyer to the seller (or seller's agent) i) on the expriation date, in the case of a European style option, (ii) on each bermuda option exercise date and the expiration date, in the case of a Bermuda style option the commencement date to, and including, the expiration date, in the case of an American option.*/
ProvisionOptionExerciseEarliestTimeBusinessCenter  = 40139, /*!< 40139 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's earliest time for notice of exercise. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionOptionExerciseLatestTime                  = 40140, /*!< 40140 (LocalMktTime FIX.5.0SP2) For a Bermuda or American style option, the latest time on an exercise business day (excluding the expiration date) within the exercise period that notice can be given by the buyer to the seller or seller's agent. Notice of exercise given after this time will be deemed to have been given on the next exercise business day.*/
ProvisionOptionExerciseLatestTimeBusinessCenter    = 40141, /*!< 40141 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's latest time for notice of exercise. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoProvisionOptionExerciseFixedDates                = 40142, /*!< 40142 (NumInGroup FIX.5.0SP2) Number of provision option exercise fixed dates in the repeating group.*/
ProvisionOptionExerciseFixedDate                   = 40143, /*!< 40143 (LocalMktDate FIX.5.0SP2) A predetermined option exercise date, unadjusted or adjusted depending on ProvisionOptionExerciseFixedDateType(40144).*/
ProvisionOptionExerciseFixedDateType               = 40144, /*!< 40144 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
ProvisionOptionExpirationDateUnadjusted            = 40145, /*!< 40145 (LocalMktDate FIX.5.0SP2) The unadjusted last day within an exercise period for an American style option. For a European style option it is the only day within the exercise period.*/
ProvisionOptionExpirationDateBusinessDayConvention = 40146, /*!< 40146 (int FIX.5.0SP2) The business day convention used to adjust the instrument's provision's option expiration date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
ProvisionOptionExpirationDateBusinessCenter        = 40147, /*!< 40147 (String FIX.5.0SP2) The business center calendar used to adjust the instrument's provision's option expiration date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionOptionExpirationDateRelativeTo            = 40148, /*!< 40148 (int FIX.5.0SP2) Specifies the anchor date when the option expiration date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
ProvisionOptionExpirationDateOffsetPeriod          = 40149, /*!< 40149 (int FIX.5.0SP2) Time unit multiplier for the relative option expiration date offset.*/
ProvisionOptionExpirationDateOffsetUnit            = 40150, /*!< 40150 (String FIX.5.0SP2) Time unit associated with the relative option expiration date offset.*/
ProvisionOptionExpirationDateOffsetDayType         = 40151, /*!< 40151 (int FIX.5.0SP2) Specifies the day type of the provision's relative option expiration date offset.*/
ProvisionOptionExpirationDateAdjusted              = 40152, /*!< 40152 (LocalMktDate FIX.5.0SP2) The adjusted last date within an exercise period for an American style option. For a European style option it is the only date within the exercise period.*/
ProvisionOptionExpirationTime                      = 40153, /*!< 40153 (LocalMktTime FIX.5.0SP2) The latest time for exercise on the expiration date.*/
ProvisionOptionExpirationTimeBusinessCenter        = 40154, /*!< 40154 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's latest exercise time on expiration date. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionOptionRelevantUnderlyingDateUnadjusted    = 40155, /*!< 40155 (LocalMktDate FIX.5.0SP2) The unadjusted date on the underlying set by the exercise of an option. What this date is depends on the option (e.g. in a swaption it is the swap effective date, in an extendible/cancelable provision it is the swap termination date).*/
ProvisionOptionRelevantUnderlyingDateBusinessDayConvention = 40156, /*!< 40156 (int FIX.5.0SP2) The business day convention used to adjust the instrument's provision's option underlying date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
ProvisionOptionRelevantUnderlyingDateBusinessCenter = 40157, /*!< 40157 (String FIX.5.0SP2) The business center calendar used to adjust the instrument's provision's option underlying date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionOptionRelevantUnderlyingDateRelativeTo    = 40158, /*!< 40158 (int FIX.5.0SP2) Specifies the anchor date when the date relevant to the underlying trade on exercise is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
ProvisionOptionRelevantUnderlyingDateOffsetPeriod  = 40159, /*!< 40159 (int FIX.5.0SP2) Time unit multiplier for the relative option relevant underlying date offset.*/
ProvisionOptionRelevantUnderlyingDateOffsetUnit    = 40160, /*!< 40160 (String FIX.5.0SP2) Time unit associated with the relative option relevant underlying date offset.*/
ProvisionOptionRelevantUnderlyingDateOffsetDayType = 40161, /*!< 40161 (int FIX.5.0SP2) Specifies the day type of the provision's relative option relevant underlying date offset.*/
ProvisionOptionRelevantUnderlyingDateAdjusted      = 40162, /*!< 40162 (LocalMktDate FIX.5.0SP2) The adjusted date on the underlying set by the exercise of an option. What this date is depends on the option (e.g. in a swaption it is the swap effective date, in an extendible/cancelable provision it is the swap termination date).*/
ProvisionCashSettlPaymentDateBusinessDayConvention = 40163, /*!< 40163 (int FIX.5.0SP2) The business day convention used to adjust the provisional cash settlement payment's termination or relative termination date.  Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
ProvisionCashSettlPaymentDateBusinessCenter        = 40164, /*!< 40164 (String FIX.5.0SP2) The business center calendar used to adjust the provisional cash settlement payment's termination or relative termination date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionCashSettlPaymentDateRelativeTo            = 40165, /*!< 40165 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
ProvisionCashSettlPaymentDateOffsetPeriod          = 40166, /*!< 40166 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement payment date offset.*/
ProvisionCashSettlPaymentDateOffsetUnit            = 40167, /*!< 40167 (String FIX.5.0SP2) Time unit associated with the relative cash settlement payment date offset.*/
ProvisionCashSettlPaymentDateOffsetDayType         = 40168, /*!< 40168 (int FIX.5.0SP2) Specifies the day type of the provision's relative cash settlement payment date offset.*/
ProvisionCashSettlPaymentDateRangeFirst            = 40169, /*!< 40169 (LocalMktDate FIX.5.0SP2) First date in range when a settlement date range is provided.*/
ProvisionCashSettlPaymentDateRangeLast             = 40170, /*!< 40170 (LocalMktDate FIX.5.0SP2) The last date in range when a settlement date range is provided.*/
NoProvisionCashSettlPaymentDates                   = 40171, /*!< 40171 (NumInGroup FIX.5.0SP2) Number of provision cash settlement payment dates in the repeating group.*/
ProvisionCashSettlPaymentDate                      = 40172, /*!< 40172 (LocalMktDate FIX.5.0SP2) The cash settlement payment date, unadjusted or adjusted depending on ProvisionCashSettlPaymentDateType(40173).*/
ProvisionCashSettlPaymentDateType                  = 40173, /*!< 40173 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
NoProvisionPartyIDs                                = 40174, /*!< 40174 (NumInGroup FIX.5.0SP2) Number of parties identified in the contract provision.*/
ProvisionPartyID                                   = 40175, /*!< 40175 (String FIX.5.0SP2) The party identifier/code for the payment settlement party. */
ProvisionPartyIDSource                             = 40176, /*!< 40176 (char FIX.5.0SP2) Identifies class or source of the ProvisionPartyID(40175) value. */
ProvisionPartyRole                                 = 40177, /*!< 40177 (int FIX.5.0SP2) Identifies the type or role of ProvisionPartyID(40175) specified. */
NoProvisionPartySubIDs                             = 40178, /*!< 40178 (NumInGroup FIX.5.0SP2) Number of sub-party IDs to be reported for the party.*/
ProvisionPartySubID                                = 40179, /*!< 40179 (String FIX.5.0SP2) Party sub-identifier, if applicable, for ProvisionPartyID(40175). */
ProvisionPartySubIDType                            = 40180, /*!< 40180 (int FIX.5.0SP2) The type of ProvisionPartySubID(40179). */
NoProtectionTerms                                  = 40181, /*!< 40181 (NumInGroup FIX.5.0SP2) Number of protection terms in the repeating group.*/
ProtectionTermNotional                             = 40182, /*!< 40182 (Amt FIX.5.0SP2) The notional amount of protection coverage. */
ProtectionTermCurrency                             = 40183, /*!< 40183 (Currency FIX.5.0SP2) The currency of ProtectionTermNotional(40182).  Uses ISO 4217 currency codes.*/
ProtectionTermSellerNotifies                       = 40184, /*!< 40184 (Boolean FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring. 

ProtectionTermSellerNotifies(40184)=Y indicates that the seller notifies.*/
ProtectionTermBuyerNotifies                        = 40185, /*!< 40185 (Boolean FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring. 

ProtectionTermBuyerNotifies(40185)=Y indicates that the buyer notifies. */
ProtectionTermEventBusinessCenter                  = 40186, /*!< 40186 (String FIX.5.0SP2) When used, the business center indicates the local time of the business center that replaces the Greenwich Mean Time  in Section 3.3 of the 2003 ISDA Credit Derivatives Definitions. See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProtectionTermStandardSources                      = 40187, /*!< 40187 (Boolean FIX.5.0SP2) Indicates whether ISDA defined Standard Public Sources are applicable (ProtectionTermStandardSources(40187)=Y) or not.*/
ProtectionTermEventMinimumSources                  = 40188, /*!< 40188 (int FIX.5.0SP2) The minimum number of the specified public information sources that must publish information that reasonably confirms that a credit event has occurred. The market convention is two. */
ProtectionTermEventNewsSource                      = 40189, /*!< 40189 (String FIX.5.0SP2) Newspaper or electronic news service or source that may publish relevant information used in the determination of whether or not a credit event has occurred.*/
ProtectionTermXID                                  = 40190, /*!< 40190 (XID FIX.5.0SP2) A named string value referenced by UnderlyingProtectionTermXIDRef(41314).*/
NoProtectionTermEvents                             = 40191, /*!< 40191 (NumInGroup FIX.5.0SP2) Number of protection term events in the repeating group.*/
ProtectionTermEventType                            = 40192, /*!< 40192 (String FIX.5.0SP2) Specifies the type of credit event applicable to the protection terms. 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Event_Types for code list of applicable event types. */
ProtectionTermEventValue                           = 40193, /*!< 40193 (String FIX.5.0SP2) Protection term event value appropriate to ProtectionTermEvenType(40192). 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Event_Types for applicable event type values. */
ProtectionTermEventCurrency                        = 40194, /*!< 40194 (Currency FIX.5.0SP2) Applicable currency if ProtectionTermEventValue(40193) is an amount.  Uses ISO 4217 currency codes.*/
ProtectionTermEventPeriod                          = 40195, /*!< 40195 (int FIX.5.0SP2) Time unit multiplier for protection term events.*/
ProtectionTermEventUnit                            = 40196, /*!< 40196 (String FIX.5.0SP2) Time unit associated with protection term events.*/
ProtectionTermEventDayType                         = 40197, /*!< 40197 (int FIX.5.0SP2) Day type for events that specify a period and unit. */
ProtectionTermEventRateSource                      = 40198, /*!< 40198 (String FIX.5.0SP2) Rate source for events that specify a rate source, e.g. Floating rate interest shortfall.*/
NoProtectionTermEventQualifiers                    = 40199, /*!< 40199 (NumInGroup FIX.5.0SP2) Number of qualifiers in the repeating group.*/
ProtectionTermEventQualifier                       = 40200, /*!< 40200 (char FIX.5.0SP2) Protection term event qualifier.  Used to further qualify ProtectionTermEventType(40192). */
NoProtectionTermObligations                        = 40201, /*!< 40201 (NumInGroup FIX.5.0SP2) Number of obligations in the repeating group.*/
ProtectionTermObligationType                       = 40202, /*!< 40202 (String FIX.5.0SP2) Specifies the type of obligation applicable to the protection terms. 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Obligation_Types for code list of applicable obligation types.*/
ProtectionTermObligationValue                      = 40203, /*!< 40203 (String FIX.5.0SP2) Protection term obligation value appropriate to ProtectionTermObligationType(40202). 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Obligation_Types for applicable obligation type values. */
NoPhysicalSettlTerms                               = 40204, /*!< 40204 (NumInGroup FIX.5.0SP2) Number of entries in the repeating group.*/
PhysicalSettlCurrency                              = 40205, /*!< 40205 (Currency FIX.5.0SP2) Specifies the currency of physical settlement.  Uses ISO 4217 currency codes.*/
PhysicalSettlBusinessDays                          = 40206, /*!< 40206 (int FIX.5.0SP2) The number of business days used in the determination of physical settlement. Its precise meaning is dependant on the context in which this element is used. */
PhysicalSettlMaximumBusinessDays                   = 40207, /*!< 40207 (int FIX.5.0SP2) A maximum number of business days. Its precise meaning is dependant on the context in which this element is used. Intended to be used to limit a particular ISDA fallback provision.*/
PhysicalSettlTermXID                               = 40208, /*!< 40208 (XID FIX.5.0SP2) A named string value referenced by UnderlyingSettlTermXIDRef(41315). */
NoPhysicalSettlDeliverableObligations              = 40209, /*!< 40209 (NumInGroup FIX.5.0SP2) Number of entries in the repeating group.*/
PhysicalSettlDeliverableObligationType             = 40210, /*!< 40210 (String FIX.5.0SP2) Specifies the type of deliverable obligation applicable for physical settlement. See http://www.fixtradingcommunity.org/codelists#Deliverable_Obligation_Types for code list for applicable deliverable obligation types.*/
PhysicalSettlDeliverableObligationValue            = 40211, /*!< 40211 (String FIX.5.0SP2) Physical settlement deliverable obligation value appropriate to PhysicalSettlDeliverableObligationType(40210). See http://www.fixtradingcommunity.org/codelists#Deliverable_Obligation_Types for applicable obligation type values.*/
NoPayments                                         = 40212, /*!< 40212 (NumInGroup FIX.5.0SP2) Number of additional settlement or bullet payments.*/
PaymentType                                        = 40213, /*!< 40213 (int FIX.5.0SP2) Type of payment.*/
PaymentPaySide                                     = 40214, /*!< 40214 (int FIX.5.0SP2) The side of the party paying the payment.*/
PaymentReceiveSide                                 = 40215, /*!< 40215 (int FIX.5.0SP2) The side of the party receiving the payment.*/
PaymentCurrency                                    = 40216, /*!< 40216 (Currency FIX.5.0SP2) Specifies the currency in which PaymentAmount(40217)  is denominated.  Uses ISO 4271 currency codes.*/
PaymentAmount                                      = 40217, /*!< 40217 (Amt FIX.5.0SP2) The total payment amount.*/
PaymentPrice                                       = 40218, /*!< 40218 (Price FIX.5.0SP2) The price determining the payment amount expressed in terms specified in PaymentPriceType(40919) and expressed in market format. */
PaymentDateUnadjusted                              = 40219, /*!< 40219 (LocalMktDate FIX.5.0SP2) The unadjusted payment date.*/
PaymentBusinessDayConvention                       = 40220, /*!< 40220 (int FIX.5.0SP2) The business day convention used to adjust the payment date.  Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
PaymentBusinessCenter                              = 40221, /*!< 40221 (String FIX.5.0SP2) The business center calendar used to adjust the payment date, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentDateAdjusted                                = 40222, /*!< 40222 (LocalMktDate FIX.5.0SP2) The adjusted payment date.*/
LegMarketDisruptionValue                           = 40223, /*!< 40223 (String FIX.5.0SP2) Applicable value for LegMarketDisruptionEvent(41468).*/
PaymentDiscountFactor                              = 40224, /*!< 40224 (float FIX.5.0SP2) The value representing the discount factor used to calculate the present value of the cash flow.*/
PaymentPresentValueAmount                          = 40225, /*!< 40225 (Amt FIX.5.0SP2) The amount representing the present value of the forecast payment.*/
PaymentPresentValueCurrency                        = 40226, /*!< 40226 (Currency FIX.5.0SP2) Specifies the currency the PaymentPresentValueAmount(40225) is denominated in.  Uses ISO 4217 currency codes.*/
PaymentSettlStyle                                  = 40227, /*!< 40227 (int FIX.5.0SP2) Payment settlement style.*/
LegPaymentStreamNonDeliverableSettlReferencePage   = 40228, /*!< 40228 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

When LegPaymentStreamNonDeliverableSettlRateSource(40087) = 3 (ISDA Settlement Rate Option) this contains a value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
PaymentText                                        = 40229, /*!< 40229 (String FIX.5.0SP2) Free form text to specify additional information or enumeration description when a sdtandard value does not apply.  Identifies the payment type when PaymentType(40213) = 99 (Other).*/
NoPaymentSettls                                    = 40230, /*!< 40230 (NumInGroup FIX.5.0SP2) Number of additional settlements or bullet payments. */
PaymentSettlAmount                                 = 40231, /*!< 40231 (Amt FIX.5.0SP2) The payment settlement amount.*/
PaymentSettlCurrency                               = 40232, /*!< 40232 (Currency FIX.5.0SP2) Specifies the currency the PaymentSettlAmount(40231) is denominated in.  Uses ISO 4217 currency codes.*/
NoPaymentSettlPartyIDs                             = 40233, /*!< 40233 (NumInGroup FIX.5.0SP2) Number of parties identified in the additional settlement or bullet payment.*/
PaymentSettlPartyID                                = 40234, /*!< 40234 (String FIX.5.0SP2) The payment settlement party identifier. */
PaymentSettlPartyIDSource                          = 40235, /*!< 40235 (char FIX.5.0SP2) Identifies the class or source of PaymentSettlPartyID(40234) value (e.g. BIC). */
PaymentSettlPartyRole                              = 40236, /*!< 40236 (int FIX.5.0SP2) Identifies the role of PaymentSettlPartyID(40234) (e.g. the beneficiary's bank or depository institution).*/
PaymentSettlPartyRoleQualifier                     = 40237, /*!< 40237 (int FIX.5.0SP2) Qualifies the value of PaymentSettlPartyRole(40236).*/
NoPaymentSettlPartySubIDs                          = 40238, /*!< 40238 (NumInGroup FIX.5.0SP2) Number of sub-party IDs to be reported for the party.*/
PaymentSettlPartySubID                             = 40239, /*!< 40239 (String FIX.5.0SP2) Party sub-identifier, if applicable, for PaymentSettlPartyRole(40236). */
PaymentSettlPartySubIDType                         = 40240, /*!< 40240 (int FIX.5.0SP2) The type of PaymentSettlPartySubID(40239) value. */
NoLegStreams                                       = 40241, /*!< 40241 (NumInGroup FIX.5.0SP2) Number of swap streams in the repeating group.*/
LegStreamType                                      = 40242, /*!< 40242 (int FIX.5.0SP2) Type of swap stream.*/
LegStreamDesc                                      = 40243, /*!< 40243 (String FIX.5.0SP2) A short descriptive name given to the payment stream, e.g. CDS, Fixed, Float, Float2, GBP. The description has no intrinsic meaning but should be arbitrarily chosen by the remitter as a reference.*/
LegStreamPaySide                                   = 40244, /*!< 40244 (int FIX.5.0SP2) The side of the party paying the stream.*/
LegStreamReceiveSide                               = 40245, /*!< 40245 (int FIX.5.0SP2) The side of the party receiving the stream.*/
LegStreamNotional                                  = 40246, /*!< 40246 (Amt FIX.5.0SP2) Notional, or initial notional value for the payment stream. The LegPaymentSchedule component should be used for specifying the steps.*/
LegStreamCurrency                                  = 40247, /*!< 40247 (Currency FIX.5.0SP2) Specifies the currency the LegStreamNotional(40246) is denominated in.  Uses ISO 4217 currency codes.*/
LegStreamText                                      = 40248, /*!< 40248 (String FIX.5.0SP2) Free form text to specify additional information or enumeration description when a standard value does not apply.*/
LegStreamEffectiveDateUnadjusted                   = 40249, /*!< 40249 (LocalMktDate FIX.5.0SP2) The unadjusted effective date.*/
LegStreamEffectiveDateBusinessDayConvention        = 40250, /*!< 40250 (int FIX.5.0SP2) The business day convention used to adjust the instrument leg's stream's effective date or relative effective date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegStreamEffectiveDateBusinessCenter               = 40251, /*!< 40251 (String FIX.5.0SP2) The business center calendar used to adjust the instrument leg's stream's effective date or relative effective date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegStreamEffectiveDateRelativeTo                   = 40252, /*!< 40252 (int FIX.5.0SP2) Specifies the anchor date when the effective date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values*/
LegStreamEffectiveDateOffsetPeriod                 = 40253, /*!< 40253 (int FIX.5.0SP2) Time unit multiplier for the relative effective date offset.  */
LegStreamEffectiveDateOffsetUnit                   = 40254, /*!< 40254 (String FIX.5.0SP2) Time unit associated with the relative effective date offset. */
LegStreamEffectiveDateOffsetDayType                = 40255, /*!< 40255 (int FIX.5.0SP2) Specifies the day type of the relative effective date offset.*/
LegStreamEffectiveDateAdjusted                     = 40256, /*!< 40256 (LocalMktDate FIX.5.0SP2) The adjusted effective date.*/
LegStreamTerminationDateUnadjusted                 = 40257, /*!< 40257 (LocalMktDate FIX.5.0SP2) The unadjusted termination date.*/
LegStreamTerminationDateBusinessDayConvention      = 40258, /*!< 40258 (int FIX.5.0SP2) The business day convention used to adjust the instrument leg's stream's termination, or relative termination, date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegStreamTerminationDateBusinessCenter             = 40259, /*!< 40259 (String FIX.5.0SP2) The business center calendar used to adjust the instrument leg's stream's termination, or relative termination, date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegStreamTerminationDateRelativeTo                 = 40260, /*!< 40260 (int FIX.5.0SP2) Specifies the anchor date when the termination date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegStreamTerminationDateOffsetPeriod               = 40261, /*!< 40261 (int FIX.5.0SP2) Time unit multiplier for the relative termination date offset.*/
LegStreamTerminationDateOffsetUnit                 = 40262, /*!< 40262 (String FIX.5.0SP2) Time unit associated with the relative termination date offset.*/
LegStreamTerminationDateOffsetDayType              = 40263, /*!< 40263 (int FIX.5.0SP2) Specifies the day type of the relative termination date offset.*/
LegStreamTerminationDateAdjusted                   = 40264, /*!< 40264 (LocalMktDate FIX.5.0SP2) The adjusted termination date.*/
LegStreamCalculationPeriodBusinessDayConvention    = 40265, /*!< 40265 (int FIX.5.0SP2) The business day convention used to adjust calculation periods.  Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegStreamCalculationPeriodBusinessCenter           = 40266, /*!< 40266 (String FIX.5.0SP2) The business center calendar used to adjust calculation periods, e.g. "GLBO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegStreamFirstPeriodStartDateUnadjusted            = 40267, /*!< 40267 (LocalMktDate FIX.5.0SP2) The unadjusted first calculation period start date if before the effective date.*/
LegStreamFirstPeriodStartDateBusinessDayConvention = 40268, /*!< 40268 (int FIX.5.0SP2) The business day convention used to adjust the instrument leg's stream's first calculation period start date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegStreamFirstPeriodStartDateBusinessCenter        = 40269, /*!< 40269 (String FIX.5.0SP2) The business center calendar used to adjust the instrument leg's stream's first calculation period start date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
LegStreamFirstPeriodStartDateAdjusted              = 40270, /*!< 40270 (LocalMktDate FIX.5.0SP2) The adjusted first calculation period start date, if it is before the effective date.*/
LegStreamFirstRegularPeriodStartDateUnadjusted     = 40271, /*!< 40271 (LocalMktDate FIX.5.0SP2) The unadjusted first start date of the regular calculation period, if there is an initial stub period.*/
LegStreamFirstCompoundingPeriodEndDateUnadjusted   = 40272, /*!< 40272 (LocalMktDate FIX.5.0SP2) The unadjusted end date of the initial compounding period.*/
LegStreamLastRegularPeriodEndDateUnadjusted        = 40273, /*!< 40273 (LocalMktDate FIX.5.0SP2) The unadjusted last regular period end date if there is a final stub period.*/
LegStreamCalculationFrequencyPeriod                = 40274, /*!< 40274 (int FIX.5.0SP2) Time unit multiplier for the frequency at which calculation period end dates occur. */
LegStreamCalculationFrequencyUnit                  = 40275, /*!< 40275 (String FIX.5.0SP2) Time unit associated with the frequency at which calculation period end dates occur. */
LegStreamCalculationRollConvention                 = 40276, /*!< 40276 (String FIX.5.0SP2) The convention for determining the sequence of end dates. It is used in conjunction with a specified frequency.  Used only to override the roll convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
NoCashSettlDealers                                 = 40277, /*!< 40277 (NumInGroup FIX.5.0SP2) Number of dealers in the repeating group.*/
NoBusinessCenters                                  = 40278, /*!< 40278 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegPaymentStreamType                               = 40279, /*!< 40279 (int FIX.5.0SP2) Identifies the type of payment stream applicable to the swap stream associated with the instrument leg.*/
LegPaymentStreamMarketRate                         = 40280, /*!< 40280 (int FIX.5.0SP2) Used only for credit index trade. This contains the credit spread ("fair value") at which the trade was executed. The market rate varies over the life of the index depending on market conditions. This is the price of the index as quoted by trading desks.*/
LegPaymentStreamDelayIndicator                     = 40281, /*!< 40281 (Boolean FIX.5.0SP2) Applicable to credit default swaps on mortgage backed securities to specify whether payment delays are applicable to the fixed amount. 

Residential mortgage backed securities typically have a payment delay of 5 days between the coupon date of the reference obligation and the payment date of the synthetic swap. 

Commercial mortage backed securities  do not typically have a payment delay, with both payment dates (the coupon date of the reference obligation and the payment date of the synthetic swap) being on the 25th of each month.*/
LegPaymentStreamSettlCurrency                      = 40282, /*!< 40282 (Currency FIX.5.0SP2) Specifies the currency that the stream settles in (to support swaps that settle in a currency different from the notional currency).  Uses ISO 4217 currency codes.*/
LegPaymentStreamDayCount                           = 40283, /*!< 40283 (int FIX.5.0SP2) The day count convention used in the payment stream calculations.*/
LegPaymentStreamAccrualDays                        = 40284, /*!< 40284 (int FIX.5.0SP2) The number of days from the adjusted calculation period start date to the adjusted value date, calculated in accordance with the applicable day count fraction.*/
LegPaymentStreamDiscountType                       = 40285, /*!< 40285 (int FIX.5.0SP2) The method of calculating discounted payment amounts.*/
LegPaymentStreamDiscountRate                       = 40286, /*!< 40286 (Percentage FIX.5.0SP2) Discount rate. The rate is expressed in decimal, e.g. 5% is expressed as 0.05.*/
LegPaymentStreamDiscountRateDayCount               = 40287, /*!< 40287 (int FIX.5.0SP2) The day count convention applied to the LegPaymentStreamDiscountRate(40286).*/
LegPaymentStreamCompoundingMethod                  = 40288, /*!< 40288 (int FIX.5.0SP2) Compounding method.*/
LegPaymentStreamInitialPrincipalExchangeIndicator  = 40289, /*!< 40289 (Boolean FIX.5.0SP2) Indicates whether there is an initial exchange of principal on the effective date.*/
LegPaymentStreamInterimPrincipalExchangeIndicator  = 40290, /*!< 40290 (Boolean FIX.5.0SP2) Indicates whether there are intermediate or interim exchanges of principal during the term of the swap.*/
LegPaymentStreamFinalPrincipalExchangeIndicator    = 40291, /*!< 40291 (Boolean FIX.5.0SP2) Indicates whether there is a final exchange of principal on the termination date.*/
LegPaymentStreamPaymentDateBusinessDayConvention   = 40292, /*!< 40292 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's payment date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPaymentStreamPaymentDateBusinessCenter          = 40293, /*!< 40293 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's payment date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
LegPaymentStreamPaymentFrequencyPeriod             = 40294, /*!< 40294 (int FIX.5.0SP2) Time unit multiplier for the frequency of payments. */
LegPaymentStreamPaymentFrequencyUnit               = 40295, /*!< 40295 (String FIX.5.0SP2) Time unit associated with the frequency of payments. */
LegPaymentStreamPaymentRollConvention              = 40296, /*!< 40296 (String FIX.5.0SP2) The convention for determining the sequence of end dates. It is used in conjunction with a specified frequency.  Used only to override the roll convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPaymentStreamFirstPaymentDateUnadjusted         = 40297, /*!< 40297 (LocalMktDate FIX.5.0SP2) The unadjusted first payment date.*/
LegPaymentStreamLastRegularPaymentDateUnadjusted   = 40298, /*!< 40298 (LocalMktDate FIX.5.0SP2) The unadjusted last regular payment date.*/
LegPaymentStreamPaymentDateRelativeTo              = 40299, /*!< 40299 (int FIX.5.0SP2) Specifies the anchor date when payment dates are relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamPaymentDateOffsetPeriod            = 40300, /*!< 40300 (int FIX.5.0SP2) Time unit multiplier for the relative payment date offset.  */
LegPaymentStreamPaymentDateOffsetUnit              = 40301, /*!< 40301 (String FIX.5.0SP2) Time unit associated with the relative payment date offset. */
LegPaymentStreamPaymentDateOffsetDayType           = 40302, /*!< 40302 (int FIX.5.0SP2) Specifies the day type of the relative payment date offset.*/
LegPaymentStreamResetDateRelativeTo                = 40303, /*!< 40303 (int FIX.5.0SP2) Specifies the anchor date when the reset dates are relative to an anchor date. 

If the reset frequency is specified as daily this element must not be included. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamResetDateBusinessDayConvention     = 40304, /*!< 40304 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's reset date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPaymentStreamResetDateBusinessCenter            = 40305, /*!< 40305 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's reset date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
LegPaymentStreamResetFrequencyPeriod               = 40306, /*!< 40306 (int FIX.5.0SP2) Time unit multiplier for frequency of resets.*/
LegPaymentStreamResetFrequencyUnit                 = 40307, /*!< 40307 (String FIX.5.0SP2) Time unit associated with frequency of resets.*/
LegPaymentStreamResetWeeklyRollConvention          = 40308, /*!< 40308 (String FIX.5.0SP2) Used to specify the day of the week in which the reset occurs for payments that reset on a weekly basis.*/
LegPaymentStreamInitialFixingDateRelativeTo        = 40309, /*!< 40309 (int FIX.5.0SP2) Specifies the anchor date when the initial fixing date is relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamInitialFixingDateBusinessDayConvention = 40310, /*!< 40310 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's initial fixing date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPaymentStreamInitialFixingDateBusinessCenter    = 40311, /*!< 40311 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's initial fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
LegPaymentStreamInitialFixingDateOffsetPeriod      = 40312, /*!< 40312 (int FIX.5.0SP2) Time unit multiplier for the relative initial fixing date offset.*/
LegPaymentStreamInitialFixingDateOffsetUnit        = 40313, /*!< 40313 (String FIX.5.0SP2) Time unit associated with the relative initial fixing date offset.*/
LegPaymentStreamInitialFixingDateOffsetDayType     = 40314, /*!< 40314 (int FIX.5.0SP2) Specifies the day type of the relative initial fixing date offset.*/
LegPaymentStreamInitialFixingDateAdjusted          = 40315, /*!< 40315 (LocalMktDate FIX.5.0SP2) The adjusted initial fixing date.*/
LegPaymentStreamFixingDateRelativeTo               = 40316, /*!< 40316 (int FIX.5.0SP2) Specifies the anchor date when the fixing date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamFixingDateBusinessDayConvention    = 40317, /*!< 40317 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's fixing date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPaymentStreamFixingDateBusinessCenter           = 40318, /*!< 40318 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
LegPaymentStreamFixingDateOffsetPeriod             = 40319, /*!< 40319 (int FIX.5.0SP2) Time unit multiplier for the relative fixing date offset.*/
LegPaymentStreamFixingDateOffsetUnit               = 40320, /*!< 40320 (String FIX.5.0SP2) Time unit associated with the relative fixing date offset.*/
LegPaymentStreamFixingDateOffsetDayType            = 40321, /*!< 40321 (int FIX.5.0SP2) Specifies the day type of the relative fixing date offset.*/
LegPaymentStreamFixingDateAdjusted                 = 40322, /*!< 40322 (LocalMktDate FIX.5.0SP2) The adjusted fixing date.*/
LegPaymentStreamRateCutoffDateOffsetPeriod         = 40323, /*!< 40323 (int FIX.5.0SP2) Time unit multiplier for the relative rate cut-off date offset.*/
LegPaymentStreamRateCutoffDateOffsetUnit           = 40324, /*!< 40324 (String FIX.5.0SP2) Time unit associated with the relative rate cut-off date offset.*/
LegPaymentStreamRateCutoffDateOffsetDayType        = 40325, /*!< 40325 (int FIX.5.0SP2) Specifies the day type of the relative rate cut-off date offset.*/
LegPaymentStreamRate                               = 40326, /*!< 40326 (Percentage FIX.5.0SP2) The rate applicable to the fixed rate payment stream.*/
LegPaymentStreamFixedAmount                        = 40327, /*!< 40327 (Amt FIX.5.0SP2) The leg instrument payment stream's fixed payment amount. In a CDS, this can be an alternative to LegPaymentStreamRate(40326).*/
LegPaymentStreamRateOrAmountCurrency               = 40328, /*!< 40328 (Currency FIX.5.0SP2) Specifies the currency in which LegPaymentStreamFixedAmount(40327) or LegPaymentStreamRate(40326) is denominated.  Uses ISO 4217 currency codes.*/
LegPaymentStreamFutureValueNotional                = 40329, /*!< 40329 (Amt FIX.5.0SP2) The future value notional is normally only required for certain non-deliverable interest rate swaps (e.g. Brazillian Real (BRL) vs. CETIP Interbank Deposit Rate (CDI)). The value is calculated as follows: Future Value Notional = Notional Amount * (1 + Fixed Rate) ^ (Fixed Rate Day Count Fraction). The currency is the same as the stream notional.*/
LegPaymentStreamFutureValueDateAdjusted            = 40330, /*!< 40330 (LocalMktDate FIX.5.0SP2) The adjusted value date of the future value amount.*/
LegPaymentStreamRateIndex                          = 40331, /*!< 40331 (String FIX.5.0SP2) The payment stream floating rate index.*/
LegPaymentStreamRateIndexSource                    = 40332, /*!< 40332 (int FIX.5.0SP2) The source of the payment stream floating rate index.*/
LegPaymentStreamRateIndexCurveUnit                 = 40333, /*!< 40333 (String FIX.5.0SP2) Time unit associated with the payment stream's floating rate index curve period. */
LegPaymentStreamRateIndexCurvePeriod               = 40334, /*!< 40334 (int FIX.5.0SP2) Time unit multiplier for the payment stream's floating rate index curve period. */
LegPaymentStreamRateMultiplier                     = 40335, /*!< 40335 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
LegPaymentStreamRateSpread                         = 40336, /*!< 40336 (PriceOffset FIX.5.0SP2) The basis points spread from the index specified in LegPaymentStreamRateIndex(40331).*/
LegPaymentStreamRateSpreadPositionType             = 40337, /*!< 40337 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
LegPaymentStreamRateTreatment                      = 40338, /*!< 40338 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
LegPaymentStreamCapRate                            = 40339, /*!< 40339 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. It is only required where the floating rate on a swap stream is capped at a certain level The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
LegPaymentStreamCapRateBuySide                     = 40340, /*!< 40340 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
LegPaymentStreamCapRateSellSide                    = 40341, /*!< 40341 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
LegPaymentStreamFloorRate                          = 40342, /*!< 40342 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level The floor rate is assumed to be exclusive of any spread and is a per annum rate. The rate is expressed as a decimal, e.g. 5% is represented as 0.05.*/
LegPaymentStreamFloorRateBuySide                   = 40343, /*!< 40343 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
LegPaymentStreamFloorRateSellSide                  = 40344, /*!< 40344 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
LegPaymentStreamInitialRate                        = 40345, /*!< 40345 (Percentage FIX.5.0SP2) The initial floating rate reset agreed between the principal parties involved in the trade. This is assumed to be the first required reset rate for the first regular calculation period. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. The initial rate is expressed in decimal form, e.g. 5% is represented as 0.05.*/
LegPaymentStreamFinalRateRoundingDirection         = 40346, /*!< 40346 (char FIX.5.0SP2) Specifies the rounding direction.*/
LegPaymentStreamFinalRatePrecision                 = 40347, /*!< 40347 (int FIX.5.0SP2) Specifies the rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
LegPaymentStreamAveragingMethod                    = 40348, /*!< 40348 (int FIX.5.0SP2) When averaging is applicable, used to specify whether a weighted or unweighted average method of calculation is to be used. */
LegPaymentStreamNegativeRateTreatment              = 40349, /*!< 40349 (int FIX.5.0SP2) The specification of any provisions for calculating payment obligations when a floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
LegPaymentStreamInflationLagPeriod                 = 40350, /*!< 40350 (int FIX.5.0SP2) Time unit multiplier for the inflation lag period.  The lag period is the offsetting period from the payment date which determineds the reference period for which the inflation index is observed.  */
LegPaymentStreamInflationLagUnit                   = 40351, /*!< 40351 (String FIX.5.0SP2) Time unit associated with the inflation lag period. */
LegPaymentStreamInflationLagDayType                = 40352, /*!< 40352 (int FIX.5.0SP2) The inflation lag period day type.*/
LegPaymentStreamInflationInterpolationMethod       = 40353, /*!< 40353 (int FIX.5.0SP2) The method used when calculating the inflation index level from multiple points. The most common is linear method.*/
LegPaymentStreamInflationIndexSource               = 40354, /*!< 40354 (int FIX.5.0SP2) The inflation index reference source. */
LegPaymentStreamInflationPublicationSource         = 40355, /*!< 40355 (String FIX.5.0SP2) The publication source, such as relevant web site, news publication or a government body, where inflation information is obtained.*/
LegPaymentStreamInflationInitialIndexLevel         = 40356, /*!< 40356 (float FIX.5.0SP2) Initial known index level for the first calculation period.*/
LegPaymentStreamInflationFallbackBondApplicable    = 40357, /*!< 40357 (Boolean FIX.5.0SP2) Indicates whether a fallback bond as defined in the 2006 ISDA Inflation Derivatives Definitions, sections 1.3 and 1.8, is applicable or not. If not specified, the default value is "Y" (True/Yes).*/
LegPaymentStreamFRADiscounting                     = 40358, /*!< 40358 (int FIX.5.0SP2) The method of Forward Rate Agreement (FRA) discounting, if any, that will apply.*/
LegPaymentStreamNonDeliverableRefCurrency          = 40359, /*!< 40359 (Currency FIX.5.0SP2) Non-deliverable settlement reference currency.  Uses ISO 4217 currency codes.*/
LegPaymentStreamNonDeliverableFixingDatesBusinessDayConvention = 40360, /*!< 40360 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's fixing date for the non-deliverable settlement terms. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPaymentStreamNonDeliverableFixingDatesBusinessCenter = 40361, /*!< 40361 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's fixing date for the non-deliverable terms, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
LegPaymentStreamNonDeliverableFixingDatesRelativeTo = 40362, /*!< 40362 (int FIX.5.0SP2) Specifies the anchor date when the non-deliverable fixing dates are relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamNonDeliverableFixingDatesOffsetPeriod = 40363, /*!< 40363 (int FIX.5.0SP2) Time unit multiplier for the relative non-deliverable fixing date offset.*/
LegPaymentStreamNonDeliverableFixingDatesOffsetUnit = 40364, /*!< 40364 (String FIX.5.0SP2) Time unit associated with the relative non-deliverable fixing date offset.*/
LegPaymentStreamNonDeliverableFixingDatesOffsetDayType = 40365, /*!< 40365 (int FIX.5.0SP2) Specifies the day type of the relative non-deliverable fixing date offset.*/
LegSettlRateFallbackRateSource                     = 40366, /*!< 40366 (int FIX.5.0SP2) Identifies the source of rate information.*/
NoLegNonDeliverableFixingDates                     = 40367, /*!< 40367 (NumInGroup FIX.5.0SP2) Number of fixing dates in the repeating group.*/
LegNonDeliverableFixingDate                        = 40368, /*!< 40368 (LocalMktDate FIX.5.0SP2) The non-deliverable fixing date.  Type of date is specified in LegNonDeliverableFixingDateType(40369).*/
LegNonDeliverableFixingDateType                    = 40369, /*!< 40369 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
LegSettlRateFallbackReferencePage                  = 40370, /*!< 40370 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

When LegSettlRateFallbackRateSource(40366) = 3(ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
PaymentStreamNonDeliverableSettlRateSource         = 40371, /*!< 40371 (int FIX.5.0SP2) Identifies the source of rate information.*/
PaymentStreamNonDeliverableSettlReferencePage      = 40372, /*!< 40372 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

When PaymentStreamNonDeliverableSettlRateSource(40371) = 3(ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
SettlRateFallbackRateSource                        = 40373, /*!< 40373 (int FIX.5.0SP2) Identifies the source of rate information.*/
NoLegPaymentSchedules                              = 40374, /*!< 40374 (NumInGroup FIX.5.0SP2) Number of swap schedules in the repeating group*/
LegPaymentScheduleType                             = 40375, /*!< 40375 (int FIX.5.0SP2) Specifies the type of schedule. */
LegPaymentScheduleStubType                         = 40376, /*!< 40376 (int FIX.5.0SP2) Indicates to which stub this schedule applies. */
LegPaymentScheduleStartDateUnadjusted              = 40377, /*!< 40377 (LocalMktDate FIX.5.0SP2) The unadjusted date on which the value is adjusted, or calculated if a future value notional for certain non-deliverable interest rate swaps (e.g. Brazillian Real (BRL) vs. CETIP Interbank Deposit Rate (CDI)), or the start date of a cashflow payment.*/
LegPaymentScheduleEndDateUnadjusted                = 40378, /*!< 40378 (LocalMktDate FIX.5.0SP2) The unadjusted end date of a cashflow payment.*/
LegPaymentSchedulePaySide                          = 40379, /*!< 40379 (int FIX.5.0SP2) The side of the party paying the step schedule.*/
LegPaymentScheduleReceiveSide                      = 40380, /*!< 40380 (int FIX.5.0SP2) The side of the party receiving the step schedule.*/
LegPaymentScheduleNotional                         = 40381, /*!< 40381 (Amt FIX.5.0SP2) The notional value for this step schedule, or amount of a cashflow payment.*/
LegPaymentScheduleCurrency                         = 40382, /*!< 40382 (Currency FIX.5.0SP2) The currency for this step schedule.  Uses ISO 4217 currency codes.*/
LegPaymentScheduleRate                             = 40383, /*!< 40383 (Percentage FIX.5.0SP2) The rate value for this step schedule.*/
LegPaymentScheduleRateMultiplier                   = 40384, /*!< 40384 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
LegPaymentScheduleRateSpread                       = 40385, /*!< 40385 (PriceOffset FIX.5.0SP2) The spread value for this step schedule.*/
LegPaymentScheduleRateSpreadPositionType           = 40386, /*!< 40386 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or a short position.*/
LegPaymentScheduleRateTreatment                    = 40387, /*!< 40387 (int FIX.5.0SP2) Specifies the yield calculation treatment for the step schedule.*/
LegPaymentScheduleFixedAmount                      = 40388, /*!< 40388 (Amt FIX.5.0SP2) The explicit payment amount for this step schedule.*/
LegPaymentScheduleFixedCurrency                    = 40389, /*!< 40389 (Currency FIX.5.0SP2) The currency of the fixed amount.  Uses ISO 4217 currency codes.*/
LegPaymentScheduleStepFrequencyPeriod              = 40390, /*!< 40390 (int FIX.5.0SP2) Time unit multiplier for the step frequency.*/
LegPaymentScheduleStepFrequencyUnit                = 40391, /*!< 40391 (String FIX.5.0SP2) Time unit associated with the step frequency. */
LegPaymentScheduleStepOffsetValue                  = 40392, /*!< 40392 (Amt FIX.5.0SP2) The explicit amount that the notional changes on each step date. This can be a positive or negative amount.*/
LegPaymentScheduleStepRate                         = 40393, /*!< 40393 (Percentage FIX.5.0SP2) The percentage by which the notional changes on each step date. The percentage is either a percentage applied to the initial notional amount or the previous outstanding notional, depending on the value specified in LegPaymentScheduleStepRelativeTo(40395). The percentage can be either positive or negative.*/
LegPaymentScheduleStepOffsetRate                   = 40394, /*!< 40394 (Percentage FIX.5.0SP2) The explicit amount that the rate changes on each step date. This can be a positive or negative value.*/
LegPaymentScheduleStepRelativeTo                   = 40395, /*!< 40395 (int FIX.5.0SP2) Specifies whether the LegPaymentScheduleStepRate(40393) or LegPaymentScheduleStepOffsetValue(40392) should be applied to the initial notional or the previous notional in order to calculate the notional step change amount.*/
LegPaymentScheduleFixingDateUnadjusted             = 40396, /*!< 40396 (LocalMktDate FIX.5.0SP2) The unadjusted fixing date.*/
LegPaymentScheduleWeight                           = 40397, /*!< 40397 (float FIX.5.0SP2) Floating rate observation weight for cashflow payment.*/
LegPaymentScheduleFixingDateRelativeTo             = 40398, /*!< 40398 (int FIX.5.0SP2) Specifies the anchor date when the fixing date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values. */
LegPaymentScheduleFixingDateBusinessDayConvention  = 40399, /*!< 40399 (int FIX.5.0SP2) The business day convention used to adjust the payment schedule's fixing date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPaymentScheduleFixingDateBusinessCenter         = 40400, /*!< 40400 (String FIX.5.0SP2) The business center calendar used to adjust the payment schedule's fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegPaymentScheduleFixingDateOffsetPeriod           = 40401, /*!< 40401 (int FIX.5.0SP2) Time unit multiplier for the relative fixing date offset.*/
LegPaymentScheduleFixingDateOffsetUnit             = 40402, /*!< 40402 (String FIX.5.0SP2) Time unit associated with the relative fixing date offset.*/
LegPaymentScheduleFixingDateOffsetDayType          = 40403, /*!< 40403 (int FIX.5.0SP2) Specifies the day type of the relative fixing date offset.*/
LegPaymentScheduleFixingDateAdjusted               = 40404, /*!< 40404 (LocalMktDate FIX.5.0SP2) The adjusted fixing date.*/
LegPaymentScheduleFixingTime                       = 40405, /*!< 40405 (LocalMktTime FIX.5.0SP2) The fxing time associated with the step schedule.*/
LegPaymentScheduleFixingTimeBusinessCenter         = 40406, /*!< 40406 (String FIX.5.0SP2) Business center for determining fixing time. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegPaymentScheduleInterimExchangePaymentDateRelativeTo = 40407, /*!< 40407 (int FIX.5.0SP2) Specifies the anchor date when the interim exchange payment date is relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentScheduleInterimExchangeDatesBusinessDayConvention = 40408, /*!< 40408 (int FIX.5.0SP2) The business day convention used to adjust the payment schedule's interim exchange date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPaymentScheduleInterimExchangeDatesBusinessCenter = 40409, /*!< 40409 (String FIX.5.0SP2) The business center calendar used to adjust the payment schedule's interim exchange date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
LegPaymentScheduleInterimExchangeDatesOffsetPeriod = 40410, /*!< 40410 (int FIX.5.0SP2) Time unit multiplier for the relative interim exchange date offset.*/
LegPaymentScheduleInterimExchangeDatesOffsetUnit   = 40411, /*!< 40411 (String FIX.5.0SP2) Time unit associated with the relative interim exchange date offset.*/
LegPaymentScheduleInterimExchangeDatesOffsetDayType = 40412, /*!< 40412 (int FIX.5.0SP2) Specifies the day type of the relative interim exchange date offset.*/
LegPaymentScheduleInterimExchangeDateAdjusted      = 40413, /*!< 40413 (LocalMktDate FIX.5.0SP2) The adjusted interim exchange date.*/
NoLegPaymentScheduleRateSources                    = 40414, /*!< 40414 (NumInGroup FIX.5.0SP2) Number of rate sources in the repeating group*/
LegPaymentScheduleRateSource                       = 40415, /*!< 40415 (int FIX.5.0SP2) Identifies the source of rate information.*/
LegPaymentScheduleRateSourceType                   = 40416, /*!< 40416 (int FIX.5.0SP2) Rate source type.*/
LegPaymentScheduleReferencePage                    = 40417, /*!< 40417 (String FIX.5.0SP2) Identifies the reference "page" from the rate source. 



For FX, the reference page to the spot rate to be used for the reference FX spot rate.



When RateSource(1446) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
NoLegPaymentStubs                                  = 40418, /*!< 40418 (NumInGroup FIX.5.0SP2) Number of stubs in the repeating group*/
LegPaymentStubType                                 = 40419, /*!< 40419 (int FIX.5.0SP2) Stub type.*/
LegPaymentStubLength                               = 40420, /*!< 40420 (int FIX.5.0SP2) Optional indication whether stub is shorter or longer than the regular swap period.*/
LegPaymentStubRate                                 = 40421, /*!< 40421 (Percentage FIX.5.0SP2) The agreed upon fixed rate for this stub.*/
LegPaymentStubFixedAmount                          = 40422, /*!< 40422 (Amt FIX.5.0SP2) A fixed payment amount for the stub.*/
LegPaymentStubFixedCurrency                        = 40423, /*!< 40423 (Currency FIX.5.0SP2) The currency of the fixed payment amount.  Uses ISO 4217 currency codes.*/
LegPaymentStubIndex                                = 40424, /*!< 40424 (String FIX.5.0SP2) The stub floating rate index.*/
LegPaymentStubIndexSource                          = 40425, /*!< 40425 (int FIX.5.0SP2) The source for the stub floating rate index.*/
LegPaymentStubIndexCurvePeriod                     = 40426, /*!< 40426 (int FIX.5.0SP2) Time unit multiplier for the floating rate index.*/
LegPaymentStubIndexCurveUnit                       = 40427, /*!< 40427 (String FIX.5.0SP2) Time unit associated with the floating rate index.*/
LegPaymentStubIndexRateMultiplier                  = 40428, /*!< 40428 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
LegPaymentStubIndexRateSpread                      = 40429, /*!< 40429 (PriceOffset FIX.5.0SP2) Spread from floating rate index.*/
LegPaymentStubIndexRateSpreadPositionType          = 40430, /*!< 40430 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or a short position.*/
LegPaymentStubIndexRateTreatment                   = 40431, /*!< 40431 (int FIX.5.0SP2) Specifies the yield calculation treatment for the stub index.*/
LegPaymentStubIndexCapRate                         = 40432, /*!< 40432 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
LegPaymentStubIndexCapRateBuySide                  = 40433, /*!< 40433 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
LegPaymentStubIndexCapRateSellSide                 = 40434, /*!< 40434 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
LegPaymentStubIndexFloorRate                       = 40435, /*!< 40435 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.*/
LegPaymentStubIndexFloorRateBuySide                = 40436, /*!< 40436 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
LegPaymentStubIndexFloorRateSellSide               = 40437, /*!< 40437 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
LegPaymentStubIndex2                               = 40438, /*!< 40438 (String FIX.5.0SP2) The second stub floating rate index.*/
LegPaymentStubIndex2Source                         = 40439, /*!< 40439 (int FIX.5.0SP2) The source for the second stub floating rate index.*/
LegPaymentStubIndex2CurvePeriod                    = 40440, /*!< 40440 (int FIX.5.0SP2) Secondary time unit multiplier for the stub floating rate index curve.*/
LegPaymentStubIndex2CurveUnit                      = 40441, /*!< 40441 (String FIX.5.0SP2) Secondary time unit associated with the stub floating rate index curve.*/
LegPaymentStubIndex2RateMultiplier                 = 40442, /*!< 40442 (float FIX.5.0SP2) A rate multiplier to apply to the second floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
LegPaymentStubIndex2RateSpread                     = 40443, /*!< 40443 (PriceOffset FIX.5.0SP2) Spread from the second floating rate index.*/
LegPaymentStubIndex2RateSpreadPositionType         = 40444, /*!< 40444 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or a short position.*/
LegPaymentStubIndex2RateTreatment                  = 40445, /*!< 40445 (int FIX.5.0SP2) Specifies the yield calculation treatment for the second stub index.*/
LegPaymentStubIndex2CapRate                        = 40446, /*!< 40446 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the second floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
LegPaymentStubIndex2FloorRate                      = 40447, /*!< 40447 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the second floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.*/
NoLegProvisions                                    = 40448, /*!< 40448 (NumInGroup FIX.5.0SP2) Number of provisions in the repeating group.*/
LegProvisionType                                   = 40449, /*!< 40449 (int FIX.5.0SP2) Type of provisions.*/
LegProvisionDateUnadjusted                         = 40450, /*!< 40450 (LocalMktDate FIX.5.0SP2) The unadjusted date of the provision.*/
LegProvisionDateBusinessDayConvention              = 40451, /*!< 40451 (int FIX.5.0SP2) The business day convention used to adjust the instrument leg's provision's date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegProvisionDateBusinessCenter                     = 40452, /*!< 40452 (String FIX.5.0SP2) The business center calendar used to adjust the instrument leg's provision's date, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionDateAdjusted                           = 40453, /*!< 40453 (LocalMktDate FIX.5.0SP2) The adjusted date of the provision.*/
LegProvisionDateTenorPeriod                        = 40454, /*!< 40454 (int FIX.5.0SP2) Time unit multiplier for the leg provision's tenor period.*/
LegProvisionDateTenorUnit                          = 40455, /*!< 40455 (String FIX.5.0SP2) Time unit associated with the leg provision's tenor period.*/
LegProvisionCalculationAgent                       = 40456, /*!< 40456 (int FIX.5.0SP2) Used to identify the calculation agent.  The calculation agent may be identified in LegProvisionCalculationAgent(40456) or in the ProvisionParties component.*/
LegProvisionOptionSinglePartyBuyerSide             = 40457, /*!< 40457 (int FIX.5.0SP2) If optional early termination is not available to both parties then this component identifies the buyer of the option through its side of the trade.*/
LegProvisionOptionSinglePartySellerSide            = 40458, /*!< 40458 (int FIX.5.0SP2) If optional early termination is not available to both parties then this component identifies the seller of the option through its side of the trade.*/
LegProvisionOptionExerciseStyle                    = 40459, /*!< 40459 (int FIX.5.0SP2) The instrument provision option exercise style.*/
LegProvisionOptionExerciseMultipleNotional         = 40460, /*!< 40460 (Amt FIX.5.0SP2) A notional amount which restricts the amount of notional that can be exercised when partial exercise or multiple exercise is applicable. The integral multiple amount defines a lower limit of notional that can be exercised and also defines a unit multiple of notional that can be exercised, i.e. only integer multiples of this amount can be exercised.*/
LegProvisionOptionExerciseMinimumNotional          = 40461, /*!< 40461 (Amt FIX.5.0SP2) The minimum notional amount that can be exercised on a given exercise date.*/
LegProvisionOptionExerciseMaximumNotional          = 40462, /*!< 40462 (Amt FIX.5.0SP2) The maximum notional amount that can be exercised on a given exercise date.*/
LegProvisionOptionMinimumNumber                    = 40463, /*!< 40463 (int FIX.5.0SP2) The minimum number of options that can be exercised on a given exercise date.*/
LegProvisionOptionMaximumNumber                    = 40464, /*!< 40464 (int FIX.5.0SP2) The maximum number of options that can be exercised on a given exercise date. If the number is not specified, it means that the maximum number of options corresponds to the remaining unexercised options.*/
LegProvisionOptionExerciseConfirmation             = 40465, /*!< 40465 (Boolean FIX.5.0SP2) Used to indicate whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller's agent.*/
LegProvisionCashSettlMethod                        = 40466, /*!< 40466 (int FIX.5.0SP2) An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (e).*/
LegProvisionCashSettlCurrency                      = 40467, /*!< 40467 (Currency FIX.5.0SP2) Specifies the currency of settlement.  Uses ISO 4217 currency codes.*/
LegProvisionCashSettlCurrency2                     = 40468, /*!< 40468 (Currency FIX.5.0SP2) Specifies the currency of settlement for a cross-currency provision.  Uses ISO 4217 currency codes.*/
LegProvisionCashSettlQuoteType                     = 40469, /*!< 40469 (int FIX.5.0SP2) Identifies the type of quote to be used.  */
LegProvisionCashSettlQuoteSource                   = 40470, /*!< 40470 (int FIX.5.0SP2) Identifies the source of quote information. */
BusinessCenter                                     = 40471, /*!< 40471 (String FIX.5.0SP2) A business center whose calendar is used for date adjustment, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionText                                   = 40472, /*!< 40472 (String FIX.5.0SP2) Free form text to specify additional information or enumeration description when a standard value does not apply.*/
NoLegProvisionCashSettlPaymentDates                = 40473, /*!< 40473 (NumInGroup FIX.5.0SP2) Number of provision cash settlement payment dates in the repeating group.*/
LegProvisionCashSettlPaymentDate                   = 40474, /*!< 40474 (LocalMktDate FIX.5.0SP2) The cash settlement payment date, unadjusted or adjusted depending on LegProvisionCashSettlPaymentDateType(40521).*/
LegProvisionCashSettlPaymentDateType               = 40475, /*!< 40475 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
LegProvisionOptionExerciseBusinessDayConvention    = 40476, /*!< 40476 (int FIX.5.0SP2) The business day convention used to adjust the instrument leg's provision's option exercise date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegProvisionOptionExerciseBusinessCenter           = 40477, /*!< 40477 (String FIX.5.0SP2) The business center calendar used to adjust the instrument leg's provision's option exercise date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionOptionExerciseEarliestDateOffsetPeriod = 40478, /*!< 40478 (int FIX.5.0SP2) Time unit multiplier for the interval to the first (and possibly only) exercise date in the exercise period.*/
LegProvisionOptionExerciseEarliestDateOffsetUnit   = 40479, /*!< 40479 (String FIX.5.0SP2) Time unit associated with the interval to the first (and possibly only) exercise date in the exercise period.*/
LegProvisionOptionExerciseFrequencyPeriod          = 40480, /*!< 40480 (int FIX.5.0SP2) Time unit multiplier for subsequent exercise dates in the exercise period following the earliest exercise date. An interval of 1 day should be used to indicate an American style exercise period.*/
LegProvisionOptionExerciseFrequencyUnit            = 40481, /*!< 40481 (String FIX.5.0SP2) Time unit associated with subsequent exercise dates in the exercise period following the earliest exercise date.*/
LegProvisionOptionExerciseStartDateUnadjusted      = 40482, /*!< 40482 (LocalMktDate FIX.5.0SP2) The unadjusted first day of the exercise period for an American style option.*/
LegProvisionOptionExerciseStartDateRelativeTo      = 40483, /*!< 40483 (int FIX.5.0SP2) Specifies the anchor date when the option exercise start date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegProvisionOptionExerciseStartDateOffsetPeriod    = 40484, /*!< 40484 (int FIX.5.0SP2) Time unit multiplier for the relative option exercise start date offset.*/
LegProvisionOptionExerciseStartDateOffsetUnit      = 40485, /*!< 40485 (String FIX.5.0SP2) Time unit associated with the relative option exercise start date offset.*/
LegProvisionOptionExerciseStartDateOffsetDayType   = 40486, /*!< 40486 (int FIX.5.0SP2) Specifies the day type of the provision's relative option exercise start date offset.*/
LegProvisionOptionExerciseStartDateAdjusted        = 40487, /*!< 40487 (LocalMktDate FIX.5.0SP2) The adjusted first day of the exercise period for an American style option.*/
LegProvisionOptionExercisePeriodSkip               = 40488, /*!< 40488 (int FIX.5.0SP2) The number of periods in the referenced date schedule that are between each date in the relative date schedule. Thus a skip of 2 would mean that dates are relative to every second date in the referenced schedule. If present this should have a value greater than 1.*/
LegProvisionOptionExerciseBoundsFirstDateUnadjusted = 40489, /*!< 40489 (LocalMktDate FIX.5.0SP2) The unadjusted first date of a schedule. This can be used to restrict the range of exercise dates when they are relative.*/
LegProvisionOptionExerciseBoundsLastDateUnadjusted = 40490, /*!< 40490 (LocalMktDate FIX.5.0SP2) The unadjusted last date of a schedule. This can be used to restrict the range of exercise dates when they are relative.*/
LegProvisionOptionExerciseEarliestTime             = 40491, /*!< 40491 (LocalMktTime FIX.5.0SP2) The earliest time at which notice of exercise can be given by the buyer to the seller (or seller's agent) (i) on the expriation date, in the case of a European style option, (ii) on each bermuda option exercise date and the expiration date, in the case of a Bermuda style option the commencement date to, and including, the expiration date, in the case of an American option.*/
LegProvisionOptionExerciseEarliestTimeBusinessCenter = 40492, /*!< 40492 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's earliest time for notice of exercise. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionOptionExerciseLatestTime               = 40493, /*!< 40493 (LocalMktTime FIX.5.0SP2) For a Bermuda or American style option, the latest time on an exercise business day (excluding the expiration date) within the exercise period that notice can be given by the buyer to the seller or seller's agent. Notice of exercise given after this time will be deemed to have been given on the next exercise business day.*/
LegProvisionOptionExerciseLatestTimeBusinessCenter = 40494, /*!< 40494 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's latest time for notice of exercise. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegProvisionOptionExerciseFixedDates             = 40495, /*!< 40495 (NumInGroup FIX.5.0SP2) Number of provision option exercise fixed dates in the repeating group.*/
LegProvisionOptionExerciseFixedDate                = 40496, /*!< 40496 (LocalMktDate FIX.5.0SP2) A predetermined option exercise date unadjusted or adjusted depending on LegProvisionOptionExerciseFixedDateType(40497).*/
LegProvisionOptionExerciseFixedDateType            = 40497, /*!< 40497 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
LegProvisionOptionExpirationDateUnadjusted         = 40498, /*!< 40498 (LocalMktDate FIX.5.0SP2) The unadjusted last day within an exercise period for an American style option. For a European style option it is the only day within the exercise period.*/
LegProvisionOptionExpirationDateBusinessDayConvention = 40499, /*!< 40499 (int FIX.5.0SP2) The business day convention used to adjust the instrument leg's provision's option expiration date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegProvisionOptionExpirationDateBusinessCenter     = 40500, /*!< 40500 (String FIX.5.0SP2) The business center calendar used to adjust the instrument leg's provision's option expiration date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionOptionExpirationDateRelativeTo         = 40501, /*!< 40501 (int FIX.5.0SP2) Specifies the anchor date when the option expiration date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegProvisionOptionExpirationDateOffsetPeriod       = 40502, /*!< 40502 (int FIX.5.0SP2) Time unit multiplier for the relative option expiration date offset.*/
LegProvisionOptionExpirationDateOffsetUnit         = 40503, /*!< 40503 (String FIX.5.0SP2) Time unit associated with the relative option expiration date offset.*/
LegProvisionOptionExpirationDateOffsetDayType      = 40504, /*!< 40504 (int FIX.5.0SP2) Specifies the day type of the provision's relative option expiration date offset.*/
LegProvisionOptionExpirationDateAdjusted           = 40505, /*!< 40505 (LocalMktDate FIX.5.0SP2) The adjusted last date within an exercise period for an American style option. For a European style option it is the only date within the exercise period.*/
LegProvisionOptionExpirationTime                   = 40506, /*!< 40506 (LocalMktTime FIX.5.0SP2) The latest time for exercise on the expiration date.*/
LegProvisionOptionExpirationTimeBusinessCenter     = 40507, /*!< 40507 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's latest exercise time on expiration date. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionOptionRelevantUnderlyingDateUnadjusted = 40508, /*!< 40508 (LocalMktDate FIX.5.0SP2) The unadjusted date on the underlying set by the exercise of an option. What this date is depends on the option (e.g. in a swaption it is the swap effective date, in an extendible/cancelable provision it is the swap termination date).*/
LegProvisionOptionRelevantUnderlyingDateBusinessDayConvention = 40509, /*!< 40509 (int FIX.5.0SP2) The business day convention used to adjust the instrument leg's provision's option relevant underlying date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegProvisionOptionRelevantUnderlyingDateBusinessCenter = 40510, /*!< 40510 (String FIX.5.0SP2) The business center calendar used to adjust the instrument leg's provision's option underlying date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionOptionRelevantUnderlyingDateRelativeTo = 40511, /*!< 40511 (int FIX.5.0SP2) Specifies the anchor date when the date relevant to the underlying trade on exercise is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegProvisionOptionRelevantUnderlyingDateOffsetPeriod = 40512, /*!< 40512 (int FIX.5.0SP2) Time unit multiplier for the relative option relevant underlying date offset.*/
LegProvisionOptionRelevantUnderlyingDateOffsetUnit = 40513, /*!< 40513 (String FIX.5.0SP2) Time unit associated with the relative option relevant underlying date offset.*/
LegProvisionOptionRelevantUnderlyingDateOffsetDayType = 40514, /*!< 40514 (int FIX.5.0SP2) Specifies the day type of the provision's relative option relevant underlying date offset.*/
LegProvisionOptionRelevantUnderlyingDateAdjusted   = 40515, /*!< 40515 (LocalMktDate FIX.5.0SP2) The adjusted date on the underlying set by the exercise of an option. What this date is depends on the option (e.g. in a swaption it is the swap effective date, in an extendible/cancelable provision it is the swap termination date).*/
LegProvisionCashSettlPaymentDateBusinessDayConvention = 40516, /*!< 40516 (int FIX.5.0SP2) The business day convention used to adjust the provisional cash settlement payment's termination, or relative termination, date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegProvisionCashSettlPaymentDateBusinessCenter     = 40517, /*!< 40517 (String FIX.5.0SP2) The business center calendar used to adjust the provisional cash settlement payment's termination, or relative termination, date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionCashSettlPaymentDateRelativeTo         = 40518, /*!< 40518 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement payment date is relative to an anchor date.  

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegProvisionCashSettlPaymentDateOffsetPeriod       = 40519, /*!< 40519 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement payment date offset.*/
LegProvisionCashSettlPaymentDateOffsetUnit         = 40520, /*!< 40520 (String FIX.5.0SP2) Time unit associated with the relative cash settlement payment date offset.*/
LegProvisionCashSettlPaymentDateOffsetDayType      = 40521, /*!< 40521 (int FIX.5.0SP2) Specifies the day type of the provision's relative cash settlement payment date offset.*/
LegProvisionCashSettlPaymentDateRangeFirst         = 40522, /*!< 40522 (LocalMktDate FIX.5.0SP2) The first date in range when a settlement date range is provided.*/
LegProvisionCashSettlPaymentDateRangeLast          = 40523, /*!< 40523 (LocalMktDate FIX.5.0SP2) The last date in range when a settlement date range is provided.*/
LegProvisionCashSettlValueTime                     = 40524, /*!< 40524 (LocalMktTime FIX.5.0SP2) A time specified in 24-hour format, e.g. 11am would be represented as 11:00:00. The time of the cash settlement valuation date when the cash settlement amount will be determined according to the cash settlement method if the parties have not otherwise been able to agree to the cash settlement amount.*/
LegProvisionCashSettlValueTimeBusinessCenter       = 40525, /*!< 40525 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's cash settlement valuation time. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionCashSettlValueDateBusinessDayConvention = 40526, /*!< 40526 (int FIX.5.0SP2) The business day convention used to adjust the provision's cash settlement valuation date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegProvisionCashSettlValueDateBusinessCenter       = 40527, /*!< 40527 (String FIX.5.0SP2) The business center calendar used to adjust the provision's cash settlement valuation date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionCashSettlValueDateRelativeTo           = 40528, /*!< 40528 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement value date is relative to an anchor date.  

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegProvisionCashSettlValueDateOffsetPeriod         = 40529, /*!< 40529 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement value date offset.*/
LegProvisionCashSettlValueDateOffsetUnit           = 40530, /*!< 40530 (String FIX.5.0SP2) Time unit associated with the relative cash settlement value date offset.*/
LegProvisionCashSettlValueDateOffsetDayType        = 40531, /*!< 40531 (int FIX.5.0SP2) Specifies the day type of the provision's relative cash settlement value date offset.*/
LegProvisionCashSettlValueDateAdjusted             = 40532, /*!< 40532 (LocalMktDate FIX.5.0SP2) The adjusted cash settlement value date.*/
NoLegProvisionPartyIDs                             = 40533, /*!< 40533 (NumInGroup FIX.5.0SP2) Number of parties identified in the contract provision.*/
LegProvisionPartyID                                = 40534, /*!< 40534 (String FIX.5.0SP2) The party identifier/code for the payment settlement party. */
LegProvisionPartyIDSource                          = 40535, /*!< 40535 (char FIX.5.0SP2) Identifies the class or source of LegProvisionPartyID(40534). */
LegProvisionPartyRole                              = 40536, /*!< 40536 (int FIX.5.0SP2) Identifies the type or role of LegProvisionPartyID(40534) specified. */
NoLegProvisionPartySubIDs                          = 40537, /*!< 40537 (NumInGroup FIX.5.0SP2) Number of sub-party IDs to be reported for the party.*/
LegProvisionPartySubID                             = 40538, /*!< 40538 (String FIX.5.0SP2) Party sub-identifier, if applicable, for LegProvisionPartyRole(40536). */
LegProvisionPartySubIDType                         = 40539, /*!< 40539 (int FIX.5.0SP2) The type of LegProvisionPartySubID(40538) value. */
NoUnderlyingStreams                                = 40540, /*!< 40540 (NumInGroup FIX.5.0SP2) Number of swap streams in the repeating group.*/
UnderlyingStreamType                               = 40541, /*!< 40541 (int FIX.5.0SP2) Type of swap stream.*/
UnderlyingStreamDesc                               = 40542, /*!< 40542 (String FIX.5.0SP2) A short descriptive name given to payment stream. Eg. CDS, Fixed, Float, Float2, GBP. The description has no intrinsic meaning but should be arbitrarily chosen by the remitter as a reference.*/
UnderlyingStreamPaySide                            = 40543, /*!< 40543 (int FIX.5.0SP2) The side of the party paying the stream.*/
UnderlyingStreamReceiveSide                        = 40544, /*!< 40544 (int FIX.5.0SP2) The side of the party receiving the stream.*/
UnderlyingStreamNotional                           = 40545, /*!< 40545 (Amt FIX.5.0SP2) Notional, or initial notional value for the payment stream. Use SwapSchedule for steps.*/
UnderlyingStreamCurrency                           = 40546, /*!< 40546 (Currency FIX.5.0SP2) Specifies the currency the UnderlyingStreamNotional(40545) is denominated in.  Uses ISO 4217 currency codes.*/
UnderlyingStreamText                               = 40547, /*!< 40547 (String FIX.5.0SP2) Free form text to specify additional information or enumeration description when a standard value does not apply.*/
UnderlyingStreamTerminationDateUnadjusted          = 40548, /*!< 40548 (LocalMktDate FIX.5.0SP2) The unadjusted termination date.*/
UnderlyingStreamTerminationDateBusinessDayConvention = 40549, /*!< 40549 (int FIX.5.0SP2) The business day convention used to adjust the underlying instrument's stream's termination, or relative termination, date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingStreamTerminationDateBusinessCenter      = 40550, /*!< 40550 (String FIX.5.0SP2) The business center calendar used to adjust the underlying instrument's stream's termination, or relative termination, date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingStreamTerminationDateRelativeTo          = 40551, /*!< 40551 (int FIX.5.0SP2) Specifies the anchor date when the termination date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingStreamTerminationDateOffsetPeriod        = 40552, /*!< 40552 (int FIX.5.0SP2) Time unit multiplier for the relative termination date offset.*/
UnderlyingStreamTerminationDateOffsetUnit          = 40553, /*!< 40553 (String FIX.5.0SP2) Time unit associated with the relative termination date offset.*/
UnderlyingStreamTerminationDateOffsetDayType       = 40554, /*!< 40554 (int FIX.5.0SP2) Specifies the day type of the relative termination date offset.*/
UnderlyingStreamTerminationDateAdjusted            = 40555, /*!< 40555 (LocalMktDate FIX.5.0SP2) The adjusted termination date.*/
UnderlyingStreamCalculationPeriodBusinessDayConvention = 40556, /*!< 40556 (int FIX.5.0SP2) The business day convention used to adjust the calculation periods. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingStreamCalculationPeriodBusinessCenter    = 40557, /*!< 40557 (String FIX.5.0SP2) The business center calendar used to adjust the calculation periods, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingStreamFirstPeriodStartDateUnadjusted     = 40558, /*!< 40558 (LocalMktDate FIX.5.0SP2) The unadjusted first calculation period start date if before the effective date.*/
UnderlyingStreamFirstPeriodStartDateBusinessDayConvention = 40559, /*!< 40559 (int FIX.5.0SP2) The business day convention used to adjust the underlying instrument's stream's first calculation period start date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingStreamFirstPeriodStartDateBusinessCenter = 40560, /*!< 40560 (String FIX.5.0SP2) The business center calendar used to adjust the underlying instrument's stream's first calculation period start date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingStreamFirstPeriodStartDateAdjusted       = 40561, /*!< 40561 (LocalMktDate FIX.5.0SP2) The adjusted first calculation period start date, if it is before the effective date.*/
UnderlyingStreamFirstRegularPeriodStartDateUnadjusted = 40562, /*!< 40562 (LocalMktDate FIX.5.0SP2) The unadjusted first start date of the regular calculation period, if there is an initial stub period.*/
UnderlyingStreamFirstCompoundingPeriodEndDateUnadjusted = 40563, /*!< 40563 (LocalMktDate FIX.5.0SP2) The unadjusted end date of the initial compounding period.*/
UnderlyingStreamLastRegularPeriodEndDateUnadjusted = 40564, /*!< 40564 (LocalMktDate FIX.5.0SP2) The unadjusted last regular period end date if there is a final stub period.*/
UnderlyingStreamCalculationFrequencyPeriod         = 40565, /*!< 40565 (int FIX.5.0SP2) Time unit multiplier for the frequency at which calculation period end dates occur.*/
UnderlyingStreamCalculationFrequencyUnit           = 40566, /*!< 40566 (String FIX.5.0SP2) Time unit associated with the frequency at which calculation period end dates occur.*/
UnderlyingStreamCalculationRollConvention          = 40567, /*!< 40567 (String FIX.5.0SP2) The convention for determining the sequence of end dates. It is used in conjunction with a specified frequency. Used only to override the roll convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentStreamType                        = 40568, /*!< 40568 (int FIX.5.0SP2) Identifies the type of payment stream applicable to the swap stream associated with the underlying instrument. */
UnderlyingPaymentStreamMarketRate                  = 40569, /*!< 40569 (int FIX.5.0SP2) Used only for credit index trade. This contains the credit spread ("fair value") at which the trade was executed. The market rate varies over the life of the index depending on market conditions. This is the price of the index as quoted by trading desks.*/
UnderlyingPaymentStreamDelayIndicator              = 40570, /*!< 40570 (Boolean FIX.5.0SP2) Applicable to credit default swaps on mortgage backed securities to specify whether payment delays are applicable to the fixed amount. 

Residential mortgage backed securities typically have a payment delay of 5 days between the coupon date of the reference obligation and the payment date of the synthetic swap. 

Commercial mortage backed securities  do not typically have a payment delay, with both payment dates (the coupon date of the reference obligation and the payment date of the synthetic swap) being on the 25th of each month.*/
UnderlyingPaymentStreamSettlCurrency               = 40571, /*!< 40571 (Currency FIX.5.0SP2) Specifies the currency that the stream settles in (to support swaps that settle in a currency different from the notional currency).  Uses ISO 4217 currency codes.*/
UnderlyingPaymentStreamDayCount                    = 40572, /*!< 40572 (int FIX.5.0SP2) The day count convention used in the payment stream calculations.*/
UnderlyingPaymentStreamAccrualDays                 = 40573, /*!< 40573 (int FIX.5.0SP2) The number of days from the adjusted calculation period start date to the adjusted value date, calculated in accordance with the applicable day count fraction.*/
UnderlyingPaymentStreamDiscountType                = 40574, /*!< 40574 (int FIX.5.0SP2) The method of calculating discounted payment amounts*/
UnderlyingPaymentStreamDiscountRate                = 40575, /*!< 40575 (Percentage FIX.5.0SP2) Discount rate.  The rate is expressed in decimal, e.g. 5% is expressed as 0.05.*/
UnderlyingPaymentStreamDiscountRateDayCount        = 40576, /*!< 40576 (int FIX.5.0SP2) The day count convention applied to the UnderlyingPaymentStreamDiscountRate(40575).*/
UnderlyingPaymentStreamCompoundingMethod           = 40577, /*!< 40577 (int FIX.5.0SP2) Compounding Method.*/
UnderlyingPaymentStreamInitialPrincipalExchangeIndicator = 40578, /*!< 40578 (Boolean FIX.5.0SP2) Indicates whether there is an initial exchange of principal on the effective date.*/
UnderlyingPaymentStreamInterimPrincipalExchangeIndicator = 40579, /*!< 40579 (Boolean FIX.5.0SP2) Indicates whether there are intermediate or interim exchanges of principal during the term of the swap.*/
UnderlyingPaymentStreamFinalPrincipalExchangeIndicator = 40580, /*!< 40580 (Boolean FIX.5.0SP2) Indicates whether there is a final exchange of principal on the termination date.*/
UnderlyingPaymentStreamPaymentDateBusinessDayConvention = 40581, /*!< 40581 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's payment date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentStreamPaymentDateBusinessCenter   = 40582, /*!< 40582 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's payment date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentStreamPaymentFrequencyPeriod      = 40583, /*!< 40583 (int FIX.5.0SP2) Time unit multiplier for the frequency of payments.*/
UnderlyingPaymentStreamPaymentFrequencyUnit        = 40584, /*!< 40584 (String FIX.5.0SP2) Time unit associated with the frequency of payments.*/
UnderlyingPaymentStreamPaymentRollConvention       = 40585, /*!< 40585 (String FIX.5.0SP2) The convention for determining the sequence of end dates. It is used in conjunction with a specified frequency. Used only to override the roll convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentStreamFirstPaymentDateUnadjusted  = 40586, /*!< 40586 (LocalMktDate FIX.5.0SP2) The unadjusted first payment date.*/
UnderlyingPaymentStreamLastRegularPaymentDateUnadjusted = 40587, /*!< 40587 (LocalMktDate FIX.5.0SP2) The unadjusted last regular payment date.*/
UnderlyingPaymentStreamPaymentDateRelativeTo       = 40588, /*!< 40588 (int FIX.5.0SP2) Specifies the anchor date when payment dates are relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamPaymentDateOffsetPeriod     = 40589, /*!< 40589 (int FIX.5.0SP2) Time unit multiplier for the relative payment date offset.*/
UnderlyingPaymentStreamPaymentDateOffsetUnit       = 40590, /*!< 40590 (String FIX.5.0SP2) Time unit associated with the relative payment date offset.*/
UnderlyingPaymentStreamPaymentDateOffsetDayType    = 40591, /*!< 40591 (int FIX.5.0SP2) Specifies the day type of the relative payment date offset.*/
UnderlyingPaymentStreamResetDateRelativeTo         = 40592, /*!< 40592 (int FIX.5.0SP2) Specifies the anchor date when the reset dates are relative to an anchor date. 

If the reset frequency is specified as daily this element must not be included. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamResetDateBusinessDayConvention = 40593, /*!< 40593 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's reset date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentStreamResetDateBusinessCenter     = 40594, /*!< 40594 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's reset date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentStreamResetFrequencyPeriod        = 40595, /*!< 40595 (int FIX.5.0SP2) Time unit multiplier for frequency of resets.*/
UnderlyingPaymentStreamResetFrequencyUnit          = 40596, /*!< 40596 (String FIX.5.0SP2) Time unit associated with frequency of resets.*/
UnderlyingPaymentStreamResetWeeklyRollConvention   = 40597, /*!< 40597 (String FIX.5.0SP2) Used to specify the day of the week in which the reset occurs for payments that reset on a weekly basis.  */
UnderlyingPaymentStreamInitialFixingDateRelativeTo = 40598, /*!< 40598 (int FIX.5.0SP2) Specifies the anchor date when the initial fixing date is relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamInitialFixingDateBusinessDayConvention = 40599, /*!< 40599 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's initial fixing date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentStreamInitialFixingDateBusinessCenter = 40600, /*!< 40600 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's initial fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentStreamInitialFixingDateOffsetPeriod = 40601, /*!< 40601 (int FIX.5.0SP2) Time unit multiplier for the relative initial fixing date offset.*/
UnderlyingPaymentStreamInitialFixingDateOffsetUnit = 40602, /*!< 40602 (String FIX.5.0SP2) Time unit associated with the relative initial fixing date offset.*/
UnderlyingPaymentStreamInitialFixingDateOffsetDayType = 40603, /*!< 40603 (int FIX.5.0SP2) Specifies the day type of the relative initial fixing date offset.*/
UnderlyingPaymentStreamInitialFixingDateAdjusted   = 40604, /*!< 40604 (LocalMktDate FIX.5.0SP2) The adjusted initial fixing date.*/
UnderlyingPaymentStreamFixingDateRelativeTo        = 40605, /*!< 40605 (int FIX.5.0SP2) Specifies the anchor date when the fixing date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values. */
UnderlyingPaymentStreamFixingDateBusinessDayConvention = 40606, /*!< 40606 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's fixing date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentStreamFixingDateBusinessCenter    = 40607, /*!< 40607 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values. */
UnderlyingPaymentStreamFixingDateOffsetPeriod      = 40608, /*!< 40608 (int FIX.5.0SP2) Time unit multiplier for the relative fixing date offset.*/
UnderlyingPaymentStreamFixingDateOffsetUnit        = 40609, /*!< 40609 (String FIX.5.0SP2) Time unit associated with the relative fixing date offset.*/
UnderlyingPaymentStreamFixingDateOffsetDayType     = 40610, /*!< 40610 (int FIX.5.0SP2) Specifies the day type of the relative fixing date offset.*/
UnderlyingPaymentStreamFixingDateAdjusted          = 40611, /*!< 40611 (LocalMktDate FIX.5.0SP2) The adjusted fixing date.*/
UnderlyingPaymentStreamRateCutoffDateOffsetPeriod  = 40612, /*!< 40612 (int FIX.5.0SP2) Time unit multiplier for the relative rate cut-off date offset.*/
UnderlyingPaymentStreamRateCutoffDateOffsetUnit    = 40613, /*!< 40613 (String FIX.5.0SP2) Time unit associated with the relative rate cut-off date offset.*/
UnderlyingPaymentStreamRateCutoffDateOffsetDayType = 40614, /*!< 40614 (int FIX.5.0SP2) Specifies the day type of the relative rate cut-off date offset.*/
UnderlyingPaymentStreamRate                        = 40615, /*!< 40615 (Percentage FIX.5.0SP2) The rate applicable to the fixed rate payment stream. */
UnderlyingPaymentStreamFixedAmount                 = 40616, /*!< 40616 (Amt FIX.5.0SP2) The underlying payment stream's fixed payment amount. In CDS an alternative to UnderlyingPaymentStreamRate(40615).*/
UnderlyingPaymentStreamRateOrAmountCurrency        = 40617, /*!< 40617 (Currency FIX.5.0SP2) Specifies the currency in which UnderlyingPaymentStreamFixedAmount(40616) or UnderlyingPaymentStreamRate(40615) is denominated.  Users ISO 4271 currency codes.*/
UnderlyingPaymentStreamFutureValueNotional         = 40618, /*!< 40618 (Amt FIX.5.0SP2) The future value notional is normally only required for certain non-deliverable interest rate swaps (e.g. Brazillian Real (BRL) vs. CETIP Interbank Deposit Rate (CDI)). The value is calculated as follows: Future Value Notional = Notional Amount * (1 + Fixed Rate) ^ (Fixed Rate Day Count Fraction). The currency is the same as the stream notional.*/
UnderlyingPaymentStreamFutureValueDateAdjusted     = 40619, /*!< 40619 (LocalMktDate FIX.5.0SP2) The adjusted value date of the future value amount.*/
UnderlyingPaymentStreamRateIndex                   = 40620, /*!< 40620 (String FIX.5.0SP2) The payment stream's floating rate index.*/
UnderlyingPaymentStreamRateIndexSource             = 40621, /*!< 40621 (int FIX.5.0SP2) The source of the payment stream floating rate index.*/
UnderlyingPaymentStreamRateIndexCurveUnit          = 40622, /*!< 40622 (String FIX.5.0SP2) Time unit associated with the underlying instrument’s floating rate index.*/
UnderlyingPaymentStreamRateIndexCurvePeriod        = 40623, /*!< 40623 (int FIX.5.0SP2) Time unit multiplier for the underlying instrument’s floating rate index.*/
UnderlyingPaymentStreamRateMultiplier              = 40624, /*!< 40624 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. A multiplier schedule is expressed as explicit multipliers and dates. In the case of a schedule, the step dates may be subject to adjustment in accordance with any adjustments specified in the calculationPeriodDatesAdjustments. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
UnderlyingPaymentStreamRateSpread                  = 40625, /*!< 40625 (PriceOffset FIX.5.0SP2) Spread from floating rate index.*/
UnderlyingPaymentStreamRateSpreadPositionType      = 40626, /*!< 40626 (int FIX.5.0SP2) Identifies a short or long spread value.*/
UnderlyingPaymentStreamRateTreatment               = 40627, /*!< 40627 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
UnderlyingPaymentStreamCapRate                     = 40628, /*!< 40628 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
UnderlyingPaymentStreamCapRateBuySide              = 40629, /*!< 40629 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
UnderlyingPaymentStreamCapRateSellSide             = 40630, /*!< 40630 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
UnderlyingPaymentStreamFloorRate                   = 40631, /*!< 40631 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.*/
UnderlyingPaymentStreamFloorRateBuySide            = 40632, /*!< 40632 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
UnderlyingPaymentStreamFloorRateSellSide           = 40633, /*!< 40633 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
UnderlyingPaymentStreamInitialRate                 = 40634, /*!< 40634 (Percentage FIX.5.0SP2) The initial floating rate reset agreed between the principal parties involved in the trade. This is assumed to be the first required reset rate for the first regular calculation period. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. An initial rate of 5% would be represented as 0.05.*/
UnderlyingPaymentStreamFinalRateRoundingDirection  = 40635, /*!< 40635 (char FIX.5.0SP2) Specifies the rounding direction.*/
UnderlyingPaymentStreamFinalRatePrecision          = 40636, /*!< 40636 (int FIX.5.0SP2) Specifies the rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
UnderlyingPaymentStreamAveragingMethod             = 40637, /*!< 40637 (int FIX.5.0SP2) When rate averaging is applicable, used to specify whether a weighted or unweighted average calculation method is to be used. */
UnderlyingPaymentStreamNegativeRateTreatment       = 40638, /*!< 40638 (int FIX.5.0SP2) The specification of any provisions for calculating payment obligations when a floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
UnderlyingPaymentStreamInflationLagPeriod          = 40639, /*!< 40639 (int FIX.5.0SP2) Time unit multiplier for the inflation lag period.  The lag period is the offsetting period from the payment date which determines the reference period for which the inflation index is observed.*/
UnderlyingPaymentStreamInflationLagUnit            = 40640, /*!< 40640 (String FIX.5.0SP2) Time unit associated with the inflation lag period.*/
UnderlyingPaymentStreamInflationLagDayType         = 40641, /*!< 40641 (int FIX.5.0SP2) The inflation lag period day type.*/
UnderlyingPaymentStreamInflationInterpolationMethod = 40642, /*!< 40642 (int FIX.5.0SP2) The method used when calculating the Inflation Index Level from multiple points - the most common is Linear.*/
UnderlyingPaymentStreamInflationIndexSource        = 40643, /*!< 40643 (int FIX.5.0SP2) The inflation index reference source. */
UnderlyingPaymentStreamInflationPublicationSource  = 40644, /*!< 40644 (String FIX.5.0SP2) The current main publication source such as relevant web site or a government body.*/
UnderlyingPaymentStreamInflationInitialIndexLevel  = 40645, /*!< 40645 (float FIX.5.0SP2) Initial known index level for the first calculation period.*/
UnderlyingPaymentStreamInflationFallbackBondApplicable = 40646, /*!< 40646 (Boolean FIX.5.0SP2) Indicates whether a fallback bond as defined in the 2006 ISDA Inflation Derivatives Definitions, sections 1.3 and 1.8, is applicable or not.  If not specified, the default value is "Y" (True/Yes).*/
UnderlyingPaymentStreamFRADiscounting              = 40647, /*!< 40647 (int FIX.5.0SP2) The method of Forward Rate Agreement (FRA) discounting, if any, that will apply.*/
UnderlyingPaymentStreamNonDeliverableRefCurrency   = 40648, /*!< 40648 (Currency FIX.5.0SP2) The non-deliverable settlement reference currency.  Uses ISO 4217 currency codes.*/
UnderlyingPaymentStreamNonDeliverableFixingDatesBusinessDayConvention = 40649, /*!< 40649 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's fixing date for the non-deliverable terms. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentStreamNonDeliverableFixingDatesBusinessCenter = 40650, /*!< 40650 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's fixing date for the non-deliverable terms, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentStreamNonDeliverableFixingDatesRelativeTo = 40651, /*!< 40651 (int FIX.5.0SP2) Specifies the anchor date when the non-deliverable fixing dates are relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetPeriod = 40652, /*!< 40652 (int FIX.5.0SP2) Time unit multiplier for the relative non-deliverable fixing date offset.*/
UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetUnit = 40653, /*!< 40653 (String FIX.5.0SP2) Time unit associated with the relative non-deliverable fixing date offset.*/
UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetDayType = 40654, /*!< 40654 (int FIX.5.0SP2) Specifies the day type of the relative non-deliverable fixing date offset.*/
SettlRateFallbackReferencePage                     = 40655, /*!< 40655 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

When SettlRateFallbackRateSource(40373) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
NoUnderlyingNonDeliverableFixingDates              = 40656, /*!< 40656 (NumInGroup FIX.5.0SP2) Number of Fixing dates in the repeating group*/
UnderlyingNonDeliverableFixingDate                 = 40657, /*!< 40657 (LocalMktDate FIX.5.0SP2) The non-deliverable fixing date unadjusted or adjusted depending on UnderlyingNonDeliverableFixingDateType(40658).*/
UnderlyingNonDeliverableFixingDateType             = 40658, /*!< 40658 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
NoUnderlyingSettlRateFallbacks                     = 40659, /*!< 40659 (NumInGroup FIX.5.0SP2) Number of settlement rate fallbacks in the repeating group*/
UnderlyingSettlRatePostponementMaximumDays         = 40660, /*!< 40660 (int FIX.5.0SP2) The maximum number of days to wait for a quote from the disrupted settlement rate option before proceding to this method.*/
UnderlyingPaymentStreamNonDeliverableSettlRateSource = 40661, /*!< 40661 (int FIX.5.0SP2) Identifies the source of rate information.*/
UnderlyingSettlRatePostponementSurvey              = 40662, /*!< 40662 (Boolean FIX.5.0SP2) Indicates whether to request a settlement rate quote from the market.*/
UnderlyingSettlRatePostponementCalculationAgent    = 40663, /*!< 40663 (int FIX.5.0SP2) Used to identify the settlement rate postponement calculation agent.*/
NoUnderlyingPaymentSchedules                       = 40664, /*!< 40664 (NumInGroup FIX.5.0SP2) Number of swap schedules in the repeating group*/
UnderlyingPaymentScheduleType                      = 40665, /*!< 40665 (int FIX.5.0SP2) Type of schedule. */
UnderlyingPaymentScheduleStubType                  = 40666, /*!< 40666 (int FIX.5.0SP2) Indicates to which stub this schedule applies. */
UnderlyingPaymentScheduleStartDateUnadjusted       = 40667, /*!< 40667 (LocalMktDate FIX.5.0SP2) The unadjusted date on which the value is adjusted, or calculated if a future value notional for certain non-deliverable interest rate swaps (e.g. Brazillian Real (BRL) vs. CETIP Interbank Deposit Rate (CDI)), or the start date of a cashflow payment.*/
UnderlyingPaymentScheduleEndDateUnadjusted         = 40668, /*!< 40668 (LocalMktDate FIX.5.0SP2) The unadjusted end date of a cashflow payment.*/
UnderlyingPaymentSchedulePaySide                   = 40669, /*!< 40669 (int FIX.5.0SP2) The side of the party paying the step schedule.*/
UnderlyingPaymentScheduleReceiveSide               = 40670, /*!< 40670 (int FIX.5.0SP2) The side of the party receiving the step schedule.*/
UnderlyingPaymentScheduleNotional                  = 40671, /*!< 40671 (Amt FIX.5.0SP2) The notional value for this step, or amount of a cashflow payment.*/
UnderlyingPaymentScheduleCurrency                  = 40672, /*!< 40672 (Currency FIX.5.0SP2) The currency for this step.  Uses ISO 4217 currency codes.*/
UnderlyingPaymentScheduleRate                      = 40673, /*!< 40673 (Percentage FIX.5.0SP2) The rate value for this step.*/
UnderlyingPaymentScheduleRateMultiplier            = 40674, /*!< 40674 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
UnderlyingPaymentScheduleRateSpread                = 40675, /*!< 40675 (PriceOffset FIX.5.0SP2) The spread value for this step.*/
UnderlyingPaymentScheduleRateSpreadPositionType    = 40676, /*!< 40676 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
UnderlyingPaymentScheduleRateTreatment             = 40677, /*!< 40677 (int FIX.5.0SP2) Specifies the yield calculation treatment for the step schedule.*/
UnderlyingPaymentScheduleFixedAmount               = 40678, /*!< 40678 (Amt FIX.5.0SP2) The explicit payment amount for this step.*/
UnderlyingPaymentScheduleFixedCurrency             = 40679, /*!< 40679 (Currency FIX.5.0SP2) The currency of the fixed amount.  Uses ISO 4217 currency codes.*/
UnderlyingPaymentScheduleStepFrequencyPeriod       = 40680, /*!< 40680 (int FIX.5.0SP2) Time unit multiplier for the step frequency.*/
UnderlyingPaymentScheduleStepFrequencyUnit         = 40681, /*!< 40681 (String FIX.5.0SP2) Time unit associated with the step frequency. */
UnderlyingPaymentScheduleStepOffsetValue           = 40682, /*!< 40682 (Amt FIX.5.0SP2) The explicit amount that the notional changes on each step date. This can be a positive or negative amount.*/
UnderlyingPaymentScheduleStepRate                  = 40683, /*!< 40683 (Percentage FIX.5.0SP2) The percentage by which the notional changes on each step date. The percentage is either a percentage applied to the initial notional amount or the previous outstanding notional, depending on the value specified in UnderlyingPaymentScheduleStepRelativeTo(40685). The percentage can be either positive or negative.*/
UnderlyingPaymentScheduleStepOffsetRate            = 40684, /*!< 40684 (Percentage FIX.5.0SP2) The explicit amount that the rate changes on each step date. This can be a positive or negative value.*/
UnderlyingPaymentScheduleStepRelativeTo            = 40685, /*!< 40685 (int FIX.5.0SP2) Specifies whether the UnderlyingPaymentScheduleStepRate(40683) or UnderlyingPaymentScheduleStepOffsetValue(40682) should be applied to the initial notional or the previous notional in order to calculate the notional step change amount.*/
UnderlyingPaymentScheduleFixingDateUnadjusted      = 40686, /*!< 40686 (LocalMktDate FIX.5.0SP2) The unadjusted fixing date.*/
UnderlyingPaymentScheduleWeight                    = 40687, /*!< 40687 (float FIX.5.0SP2) Floating rate observation weight for cashflow payment.*/
UnderlyingPaymentScheduleFixingDateRelativeTo      = 40688, /*!< 40688 (int FIX.5.0SP2) Specifies the anchor date when the fixing date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values. */
UnderlyingPaymentScheduleFixingDateBusinessDayCnvtn = 40689, /*!< 40689 (int FIX.5.0SP2) The business day convention used to adjust the payment schedule's fixing date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentScheduleFixingDateBusinessCenter  = 40690, /*!< 40690 (String FIX.5.0SP2) The business center calendar used to adjust the payment schedule's fixing date, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentScheduleFixingDateOffsetPeriod    = 40691, /*!< 40691 (int FIX.5.0SP2) Time unit multiplier for the relative fixing date offset.*/
UnderlyingPaymentScheduleFixingDateOffsetUnit      = 40692, /*!< 40692 (String FIX.5.0SP2) Time unit associated with the relative fixing date offset.*/
UnderlyingPaymentScheduleFixingDateOffsetDayType   = 40693, /*!< 40693 (int FIX.5.0SP2) Specifies the day type of the relative fixing date offset.*/
UnderlyingPaymentScheduleFixingDateAdjusted        = 40694, /*!< 40694 (LocalMktDate FIX.5.0SP2) The adjusted fixing date.*/
UnderlyingPaymentScheduleFixingTime                = 40695, /*!< 40695 (LocalMktTime FIX.5.0SP2) The fixing time.*/
UnderlyingPaymentScheduleFixingTimeBusinessCenter  = 40696, /*!< 40696 (String FIX.5.0SP2) Business center for determining fixing time. See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentScheduleInterimExchangePaymentDateRelativeTo = 40697, /*!< 40697 (int FIX.5.0SP2) Specifies the anchor date when the interim exchange payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentScheduleInterimExchangeDatesBusinessDayConvention = 40698, /*!< 40698 (int FIX.5.0SP2) The business day convention used to adjust the payment schedule's interim exchange date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPaymentScheduleInterimExchangeDatesBusinessCenter = 40699, /*!< 40699 (String FIX.5.0SP2) The business center calendar used to adjust the payment schedule's interim exchange date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentScheduleInterimExchangeDatesOffsetPeriod = 40700, /*!< 40700 (int FIX.5.0SP2) Time unit multiplier for the relative interim exchange date offset.*/
UnderlyingPaymentScheduleInterimExchangeDatesOffsetUnit = 40701, /*!< 40701 (String FIX.5.0SP2) Time unit associated with the relative interim exchange date offset.*/
UnderlyingPaymentScheduleInterimExchangeDatesOffsetDayType = 40702, /*!< 40702 (int FIX.5.0SP2) Specifies the day type of the relative interim exchange date offset.*/
UnderlyingPaymentScheduleInterimExchangeDateAdjusted = 40703, /*!< 40703 (LocalMktDate FIX.5.0SP2) The adjusted interim exchange date.*/
NoUnderlyingPaymentScheduleRateSources             = 40704, /*!< 40704 (NumInGroup FIX.5.0SP2) Number of rate sources in the repeating group*/
UnderlyingPaymentScheduleRateSource                = 40705, /*!< 40705 (int FIX.5.0SP2) Identifies the source of rate information.*/
UnderlyingPaymentScheduleRateSourceType            = 40706, /*!< 40706 (int FIX.5.0SP2) Rate source type.*/
UnderlyingPaymentScheduleReferencePage             = 40707, /*!< 40707 (String FIX.5.0SP2) Identifies the reference “page” from the rate source. 



For FX, the reference page to the spot rate to be used for the reference FX spot rate.



When RateSource(1446) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
NoUnderlyingPaymentStubs                           = 40708, /*!< 40708 (NumInGroup FIX.5.0SP2) Number of stubs in the repeating group*/
UnderlyingPaymentStubType                          = 40709, /*!< 40709 (int FIX.5.0SP2) Stub type.*/
UnderlyingPaymentStubLength                        = 40710, /*!< 40710 (int FIX.5.0SP2) Optional indication whether stub is shorter or longer than the regular swap period.*/
UnderlyingPaymentStubRate                          = 40711, /*!< 40711 (Percentage FIX.5.0SP2) The agreed upon fixed rate for this stub.*/
UnderlyingPaymentStubFixedAmount                   = 40712, /*!< 40712 (Amt FIX.5.0SP2) A fixed payment amount for the stub.*/
UnderlyingPaymentStubFixedCurrency                 = 40713, /*!< 40713 (Currency FIX.5.0SP2) The currency of the fixed payment amount.  Uses ISO 4217 currency codes.*/
UnderlyingPaymentStubIndex                         = 40714, /*!< 40714 (String FIX.5.0SP2) The stub floating rate index.*/
UnderlyingPaymentStubIndexSource                   = 40715, /*!< 40715 (int FIX.5.0SP2) The source for the underlying payment stub floating rate index.*/
UnderlyingPaymentStubIndexCurvePeriod              = 40716, /*!< 40716 (int FIX.5.0SP2) Time unit multiplier for the underlying payment stub floating rate index.*/
UnderlyingPaymentStubIndexCurveUnit                = 40717, /*!< 40717 (String FIX.5.0SP2) Time unit associated with the underlying payment stub floating rate index.*/
UnderlyingPaymentStubIndexRateMultiplier           = 40718, /*!< 40718 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
UnderlyingPaymentStubIndexRateSpread               = 40719, /*!< 40719 (PriceOffset FIX.5.0SP2) Spread from floating rate index.*/
UnderlyingPaymentStubIndexRateSpreadPositionType   = 40720, /*!< 40720 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position. */
UnderlyingPaymentStubIndexRateTreatment            = 40721, /*!< 40721 (int FIX.5.0SP2) Specifies the yield calculation treatment for the stub index.*/
UnderlyingPaymentStubIndexCapRate                  = 40722, /*!< 40722 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
UnderlyingPaymentStubIndexCapRateBuySide           = 40723, /*!< 40723 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
UnderlyingPaymentStubIndexCapRateSellSide          = 40724, /*!< 40724 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
UnderlyingPaymentStubIndexFloorRate                = 40725, /*!< 40725 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.*/
UnderlyingPaymentStubIndexFloorRateBuySide         = 40726, /*!< 40726 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
UnderlyingPaymentStubIndexFloorRateSellSide        = 40727, /*!< 40727 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
UnderlyingPaymentStubIndex2                        = 40728, /*!< 40728 (String FIX.5.0SP2) The second stub floating rate index.*/
UnderlyingPaymentStubIndex2Source                  = 40729, /*!< 40729 (int FIX.5.0SP2) The source of the second stub floating rate index.*/
UnderlyingPaymentStubIndex2CurvePeriod             = 40730, /*!< 40730 (int FIX.5.0SP2) Secondary time unit multiplier for the stub floating rate index curve.*/
UnderlyingPaymentStubIndex2CurveUnit               = 40731, /*!< 40731 (String FIX.5.0SP2) Secondary time unit associated with the stub floating rate index curve.*/
UnderlyingPaymentStubIndex2RateMultiplier          = 40732, /*!< 40732 (float FIX.5.0SP2) A rate multiplier to apply to the second floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
UnderlyingPaymentStubIndex2RateSpread              = 40733, /*!< 40733 (PriceOffset FIX.5.0SP2) Spread from the second floating rate index.*/
UnderlyingPaymentStubIndex2RateSpreadPositionType  = 40734, /*!< 40734 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position. */
UnderlyingPaymentStubIndex2RateTreatment           = 40735, /*!< 40735 (int FIX.5.0SP2) Specifies the yield calculation treatment for the second stub index.*/
UnderlyingPaymentStubIndex2CapRate                 = 40736, /*!< 40736 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the second floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
UnderlyingPaymentStubIndex2FloorRate               = 40737, /*!< 40737 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the second floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.*/
PaymentStreamType                                  = 40738, /*!< 40738 (int FIX.5.0SP2) Identifies the type of payment stream associated with the swap.*/
PaymentStreamMarketRate                            = 40739, /*!< 40739 (int FIX.5.0SP2) Used only for credit index trade. This contains the credit spread ("fair value") at which the trade was executed. The market rate varies over the life of the index depending on market conditions. This is the price of the index as quoted by trading desks.*/
PaymentStreamDelayIndicator                        = 40740, /*!< 40740 (Boolean FIX.5.0SP2) Applicable to credit default swaps on mortgage backed securities to specify whether payment delays are applicable to the fixed amount. 

Residential mortgage backed securities typically have a payment delay of 5 days between the coupon date of the reference obligation and the payment date of the synthetic swap. 

Commercial mortgage backed securities do not typically have a payment delay,  with both payment dates (the coupon date of the reference obligation and the payment date of the synthetic swap) being on the 25th of each month.*/
PaymentStreamSettlCurrency                         = 40741, /*!< 40741 (Currency FIX.5.0SP2) Specifies the currency that the stream settles in (to support swaps that settle in a currency different from the notional currency).  Uses ISO 4217 currency codes.*/
PaymentStreamDayCount                              = 40742, /*!< 40742 (int FIX.5.0SP2) The day count convention used in the payment stream calculations.*/
PaymentStreamAccrualDays                           = 40743, /*!< 40743 (int FIX.5.0SP2) The number of days from the adjusted calculation period start date to the adjusted value date, calculated in accordance with the applicable day count fraction.*/
PaymentStreamDiscountType                          = 40744, /*!< 40744 (int FIX.5.0SP2) The method of calculating discounted payment amounts*/
PaymentStreamDiscountRate                          = 40745, /*!< 40745 (Percentage FIX.5.0SP2) Discount rate.  The rate is expressed in decimal, e.g. 5% is expressed as 0.05.*/
PaymentStreamDiscountRateDayCount                  = 40746, /*!< 40746 (int FIX.5.0SP2) The day count convention applied to the PaymentStreamDiscountRate(40745). */
PaymentStreamCompoundingMethod                     = 40747, /*!< 40747 (int FIX.5.0SP2) Compounding method.*/
PaymentStreamInitialPrincipalExchangeIndicator     = 40748, /*!< 40748 (Boolean FIX.5.0SP2) Indicates whether there is an initial exchange of principal on the effective date.*/
PaymentStreamInterimPrincipalExchangeIndicator     = 40749, /*!< 40749 (Boolean FIX.5.0SP2) Indicates whether there are intermediate or interim exchanges of principal during the term of the swap.*/
PaymentStreamFinalPrincipalExchangeIndicator       = 40750, /*!< 40750 (Boolean FIX.5.0SP2) Indicates whether there is a final exchange of principal on the termination date.*/
PaymentStreamPaymentDateBusinessDayConvention      = 40751, /*!< 40751 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's payment date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
PaymentStreamPaymentDateBusinessCenter             = 40752, /*!< 40752 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's payment date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentStreamPaymentFrequencyPeriod                = 40753, /*!< 40753 (int FIX.5.0SP2) Time unit multiplier for the frequency of payments.*/
PaymentStreamPaymentFrequencyUnit                  = 40754, /*!< 40754 (String FIX.5.0SP2) Time unit associated with the frequency of payments. */
PaymentStreamPaymentRollConvention                 = 40755, /*!< 40755 (String FIX.5.0SP2) The convention for determining the sequence of end dates. It is used in conjunction with a specified frequency. Used only to override the roll convention specified in the DateAdjustment component within the Instrument component.*/
PaymentStreamFirstPaymentDateUnadjusted            = 40756, /*!< 40756 (LocalMktDate FIX.5.0SP2) The unadjusted first payment date.*/
PaymentStreamLastRegularPaymentDateUnadjusted      = 40757, /*!< 40757 (LocalMktDate FIX.5.0SP2) The unadjusted last regular payment date.*/
PaymentStreamPaymentDateRelativeTo                 = 40758, /*!< 40758 (int FIX.5.0SP2) Specifies the anchor date when payment dates are relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamPaymentDateOffsetPeriod               = 40759, /*!< 40759 (int FIX.5.0SP2) Time unit multiplier for the relative payment date offset.*/
PaymentStreamPaymentDateOffsetUnit                 = 40760, /*!< 40760 (String FIX.5.0SP2) Time unit multiplier for the relative initial fixing date offset.*/
PaymentStreamResetDateRelativeTo                   = 40761, /*!< 40761 (int FIX.5.0SP2) Specifies the anchor date when the reset dates are relative to an anchor date. 

If the reset frequency is specified as daily this element must not be included. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamResetDateBusinessDayConvention        = 40762, /*!< 40762 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's reset date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
PaymentStreamResetDateBusinessCenter               = 40763, /*!< 40763 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's reset date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentStreamResetFrequencyPeriod                  = 40764, /*!< 40764 (int FIX.5.0SP2) Time unit multiplier for the frequency of resets.*/
PaymentStreamResetFrequencyUnit                    = 40765, /*!< 40765 (String FIX.5.0SP2) Time unit associated with the frequency of resets.*/
PaymentStreamResetWeeklyRollConvention             = 40766, /*!< 40766 (String FIX.5.0SP2) Used to specify the day of the week in which the reset occurs for payments that reset on a weekly basis.  */
PaymentStreamInitialFixingDateRelativeTo           = 40767, /*!< 40767 (int FIX.5.0SP2) Specifies the anchor date when the initial fixing date is relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamInitialFixingDateBusinessDayConvention = 40768, /*!< 40768 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's initial fixing date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
PaymentStreamInitialFixingDateBusinessCenter       = 40769, /*!< 40769 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's initial fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentStreamInitialFixingDateOffsetPeriod         = 40770, /*!< 40770 (int FIX.5.0SP2) Time unit multiplier for the relative initial fixing date offset.*/
PaymentStreamInitialFixingDateOffsetUnit           = 40771, /*!< 40771 (String FIX.5.0SP2) Time unit associated with the relative initial fixing date offset.*/
PaymentStreamInitialFixingDateOffsetDayType        = 40772, /*!< 40772 (int FIX.5.0SP2) Specifies the day type of the relative initial fixing date offset.*/
PaymentStreamInitialFixingDateAdjusted             = 40773, /*!< 40773 (LocalMktDate FIX.5.0SP2) The adjusted initial fixing date.*/
PaymentStreamFixingDateRelativeTo                  = 40774, /*!< 40774 (int FIX.5.0SP2) Specifies the anchor date when the fixing date is relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamFixingDateBusinessDayConvention       = 40775, /*!< 40775 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's fixing date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
PaymentStreamFixingDateBusinessCenter              = 40776, /*!< 40776 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentStreamFixingDateOffsetPeriod                = 40777, /*!< 40777 (int FIX.5.0SP2) Time unit multiplier for the relative fixing date offset.*/
PaymentStreamFixingDateOffsetUnit                  = 40778, /*!< 40778 (String FIX.5.0SP2) Time unit associated with the relative fixing date offset.*/
PaymentStreamFixingDateOffsetDayType               = 40779, /*!< 40779 (int FIX.5.0SP2) Specifies the day type of the relative fixing date offset.*/
PaymentStreamFixingDateAdjusted                    = 40780, /*!< 40780 (LocalMktDate FIX.5.0SP2) The adjusted fixing date.*/
PaymentStreamRateCutoffDateOffsetPeriod            = 40781, /*!< 40781 (int FIX.5.0SP2) Time unit multiplier for the relative rate cut-off date offset.*/
PaymentStreamRateCutoffDateOffsetUnit              = 40782, /*!< 40782 (String FIX.5.0SP2) Time unit associated with the relative rate cut-off date offset.*/
PaymentStreamRateCutoffDateOffsetDayType           = 40783, /*!< 40783 (int FIX.5.0SP2) Specifies the day type of the relative rate cut-off date offset.*/
PaymentStreamRate                                  = 40784, /*!< 40784 (Percentage FIX.5.0SP2) The rate applicable to the fixed rate payment stream. */
PaymentStreamFixedAmount                           = 40785, /*!< 40785 (Amt FIX.5.0SP2) The payment stream's fixed payment amount. In CDS an alternative to PaymentStreamRate(40784).*/
PaymentStreamRateOrAmountCurrency                  = 40786, /*!< 40786 (Currency FIX.5.0SP2) Specifies the currency in which PaymentStreamFixedAmount(40785) or PaymentStreamRate(40784) is denominated.  Uses ISO 4271 currency codes.*/
PaymentStreamFutureValueNotional                   = 40787, /*!< 40787 (Amt FIX.5.0SP2) The future value notional is normally only required for certain non-deliverable interest rate swaps (e.g. Brazillian Real (BRL) vs. CETIP Interbank Deposit Rate (CDI)). The value is calculated as follows: Future Value Notional = Notional Amount * (1 + Fixed Rate) ^ (Fixed Rate Day Count Fraction). The currency is the same as the stream notional.*/
PaymentStreamFutureValueDateAdjusted               = 40788, /*!< 40788 (LocalMktDate FIX.5.0SP2) The adjusted value date of the future value amount.*/
PaymentStreamRateIndex                             = 40789, /*!< 40789 (String FIX.5.0SP2) The payment stream floating rate index.*/
PaymentStreamRateIndexSource                       = 40790, /*!< 40790 (int FIX.5.0SP2) The source of the payment stream floating rate index.*/
PaymentStreamRateIndexCurveUnit                    = 40791, /*!< 40791 (String FIX.5.0SP2) Time unit associated with the floating rate index.*/
PaymentStreamRateIndexCurvePeriod                  = 40792, /*!< 40792 (int FIX.5.0SP2) Time unit multiplier for the floating rate index.*/
PaymentStreamRateMultiplier                        = 40793, /*!< 40793 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. A multiplier schedule is expressed as explicit multipliers and dates. In the case of a schedule, the step dates may be subject to adjustment in accordance with any adjustments specified in the calculationPeriodDatesAdjustments. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
PaymentStreamRateSpread                            = 40794, /*!< 40794 (PriceOffset FIX.5.0SP2) Spread from floating rate index.*/
PaymentStreamRateSpreadPositionType                = 40795, /*!< 40795 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
PaymentStreamRateTreatment                         = 40796, /*!< 40796 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
PaymentStreamCapRate                               = 40797, /*!< 40797 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
PaymentStreamCapRateBuySide                        = 40798, /*!< 40798 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
PaymentStreamCapRateSellSide                       = 40799, /*!< 40799 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
PaymentStreamFloorRate                             = 40800, /*!< 40800 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.*/
PaymentStreamFloorRateBuySide                      = 40801, /*!< 40801 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
PaymentStreamFloorRateSellSide                     = 40802, /*!< 40802 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
PaymentStreamInitialRate                           = 40803, /*!< 40803 (Percentage FIX.5.0SP2) The initial floating rate reset agreed between the principal parties involved in the trade. This is assumed to be the first required reset rate for the first regular calculation period. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. An initial rate of 5% would be represented as 0.05.*/
PaymentStreamFinalRateRoundingDirection            = 40804, /*!< 40804 (char FIX.5.0SP2) Specifies the rounding direction.*/
PaymentStreamFinalRatePrecision                    = 40805, /*!< 40805 (int FIX.5.0SP2) Specifies the rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
PaymentStreamAveragingMethod                       = 40806, /*!< 40806 (int FIX.5.0SP2) When rate averaging is applicable, used to specify whether a weighted or unweighted average calculation method is to be used.*/
PaymentStreamNegativeRateTreatment                 = 40807, /*!< 40807 (int FIX.5.0SP2) The specification of any provisions for calculating payment obligations when a floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
PaymentStreamInflationLagPeriod                    = 40808, /*!< 40808 (int FIX.5.0SP2) Time unit multiplier for the inflation lag period.  The lag period is the offsetting period from the payment date which determines the reference period for which the inflation index is observed.*/
PaymentStreamInflationLagUnit                      = 40809, /*!< 40809 (String FIX.5.0SP2) Time unit associated with the inflation lag period.*/
PaymentStreamInflationLagDayType                   = 40810, /*!< 40810 (int FIX.5.0SP2) The inflation lag period day type.*/
PaymentStreamInflationInterpolationMethod          = 40811, /*!< 40811 (int FIX.5.0SP2) The method used when calculating the Inflation Index Level from multiple points - the most common is Linear.*/
PaymentStreamInflationIndexSource                  = 40812, /*!< 40812 (int FIX.5.0SP2) The inflation index reference source. */
PaymentStreamInflationPublicationSource            = 40813, /*!< 40813 (String FIX.5.0SP2) The current main publication source such as relevant web site or a government body.*/
PaymentStreamInflationInitialIndexLevel            = 40814, /*!< 40814 (float FIX.5.0SP2) Initial known index level for the first calculation period.*/
PaymentStreamInflationFallbackBondApplicable       = 40815, /*!< 40815 (Boolean FIX.5.0SP2) Indicates whether a fallback bond as defined in the 2006 ISDA Inflation Derivatives Definitions, sections 1.3 and 1.8, is applicable or not.  If not specified, the default value is "Y" (True/Yes).*/
PaymentStreamFRADiscounting                        = 40816, /*!< 40816 (int FIX.5.0SP2) The method of Forward Rate Agreement (FRA) discounting, if any, that will apply.*/
PaymentStreamNonDeliverableRefCurrency             = 40817, /*!< 40817 (Currency FIX.5.0SP2) The non-deliverable settlement reference currency.  Uses ISO 4217 currency codes.*/
PaymentStreamNonDeliverableFixingDatesBusinessDayConvention = 40818, /*!< 40818 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's fixing date for the non-deliverable settlement terms. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component*/
PaymentStreamNonDeliverableFixingDatesBusinessCenter = 40819, /*!< 40819 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's fixing date for the non-deliverable terms, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentStreamNonDeliverableFixingDatesRelativeTo   = 40820, /*!< 40820 (int FIX.5.0SP2) Specifies the anchor date when the non-deliverable fixing dates are relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamNonDeliverableFixingDatesOffsetPeriod = 40821, /*!< 40821 (int FIX.5.0SP2) Time unit multiplier for the relative non-deliverable fixing date offset.*/
PaymentStreamNonDeliverableFixingDatesOffsetUnit   = 40822, /*!< 40822 (String FIX.5.0SP2) Time unit associated with the relative non-deliverable fixing date offset.*/
PaymentStreamNonDeliverableFixingDatesOffsetDayType = 40823, /*!< 40823 (int FIX.5.0SP2) Specifies the day type of the relative non-deliverable fixing date offset.*/
UnderlyingPaymentStreamNonDeliverableSettlReferencePage = 40824, /*!< 40824 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

When UnderlyingPaymentStreamNonDeliverableSettlRateSource(40661) = 3(ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
NoNonDeliverableFixingDates                        = 40825, /*!< 40825 (NumInGroup FIX.5.0SP2) Number of Fixing dates in the repeating group*/
NonDeliverableFixingDate                           = 40826, /*!< 40826 (LocalMktDate FIX.5.0SP2) Non-deliverable fixing date unadjusted or adjusted depending on NonDeliverableFixingDateType(40827).*/
NonDeliverableFixingDateType                       = 40827, /*!< 40827 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
NoPaymentSchedules                                 = 40828, /*!< 40828 (NumInGroup FIX.5.0SP2) Number of swap schedules in the repeating group*/
PaymentScheduleType                                = 40829, /*!< 40829 (int FIX.5.0SP2) Type of schedule.*/
PaymentScheduleStubType                            = 40830, /*!< 40830 (int FIX.5.0SP2) Indicates to which stub this schedule applies. */
PaymentScheduleStartDateUnadjusted                 = 40831, /*!< 40831 (LocalMktDate FIX.5.0SP2) The date on which the value is adjusted, or calculated if a future value notional for certain non-deliverable interest rate swaps (e.g. Brazillian Real (BRL) vs. CETIP Interbank Deposit Rate (CDI)), or the start date of a cashflow payment.*/
PaymentScheduleEndDateUnadjusted                   = 40832, /*!< 40832 (LocalMktDate FIX.5.0SP2) The unadjusted end date of a cash flow payment.*/
PaymentSchedulePaySide                             = 40833, /*!< 40833 (int FIX.5.0SP2) The side of the party paying the step schedule.*/
PaymentScheduleReceiveSide                         = 40834, /*!< 40834 (int FIX.5.0SP2) The side of the party receiving the stepf schedule.*/
PaymentScheduleNotional                            = 40835, /*!< 40835 (Amt FIX.5.0SP2) The notional value for this step, or amount of a cashflow payment.*/
PaymentScheduleCurrency                            = 40836, /*!< 40836 (Currency FIX.5.0SP2) The currency for this step.  Uses ISO 4217 currency codes.*/
PaymentScheduleRate                                = 40837, /*!< 40837 (Percentage FIX.5.0SP2) The rate value for this step schedule.*/
PaymentScheduleRateMultiplier                      = 40838, /*!< 40838 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
PaymentScheduleRateSpread                          = 40839, /*!< 40839 (PriceOffset FIX.5.0SP2) The spread value for this step schedule.*/
PaymentScheduleRateSpreadPositionType              = 40840, /*!< 40840 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
PaymentScheduleRateTreatment                       = 40841, /*!< 40841 (int FIX.5.0SP2) Specifies the yield calculation treatment for the step schedule.*/
PaymentScheduleFixedAmount                         = 40842, /*!< 40842 (Amt FIX.5.0SP2) The explicit payment amount for this step schedule.*/
PaymentScheduleFixedCurrency                       = 40843, /*!< 40843 (Currency FIX.5.0SP2) The currency of the fixed amount.  Uses ISO 4217 currency codes.*/
PaymentScheduleStepFrequencyPeriod                 = 40844, /*!< 40844 (int FIX.5.0SP2) Time unit multiplier for the step frequency.*/
PaymentScheduleStepFrequencyUnit                   = 40845, /*!< 40845 (String FIX.5.0SP2) Time unit associated with the step frequency. */
PaymentScheduleStepOffsetValue                     = 40846, /*!< 40846 (Amt FIX.5.0SP2) The explicit amount that the notional changes on each step date. This can be a positive or negative amount.*/
PaymentScheduleStepRate                            = 40847, /*!< 40847 (Percentage FIX.5.0SP2) The percentage by which the notional changes on each step date. The percentage is either a percentage applied to the initial notional amount or the previous outstanding notional, depending on the value specified in PaymentScheduleStepRelativeTo(40849). The percentage can be either positive or negative.*/
PaymentScheduleStepOffsetRate                      = 40848, /*!< 40848 (Percentage FIX.5.0SP2) The explicit amount that the rate changes on each step date. This can be a positive or negative value.*/
PaymentScheduleStepRelativeTo                      = 40849, /*!< 40849 (int FIX.5.0SP2) Specifies whether the PaymentScheduleStepRate(40847) or PaymentScheduleStepOffsetValue(40846) should be applied to the initial notional or the previous notional in order to calculate the notional step change amount. */
PaymentScheduleFixingDateUnadjusted                = 40850, /*!< 40850 (LocalMktDate FIX.5.0SP2) The unadjusted fixing date.*/
PaymentScheduleWeight                              = 40851, /*!< 40851 (float FIX.5.0SP2) Floating rate observation weight for cashflow payment.*/
PaymentScheduleFixingDateRelativeTo                = 40852, /*!< 40852 (int FIX.5.0SP2) Specifies the anchor date when the fixing date is relative to an anchor date.  

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentScheduleFixingDateBusinessDayConvention     = 40853, /*!< 40853 (int FIX.5.0SP2) The business day convention used to adjust the payment schedule's fixing date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
PaymentScheduleFixingDateBusinessCenter            = 40854, /*!< 40854 (String FIX.5.0SP2) The business center calendar used to adjust the payment schedule's fixing date, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentScheduleFixingDateOffsetPeriod              = 40855, /*!< 40855 (int FIX.5.0SP2) Time unit multiplier for the relative fixing date offset.*/
PaymentScheduleFixingDateOffsetUnit                = 40856, /*!< 40856 (String FIX.5.0SP2) Time unit associated with the relative fixing date offset.*/
PaymentScheduleFixingDateOffsetDayType             = 40857, /*!< 40857 (int FIX.5.0SP2) Specifies the day type of the relative fixing date offset.*/
PaymentScheduleFixingDateAdjusted                  = 40858, /*!< 40858 (LocalMktDate FIX.5.0SP2) The adjusted fixing date.*/
PaymentScheduleFixingTime                          = 40859, /*!< 40859 (LocalMktTime FIX.5.0SP2) The fixing time associated with the step schedule.*/
PaymentScheduleFixingTimeBusinessCenter            = 40860, /*!< 40860 (String FIX.5.0SP2) Business center for determining fixing time. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentScheduleInterimExchangePaymentDateRelativeTo = 40861, /*!< 40861 (int FIX.5.0SP2) Specifies the anchor date when the interim exchange payment date is relative to an anchor date.    

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values. */
PaymentScheduleInterimExchangeDatesBusinessDayConvention = 40862, /*!< 40862 (int FIX.5.0SP2) The business day convention used to adjust the payment schedule's interim exchange date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
PaymentScheduleInterimExchangeDatesBusinessCenter  = 40863, /*!< 40863 (String FIX.5.0SP2) The business center calendar used to adjust the payment schedule's interim exchange date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentScheduleInterimExchangeDatesOffsetPeriod    = 40864, /*!< 40864 (int FIX.5.0SP2) Time unit multiplier for the relative interim exchange date offset.*/
PaymentScheduleInterimExchangeDatesOffsetUnit      = 40865, /*!< 40865 (String FIX.5.0SP2) Time unit associated with the relative interim exchange date offset.*/
PaymentScheduleInterimExchangeDatesOffsetDayType   = 40866, /*!< 40866 (int FIX.5.0SP2) Specifies the day type of the relative interim exchange date offset.*/
PaymentScheduleInterimExchangeDateAdjusted         = 40867, /*!< 40867 (LocalMktDate FIX.5.0SP2) The adjusted interim exchange date.*/
NoPaymentScheduleRateSources                       = 40868, /*!< 40868 (NumInGroup FIX.5.0SP2) Number of swap schedule rate sources.*/
PaymentScheduleRateSource                          = 40869, /*!< 40869 (int FIX.5.0SP2) Identifies the source of rate information.*/
PaymentScheduleRateSourceType                      = 40870, /*!< 40870 (int FIX.5.0SP2) Rate source type.*/
PaymentScheduleReferencePage                       = 40871, /*!< 40871 (String FIX.5.0SP2) Identifies the reference “page” from the rate source. 



For FX, the reference page to the spot rate to be used for the reference FX spot rate.



When RateSource(1446) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
NoPaymentStubs                                     = 40872, /*!< 40872 (NumInGroup FIX.5.0SP2) Number of stubs in the repeating group*/
PaymentStubType                                    = 40873, /*!< 40873 (int FIX.5.0SP2) Stub type.*/
PaymentStubLength                                  = 40874, /*!< 40874 (int FIX.5.0SP2) Optional indication whether stub is shorter or longer than the regular swap period.*/
PaymentStubRate                                    = 40875, /*!< 40875 (Percentage FIX.5.0SP2) The agreed upon fixed rate for this stub.*/
PaymentStubFixedAmount                             = 40876, /*!< 40876 (Amt FIX.5.0SP2) A fixed payment amount for the stub.*/
PaymentStubFixedCurrency                           = 40877, /*!< 40877 (Currency FIX.5.0SP2) The currency of the fixed payment amount.  Uses ISO 4217 currency codes.*/
PaymentStubIndex                                   = 40878, /*!< 40878 (String FIX.5.0SP2) The stub floating rate index.*/
PaymentStubIndexSource                             = 40879, /*!< 40879 (int FIX.5.0SP2) The source of the stub floating rate index.*/
PaymentStubIndexCurvePeriod                        = 40880, /*!< 40880 (int FIX.5.0SP2) Time unit multiplier for the stub floating rate index.*/
PaymentStubIndexCurveUnit                          = 40881, /*!< 40881 (String FIX.5.0SP2) Time unit associated with the stub floating rate index.*/
PaymentStubIndexRateMultiplier                     = 40882, /*!< 40882 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
PaymentStubIndexRateSpread                         = 40883, /*!< 40883 (PriceOffset FIX.5.0SP2) Spread from floating rate index.*/
PaymentStubIndexRateSpreadPositionType             = 40884, /*!< 40884 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
PaymentStubIndexRateTreatment                      = 40885, /*!< 40885 (int FIX.5.0SP2) Specifies the yield calculation treatment for the payment stub index.*/
PaymentStubIndexCapRate                            = 40886, /*!< 40886 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
PaymentStubIndexCapRateBuySide                     = 40887, /*!< 40887 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
PaymentStubIndexCapRateSellSide                    = 40888, /*!< 40888 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
PaymentStubIndexFloorRate                          = 40889, /*!< 40889 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.*/
PaymentStubIndexFloorRateBuySide                   = 40890, /*!< 40890 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
PaymentStubIndexFloorRateSellSide                  = 40891, /*!< 40891 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
PaymentStubIndex2                                  = 40892, /*!< 40892 (String FIX.5.0SP2) The second stub floating rate index.*/
PaymentStubIndex2Source                            = 40893, /*!< 40893 (int FIX.5.0SP2) The source of the second stub floating rate index.*/
PaymentStubIndex2CurvePeriod                       = 40894, /*!< 40894 (int FIX.5.0SP2) Secondary time unit multiplier for the stub floating rate index curve.*/
PaymentStubIndex2CurveUnit                         = 40895, /*!< 40895 (String FIX.5.0SP2) Secondary time unit associated with the stub floating rate index curve.*/
PaymentStubIndex2RateMultiplier                    = 40896, /*!< 40896 (float FIX.5.0SP2) A rate multiplier to apply to the second floating rate. The multiplier can be less than or greater than 1 (one). This element should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
PaymentStubIndex2RateSpread                        = 40897, /*!< 40897 (PriceOffset FIX.5.0SP2) Spread from the second floating rate index.*/
PaymentStubIndex2RateSpreadPositionType            = 40898, /*!< 40898 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
PaymentStubIndex2RateTreatment                     = 40899, /*!< 40899 (int FIX.5.0SP2) Specifies the yield calculation treatment for the second stub index.*/
PaymentStubIndex2CapRate                           = 40900, /*!< 40900 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the second floating rate. The cap rate (strike) is only required where the floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as 0.05.*/
PaymentStubIndex2FloorRate                         = 40901, /*!< 40901 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the second floating rate. The floor rate (strike) is only required where the floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A floor rate of 5% would be represented as 0.05.*/
NoLegSettlRateFallbacks                            = 40902, /*!< 40902 (NumInGroup FIX.5.0SP2) Number of settlement rate fallbacks in the repeating group*/
LegSettlRatePostponementMaximumDays                = 40903, /*!< 40903 (int FIX.5.0SP2) The maximum number of days to wait for a quote from the disrupted settlement rate option before proceding to this method.*/
UnderlyingSettlRateFallbackRateSource              = 40904, /*!< 40904 (int FIX.5.0SP2) Identifies the source of rate information.*/
LegSettlRatePostponementSurvey                     = 40905, /*!< 40905 (Boolean FIX.5.0SP2) Indicates whether to request a settlement rate quote from the market.*/
LegSettlRatePostponementCalculationAgent           = 40906, /*!< 40906 (int FIX.5.0SP2) Used to identify the settlement rate postponement calculation agent.*/
StreamEffectiveDateUnadjusted                      = 40907, /*!< 40907 (LocalMktDate FIX.5.0SP2) The unadjusted effective date.*/
StreamEffectiveDateBusinessDayConvention           = 40908, /*!< 40908 (int FIX.5.0SP2) The business day convention used to adjust the instrument's stream's effective, or relative effective, date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
StreamEffectiveDateBusinessCenter                  = 40909, /*!< 40909 (String FIX.5.0SP2) The business center calendar used to adjust the instrument's stream's effective, or relative effective, date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
StreamEffectiveDateRelativeTo                      = 40910, /*!< 40910 (int FIX.5.0SP2) Specifies the anchor date when the effective date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
StreamEffectiveDateOffsetPeriod                    = 40911, /*!< 40911 (int FIX.5.0SP2) Time unit multiplier for the relative effective date offset.*/
StreamEffectiveDateOffsetUnit                      = 40912, /*!< 40912 (String FIX.5.0SP2) Time unit associated with the relative effective date offset.*/
StreamEffectiveDateOffsetDayType                   = 40913, /*!< 40913 (int FIX.5.0SP2) Specifies the day type of the relative effective date offset.*/
StreamEffectiveDateAdjusted                        = 40914, /*!< 40914 (LocalMktDate FIX.5.0SP2) The adjusted effective date.*/
UnderlyingSettlRateFallbackReferencePage           = 40915, /*!< 40915 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

When UnderlyingSettlRateFallbackRateSource(40904) = 3(ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
CashSettlValuationSubsequentBusinessDaysOffset     = 40916, /*!< 40916 (int FIX.5.0SP2) The number of business days between successive valuation dates when multiple valuation dates are applicable for cash settlement.*/
CashSettlNumOfValuationDates                       = 40917, /*!< 40917 (int FIX.5.0SP2) Where multiple valuation dates are specified as being applicable for cash settlement, this specifies the number of applicable valuation dates.*/
UnderlyingProvisionPartyRoleQualifier              = 40918, /*!< 40918 (int FIX.5.0SP2) Used to further qualify the value of UnderlyingProvisionPartyRole(42176).*/
PaymentPriceType                                   = 40919, /*!< 40919 (int FIX.5.0SP2) Specifies the type of price for PaymentPrice(40218).*/
PaymentStreamPaymentDateOffsetDayType              = 40920, /*!< 40920 (int FIX.5.0SP2) Specifies the day type of the relative payment date offset.*/
BusinessDayConvention                              = 40921, /*!< 40921 (int FIX.5.0SP2) The business day convention used for adjusting dates. The value defined here applies to all adjustable dates in the instrument unless specifically overridden.*/
DateRollConvention                                 = 40922, /*!< 40922 (String FIX.5.0SP2) The convention for determining a sequence of dates. It is used in conjunction with a specified frequency. The value defined here applies to all adjustable dates in the instrument unless specifically overridden. Additional values may be used by mutual agreement of the counterparties.*/
NoLegBusinessCenters                               = 40923, /*!< 40923 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegBusinessCenter                                  = 40924, /*!< 40924 (String FIX.5.0SP2) A business center whose calendar is used for date adjustment, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegBusinessDayConvention                           = 40925, /*!< 40925 (int FIX.5.0SP2) The business day convention used for adjusting dates. The value defined here applies to all adjustable dates in the instrument leg unless specifically overridden.*/
LegDateRollConvention                              = 40926, /*!< 40926 (String FIX.5.0SP2) The convention for determining a sequence of dates. It is used in conjunction with a specified frequency. The value defined here applies to all adjustable dates in the instrument leg unless specifically overridden.*/
NoLegPaymentScheduleFixingDateBusinessCenters      = 40927, /*!< 40927 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegPaymentScheduleInterimExchangeDateBusinessCenters = 40928, /*!< 40928 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegPaymentStreamNonDeliverableFixingDateBusinessCenters = 40929, /*!< 40929 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegPaymentStreamPaymentDateBusinessCenters       = 40930, /*!< 40930 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegPaymentStreamResetDateBusinessCenters         = 40931, /*!< 40931 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegPaymentStreamInitialFixingDateBusinessCenters = 40932, /*!< 40932 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegPaymentStreamFixingDateBusinessCenters        = 40933, /*!< 40933 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegProvisionCashSettlPaymentDateBusinessCenters  = 40934, /*!< 40934 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegProvisionCashSettlValueDateBusinessCenters    = 40935, /*!< 40935 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegProvisionOptionExerciseBusinessCenters        = 40936, /*!< 40936 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegProvisionOptionExpirationDateBusinessCenters  = 40937, /*!< 40937 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegProvisionOptionRelevantUnderlyingDateBusinessCenters = 40938, /*!< 40938 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegProvisionDateBusinessCenters                  = 40939, /*!< 40939 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegStreamCalculationPeriodBusinessCenters        = 40940, /*!< 40940 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegStreamFirstPeriodStartDateBusinessCenters     = 40941, /*!< 40941 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegStreamEffectiveDateBusinessCenters            = 40942, /*!< 40942 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoLegStreamTerminationDateBusinessCenters          = 40943, /*!< 40943 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoPaymentBusinessCenters                           = 40944, /*!< 40944 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoPaymentScheduleInterimExchangeDateBusinessCenters = 40945, /*!< 40945 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoPaymentStreamNonDeliverableFixingDatesBusinessCenters = 40946, /*!< 40946 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoPaymentStreamPaymentDateBusinessCenters          = 40947, /*!< 40947 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoPaymentStreamResetDateBusinessCenters            = 40948, /*!< 40948 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoPaymentStreamInitialFixingDateBusinessCenters    = 40949, /*!< 40949 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoPaymentStreamFixingDateBusinessCenters           = 40950, /*!< 40950 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoProtectionTermEventNewsSources                   = 40951, /*!< 40951 (NumInGroup FIX.5.0SP2) Number of event news sources in the repeating group.*/
NoProvisionCashSettlPaymentDateBusinessCenters     = 40952, /*!< 40952 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoProvisionCashSettlValueDateBusinessCenters       = 40953, /*!< 40953 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoProvisionOptionExerciseBusinessCenters           = 40954, /*!< 40954 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoProvisionOptionExpirationDateBusinessCenters     = 40955, /*!< 40955 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoProvisionOptionRelevantUnderlyingDateBusinessCenters = 40956, /*!< 40956 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoProvisionDateBusinessCenters                     = 40957, /*!< 40957 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoStreamCalculationPeriodBusinessCenters           = 40958, /*!< 40958 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoStreamFirstPeriodStartDateBusinessCenters        = 40959, /*!< 40959 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoStreamEffectiveBusinessCenters                   = 40960, /*!< 40960 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoStreamTerminationDateBusinessCenters             = 40961, /*!< 40961 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingBusinessCenters                        = 40962, /*!< 40962 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingBusinessCenter                           = 40963, /*!< 40963 (String FIX.5.0SP2) A business center whose calendar is used for date adjustment, e.g. "GBLO". See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingBusinessDayConvention                    = 40964, /*!< 40964 (int FIX.5.0SP2) The business day convention used for adjusting dates. The value defined here applies to all adjustable dates in the underlying instrument unless specifically overridden.*/
UnderlyingDateRollConvention                       = 40965, /*!< 40965 (String FIX.5.0SP2) The convention for determining a sequence of dates. It is used in conjunction with a specified frequency. The value defined here applies to all adjustable dates in the underlying instrument unless specifically overridden.*/
NoUnderlyingPaymentScheduleFixingDateBusinessCenters = 40966, /*!< 40966 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingPaymentScheduleInterimExchangeDateBusinessCenters = 40967, /*!< 40967 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingPaymentStreamNonDeliverableFixingDatesBusinessCenters = 40968, /*!< 40968 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingPaymentStreamPaymentDateBusinessCenters = 40969, /*!< 40969 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingPaymentStreamResetDateBusinessCenters  = 40970, /*!< 40970 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingPaymentStreamInitialFixingDateBusinessCenters = 40971, /*!< 40971 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingPaymentStreamFixingDateBusinessCenters = 40972, /*!< 40972 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingStreamCalculationPeriodBusinessCenters = 40973, /*!< 40973 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingStreamFirstPeriodStartDateBusinessCenters = 40974, /*!< 40974 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingStreamEffectiveDateBusinessCenters     = 40975, /*!< 40975 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoUnderlyingStreamTerminationDateBusinessCenters   = 40976, /*!< 40976 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
NoPaymentScheduleFixingDateBusinessCenters         = 40977, /*!< 40977 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
EncodedLegStreamTextLen                            = 40978, /*!< 40978 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegStreamText(40979) field.*/
EncodedLegStreamText                               = 40979, /*!< 40979 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegStreamText(40248) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegStreamText(40248) field.*/
EncodedLegProvisionTextLen                         = 40980, /*!< 40980 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegProvisionText(40472) field.*/
EncodedLegProvisionText                            = 40981, /*!< 40981 (data FIX.5.0SP2) 	Encoded (non-ASCII characters) representation of the LegProvisionText(40472) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegProvisionText(40472) field.*/
EncodedStreamTextLen                               = 40982, /*!< 40982 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedStreamText(40983) field.*/
EncodedStreamText                                  = 40983, /*!< 40983 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the StreamText(40056) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the StreamText(40056) field.*/
EncodedPaymentTextLen                              = 40984, /*!< 40984 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedPaymentText(40985) field.*/
EncodedPaymentText                                 = 40985, /*!< 40985 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the PaymentText(40229) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the PaymentText(40229) field.*/
EncodedProvisionTextLen                            = 40986, /*!< 40986 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedProvisionText(40987) field.*/
EncodedProvisionText                               = 40987, /*!< 40987 (data FIX.5.0SP2) 	Encoded (non-ASCII characters) representation of the ProvisionText(40113) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the ProvisionText(40113) field.*/
EncodedUnderlyingStreamTextLen                     = 40988, /*!< 40988 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingStreamText(40989) field.*/
EncodedUnderlyingStreamText                        = 40989, /*!< 40989 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingStreamText(40547) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingStreamText(40547) field.*/
LegMarketDisruptionFallbackValue                   = 40990, /*!< 40990 (String FIX.5.0SP2) Applicable value for LegMarketDisruptionFallbackType(41470).*/
MarketDisruptionValue                              = 40991, /*!< 40991 (String FIX.5.0SP2) Applicable value for MarketDisruptionEvent(41093).*/
MarketDisruptionFallbackValue                      = 40992, /*!< 40992 (String FIX.5.0SP2) Applicable value for MarketDisruptionFallbackType(41095).*/
PaymentSubType                                     = 40993, /*!< 40993 (int FIX.5.0SP2) Used to further clarify the value of PaymentType(40213).*/
NoComplexEventAveragingObservations                = 40994, /*!< 40994 (NumInGroup FIX.5.0SP2) The number of averaging observations in the repeating group.*/
ComplexEventAveragingObservationNumber             = 40995, /*!< 40995 (int FIX.5.0SP2) Cross reference to the ordinal observation as specified either in the ComplexEventScheduleGrp or ComplexEventPeriodDateGrp components. */
ComplexEventAveragingWeight                        = 40996, /*!< 40996 (float FIX.5.0SP2) The weight factor to be applied to the observation.*/
NoComplexEventCreditEvents                         = 40997, /*!< 40997 (NumInGroup FIX.5.0SP2) The number of credit events specified in the repeating group.*/
ComplexEventCreditEventType                        = 40998, /*!< 40998 (String FIX.5.0SP2) Specifies the type of credit event. 

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Types for code list of applicable event types.*/
ComplexEventCreditEventValue                       = 40999, /*!< 40999 (String FIX.5.0SP2) The credit event value appropriate to ComplexEventCreditEventType(40998). 

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Types for applicable event type values. */
ComplexEventCreditEventCurrency                    = 41000, /*!< 41000 (Currency FIX.5.0SP2) Specifies the applicable currency when ComplexEventCreditEventValue(40999) is an amount. Uses ISO 4217 currency codes.*/
ComplexEventCreditEventPeriod                      = 41001, /*!< 41001 (int FIX.5.0SP2) Time unit multiplier for complex credit events.*/
ComplexEventCreditEventUnit                        = 41002, /*!< 41002 (String FIX.5.0SP2) Time unit associated with complex credit events.*/
ComplexEventCreditEventDayType                     = 41003, /*!< 41003 (int FIX.5.0SP2) Specifies the day type for the complex credit events.*/
ComplexEventCreditEventRateSource                  = 41004, /*!< 41004 (int FIX.5.0SP2) Identifies the source of rate information used for credit events. 

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Rate_Source for code list of applicable sources.*/
NoComplexEventCreditEventQualifiers                = 41005, /*!< 41005 (NumInGroup FIX.5.0SP2) The number of qualifiers in the repeating group.*/
ComplexEventCreditEventQualifier                   = 41006, /*!< 41006 (char FIX.5.0SP2) Specifies a complex event qualifier. Used to further qualify ComplexEventCreditEventType(40998).*/
NoComplexEventPeriodDateTimes                      = 41007, /*!< 41007 (NumInGroup FIX.5.0SP2) The number of entries in the date-time repeating group.*/
ComplexEventPeriodDate                             = 41008, /*!< 41008 (LocalMktDate FIX.5.0SP2) The averaging date for an Asian option. 

The trigger date for a Barrier or Knock option.*/
ComplexEventPeriodTime                             = 41009, /*!< 41009 (LocalMktTime FIX.5.0SP2) The averaging time for an Asian option. */
NoComplexEventPeriods                              = 41010, /*!< 41010 (NumInGroup FIX.5.0SP2) The number of periods in the repeating group.*/
ComplexEventPeriodType                             = 41011, /*!< 41011 (int FIX.5.0SP2) Specifies the period type.*/
ComplexEventBusinessCenter                         = 41012, /*!< 41012 (String FIX.5.0SP2) The business center used to determine dates and times in the schedule or date-time group.  

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoComplexEventRateSources                          = 41013, /*!< 41013 (NumInGroup FIX.5.0SP2) The number of rate sources in the repeating group.*/
ComplexEventRateSource                             = 41014, /*!< 41014 (int FIX.5.0SP2) Identifies the source of rate information.*/
ComplexEventRateSourceType                         = 41015, /*!< 41015 (int FIX.5.0SP2) Indicates whether the rate source specified is a primary or secondary source.*/
ComplexEventReferencePage                          = 41016, /*!< 41016 (String FIX.5.0SP2) Identifies the reference page from the rate source.

For FX, the reference page to the spot rate is to be used for the reference FX spot rate.

When ComplexEventRateSource(41014) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option. */
ComplexEventReferencePageHeading                   = 41017, /*!< 41017 (String FIX.5.0SP2) Identifies the reference page heading from the rate source.*/
NoComplexEventDateBusinessCenters                  = 41018, /*!< 41018 (NumInGroup FIX.5.0SP2) The number of business centers in the repeating group.*/
ComplexEventDateBusinessCenter                     = 41019, /*!< 41019 (String FIX.5.0SP2) The business center calendar used to adjust the complex event date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ComplexEventDateUnadjusted                         = 41020, /*!< 41020 (LocalMktDate FIX.5.0SP2) The unadjusted complex event date.*/
ComplexEventDateRelativeTo                         = 41021, /*!< 41021 (int FIX.5.0SP2) Specifies the anchor date when the complex event date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
ComplexEventDateOffsetPeriod                       = 41022, /*!< 41022 (int FIX.5.0SP2) Time unit multiplier for the relative date offset.*/
ComplexEventDateOffsetUnit                         = 41023, /*!< 41023 (String FIX.5.0SP2) Time unit associated with the relative date offset.*/
ComplexEventDateOffsetDayType                      = 41024, /*!< 41024 (int FIX.5.0SP2) Specifies the day type of the relative date offset.*/
ComplexEventDateBusinessDayConvention              = 41025, /*!< 41025 (int FIX.5.0SP2) The business day convention used to adjust the complex event date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
ComplexEventDateAdjusted                           = 41026, /*!< 41026 (LocalMktDate FIX.5.0SP2) The adjusted complex event date.*/
ComplexEventFixingTime                             = 41027, /*!< 41027 (LocalMktTime FIX.5.0SP2) The local market fixing time.*/
ComplexEventFixingTimeBusinessCenter               = 41028, /*!< 41028 (String FIX.5.0SP2) The business center calendar used to determine the actual fixing times.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoComplexEventCreditEventSources                   = 41029, /*!< 41029 (NumInGroup FIX.5.0SP2) Number of event sources in the repeating group.*/
ComplexEventCreditEventSource                      = 41030, /*!< 41030 (String FIX.5.0SP2) A newspaper or electronic news service that may publish relevant information used in the determination of whether or not a credit event has occurred.*/
NoComplexEventSchedules                            = 41031, /*!< 41031 (NumInGroup FIX.5.0SP2) Number of schedules in the repeating group.*/
ComplexEventScheduleStartDate                      = 41032, /*!< 41032 (LocalMktDate FIX.5.0SP2) The start date of the schedule.*/
ComplexEventScheduleEndDate                        = 41033, /*!< 41033 (LocalMktDate FIX.5.0SP2) The end date of the schedule.*/
ComplexEventScheduleFrequencyPeriod                = 41034, /*!< 41034 (int FIX.5.0SP2) Time unit multiplier for the schedule date frequency.*/
ComplexEventScheduleFrequencyUnit                  = 41035, /*!< 41035 (String FIX.5.0SP2) Time unit associated with the schedule date frequency.*/
ComplexEventScheduleRollConvention                 = 41036, /*!< 41036 (String FIX.5.0SP2) The convention for determining the sequence of dates.  It is used in conjunction with a specified frequency.  Used only to override the roll convention defined in the DateAdjustment component in Instrument.*/
NoDeliverySchedules                                = 41037, /*!< 41037 (NumInGroup FIX.5.0SP2) Number of delivery schedules in the repeating group.*/
DeliveryScheduleType                               = 41038, /*!< 41038 (int FIX.5.0SP2) Specifies the type of delivery schedule.*/
DeliveryScheduleXID                                = 41039, /*!< 41039 (XID FIX.5.0SP2) Identifier for this instance of delivery schedule for cross referencing elsewhere in the message.*/
DeliveryScheduleNotional                           = 41040, /*!< 41040 (Qty FIX.5.0SP2) Physical delivery quantity.*/
DeliveryScheduleNotionalUnitOfMeasure              = 41041, /*!< 41041 (String FIX.5.0SP2) Specifies the delivery quantity unit of measure (UOM).*/
DeliveryScheduleNotionalCommodityFrequency         = 41042, /*!< 41042 (int FIX.5.0SP2) The frequency of notional delivery.*/
DeliveryScheduleNegativeTolerance                  = 41043, /*!< 41043 (float FIX.5.0SP2) Specifies the negative tolerance value.  The value may be an absolute quantity or a percentage, as specified in DeliveryScheduleToleranceType(41046).  Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
DeliverySchedulePositiveTolerance                  = 41044, /*!< 41044 (float FIX.5.0SP2) Specifies the positive tolerance value. The value may be an absolute quantity or a percentage, as specified in DeliveryScheduleToleranceType(41046). Value may exceed agreed upon value. Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
DeliveryScheduleToleranceUnitOfMeasure             = 41045, /*!< 41045 (String FIX.5.0SP2) Specifies the tolerance value's unit of measure (UOM).*/
DeliveryScheduleToleranceType                      = 41046, /*!< 41046 (int FIX.5.0SP2) Specifies the tolerance value type.*/
DeliveryScheduleSettlCountry                       = 41047, /*!< 41047 (Country FIX.5.0SP2) Specifies the country where delivery takes place. Uses ISO 3166 2-character country code.*/
DeliveryScheduleSettlTimeZone                      = 41048, /*!< 41048 (String FIX.5.0SP2) Delivery timezone specified as "prevailing" rather than "standard" or "daylight".  

See http://www.fixtradingcommunity.org/codelists#Prevailing_Timezones for code list of applicable prevailing timezones.*/
DeliveryScheduleSettlFlowType                      = 41049, /*!< 41049 (int FIX.5.0SP2) Specifies the commodity delivery flow type.*/
DeliveryScheduleSettlHolidaysProcessingInstruction = 41050, /*!< 41050 (int FIX.5.0SP2) Indicates whether holidays are included in the settlement periods. Required for electricity contracts.*/
NoDeliveryScheduleSettlDays                        = 41051, /*!< 41051 (NumInGroup FIX.5.0SP2) Number of delivery schedules in the repeating group.*/
DeliveryScheduleSettlDay                           = 41052, /*!< 41052 (int FIX.5.0SP2) Specifies the day or group of days for delivery.  */
DeliveryScheduleSettlTotalHours                    = 41053, /*!< 41053 (int FIX.5.0SP2) The sum of the total hours specified in the DeliveryScheduleSettlTimeGrp component.*/
NoDeliveryScheduleSettlTimes                       = 41054, /*!< 41054 (NumInGroup FIX.5.0SP2) Number of hour ranges in the repeating group.*/
DeliveryScheduleSettlStart                         = 41055, /*!< 41055 (String FIX.5.0SP2) The scheduled start time for the delivery of the commodity where delivery occurs over specified times. The format of the time value is specified in DeliveryScheduleSettlTimeType(41057).*/
DeliveryScheduleSettlEnd                           = 41056, /*!< 41056 (String FIX.5.0SP2) The scheduled end time for the delivery of the commodity where delivery occurs over specified times. The format of the time value is specified in DeliveryScheduleSettlTimeType(41057).*/
DeliveryScheduleSettlTimeType                      = 41057, /*!< 41057 (int FIX.5.0SP2) Specifies the format of the delivery start and end time values.*/
DeliveryStreamType                                 = 41058, /*!< 41058 (int FIX.5.0SP2) Specifies the type of delivery stream.*/
DeliveryStreamPipeline                             = 41059, /*!< 41059 (String FIX.5.0SP2) The name of the oil delivery pipeline.*/
DeliveryStreamEntryPoint                           = 41060, /*!< 41060 (String FIX.5.0SP2) The point at which the commodity will enter the delivery mechanism or pipeline.*/
DeliveryStreamWithdrawalPoint                      = 41061, /*!< 41061 (String FIX.5.0SP2) The point at which the commodity product will be withdrawn prior to delivery.*/
DeliveryStreamDeliveryPoint                        = 41062, /*!< 41062 (String FIX.5.0SP2) The point at which the commodity product will be delivered and received. Value specified should follow market convention appropriate for the commodity product. 

For bullion, see http://www.fpml.org/coding-scheme/bullion-delivery-location for values.*/
DeliveryStreamDeliveryRestriction                  = 41063, /*!< 41063 (int FIX.5.0SP2) Specifies under what conditions the buyer and seller should be excused of their delivery obligations.*/
DeliveryStreamDeliveryContingency                  = 41064, /*!< 41064 (String FIX.5.0SP2) Specifies the electricity delivery contingency. 

See http://www.fpml.org/coding-scheme/electricity-transmission-contingency for values.*/
DeliveryStreamDeliveryContingentPartySide          = 41065, /*!< 41065 (int FIX.5.0SP2) The trade side value of the party responsible for electricity delivery contingency.*/
DeliveryStreamDeliverAtSourceIndicator             = 41066, /*!< 41066 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', delivery of the coal product is to be at its source.*/
DeliveryStreamRiskApportionment                    = 41067, /*!< 41067 (String FIX.5.0SP2) Specifies how the parties to the trade apportion responsibility for the delivery of the commodity product. 

See http://www.fixtradingcommunity.org/codelists#Risk_Apportionment for the details of the external code list.*/
DeliveryStreamTitleTransferLocation                = 41068, /*!< 41068 (String FIX.5.0SP2) Specifies the title transfer location.*/
DeliveryStreamTitleTransferCondition               = 41069, /*!< 41069 (int FIX.5.0SP2) Specifies the condition of title transfer.*/
DeliveryStreamImporterOfRecord                     = 41070, /*!< 41070 (String FIX.5.0SP2) A party, not necessarily of the trade, who is the Importer of Record for the purposes of paying customs duties and applicable taxes or costs related to importation.*/
DeliveryStreamNegativeTolerance                    = 41071, /*!< 41071 (float FIX.5.0SP2) Specifies the negative tolerance value.  The value may be an absolute quantity or a percentage, as specified in DeliveryStreamToleranceType(41074).  Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
DeliveryStreamPositiveTolerance                    = 41072, /*!< 41072 (float FIX.5.0SP2) Specifies the positive tolerance value. The value may be an absolute quantity or a percentage, as specified in DeliveryStreamToleranceType(41074). Value may exceed agreed upon value. Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
DeliveryStreamToleranceUnitOfMeasure               = 41073, /*!< 41073 (String FIX.5.0SP2) Specifies the tolerance value's unit of measure (UOM).*/
DeliveryStreamToleranceType                        = 41074, /*!< 41074 (int FIX.5.0SP2) Specifies the tolerance value type.*/
DeliveryStreamToleranceOptionSide                  = 41075, /*!< 41075 (int FIX.5.0SP2) Indicates whether the tolerance is at the seller's or buyer's option.*/
DeliveryStreamTotalPositiveTolerance               = 41076, /*!< 41076 (Percentage FIX.5.0SP2) The positive percent tolerance which applies to the total quantity delivered over all shipment periods.

Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%.).*/
DeliveryStreamTotalNegativeTolerance               = 41077, /*!< 41077 (Percentage FIX.5.0SP2) The negative percent tolerance which applies to the total quantity delivered over all shipment periods.

Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%.).*/
DeliveryStreamNotionalConversionFactor             = 41078, /*!< 41078 (float FIX.5.0SP2) If the notional quantity is specified in a unit that does not match the unit in which the commodity reference price is quoted, the scaling or conversion factor used to convert the commodity reference price unit into the notional quantity unit should be stated here. If there is no conversion, this field is not intended to be used.*/
DeliveryStreamTransportEquipment                   = 41079, /*!< 41079 (String FIX.5.0SP2) The transportation equipment with which the commodity product will be delivered and received. */
DeliveryStreamElectingPartySide                    = 41080, /*!< 41080 (int FIX.5.0SP2) A reference to the party able to choose whether the gas is delivered for a particular period as found in a swing or interruptible contract.*/
NoDeliveryStreamCycles                             = 41081, /*!< 41081 (NumInGroup FIX.5.0SP2) Number of delivery cycles in the repeating group.*/
DeliveryStreamCycleDesc                            = 41082, /*!< 41082 (String FIX.5.0SP2) The delivery cycles during which the oil product will be transported in the pipeline.*/
EncodedDeliveryStreamCycleDescLen                  = 41083, /*!< 41083 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedDeliveryStreamCycleDesc(41084) field.*/
EncodedDeliveryStreamCycleDesc                     = 41084, /*!< 41084 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the DeliveryStreamCycleDesc(41082) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the DeliveryStreamCycleDesc(41082) field.*/
NoDeliveryStreamCommoditySources                   = 41085, /*!< 41085 (NumInGroup FIX.5.0SP2) Number of commodity sources in the repeating group.*/
DeliveryStreamCommoditySource                      = 41086, /*!< 41086 (String FIX.5.0SP2) The SCoTA coal cargo origin, mining region, mine(s), mining complex(es), loadout(s) or river dock(s) or other point(s) of origin that seller and buyer agree are acceptable origins for the coal product. For international coal transactions, this is the origin of the coal product. 

See http://www.fpml.org/coding-scheme/commodity-coal-product-source for values.*/
MarketDisruptionProvision                          = 41087, /*!< 41087 (int FIX.5.0SP2) The consequences of market disruption events.*/
MarketDisruptionFallbackProvision                  = 41088, /*!< 41088 (int FIX.5.0SP2) Specifies the location of the fallback provision documentation.*/
MarketDisruptionMaximumDays                        = 41089, /*!< 41089 (int FIX.5.0SP2) Specifies the maximum number of market disruption days (commodity or bullion business days) in a contract or confirmation. If none are specified, the maximum number of market disruption days is five (5).*/
MarketDisruptionMaterialityPercentage              = 41090, /*!< 41090 (Percentage FIX.5.0SP2) Used when a price materiality percentage applies to the price source disruption event and this event has been specified.*/
MarketDisruptionMinimumFuturesContracts            = 41091, /*!< 41091 (int FIX.5.0SP2) Specifies the minimum futures contracts level that dictates whether or not a 'De Minimis Trading' event has occurred. */
NoMarketDisruptionEvents                           = 41092, /*!< 41092 (NumInGroup FIX.5.0SP2) Number of disruption events in the repeating group.*/
MarketDisruptionEvent                              = 41093, /*!< 41093 (String FIX.5.0SP2) Specifies the market disruption event. 

For commodities see http://www.fpml.org/coding-scheme/commodity-market-disruption for values. 

For foreign exchange, see http://www.fixtradingcommunity.org/codelists#Market_Disruption_Event for code list of applicable event types.*/
NoMarketDisruptionFallbacks                        = 41094, /*!< 41094 (NumInGroup FIX.5.0SP2) Number of fallbacks in the repeating group.*/
MarketDisruptionFallbackType                       = 41095, /*!< 41095 (String FIX.5.0SP2) Specifies the type of disruption fallback.  

See http://www.fpml.org/coding-scheme/commodity-market-disruption-fallback for values. */
NoMarketDisruptionFallbackReferencePrices          = 41096, /*!< 41096 (NumInGroup FIX.5.0SP2) Number of fallback reference securities in the repeating group.*/
MarketDisruptionFallbackUnderlierType              = 41097, /*!< 41097 (int FIX.5.0SP2) The type of reference price underlier.*/
MarketDisruptionFallbackUnderlierSecurityID        = 41098, /*!< 41098 (String FIX.5.0SP2) Specifies the identifier value of the security.*/
MarketDisruptionFallbackUnderlierSecurityIDSource  = 41099, /*!< 41099 (String FIX.5.0SP2) Specifies the class or source scheme of the security identifier.*/
MarketDisruptionFallbackUnderlierSecurityDesc      = 41100, /*!< 41100 (String FIX.5.0SP2) Specifies the description of the underlying security.*/
EncodedMarketDisruptionFallbackUnderlierSecurityDescLen = 41101, /*!< 41101 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedMarketDisruptionFallbackUnderlierSecurityDesc(41102) field.*/
EncodedMarketDisruptionFallbackUnderlierSecurityDesc = 41102, /*!< 41102 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the MarketDisruptionFallbackUnderlierSecurityDesc(41100) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the MarketDisruptionFallbackUnderlierSecurityDesc(41100) field.*/
MarketDisruptionFallbackOpenUnits                  = 41103, /*!< 41103 (Qty FIX.5.0SP2) If there are multiple underlying assets, this specifies the number of units (index or securities) that constitute the underlier of the swap. In the case of a basket swap, this is used to reference both the number of basket units, and the number of each asset components of the basket when these are expressed in absolute terms.*/
MarketDisruptionFallbackBasketCurrency             = 41104, /*!< 41104 (Currency FIX.5.0SP2) Specifies the currency if the underlier is a basket. Uses ISO 4217 currency codes.*/
MarketDisruptionFallbackBasketDivisor              = 41105, /*!< 41105 (float FIX.5.0SP2) Specifies the basket divisor amount. This value is normally used to adjust the constituent weight for pricing or to adjust for dividends, or other corporate actions.*/
ExerciseDesc                                       = 41106, /*!< 41106 (String FIX.5.0SP2) A description of the option exercise.*/
EncodedExerciseDescLen                             = 41107, /*!< 41107 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedExerciseDesc(41102) field.*/
EncodedExerciseDesc                                = 41108, /*!< 41108 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the ExerciseDesc(41106) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the ExerciseDesc(41106) field.*/
AutomaticExerciseIndicator                         = 41109, /*!< 41109 (Boolean FIX.5.0SP2) Indicates (when 'Y') that exercise is automatic when the strike price is crossed or the underlying trade is in the money.*/
AutomaticExerciseThresholdRate                     = 41110, /*!< 41110 (float FIX.5.0SP2) The threshold rate for triggering automatic exercise.*/
ExerciseConfirmationMethod                         = 41111, /*!< 41111 (int FIX.5.0SP2) Indicates whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller's agent. */
ManualNoticeBusinessCenter                         = 41112, /*!< 41112 (String FIX.5.0SP2) Identifies the business center used for adjusting the time for manual exercise notice.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
FallbackExerciseIndicator                          = 41113, /*!< 41113 (Boolean FIX.5.0SP2) Indicates whether the notional amount of the underlying swap, not previously exercised under the option, will be automatically exercised at the expiration time on the expiration date if at such time the buyer is in-the-money, provided that the difference between the settlement rate and the fixed rate under the relevant underlying swap is not less than one tenth of a percentage point (0.10% or 0.001).*/
LimitedRightToConfirmIndicator                     = 41114, /*!< 41114 (Boolean FIX.5.0SP2) Indicates whether the Seller may request the Buyer to confirm its intent to exercise if not done on or before the expiration time on the expiration date. If true ("Y") specific rules will apply in relation to the settlement mode.*/
ExerciseSplitTicketIndicator                       = 41115, /*!< 41115 (Boolean FIX.5.0SP2) Indicates in physical settlement of bond and convertible bond options whether the party required to deliver the bonds will divide those to be delivered as notifying party desires to facilitate delivery obligations.*/
NoOptionExerciseBusinessCenters                    = 41116, /*!< 41116 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
OptionExerciseBusinessCenter                       = 41117, /*!< 41117 (String FIX.5.0SP2) The business center calendar used to adjust the option exercise dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
OptionExerciseBusinessDayConvention                = 41118, /*!< 41118 (int FIX.5.0SP2) The business day convention used to adjust the option exercise dates. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
OptionExerciseEarliestDateOffsetDayType            = 41119, /*!< 41119 (int FIX.5.0SP2) Specifies the day type of the relative earliest option exercise date offset.*/
OptionExerciseEarliestDateOffsetPeriod             = 41120, /*!< 41120 (int FIX.5.0SP2) Time unit multiplier for the relative earliest exercise date offset.  */
OptionExerciseEarliestDateOffsetUnit               = 41121, /*!< 41121 (String FIX.5.0SP2) Time unit associated with the relative earliest exercise date offset. */
OptionExerciseFrequencyPeriod                      = 41122, /*!< 41122 (int FIX.5.0SP2) Time unit multiplier for the frequency of exercise dates. */
OptionExerciseFrequencyUnit                        = 41123, /*!< 41123 (String FIX.5.0SP2) Time unit associated with the frequency of exercise dates. */
OptionExerciseStartDateUnadjusted                  = 41124, /*!< 41124 (LocalMktDate FIX.5.0SP2) The unadjusted start date for calculating periodic exercise dates.*/
OptionExerciseStartDateRelativeTo                  = 41125, /*!< 41125 (int FIX.5.0SP2) Specifies the anchor date when the option exercise start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
OptionExerciseStartDateOffsetPeriod                = 41126, /*!< 41126 (int FIX.5.0SP2) Time unit multiplier for the relative exercise start date offset.  */
OptionExerciseStartDateOffsetUnit                  = 41127, /*!< 41127 (String FIX.5.0SP2) Time unit associated with the relative exercise start date offset. */
OptionExerciseStartDateOffsetDayType               = 41128, /*!< 41128 (int FIX.5.0SP2) Specifies the day type of the relative option exercise start date offset.*/
OptionExerciseStartDateAdjusted                    = 41129, /*!< 41129 (LocalMktDate FIX.5.0SP2) The adjusted start date for calculating periodic exercise dates.*/
OptionExerciseSkip                                 = 41130, /*!< 41130 (int FIX.5.0SP2) The number of periods in the referenced date schedule that are between each date in the relative date schedule. Thus a skip of 2 would mean that dates are relative to every second date in the referenced schedule. If present this should have a value greater than 1.*/
OptionExerciseNominationDeadline                   = 41131, /*!< 41131 (LocalMktDate FIX.5.0SP2) Last date (adjusted) for establishing the option exercise terms.*/
OptionExerciseFirstDateUnadjusted                  = 41132, /*!< 41132 (LocalMktDate FIX.5.0SP2) The unadjusted first exercise date.*/
OptionExerciseLastDateUnadjusted                   = 41133, /*!< 41133 (LocalMktDate FIX.5.0SP2) The unadjusted last exercise date.*/
OptionExerciseEarliestTime                         = 41134, /*!< 41134 (LocalMktTime FIX.5.0SP2) The earliest time at which notice of exercise can be given by the buyer to the seller (or seller's agent) (i) on the expriation date, in the case of a European style option, (ii) on each Bermuda option exercise date and the expiration date, in the case of a Bermuda style option, (iii) the commencement date to, and including, the expiration date, in the case of an American option.*/
OptionExerciseLatestTime                           = 41135, /*!< 41135 (LocalMktTime FIX.5.0SP2) The latest exercise time.  See also OptionExerciseEarliestTime(41134).*/
OptionExerciseTimeBusinessCenter                   = 41136, /*!< 41136 (String FIX.5.0SP2) The business center used to determine the locale for option exercise time, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values*/
NoOptionExerciseDates                              = 41137, /*!< 41137 (NumInGroup FIX.5.0SP2) Number of dates in the repeating group.*/
OptionExerciseDate                                 = 41138, /*!< 41138 (LocalMktDate FIX.5.0SP2) The option exercise fixed date, unadjusted or adjusted depending on OptionExerciseDateType(41139).*/
OptionExerciseDateType                             = 41139, /*!< 41139 (int FIX.5.0SP2) Specifies the type of date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
NoOptionExerciseExpirationDateBusinessCenters      = 41140, /*!< 41140 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
OptionExerciseExpirationDateBusinessCenter         = 41141, /*!< 41141 (String FIX.5.0SP2) The business center calendar used to adjust the option exercise expiration dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
OptionExerciseExpirationDateBusinessDayConvention  = 41142, /*!< 41142 (int FIX.5.0SP2) The business day convention used to adjust the option exercise expiration dates. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
OptionExerciseExpirationDateRelativeTo             = 41143, /*!< 41143 (int FIX.5.0SP2) Specifies the anchor date when the option exercise expiration date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
OptionExerciseExpirationDateOffsetPeriod           = 41144, /*!< 41144 (int FIX.5.0SP2) Time unit multiplier for the relative exercise expiration date offset.  */
OptionExerciseExpirationDateOffsetUnit             = 41145, /*!< 41145 (String FIX.5.0SP2) Time unit associated with the relative exercise expiration date offset.*/
OptionExerciseExpirationFrequencyPeriod            = 41146, /*!< 41146 (int FIX.5.0SP2) Time unit multiplier for the frequency of exercise expiration dates. */
OptionExerciseExpirationFrequencyUnit              = 41147, /*!< 41147 (String FIX.5.0SP2) Time unit associated with the frequency of exercise expiration dates. */
OptionExerciseExpirationRollConvention             = 41148, /*!< 41148 (String FIX.5.0SP2) The convention for determining the sequence of exercise expiration dates. It is used in conjunction with a specified frequency. Used only to override the roll convention defined in the DateAdjustment component in Instrument.*/
OptionExerciseExpirationDateOffsetDayType          = 41149, /*!< 41149 (int FIX.5.0SP2) Specifies the day type of the relative option exercise expiration date offset.*/
OptionExerciseExpirationTime                       = 41150, /*!< 41150 (LocalMktTime FIX.5.0SP2) The option exercise expiration time.*/
OptionExerciseExpirationTimeBusinessCenter         = 41151, /*!< 41151 (String FIX.5.0SP2) The business center used to determine the locale for option exercise expiration time, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoOptionExerciseExpirationDates                    = 41152, /*!< 41152 (NumInGroup FIX.5.0SP2) Number of fixed exercise expiration dates in the repeating group.*/
OptionExerciseExpirationDate                       = 41153, /*!< 41153 (LocalMktDate FIX.5.0SP2) An adjusted or unadjusted fixed option exercise expiration date.*/
OptionExerciseExpirationDateType                   = 41154, /*!< 41154 (int FIX.5.0SP2) Specifies the type of option exercise expiration date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
PaymentUnitOfMeasure                               = 41155, /*!< 41155 (String FIX.5.0SP2) Used to express the unit of measure (UOM) of the payment amount if not in the currency of the trade.*/
PaymentDateRelativeTo                              = 41156, /*!< 41156 (int FIX.5.0SP2) Specifies the anchor date when the payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentDateOffsetPeriod                            = 41157, /*!< 41157 (int FIX.5.0SP2) Time unit multiplier for the relative payment date offset. */
PaymentDateOffsetUnit                              = 41158, /*!< 41158 (String FIX.5.0SP2) Time unit associated with the relative payment date offset. */
PaymentDateOffsetDayType                           = 41159, /*!< 41159 (int FIX.5.0SP2) Specifies the day type of the relative payment date offset.*/
PaymentForwardStartType                            = 41160, /*!< 41160 (int FIX.5.0SP2) Forward start premium type.*/
NoPaymentScheduleFixingDays                        = 41161, /*!< 41161 (NumInGroup FIX.5.0SP2) Number of fixing days in the repeating group.*/
PaymentScheduleFixingDayOfWeek                     = 41162, /*!< 41162 (int FIX.5.0SP2) The day of the week on which fixing will take place.  */
PaymentScheduleFixingDayNumber                     = 41163, /*!< 41163 (int FIX.5.0SP2) The occurrence of the day of week on which fixing takes place. */
PaymentScheduleXID                                 = 41164, /*!< 41164 (XID FIX.5.0SP2) Identifier of this PaymentSchedule for cross referencing elsewhere in the message.*/
PaymentScheduleXIDRef                              = 41165, /*!< 41165 (XIDREF FIX.5.0SP2) Reference to payment schedule elsewhere in the message.*/
PaymentScheduleRateCurrency                        = 41166, /*!< 41166 (Currency FIX.5.0SP2) The currency of the schedule rate. Uses ISO 4217 currency codes.*/
PaymentScheduleRateUnitOfMeasure                   = 41167, /*!< 41167 (String FIX.5.0SP2) The schedule rate unit of measure (UOM).*/
PaymentScheduleRateConversionFactor                = 41168, /*!< 41168 (float FIX.5.0SP2) The number to be multiplied by the derived floating rate of the payment schedule in order to arrive at the payment rate. If omitted, the schedule rate conversion factor is 1. */
PaymentScheduleRateSpreadType                      = 41169, /*!< 41169 (int FIX.5.0SP2) Identifies whether the rate spread is an absolute value to be added to the index rate or a percentage of the index rate.*/
PaymentScheduleSettlPeriodPrice                    = 41170, /*!< 41170 (Price FIX.5.0SP2) The schedule settlement period price.*/
PaymentScheduleSettlPeriodPriceCurrency            = 41171, /*!< 41171 (Currency FIX.5.0SP2) Specifies the currency of the schedule settlement period price.  Uses ISO 4217 currency codes.*/
PaymentScheduleSettlPeriodPriceUnitOfMeasure       = 41172, /*!< 41172 (String FIX.5.0SP2) The settlement period price unit of measure (UOM).*/
PaymentScheduleStepUnitOfMeasure                   = 41173, /*!< 41173 (String FIX.5.0SP2) The schedule step unit of measure (UOM).*/
PaymentScheduleFixingDayDistribution               = 41174, /*!< 41174 (int FIX.5.0SP2) The distribution of fixing days.*/
PaymentScheduleFixingDayCount                      = 41175, /*!< 41175 (int FIX.5.0SP2) The number of days over which fixing should take place.*/
PaymentScheduleFixingLagPeriod                     = 41176, /*!< 41176 (int FIX.5.0SP2) Time unit multiplier for the fixing lag duration.  */
PaymentScheduleFixingLagUnit                       = 41177, /*!< 41177 (String FIX.5.0SP2) Time unit associated with the fixing lag duration. */
PaymentScheduleFixingFirstObservationDateOffsetPeriod = 41178, /*!< 41178 (int FIX.5.0SP2) Time unit multiplier for the relative first observation date offset.*/
PaymentScheduleFixingFirstObservationDateOffsetUnit = 41179, /*!< 41179 (String FIX.5.0SP2) Time unit associated with the relative first observation date offset.*/
PaymentStreamFlatRateIndicator                     = 41180, /*!< 41180 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', the Flat Rate is the New Worldwide Tanker Nominal Freight Scale for the Freight Index Route taken at the Trade Date of the transaction “Fixed”. If  'N' it is taken on each Pricing Date “Floating”.*/
PaymentStreamFlatRateAmount                        = 41181, /*!< 41181 (Amt FIX.5.0SP2) Specifies the actual monetary value of the flat rate when PaymentStreamFlatRateIndicator(41180) = 'Y'.*/
PaymentStreamFlatRateCurrency                      = 41182, /*!< 41182 (Currency FIX.5.0SP2) Specifies the currency of the actual flat rate.  Uses ISO 4217 currency codes.*/
PaymentStreamMaximumPaymentAmount                  = 41183, /*!< 41183 (Amt FIX.5.0SP2) Specifies the limit on the total payment amount.*/
PaymentStreamMaximumPaymentCurrency                = 41184, /*!< 41184 (Currency FIX.5.0SP2) Specifies the currency of total payment amount limit.  Uses ISO 4217 currency codes.*/
PaymentStreamMaximumTransactionAmount              = 41185, /*!< 41185 (Amt FIX.5.0SP2) Specifies the limit on the payment amount that goes out in any particular calculation period.*/
PaymentStreamMaximumTransactionCurrency            = 41186, /*!< 41186 (Currency FIX.5.0SP2) Specifies the currency of the period payment amount limit. Uses ISO 4217 currency codes.*/
PaymentStreamFixedAmountUnitOfMeasure              = 41187, /*!< 41187 (String FIX.5.0SP2) Specifies the fixed payment amount unit of measure (UOM).*/
PaymentStreamTotalFixedAmount                      = 41188, /*!< 41188 (Amt FIX.5.0SP2) Specifies the total fixed payment amount.*/
PaymentStreamWorldScaleRate                        = 41189, /*!< 41189 (float FIX.5.0SP2) The number of Worldscale points for purposes of the calculation of a fixed amount for a wet voyage charter commodity swap.*/
PaymentStreamContractPrice                         = 41190, /*!< 41190 (Price FIX.5.0SP2) The price per relevant unit for purposes of the calculation of a fixed amount for a dry voyage charter or time charter commodity swap.*/
PaymentStreamContractPriceCurrency                 = 41191, /*!< 41191 (Currency FIX.5.0SP2) Specifies the currency of PaymentStreamContractPrice(41190). Uses ISO 4217 currency codes.*/
NoPaymentStreamPricingBusinessCenters              = 41192, /*!< 41192 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
PaymentStreamPricingBusinessCenter                 = 41193, /*!< 41193 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's pricing dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentStreamRateIndex2CurvePeriod                 = 41194, /*!< 41194 (int FIX.5.0SP2) Secondary time unit multiplier for the payment stream's floating rate index curve.  */
PaymentStreamRateIndex2CurveUnit                   = 41195, /*!< 41195 (String FIX.5.0SP2) Secondary time unit associated with the payment stream's floating rate index curve.  */
PaymentStreamRateIndexLocation                     = 41196, /*!< 41196 (String FIX.5.0SP2) Specifies the location of the floating rate index.*/
PaymentStreamRateIndexLevel                        = 41197, /*!< 41197 (Qty FIX.5.0SP2) This is the weather Cooling Degree Days (CDD), Heating Degree Days (HDD) or HDD index level specified as the number of (amount of) weather index units specified by the parties in the related confirmation.*/
PaymentStreamRateIndexUnitOfMeasure                = 41198, /*!< 41198 (String FIX.5.0SP2) The unit of measure (UOM) of the rate index level.*/
PaymentStreamSettlLevel                            = 41199, /*!< 41199 (int FIX.5.0SP2) Specifies how weather index units are to be calculated.*/
PaymentStreamReferenceLevel                        = 41200, /*!< 41200 (Qty FIX.5.0SP2) This is the weather Cooling Degree Days (CDD), Heating Degree Days (HDD) or HDD reference level specified as the number of (amount of) weather index units specified by the parties in the related confirmation.*/
PaymentStreamReferenceLevelUnitOfMeasure           = 41201, /*!< 41201 (String FIX.5.0SP2) The unit of measure (UOM) of the rate reference level. */
PaymentStreamReferenceLevelEqualsZeroIndicator     = 41202, /*!< 41202 (Boolean FIX.5.0SP2) When set to 'Y', it indicates the weather reference level equals zero.*/
PaymentStreamRateSpreadCurrency                    = 41203, /*!< 41203 (Currency FIX.5.0SP2) Specifies the currency of the floating rate spread. Uses ISO 4217 currency codes.*/
PaymentStreamRateSpreadUnitOfMeasure               = 41204, /*!< 41204 (String FIX.5.0SP2) Species the unit of measure (UOM) of the floating rate spread. */
PaymentStreamRateConversionFactor                  = 41205, /*!< 41205 (float FIX.5.0SP2) The number to be multiplied by the derived floating rate of the payment stream in order to arrive at the payment rate. If omitted, the floating rate conversion factor is 1. */
PaymentStreamRateSpreadType                        = 41206, /*!< 41206 (int FIX.5.0SP2) Identifies whether the rate spread is an absolute value to be added to the index rate or a percentage of the index rate.*/
PaymentStreamLastResetRate                         = 41207, /*!< 41207 (Percentage FIX.5.0SP2) The floating rate determined at the most recent reset. The rate is expressed in decimal form, e.g. 5% is represented as 0.05.*/
PaymentStreamFinalRate                             = 41208, /*!< 41208 (Percentage FIX.5.0SP2) The floating rate determined at the final reset. The rate is expressed in decimal form, e.g. 5% is represented as 0.05.*/
PaymentStreamCalculationLagPeriod                  = 41209, /*!< 41209 (int FIX.5.0SP2) Time unit multiplier for the calculation lag duration.*/
PaymentStreamCalculationLagUnit                    = 41210, /*!< 41210 (String FIX.5.0SP2) Time unit associated with the calculation lag duration.*/
PaymentStreamFirstObservationDateOffsetPeriod      = 41211, /*!< 41211 (int FIX.5.0SP2) Time unit multiplier for the relative first observation date offset.*/
PaymentStreamFirstObservationDateOffsetUnit        = 41212, /*!< 41212 (String FIX.5.0SP2) Time unit associated with the relative first observation date offset.*/
PaymentStreamPricingDayType                        = 41213, /*!< 41213 (int FIX.5.0SP2) Specifies the commodity pricing day type.*/
PaymentStreamPricingDayDistribution                = 41214, /*!< 41214 (int FIX.5.0SP2) The distribution of pricing days.*/
PaymentStreamPricingDayCount                       = 41215, /*!< 41215 (int FIX.5.0SP2) The number of days over which pricing should take place.*/
PaymentStreamPricingBusinessCalendar               = 41216, /*!< 41216 (String FIX.5.0SP2) Specifies the business calendar to use for pricing. 

See http://www.fpml.org/coding-scheme/commodity-business-calendar for values.*/
PaymentStreamPricingBusinessDayConvention          = 41217, /*!< 41217 (int FIX.5.0SP2) The business day convention used to adjust the payent stream's pricing dates. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
DeliveryStreamRiskApportionmentSource              = 41218, /*!< 41218 (String FIX.5.0SP2) Specifies the source or legal framework for the risk apportionment.  

See http://www.fixtradingcommunity.org/codelists#Risk_Apportionment_Source for the details of the external code list.*/
LegDeliveryStreamRiskApportionmentSource           = 41219, /*!< 41219 (String FIX.5.0SP2) Specifies the source or legal framework for the risk apportionment.  

See http://www.fixtradingcommunity.org/codelists#Risk_Apportionment_Source for the details of the external code list.*/
NoPaymentStreamPaymentDates                        = 41220, /*!< 41220 (NumInGroup FIX.5.0SP2) Number of payment dates in the repeating group.*/
PaymentStreamPaymentDate                           = 41221, /*!< 41221 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted fixed stream payment date.*/
PaymentStreamPaymentDateType                       = 41222, /*!< 41222 (int FIX.5.0SP2) Specifies the type of payment date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
PaymentStreamMasterAgreementPaymentDatesIndicator  = 41223, /*!< 41223 (Boolean FIX.5.0SP2) When set to 'Y', it indicates that payment dates are specified in the relevant master agreement.*/
NoPaymentStreamPricingDates                        = 41224, /*!< 41224 (NumInGroup FIX.5.0SP2) Number of pricing dates in the repeating group.*/
PaymentStreamPricingDate                           = 41225, /*!< 41225 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted fixed stream pricing date.*/
PaymentStreamPricingDateType                       = 41226, /*!< 41226 (int FIX.5.0SP2) Specifies the type of pricing date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
NoPaymentStreamPricingDays                         = 41227, /*!< 41227 (NumInGroup FIX.5.0SP2) Number of pricing days in the repeating group.*/
PaymentStreamPricingDayOfWeek                      = 41228, /*!< 41228 (int FIX.5.0SP2) The day of the week on which pricing takes place.*/
PaymentStreamPricingDayNumber                      = 41229, /*!< 41229 (int FIX.5.0SP2) The occurrence of the day of week on which pricing takes place.  */
NoPricingDateBusinessCenters                       = 41230, /*!< 41230 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
PricingDateBusinessCenter                          = 41231, /*!< 41231 (String FIX.5.0SP2) The business center calendar used to adjust pricing or fixing dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PricingDateUnadjusted                              = 41232, /*!< 41232 (LocalMktDate FIX.5.0SP2) The unadjusted pricing or fixing date.*/
PricingDateBusinessDayConvention                   = 41233, /*!< 41233 (int FIX.5.0SP2) The business day convention used to adjust pricing or fixing dates. Used only to override the business day convention defined in the DateAdjustment component within the Instrument component.*/
PricingDateAdjusted                                = 41234, /*!< 41234 (LocalMktDate FIX.5.0SP2) The adjusted pricing or fixing date.*/
PricingTime                                        = 41235, /*!< 41235 (LocalMktTime FIX.5.0SP2) Specifies the local market time of the pricing or fixing.*/
PricingTimeBusinessCenter                          = 41236, /*!< 41236 (String FIX.5.0SP2) Specifies the business center for determining the pricing or fixing time. See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoStreamAssetAttributes                            = 41237, /*!< 41237 (NumInGroup FIX.5.0SP2) Number of asset attribute entries in the group.*/
StreamAssetAttributeType                           = 41238, /*!< 41238 (String FIX.5.0SP2) Specifies the name of the attribute.  

See http://www.fixtradingcommunity.org/codelists#Asset_Attribute_Types for code list of applicable asset attribute types.*/
StreamAssetAttributeValue                          = 41239, /*!< 41239 (String FIX.5.0SP2) Specifies the value of the attribute.*/
StreamAssetAttributeLimit                          = 41240, /*!< 41240 (String FIX.5.0SP2) Limit or lower acceptable value of the attribute.*/
NoStreamCalculationPeriodDates                     = 41241, /*!< 41241 (NumInGroup FIX.5.0SP2) Number of calculation period dates in the repeating group.*/
StreamCalculationPeriodDate                        = 41242, /*!< 41242 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted fixed calculation period date.*/
StreamCalculationPeriodDateType                    = 41243, /*!< 41243 (int FIX.5.0SP2) Specifies the type of fixed calculation period date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
StreamCalculationPeriodDatesXID                    = 41244, /*!< 41244 (XID FIX.5.0SP2) Identifier of this calculation period for cross referencing elsewhere in the message.*/
StreamCalculationPeriodDatesXIDRef                 = 41245, /*!< 41245 (XIDREF FIX.5.0SP2) Cross reference to another calculation period for duplicating its properties.*/
StreamCalculationBalanceOfFirstPeriod              = 41246, /*!< 41246 (Boolean FIX.5.0SP2) When specified and set to 'Y', it indicates that the first calculation period should run from the effective date to the end of the calendar period in which the effective date falls (e.g. Jan 15 - Jan 31 if the calculation periods are one month long and effective date is Jan 15.). If 'N' or not specified, it indicates that the first calculation period should run from the effective date for one whole period (e.g. Jan 15 to Feb 14 if the calculation periods are one month long and the effective date is Jan 15.).*/
StreamCalculationCorrectionPeriod                  = 41247, /*!< 41247 (int FIX.5.0SP2) Time unit multiplier for the length of time after the publication of the data when corrections can be made.*/
StreamCalculationCorrectionUnit                    = 41248, /*!< 41248 (String FIX.5.0SP2) Time unit associated with the length of time after the publication of the data when corrections can be made.*/
NoStreamCommoditySettlBusinessCenters              = 41249, /*!< 41249 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
StreamCommoditySettlBusinessCenter                 = 41250, /*!< 41250 (String FIX.5.0SP2) The business center calendar used to adjust the commodity delivery date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
StreamCommodityBase                                = 41251, /*!< 41251 (String FIX.5.0SP2) Specifies the general base type of the commodity traded. Where possible, this should follow the naming convention used in the 2005 ISDA Commodity Definitions. */
StreamCommodityType                                = 41252, /*!< 41252 (String FIX.5.0SP2) Specifies the type of commodity product.   

For coal see http://www.fpml.org/coding-scheme/commodity-coal-product-type for values. 

For metals see http://www.fpml.org/coding-scheme/commodity-metal-product-type for values.

For bullion see http://www.fixtradingcommunity.org/codelists#Bullion_Types for the external code list of bullion types.*/
StreamCommoditySecurityID                          = 41253, /*!< 41253 (String FIX.5.0SP2) Specifies the market identifier for the commodity.*/
StreamCommoditySecurityIDSource                    = 41254, /*!< 41254 (String FIX.5.0SP2) Identifies the class or source of the StreamCommoditySecurityIDSource(41253) value.*/
StreamCommodityDesc                                = 41255, /*!< 41255 (String FIX.5.0SP2) Description of the commodity asset.*/
EncodedStreamCommodityDescLen                      = 41256, /*!< 41256 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedStreamCommodityDesc(41257) field.*/
EncodedStreamCommodityDesc                         = 41257, /*!< 41257 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the StreamCommodityDesc(41255) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the StreamCommodityDesc(41255) field.*/
StreamCommodityUnitOfMeasure                       = 41258, /*!< 41258 (String FIX.5.0SP2) The unit of measure (UOM) of the commodity asset.*/
StreamCommodityCurrency                            = 41259, /*!< 41259 (Currency FIX.5.0SP2) Identifies the currency of the commodity asset. Uses ISO 4217 currency codes.*/
StreamCommodityExchange                            = 41260, /*!< 41260 (Exchange FIX.5.0SP2) Identifies the exchange where the commodity is traded.*/
StreamCommodityRateSource                          = 41261, /*!< 41261 (int FIX.5.0SP2) Identifies the source of rate information used for commodities. 

See http://www.fixtradingcommunity.org/codelists#Commodity_Rate_Source for code list of applicable sources.*/
StreamCommodityRateReferencePage                   = 41262, /*!< 41262 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.*/
StreamCommodityRateReferencePageHeading            = 41263, /*!< 41263 (String FIX.5.0SP2) Identifies the page heading from the rate source.*/
StreamDataProvider                                 = 41264, /*!< 41264 (String FIX.5.0SP2) Specifies the commodity data or information provider. 

See http://www.fpml.org/coding-scheme/commodity-information-provider for values.*/
StreamCommodityPricingType                         = 41265, /*!< 41265 (String FIX.5.0SP2) Specifies how the pricing or rate setting of the trade is to be determined or based upon.  

See http://www.fixtradingcommunity.org/codelists#Commodity_Rate_Pricing_Type for code list of applicable commodity pricing types.*/
StreamCommodityNearbySettlDayPeriod                = 41266, /*!< 41266 (int FIX.5.0SP2) Time unit multiplier for the nearby settlement day.*/
StreamCommodityNearbySettlDayUnit                  = 41267, /*!< 41267 (String FIX.5.0SP2) Time unit associated with the nearby settlement day.*/
StreamCommoditySettlDateUnadjusted                 = 41268, /*!< 41268 (LocalMktDate FIX.5.0SP2) The unadjusted commodity delivery date.*/
StreamCommoditySettlDateBusinessDayConvention      = 41269, /*!< 41269 (int FIX.5.0SP2) The business day convention used to adjust the commodity delivery date. Used only to override the business day convention specified in the DateAdjustment component within the Instrument component.*/
StreamCommoditySettlDateAdjusted                   = 41270, /*!< 41270 (LocalMktDate FIX.5.0SP2) The adjusted commodity delivery date.*/
StreamCommoditySettlMonth                          = 41271, /*!< 41271 (int FIX.5.0SP2) Specifies a fixed single month for commodity delivery. */
StreamCommoditySettlDateRollPeriod                 = 41272, /*!< 41272 (int FIX.5.0SP2) Time unit multiplier for the commodity delivery date roll.  */
StreamCommoditySettlDateRollUnit                   = 41273, /*!< 41273 (String FIX.5.0SP2) Time unit associated with the commodity delivery date roll.*/
StreamCommoditySettlDayType                        = 41274, /*!< 41274 (int FIX.5.0SP2) Specifies the commodity delivery roll day type.*/
StreamCommodityXID                                 = 41275, /*!< 41275 (XID FIX.5.0SP2) Identifier of this stream commodity for cross referencing elsewhere in the message.*/
StreamCommodityXIDRef                              = 41276, /*!< 41276 (XIDREF FIX.5.0SP2) Reference to a stream commodity elsewhere in the message.*/
NoStreamCommodityAltIDs                            = 41277, /*!< 41277 (NumInGroup FIX.5.0SP2) Number of alternate security identifers.*/
StreamCommodityAltID                               = 41278, /*!< 41278 (String FIX.5.0SP2) Alternate security identifier value for the commodity.*/
StreamCommodityAltIDSource                         = 41279, /*!< 41279 (String FIX.5.0SP2) Identifies the class or source of the alternate commodity security identifier.*/
NoStreamCommodityDataSources                       = 41280, /*!< 41280 (NumInGroup FIX.5.0SP2) Number of data sources in the repeating group. The order of entry determines priority – first is the main source, second is fallback, third is second fallback.*/
StreamCommodityDataSourceID                        = 41281, /*!< 41281 (String FIX.5.0SP2) Data source identifier.*/
StreamCommodityDataSourceIDType                    = 41282, /*!< 41282 (int FIX.5.0SP2) Type of data source identifier.*/
NoStreamCommoditySettlDays                         = 41283, /*!< 41283 (NumInGroup FIX.5.0SP2) Number of days in the repeating group.*/
StreamCommoditySettlDay                            = 41284, /*!< 41284 (int FIX.5.0SP2) Specifies the day or group of days for delivery.  */
StreamCommoditySettlTotalHours                     = 41285, /*!< 41285 (int FIX.5.0SP2) Sum of the hours specified in StreamCommoditySettlTimeGrp.*/
NoStreamCommoditySettlTimes                        = 41286, /*!< 41286 (NumInGroup FIX.5.0SP2) Number of hour ranges in the repeating group.*/
StreamCommoditySettlStart                          = 41287, /*!< 41287 (String FIX.5.0SP2) The start time for commodities settlement where delivery occurs over time. The time format is specified by the settlement time type.*/
StreamCommoditySettlEnd                            = 41288, /*!< 41288 (String FIX.5.0SP2) The end time for commodities settlement where delivery occurs over time. The time format is specified by the settlement time type.*/
NoStreamCommoditySettlPeriods                      = 41289, /*!< 41289 (NumInGroup FIX.5.0SP2) Number of commodity settlement periods in the repeating group.*/
StreamCommoditySettlCountry                        = 41290, /*!< 41290 (Country FIX.5.0SP2) Specifies the country where delivery takes place. Uses ISO 3166 2-character country code.*/
StreamCommoditySettlTimeZone                       = 41291, /*!< 41291 (String FIX.5.0SP2) Commodity delivery timezone specified as "prevailing" rather than "standard" or "daylight".  

See http://www.fixtradingcommunity.org/codelists#Prevailing_Timezones for code list of applicable prevailing timezones.*/
StreamCommoditySettlFlowType                       = 41292, /*!< 41292 (int FIX.5.0SP2) Specifies the commodity delivery flow type.*/
StreamCommoditySettlPeriodNotional                 = 41293, /*!< 41293 (Qty FIX.5.0SP2) Specifies the delivery quantity associated with this settlement period.*/
StreamCommoditySettlPeriodNotionalUnitOfMeasure    = 41294, /*!< 41294 (String FIX.5.0SP2) Specifies the unit of measure (UOM) of the delivery quantity associated with this settlement period.*/
StreamCommoditySettlPeriodFrequencyPeriod          = 41295, /*!< 41295 (int FIX.5.0SP2) Time unit multiplier for the settlement period frequency. */
StreamCommoditySettlPeriodFrequencyUnit            = 41296, /*!< 41296 (String FIX.5.0SP2) Time unit associated with the settlement period frequency. */
StreamCommoditySettlPeriodPrice                    = 41297, /*!< 41297 (Price FIX.5.0SP2) The settlement period price.*/
StreamCommoditySettlPeriodPriceUnitOfMeasure       = 41298, /*!< 41298 (String FIX.5.0SP2) Specifies the settlement period price unit of measure (UOM). */
StreamCommoditySettlPeriodPriceCurrency            = 41299, /*!< 41299 (Currency FIX.5.0SP2) The currency of the settlement period price. Uses ISO 4217 currency codes.*/
StreamCommoditySettlHolidaysProcessingInstruction  = 41300, /*!< 41300 (int FIX.5.0SP2) Indicates whether holidays are included in the settlement periods. Required for electricity contracts.*/
StreamCommoditySettlPeriodXID                      = 41301, /*!< 41301 (XID FIX.5.0SP2) Identifier of this settlement period for cross referencing elsewhere in the message.*/
StreamCommoditySettlPeriodXIDRef                   = 41302, /*!< 41302 (XIDREF FIX.5.0SP2) Cross reference to another settlement period for duplicating its properties.*/
StreamXID                                          = 41303, /*!< 41303 (XID FIX.5.0SP2) Identifier of this Stream for cross referencing elsewhere in the message.*/
PaymentLegRefID                                    = 41304, /*!< 41304 (String FIX.5.0SP2) Identifies the instrument leg in which this payment applies to by referencing the leg's LegID(1788).*/
StreamNotionalXIDRef                               = 41305, /*!< 41305 (XIDREF FIX.5.0SP2) Cross reference to another Stream notional for duplicating its properties.*/
StreamNotionalFrequencyPeriod                      = 41306, /*!< 41306 (int FIX.5.0SP2) Time unit multiplier for the swap stream's notional frequency. */
StreamNotionalFrequencyUnit                        = 41307, /*!< 41307 (String FIX.5.0SP2) Time unit associated with the swap stream's notional frequency. */
StreamNotionalCommodityFrequency                   = 41308, /*!< 41308 (int FIX.5.0SP2) The commodity's notional or quantity delivery frequency.*/
StreamNotionalUnitOfMeasure                        = 41309, /*!< 41309 (String FIX.5.0SP2) Specifies the delivery stream quantity unit of measure (UOM). */
StreamTotalNotional                                = 41310, /*!< 41310 (Qty FIX.5.0SP2) Total notional or delivery quantity over the term of the contract.*/
StreamTotalNotionalUnitOfMeasure                   = 41311, /*!< 41311 (String FIX.5.0SP2) Specifies the unit of measure (UOM) for the total notional or delivery quantity over the term of the contract.*/
NoMandatoryClearingJurisdictions                   = 41312, /*!< 41312 (NumInGroup FIX.5.0SP2) Number of mandatory clearing jurisdictions.*/
MandatoryClearingJurisdiction                      = 41313, /*!< 41313 (String FIX.5.0SP2) Identifier of the regulatory jurisdiction requiring the trade to be cleared.*/
UnderlyingProtectionTermXIDRef                     = 41314, /*!< 41314 (XIDREF FIX.5.0SP2) Reference to the protection terms applicable to this entity or obligation. Contains the same XID named string value of the instance in the ProtectionTerms repeating group that applies to this Underlying.*/
UnderlyingSettlTermXIDRef                          = 41315, /*!< 41315 (XIDREF FIX.5.0SP2) Reference to the cash or physical settlement terms applicable to this entity or obligation. Contains the same XID named string value of the instance in the appropriate repeating group that applies to this Underlying.*/
NoLegAdditionalTermBondRefs                        = 41316, /*!< 41316 (NumInGroup FIX.5.0SP2) Number of bonds in the repeating group.*/
LegAdditionalTermBondSecurityID                    = 41317, /*!< 41317 (String FIX.5.0SP2) Security identifier of the bond.*/
LegAdditionalTermBondSecurityIDSource              = 41318, /*!< 41318 (String FIX.5.0SP2) Identifies the source scheme of the LegAdditionalTermBondSecurityID(41317) value.*/
LegAdditionalTermBondDesc                          = 41319, /*!< 41319 (String FIX.5.0SP2) Description of the bond.*/
EncodedLegAdditionalTermBondDescLen                = 41320, /*!< 41320 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegAdditionalTermBondDesc(41321) field.*/
EncodedLegAdditionalTermBondDesc                   = 41321, /*!< 41321 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegAdditionalTermBondDesc(41319) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegAdditionalTermBondDesc(41319) field.*/
LegAdditionalTermBondCurrency                      = 41322, /*!< 41322 (Currency FIX.5.0SP2) Specifies the currency the bond value is denominated in.  Uses ISO 4217 currency codes.*/
LegAdditionalTermBondIssuer                        = 41323, /*!< 41323 (String FIX.5.0SP2) Issuer of the bond.*/
EncodedLegAdditionalTermBondIssuerLen              = 41324, /*!< 41324 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegAdditionalTermBondIssuer(41325) field.*/
EncodedLegAdditionalTermBondIssuer                 = 41325, /*!< 41325 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegAdditionalTermBondIssuer(41323) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegAdditionalTermBondIssuer(41323) field.*/
LegAdditionalTermBondSeniority                     = 41326, /*!< 41326 (String FIX.5.0SP2) Specifies the bond's payment priority in the event of a default.*/
LegAdditionalTermBondCouponType                    = 41327, /*!< 41327 (int FIX.5.0SP2) Specifies the coupon type of the bond.*/
LegAdditionalTermBondCouponRate                    = 41328, /*!< 41328 (Percentage FIX.5.0SP2) Coupon rate of the bond.  See also CouponRate(223).*/
LegAdditionalTermBondMaturityDate                  = 41329, /*!< 41329 (LocalMktDate FIX.5.0SP2) The maturity date of the bond.*/
LegAdditionalTermBondParValue                      = 41330, /*!< 41330 (Amt FIX.5.0SP2) The par value of the bond.*/
LegAdditionalTermBondCurrentTotalIssuedAmount      = 41331, /*!< 41331 (Amt FIX.5.0SP2) Total issued amount of the bond.*/
LegAdditionalTermBondCouponFrequencyPeriod         = 41332, /*!< 41332 (int FIX.5.0SP2) Time unit multiplier for the frequency of the bond's coupon payment. */
LegAdditionalTermBondCouponFrequencyUnit           = 41333, /*!< 41333 (String FIX.5.0SP2) Time unit associated with the frequency of the bond's coupon payment.  */
LegAdditionalTermBondDayCount                      = 41334, /*!< 41334 (int FIX.5.0SP2) The day count convention used in interest calculations for a bond or an interest bearing security.*/
NoLegAdditionalTerms                               = 41335, /*!< 41335 (NumInGroup FIX.5.0SP2) Number of additional terms in the repeating group.*/
LegAdditionalTermConditionPrecedentBondIndicator   = 41336, /*!< 41336 (Boolean FIX.5.0SP2) Indicates whether the condition precedent bond is applicable. The swap contract is only valid if the bond is issued and if there is any dispute over the terms of fixed stream then the bond terms would be used.*/
LegAdditionalTermDiscrepancyClauseIndicator        = 41337, /*!< 41337 (Boolean FIX.5.0SP2) Indicates whether the discrepancy clause is applicable.*/
UnderlyingMarketDisruptionValue                    = 41338, /*!< 41338 (String FIX.5.0SP2) Applicable value for UnderlyingMarketDisruptionEvent(41865).*/
UnderlyingMarketDisruptionFallbackValue            = 41339, /*!< 41339 (String FIX.5.0SP2) Applicable value for UnderlyingMarketDisruptionFallbackType(41867).*/
NoUnderlyingAdditionalTermBondRefs                 = 41340, /*!< 41340 (NumInGroup FIX.5.0SP2) Number of bonds in the repeating group.*/
UnderlyingAdditionalTermBondSecurityID             = 41341, /*!< 41341 (String FIX.5.0SP2) Security identifier of the bond.  */
NoLegCashSettlDealers                              = 41342, /*!< 41342 (NumInGroup FIX.5.0SP2) Number of dealers in the repeating group.*/
LegCashSettlDealer                                 = 41343, /*!< 41343 (String FIX.5.0SP2) Identifies the dealer from whom price quotations for the reference obligation are obtained for the purpose of cash settlement valuation calculation.*/
NoLegCashSettlTerms                                = 41344, /*!< 41344 (NumInGroup FIX.5.0SP2) Number of elements in the repeating group.*/
LegCashSettlCurrency                               = 41345, /*!< 41345 (Currency FIX.5.0SP2) Specifies the currency the LegCashSettlAmount(41357) is denominated in.  Uses ISO 4217 currency codes.*/
LegCasSettlValuationFirstBusinessDayOffset         = 41346, /*!< 41346 (int FIX.5.0SP2) The number of business days after settlement conditions have been satisfied, when the calculation agent is to obtain a price quotation on the reference obligation for purposes of cash settlement.*/
LegCashSettlValuationSubsequentBusinessDaysOffset  = 41347, /*!< 41347 (int FIX.5.0SP2) The number of business days between successive valuation dates when multiple valuation dates are applicable for cash settlement.*/
LegCashSettlNumOfValuationDates                    = 41348, /*!< 41348 (int FIX.5.0SP2) Where multiple valuation dates are specified as being applicable for cash settlement, this element specifies the number of applicable valuation dates.*/
LegCashSettlValuationTime                          = 41349, /*!< 41349 (LocalMktTime FIX.5.0SP2) Time of valuation.*/
LegCashSettlBusinessCenter                         = 41350, /*!< 41350 (String FIX.5.0SP2) Identifies the business center calendar used at valuation time for cash settlement purposes e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegCashSettlQuoteMethod                            = 41351, /*!< 41351 (int FIX.5.0SP2) The type of quote used to determine the cash settlement price.*/
LegCashSettlQuoteAmount                            = 41352, /*!< 41352 (Amt FIX.5.0SP2) When determining the cash settlement amount, if weighted average price quotes are to be obtained for the reference obligation, this is the upper limit to the outstanding principal balance of the reference obligation for which the quote should be obtained.  If not specifed, the ISDA definitions provide for a fallback amount equal to floating rate payer calculation amount.*/
LegCashSettlQuoteCurrency                          = 41353, /*!< 41353 (Currency FIX.5.0SP2) Specifies the currency the LegCashSettlQuoteAmount(41352) is denominated in.  Uses ISO 4217 Currency Code.*/
LegCashSettlMinimumQuoteAmount                     = 41354, /*!< 41354 (Amt FIX.5.0SP2) When determining the cash settlement amount, if weighted average price quotes are to be obtained for the reference obligation, this is the minimum intended threshold amount of outstanding principal balance of the reference obligation for which the quote should be obtained.  If not specified, the ISDA definitions provide for a fallback amount of the lower of either USD1,000,000 (or its equivalent in the relevent obligation currency) or the (minimum) quoted amount.*/
LegCashSettlMinimumQuoteCurrency                   = 41355, /*!< 41355 (Currency FIX.5.0SP2) Specifies the currency the LegCashSettlQuoteMinimumAmount(41354) is denominated in.  Uses ISO 4217 Currency Code. */
LegCashSettlBusinessDays                           = 41356, /*!< 41356 (int FIX.5.0SP2) The number of business days used in the determination of the cash settlement payment date. */
LegCashSettlAmount                                 = 41357, /*!< 41357 (Amt FIX.5.0SP2) The amount paid between the trade parties, seller to the buyer, for cash settlement on the cash settlement date.*/
LegCashSettlRecoveryFactor                         = 41358, /*!< 41358 (float FIX.5.0SP2) Used for fixed recovery, this specifies the recovery level as determined at contract inception, to be applied in the event of a default.  The factor is used to calculate the amount paid by the seller to the buyer for cash settlement on the cash settlement date.  The amount calculated is (1 - LegCashSettlRecoveryFactor(41358)) x floating rate payer calculation amount.  The currency is derived from the floating rate payer calculation amount.*/
LegCashSettlFixedTermIndicator                     = 41359, /*!< 41359 (Boolean FIX.5.0SP2) Indicates whether fixed settlement is applicable or not applicable in a recovery lock.*/
LegCashSettlAccruedInterestIndicator               = 41360, /*!< 41360 (Boolean FIX.5.0SP2) Indicates whether accrued interest is included or not in the value provided in LegCashSettlAmount(41357).

For cash settlement this specifies whether quotations should be obtained inclusive or not of accrued interest. 

For physical settlement this specifies whether the buyer should deliver the obligation with an outstanding principal balance that includes or excludes accrued interest. */
LegCashSettlValuationMethod                        = 41361, /*!< 41361 (int FIX.5.0SP2) The ISDA defined methodology for determining the final price of the reference obligation for purposes of cash settlement. */
LegCashSettlTermXID                                = 41362, /*!< 41362 (XID FIX.5.0SP2) A named string value referenced by UnderlyingSettlTermXIDRef(41315).*/
NoLegComplexEventAveragingObservations             = 41363, /*!< 41363 (NumInGroup FIX.5.0SP2) The number of averaging observations in the repeating group.*/
LegComplexEventAveragingObservationNumber          = 41364, /*!< 41364 (int FIX.5.0SP2) Cross reference to the ordinal observation as specified either in the LegComplexEventScheduleGrp or LegComplexEventPeriodDateGrp components.*/
LegComplexEventAveragingWeight                     = 41365, /*!< 41365 (float FIX.5.0SP2) The weight factor to be applied to the observation.*/
NoLegComplexEventCreditEvents                      = 41366, /*!< 41366 (NumInGroup FIX.5.0SP2) The number of credit events specified in the repeating group.*/
LegComplexEventCreditEventType                     = 41367, /*!< 41367 (String FIX.5.0SP2) Specifies the type of credit event.  

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Types for code list of applicable event types.*/
LegComplexEventCreditEventValue                    = 41368, /*!< 41368 (String FIX.5.0SP2) The credit event value appropriate to LegComplexEventCreditEventType(41367). 

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Types for applicable event type values. */
LegComplexEventCreditEventCurrency                 = 41369, /*!< 41369 (Currency FIX.5.0SP2) Specifies the applicable currency when LegComplexEventCreditEventCurrency(41368) is an amount. Uses ISO 4217 currency codes.*/
LegComplexEventCreditEventPeriod                   = 41370, /*!< 41370 (int FIX.5.0SP2) Time unit multiplier for complex credit events.*/
LegComplexEventCreditEventUnit                     = 41371, /*!< 41371 (String FIX.5.0SP2) Time unit associated with complex credit events.*/
LegComplexEventCreditEventDayType                  = 41372, /*!< 41372 (int FIX.5.0SP2) Specifies the day type for the complex credit events.*/
LegComplexEventCreditEventRateSource               = 41373, /*!< 41373 (int FIX.5.0SP2) Identifies the source of rate information used for credit events. 

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Rate_Source for code list of applicable sources.*/
NoLegComplexEventCreditEventQualifiers             = 41374, /*!< 41374 (NumInGroup FIX.5.0SP2) Number of qualifiers in the repeating group.*/
LegComplexEventCreditEventQualifier                = 41375, /*!< 41375 (char FIX.5.0SP2) Specifies a complex event qualifier. Used to further qualify LegComplexEventCreditEventType(41367). */
NoLegComplexEventPeriodDateTimes                   = 41376, /*!< 41376 (NumInGroup FIX.5.0SP2) Number of entries in the date-time repeating group.*/
LegComplexEventPeriodDate                          = 41377, /*!< 41377 (LocalMktDate FIX.5.0SP2) Averaging date for an Asian option. 

Trigger date for a Barrier or Knock option.*/
LegComplexEventPeriodTime                          = 41378, /*!< 41378 (LocalMktTime FIX.5.0SP2) Averaging time for an Asian option.*/
NoLegComplexEventPeriods                           = 41379, /*!< 41379 (NumInGroup FIX.5.0SP2) Number of periods in the repeating group.*/
LegComplexEventPeriodType                          = 41380, /*!< 41380 (int FIX.5.0SP2) Specifies the period type.*/
LegComplexEventBusinessCenter                      = 41381, /*!< 41381 (String FIX.5.0SP2) The business center for adjusting dates and times in the schedule or date-time group.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegComplexEventRateSources                       = 41382, /*!< 41382 (NumInGroup FIX.5.0SP2) Number of rate sources in the repeating group.*/
LegComplexEventRateSource                          = 41383, /*!< 41383 (int FIX.5.0SP2) Identifies the source of rate information.

For FX, the reference source to be used for the FX spot rate.*/
LegComplexEventRateSourceType                      = 41384, /*!< 41384 (int FIX.5.0SP2) Indicates whether the rate source specified is a primary or secondary source.*/
LegComplexEventReferencePage                       = 41385, /*!< 41385 (String FIX.5.0SP2) Identifies the reference page from the rate source.

For FX, the reference page to the spot rate is to be used for the reference FX spot rate.

When LegComplexEventRateSource(41383) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option.*/
LegComplexEvenReferencePageHeading                 = 41386, /*!< 41386 (String FIX.5.0SP2) Identifies the reference page heading from the rate source.*/
NoLegComplexEventDateBusinessCenters               = 41387, /*!< 41387 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegComplexEventDateBusinessCenter                  = 41388, /*!< 41388 (String FIX.5.0SP2) The business center calendar used to adjust the event date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegComplexEventDateUnadjusted                      = 41389, /*!< 41389 (LocalMktDate FIX.5.0SP2) The unadjusted complex event date.*/
LegComplexEventDateRelativeTo                      = 41390, /*!< 41390 (int FIX.5.0SP2) Specifies the anchor date when the complex event date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegComplexEventDateOffsetPeriod                    = 41391, /*!< 41391 (int FIX.5.0SP2) Time unit multiplier for the relative date offset.*/
LegComplexEventDateOffsetUnit                      = 41392, /*!< 41392 (String FIX.5.0SP2) Time unit associated with the relative date offset.*/
LegComplexEventDateOffsetDayType                   = 41393, /*!< 41393 (int FIX.5.0SP2) Specifies the day type of the relative date offset.*/
LegComplexEventDateBusinessDayConvention           = 41394, /*!< 41394 (int FIX.5.0SP2) The business day convention used to adjust the event date.  Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegComplexEventDateAdjusted                        = 41395, /*!< 41395 (LocalMktDate FIX.5.0SP2) The adjusted complex event date.*/
LegComplexEventFixingTime                          = 41396, /*!< 41396 (LocalMktTime FIX.5.0SP2) The local market fixing time.*/
LegComplexEventFixingTimeBusinessCenter            = 41397, /*!< 41397 (String FIX.5.0SP2) The business center for determining the actual fixing times.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegComplexEventCreditEventSources                = 41398, /*!< 41398 (NumInGroup FIX.5.0SP2) Number of event sources in the repeating group.*/
LegComplexEventCreditEventSource                   = 41399, /*!< 41399 (String FIX.5.0SP2) A newspaper or electronic news service that may publish relevant information used in the determination of whether or not a credit event has occurred.*/
NoLegComplexEventSchedules                         = 41400, /*!< 41400 (NumInGroup FIX.5.0SP2) Number of schedules in the repeating group.*/
LegComplexEventScheduleStartDate                   = 41401, /*!< 41401 (LocalMktDate FIX.5.0SP2) The start date of the schedule.*/
LegComplexEventScheduleEndDate                     = 41402, /*!< 41402 (LocalMktDate FIX.5.0SP2) The end date of the schedule.*/
LegComplexEventScheduleFrequencyPeriod             = 41403, /*!< 41403 (int FIX.5.0SP2) Time unit multiplier for the schedule date frequency. */
LegComplexEventScheduleFrequencyUnit               = 41404, /*!< 41404 (String FIX.5.0SP2) Time unit associated with the schedule date frequency.*/
LegComplexEventScheduleRollConvention              = 41405, /*!< 41405 (String FIX.5.0SP2) The convention for determining the sequence of dates. It is used in conjunction with a specified frequency. Used only to override the roll convention defined in the LegDateAdjustment component in InstrumentLeg.*/
ProvisionCashSettlQuoteReferencePage               = 41406, /*!< 41406 (String FIX.5.0SP2) Identifies the reference "page" from the quote source.*/
LegProvisionCashSettlQuoteReferencePage            = 41407, /*!< 41407 (String FIX.5.0SP2) Identifies the reference "page" from the quote source.*/
NoLegDeliverySchedules                             = 41408, /*!< 41408 (NumInGroup FIX.5.0SP2) Number of delivery schedules in the repeating group.*/
LegDeliveryScheduleType                            = 41409, /*!< 41409 (int FIX.5.0SP2) Specifies the type of delivery schedule.*/
LegDeliveryScheduleXID                             = 41410, /*!< 41410 (XID FIX.5.0SP2) Identifier for this instance of delivery schedule for cross referencing elsewhere in the message.*/
LegDeliveryScheduleNotional                        = 41411, /*!< 41411 (Qty FIX.5.0SP2) Physical delivery quantity.*/
LegDeliveryScheduleNotionalUnitOfMeasure           = 41412, /*!< 41412 (String FIX.5.0SP2) Specifies the delivery quantity unit of measure (UOM). */
LegDeliveryScheduleNotionalCommodityFrequency      = 41413, /*!< 41413 (int FIX.5.0SP2) The frequency of notional delivery.*/
LegDeliveryScheduleNegativeTolerance               = 41414, /*!< 41414 (float FIX.5.0SP2) Specifies the negative tolerance value.  The value may be an absolute quantity or a percentage, as specified in LegDeliveryScheduleToleranceType(41417).  Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
LegDeliverySchedulePositiveTolerance               = 41415, /*!< 41415 (float FIX.5.0SP2) Specifies the positive tolerance value. The value may be an absolute quantity or a percentage, as specified in LegDeliveryScheduleToleranceType(41417). Value may exceed agreed upon value. Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
LegDeliveryScheduleToleranceUnitOfMeasure          = 41416, /*!< 41416 (String FIX.5.0SP2) Specifies the tolerance value's unit of measure (UOM).*/
LegDeliveryScheduleToleranceType                   = 41417, /*!< 41417 (int FIX.5.0SP2) Specifies the tolerance value type.*/
LegDeliveryScheduleSettlCountry                    = 41418, /*!< 41418 (Country FIX.5.0SP2) Specifies the country where delivery takes place. Uses ISO 3166 2-character country code.*/
LegDeliveryScheduleSettlTimeZone                   = 41419, /*!< 41419 (String FIX.5.0SP2) Delivery timezone specified as "prevailing" rather than "standard" or "daylight".  

See http://www.fixtradingcommunity.org/codelists#Prevailing_Timezones for code list of applicable prevailing timezones.*/
LegDeliveryScheduleSettlFlowType                   = 41420, /*!< 41420 (int FIX.5.0SP2) Specifies the delivery flow type.*/
LegDeliveryScheduleSettlHolidaysProcessingInstruction = 41421, /*!< 41421 (int FIX.5.0SP2) Indicates whether holidays are included in the settlement periods. Required for electricity contracts.*/
NoLegDeliveryScheduleSettlDays                     = 41422, /*!< 41422 (NumInGroup FIX.5.0SP2) Number of delivery schedules in the repeating group.*/
LegDeliveryScheduleSettlDay                        = 41423, /*!< 41423 (int FIX.5.0SP2) Specifies the day or group of days for delivery.*/
LegDeliveryScheduleSettlTotalHours                 = 41424, /*!< 41424 (int FIX.5.0SP2) The sum of the total hours specified in the LegDeliveryScheduleSettlTimeGrp component.*/
NoLegDeliveryScheduleSettlTimes                    = 41425, /*!< 41425 (NumInGroup FIX.5.0SP2) Number of hour ranges in the repeating group.*/
LegDeliveryScheduleSettlStart                      = 41426, /*!< 41426 (String FIX.5.0SP2) The scheduled start time for the delivery of the commodity where delivery occurs over specified times. The format of the time value is specified in LegDeliveryScheduleSettlTimeType(41428).*/
LegDeliveryScheduleSettlEnd                        = 41427, /*!< 41427 (String FIX.5.0SP2) The scheduled end time for the delivery of the commodity where delivery occurs over specified times. The format of the time value is specified in LegDeliveryScheduleSettlTimeType(41428).*/
LegDeliveryScheduleSettlTimeType                   = 41428, /*!< 41428 (int FIX.5.0SP2) Specifies the format of the delivery start and end time values.*/
LegDeliveryStreamType                              = 41429, /*!< 41429 (int FIX.5.0SP2) Specifies the type of delivery stream.*/
LegDeliveryStreamPipeline                          = 41430, /*!< 41430 (String FIX.5.0SP2) The name of the oil delivery pipeline.*/
LegDeliveryStreamEntryPoint                        = 41431, /*!< 41431 (String FIX.5.0SP2) The point at which the commodity will enter the delivery mechanism or pipeline.*/
LegDeliveryStreamWithdrawalPoint                   = 41432, /*!< 41432 (String FIX.5.0SP2) The point at which the commodity product will be withdrawn prior to delivery.*/
LegDeliveryStreamDeliveryPoint                     = 41433, /*!< 41433 (String FIX.5.0SP2) The point at which the commodity product will be delivered and received. Value specified should follow market convention appropriate for the commodity product. 

For bullion, see http://www.fpml.org/coding-scheme/bullion-delivery-location for values.*/
LegDeliveryStreamDeliveryRestriction               = 41434, /*!< 41434 (int FIX.5.0SP2) Specifies under what conditions the buyer and seller should be excused of their delivery obligations.*/
LegDeliveryStreamDeliveryContingency               = 41435, /*!< 41435 (String FIX.5.0SP2) Specifies the electricity delivery contingency. See

http://www.fpml.org/coding-scheme/electricity-transmission-contingency for values.*/
LegDeliveryStreamDeliveryContingentPartySide       = 41436, /*!< 41436 (int FIX.5.0SP2) The trade side value of the party responsible for electricity delivery contingency.*/
LegDeliveryStreamDeliverAtSourceIndicator          = 41437, /*!< 41437 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', delivery of the coal product is to be at its source.*/
LegDeliveryStreamRiskApportionment                 = 41438, /*!< 41438 (String FIX.5.0SP2) Specifies how the parties to the trade apportion responsibility for the delivery of the commodity product. 

See http://www.fixtradingcommunity.org/codelists#Risk_Apportionment for the details of the external code list.*/
LegDeliveryStreamTitleTransferLocation             = 41439, /*!< 41439 (String FIX.5.0SP2) Specifies the title transfer location.*/
LegDeliveryStreamTitleTransferCondition            = 41440, /*!< 41440 (int FIX.5.0SP2) Specifies the condition of title transfer.*/
LegDeliveryStreamImporterOfRecord                  = 41441, /*!< 41441 (String FIX.5.0SP2) A party, not necessarily of the trade, who is the Importer of Record for the purposes of paying customs duties and applicable taxes or costs related to importation.*/
LegDeliveryStreamNegativeTolerance                 = 41442, /*!< 41442 (float FIX.5.0SP2) Specifies the negative tolerance value.  The value may be an absolute quantity or a percentage, as specified in LegDeliveryStreamToleranceType(41445).  Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
LegDeliveryStreamPositiveTolerance                 = 41443, /*!< 41443 (float FIX.5.0SP2) Specifies the positive tolerance value. The value may be an absolute quantity or a percentage, as specified in LegDeliveryStreamToleranceType(41445). Value may exceed agreed upon value. Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
LegDeliveryStreamToleranceUnitOfMeasure            = 41444, /*!< 41444 (String FIX.5.0SP2) Specifies the tolerance value's unit of measure (UOM).*/
LegDeliveryStreamToleranceType                     = 41445, /*!< 41445 (int FIX.5.0SP2) Specifies the tolerance value type.*/
LegDeliveryStreamToleranceOptionSide               = 41446, /*!< 41446 (int FIX.5.0SP2) Indicates whether the tolerance is at the seller's or buyer's option.*/
LegDeliveryStreamTotalPositiveTolerance            = 41447, /*!< 41447 (Percentage FIX.5.0SP2) The positive percent tolerance which applies to the total quantity delivered over all shipment periods.

Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%.).*/
LegDeliveryStreamTotalNegativeTolerance            = 41448, /*!< 41448 (Percentage FIX.5.0SP2) The negative percent tolerance which applies to the total quantity delivered over all shipment periods.

Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%.).*/
LegDeliveryStreamNotionalConversionFactor          = 41449, /*!< 41449 (float FIX.5.0SP2) If the notional quantity is specified in a unit that does not match the unit in which the commodity reference price is quoted, the scaling or conversion factor used to convert the commodity reference price unit into the notional quantity unit should be stated here. If there is no conversion, this field is not intended to be used.*/
LegDeliveryStreamTransportEquipment                = 41450, /*!< 41450 (String FIX.5.0SP2) The transportation equipment with which the commodity product will be delivered and received.*/
LegDeliveryStreamElectingPartySide                 = 41451, /*!< 41451 (int FIX.5.0SP2) A reference to the party able to choose whether the gas is delivered for a particular period e.g. a swing or interruptible contract.*/
NoLegStreamAssetAttributes                         = 41452, /*!< 41452 (NumInGroup FIX.5.0SP2) Number of asset attribute entries in the group.*/
LegStreamAssetAttributeType                        = 41453, /*!< 41453 (String FIX.5.0SP2) Specifies the name of the attribute.  

See http://www.fixtradingcommunity.org/codelists#Asset_Attribute_Types for code list of applicable asset attribute types.*/
LegStreamAssetAttributeValue                       = 41454, /*!< 41454 (String FIX.5.0SP2) Specifies the value of the attribute.*/
LegStreamAssetAttributeLimit                       = 41455, /*!< 41455 (String FIX.5.0SP2) Limit or lower acceptable value of the attribute.*/
NoLegDeliveryStreamCycles                          = 41456, /*!< 41456 (NumInGroup FIX.5.0SP2) Number of commodity sources in the repeating group.*/
LegDeliveryStreamCycleDesc                         = 41457, /*!< 41457 (String FIX.5.0SP2) The delivery cycles during which the oil product will be transported in the pipeline. */
EncodedLegDeliveryStreamCycleDescLen               = 41458, /*!< 41458 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegDeliveryStreamCycleDesc(41459) field.*/
EncodedLegDeliveryStreamCycleDesc                  = 41459, /*!< 41459 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegDeliveryStreamCycleDesc(41457) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegLeg DeliveryStream(41457) field.*/
NoLegDeliveryStreamCommoditySources                = 41460, /*!< 41460 (NumInGroup FIX.5.0SP2) Number of commodity sources in the repeating group.*/
LegDeliveryStreamCommoditySource                   = 41461, /*!< 41461 (String FIX.5.0SP2) The SCoTA coal cargo origin, mining region, mine(s), mining complex(es), loadout(s) or river dock(s) or other point(s) of origin that seller and buyer agree are acceptable origins for the coal product. For international coal transactions, this is the origin of the coal product. 

See http://www.fpml.org/coding-scheme/commodity-coal-product-source for values.*/
LegMarketDisruptionProvision                       = 41462, /*!< 41462 (int FIX.5.0SP2) The consequences of market disruption events.*/
LegMarketDisruptionFallbackProvision               = 41463, /*!< 41463 (int FIX.5.0SP2) Specifies the location of the fallback provision documentation.*/
LegMarketDisruptionMaximumDays                     = 41464, /*!< 41464 (int FIX.5.0SP2) Specifies the maximum number of market disruption days (commodity or bullion business days) in a contract or confirmation. If none are specified, the maximum number of market disruption days is five (5).*/
LegMarketDisruptionMaterialityPercentage           = 41465, /*!< 41465 (Percentage FIX.5.0SP2) Used when a price materiality percentage applies to the price source disruption event and this event has been specified.*/
LegMarketDisruptionMinimumFuturesContracts         = 41466, /*!< 41466 (int FIX.5.0SP2) Specifies the minimum futures contracts level that dictates whether or not a 'De Minimis Trading' event has occurred. */
NoLegMarketDisruptionEvents                        = 41467, /*!< 41467 (NumInGroup FIX.5.0SP2) Number of disruption events in the repeating group.*/
LegMarketDisruptionEvent                           = 41468, /*!< 41468 (String FIX.5.0SP2) Specifies the market disruption event. 

For commodities see http://www.fpml.org/coding-scheme/commodity-market-disruption for values. 

For foreign exchange, see http://www.fixtradingcommunity.org/codelists#Market_Disruption_Event for code list of applicable event types.*/
NoLegMarketDisruptionFallbacks                     = 41469, /*!< 41469 (NumInGroup FIX.5.0SP2) Number of fallbacks in the repeating group.*/
LegMarketDisruptionFallbackType                    = 41470, /*!< 41470 (String FIX.5.0SP2) Specifies the type of disruption fallback.  

See http://www.fpml.org/coding-scheme/commodity-market-disruption-fallback for values. */
NoLegMarketDisruptionFallbackReferencePrices       = 41471, /*!< 41471 (NumInGroup FIX.5.0SP2) Number of fallback reference securities in the repeating group.*/
LegMarketDisruptionFallbackUnderlierType           = 41472, /*!< 41472 (int FIX.5.0SP2) The type of reference price underlier.*/
LegMarketDisruptionFallbackUnderlierSecurityID     = 41473, /*!< 41473 (String FIX.5.0SP2) Specifies the identifier value of the security.*/
LegMarketDisruptionFallbackUnderlierSecurityIDSource = 41474, /*!< 41474 (String FIX.5.0SP2) Specifies the class or source scheme of the security identifier.*/
LegMarketDisruptionFallbackUnderlierSecurityDesc   = 41475, /*!< 41475 (String FIX.5.0SP2) Specifies the description of the underlying security.*/
EncodedLegMarketDisruptionFallbackUnderlierSecurityDescLen = 41476, /*!< 41476 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegMarketDisruptionFallbackUnderlierSecurityDesc (41477) field.*/
EncodedLegMarketDisruptionFallbackUnderlierSecurityDesc = 41477, /*!< 41477 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegMarketDisruptionFallbackUnderlierSecurityDesc(41475) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegMarketDisruptionFallbackUnderlierSecurityDesc(41475) field.*/
LegMarketDisruptionFallbackOpenUnits               = 41478, /*!< 41478 (Qty FIX.5.0SP2) If there are multiple underlying assets, this specifies the number of units (index or securities) that constitute the underlier of the swap. In the case of a basket swap, this is used to reference both the number of basket units, and the number of each asset components of the basket when these are expressed in absolute terms.*/
LegMarketDisruptionFallbackBasketCurrency          = 41479, /*!< 41479 (Currency FIX.5.0SP2) Specifies the currency if the underlier is a basket.  Uses ISO 4217 currency codes.*/
LegMarketDisruptionFallbackBasketDivisor           = 41480, /*!< 41480 (float FIX.5.0SP2) Specifies the basket divisor amount. This value is normally used to adjust the constituent weight for pricing or to adjust for dividends, or other corporate actions.*/
LegExerciseDesc                                    = 41481, /*!< 41481 (String FIX.5.0SP2) A description of the option exercise.*/
EncodedLegExerciseDescLen                          = 41482, /*!< 41482 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegExerciseDesc(41483) field.*/
EncodedLegExerciseDesc                             = 41483, /*!< 41483 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegExerciseDesc(41481) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegExerciseDesc(41481) field.*/
LegAutomaticExerciseIndicator                      = 41484, /*!< 41484 (Boolean FIX.5.0SP2) Indicates (when 'Y') that exercise is automatic when the strike price is crossed or the underlying trade is in the money.*/
LegAutomaticExerciseThresholdRate                  = 41485, /*!< 41485 (float FIX.5.0SP2) The threshold rate for triggering automatic exercise.*/
LegExerciseConfirmationMethod                      = 41486, /*!< 41486 (int FIX.5.0SP2) Indicates whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller's agent.*/
LegManualNoticeBusinessCenter                      = 41487, /*!< 41487 (String FIX.5.0SP2) Identifies the business center used for adjusting the time for manual exercise notice.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegFallbackExerciseIndicator                       = 41488, /*!< 41488 (Boolean FIX.5.0SP2) Indicates whether the notional amount of the underlying swap, not previously exercised under the option, will be automatically exercised at the expiration time on the expiration date if at such time the buyer is in-the-money, provided that the difference between the settlement rate and the fixed rate under the relevant underlying swap is not less than one tenth of a percentage point (0.10% or 0.001).*/
LegLimitRightToConfirmIndicator                    = 41489, /*!< 41489 (Boolean FIX.5.0SP2) Indicates whether the Seller may request the Buyer to confirm its intent to exercise if not done on or before the expiration time on the expiration date. If true ("Y") specific rules will apply in relation to the settlement mode.*/
LegExerciseSplitTicketIndicator                    = 41490, /*!< 41490 (Boolean FIX.5.0SP2) Indicates in physical settlement of bond and convertible bond options whether the party required to deliver the bonds will divide those to be delivered as notifying party desires to facilitate delivery obligations.*/
NoLegOptionExerciseBusinessCenters                 = 41491, /*!< 41491 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group. */
LegOptionExerciseBusinessCenter                    = 41492, /*!< 41492 (String FIX.5.0SP2) The business center calendar used to adjust the option exercise dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegOptionExerciseBusinessDayConvention             = 41493, /*!< 41493 (int FIX.5.0SP2) The business day convention used to adjust the option exercise dates. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegOptionExerciseEarliestDateOffsetDayType         = 41494, /*!< 41494 (int FIX.5.0SP2) Specifies the day type of the relative earliest exercise date offset.*/
LegOptionExerciseEarliestDateOffsetPeriod          = 41495, /*!< 41495 (int FIX.5.0SP2) Time unit multiplier for the relative earliest exercise date offset.*/
LegOptionExerciseEarliestDateOffsetUnit            = 41496, /*!< 41496 (String FIX.5.0SP2) Time unit associated with the relative earliest exercise date offset.*/
LegOptionExerciseFrequencyPeriod                   = 41497, /*!< 41497 (int FIX.5.0SP2) Time unit multiplier for the frequency of exercise dates. */
LegOptionExerciseFrequencyUnit                     = 41498, /*!< 41498 (String FIX.5.0SP2) Time unit associated with the frequency of exercise dates.*/
LegOptionExerciseStartDateUnadjusted               = 41499, /*!< 41499 (LocalMktDate FIX.5.0SP2) The unadjusted start date for calculating periodic exercise dates.*/
LegOptionExerciseStartDateRelativeTo               = 41500, /*!< 41500 (int FIX.5.0SP2) Specifies the anchor date when the option exercise start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegOptionExerciseStartDateOffsetPeriod             = 41501, /*!< 41501 (int FIX.5.0SP2) Time unit multiplier for the relative exercise start date offset. */
LegOptionExerciseStartDateOffsetUnit               = 41502, /*!< 41502 (String FIX.5.0SP2) Time unit associated with the relative exercise start date offset. */
LegOptionExerciseStartDateOffsetDayType            = 41503, /*!< 41503 (int FIX.5.0SP2) Specifies the day type of the relative option exercise start date offset.*/
LegOptionExerciseStartDateAdjusted                 = 41504, /*!< 41504 (LocalMktDate FIX.5.0SP2) The adjusted start date for calculating periodic exercise dates.*/
LegOptionExerciseSkip                              = 41505, /*!< 41505 (int FIX.5.0SP2) The number of periods in the referenced date schedule that are between each date in the relative date schedule. Thus a skip of 2 would mean that dates are relative to every second date in the referenced schedule. If present this should have a value greater than 1.*/
LegOptionExerciseNominationDeadline                = 41506, /*!< 41506 (LocalMktDate FIX.5.0SP2) The last date (adjusted) for establishing the option exercise terms.*/
LegOptionExerciseFirstDateUnadjusted               = 41507, /*!< 41507 (LocalMktDate FIX.5.0SP2) The unadjusted first exercise date.*/
LegOptionExerciseLastDateUnadjusted                = 41508, /*!< 41508 (LocalMktDate FIX.5.0SP2) The unadjusted last exercise date.*/
LegOptionExerciseEarliestTime                      = 41509, /*!< 41509 (LocalMktTime FIX.5.0SP2) The earliest time at which notice of exercise can be given by the buyer to the seller (or seller's agent) (i) on the expriation date, in the case of a European style option, (ii) on each Bermuda option exercise date and the expiration date, in the case of a Bermuda style option, (iii) the commencement date to, and including, the expiration date, in the case of an American option.*/
LegOptionExerciseLatestTime                        = 41510, /*!< 41510 (LocalMktTime FIX.5.0SP2) The latest exercise time.  See also LegOptionExerciseEarliestTime(41509).*/
LegOptionExerciseTimeBusinessCenter                = 41511, /*!< 41511 (String FIX.5.0SP2) The business center used to determine the locale for option exercise time, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegOptionExerciseDates                           = 41512, /*!< 41512 (NumInGroup FIX.5.0SP2) Number of dates in the repeating group.*/
LegOptionExerciseDate                              = 41513, /*!< 41513 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted option exercise fixed date.*/
LegOptionExerciseDateType                          = 41514, /*!< 41514 (int FIX.5.0SP2) Specifies the type of option exercise date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
NoLegOptionExerciseExpirationDateBusinessCenters   = 41515, /*!< 41515 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegOptionExerciseExpirationDateBusinessCenter      = 41516, /*!< 41516 (String FIX.5.0SP2) The business center calendar used to adjust the option exercise expiration dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegOptionExerciseExpirationDateBusinessDayConvention = 41517, /*!< 41517 (int FIX.5.0SP2) The business day convention used to adjust the option exercise expiration dates. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegOptionExerciseExpirationDateRelativeTo          = 41518, /*!< 41518 (int FIX.5.0SP2) Specifies the anchor date when the option exercise expiration date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegOptionExerciseExpirationDateOffsetPeriod        = 41519, /*!< 41519 (int FIX.5.0SP2) Time unit multiplier for the relative exercise expiration date offset.*/
LegOptionExerciseExpirationDateOffsetUnit          = 41520, /*!< 41520 (String FIX.5.0SP2) Time unit associated with the relative exercise expiration date offset.*/
LegOptionExerciseExpirationFrequencyPeriod         = 41521, /*!< 41521 (int FIX.5.0SP2) Time unit multiplier for the frequency of exercise expiration dates.*/
LegOptionExerciseExpirationFrequencyUnit           = 41522, /*!< 41522 (String FIX.5.0SP2) Time unit associated with the frequency of exercise expiration dates.*/
LegOptionExerciseExpirationRollConvention          = 41523, /*!< 41523 (String FIX.5.0SP2) The convention for determining the sequence of exercise expiration dates. It is used in conjunction with a specified frequency. Used only to override the roll convention defined in the LegDateAdjustment component in InstrumentLeg.*/
LegOptionExerciseExpirationDateOffsetDayType       = 41524, /*!< 41524 (int FIX.5.0SP2) Specifies the day type of the relative option exercise expiration date offset.*/
LegOptionExerciseExpirationTime                    = 41525, /*!< 41525 (LocalMktTime FIX.5.0SP2) The option exercise expiration time.*/
LegOptionExerciseExpirationTimeBusinessCenter      = 41526, /*!< 41526 (String FIX.5.0SP2) The business center used to determine the locale for option exercise expiration time, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegOptionExerciseExpirationDates                 = 41527, /*!< 41527 (NumInGroup FIX.5.0SP2) Number of fixed exercise expiration dates in the repeating group.*/
LegOptionExerciseExpirationDate                    = 41528, /*!< 41528 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted option exercise expiration fixed date.*/
LegOptionExerciseExpirationDateType                = 41529, /*!< 41529 (int FIX.5.0SP2) Specifies the type of option exercise expiration date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
NoLegPaymentScheduleFixingDays                     = 41530, /*!< 41530 (NumInGroup FIX.5.0SP2) Number of fixing days in the repeating group.*/
LegPaymentScheduleFixingDayOfWeek                  = 41531, /*!< 41531 (int FIX.5.0SP2) The day of the week on which fixing takes place. */
LegPaymentScheduleFixingDayNumber                  = 41532, /*!< 41532 (int FIX.5.0SP2) The occurrence of the day of week on which fixing takes place.*/
LegPaymentScheduleXID                              = 41533, /*!< 41533 (XID FIX.5.0SP2) Identifier of this LegPaymentSchedule for cross referencing elsewhere in the message.*/
LegPaymentScheduleXIDRef                           = 41534, /*!< 41534 (XIDREF FIX.5.0SP2) Reference to payment schedule elsewhere in the message.*/
LegPaymentScheduleRateCurrency                     = 41535, /*!< 41535 (Currency FIX.5.0SP2) The currency of the schedule rate.  Uses ISO 4217 currency codes.*/
LegPaymentScheduleRateUnitOfMeasure                = 41536, /*!< 41536 (String FIX.5.0SP2) The schedule rate unit of measure (UOM).*/
LegPaymentScheduleRateConversionFactor             = 41537, /*!< 41537 (float FIX.5.0SP2) The number multipled by the derived floating rate of the leg's payment schedule in order to arrive at the payment rate. If omitted, the schedule rate conversion factor is 1. */
LegPaymentScheduleRateSpreadType                   = 41538, /*!< 41538 (int FIX.5.0SP2) Identifies whether the rate spread is an absolute value to be added to the index rate or a percentage of the index rate.*/
LegPaymentScheduleSettlPeriodPrice                 = 41539, /*!< 41539 (Price FIX.5.0SP2) The schedule settlement period price.*/
LegPaymentScheduleSettlPeriodPriceCurrency         = 41540, /*!< 41540 (Currency FIX.5.0SP2) The currency of the schedule settlement period price. Uses ISO 4217 currency codes.*/
LegPaymentScheduleSettlPeriodPriceUnitOfMeasure    = 41541, /*!< 41541 (String FIX.5.0SP2) The settlement period price unit of measure (UOM).*/
LegPaymentScheduleStepUnitOfMeasure                = 41542, /*!< 41542 (String FIX.5.0SP2) The schedule step unit of measure (UOM).*/
LegPaymentScheduleFixingDayDistribution            = 41543, /*!< 41543 (int FIX.5.0SP2) The distribution of fixing days.*/
LegPaymentScheduleFixingDayCount                   = 41544, /*!< 41544 (int FIX.5.0SP2) The number of days over which fixing should take place. */
LegPaymentScheduleFixingLagPeriod                  = 41545, /*!< 41545 (int FIX.5.0SP2) Time unit multiplier for the fixing lag duration. */
LegPaymentScheduleFixingLagUnit                    = 41546, /*!< 41546 (String FIX.5.0SP2) Time unit associated with the fixing lag duration.*/
LegPaymentScheduleFixingFirstObservationDateOffsetPeriod = 41547, /*!< 41547 (int FIX.5.0SP2) Time unit multiplier for the relative first observation date offset.*/
LegPaymentScheduleFixingFirstObservationDateOffsetUnit = 41548, /*!< 41548 (String FIX.5.0SP2) Time unit associated with the relative first observation date offset.*/
LegPaymentStreamFlatRateIndicator                  = 41549, /*!< 41549 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', the Flat Rate is the New Worldwide Tanker Nominal Freight Scale for the Freight Index Route taken at the trade date of the transaction "Fixed". If 'N' it is taken on each pricing date "Floating".*/
LegPaymentStreamFlatRateAmount                     = 41550, /*!< 41550 (Amt FIX.5.0SP2) Specifies the actual monetary value of the flat rate when LegPaymentStreamFlatRateIndicator(41549) = 'Y'.*/
LegPaymentStreamFlatRateCurrency                   = 41551, /*!< 41551 (Currency FIX.5.0SP2) Specifies the currency of the actual flat rate.  Uses ISO 4217 currency codes.*/
LegStreamMaximumPaymentAmount                      = 41552, /*!< 41552 (Amt FIX.5.0SP2) Specifies the limit on the total payment amount.*/
LegStreamMaximumPaymentCurrency                    = 41553, /*!< 41553 (Currency FIX.5.0SP2) Specifies the currency of total payment amount limit.  Uses ISO 4217 currency codes.*/
LegStreamMaximumTransactionAmount                  = 41554, /*!< 41554 (Amt FIX.5.0SP2) Specifies the limit on the payment amount that goes out in any particular calculation period.*/
LegStreamMaximumTransactionCurrency                = 41555, /*!< 41555 (Currency FIX.5.0SP2) Specifies the currency of the period payment amount limit. Uses ISO 4217 currency codes.*/
LegPaymentStreamFixedAmountUnitOfMeasure           = 41556, /*!< 41556 (String FIX.5.0SP2) The fixed payment amount unit of measure (UOM).*/
LegPaymentStreamTotalFixedAmount                   = 41557, /*!< 41557 (Amt FIX.5.0SP2) Specifies the total fixed payment amount.*/
LegPaymentStreamWorldScaleRate                     = 41558, /*!< 41558 (float FIX.5.0SP2) The number of Worldscale points for purposes of the calculation of a fixed amount for a wet voyage charter commodity swap.*/
LegPaymentStreamContractPrice                      = 41559, /*!< 41559 (Price FIX.5.0SP2) The price per relevant unit for purposes of the calculation of a fixed amount for a dry voyage charter or time charter commodity swap.*/
LegPaymentStreamContractPriceCurrency              = 41560, /*!< 41560 (Currency FIX.5.0SP2) Specifies the currency of LegPaymentStreamContractPrice(41559). Uses ISO 4217 currency codes.*/
NoLegPaymentStreamPricingBusinessCenters           = 41561, /*!< 41561 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegPaymentStreamPricingBusinessCenter              = 41562, /*!< 41562 (String FIX.5.0SP2) The business center calendar used to adjust the pricing dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegPaymentStreamRateIndex2CurveUnit                = 41563, /*!< 41563 (String FIX.5.0SP2) Secondary time unit associated with the payment stream's floating rate index curve.  */
LegPaymentStreamRateIndex2CurvePeriod              = 41564, /*!< 41564 (int FIX.5.0SP2) Secondary time unit multiplier for the payment stream's floating rate index curve.  */
LegPaymentStreamRateIndexLocation                  = 41565, /*!< 41565 (String FIX.5.0SP2) Specifies the location of the floating rate index.*/
LegPaymentStreamRateIndexLevel                     = 41566, /*!< 41566 (Qty FIX.5.0SP2) This is the weather Cooling Degree Days (CDD), Heating Degree Days (HDD) or HDD index level specified as the number of (amount of) weather index units specified by the parties in the related confirmation.*/
LegPaymentStreamRateIndexUnitOfMeasure             = 41567, /*!< 41567 (String FIX.5.0SP2) The unit of measure (UOM) of the rate index level. */
LegPaymentStreamSettlLevel                         = 41568, /*!< 41568 (int FIX.5.0SP2) Specifies how weather index units are to be calculated.*/
LegPaymentStreamReferenceLevel                     = 41569, /*!< 41569 (Qty FIX.5.0SP2) This is the weather Cooling Degree Days (CDD), Heating Degree Days (HDD) or HDD reference level specified as the number of (amount of) weather index units specified by the parties in the related confirmation.*/
LegPaymentStreamReferenceLevelUnitOfMeasure        = 41570, /*!< 41570 (String FIX.5.0SP2) The unit of measure (UOM) of the rate reference level. */
LegPaymentStreamReferenceLevelEqualsZeroIndicator  = 41571, /*!< 41571 (Boolean FIX.5.0SP2) When set to 'Y', it indicates that the weather reference level equals zero.*/
LegPaymentStreamRateSpreadCurrency                 = 41572, /*!< 41572 (Currency FIX.5.0SP2) Specifies the currency of the floating rate spread. Uses ISO 4217 currency codes.*/
LegPaymentStreamRateSpreadUnitOfMeasure            = 41573, /*!< 41573 (String FIX.5.0SP2) Specifies the unit of measure (UOM) of the floating rate spread. */
LegPaymentStreamRateConversionFactor               = 41574, /*!< 41574 (float FIX.5.0SP2) The number to be multiplied by the derived floating rate of the leg's payment stream in order to arrive at the payment rate. If omitted, the floating rate conversion factor is 1. */
LegPaymentStreamRateSpreadType                     = 41575, /*!< 41575 (int FIX.5.0SP2) Identifies whether the rate spread is an absolute value to be added to the index rate or a percentage of the index rate.*/
LegPaymentStreamLastResetRate                      = 41576, /*!< 41576 (Percentage FIX.5.0SP2) The floating rate determined at the most recent reset. The rate is expressed in decimal form, e.g. 5% is represented as 0.05.*/
LegPaymentStreamFinalRate                          = 41577, /*!< 41577 (Percentage FIX.5.0SP2) The floating rate determined at the final reset. The rate is expressed in decimal form, e.g. 5% is represented as 0.05.*/
LegPaymentStreamCalculationLagPeriod               = 41578, /*!< 41578 (int FIX.5.0SP2) Time unit multiplier for the calculation lag duration.*/
LegPaymentStreamCalculationLagUnit                 = 41579, /*!< 41579 (String FIX.5.0SP2) Time unit associated with the calculation lag duration. */
LegPaymentStreamFirstObservationDateOffsetPeriod   = 41580, /*!< 41580 (int FIX.5.0SP2) Time unit multiplier for the relative first observation date offset.*/
LegPaymentStreamFirstObservationDateOffsetUnit     = 41581, /*!< 41581 (String FIX.5.0SP2) Time unit associated with the relative first observation date offset.*/
LegPaymentStreamPricingDayType                     = 41582, /*!< 41582 (int FIX.5.0SP2) Specifies the commodity pricing day type.*/
LegPaymentStreamPricingDayDistribution             = 41583, /*!< 41583 (int FIX.5.0SP2) The distribution of pricing days.*/
LegPaymentStreamPricingDayCount                    = 41584, /*!< 41584 (int FIX.5.0SP2) The number of days over which pricing should take place.*/
LegPaymentStreamPricingBusinessCalendar            = 41585, /*!< 41585 (String FIX.5.0SP2) Specifies the business calendar to use for pricing. 

See http://www.fpml.org/coding-scheme/commodity-business-calendar for values.*/
LegPaymentStreamPricingBusinessDayConvention       = 41586, /*!< 41586 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's pricing dates. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
UnderlyingDeliveryStreamRiskApportionmentSource    = 41587, /*!< 41587 (String FIX.5.0SP2) Specifies the source or legal framework for the risk apportionment.  

See http://www.fixtradingcommunity.org/codelists#Risk_Apportionment_Source for the details of the external code list.*/
StreamCommoditySettlTimeType                       = 41588, /*!< 41588 (int FIX.5.0SP2) Specifies the format of the commodities settlement start and end times.*/
NoLegPaymentStreamPaymentDates                     = 41589, /*!< 41589 (NumInGroup FIX.5.0SP2) Number of payment dates in the repeating group.*/
LegPaymentStreamPaymentDate                        = 41590, /*!< 41590 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted fixed stream payment date.*/
LegPaymentStreamPaymentDateType                    = 41591, /*!< 41591 (int FIX.5.0SP2) Specifies the type of payment date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
LegPaymentStreamMasterAgreementPaymentDatesIndicator = 41592, /*!< 41592 (Boolean FIX.5.0SP2) When set to 'Y', it indicates that payment dates are specified in the relevant master agreement.*/
NoLegPaymentStreamPricingDates                     = 41593, /*!< 41593 (NumInGroup FIX.5.0SP2) Number of pricing dates in the repeating group.*/
LegPaymentStreamPricingDate                        = 41594, /*!< 41594 (LocalMktDate FIX.5.0SP2) The adjusted or unadusted fixed stream pricing date.*/
LegPaymentStreamPricingDateType                    = 41595, /*!< 41595 (int FIX.5.0SP2) Specifies the type of pricing date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
NoLegPaymentStreamPricingDays                      = 41596, /*!< 41596 (NumInGroup FIX.5.0SP2) Number of pricing days in the repeating group.*/
LegPaymentStreamPricingDayOfWeek                   = 41597, /*!< 41597 (int FIX.5.0SP2) The day of the week on which pricing takes place.. */
LegPaymentStreamPricingDayNumber                   = 41598, /*!< 41598 (int FIX.5.0SP2) The occurrence of the day of week on which pricing takes place.*/
NoLegPhysicalSettlTerms                            = 41599, /*!< 41599 (NumInGroup FIX.5.0SP2) Number of entries in the repeating group.*/
LegPhysicalSettlTermXID                            = 41600, /*!< 41600 (XID FIX.5.0SP2) A named string value referenced by UnderlyingSettlTermXIDRef(41315).*/
LegPhysicalSettlCurency                            = 41601, /*!< 41601 (Currency FIX.5.0SP2) Specifies the currency of physical settlement.  Uses ISO 4217 currency codes.*/
LegPhysicalSettlBusinessDays                       = 41602, /*!< 41602 (int FIX.5.0SP2) The number of business days used in the determination of physical settlement. Its precise meaning is dependant on the context in which this is used.*/
LegPhysicalSettlMaximumBusinessDays                = 41603, /*!< 41603 (int FIX.5.0SP2) A maximum number of business days. Its precise meaning is dependant on the context in which this element is used. Intended to be used to limit a particular ISDA fallback provision.*/
NoLegPhysicalSettlDeliverableObligations           = 41604, /*!< 41604 (NumInGroup FIX.5.0SP2) Number of entries in the repeating group.*/
LegPhysicalSettlDeliverableObligationType          = 41605, /*!< 41605 (String FIX.5.0SP2) Specifies the type of delivery obligation applicable for physical settlement. 

See http://www.fixptradingcommunity.org/codelists#Deliverable_Obligation_Types for code list for applicable deliverable obligation types. */
LegPhysicalSettlDeliverableObligationValue         = 41606, /*!< 41606 (String FIX.5.0SP2) Physical settlement delivery obligation value appropriate to LegPhysicalSettlDeliverableObligationType(41605).

See http://www.fixtradingcommunity.org/codelists#Deliverable_Obligation_Types for code list for applicable deliverable obligation types.*/
NoLegPricingDateBusinessCenters                    = 41607, /*!< 41607 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegPricingDateBusinessCenter                       = 41608, /*!< 41608 (String FIX.5.0SP2) The business center calendar used to adjust the pricing or fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegPricingDateUnadjusted                           = 41609, /*!< 41609 (LocalMktDate FIX.5.0SP2) The unadjusted pricing or fixing date.*/
LegPricingDateBusinessDayConvention                = 41610, /*!< 41610 (int FIX.5.0SP2) The business day convention used to adjust the pricing or fixing date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegPricingDateAdjusted                             = 41611, /*!< 41611 (LocalMktDate FIX.5.0SP2) The adjusted pricing or fixing date.*/
LegPricingTime                                     = 41612, /*!< 41612 (LocalMktTime FIX.5.0SP2) The local market pricing or fixing time.*/
LegPricingTimeBusinessCenter                       = 41613, /*!< 41613 (String FIX.5.0SP2) Specifies the business center for determining the pricing or fixing time. See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegProtectionTermEventNewsSources                = 41614, /*!< 41614 (NumInGroup FIX.5.0SP2) Number of event sources in the repeating group.*/
LegProtectionTermEventNewsSource                   = 41615, /*!< 41615 (String FIX.5.0SP2) A newspaper or electronic news service that may publish relevant information used in the determination of whether or not a credit event has occurred.*/
NoLegProtectionTerms                               = 41616, /*!< 41616 (NumInGroup FIX.5.0SP2) Number of protection terms in the repeating group.*/
LegProtectionTermXID                               = 41617, /*!< 41617 (XID FIX.5.0SP2) A named string value referenced from UnderlyingLegProtectionTermXIDRef(41314).*/
LegProtectionTermNotional                          = 41618, /*!< 41618 (Amt FIX.5.0SP2) The notional amount of protection coverage. */
LegProtectionTermCurrency                          = 41619, /*!< 41619 (Currency FIX.5.0SP2) The currency of LegProtectionTermNotional(41618). Uses ISO 4217 currency codes.*/
LegProtectionTermSellerNotifies                    = 41620, /*!< 41620 (Boolean FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring. LegProtectionTermSellerNotifies(41620)=Y indicates that the seller notifies.*/
LegProtectionTermBuyerNotifies                     = 41621, /*!< 41621 (Boolean FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring. LegProtectionTermBuyerNotifies(41621)=Y indicates that the buyer notifies.*/
LegProtectionTermEventBusinessCenter               = 41622, /*!< 41622 (String FIX.5.0SP2) When used, the business center indicates the local time of the business center that replaces the Greenwich Mean Time in Section 3.3 of the 2003 ISDA Credit Derivatives Definitions. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProtectionTermStandardSources                   = 41623, /*!< 41623 (Boolean FIX.5.0SP2) Indicates whether ISDA defined Standard Public Sources are applicable (LegProtectionTermStandardSources(41623)=Y) or not.*/
LegProtectionTermEventMinimumSources               = 41624, /*!< 41624 (int FIX.5.0SP2) The minimum number of the specified public information sources that must publish information that reasonably confirms that a credit event has occurred. The market convention is two. */
NoLegProtectionTermEvents                          = 41625, /*!< 41625 (NumInGroup FIX.5.0SP2) Number of protection term events in the repeating group.*/
LegProtectionTermEventType                         = 41626, /*!< 41626 (String FIX.5.0SP2) Specifies the type of credit event applicable to the protection terms. 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Event_Types for code list of applicable event types.*/
LegProtectionTermEventValue                        = 41627, /*!< 41627 (String FIX.5.0SP2) Specifies the protection term event value appropriate to LegProtectionTermEventType(41626).  See http:///www.fixtradingcommunity.org/codelists#Protection_Term_Event_Types for applicable event type values.*/
LegProtectionTermEventCurrency                     = 41628, /*!< 41628 (Currency FIX.5.0SP2) Applicable currency if the event value is an amount. Uses ISO 4217 currency codes.*/
LegProtectionTermEventPeriod                       = 41629, /*!< 41629 (int FIX.5.0SP2) Time unit multiplier for protection term events.  */
LegProtectionTermEventUnit                         = 41630, /*!< 41630 (String FIX.5.0SP2) Time unit associated with protection term events.*/
LegProtectionTermEventDayType                      = 41631, /*!< 41631 (int FIX.5.0SP2) Specifies the day type for protection term events.*/
LegProtectionTermEventRateSource                   = 41632, /*!< 41632 (String FIX.5.0SP2) Rate source for events that specify a rate source, e.g. floating rate interest shortfall.*/
NoLegProtectionTermEventQualifiers                 = 41633, /*!< 41633 (NumInGroup FIX.5.0SP2) Number of qualifiers in the repeating group.*/
LegProtectionTermEventQualifier                    = 41634, /*!< 41634 (char FIX.5.0SP2) Specifies the protection term event qualifier. Used to further qualify LegProtectionTermEventType(41626).*/
NoLegProtectionTermObligations                     = 41635, /*!< 41635 (NumInGroup FIX.5.0SP2) Number of obligations in the repeating group.*/
LegProtectionTermObligationType                    = 41636, /*!< 41636 (String FIX.5.0SP2) Specifies the type of obligation applicable to the protection terms. 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Obligation_Types for code list of applicable obligation types.*/
LegProtectionTermObligationValue                   = 41637, /*!< 41637 (String FIX.5.0SP2) The value associated with the protection term obligation specified in LegProtectionTermObligationType(41636). See http://www.fixtradingcommunity.org/codelists#Protection_Term_Obligation_Types for applicable obligation type values.*/
NoLegStreamCalculationPeriodDates                  = 41638, /*!< 41638 (NumInGroup FIX.5.0SP2) Number of calculation period dates in the repeating group.*/
LegStreamCalculationPeriodDate                     = 41639, /*!< 41639 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted fixed calculation period date.*/
LegStreamCalculationPeriodDateType                 = 41640, /*!< 41640 (int FIX.5.0SP2) Specifies the type of fixed calculation period date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
LegStreamCalculationPeriodDatesXID                 = 41641, /*!< 41641 (XID FIX.5.0SP2) Identifier of this calculation period for cross referencing elsewhere in the message.*/
LegStreamCalculationPeriodDatesXIDRef              = 41642, /*!< 41642 (XIDREF FIX.5.0SP2) Cross reference to another calculation period for duplicating its properties.*/
LegStreamCalculationBalanceOfFirstPeriod           = 41643, /*!< 41643 (Boolean FIX.5.0SP2) When specified and set to 'Y', it indicates that the first calculation period should run from the effective date to the end of the calendar period in which the effective date falls (e.g. Jan 15 - Jan 31 if the calculation periods are one month long and effective date is Jan 15.). If 'N' or not specified, it indicates that the first calculation period should run from the effective date for one whole period (e.g. Jan 15 to Feb 14 if the calculation periods are one month long and the effective date is Jan 15.).*/
LegStreamCalculationCorrectionPeriod               = 41644, /*!< 41644 (int FIX.5.0SP2) Time unit multiplier for the length of time after the publication of the data when corrections can be made.*/
LegStreamCalculationCorrectionUnit                 = 41645, /*!< 41645 (String FIX.5.0SP2) Time unit associated with the length of time after the publication of the data when corrections can be made.*/
NoLegStreamCommoditySettlBusinessCenters           = 41646, /*!< 41646 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegStreamCommoditySettlBusinessCenter              = 41647, /*!< 41647 (String FIX.5.0SP2) The business center calendar used to adjust the commodity delivery date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegStreamCommodityBase                             = 41648, /*!< 41648 (String FIX.5.0SP2) Specifies the general base type of the commodity traded. Where possible, this should follow the naming convention used in the 2005 ISDA Commodity Definitions. */
LegStreamCommodityType                             = 41649, /*!< 41649 (String FIX.5.0SP2) Specifies the type of commodity product.

For coal see http://www.fpml.org/coding-scheme/commodity-coal-product-type for values. 

For metals see http://www.fpml.org/coding-scheme/commodity-metal-product-type for values. 

For bullion see http://www.fixtradingcommunity.org/codelists#Bullion_Types for the external code list of bullion types.*/
LegStreamCommoditySecurityID                       = 41650, /*!< 41650 (String FIX.5.0SP2) Specifies the market identifier for the commodity.*/
LegStreamCommoditySecurityIDSource                 = 41651, /*!< 41651 (String FIX.5.0SP2) Identifies the class or source of the LegStreamCommoditySecurityIDSource(41650) value.*/
LegStreamCommodityDesc                             = 41652, /*!< 41652 (String FIX.5.0SP2) Description of the commodity asset.*/
EncodedLegStreamCommodityDescLen                   = 41653, /*!< 41653 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedLegStreamCommodityDesc(41654) field.*/
EncodedLegStreamCommodityDesc                      = 41654, /*!< 41654 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the LegStreamCommodityDesc(41652) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the LegStreamCommodityDesc(41652) field.*/
LegStreamCommodityUnitOfMeasure                    = 41655, /*!< 41655 (String FIX.5.0SP2) The unit of measure (UOM) of the commodity asset.*/
LegStreamCommodityCurrency                         = 41656, /*!< 41656 (Currency FIX.5.0SP2) Identifies the currency of the commodity asset. Uses ISO 4217 currency codes.*/
LegStreamCommodityExchange                         = 41657, /*!< 41657 (Exchange FIX.5.0SP2) Identifies the exchange where the commodity is traded.*/
LegStreamCommodityRateSource                       = 41658, /*!< 41658 (int FIX.5.0SP2) Identifies the source of rate information used for commodities. 

See http://www.fixtradingcommunity.org/codelists#Commodity_Rate_Source for code list of applicable sources.*/
LegStreamCommodityRateReferencePage                = 41659, /*!< 41659 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.*/
LegStreamCommodityRateReferencePageHeading         = 41660, /*!< 41660 (String FIX.5.0SP2) Identifies the page heading from the rate source.*/
LegStreamDataProvider                              = 41661, /*!< 41661 (String FIX.5.0SP2) Specifies the commodity data or information provider. 

See http://www.fpml.org/coding-scheme/commodity-information-provider for values.*/
LegStreamCommodityPricingType                      = 41662, /*!< 41662 (String FIX.5.0SP2) Specifies how the pricing or rate setting of the trade is to be determined or based upon.  

See http://www.fixtradingcommunity.org/codelists#Commodity_Rate_Pricing_Type for code list of applicable commodity pricing types.*/
LegStreamCommodityNearbySettlDayPeriod             = 41663, /*!< 41663 (int FIX.5.0SP2) Time unit multiplier for the nearby settlement day.  */
LegStreamCommodityNearbySettlDayUnit               = 41664, /*!< 41664 (String FIX.5.0SP2) Time unit associated with the nearby settlement day.  */
LegStreamCommoditySettlDateUnadjusted              = 41665, /*!< 41665 (LocalMktDate FIX.5.0SP2) The unadjusted commodity delivery date.*/
LegStreamCommoditySettlDateBusinessDayConvention   = 41666, /*!< 41666 (int FIX.5.0SP2) The business day convention used to adjust the commodity delivery date. Used only to override the business day convention specified in the LegDateAdjustment component within the InstrumentLeg component.*/
LegStreamCommoditySettlDateAdjusted                = 41667, /*!< 41667 (LocalMktDate FIX.5.0SP2) The adjusted commodity delivery date.*/
LegStreamCommoditySettlMonth                       = 41668, /*!< 41668 (int FIX.5.0SP2) Specifies a fixed single month for commodity delivery. */
LegStreamCommoditySettlDateRollPeriod              = 41669, /*!< 41669 (int FIX.5.0SP2) Time unit multiplier for the commodity delivery date roll.*/
LegStreamCommoditySettlDateRollUnit                = 41670, /*!< 41670 (String FIX.5.0SP2) Time unit associated with the commodity delivery date roll.*/
LegStreamCommoditySettlDayType                     = 41671, /*!< 41671 (int FIX.5.0SP2) Specifies the commodity delivery roll day type.*/
LegStreamCommodityXID                              = 41672, /*!< 41672 (XID FIX.5.0SP2) Identifier of this stream commodity for cross referencing elsewhere in the message.*/
LegStreamCommodityXIDRef                           = 41673, /*!< 41673 (XIDREF FIX.5.0SP2) Reference to a stream commodity elsewhere in the message.*/
NoLegStreamCommodityAltIDs                         = 41674, /*!< 41674 (NumInGroup FIX.5.0SP2) Number of alternate security identifers.*/
LegStreamCommodityAltID                            = 41675, /*!< 41675 (String FIX.5.0SP2) Alternate security identifier value for the commodity.*/
LegStreamCommodityAltIDSource                      = 41676, /*!< 41676 (String FIX.5.0SP2) Identifies the class or source of the alternate commodity security identifier. */
NoLegStreamCommodityDataSources                    = 41677, /*!< 41677 (NumInGroup FIX.5.0SP2) Number of data sources in the repeating group. The order of entry determines priority – first is the main source, second is fallback, third is second fallback.*/
LegStreamCommodityDataSourceID                     = 41678, /*!< 41678 (String FIX.5.0SP2) Specifies the data source identifier.*/
LegStreamCommodityDataSourceIDType                 = 41679, /*!< 41679 (int FIX.5.0SP2) Specifies the type of data source identifier.*/
NoLegStreamCommoditySettlDays                      = 41680, /*!< 41680 (NumInGroup FIX.5.0SP2) Number of days in the repeating group.*/
LegStreamCommoditySettlDay                         = 41681, /*!< 41681 (int FIX.5.0SP2) Specifies the day or group of days for delivery.*/
LegStreamCommoditySettlTotalHours                  = 41682, /*!< 41682 (int FIX.5.0SP2) Sum of the hours specified in LegStreamCommoditySettlTimeGrp.*/
NoLegStreamCommoditySettlTimes                     = 41683, /*!< 41683 (NumInGroup FIX.5.0SP2) Number of hour ranges in the repeating group.*/
LegStreamCommoditySettlStart                       = 41684, /*!< 41684 (String FIX.5.0SP2) The start time for commodity settlement where delivery occurs over time. The time format is specified by the settlement time type.*/
LegStreamCommoditySettlEnd                         = 41685, /*!< 41685 (String FIX.5.0SP2) The end time for commodity settlement where delivery occurs over time. The time format is specified by the settlement time type.*/
NoLegStreamCommoditySettlPeriods                   = 41686, /*!< 41686 (NumInGroup FIX.5.0SP2) Number of commodity settlement periods in the repeating group.*/
LegStreamCommoditySettlCountry                     = 41687, /*!< 41687 (Country FIX.5.0SP2) Specifies the country where delivery takes place. Uses ISO 3166 2-character country code.*/
LegStreamCommoditySettlTimeZone                    = 41688, /*!< 41688 (String FIX.5.0SP2) Commodity delivery timezone specified as "prevailing" rather than "standard" or "daylight".  

See http://www.fixtradingcommunity.org/codelists#Prevailing_Timezones for code list of applicable prevailing timezones.*/
LegStreamCommoditySettlFlowType                    = 41689, /*!< 41689 (int FIX.5.0SP2) Specifies the commodity delivery flow type.*/
LegStreamCommoditySettlPeriodNotional              = 41690, /*!< 41690 (Qty FIX.5.0SP2) Delivery quantity associated with this settlement period.*/
LegStreamCommoditySettlPeriodNotionalUnitOfMeasure = 41691, /*!< 41691 (String FIX.5.0SP2) Specifies the unit of measure (UOM) of the delivery quantity associated with this settlement period.*/
LegStreamCommoditySettlPeriodFrequencyPeriod       = 41692, /*!< 41692 (int FIX.5.0SP2) Time unit multiplier for the settlement period frequency. */
LegStreamCommoditySettlPeriodFrequencyUnit         = 41693, /*!< 41693 (String FIX.5.0SP2) Time unit associated with the settlement period frequency.*/
LegStreamCommoditySettlPeriodPrice                 = 41694, /*!< 41694 (Price FIX.5.0SP2) The settlement period price.*/
LegStreamCommoditySettlPeriodPriceUnitOfMeasure    = 41695, /*!< 41695 (String FIX.5.0SP2) The settlement period price unit of measure (UOM).*/
LegStreamCommoditySettlPeriodPriceCurrency         = 41696, /*!< 41696 (Currency FIX.5.0SP2) The currency of the settlement period price. Uses ISO 4217 currency codes.*/
LegStreamCommoditySettlHolidaysProcessingInstruction = 41697, /*!< 41697 (int FIX.5.0SP2) Indicates whether holidays are included in the settlement periods. Required for electricity contracts.*/
LegStreamCommoditySettlPeriodXID                   = 41698, /*!< 41698 (XID FIX.5.0SP2) Identifier of this settlement period for cross referencing elsewhere in the message.*/
LegStreamCommoditySettlPeriodXIDRef                = 41699, /*!< 41699 (XIDREF FIX.5.0SP2) Cross reference to another settlement period for duplicating its properties.*/
LegStreamXID                                       = 41700, /*!< 41700 (XID FIX.5.0SP2) Identifier of this LegStream for cross referencing elsewhere in the message.*/
UnderlyingAdditionalTermBondSecurityIDSource       = 41701, /*!< 41701 (String FIX.5.0SP2) Identifies the source scheme of the UnderlyingAdditionalTermBondSecurityID(41341) value. */
LegStreamNotionalXIDRef                            = 41702, /*!< 41702 (XIDREF FIX.5.0SP2) Cross reference to another LegStream notional for duplicating its properties.*/
LegStreamNotionalFrequencyPeriod                   = 41703, /*!< 41703 (int FIX.5.0SP2) Time unit multiplier for the swap stream's notional frequency.*/
LegStreamNotionalFrequencyUnit                     = 41704, /*!< 41704 (String FIX.5.0SP2) Time unit associated with the swap stream's notional frequency. */
LegStreamNotionalCommodityFrequency                = 41705, /*!< 41705 (int FIX.5.0SP2) The commodity's notional or quantity delivery frequency.*/
LegStreamNotionalUnitOfMeasure                     = 41706, /*!< 41706 (String FIX.5.0SP2) Specifies the delivery quantity unit of measure (UOM).*/
LegStreamTotalNotional                             = 41707, /*!< 41707 (Qty FIX.5.0SP2) Specifies the total notional or delivery quantity over the term of the contract.*/
LegStreamTotalNotionalUnitOfMeasure                = 41708, /*!< 41708 (String FIX.5.0SP2) Specifies the unit of measure (UOM) for the total notional or delivery quantity over the term of the contract.*/
UnderlyingAdditionalTermBondDesc                   = 41709, /*!< 41709 (String FIX.5.0SP2) Description of the bond.*/
EncodedUnderlyingAdditionalTermBondDescLen         = 41710, /*!< 41710 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingAdditionalTermBondDesc(41711) field.*/
EncodedUnderlyingAdditionalTermBondDesc            = 41711, /*!< 41711 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingAdditionalTermBondDesc(41709) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingAdditionalTermBondDesc(41709) field.*/
UnderlyingAdditionalTermBondCurrency               = 41712, /*!< 41712 (Currency FIX.5.0SP2) Specifies the currency the bond value is denominated in.  Uses ISO 4217 currency codes.*/
NoUnderlyingComplexEventAveragingObservations      = 41713, /*!< 41713 (NumInGroup FIX.5.0SP2) The number of averaging observations in the repeating group.*/
UnderlyingComplexEventAveragingObservationNumber   = 41714, /*!< 41714 (int FIX.5.0SP2) Cross reference to the ordinal observation as specified either in the UnderlyingComplexEventScheduleGrp or UnderlyingComplexEventPeriodDateGrp components.*/
UnderlyingComplexEventAveragingWeight              = 41715, /*!< 41715 (float FIX.5.0SP2) The weight factor to be applied to the observation.*/
NoUnderlyingComplexEventCreditEvents               = 41716, /*!< 41716 (NumInGroup FIX.5.0SP2) The number of credit events specified in the repeating group.*/
UnderlyingComplexEventCreditEventType              = 41717, /*!< 41717 (String FIX.5.0SP2) Specifies the type of credit event. 

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Types for code list of applicable event types.*/
UnderlyingComplexEventCreditEventValue             = 41718, /*!< 41718 (String FIX.5.0SP2) The credit event value appropriate to UnderlyingComplexEventCreditEventType(41717). 

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Types for applicable event type values. */
UnderlyingComplexEventCreditEventCurrency          = 41719, /*!< 41719 (Currency FIX.5.0SP2) Specifies the applicable currency when UnderlyingComplexEventCreditEventValue(41718) is an amount. Uses ISO 4217 currency codes. */
UnderlyingComplexEventCreditEventPeriod            = 41720, /*!< 41720 (int FIX.5.0SP2) Time unit multiplier for complex credit events.*/
UnderlyingComplexEventCreditEventUnit              = 41721, /*!< 41721 (String FIX.5.0SP2) Time unit associated with complex credit events.*/
UnderlyingComplexEventCreditEventDayType           = 41722, /*!< 41722 (int FIX.5.0SP2) Specifies the day type for the complex credit events. */
UnderlyingComplexEventCreditEventRateSource        = 41723, /*!< 41723 (int FIX.5.0SP2) Identifies the source of rate information used for credit events. 

See http://www.fixtradingcommunity.org/codelists#Credit_Event_Rate_Source for code list of applicable sources.*/
NoUnderlyingComplexEventCreditEventQualifiers      = 41724, /*!< 41724 (NumInGroup FIX.5.0SP2) Number of qualifiers in the repeating group.*/
UnderlyingComplexEventCreditEventQualifier         = 41725, /*!< 41725 (char FIX.5.0SP2) Specifies a complex event qualifier. Used to further qualify UnderlyingComplexEventCreditEventType(41717). */
NoUnderlyingComplexEventPeriodDateTimes            = 41726, /*!< 41726 (NumInGroup FIX.5.0SP2) Number of entries in the date-time repeating group.*/
UnderlyingComplexEventPeriodDate                   = 41727, /*!< 41727 (LocalMktDate FIX.5.0SP2) The averaging date for an Asian option. 

The trigger date for a Barrier or Knock option.*/
UnderlyingComplexEventPeriodTime                   = 41728, /*!< 41728 (LocalMktTime FIX.5.0SP2) The averaging time for an Asian option.*/
NoUnderlyingComplexEventPeriods                    = 41729, /*!< 41729 (NumInGroup FIX.5.0SP2) Number of periods in the repeating group.*/
UnderlyingComplexEventPeriodType                   = 41730, /*!< 41730 (int FIX.5.0SP2) Specifies the period type.*/
UnderlyingComplexEventBusinessCenter               = 41731, /*!< 41731 (String FIX.5.0SP2) The business center for adjusting dates and times in the schedule or date-time group.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingComplexEventRateSources                = 41732, /*!< 41732 (NumInGroup FIX.5.0SP2) Number of rate sources in the repeating group.*/
UnderlyingComplexEventRateSource                   = 41733, /*!< 41733 (int FIX.5.0SP2) Identifies the source of rate information.

*/
UnderlyingComplexEventRateSourceType               = 41734, /*!< 41734 (int FIX.5.0SP2) Indicates whether the rate source specified is a primary or secondary source.*/
UnderlyingComplexEventReferencePage                = 41735, /*!< 41735 (String FIX.5.0SP2) Identifies the reference page from the rate source.

For FX, the reference page to the spot rate is to be used for the reference FX spot rate.

When UnderlyingComplexEventRateSource(41733) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. See: http://www.fpml.org/coding-scheme/settlement-rate-option.*/
UnderlyingComplexEventReferencePageHeading         = 41736, /*!< 41736 (String FIX.5.0SP2) Identifies the reference page heading from the rate source.*/
NoUnderlyingComplexEventDateBusinessCenters        = 41737, /*!< 41737 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingComplexEventDateBusinessCenter           = 41738, /*!< 41738 (String FIX.5.0SP2) The business center calendar is used to adjust the event date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingComplexEventDateUnadjusted               = 41739, /*!< 41739 (LocalMktDate FIX.5.0SP2) The unadjusted complex event date.*/
UnderlyingComplexEventDateRelativeTo               = 41740, /*!< 41740 (int FIX.5.0SP2) Specifies the anchor date when the complex event date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingComplexEventDateOffsetPeriod             = 41741, /*!< 41741 (int FIX.5.0SP2) Time unit multiplier for the relative date offset.*/
UnderlyingComplexEventDateOffsetUnit               = 41742, /*!< 41742 (String FIX.5.0SP2) Time unit associated with the relative date offset.*/
UnderlyingComplexEventDateOffsetDayType            = 41743, /*!< 41743 (int FIX.5.0SP2) Specifies the day type of the relative date offset.*/
UnderlyingComplexEventDateBusinessDayConvention    = 41744, /*!< 41744 (int FIX.5.0SP2) The business day convention used to adjust the event date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingComplexEventDateAdjusted                 = 41745, /*!< 41745 (LocalMktDate FIX.5.0SP2) The adjusted complex event date.*/
UnderlyingComplexEventFixingTime                   = 41746, /*!< 41746 (LocalMktTime FIX.5.0SP2) The local market fixing time.*/
UnderlyingComplexEventFixingTimeBusinessCenter     = 41747, /*!< 41747 (String FIX.5.0SP2) The business center for determining the actual fixing times.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingComplexEventCreditEventSources         = 41748, /*!< 41748 (NumInGroup FIX.5.0SP2) Number of event sources in the repeating group.*/
UnderlyingComplexEventCreditEventSource            = 41749, /*!< 41749 (String FIX.5.0SP2) A newspaper or electronic news service that may publish relevant information used in the determination of whether or not a credit event has occurred.*/
NoUnderlyingComplexEventSchedules                  = 41750, /*!< 41750 (NumInGroup FIX.5.0SP2) Number of schedules in the repeating group.*/
UnderlyingComplexEventScheduleStartDate            = 41751, /*!< 41751 (LocalMktDate FIX.5.0SP2) The start date of the schedule.*/
UnderlyingComplexEventScheduleEndDate              = 41752, /*!< 41752 (LocalMktDate FIX.5.0SP2) The end date of the schedule.*/
UnderlyingComplexEventScheduleFrequencyPeriod      = 41753, /*!< 41753 (int FIX.5.0SP2) Time unit multiplier for the schedule date frequency.*/
UnderlyingComplexEventScheduleFrequencyUnit        = 41754, /*!< 41754 (String FIX.5.0SP2) Time unit associated with the schedule date frequency.*/
UnderlyingComplexEventScheduleRollConvention       = 41755, /*!< 41755 (String FIX.5.0SP2) The convention for determining the sequence of dates. It is used in conjunction with a specified frequency. Used only to override the roll convention defined in the UnderlyingDateAdjustment component in UnderlyingInstrument.*/
NoUnderlyingDeliverySchedules                      = 41756, /*!< 41756 (NumInGroup FIX.5.0SP2) Number of delivery schedules in the repeating group.*/
UnderlyingDeliveryScheduleType                     = 41757, /*!< 41757 (int FIX.5.0SP2) Specifies the type of delivery schedule.*/
UnderlyingDeliveryScheduleXID                      = 41758, /*!< 41758 (XID FIX.5.0SP2) Identifier for this instance of delivery schedule for cross referencing elsewhere in the message.*/
UnderlyingDeliveryScheduleNotional                 = 41759, /*!< 41759 (Qty FIX.5.0SP2) Physical delivery quantity.*/
UnderlyingDeliveryScheduleNotionalUnitOfMeasure    = 41760, /*!< 41760 (String FIX.5.0SP2) Specifies the delivery quantity unit of measure (UOM).*/
UnderlyingDeliveryScheduleNotionalCommodityFrequency = 41761, /*!< 41761 (int FIX.5.0SP2) The frequency of notional delivery.*/
UnderlyingDeliveryScheduleNegativeTolerance        = 41762, /*!< 41762 (float FIX.5.0SP2) Specifies the negative tolerance value.  The value may be an absolute quantity or a percentage, as specified in UnderlyingDeliveryScheduleToleranceType(41765).  Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
UnderlyingDeliverySchedulePositiveTolerance        = 41763, /*!< 41763 (float FIX.5.0SP2) Specifies the positive tolerance value. The value may be an absolute quantity or a percentage, as specified in UnderlyingDeliveryScheduleToleranceType(41765). Value may exceed agreed upon value. Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
UnderlyingDeliveryScheduleToleranceUnitOfMeasure   = 41764, /*!< 41764 (String FIX.5.0SP2) Specifies the tolerance value's unit of measure (UOM).*/
UnderlyingDeliveryScheduleToleranceType            = 41765, /*!< 41765 (int FIX.5.0SP2) Specifies the tolerance value type.*/
UnderlyingDeliveryScheduleSettlCountry             = 41766, /*!< 41766 (Country FIX.5.0SP2) Specifies the country where delivery takes place. Uses ISO 3166 2-character country code.*/
UnderlyingDeliveryScheduleSettlTimeZone            = 41767, /*!< 41767 (String FIX.5.0SP2) Delivery timezone specified as "prevailing" rather than "standard" or "daylight".  

See http://www.fixtradingcommunity.org/codelists#Prevailing_Timezones for code list of applicable prevailing timezones.*/
UnderlyingDeliveryScheduleSettlFlowType            = 41768, /*!< 41768 (int FIX.5.0SP2) Specifies the delivery flow type.*/
UnderlyingDeliveryScheduleSettlHolidaysProcessingInstruction = 41769, /*!< 41769 (int FIX.5.0SP2) Indicates whether holidays are included in the settlement periods. Required for electricity contracts.*/
NoUnderlyingDeliveryScheduleSettlDays              = 41770, /*!< 41770 (NumInGroup FIX.5.0SP2) Number of delivery schedules in the repeating group.*/
UnderlyingDeliveryScheduleSettlDay                 = 41771, /*!< 41771 (int FIX.5.0SP2) Specifies the day or group of days for delivery.*/
UnderlyingDeliveryScheduleSettlTotalHours          = 41772, /*!< 41772 (int FIX.5.0SP2) The sum of the total hours specified in the UnderlyingDeliveryScheduleSettlTimeGrp component.*/
NoUnderlyingDeliveryScheduleSettlTimes             = 41773, /*!< 41773 (NumInGroup FIX.5.0SP2) Number of hour ranges in the repeating group.*/
UnderlyingDeliveryScheduleSettlStart               = 41774, /*!< 41774 (String FIX.5.0SP2) The scheduled start time for the delivery of the commodity where delivery occurs over specified times. The format of the time value is specified in UnderlyingDeliveryScheduleSettlTimeType(41776).*/
UnderlyingDeliveryScheduleSettlEnd                 = 41775, /*!< 41775 (String FIX.5.0SP2) The scheduled end time for the delivery of the commodity where delivery occurs over specified times. The format of the time value is specified in UnderlyingDeliveryScheduleSettlTimeType(41776).*/
UnderlyingDeliveryScheduleSettlTimeType            = 41776, /*!< 41776 (int FIX.5.0SP2) Specifies the format of the delivery start and end time values.*/
UnderlyingDeliveryStreamType                       = 41777, /*!< 41777 (int FIX.5.0SP2) Specifies the type of delivery stream.*/
UnderlyingDeliveryStreamPipeline                   = 41778, /*!< 41778 (String FIX.5.0SP2) The name of the oil delivery pipeline.*/
UnderlyingDeliveryStreamEntryPoint                 = 41779, /*!< 41779 (String FIX.5.0SP2) The point at which the commodity will enter the delivery mechanism or pipeline.*/
UnderlyingDeliveryStreamWithdrawalPoint            = 41780, /*!< 41780 (String FIX.5.0SP2) The point at which the commodity product will be withdrawn prior to delivery.*/
UnderlyingDeliveryStreamDeliveryPoint              = 41781, /*!< 41781 (String FIX.5.0SP2) The point at which the commodity product will be delivered and received. Value specified should follow market convention appropriate for the commodity product.

For bullion see http://www.fpml.org/coding-scheme/bullion-delivery-location for values.*/
UnderlyingDeliveryStreamDeliveryRestriction        = 41782, /*!< 41782 (int FIX.5.0SP2) Specifies under what conditions the buyer and seller should be excused of their delivery obligations.*/
UnderlyingDeliveryStreamDeliveryContingency        = 41783, /*!< 41783 (String FIX.5.0SP2) Specifies the electricity delivery contingency. 

See http://www.fpml.org/coding-scheme/electricity-transmission-contingency for values.*/
UnderlyingDeliveryStreamDeliveryContingentPartySide = 41784, /*!< 41784 (int FIX.5.0SP2) The trade side value of the party responsible for electricity delivery contingency.*/
UnderlyingDeliveryStreamDeliverAtSourceIndicator   = 41785, /*!< 41785 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', delivery of the coal product is to be at its source.*/
UnderlyingDeliveryStreamRiskApportionment          = 41786, /*!< 41786 (String FIX.5.0SP2) Specifies how the parties to the trade apportion responsibility for the delivery of the commodity product. 

See http://www.fixtradingcommunity.org/codelists#Risk_Apportionment for the details of the external code list.*/
UnderlyingDeliveryStreamTitleTransferLocation      = 41787, /*!< 41787 (String FIX.5.0SP2) Specifies the title transfer location.*/
UnderlyingDeliveryStreamTitleTransferCondition     = 41788, /*!< 41788 (int FIX.5.0SP2) Specifies the title transfer condition.*/
UnderlyingDeliveryStreamImporterOfRecord           = 41789, /*!< 41789 (String FIX.5.0SP2) A party, not necessarily of the trade, who is the Importer of Record for the purposes of paying customs duties and applicable taxes or costs related to importation.*/
UnderlyingDeliveryStreamNegativeTolerance          = 41790, /*!< 41790 (float FIX.5.0SP2) Specifies the negative tolerance value.  The value may be an absolute quantity or a percentage, as specified in UnderlyingDeliveryStreamToleranceType(41793).  Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
UnderlyingDeliveryStreamPositiveTolerance          = 41791, /*!< 41791 (float FIX.5.0SP2) Specifies the positive tolerance value. The value may be an absolute quantity or a percentage, as specified in UnderlyingDeliveryStreamToleranceType(41793). Value may exceed agreed upon value. Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%).*/
UnderlyingDeliveryStreamToleranceUnitOfMeasure     = 41792, /*!< 41792 (String FIX.5.0SP2) Specifies the tolerance value's unit of measure (UOM).*/
UnderlyingDeliveryStreamToleranceType              = 41793, /*!< 41793 (int FIX.5.0SP2) Specifies the tolerance value type.*/
UnderlyingDeliveryStreamToleranceOptionSide        = 41794, /*!< 41794 (int FIX.5.0SP2) Indicates whether the tolerance is at the seller's or buyer's option.*/
UnderlyingDeliveryStreamTotalPositiveTolerance     = 41795, /*!< 41795 (Percentage FIX.5.0SP2) The positive percent tolerance which applies to the total quantity delivered over all shipment periods.

Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%.).*/
UnderlyingDeliveryStreamTotalNegativeTolerance     = 41796, /*!< 41796 (Percentage FIX.5.0SP2) The negative percent tolerance which applies to the total quantity delivered over all shipment periods.

Percentage value is to be expressed relative to "1.0" representing 100% (e.g. a value of "0.0575" represents 5.75%.).*/
UnderlyingDeliveryStreamNotionalConversionFactor   = 41797, /*!< 41797 (float FIX.5.0SP2) If the notional quantity is specified in a unit that does not match the unit in which the commodity reference price is quoted, the scaling or conversion factor used to convert the commodity reference price unit into the notional quantity unit should be stated here. If there is no conversion, this field is not intended to be used.*/
UnderlyingDeliveryStreamTransportEquipment         = 41798, /*!< 41798 (String FIX.5.0SP2) The transportation equipment with which the commodity product will be delivered and received.*/
UnderlyingDeliveryStreamElectingPartySide          = 41799, /*!< 41799 (int FIX.5.0SP2) A reference to the party able to choose whether the gas is delivered for a particular period e.g. a swing or interruptible contract.*/
NoUnderlyingStreamAssetAttributes                  = 41800, /*!< 41800 (NumInGroup FIX.5.0SP2) Number of asset attribute entries in the group.*/
UnderlyingStreamAssetAttributeType                 = 41801, /*!< 41801 (String FIX.5.0SP2) Specifies the name of the attribute.  

See http://www.fixtradingcommunity.org/codelists#Asset_Attribute_Types for code list of applicable asset attribute types.*/
UnderlyingStreamAssetAttributeValue                = 41802, /*!< 41802 (String FIX.5.0SP2) Specifies the value of the attribute.*/
UnderlyingStreamAssetAttributeLimit                = 41803, /*!< 41803 (String FIX.5.0SP2) The limit or lower acceptable value of the attribute.*/
NoUnderlyingDeliveryStreamCycles                   = 41804, /*!< 41804 (NumInGroup FIX.5.0SP2) Number of delivery cycles in the repeating group.*/
UnderlyingDeliveryStreamCycleDesc                  = 41805, /*!< 41805 (String FIX.5.0SP2) The delivery cycles during which the oil product will be transported in the pipeline.*/
EncodedUnderlyingDeliveryStreamCycleDescLen        = 41806, /*!< 41806 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingDeliveryStreamCycleDesc(41807) field.*/
EncodedUnderlyingDeliveryStreamCycleDesc           = 41807, /*!< 41807 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingDeliveryStreamCycleDesc(41805) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingDeliveryStreamCycleDesc(41805) field.*/
NoUnderlyingDeliveryStreamCommoditySources         = 41808, /*!< 41808 (NumInGroup FIX.5.0SP2) Number of commodity sources in the repeating group.*/
UnderlyingDeliveryStreamCommoditySource            = 41809, /*!< 41809 (String FIX.5.0SP2) The SCoTA coal cargo origin, mining region, mine(s), mining complex(es), loadout(s) or river dock(s) or other point(s) of origin that seller and buyer agree are acceptable origins for the coal product. For international coal transactions, this is the origin of the coal product. 

See http://www.fpml.org/coding-scheme/commodity-coal-product-source for values.*/
UnderlyingExerciseDesc                             = 41810, /*!< 41810 (String FIX.5.0SP2) A description of  the option exercise.*/
EncodedUnderlyingExerciseDescLen                   = 41811, /*!< 41811 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingExerciseDesc(41812) field.*/
EncodedUnderlyingExerciseDesc                      = 41812, /*!< 41812 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingExerciseDesc(41810) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingExerciseDesc(41810) field.*/
UnderlyingAutomaticExerciseIndicator               = 41813, /*!< 41813 (Boolean FIX.5.0SP2) Indicates (when 'Y') that exercise is automatic when the strike price is crossed or the underlying trade is in the money.*/
UnderlyingAutomaticExerciseThresholdRate           = 41814, /*!< 41814 (float FIX.5.0SP2) The threshold rate for triggering automatic exercise.*/
UnderlyingExerciseConfirmationMethod               = 41815, /*!< 41815 (int FIX.5.0SP2) Indicates whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller's agent.*/
UnderlyingManualNoticeBusinessCenter               = 41816, /*!< 41816 (String FIX.5.0SP2) Identifies the business center used for adjusting the time for manual exercise notice.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingFallbackExerciseIndicator                = 41817, /*!< 41817 (Boolean FIX.5.0SP2) Indicates whether the notional amount of the underlying swap, not previously exercised under the option, will be automatically exercised at the expiration time on the expiration date if at such time the buyer is in-the-money, provided that the difference between the settlement rate and the fixed rate under the relevant underlying swap is not less than one tenth of a percentage point (0.10% or 0.001).*/
UnderlyingLimitedRightToConfirmIndicator           = 41818, /*!< 41818 (Boolean FIX.5.0SP2) Indicates whether the Seller may request the Buyer to confirm its intent to exercise if not done on or before the expiration time on the Expiration date. If true ("Y") specific rules will apply in relation to the settlement mode.*/
UnderlyingExerciseSplitTicketIndicator             = 41819, /*!< 41819 (Boolean FIX.5.0SP2) Indicates in physical settlement of bond and convertible bond options whether the party required to deliver the bonds will divide those to be delivered as notifying party desires to facilitate delivery obligations.*/
NoUnderlyingOptionExerciseBusinessCenters          = 41820, /*!< 41820 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingOptionExerciseBusinessCenter             = 41821, /*!< 41821 (String FIX.5.0SP2) The business center calendar used to adjust the option exercise dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingOptionExerciseBusinessDayConvention      = 41822, /*!< 41822 (int FIX.5.0SP2) The business day convention used to adjust the option exercise dates. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingOptionExerciseEarliestDateOffsetDayType  = 41823, /*!< 41823 (int FIX.5.0SP2) Specifies the day type of the relative earliest exercise date offset.*/
UnderlyingOptionExerciseEarliestDateOffsetPeriod   = 41824, /*!< 41824 (int FIX.5.0SP2) Time unit multiplier for the relative earliest exercise date offset. */
UnderlyingOptionExerciseEarliestDateOffsetUnit     = 41825, /*!< 41825 (String FIX.5.0SP2) Time unit associated with the relative earliest exercise date offset.*/
UnderlyingOptionExerciseFrequencyPeriod            = 41826, /*!< 41826 (int FIX.5.0SP2) Time unit multiplier for the frequency of exercise dates. */
UnderlyingOptionExerciseFrequencyUnit              = 41827, /*!< 41827 (String FIX.5.0SP2) Time unit associated with the frequency of exercise dates. */
UnderlyingOptionExerciseStartDateUnadjusted        = 41828, /*!< 41828 (LocalMktDate FIX.5.0SP2) The unadjusted start date for calculating periodic exercise dates.*/
UnderlyingOptionExerciseStartDateRelativeTo        = 41829, /*!< 41829 (int FIX.5.0SP2) Specifies the anchor date when the option exercise start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingOptionExerciseStartDateOffsetPeriod      = 41830, /*!< 41830 (int FIX.5.0SP2) Time unit multiplier for the relative exercise start date offset.  */
UnderlyingOptionExerciseStartDateOffsetUnit        = 41831, /*!< 41831 (String FIX.5.0SP2) Time unit associated with the relative exercise start date offset.*/
UnderlyingOptionExerciseStartDateOffsetDayType     = 41832, /*!< 41832 (int FIX.5.0SP2) Specifies the day type of the relative option exercise start date offset.*/
UnderlyingOptionExerciseStartDateAdjusted          = 41833, /*!< 41833 (LocalMktDate FIX.5.0SP2) The adjusted start date for calculating periodic exercise dates.*/
UnderlyingOptionExerciseSkip                       = 41834, /*!< 41834 (int FIX.5.0SP2) The number of periods in the referenced date schedule that are between each date in the relative date schedule. Thus a skip of 2 would mean that dates are relative to every second date in the referenced schedule. If present this should have a value greater than 1.*/
UnderlyingOptionExerciseNominationDeadline         = 41835, /*!< 41835 (LocalMktDate FIX.5.0SP2) The last date (adjusted) for establishing the option exercise terms.*/
UnderlyingOptionExerciseFirstDateUnadjusted        = 41836, /*!< 41836 (LocalMktDate FIX.5.0SP2) The unadjusted first exercise date.*/
UnderlyingOptionExerciseLastDateUnadjusted         = 41837, /*!< 41837 (LocalMktDate FIX.5.0SP2) The unadjusted last exercise date.*/
UnderlyingOptionExerciseEarliestTime               = 41838, /*!< 41838 (LocalMktTime FIX.5.0SP2) The earliest time at which notice of exercise can be given by the buyer to the seller (or seller's agent) (i) on the expriation date, in the case of a European style option, (ii) on each Bermuda option exercise date and the expiration date, in the case of a Bermuda style option, (iii) the commencement date to, and including, the expiration date, in the case of an American option.*/
UnderlyingOptionExerciseLatestTime                 = 41839, /*!< 41839 (LocalMktTime FIX.5.0SP2) Latest exercise time.  See also UnderlyingOptionExerciseEarliestTime(41838).*/
UnderlyingOptionExerciseTimeBusinessCenter         = 41840, /*!< 41840 (String FIX.5.0SP2) The business center used to determine the locale for option exercise time, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values*/
NoUnderlyingOptionExerciseDates                    = 41841, /*!< 41841 (NumInGroup FIX.5.0SP2) Number of dates in the repeating group.*/
UnderlyingOptionExerciseDate                       = 41842, /*!< 41842 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted option exercise fixed date.*/
UnderlyingOptionExerciseDateType                   = 41843, /*!< 41843 (int FIX.5.0SP2) Specifies the type of option exercise date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
NoUnderlyingOptionExerciseExpirationDateBusinessCenters = 41844, /*!< 41844 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingOptionExerciseExpirationDateBusinessCenter = 41845, /*!< 41845 (String FIX.5.0SP2) The business center calendar used to adjust the option exercise expiration dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingOptionExerciseExpirationDateBusinessDayConvention = 41846, /*!< 41846 (int FIX.5.0SP2) The business day convention used to adjust the option exercise expiration dates. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingOptionExerciseExpirationDateRelativeTo   = 41847, /*!< 41847 (int FIX.5.0SP2) Specifies the anchor date when the option exercise expiration date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingOptionExerciseExpirationDateOffsetPeriod = 41848, /*!< 41848 (int FIX.5.0SP2) Time unit multiplier for the relative exercise expiration date offset. */
UnderlyingOptionExerciseExpirationDateOffsetUnit   = 41849, /*!< 41849 (String FIX.5.0SP2) Time unit associated with the relative exercise expiration date offset.*/
UnderlyingOptionExerciseExpirationFrequencyPeriod  = 41850, /*!< 41850 (int FIX.5.0SP2) Time unit multiplier for the frequency of exercise expiration dates. */
UnderlyingOptionExerciseExpirationFrequencyUnit    = 41851, /*!< 41851 (String FIX.5.0SP2) Time unit associated with the frequency of exercise expiration dates. */
UnderlyingOptionExerciseExpirationRollConvention   = 41852, /*!< 41852 (String FIX.5.0SP2) The convention for determining the sequence of exercise expiration dates. It is used in conjunction with a specified frequency. Used only to override the roll convention defined in the UnderlyingDateAdjustment component in UnderlyingInstrument.*/
UnderlyingOptionExerciseExpirationDateOffsetDayType = 41853, /*!< 41853 (int FIX.5.0SP2) Specifies the day type of the relative option exercise expiration date offset.*/
UnderlyingOptionExerciseExpirationTime             = 41854, /*!< 41854 (LocalMktTime FIX.5.0SP2) The option exercise expiration time.*/
UnderlyingOptionExerciseExpirationTimeBusinessCenter = 41855, /*!< 41855 (String FIX.5.0SP2) The business center used to determine the locale for option exercise expiration time, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingOptionExerciseExpirationDates          = 41856, /*!< 41856 (NumInGroup FIX.5.0SP2) Number of fixed exercise expiration dates in the repeating group.*/
UnderlyingOptionExerciseExpirationDate             = 41857, /*!< 41857 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted option exercise expiration fixed date.*/
UnderlyingOptionExerciseExpirationDateType         = 41858, /*!< 41858 (int FIX.5.0SP2) Specifies the type of option exercise expiration date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
UnderlyingMarketDisruptionProvision                = 41859, /*!< 41859 (int FIX.5.0SP2) The consequences of market disruption events.*/
UnderlyingMarketDisruptionFallbackProvision        = 41860, /*!< 41860 (int FIX.5.0SP2) Specifies the location of the fallback provision documentation.*/
UnderlyingMarketDisruptionMaximumDays              = 41861, /*!< 41861 (int FIX.5.0SP2) Specifies the maximum number of market disruption days (commodity or bullion business days) in a contract or confirmation. If none are specified, the maximum number of market disruption days is five (5).*/
UnderlyingMarketDisruptionMaterialityPercentage    = 41862, /*!< 41862 (Percentage FIX.5.0SP2) Used when a price materiality percentage applies to the price source disruption event and this event has been specified.*/
UnderlyingMarketDisruptionMinimumFuturesContracts  = 41863, /*!< 41863 (int FIX.5.0SP2) Specifies the minimum futures contracts level that dictates whether or not a 'De Minimis Trading' event has occurred. */
NoUnderlyingMarketDisruptionEvents                 = 41864, /*!< 41864 (NumInGroup FIX.5.0SP2) Number of disruption events in the repeating group.*/
UnderlyingMarketDisruptionEvent                    = 41865, /*!< 41865 (String FIX.5.0SP2) Specifies the market disruption event. 

For commodities see http://www.fpml.org/coding-scheme/commodity-market-disruption for values. 

For foreign exchange, see http://www.fixtradingcommunity.org/codelists#Market_Disruption_Event for code list of applicable event types.*/
NoUnderlyingMarketDisruptionFallbacks              = 41866, /*!< 41866 (NumInGroup FIX.5.0SP2) Number of fallbacks in the repeating group.*/
UnderlyingMarketDisruptionFallbackType             = 41867, /*!< 41867 (String FIX.5.0SP2) Specifies the type of disruption fallback. 

See http://www.fpml.org/coding-scheme/commodity-market-disruption-fallback for values. */
NoUnderlyingMarketDisruptionFallbackReferencePrices = 41868, /*!< 41868 (NumInGroup FIX.5.0SP2) Number of fallback reference securities in the repeating group.*/
UnderlyingMarketDisruptionFallbackUnderlierType    = 41869, /*!< 41869 (int FIX.5.0SP2) The type of reference price underlier.*/
UnderlyingMarketDisruptionFallbackUnderlierSecurityID = 41870, /*!< 41870 (String FIX.5.0SP2) Specifies the identifier value of the security.*/
UnderlyingMarketDisruptionFallbackUnderlierSecurityIDSource = 41871, /*!< 41871 (String FIX.5.0SP2) Specifies the class or source scheme of the security identifier.*/
UnderlyingMarketDisruptionFallbackUnderlierSecurityDesc = 41872, /*!< 41872 (String FIX.5.0SP2) Specifies the description of underlying security.*/
EncodedUnderlyingMarketDisruptionFallbackUnderlierSecurityDescLen = 41873, /*!< 41873 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingMarketDisruptionFallbackUnderlierSecurityDesc(41874) field.*/
EncodedUnderlyingMarketDisruptionFallbackUnderlierSecurityDesc = 41874, /*!< 41874 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingMarketDisruptionFallbackUnderlierSecurityDesc(41872) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingMarketDisruptionFallbackUnderlierSecurityDesc(41872).*/
UnderlyingMarketDisruptionFallbackOpenUnits        = 41875, /*!< 41875 (Qty FIX.5.0SP2) If there are multiple underlying assets, this specifies the number of units (index or securities) that constitute the underlier of the swap. In the case of a basket swap, this is used to reference both the number of basket units, and the number of each asset components of the basket when these are expressed in absolute terms.*/
UnderlyingMarketDisruptionFallbackBasketCurrency   = 41876, /*!< 41876 (Currency FIX.5.0SP2) Specifies the currency if the underlier is a basket.  Uses ISO 4217 currency codes.*/
UnderlyingMarketDisruptionFallbackBasketDivisor    = 41877, /*!< 41877 (float FIX.5.0SP2) Specifies the basket divisor amount. This value is normally used to adjust the constituent weight for pricing or to adjust for dividends, or other corporate actions.*/
NoUnderlyingPaymentScheduleFixingDays              = 41878, /*!< 41878 (NumInGroup FIX.5.0SP2) Number of fixing days in the repeating group.*/
UnderlyingPaymentScheduleFixingDayOfWeek           = 41879, /*!< 41879 (int FIX.5.0SP2) The day of the week on which fixing takes place. */
UnderlyingPaymentScheduleFixingDayNumber           = 41880, /*!< 41880 (int FIX.5.0SP2) The occurrence of the day of week on which fixing takes place. */
UnderlyingPaymentScheduleXID                       = 41881, /*!< 41881 (XID FIX.5.0SP2) Identifier of this UnderlyingPaymentSchedule for cross referencing elsewhere in the message.*/
UnderlyingPaymentScheduleXIDRef                    = 41882, /*!< 41882 (XIDREF FIX.5.0SP2) Reference to payment schedule elsewhere in the message.*/
UnderlyingPaymentScheduleRateCurrency              = 41883, /*!< 41883 (Currency FIX.5.0SP2) Specifies the currency of the schedule rate.  Uses ISO 4217 currency codes.*/
UnderlyingPaymentScheduleRateUnitOfMeasure         = 41884, /*!< 41884 (String FIX.5.0SP2) The schedule rate unit of measure (UOM).*/
UnderlyingPaymentScheduleRateConversionFactor      = 41885, /*!< 41885 (float FIX.5.0SP2) The number to be multiplied by the derived floating rate of the underlying's payment schedule in order to arrive at the payment rate. If ommitted, the schedule rate conversion factor is 1. */
UnderlyingPaymentScheduleRateSpreadType            = 41886, /*!< 41886 (int FIX.5.0SP2) Specifies whether the rate spread is an absolute value to be added to the index rate or a percentage of the index rate.*/
UnderlyingPaymentScheduleSettlPeriodPrice          = 41887, /*!< 41887 (Price FIX.5.0SP2) The schedule settlement period price.*/
UnderlyingPaymentScheduleSettlPeriodPriceCurrency  = 41888, /*!< 41888 (Currency FIX.5.0SP2) The currency of the schedule settlement period price. Uses ISO 4217 currency codes.*/
UnderlyingPaymentScheduleSettlPeriodPriceUnitOfMeasure = 41889, /*!< 41889 (String FIX.5.0SP2) The settlement period price unit of measure (UOM).*/
UnderlyingPaymentScheduleStepUnitOfMeasure         = 41890, /*!< 41890 (String FIX.5.0SP2) The schedule step unit of measure (UOM).*/
UnderlyingPaymentScheduleFixingDayDistribution     = 41891, /*!< 41891 (int FIX.5.0SP2) The distribution of fixing days.*/
UnderlyingPaymentScheduleFixingDayCount            = 41892, /*!< 41892 (int FIX.5.0SP2) The number of days over which fixing should take place.*/
UnderlyingPaymentScheduleFixingLagPeriod           = 41893, /*!< 41893 (int FIX.5.0SP2) Time unit multiplier for the fixing lag duration. */
UnderlyingPaymentScheduleFixingLagUnit             = 41894, /*!< 41894 (String FIX.5.0SP2) Time unit associated with the fixing lag duration.*/
UnderlyingPaymentScheduleFixingFirstObservationDateOffsetPeriod = 41895, /*!< 41895 (int FIX.5.0SP2) Time unit multiplier for the relative first observation date offset.*/
UnderlyingPaymentScheduleFixingFirstObservationDateOffsetUnit = 41896, /*!< 41896 (String FIX.5.0SP2) Time unit associated with the relative first observation date offset.*/
UnderlyingPaymentStreamFlatRateIndicator           = 41897, /*!< 41897 (Boolean FIX.5.0SP2) When this element is specified and set to 'Y', the Flat Rate is the New Worldwide Tanker Nominal Freight Scale for the Freight Index Route taken at the Trade Date of the transaction "Fixed". If 'N' it is taken on each Pricing Date "Floating".*/
UnderlyingPaymentStreamFlatRateAmount              = 41898, /*!< 41898 (Amt FIX.5.0SP2) Specifies the actual monetary value of the flat rate when UnderlyingPaymentStreamFlatRateIndicator(41897) = 'Y'.*/
UnderlyingPaymentStreamFlatRateCurrency            = 41899, /*!< 41899 (Currency FIX.5.0SP2) Specifies the currency of the actual flat rate.  Uses ISO 4217 currency codes.*/
UnderlyingPaymentStreamMaximumPaymentAmount        = 41900, /*!< 41900 (Amt FIX.5.0SP2) Specifies the limit on the total payment amount.*/
UnderlyingPaymentStreamMaximumPaymentCurrency      = 41901, /*!< 41901 (Currency FIX.5.0SP2) Specifies the currency of total payment amount limit. Uses ISO 4217 currency codes.*/
UnderlyingPaymentStreamMaximumTransactionAmount    = 41902, /*!< 41902 (Amt FIX.5.0SP2) Specifies the limit on the payment amount that goes out in any particular calculation period.*/
UnderlyingPaymentStreamMaximumTransactionCurrency  = 41903, /*!< 41903 (Currency FIX.5.0SP2) Specifies the currency of the period payment amount limit. Uses ISO 4217 currency codes. */
UnderlyingPaymentStreamFixedAmountUnitOfMeasure    = 41904, /*!< 41904 (String FIX.5.0SP2) Fixed payment amount unit of measure (UOM).*/
UnderlyingPaymentStreamTotalFixedAmount            = 41905, /*!< 41905 (Amt FIX.5.0SP2) Specifies the total fixed payment amount.*/
UnderlyingPaymentStreamWorldScaleRate              = 41906, /*!< 41906 (float FIX.5.0SP2) The number of Worldscale points for purposes of the calculation of a fixed amount for a wet voyage charter commodity swap.*/
UnderlyingPaymentStreamContractPrice               = 41907, /*!< 41907 (Price FIX.5.0SP2) The price per relevant unit for purposes of the calculation of a fixed amount for a dry voyage charter or time charter commodity swap.*/
UnderlyingPaymentStreamContractPriceCurrency       = 41908, /*!< 41908 (Currency FIX.5.0SP2) Specifies the currency of UnderlyingPaymentStreamContractPrice(41907).  Uses ISO 4217 currency codes.*/
NoUnderlyingPaymentStreamPricingBusinessCenters    = 41909, /*!< 41909 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingPaymentStreamPricingBusinessCenter       = 41910, /*!< 41910 (String FIX.5.0SP2) The business center calendar used to adjust the payment stream's pricing dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentStreamRateIndex2CurveUnit         = 41911, /*!< 41911 (String FIX.5.0SP2) Secondary time unit associated with the payment stream’s floating rate index curve.  */
UnderlyingPaymentStreamRateIndex2CurvePeriod       = 41912, /*!< 41912 (int FIX.5.0SP2) Secondary time unit multiplier for the payment stream’s floating rate index curve.  */
UnderlyingPaymentStreamRateIndexLocation           = 41913, /*!< 41913 (String FIX.5.0SP2) Specifies the location of the floating rate index.*/
UnderlyingPaymentStreamRateIndexLevel              = 41914, /*!< 41914 (Qty FIX.5.0SP2) This is the weather Cooling Degree Days (CDD), Heating Degree Days (HDD) or HDD index level specified as the number of (amount of) weather index units specified by the parties in the related confirmation.*/
UnderlyingPaymentStreamRateIndexUnitOfMeasure      = 41915, /*!< 41915 (String FIX.5.0SP2) The unit of measure (UOM) of the rate index level.*/
UnderlyingPaymentStreamSettlLevel                  = 41916, /*!< 41916 (int FIX.5.0SP2) Specifies how weather index units are to be calculated.*/
UnderlyingPaymentStreamReferenceLevel              = 41917, /*!< 41917 (Qty FIX.5.0SP2) This is the weather Cooling Degree Days (CDD), Heating Degree Days (HDD) or HDD reference level specified as the number of (amount of) weather index units specified by the parties in the related confirmation.*/
UnderlyingPaymentStreamReferenceLevelUnitOfMeasure = 41918, /*!< 41918 (String FIX.5.0SP2) The unit of measure (UOM) of the rate reference level.*/
UnderlyingPaymentStreamReferenceLevelEqualsZeroIndicator = 41919, /*!< 41919 (Boolean FIX.5.0SP2) When set to 'Y', it indicates that the weather reference level equals zero.*/
UnderlyingPaymentStreamRateSpreadCurrency          = 41920, /*!< 41920 (Currency FIX.5.0SP2) Specifies the currency of the floating rate spread. Uses ISO 4217 currency codes.*/
UnderlyingPaymentStreamRateSpreadUnitOfMeasure     = 41921, /*!< 41921 (String FIX.5.0SP2) Specifies the unit of measure (UOM) of the floating rate spread. */
UnderlyingPaymentStreamRateConversionFactor        = 41922, /*!< 41922 (float FIX.5.0SP2) The number to be multiplied by the derived floating rate of the underlying's payment stream in order to arrive at the payment rate. If omitted, the floating rate conversion factor is 1. */
UnderlyingPaymentStreamRateSpreadType              = 41923, /*!< 41923 (int FIX.5.0SP2) Identifies whether the rate spread is an absolute value to be added to the index rate or a percentage of the index rate.*/
UnderlyingPaymentStreamLastResetRate               = 41924, /*!< 41924 (Percentage FIX.5.0SP2) The floating rate determined at the most recent reset. The rate is expressed in decimal form, e.g. 5% is represented as 0.05.*/
UnderlyingPaymentStreamFinalRate                   = 41925, /*!< 41925 (Percentage FIX.5.0SP2) The floating rate determined at the final reset. The rate is expressed in decimal form, e.g. 5% is represented as 0.05.*/
UnderlyingPaymentStreamCalculationLagPeriod        = 41926, /*!< 41926 (int FIX.5.0SP2) Time unit multiplier for the calculation lag duration.  */
UnderlyingPaymentStreamCalculationLagUnit          = 41927, /*!< 41927 (String FIX.5.0SP2) Time unit associated with the calculation lag duration.*/
UnderlyingPaymentStreamFirstObservationDateOffsetPeriod = 41928, /*!< 41928 (int FIX.5.0SP2) Time unit multiplier for the relative first observation date offset.*/
UnderlyingPaymentStreamFirstObservationDateOffsetUnit = 41929, /*!< 41929 (String FIX.5.0SP2) Time unit associated with the relative first observation date offset.*/
UnderlyingPaymentStreamPricingDayType              = 41930, /*!< 41930 (int FIX.5.0SP2) Specifies the commodity pricing day type.*/
UnderlyingPaymentStreamPricingDayDistribution      = 41931, /*!< 41931 (int FIX.5.0SP2) The distribution of pricing days.*/
UnderlyingPaymentStreamPricingDayCount             = 41932, /*!< 41932 (int FIX.5.0SP2) The number of days over which pricing should take place.*/
UnderlyingPaymentStreamPricingBusinessCalendar     = 41933, /*!< 41933 (String FIX.5.0SP2) Specifies the business calendar to use for pricing. 

See http://www.fpml.org/coding-scheme/commodity-business-calendar for values.*/
UnderlyingPaymentStreamPricingBusinessDayConvention = 41934, /*!< 41934 (int FIX.5.0SP2) The business day convention used to adjust the payment stream's pricing dates. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
LegStreamCommoditySettlTimeType                    = 41935, /*!< 41935 (int FIX.5.0SP2) Specifies the format of the commodity settlement start and end times.*/
UnderlyingStreamCommoditySettlTimeType             = 41936, /*!< 41936 (int FIX.5.0SP2) Specifies the format of the commodity settlement start and end times.*/
NoUnderlyingPaymentStreamPaymentDates              = 41937, /*!< 41937 (NumInGroup FIX.5.0SP2) Number of payment dates in the repeating group.*/
UnderlyingPaymentStreamPaymentDate                 = 41938, /*!< 41938 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted fixed stream payment date.*/
UnderlyingPaymentStreamPaymentDateType             = 41939, /*!< 41939 (int FIX.5.0SP2) Specifies the type of payment date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
UnderlyingPaymentStreamMasterAgreementPaymentDatesIndicator = 41940, /*!< 41940 (Boolean FIX.5.0SP2) When set to 'Y', it indicates that payment dates are specified in the relevant master agreement.*/
NoUnderlyingPaymentStreamPricingDates              = 41941, /*!< 41941 (NumInGroup FIX.5.0SP2) Number of pricing dates in the repeating group.*/
UnderlyingPaymentStreamPricingDate                 = 41942, /*!< 41942 (LocalMktDate FIX.5.0SP2) An adjusted or unadjusted fixed pricing date. */
UnderlyingPaymentStreamPricingDateType             = 41943, /*!< 41943 (int FIX.5.0SP2) Specifies the type of pricing date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
NoUnderlyingPaymentStreamPricingDays               = 41944, /*!< 41944 (NumInGroup FIX.5.0SP2) Number of pricing days in the repeating group.*/
UnderlyingPaymentStreamPricingDayOfWeek            = 41945, /*!< 41945 (int FIX.5.0SP2) The day of the week on which pricing takes place.*/
UnderlyingPaymentStreamPricingDayNumber            = 41946, /*!< 41946 (int FIX.5.0SP2) The occurrence of the day of week on which pricing takes place. */
NoUnderlyingPricingDateBusinessCenters             = 41947, /*!< 41947 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingPricingDateBusinessCenter                = 41948, /*!< 41948 (String FIX.5.0SP2) The business center calendar used to adjust the pricing or fixing date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPricingDateUnadjusted                    = 41949, /*!< 41949 (LocalMktDate FIX.5.0SP2) The unadjusted pricing or fixing date.*/
UnderlyingPricingDateBusinessDayConvention         = 41950, /*!< 41950 (int FIX.5.0SP2) The business day convention used to adjust the pricing or fixing date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingPricingDateAdjusted                      = 41951, /*!< 41951 (LocalMktDate FIX.5.0SP2) The adjusted pricing or fixing date.*/
UnderlyingPricingTime                              = 41952, /*!< 41952 (LocalMktTime FIX.5.0SP2) The local market pricing or fixing time.*/
UnderlyingPricingTimeBusinessCenter                = 41953, /*!< 41953 (String FIX.5.0SP2) Specifies the business center for determining the pricing or fixing time. See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingStreamCalculationPeriodDates           = 41954, /*!< 41954 (NumInGroup FIX.5.0SP2) Number of calculation period dates in the repeating group.*/
UnderlyingStreamCalculationPeriodDate              = 41955, /*!< 41955 (LocalMktDate FIX.5.0SP2) The adjusted or unadjusted fixed calculation period date.*/
UnderlyingStreamCalculationPeriodDateType          = 41956, /*!< 41956 (int FIX.5.0SP2) Specifies the type of fixed calculation period date. When specified it applies not only to the current date but to all subsequent dates in the group until overridden with a new type.*/
UnderlyingStreamCalculationPeriodDatesXID          = 41957, /*!< 41957 (XID FIX.5.0SP2) Identifier of this calculation period for cross referencing elsewhere in the message.*/
UnderlyingStreamCalculationPeriodDatesXIDRef       = 41958, /*!< 41958 (XIDREF FIX.5.0SP2) Cross reference to another calculation period for duplicating its properties.*/
UnderlyingStreamCalculationBalanceOfFirstPeriod    = 41959, /*!< 41959 (Boolean FIX.5.0SP2) When specified and set to 'Y', it indicates that the first calculation period should run from the effective date to the end of the calendar period in which the effective date falls (e.g. Jan 15 - Jan 31 if the calculation periods are one month long and effective date is Jan 15.). If 'N' or not specified, it indicates that the first calculation period should run from the effective date for one whole period (e.g. Jan 15 to Feb 14 if the calculation periods are one month long and the effective date is Jan 15.).*/
UnderlyingStreamCalculationCorrectionPeriod        = 41960, /*!< 41960 (int FIX.5.0SP2) Time unit multiplier for the length of time after the publication of the data when corrections can be made.*/
UnderlyingStreamCalculationCorrectionUnit          = 41961, /*!< 41961 (String FIX.5.0SP2) Time unit associated with the length of time after the publication of the data when corrections can be made. */
NoUnderlyingStreamCommoditySettlBusinessCenters    = 41962, /*!< 41962 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingStreamCommoditySettlBusinessCenter       = 41963, /*!< 41963 (String FIX.5.0SP2) The business center calendar used to adjust the commodity delivery date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingStreamCommodityBase                      = 41964, /*!< 41964 (String FIX.5.0SP2) Specifies the general base type of the commodity traded. Where possible, this should follow the naming convention used in the 2005 ISDA Commodity Definitions. */
UnderlyingStreamCommodityType                      = 41965, /*!< 41965 (String FIX.5.0SP2) Specifies the type of commodity product.

For coal see http://www.fpml.org/coding-scheme/commodity-coal-product-type for values. 

For metals see http://www.fpml.org/coding-scheme/commodity-metal-product-type for values. 

For bullion see http://www.fixtradingcommunity.org/codelists#Bullion_Types for the external code list of bullion types.*/
UnderlyingStreamCommoditySecurityID                = 41966, /*!< 41966 (String FIX.5.0SP2) Specifies the market identifier for the commodity.*/
UnderlyingStreamCommoditySecurityIDSource          = 41967, /*!< 41967 (String FIX.5.0SP2) Identifies the class or source of the UnderlyingStreamCommoditySecurityIDSource(41966) value.*/
UnderlyingStreamCommodityDesc                      = 41968, /*!< 41968 (String FIX.5.0SP2) Description of the commodity asset.*/
EncodedUnderlyingStreamCommodityDescLen            = 41969, /*!< 41969 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingStreamCommodityDesc(41970) field.*/
EncodedUnderlyingStreamCommodityDesc               = 41970, /*!< 41970 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingStreamCommodityDesc(41968) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingStreamCommodityDesc(41968) field.*/
UnderlyingStreamCommodityUnitOfMeasure             = 41971, /*!< 41971 (String FIX.5.0SP2) The unit of measure (UOM) of the commodity asset.*/
UnderlyingStreamCommodityCurrency                  = 41972, /*!< 41972 (Currency FIX.5.0SP2) Identifies the currency of the commodity asset.  Uses ISO 4217 currency codes.*/
UnderlyingStreamCommodityExchange                  = 41973, /*!< 41973 (Exchange FIX.5.0SP2) Identifies the exchange where the commodity is traded.*/
UnderlyingStreamCommodityRateSource                = 41974, /*!< 41974 (int FIX.5.0SP2) Identifies the source of rate information used for commodities. 

See http://www.fixtradingcommunity.org/codelists#Commodity_Rate_Source for code list of applicable sources.*/
UnderlyingStreamCommodityRateReferencePage         = 41975, /*!< 41975 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.*/
UnderlyingStreamCommodityRateReferencePageHeading  = 41976, /*!< 41976 (String FIX.5.0SP2) Identifies the page heading from the rate source.*/
UnderlyingStreamDataProvider                       = 41977, /*!< 41977 (String FIX.5.0SP2) Specifies the commodity data or information provider. 

See http://www.fpml.org/coding-scheme/commodity-information-provider for values.*/
UnderlyingStreamCommodityPricingType               = 41978, /*!< 41978 (String FIX.5.0SP2) Specifies how the pricing or rate setting of the trade is to be determined or based upon.  

See http://www.fixtradingcommunity.org/codelists#Commodity_Rate_Pricing_Type for code list of applicable commodity pricing types.*/
UnderlyingStreamCommodityNearbySettlDayPeriod      = 41979, /*!< 41979 (int FIX.5.0SP2) Time unit multiplier for the nearby settlement day. */
UnderlyingStreamCommodityNearbySettlDayUnit        = 41980, /*!< 41980 (String FIX.5.0SP2) Time unit associated with the nearby settlement day.*/
UnderlyingStreamCommoditySettlDateUnadjusted       = 41981, /*!< 41981 (LocalMktDate FIX.5.0SP2) The unadjusted commodity delivery date.*/
UnderlyingStreamCommoditySettlDateBusinessDayConvention = 41982, /*!< 41982 (int FIX.5.0SP2) The business day convention used to adjust the commodity delivery date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingStreamCommoditySettlDateAdjusted         = 41983, /*!< 41983 (LocalMktDate FIX.5.0SP2) The adjusted commodity delivery date.*/
UnderlyingStreamCommoditySettlMonth                = 41984, /*!< 41984 (int FIX.5.0SP2) Specifies a fixed single month for commodity delivery. */
UnderlyingStreamCommoditySettlDateRollPeriod       = 41985, /*!< 41985 (int FIX.5.0SP2) Time unit multiplier for the commodity delivery date roll.  */
UnderlyingStreamCommoditySettlDateRollUnit         = 41986, /*!< 41986 (String FIX.5.0SP2) Time unit associated with the commodity delivery date roll.*/
UnderlyingStreamCommoditySettlDayType              = 41987, /*!< 41987 (int FIX.5.0SP2) Specifies the commodity delivery roll day type.*/
UnderlyingStreamCommodityXID                       = 41988, /*!< 41988 (XID FIX.5.0SP2) Identifier of this stream commodity for cross referencing elsewhere in the message.*/
UnderlyingStreamCommodityXIDRef                    = 41989, /*!< 41989 (XIDREF FIX.5.0SP2) Reference to a stream commodity elsewhere in the message.*/
NoUnderlyingStreamCommodityAltIDs                  = 41990, /*!< 41990 (NumInGroup FIX.5.0SP2) Number of alternate security identifers.*/
UnderlyingStreamCommodityAltID                     = 41991, /*!< 41991 (String FIX.5.0SP2) Alternate security identifier value for the commodity.*/
UnderlyingStreamCommodityAltIDSource               = 41992, /*!< 41992 (String FIX.5.0SP2) Identifies the class or source of the alternate commodity security identifier.*/
NoUnderlyingStreamCommodityDataSources             = 41993, /*!< 41993 (NumInGroup FIX.5.0SP2) Number of commodity data sources in the repeating group.  */
UnderlyingStreamCommodityDataSourceID              = 41994, /*!< 41994 (String FIX.5.0SP2) Data source identifier.*/
UnderlyingStreamCommodityDataSourceIDType          = 41995, /*!< 41995 (int FIX.5.0SP2) Specifies the type of data source identifier.*/
NoUnderlyingStreamCommoditySettlDays               = 41996, /*!< 41996 (NumInGroup FIX.5.0SP2) Number of days in the repeating group.*/
UnderlyingStreamCommoditySettlDay                  = 41997, /*!< 41997 (int FIX.5.0SP2) Specifies the day or group of days for delivery.*/
UnderlyingStreamCommoditySettlTotalHours           = 41998, /*!< 41998 (int FIX.5.0SP2) Sum of the hours specified in UnderlyingStreamCommoditySettlTimeGrp.*/
NoUnderlyingStreamCommoditySettlTimes              = 41999, /*!< 41999 (NumInGroup FIX.5.0SP2) Number of hour ranges in the repeating group.*/
UnderlyingStreamCommoditySettlStart                = 42000, /*!< 42000 (String FIX.5.0SP2) The start time for commodity settlement where delivery occurs over time. The time format is specified by the settlement time type.*/
UnderlyingStreamCommoditySettlEnd                  = 42001, /*!< 42001 (String FIX.5.0SP2) The end time for commodity settlement where delivery occurs over time. The time format is specified by the settlement time type.*/
NoUnderlyingStreamCommoditySettlPeriods            = 42002, /*!< 42002 (NumInGroup FIX.5.0SP2) Number of commodity settlement periods in the repeating group.*/
UnderlyingStreamCommoditySettlCountry              = 42003, /*!< 42003 (Country FIX.5.0SP2) Specifies the country where delivery takes place. Uses ISO 3166 2-character country code.*/
UnderlyingStreamCommoditySettlTimeZone             = 42004, /*!< 42004 (String FIX.5.0SP2) Commodity delivery timezone specified as "prevailing" rather than "standard" or "daylight".  

See http://www.fixtradingcommunity.org/codelists#Prevailing_Timezones for code list of applicable prevailing timezones.*/
UnderlyingStreamCommoditySettlFlowType             = 42005, /*!< 42005 (int FIX.5.0SP2) Specifies the commodity delivery flow type.*/
UnderlyingStreamCommoditySettlPeriodNotional       = 42006, /*!< 42006 (Qty FIX.5.0SP2) Specifies the delivery quantity associated with this settlement period.*/
UnderlyingStreamCommoditySettlPeriodNotionalUnitOfMeasure = 42007, /*!< 42007 (String FIX.5.0SP2) Specifies the unit of measure (UOM) of the delivery quantity associated with this settlement period.*/
UnderlyingStreamCommoditySettlPeriodFrequencyPeriod = 42008, /*!< 42008 (int FIX.5.0SP2) Time unit multiplier for the settlement period frequency.*/
UnderlyingStreamCommoditySettlPeriodFrequencyUnit  = 42009, /*!< 42009 (String FIX.5.0SP2) Time unit associated with the settlement period frequency.*/
UnderlyingStreamCommoditySettlPeriodPrice          = 42010, /*!< 42010 (Price FIX.5.0SP2) The settlement period price.*/
UnderlyingStreamCommoditySettlPeriodPriceUnitOfMeasure = 42011, /*!< 42011 (String FIX.5.0SP2) Specifies the settlement period price unit of measure (UOM).*/
UnderlyingStreamCommoditySettlPeriodPriceCurrency  = 42012, /*!< 42012 (Currency FIX.5.0SP2) The currency of the settlement period price.  Uses ISO 4217 currency codes.*/
UnderlyingStreamCommoditySettlHolidaysProcessingInstruction = 42013, /*!< 42013 (int FIX.5.0SP2) Indicates whether holidays are included in the settlement periods. Required for electricity contracts.*/
UnderlyingStreamCommoditySettlPeriodXID            = 42014, /*!< 42014 (XID FIX.5.0SP2) Identifier of this settlement period for cross referencing elsewhere in the message.*/
UnderlyingStreamCommoditySettlPeriodXIDRef         = 42015, /*!< 42015 (XIDREF FIX.5.0SP2) Cross reference to another settlement period for duplicating its properties.*/
UnderlyingStreamXID                                = 42016, /*!< 42016 (XID FIX.5.0SP2) Identifier of this UnderlyingStream for cross referencing elsewhere in the message.*/
UnderlyingAdditionalTermBondIssuer                 = 42017, /*!< 42017 (String FIX.5.0SP2) Issuer of the bond.*/
UnderlyingStreamNotionalXIDRef                     = 42018, /*!< 42018 (XIDREF FIX.5.0SP2) Cross reference to another UnderlyingStream notional for duplicating its properties.*/
UnderlyingStreamNotionalFrequencyPeriod            = 42019, /*!< 42019 (int FIX.5.0SP2) Time unit multiplier for the swap stream's notional frequency.*/
UnderlyingStreamNotionalFrequencyUnit              = 42020, /*!< 42020 (String FIX.5.0SP2) Time unit associated with the swap stream's notional frequency.*/
UnderlyingStreamNotionalCommodityFrequency         = 42021, /*!< 42021 (int FIX.5.0SP2) The commodity's notional or quantity delivery frequency.*/
UnderlyingStreamNotionalUnitOfMeasure              = 42022, /*!< 42022 (String FIX.5.0SP2) Specifies the delivery quantity unit of measure (UOM).*/
UnderlyingStreamTotalNotional                      = 42023, /*!< 42023 (Qty FIX.5.0SP2) Specifies the total notional or delivery quantity over the term of the contract.*/
UnderlyingStreamTotalNotionalUnitOfMeasure         = 42024, /*!< 42024 (String FIX.5.0SP2) Specifies the unit of measure (UOM) for the total notional or delivery quantity over the term of the contract.*/
EncodedUnderlyingAdditionalTermBondIssuerLen       = 42025, /*!< 42025 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingAdditionalTermBondIssuer(42026) field.*/
EncodedUnderlyingAdditionalTermBondIssuer          = 42026, /*!< 42026 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingAdditionalTermBondIssuer(42017) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingAdditionalTermBondIssuer(42017) field.*/
UnderlyingAdditionalTermBondSeniority              = 42027, /*!< 42027 (String FIX.5.0SP2) Specifies the bond's payment priority in the event of a default. */
UnderlyingAdditionalTermBondCouponType             = 42028, /*!< 42028 (int FIX.5.0SP2) Coupon type of the bond.*/
UnderlyingAdditionalTermBondCouponRate             = 42029, /*!< 42029 (Percentage FIX.5.0SP2) Coupon rate of the bond.  See also CouponRate(223).*/
UnderlyingAdditionalTermBondMaturityDate           = 42030, /*!< 42030 (LocalMktDate FIX.5.0SP2) The maturity date of the bond.*/
UnderlyingAdditionalTermBondParValue               = 42031, /*!< 42031 (Amt FIX.5.0SP2) The par value of the bond.*/
UnderlyingAdditionalTermBondCurrentTotalIssuedAmount = 42032, /*!< 42032 (Amt FIX.5.0SP2) Total issued amount of the bond.*/
UnderlyingAdditionalTermBondCouponFrequencyPeriod  = 42033, /*!< 42033 (int FIX.5.0SP2) Time unit multiplier for the frequency of the bond's coupon payment.*/
UnderlyingAdditionalTermBondCouponFrequencyUnit    = 42034, /*!< 42034 (String FIX.5.0SP2) Time unit associated with the frequency of the bond's coupon payment.  */
UnderlyingAdditionalTermBondDayCount               = 42035, /*!< 42035 (int FIX.5.0SP2) The day count convention used in interest calculations for a bond or an interest bearing security.*/
NoUnderlyingAdditionalTerms                        = 42036, /*!< 42036 (NumInGroup FIX.5.0SP2) Number of additional terms in the repeating group.*/
UnderlyingAdditionalTermConditionPrecedentBondIndicator = 42037, /*!< 42037 (Boolean FIX.5.0SP2) Indicates whether the condition precedent bond is applicable. The swap contract is only valid if the bond is issued and if there is any dispute over the terms of fixed stream then the bond terms would be used.*/
UnderlyingAdditionalTermDiscrepancyClauseIndicator = 42038, /*!< 42038 (Boolean FIX.5.0SP2) Indicates whether the discrepancy clause is applicable.*/
NoUnderlyingCashSettlDealers                       = 42039, /*!< 42039 (NumInGroup FIX.5.0SP2) Number of dealers in the repeating group.*/
UnderlyingCashSettlDealer                          = 42040, /*!< 42040 (String FIX.5.0SP2) Identifies the dealer from whom price quotations for the reference obligation are obtained for the purpose of cash settlement valuation calculation.*/
NoUnderlyingCashSettlTerms                         = 42041, /*!< 42041 (NumInGroup FIX.5.0SP2) Number of elements in the repeating group.*/
UnderlyingCashSettlCurrency                        = 42042, /*!< 42042 (Currency FIX.5.0SP2) Specifies the currency the UnderlyingCashSettlAmount(42054) is denominated in.  Uses ISO 4217 currency codes.*/
UnderlyingCashSettlValuationFirstBusinessDayOffset = 42043, /*!< 42043 (int FIX.5.0SP2) The number of business days after settlement conditions have been satisfied, when the calculation agent is to obtain a price quotation on the reference obligation for purposes of cash settlement. */
UnderlyingCashSettlValuationSubsequentBusinessDaysOffset = 42044, /*!< 42044 (int FIX.5.0SP2) The number of business days between successive valuation dates when multiple valuation dates are applicable for cash settlement.*/
UnderlyingCashSettlNumOfValuationDates             = 42045, /*!< 42045 (int FIX.5.0SP2) Where multiple valuation dates are specified as being applicable for cash settlement, this element specifies the number of applicable valuation dates.*/
UnderlyingCashSettlValuationTime                   = 42046, /*!< 42046 (LocalMktTime FIX.5.0SP2) Time of valuation.*/
UnderlyingCashSettlBusinessCenter                  = 42047, /*!< 42047 (String FIX.5.0SP2) Identifies the business center calendar  used at valuation time for cash settlement purposes e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingCashSettlQuoteMethod                     = 42048, /*!< 42048 (int FIX.5.0SP2) The type of quote used to determine the cash settlement price.*/
UnderlyingCashSettlQuoteAmount                     = 42049, /*!< 42049 (Amt FIX.5.0SP2) When determining the cash settlement amount, if weighted average price quotes are to be obtained for the reference obligation, this is the upper limit to the outstanding principal balance of the reference obligation for which the quote should be obtained.  If not specifed, the ISDA definitions provide for a fallback amount equal to floating rate payer calculation amount.*/
UnderlyingCashSettlQuoteCurrency                   = 42050, /*!< 42050 (Currency FIX.5.0SP2) Specifies the currency the UnderlyingCashSettlQuoteAmount(42049) is denominated in.  Uses ISO 4217 currency codes.*/
UnderlyingCashSettlMinimumQuoteAmount              = 42051, /*!< 42051 (Amt FIX.5.0SP2) When determining the cash settlement amount, if weighted average price quotes are to be obtained for the reference obligation, this is the minimum intended threshold amount of outstanding principal balance of the reference obligation for which the quote should be obtained.  If not specified, the ISDA definitions provide for a fallback amount of the lower of either USD1,000,000 (or its equivalent in the relevent obligation currency) or the (minimum) quoted amount.*/
UnderlyingCashSettlMinimumQuoteCurrency            = 42052, /*!< 42052 (Currency FIX.5.0SP2) Specifies the currency the UnderlyingCashSettlQuoteAmount(42049) is denominated in.  Uses ISO 4217 currency codes.*/
UnderlyingCashSettlBusinessDays                    = 42053, /*!< 42053 (int FIX.5.0SP2) The number of business days used in the determination of the cash settlement payment date. */
UnderlyingCashSettlAmount                          = 42054, /*!< 42054 (Amt FIX.5.0SP2) The amount paid between the trade parties, seller to the buyer, for cash settlement on the cash settlement date.	*/
UnderlyingCashSettlRecoveryFactor                  = 42055, /*!< 42055 (float FIX.5.0SP2) Used for fixed recovery, this specifies the recovery level as determined at contract inception, to be applied in the event of a default.  The factor is used to calculate the amount paid by the seller to the buyer for cash settlement on the cash settlement date.  The amount is calculated is (1 - UnderlyingCashSettlRecoveryFactor(42055)) x floating rate payer calculation amount.  The currency is derived from the floating rate payer calculation amount.*/
UnderlyingCashSettlFixedTermIndicator              = 42056, /*!< 42056 (Boolean FIX.5.0SP2) Indicates whether fixed settlement is applicable or not applicable in a recovery lock.*/
UnderlyingCashSettlAccruedInterestIndicator        = 42057, /*!< 42057 (Boolean FIX.5.0SP2) Indicates whether accrued interest is included or not in the value provided in UnderlyingCashSettlAmount(42054).

For cash settlement this specifies whether quotations should be obtained inclusive or not of accrued interest. 

For physical settlement this specifies whether the buyer should deliver the obligation with an outstanding principal balance that includes or excludes accrued interest. 

*/
UnderlyingCashSettlValuationMethod                 = 42058, /*!< 42058 (int FIX.5.0SP2) The ISDA defined methodology for determining the final price of the reference obligation for purposes of cash settlement. */
UnderlyingCashSettlTermXID                         = 42059, /*!< 42059 (XID FIX.5.0SP2) Name referenced from UnderlyingSettlementTermXIDRef(41315).*/
NoUnderlyingPhysicalSettlTerms                     = 42060, /*!< 42060 (NumInGroup FIX.5.0SP2) Number of entries in the repeating group.*/
UnderlyingPhysicalSettlCurrency                    = 42061, /*!< 42061 (Currency FIX.5.0SP2) Currency of physical settlement.  Uses ISO 4217 currency codes.*/
UnderlyingPhysicalSettlBusinessDays                = 42062, /*!< 42062 (int FIX.5.0SP2) A number of business days. Its precise meaning is dependent on the context in which this element is used. */
UnderlyingPhysicalSettlMaximumBusinessDays         = 42063, /*!< 42063 (int FIX.5.0SP2) A maximum number of business days. Its precise meaning is dependent on the context in which this element is used. Intended to be used to limit a particular ISDA fallback provision.*/
UnderlyingPhysicalSettlTermXID                     = 42064, /*!< 42064 (XID FIX.5.0SP2) A named string value referenced by UnderlyingSettlementTermXIDRef(41315).*/
NoUnderlyingPhysicalSettlDeliverableObligations    = 42065, /*!< 42065 (NumInGroup FIX.5.0SP2) Number of entries in the repeating group.*/
UnderlyingPhysicalSettlDeliverableObligationType   = 42066, /*!< 42066 (String FIX.5.0SP2) Specifies the type of delivery obligation applicable for physical settlement. 

See http://www.fixtradingcommunity.org/codelists#Deliverable_Obligation_Types for code list for applicable deliverable obligation types.*/
UnderlyingPhysicalSettlDeliverableObligationValue  = 42067, /*!< 42067 (String FIX.5.0SP2) Physical settlement delivery obligation value appropriate to UnderlyingPhysicalSettlDeliverableObligationType(42066). 

See http://www.fixtradingcommunity.org/codelists#Deliverable_Obligation_Types for applicable obligation type values.*/
NoUnderlyingProtectionTerms                        = 42068, /*!< 42068 (NumInGroup FIX.5.0SP2) Number of protection terms in the repeating group.*/
UnderlyingProtectionTermNotional                   = 42069, /*!< 42069 (Amt FIX.5.0SP2) The notional amount of protection coverage for a floating rate. */
UnderlyingProtectionTermCurrency                   = 42070, /*!< 42070 (Currency FIX.5.0SP2) The currency of UnderlyingProtectionTermNotional(42069). Uses ISO 4217 currency codes.*/
UnderlyingProtectionTermSellerNotifies             = 42071, /*!< 42071 (Boolean FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring.

UnderlyingProtectionTermSellerNotifies(42071)=Y indicates that the seller notifies.

*/
UnderlyingProtectionTermBuyerNotifies              = 42072, /*!< 42072 (Boolean FIX.5.0SP2) The notifying party is the party that notifies the other party when a credit event has occurred by means of a credit event notice. If more than one party is referenced as being the notifying party then either party may notify the other of a credit event occurring.

UnderlyingProtectionTermBuyerNotifies(42072)=Y indicates that the buyer notifies.*/
UnderlyingProtectionTermEventBusinessCenter        = 42073, /*!< 42073 (String FIX.5.0SP2) When used, the business center indicates the local time of the business center that replaces the Greenwich Mean Time in Section 3.3 of the 2003 ISDA Credit Derivatives Definitions. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingProtectionTermStandardSources            = 42074, /*!< 42074 (Boolean FIX.5.0SP2) Indicates whether ISDA defined Standard Public Sources are applicable (UnderlyingProtectionTermStandardSources(42074)=Y) or not.*/
UnderlyingProtectionTermEventMinimumSources        = 42075, /*!< 42075 (int FIX.5.0SP2) The minimum number of the specified public information sources that must publish information that reasonably confirms that a credit event has occurred. The market convention is two. */
UnderlyingProtectionTermXID                        = 42076, /*!< 42076 (XID FIX.5.0SP2) A named string value referenced by UnderlyingProtectionTermXIDRef(41314).*/
NoUnderlyingProtectionTermEvents                   = 42077, /*!< 42077 (NumInGroup FIX.5.0SP2) Number of protection term events in the repeating group.*/
UnderlyingProtectionTermEventType                  = 42078, /*!< 42078 (String FIX.5.0SP2) Specifies the type of credit event applicable to the protection terms. 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Event_Types for code list of applicable event types.*/
UnderlyingProtectionTermEventValue                 = 42079, /*!< 42079 (String FIX.5.0SP2) Protection term event value appropriate to UnderlyingProtectionTermEventType(42078). 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Event_Types for applicable event type values.*/
UnderlyingProtectionTermEventCurrency              = 42080, /*!< 42080 (Currency FIX.5.0SP2) Applicable currency if UnderlyingProtectionTermEventValue(42079) is an amount. Uses ISO 4217 currency codes.*/
UnderlyingProtectionTermEventPeriod                = 42081, /*!< 42081 (int FIX.5.0SP2) Time unit multiplier for protection term events.*/
UnderlyingProtectionTermEventUnit                  = 42082, /*!< 42082 (String FIX.5.0SP2) Time unit associated with protection term events.*/
UnderlyingProtectionTermEventDayType               = 42083, /*!< 42083 (int FIX.5.0SP2) Day type for events that specify a period and unit.*/
UnderlyingProtectionTermEventRateSource            = 42084, /*!< 42084 (String FIX.5.0SP2) Rate source for events that specify a rate source, e.g. Floating rate interest shortfall.*/
NoUnderlyingProtectionTermEventQualifiers          = 42085, /*!< 42085 (NumInGroup FIX.5.0SP2) Number of qualifiers in the repeating group.*/
UnderlyingProtectionTermEventQualifier             = 42086, /*!< 42086 (char FIX.5.0SP2) Protection term event qualifier. Used to further qualify UnderlyingProtectionTermEventType(43078).*/
NoUnderlyingProtectionTermObligations              = 42087, /*!< 42087 (NumInGroup FIX.5.0SP2) Number of obligations in the repeating group.*/
UnderlyingProtectionTermObligationType             = 42088, /*!< 42088 (String FIX.5.0SP2) Specifies the type of obligation applicable to the protection terms. 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Obligation_Types for code list of applicable obligation types.*/
UnderlyingProtectionTermObligationValue            = 42089, /*!< 42089 (String FIX.5.0SP2) Protection term obligation value appropriate to UnderlyingProtectionTermObligationType(42088). 

See http://www.fixtradingcommunity.org/codelists#Protection_Term_Obligation_Types for applicable obligation type values.*/
NoUnderlyingProtectionTermEventNewsSources         = 42090, /*!< 42090 (NumInGroup FIX.5.0SP2) Number of event news sources in the repeating group.*/
UnderlyingProtectionTermEventNewsSource            = 42091, /*!< 42091 (String FIX.5.0SP2) Newspaper or electronic news service or source that may publish relevant information used in the determination of whether or not a credit event has occurred.*/
UnderlyingProvisionCashSettlPaymentDateBusinessDayConvention = 42092, /*!< 42092 (int FIX.5.0SP2) The business day convention used to adjust the provisional cash settlement payment's termination, or relative termination, date.  Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingProvisionCashSettlPaymentDateRelativeTo  = 42093, /*!< 42093 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingProvisionCashSettlPaymentDateOffsetPeriod = 42094, /*!< 42094 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement payment date offset.*/
UnderlyingProvisionCashSettlPaymentDateOffsetUnit  = 42095, /*!< 42095 (String FIX.5.0SP2) Time unit associated with the relative cash settlement payment date offset.*/
UnderlyingProvisionCashSettlPaymentDateOffsetDayType = 42096, /*!< 42096 (int FIX.5.0SP2) Specifies the day type of the provision's relative cash settlement payment date offset.*/
UnderlyingProvisionCashSettlPaymentDateRangeFirst  = 42097, /*!< 42097 (LocalMktDate FIX.5.0SP2) First date in range when a settlement date range is provided.*/
UnderlyingProvisionCashSettlPaymentDateRangeLast   = 42098, /*!< 42098 (LocalMktDate FIX.5.0SP2) Last date in range when a settlement date range is provided.*/
NoUnderlyingProvisionCashSettlPaymentDates         = 42099, /*!< 42099 (NumInGroup FIX.5.0SP2) Number of UnderlyingProvision cash settlement payment dates in the repeating group.*/
UnderlyingProvisionCashSettlPaymentDate            = 42100, /*!< 42100 (LocalMktDate FIX.5.0SP2) The cash settlement payment date, unadjusted or adjusted depending on UnderlyingProvisionCashSettlPaymentDateType(42101).*/
UnderlyingProvisionCashSettlPaymentDateType        = 42101, /*!< 42101 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
UnderlyingProvisionCashSettlQuoteSource            = 42102, /*!< 42102 (int FIX.5.0SP2) Identifies the source of quote information.*/
UnderlyingProvisionCashSettlQuoteReferencePage     = 42103, /*!< 42103 (String FIX.5.0SP2) Identifies the reference "page" from the quote source.*/
UnderlyingProvisionCashSettlValueTime              = 42104, /*!< 42104 (LocalMktTime FIX.5.0SP2) A time specified in 24-hour format, e.g. 11am would be represented as 11:00:00. The time of the cash settlement valuation date when the cash settlement amount will be determined according to the cash settlement method if the parties have not otherwise been able to agree to the cash settlement amount.*/
UnderlyingProvisionCashSettlValueTimeBusinessCenter = 42105, /*!< 42105 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's cash settlement valuation time.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingProvisionCashSettlValueDateBusinessDayConvention = 42106, /*!< 42106 (int FIX.5.0SP2) The business day convention used to adjust the cash settlement valuation date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingProvisionCashSettlValueDateRelativeTo    = 42107, /*!< 42107 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement value date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingProvisionCashSettlValueDateOffsetPeriod  = 42108, /*!< 42108 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement value date offset.*/
UnderlyingProvisionCashSettlValueDateOffsetUnit    = 42109, /*!< 42109 (String FIX.5.0SP2) Time unit associated with the relative cash settlement value date offset.*/
UnderlyingProvisionCashSettlValueDateOffsetDayType = 42110, /*!< 42110 (int FIX.5.0SP2) Specifies the day type of the provision's relative cash settlement value date offset.*/
UnderlyingProvisionCashSettlValueDateAdjusted      = 42111, /*!< 42111 (LocalMktDate FIX.5.0SP2) The adjusted cash settlement value date.*/
NoUnderlyingProvisionOptionExerciseFixedDates      = 42112, /*!< 42112 (NumInGroup FIX.5.0SP2) Number of UnderlyingProvision option exercise fixed dates in the repeating group.*/
UnderlyingProvisionOptionExerciseFixedDate         = 42113, /*!< 42113 (LocalMktDate FIX.5.0SP2) A predetermined option exercise date, unadjusted or adjusted depending on UnderlyingProvisionOptionExerciseFixedDateType(42114).*/
UnderlyingProvisionOptionExerciseFixedDateType     = 42114, /*!< 42114 (int FIX.5.0SP2) Specifies the type of date (e.g. adjusted for holidays).*/
UnderlyingProvisionOptionExerciseBusinessDayConvention = 42115, /*!< 42115 (int FIX.5.0SP2) The business day convention used to adjust the underlying instrument's provision's option exercise date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingProvisionOptionExerciseEarliestDateOffsetPeriod = 42116, /*!< 42116 (int FIX.5.0SP2) Time unit multiplier for  the interval to the first (and possibly only) exercise date in the exercise period.*/
UnderlyingProvisionOptionExerciseEarliestDateOffsetUnit = 42117, /*!< 42117 (String FIX.5.0SP2) Time unit associated with the interval to the first (and possibly only) exercise date in the exercise period.*/
UnderlyingProvisionOptionExerciseFrequencyPeriod   = 42118, /*!< 42118 (int FIX.5.0SP2) Time unit multiplier for the frequency of subsequent exercise dates in the exercise period following the earliest exercise date. An interval of 1 day should be used to indicate an American style exercise frequency.*/
UnderlyingProvisionOptionExerciseFrequencyUnit     = 42119, /*!< 42119 (String FIX.5.0SP2) Time unit associated with the frequency of subsequent exercise dates in the exercise period following the earliest exercise date.*/
UnderlyingProvisionOptionExerciseStartDateUnadjusted = 42120, /*!< 42120 (LocalMktDate FIX.5.0SP2) The unadjusted first day of the exercise period for an American style option.*/
UnderlyingProvisionOptionExerciseStartDateRelativeTo = 42121, /*!< 42121 (int FIX.5.0SP2) Specifies the anchor date when the option exercise start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingProvisionOptionExerciseStartDateOffsetPeriod = 42122, /*!< 42122 (int FIX.5.0SP2) Time unit multiplier for the relative option exercise start date offset.*/
UnderlyingProvisionOptionExerciseStartDateOffsetUnit = 42123, /*!< 42123 (String FIX.5.0SP2) Time unit associated with the relative option exercise start date offset.*/
UnderlyingProvisionOptionExerciseStartDateOffsetDayType = 42124, /*!< 42124 (int FIX.5.0SP2) Specifies the day type of the provision's relative option exercise start date offset.*/
UnderlyingProvisionOptionExerciseStartDateAdjusted = 42125, /*!< 42125 (LocalMktDate FIX.5.0SP2) The adjusted first day of the exercise period for an American style option.*/
UnderlyingProvisionOptionExercisePeriodSkip        = 42126, /*!< 42126 (int FIX.5.0SP2) The number of periods in the referenced date schedule that are between each date in the relative date schedule. Thus a skip of 2 would mean that dates are relative to every second date in the referenced schedule. If present this should have a value greater than 1.*/
UnderlyingProvisionOptionExerciseBoundsFirstDateUnadjusted = 42127, /*!< 42127 (LocalMktDate FIX.5.0SP2) The unadjusted first date of a schedule. This can be used to restrict the range of exercise dates when they are relative.*/
UnderlyingProvisionOptionExerciseBoundsLastDateUnadjusted = 42128, /*!< 42128 (LocalMktDate FIX.5.0SP2) The unadjusted last date of a schedule. This can be used to restrict the range of exercise dates when they are relative.*/
UnderlyingProvisionOptionExerciseEarliestTime      = 42129, /*!< 42129 (LocalMktTime FIX.5.0SP2) The earliest time at which notice of exercise can be given by the buyer to the seller (or seller's agent) i) on the expriation date, in the case of a European style option, (ii) on each bermuda option exercise date and the expiration date, in the case of a Bermuda style option the commencement date to, and including, the expiration date, in the case of an American option.*/
UnderlyingProvisionOptionExerciseEarliestTimeBusinessCenter = 42130, /*!< 42130 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's earliest time for notice of exercise. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingProvisionOptionExerciseLatestTime        = 42131, /*!< 42131 (LocalMktTime FIX.5.0SP2) For a Bermuda or American style option, the latest time on an exercise business day (excluding the expiration date) within the exercise period that notice can be given by the buyer to the seller or seller's agent. Notice of exercise given after this time will be deemed to have been given on the next exercise business day.*/
UnderlyingProvisionOptionExerciseLatestTimeBusinessCenter = 42132, /*!< 42132 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's latest time for notice of exercise.

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingProvisionOptionExpirationDateUnadjusted  = 42133, /*!< 42133 (LocalMktDate FIX.5.0SP2) The unadjusted last day within an exercise period for an American style option. For a European style option it is the only day within the exercise period.*/
UnderlyingProvisionOptionExpirationDateBusinessDayConvention = 42134, /*!< 42134 (int FIX.5.0SP2) The business day convention used to adjust the underlying instrument's provision's option expiration date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingProvisionOptionExpirationDateRelativeTo  = 42135, /*!< 42135 (int FIX.5.0SP2) Specifies the anchor date when the option expiration date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingProvisionOptionExpirationDateOffsetPeriod = 42136, /*!< 42136 (int FIX.5.0SP2) Time unit multiplier for the relative option expiration date offset.*/
UnderlyingProvisionOptionExpirationDateOffsetUnit  = 42137, /*!< 42137 (String FIX.5.0SP2) Time unit associated with the relative option expiration date offset.*/
UnderlyingProvisionOptionExpirationDateOffsetDayType = 42138, /*!< 42138 (int FIX.5.0SP2) Specifies the day type of the provision's relative option expiration date offset.*/
UnderlyingProvisionOptionExpirationDateAdjusted    = 42139, /*!< 42139 (LocalMktDate FIX.5.0SP2) The adjusted last date within an exercise period for an American style option. For a European style option it is the only date within the exercise period.*/
UnderlyingProvisionOptionExpirationTime            = 42140, /*!< 42140 (LocalMktTime FIX.5.0SP2) The latest time for exercise on the expiration date.*/
UnderlyingProvisionOptionExpirationTimeBusinessCenter = 42141, /*!< 42141 (String FIX.5.0SP2) Identifies the business center calendar used with the provision's latest exercise time on expiration date. 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingProvisionOptionRelevantUnderlyingDateUnadjusted = 42142, /*!< 42142 (LocalMktDate FIX.5.0SP2) The unadjusted date on the underlying set by the exercise of an option. What this date is depends on the option (e.g. in a swaption it is the swap effective date, in an extendible/cancelable provision it is the swap termination date).*/
UnderlyingProvisionOptionRelevantUnderlyingDateBusinessDayConvention = 42143, /*!< 42143 (int FIX.5.0SP2) The business day convnetion used to adjust the underlying instrument provision's option underlying date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingProvisionOptionRelevantUnderlyingDateRelativeTo = 42144, /*!< 42144 (int FIX.5.0SP2) Specifies the anchor date when the date relevant to the underlying trade on exercise is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingProvisionOptionRelevantUnderlyingDateOffsetPeriod = 42145, /*!< 42145 (int FIX.5.0SP2) Time unit multiplier for the relative option relevant underlying date offset.*/
UnderlyingProvisionOptionRelevantUnderlyingDateOffsetUnit = 42146, /*!< 42146 (String FIX.5.0SP2) Time unit associated with the relative option relevant underlying date offset.*/
UnderlyingProvisionOptionRelevantUnderlyingDateOffsetDayType = 42147, /*!< 42147 (int FIX.5.0SP2) Specifies the day type of the provision's relative option relevant underlying date offset.*/
UnderlyingProvisionOptionRelevantUnderlyingDateAdjusted = 42148, /*!< 42148 (LocalMktDate FIX.5.0SP2) The adjusted date on the underlying set by the exercise of an option. What this date is depends on the option (e.g. in a swaption it is the swap effective date, in an extendible/cancelable provision it is the swap termination date).*/
NoUnderlyingProvisions                             = 42149, /*!< 42149 (NumInGroup FIX.5.0SP2) Number of provisions in the repeating group.*/
UnderlyingProvisionType                            = 42150, /*!< 42150 (int FIX.5.0SP2) Type of provision.*/
UnderlyingProvisionDateUnadjusted                  = 42151, /*!< 42151 (LocalMktDate FIX.5.0SP2) The unadjusted date of the provision.*/
UnderlyingProvisionDateBusinessDayConvention       = 42152, /*!< 42152 (int FIX.5.0SP2) The business day convention used to adjust the underlying instrument's provision's date. Used only to override the business day convention specified in the UnderlyingDateAdjustment component within the UnderlyingInstrument component.*/
UnderlyingProvisionDateAdjusted                    = 42153, /*!< 42153 (LocalMktDate FIX.5.0SP2) The adjusted date of the provision.*/
UnderlyingProvisionDateTenorPeriod                 = 42154, /*!< 42154 (int FIX.5.0SP2) Time unit multiplier for the provision's tenor period.*/
UnderlyingProvisionDateTenorUnit                   = 42155, /*!< 42155 (String FIX.5.0SP2) Time unit associated with the provision's tenor period.*/
UnderlyingProvisionCalculationAgent                = 42156, /*!< 42156 (int FIX.5.0SP2) Used to identify the calculation agent.  The calculation agent may be identified in UnderlyingProvisionCalculationAgent(42156) or in the underlying provision parties component.*/
UnderlyingProvisionOptionSinglePartyBuyerSide      = 42157, /*!< 42157 (int FIX.5.0SP2) If optional early termination is not available to both parties then this component identifies the buyer of the option through its side of the trade.*/
UnderlyingProvisionOptionSinglePartySellerSide     = 42158, /*!< 42158 (int FIX.5.0SP2) If optional early termination is not available to both parties then this component identifies the seller of the option through its side of the trade.*/
UnderlyingProvisionOptionExerciseStyle             = 42159, /*!< 42159 (int FIX.5.0SP2) The instrument provision's exercise style.*/
UnderlyingProvisionOptionExerciseMultipleNotional  = 42160, /*!< 42160 (Amt FIX.5.0SP2) A notional amount which restricts the amount of notional that can be exercised when partial exercise or multiple exercise is applicable. The integral multiple amount defines a lower limit of notional that can be exercised and also defines a unit multiple of notional that can be exercised, i.e. only integer multiples of this amount can be exercised.*/
UnderlyingProvisionOptionExerciseMinimumNotional   = 42161, /*!< 42161 (Amt FIX.5.0SP2) The minimum notional amount that can be exercised on a given exercise date.*/
UnderlyingProvisionOptionExerciseMaximumNotional   = 42162, /*!< 42162 (Amt FIX.5.0SP2) The maximum notional amount that can be exercised on a given exercise date.*/
UnderlyingProvisionOptionMinimumNumber             = 42163, /*!< 42163 (int FIX.5.0SP2) The minimum number of options that can be exercised on a given exercise date.*/
UnderlyingProvisionOptionMaximumNumber             = 42164, /*!< 42164 (int FIX.5.0SP2) The maximum number of options that can be exercised on a given exercise date. If the number is not specified, it means that the maximum number of options corresponds to the remaining unexercised options.*/
UnderlyingProvisionOptionExerciseConfirmation      = 42165, /*!< 42165 (Boolean FIX.5.0SP2) Used to indicate whether follow-up confirmation of exercise (written or electronic) is required following telephonic notice by the buyer to the seller or seller's agent.*/
UnderlyingProvisionCashSettlMethod                 = 42166, /*!< 42166 (int FIX.5.0SP2) 	An ISDA defined cash settlement method used for the determination of the applicable cash settlement amount. The method is defined in the 2006 ISDA Definitions, Section 18.3. Cash Settlement Methods, paragraph (e).*/
UnderlyingProvisionCashSettlCurrency               = 42167, /*!< 42167 (Currency FIX.5.0SP2) Specifies the currency of settlement. Uses ISO 4217 currency codes.*/
UnderlyingProvisionCashSettlCurrency2              = 42168, /*!< 42168 (Currency FIX.5.0SP2) Specifies the currency of settlement for a cross-currency provision.  Uses ISO 4217 currency codes.*/
UnderlyingProvisionCashSettlQuoteType              = 42169, /*!< 42169 (int FIX.5.0SP2) Identifies the type of quote to be used.*/
UnderlyingProvisionText                            = 42170, /*!< 42170 (String FIX.5.0SP2) Free form text to specify additional information or enumeration description when a standard value does not apply.*/
EncodedUnderlyingProvisionTextLen                  = 42171, /*!< 42171 (Length FIX.5.0SP2) Byte length of encoded (non-ASCII characters) EncodedUnderlyingProvisionText(42712) field.*/
EncodedUnderlyingProvisionText                     = 42172, /*!< 42172 (data FIX.5.0SP2) Encoded (non-ASCII characters) representation of the UnderlyingProvisionText(42170) field in the encoded format specified via the MessageEncoding (347) field. If used, the ASCII (English) representation should also be specified in the UnderlyingProvisionText(42170) field.*/
NoUnderlyingProvisionPartyIDs                      = 42173, /*!< 42173 (NumInGroup FIX.5.0SP2) Number of parties identified in the contract provision.*/
UnderlyingProvisionPartyID                         = 42174, /*!< 42174 (String FIX.5.0SP2) The party identifier for the payment settlement party. */
UnderlyingProvisionPartyIDSource                   = 42175, /*!< 42175 (char FIX.5.0SP2) Identifies the class or source of the UnderlyingProvisionPartyID(42174) value. */
UnderlyingProvisionPartyRole                       = 42176, /*!< 42176 (int FIX.5.0SP2) Identifies the type or role of UnderlyingProvisionPartyID(42174) specified.*/
NoUnderlyingProvisionPartySubIDs                   = 42177, /*!< 42177 (NumInGroup FIX.5.0SP2) Number of sub-party IDs to be reported for the party.*/
UnderlyingProvisionPartySubID                      = 42178, /*!< 42178 (String FIX.5.0SP2) Underlying provision party sub-identifier, if applicable for UnderlyingProvisionPartyID(42174). */
UnderlyingProvisionPartySubIDType                  = 42179, /*!< 42179 (int FIX.5.0SP2) The type of UnderlyingProvisionPartySubID(42178).*/
NoUnderlyingProvisionCashSettlPaymentDateBusinessCenters = 42180, /*!< 42180 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingProvisionCashSettlPaymentDateBusinessCenter = 42181, /*!< 42181 (String FIX.5.0SP2) The business center calendar used to adjust the provision's cash settlement payment's termination, or relative termination, date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingProvisionCashSettlValueDateBusinessCenters = 42182, /*!< 42182 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingProvisionCashSettlValueDateBusinessCenter = 42183, /*!< 42183 (String FIX.5.0SP2) The business center calendar used to adjust the cash settlement valuation date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingProvisionOptionExerciseBusinessCenters = 42184, /*!< 42184 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingProvisionOptionExerciseBusinessCenter    = 42185, /*!< 42185 (String FIX.5.0SP2) The business center calendar used to adjust the underlying instrument's provision's option exercise date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingProvisionOptionExpirationDateBusinessCenters = 42186, /*!< 42186 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingProvisionOptionExpirationDateBusinessCenter = 42187, /*!< 42187 (String FIX.5.0SP2) The business center calendar used to adjust the underlying instrument's provision's option expiration date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingProvisionOptionRelevantUnderlyingDateBusinessCenters = 42188, /*!< 42188 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingProvisionOptionRelevantUnderlyingDateBusinessCenter = 42189, /*!< 42189 (String FIX.5.0SP2) The business center calendar used to adjust the underlying instrument's provision's option underlying date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingProvisionDateBusinessCenters           = 42190, /*!< 42190 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingProvisionDateBusinessCenter              = 42191, /*!< 42191 (String FIX.5.0SP2) The business center calendar used to adjust the underlying instrument's provision's date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
DeliveryStreamDeliveryPointSource                  = 42192, /*!< 42192 (int FIX.5.0SP2) Identifies the class or source of DeliveryStreamDeliveryPoint(41062).*/
DeliveryStreamDeliveryPointDesc                    = 42193, /*!< 42193 (String FIX.5.0SP2) Description of the delivery point identified in DeliveryStreamDeliveryPoint(41062).*/
LegDeliveryStreamDeliveryPointSource               = 42194, /*!< 42194 (int FIX.5.0SP2) Identifies the class or source of LegDeliveryStreamDeliveryPoint(41433).*/
LegDeliveryStreamDeliveryPointDesc                 = 42195, /*!< 42195 (String FIX.5.0SP2) Description of the delivery point identified in LegDeliveryStreamDeliveryPoint(41433).*/
UnderlyingDeliveryStreamDeliveryPointSource        = 42196, /*!< 42196 (int FIX.5.0SP2) Identifies the class or source of UnderlyingDeliveryStreamDeliveryPoint(41781).*/
UnderlyingDeliveryStreamDeliveryPointDesc          = 42197, /*!< 42197 (String FIX.5.0SP2) Description of the delivery point identified in UnderlyingDeliveryStreamDeliveryPoint(41781).*/
NoLegContractualDefinitions                        = 42198, /*!< 42198 (NumInGroup FIX.5.0SP2) Number of financing definitions in the repeating group.*/
LegContractualDefinition                           = 42199, /*!< 42199 (String FIX.5.0SP2) Specifies which contract definition, such as those published by ISDA, will apply for the terms of the trade. See http://www.fpml.org/coding-scheme/contractual-definitions for values.*/
NoLegFinancingTermSupplements                      = 42200, /*!< 42200 (NumInGroup FIX.5.0SP2) Number of financing terms supplements in the repeating group.*/
LegFinancingTermSupplementDesc                     = 42201, /*!< 42201 (String FIX.5.0SP2) Identifies the applicable contractual supplement. See http://www.fpml.org/coding-scheme/contractual-supplement for values.*/
LegFinancingTermSupplementDate                     = 42202, /*!< 42202 (LocalMktDate FIX.5.0SP2) Specifies the publication date of the applicable version of the contractual supplement.*/
NoLegContractualMatrices                           = 42203, /*!< 42203 (NumInGroup FIX.5.0SP2) Number of contractual matrices in the repeating group.*/
LegContractualMatrixSource                         = 42204, /*!< 42204 (String FIX.5.0SP2) Identifies the applicable contract matrix. See http://www.fpml.org/coding-scheme/matrix-type-1-0.xml for values.*/
LegContractualMatrixDate                           = 42205, /*!< 42205 (LocalMktDate FIX.5.0SP2) Specifies the publication date of the applicable version of the contract matrix. If not specified, the ISDA Standard Terms Supplement defines rules for which version of the matrix is applicable.*/
LegContractualMatrixTerm                           = 42206, /*!< 42206 (String FIX.5.0SP2) Specifies the applicable key into the relevent contract matrix. In the case of 2000 ISDA Definitions Settlement Matrix for Early Termination and Swaptions, the LegContractualMatrixTerm(42206) is not applicable and is to be omitted. See http://www.fpml.org/coding-scheme/credit-matrix-transaction-type for values.*/
CashSettlDateUnadjusted                            = 42207, /*!< 42207 (LocalMktDate FIX.5.0SP2) The unadjusted cash settlement date.*/
CashSettlDateBusinessDayConvention                 = 42208, /*!< 42208 (int FIX.5.0SP2) The business day convention used to adjust the cash settlement provision's date. Used only to override the business day convention defined in the Instrument component.*/
CashSettlDateRelativeTo                            = 42209, /*!< 42209 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
CashSettlDateOffsetPeriod                          = 42210, /*!< 42210 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement date offset.*/
CashSettlDateOffsetUnit                            = 42211, /*!< 42211 (String FIX.5.0SP2) Time unit associated with the relative cash settlement date offset.*/
CashSettlDateOffsetDayType                         = 42212, /*!< 42212 (int FIX.5.0SP2) Specifies the day type of the relative cash settlement date offset.*/
CashSettlDateAdjusted                              = 42213, /*!< 42213 (LocalMktDate FIX.5.0SP2) The adjusted cash settlement date.*/
NoCashSettlDateBusinessCenters                     = 42214, /*!< 42214 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
CashSettlDateBusinessCenter                        = 42215, /*!< 42215 (String FIX.5.0SP2) The business center calendar used for date adjustment of the cash settlement unadjusted or relative date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
CashSettlPriceSource                               = 42216, /*!< 42216 (String FIX.5.0SP2) The source from which the settlement price is to be obtained.

See http://www.fpml.org/coding-scheme/settlement-price-source for values.*/
CashSettlPriceDefault                              = 42217, /*!< 42217 (int FIX.5.0SP2) The default election for determining settlement price.*/
DividendFloatingRateIndex                          = 42218, /*!< 42218 (String FIX.5.0SP2) The dividend accrual floating rate index.*/
DividendFloatingRateIndexCurvePeriod               = 42219, /*!< 42219 (int FIX.5.0SP2) Time unit multiplier for the dividend accrual floating rate index curve.*/
DividendFloatingRateIndexCurveUnit                 = 42220, /*!< 42220 (String FIX.5.0SP2) Time unit associated with the dividend accrual floating rate index curve period.

*/
DividendFloatingRateMultiplier                     = 42221, /*!< 42221 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This should only be included if the multiplier is not equal to 1 (one) for the term of the contract.*/
DividendFloatingRateSpread                         = 42222, /*!< 42222 (PriceOffset FIX.5.0SP2) The basis points spread from the index specified in DividendFloatingRateIndex(42218).*/
DividendFloatingRateSpreadPositionType             = 42223, /*!< 42223 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
DividendFloatingRateTreatment                      = 42224, /*!< 42224 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
DividendCapRate                                    = 42225, /*!< 42225 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. It is only required where the floating rate is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as "0.05".*/
DividendCapRateBuySide                             = 42226, /*!< 42226 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
DividendCapRateSellSide                            = 42227, /*!< 42227 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
DividendFloorRate                                  = 42228, /*!< 42228 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate. The rate is expressed as a decimal, e.g. 5% is represented as "0.05".*/
DividendFloorRateBuySide                           = 42229, /*!< 42229 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
DividendFloorRateSellSide                          = 42230, /*!< 42230 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
DividendInitialRate                                = 42231, /*!< 42231 (Percentage FIX.5.0SP2) The initial floating rate reset agreed between the principal parties involved in the trade. This is assumed to be the first required reset rate for the first regular calculation period. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. The initial rate is expressed in decimal form, e.g. 5% is represented as "0.05".*/
DividendFinalRateRoundingDirection                 = 42232, /*!< 42232 (char FIX.5.0SP2) Specifies the rounding direction of the final rate.*/
DividendFinalRatePrecision                         = 42233, /*!< 42233 (int FIX.5.0SP2) Specifies the rounding precision of the final rate in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
DividendAveragingMethod                            = 42234, /*!< 42234 (int FIX.5.0SP2) When averaging is applicable, used to specify whether a weighted or unweighted average method of calculation is to be used. */
DividendNegativeRateTreatment                      = 42235, /*!< 42235 (int FIX.5.0SP2) The specification of any provisions for calculating payment obligations when a floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
NoDividendAccrualPaymentDateBusinessCenters        = 42236, /*!< 42236 (NumInGroup FIX.5.0SP2) Number of entries in the DividendAccrualPaymentDateBusinessCenterGrp.*/
DividendAccrualPaymentDateBusinessCenter           = 42237, /*!< 42237 (String FIX.5.0SP2) The business center calendar used for date adjustment of the  instrument's dividend accrual payment date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
DividendAccrualPaymentDateRelativeTo               = 42238, /*!< 42238 (int FIX.5.0SP2) Specifies the anchor date when the accrual payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
DividendAccrualPaymentDateOffsetPeriod             = 42239, /*!< 42239 (int FIX.5.0SP2) Time unit multiplier for the relative accrual payment date offset.*/
DividendAccrualPaymentDateOffsetUnit               = 42240, /*!< 42240 (String FIX.5.0SP2) Time unit associated with the relative accrual payment date offset.*/
DividendAccrualPaymentDateOffsetDayType            = 42241, /*!< 42241 (int FIX.5.0SP2) Specifies the day type of the relative accrual payment date offset.*/
DividendAccrualPaymentDateUnadjusted               = 42242, /*!< 42242 (LocalMktDate FIX.5.0SP2) The unadjusted accrual payment date.*/
DividendAccrualPaymeentDateBusinessDayConvention   = 42243, /*!< 42243 (int FIX.5.0SP2) Accrual payment date adjustment business day convention.*/
DividendAccrualPaymentDateAdjusted                 = 42244, /*!< 42244 (LocalMktDate FIX.5.0SP2) The adjusted accrual payment date.*/
DividendReinvestmentIndicator                      = 42245, /*!< 42245 (Boolean FIX.5.0SP2) Indicates whether the dividend will be reinvested.*/
DividendEntitlementEvent                           = 42246, /*!< 42246 (int FIX.5.0SP2) Defines the contract event which the receiver of the derivative is entitled to the dividend.*/
DividendAmountType                                 = 42247, /*!< 42247 (int FIX.5.0SP2) Indicates how the gross cash dividend amount per share is determined.*/
DividendUnderlierRefID                             = 42248, /*!< 42248 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
ExtraordinaryDividendPartySide                     = 42249, /*!< 42249 (int FIX.5.0SP2) Reference to the party through its side in the trade who makes the determination whether dividends are extraordinary in relation to normal levels.*/
ExtraordinaryDividendAmountType                    = 42250, /*!< 42250 (int FIX.5.0SP2) Indicates how the extraordinary gross cash dividend per share is determined.*/
ExtraordinaryDividendCurrency                      = 42251, /*!< 42251 (Currency FIX.5.0SP2) The currency in which the excess dividend is denominated. Uses ISO 4217 currency codes.*/
ExtraordinaryDividendDeterminationMethod           = 42252, /*!< 42252 (String FIX.5.0SP2) Specifies the method in which the excess amount is determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
DividendAccrualFixedRate                           = 42253, /*!< 42253 (Percentage FIX.5.0SP2) The dividend accrual fixed rate per annum expressed as a decimal. 

A value of 5% would be represented as "0.05".*/
DividendCompoundingMethod                          = 42254, /*!< 42254 (int FIX.5.0SP2) The compounding method to be used when more than one dividend period contributes to a single payment.*/
DividendNumOfIndexUnits                            = 42255, /*!< 42255 (int FIX.5.0SP2) The number of index units applicable to dividends.*/
DividendCashPercentage                             = 42256, /*!< 42256 (Percentage FIX.5.0SP2) Declared cash dividend percentage. 

A value of 5% would be represented as "0.05".*/
DividendCashEquivalentPercentage                   = 42257, /*!< 42257 (Percentage FIX.5.0SP2) Declared cash-equivalent dividend percentage. 

A value of 5% would be represented as "0.05".*/
NonCashDividendTreatment                           = 42258, /*!< 42258 (int FIX.5.0SP2) Defines the treatment of non-cash dividends.*/
DividendComposition                                = 42259, /*!< 42259 (int FIX.5.0SP2) Defines how the composition of dividends is to be determined.*/
SpecialDividendsIndicator                          = 42260, /*!< 42260 (Boolean FIX.5.0SP2) Indicates whether special dividends are applicable.*/
MaterialDividendsIndicator                         = 42261, /*!< 42261 (Boolean FIX.5.0SP2) Indicates whether material non-cash dividends are applicable.*/
OptionsExchangeDividendsIndicator                  = 42262, /*!< 42262 (Boolean FIX.5.0SP2) Indicates whether option exchange dividends are applicable.*/
AdditionalDividendsIndicator                       = 42263, /*!< 42263 (Boolean FIX.5.0SP2) Indicates whether additional dividends are applicable.*/
AllDividendsIndicator                              = 42264, /*!< 42264 (Boolean FIX.5.0SP2) Represents the European Master Confirmation value of 'All Dividends' which, when applicable, signifies that, for a given Ex-Date, the daily observed share price for that day is adjusted (reduced) by the cash dividend and/or the cash value of any non-cash dividend per share (including extraordinary dividends) declared by the issuer.*/
DividendFXTriggerDateRelativeTo                    = 42265, /*!< 42265 (int FIX.5.0SP2) Specifies the anchor date when the FX trigger date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
DividendFXTriggerDateOffsetPeriod                  = 42266, /*!< 42266 (int FIX.5.0SP2) Time unit multiplier for the relative FX trigger date offset.*/
DividendFXTriggerDateOffsetUnit                    = 42267, /*!< 42267 (String FIX.5.0SP2) Time unit associated with the relative FX trigger date offset.*/
DividendFXTriggerDateOffsetDayType                 = 42268, /*!< 42268 (int FIX.5.0SP2) Specifies the day type of the relative FX trigger date offset.*/
DividendFXTriggerDateUnadjusted                    = 42269, /*!< 42269 (LocalMktDate FIX.5.0SP2) The unadjusted FX trigger date.*/
DividendFXTriggerDateBusinessDayConvention         = 42270, /*!< 42270 (int FIX.5.0SP2) The business day convention used for the FX trigger date adjustment.*/
DividendFXTriggerDateAdjusted                      = 42271, /*!< 42271 (LocalMktDate FIX.5.0SP2) The adjusted FX trigger date.*/
NoDividendFXTriggerDateBusinessCenters             = 42272, /*!< 42272 (NumInGroup FIX.5.0SP2) Number of entries in the DividendFXTriggerDateBusinessCenterGrp.*/
DividendFXTriggerDateBusinessCenter                = 42273, /*!< 42273 (String FIX.5.0SP2) The business center calendar used for date adjustment of the  instrument's FX trigger date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoDividendPeriods                                  = 42274, /*!< 42274 (NumInGroup FIX.5.0SP2) Number of entries in the DividendPeriodGrp component.*/
DividendPeriodSequence                             = 42275, /*!< 42275 (int FIX.5.0SP2) Defines the ordinal dividend period. E.g. 1 = First period, 2 = Second period, etc.*/
DividendPeriodStartDateUnadjusted                  = 42276, /*!< 42276 (LocalMktDate FIX.5.0SP2) The unadjusted date on which the dividend period will begin.*/
DividendPeriodEndDateUnadjusted                    = 42277, /*!< 42277 (LocalMktDate FIX.5.0SP2) The unadjusted date on which the dividend period will end.*/
DividendPeriodUnderlierRefID                       = 42278, /*!< 42278 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
DividendPeriodStrikePrice                          = 42279, /*!< 42279 (Price FIX.5.0SP2) Specifies the fixed strike price of the dividend period.*/
DividendPeriodBusinessDayConvention                = 42280, /*!< 42280 (int FIX.5.0SP2) The dividend period dates business day convention.*/
DividendPeriodValuationDateUnadjusted              = 42281, /*!< 42281 (LocalMktDate FIX.5.0SP2) The unadjusted dividend period valuation date.*/
DividendPeriodValuationDateRelativeTo              = 42282, /*!< 42282 (int FIX.5.0SP2) Specifies the anchor date when the dividend period valuation date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
DividendPeriodValuationDateOffsetPeriod            = 42283, /*!< 42283 (int FIX.5.0SP2) Time unit multiplier for the relative dividend period valuation date offset.*/
DividendPeriodValuationDateOffsetUnit              = 42284, /*!< 42284 (String FIX.5.0SP2) Time unit associated with the relative dividend period valuation date offset.*/
DividendPeriodValuationDateOffsetDayType           = 42285, /*!< 42285 (int FIX.5.0SP2) Specifies the day type of the relative dividend period valuation date offset.*/
DividendPeriodValuationDateAdjusted                = 42286, /*!< 42286 (LocalMktDate FIX.5.0SP2) The adjusted dividend period valuation date.*/
DividendPeriodPaymentDateUnadjusted                = 42287, /*!< 42287 (LocalMktDate FIX.5.0SP2) The unadjusted dividend period payment date.*/
DividendPeriodPaymentDateRelativeTo                = 42288, /*!< 42288 (int FIX.5.0SP2) Specifies the anchor date when the dividend period payment date is relative to an anchor date.  

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
DividendPeriodPaymentDateOffsetPeriod              = 42289, /*!< 42289 (int FIX.5.0SP2) Time unit multiplier for the relative dividend period payment date offset.*/
DividendPeriodPaymentDateOffsetUnit                = 42290, /*!< 42290 (String FIX.5.0SP2) Time unit associated with the relative dividend period payment date offset.*/
DividendPeriodPaymentDateOffsetDayType             = 42291, /*!< 42291 (int FIX.5.0SP2) Specifies the day type of the relative dividend period payment date offset.*/
DividendPeriodPaymentDateAdjusted                  = 42292, /*!< 42292 (LocalMktDate FIX.5.0SP2) The adjusted dividend period payment date.*/
DividendPeriodXID                                  = 42293, /*!< 42293 (XID FIX.5.0SP2) Identifier for linking this stream dividend period to an underlier through an instance of RelatedInstrumentGrp.*/
NoDividendPeriodBusinessCenters                    = 42294, /*!< 42294 (NumInGroup FIX.5.0SP2) Number of entries in the DividendPeriodBusinessCenterGrp.*/
DividendPeriodBusinessCenter                       = 42295, /*!< 42295 (String FIX.5.0SP2) The business center calendar used for date adjustment of the instrument's dividend period date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoExtraordinaryEvents                              = 42296, /*!< 42296 (NumInGroup FIX.5.0SP2) Number of extraordinary events in the repeating group.*/
ExtraordinaryEventType                             = 42297, /*!< 42297 (String FIX.5.0SP2) Identifies the type of extraordinary or disruptive event applicable to the reference entity.

See http://www.fixtradingcommunity.org/codelists#Extraordinary_Event_Type for code list of extraordinary event types and values.*/
ExtraordinaryEventValue                            = 42298, /*!< 42298 (String FIX.5.0SP2) The extraordinary or disruptive event value appropriate to ExtraordinaryEventType(42297). 

See http://www.fixtradingcommunity.org/codelists#Extraordinary_Event_Type for code list of extraordinary event types and values.*/
LegCashSettlDateUnadjusted                         = 42299, /*!< 42299 (LocalMktDate FIX.5.0SP2) The unadjusted cash settlement date.*/
LegCashSettlDateBusinessDayConvention              = 42300, /*!< 42300 (int FIX.5.0SP2) The business day convention used to adjust the cash settlement provision's date.  Used only to override the business day convention defined in the InstrumentLeg component.*/
LegCashSettlDateRelativeTo                         = 42301, /*!< 42301 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegCashSettlDateOffsetPeriod                       = 42302, /*!< 42302 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement date offset.*/
LegCashSettlDateOffsetUnit                         = 42303, /*!< 42303 (String FIX.5.0SP2) Time unit associated with the relative cash settlement date offset.*/
LegCashSettlDateOffsetDayType                      = 42304, /*!< 42304 (int FIX.5.0SP2) Specifies the day type of the relative cash settlement date offset.*/
LegCashSettlDateAdjusted                           = 42305, /*!< 42305 (LocalMktDate FIX.5.0SP2) The adjusted cash settlement date.*/
NoLegCashSettlDateBusinessCenters                  = 42306, /*!< 42306 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegCashSettlDateBusinessCenter                     = 42307, /*!< 42307 (String FIX.5.0SP2) The business center calendar used for date adjustment of the cash settlement unadjusted or relative date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegCashSettlPriceSource                            = 42308, /*!< 42308 (String FIX.5.0SP2) The source from which the settlement price is to be obtained.

See http://www.fpml.org/coding-scheme/settlement-price-source for values.*/
LegCashSettlPriceDefault                           = 42309, /*!< 42309 (int FIX.5.0SP2) The default election for determining settlement price. */
NoLegDividendAccrualPaymentDateBusinessCenters     = 42310, /*!< 42310 (NumInGroup FIX.5.0SP2) Number of entries in the LegDividendAccrualPaymentDateBusinessCenterGrp.*/
LegDividendAccrualPaymentDateBusinessCenter        = 42311, /*!< 42311 (String FIX.5.0SP2) The business center calendar used for date adjustment of the instrument's dividend accrual payment date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegDividendFloatingRateIndex                       = 42312, /*!< 42312 (String FIX.5.0SP2) The dividend accrual floating rate index.*/
LegDividendFloatingRateIndexCurvePeriod            = 42313, /*!< 42313 (int FIX.5.0SP2) Time unit multiplier for the dividend accrual floating rate index curve.*/
LegDividendFloatingRateIndexCurveUnit              = 42314, /*!< 42314 (String FIX.5.0SP2) Time unit associated with the dividend accrual floating rate index curve period.

*/
LegDividendFloatingRateMultiplier                  = 42315, /*!< 42315 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This should only be included if the multiplier is not equal to 1 (one) for the term of the contract.*/
LegDividendFloatingRateSpread                      = 42316, /*!< 42316 (PriceOffset FIX.5.0SP2) The basis points spread from the index specified in LegDividendFloatingRateIndex(42312).*/
LegDividendFloatingRateSpreadPositionType          = 42317, /*!< 42317 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
LegDividendFloatingRateTreatment                   = 42318, /*!< 42318 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
LegDividendCapRate                                 = 42319, /*!< 42319 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. It is only required where the floating rate is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as "0.05".*/
LegDividendCapRateBuySide                          = 42320, /*!< 42320 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
LegDividendCapRateSellSide                         = 42321, /*!< 42321 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
LegDividendFloorRate                               = 42322, /*!< 42322 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate. The rate is expressed as a decimal, e.g. 5% is represented as "0.05".*/
LegDividendFloorRateBuySide                        = 42323, /*!< 42323 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
LegDividendFloorRateSellSide                       = 42324, /*!< 42324 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
LegDividendInitialRate                             = 42325, /*!< 42325 (Percentage FIX.5.0SP2) The initial floating rate reset agreed between the principal parties involved in the trade. This is assumed to be the first required reset rate for the first regular calculation period. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. The initial rate is expressed in decimal form, e.g. 5% is represented as "0.05".*/
LegDividendFinalRateRoundingDirection              = 42326, /*!< 42326 (char FIX.5.0SP2) Specifies the rounding direction of the final rate.*/
LegDividendFinalRatePrecision                      = 42327, /*!< 42327 (int FIX.5.0SP2) Specifies the rounding precision of the final rate in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
LegDividendAveragingMethod                         = 42328, /*!< 42328 (int FIX.5.0SP2) When averaging is applicable, used to specify whether a weighted or unweighted average method of calculation is to be used. */
LegDividendNegativeRateTreatment                   = 42329, /*!< 42329 (int FIX.5.0SP2) The specification of any provisions for calculating payment obligations when a floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
LegDividendAccrualPaymentDateRelativeTo            = 42330, /*!< 42330 (int FIX.5.0SP2) Specifies the anchor date when the accrual payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegDividendAccrualPaymentDateOffsetPeriod          = 42331, /*!< 42331 (int FIX.5.0SP2) Time unit multiplier for the relative accrual payment date offset.*/
LegDividendAccrualPaymentDateOffsetUnit            = 42332, /*!< 42332 (String FIX.5.0SP2) Time unit associated with the relative accrual payment date offset.*/
LegDividendAccrualPaymentDateOffsetDayType         = 42333, /*!< 42333 (int FIX.5.0SP2) Specifies the day type of the relative accrual payment date offset.*/
LegDividendAccrualPaymentDateUnadjusted            = 42334, /*!< 42334 (LocalMktDate FIX.5.0SP2) The unadjusted accrual payment date.*/
LegDividendAccrualPaymentDateBusinessDayConvention = 42335, /*!< 42335 (int FIX.5.0SP2) Accrual payment date adjustment business day convention.*/
LegDividendAccrualPaymentDateAdjusted              = 42336, /*!< 42336 (LocalMktDate FIX.5.0SP2) The adjusted accrual payment date.*/
LegDividendReinvestmentIndicator                   = 42337, /*!< 42337 (Boolean FIX.5.0SP2) Indicates whether the dividend will be reinvested.*/
LegDividendEntitlementEvent                        = 42338, /*!< 42338 (int FIX.5.0SP2) Defines the contract event which the receiver of the derivative is entitled to the dividend.*/
LegDividendAmountType                              = 42339, /*!< 42339 (int FIX.5.0SP2) Indicates how the gross cash dividend amount per share is determined.*/
LegDividendUnderlierRefID                          = 42340, /*!< 42340 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
LegExtraordinaryDividendPartySide                  = 42341, /*!< 42341 (int FIX.5.0SP2) Reference to the party through its side in the trade who makes the determination whether dividends are extraordinary in relation to normal levels.*/
LegExtraordinaryDividendAmountType                 = 42342, /*!< 42342 (int FIX.5.0SP2) Indicates how the extraordinary gross cash dividend per share is determined.*/
LegExtraordinaryDividendCurrency                   = 42343, /*!< 42343 (Currency FIX.5.0SP2) The currency in which the excess dividend is denominated. Uses ISO 4217 currency codes.*/
LegExtraordinaryDividendDeterminationMethod        = 42344, /*!< 42344 (String FIX.5.0SP2) Specifies the method in which the excess amount is determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
LegDividendAccrualFixedRate                        = 42345, /*!< 42345 (Percentage FIX.5.0SP2) The dividend accrual fixed rate per annum expressed as a decimal. 

A value of 5% would be represented as "0.05".*/
LegDividendCompoundingMethod                       = 42346, /*!< 42346 (int FIX.5.0SP2) The compounding method to be used when more than one dividend period contributes to a single payment.*/
LegDividendNumOfIndexUnits                         = 42347, /*!< 42347 (int FIX.5.0SP2) The number of index units applicable to dividends.*/
LegDividendCashPercentage                          = 42348, /*!< 42348 (Percentage FIX.5.0SP2) Declared cash dividend percentage. 

A value of 5% would be represented as "0.05".*/
LegDividendCashEquivalentPercentage                = 42349, /*!< 42349 (Percentage FIX.5.0SP2) Declared cash-equivalent dividend percentage. 

A value of 5% would be represented as "0.05".*/
LegNonCashDividendTreatment                        = 42350, /*!< 42350 (int FIX.5.0SP2) Defines the treatment of non-cash dividends.*/
LegDividendComposition                             = 42351, /*!< 42351 (int FIX.5.0SP2) Defines how the composition of dividends is to be determined.*/
LegSpecialDividendsIndicator                       = 42352, /*!< 42352 (Boolean FIX.5.0SP2) Indicates whether special dividends are applicable.*/
LegMaterialDividendsIndicator                      = 42353, /*!< 42353 (Boolean FIX.5.0SP2) Indicates whether material non-cash dividends are applicable.*/
LegOptionsExchangeDividendsIndicator               = 42354, /*!< 42354 (Boolean FIX.5.0SP2) Indicates whether option exchange dividends are applicable.*/
LegAdditionalDividendsIndicator                    = 42355, /*!< 42355 (Boolean FIX.5.0SP2) Indicates whether additional dividends are applicable.*/
LegAllDividendsIndicator                           = 42356, /*!< 42356 (Boolean FIX.5.0SP2) Represents the European Master Confirmation value of 'All Dividends' which, when applicable, signifies that, for a given Ex-Date, the daily observed share price for that day is adjusted (reduced) by the cash dividend and/or the cash value of any non-cash dividend per share (including extraordinary dividends) declared by the issuer.*/
LegDividendFXTriggerDateRelativeTo                 = 42357, /*!< 42357 (int FIX.5.0SP2) Specifies the anchor date when the FX trigger date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegDividendFXTriggerDateOffsetPeriod               = 42358, /*!< 42358 (int FIX.5.0SP2) Time unit multiplier for the relative FX trigger date offset.*/
LegDividendFXTriggerDateOffsetUnit                 = 42359, /*!< 42359 (String FIX.5.0SP2) Time unit associated with the relative FX trigger date offset.*/
LegDividendFXTriggerDateOffsetDayType              = 42360, /*!< 42360 (int FIX.5.0SP2) Specifies the day type of the relative FX trigger date offset.*/
LegDividendFXTriggerDateUnadjusted                 = 42361, /*!< 42361 (LocalMktDate FIX.5.0SP2) The unadjusted FX trigger date.*/
LegDividendFXTriggerDateBusinessDayConvention      = 42362, /*!< 42362 (int FIX.5.0SP2) The business day convention used for the FX trigger date adjustment.*/
LegDividendFXTriggerDateAdjusted                   = 42363, /*!< 42363 (LocalMktDate FIX.5.0SP2) The adjusted FX trigger date.*/
NoLegDividendFXTriggerDateBusinessCenters          = 42364, /*!< 42364 (NumInGroup FIX.5.0SP2) Number of entries in the LegDividendFXTriggerDateBusinessCenterGrp.*/
LegDividendFXTriggerDateBusinessCenter             = 42365, /*!< 42365 (String FIX.5.0SP2) The business center calendar used for date adjustment of the  instrument's FX trigger date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegDividendPeriods                               = 42366, /*!< 42366 (NumInGroup FIX.5.0SP2) Number of entries in the LegDividendPeriodGrp component.*/
LegDividendPeriodSequence                          = 42367, /*!< 42367 (int FIX.5.0SP2) Defines the ordinal dividend period. E.g. 1 = First period, 2 = Second period, etc.*/
LegDividendPeriodStartDateUnadjusted               = 42368, /*!< 42368 (LocalMktDate FIX.5.0SP2) The unadjusted date on which the dividend period will begin.*/
LegDividendPeriodEndDateUnadjusted                 = 42369, /*!< 42369 (LocalMktDate FIX.5.0SP2) The unadjusted date on which the dividend period will end.*/
LegDividendPeriodUnderlierRefID                    = 42370, /*!< 42370 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
LegDividendPeriodStrikePrice                       = 42371, /*!< 42371 (Price FIX.5.0SP2) Specifies the fixed strike price of the dividend period.*/
LegDividendPeriodBusinessDayConvention             = 42372, /*!< 42372 (int FIX.5.0SP2) The dividend period dates business day convention.*/
LegDividendPeriodValuationDateUnadjusted           = 42373, /*!< 42373 (LocalMktDate FIX.5.0SP2) The unadjusted dividend period valuation date.*/
LegDividendPeriodValuationDateRelativeTo           = 42374, /*!< 42374 (int FIX.5.0SP2) Specifies the anchor date when the dividend period valuation date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegDividendPeriodValuationDateOffsetPeriod         = 42375, /*!< 42375 (int FIX.5.0SP2) Time unit multiplier for the relative dividend period valuation date offset.*/
LegDividendPeriodValuationDateOffsetUnit           = 42376, /*!< 42376 (String FIX.5.0SP2) Time unit associated with the relative dividend period valuation date offset.*/
LegDividendPeriodValuationDateOffsetDayType        = 42377, /*!< 42377 (int FIX.5.0SP2) Specifies the day type of the relative dividend period valuation date offset.*/
LegDividendPeriodValuationDateAdjusted             = 42378, /*!< 42378 (LocalMktDate FIX.5.0SP2) The adjusted dividend period valuation date.*/
LegDividendPeriodPaymentDateUnadjusted             = 42379, /*!< 42379 (LocalMktDate FIX.5.0SP2) The unadjusted dividend period payment date.*/
LegDividendPeriodPaymentDateRelativeTo             = 42380, /*!< 42380 (int FIX.5.0SP2) Specifies the anchor date when the dividend period payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegDividendPeriodPaymentDateOffsetPeriod           = 42381, /*!< 42381 (int FIX.5.0SP2) Time unit multiplier for the relative dividend period payment date offset.*/
LegDividendPeriodPaymentDateOffsetUnit             = 42382, /*!< 42382 (String FIX.5.0SP2) Time unit associated with the relative dividend period payment date offset.*/
LegDividendPeriodPaymentDateOffsetDayType          = 42383, /*!< 42383 (int FIX.5.0SP2) Specifies the day type of the relative dividend period payment date offset.*/
LegDividendPeriodPaymentDateAdjusted               = 42384, /*!< 42384 (LocalMktDate FIX.5.0SP2) The adjusted dividend period payment date.*/
LegDividendPeriodXID                               = 42385, /*!< 42385 (XID FIX.5.0SP2) Identifier for linking this stream dividend period to an underlier through an instance of RelatedInstrumentGrp.*/
NoLegDividendPeriodBusinessCenters                 = 42386, /*!< 42386 (NumInGroup FIX.5.0SP2) The number of entries in the LegDividendPeriodBusinessCentersGrp component.*/
LegDividendPeriodBusinessCenter                    = 42387, /*!< 42387 (String FIX.5.0SP2) The business center calendar used for date adjustment of the instrument's dividend period date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegExtraordinaryEvents                           = 42388, /*!< 42388 (NumInGroup FIX.5.0SP2) Number of extraordinary events in the repeating group.*/
LegExtraordinaryEventType                          = 42389, /*!< 42389 (String FIX.5.0SP2) Identifies the type of extraordinary or disruptive event applicable to the reference entity.

See http://www.fixtradingcommunity.org/codelists#Extraordinary_Event_Type for code list of extraordinary event types and values.*/
LegExtraordinaryEventValue                         = 42390, /*!< 42390 (String FIX.5.0SP2) The extraordinary or disruptive event value appropriate to LegExtraordinaryEventType(42389). 

See http://www.fixtradingcommunity.org/codelists#Extraordinary_Event_Type for code list of extraordinary event types and values.*/
LegSettlMethodElectingPartySide                    = 42391, /*!< 42391 (int FIX.5.0SP2) Side value of the party electing the settlement method.*/
LegMakeWholeDate                                   = 42392, /*!< 42392 (LocalMktDate FIX.5.0SP2) The date through which option cannot be exercised without penalty.*/
LegMakeWholeAmount                                 = 42393, /*!< 42393 (Amt FIX.5.0SP2) Amount to be paid by the buyer of the option if the option is exercised prior to the LegMakeWholeDate(42392).*/
LegMakeWholeBenchmarkCurveName                     = 42394, /*!< 42394 (String FIX.5.0SP2) Identifies the benchmark floating rate index.*/
LegMakeWholeBenchmarkCurvePoint                    = 42395, /*!< 42395 (String FIX.5.0SP2) The point on the floating rate index curve. 

Sample values:

M = combination of a number between 1-12 and an "M" for month, e.g. 3M

Y = combination of number between 1-100 and a "Y" for year, e.g. 10Y

10Y-OLD = see above, then add "-OLD" when appropriate

INTERPOLATED = the point is mathematically derived

2/2031 5 3/8 = the point is stated via a combination of maturity month / year and coupon.*/
LegMakeWholeRecallSpread                           = 42396, /*!< 42396 (PriceOffset FIX.5.0SP2) Spread over the floating rate index.*/
LegMakeWholeBenchmarkQuote                         = 42397, /*!< 42397 (int FIX.5.0SP2) The quote side of the benchmark to be used for calculating the "make whole" amount.*/
LegMakeWholeInterpolationMethod                    = 42398, /*!< 42398 (int FIX.5.0SP2) The method used when calculating the "make whole" amount. The most common is linear method.*/
LegPaymentStreamCashSettlIndicator                 = 42399, /*!< 42399 (Boolean FIX.5.0SP2) Indicates whether cash settlement is applicable.*/
LegPaymentStreamCompoundingXIDRef                  = 42400, /*!< 42400 (XIDREF FIX.5.0SP2) Reference to the stream which details the compounding fixed or floating rate.*/
LegPaymentStreamCompoundingSpread                  = 42401, /*!< 42401 (PriceOffset FIX.5.0SP2) The spread to be used for compounding. Used in scenarios where the interest payment is based on a compounding formula that uses a compounding spread in addition to the regular spread.*/
LegPaymentStreamInterpolationMethod                = 42402, /*!< 42402 (int FIX.5.0SP2) The method used when calculating the index rate from multiple points on the curve. The most common is linear method.*/
LegPaymentStreamInterpolationPeriod                = 42403, /*!< 42403 (int FIX.5.0SP2) Defines applicable periods for interpolation.*/
LegPaymentStreamCompoundingFixedRate               = 42404, /*!< 42404 (float FIX.5.0SP2) The compounding fixed rate applicable to the payment stream.*/
NoLegPaymentStreamCompoundingDates                 = 42405, /*!< 42405 (NumInGroup FIX.5.0SP2) Number of dates in the repeating group.*/
LegPaymentStreamCompoundingDate                    = 42406, /*!< 42406 (LocalMktDate FIX.5.0SP2) The compounding date.  Type of date is specified in LegPaymentStreamCompoundingDateType(42407).*/
LegPaymentStreamCompoundingDateType                = 42407, /*!< 42407 (int FIX.5.0SP2) Specifies the type of payment compounding date (e.g. adjusted for holidays).  */
LegPaymentStreamCompoundingDatesBusinessDayConvention = 42408, /*!< 42408 (int FIX.5.0SP2) The compounding dates business day convention.*/
LegPaymentStreamCompoundingDatesRelativeTo         = 42409, /*!< 42409 (int FIX.5.0SP2) Specifies the anchor date when the compounding dates are relative to an anchor  date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamCompoundingDatesOffsetPeriod       = 42410, /*!< 42410 (int FIX.5.0SP2) Time unit multiplier for the relative compounding date offset.*/
LegPaymentStreamCompoundingDatesOffsetUnit         = 42411, /*!< 42411 (String FIX.5.0SP2) Time unit associated with the relative compounding date offset.*/
LegPaymentStreamCompoundingDatesOffsetDayType      = 42412, /*!< 42412 (int FIX.5.0SP2) Specifies the day type of the relative compounding date offset.*/
LegPaymentStreamCompoundingPeriodSkip              = 42413, /*!< 42413 (int FIX.5.0SP2) The number of periods in the "RelativeTo" schedule that are between each date in the compounding schedule. A skip of 2 would mean that compounding dates are relative to every second date in the "RelativeTo" schedule. If present this should have a value greater than 1.*/
LegPaymentStreamCompoundingFrequencyPeriod         = 42414, /*!< 42414 (int FIX.5.0SP2) Time unit multiplier for the frequency at which compounding dates occur. */
LegPaymentStreamCompoundingFrequencyUnit           = 42415, /*!< 42415 (String FIX.5.0SP2) Time unit associated with the frequency at which compounding dates occur.*/
LegPaymentStreamCompoundingRollConvention          = 42416, /*!< 42416 (String FIX.5.0SP2) The convention for determining the sequence of compounding dates. It is used in conjunction with a specified frequency. */
LegPaymentStreamBoundsFirstDateUnadjusted          = 42417, /*!< 42417 (LocalMktDate FIX.5.0SP2) The unadjusted first date of the compounding schedule. This can be used to restrict the range of dates when they are relative.*/
LegPaymentStreamBoundsLastDateUnadjusted           = 42418, /*!< 42418 (LocalMktDate FIX.5.0SP2) The unadjusted last date of the compounding schedule. This can be used to restrict the range of dates when they are relative.*/
NoLegPaymentStreamCompoundingDatesBusinessCenters  = 42419, /*!< 42419 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegPaymentStreamCompoundingDatesBusinessCenter     = 42420, /*!< 42420 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stream compounding dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegPaymentStreamCompoundingEndDateUnadjusted       = 42421, /*!< 42421 (LocalMktDate FIX.5.0SP2) The unadjusted compounding end date.*/
LegPaymentStreamCompoundingEndDateRelativeTo       = 42422, /*!< 42422 (int FIX.5.0SP2) Specifies the anchor date when the compounding end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamCompoundingEndDateOffsetPeriod     = 42423, /*!< 42423 (int FIX.5.0SP2) Time unit multiplier for the relative compounding end date offset.*/
LegPaymentStreamCompoundingEndDateOffsetUnit       = 42424, /*!< 42424 (String FIX.5.0SP2) Time unit associated with the relative compounding end date offset.*/
LegPaymentStreamCompoundingEndDateOffsetDayType    = 42425, /*!< 42425 (int FIX.5.0SP2) Specifies the day type of the relative compounding end date offset.*/
LegPaymentStreamCompoundingEndDateAdjusted         = 42426, /*!< 42426 (LocalMktDate FIX.5.0SP2) The adjusted compounding end date.*/
LegPaymentStreamCompoundingRateIndex               = 42427, /*!< 42427 (String FIX.5.0SP2) The payment stream's compounding floating rate index.*/
LegPaymentStreamCompoundingRateIndexCurvePeriod    = 42428, /*!< 42428 (int FIX.5.0SP2) Time unit multiplier for the payment stream's compounding floating rate index curve period.*/
LegPaymentStreamCompoundingRateIndexCurveUnit      = 42429, /*!< 42429 (String FIX.5.0SP2) Time unit associated with the payment stream's compounding floating rate index curve period. */
LegPaymentStreamCompoundingRateMultiplier          = 42430, /*!< 42430 (float FIX.5.0SP2) A rate multiplier to apply to the compounding floating rate. The multiplier can be less than or greater than 1 (one). This should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
LegPaymentStreamCompoundingRateSpread              = 42431, /*!< 42431 (PriceOffset FIX.5.0SP2) The basis points spread from the index specified in LegPaymentStreamCompoundingRateIndex(42427).*/
LegPaymentStreamCompoundingRateSpreadPositionType  = 42432, /*!< 42432 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
LegPaymentStreamCompoundingRateTreatment           = 42433, /*!< 42433 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
LegPaymentStreamCompoundingCapRate                 = 42434, /*!< 42434 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the compounding floating rate. It is only required where the compounding floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as "0.05".*/
LegPaymentStreamCompoundingCapRateBuySide          = 42435, /*!< 42435 (int FIX.5.0SP2) Reference to the buyer of the compounding cap rate option through its trade side.*/
LegPaymentStreamCompoundingCapRateSellSide         = 42436, /*!< 42436 (int FIX.5.0SP2) Reference to the seller of the compounding cap rate option through its trade side.*/
LegPaymentStreamCompoundingFloorRate               = 42437, /*!< 42437 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the compounding floating rate. The floor rate (strike) is only required where the compounding floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate. The rate is expressed as a decimal, e.g. 5% is represented as "0.05".*/
LegPaymentStreamCompoundingFloorRateBuySide        = 42438, /*!< 42438 (int FIX.5.0SP2) Reference to the buyer of the compounding floor rate option through its trade side.*/
LegPaymentStreamCompoundingFloorRateSellSide       = 42439, /*!< 42439 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
LegPaymentStreamCompoundingInitialRate             = 42440, /*!< 42440 (Percentage FIX.5.0SP2) The initial compounding floating rate reset agreed between the principal parties involved in the trade. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. The initial rate is expressed in decimal form, e.g. 5% is represented as "0.05".*/
LegPaymentStreamCompoundingFinalRateRoundingDirection = 42441, /*!< 42441 (char FIX.5.0SP2) Specifies the rounding direction for the compounding floating rate.*/
LegPaymentStreamCompoundingFinalRatePrecision      = 42442, /*!< 42442 (int FIX.5.0SP2) Specifies the compounding floating rate rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
LegPaymentStreamCompoundingAveragingMethod         = 42443, /*!< 42443 (int FIX.5.0SP2) Specifies the averaging method when compounding floating rate averaging is applicable (e.g. weighted or unweighted). */
LegPaymentStreamCompoundingNegativeRateTreatment   = 42444, /*!< 42444 (int FIX.5.0SP2) Specifies the method for calculating payment obligations when a compounding floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
LegPaymentStreamCompoundingStartDateUnadjusted     = 42445, /*!< 42445 (LocalMktDate FIX.5.0SP2) The unadjusted compounding start date.*/
LegPaymentStreamCompoundingStartDateRelativeTo     = 42446, /*!< 42446 (int FIX.5.0SP2) Specifies the anchor date when the compounding start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamCompoundingStartDateOffsetPeriod   = 42447, /*!< 42447 (int FIX.5.0SP2) Time unit multiplier for the relative compounding start date offset.*/
LegPaymentStreamCompoundingStartDateOffsetUnit     = 42448, /*!< 42448 (String FIX.5.0SP2) Time unit associated with the relative compounding start date offset.*/
LegPaymentStreamCompoundingStartDateOffsetDayType  = 42449, /*!< 42449 (int FIX.5.0SP2) Specifies the day type of the relative compounding start date offset.*/
LegPaymentStreamCompoundingStartDateAdjusted       = 42450, /*!< 42450 (LocalMktDate FIX.5.0SP2) The adjusted compounding start date.*/
LegPaymentStreamFormulaImageLength                 = 42451, /*!< 42451 (Length FIX.5.0SP2) Length in bytes of the LegPaymentStreamFormulaImage(42452) field.*/
LegPaymentStreamFormulaImage                       = 42452, /*!< 42452 (data FIX.5.0SP2) Image of the formula image when represented through an encoded clip in base64Binary.*/
LegPaymentStreamFinalPricePaymentDateUnadjusted    = 42453, /*!< 42453 (LocalMktDate FIX.5.0SP2) The unadjusted final price payment date.*/
LegPaymentStreamFinalPricePaymentDateRelativeTo    = 42454, /*!< 42454 (int FIX.5.0SP2) Specifies the anchor date when the final price payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamFinalPricePaymentDateOffsetPeriod  = 42455, /*!< 42455 (int FIX.5.0SP2) Time unit multiplier for the relative final price payment date offset.*/
LegPaymentStreamFinalPricePaymentDateOffsetUnit    = 42456, /*!< 42456 (String FIX.5.0SP2) Time unit associated with the relative final price payment date offset.*/
LegPaymentStreamFinalPricePaymentDateOffsetDayType = 42457, /*!< 42457 (int FIX.5.0SP2) Specifies the day type of the relative final price payment date offset.*/
LegPaymentStreamFinalPricePaymentDateAdjusted      = 42458, /*!< 42458 (LocalMktDate FIX.5.0SP2) The adjusted final price payment date.*/
NoLegPaymentStreamFixingDates                      = 42459, /*!< 42459 (NumInGroup FIX.5.0SP2) Number of fixing dates in the repeating group.*/
LegPaymentStreamFixingDate                         = 42460, /*!< 42460 (LocalMktDate FIX.5.0SP2) The fixing date.  Type of date is specified in LegPaymentStreamFixingDateType(42461).*/
LegPaymentStreamFixingDateType                     = 42461, /*!< 42461 (int FIX.5.0SP2) Specifies the type of fixing date (e.g. adjusted for holidays).  */
LegPaymentStreamFirstObservationDateUnadjusted     = 42462, /*!< 42462 (LocalMktDate FIX.5.0SP2) The unadjusted initial price observation date.*/
LegPaymentStreamFirstObservationDateRelativeTo     = 42463, /*!< 42463 (int FIX.5.0SP2) Specifies the anchor date when the initial price observation date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStreamFirstObservationDateOffsetDayType  = 42464, /*!< 42464 (int FIX.5.0SP2) Specifies the day type of the initial price observation date offset.*/
LegPaymentStreamFirstObservationDateAdjusted       = 42465, /*!< 42465 (LocalMktDate FIX.5.0SP2) The adjusted initial price observation date.*/
LegPaymentStreamUnderlierRefID                     = 42466, /*!< 42466 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
LegReturnRateNotionalReset                         = 42467, /*!< 42467 (Boolean FIX.5.0SP2) Indicates whether the term "Equity Notional Reset" as defined in the ISDA 2002 Equity Derivatives Definitions is applicable ("Y") or not.*/
LegPaymentStreamLinkInitialLevel                   = 42468, /*!< 42468 (Price FIX.5.0SP2) Price level at which the correlation or variance swap contract will strike.*/
LegPaymentStreamLinkClosingLevelIndicator          = 42469, /*!< 42469 (Boolean FIX.5.0SP2) Indicates whether the correlation or variance swap contract will ("Y") strike off the closing level of the default exchange traded contract or not.*/
LegPaymentStreamLinkExpiringLevelIndicator         = 42470, /*!< 42470 (Boolean FIX.5.0SP2) Indicates whether the correlation or variance swap contract will ("Y") strike off the expiring level of the default exchange traded contract or not.*/
LegPaymentStreamLinkEstimatedTradingDays           = 42471, /*!< 42471 (int FIX.5.0SP2) The expected number of trading days in the variance or correlation swap stream.*/
LegPaymentStreamLinkStrikePrice                    = 42472, /*!< 42472 (Price FIX.5.0SP2) The strike price of a correlation or variance swap stream.*/
LegPaymentStreamLinkStrikePriceType                = 42473, /*!< 42473 (int FIX.5.0SP2) For a variance swap specifies how LegPaymentStreamLinkStrikePrice(42472) is expressed.*/
LegPaymentStreamLinkMaximumBoundary                = 42474, /*!< 42474 (float FIX.5.0SP2) Specifies the maximum or upper boundary for variance or strike determination.

For a variation swap stream all observations above this price level will be excluded from the variance calculation. 

For a correlation swap stream the maximum boundary is a percentage of the strike price.*/
LegPaymentStreamLinkMinimumBoundary                = 42475, /*!< 42475 (float FIX.5.0SP2) Specifies the minimum or lower boundary for variance or strike determination.

For a variation swap stream all observations below this price level will be excluded from the variance calculation.

For a correlation swap stream the minimum boundary is a percentage of the strike price.*/
LegPaymentStreamLinkNumberOfDataSeries             = 42476, /*!< 42476 (int FIX.5.0SP2) Number of data series for a correlation swap.  Normal market practice is that correlation data sets are drawn from geographic market areas, such as America, Europe and Asia Pacific.  Each of these geographic areas will have its own data series to avoid contagion.*/
LegPaymentStreamVarianceUnadjustedCap              = 42477, /*!< 42477 (float FIX.5.0SP2) Indicates the scaling factor to be multiplied by the variance strike price thereby making variance cap applicable.*/
LegPaymentStreamRealizedVarianceMethod             = 42478, /*!< 42478 (int FIX.5.0SP2) Indicates which price to use to satisfy the boundary condition.*/
LegPaymentStreamDaysAdjustmentIndicator            = 42479, /*!< 42479 (Boolean FIX.5.0SP2) Indicates whether the contract specifies that the notional should be scaled by the number of days in range divided by the estimate trading days or not. The number of "days in range" refers to the number of returns that contribute to the realized volatility.*/
LegPaymentStreamNearestExchangeContractRefID       = 42480, /*!< 42480 (String FIX.5.0SP2) References a contract listed on an exchange through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
LegPaymentStreamVegaNotionalAmount                 = 42481, /*!< 42481 (float FIX.5.0SP2) Vega Notional represents the approximate gain/loss at maturity for a 1% difference between RVol (realized volatility) and KVol (strike volatility). It does not necessarily represent the Vega risk of the trade.*/
LegPaymentStreamFormulaCurrency                    = 42482, /*!< 42482 (Currency FIX.5.0SP2) The currency in which the formula amount is denominated.  Uses ISO 4217 currency codes.*/
LegPaymentStreamFormulaCurrencyDeterminationMethod = 42483, /*!< 42483 (String FIX.5.0SP2) Specifies the method according to which the formula amount currency is determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
LegPaymentStreamFormulaReferenceAmount             = 42484, /*!< 42484 (int FIX.5.0SP2) Specifies the reference amount when this term either corresponds to the standard ISDA Definition (either the 2002 Equity Definition for the Equity Amount, or the 2000 Definition for the Interest Amount), or refers to a term defined elsewhere in the swap document.

See http://www.fixtradingcommunity.org/codelists#Payment_Amount_Relative_To for code list of reference amounts.*/
NoLegPaymentStreamFormulas                         = 42485, /*!< 42485 (NumInGroup FIX.5.0SP2) Number of formulas in the repeating group.*/
LegPaymentStreamFormula                            = 42486, /*!< 42486 (XMLData FIX.5.0SP2) Contains an XML representation of the formula. Defined for flexibility in choice of language (MathML, OpenMath or text).*/
LegPaymentStreamFormulaDesc                        = 42487, /*!< 42487 (String FIX.5.0SP2) A description of the math formula in LegPaymentStreamFormula(42486).*/
LegPaymentStubEndDateUnadjusted                    = 42488, /*!< 42488 (LocalMktDate FIX.5.0SP2) The unadjusted stub end date.*/
LegPaymentStubEndDateBusinessDayConvention         = 42489, /*!< 42489 (int FIX.5.0SP2) The stub end date business day convention.*/
LegPaymentStubEndDateRelativeTo                    = 42490, /*!< 42490 (int FIX.5.0SP2) Specifies the anchor date when the stub end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStubEndDateOffsetPeriod                  = 42491, /*!< 42491 (int FIX.5.0SP2) Time unit multiplier for the relative stub end date offset.*/
LegPaymentStubEndDateOffsetUnit                    = 42492, /*!< 42492 (String FIX.5.0SP2) Time unit associated with the relative stub end date offset.*/
LegPaymentStubEndDateOffsetDayType                 = 42493, /*!< 42493 (int FIX.5.0SP2) Specifies the day type of the relative stub end date offset.*/
LegPaymentStubEndDateAdjusted                      = 42494, /*!< 42494 (LocalMktDate FIX.5.0SP2) The adjusted stub end date.*/
NoLegPaymentStubEndDateBusinessCenters             = 42495, /*!< 42495 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegPaymentStubEndDateBusinessCenter                = 42496, /*!< 42496 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stub end date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegPaymentStubStartDateUnadjusted                  = 42497, /*!< 42497 (LocalMktDate FIX.5.0SP2) The unadjusted stub start date.*/
LegPaymentStubStartDateBusinessDayConvention       = 42498, /*!< 42498 (int FIX.5.0SP2) The stub start date business day convention.*/
LegPaymentStubStartDateRelativeTo                  = 42499, /*!< 42499 (int FIX.5.0SP2) Specifies the anchor date when the stub start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegPaymentStubStartDateOffsetPeriod                = 42500, /*!< 42500 (int FIX.5.0SP2) Time unit multiplier for the relative stub start date offset.*/
LegPaymentStubStartDateOffsetUnit                  = 42501, /*!< 42501 (String FIX.5.0SP2) Time unit associated with the relative stub start date offset.*/
LegPaymentStubStartDateOffsetDayType               = 42502, /*!< 42502 (int FIX.5.0SP2) Specifies the day type of the relative stub start date offset.*/
LegPaymentStubStartDateAdjusted                    = 42503, /*!< 42503 (LocalMktDate FIX.5.0SP2) The adjusted stub start date.*/
NoLegPaymentStubStartDateBusinessCenters           = 42504, /*!< 42504 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegPaymentStubStartDateBusinessCenter              = 42505, /*!< 42505 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stub start date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegProvisionBreakFeeElection                       = 42506, /*!< 42506 (int FIX.5.0SP2) Type of fee elected for the break provision. */
LegProvisionBreakFeeRate                           = 42507, /*!< 42507 (Percentage FIX.5.0SP2) Break fee election rate when the break fee is proportional to the notional. A fee rate of 5% would be represented as "0.05".*/
NoLegReturnRateDates                               = 42508, /*!< 42508 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate date repeating group.*/
LegReturnRateDateMode                              = 42509, /*!< 42509 (int FIX.5.0SP2) Specifies the valuation type applicable to the return rate date.*/
LegReturnRateValuationDateRelativeTo               = 42510, /*!< 42510 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation dates are relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegReturnRateValuationDateOffsetPeriod             = 42511, /*!< 42511 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation date offset.*/
LegReturnRateValuationDateOffsetUnit               = 42512, /*!< 42512 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation date offset.*/
LegReturnRateValuationDateOffsetDayType            = 42513, /*!< 42513 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation date offset.*/
LegReturnRateValuationStartDateUnadjusted          = 42514, /*!< 42514 (LocalMktDate FIX.5.0SP2) The unadjusted start date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
LegReturnRateValuationStartDateRelativeTo          = 42515, /*!< 42515 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegReturnRateValuationStartDateOffsetPeriod        = 42516, /*!< 42516 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation start date offset.*/
LegReturnRateValuationStartDateOffsetUnit          = 42517, /*!< 42517 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation start date offset.*/
LegReturnRateValuationStartDateOffsetDayType       = 42518, /*!< 42518 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation start date offset.*/
LegReturnRateValuationStartDateAdjusted            = 42519, /*!< 42519 (LocalMktDate FIX.5.0SP2) The adjusted start date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
LegReturnRateValuationEndDateUnadjusted            = 42520, /*!< 42520 (LocalMktDate FIX.5.0SP2) The unadjusted end date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
LegReturnRateValuationEndDateRelativeTo            = 42521, /*!< 42521 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegReturnRateValuationEndDateOffsetPeriod          = 42522, /*!< 42522 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation end date offset.*/
LegReturnRateValuationEndDateOffsetUnit            = 42523, /*!< 42523 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation end date offset.*/
LegReturnRateValuationEndDateOffsetDayType         = 42524, /*!< 42524 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation end date offset.*/
LegReturnRateValuationEndDateAdjusted              = 42525, /*!< 42525 (LocalMktDate FIX.5.0SP2) The adjusted end date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
LegReturnRateValuationFrequencyPeriod              = 42526, /*!< 42526 (int FIX.5.0SP2) Time unit multiplier for the frequency at which return rate valuation dates occur. */
LegReturnRateValuationFrequencyUnit                = 42527, /*!< 42527 (String FIX.5.0SP2) Time unit associated with the frequency at which return rate valuation dates occur.*/
LegReturnRateValuationFrequencyRollConvention      = 42528, /*!< 42528 (String FIX.5.0SP2) The convention for determining the sequence of return rate valuation dates. It is used in conjunction with a specified frequency.*/
LegReturnRateValuationDateBusinessDayConvention    = 42529, /*!< 42529 (int FIX.5.0SP2) The return rate valuation dates business day convention.*/
NoLegReturnRateFXConversions                       = 42530, /*!< 42530 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate FX conversion repeating group. */
LegReturnRateFXCurrencySymbol                      = 42531, /*!< 42531 (String FIX.5.0SP2) Specifies the currency pair for the FX conversion expressed using the CCY1/CCY2 convention.  Uses ISO 4217 currency codes.*/
LegReturnRateFXRate                                = 42532, /*!< 42532 (float FIX.5.0SP2) The rate of exchange between the two currencies specified in LegReturnRateFXCurrencySymbol(42531).*/
LegReturnRateFXRateCalc                            = 42533, /*!< 42533 (char FIX.5.0SP2) The rate of exchange between the two currencies specified in LegReturnRateFXCurrencySymbol(42531).*/
NoLegReturnRates                                   = 42534, /*!< 42534 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate repeating group.*/
LegReturnRatePriceSequence                         = 42535, /*!< 42535 (int FIX.5.0SP2) Specifies the type of price sequence of the return rate.*/
LegReturnRateCommissionBasis                       = 42536, /*!< 42536 (char FIX.5.0SP2) Specifies the basis or unit used to calculate the commission.*/
LegReturnRateCommissionAmount                      = 42537, /*!< 42537 (Amt FIX.5.0SP2) The commission amount.*/
LegReturnRateCommissionCurrency                    = 42538, /*!< 42538 (Currency FIX.5.0SP2) Specifies the currency the commission amount is denominated in. Uses ISO 4217 currency codes.*/
LegReturnRateTotalCommissionPerTrade               = 42539, /*!< 42539 (Amt FIX.5.0SP2) The total commission per trade.*/
LegReturnRateDeterminationMethod                   = 42540, /*!< 42540 (String FIX.5.0SP2) Specifies the method by which the underlier prices are determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
LegReturnRateAmountRelativeTo                      = 42541, /*!< 42541 (int FIX.5.0SP2) Specifies the reference amount when the return rate amount is relative to another amount in the trade.

See http://www.fixtradingcommunity.org/codelists#Amount_Relative_To for code list of relative amounts.*/
LegReturnRateQuoteMeasureType                      = 42542, /*!< 42542 (String FIX.5.0SP2) Specifies the type of the measure applied to the return rate's asset, e.g. valuation, sensitivity risk. This could be an NPV, a cash flow, a clean price, etc.

See http://www.fpml.org/coding-scheme/asset-measure for values.*/
LegReturnRateQuoteUnits                            = 42543, /*!< 42543 (String FIX.5.0SP2) Specifies the units that the measure is expressed in. If not specified, the default is a price/value in currency units. 

See http://www.fpml.org/coding-scheme/price-quote-units for values.*/
LegReturnRateQuoteMethod                           = 42544, /*!< 42544 (int FIX.5.0SP2) Specifies the type of quote used to determine the return rate of the swap.*/
LegReturnRateQuoteCurrency                         = 42545, /*!< 42545 (Currency FIX.5.0SP2) Specifies the currency the return rate quote is denominated in.  Uses ISO 4217 Currency Code.*/
LegReturnRateQuoteCurrencyType                     = 42546, /*!< 42546 (String FIX.5.0SP2) Specifies the type of currency, e.g. settlement currency, base currency, etc., that the quote is reported in. 

See http://www.fpml.org/coding-scheme/reporting-currency-type for values.*/
LegReturnRateQuoteTimeType                         = 42547, /*!< 42547 (int FIX.5.0SP2) Specifies how or the timing when the quote is to be obtained.*/
LegReturnRateQuoteTime                             = 42548, /*!< 42548 (LocalMktTime FIX.5.0SP2) The time when the quote is to be generated. */
LegReturnRateQuoteDate                             = 42549, /*!< 42549 (LocalMktDate FIX.5.0SP2) The date when the quote is to be generated.*/
LegReturnRateQuoteExpirationTime                   = 42550, /*!< 42550 (LocalMktTime FIX.5.0SP2) The time when the quote ceases to be valid.*/
LegReturnRateQuoteBusinessCenter                   = 42551, /*!< 42551 (String FIX.5.0SP2) The business center calendar used for adjustments associated with LegReturnRateQuoteTimeType(42547) or LegReturnRateQuoteTime(42548) and LegReturnRateQuoteDate(42549), e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegReturnRateQuoteExchange                         = 42552, /*!< 42552 (Exchange FIX.5.0SP2) Specifies the exchange (e.g. stock or listed futures/options exchange) from which the quote is obtained.*/
LegReturnRateQuotePricingModel                     = 42553, /*!< 42553 (String FIX.5.0SP2) Specifies the pricing model used to evaluate the underlying asset price. 

See http://www.fpml.org/coding-scheme/pricing-model for values.*/
LegReturnRateCashFlowType                          = 42554, /*!< 42554 (String FIX.5.0SP2) Specifies the type of cash flows, e.g. coupon payment, premium fee, settlement fee, etc. 

See http://www.fpml.org/coding-scheme/cashflow-type for values.*/
LegReturnRateValuationTimeType                     = 42555, /*!< 42555 (int FIX.5.0SP2) Specifies the timing at which the calculation agent values the underlying. */
LegReturnRateValuationTime                         = 42556, /*!< 42556 (LocalMktTime FIX.5.0SP2) The time at which the calculation agent values the underlying asset. */
LegReturnRateValuationTimeBusinessCenter           = 42557, /*!< 42557 (String FIX.5.0SP2) The business center calendar used for adjustments associated with LegReturnRateValuationTimeType(42555) or LegReturnRateValuationTime(42556), e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegReturnRateValuationPriceOption                  = 42558, /*!< 42558 (int FIX.5.0SP2) Indicates whether an ISDA price option applies, and if applicable which type of price.*/
LegReturnRateFinalPriceFallback                    = 42559, /*!< 42559 (int FIX.5.0SP2) Specifies the fallback provision for the hedging party in the determination of the final price.*/
NoLegReturnRateInformationSources                  = 42560, /*!< 42560 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate information source repeating group.*/
LegReturnRateInformationSource                     = 42561, /*!< 42561 (int FIX.5.0SP2) Identifies the source of rate information. For FX the references source to be used for the FX spot rate.*/
LegReturnRateReferencePage                         = 42562, /*!< 42562 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

For FX, the reference page to the spot rate to be used for the reference FX spot rate.

When LegReturnRateInformationSource(42561) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. 

See: http://www.fpml.org/coding-scheme/settlement-rate-option.*/
LegReturnRateReferencePageHeading                  = 42563, /*!< 42563 (String FIX.5.0SP2) Identifies the page heading from the rate source.*/
NoLegReturnRatePrices                              = 42564, /*!< 42564 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate price repeating group.*/
LegReturnRatePriceBasis                            = 42565, /*!< 42565 (int FIX.5.0SP2) The basis of the return price.*/
LegReturnRatePrice                                 = 42566, /*!< 42566 (Price FIX.5.0SP2) Specifies the price of the underlying swap asset.*/
LegReturnRatePriceCurrency                         = 42567, /*!< 42567 (Currency FIX.5.0SP2) Specifies the currency of the price of the leg swap asset. Uses ISO 4217 currency codes.*/
LegReturnRatePriceType                             = 42568, /*!< 42568 (int FIX.5.0SP2) Specifies whether the LegReturnRatePrice(42566) is expressed in absolute or relative terms.*/
NoLegReturnRateValuationDateBusinessCenters        = 42569, /*!< 42569 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate valuation date business center repeating group.*/
LegReturnRateValuationDateBusinessCenter           = 42570, /*!< 42570 (String FIX.5.0SP2) The business center calendar used for date adjustment of the return rate valuation unadjusted or relative dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoLegReturnRateValuationDates                      = 42571, /*!< 42571 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate valuation date repeating group.*/
LegReturnRateValuationDate                         = 42572, /*!< 42572 (LocalMktDate FIX.5.0SP2) The return rate valuation date.  The type of date is specified in LegReturnRateValuationDateType(42573).*/
LegReturnRateValuationDateType                     = 42573, /*!< 42573 (int FIX.5.0SP2) Specifies the type of return rate valuation date (e.g. adjusted for holidays).  */
LegSettlMethodElectionDateUnadjusted               = 42574, /*!< 42574 (LocalMktDate FIX.5.0SP2) The unadjusted settlement method election date.*/
LegSettlMethodElectionDateBusinessDayConvention    = 42575, /*!< 42575 (int FIX.5.0SP2) The settlement method election date adjustment business day convention. */
LegSettlMethodElectionDateRelativeTo               = 42576, /*!< 42576 (int FIX.5.0SP2) Specifies the anchor date when the settlement method election date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
LegSettlMethodElectionDateOffsetPeriod             = 42577, /*!< 42577 (int FIX.5.0SP2) Time unit multiplier for the relative settlement method election date offset.*/
LegSettlMethodElectionDateOffsetUnit               = 42578, /*!< 42578 (String FIX.5.0SP2) Time unit associated with the relative settlement method election date offset.*/
LegSettlMethodElectionDateOffsetDayType            = 42579, /*!< 42579 (int FIX.5.0SP2) Specifies the day type of the relative settlement method election date offset.*/
LegSettlMethodElectionDateAdjusted                 = 42580, /*!< 42580 (LocalMktDate FIX.5.0SP2) The adjusted settlement method election date.*/
NoLegSettlMethodElectionDateBusinessCenters        = 42581, /*!< 42581 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
LegSettlMethodElectionDateBusinessCenter           = 42582, /*!< 42582 (String FIX.5.0SP2) The business center calendar used for date adjustment of the settlement method election unadjusted or relative date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
LegStreamVersion                                   = 42583, /*!< 42583 (String FIX.5.0SP2) The stream version identifier when there have been modifications to the contract over time. Helps signal when there are embedded changes.*/
LegStreamVersionEffectiveDate                      = 42584, /*!< 42584 (LocalMktDate FIX.5.0SP2) The effective date of the LegStreamVersion(42583).*/
LegStreamNotionalDeterminationMethod               = 42585, /*!< 42585 (String FIX.5.0SP2) Specifies the method for determining the floating notional value for equity swaps. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
LegStreamNotionalAdjustments                       = 42586, /*!< 42586 (int FIX.5.0SP2) For equity swaps this specifies the conditions that govern the adjustment to the number of units of the swap.*/
StreamCommodityDeliveryPricingRegion               = 42587, /*!< 42587 (String FIX.5.0SP2) The delivery or pricing region associated with the commodity swap. See http://www.ecfr.gov/cgi-bin/text-idx?SID=660d6a40f836aa6ddf213cba080c5b22&node=ap17.2.43_17.e&rgn=div9  for the external code list.*/
LegStreamCommodityDeliveryPricingRegion            = 42588, /*!< 42588 (String FIX.5.0SP2) The delivery or pricing region associated with the commodity swap. See http://www.ecfr.gov/cgi-bin/text-idx?SID=660d6a40f836aa6ddf213cba080c5b22&node=ap17.2.43_17.e&rgn=div9 for the external code list.*/
UnderlyingStreamCommodityDeliveryPricingRegion     = 42589, /*!< 42589 (String FIX.5.0SP2) The delivery or pricing region associated with the commodity swap. See http://www.ecfr.gov/cgi-bin/text-idx?SID=660d6a40f836aa6ddf213cba080c5b22&node=ap17.2.43_17.e&rgn=div9 for the external code list.*/
SettlMethodElectingPartySide                       = 42590, /*!< 42590 (int FIX.5.0SP2) Side value of the party electing the settlement method.*/
MakeWholeDate                                      = 42591, /*!< 42591 (LocalMktDate FIX.5.0SP2) The date through which option cannot be exercised without penalty.*/
MakeWholeAmount                                    = 42592, /*!< 42592 (Amt FIX.5.0SP2) Amount to be paid by the buyer of the option if the option is exercised prior to the MakeWholeDate(42591).*/
MakeWholeBenchmarkCurveName                        = 42593, /*!< 42593 (String FIX.5.0SP2) Identifies the benchmark floating rate index.*/
MakeWholeBenchmarkCurvePoint                       = 42594, /*!< 42594 (String FIX.5.0SP2) The point on the floating rate index curve. 

Sample values:

M = combination of a number between 1-12 and an "M" for month, e.g. 3M

Y = combination of number between 1-100 and a "Y" for year, e.g. 10Y

10Y-OLD = see above, then add "-OLD" when appropriate

INTERPOLATED = the point is mathematically derived

2/2031 5 3/8 = the point is stated via a combination of maturity month / year and coupon.*/
MakeWholeRecallSpread                              = 42595, /*!< 42595 (PriceOffset FIX.5.0SP2) Spread over the floating rate index.*/
MakeWholeBenchmarkQuote                            = 42596, /*!< 42596 (int FIX.5.0SP2) The quote side of the benchmark to be used for calculating the "make whole" amount.*/
MakeWholeInterpolationMethod                       = 42597, /*!< 42597 (int FIX.5.0SP2) The method used when calculating the "make whole" amount. The most common is linear method.*/
PaymentAmountRelativeTo                            = 42598, /*!< 42598 (int FIX.5.0SP2) Specifies the reference amount when the payment amount is relative to another amount in the message.

See http://www.fixtradingcommunity.org/codelists#Payment_Amount_Relative_To for code list of relative amounts.

*/
PaymentAmountDeterminationMethod                   = 42599, /*!< 42599 (String FIX.5.0SP2) Specifies the method by which a payment amount is determined.

See http://www.fpml.org/coding-scheme/determination-method for values.*/
PaymentStreamCashSettlIndicator                    = 42600, /*!< 42600 (Boolean FIX.5.0SP2) Indicates whether cash settlement is applicable.*/
PaymentStreamCompoundingXIDRef                     = 42601, /*!< 42601 (XIDREF FIX.5.0SP2) Reference to the stream which details the compounding fixed or floating rate.*/
PaymentStreamCompoundingSpread                     = 42602, /*!< 42602 (PriceOffset FIX.5.0SP2) The spread to be used for compounding. Used in scenarios where the interest payment is based on a compounding formula that uses a compounding spread in addition to the regular spread.*/
PaymentStreamInterpolationMethod                   = 42603, /*!< 42603 (int FIX.5.0SP2) The method used when calculating the index rate from multiple points on the curve. The most common is linear method.*/
PaymentStreamInterpolationPeriod                   = 42604, /*!< 42604 (int FIX.5.0SP2) Defines applicable periods for interpolation.*/
PaymentStreamCompoundingFixedRate                  = 42605, /*!< 42605 (float FIX.5.0SP2) The compounding fixed rate applicable to the payment stream.*/
NoPaymentStreamCompoundingDates                    = 42606, /*!< 42606 (NumInGroup FIX.5.0SP2) Number of dates in the repeating group.*/
PaymentStreamCompoundingDate                       = 42607, /*!< 42607 (LocalMktDate FIX.5.0SP2) The compounding date.  The type of date is specified in PaymentStreamCompoundingDateType(42608).*/
PaymentStreamCompoundingDateType                   = 42608, /*!< 42608 (int FIX.5.0SP2) Specifies the type of payment compounding date (e.g. adjusted for holidays).  */
PaymentStreamCompoundingDatesBusinessDayConvention = 42609, /*!< 42609 (int FIX.5.0SP2) The compounding dates business day convention.*/
PaymentStreamCompoundingDatesRelativeTo            = 42610, /*!< 42610 (int FIX.5.0SP2) Specifies the anchor date when the compounding dates are relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamCompoundingDatesOffsetPeriod          = 42611, /*!< 42611 (int FIX.5.0SP2) Time unit multiplier for the relative compounding date offset.*/
PaymentStreamCompoundingDatesOffsetUnit            = 42612, /*!< 42612 (String FIX.5.0SP2) Time unit associated with the relative compounding date offset.*/
PaymentStreamCompoundingDatesOffsetDayType         = 42613, /*!< 42613 (int FIX.5.0SP2) Specifies the day type of the relative compounding date offset.*/
PaymentStreamCompoundingPeriodSkip                 = 42614, /*!< 42614 (int FIX.5.0SP2) The number of periods in the "RelativeTo" schedule that are between each date in the compounding schedule. A skip of 2 would mean that compounding dates are relative to every second date in the "RelativeTo" schedule. If present this should have a value greater than 1.*/
PaymentStreamCompoundingFrequencyPeriod            = 42615, /*!< 42615 (int FIX.5.0SP2) Time unit multiplier for the frequency at which compounding dates occur. */
PaymentStreamCompoundingFrequencyUnit              = 42616, /*!< 42616 (String FIX.5.0SP2) Time unit associated with the frequency at which compounding dates occur.*/
PaymentStreamCompoundingRollConvention             = 42617, /*!< 42617 (String FIX.5.0SP2) The convention for determining the sequence of compounding dates. It is used in conjunction with a specified frequency. */
PaymentStreamBoundsFirstDateUnadjusted             = 42618, /*!< 42618 (LocalMktDate FIX.5.0SP2) The unadjusted first date of the compounding schedule. This can be used to restrict the range of dates when they are relative.*/
PaymentStreamBoundsLastDateUnadjusted              = 42619, /*!< 42619 (LocalMktDate FIX.5.0SP2) The unadjusted last date of the compounding schedule. This can be used to restrict the range of dates when they are relative.*/
NoPaymentStreamCompoundingDatesBusinessCenters     = 42620, /*!< 42620 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
PaymentStreamCompoundingDatesBusinessCenter        = 42621, /*!< 42621 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stream compounding dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentStreamCompoundingEndDateUnadjusted          = 42622, /*!< 42622 (LocalMktDate FIX.5.0SP2) The unadjusted compounding end date.*/
PaymentStreamCompoundingEndDateRelativeTo          = 42623, /*!< 42623 (int FIX.5.0SP2) Specifies the anchor date when the compounding end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamCompoundingEndDateOffsetPeriod        = 42624, /*!< 42624 (int FIX.5.0SP2) Time unit multiplier for the relative compounding end date offset.*/
PaymentStreamCompoundingEndDateOffsetUnit          = 42625, /*!< 42625 (String FIX.5.0SP2) Time unit associated with the relative compounding end date offset.*/
PaymentStreamCompoundingEndDateOffsetDayType       = 42626, /*!< 42626 (int FIX.5.0SP2) Specifies the day type of the relative compounding end date offset.*/
PaymentStreamCompoundingEndDateAdjusted            = 42627, /*!< 42627 (LocalMktDate FIX.5.0SP2) The adjusted compounding end date.*/
PaymentStreamCompoundingRateIndex                  = 42628, /*!< 42628 (String FIX.5.0SP2) The payment stream's compounding floating rate index.*/
PaymentStreamCompoundingRateIndexCurvePeriod       = 42629, /*!< 42629 (int FIX.5.0SP2) Time unit multiplier for the payment stream's compounding floating rate index curve period.*/
PaymentStreamCompoundingRateIndexCurveUnit         = 42630, /*!< 42630 (String FIX.5.0SP2) Time unit associated with the payment stream's compounding floating rate index curve period. */
PaymentStreamCompoundingRateMultiplier             = 42631, /*!< 42631 (float FIX.5.0SP2) A rate multiplier to apply to the compounding floating rate. The multiplier can be less than or greater than 1 (one). This should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
PaymentStreamCompoundingRateSpread                 = 42632, /*!< 42632 (PriceOffset FIX.5.0SP2) The basis points spread from the index specified in PaymentStreamCompoundingRateIndex(42628).*/
PaymentStreamCompoundingRateSpreadPositionType     = 42633, /*!< 42633 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
PaymentStreamCompoundingRateTreatment              = 42634, /*!< 42634 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
PaymentStreamCompoundingCapRate                    = 42635, /*!< 42635 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the compounding floating rate. It is only required where the compounding floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as "0.05".*/
PaymentStreamCompoundingCapRateBuySide             = 42636, /*!< 42636 (int FIX.5.0SP2) Reference to the buyer of the compounding cap rate option through its trade side.*/
PaymentStreamCompoundingCapRateSellSide            = 42637, /*!< 42637 (int FIX.5.0SP2) Reference to the seller of the compounding cap rate option through its trade side.*/
PaymentStreamCompoundingFloorRate                  = 42638, /*!< 42638 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the compounding floating rate. The floor rate (strike) is only required where the compounding floating rate on a swap stream is floored at a certain strike level.  The floor rate is assumed to be exclusive of any spread and is a per annum rate. The rate is expressed as a decimal, e.g. 5% is represented as "0.05".*/
PaymentStreamCompoundingFloorRateBuySide           = 42639, /*!< 42639 (int FIX.5.0SP2) Reference to the buyer of the compounding floor rate option through its trade side.*/
PaymentStreamCompoundingFloorRateSellSide          = 42640, /*!< 42640 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
PaymentStreamCompoundingInitialRate                = 42641, /*!< 42641 (Percentage FIX.5.0SP2) The initial compounding floating rate reset agreed between the principal parties involved in the trade. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. The initial rate is expressed in decimal form, e.g. 5% is represented as "0.05".*/
PaymentStreamCompoundingFinalRateRoundingDirection = 42642, /*!< 42642 (char FIX.5.0SP2) Specifies the rounding direction for the compounding floating rate.*/
PaymentStreamCompoundingFinalRatePrecision         = 42643, /*!< 42643 (int FIX.5.0SP2) Specifies the compounding floating rate rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
PaymentStreamCompoundingAveragingMethod            = 42644, /*!< 42644 (int FIX.5.0SP2) Specifies the averaging method when compounding floating rate averaging is applicable (e.g. weighted or unweighted). */
PaymentStreamCompoundingNegativeRateTreatment      = 42645, /*!< 42645 (int FIX.5.0SP2) Specifies the method for calculating payment obligations when a compounding floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
PaymentStreamCompoundingStartDateUnadjusted        = 42646, /*!< 42646 (LocalMktDate FIX.5.0SP2) The unadjusted compounding start date.*/
PaymentStreamCompoundingStartDateRelativeTo        = 42647, /*!< 42647 (int FIX.5.0SP2) Specifies the anchor date when the compounding start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamCompoundingStartDateOffsetPeriod      = 42648, /*!< 42648 (int FIX.5.0SP2) Time unit multiplier for the relative compounding start date offset.*/
PaymentStreamCompoundingStartDateOffsetUnit        = 42649, /*!< 42649 (String FIX.5.0SP2) Time unit associated with the relative compounding start date offset.*/
PaymentStreamCompoundingStartDateOffsetDayType     = 42650, /*!< 42650 (int FIX.5.0SP2) Specifies the day type of the relative compounding start date offset.*/
PaymentStreamCompoundingStartDateAdjusted          = 42651, /*!< 42651 (LocalMktDate FIX.5.0SP2) The adjusted compounding start date.*/
PaymentStreamFormulaImageLength                    = 42652, /*!< 42652 (Length FIX.5.0SP2) Length in bytes of the PaymentStreamFormulaImage(42563) field.*/
PaymentStreamFormulaImage                          = 42653, /*!< 42653 (data FIX.5.0SP2) Image of the formula image when represented through an encoded clip in base64Binary.*/
PaymentStreamFinalPricePaymentDateUnadjusted       = 42654, /*!< 42654 (LocalMktDate FIX.5.0SP2) The unadjusted final price payment date.*/
PaymentStreamFinalPricePaymentDateRelativeTo       = 42655, /*!< 42655 (int FIX.5.0SP2) Specifies the anchor date when the final price payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamFinalPricePaymentDateOffsetfPeriod    = 42656, /*!< 42656 (int FIX.5.0SP2) Time unit multiplier for the relative final price payment date offset.*/
PaymentStreamFinalPricePaymentDateOffsetUnit       = 42657, /*!< 42657 (String FIX.5.0SP2) Time unit associated with the relative final price payment date offset.*/
PaymentStreamFinalPricePaymentDateOffsetDayType    = 42658, /*!< 42658 (int FIX.5.0SP2) Specifies the day type of the relative final price payment date offset.*/
PaymentStreamFinalPricePaymentDateAdjusted         = 42659, /*!< 42659 (LocalMktDate FIX.5.0SP2) The adjusted final price payment date.*/
NoPaymentStreamFixingDates                         = 42660, /*!< 42660 (NumInGroup FIX.5.0SP2) Number of fixing dates in the repeating group.*/
PaymentStreamFixingDate                            = 42661, /*!< 42661 (LocalMktDate FIX.5.0SP2) The fixing date.  The type of date is specified in PaymentStreamFixingDateType(42662).*/
PaymentStreamFixingDateType                        = 42662, /*!< 42662 (int FIX.5.0SP2) Specifies the type of fixing date (e.g. adjusted for holidays).  */
PaymentStreamFirstObservationDateUnadjusted        = 42663, /*!< 42663 (LocalMktDate FIX.5.0SP2) The unadjusted initial price observation date.*/
PaymentStreamFirstObservationDateRelativeTo        = 42664, /*!< 42664 (int FIX.5.0SP2) Specifies the anchor date when the initial price observation date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStreamFirstObservationDateOffsetDayType     = 42665, /*!< 42665 (int FIX.5.0SP2) Specifies the day type of the initial price observation date offset.*/
PaymentStreamFirstObservationDateAdjusted          = 42666, /*!< 42666 (LocalMktDate FIX.5.0SP2) The adjusted initial price observation date.*/
PaymentStreamUnderlierRefID                        = 42667, /*!< 42667 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
ReturnRateNotionalReset                            = 42668, /*!< 42668 (Boolean FIX.5.0SP2) Indicates whether the term "Equity Notional Reset" as defined in the ISDA 2002 Equity Derivatives Definitions is applicable ("Y") or not.*/
PaymentStreamLinkInitialLevel                      = 42669, /*!< 42669 (Price FIX.5.0SP2) Price level at which the correlation or variance swap contract will strike.*/
PaymentStreamLinkClosingLevelIndicator             = 42670, /*!< 42670 (Boolean FIX.5.0SP2) Indicates whether the correlation or variance swap contract will ("Y") strike off the closing level of the default exchange traded contract or not.*/
PaymentStreamLinkExpiringLevelIndicator            = 42671, /*!< 42671 (Boolean FIX.5.0SP2) Indicates whether the correlation or variance swap contract will ("Y") strike off the expiring level of the default exchange traded contract or not.*/
PaymentStreamLinkEstimatedTradingDays              = 42672, /*!< 42672 (int FIX.5.0SP2) The expected number of trading days in the variance or correlation swap stream.*/
PaymentStreamLinkStrikePrice                       = 42673, /*!< 42673 (Price FIX.5.0SP2) The strike price of a correlation or variance swap stream.*/
PaymentStreamLinkStrikePriceType                   = 42674, /*!< 42674 (int FIX.5.0SP2) For a variance swap specifies how PaymentStreamLinkStrikePrice(42673) is expressed.*/
PaymentStreamLinkMaximumBoundary                   = 42675, /*!< 42675 (float FIX.5.0SP2) Specifies the maximum or upper boundary for variance or strike determination.

For a variation swap stream all observations above this price level will be excluded from the variance calculation. 

For a correlation swap stream the maximum boundary is a percentage of the strike price.*/
PaymentStreamLinkMinimumBoundary                   = 42676, /*!< 42676 (float FIX.5.0SP2) Specifies the minimum or lower boundary for variance or strike determination.

For a variation swap stream all observations below this price level will be excluded from the variance calculation.

For a correlation swap stream the minimum boundary is a percentage of the strike price.*/
PaymentStreamLinkNumberOfDataSeries                = 42677, /*!< 42677 (int FIX.5.0SP2) Number of data series for a correlation swap.  Normal market practice is that correlation data sets are drawn from geographic market areas, such as America, Europe and Asia Pacific.  Each of these geographic areas will have its own data series to avoid contagion.*/
PaymentStreamVarianceUnadjustedCap                 = 42678, /*!< 42678 (float FIX.5.0SP2) Indicates the scaling factor to be multiplied by the variance strike price thereby making variance cap applicable.*/
PaymentStreamRealizedVarianceMethod                = 42679, /*!< 42679 (int FIX.5.0SP2) Indicates which price to use to satisfy the boundary condition.*/
PaymentStreamDaysAdjustmentIndicator               = 42680, /*!< 42680 (Boolean FIX.5.0SP2) Indicates whether the contract specifies that the notional should be scaled by the number of days in range divided by the estimate trading days or not. The number of "days in range" refers to the number of returns that contribute to the realized volatility.*/
PaymentStreamNearestExchangeContractRefID          = 42681, /*!< 42681 (String FIX.5.0SP2) References a contract listed on an exchange through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
PaymentStreamVegaNotionalAmount                    = 42682, /*!< 42682 (float FIX.5.0SP2) "Vega Notional" represents the approximate gain/loss at maturity for a 1% difference between RVol (realised volatility) and KVol (strike volatility). It does not necessarily represent the Vega risk of the trade.*/
NoPaymentStreamFormulas                            = 42683, /*!< 42683 (NumInGroup FIX.5.0SP2) Number of formulas in the repeating group.*/
PaymentStreamFormula                               = 42684, /*!< 42684 (XMLData FIX.5.0SP2) Contains an XML representation of the formula. Defined for flexibility in choice of language (MathML, OpenMath or text).*/
PaymentStreamFormulaDesc                           = 42685, /*!< 42685 (String FIX.5.0SP2) A description of the math formula in PaymentStreamFormula(42684).*/
PaymentStreamFormulaCurrency                       = 42686, /*!< 42686 (Currency FIX.5.0SP2) The currency in which the formula amount is denominated.  Uses ISO 4217 currency codes.*/
PaymentStreamFormulaCurrencyDeterminationMethod    = 42687, /*!< 42687 (String FIX.5.0SP2) Specifies the method according to which the formula amount currency is determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
PaymentStreamFormulaReferenceAmount                = 42688, /*!< 42688 (int FIX.5.0SP2) Specifies the reference amount when this term either corresponds to the standard ISDA Definition (either the 2002 Equity Definition for the Equity Amount, or the 2000 Definition for the Interest Amount), or refers to a term defined elsewhere in the swap document.

See http://www.fixtradingcommunity.org/codelists#Payment_Amount_Relative_To for code list of reference amounts.*/
PaymentStubEndDateUnadjusted                       = 42689, /*!< 42689 (LocalMktDate FIX.5.0SP2) The unadjusted stub end date.*/
PaymentStubEndDateBusinessDayConvention            = 42690, /*!< 42690 (int FIX.5.0SP2) The stub end date business day convention.*/
PaymentStubEndDateRelativeTo                       = 42691, /*!< 42691 (int FIX.5.0SP2) Specifies the anchor date when the stub end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStubEndDateOffsetPeriod                     = 42692, /*!< 42692 (int FIX.5.0SP2) Time unit multiplier for the relative stub end date offset.*/
PaymentStubEndDateOffsetUnit                       = 42693, /*!< 42693 (String FIX.5.0SP2) Time unit associated with the relative stub end date offset.*/
PaymentStubEndDateOffsetDayType                    = 42694, /*!< 42694 (int FIX.5.0SP2) Specifies the day type of the relative stub end date offset.*/
PaymentStubEndDateAdjusted                         = 42695, /*!< 42695 (LocalMktDate FIX.5.0SP2) The adjusted stub end date.*/
NoPaymentStubEndDateBusinessCenters                = 42696, /*!< 42696 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
PaymentStubEndDateBusinessCenter                   = 42697, /*!< 42697 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stub end date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
PaymentStubStartDateUnadjusted                     = 42698, /*!< 42698 (LocalMktDate FIX.5.0SP2) The unadjusted stub start date.*/
PaymentStubStartDateBusinessDayConvention          = 42699, /*!< 42699 (int FIX.5.0SP2) The stub start date business day convention.*/
PaymentStubStartDateRelativeTo                     = 42700, /*!< 42700 (int FIX.5.0SP2) Specifies the anchor date when the stub start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
PaymentStubStartDateOffsetPeriod                   = 42701, /*!< 42701 (int FIX.5.0SP2) Time unit multiplier for the relative stub start date offset.*/
PaymentStubStartDateOffsetUnit                     = 42702, /*!< 42702 (String FIX.5.0SP2) Time unit associated with the relative stub start date offset.*/
PaymentStubStartDateOffsetDayType                  = 42703, /*!< 42703 (int FIX.5.0SP2) Specifies the day type of the relative stub start date offset.*/
PaymentStubStartDateAdjusted                       = 42704, /*!< 42704 (LocalMktDate FIX.5.0SP2) The adjusted stub start date.*/
NoPaymentStubStartDateBusinessCenters              = 42705, /*!< 42705 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
PaymentStubStartDateBusinessCenter                 = 42706, /*!< 42706 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stub start date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ProvisionBreakFeeElection                          = 42707, /*!< 42707 (int FIX.5.0SP2) Type of fee elected for the break provision. */
ProvisionBreakFeeRate                              = 42708, /*!< 42708 (Percentage FIX.5.0SP2) Break fee election rate when the break fee is proportional to the notional.  A fee rate of 5% would be represented as "0.05".*/
NoReturnRateDates                                  = 42709, /*!< 42709 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate date repeating group.*/
ReturnRateDateMode                                 = 42710, /*!< 42710 (int FIX.5.0SP2) Specifies the valuation type applicable to the return rate date.*/
ReturnRateValuationDateRelativeTo                  = 42711, /*!< 42711 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation dates are relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
ReturnRateValuationDateOffsetPeriod                = 42712, /*!< 42712 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation date offset.*/
ReturnRateValuationDateOffsetUnit                  = 42713, /*!< 42713 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation date offset.*/
ReturnRateValuationDateOffsetDayType               = 42714, /*!< 42714 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation date offset.*/
ReturnRateValuationStartDateUnadjusted             = 42715, /*!< 42715 (LocalMktDate FIX.5.0SP2) The unadjusted start date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
ReturnRateValuationStartDateRelativeTo             = 42716, /*!< 42716 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
ReturnRateValuationStartDateOffsetPeriod           = 42717, /*!< 42717 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation start date offset.*/
ReturnRateValuationStartDateOffsetUnit             = 42718, /*!< 42718 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation start date offset.*/
ReturnRateValuationStartDateOffsetDayType          = 42719, /*!< 42719 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation start date offset.*/
ReturnRateValuationStartDateAdjusted               = 42720, /*!< 42720 (LocalMktDate FIX.5.0SP2) The adjusted start date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
ReturnRateValuationEndDateUnadjusted               = 42721, /*!< 42721 (LocalMktDate FIX.5.0SP2) The unadjusted end date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
ReturnRateValuationEndDateRelativeTo               = 42722, /*!< 42722 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
ReturnRateValuationEndDateOffsetPeriod             = 42723, /*!< 42723 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation end date offset.*/
ReturnRateValuationEndDateOffsetUnit               = 42724, /*!< 42724 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation end date offset.*/
ReturnRateValuationEndDateOffsetDayType            = 42725, /*!< 42725 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation end date offset.*/
ReturnRateValuationEndDateAdjusted                 = 42726, /*!< 42726 (LocalMktDate FIX.5.0SP2) The adjusted end date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
ReturnRateValuationFrequencyPeriod                 = 42727, /*!< 42727 (int FIX.5.0SP2) Time unit multiplier for the frequency at which return rate valuation dates occur. */
ReturnRateValuationFrequencyUnit                   = 42728, /*!< 42728 (String FIX.5.0SP2) Time unit associated with the frequency at which return rate valuation dates occur.*/
ReturnRateValuationFrequencyRollConvention         = 42729, /*!< 42729 (String FIX.5.0SP2) The convention for determining the sequence of return rate valuation dates. It is used in conjunction with a specified frequency.*/
ReturnRateValuationDateBusinessDayConvention       = 42730, /*!< 42730 (int FIX.5.0SP2) The return rate valuation dates business day convention.*/
NoReturnRateFXConversions                          = 42731, /*!< 42731 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate FX conversion repeating group. */
ReturnRateFXCurrencySymbol                         = 42732, /*!< 42732 (String FIX.5.0SP2) Specifies the currency pair for the FX conversion expressed using the CCY1/CCY2 convention.  Uses ISO 4217 currency codes.*/
ReturnRateFXRate                                   = 42733, /*!< 42733 (float FIX.5.0SP2) The rate of exchange between the two currencies specified in ReturnRateFXCurrencySymbol(42732).*/
ReturnRateFXRateCalc                               = 42734, /*!< 42734 (char FIX.5.0SP2) Specifies whether ReturnRateFXRate(42733) should be multiplied or divided.*/
NoReturnRates                                      = 42735, /*!< 42735 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate repeating group.*/
ReturnRatePriceSequence                            = 42736, /*!< 42736 (int FIX.5.0SP2) Specifies the type of price sequence of the return rate.*/
ReturnRateCommissionBasis                          = 42737, /*!< 42737 (char FIX.5.0SP2) Specifies the basis or unit used to calculate the commission.*/
ReturnRateCommissionAmount                         = 42738, /*!< 42738 (Amt FIX.5.0SP2) The commission amount.*/
ReturnRateCommissionCurrency                       = 42739, /*!< 42739 (Currency FIX.5.0SP2) Specifies the currency the commission amount is denominated in. Uses ISO 4217 currency codes.*/
ReturnRateTotalCommissionPerTrade                  = 42740, /*!< 42740 (Amt FIX.5.0SP2) The total commission per trade.*/
ReturnRateDeterminationMethod                      = 42741, /*!< 42741 (String FIX.5.0SP2) Specifies the method by which the underlier prices are determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
ReturnRateAmountRelativeTo                         = 42742, /*!< 42742 (int FIX.5.0SP2) Specifies the reference amount when the return rate amount is relative to another amount in the trade.

See http://www.fixtradingcommunity.org/codelists#Payment_Amount_Relative_To for code list of relative amounts.*/
ReturnRateQuoteMeasureType                         = 42743, /*!< 42743 (String FIX.5.0SP2) Specifies the type of the measure applied to the return rate's asset, e.g. valuation, sensitivity risk. This could be an NPV, a cash flow, a clean price, etc. 

See http://www.fpml.org/coding-scheme/asset-measure for values.*/
ReturnRateQuoteUnits                               = 42744, /*!< 42744 (String FIX.5.0SP2) Specifies the units that the measure is expressed in. If not specified, the default is a price/value in currency units. 

See http://www.fpml.org/coding-scheme/price-quote-units for values.*/
ReturnRateQuoteMethod                              = 42745, /*!< 42745 (int FIX.5.0SP2) Specifies the type of quote used to determine the return rate of the swap.*/
ReturnRateQuoteCurrency                            = 42746, /*!< 42746 (Currency FIX.5.0SP2) Specifies the currency the return rate quote is denominated in.  Uses ISO 4217 Currency Code.*/
ReturnRateQuoteCurrencyType                        = 42747, /*!< 42747 (String FIX.5.0SP2) Specifies the type of currency, e.g. settlement currency, base currency, etc., that the quote is reported in. 

See http://www.fpml.org/coding-scheme/reporting-currency-type for values.*/
ReturnRateQuoteTimeType                            = 42748, /*!< 42748 (int FIX.5.0SP2) Specifies how or the timing when the quote is to be obtained.*/
ReturnRateQuoteTime                                = 42749, /*!< 42749 (LocalMktTime FIX.5.0SP2) The time when the quote is to be generated.*/
ReturnRateQuoteDate                                = 42750, /*!< 42750 (LocalMktDate FIX.5.0SP2) The date when the quote is to be generated.*/
ReturnRateQuoteExpirationTime                      = 42751, /*!< 42751 (LocalMktTime FIX.5.0SP2) The time when the quote ceases to be valid.*/
ReturnRateQuoteBusinessCenter                      = 42752, /*!< 42752 (String FIX.5.0SP2) The business center calendar used for adjustments associated with ReturnRateQuoteTimeType(42748) or ReturnRateQuoteTime(42749) and ReturnRateQuoteDate(42750), e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ReturnRateQuoteExchange                            = 42753, /*!< 42753 (Exchange FIX.5.0SP2) Specifies the exchange (e.g. stock or listed futures/options exchange) from which the quote is obtained.*/
ReturnRateQuotePricingModel                        = 42754, /*!< 42754 (String FIX.5.0SP2) Specifies the pricing model used to evaluate the underlying asset price. 

See http://www.fpml.org/coding-scheme/pricing-model for values.*/
ReturnRateCashFlowType                             = 42755, /*!< 42755 (String FIX.5.0SP2) Specifies the type of cash flows, e.g. coupon payment, premium fee, settlement fee, etc. 

See http://www.fpml.org/coding-scheme/cashflow-type for values.*/
ReturnRateValuationTimeType                        = 42756, /*!< 42756 (int FIX.5.0SP2) Specifies the timing at which the calculation agent values the underlying.*/
ReturnRateValuationTime                            = 42757, /*!< 42757 (LocalMktTime FIX.5.0SP2) The time at which the calculation agent values the underlying asset.*/
ReturnRateValuationTimeBusinessCenter              = 42758, /*!< 42758 (String FIX.5.0SP2) The business center calendar used for adjustments associated with ReturnRateValuationTimeType(42756) or ReturnRateValuationTime(42757), e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
ReturnRateValuationPriceOption                     = 42759, /*!< 42759 (int FIX.5.0SP2) Indicates whether an ISDA price option applies, and if applicable which type of price.*/
ReturnRateFinalPriceFallback                       = 42760, /*!< 42760 (int FIX.5.0SP2) Specifies the fallback provision for the hedging party in the determination of the final price.*/
NoReturnRateInformationSources                     = 42761, /*!< 42761 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate information source repeating group.*/
ReturnRateInformationSource                        = 42762, /*!< 42762 (int FIX.5.0SP2) Identifies the source of rate information. For FX the references source to be used for the FX spot rate.*/
ReturnRateReferencePage                            = 42763, /*!< 42763 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

For FX, the reference page to the spot rate to be used for the reference FX spot rate.

When ReturnRateInformationSource(42762) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. 

See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
ReturnRateReferencePageHeading                     = 42764, /*!< 42764 (String FIX.5.0SP2) Identifies the page heading from the rate source.*/
NoReturnRatePrices                                 = 42765, /*!< 42765 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate price repeating group.*/
ReturnRatePriceBasis                               = 42766, /*!< 42766 (int FIX.5.0SP2) The basis of the return price.*/
ReturnRatePrice                                    = 42767, /*!< 42767 (Price FIX.5.0SP2) Specifies the price of the underlying swap asset.*/
ReturnRatePriceCurrency                            = 42768, /*!< 42768 (Currency FIX.5.0SP2) Specifies the currency of the price of the underlying swap asset. Uses ISO 4217 currency codes.*/
ReturnRatePriceType                                = 42769, /*!< 42769 (int FIX.5.0SP2) Specifies whether the ReturnRatePrice(42767) is expressed in absolute or relative terms.*/
NoReturnRateValuationDateBusinessCenters           = 42770, /*!< 42770 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate valuation date business center repeating group.*/
ReturnRateValuationDateBusinessCenter              = 42771, /*!< 42771 (String FIX.5.0SP2) The business center calendar used for date adjustment of the return rate valuation unadjusted or relative dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoReturnRateValuationDates                         = 42772, /*!< 42772 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate valuation date repeating group.*/
ReturnRateValuationDate                            = 42773, /*!< 42773 (LocalMktDate FIX.5.0SP2) The return rate valuation date.  Type of date is specified in ReturnRateValuationDateType(42774).*/
ReturnRateValuationDateType                        = 42774, /*!< 42774 (int FIX.5.0SP2) Specifies the type of return rate valuation date (e.g. adjusted for holidays).  */
NoSettlMethodElectionDateBusinessCenters           = 42775, /*!< 42775 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
SettlMethodElectionDateBusinessCenter              = 42776, /*!< 42776 (String FIX.5.0SP2) The business center calendar used for date adjustment of the settlement method election unadjusted or relative date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
SettlMethodElectionDateUnadjusted                  = 42777, /*!< 42777 (LocalMktDate FIX.5.0SP2) The unadjusted settlement method election date.*/
SettlMethodElectionDateBusinessDayConvention       = 42778, /*!< 42778 (int FIX.5.0SP2) The settlement method election date adjustment business day convention. */
SettlMethodElectionDateRelativeTo                  = 42779, /*!< 42779 (int FIX.5.0SP2) Specifies the anchor date when the settlement method election date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
SettlMethodElectionDateOffsetPeriod                = 42780, /*!< 42780 (int FIX.5.0SP2) Time unit multiplier for the relative settlement method election date offset.*/
SettlMethodElectionDateOffsetUnit                  = 42781, /*!< 42781 (String FIX.5.0SP2) Time unit associated with the relative settlement method election date offset.*/
SettlMethodElectionDateOffsetDayType               = 42782, /*!< 42782 (int FIX.5.0SP2) Specifies the day type of the relative settlement method election date offset.*/
SettlMethodElectionDateAdjusted                    = 42783, /*!< 42783 (LocalMktDate FIX.5.0SP2) The adjusted settlement method election date.*/
StreamVersion                                      = 42784, /*!< 42784 (String FIX.5.0SP2) The stream version identifier when there have been modifications to the contract over time. Helps signal when there are embedded changes.*/
StreamVersionEffectiveDate                         = 42785, /*!< 42785 (LocalMktDate FIX.5.0SP2) The effective date of the StreamVersion(42784).*/
StreamNotionalDeterminationMethod                  = 42786, /*!< 42786 (String FIX.5.0SP2) Specifies the method for determining the floating notional value for equity swaps. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
StreamNotionalAdjustments                          = 42787, /*!< 42787 (int FIX.5.0SP2) For equity swaps this specifies the conditions that govern the adjustment to the number of units of the swap.*/
NoUnderlyingCashSettlDateBusinessCenters           = 42788, /*!< 42788 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingCashSettlDateBusinessCenter              = 42789, /*!< 42789 (String FIX.5.0SP2) The business center calendar used for date adjustment of the cash settlement unadjusted or relative date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingCashSettlDateUnadjusted                  = 42790, /*!< 42790 (LocalMktDate FIX.5.0SP2) The unadjusted cash settlement date.*/
UnderlyingCashSettlDateBusinessDayConvention       = 42791, /*!< 42791 (int FIX.5.0SP2) The business day convention used to adjust the cash settlement provision's date.  Used only to override the business day convention defined in the UnderlyingInstrument component.*/
UnderlyingCashSettlDateRelativeTo                  = 42792, /*!< 42792 (int FIX.5.0SP2) Specifies the anchor date when the cash settlement date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingCashSettlDateOffsetPeriod                = 42793, /*!< 42793 (int FIX.5.0SP2) Time unit multiplier for the relative cash settlement date offset.*/
UnderlyingCashSettlDateOffsetUnit                  = 42794, /*!< 42794 (String FIX.5.0SP2) Time unit associated with the relative cash settlement date offset.*/
UnderlyingCashSettlDateOffsetDayType               = 42795, /*!< 42795 (int FIX.5.0SP2) Specifies the day type of the relative cash settlement date offset.*/
UnderlyingCashSettlDateAdjusted                    = 42796, /*!< 42796 (LocalMktDate FIX.5.0SP2) The adjusted cash settlement date.*/
UnderlyingCashSettlPriceSource                     = 42797, /*!< 42797 (String FIX.5.0SP2) The source from which the settlement price is to be obtained. 

See http://www.fpml.org/coding-scheme/settlement-price-source for values.*/
UnderlyingCashSettlPriceDefault                    = 42798, /*!< 42798 (int FIX.5.0SP2) The default election for determining settlement price.*/
NoUnderlyingDividendAccrualPaymentDateBusinessCenters = 42799, /*!< 42799 (NumInGroup FIX.5.0SP2) Number of entries in the UnderlyingDividendAccrualPaymentDateBusinessCenterGrp.*/
UnderlyingDividendAccrualPaymentDateBusinessCenter = 42800, /*!< 42800 (String FIX.5.0SP2) The business center calendar used for date adjustment of the  instrument's dividend accrual payment date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingDividendFloatingRateIndex                = 42801, /*!< 42801 (String FIX.5.0SP2) The dividend accrual floating rate index.*/
UnderlyingDividendFloatingRateIndexCurvePeriod     = 42802, /*!< 42802 (int FIX.5.0SP2) Time unit multiplier for the dividend accrual floating rate index curve.*/
UnderlyingDividendFloatingRateIndexCurveUnit       = 42803, /*!< 42803 (String FIX.5.0SP2) Time unit associated with the dividend accrual floating rate index curve period.*/
UnderlyingDividendFloatingRateMultiplier           = 42804, /*!< 42804 (float FIX.5.0SP2) A rate multiplier to apply to the floating rate. The multiplier can be less than or greater than 1 (one). This should only be included if the multiplier is not equal to 1 (one) for the term of the contract.*/
UnderlyingDividendFloatingRateSpread               = 42805, /*!< 42805 (PriceOffset FIX.5.0SP2) The basis points spread from the index specified in UnderlyingDividendFloatingRateIndex(42801).*/
UnderlyingDividendFloatingRateSpreadPositionType   = 42806, /*!< 42806 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
UnderlyingDividendFloatingRateTreatment            = 42807, /*!< 42807 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
UnderlyingDividendCapRate                          = 42808, /*!< 42808 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the floating rate. It is only required where the floating rate is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as "0.05".*/
UnderlyingDividendCapRateBuySide                   = 42809, /*!< 42809 (int FIX.5.0SP2) Reference to the buyer of the cap rate option through its trade side.*/
UnderlyingDividendCapRateSellSide                  = 42810, /*!< 42810 (int FIX.5.0SP2) Reference to the seller of the cap rate option through its trade side.*/
UnderlyingDividendFloorRate                        = 42811, /*!< 42811 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the floating rate. The floor rate (strike) is only required where the floating rate is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate. The rate is expressed as a decimal, e.g. 5% is represented as "0.05".*/
UnderlyingDividendFloorRateBuySide                 = 42812, /*!< 42812 (int FIX.5.0SP2) Reference to the buyer of the floor rate option through its trade side.*/
UnderlyingDividendFloorRateSellSide                = 42813, /*!< 42813 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
UnderlyingDividendInitialRate                      = 42814, /*!< 42814 (Percentage FIX.5.0SP2) The initial floating rate reset agreed between the principal parties involved in the trade. This is assumed to be the first required reset rate for the first regular calculation period. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. The initial rate is expressed in decimal form, e.g. 5% is represented as "0.05".*/
UnderlyingDividendFinalRateRoundingDirection       = 42815, /*!< 42815 (char FIX.5.0SP2) Specifies the rounding direction of the final rate.*/
UnderlyingDividendFinalRatePrecision               = 42816, /*!< 42816 (int FIX.5.0SP2) Specifies the rounding precision of the final rate in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
UnderlyingDividendAveragingMethod                  = 42817, /*!< 42817 (int FIX.5.0SP2) When averaging is applicable, used to specify whether a weighted or unweighted average method of calculation is to be used. */
UnderlyingDividendNegativeRateTreatment            = 42818, /*!< 42818 (int FIX.5.0SP2) The specification of any provisions for calculating payment obligations when a floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
UnderlyingDividendAccrualPaymentDateRelativeTo     = 42819, /*!< 42819 (int FIX.5.0SP2) Specifies the anchor date when the accrual payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingDividendAccrualPaymentDateOffsetPeriod   = 42820, /*!< 42820 (int FIX.5.0SP2) Time unit multiplier for the relative accrual payment date offset.*/
UnderlyingDividendAccrualPaymentDateOffsetUnit     = 42821, /*!< 42821 (String FIX.5.0SP2) Time unit associated with the relative accrual payment date offset.*/
UnderlyingDividendAccrualPaymentDateOffsetDayType  = 42822, /*!< 42822 (int FIX.5.0SP2) Specifies the day type of the relative accrual payment date offset.*/
UnderlyingDividendAccrualPaymentDateUnadjusted     = 42823, /*!< 42823 (LocalMktDate FIX.5.0SP2) The unadjusted accrual payment date.*/
UnderlyingDividendAccrualPaymentDateBusinessDayConvention = 42824, /*!< 42824 (int FIX.5.0SP2) Accrual payment date adjustment business day convention.*/
UnderlyingDividendAccrualPaymentDateAdjusted       = 42825, /*!< 42825 (LocalMktDate FIX.5.0SP2) The adjusted accrual payment date.*/
UnderlyingDividendReinvestmentIndicator            = 42826, /*!< 42826 (Boolean FIX.5.0SP2) Indicates whether the dividend will be reinvested.*/
UnderlyingDividendEntitlementEvent                 = 42827, /*!< 42827 (int FIX.5.0SP2) Defines the contract event which the receiver of the derivative is entitled to the dividend.*/
UnderlyingDividendAmountType                       = 42828, /*!< 42828 (int FIX.5.0SP2) Indicates how the gross cash dividend amount per share is determined.*/
UnderlyingDividendUnderlierRefID                   = 42829, /*!< 42829 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in a separate instance of the UnderlyingInstrument component.*/
UnderlyingExtraordinaryDividendPartySide           = 42830, /*!< 42830 (int FIX.5.0SP2) Reference to the party through its side in the trade who makes the determination whether dividends are extraordinary in relation to normal levels.*/
UnderlyingExtraordinaryDividendAmountType          = 42831, /*!< 42831 (int FIX.5.0SP2) Indicates how the extraordinary gross cash dividend per share is determined.*/
UnderlyingExtraordinaryDividendCurrency            = 42832, /*!< 42832 (Currency FIX.5.0SP2) The currency in which the excess dividend is denominated. Uses ISO 4217 currency codes.*/
UnderlyingExtraordinaryDividendDeterminationMethod = 42833, /*!< 42833 (String FIX.5.0SP2) Specifies the method in which the excess amount is determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
UnderlyingDividendAccrualFixedRate                 = 42834, /*!< 42834 (Percentage FIX.5.0SP2) The dividend accrual fixed rate per annum expressed as a decimal. 

A value of 5% would be represented as "0.05".*/
UnderlyingDividendCompoundingMethod                = 42835, /*!< 42835 (int FIX.5.0SP2) The compounding method to be used when more than one dividend period contributes to a single payment.*/
UnderlyingDividendNumOfIndexUnits                  = 42836, /*!< 42836 (int FIX.5.0SP2) The number of index units applicable to dividends.*/
UnderlyingDividendCashPercentage                   = 42837, /*!< 42837 (Percentage FIX.5.0SP2) Declared cash dividend percentage. 

A value of 5% would be represented as "0.05".*/
UnderlyingDividendCashEquivalentPercentage         = 42838, /*!< 42838 (Percentage FIX.5.0SP2) Declared cash-equivalent dividend percentage. A value of 5% would be represented as "0.05".*/
UnderlyingNonCashDividendTreatment                 = 42839, /*!< 42839 (int FIX.5.0SP2) Defines the treatment of non-cash dividends.*/
UnderlyingDividendComposition                      = 42840, /*!< 42840 (int FIX.5.0SP2) Defines how the composition of dividends is to be determined.*/
UnderlyingSpecialDividendsIndicator                = 42841, /*!< 42841 (Boolean FIX.5.0SP2) Indicates whether special dividends are applicable.*/
UnderlyingMaterialDividendsIndicator               = 42842, /*!< 42842 (Boolean FIX.5.0SP2) Indicates whether material non-cash dividends are applicable.*/
UnderlyingOptionsExchangeDividendsIndicator        = 42843, /*!< 42843 (Boolean FIX.5.0SP2) Indicates whether option exchange dividends are applicable.*/
UnderlyingAdditionalDividendsIndicator             = 42844, /*!< 42844 (Boolean FIX.5.0SP2) Indicates whether additional dividends are applicable.*/
UnderlyingAllDividendsIndicator                    = 42845, /*!< 42845 (Boolean FIX.5.0SP2) Represents the European Master Confirmation value of 'All Dividends' which, when applicable, signifies that, for a given Ex-Date, the daily observed share price for that day is adjusted (reduced) by the cash dividend and/or the cash value of any non-cash dividend per share (including extraordinary dividends) declared by the issuer.*/
UnderlyingDividendFXTriggerDateRelativeTo          = 42846, /*!< 42846 (int FIX.5.0SP2) Specifies the anchor date when the FX trigger date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingDividendFXTriggerDateOffsetPeriod        = 42847, /*!< 42847 (int FIX.5.0SP2) Time unit multiplier for the relative FX trigger date offset.*/
UnderlyingDividendFXTriggerDateOffsetUnit          = 42848, /*!< 42848 (String FIX.5.0SP2) Time unit associated with the relative FX trigger date offset.*/
UnderlyingDividendFXTriggerDateOffsetDayType       = 42849, /*!< 42849 (int FIX.5.0SP2) Specifies the day type of the relative FX trigger date offset.*/
UnderlyingDividendFXTriggerDateUnadjusted          = 42850, /*!< 42850 (LocalMktDate FIX.5.0SP2) The unadjusted FX trigger date.*/
UnderlyingDividendFXTriggerDateBusinessDayConvention = 42851, /*!< 42851 (int FIX.5.0SP2) The business day convention used for the FX trigger date adjustment.*/
UnderlyingDividendFXTriggerDateAdjusted            = 42852, /*!< 42852 (LocalMktDate FIX.5.0SP2) The adjusted FX trigger date.*/
NoUnderlyingDividendFXTriggerDateBusinessCenters   = 42853, /*!< 42853 (NumInGroup FIX.5.0SP2) Number of entries in the UnderlyingDividendFXTriggerDateBusinessCenterGrp.*/
UnderlyingDividendFXTriggerDateBusinessCenter      = 42854, /*!< 42854 (String FIX.5.0SP2) The business center calendar used for date adjustment of the instrument's FX trigger date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingDividendPayments                       = 42855, /*!< 42855 (NumInGroup FIX.5.0SP2) Number of entries in the repeating group.*/
UnderlyingDividendPaymentDate                      = 42856, /*!< 42856 (LocalMktDate FIX.5.0SP2) Specifies the date that the dividend or coupon payment is due.*/
UnderlyingDividendPaymentAmount                    = 42857, /*!< 42857 (Amt FIX.5.0SP2) The amount of the dividend or coupon payment.*/
UnderlyingDividendPaymentCurrency                  = 42858, /*!< 42858 (Currency FIX.5.0SP2) Specifies the currency the UnderlyingDividendPaymentAmount(42857) is denominated in.  Uses ISO 4217 currency codes.*/
UnderlyingDividendAccruedInterest                  = 42859, /*!< 42859 (Amt FIX.5.0SP2) Accrued interest on the dividend or coupon payment.*/
UnderlyingDividendPayoutRatio                      = 42860, /*!< 42860 (float FIX.5.0SP2) Specifies the actual dividend payout ratio associated with the equity or bond underlier.*/
UnderlyingDividendPayoutConditions                 = 42861, /*!< 42861 (String FIX.5.0SP2) Specifies the dividend payout conditions that will be applied in the case where the actual ratio is not known, typically because of regulatory or legal uncertainties. */
NoUnderlyingDividendPeriods                        = 42862, /*!< 42862 (NumInGroup FIX.5.0SP2) Number of entries in the UnderlyingDividendPeriodGrp component.*/
UnderlyingDividendPeriodSequence                   = 42863, /*!< 42863 (int FIX.5.0SP2) Defines the ordinal dividend period. E.g. 1 = First period, 2 = Second period, etc.*/
UnderlyingDividendPeriodStartDateUnadjusted        = 42864, /*!< 42864 (LocalMktDate FIX.5.0SP2) The unadjusted date on which the dividend period will begin.*/
UnderlyingDividendPeriodEndDateUnadjusted          = 42865, /*!< 42865 (LocalMktDate FIX.5.0SP2) The unadjusted date on which the dividend period will end.*/
UnderlyingDividendPeriodUnderlierRefID             = 42866, /*!< 42866 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
UnderlyingDividendPeriodStrikePrice                = 42867, /*!< 42867 (Price FIX.5.0SP2) Specifies the fixed strike price of the dividend period.*/
UnderlyingDividendPeriodBusinessDayConvention      = 42868, /*!< 42868 (int FIX.5.0SP2) The dividend period dates business day convention.*/
UnderlyingDividendPeriodValuationDateUnadjusted    = 42869, /*!< 42869 (LocalMktDate FIX.5.0SP2) The unadjusted dividend period valuation date.*/
UnderlyingDividendPeriodValuationDateRelativeTo    = 42870, /*!< 42870 (int FIX.5.0SP2) Specifies the anchor date when the dividend period valuation date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingDividendPeriodValuationDateOffsetPeriod  = 42871, /*!< 42871 (int FIX.5.0SP2) Time unit multiplier for the relative dividend period valuation date offset.*/
UnderlyingDividendPeriodValuationDateOffsetUnit    = 42872, /*!< 42872 (String FIX.5.0SP2) Time unit associated with the relative dividend period valuation date offset.*/
UnderlyingDividendPeriodValuationDateOffsetDayType = 42873, /*!< 42873 (int FIX.5.0SP2) Specifies the day type of the relative dividend period valuation date offset.*/
UnderlyingDividendPeriodValuationDateAdjusted      = 42874, /*!< 42874 (LocalMktDate FIX.5.0SP2) The adjusted dividend period valuation date.*/
UnderlyingDividendPeriodPaymentDateUnadjusted      = 42875, /*!< 42875 (LocalMktDate FIX.5.0SP2) The unadjusted dividend period payment date.*/
UnderlyingDividendPeriodPaymentDateRelativeTo      = 42876, /*!< 42876 (int FIX.5.0SP2) Specifies the anchor date when the dividend period payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingDividendPeriodPaymentDateOffsetPeriod    = 42877, /*!< 42877 (int FIX.5.0SP2) Time unit multiplier for the relative dividend period payment date offset.*/
UnderlyingDividendPeriodPaymentDateOffsetUnit      = 42878, /*!< 42878 (String FIX.5.0SP2) Time unit associated with the relative dividend period payment date offset.*/
UnderlyingDividendPeriodPaymentDateOffsetDayType   = 42879, /*!< 42879 (int FIX.5.0SP2) Specifies the day type of the relative dividend period payment date offset.*/
UnderlyingDividendPeriodPaymentDateAdjusted        = 42880, /*!< 42880 (LocalMktDate FIX.5.0SP2) The adjusted dividend period payment date.*/
UnderlyingDividendPeriodXID                        = 42881, /*!< 42881 (XID FIX.5.0SP2) Identifier for linking this stream dividend period to an underlier through an instance of RelatedInstrumentGrp.*/
NoUnderlyingDividendPeriodBusinessCenters          = 42882, /*!< 42882 (NumInGroup FIX.5.0SP2) Number of entries in UnderlyingDividendPeriodBusinessCenterGrp.*/
UnderlyingDividendPeriodBusinessCenter             = 42883, /*!< 42883 (String FIX.5.0SP2) The business center calendar used for date adjustment of the instrument's dividend period date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingExtraordinaryEvents                    = 42884, /*!< 42884 (NumInGroup FIX.5.0SP2) Number of extraordinary events in the repeating group.*/
UnderlyingExtraordinaryEventType                   = 42885, /*!< 42885 (String FIX.5.0SP2) Identifies the type of extraordinary or disruptive event applicable to UnderlyingExtraordinaryEventType(42885).

See http://www.fixtradingcommunity.org/codelists#Extraordinary_Event_Type for code list of extraordinary event types and values.*/
UnderlyingExtraordinaryEventValue                  = 42886, /*!< 42886 (String FIX.5.0SP2) The extraordinary or disruptive event value appropriate to UnderlyingExtraordinaryEventType(42885).

See http://www.fixtradingcommunity.org/codelists#Extraordinary_Event_Type for code list of extraordinary event types and values.*/
UnderlyingSettlMethodElectingPartySide             = 42887, /*!< 42887 (int FIX.5.0SP2) Side value of the party electing the settlement method.*/
UnderlyingMakeWholeDate                            = 42888, /*!< 42888 (LocalMktDate FIX.5.0SP2) The date through which the option cannot be exercised without penalty.*/
UnderlyingMakeWholeAmount                          = 42889, /*!< 42889 (Amt FIX.5.0SP2) Amount to be paid by the buyer of the option if the option is exercised prior to the UnderlyingMakeWholeDate(42888).*/
UnderlyingMakeWholeBenchmarkCurveName              = 42890, /*!< 42890 (String FIX.5.0SP2) Identifies the benchmark floating rate index.*/
UnderlyingMakeWholeBenchmarkCurvePoint             = 42891, /*!< 42891 (String FIX.5.0SP2) The point on the floating rate index curve. 

Sample values:

M = combination of a number between 1-12 and an "M" for month, e.g. 3M

Y = combination of number between 1-100 and a "Y" for year, e.g. 10Y

10Y-OLD = see above, then add "-OLD" when appropriate

INTERPOLATED = the point is mathematically derived

2/2031 5 3/8 = the point is stated via a combination of maturity month / year and coupon.*/
UnderlyingMakeWholeRecallSpread                    = 42892, /*!< 42892 (PriceOffset FIX.5.0SP2) Spread over the floating rate index.*/
UnderlyingMakeWholeBenchmarkQuote                  = 42893, /*!< 42893 (int FIX.5.0SP2) The quote side of the benchmark to be used for calculating the "make whole" amount.*/
UnderlyingMakeWholeInterpolationMethod             = 42894, /*!< 42894 (int FIX.5.0SP2) The method used when calculating the "make whole" amount. The most common is linear method.*/
UnderlyingPaymentStreamCashSettlIndicator          = 42895, /*!< 42895 (Boolean FIX.5.0SP2) Indicates whether cash settlement is applicable.*/
UnderlyingPaymentStreamCompoundingXIDRef           = 42896, /*!< 42896 (XIDREF FIX.5.0SP2) Reference to the stream which details the compounding fixed or floating rate.*/
UnderlyingPaymentStreamCompoundingSpread           = 42897, /*!< 42897 (PriceOffset FIX.5.0SP2) The spread to be used for compounding. Used in scenarios where the interest payment is based on a compounding formula that uses a compounding spread in addition to the regular spread.*/
UnderlyingPaymentStreamInterpolationMethod         = 42898, /*!< 42898 (int FIX.5.0SP2) The method used when calculating the index rate from multiple points on the curve. The most common is linear method.*/
UnderlyingPaymentStreamInterpolationPeriod         = 42899, /*!< 42899 (int FIX.5.0SP2) Defines applicable periods for interpolation.*/
UnderlyingPaymentStreamCompoundingFixedRate        = 42900, /*!< 42900 (float FIX.5.0SP2) The compounding fixed rate applicable to the payment stream.*/
NoUnderlyingPaymentStreamCompoundingDates          = 42901, /*!< 42901 (NumInGroup FIX.5.0SP2) Number of dates in the repeating group.*/
UnderlyingPaymentStreamCompoundingDate             = 42902, /*!< 42902 (LocalMktDate FIX.5.0SP2) The compounding date.  Type of date is specified in UnderlyingPaymentStreamCompoundingDateType(42903).*/
UnderlyingPaymentStreamCompoundingDateType         = 42903, /*!< 42903 (int FIX.5.0SP2) Specifies the type of payment compounding date (e.g. adjusted for holidays).  */
UnderlyingPaymentStreamCompoundingDatesBusinessDayConvention = 42904, /*!< 42904 (int FIX.5.0SP2) The compounding dates business day convention.*/
UnderlyingPaymentStreamCompoundingDatesRelativeTo  = 42905, /*!< 42905 (int FIX.5.0SP2) Specifies the anchor date when the compounding dates are relative to an anchor  date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamCompoundingDatesOffsetPeriod = 42906, /*!< 42906 (int FIX.5.0SP2) Time unit multiplier for the relative compounding date offset.*/
UnderlyingPaymentStreamCompoundingDatesOffsetUnit  = 42907, /*!< 42907 (String FIX.5.0SP2) Time unit associated with the relative compounding date offset.*/
UnderlyingPaymentStreamCompoundingDatesOffsetDayType = 42908, /*!< 42908 (int FIX.5.0SP2) Specifies the day type of the relative compounding date offset.*/
UnderlyingPaymentStreamCompoundingPeriodSkip       = 42909, /*!< 42909 (int FIX.5.0SP2) The number of periods in the "RelativeTo" schedule that are between each date in the compounding schedule. A skip of 2 would mean that compounding dates are relative to every second date in the "RelativeTo" schedule. If present this should have a value greater than 1.*/
UnderlyingPaymentStreamCompoundingFrequencyPeriod  = 42910, /*!< 42910 (int FIX.5.0SP2) Time unit multiplier for the frequency at which compounding dates occur. */
UnderlyingPaymentStreamCompoundingFrequencyUnit    = 42911, /*!< 42911 (String FIX.5.0SP2) Time unit associated with the frequency at which compounding dates occur.*/
UnderlyingPaymentStreamCompoundingRollConvention   = 42912, /*!< 42912 (String FIX.5.0SP2) The convention for determining the sequence of compounding dates. It is used in conjunction with a specified frequency.

*/
UnderlyingPaymentStreamBoundsFirstDateUnadjusted   = 42913, /*!< 42913 (LocalMktDate FIX.5.0SP2) The unadjusted first date of the compounding schedule. This can be used to restrict the range of dates when they are relative.*/
UnderlyingPaymentStreamBoundsLastDateUnadjusted    = 42914, /*!< 42914 (LocalMktDate FIX.5.0SP2) The unadjusted last date of the compounding schedule. This can be used to restrict the range of dates when they are relative.*/
NoUnderlyingPaymentStreamCompoundingDatesBusinessCenters = 42915, /*!< 42915 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingPaymentStreamCompoundingDatesBusinessCenter = 42916, /*!< 42916 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stream compounding dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentStreamCompoundingEndDateUnadjusted = 42917, /*!< 42917 (LocalMktDate FIX.5.0SP2) The unadjusted compounding end date.*/
UnderlyingPaymentStreamCompoundingEndDateRelativeTo = 42918, /*!< 42918 (int FIX.5.0SP2) Specifies the anchor date when the compounding end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamCompoundingEndDateOffsetPeriod = 42919, /*!< 42919 (int FIX.5.0SP2) Time unit multiplier for the relative compounding end date offset.*/
UnderlyingPaymentStreamCompoundingEndDateOffsetUnit = 42920, /*!< 42920 (String FIX.5.0SP2) Time unit associated with the relative compounding end date offset.*/
UnderlyingPaymentStreamCompoundingEndDateOffsetDayType = 42921, /*!< 42921 (int FIX.5.0SP2) Specifies the day type of the relative compounding end date offset.*/
UnderlyingPaymentStreamCompoundingEndDateAdjusted  = 42922, /*!< 42922 (LocalMktDate FIX.5.0SP2) The adjusted compounding end date.*/
UnderlyingPaymentStreamCompoundingRateIndex        = 42923, /*!< 42923 (String FIX.5.0SP2) The payment stream's compounding floating rate index.*/
UnderlyingPaymentStreamCompoundingRateIndexCurvePeriod = 42924, /*!< 42924 (int FIX.5.0SP2) Time unit multiplier for the payment stream's compounding floating rate index curve period.*/
UnderlyingPaymentStreamCompoundingRateIndexCurveUnit = 42925, /*!< 42925 (String FIX.5.0SP2) Time unit associated with the payment stream's compounding floating rate index curve period. */
UnderlyingPaymentStreamCompoundingRateMultiplier   = 42926, /*!< 42926 (float FIX.5.0SP2) A rate multiplier to apply to the compounding floating rate. The multiplier can be less than or greater than 1 (one). This should only be included if the multiplier is not equal to 1 (one) for the term of the stream.*/
UnderlyingPaymentStreamCompoundingRateSpread       = 42927, /*!< 42927 (PriceOffset FIX.5.0SP2) The basis points spread from the index specified in UnderlyingPaymentStreamCompoundingRateIndex(42923).*/
UnderlyingPaymentStreamCompoundingRateSpreadPositionType = 42928, /*!< 42928 (int FIX.5.0SP2) Identifies whether the rate spread is applied to a long or short position.*/
UnderlyingPaymentStreamCompoundingRateTreatment    = 42929, /*!< 42929 (int FIX.5.0SP2) Specifies the yield calculation treatment for the index.*/
UnderlyingPaymentStreamCompoundingCapRate          = 42930, /*!< 42930 (Percentage FIX.5.0SP2) The cap rate, if any, which applies to the compounding floating rate. It is only required where the compounding floating rate on a swap stream is capped at a certain level. The cap rate is assumed to be exclusive of any spread and is a per annum rate, expressed as a decimal. A cap rate of 5% would be represented as "0.05".*/
UnderlyingPaymentStreamCompoundingCapRateBuySide   = 42931, /*!< 42931 (int FIX.5.0SP2) Reference to the buyer of the compounding cap rate option through its trade side.*/
UnderlyingPaymentStreamCompoundingCapRateSellSide  = 42932, /*!< 42932 (int FIX.5.0SP2) Reference to the seller of the compounding cap rate option through its trade side.*/
UnderlyingPaymentStreamCompoundingFloorRate        = 42933, /*!< 42933 (Percentage FIX.5.0SP2) The floor rate, if any, which applies to the compounding floating rate. The floor rate (strike) is only required where the compounding floating rate on a swap stream is floored at a certain strike level. The floor rate is assumed to be exclusive of any spread and is a per annum rate. The rate is expressed as a decimal, e.g. 5% is represented as "0.05".*/
UnderlyingPaymentStreamCompoundingFloorRateBuySide = 42934, /*!< 42934 (int FIX.5.0SP2) Reference to the buyer of the compounding floor rate option through its trade side.*/
UnderlyingPaymentStreamCompoundingFloorRateSellSide = 42935, /*!< 42935 (int FIX.5.0SP2) Reference to the seller of the floor rate option through its trade side.*/
UnderlyingPaymentStreamCompoundingInitialRate      = 42936, /*!< 42936 (Percentage FIX.5.0SP2) The initial compounding floating rate reset agreed between the principal parties involved in the trade. It should only be included when the rate is not equal to the rate published on the source implied by the floating rate index. The initial rate is expressed in decimal form, e.g. 5% is represented as "0.05".*/
UnderlyingPaymentStreamCompoundingFinalRateRoundingDirection = 42937, /*!< 42937 (char FIX.5.0SP2) Specifies the rounding direction for the compounding floating rate.*/
UnderlyingPaymentStreamCompoundingFinalRatePrecision = 42938, /*!< 42938 (int FIX.5.0SP2) Specifies the compounding floating rate rounding precision in terms of a number of decimal places. Note how a percentage rate rounding of 5 decimal places is expressed as a rounding precision of 7.*/
UnderlyingPaymentStreamCompoundingAveragingMethod  = 42939, /*!< 42939 (int FIX.5.0SP2) Specifies the averaging method when compounding floating rate averaging is applicable (e.g. weighted or unweighted). */
UnderlyingPaymentStreamCompoundingNegativeRateTreatment = 42940, /*!< 42940 (int FIX.5.0SP2) Specifies the method for calculating payment obligations when a compounding floating rate is negative (either due to a quoted negative floating rate or by operation of a spread that is subtracted from the floating rate).*/
UnderlyingPaymentStreamCompoundingStartDateUnadjusted = 42941, /*!< 42941 (LocalMktDate FIX.5.0SP2) The unadjusted compounding start date.*/
UnderlyingPaymentStreamCompoundingStartDateRelativeTo = 42942, /*!< 42942 (int FIX.5.0SP2) Specifies the anchor date when the compounding start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamCompoundingStartDateOffsetPeriod = 42943, /*!< 42943 (int FIX.5.0SP2) Time unit multiplier for the relative compounding start date offset.*/
UnderlyingPaymentStreamCompoundingStartDateOffsetUnit = 42944, /*!< 42944 (String FIX.5.0SP2) Time unit associated with the relative compounding start date offset.*/
UnderlyingPaymentStreamCompoundingStartDateOffsetDayType = 42945, /*!< 42945 (int FIX.5.0SP2) Specifies the day type of the relative compounding start date offset.*/
UnderlyingPaymentStreamCompoundingStartDateAdjusted = 42946, /*!< 42946 (LocalMktDate FIX.5.0SP2) The adjusted compounding start date.*/
UnderlyingPaymentStreamFormulaImageLength          = 42947, /*!< 42947 (Length FIX.5.0SP2) Length in bytes of the UnderlyingPaymentStreamFormulaImage(42948) field.*/
UnderlyingPaymentStreamFormulaImage                = 42948, /*!< 42948 (data FIX.5.0SP2) Image of the formula image when represented through an encoded clip in base64Binary.*/
UnderlyingPaymentStreamFinalPricePaymentDateUnadjusted = 42949, /*!< 42949 (LocalMktDate FIX.5.0SP2) The unadjusted final price payment date.*/
UnderlyingPaymentStreamFinalPricePaymentDateRelativeTo = 42950, /*!< 42950 (int FIX.5.0SP2) Specifies the anchor date when the final price payment date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamFinalPricePaymentDateOffsetPeriod = 42951, /*!< 42951 (int FIX.5.0SP2) Time unit multiplier for the relative final price payment date offset.*/
UnderlyingPaymentStreamFinalPricePaymentDateOffsetUnit = 42952, /*!< 42952 (String FIX.5.0SP2) Time unit associated with the relative final price payment date offset.*/
UnderlyingPaymentStreamFinalPricePaymentDateOffsetDayType = 42953, /*!< 42953 (int FIX.5.0SP2) Specifies the day type of the relative final price payment date offset.*/
UnderlyingPaymentStreamFinalPricePaymentDateAdjusted = 42954, /*!< 42954 (LocalMktDate FIX.5.0SP2) The adjusted final price payment date.*/
NoUnderlyingPaymentStreamFixingDates               = 42955, /*!< 42955 (NumInGroup FIX.5.0SP2) Number of fixing dates in the repeating group.*/
UnderlyingPaymentStreamFixingDate                  = 42956, /*!< 42956 (LocalMktDate FIX.5.0SP2) The fixing date.  Type of date is specified in UnderlyingPaymentStreamFixingDateType(42957).*/
UnderlyingPaymentStreamFixingDateType              = 42957, /*!< 42957 (int FIX.5.0SP2) Specifies the type of fixing date (e.g. adjusted for holidays).  */
UnderlyingPaymentStreamFirstObservationDateUnadjusted = 42958, /*!< 42958 (LocalMktDate FIX.5.0SP2) The unadjusted initial price observation date.*/
UnderlyingPaymentStreamFirstObservationDateRelativeTo = 42959, /*!< 42959 (int FIX.5.0SP2) Specifies the anchor date when the initial price observation date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStreamFirstObservationDateOffsetDayType = 42960, /*!< 42960 (int FIX.5.0SP2) Specifies the day type of the initial price observation date offset.*/
UnderlyingPaymentStreamFirstObservationDateAdjusted = 42961, /*!< 42961 (LocalMktDate FIX.5.0SP2) The adjusted initial price observation date.*/
UnderlyingPaymentStreamUnderlierRefID              = 42962, /*!< 42962 (String FIX.5.0SP2) References the dividend underlier through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
UnderlyingReturnRateNotionalReset                  = 42963, /*!< 42963 (Boolean FIX.5.0SP2) Indicates whether the term "Equity Notional Reset" as defined in the ISDA 2002 Equity Derivatives Definitions is applicable ("Y") or not.*/
UnderlyingPaymentStreamLinkInitialLevel            = 42964, /*!< 42964 (Price FIX.5.0SP2) Price level at which the correlation or variance swap contract will strike.*/
UnderlyingPaymentStreamLinkClosingLevelIndicator   = 42965, /*!< 42965 (Boolean FIX.5.0SP2) Indicates whether the correlation or variance swap contract will ("Y") strike off the closing level of the default exchange traded contract or not.*/
UnderlyingPaymentStreamLinkExpiringLevelIndicator  = 42966, /*!< 42966 (Boolean FIX.5.0SP2) Indicates whether the correlation or variance swap contract will ("Y") strike off the expiring level of the default exchange traded contract or not.*/
UnderlyingPaymentStreamLinkEstimatedTradingDays    = 42967, /*!< 42967 (int FIX.5.0SP2) The expected number of trading days in the variance or correlation swap stream.*/
UnderlyingPaymentStreamLinkStrikePrice             = 42968, /*!< 42968 (Price FIX.5.0SP2) The strike price of a correlation or variance swap stream.*/
UnderlyingPaymentStreamLinkStrikePriceType         = 42969, /*!< 42969 (int FIX.5.0SP2) For a variance swap specifies how UnderlyingPaymentStreamLinkStrikePrice(42968) is expressed.*/
UnderlyingPaymentStreamLinkMaximumBoundary         = 42970, /*!< 42970 (float FIX.5.0SP2) Specifies the maximum or upper boundary for variance or strike determination.

For a variation swap stream all observations above this price level will be excluded from the variance calculation. 

For a correlation swap stream the maximum boundary is a percentage of the strike price.

*/
UnderlyingPaymentStreamLinkMinimumBoundary         = 42971, /*!< 42971 (float FIX.5.0SP2) Specifies the minimum or lower boundary for variance or strike determination.

For a variation swap stream all observations below this price level will be excluded from the variance calculation.

For a correlation swap stream the minimum boundary is a percentage of the strike price.

*/
UnderlyingPaymentStreamLinkNumberOfDataSeries      = 42972, /*!< 42972 (int FIX.5.0SP2) Number of data series for a correlation swap.  Normal market practice is that correlation data sets are drawn from geographic market areas, such as America, Europe and Asia Pacific.  Each of these geographic areas will have its own data series to avoid contagion.*/
UnderlyingPaymentStreamVarianceUnadjustedCap       = 42973, /*!< 42973 (float FIX.5.0SP2) Indicates the scaling factor to be multiplied by the variance strike price thereby making variance cap applicable.*/
UnderlyingPaymentStreamRealizedVarianceMethod      = 42974, /*!< 42974 (int FIX.5.0SP2) Indicates which price to use to satisfy the boundary condition.*/
UnderlyingPaymentStreamDaysAdjustmentIndicator     = 42975, /*!< 42975 (Boolean FIX.5.0SP2) Indicates whether the contract specifies that the notional should be scaled by the number of days in range divided by the estimate trading days or not. The number of "days in range" refers to the number of returns that contribute to the realized volatility.*/
UnderlyingPaymentStreamNearestExchangeContractRefID = 42976, /*!< 42976 (String FIX.5.0SP2) References a contract listed on an exchange through the instrument's UnderlyingSecurityID(309) which must be fully specified in an instance of the UnderlyingInstrument component.*/
UnderlyingPaymentStreamVegaNotionalAmount          = 42977, /*!< 42977 (float FIX.5.0SP2) Vega Notional represents the approximate gain/loss at maturity for a 1% difference between RVol (realised volatility) and KVol (strike volatility). It does not necessarily represent the Vega risk of the trade.*/
UnderlyingPaymentStreamFormulaCurrency             = 42978, /*!< 42978 (Currency FIX.5.0SP2) The currency in which the formula amount is denominated.  Uses ISO 4217 currency codes.*/
UnderlyingPaymentStreamFormulaCurrencyDeterminationMethod = 42979, /*!< 42979 (String FIX.5.0SP2) Specifies the method according to which the formula amount currency is determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
UnderlyingPaymentStreamFormulaReferenceAmount      = 42980, /*!< 42980 (int FIX.5.0SP2) Specifies the reference amount when this term either corresponds to the standard ISDA Definition (either the 2002 Equity Definition for the Equity Amount, or the 2000 Definition for the Interest Amount), or refers to a term defined elsewhere in the swap document.

See http://www.fixtradingcommunity.org/codelists#Payment_Amount_Relative_To for code list of reference amounts.*/
NoUnderlyingPaymentStreamFormulas                  = 42981, /*!< 42981 (NumInGroup FIX.5.0SP2) Number of formulas in the repeating group.*/
UnderlyingPaymentStreamFormula                     = 42982, /*!< 42982 (XMLData FIX.5.0SP2) Contains an XML representation of the formula. Defined for flexibility in choice of language (MathML, OpenMath or text).*/
UnderlyingPaymentStreamFormulaDesc                 = 42983, /*!< 42983 (String FIX.5.0SP2) A description of the math formula in UnderlyingPaymentStreamFormula(42982).*/
UnderlyingPaymentStubEndDateUnadjusted             = 42984, /*!< 42984 (LocalMktDate FIX.5.0SP2) The unadjusted stub end date.*/
UnderlyingPaymentStubEndDateBusinessDayConvention  = 42985, /*!< 42985 (int FIX.5.0SP2) The stub end date business day convention.*/
UnderlyingPaymentStubEndDateRelativeTo             = 42986, /*!< 42986 (int FIX.5.0SP2) Specifies the anchor date when the stub end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStubEndDateOffsetPeriod           = 42987, /*!< 42987 (int FIX.5.0SP2) Time unit multiplier for the relative stub end date offset.*/
UnderlyingPaymentStubEndDateOffsetUnit             = 42988, /*!< 42988 (String FIX.5.0SP2) Time unit associated with the relative stub end date offset.*/
UnderlyingPaymentStubEndDateOffsetDayType          = 42989, /*!< 42989 (int FIX.5.0SP2) Specifies the day type of the relative stub end date offset.*/
UnderlyingPaymentStubEndDateAdjusted               = 42990, /*!< 42990 (LocalMktDate FIX.5.0SP2) The adjusted stub end date.*/
NoUnderlyingPaymentStubEndDateBusinessCenters      = 42991, /*!< 42991 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingPaymentStubEndDateBusinessCenter         = 42992, /*!< 42992 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stub end date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingPaymentStubStartDateUnadjusted           = 42993, /*!< 42993 (LocalMktDate FIX.5.0SP2) The unadjusted stub start date.*/
UnderlyingPaymentStubStartDateBusinessDayConvention = 42994, /*!< 42994 (int FIX.5.0SP2) The stub start date business day convention.*/
UnderlyingPaymentStubStartDateRelativeTo           = 42995, /*!< 42995 (int FIX.5.0SP2) Specifies the anchor date when the stub start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingPaymentStubStartDateOffsetPeriod         = 42996, /*!< 42996 (int FIX.5.0SP2) Time unit multiplier for the relative stub start date offset.*/
UnderlyingPaymentStubStartDateOffsetUnit           = 42997, /*!< 42997 (String FIX.5.0SP2) Time unit associated with the relative stub start date offset.*/
UnderlyingPaymentStubStartDateOffsetDayType        = 42998, /*!< 42998 (int FIX.5.0SP2) Specifies the day type of the relative stub start date offset.*/
UnderlyingPaymentStubStartDateAdjusted             = 42999, /*!< 42999 (LocalMktDate FIX.5.0SP2) The adjusted stub start date.*/
NoUnderlyingPaymentStubStartDateBusinessCenters    = 43000, /*!< 43000 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingPaymentStubStartDateBusinessCenter       = 43001, /*!< 43001 (String FIX.5.0SP2) The business center calendar used for date adjustment of the payment stub start date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingProvisionBreakFeeElection                = 43002, /*!< 43002 (int FIX.5.0SP2) Type of fee elected for the break provision. */
UnderlyingProvisionBreakFeeRate                    = 43003, /*!< 43003 (Percentage FIX.5.0SP2) Break fee election rate when the break fee is proportional to the notional.  A fee rate of 5% would be represented as "0.05".*/
UnderlyingRateSpreadInitialValue                   = 43004, /*!< 43004 (float FIX.5.0SP2) Specifies the initial rate spread for a basket underlier.*/
NoUnderlyingRateSpreadSteps                        = 43005, /*!< 43005 (NumInGroup FIX.5.0SP2) Number of entries in the repeating group.*/
UnderlyingRateSpreadStepDate                       = 43006, /*!< 43006 (LocalMktDate FIX.5.0SP2) The date that the rate spread step takes affect.*/
UnderlyingRateSpreadStepValue                      = 43007, /*!< 43007 (float FIX.5.0SP2) The the value of the new rate spread as of the UnderlyingRateSpreadStepDate(43006).*/
NoUnderlyingReturnRateDates                        = 43008, /*!< 43008 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate date repeating group.*/
UnderlyingReturnRateDateMode                       = 43009, /*!< 43009 (int FIX.5.0SP2) Specifies the valuation type applicable to the return rate date.*/
UnderlyingReturnRateValuationDateRelativeTo        = 43010, /*!< 43010 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation dates are relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingReturnRateValuationDateOffsetPeriod      = 43011, /*!< 43011 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation date offset.*/
UnderlyingReturnRateValuationDateOffsetUnit        = 43012, /*!< 43012 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation date offset.*/
UnderlyingReturnRateValuationDateOffsetDayType     = 43013, /*!< 43013 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation date offset.*/
UnderlyingReturnRateValuationStartDateUnadjusted   = 43014, /*!< 43014 (LocalMktDate FIX.5.0SP2) The unadjusted start date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
UnderlyingReturnRateValuationStartDateRelativeTo   = 43015, /*!< 43015 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation start date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingReturnRateValuationStartDateOffsetPeriod = 43016, /*!< 43016 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation start date offset.*/
UnderlyingReturnRateValuationStartDateOffsetUnit   = 43017, /*!< 43017 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation start date offset.*/
UnderlyingReturnRateValuationStartDateOffsetDayType = 43018, /*!< 43018 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation start date offset.*/
UnderlyingReturnRateValuationStartDateAdjusted     = 43019, /*!< 43019 (LocalMktDate FIX.5.0SP2) The adjusted start date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
UnderlyingReturnRateValuationEndDateUnadjusted     = 43020, /*!< 43020 (LocalMktDate FIX.5.0SP2) The unadjusted end date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
UnderlyingReturnRateValuationEndDateRelativeTo     = 43021, /*!< 43021 (int FIX.5.0SP2) Specifies the anchor date when the return rate valuation end date is relative to an anchor date.

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingReturnRateValuationEndDateOffsetPeriod   = 43022, /*!< 43022 (int FIX.5.0SP2) Time unit multiplier for the relative return rate valuation end date offset.*/
UnderlyingReturnRateValuationEndDateOffsetUnit     = 43023, /*!< 43023 (String FIX.5.0SP2) Time unit associated with the relative return rate valuation end date offset.*/
UnderlyingReturnRateValuationEndDateOffsetDayType  = 43024, /*!< 43024 (int FIX.5.0SP2) Specifies the day type of the relative return rate valuation end date offset.*/
UnderlyingReturnRateValuationEndDateAdjusted       = 43025, /*!< 43025 (LocalMktDate FIX.5.0SP2) The adjusted end date for return rate valuation. This can be used to restrict the range of dates when they are relative.*/
UnderlyingReturnRateValuationFrequencyPeriod       = 43026, /*!< 43026 (int FIX.5.0SP2) Time unit multiplier for the frequency at which return rate valuation dates occur. */
UnderlyingReturnRateValuationFrequencyUnit         = 43027, /*!< 43027 (String FIX.5.0SP2) Time unit associated with the frequency at which return rate valuation dates occur.*/
UnderlyingReturnRateValuationFrequencyRollConvention = 43028, /*!< 43028 (String FIX.5.0SP2) The convention for determining the sequence of return rate valuation dates. It is used in conjunction with a specified frequency.*/
UnderlyingReturnRateValuationDateBusinessDayConvention = 43029, /*!< 43029 (int FIX.5.0SP2) The return rate valuation dates business day convention.*/
NoUnderlyingReturnRateFXConversions                = 43030, /*!< 43030 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate FX conversion repeating group. */
UnderlyingReturnRateFXCurrencySymbol               = 43031, /*!< 43031 (String FIX.5.0SP2) Specifies the currency pair for the FX conversion expressed using the CCY1/CCY2 convention.  Uses ISO 4217 currency codes.*/
UnderlyingReturnRateFXRate                         = 43032, /*!< 43032 (float FIX.5.0SP2) The rate of exchange between the two currencies specified in UnderlyingReturnRateFXCurrencySymbol(43031).*/
UnderlyingReturnRateFXRateCalc                     = 43033, /*!< 43033 (char FIX.5.0SP2) Specifies whether UnderlyingReturnRateFXRate(43032) should be multiplied or divided.*/
NoUnderlyingReturnRates                            = 43034, /*!< 43034 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate repeating group.*/
UnderlyingReturnRatePriceSequence                  = 43035, /*!< 43035 (int FIX.5.0SP2) Specifies the type of price sequence of the return rate.*/
UnderlyingReturnRateCommissionBasis                = 43036, /*!< 43036 (char FIX.5.0SP2) Specifies the basis or unit used to calculate the commission.*/
UnderlyingReturnRateCommissionAmount               = 43037, /*!< 43037 (Amt FIX.5.0SP2) The commission amount.*/
UnderlyingReturnRateCommissionCurrency             = 43038, /*!< 43038 (Currency FIX.5.0SP2) Specifies the currency the commission amount is denominated in. Uses ISO 4217 currency codes.*/
UnderlyingReturnRateTotalCommissionPerTrade        = 43039, /*!< 43039 (Amt FIX.5.0SP2) The total commission per trade.*/
UnderlyingReturnRateDeterminationMethod            = 43040, /*!< 43040 (String FIX.5.0SP2) Specifies the method by which the underlier prices are determined. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
UnderlyingReturnRateAmountRelativeTo               = 43041, /*!< 43041 (int FIX.5.0SP2) Specifies the reference amount when the return rate amount is relative to another amount in the trade.

See http://www.fixtradingcommunity.org/codelists#Payment_Amount_Relative_To for code list of relative amounts.*/
UnderlyingReturnRateQuoteMeasureType               = 43042, /*!< 43042 (String FIX.5.0SP2) Specifies the type of the measure applied to the return rate's asset, e.g. valuation, sensitivity risk. This could be an NPV, a cash flow, a clean price, etc. 

See http://www.fpml.org/coding-scheme/asset-measure for values.*/
UnderlyingReturnRateQuoteUnits                     = 43043, /*!< 43043 (String FIX.5.0SP2) Specifies the units that the measure is expressed in. If not specified, the default is a price/value in currency units. 

See http://www.fpml.org/coding-scheme/price-quote-units for values.*/
UnderlyingReturnRateQuoteMethod                    = 43044, /*!< 43044 (int FIX.5.0SP2) Specifies the type of quote used to determine the return rate of the swap.*/
UnderlyingReturnRateQuoteCurrency                  = 43045, /*!< 43045 (Currency FIX.5.0SP2) Specifies the currency the return rate quote is denominated in.  Uses ISO 4217 Currency Code.*/
UnderlyingReturnRateQuoteCurrencyType              = 43046, /*!< 43046 (String FIX.5.0SP2) Specifies the type of currency, e.g. settlement currency, base currency, etc., that the quote is reported in. 

See http://www.fpml.org/coding-scheme/reporting-currency-type for values.*/
UnderlyingReturnRateQuoteTimeType                  = 43047, /*!< 43047 (int FIX.5.0SP2) Specifies how or the timing when the quote is to be obtained.*/
UnderlyingReturnRateQuoteTime                      = 43048, /*!< 43048 (LocalMktDate FIX.5.0SP2) The time when the quote is to be generated. */
UnderlyingReturnRateQuoteDate                      = 43049, /*!< 43049 (LocalMktDate FIX.5.0SP2) The date when the quote is to be generated.*/
UnderlyingReturnRateQuoteExpirationTime            = 43050, /*!< 43050 (LocalMktTime FIX.5.0SP2) The time when the quote ceases to be valid.*/
UnderlyingReturnRateQuoteBusinessCenter            = 43051, /*!< 43051 (String FIX.5.0SP2) The business center calendar used for adjustments associated with UnderlyingReturnRateQuoteTimeType(43047) or UnderlyingReturnRateQuoteTime(43048) and UnderlyingReturnRateQuoteDate(43049), e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingReturnRateQuoteExchange                  = 43052, /*!< 43052 (Exchange FIX.5.0SP2) Specifies the exchange (e.g. stock or listed futures/options exchange) from which the quote is obtained.*/
UnderlyingReturnRateQuotePricingModel              = 43053, /*!< 43053 (String FIX.5.0SP2) Specifies the pricing model used to evaluate the underlying asset price. 

See http://www.fpml.org/coding-scheme/pricing-model for values.*/
UnderlyingReturnRateCashFlowType                   = 43054, /*!< 43054 (String FIX.5.0SP2) Specifies the type of cash flows, e.g. coupon payment, premium fee, settlement fee, etc. 

See http://www.fpml.org/coding-scheme/cashflow-type for values.*/
UnderlyingReturnRateValuationTimeType              = 43055, /*!< 43055 (int FIX.5.0SP2) Specifies the timing at which the calculation agent values the underlying.*/
UnderlyingReturnRateValuationTime                  = 43056, /*!< 43056 (LocalMktTime FIX.5.0SP2) The time at which the calculation agent values the underlying asset.*/
UnderlyingReturnRateValuationTimeBusinessCenter    = 43057, /*!< 43057 (String FIX.5.0SP2) The business center calendar used for adjustments associated with  UnderlyingReturnRateValuationTimeType(43055) or UnderlyingReturnRateValuationTime(43056) , e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingReturnRateValuationPriceOption           = 43058, /*!< 43058 (int FIX.5.0SP2) Indicates whether an ISDA price option applies, and if applicable which type of price.*/
UnderlyingReturnRateFinalPriceFallback             = 43059, /*!< 43059 (int FIX.5.0SP2) Specifies the fallback provision for the hedging party in the determination of the final price.*/
NoUnderlyingReturnRateInformationSources           = 43060, /*!< 43060 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate information source repeating group.*/
UnderlyingReturnRateInformationSource              = 43061, /*!< 43061 (int FIX.5.0SP2) Identifies the source of rate information. For FX the references source to be used for the FX spot rate.*/
UnderlyingReturnRateReferencePage                  = 43062, /*!< 43062 (String FIX.5.0SP2) Identifies the reference "page" from the rate source.

For FX, the reference page to the spot rate to be used for the reference FX spot rate.

When UnderlyingReturnRateInformationSource(43061) = 3 (ISDA Settlement Rate Option) this contains the value from the scheme that reflects the terms of the Annex A to the ISDA 1998 FX and Currency Option Definitions. 

See: http://www.fpml.org/coding-scheme/settlement-rate-option*/
UnderlyingReturnRateReferencePageHeading           = 43063, /*!< 43063 (String FIX.5.0SP2) Identifies the page heading from the rate source.*/
NoUnderlyingReturnRatePrices                       = 43064, /*!< 43064 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate price repeating group.*/
UnderlyingReturnRatePriceBasis                     = 43065, /*!< 43065 (int FIX.5.0SP2) The basis of the return price.*/
UnderlyingReturnRatePrice                          = 43066, /*!< 43066 (Price FIX.5.0SP2) Specifies the price of the underlying swap asset.*/
UnderlyingReturnRatePriceCurrency                  = 43067, /*!< 43067 (Currency FIX.5.0SP2) Specifies the currency of the price of the underlying swap asset. Uses ISO 4217 currency codes.*/
UnderlyingReturnRatePriceType                      = 43068, /*!< 43068 (int FIX.5.0SP2) Specifies whether the UnderlyingReturnRatePrice(43066) is expressed in absolute or relative terms.*/
NoUnderlyingReturnRateValuationDateBusinessCenters = 43069, /*!< 43069 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate valuation date business center repeating group.*/
UnderlyingReturnRateValuationDateBusinessCenter    = 43070, /*!< 43070 (String FIX.5.0SP2) The business center calendar used for date adjustment of the return rate valuation unadjusted or relative dates, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
NoUnderlyingReturnRateValuationDates               = 43071, /*!< 43071 (NumInGroup FIX.5.0SP2) Number of iterations in the return rate valuation date repeating group.*/
UnderlyingReturnRateValuationDate                  = 43072, /*!< 43072 (LocalMktDate FIX.5.0SP2) The return rate valuation date.  Type of date is specified in UnderlyingReturnRateValuationDateType(43073).*/
UnderlyingReturnRateValuationDateType              = 43073, /*!< 43073 (int FIX.5.0SP2) Specifies the type of return rate valuation date (e.g. adjusted for holidays).  */
NoUnderlyingSettlMethodElectionDateBusinessCenters = 43074, /*!< 43074 (NumInGroup FIX.5.0SP2) Number of business centers in the repeating group.*/
UnderlyingSettlMethodElectionDateBusinessCenter    = 43075, /*!< 43075 (String FIX.5.0SP2) The business center calendar used for date adjustment of the settlement method election unadjusted or relative date, e.g. "GBLO". 

See http://www.fpml.org/coding-scheme/business-center for standard 4-character code values.*/
UnderlyingSettlMethodElectionDateUnadjusted        = 43076, /*!< 43076 (LocalMktDate FIX.5.0SP2) The unadjusted settlement method election date.*/
UnderlyingSettlMethodElectionDateBusinessDayConvention = 43077, /*!< 43077 (int FIX.5.0SP2) The settlement method election date adjustment business day convention. */
UnderlyingSettlMethodElectionDateRelativeTo        = 43078, /*!< 43078 (int FIX.5.0SP2) Specifies the anchor date when the settlement method election date is relative to an anchor date. 

See http://www.fixtradingcommunity.org/codelists#Relative_To_Date for values.*/
UnderlyingSettlMethodElectionDateOffsetPeriod      = 43079, /*!< 43079 (int FIX.5.0SP2) Time unit multiplier for the relative settlement method election date offset.*/
UnderlyingSettlMethodElectionDateOffsetUnit        = 43080, /*!< 43080 (String FIX.5.0SP2) Time unit associated with the relative settlement method election date offset.*/
UnderlyingSettlMethodElectionDateOffsetDayType     = 43081, /*!< 43081 (int FIX.5.0SP2) Specifies the day type of the relative settlement method election date offset.*/
UnderlyingSettlMethodElectionDateAdjusted          = 43082, /*!< 43082 (LocalMktDate FIX.5.0SP2) The adjusted settlement method election date.*/
UnderlyingStreamVersion                            = 43083, /*!< 43083 (String FIX.5.0SP2) The stream version identifier when there have been modifications to the contract over time. Helps signal when there are embedded changes.*/
UnderlyingStreamVersionEffectiveDate               = 43084, /*!< 43084 (LocalMktDate FIX.5.0SP2) The effective date of the UnderlyingStreamVersion(43083).*/
UnderlyingStreamNotionalDeterminationMethod        = 43085, /*!< 43085 (String FIX.5.0SP2) Specifies the method for determining the floating notional value for equity swaps. 

See http://www.fpml.org/coding-scheme/determination-method for values.*/
UnderlyingStreamNotionalAdjustments                = 43086, /*!< 43086 (int FIX.5.0SP2) For equity swaps this specifies the conditions that govern the adjustment to the number of units of the swap.*/
PaymentDesc                                        = 43087, /*!< 43087 (String FIX.5.0SP2) A short descriptive name given to the payment, e.g. Premium, Upfront, etc. The description has no intrinsic meaning but should be arbitrarily chosen by the remitter as reference.*/
LegPaymentStreamRateIndexID                        = 43088, /*!< 43088 (String FIX.5.0SP2) Security identifier of the floating rate index.*/
LegPaymentStreamRateIndexIDSource                  = 43089, /*!< 43089 (String FIX.5.0SP2) Source for the floating rate index identified in LegPaymentStreamRateIndexID(43088).*/
PaymentStreamRateIndexID                           = 43090, /*!< 43090 (String FIX.5.0SP2) Security identifier of the floating rate index.*/
PaymentStreamRateIndexIDSource                     = 43091, /*!< 43091 (String FIX.5.0SP2) Source for the floating rate index identified in PaymentStreamRateIndexID(43090).*/
UnderlyingPaymentStreamRateIndexID                 = 43092, /*!< 43092 (String FIX.5.0SP2) Security identifier of the floating rate index.*/
UnderlyingPaymentStreamRateIndexIDSource           = 43093, /*!< 43093 (String FIX.5.0SP2) Source for the floating rate index identified in UnderlyingPaymentStreamRateIndexID(43092).*/
DeliveryStreamRouteOrCharter                       = 43094, /*!< 43094 (String FIX.5.0SP2) Specific delivery route or time charter average.  Applicable to commodity freight swaps.*/
LegDeliveryStreamRouteOrCharter                    = 43095, /*!< 43095 (String FIX.5.0SP2) Specific delivery route or time charter average.  Applicable to commodity freight swaps.*/
UnderlyingDeliveryStreamRouteOrCharter             = 43096, /*!< 43096 (String FIX.5.0SP2) Specific delivery route or time charter average.  Applicable to commodity freight swaps.*/
BatchID                                            = 50000, /*!< 50000 (String FIX.5.0SP2) Unique Identifier for a batch of messages.*/
BatchTotalMessages                                 = 50001, /*!< 50001 (int FIX.5.0SP2) Total # of messages contained within batch.*/
BatchProcessMode                                   = 50002 /*!< 50002 (int FIX.5.0SP2) Indicates the processing mode for a batch of messages.*/
};
} // namespace tag

namespace {
//! Sorted list of all field tags which are of type Length
int length_fields[] = {
tag::SecureDataLen,                 // 90 Length FIX.2.7
tag::SignatureLength,               // 93 Length FIX.2.7
tag::RawDataLength,                 // 95 Length FIX.2.7
tag::XmlDataLen,                    // 212 Length FIX.4.2
tag::EncodedIssuerLen,              // 348 Length FIX.4.2
tag::EncodedSecurityDescLen,        // 350 Length FIX.4.2
tag::EncodedListExecInstLen,        // 352 Length FIX.4.2
tag::EncodedTextLen,                // 354 Length FIX.4.2
tag::EncodedSubjectLen,             // 356 Length FIX.4.2
tag::EncodedHeadlineLen,            // 358 Length FIX.4.2
tag::EncodedAllocTextLen,           // 360 Length FIX.4.2
tag::EncodedUnderlyingIssuerLen,    // 362 Length FIX.4.2
tag::EncodedUnderlyingSecurityDescLen, // 364 Length FIX.4.2
tag::MaxMessageSize,                // 383 Length FIX.4.2
tag::EncodedListStatusTextLen,      // 445 Length FIX.4.2
tag::EncodedLegIssuerLen,           // 618 Length FIX.4.3
tag::EncodedLegSecurityDescLen,     // 621 Length FIX.4.3
tag::SecurityXMLLen,                // 1184 Length FIX.5.0
tag::DerivativeSecurityXMLLen,      // 1282 Length FIX.5.0
tag::EncodedMktSegmDescLen,         // 1397 Length FIX.5.0
tag::EncryptedPasswordLen,          // 1401 Length FIX.5.0
tag::EncryptedNewPasswordLen,       // 1403 Length FIX.5.0
tag::EncodedSecurityListDescLen,    // 1468 Length FIX.5.0SP1
tag::EncodedDocumentationTextLen,   // 1525 Length FIX.5.0SP2
tag::EncodedEventTextLen,           // 1578 Length FIX.5.0SP2
tag::InstrumentScopeEncodedSecurityDescLen, // 1620 Length FIX.5.0SP2
tag::EncodedRejectTextLen,          // 1664 Length FIX.5.0SP2
tag::EncodedOptionExpirationDescLen, // 1678 Length FIX.5.0SP2
tag::EncodedFirmAllocTextLen,       // 1733 Length FIX.5.0SP2
tag::LegSecurityXMLLen,             // 1871 Length FIX.5.0SP2
tag::UnderlyingSecurityXMLLen,      // 1874 Length FIX.5.0SP2
tag::EncodedUnderlyingEventTextLen, // 2072 Length FIX.5.0SP2
tag::EncodedLegEventTextLen,        // 2074 Length FIX.5.0SP2
tag::EncodedAttachmentLen,          // 2111 Length FIX.5.0SP2
tag::EncodedLegOptionExpirationDescLen, // 2179 Length FIX.5.0SP2
tag::EncodedUnderlyingOptionExpirationDescLen, // 2287 Length FIX.5.0SP2
tag::EncodedComplianceTextLen,      // 2351 Length FIX.5.0SP2
tag::EncodedTradeContinuationTextLen, // 2372 Length FIX.5.0SP2
tag::EncodedMDStatisticDescLen,     // 2481 Length FIX.5.0SP2
tag::EncodedLegDocumentationTextLen, // 2494 Length FIX.5.0SP2
tag::EncodedWarningTextLen,         // 2522 Length FIX.5.0SP2
tag::EncodedMiscFeeSubTypeDescLen,  // 2637 Length FIX.5.0SP2
tag::EncodedCommissionDescLen,      // 2651 Length FIX.5.0SP2
tag::EncodedAllocCommissionDescLen, // 2665 Length FIX.5.0SP2
tag::EncodedFinancialInstrumentFullNameLen, // 2715 Length FIX.5.0SP2
tag::EncodedLegFinancialInstrumentFullNameLen, // 2718 Length FIX.5.0SP2
tag::EncodedUnderlyingFinancialInstrumentFullNameLen, // 2721 Length FIX.5.0SP2
tag::EncodedAdditionalTermBondDescLen, // 40004 Length FIX.5.0SP2
tag::EncodedAdditionalTermBondIssuerLen, // 40008 Length FIX.5.0SP2
tag::EncodedLegStreamTextLen,       // 40978 Length FIX.5.0SP2
tag::EncodedLegProvisionTextLen,    // 40980 Length FIX.5.0SP2
tag::EncodedStreamTextLen,          // 40982 Length FIX.5.0SP2
tag::EncodedPaymentTextLen,         // 40984 Length FIX.5.0SP2
tag::EncodedProvisionTextLen,       // 40986 Length FIX.5.0SP2
tag::EncodedUnderlyingStreamTextLen, // 40988 Length FIX.5.0SP2
tag::EncodedDeliveryStreamCycleDescLen, // 41083 Length FIX.5.0SP2
tag::EncodedMarketDisruptionFallbackUnderlierSecurityDescLen, // 41101 Length FIX.5.0SP2
tag::EncodedExerciseDescLen,        // 41107 Length FIX.5.0SP2
tag::EncodedStreamCommodityDescLen, // 41256 Length FIX.5.0SP2
tag::EncodedLegAdditionalTermBondDescLen, // 41320 Length FIX.5.0SP2
tag::EncodedLegAdditionalTermBondIssuerLen, // 41324 Length FIX.5.0SP2
tag::EncodedLegDeliveryStreamCycleDescLen, // 41458 Length FIX.5.0SP2
tag::EncodedLegMarketDisruptionFallbackUnderlierSecurityDescLen, // 41476 Length FIX.5.0SP2
tag::EncodedLegExerciseDescLen,     // 41482 Length FIX.5.0SP2
tag::EncodedLegStreamCommodityDescLen, // 41653 Length FIX.5.0SP2
tag::EncodedUnderlyingAdditionalTermBondDescLen, // 41710 Length FIX.5.0SP2
tag::EncodedUnderlyingDeliveryStreamCycleDescLen, // 41806 Length FIX.5.0SP2
tag::EncodedUnderlyingExerciseDescLen, // 41811 Length FIX.5.0SP2
tag::EncodedUnderlyingMarketDisruptionFallbackUnderlierSecurityDescLen, // 41873 Length FIX.5.0SP2
tag::EncodedUnderlyingStreamCommodityDescLen, // 41969 Length FIX.5.0SP2
tag::EncodedUnderlyingAdditionalTermBondIssuerLen, // 42025 Length FIX.5.0SP2
tag::EncodedUnderlyingProvisionTextLen, // 42171 Length FIX.5.0SP2
tag::LegPaymentStreamFormulaImageLength, // 42451 Length FIX.5.0SP2
tag::PaymentStreamFormulaImageLength, // 42652 Length FIX.5.0SP2
tag::UnderlyingPaymentStreamFormulaImageLength // 42947 Length FIX.5.0SP2
};
}

/*!
 * \brief Populate an AssociativeContainer with the names of all the FIX fields.
 *
 * \param dictionary A reference to an AssociativeContainer<int, std::string>
 */
 template <typename AssociativeContainer> void dictionary_init_field(AssociativeContainer& dictionary) {
dictionary[tag::Account]                           = "Account";                                         // (String FIX.2.7)
dictionary[tag::AdvId]                             = "AdvId";                                           // (String FIX.2.7)
dictionary[tag::AdvRefID]                          = "AdvRefID";                                        // (String FIX.2.7)
dictionary[tag::AdvSide]                           = "AdvSide";                                         // (char FIX.2.7)
dictionary[tag::AdvTransType]                      = "AdvTransType";                                    // (String FIX.2.7)
dictionary[tag::AvgPx]                             = "AvgPx";                                           // (Price FIX.2.7)
dictionary[tag::BeginSeqNo]                        = "BeginSeqNo";                                      // (SeqNum FIX.2.7)
dictionary[tag::BeginString]                       = "BeginString";                                     // (String FIX.2.7)
dictionary[tag::BodyLength]                        = "BodyLength";                                      // (Length FIX.2.7)
dictionary[tag::CheckSum]                          = "CheckSum";                                        // (String FIX.2.7)
dictionary[tag::ClOrdID]                           = "ClOrdID";                                         // (String FIX.2.7)
dictionary[tag::Commission]                        = "Commission";                                      // (Amt FIX.2.7)
dictionary[tag::CommType]                          = "CommType";                                        // (char FIX.2.7)
dictionary[tag::CumQty]                            = "CumQty";                                          // (Qty FIX.2.7)
dictionary[tag::Currency]                          = "Currency";                                        // (Currency FIX.2.7)
dictionary[tag::EndSeqNo]                          = "EndSeqNo";                                        // (SeqNum FIX.2.7)
dictionary[tag::ExecID]                            = "ExecID";                                          // (String FIX.2.7)
dictionary[tag::ExecInst]                          = "ExecInst";                                        // (MultipleCharValue FIX.2.7)
dictionary[tag::ExecRefID]                         = "ExecRefID";                                       // (String FIX.2.7)
dictionary[tag::ExecTransType]                     = "ExecTransType";                                   // (char FIX.4.2)
dictionary[tag::HandlInst]                         = "HandlInst";                                       // (char FIX.2.7)
dictionary[tag::SecurityIDSource]                  = "SecurityIDSource";                                // (String FIX.2.7)
dictionary[tag::IOIID]                             = "IOIID";                                           // (String FIX.2.7)
dictionary[tag::IOIQltyInd]                        = "IOIQltyInd";                                      // (char FIX.2.7)
dictionary[tag::IOIRefID]                          = "IOIRefID";                                        // (String FIX.2.7)
dictionary[tag::IOIQty]                            = "IOIQty";                                          // (String FIX.2.7)
dictionary[tag::IOITransType]                      = "IOITransType";                                    // (char FIX.2.7)
dictionary[tag::LastCapacity]                      = "LastCapacity";                                    // (char FIX.2.7)
dictionary[tag::LastMkt]                           = "LastMkt";                                         // (Exchange FIX.2.7)
dictionary[tag::LastPx]                            = "LastPx";                                          // (Price FIX.2.7)
dictionary[tag::LastQty]                           = "LastQty";                                         // (Qty FIX.2.7)
dictionary[tag::NoLinesOfText]                     = "NoLinesOfText";                                   // (NumInGroup FIX.2.7)
dictionary[tag::MsgSeqNum]                         = "MsgSeqNum";                                       // (SeqNum FIX.2.7)
dictionary[tag::MsgType]                           = "MsgType";                                         // (String FIX.2.7)
dictionary[tag::NewSeqNo]                          = "NewSeqNo";                                        // (SeqNum FIX.2.7)
dictionary[tag::OrderID]                           = "OrderID";                                         // (String FIX.2.7)
dictionary[tag::OrderQty]                          = "OrderQty";                                        // (Qty FIX.2.7)
dictionary[tag::OrdStatus]                         = "OrdStatus";                                       // (char FIX.2.7)
dictionary[tag::OrdType]                           = "OrdType";                                         // (char FIX.2.7)
dictionary[tag::OrigClOrdID]                       = "OrigClOrdID";                                     // (String FIX.2.7)
dictionary[tag::OrigTime]                          = "OrigTime";                                        // (UTCTimestamp FIX.2.7)
dictionary[tag::PossDupFlag]                       = "PossDupFlag";                                     // (Boolean FIX.2.7)
dictionary[tag::Price]                             = "Price";                                           // (Price FIX.2.7)
dictionary[tag::RefSeqNum]                         = "RefSeqNum";                                       // (SeqNum FIX.2.7)
dictionary[tag::SecurityID]                        = "SecurityID";                                      // (String FIX.2.7)
dictionary[tag::SenderCompID]                      = "SenderCompID";                                    // (String FIX.2.7)
dictionary[tag::SenderSubID]                       = "SenderSubID";                                     // (String FIX.2.7)
dictionary[tag::SendingTime]                       = "SendingTime";                                     // (UTCTimestamp FIX.2.7)
dictionary[tag::Quantity]                          = "Quantity";                                        // (Qty FIX.2.7)
dictionary[tag::Side]                              = "Side";                                            // (char FIX.2.7)
dictionary[tag::Symbol]                            = "Symbol";                                          // (String FIX.2.7)
dictionary[tag::TargetCompID]                      = "TargetCompID";                                    // (String FIX.2.7)
dictionary[tag::TargetSubID]                       = "TargetSubID";                                     // (String FIX.2.7)
dictionary[tag::Text]                              = "Text";                                            // (String FIX.2.7)
dictionary[tag::TimeInForce]                       = "TimeInForce";                                     // (char FIX.2.7)
dictionary[tag::TransactTime]                      = "TransactTime";                                    // (UTCTimestamp FIX.2.7)
dictionary[tag::Urgency]                           = "Urgency";                                         // (char FIX.2.7)
dictionary[tag::ValidUntilTime]                    = "ValidUntilTime";                                  // (UTCTimestamp FIX.2.7)
dictionary[tag::SettlType]                         = "SettlType";                                       // (String FIX.2.7)
dictionary[tag::SettlDate]                         = "SettlDate";                                       // (LocalMktDate FIX.2.7)
dictionary[tag::SymbolSfx]                         = "SymbolSfx";                                       // (String FIX.2.7)
dictionary[tag::ListID]                            = "ListID";                                          // (String FIX.2.7)
dictionary[tag::ListSeqNo]                         = "ListSeqNo";                                       // (int FIX.2.7)
dictionary[tag::TotNoOrders]                       = "TotNoOrders";                                     // (int FIX.2.7)
dictionary[tag::ListExecInst]                      = "ListExecInst";                                    // (String FIX.2.7)
dictionary[tag::AllocID]                           = "AllocID";                                         // (String FIX.2.7)
dictionary[tag::AllocTransType]                    = "AllocTransType";                                  // (char FIX.2.7)
dictionary[tag::RefAllocID]                        = "RefAllocID";                                      // (String FIX.2.7)
dictionary[tag::NoOrders]                          = "NoOrders";                                        // (NumInGroup FIX.2.7)
dictionary[tag::AvgPxPrecision]                    = "AvgPxPrecision";                                  // (int FIX.2.7)
dictionary[tag::TradeDate]                         = "TradeDate";                                       // (LocalMktDate FIX.2.7)
dictionary[tag::PositionEffect]                    = "PositionEffect";                                  // (char FIX.2.7)
dictionary[tag::NoAllocs]                          = "NoAllocs";                                        // (NumInGroup FIX.2.7)
dictionary[tag::AllocAccount]                      = "AllocAccount";                                    // (String FIX.2.7)
dictionary[tag::AllocQty]                          = "AllocQty";                                        // (Qty FIX.2.7)
dictionary[tag::ProcessCode]                       = "ProcessCode";                                     // (char FIX.2.7)
dictionary[tag::NoRpts]                            = "NoRpts";                                          // (int FIX.2.7)
dictionary[tag::RptSeq]                            = "RptSeq";                                          // (int FIX.2.7)
dictionary[tag::CxlQty]                            = "CxlQty";                                          // (Qty FIX.2.7)
dictionary[tag::NoDlvyInst]                        = "NoDlvyInst";                                      // (NumInGroup FIX.2.7)
dictionary[tag::AllocStatus]                       = "AllocStatus";                                     // (int FIX.2.7)
dictionary[tag::AllocRejCode]                      = "AllocRejCode";                                    // (int FIX.2.7)
dictionary[tag::Signature]                         = "Signature";                                       // (data FIX.2.7)
dictionary[tag::SecureDataLen]                     = "SecureDataLen";                                   // (Length FIX.2.7)
dictionary[tag::SecureData]                        = "SecureData";                                      // (data FIX.2.7)
dictionary[tag::SignatureLength]                   = "SignatureLength";                                 // (Length FIX.2.7)
dictionary[tag::EmailType]                         = "EmailType";                                       // (char FIX.2.7)
dictionary[tag::RawDataLength]                     = "RawDataLength";                                   // (Length FIX.2.7)
dictionary[tag::RawData]                           = "RawData";                                         // (data FIX.2.7)
dictionary[tag::PossResend]                        = "PossResend";                                      // (Boolean FIX.2.7)
dictionary[tag::EncryptMethod]                     = "EncryptMethod";                                   // (int FIX.2.7)
dictionary[tag::StopPx]                            = "StopPx";                                          // (Price FIX.2.7)
dictionary[tag::ExDestination]                     = "ExDestination";                                   // (Exchange FIX.2.7)
dictionary[tag::CxlRejReason]                      = "CxlRejReason";                                    // (int FIX.2.7)
dictionary[tag::OrdRejReason]                      = "OrdRejReason";                                    // (int FIX.2.7)
dictionary[tag::IOIQualifier]                      = "IOIQualifier";                                    // (char FIX.3.0)
dictionary[tag::Issuer]                            = "Issuer";                                          // (String FIX.3.0)
dictionary[tag::SecurityDesc]                      = "SecurityDesc";                                    // (String FIX.3.0)
dictionary[tag::HeartBtInt]                        = "HeartBtInt";                                      // (int FIX.3.0)
dictionary[tag::MinQty]                            = "MinQty";                                          // (Qty FIX.3.0)
dictionary[tag::MaxFloor]                          = "MaxFloor";                                        // (Qty FIX.3.0)
dictionary[tag::TestReqID]                         = "TestReqID";                                       // (String FIX.3.0)
dictionary[tag::ReportToExch]                      = "ReportToExch";                                    // (Boolean FIX.3.0)
dictionary[tag::LocateReqd]                        = "LocateReqd";                                      // (Boolean FIX.4.0)
dictionary[tag::OnBehalfOfCompID]                  = "OnBehalfOfCompID";                                // (String FIX.4.0)
dictionary[tag::OnBehalfOfSubID]                   = "OnBehalfOfSubID";                                 // (String FIX.4.0)
dictionary[tag::QuoteID]                           = "QuoteID";                                         // (String FIX.4.0)
dictionary[tag::NetMoney]                          = "NetMoney";                                        // (Amt FIX.4.0)
dictionary[tag::SettlCurrAmt]                      = "SettlCurrAmt";                                    // (Amt FIX.4.0)
dictionary[tag::SettlCurrency]                     = "SettlCurrency";                                   // (Currency FIX.4.0)
dictionary[tag::ForexReq]                          = "ForexReq";                                        // (Boolean FIX.4.0)
dictionary[tag::OrigSendingTime]                   = "OrigSendingTime";                                 // (UTCTimestamp FIX.4.0)
dictionary[tag::GapFillFlag]                       = "GapFillFlag";                                     // (Boolean FIX.4.0)
dictionary[tag::NoExecs]                           = "NoExecs";                                         // (NumInGroup FIX.4.0)
dictionary[tag::ExpireTime]                        = "ExpireTime";                                      // (UTCTimestamp FIX.4.0)
dictionary[tag::DKReason]                          = "DKReason";                                        // (char FIX.4.0)
dictionary[tag::DeliverToCompID]                   = "DeliverToCompID";                                 // (String FIX.4.0)
dictionary[tag::DeliverToSubID]                    = "DeliverToSubID";                                  // (String FIX.4.0)
dictionary[tag::IOINaturalFlag]                    = "IOINaturalFlag";                                  // (Boolean FIX.4.0)
dictionary[tag::QuoteReqID]                        = "QuoteReqID";                                      // (String FIX.4.0)
dictionary[tag::BidPx]                             = "BidPx";                                           // (Price FIX.4.0)
dictionary[tag::OfferPx]                           = "OfferPx";                                         // (Price FIX.4.0)
dictionary[tag::BidSize]                           = "BidSize";                                         // (Qty FIX.4.0)
dictionary[tag::OfferSize]                         = "OfferSize";                                       // (Qty FIX.4.0)
dictionary[tag::NoMiscFees]                        = "NoMiscFees";                                      // (NumInGroup FIX.4.0)
dictionary[tag::MiscFeeAmt]                        = "MiscFeeAmt";                                      // (Amt FIX.4.0)
dictionary[tag::MiscFeeCurr]                       = "MiscFeeCurr";                                     // (Currency FIX.4.0)
dictionary[tag::MiscFeeType]                       = "MiscFeeType";                                     // (String FIX.4.0)
dictionary[tag::PrevClosePx]                       = "PrevClosePx";                                     // (Price FIX.4.0)
dictionary[tag::ResetSeqNumFlag]                   = "ResetSeqNumFlag";                                 // (Boolean FIX.4.1)
dictionary[tag::SenderLocationID]                  = "SenderLocationID";                                // (String FIX.4.1)
dictionary[tag::TargetLocationID]                  = "TargetLocationID";                                // (String FIX.4.1)
dictionary[tag::OnBehalfOfLocationID]              = "OnBehalfOfLocationID";                            // (String FIX.4.1)
dictionary[tag::DeliverToLocationID]               = "DeliverToLocationID";                             // (String FIX.4.1)
dictionary[tag::NoRelatedSym]                      = "NoRelatedSym";                                    // (NumInGroup FIX.4.1)
dictionary[tag::Subject]                           = "Subject";                                         // (String FIX.4.1)
dictionary[tag::Headline]                          = "Headline";                                        // (String FIX.4.1)
dictionary[tag::URLLink]                           = "URLLink";                                         // (String FIX.4.1)
dictionary[tag::ExecType]                          = "ExecType";                                        // (char FIX.4.1)
dictionary[tag::LeavesQty]                         = "LeavesQty";                                       // (Qty FIX.4.1)
dictionary[tag::CashOrderQty]                      = "CashOrderQty";                                    // (Qty FIX.4.1)
dictionary[tag::AllocAvgPx]                        = "AllocAvgPx";                                      // (Price FIX.4.1)
dictionary[tag::AllocNetMoney]                     = "AllocNetMoney";                                   // (Amt FIX.4.1)
dictionary[tag::SettlCurrFxRate]                   = "SettlCurrFxRate";                                 // (float FIX.4.1)
dictionary[tag::SettlCurrFxRateCalc]               = "SettlCurrFxRateCalc";                             // (char FIX.4.1)
dictionary[tag::NumDaysInterest]                   = "NumDaysInterest";                                 // (int FIX.4.1)
dictionary[tag::AccruedInterestRate]               = "AccruedInterestRate";                             // (Percentage FIX.4.1)
dictionary[tag::AccruedInterestAmt]                = "AccruedInterestAmt";                              // (Amt FIX.4.1)
dictionary[tag::SettlInstMode]                     = "SettlInstMode";                                   // (char FIX.4.1)
dictionary[tag::AllocText]                         = "AllocText";                                       // (String FIX.4.1)
dictionary[tag::SettlInstID]                       = "SettlInstID";                                     // (String FIX.4.1)
dictionary[tag::SettlInstTransType]                = "SettlInstTransType";                              // (char FIX.4.1)
dictionary[tag::EmailThreadID]                     = "EmailThreadID";                                   // (String FIX.4.1)
dictionary[tag::SettlInstSource]                   = "SettlInstSource";                                 // (char FIX.4.1)
dictionary[tag::SecurityType]                      = "SecurityType";                                    // (String FIX.4.1)
dictionary[tag::EffectiveTime]                     = "EffectiveTime";                                   // (UTCTimestamp FIX.4.1)
dictionary[tag::StandInstDbType]                   = "StandInstDbType";                                 // (int FIX.4.1)
dictionary[tag::StandInstDbName]                   = "StandInstDbName";                                 // (String FIX.4.1)
dictionary[tag::StandInstDbID]                     = "StandInstDbID";                                   // (String FIX.4.1)
dictionary[tag::SettlDeliveryType]                 = "SettlDeliveryType";                               // (int FIX.4.1)
dictionary[tag::BidSpotRate]                       = "BidSpotRate";                                     // (Price FIX.4.1)
dictionary[tag::BidForwardPoints]                  = "BidForwardPoints";                                // (PriceOffset FIX.4.1)
dictionary[tag::OfferSpotRate]                     = "OfferSpotRate";                                   // (Price FIX.4.1)
dictionary[tag::OfferForwardPoints]                = "OfferForwardPoints";                              // (PriceOffset FIX.4.1)
dictionary[tag::OrderQty2]                         = "OrderQty2";                                       // (Qty FIX.4.1)
dictionary[tag::SettlDate2]                        = "SettlDate2";                                      // (LocalMktDate FIX.4.1)
dictionary[tag::LastSpotRate]                      = "LastSpotRate";                                    // (Price FIX.4.1)
dictionary[tag::LastForwardPoints]                 = "LastForwardPoints";                               // (PriceOffset FIX.4.1)
dictionary[tag::AllocLinkID]                       = "AllocLinkID";                                     // (String FIX.4.1)
dictionary[tag::AllocLinkType]                     = "AllocLinkType";                                   // (int FIX.4.1)
dictionary[tag::SecondaryOrderID]                  = "SecondaryOrderID";                                // (String FIX.4.1)
dictionary[tag::NoIOIQualifiers]                   = "NoIOIQualifiers";                                 // (NumInGroup FIX.4.1)
dictionary[tag::MaturityMonthYear]                 = "MaturityMonthYear";                               // (MonthYear FIX.4.1)
dictionary[tag::PutOrCall]                         = "PutOrCall";                                       // (int FIX.4.1)
dictionary[tag::StrikePrice]                       = "StrikePrice";                                     // (Price FIX.4.1)
dictionary[tag::CoveredOrUncovered]                = "CoveredOrUncovered";                              // (int FIX.4.1)
dictionary[tag::OptAttribute]                      = "OptAttribute";                                    // (char FIX.4.1)
dictionary[tag::SecurityExchange]                  = "SecurityExchange";                                // (Exchange FIX.4.1)
dictionary[tag::NotifyBrokerOfCredit]              = "NotifyBrokerOfCredit";                            // (Boolean FIX.4.1)
dictionary[tag::AllocHandlInst]                    = "AllocHandlInst";                                  // (int FIX.4.1)
dictionary[tag::MaxShow]                           = "MaxShow";                                         // (Qty FIX.4.1)
dictionary[tag::PegOffsetValue]                    = "PegOffsetValue";                                  // (float FIX.4.1)
dictionary[tag::XmlDataLen]                        = "XmlDataLen";                                      // (Length FIX.4.2)
dictionary[tag::XmlData]                           = "XmlData";                                         // (data FIX.4.2)
dictionary[tag::SettlInstRefID]                    = "SettlInstRefID";                                  // (String FIX.4.2)
dictionary[tag::NoRoutingIDs]                      = "NoRoutingIDs";                                    // (NumInGroup FIX.4.2)
dictionary[tag::RoutingType]                       = "RoutingType";                                     // (int FIX.4.2)
dictionary[tag::RoutingID]                         = "RoutingID";                                       // (String FIX.4.2)
dictionary[tag::Spread]                            = "Spread";                                          // (PriceOffset FIX.4.2)
dictionary[tag::BenchmarkCurveCurrency]            = "BenchmarkCurveCurrency";                          // (Currency FIX.4.2)
dictionary[tag::BenchmarkCurveName]                = "BenchmarkCurveName";                              // (String FIX.4.2)
dictionary[tag::BenchmarkCurvePoint]               = "BenchmarkCurvePoint";                             // (String FIX.4.2)
dictionary[tag::CouponRate]                        = "CouponRate";                                      // (Percentage FIX.4.2)
dictionary[tag::CouponPaymentDate]                 = "CouponPaymentDate";                               // (LocalMktDate FIX.4.2)
dictionary[tag::IssueDate]                         = "IssueDate";                                       // (LocalMktDate FIX.4.2)
dictionary[tag::RepurchaseTerm]                    = "RepurchaseTerm";                                  // (int FIX.4.2)
dictionary[tag::RepurchaseRate]                    = "RepurchaseRate";                                  // (Percentage FIX.4.2)
dictionary[tag::Factor]                            = "Factor";                                          // (float FIX.4.2)
dictionary[tag::TradeOriginationDate]              = "TradeOriginationDate";                            // (LocalMktDate FIX.4.2)
dictionary[tag::ExDate]                            = "ExDate";                                          // (LocalMktDate FIX.4.2)
dictionary[tag::ContractMultiplier]                = "ContractMultiplier";                              // (float FIX.4.2)
dictionary[tag::NoStipulations]                    = "NoStipulations";                                  // (NumInGroup FIX.4.2)
dictionary[tag::StipulationType]                   = "StipulationType";                                 // (String FIX.4.2)
dictionary[tag::StipulationValue]                  = "StipulationValue";                                // (String FIX.4.2)
dictionary[tag::YieldType]                         = "YieldType";                                       // (String FIX.4.2)
dictionary[tag::Yield]                             = "Yield";                                           // (Percentage FIX.4.2)
dictionary[tag::TotalTakedown]                     = "TotalTakedown";                                   // (Amt FIX.4.2)
dictionary[tag::Concession]                        = "Concession";                                      // (Amt FIX.4.2)
dictionary[tag::RepoCollateralSecurityType]        = "RepoCollateralSecurityType";                      // (String FIX.4.3)
dictionary[tag::RedemptionDate]                    = "RedemptionDate";                                  // (LocalMktDate FIX.4.2)
dictionary[tag::UnderlyingCouponPaymentDate]       = "UnderlyingCouponPaymentDate";                     // (LocalMktDate FIX.4.2)
dictionary[tag::UnderlyingIssueDate]               = "UnderlyingIssueDate";                             // (LocalMktDate FIX.4.2)
dictionary[tag::UnderlyingRepoCollateralSecurityType] = "UnderlyingRepoCollateralSecurityType";            // (String FIX.4.3)
dictionary[tag::UnderlyingRepurchaseTerm]          = "UnderlyingRepurchaseTerm";                        // (int FIX.4.2)
dictionary[tag::UnderlyingRepurchaseRate]          = "UnderlyingRepurchaseRate";                        // (Percentage FIX.4.2)
dictionary[tag::UnderlyingFactor]                  = "UnderlyingFactor";                                // (float FIX.4.2)
dictionary[tag::UnderlyingRedemptionDate]          = "UnderlyingRedemptionDate";                        // (LocalMktDate FIX.4.2)
dictionary[tag::LegCouponPaymentDate]              = "LegCouponPaymentDate";                            // (LocalMktDate FIX.4.2)
dictionary[tag::LegIssueDate]                      = "LegIssueDate";                                    // (LocalMktDate FIX.4.2)
dictionary[tag::LegRepoCollateralSecurityType]     = "LegRepoCollateralSecurityType";                   // (String FIX.4.3)
dictionary[tag::LegRepurchaseTerm]                 = "LegRepurchaseTerm";                               // (int FIX.4.2)
dictionary[tag::LegRepurchaseRate]                 = "LegRepurchaseRate";                               // (Percentage FIX.4.2)
dictionary[tag::LegFactor]                         = "LegFactor";                                       // (float FIX.4.2)
dictionary[tag::LegRedemptionDate]                 = "LegRedemptionDate";                               // (LocalMktDate FIX.4.2)
dictionary[tag::CreditRating]                      = "CreditRating";                                    // (String FIX.4.2)
dictionary[tag::UnderlyingCreditRating]            = "UnderlyingCreditRating";                          // (String FIX.4.2)
dictionary[tag::LegCreditRating]                   = "LegCreditRating";                                 // (String FIX.4.2)
dictionary[tag::TradedFlatSwitch]                  = "TradedFlatSwitch";                                // (Boolean FIX.4.2)
dictionary[tag::BasisFeatureDate]                  = "BasisFeatureDate";                                // (LocalMktDate FIX.4.2)
dictionary[tag::BasisFeaturePrice]                 = "BasisFeaturePrice";                               // (Price FIX.4.2)
dictionary[tag::MDReqID]                           = "MDReqID";                                         // (String FIX.4.2)
dictionary[tag::SubscriptionRequestType]           = "SubscriptionRequestType";                         // (char FIX.4.2)
dictionary[tag::MarketDepth]                       = "MarketDepth";                                     // (int FIX.4.2)
dictionary[tag::MDUpdateType]                      = "MDUpdateType";                                    // (int FIX.4.2)
dictionary[tag::AggregatedBook]                    = "AggregatedBook";                                  // (Boolean FIX.4.2)
dictionary[tag::NoMDEntryTypes]                    = "NoMDEntryTypes";                                  // (NumInGroup FIX.4.2)
dictionary[tag::NoMDEntries]                       = "NoMDEntries";                                     // (NumInGroup FIX.4.2)
dictionary[tag::MDEntryType]                       = "MDEntryType";                                     // (char FIX.4.2)
dictionary[tag::MDEntryPx]                         = "MDEntryPx";                                       // (Price FIX.4.2)
dictionary[tag::MDEntrySize]                       = "MDEntrySize";                                     // (Qty FIX.4.2)
dictionary[tag::MDEntryDate]                       = "MDEntryDate";                                     // (UTCDateOnly FIX.4.2)
dictionary[tag::MDEntryTime]                       = "MDEntryTime";                                     // (UTCTimeOnly FIX.4.2)
dictionary[tag::TickDirection]                     = "TickDirection";                                   // (char FIX.4.2)
dictionary[tag::MDMkt]                             = "MDMkt";                                           // (Exchange FIX.4.2)
dictionary[tag::QuoteCondition]                    = "QuoteCondition";                                  // (MultipleStringValue FIX.4.2)
dictionary[tag::TradeCondition]                    = "TradeCondition";                                  // (MultipleStringValue FIX.4.2)
dictionary[tag::MDEntryID]                         = "MDEntryID";                                       // (String FIX.4.2)
dictionary[tag::MDUpdateAction]                    = "MDUpdateAction";                                  // (char FIX.4.2)
dictionary[tag::MDEntryRefID]                      = "MDEntryRefID";                                    // (String FIX.4.2)
dictionary[tag::MDReqRejReason]                    = "MDReqRejReason";                                  // (char FIX.4.2)
dictionary[tag::MDEntryOriginator]                 = "MDEntryOriginator";                               // (String FIX.4.2)
dictionary[tag::LocationID]                        = "LocationID";                                      // (String FIX.4.2)
dictionary[tag::DeskID]                            = "DeskID";                                          // (String FIX.4.2)
dictionary[tag::DeleteReason]                      = "DeleteReason";                                    // (char FIX.4.2)
dictionary[tag::OpenCloseSettlFlag]                = "OpenCloseSettlFlag";                              // (MultipleCharValue FIX.4.2)
dictionary[tag::SellerDays]                        = "SellerDays";                                      // (int FIX.4.2)
dictionary[tag::MDEntryBuyer]                      = "MDEntryBuyer";                                    // (String FIX.4.2)
dictionary[tag::MDEntrySeller]                     = "MDEntrySeller";                                   // (String FIX.4.2)
dictionary[tag::MDEntryPositionNo]                 = "MDEntryPositionNo";                               // (int FIX.4.2)
dictionary[tag::FinancialStatus]                   = "FinancialStatus";                                 // (MultipleCharValue FIX.4.2)
dictionary[tag::CorporateAction]                   = "CorporateAction";                                 // (MultipleCharValue FIX.4.2)
dictionary[tag::DefBidSize]                        = "DefBidSize";                                      // (Qty FIX.4.2)
dictionary[tag::DefOfferSize]                      = "DefOfferSize";                                    // (Qty FIX.4.2)
dictionary[tag::NoQuoteEntries]                    = "NoQuoteEntries";                                  // (NumInGroup FIX.4.2)
dictionary[tag::NoQuoteSets]                       = "NoQuoteSets";                                     // (NumInGroup FIX.4.2)
dictionary[tag::QuoteStatus]                       = "QuoteStatus";                                     // (int FIX.4.2)
dictionary[tag::QuoteCancelType]                   = "QuoteCancelType";                                 // (int FIX.4.2)
dictionary[tag::QuoteEntryID]                      = "QuoteEntryID";                                    // (String FIX.4.2)
dictionary[tag::QuoteRejectReason]                 = "QuoteRejectReason";                               // (int FIX.4.2)
dictionary[tag::QuoteResponseLevel]                = "QuoteResponseLevel";                              // (int FIX.4.2)
dictionary[tag::QuoteSetID]                        = "QuoteSetID";                                      // (String FIX.4.2)
dictionary[tag::QuoteRequestType]                  = "QuoteRequestType";                                // (int FIX.4.2)
dictionary[tag::TotNoQuoteEntries]                 = "TotNoQuoteEntries";                               // (int FIX.4.2)
dictionary[tag::UnderlyingSecurityIDSource]        = "UnderlyingSecurityIDSource";                      // (String FIX.4.2)
dictionary[tag::UnderlyingIssuer]                  = "UnderlyingIssuer";                                // (String FIX.4.2)
dictionary[tag::UnderlyingSecurityDesc]            = "UnderlyingSecurityDesc";                          // (String FIX.4.2)
dictionary[tag::UnderlyingSecurityExchange]        = "UnderlyingSecurityExchange";                      // (Exchange FIX.4.2)
dictionary[tag::UnderlyingSecurityID]              = "UnderlyingSecurityID";                            // (String FIX.4.2)
dictionary[tag::UnderlyingSecurityType]            = "UnderlyingSecurityType";                          // (String FIX.4.2)
dictionary[tag::UnderlyingSymbol]                  = "UnderlyingSymbol";                                // (String FIX.4.2)
dictionary[tag::UnderlyingSymbolSfx]               = "UnderlyingSymbolSfx";                             // (String FIX.4.2)
dictionary[tag::UnderlyingMaturityMonthYear]       = "UnderlyingMaturityMonthYear";                     // (MonthYear FIX.4.2)
dictionary[tag::UnderlyingPutOrCall]               = "UnderlyingPutOrCall";                             // (int FIX.4.2)
dictionary[tag::UnderlyingStrikePrice]             = "UnderlyingStrikePrice";                           // (Price FIX.4.2)
dictionary[tag::UnderlyingOptAttribute]            = "UnderlyingOptAttribute";                          // (char FIX.4.2)
dictionary[tag::UnderlyingCurrency]                = "UnderlyingCurrency";                              // (Currency FIX.4.2)
dictionary[tag::SecurityReqID]                     = "SecurityReqID";                                   // (String FIX.4.2)
dictionary[tag::SecurityRequestType]               = "SecurityRequestType";                             // (int FIX.4.2)
dictionary[tag::SecurityResponseID]                = "SecurityResponseID";                              // (String FIX.4.2)
dictionary[tag::SecurityResponseType]              = "SecurityResponseType";                            // (int FIX.4.2)
dictionary[tag::SecurityStatusReqID]               = "SecurityStatusReqID";                             // (String FIX.4.2)
dictionary[tag::UnsolicitedIndicator]              = "UnsolicitedIndicator";                            // (Boolean FIX.4.2)
dictionary[tag::SecurityTradingStatus]             = "SecurityTradingStatus";                           // (int FIX.4.2)
dictionary[tag::HaltReason]                        = "HaltReason";                                      // (int FIX.4.2)
dictionary[tag::InViewOfCommon]                    = "InViewOfCommon";                                  // (Boolean FIX.4.2)
dictionary[tag::DueToRelated]                      = "DueToRelated";                                    // (Boolean FIX.4.2)
dictionary[tag::BuyVolume]                         = "BuyVolume";                                       // (Qty FIX.4.2)
dictionary[tag::SellVolume]                        = "SellVolume";                                      // (Qty FIX.4.2)
dictionary[tag::HighPx]                            = "HighPx";                                          // (Price FIX.4.2)
dictionary[tag::LowPx]                             = "LowPx";                                           // (Price FIX.4.2)
dictionary[tag::Adjustment]                        = "Adjustment";                                      // (int FIX.4.2)
dictionary[tag::TradSesReqID]                      = "TradSesReqID";                                    // (String FIX.4.2)
dictionary[tag::TradingSessionID]                  = "TradingSessionID";                                // (String FIX.4.2)
dictionary[tag::ContraTrader]                      = "ContraTrader";                                    // (String FIX.4.2)
dictionary[tag::TradSesMethod]                     = "TradSesMethod";                                   // (int FIX.4.2)
dictionary[tag::TradSesMode]                       = "TradSesMode";                                     // (int FIX.4.2)
dictionary[tag::TradSesStatus]                     = "TradSesStatus";                                   // (int FIX.4.2)
dictionary[tag::TradSesStartTime]                  = "TradSesStartTime";                                // (UTCTimestamp FIX.4.2)
dictionary[tag::TradSesOpenTime]                   = "TradSesOpenTime";                                 // (UTCTimestamp FIX.4.2)
dictionary[tag::TradSesPreCloseTime]               = "TradSesPreCloseTime";                             // (UTCTimestamp FIX.4.2)
dictionary[tag::TradSesCloseTime]                  = "TradSesCloseTime";                                // (UTCTimestamp FIX.4.2)
dictionary[tag::TradSesEndTime]                    = "TradSesEndTime";                                  // (UTCTimestamp FIX.4.2)
dictionary[tag::NumberOfOrders]                    = "NumberOfOrders";                                  // (int FIX.4.2)
dictionary[tag::MessageEncoding]                   = "MessageEncoding";                                 // (String FIX.4.2)
dictionary[tag::EncodedIssuerLen]                  = "EncodedIssuerLen";                                // (Length FIX.4.2)
dictionary[tag::EncodedIssuer]                     = "EncodedIssuer";                                   // (data FIX.4.2)
dictionary[tag::EncodedSecurityDescLen]            = "EncodedSecurityDescLen";                          // (Length FIX.4.2)
dictionary[tag::EncodedSecurityDesc]               = "EncodedSecurityDesc";                             // (data FIX.4.2)
dictionary[tag::EncodedListExecInstLen]            = "EncodedListExecInstLen";                          // (Length FIX.4.2)
dictionary[tag::EncodedListExecInst]               = "EncodedListExecInst";                             // (data FIX.4.2)
dictionary[tag::EncodedTextLen]                    = "EncodedTextLen";                                  // (Length FIX.4.2)
dictionary[tag::EncodedText]                       = "EncodedText";                                     // (data FIX.4.2)
dictionary[tag::EncodedSubjectLen]                 = "EncodedSubjectLen";                               // (Length FIX.4.2)
dictionary[tag::EncodedSubject]                    = "EncodedSubject";                                  // (data FIX.4.2)
dictionary[tag::EncodedHeadlineLen]                = "EncodedHeadlineLen";                              // (Length FIX.4.2)
dictionary[tag::EncodedHeadline]                   = "EncodedHeadline";                                 // (data FIX.4.2)
dictionary[tag::EncodedAllocTextLen]               = "EncodedAllocTextLen";                             // (Length FIX.4.2)
dictionary[tag::EncodedAllocText]                  = "EncodedAllocText";                                // (data FIX.4.2)
dictionary[tag::EncodedUnderlyingIssuerLen]        = "EncodedUnderlyingIssuerLen";                      // (Length FIX.4.2)
dictionary[tag::EncodedUnderlyingIssuer]           = "EncodedUnderlyingIssuer";                         // (data FIX.4.2)
dictionary[tag::EncodedUnderlyingSecurityDescLen]  = "EncodedUnderlyingSecurityDescLen";                // (Length FIX.4.2)
dictionary[tag::EncodedUnderlyingSecurityDesc]     = "EncodedUnderlyingSecurityDesc";                   // (data FIX.4.2)
dictionary[tag::AllocPrice]                        = "AllocPrice";                                      // (Price FIX.4.2)
dictionary[tag::QuoteSetValidUntilTime]            = "QuoteSetValidUntilTime";                          // (UTCTimestamp FIX.4.2)
dictionary[tag::QuoteEntryRejectReason]            = "QuoteEntryRejectReason";                          // (int FIX.4.2)
dictionary[tag::LastMsgSeqNumProcessed]            = "LastMsgSeqNumProcessed";                          // (SeqNum FIX.4.2)
dictionary[tag::RefTagID]                          = "RefTagID";                                        // (int FIX.4.2)
dictionary[tag::RefMsgType]                        = "RefMsgType";                                      // (String FIX.4.2)
dictionary[tag::SessionRejectReason]               = "SessionRejectReason";                             // (int FIX.4.2)
dictionary[tag::BidRequestTransType]               = "BidRequestTransType";                             // (char FIX.4.2)
dictionary[tag::ContraBroker]                      = "ContraBroker";                                    // (String FIX.4.2)
dictionary[tag::ComplianceID]                      = "ComplianceID";                                    // (String FIX.4.2)
dictionary[tag::SolicitedFlag]                     = "SolicitedFlag";                                   // (Boolean FIX.4.2)
dictionary[tag::ExecRestatementReason]             = "ExecRestatementReason";                           // (int FIX.4.2)
dictionary[tag::BusinessRejectRefID]               = "BusinessRejectRefID";                             // (String FIX.4.2)
dictionary[tag::BusinessRejectReason]              = "BusinessRejectReason";                            // (int FIX.4.2)
dictionary[tag::GrossTradeAmt]                     = "GrossTradeAmt";                                   // (Amt FIX.4.2)
dictionary[tag::NoContraBrokers]                   = "NoContraBrokers";                                 // (NumInGroup FIX.4.2)
dictionary[tag::MaxMessageSize]                    = "MaxMessageSize";                                  // (Length FIX.4.2)
dictionary[tag::NoMsgTypes]                        = "NoMsgTypes";                                      // (NumInGroup FIX.4.2)
dictionary[tag::MsgDirection]                      = "MsgDirection";                                    // (char FIX.4.2)
dictionary[tag::NoTradingSessions]                 = "NoTradingSessions";                               // (NumInGroup FIX.4.2)
dictionary[tag::TotalVolumeTraded]                 = "TotalVolumeTraded";                               // (Qty FIX.4.2)
dictionary[tag::DiscretionInst]                    = "DiscretionInst";                                  // (char FIX.4.2)
dictionary[tag::DiscretionOffsetValue]             = "DiscretionOffsetValue";                           // (float FIX.4.2)
dictionary[tag::BidID]                             = "BidID";                                           // (String FIX.4.2)
dictionary[tag::ClientBidID]                       = "ClientBidID";                                     // (String FIX.4.2)
dictionary[tag::ListName]                          = "ListName";                                        // (String FIX.4.2)
dictionary[tag::TotNoRelatedSym]                   = "TotNoRelatedSym";                                 // (int FIX.4.2)
dictionary[tag::BidType]                           = "BidType";                                         // (int FIX.4.2)
dictionary[tag::NumTickets]                        = "NumTickets";                                      // (int FIX.4.2)
dictionary[tag::SideValue1]                        = "SideValue1";                                      // (Amt FIX.4.2)
dictionary[tag::SideValue2]                        = "SideValue2";                                      // (Amt FIX.4.2)
dictionary[tag::NoBidDescriptors]                  = "NoBidDescriptors";                                // (NumInGroup FIX.4.2)
dictionary[tag::BidDescriptorType]                 = "BidDescriptorType";                               // (int FIX.4.2)
dictionary[tag::BidDescriptor]                     = "BidDescriptor";                                   // (String FIX.4.2)
dictionary[tag::SideValueInd]                      = "SideValueInd";                                    // (int FIX.4.2)
dictionary[tag::LiquidityPctLow]                   = "LiquidityPctLow";                                 // (Percentage FIX.4.2)
dictionary[tag::LiquidityPctHigh]                  = "LiquidityPctHigh";                                // (Percentage FIX.4.2)
dictionary[tag::LiquidityValue]                    = "LiquidityValue";                                  // (Amt FIX.4.2)
dictionary[tag::EFPTrackingError]                  = "EFPTrackingError";                                // (Percentage FIX.4.2)
dictionary[tag::FairValue]                         = "FairValue";                                       // (Amt FIX.4.2)
dictionary[tag::OutsideIndexPct]                   = "OutsideIndexPct";                                 // (Percentage FIX.4.2)
dictionary[tag::ValueOfFutures]                    = "ValueOfFutures";                                  // (Amt FIX.4.2)
dictionary[tag::LiquidityIndType]                  = "LiquidityIndType";                                // (int FIX.4.2)
dictionary[tag::WtAverageLiquidity]                = "WtAverageLiquidity";                              // (Percentage FIX.4.2)
dictionary[tag::ExchangeForPhysical]               = "ExchangeForPhysical";                             // (Boolean FIX.4.2)
dictionary[tag::OutMainCntryUIndex]                = "OutMainCntryUIndex";                              // (Amt FIX.4.2)
dictionary[tag::CrossPercent]                      = "CrossPercent";                                    // (Percentage FIX.4.2)
dictionary[tag::ProgRptReqs]                       = "ProgRptReqs";                                     // (int FIX.4.2)
dictionary[tag::ProgPeriodInterval]                = "ProgPeriodInterval";                              // (int FIX.4.2)
dictionary[tag::IncTaxInd]                         = "IncTaxInd";                                       // (int FIX.4.2)
dictionary[tag::NumBidders]                        = "NumBidders";                                      // (int FIX.4.2)
dictionary[tag::BidTradeType]                      = "BidTradeType";                                    // (char FIX.4.2)
dictionary[tag::BasisPxType]                       = "BasisPxType";                                     // (char FIX.4.2)
dictionary[tag::NoBidComponents]                   = "NoBidComponents";                                 // (NumInGroup FIX.4.2)
dictionary[tag::Country]                           = "Country";                                         // (Country FIX.4.2)
dictionary[tag::TotNoStrikes]                      = "TotNoStrikes";                                    // (int FIX.4.2)
dictionary[tag::PriceType]                         = "PriceType";                                       // (int FIX.4.2)
dictionary[tag::DayOrderQty]                       = "DayOrderQty";                                     // (Qty FIX.4.2)
dictionary[tag::DayCumQty]                         = "DayCumQty";                                       // (Qty FIX.4.2)
dictionary[tag::DayAvgPx]                          = "DayAvgPx";                                        // (Price FIX.4.2)
dictionary[tag::GTBookingInst]                     = "GTBookingInst";                                   // (int FIX.4.2)
dictionary[tag::NoStrikes]                         = "NoStrikes";                                       // (NumInGroup FIX.4.2)
dictionary[tag::ListStatusType]                    = "ListStatusType";                                  // (int FIX.4.2)
dictionary[tag::NetGrossInd]                       = "NetGrossInd";                                     // (int FIX.4.2)
dictionary[tag::ListOrderStatus]                   = "ListOrderStatus";                                 // (int FIX.4.2)
dictionary[tag::ExpireDate]                        = "ExpireDate";                                      // (LocalMktDate FIX.4.2)
dictionary[tag::ListExecInstType]                  = "ListExecInstType";                                // (char FIX.4.2)
dictionary[tag::CxlRejResponseTo]                  = "CxlRejResponseTo";                                // (char FIX.4.2)
dictionary[tag::UnderlyingCouponRate]              = "UnderlyingCouponRate";                            // (Percentage FIX.4.2)
dictionary[tag::UnderlyingContractMultiplier]      = "UnderlyingContractMultiplier";                    // (float FIX.4.2)
dictionary[tag::ContraTradeQty]                    = "ContraTradeQty";                                  // (Qty FIX.4.2)
dictionary[tag::ContraTradeTime]                   = "ContraTradeTime";                                 // (UTCTimestamp FIX.4.2)
dictionary[tag::LiquidityNumSecurities]            = "LiquidityNumSecurities";                          // (int FIX.4.2)
dictionary[tag::MultiLegReportingType]             = "MultiLegReportingType";                           // (char FIX.4.2)
dictionary[tag::StrikeTime]                        = "StrikeTime";                                      // (UTCTimestamp FIX.4.2)
dictionary[tag::ListStatusText]                    = "ListStatusText";                                  // (String FIX.4.2)
dictionary[tag::EncodedListStatusTextLen]          = "EncodedListStatusTextLen";                        // (Length FIX.4.2)
dictionary[tag::EncodedListStatusText]             = "EncodedListStatusText";                           // (data FIX.4.2)
dictionary[tag::PartyIDSource]                     = "PartyIDSource";                                   // (char FIX.4.3)
dictionary[tag::PartyID]                           = "PartyID";                                         // (String FIX.4.3)
dictionary[tag::NetChgPrevDay]                     = "NetChgPrevDay";                                   // (PriceOffset FIX.4.3)
dictionary[tag::PartyRole]                         = "PartyRole";                                       // (int FIX.4.3)
dictionary[tag::NoPartyIDs]                        = "NoPartyIDs";                                      // (NumInGroup FIX.4.3)
dictionary[tag::NoSecurityAltID]                   = "NoSecurityAltID";                                 // (NumInGroup FIX.4.3)
dictionary[tag::SecurityAltID]                     = "SecurityAltID";                                   // (String FIX.4.3)
dictionary[tag::SecurityAltIDSource]               = "SecurityAltIDSource";                             // (String FIX.4.3)
dictionary[tag::NoUnderlyingSecurityAltID]         = "NoUnderlyingSecurityAltID";                       // (NumInGroup FIX.4.3)
dictionary[tag::UnderlyingSecurityAltID]           = "UnderlyingSecurityAltID";                         // (String FIX.4.3)
dictionary[tag::UnderlyingSecurityAltIDSource]     = "UnderlyingSecurityAltIDSource";                   // (String FIX.4.3)
dictionary[tag::Product]                           = "Product";                                         // (int FIX.4.3)
dictionary[tag::CFICode]                           = "CFICode";                                         // (String FIX.4.3)
dictionary[tag::UnderlyingProduct]                 = "UnderlyingProduct";                               // (int FIX.4.3)
dictionary[tag::UnderlyingCFICode]                 = "UnderlyingCFICode";                               // (String FIX.4.3)
dictionary[tag::TestMessageIndicator]              = "TestMessageIndicator";                            // (Boolean FIX.4.3)
dictionary[tag::BookingRefID]                      = "BookingRefID";                                    // (String FIX.4.3)
dictionary[tag::IndividualAllocID]                 = "IndividualAllocID";                               // (String FIX.4.3)
dictionary[tag::RoundingDirection]                 = "RoundingDirection";                               // (char FIX.4.3)
dictionary[tag::RoundingModulus]                   = "RoundingModulus";                                 // (float FIX.4.3)
dictionary[tag::CountryOfIssue]                    = "CountryOfIssue";                                  // (Country FIX.4.3)
dictionary[tag::StateOrProvinceOfIssue]            = "StateOrProvinceOfIssue";                          // (String FIX.4.3)
dictionary[tag::LocaleOfIssue]                     = "LocaleOfIssue";                                   // (String FIX.4.3)
dictionary[tag::NoRegistDtls]                      = "NoRegistDtls";                                    // (NumInGroup FIX.4.3)
dictionary[tag::MailingDtls]                       = "MailingDtls";                                     // (String FIX.4.3)
dictionary[tag::InvestorCountryOfResidence]        = "InvestorCountryOfResidence";                      // (Country FIX.4.3)
dictionary[tag::PaymentRef]                        = "PaymentRef";                                      // (String FIX.4.3)
dictionary[tag::DistribPaymentMethod]              = "DistribPaymentMethod";                            // (int FIX.4.3)
dictionary[tag::CashDistribCurr]                   = "CashDistribCurr";                                 // (Currency FIX.4.3)
dictionary[tag::CommCurrency]                      = "CommCurrency";                                    // (Currency FIX.4.3)
dictionary[tag::CancellationRights]                = "CancellationRights";                              // (char FIX.4.3)
dictionary[tag::MoneyLaunderingStatus]             = "MoneyLaunderingStatus";                           // (char FIX.4.3)
dictionary[tag::MailingInst]                       = "MailingInst";                                     // (String FIX.4.3)
dictionary[tag::TransBkdTime]                      = "TransBkdTime";                                    // (UTCTimestamp FIX.4.3)
dictionary[tag::ExecPriceType]                     = "ExecPriceType";                                   // (char FIX.4.3)
dictionary[tag::ExecPriceAdjustment]               = "ExecPriceAdjustment";                             // (float FIX.4.3)
dictionary[tag::DateOfBirth]                       = "DateOfBirth";                                     // (LocalMktDate FIX.4.3)
dictionary[tag::TradeReportTransType]              = "TradeReportTransType";                            // (int FIX.4.3)
dictionary[tag::CardHolderName]                    = "CardHolderName";                                  // (String FIX.4.3)
dictionary[tag::CardNumber]                        = "CardNumber";                                      // (String FIX.4.3)
dictionary[tag::CardExpDate]                       = "CardExpDate";                                     // (LocalMktDate FIX.4.3)
dictionary[tag::CardIssNum]                        = "CardIssNum";                                      // (String FIX.4.3)
dictionary[tag::PaymentMethod]                     = "PaymentMethod";                                   // (int FIX.4.3)
dictionary[tag::RegistAcctType]                    = "RegistAcctType";                                  // (String FIX.4.3)
dictionary[tag::Designation]                       = "Designation";                                     // (String FIX.4.3)
dictionary[tag::TaxAdvantageType]                  = "TaxAdvantageType";                                // (int FIX.4.3)
dictionary[tag::RegistRejReasonText]               = "RegistRejReasonText";                             // (String FIX.4.3)
dictionary[tag::FundRenewWaiv]                     = "FundRenewWaiv";                                   // (char FIX.4.3)
dictionary[tag::CashDistribAgentName]              = "CashDistribAgentName";                            // (String FIX.4.3)
dictionary[tag::CashDistribAgentCode]              = "CashDistribAgentCode";                            // (String FIX.4.3)
dictionary[tag::CashDistribAgentAcctNumber]        = "CashDistribAgentAcctNumber";                      // (String FIX.4.3)
dictionary[tag::CashDistribPayRef]                 = "CashDistribPayRef";                               // (String FIX.4.3)
dictionary[tag::CashDistribAgentAcctName]          = "CashDistribAgentAcctName";                        // (String FIX.4.3)
dictionary[tag::CardStartDate]                     = "CardStartDate";                                   // (LocalMktDate FIX.4.3)
dictionary[tag::PaymentDate]                       = "PaymentDate";                                     // (LocalMktDate FIX.4.3)
dictionary[tag::PaymentRemitterID]                 = "PaymentRemitterID";                               // (String FIX.4.3)
dictionary[tag::RegistStatus]                      = "RegistStatus";                                    // (char FIX.4.3)
dictionary[tag::RegistRejReasonCode]               = "RegistRejReasonCode";                             // (int FIX.4.3)
dictionary[tag::RegistRefID]                       = "RegistRefID";                                     // (String FIX.4.3)
dictionary[tag::RegistDtls]                        = "RegistDtls";                                      // (String FIX.4.3)
dictionary[tag::NoDistribInsts]                    = "NoDistribInsts";                                  // (NumInGroup FIX.4.3)
dictionary[tag::RegistEmail]                       = "RegistEmail";                                     // (String FIX.4.3)
dictionary[tag::DistribPercentage]                 = "DistribPercentage";                               // (Percentage FIX.4.3)
dictionary[tag::RegistID]                          = "RegistID";                                        // (String FIX.4.3)
dictionary[tag::RegistTransType]                   = "RegistTransType";                                 // (char FIX.4.3)
dictionary[tag::ExecValuationPoint]                = "ExecValuationPoint";                              // (UTCTimestamp FIX.4.3)
dictionary[tag::OrderPercent]                      = "OrderPercent";                                    // (Percentage FIX.4.3)
dictionary[tag::OwnershipType]                     = "OwnershipType";                                   // (char FIX.4.3)
dictionary[tag::NoContAmts]                        = "NoContAmts";                                      // (NumInGroup FIX.4.3)
dictionary[tag::ContAmtType]                       = "ContAmtType";                                     // (int FIX.4.3)
dictionary[tag::ContAmtValue]                      = "ContAmtValue";                                    // (float FIX.4.3)
dictionary[tag::ContAmtCurr]                       = "ContAmtCurr";                                     // (Currency FIX.4.3)
dictionary[tag::OwnerType]                         = "OwnerType";                                       // (int FIX.4.3)
dictionary[tag::PartySubID]                        = "PartySubID";                                      // (String FIX.4.3)
dictionary[tag::NestedPartyID]                     = "NestedPartyID";                                   // (String FIX.4.3)
dictionary[tag::NestedPartyIDSource]               = "NestedPartyIDSource";                             // (char FIX.4.3)
dictionary[tag::SecondaryClOrdID]                  = "SecondaryClOrdID";                                // (String FIX.4.3)
dictionary[tag::SecondaryExecID]                   = "SecondaryExecID";                                 // (String FIX.4.3)
dictionary[tag::OrderCapacity]                     = "OrderCapacity";                                   // (char FIX.4.3)
dictionary[tag::OrderRestrictions]                 = "OrderRestrictions";                               // (MultipleCharValue FIX.4.3)
dictionary[tag::MassCancelRequestType]             = "MassCancelRequestType";                           // (char FIX.4.3)
dictionary[tag::MassCancelResponse]                = "MassCancelResponse";                              // (char FIX.4.3)
dictionary[tag::MassCancelRejectReason]            = "MassCancelRejectReason";                          // (int FIX.4.3)
dictionary[tag::TotalAffectedOrders]               = "TotalAffectedOrders";                             // (int FIX.4.3)
dictionary[tag::NoAffectedOrders]                  = "NoAffectedOrders";                                // (NumInGroup FIX.4.3)
dictionary[tag::AffectedOrderID]                   = "AffectedOrderID";                                 // (String FIX.4.3)
dictionary[tag::AffectedSecondaryOrderID]          = "AffectedSecondaryOrderID";                        // (String FIX.4.3)
dictionary[tag::QuoteType]                         = "QuoteType";                                       // (int FIX.4.3)
dictionary[tag::NestedPartyRole]                   = "NestedPartyRole";                                 // (int FIX.4.3)
dictionary[tag::NoNestedPartyIDs]                  = "NoNestedPartyIDs";                                // (NumInGroup FIX.4.3)
dictionary[tag::TotalAccruedInterestAmt]           = "TotalAccruedInterestAmt";                         // (Amt FIX.4.3)
dictionary[tag::MaturityDate]                      = "MaturityDate";                                    // (LocalMktDate FIX.4.3)
dictionary[tag::UnderlyingMaturityDate]            = "UnderlyingMaturityDate";                          // (LocalMktDate FIX.4.3)
dictionary[tag::InstrRegistry]                     = "InstrRegistry";                                   // (String FIX.4.3)
dictionary[tag::CashMargin]                        = "CashMargin";                                      // (char FIX.4.3)
dictionary[tag::NestedPartySubID]                  = "NestedPartySubID";                                // (String FIX.4.3)
dictionary[tag::Scope]                             = "Scope";                                           // (MultipleCharValue FIX.4.3)
dictionary[tag::MDImplicitDelete]                  = "MDImplicitDelete";                                // (Boolean FIX.4.3)
dictionary[tag::CrossID]                           = "CrossID";                                         // (String FIX.4.3)
dictionary[tag::CrossType]                         = "CrossType";                                       // (int FIX.4.3)
dictionary[tag::CrossPrioritization]               = "CrossPrioritization";                             // (int FIX.4.3)
dictionary[tag::OrigCrossID]                       = "OrigCrossID";                                     // (String FIX.4.3)
dictionary[tag::NoSides]                           = "NoSides";                                         // (NumInGroup FIX.4.3)
dictionary[tag::Username]                          = "Username";                                        // (String FIX.4.3)
dictionary[tag::Password]                          = "Password";                                        // (String FIX.4.3)
dictionary[tag::NoLegs]                            = "NoLegs";                                          // (NumInGroup FIX.4.3)
dictionary[tag::LegCurrency]                       = "LegCurrency";                                     // (Currency FIX.4.3)
dictionary[tag::TotNoSecurityTypes]                = "TotNoSecurityTypes";                              // (int FIX.4.3)
dictionary[tag::NoSecurityTypes]                   = "NoSecurityTypes";                                 // (NumInGroup FIX.4.3)
dictionary[tag::SecurityListRequestType]           = "SecurityListRequestType";                         // (int FIX.4.3)
dictionary[tag::SecurityRequestResult]             = "SecurityRequestResult";                           // (int FIX.4.3)
dictionary[tag::RoundLot]                          = "RoundLot";                                        // (Qty FIX.4.3)
dictionary[tag::MinTradeVol]                       = "MinTradeVol";                                     // (Qty FIX.4.3)
dictionary[tag::MultiLegRptTypeReq]                = "MultiLegRptTypeReq";                              // (int FIX.4.3)
dictionary[tag::LegPositionEffect]                 = "LegPositionEffect";                               // (char FIX.4.3)
dictionary[tag::LegCoveredOrUncovered]             = "LegCoveredOrUncovered";                           // (int FIX.4.3)
dictionary[tag::LegPrice]                          = "LegPrice";                                        // (Price FIX.4.3)
dictionary[tag::TradSesStatusRejReason]            = "TradSesStatusRejReason";                          // (int FIX.4.3)
dictionary[tag::TradeRequestID]                    = "TradeRequestID";                                  // (String FIX.4.3)
dictionary[tag::TradeRequestType]                  = "TradeRequestType";                                // (int FIX.4.3)
dictionary[tag::PreviouslyReported]                = "PreviouslyReported";                              // (Boolean FIX.4.3)
dictionary[tag::TradeReportID]                     = "TradeReportID";                                   // (String FIX.4.3)
dictionary[tag::TradeReportRefID]                  = "TradeReportRefID";                                // (String FIX.4.3)
dictionary[tag::MatchStatus]                       = "MatchStatus";                                     // (char FIX.4.3)
dictionary[tag::MatchType]                         = "MatchType";                                       // (String FIX.4.3)
dictionary[tag::OddLot]                            = "OddLot";                                          // (Boolean FIX.4.3)
dictionary[tag::NoClearingInstructions]            = "NoClearingInstructions";                          // (NumInGroup FIX.4.3)
dictionary[tag::ClearingInstruction]               = "ClearingInstruction";                             // (int FIX.4.3)
dictionary[tag::TradeInputSource]                  = "TradeInputSource";                                // (String FIX.4.3)
dictionary[tag::TradeInputDevice]                  = "TradeInputDevice";                                // (String FIX.4.3)
dictionary[tag::NoDates]                           = "NoDates";                                         // (NumInGroup FIX.4.3)
dictionary[tag::AccountType]                       = "AccountType";                                     // (int FIX.4.3)
dictionary[tag::CustOrderCapacity]                 = "CustOrderCapacity";                               // (int FIX.4.3)
dictionary[tag::ClOrdLinkID]                       = "ClOrdLinkID";                                     // (String FIX.4.3)
dictionary[tag::MassStatusReqID]                   = "MassStatusReqID";                                 // (String FIX.4.3)
dictionary[tag::MassStatusReqType]                 = "MassStatusReqType";                               // (int FIX.4.3)
dictionary[tag::OrigOrdModTime]                    = "OrigOrdModTime";                                  // (UTCTimestamp FIX.4.3)
dictionary[tag::LegSettlType]                      = "LegSettlType";                                    // (String FIX.4.3)
dictionary[tag::LegSettlDate]                      = "LegSettlDate";                                    // (LocalMktDate FIX.4.3)
dictionary[tag::DayBookingInst]                    = "DayBookingInst";                                  // (char FIX.4.3)
dictionary[tag::BookingUnit]                       = "BookingUnit";                                     // (char FIX.4.3)
dictionary[tag::PreallocMethod]                    = "PreallocMethod";                                  // (char FIX.4.3)
dictionary[tag::UnderlyingCountryOfIssue]          = "UnderlyingCountryOfIssue";                        // (Country FIX.4.3)
dictionary[tag::UnderlyingStateOrProvinceOfIssue]  = "UnderlyingStateOrProvinceOfIssue";                // (String FIX.4.3)
dictionary[tag::UnderlyingLocaleOfIssue]           = "UnderlyingLocaleOfIssue";                         // (String FIX.4.3)
dictionary[tag::UnderlyingInstrRegistry]           = "UnderlyingInstrRegistry";                         // (String FIX.4.3)
dictionary[tag::LegCountryOfIssue]                 = "LegCountryOfIssue";                               // (Country FIX.4.3)
dictionary[tag::LegStateOrProvinceOfIssue]         = "LegStateOrProvinceOfIssue";                       // (String FIX.4.3)
dictionary[tag::LegLocaleOfIssue]                  = "LegLocaleOfIssue";                                // (String FIX.4.3)
dictionary[tag::LegInstrRegistry]                  = "LegInstrRegistry";                                // (String FIX.4.3)
dictionary[tag::LegSymbol]                         = "LegSymbol";                                       // (String FIX.4.3)
dictionary[tag::LegSymbolSfx]                      = "LegSymbolSfx";                                    // (String FIX.4.3)
dictionary[tag::LegSecurityID]                     = "LegSecurityID";                                   // (String FIX.4.3)
dictionary[tag::LegSecurityIDSource]               = "LegSecurityIDSource";                             // (String FIX.4.3)
dictionary[tag::NoLegSecurityAltID]                = "NoLegSecurityAltID";                              // (NumInGroup FIX.4.3)
dictionary[tag::LegSecurityAltID]                  = "LegSecurityAltID";                                // (String FIX.4.3)
dictionary[tag::LegSecurityAltIDSource]            = "LegSecurityAltIDSource";                          // (String FIX.4.3)
dictionary[tag::LegProduct]                        = "LegProduct";                                      // (int FIX.4.3)
dictionary[tag::LegCFICode]                        = "LegCFICode";                                      // (String FIX.4.3)
dictionary[tag::LegSecurityType]                   = "LegSecurityType";                                 // (String FIX.4.3)
dictionary[tag::LegMaturityMonthYear]              = "LegMaturityMonthYear";                            // (MonthYear FIX.4.3)
dictionary[tag::LegMaturityDate]                   = "LegMaturityDate";                                 // (LocalMktDate FIX.4.3)
dictionary[tag::LegStrikePrice]                    = "LegStrikePrice";                                  // (Price FIX.4.3)
dictionary[tag::LegOptAttribute]                   = "LegOptAttribute";                                 // (char FIX.4.3)
dictionary[tag::LegContractMultiplier]             = "LegContractMultiplier";                           // (float FIX.4.3)
dictionary[tag::LegCouponRate]                     = "LegCouponRate";                                   // (Percentage FIX.4.3)
dictionary[tag::LegSecurityExchange]               = "LegSecurityExchange";                             // (Exchange FIX.4.3)
dictionary[tag::LegIssuer]                         = "LegIssuer";                                       // (String FIX.4.3)
dictionary[tag::EncodedLegIssuerLen]               = "EncodedLegIssuerLen";                             // (Length FIX.4.3)
dictionary[tag::EncodedLegIssuer]                  = "EncodedLegIssuer";                                // (data FIX.4.3)
dictionary[tag::LegSecurityDesc]                   = "LegSecurityDesc";                                 // (String FIX.4.3)
dictionary[tag::EncodedLegSecurityDescLen]         = "EncodedLegSecurityDescLen";                       // (Length FIX.4.3)
dictionary[tag::EncodedLegSecurityDesc]            = "EncodedLegSecurityDesc";                          // (data FIX.4.3)
dictionary[tag::LegRatioQty]                       = "LegRatioQty";                                     // (float FIX.4.3)
dictionary[tag::LegSide]                           = "LegSide";                                         // (char FIX.4.3)
dictionary[tag::TradingSessionSubID]               = "TradingSessionSubID";                             // (String FIX.4.3)
dictionary[tag::AllocType]                         = "AllocType";                                       // (int FIX.4.3)
dictionary[tag::NoHops]                            = "NoHops";                                          // (NumInGroup FIX.4.3)
dictionary[tag::HopCompID]                         = "HopCompID";                                       // (String FIX.4.3)
dictionary[tag::HopSendingTime]                    = "HopSendingTime";                                  // (UTCTimestamp FIX.4.3)
dictionary[tag::HopRefID]                          = "HopRefID";                                        // (SeqNum FIX.4.3)
dictionary[tag::MidPx]                             = "MidPx";                                           // (Price FIX.4.3)
dictionary[tag::BidYield]                          = "BidYield";                                        // (Percentage FIX.4.3)
dictionary[tag::MidYield]                          = "MidYield";                                        // (Percentage FIX.4.3)
dictionary[tag::OfferYield]                        = "OfferYield";                                      // (Percentage FIX.4.3)
dictionary[tag::ClearingFeeIndicator]              = "ClearingFeeIndicator";                            // (String FIX.4.3)
dictionary[tag::WorkingIndicator]                  = "WorkingIndicator";                                // (Boolean FIX.4.3)
dictionary[tag::LegLastPx]                         = "LegLastPx";                                       // (Price FIX.4.3)
dictionary[tag::PriorityIndicator]                 = "PriorityIndicator";                               // (int FIX.4.3)
dictionary[tag::PriceImprovement]                  = "PriceImprovement";                                // (PriceOffset FIX.4.3)
dictionary[tag::Price2]                            = "Price2";                                          // (Price FIX.4.3)
dictionary[tag::LastForwardPoints2]                = "LastForwardPoints2";                              // (PriceOffset FIX.4.3)
dictionary[tag::BidForwardPoints2]                 = "BidForwardPoints2";                               // (PriceOffset FIX.4.3)
dictionary[tag::OfferForwardPoints2]               = "OfferForwardPoints2";                             // (PriceOffset FIX.4.3)
dictionary[tag::RFQReqID]                          = "RFQReqID";                                        // (String FIX.4.3)
dictionary[tag::MktBidPx]                          = "MktBidPx";                                        // (Price FIX.4.3)
dictionary[tag::MktOfferPx]                        = "MktOfferPx";                                      // (Price FIX.4.3)
dictionary[tag::MinBidSize]                        = "MinBidSize";                                      // (Qty FIX.4.3)
dictionary[tag::MinOfferSize]                      = "MinOfferSize";                                    // (Qty FIX.4.3)
dictionary[tag::QuoteStatusReqID]                  = "QuoteStatusReqID";                                // (String FIX.4.3)
dictionary[tag::LegalConfirm]                      = "LegalConfirm";                                    // (Boolean FIX.4.3)
dictionary[tag::UnderlyingLastPx]                  = "UnderlyingLastPx";                                // (Price FIX.4.3)
dictionary[tag::UnderlyingLastQty]                 = "UnderlyingLastQty";                               // (Qty FIX.4.3)
dictionary[tag::LegRefID]                          = "LegRefID";                                        // (String FIX.4.3)
dictionary[tag::ContraLegRefID]                    = "ContraLegRefID";                                  // (String FIX.4.3)
dictionary[tag::SettlCurrBidFxRate]                = "SettlCurrBidFxRate";                              // (float FIX.4.3)
dictionary[tag::SettlCurrOfferFxRate]              = "SettlCurrOfferFxRate";                            // (float FIX.4.3)
dictionary[tag::QuoteRequestRejectReason]          = "QuoteRequestRejectReason";                        // (int FIX.4.3)
dictionary[tag::SideComplianceID]                  = "SideComplianceID";                                // (String FIX.4.3)
dictionary[tag::AcctIDSource]                      = "AcctIDSource";                                    // (int FIX.4.4)
dictionary[tag::AllocAcctIDSource]                 = "AllocAcctIDSource";                               // (int FIX.4.4)
dictionary[tag::BenchmarkPrice]                    = "BenchmarkPrice";                                  // (Price FIX.4.4)
dictionary[tag::BenchmarkPriceType]                = "BenchmarkPriceType";                              // (int FIX.4.4)
dictionary[tag::ConfirmID]                         = "ConfirmID";                                       // (String FIX.4.4)
dictionary[tag::ConfirmStatus]                     = "ConfirmStatus";                                   // (int FIX.4.4)
dictionary[tag::ConfirmTransType]                  = "ConfirmTransType";                                // (int FIX.4.4)
dictionary[tag::ContractSettlMonth]                = "ContractSettlMonth";                              // (MonthYear FIX.4.4)
dictionary[tag::DeliveryForm]                      = "DeliveryForm";                                    // (int FIX.4.4)
dictionary[tag::LastParPx]                         = "LastParPx";                                       // (Price FIX.4.4)
dictionary[tag::NoLegAllocs]                       = "NoLegAllocs";                                     // (NumInGroup FIX.4.4)
dictionary[tag::LegAllocAccount]                   = "LegAllocAccount";                                 // (String FIX.4.4)
dictionary[tag::LegIndividualAllocID]              = "LegIndividualAllocID";                            // (String FIX.4.4)
dictionary[tag::LegAllocQty]                       = "LegAllocQty";                                     // (Qty FIX.4.4)
dictionary[tag::LegAllocAcctIDSource]              = "LegAllocAcctIDSource";                            // (String FIX.4.4)
dictionary[tag::LegSettlCurrency]                  = "LegSettlCurrency";                                // (Currency FIX.4.4)
dictionary[tag::LegBenchmarkCurveCurrency]         = "LegBenchmarkCurveCurrency";                       // (Currency FIX.4.4)
dictionary[tag::LegBenchmarkCurveName]             = "LegBenchmarkCurveName";                           // (String FIX.4.4)
dictionary[tag::LegBenchmarkCurvePoint]            = "LegBenchmarkCurvePoint";                          // (String FIX.4.4)
dictionary[tag::LegBenchmarkPrice]                 = "LegBenchmarkPrice";                               // (Price FIX.4.4)
dictionary[tag::LegBenchmarkPriceType]             = "LegBenchmarkPriceType";                           // (int FIX.4.4)
dictionary[tag::LegBidPx]                          = "LegBidPx";                                        // (Price FIX.4.4)
dictionary[tag::LegIOIQty]                         = "LegIOIQty";                                       // (String FIX.4.4)
dictionary[tag::NoLegStipulations]                 = "NoLegStipulations";                               // (NumInGroup FIX.4.4)
dictionary[tag::LegOfferPx]                        = "LegOfferPx";                                      // (Price FIX.4.4)
dictionary[tag::LegOrderQty]                       = "LegOrderQty";                                     // (Qty FIX.4.4)
dictionary[tag::LegPriceType]                      = "LegPriceType";                                    // (int FIX.4.4)
dictionary[tag::LegQty]                            = "LegQty";                                          // (Qty FIX.4.4)
dictionary[tag::LegStipulationType]                = "LegStipulationType";                              // (String FIX.4.4)
dictionary[tag::LegStipulationValue]               = "LegStipulationValue";                             // (String FIX.4.4)
dictionary[tag::LegSwapType]                       = "LegSwapType";                                     // (int FIX.4.4)
dictionary[tag::Pool]                              = "Pool";                                            // (String FIX.4.4)
dictionary[tag::QuotePriceType]                    = "QuotePriceType";                                  // (int FIX.4.4)
dictionary[tag::QuoteRespID]                       = "QuoteRespID";                                     // (String FIX.4.4)
dictionary[tag::QuoteRespType]                     = "QuoteRespType";                                   // (int FIX.4.4)
dictionary[tag::QuoteQualifier]                    = "QuoteQualifier";                                  // (char FIX.4.4)
dictionary[tag::YieldRedemptionDate]               = "YieldRedemptionDate";                             // (LocalMktDate FIX.4.4)
dictionary[tag::YieldRedemptionPrice]              = "YieldRedemptionPrice";                            // (Price FIX.4.4)
dictionary[tag::YieldRedemptionPriceType]          = "YieldRedemptionPriceType";                        // (int FIX.4.4)
dictionary[tag::BenchmarkSecurityID]               = "BenchmarkSecurityID";                             // (String FIX.4.4)
dictionary[tag::ReversalIndicator]                 = "ReversalIndicator";                               // (Boolean FIX.4.4)
dictionary[tag::YieldCalcDate]                     = "YieldCalcDate";                                   // (LocalMktDate FIX.4.4)
dictionary[tag::NoPositions]                       = "NoPositions";                                     // (NumInGroup FIX.4.4)
dictionary[tag::PosType]                           = "PosType";                                         // (String FIX.4.4)
dictionary[tag::LongQty]                           = "LongQty";                                         // (Qty FIX.4.4)
dictionary[tag::ShortQty]                          = "ShortQty";                                        // (Qty FIX.4.4)
dictionary[tag::PosQtyStatus]                      = "PosQtyStatus";                                    // (int FIX.4.4)
dictionary[tag::PosAmtType]                        = "PosAmtType";                                      // (String FIX.4.4)
dictionary[tag::PosAmt]                            = "PosAmt";                                          // (Amt FIX.4.4)
dictionary[tag::PosTransType]                      = "PosTransType";                                    // (int FIX.4.4)
dictionary[tag::PosReqID]                          = "PosReqID";                                        // (String FIX.4.4)
dictionary[tag::NoUnderlyings]                     = "NoUnderlyings";                                   // (NumInGroup FIX.4.4)
dictionary[tag::PosMaintAction]                    = "PosMaintAction";                                  // (int FIX.4.4)
dictionary[tag::OrigPosReqRefID]                   = "OrigPosReqRefID";                                 // (String FIX.4.4)
dictionary[tag::PosMaintRptRefID]                  = "PosMaintRptRefID";                                // (String FIX.4.4)
dictionary[tag::ClearingBusinessDate]              = "ClearingBusinessDate";                            // (LocalMktDate FIX.4.4)
dictionary[tag::SettlSessID]                       = "SettlSessID";                                     // (String FIX.4.4)
dictionary[tag::SettlSessSubID]                    = "SettlSessSubID";                                  // (String FIX.4.4)
dictionary[tag::AdjustmentType]                    = "AdjustmentType";                                  // (int FIX.4.4)
dictionary[tag::ContraryInstructionIndicator]      = "ContraryInstructionIndicator";                    // (Boolean FIX.4.4)
dictionary[tag::PriorSpreadIndicator]              = "PriorSpreadIndicator";                            // (Boolean FIX.4.4)
dictionary[tag::PosMaintRptID]                     = "PosMaintRptID";                                   // (String FIX.4.4)
dictionary[tag::PosMaintStatus]                    = "PosMaintStatus";                                  // (int FIX.4.4)
dictionary[tag::PosMaintResult]                    = "PosMaintResult";                                  // (int FIX.4.4)
dictionary[tag::PosReqType]                        = "PosReqType";                                      // (int FIX.4.4)
dictionary[tag::ResponseTransportType]             = "ResponseTransportType";                           // (int FIX.4.4)
dictionary[tag::ResponseDestination]               = "ResponseDestination";                             // (String FIX.4.4)
dictionary[tag::TotalNumPosReports]                = "TotalNumPosReports";                              // (int FIX.4.4)
dictionary[tag::PosReqResult]                      = "PosReqResult";                                    // (int FIX.4.4)
dictionary[tag::PosReqStatus]                      = "PosReqStatus";                                    // (int FIX.4.4)
dictionary[tag::SettlPrice]                        = "SettlPrice";                                      // (Price FIX.4.4)
dictionary[tag::SettlPriceType]                    = "SettlPriceType";                                  // (int FIX.4.4)
dictionary[tag::UnderlyingSettlPrice]              = "UnderlyingSettlPrice";                            // (Price FIX.4.4)
dictionary[tag::UnderlyingSettlPriceType]          = "UnderlyingSettlPriceType";                        // (int FIX.4.4)
dictionary[tag::PriorSettlPrice]                   = "PriorSettlPrice";                                 // (Price FIX.4.4)
dictionary[tag::NoQuoteQualifiers]                 = "NoQuoteQualifiers";                               // (NumInGroup FIX.4.4)
dictionary[tag::AllocSettlCurrency]                = "AllocSettlCurrency";                              // (Currency FIX.4.4)
dictionary[tag::AllocSettlCurrAmt]                 = "AllocSettlCurrAmt";                               // (Amt FIX.4.4)
dictionary[tag::InterestAtMaturity]                = "InterestAtMaturity";                              // (Amt FIX.4.4)
dictionary[tag::LegDatedDate]                      = "LegDatedDate";                                    // (LocalMktDate FIX.4.4)
dictionary[tag::LegPool]                           = "LegPool";                                         // (String FIX.4.4)
dictionary[tag::AllocInterestAtMaturity]           = "AllocInterestAtMaturity";                         // (Amt FIX.4.4)
dictionary[tag::AllocAccruedInterestAmt]           = "AllocAccruedInterestAmt";                         // (Amt FIX.4.4)
dictionary[tag::DeliveryDate]                      = "DeliveryDate";                                    // (LocalMktDate FIX.4.4)
dictionary[tag::AssignmentMethod]                  = "AssignmentMethod";                                // (char FIX.4.4)
dictionary[tag::AssignmentUnit]                    = "AssignmentUnit";                                  // (Qty FIX.4.4)
dictionary[tag::OpenInterest]                      = "OpenInterest";                                    // (Amt FIX.4.4)
dictionary[tag::ExerciseMethod]                    = "ExerciseMethod";                                  // (char FIX.4.4)
dictionary[tag::TotNumTradeReports]                = "TotNumTradeReports";                              // (int FIX.4.4)
dictionary[tag::TradeRequestResult]                = "TradeRequestResult";                              // (int FIX.4.4)
dictionary[tag::TradeRequestStatus]                = "TradeRequestStatus";                              // (int FIX.4.4)
dictionary[tag::TradeReportRejectReason]           = "TradeReportRejectReason";                         // (int FIX.4.4)
dictionary[tag::SideMultiLegReportingType]         = "SideMultiLegReportingType";                       // (int FIX.4.4)
dictionary[tag::NoPosAmt]                          = "NoPosAmt";                                        // (NumInGroup FIX.4.4)
dictionary[tag::AutoAcceptIndicator]               = "AutoAcceptIndicator";                             // (Boolean FIX.4.4)
dictionary[tag::AllocReportID]                     = "AllocReportID";                                   // (String FIX.4.4)
dictionary[tag::NoNested2PartyIDs]                 = "NoNested2PartyIDs";                               // (NumInGroup FIX.4.4)
dictionary[tag::Nested2PartyID]                    = "Nested2PartyID";                                  // (String FIX.4.4)
dictionary[tag::Nested2PartyIDSource]              = "Nested2PartyIDSource";                            // (char FIX.4.4)
dictionary[tag::Nested2PartyRole]                  = "Nested2PartyRole";                                // (int FIX.4.4)
dictionary[tag::Nested2PartySubID]                 = "Nested2PartySubID";                               // (String FIX.4.4)
dictionary[tag::BenchmarkSecurityIDSource]         = "BenchmarkSecurityIDSource";                       // (String FIX.4.4)
dictionary[tag::SecuritySubType]                   = "SecuritySubType";                                 // (String FIX.4.4)
dictionary[tag::UnderlyingSecuritySubType]         = "UnderlyingSecuritySubType";                       // (String FIX.4.4)
dictionary[tag::LegSecuritySubType]                = "LegSecuritySubType";                              // (String FIX.4.4)
dictionary[tag::AllowableOneSidednessPct]          = "AllowableOneSidednessPct";                        // (Percentage FIX.4.4)
dictionary[tag::AllowableOneSidednessValue]        = "AllowableOneSidednessValue";                      // (Amt FIX.4.4)
dictionary[tag::AllowableOneSidednessCurr]         = "AllowableOneSidednessCurr";                       // (Currency FIX.4.4)
dictionary[tag::NoTrdRegTimestamps]                = "NoTrdRegTimestamps";                              // (NumInGroup FIX.4.4)
dictionary[tag::TrdRegTimestamp]                   = "TrdRegTimestamp";                                 // (UTCTimestamp FIX.4.4)
dictionary[tag::TrdRegTimestampType]               = "TrdRegTimestampType";                             // (int FIX.4.4)
dictionary[tag::TrdRegTimestampOrigin]             = "TrdRegTimestampOrigin";                           // (String FIX.4.4)
dictionary[tag::ConfirmRefID]                      = "ConfirmRefID";                                    // (String FIX.4.4)
dictionary[tag::ConfirmType]                       = "ConfirmType";                                     // (int FIX.4.4)
dictionary[tag::ConfirmRejReason]                  = "ConfirmRejReason";                                // (int FIX.4.4)
dictionary[tag::BookingType]                       = "BookingType";                                     // (int FIX.4.4)
dictionary[tag::IndividualAllocRejCode]            = "IndividualAllocRejCode";                          // (int FIX.4.4)
dictionary[tag::SettlInstMsgID]                    = "SettlInstMsgID";                                  // (String FIX.4.4)
dictionary[tag::NoSettlInst]                       = "NoSettlInst";                                     // (NumInGroup FIX.4.4)
dictionary[tag::LastUpdateTime]                    = "LastUpdateTime";                                  // (UTCTimestamp FIX.4.4)
dictionary[tag::AllocSettlInstType]                = "AllocSettlInstType";                              // (int FIX.4.4)
dictionary[tag::NoSettlPartyIDs]                   = "NoSettlPartyIDs";                                 // (NumInGroup FIX.4.4)
dictionary[tag::SettlPartyID]                      = "SettlPartyID";                                    // (String FIX.4.4)
dictionary[tag::SettlPartyIDSource]                = "SettlPartyIDSource";                              // (char FIX.4.4)
dictionary[tag::SettlPartyRole]                    = "SettlPartyRole";                                  // (int FIX.4.4)
dictionary[tag::SettlPartySubID]                   = "SettlPartySubID";                                 // (String FIX.4.4)
dictionary[tag::SettlPartySubIDType]               = "SettlPartySubIDType";                             // (int FIX.4.4)
dictionary[tag::DlvyInstType]                      = "DlvyInstType";                                    // (char FIX.4.4)
dictionary[tag::TerminationType]                   = "TerminationType";                                 // (int FIX.4.4)
dictionary[tag::NextExpectedMsgSeqNum]             = "NextExpectedMsgSeqNum";                           // (SeqNum FIX.4.4)
dictionary[tag::OrdStatusReqID]                    = "OrdStatusReqID";                                  // (String FIX.4.4)
dictionary[tag::SettlInstReqID]                    = "SettlInstReqID";                                  // (String FIX.4.4)
dictionary[tag::SettlInstReqRejCode]               = "SettlInstReqRejCode";                             // (int FIX.4.4)
dictionary[tag::SecondaryAllocID]                  = "SecondaryAllocID";                                // (String FIX.4.4)
dictionary[tag::AllocReportType]                   = "AllocReportType";                                 // (int FIX.4.4)
dictionary[tag::AllocReportRefID]                  = "AllocReportRefID";                                // (String FIX.4.4)
dictionary[tag::AllocCancReplaceReason]            = "AllocCancReplaceReason";                          // (int FIX.4.4)
dictionary[tag::CopyMsgIndicator]                  = "CopyMsgIndicator";                                // (Boolean FIX.4.4)
dictionary[tag::AllocAccountType]                  = "AllocAccountType";                                // (int FIX.4.4)
dictionary[tag::OrderAvgPx]                        = "OrderAvgPx";                                      // (Price FIX.4.4)
dictionary[tag::OrderBookingQty]                   = "OrderBookingQty";                                 // (Qty FIX.4.4)
dictionary[tag::NoSettlPartySubIDs]                = "NoSettlPartySubIDs";                              // (NumInGroup FIX.4.4)
dictionary[tag::NoPartySubIDs]                     = "NoPartySubIDs";                                   // (NumInGroup FIX.4.4)
dictionary[tag::PartySubIDType]                    = "PartySubIDType";                                  // (int FIX.4.4)
dictionary[tag::NoNestedPartySubIDs]               = "NoNestedPartySubIDs";                             // (NumInGroup FIX.4.4)
dictionary[tag::NestedPartySubIDType]              = "NestedPartySubIDType";                            // (int FIX.4.4)
dictionary[tag::NoNested2PartySubIDs]              = "NoNested2PartySubIDs";                            // (NumInGroup FIX.4.4)
dictionary[tag::Nested2PartySubIDType]             = "Nested2PartySubIDType";                           // (int FIX.4.4)
dictionary[tag::AllocIntermedReqType]              = "AllocIntermedReqType";                            // (int FIX.4.4)
dictionary[tag::NoUsernames]                       = "NoUsernames";                                     // (NumInGroup FIX.4.4)
dictionary[tag::UnderlyingPx]                      = "UnderlyingPx";                                    // (Price FIX.4.4)
dictionary[tag::PriceDelta]                        = "PriceDelta";                                      // (float FIX.4.4)
dictionary[tag::ApplQueueMax]                      = "ApplQueueMax";                                    // (int FIX.4.4)
dictionary[tag::ApplQueueDepth]                    = "ApplQueueDepth";                                  // (int FIX.4.4)
dictionary[tag::ApplQueueResolution]               = "ApplQueueResolution";                             // (int FIX.4.4)
dictionary[tag::ApplQueueAction]                   = "ApplQueueAction";                                 // (int FIX.4.4)
dictionary[tag::NoAltMDSource]                     = "NoAltMDSource";                                   // (NumInGroup FIX.4.4)
dictionary[tag::AltMDSourceID]                     = "AltMDSourceID";                                   // (String FIX.4.4)
dictionary[tag::SecondaryTradeReportID]            = "SecondaryTradeReportID";                          // (String FIX.4.4)
dictionary[tag::AvgPxIndicator]                    = "AvgPxIndicator";                                  // (int FIX.4.4)
dictionary[tag::TradeLinkID]                       = "TradeLinkID";                                     // (String FIX.4.4)
dictionary[tag::OrderInputDevice]                  = "OrderInputDevice";                                // (String FIX.4.4)
dictionary[tag::UnderlyingTradingSessionID]        = "UnderlyingTradingSessionID";                      // (String FIX.4.4)
dictionary[tag::UnderlyingTradingSessionSubID]     = "UnderlyingTradingSessionSubID";                   // (String FIX.4.4)
dictionary[tag::TradeLegRefID]                     = "TradeLegRefID";                                   // (String FIX.4.4)
dictionary[tag::ExchangeRule]                      = "ExchangeRule";                                    // (String FIX.4.4)
dictionary[tag::TradeAllocIndicator]               = "TradeAllocIndicator";                             // (int FIX.4.4)
dictionary[tag::ExpirationCycle]                   = "ExpirationCycle";                                 // (int FIX.4.4)
dictionary[tag::TrdType]                           = "TrdType";                                         // (int FIX.4.4)
dictionary[tag::TrdSubType]                        = "TrdSubType";                                      // (int FIX.4.4)
dictionary[tag::TransferReason]                    = "TransferReason";                                  // (String FIX.4.4)
dictionary[tag::TotNumAssignmentReports]           = "TotNumAssignmentReports";                         // (int FIX.4.4)
dictionary[tag::AsgnRptID]                         = "AsgnRptID";                                       // (String FIX.4.4)
dictionary[tag::ThresholdAmount]                   = "ThresholdAmount";                                 // (PriceOffset FIX.4.4)
dictionary[tag::PegMoveType]                       = "PegMoveType";                                     // (int FIX.4.4)
dictionary[tag::PegOffsetType]                     = "PegOffsetType";                                   // (int FIX.4.4)
dictionary[tag::PegLimitType]                      = "PegLimitType";                                    // (int FIX.4.4)
dictionary[tag::PegRoundDirection]                 = "PegRoundDirection";                               // (int FIX.4.4)
dictionary[tag::PeggedPrice]                       = "PeggedPrice";                                     // (Price FIX.4.4)
dictionary[tag::PegScope]                          = "PegScope";                                        // (int FIX.4.4)
dictionary[tag::DiscretionMoveType]                = "DiscretionMoveType";                              // (int FIX.4.4)
dictionary[tag::DiscretionOffsetType]              = "DiscretionOffsetType";                            // (int FIX.4.4)
dictionary[tag::DiscretionLimitType]               = "DiscretionLimitType";                             // (int FIX.4.4)
dictionary[tag::DiscretionRoundDirection]          = "DiscretionRoundDirection";                        // (int FIX.4.4)
dictionary[tag::DiscretionPrice]                   = "DiscretionPrice";                                 // (Price FIX.4.4)
dictionary[tag::DiscretionScope]                   = "DiscretionScope";                                 // (int FIX.4.4)
dictionary[tag::TargetStrategy]                    = "TargetStrategy";                                  // (int FIX.4.4)
dictionary[tag::TargetStrategyParameters]          = "TargetStrategyParameters";                        // (String FIX.4.4)
dictionary[tag::ParticipationRate]                 = "ParticipationRate";                               // (Percentage FIX.4.4)
dictionary[tag::TargetStrategyPerformance]         = "TargetStrategyPerformance";                       // (float FIX.4.4)
dictionary[tag::LastLiquidityInd]                  = "LastLiquidityInd";                                // (int FIX.4.4)
dictionary[tag::PublishTrdIndicator]               = "PublishTrdIndicator";                             // (Boolean FIX.4.4)
dictionary[tag::ShortSaleReason]                   = "ShortSaleReason";                                 // (int FIX.4.4)
dictionary[tag::QtyType]                           = "QtyType";                                         // (int FIX.4.4)
dictionary[tag::SecondaryTrdType]                  = "SecondaryTrdType";                                // (int FIX.4.4)
dictionary[tag::TradeReportType]                   = "TradeReportType";                                 // (int FIX.4.4)
dictionary[tag::AllocNoOrdersType]                 = "AllocNoOrdersType";                               // (int FIX.4.4)
dictionary[tag::SharedCommission]                  = "SharedCommission";                                // (Amt FIX.4.4)
dictionary[tag::ConfirmReqID]                      = "ConfirmReqID";                                    // (String FIX.4.4)
dictionary[tag::AvgParPx]                          = "AvgParPx";                                        // (Price FIX.4.4)
dictionary[tag::ReportedPx]                        = "ReportedPx";                                      // (Price FIX.4.4)
dictionary[tag::NoCapacities]                      = "NoCapacities";                                    // (NumInGroup FIX.4.4)
dictionary[tag::OrderCapacityQty]                  = "OrderCapacityQty";                                // (Qty FIX.4.4)
dictionary[tag::NoEvents]                          = "NoEvents";                                        // (NumInGroup FIX.4.4)
dictionary[tag::EventType]                         = "EventType";                                       // (int FIX.4.4)
dictionary[tag::EventDate]                         = "EventDate";                                       // (LocalMktDate FIX.4.4)
dictionary[tag::EventPx]                           = "EventPx";                                         // (Price FIX.4.4)
dictionary[tag::EventText]                         = "EventText";                                       // (String FIX.4.4)
dictionary[tag::PctAtRisk]                         = "PctAtRisk";                                       // (Percentage FIX.4.4)
dictionary[tag::NoInstrAttrib]                     = "NoInstrAttrib";                                   // (NumInGroup FIX.4.4)
dictionary[tag::InstrAttribType]                   = "InstrAttribType";                                 // (int FIX.4.4)
dictionary[tag::InstrAttribValue]                  = "InstrAttribValue";                                // (String FIX.4.4)
dictionary[tag::DatedDate]                         = "DatedDate";                                       // (LocalMktDate FIX.4.4)
dictionary[tag::InterestAccrualDate]               = "InterestAccrualDate";                             // (LocalMktDate FIX.4.4)
dictionary[tag::CPProgram]                         = "CPProgram";                                       // (int FIX.4.4)
dictionary[tag::CPRegType]                         = "CPRegType";                                       // (String FIX.4.4)
dictionary[tag::UnderlyingCPProgram]               = "UnderlyingCPProgram";                             // (int FIX.4.4)
dictionary[tag::UnderlyingCPRegType]               = "UnderlyingCPRegType";                             // (String FIX.4.4)
dictionary[tag::UnderlyingQty]                     = "UnderlyingQty";                                   // (Qty FIX.4.4)
dictionary[tag::TrdMatchID]                        = "TrdMatchID";                                      // (String FIX.4.4)
dictionary[tag::SecondaryTradeReportRefID]         = "SecondaryTradeReportRefID";                       // (String FIX.4.4)
dictionary[tag::UnderlyingDirtyPrice]              = "UnderlyingDirtyPrice";                            // (Price FIX.4.4)
dictionary[tag::UnderlyingEndPrice]                = "UnderlyingEndPrice";                              // (Price FIX.4.4)
dictionary[tag::UnderlyingStartValue]              = "UnderlyingStartValue";                            // (Amt FIX.4.4)
dictionary[tag::UnderlyingCurrentValue]            = "UnderlyingCurrentValue";                          // (Amt FIX.4.4)
dictionary[tag::UnderlyingEndValue]                = "UnderlyingEndValue";                              // (Amt FIX.4.4)
dictionary[tag::NoUnderlyingStips]                 = "NoUnderlyingStips";                               // (NumInGroup FIX.4.4)
dictionary[tag::UnderlyingStipType]                = "UnderlyingStipType";                              // (String FIX.4.4)
dictionary[tag::UnderlyingStipValue]               = "UnderlyingStipValue";                             // (String FIX.4.4)
dictionary[tag::MaturityNetMoney]                  = "MaturityNetMoney";                                // (Amt FIX.4.4)
dictionary[tag::MiscFeeBasis]                      = "MiscFeeBasis";                                    // (int FIX.4.4)
dictionary[tag::TotNoAllocs]                       = "TotNoAllocs";                                     // (int FIX.4.4)
dictionary[tag::LastFragment]                      = "LastFragment";                                    // (Boolean FIX.4.4)
dictionary[tag::CollReqID]                         = "CollReqID";                                       // (String FIX.4.4)
dictionary[tag::CollAsgnReason]                    = "CollAsgnReason";                                  // (int FIX.4.4)
dictionary[tag::CollInquiryQualifier]              = "CollInquiryQualifier";                            // (int FIX.4.4)
dictionary[tag::NoTrades]                          = "NoTrades";                                        // (NumInGroup FIX.4.4)
dictionary[tag::MarginRatio]                       = "MarginRatio";                                     // (Percentage FIX.4.4)
dictionary[tag::MarginExcess]                      = "MarginExcess";                                    // (Amt FIX.4.4)
dictionary[tag::TotalNetValue]                     = "TotalNetValue";                                   // (Amt FIX.4.4)
dictionary[tag::CashOutstanding]                   = "CashOutstanding";                                 // (Amt FIX.4.4)
dictionary[tag::CollAsgnID]                        = "CollAsgnID";                                      // (String FIX.4.4)
dictionary[tag::CollAsgnTransType]                 = "CollAsgnTransType";                               // (int FIX.4.4)
dictionary[tag::CollRespID]                        = "CollRespID";                                      // (String FIX.4.4)
dictionary[tag::CollAsgnRespType]                  = "CollAsgnRespType";                                // (int FIX.4.4)
dictionary[tag::CollAsgnRejectReason]              = "CollAsgnRejectReason";                            // (int FIX.4.4)
dictionary[tag::CollAsgnRefID]                     = "CollAsgnRefID";                                   // (String FIX.4.4)
dictionary[tag::CollRptID]                         = "CollRptID";                                       // (String FIX.4.4)
dictionary[tag::CollInquiryID]                     = "CollInquiryID";                                   // (String FIX.4.4)
dictionary[tag::CollStatus]                        = "CollStatus";                                      // (int FIX.4.4)
dictionary[tag::TotNumReports]                     = "TotNumReports";                                   // (int FIX.4.4)
dictionary[tag::LastRptRequested]                  = "LastRptRequested";                                // (Boolean FIX.4.4)
dictionary[tag::AgreementDesc]                     = "AgreementDesc";                                   // (String FIX.4.4)
dictionary[tag::AgreementID]                       = "AgreementID";                                     // (String FIX.4.4)
dictionary[tag::AgreementDate]                     = "AgreementDate";                                   // (LocalMktDate FIX.4.4)
dictionary[tag::StartDate]                         = "StartDate";                                       // (LocalMktDate FIX.4.4)
dictionary[tag::EndDate]                           = "EndDate";                                         // (LocalMktDate FIX.4.4)
dictionary[tag::AgreementCurrency]                 = "AgreementCurrency";                               // (Currency FIX.4.4)
dictionary[tag::DeliveryType]                      = "DeliveryType";                                    // (int FIX.4.4)
dictionary[tag::EndAccruedInterestAmt]             = "EndAccruedInterestAmt";                           // (Amt FIX.4.4)
dictionary[tag::StartCash]                         = "StartCash";                                       // (Amt FIX.4.4)
dictionary[tag::EndCash]                           = "EndCash";                                         // (Amt FIX.4.4)
dictionary[tag::UserRequestID]                     = "UserRequestID";                                   // (String FIX.4.4)
dictionary[tag::UserRequestType]                   = "UserRequestType";                                 // (int FIX.4.4)
dictionary[tag::NewPassword]                       = "NewPassword";                                     // (String FIX.4.4)
dictionary[tag::UserStatus]                        = "UserStatus";                                      // (int FIX.4.4)
dictionary[tag::UserStatusText]                    = "UserStatusText";                                  // (String FIX.4.4)
dictionary[tag::StatusValue]                       = "StatusValue";                                     // (int FIX.4.4)
dictionary[tag::StatusText]                        = "StatusText";                                      // (String FIX.4.4)
dictionary[tag::RefCompID]                         = "RefCompID";                                       // (String FIX.4.4)
dictionary[tag::RefSubID]                          = "RefSubID";                                        // (String FIX.4.4)
dictionary[tag::NetworkResponseID]                 = "NetworkResponseID";                               // (String FIX.4.4)
dictionary[tag::NetworkRequestID]                  = "NetworkRequestID";                                // (String FIX.4.4)
dictionary[tag::LastNetworkResponseID]             = "LastNetworkResponseID";                           // (String FIX.4.4)
dictionary[tag::NetworkRequestType]                = "NetworkRequestType";                              // (int FIX.4.4)
dictionary[tag::NoCompIDs]                         = "NoCompIDs";                                       // (NumInGroup FIX.4.4)
dictionary[tag::NetworkStatusResponseType]         = "NetworkStatusResponseType";                       // (int FIX.4.4)
dictionary[tag::NoCollInquiryQualifier]            = "NoCollInquiryQualifier";                          // (NumInGroup FIX.4.4)
dictionary[tag::TrdRptStatus]                      = "TrdRptStatus";                                    // (int FIX.4.4)
dictionary[tag::AffirmStatus]                      = "AffirmStatus";                                    // (int FIX.4.4)
dictionary[tag::UnderlyingStrikeCurrency]          = "UnderlyingStrikeCurrency";                        // (Currency FIX.4.4)
dictionary[tag::LegStrikeCurrency]                 = "LegStrikeCurrency";                               // (Currency FIX.4.4)
dictionary[tag::TimeBracket]                       = "TimeBracket";                                     // (String FIX.4.4)
dictionary[tag::CollAction]                        = "CollAction";                                      // (int FIX.4.4)
dictionary[tag::CollInquiryStatus]                 = "CollInquiryStatus";                               // (int FIX.4.4)
dictionary[tag::CollInquiryResult]                 = "CollInquiryResult";                               // (int FIX.4.4)
dictionary[tag::StrikeCurrency]                    = "StrikeCurrency";                                  // (Currency FIX.4.4)
dictionary[tag::NoNested3PartyIDs]                 = "NoNested3PartyIDs";                               // (NumInGroup FIX.4.4)
dictionary[tag::Nested3PartyID]                    = "Nested3PartyID";                                  // (String FIX.4.4)
dictionary[tag::Nested3PartyIDSource]              = "Nested3PartyIDSource";                            // (char FIX.4.4)
dictionary[tag::Nested3PartyRole]                  = "Nested3PartyRole";                                // (int FIX.4.4)
dictionary[tag::NoNested3PartySubIDs]              = "NoNested3PartySubIDs";                            // (NumInGroup FIX.4.4)
dictionary[tag::Nested3PartySubID]                 = "Nested3PartySubID";                               // (String FIX.4.4)
dictionary[tag::Nested3PartySubIDType]             = "Nested3PartySubIDType";                           // (int FIX.4.4)
dictionary[tag::LegContractSettlMonth]             = "LegContractSettlMonth";                           // (MonthYear FIX.4.4)
dictionary[tag::LegInterestAccrualDate]            = "LegInterestAccrualDate";                          // (LocalMktDate FIX.4.4)
dictionary[tag::NoStrategyParameters]              = "NoStrategyParameters";                            // (NumInGroup FIX.4.4)
dictionary[tag::StrategyParameterName]             = "StrategyParameterName";                           // (String FIX.4.4)
dictionary[tag::StrategyParameterType]             = "StrategyParameterType";                           // (int FIX.4.4)
dictionary[tag::StrategyParameterValue]            = "StrategyParameterValue";                          // (String FIX.4.4)
dictionary[tag::HostCrossID]                       = "HostCrossID";                                     // (String FIX.4.4)
dictionary[tag::SideTimeInForce]                   = "SideTimeInForce";                                 // (UTCTimestamp FIX.4.4)
dictionary[tag::MDReportID]                        = "MDReportID";                                      // (int FIX.4.4)
dictionary[tag::SecurityReportID]                  = "SecurityReportID";                                // (int FIX.4.4)
dictionary[tag::SecurityStatus]                    = "SecurityStatus";                                  // (String FIX.4.4)
dictionary[tag::SettleOnOpenFlag]                  = "SettleOnOpenFlag";                                // (String FIX.4.4)
dictionary[tag::StrikeMultiplier]                  = "StrikeMultiplier";                                // (float FIX.4.4)
dictionary[tag::StrikeValue]                       = "StrikeValue";                                     // (float FIX.4.4)
dictionary[tag::MinPriceIncrement]                 = "MinPriceIncrement";                               // (float FIX.4.4)
dictionary[tag::PositionLimit]                     = "PositionLimit";                                   // (int FIX.4.4)
dictionary[tag::NTPositionLimit]                   = "NTPositionLimit";                                 // (int FIX.4.4)
dictionary[tag::UnderlyingAllocationPercent]       = "UnderlyingAllocationPercent";                     // (Percentage FIX.4.4)
dictionary[tag::UnderlyingCashAmount]              = "UnderlyingCashAmount";                            // (Amt FIX.4.4)
dictionary[tag::UnderlyingCashType]                = "UnderlyingCashType";                              // (String FIX.4.4)
dictionary[tag::UnderlyingSettlementType]          = "UnderlyingSettlementType";                        // (int FIX.4.4)
dictionary[tag::QuantityDate]                      = "QuantityDate";                                    // (LocalMktDate FIX.4.4)
dictionary[tag::ContIntRptID]                      = "ContIntRptID";                                    // (String FIX.4.4)
dictionary[tag::LateIndicator]                     = "LateIndicator";                                   // (Boolean FIX.4.4)
dictionary[tag::InputSource]                       = "InputSource";                                     // (String FIX.4.4)
dictionary[tag::NoExpiration]                      = "NoExpiration";                                    // (NumInGroup FIX.4.4)
dictionary[tag::ExpirationQtyType]                 = "ExpirationQtyType";                               // (int FIX.4.4)
dictionary[tag::ExpQty]                            = "ExpQty";                                          // (Qty FIX.4.4)
dictionary[tag::NoUnderlyingAmounts]               = "NoUnderlyingAmounts";                             // (NumInGroup FIX.4.4)
dictionary[tag::UnderlyingPayAmount]               = "UnderlyingPayAmount";                             // (Amt FIX.4.4)
dictionary[tag::UnderlyingCollectAmount]           = "UnderlyingCollectAmount";                         // (Amt FIX.4.4)
dictionary[tag::UnderlyingSettlementDate]          = "UnderlyingSettlementDate";                        // (LocalMktDate FIX.4.4)
dictionary[tag::UnderlyingSettlementStatus]        = "UnderlyingSettlementStatus";                      // (String FIX.4.4)
dictionary[tag::SecondaryIndividualAllocID]        = "SecondaryIndividualAllocID";                      // (String FIX.4.4)
dictionary[tag::LegReportID]                       = "LegReportID";                                     // (String FIX.4.4)
dictionary[tag::RndPx]                             = "RndPx";                                           // (Price FIX.4.4)
dictionary[tag::IndividualAllocType]               = "IndividualAllocType";                             // (int FIX.4.4)
dictionary[tag::AllocCustomerCapacity]             = "AllocCustomerCapacity";                           // (String FIX.4.4)
dictionary[tag::TierCode]                          = "TierCode";                                        // (String FIX.4.4)
dictionary[tag::UnitOfMeasure]                     = "UnitOfMeasure";                                   // (String FIX.4.4)
dictionary[tag::TimeUnit]                          = "TimeUnit";                                        // (String FIX.4.4)
dictionary[tag::UnderlyingUnitOfMeasure]           = "UnderlyingUnitOfMeasure";                         // (String FIX.4.4)
dictionary[tag::LegUnitOfMeasure]                  = "LegUnitOfMeasure";                                // (String FIX.4.4)
dictionary[tag::UnderlyingTimeUnit]                = "UnderlyingTimeUnit";                              // (String FIX.4.4)
dictionary[tag::LegTimeUnit]                       = "LegTimeUnit";                                     // (String FIX.4.4)
dictionary[tag::AllocMethod]                       = "AllocMethod";                                     // (int FIX.4.4)
dictionary[tag::TradeID]                           = "TradeID";                                         // (String FIX.4.4)
dictionary[tag::SideTradeReportID]                 = "SideTradeReportID";                               // (String FIX.4.4)
dictionary[tag::SideFillStationCd]                 = "SideFillStationCd";                               // (String FIX.4.4)
dictionary[tag::SideReasonCd]                      = "SideReasonCd";                                    // (String FIX.4.4)
dictionary[tag::SideTrdSubTyp]                     = "SideTrdSubTyp";                                   // (int FIX.4.4)
dictionary[tag::SideLastQty]                       = "SideLastQty";                                     // (Qty FIX.4.4)
dictionary[tag::MessageEventSource]                = "MessageEventSource";                              // (String FIX.4.4)
dictionary[tag::SideTrdRegTimestamp]               = "SideTrdRegTimestamp";                             // (UTCTimestamp FIX.4.4)
dictionary[tag::SideTrdRegTimestampType]           = "SideTrdRegTimestampType";                         // (int FIX.4.4)
dictionary[tag::SideTrdRegTimestampSrc]            = "SideTrdRegTimestampSrc";                          // (String FIX.4.4)
dictionary[tag::AsOfIndicator]                     = "AsOfIndicator";                                   // (char FIX.4.4)
dictionary[tag::NoSideTrdRegTS]                    = "NoSideTrdRegTS";                                  // (NumInGroup FIX.4.4)
dictionary[tag::LegOptionRatio]                    = "LegOptionRatio";                                  // (float FIX.4.4)
dictionary[tag::NoInstrumentParties]               = "NoInstrumentParties";                             // (NumInGroup FIX.4.4)
dictionary[tag::InstrumentPartyID]                 = "InstrumentPartyID";                               // (String FIX.4.4)
dictionary[tag::TradeVolume]                       = "TradeVolume";                                     // (Qty FIX.4.4)
dictionary[tag::MDBookType]                        = "MDBookType";                                      // (int FIX.4.4)
dictionary[tag::MDFeedType]                        = "MDFeedType";                                      // (String FIX.4.4)
dictionary[tag::MDPriceLevel]                      = "MDPriceLevel";                                    // (int FIX.4.4)
dictionary[tag::MDOriginType]                      = "MDOriginType";                                    // (int FIX.4.4)
dictionary[tag::FirstPx]                           = "FirstPx";                                         // (Price FIX.4.4)
dictionary[tag::MDEntrySpotRate]                   = "MDEntrySpotRate";                                 // (float FIX.4.4)
dictionary[tag::MDEntryForwardPoints]              = "MDEntryForwardPoints";                            // (PriceOffset FIX.4.4)
dictionary[tag::ManualOrderIndicator]              = "ManualOrderIndicator";                            // (Boolean FIX.4.4)
dictionary[tag::CustDirectedOrder]                 = "CustDirectedOrder";                               // (Boolean FIX.4.4)
dictionary[tag::ReceivedDeptID]                    = "ReceivedDeptID";                                  // (String FIX.4.4)
dictionary[tag::CustOrderHandlingInst]             = "CustOrderHandlingInst";                           // (MultipleStringValue FIX.4.4)
dictionary[tag::OrderHandlingInstSource]           = "OrderHandlingInstSource";                         // (int FIX.4.4)
dictionary[tag::DeskType]                          = "DeskType";                                        // (String FIX.4.4)
dictionary[tag::DeskTypeSource]                    = "DeskTypeSource";                                  // (int FIX.4.4)
dictionary[tag::DeskOrderHandlingInst]             = "DeskOrderHandlingInst";                           // (MultipleStringValue FIX.4.4)
dictionary[tag::ExecAckStatus]                     = "ExecAckStatus";                                   // (char FIX.4.4)
dictionary[tag::UnderlyingDeliveryAmount]          = "UnderlyingDeliveryAmount";                        // (Amt FIX.4.4)
dictionary[tag::UnderlyingCapValue]                = "UnderlyingCapValue";                              // (Amt FIX.4.4)
dictionary[tag::UnderlyingSettlMethod]             = "UnderlyingSettlMethod";                           // (String FIX.4.4)
dictionary[tag::SecondaryTradeID]                  = "SecondaryTradeID";                                // (String FIX.4.4)
dictionary[tag::FirmTradeID]                       = "FirmTradeID";                                     // (String FIX.4.4)
dictionary[tag::SecondaryFirmTradeID]              = "SecondaryFirmTradeID";                            // (String FIX.4.4)
dictionary[tag::CollApplType]                      = "CollApplType";                                    // (int FIX.4.4)
dictionary[tag::UnderlyingAdjustedQuantity]        = "UnderlyingAdjustedQuantity";                      // (Qty FIX.4.4)
dictionary[tag::UnderlyingFXRate]                  = "UnderlyingFXRate";                                // (float FIX.4.4)
dictionary[tag::UnderlyingFXRateCalc]              = "UnderlyingFXRateCalc";                            // (char FIX.4.4)
dictionary[tag::AllocPositionEffect]               = "AllocPositionEffect";                             // (char FIX.4.4)
dictionary[tag::DealingCapacity]                   = "DealingCapacity";                                 // (char FIX.4.4)
dictionary[tag::InstrmtAssignmentMethod]           = "InstrmtAssignmentMethod";                         // (char FIX.4.4)
dictionary[tag::InstrumentPartyIDSource]           = "InstrumentPartyIDSource";                         // (char FIX.4.4)
dictionary[tag::InstrumentPartyRole]               = "InstrumentPartyRole";                             // (int FIX.4.4)
dictionary[tag::NoInstrumentPartySubIDs]           = "NoInstrumentPartySubIDs";                         // (NumInGroup FIX.4.4)
dictionary[tag::InstrumentPartySubID]              = "InstrumentPartySubID";                            // (String FIX.4.4)
dictionary[tag::InstrumentPartySubIDType]          = "InstrumentPartySubIDType";                        // (int FIX.4.4)
dictionary[tag::PositionCurrency]                  = "PositionCurrency";                                // (String FIX.4.4)
dictionary[tag::CalculatedCcyLastQty]              = "CalculatedCcyLastQty";                            // (Qty FIX.4.4)
dictionary[tag::AggressorIndicator]                = "AggressorIndicator";                              // (Boolean FIX.4.4)
dictionary[tag::NoUndlyInstrumentParties]          = "NoUndlyInstrumentParties";                        // (NumInGroup FIX.4.4)
dictionary[tag::UnderlyingInstrumentPartyID]       = "UnderlyingInstrumentPartyID";                     // (String FIX.4.4)
dictionary[tag::UnderlyingInstrumentPartyIDSource] = "UnderlyingInstrumentPartyIDSource";               // (char FIX.4.4)
dictionary[tag::UnderlyingInstrumentPartyRole]     = "UnderlyingInstrumentPartyRole";                   // (int FIX.4.4)
dictionary[tag::NoUndlyInstrumentPartySubIDs]      = "NoUndlyInstrumentPartySubIDs";                    // (NumInGroup FIX.4.4)
dictionary[tag::UnderlyingInstrumentPartySubID]    = "UnderlyingInstrumentPartySubID";                  // (String FIX.4.4)
dictionary[tag::UnderlyingInstrumentPartySubIDType] = "UnderlyingInstrumentPartySubIDType";              // (int FIX.4.4)
dictionary[tag::BidSwapPoints]                     = "BidSwapPoints";                                   // (PriceOffset FIX.4.4)
dictionary[tag::OfferSwapPoints]                   = "OfferSwapPoints";                                 // (PriceOffset FIX.4.4)
dictionary[tag::LegBidForwardPoints]               = "LegBidForwardPoints";                             // (PriceOffset FIX.4.4)
dictionary[tag::LegOfferForwardPoints]             = "LegOfferForwardPoints";                           // (PriceOffset FIX.4.4)
dictionary[tag::SwapPoints]                        = "SwapPoints";                                      // (PriceOffset FIX.4.4)
dictionary[tag::MDQuoteType]                       = "MDQuoteType";                                     // (int FIX.4.4)
dictionary[tag::LastSwapPoints]                    = "LastSwapPoints";                                  // (PriceOffset FIX.4.4)
dictionary[tag::SideGrossTradeAmt]                 = "SideGrossTradeAmt";                               // (Amt FIX.4.4)
dictionary[tag::LegLastForwardPoints]              = "LegLastForwardPoints";                            // (PriceOffset FIX.4.4)
dictionary[tag::LegCalculatedCcyLastQty]           = "LegCalculatedCcyLastQty";                         // (Qty FIX.4.4)
dictionary[tag::LegGrossTradeAmt]                  = "LegGrossTradeAmt";                                // (Amt FIX.4.4)
dictionary[tag::MaturityTime]                      = "MaturityTime";                                    // (TZTimeOnly FIX.4.4)
dictionary[tag::RefOrderID]                        = "RefOrderID";                                      // (String FIX.4.4)
dictionary[tag::RefOrderIDSource]                  = "RefOrderIDSource";                                // (char FIX.4.4)
dictionary[tag::SecondaryDisplayQty]               = "SecondaryDisplayQty";                             // (Qty FIX.4.4)
dictionary[tag::DisplayWhen]                       = "DisplayWhen";                                     // (char FIX.4.4)
dictionary[tag::DisplayMethod]                     = "DisplayMethod";                                   // (char FIX.4.4)
dictionary[tag::DisplayLowQty]                     = "DisplayLowQty";                                   // (Qty FIX.4.4)
dictionary[tag::DisplayHighQty]                    = "DisplayHighQty";                                  // (Qty FIX.4.4)
dictionary[tag::DisplayMinIncr]                    = "DisplayMinIncr";                                  // (Qty FIX.4.4)
dictionary[tag::RefreshQty]                        = "RefreshQty";                                      // (Qty FIX.4.4)
dictionary[tag::MatchIncrement]                    = "MatchIncrement";                                  // (Qty FIX.4.4)
dictionary[tag::MaxPriceLevels]                    = "MaxPriceLevels";                                  // (int FIX.4.4)
dictionary[tag::PreTradeAnonymity]                 = "PreTradeAnonymity";                               // (Boolean FIX.4.4)
dictionary[tag::PriceProtectionScope]              = "PriceProtectionScope";                            // (char FIX.4.4)
dictionary[tag::LotType]                           = "LotType";                                         // (char FIX.4.4)
dictionary[tag::PegPriceType]                      = "PegPriceType";                                    // (int FIX.4.4)
dictionary[tag::PeggedRefPrice]                    = "PeggedRefPrice";                                  // (Price FIX.4.4)
dictionary[tag::PegSecurityIDSource]               = "PegSecurityIDSource";                             // (String FIX.4.4)
dictionary[tag::PegSecurityID]                     = "PegSecurityID";                                   // (String FIX.4.4)
dictionary[tag::PegSymbol]                         = "PegSymbol";                                       // (String FIX.4.4)
dictionary[tag::PegSecurityDesc]                   = "PegSecurityDesc";                                 // (String FIX.4.4)
dictionary[tag::TriggerType]                       = "TriggerType";                                     // (char FIX.5.0)
dictionary[tag::TriggerAction]                     = "TriggerAction";                                   // (char FIX.5.0)
dictionary[tag::TriggerPrice]                      = "TriggerPrice";                                    // (Price FIX.5.0)
dictionary[tag::TriggerSymbol]                     = "TriggerSymbol";                                   // (String FIX.5.0)
dictionary[tag::TriggerSecurityID]                 = "TriggerSecurityID";                               // (String FIX.5.0)
dictionary[tag::TriggerSecurityIDSource]           = "TriggerSecurityIDSource";                         // (String FIX.5.0)
dictionary[tag::TriggerSecurityDesc]               = "TriggerSecurityDesc";                             // (String FIX.5.0)
dictionary[tag::TriggerPriceType]                  = "TriggerPriceType";                                // (char FIX.5.0)
dictionary[tag::TriggerPriceTypeScope]             = "TriggerPriceTypeScope";                           // (char FIX.5.0)
dictionary[tag::TriggerPriceDirection]             = "TriggerPriceDirection";                           // (char FIX.5.0)
dictionary[tag::TriggerNewPrice]                   = "TriggerNewPrice";                                 // (Price FIX.5.0)
dictionary[tag::TriggerOrderType]                  = "TriggerOrderType";                                // (char FIX.5.0)
dictionary[tag::TriggerNewQty]                     = "TriggerNewQty";                                   // (Qty FIX.5.0)
dictionary[tag::TriggerTradingSessionID]           = "TriggerTradingSessionID";                         // (String FIX.5.0)
dictionary[tag::TriggerTradingSessionSubID]        = "TriggerTradingSessionSubID";                      // (String FIX.5.0)
dictionary[tag::OrderCategory]                     = "OrderCategory";                                   // (char FIX.4.4)
dictionary[tag::NoRootPartyIDs]                    = "NoRootPartyIDs";                                  // (NumInGroup FIX.4.4)
dictionary[tag::RootPartyID]                       = "RootPartyID";                                     // (String FIX.4.4)
dictionary[tag::RootPartyIDSource]                 = "RootPartyIDSource";                               // (char FIX.4.4)
dictionary[tag::RootPartyRole]                     = "RootPartyRole";                                   // (int FIX.4.4)
dictionary[tag::NoRootPartySubIDs]                 = "NoRootPartySubIDs";                               // (NumInGroup FIX.4.4)
dictionary[tag::RootPartySubID]                    = "RootPartySubID";                                  // (String FIX.4.4)
dictionary[tag::RootPartySubIDType]                = "RootPartySubIDType";                              // (int FIX.4.4)
dictionary[tag::TradeHandlingInstr]                = "TradeHandlingInstr";                              // (char FIX.4.4)
dictionary[tag::OrigTradeHandlingInstr]            = "OrigTradeHandlingInstr";                          // (char FIX.4.4)
dictionary[tag::OrigTradeDate]                     = "OrigTradeDate";                                   // (LocalMktDate FIX.4.4)
dictionary[tag::OrigTradeID]                       = "OrigTradeID";                                     // (String FIX.4.4)
dictionary[tag::OrigSecondaryTradeID]              = "OrigSecondaryTradeID";                            // (String FIX.4.4)
dictionary[tag::ApplVerID]                         = "ApplVerID";                                       // (String FIX.4.4)
dictionary[tag::CstmApplVerID]                     = "CstmApplVerID";                                   // (String FIX.4.4)
dictionary[tag::RefApplVerID]                      = "RefApplVerID";                                    // (String FIX.4.4)
dictionary[tag::RefCstmApplVerID]                  = "RefCstmApplVerID";                                // (String FIX.4.4)
dictionary[tag::TZTransactTime]                    = "TZTransactTime";                                  // (TZTimestamp FIX.4.4)
dictionary[tag::ExDestinationIDSource]             = "ExDestinationIDSource";                           // (char FIX.4.4)
dictionary[tag::ReportedPxDiff]                    = "ReportedPxDiff";                                  // (Boolean FIX.4.4)
dictionary[tag::RptSys]                            = "RptSys";                                          // (String FIX.4.4)
dictionary[tag::AllocClearingFeeIndicator]         = "AllocClearingFeeIndicator";                       // (String FIX.4.4)
dictionary[tag::DefaultApplVerID]                  = "DefaultApplVerID";                                // (String FIX.4.4)
dictionary[tag::DisplayQty]                        = "DisplayQty";                                      // (Qty FIX.4.4)
dictionary[tag::ExchangeSpecialInstructions]       = "ExchangeSpecialInstructions";                     // (String FIX.4.4)
dictionary[tag::MaxTradeVol]                       = "MaxTradeVol";                                     // (Qty FIX.5.0)
dictionary[tag::NoMDFeedTypes]                     = "NoMDFeedTypes";                                   // (NumInGroup FIX.5.0)
dictionary[tag::MatchAlgorithm]                    = "MatchAlgorithm";                                  // (String FIX.5.0)
dictionary[tag::MaxPriceVariation]                 = "MaxPriceVariation";                               // (float FIX.5.0)
dictionary[tag::ImpliedMarketIndicator]            = "ImpliedMarketIndicator";                          // (int FIX.5.0)
dictionary[tag::EventTime]                         = "EventTime";                                       // (UTCTimestamp FIX.5.0)
dictionary[tag::MinPriceIncrementAmount]           = "MinPriceIncrementAmount";                         // (Amt FIX.5.0)
dictionary[tag::UnitOfMeasureQty]                  = "UnitOfMeasureQty";                                // (Qty FIX.5.0)
dictionary[tag::LowLimitPrice]                     = "LowLimitPrice";                                   // (Price FIX.5.0)
dictionary[tag::HighLimitPrice]                    = "HighLimitPrice";                                  // (Price FIX.5.0)
dictionary[tag::TradingReferencePrice]             = "TradingReferencePrice";                           // (Price FIX.5.0)
dictionary[tag::SecurityGroup]                     = "SecurityGroup";                                   // (String FIX.5.0)
dictionary[tag::LegNumber]                         = "LegNumber";                                       // (int FIX.5.0)
dictionary[tag::SettlementCycleNo]                 = "SettlementCycleNo";                               // (int FIX.5.0)
dictionary[tag::SideCurrency]                      = "SideCurrency";                                    // (Currency FIX.5.0)
dictionary[tag::SideSettlCurrency]                 = "SideSettlCurrency";                               // (Currency FIX.5.0)
dictionary[tag::ApplExtID]                         = "ApplExtID";                                       // (int FIX.5.0)
dictionary[tag::CcyAmt]                            = "CcyAmt";                                          // (Amt FIX.5.0)
dictionary[tag::NoSettlDetails]                    = "NoSettlDetails";                                  // (NumInGroup FIX.5.0)
dictionary[tag::SettlObligMode]                    = "SettlObligMode";                                  // (int FIX.5.0)
dictionary[tag::SettlObligMsgID]                   = "SettlObligMsgID";                                 // (String FIX.5.0)
dictionary[tag::SettlObligID]                      = "SettlObligID";                                    // (String FIX.5.0)
dictionary[tag::SettlObligTransType]               = "SettlObligTransType";                             // (char FIX.5.0)
dictionary[tag::SettlObligRefID]                   = "SettlObligRefID";                                 // (String FIX.5.0)
dictionary[tag::SettlObligSource]                  = "SettlObligSource";                                // (char FIX.5.0)
dictionary[tag::NoSettlOblig]                      = "NoSettlOblig";                                    // (NumInGroup FIX.5.0)
dictionary[tag::QuoteMsgID]                        = "QuoteMsgID";                                      // (String FIX.5.0)
dictionary[tag::QuoteEntryStatus]                  = "QuoteEntryStatus";                                // (int FIX.5.0)
dictionary[tag::TotNoCxldQuotes]                   = "TotNoCxldQuotes";                                 // (int FIX.5.0)
dictionary[tag::TotNoAccQuotes]                    = "TotNoAccQuotes";                                  // (int FIX.5.0)
dictionary[tag::TotNoRejQuotes]                    = "TotNoRejQuotes";                                  // (int FIX.5.0)
dictionary[tag::PrivateQuote]                      = "PrivateQuote";                                    // (Boolean FIX.5.0)
dictionary[tag::RespondentType]                    = "RespondentType";                                  // (int FIX.5.0)
dictionary[tag::MDSubBookType]                     = "MDSubBookType";                                   // (int FIX.5.0)
dictionary[tag::SecurityTradingEvent]              = "SecurityTradingEvent";                            // (int FIX.5.0)
dictionary[tag::NoStatsIndicators]                 = "NoStatsIndicators";                               // (NumInGroup FIX.5.0)
dictionary[tag::StatsType]                         = "StatsType";                                       // (int FIX.5.0)
dictionary[tag::NoOfSecSizes]                      = "NoOfSecSizes";                                    // (NumInGroup FIX.5.0)
dictionary[tag::MDSecSizeType]                     = "MDSecSizeType";                                   // (int FIX.5.0)
dictionary[tag::MDSecSize]                         = "MDSecSize";                                       // (Qty FIX.5.0)
dictionary[tag::ApplID]                            = "ApplID";                                          // (String FIX.5.0)
dictionary[tag::ApplSeqNum]                        = "ApplSeqNum";                                      // (SeqNum FIX.5.0)
dictionary[tag::ApplBegSeqNum]                     = "ApplBegSeqNum";                                   // (SeqNum FIX.5.0)
dictionary[tag::ApplEndSeqNum]                     = "ApplEndSeqNum";                                   // (SeqNum FIX.5.0)
dictionary[tag::SecurityXMLLen]                    = "SecurityXMLLen";                                  // (Length FIX.5.0)
dictionary[tag::SecurityXML]                       = "SecurityXML";                                     // (XMLData FIX.5.0)
dictionary[tag::SecurityXMLSchema]                 = "SecurityXMLSchema";                               // (String FIX.5.0)
dictionary[tag::RefreshIndicator]                  = "RefreshIndicator";                                // (Boolean FIX.5.0)
dictionary[tag::Volatility]                        = "Volatility";                                      // (float FIX.5.0)
dictionary[tag::TimeToExpiration]                  = "TimeToExpiration";                                // (float FIX.5.0)
dictionary[tag::RiskFreeRate]                      = "RiskFreeRate";                                    // (float FIX.5.0)
dictionary[tag::PriceUnitOfMeasure]                = "PriceUnitOfMeasure";                              // (String FIX.5.0)
dictionary[tag::PriceUnitOfMeasureQty]             = "PriceUnitOfMeasureQty";                           // (Qty FIX.5.0)
dictionary[tag::SettlMethod]                       = "SettlMethod";                                     // (String FIX.5.0)
dictionary[tag::ExerciseStyle]                     = "ExerciseStyle";                                   // (int FIX.5.0)
dictionary[tag::OptPayoutAmount]                   = "OptPayoutAmount";                                 // (Amt FIX.5.0)
dictionary[tag::PriceQuoteMethod]                  = "PriceQuoteMethod";                                // (String FIX.5.0)
dictionary[tag::ValuationMethod]                   = "ValuationMethod";                                 // (String FIX.5.0)
dictionary[tag::ListMethod]                        = "ListMethod";                                      // (int FIX.5.0)
dictionary[tag::CapPrice]                          = "CapPrice";                                        // (Price FIX.5.0)
dictionary[tag::FloorPrice]                        = "FloorPrice";                                      // (Price FIX.5.0)
dictionary[tag::NoStrikeRules]                     = "NoStrikeRules";                                   // (NumInGroup FIX.5.0)
dictionary[tag::StartStrikePxRange]                = "StartStrikePxRange";                              // (Price FIX.5.0)
dictionary[tag::EndStrikePxRange]                  = "EndStrikePxRange";                                // (Price FIX.5.0)
dictionary[tag::StrikeIncrement]                   = "StrikeIncrement";                                 // (float FIX.5.0)
dictionary[tag::NoTickRules]                       = "NoTickRules";                                     // (NumInGroup FIX.5.0)
dictionary[tag::StartTickPriceRange]               = "StartTickPriceRange";                             // (Price FIX.5.0)
dictionary[tag::EndTickPriceRange]                 = "EndTickPriceRange";                               // (Price FIX.5.0)
dictionary[tag::TickIncrement]                     = "TickIncrement";                                   // (Price FIX.5.0)
dictionary[tag::TickRuleType]                      = "TickRuleType";                                    // (int FIX.5.0)
dictionary[tag::NestedInstrAttribType]             = "NestedInstrAttribType";                           // (int FIX.5.0)
dictionary[tag::NestedInstrAttribValue]            = "NestedInstrAttribValue";                          // (String FIX.5.0)
dictionary[tag::LegMaturityTime]                   = "LegMaturityTime";                                 // (TZTimeOnly FIX.5.0)
dictionary[tag::UnderlyingMaturityTime]            = "UnderlyingMaturityTime";                          // (TZTimeOnly FIX.5.0)
dictionary[tag::DerivativeSymbol]                  = "DerivativeSymbol";                                // (String FIX.5.0)
dictionary[tag::DerivativeSymbolSfx]               = "DerivativeSymbolSfx";                             // (String FIX.5.0)
dictionary[tag::DerivativeSecurityID]              = "DerivativeSecurityID";                            // (String FIX.5.0)
dictionary[tag::DerivativeSecurityIDSource]        = "DerivativeSecurityIDSource";                      // (String FIX.5.0)
dictionary[tag::NoDerivativeSecurityAltID]         = "NoDerivativeSecurityAltID";                       // (NumInGroup FIX.5.0)
dictionary[tag::DerivativeSecurityAltID]           = "DerivativeSecurityAltID";                         // (String FIX.5.0)
dictionary[tag::DerivativeSecurityAltIDSource]     = "DerivativeSecurityAltIDSource";                   // (String FIX.5.0)
dictionary[tag::SecondaryLowLimitPrice]            = "SecondaryLowLimitPrice";                          // (Price FIX.5.0)
dictionary[tag::MaturityRuleID]                    = "MaturityRuleID";                                  // (String FIX.5.0)
dictionary[tag::StrikeRuleID]                      = "StrikeRuleID";                                    // (String FIX.5.0)
dictionary[tag::LegUnitOfMeasureQty]               = "LegUnitOfMeasureQty";                             // (Qty FIX.5.0)
dictionary[tag::DerivativeOptPayAmount]            = "DerivativeOptPayAmount";                          // (Amt FIX.5.0)
dictionary[tag::EndMaturityMonthYear]              = "EndMaturityMonthYear";                            // (MonthYear FIX.5.0)
dictionary[tag::ProductComplex]                    = "ProductComplex";                                  // (String FIX.5.0)
dictionary[tag::DerivativeProductComplex]          = "DerivativeProductComplex";                        // (String FIX.5.0)
dictionary[tag::MaturityMonthYearIncrement]        = "MaturityMonthYearIncrement";                      // (int FIX.5.0)
dictionary[tag::SecondaryHighLimitPrice]           = "SecondaryHighLimitPrice";                         // (Price FIX.5.0)
dictionary[tag::MinLotSize]                        = "MinLotSize";                                      // (Qty FIX.5.0)
dictionary[tag::NoExecInstRules]                   = "NoExecInstRules";                                 // (NumInGroup FIX.5.0)
dictionary[tag::CommRate]                          = "CommRate";                                        // (float FIX.5.0SP2)
dictionary[tag::NoLotTypeRules]                    = "NoLotTypeRules";                                  // (NumInGroup FIX.5.0)
dictionary[tag::NoMatchRules]                      = "NoMatchRules";                                    // (NumInGroup FIX.5.0)
dictionary[tag::NoMaturityRules]                   = "NoMaturityRules";                                 // (NumInGroup FIX.5.0)
dictionary[tag::NoOrdTypeRules]                    = "NoOrdTypeRules";                                  // (NumInGroup FIX.5.0)
dictionary[tag::CommUnitOfMeasure]                 = "CommUnitOfMeasure";                               // (String FIX.5.0SP2)
dictionary[tag::NoTimeInForceRules]                = "NoTimeInForceRules";                              // (NumInGroup FIX.5.0)
dictionary[tag::SecondaryTradingReferencePrice]    = "SecondaryTradingReferencePrice";                  // (Price FIX.5.0)
dictionary[tag::StartMaturityMonthYear]            = "StartMaturityMonthYear";                          // (MonthYear FIX.5.0)
dictionary[tag::FlexProductEligibilityIndicator]   = "FlexProductEligibilityIndicator";                 // (Boolean FIX.5.0)
dictionary[tag::DerivFlexProductEligibilityIndicator] = "DerivFlexProductEligibilityIndicator";            // (Boolean FIX.5.0)
dictionary[tag::FlexibleIndicator]                 = "FlexibleIndicator";                               // (Boolean FIX.5.0)
dictionary[tag::TradingCurrency]                   = "TradingCurrency";                                 // (Currency FIX.5.0)
dictionary[tag::DerivativeSecurityXMLLen]          = "DerivativeSecurityXMLLen";                        // (Length FIX.5.0)
dictionary[tag::DerivativeSecurityXML]             = "DerivativeSecurityXML";                           // (data FIX.5.0)
dictionary[tag::DerivativeSecurityXMLSchema]       = "DerivativeSecurityXMLSchema";                     // (String FIX.5.0)
dictionary[tag::NoDerivativeInstrumentParties]     = "NoDerivativeInstrumentParties";                   // (NumInGroup FIX.5.0)
dictionary[tag::DerivativeInstrumentPartyID]       = "DerivativeInstrumentPartyID";                     // (String FIX.5.0)
dictionary[tag::DerivativeInstrumentPartyIDSource] = "DerivativeInstrumentPartyIDSource";               // (char FIX.5.0)
dictionary[tag::DerivativeInstrumentPartyRole]     = "DerivativeInstrumentPartyRole";                   // (int FIX.5.0)
dictionary[tag::NoDerivativeInstrumentPartySubIDs] = "NoDerivativeInstrumentPartySubIDs";               // (NumInGroup FIX.5.0)
dictionary[tag::DerivativeInstrumentPartySubID]    = "DerivativeInstrumentPartySubID";                  // (String FIX.5.0)
dictionary[tag::DerivativeInstrumentPartySubIDType] = "DerivativeInstrumentPartySubIDType";              // (int FIX.5.0)
dictionary[tag::DerivativeExerciseStyle]           = "DerivativeExerciseStyle";                         // (int FIX.5.0)
dictionary[tag::MarketSegmentID]                   = "MarketSegmentID";                                 // (String FIX.5.0)
dictionary[tag::MarketID]                          = "MarketID";                                        // (Exchange FIX.5.0)
dictionary[tag::MaturityMonthYearIncrementUnits]   = "MaturityMonthYearIncrementUnits";                 // (int FIX.5.0)
dictionary[tag::MaturityMonthYearFormat]           = "MaturityMonthYearFormat";                         // (int FIX.5.0)
dictionary[tag::StrikeExerciseStyle]               = "StrikeExerciseStyle";                             // (int FIX.5.0)
dictionary[tag::SecondaryPriceLimitType]           = "SecondaryPriceLimitType";                         // (int FIX.5.0)
dictionary[tag::PriceLimitType]                    = "PriceLimitType";                                  // (int FIX.5.0)
dictionary[tag::ExecInstValue]                     = "ExecInstValue";                                   // (MultipleCharValue FIX.5.0)
dictionary[tag::NoTradingSessionRules]             = "NoTradingSessionRules";                           // (NumInGroup FIX.5.0)
dictionary[tag::NoMarketSegments]                  = "NoMarketSegments";                                // (NumInGroup FIX.5.0)
dictionary[tag::DerivativeInstrAttribType]         = "DerivativeInstrAttribType";                       // (int FIX.5.0)
dictionary[tag::DerivativeInstrAttribValue]        = "DerivativeInstrAttribValue";                      // (String FIX.5.0)
dictionary[tag::DerivativePriceUnitOfMeasure]      = "DerivativePriceUnitOfMeasure";                    // (String FIX.5.0)
dictionary[tag::DerivativePriceUnitOfMeasureQty]   = "DerivativePriceUnitOfMeasureQty";                 // (Qty FIX.5.0)
dictionary[tag::DerivativeSettlMethod]             = "DerivativeSettlMethod";                           // (String FIX.5.0)
dictionary[tag::DerivativePriceQuoteMethod]        = "DerivativePriceQuoteMethod";                      // (String FIX.5.0)
dictionary[tag::DerivativeValuationMethod]         = "DerivativeValuationMethod";                       // (String FIX.5.0)
dictionary[tag::DerivativeListMethod]              = "DerivativeListMethod";                            // (int FIX.5.0)
dictionary[tag::DerivativeCapPrice]                = "DerivativeCapPrice";                              // (Price FIX.5.0)
dictionary[tag::DerivativeFloorPrice]              = "DerivativeFloorPrice";                            // (Price FIX.5.0)
dictionary[tag::DerivativePutOrCall]               = "DerivativePutOrCall";                             // (int FIX.5.0)
dictionary[tag::ListUpdateAction]                  = "ListUpdateAction";                                // (char FIX.5.0)
dictionary[tag::ParentMktSegmID]                   = "ParentMktSegmID";                                 // (String FIX.5.0)
dictionary[tag::TradingSessionDesc]                = "TradingSessionDesc";                              // (String FIX.5.0)
dictionary[tag::TradSesUpdateAction]               = "TradSesUpdateAction";                             // (char FIX.5.0)
dictionary[tag::RejectText]                        = "RejectText";                                      // (String FIX.5.0)
dictionary[tag::FeeMultiplier]                     = "FeeMultiplier";                                   // (float FIX.5.0)
dictionary[tag::UnderlyingLegSymbol]               = "UnderlyingLegSymbol";                             // (String FIX.5.0)
dictionary[tag::UnderlyingLegSymbolSfx]            = "UnderlyingLegSymbolSfx";                          // (String FIX.5.0)
dictionary[tag::UnderlyingLegSecurityID]           = "UnderlyingLegSecurityID";                         // (String FIX.5.0)
dictionary[tag::UnderlyingLegSecurityIDSource]     = "UnderlyingLegSecurityIDSource";                   // (String FIX.5.0)
dictionary[tag::NoUnderlyingLegSecurityAltID]      = "NoUnderlyingLegSecurityAltID";                    // (NumInGroup FIX.5.0)
dictionary[tag::UnderlyingLegSecurityAltID]        = "UnderlyingLegSecurityAltID";                      // (String FIX.5.0)
dictionary[tag::UnderlyingLegSecurityAltIDSource]  = "UnderlyingLegSecurityAltIDSource";                // (String FIX.5.0)
dictionary[tag::UnderlyingLegSecurityType]         = "UnderlyingLegSecurityType";                       // (String FIX.5.0)
dictionary[tag::UnderlyingLegSecuritySubType]      = "UnderlyingLegSecuritySubType";                    // (String FIX.5.0)
dictionary[tag::UnderlyingLegMaturityMonthYear]    = "UnderlyingLegMaturityMonthYear";                  // (MonthYear FIX.5.0)
dictionary[tag::UnderlyingLegStrikePrice]          = "UnderlyingLegStrikePrice";                        // (Price FIX.5.0)
dictionary[tag::UnderlyingLegSecurityExchange]     = "UnderlyingLegSecurityExchange";                   // (String FIX.5.0)
dictionary[tag::NoOfLegUnderlyings]                = "NoOfLegUnderlyings";                              // (NumInGroup FIX.5.0)
dictionary[tag::UnderlyingLegPutOrCall]            = "UnderlyingLegPutOrCall";                          // (int FIX.5.0)
dictionary[tag::UnderlyingLegCFICode]              = "UnderlyingLegCFICode";                            // (String FIX.5.0)
dictionary[tag::UnderlyingLegMaturityDate]         = "UnderlyingLegMaturityDate";                       // (LocalMktDate FIX.5.0)
dictionary[tag::ApplReqID]                         = "ApplReqID";                                       // (String FIX.5.0)
dictionary[tag::ApplReqType]                       = "ApplReqType";                                     // (int FIX.5.0)
dictionary[tag::ApplResponseType]                  = "ApplResponseType";                                // (int FIX.5.0)
dictionary[tag::ApplTotalMessageCount]             = "ApplTotalMessageCount";                           // (int FIX.5.0)
dictionary[tag::ApplLastSeqNum]                    = "ApplLastSeqNum";                                  // (SeqNum FIX.5.0)
dictionary[tag::NoApplIDs]                         = "NoApplIDs";                                       // (NumInGroup FIX.5.0)
dictionary[tag::ApplResendFlag]                    = "ApplResendFlag";                                  // (Boolean FIX.5.0)
dictionary[tag::ApplResponseID]                    = "ApplResponseID";                                  // (String FIX.5.0)
dictionary[tag::ApplResponseError]                 = "ApplResponseError";                               // (int FIX.5.0)
dictionary[tag::RefApplID]                         = "RefApplID";                                       // (String FIX.5.0)
dictionary[tag::ApplReportID]                      = "ApplReportID";                                    // (String FIX.5.0)
dictionary[tag::RefApplLastSeqNum]                 = "RefApplLastSeqNum";                               // (SeqNum FIX.5.0)
dictionary[tag::LegPutOrCall]                      = "LegPutOrCall";                                    // (int FIX.5.0)
dictionary[tag::TotNoFills]                        = "TotNoFills";                                      // (int FIX.5.0)
dictionary[tag::FillExecID]                        = "FillExecID";                                      // (String FIX.5.0)
dictionary[tag::FillPx]                            = "FillPx";                                          // (Price FIX.5.0)
dictionary[tag::FillQty]                           = "FillQty";                                         // (Qty FIX.5.0)
dictionary[tag::LegAllocID]                        = "LegAllocID";                                      // (String FIX.5.0)
dictionary[tag::LegAllocSettlCurrency]             = "LegAllocSettlCurrency";                           // (Currency FIX.5.0)
dictionary[tag::TradSesEvent]                      = "TradSesEvent";                                    // (int FIX.5.0)
dictionary[tag::MassActionReportID]                = "MassActionReportID";                              // (String FIX.5.0)
dictionary[tag::NoNotAffectedOrders]               = "NoNotAffectedOrders";                             // (NumInGroup FIX.5.0)
dictionary[tag::NotAffectedOrderID]                = "NotAffectedOrderID";                              // (String FIX.5.0)
dictionary[tag::NotAffOrigClOrdID]                 = "NotAffOrigClOrdID";                               // (String FIX.5.0)
dictionary[tag::MassActionType]                    = "MassActionType";                                  // (int FIX.5.0)
dictionary[tag::MassActionScope]                   = "MassActionScope";                                 // (int FIX.5.0)
dictionary[tag::MassActionResponse]                = "MassActionResponse";                              // (int FIX.5.0)
dictionary[tag::MassActionRejectReason]            = "MassActionRejectReason";                          // (int FIX.5.0)
dictionary[tag::MultilegModel]                     = "MultilegModel";                                   // (int FIX.5.0)
dictionary[tag::MultilegPriceMethod]               = "MultilegPriceMethod";                             // (int FIX.5.0)
dictionary[tag::LegVolatility]                     = "LegVolatility";                                   // (float FIX.5.0)
dictionary[tag::DividendYield]                     = "DividendYield";                                   // (Percentage FIX.5.0)
dictionary[tag::LegDividendYield]                  = "LegDividendYield";                                // (Percentage FIX.5.0)
dictionary[tag::CurrencyRatio]                     = "CurrencyRatio";                                   // (float FIX.5.0)
dictionary[tag::LegCurrencyRatio]                  = "LegCurrencyRatio";                                // (float FIX.5.0)
dictionary[tag::LegExecInst]                       = "LegExecInst";                                     // (MultipleCharValue FIX.5.0)
dictionary[tag::ContingencyType]                   = "ContingencyType";                                 // (int FIX.5.0)
dictionary[tag::ListRejectReason]                  = "ListRejectReason";                                // (int FIX.5.0)
dictionary[tag::NoTrdRepIndicators]                = "NoTrdRepIndicators";                              // (NumInGroup FIX.5.0)
dictionary[tag::TrdRepPartyRole]                   = "TrdRepPartyRole";                                 // (int FIX.5.0)
dictionary[tag::TrdRepIndicator]                   = "TrdRepIndicator";                                 // (Boolean FIX.5.0)
dictionary[tag::TradePublishIndicator]             = "TradePublishIndicator";                           // (int FIX.5.0)
dictionary[tag::UnderlyingLegOptAttribute]         = "UnderlyingLegOptAttribute";                       // (char FIX.5.0)
dictionary[tag::UnderlyingLegSecurityDesc]         = "UnderlyingLegSecurityDesc";                       // (String FIX.5.0)
dictionary[tag::MarketReqID]                       = "MarketReqID";                                     // (String FIX.5.0)
dictionary[tag::MarketReportID]                    = "MarketReportID";                                  // (String FIX.5.0)
dictionary[tag::MarketUpdateAction]                = "MarketUpdateAction";                              // (char FIX.5.0)
dictionary[tag::MarketSegmentDesc]                 = "MarketSegmentDesc";                               // (String FIX.5.0)
dictionary[tag::EncodedMktSegmDescLen]             = "EncodedMktSegmDescLen";                           // (Length FIX.5.0)
dictionary[tag::EncodedMktSegmDesc]                = "EncodedMktSegmDesc";                              // (data FIX.5.0)
dictionary[tag::ApplNewSeqNum]                     = "ApplNewSeqNum";                                   // (SeqNum FIX.5.0)
dictionary[tag::EncryptedPasswordMethod]           = "EncryptedPasswordMethod";                         // (int FIX.5.0)
dictionary[tag::EncryptedPasswordLen]              = "EncryptedPasswordLen";                            // (Length FIX.5.0)
dictionary[tag::EncryptedPassword]                 = "EncryptedPassword";                               // (data FIX.5.0)
dictionary[tag::EncryptedNewPasswordLen]           = "EncryptedNewPasswordLen";                         // (Length FIX.5.0)
dictionary[tag::EncryptedNewPassword]              = "EncryptedNewPassword";                            // (data FIX.5.0)
dictionary[tag::UnderlyingLegMaturityTime]         = "UnderlyingLegMaturityTime";                       // (TZTimeOnly FIX.5.0)
dictionary[tag::RefApplExtID]                      = "RefApplExtID";                                    // (int FIX.5.0)
dictionary[tag::DefaultApplExtID]                  = "DefaultApplExtID";                                // (int FIX.5.0)
dictionary[tag::DefaultCstmApplVerID]              = "DefaultCstmApplVerID";                            // (String FIX.5.0)
dictionary[tag::SessionStatus]                     = "SessionStatus";                                   // (int FIX.5.0)
dictionary[tag::Nested4PartySubIDType]             = "Nested4PartySubIDType";                           // (int FIX.5.0)
dictionary[tag::Nested4PartySubID]                 = "Nested4PartySubID";                               // (String FIX.5.0)
dictionary[tag::NoNested4PartySubIDs]              = "NoNested4PartySubIDs";                            // (NumInGroup FIX.5.0)
dictionary[tag::NoNested4PartyIDs]                 = "NoNested4PartyIDs";                               // (NumInGroup FIX.5.0)
dictionary[tag::Nested4PartyID]                    = "Nested4PartyID";                                  // (String FIX.5.0)
dictionary[tag::Nested4PartyIDSource]              = "Nested4PartyIDSource";                            // (char FIX.5.0)
dictionary[tag::Nested4PartyRole]                  = "Nested4PartyRole";                                // (int FIX.5.0)
dictionary[tag::LegLastQty]                        = "LegLastQty";                                      // (Qty FIX.5.0)
dictionary[tag::UnderlyingExerciseStyle]           = "UnderlyingExerciseStyle";                         // (int FIX.5.0)
dictionary[tag::LegExerciseStyle]                  = "LegExerciseStyle";                                // (int FIX.5.0)
dictionary[tag::LegPriceUnitOfMeasure]             = "LegPriceUnitOfMeasure";                           // (String FIX.5.0)
dictionary[tag::LegPriceUnitOfMeasureQty]          = "LegPriceUnitOfMeasureQty";                        // (Qty FIX.5.0)
dictionary[tag::UnderlyingUnitOfMeasureQty]        = "UnderlyingUnitOfMeasureQty";                      // (Qty FIX.5.0)
dictionary[tag::UnderlyingPriceUnitOfMeasure]      = "UnderlyingPriceUnitOfMeasure";                    // (String FIX.5.0)
dictionary[tag::UnderlyingPriceUnitOfMeasureQty]   = "UnderlyingPriceUnitOfMeasureQty";                 // (Qty FIX.5.0)
dictionary[tag::ApplReportType]                    = "ApplReportType";                                  // (int FIX.5.0SP2)
dictionary[tag::SideExecID]                        = "SideExecID";                                      // (String FIX.5.0SP1)
dictionary[tag::OrderDelay]                        = "OrderDelay";                                      // (int FIX.5.0SP1)
dictionary[tag::OrderDelayUnit]                    = "OrderDelayUnit";                                  // (int FIX.5.0SP1)
dictionary[tag::VenueType]                         = "VenueType";                                       // (char FIX.5.0SP1)
dictionary[tag::RefOrdIDReason]                    = "RefOrdIDReason";                                  // (int FIX.5.0SP1)
dictionary[tag::OrigCustOrderCapacity]             = "OrigCustOrderCapacity";                           // (int FIX.5.0SP1)
dictionary[tag::RefApplReqID]                      = "RefApplReqID";                                    // (String FIX.5.0SP1)
dictionary[tag::ModelType]                         = "ModelType";                                       // (int FIX.5.0SP1)
dictionary[tag::ContractMultiplierUnit]            = "ContractMultiplierUnit";                          // (int FIX.5.0SP1)
dictionary[tag::LegContractMultiplierUnit]         = "LegContractMultiplierUnit";                       // (int FIX.5.0SP1)
dictionary[tag::UnderlyingContractMultiplierUnit]  = "UnderlyingContractMultiplierUnit";                // (int FIX.5.0SP1)
dictionary[tag::DerivativeContractMultiplierUnit]  = "DerivativeContractMultiplierUnit";                // (int FIX.5.0SP1)
dictionary[tag::FlowScheduleType]                  = "FlowScheduleType";                                // (int FIX.5.0SP1)
dictionary[tag::LegFlowScheduleType]               = "LegFlowScheduleType";                             // (int FIX.5.0SP1)
dictionary[tag::UnderlyingFlowScheduleType]        = "UnderlyingFlowScheduleType";                      // (int FIX.5.0SP1)
dictionary[tag::DerivativeFlowScheduleType]        = "DerivativeFlowScheduleType";                      // (int FIX.5.0SP1)
dictionary[tag::FillLiquidityInd]                  = "FillLiquidityInd";                                // (int FIX.5.0SP1)
dictionary[tag::SideLiquidityInd]                  = "SideLiquidityInd";                                // (int FIX.5.0SP1)
dictionary[tag::NoRateSources]                     = "NoRateSources";                                   // (NumInGroup FIX.5.0SP1)
dictionary[tag::RateSource]                        = "RateSource";                                      // (int FIX.5.0SP1)
dictionary[tag::RateSourceType]                    = "RateSourceType";                                  // (int FIX.5.0SP1)
dictionary[tag::ReferencePage]                     = "ReferencePage";                                   // (String FIX.5.0SP1)
dictionary[tag::RestructuringType]                 = "RestructuringType";                               // (String FIX.5.0SP1)
dictionary[tag::Seniority]                         = "Seniority";                                       // (String FIX.5.0SP1)
dictionary[tag::NotionalPercentageOutstanding]     = "NotionalPercentageOutstanding";                   // (Percentage FIX.5.0SP1)
dictionary[tag::OriginalNotionalPercentageOutstanding] = "OriginalNotionalPercentageOutstanding";           // (Percentage FIX.5.0SP1)
dictionary[tag::UnderlyingRestructuringType]       = "UnderlyingRestructuringType";                     // (String FIX.5.0SP1)
dictionary[tag::UnderlyingSeniority]               = "UnderlyingSeniority";                             // (String FIX.5.0SP1)
dictionary[tag::UnderlyingNotionalPercentageOutstanding] = "UnderlyingNotionalPercentageOutstanding";         // (Percentage FIX.5.0SP1)
dictionary[tag::UnderlyingOriginalNotionalPercentageOutstanding] = "UnderlyingOriginalNotionalPercentageOutstanding"; // (Percentage FIX.5.0SP1)
dictionary[tag::AttachmentPoint]                   = "AttachmentPoint";                                 // (Percentage FIX.5.0SP1)
dictionary[tag::DetachmentPoint]                   = "DetachmentPoint";                                 // (Percentage FIX.5.0SP1)
dictionary[tag::UnderlyingAttachmentPoint]         = "UnderlyingAttachmentPoint";                       // (Percentage FIX.5.0SP1)
dictionary[tag::UnderlyingDetachmentPoint]         = "UnderlyingDetachmentPoint";                       // (Percentage FIX.5.0SP1)
dictionary[tag::NoTargetPartyIDs]                  = "NoTargetPartyIDs";                                // (NumInGroup FIX.5.0SP1)
dictionary[tag::TargetPartyID]                     = "TargetPartyID";                                   // (String FIX.5.0SP1)
dictionary[tag::TargetPartyIDSource]               = "TargetPartyIDSource";                             // (char FIX.5.0SP1)
dictionary[tag::TargetPartyRole]                   = "TargetPartyRole";                                 // (int FIX.5.0SP1)
dictionary[tag::SecurityListID]                    = "SecurityListID";                                  // (String FIX.5.0SP1)
dictionary[tag::SecurityListRefID]                 = "SecurityListRefID";                               // (String FIX.5.0SP1)
dictionary[tag::SecurityListDesc]                  = "SecurityListDesc";                                // (String FIX.5.0SP1)
dictionary[tag::EncodedSecurityListDescLen]        = "EncodedSecurityListDescLen";                      // (Length FIX.5.0SP1)
dictionary[tag::EncodedSecurityListDesc]           = "EncodedSecurityListDesc";                         // (data FIX.5.0SP1)
dictionary[tag::SecurityListType]                  = "SecurityListType";                                // (int FIX.5.0SP1)
dictionary[tag::SecurityListTypeSource]            = "SecurityListTypeSource";                          // (int FIX.5.0SP1)
dictionary[tag::NewsID]                            = "NewsID";                                          // (String FIX.5.0SP1)
dictionary[tag::NewsCategory]                      = "NewsCategory";                                    // (int FIX.5.0SP1)
dictionary[tag::LanguageCode]                      = "LanguageCode";                                    // (Language FIX.5.0SP1)
dictionary[tag::NoNewsRefIDs]                      = "NoNewsRefIDs";                                    // (NumInGroup FIX.5.0SP1)
dictionary[tag::NewsRefID]                         = "NewsRefID";                                       // (String FIX.5.0SP1)
dictionary[tag::NewsRefType]                       = "NewsRefType";                                     // (int FIX.5.0SP1)
dictionary[tag::StrikePriceDeterminationMethod]    = "StrikePriceDeterminationMethod";                  // (int FIX.5.0SP1)
dictionary[tag::StrikePriceBoundaryMethod]         = "StrikePriceBoundaryMethod";                       // (int FIX.5.0SP1)
dictionary[tag::StrikePriceBoundaryPrecision]      = "StrikePriceBoundaryPrecision";                    // (Percentage FIX.5.0SP1)
dictionary[tag::UnderlyingPriceDeterminationMethod] = "UnderlyingPriceDeterminationMethod";              // (int FIX.5.0SP1)
dictionary[tag::OptPayoutType]                     = "OptPayoutType";                                   // (int FIX.5.0SP1)
dictionary[tag::NoComplexEvents]                   = "NoComplexEvents";                                 // (NumInGroup FIX.5.0SP1)
dictionary[tag::ComplexEventType]                  = "ComplexEventType";                                // (int FIX.5.0SP1)
dictionary[tag::ComplexOptPayoutAmount]            = "ComplexOptPayoutAmount";                          // (Amt FIX.5.0SP1)
dictionary[tag::ComplexEventPrice]                 = "ComplexEventPrice";                               // (Price FIX.5.0SP1)
dictionary[tag::ComplexEventPriceBoundaryMethod]   = "ComplexEventPriceBoundaryMethod";                 // (int FIX.5.0SP1)
dictionary[tag::ComplexEventPriceBoundaryPrecision] = "ComplexEventPriceBoundaryPrecision";              // (Percentage FIX.5.0SP1)
dictionary[tag::ComplexEventPriceTimeType]         = "ComplexEventPriceTimeType";                       // (int FIX.5.0SP1)
dictionary[tag::ComplexEventCondition]             = "ComplexEventCondition";                           // (int FIX.5.0SP1)
dictionary[tag::NoComplexEventDates]               = "NoComplexEventDates";                             // (NumInGroup FIX.5.0SP1)
dictionary[tag::ComplexEventStartDate]             = "ComplexEventStartDate";                           // (UTCDateOnly FIX.5.0SP1)
dictionary[tag::ComplexEventEndDate]               = "ComplexEventEndDate";                             // (UTCDateOnly FIX.5.0SP1)
dictionary[tag::NoComplexEventTimes]               = "NoComplexEventTimes";                             // (NumInGroup FIX.5.0SP1)
dictionary[tag::ComplexEventStartTime]             = "ComplexEventStartTime";                           // (UTCTimeOnly FIX.5.0SP1)
dictionary[tag::ComplexEventEndTime]               = "ComplexEventEndTime";                             // (UTCTimeOnly FIX.5.0SP1)
dictionary[tag::StreamAsgnReqID]                   = "StreamAsgnReqID";                                 // (String FIX.5.0SP1)
dictionary[tag::StreamAsgnReqType]                 = "StreamAsgnReqType";                               // (int FIX.5.0SP1)
dictionary[tag::NoAsgnReqs]                        = "NoAsgnReqs";                                      // (NumInGroup FIX.5.0SP1)
dictionary[tag::MDStreamID]                        = "MDStreamID";                                      // (String FIX.5.0SP1)
dictionary[tag::StreamAsgnRptID]                   = "StreamAsgnRptID";                                 // (String FIX.5.0SP1)
dictionary[tag::StreamAsgnRejReason]               = "StreamAsgnRejReason";                             // (int FIX.5.0SP1)
dictionary[tag::StreamAsgnAckType]                 = "StreamAsgnAckType";                               // (int FIX.5.0SP1)
dictionary[tag::RelSymTransactTime]                = "RelSymTransactTime";                              // (UTCTimestamp FIX.5.0SP1)
dictionary[tag::PartyDetailsListRequestID]         = "PartyDetailsListRequestID";                       // (String FIX.5.0SP2)
dictionary[tag::SideTradeID]                       = "SideTradeID";                                     // (String FIX.5.0SP2)
dictionary[tag::SideOrigTradeID]                   = "SideOrigTradeID";                                 // (String FIX.5.0SP2)
dictionary[tag::NoRequestedPartyRoles]             = "NoRequestedPartyRoles";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::RequestedPartyRole]                = "RequestedPartyRole";                              // (int FIX.5.0SP2)
dictionary[tag::PartyDetailsListReportID]          = "PartyDetailsListReportID";                        // (String FIX.5.0SP2)
dictionary[tag::RequestResult]                     = "RequestResult";                                   // (int FIX.5.0SP2)
dictionary[tag::TotNoParties]                      = "TotNoParties";                                    // (int FIX.5.0SP2)
dictionary[tag::DocumentationText]                 = "DocumentationText";                               // (String FIX.5.0SP2)
dictionary[tag::NoPartyRelationships]              = "NoPartyRelationships";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::PartyRelationship]                 = "PartyRelationship";                               // (int FIX.5.0SP2)
dictionary[tag::NoPartyDetailAltID]                = "NoPartyDetailAltID";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::PartyDetailAltID]                  = "PartyDetailAltID";                                // (String FIX.5.0SP2)
dictionary[tag::PartyDetailAltIDSource]            = "PartyDetailAltIDSource";                          // (char FIX.5.0SP2)
dictionary[tag::NoPartyDetailAltSubIDs]            = "NoPartyDetailAltSubIDs";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::PartyDetailAltSubID]               = "PartyDetailAltSubID";                             // (String FIX.5.0SP2)
dictionary[tag::PartyDetailAltSubIDType]           = "PartyDetailAltSubIDType";                         // (int FIX.5.0SP2)
dictionary[tag::DifferentialPrice]                 = "DifferentialPrice";                               // (PriceOffset FIX.5.0SP2)
dictionary[tag::TrdAckStatus]                      = "TrdAckStatus";                                    // (int FIX.5.0SP2)
dictionary[tag::PriceQuoteCurrency]                = "PriceQuoteCurrency";                              // (Currency FIX.5.0SP2)
dictionary[tag::EncodedDocumentationTextLen]       = "EncodedDocumentationTextLen";                     // (Length FIX.5.0SP2)
dictionary[tag::UnderlyingPriceQuoteCurrency]      = "UnderlyingPriceQuoteCurrency";                    // (Currency FIX.5.0SP2)
dictionary[tag::EncodedDocumentationText]          = "EncodedDocumentationText";                        // (data FIX.5.0SP2)
dictionary[tag::LegPriceQuoteCurrency]             = "LegPriceQuoteCurrency";                           // (Currency FIX.5.0SP2)
dictionary[tag::NoRiskLimitTypes]                  = "NoRiskLimitTypes";                                // (NumInGroup FIX.5.0SP2)
dictionary[tag::RiskLimitType]                     = "RiskLimitType";                                   // (int FIX.5.0SP2)
dictionary[tag::RiskLimitAmount]                   = "RiskLimitAmount";                                 // (Amt FIX.5.0SP2)
dictionary[tag::RiskLimitCurrency]                 = "RiskLimitCurrency";                               // (Currency FIX.5.0SP2)
dictionary[tag::RiskLimitPlatform]                 = "RiskLimitPlatform";                               // (String FIX.5.0SP2)
dictionary[tag::NoRiskInstrumentScopes]            = "NoRiskInstrumentScopes";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::InstrumentScopeOperator]           = "InstrumentScopeOperator";                         // (int FIX.5.0SP2)
dictionary[tag::InstrumentScopeSymbol]             = "InstrumentScopeSymbol";                           // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSymbolSfx]          = "InstrumentScopeSymbolSfx";                        // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecurityID]         = "InstrumentScopeSecurityID";                       // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecurityIDSource]   = "InstrumentScopeSecurityIDSource";                 // (String FIX.5.0SP2)
dictionary[tag::NoInstrumentScopeSecurityAltID]    = "NoInstrumentScopeSecurityAltID";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecurityAltID]      = "InstrumentScopeSecurityAltID";                    // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecurityAltIDSource] = "InstrumentScopeSecurityAltIDSource";              // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeProduct]            = "InstrumentScopeProduct";                          // (int FIX.5.0SP2)
dictionary[tag::InstrumentScopeProductComplex]     = "InstrumentScopeProductComplex";                   // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecurityGroup]      = "InstrumentScopeSecurityGroup";                    // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeCFICode]            = "InstrumentScopeCFICode";                          // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecurityType]       = "InstrumentScopeSecurityType";                     // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecuritySubType]    = "InstrumentScopeSecuritySubType";                  // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeMaturityMonthYear]  = "InstrumentScopeMaturityMonthYear";                // (MonthYear FIX.5.0SP2)
dictionary[tag::InstrumentScopeMaturityTime]       = "InstrumentScopeMaturityTime";                     // (TZTimeOnly FIX.5.0SP2)
dictionary[tag::InstrumentScopeRestructuringType]  = "InstrumentScopeRestructuringType";                // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSeniority]          = "InstrumentScopeSeniority";                        // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopePutOrCall]          = "InstrumentScopePutOrCall";                        // (int FIX.5.0SP2)
dictionary[tag::InstrumentScopeFlexibleIndicator]  = "InstrumentScopeFlexibleIndicator";                // (Boolean FIX.5.0SP2)
dictionary[tag::InstrumentScopeCouponRate]         = "InstrumentScopeCouponRate";                       // (Percentage FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecurityDesc]       = "InstrumentScopeSecurityDesc";                     // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeSettlType]          = "InstrumentScopeSettlType";                        // (String FIX.5.0SP2)
dictionary[tag::RiskInstrumentMultiplier]          = "RiskInstrumentMultiplier";                        // (float FIX.5.0SP2)
dictionary[tag::NoRiskWarningLevels]               = "NoRiskWarningLevels";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::RiskWarningLevelPercent]           = "RiskWarningLevelPercent";                         // (Percentage FIX.5.0SP2)
dictionary[tag::RiskWarningLevelName]              = "RiskWarningLevelName";                            // (String FIX.5.0SP2)
dictionary[tag::NoRelatedPartyDetailID]            = "NoRelatedPartyDetailID";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailID]              = "RelatedPartyDetailID";                            // (String FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailIDSource]        = "RelatedPartyDetailIDSource";                      // (char FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailRole]            = "RelatedPartyDetailRole";                          // (int FIX.5.0SP2)
dictionary[tag::NoRelatedPartyDetailSubIDs]        = "NoRelatedPartyDetailSubIDs";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailSubID]           = "RelatedPartyDetailSubID";                         // (String FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailSubIDType]       = "RelatedPartyDetailSubIDType";                     // (int FIX.5.0SP2)
dictionary[tag::NoRelatedPartyDetailAltID]         = "NoRelatedPartyDetailAltID";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailAltID]           = "RelatedPartyDetailAltID";                         // (String FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailAltIDSource]     = "RelatedPartyDetailAltIDSource";                   // (char FIX.5.0SP2)
dictionary[tag::NoRelatedPartyDetailAltSubIDs]     = "NoRelatedPartyDetailAltSubIDs";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailAltSubID]        = "RelatedPartyDetailAltSubID";                      // (String FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailAltSubIDType]    = "RelatedPartyDetailAltSubIDType";                  // (int FIX.5.0SP2)
dictionary[tag::SwapSubClass]                      = "SwapSubClass";                                    // (String FIX.5.0SP2)
dictionary[tag::DerivativePriceQuoteCurrency]      = "DerivativePriceQuoteCurrency";                    // (Currency FIX.5.0SP2)
dictionary[tag::SettlRateIndex]                    = "SettlRateIndex";                                  // (String FIX.5.0SP2)
dictionary[tag::EncodedEventTextLen]               = "EncodedEventTextLen";                             // (Length FIX.5.0SP2)
dictionary[tag::EncodedEventText]                  = "EncodedEventText";                                // (data FIX.5.0SP2)
dictionary[tag::SettlRateIndexLocation]            = "SettlRateIndexLocation";                          // (String FIX.5.0SP2)
dictionary[tag::OptionExpirationDesc]              = "OptionExpirationDesc";                            // (String FIX.5.0SP2)
dictionary[tag::NoSecurityClassifications]         = "NoSecurityClassifications";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::SecurityClassificationReason]      = "SecurityClassificationReason";                    // (int FIX.5.0SP2)
dictionary[tag::SecurityClassificationValue]       = "SecurityClassificationValue";                     // (String FIX.5.0SP2)
dictionary[tag::PosAmtReason]                      = "PosAmtReason";                                    // (int FIX.5.0SP2)
dictionary[tag::NoLegPosAmt]                       = "NoLegPosAmt";                                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPosAmt]                         = "LegPosAmt";                                       // (Amt FIX.5.0SP2)
dictionary[tag::LegPosAmtType]                     = "LegPosAmtType";                                   // (String FIX.5.0SP2)
dictionary[tag::LegPosCurrency]                    = "LegPosCurrency";                                  // (Currency FIX.5.0SP2)
dictionary[tag::LegPosAmtReason]                   = "LegPosAmtReason";                                 // (int FIX.5.0SP2)
dictionary[tag::LegQtyType]                        = "LegQtyType";                                      // (int FIX.5.0SP2)
dictionary[tag::DiscountFactor]                    = "DiscountFactor";                                  // (float FIX.5.0SP2)
dictionary[tag::ParentAllocID]                     = "ParentAllocID";                                   // (String FIX.5.0SP2)
dictionary[tag::LegSecurityGroup]                  = "LegSecurityGroup";                                // (String FIX.5.0SP2)
dictionary[tag::PositionContingentPrice]           = "PositionContingentPrice";                         // (Price FIX.5.0SP2)
dictionary[tag::ClearingTradePrice]                = "ClearingTradePrice";                              // (Price FIX.5.0SP2)
dictionary[tag::SideClearingTradePrice]            = "SideClearingTradePrice";                          // (Price FIX.5.0SP2)
dictionary[tag::SideClearingTradePriceType]        = "SideClearingTradePriceType";                      // (int FIX.5.0SP2)
dictionary[tag::SidePriceDifferential]             = "SidePriceDifferential";                           // (Price FIX.5.0SP2)
dictionary[tag::FIXEngineName]                     = "FIXEngineName";                                   // (String FIX.5.0SP2)
dictionary[tag::FIXEngineVersion]                  = "FIXEngineVersion";                                // (String FIX.5.0SP2)
dictionary[tag::FIXEngineVendor]                   = "FIXEngineVendor";                                 // (String FIX.5.0SP2)
dictionary[tag::ApplicationSystemName]             = "ApplicationSystemName";                           // (String FIX.5.0SP2)
dictionary[tag::ApplicationSystemVersion]          = "ApplicationSystemVersion";                        // (String FIX.5.0SP2)
dictionary[tag::ApplicationSystemVendor]           = "ApplicationSystemVendor";                         // (String FIX.5.0SP2)
dictionary[tag::NumOfSimpleInstruments]            = "NumOfSimpleInstruments";                          // (int FIX.5.0SP2)
dictionary[tag::SecurityRejectReason]              = "SecurityRejectReason";                            // (int FIX.5.0SP2)
dictionary[tag::InitialDisplayQty]                 = "InitialDisplayQty";                               // (Qty FIX.5.0SP2)
dictionary[tag::ThrottleStatus]                    = "ThrottleStatus";                                  // (int FIX.5.0SP2)
dictionary[tag::NoThrottles]                       = "NoThrottles";                                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::ThrottleAction]                    = "ThrottleAction";                                  // (int FIX.5.0SP2)
dictionary[tag::ThrottleType]                      = "ThrottleType";                                    // (int FIX.5.0SP2)
dictionary[tag::ThrottleNoMsgs]                    = "ThrottleNoMsgs";                                  // (int FIX.5.0SP2)
dictionary[tag::ThrottleTimeInterval]              = "ThrottleTimeInterval";                            // (int FIX.5.0SP2)
dictionary[tag::ThrottleTimeUnit]                  = "ThrottleTimeUnit";                                // (int FIX.5.0SP2)
dictionary[tag::InstrumentScopeSecurityExchange]   = "InstrumentScopeSecurityExchange";                 // (Exchange FIX.5.0SP2)
dictionary[tag::StreamAsgnType]                    = "StreamAsgnType";                                  // (int FIX.5.0SP1)
dictionary[tag::NoThrottleMsgType]                 = "NoThrottleMsgType";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::ThrottleMsgType]                   = "ThrottleMsgType";                                 // (String FIX.5.0SP2)
dictionary[tag::InstrumentScopeEncodedSecurityDescLen] = "InstrumentScopeEncodedSecurityDescLen";           // (Length FIX.5.0SP2)
dictionary[tag::InstrumentScopeEncodedSecurityDesc] = "InstrumentScopeEncodedSecurityDesc";              // (data FIX.5.0SP2)
dictionary[tag::FillYieldType]                     = "FillYieldType";                                   // (String FIX.5.0SP2)
dictionary[tag::FillYield]                         = "FillYield";                                       // (Percentage FIX.5.0SP2)
dictionary[tag::NoMatchInst]                       = "NoMatchInst";                                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::MatchInst]                         = "MatchInst";                                       // (int FIX.5.0SP2)
dictionary[tag::MatchAttribTagID]                  = "MatchAttribTagID";                                // (TagNum FIX.5.0SP2)
dictionary[tag::MatchAttribValue]                  = "MatchAttribValue";                                // (String FIX.5.0SP2)
dictionary[tag::TriggerScope]                      = "TriggerScope";                                    // (int FIX.5.0SP2)
dictionary[tag::ExposureDuration]                  = "ExposureDuration";                                // (int FIX.5.0SP2)
dictionary[tag::NoLimitAmts]                       = "NoLimitAmts";                                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::LimitAmtType]                      = "LimitAmtType";                                    // (int FIX.5.0SP2)
dictionary[tag::LastLimitAmt]                      = "LastLimitAmt";                                    // (Amt FIX.5.0SP2)
dictionary[tag::LimitAmtRemaining]                 = "LimitAmtRemaining";                               // (Amt FIX.5.0SP2)
dictionary[tag::LimitAmtCurrency]                  = "LimitAmtCurrency";                                // (Currency FIX.5.0SP2)
dictionary[tag::MarginReqmtInqID]                  = "MarginReqmtInqID";                                // (String FIX.5.0SP2)
dictionary[tag::NoMarginReqmtInqQualifier]         = "NoMarginReqmtInqQualifier";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::MarginReqmtInqQualifier]           = "MarginReqmtInqQualifier";                         // (int FIX.5.0SP2)
dictionary[tag::MarginReqmtRptType]                = "MarginReqmtRptType";                              // (int FIX.5.0SP2)
dictionary[tag::MarginClass]                       = "MarginClass";                                     // (String FIX.5.0SP2)
dictionary[tag::MarginReqmtInqStatus]              = "MarginReqmtInqStatus";                            // (int FIX.5.0SP2)
dictionary[tag::MarginReqmtInqResult]              = "MarginReqmtInqResult";                            // (int FIX.5.0SP2)
dictionary[tag::MarginReqmtRptID]                  = "MarginReqmtRptID";                                // (String FIX.5.0SP2)
dictionary[tag::NoMarginAmt]                       = "NoMarginAmt";                                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::MarginAmtType]                     = "MarginAmtType";                                   // (int FIX.5.0SP2)
dictionary[tag::MarginAmt]                         = "MarginAmt";                                       // (Amt FIX.5.0SP2)
dictionary[tag::MarginAmtCcy]                      = "MarginAmtCcy";                                    // (Currency FIX.5.0SP2)
dictionary[tag::NoRelatedInstruments]              = "NoRelatedInstruments";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelatedInstrumentType]             = "RelatedInstrumentType";                           // (int FIX.5.0SP2)
dictionary[tag::RelatedSymbol]                     = "RelatedSymbol";                                   // (String FIX.5.0SP2)
dictionary[tag::RelatedSecurityID]                 = "RelatedSecurityID";                               // (String FIX.5.0SP2)
dictionary[tag::RelatedSecurityIDSource]           = "RelatedSecurityIDSource";                         // (String FIX.5.0SP2)
dictionary[tag::RelatedSecurityType]               = "RelatedSecurityType";                             // (String FIX.5.0SP2)
dictionary[tag::RelatedMaturityMonthYear]          = "RelatedMaturityMonthYear";                        // (MonthYear FIX.5.0SP2)
dictionary[tag::CoveredQty]                        = "CoveredQty";                                      // (Qty FIX.5.0SP2)
dictionary[tag::MarketMakerActivity]               = "MarketMakerActivity";                             // (int FIX.5.0SP2)
dictionary[tag::NoInstrumentScopes]                = "NoInstrumentScopes";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoRequestingPartyIDs]              = "NoRequestingPartyIDs";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::RequestingPartyID]                 = "RequestingPartyID";                               // (String FIX.5.0SP2)
dictionary[tag::RequestingPartyIDSource]           = "RequestingPartyIDSource";                         // (char FIX.5.0SP2)
dictionary[tag::RequestingPartyRole]               = "RequestingPartyRole";                             // (int FIX.5.0SP2)
dictionary[tag::NoRequestingPartySubIDs]           = "NoRequestingPartySubIDs";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::RequestingPartySubID]              = "RequestingPartySubID";                            // (String FIX.5.0SP2)
dictionary[tag::RequestingPartySubIDType]          = "RequestingPartySubIDType";                        // (int FIX.5.0SP2)
dictionary[tag::EncodedRejectTextLen]              = "EncodedRejectTextLen";                            // (Length FIX.5.0SP2)
dictionary[tag::EncodedRejectText]                 = "EncodedRejectText";                               // (data FIX.5.0SP2)
dictionary[tag::RiskLimitRequestID]                = "RiskLimitRequestID";                              // (String FIX.5.0SP2)
dictionary[tag::RiskLimitReportID]                 = "RiskLimitReportID";                               // (String FIX.5.0SP2)
dictionary[tag::NoRequestedRiskLimitType]          = "NoRequestedRiskLimitType";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoRiskLimits]                      = "NoRiskLimits";                                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::RiskLimitID]                       = "RiskLimitID";                                     // (String FIX.5.0SP2)
dictionary[tag::NoPartyDetails]                    = "NoPartyDetails";                                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::PartyDetailStatus]                 = "PartyDetailStatus";                               // (int FIX.5.0SP2)
dictionary[tag::MatchInstMarketID]                 = "MatchInstMarketID";                               // (Exchange FIX.5.0SP2)
dictionary[tag::PartyDetailRoleQualifier]          = "PartyDetailRoleQualifier";                        // (int FIX.5.0SP2)
dictionary[tag::RelatedPartyDetailRoleQualifier]   = "RelatedPartyDetailRoleQualifier";                 // (int FIX.5.0SP2)
dictionary[tag::NoPartyUpdates]                    = "NoPartyUpdates";                                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPartyRiskLimits]                 = "NoPartyRiskLimits";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::EncodedOptionExpirationDescLen]    = "EncodedOptionExpirationDescLen";                  // (Length FIX.5.0SP2)
dictionary[tag::SecurityMassTradingStatus]         = "SecurityMassTradingStatus";                       // (int FIX.5.0SP2)
dictionary[tag::SecurityMassTradingEvent]          = "SecurityMassTradingEvent";                        // (int FIX.5.0SP2)
dictionary[tag::MassHaltReason]                    = "MassHaltReason";                                  // (int FIX.5.0SP2)
dictionary[tag::MDSecurityTradingStatus]           = "MDSecurityTradingStatus";                         // (int FIX.5.0SP2)
dictionary[tag::MDSubFeedType]                     = "MDSubFeedType";                                   // (String FIX.5.0SP2)
dictionary[tag::MDHaltReason]                      = "MDHaltReason";                                    // (int FIX.5.0SP2)
dictionary[tag::ThrottleInst]                      = "ThrottleInst";                                    // (int FIX.5.0SP2)
dictionary[tag::ThrottleCountIndicator]            = "ThrottleCountIndicator";                          // (int FIX.5.0SP2)
dictionary[tag::ShortSaleRestriction]              = "ShortSaleRestriction";                            // (int FIX.5.0SP2)
dictionary[tag::ShortSaleExemptionReason]          = "ShortSaleExemptionReason";                        // (int FIX.5.0SP2)
dictionary[tag::LegShortSaleExemptionReason]       = "LegShortSaleExemptionReason";                     // (int FIX.5.0SP2)
dictionary[tag::SideShortSaleExemptionReason]      = "SideShortSaleExemptionReason";                    // (int FIX.5.0SP2)
dictionary[tag::PartyDetailID]                     = "PartyDetailID";                                   // (String FIX.5.0SP2)
dictionary[tag::PartyDetailIDSource]               = "PartyDetailIDSource";                             // (char FIX.5.0SP2)
dictionary[tag::PartyDetailRole]                   = "PartyDetailRole";                                 // (int FIX.5.0SP2)
dictionary[tag::NoPartyDetailSubIDs]               = "NoPartyDetailSubIDs";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::PartyDetailSubID]                  = "PartyDetailSubID";                                // (String FIX.5.0SP2)
dictionary[tag::PartyDetailSubIDType]              = "PartyDetailSubIDType";                            // (int FIX.5.0SP2)
dictionary[tag::EncodedOptionExpirationDesc]       = "EncodedOptionExpirationDesc";                     // (data FIX.5.0SP2)
dictionary[tag::StrikeUnitOfMeasure]               = "StrikeUnitOfMeasure";                             // (String FIX.5.0SP2)
dictionary[tag::AccountSummaryReportID]            = "AccountSummaryReportID";                          // (String FIX.5.0SP2)
dictionary[tag::NoSettlementAmounts]               = "NoSettlementAmounts";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::SettlementAmount]                  = "SettlementAmount";                                // (Amt FIX.5.0SP2)
dictionary[tag::SettlementAmountCurrency]          = "SettlementAmountCurrency";                        // (Currency FIX.5.0SP2)
dictionary[tag::NoCollateralAmounts]               = "NoCollateralAmounts";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::CurrentCollateralAmount]           = "CurrentCollateralAmount";                         // (Amt FIX.5.0SP2)
dictionary[tag::CollateralCurrency]                = "CollateralCurrency";                              // (Currency FIX.5.0SP2)
dictionary[tag::CollateralType]                    = "CollateralType";                                  // (String FIX.5.0SP2)
dictionary[tag::NoPayCollects]                     = "NoPayCollects";                                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::PayCollectType]                    = "PayCollectType";                                  // (String FIX.5.0SP2)
dictionary[tag::PayCollectCurrency]                = "PayCollectCurrency";                              // (Currency FIX.5.0SP2)
dictionary[tag::PayAmount]                         = "PayAmount";                                       // (Amt FIX.5.0SP2)
dictionary[tag::CollectAmount]                     = "CollectAmount";                                   // (Amt FIX.5.0SP2)
dictionary[tag::PayCollectMarketSegmentID]         = "PayCollectMarketSegmentID";                       // (String FIX.5.0SP2)
dictionary[tag::PayCollectMarketID]                = "PayCollectMarketID";                              // (String FIX.5.0SP2)
dictionary[tag::MarginAmountMarketSegmentID]       = "MarginAmountMarketSegmentID";                     // (String FIX.5.0SP2)
dictionary[tag::MarginAmountMarketID]              = "MarginAmountMarketID";                            // (String FIX.5.0SP2)
dictionary[tag::UnitOfMeasureCurrency]             = "UnitOfMeasureCurrency";                           // (Currency FIX.5.0SP2)
dictionary[tag::PriceUnitOfMeasureCurrency]        = "PriceUnitOfMeasureCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingUnitOfMeasureCurrency]   = "UnderlyingUnitOfMeasureCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPriceUnitOfMeasureCurrency] = "UnderlyingPriceUnitOfMeasureCurrency";            // (Currency FIX.5.0SP2)
dictionary[tag::LegUnitOfMeasureCurrency]          = "LegUnitOfMeasureCurrency";                        // (Currency FIX.5.0SP2)
dictionary[tag::LegPriceUnitOfMeasureCurrency]     = "LegPriceUnitOfMeasureCurrency";                   // (Currency FIX.5.0SP2)
dictionary[tag::DerivativeUnitOfMeasureCurrency]   = "DerivativeUnitOfMeasureCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::DerivativePriceUnitOfMeasureCurrency] = "DerivativePriceUnitOfMeasureCurrency";            // (Currency FIX.5.0SP2)
dictionary[tag::OrderOrigination]                  = "OrderOrigination";                                // (int FIX.5.0SP2)
dictionary[tag::OriginatingDeptID]                 = "OriginatingDeptID";                               // (String FIX.5.0SP2)
dictionary[tag::ReceivingDeptID]                   = "ReceivingDeptID";                                 // (String FIX.5.0SP2)
dictionary[tag::InformationBarrierID]              = "InformationBarrierID";                            // (String FIX.5.0SP2)
dictionary[tag::FirmGroupID]                       = "FirmGroupID";                                     // (String FIX.5.0SP2)
dictionary[tag::FirmMnemonic]                      = "FirmMnemonic";                                    // (String FIX.5.0SP2)
dictionary[tag::AllocGroupID]                      = "AllocGroupID";                                    // (String FIX.5.0SP2)
dictionary[tag::AvgPxGroupID]                      = "AvgPxGroupID";                                    // (String FIX.5.0SP2)
dictionary[tag::FirmAllocText]                     = "FirmAllocText";                                   // (String FIX.5.0SP2)
dictionary[tag::EncodedFirmAllocTextLen]           = "EncodedFirmAllocTextLen";                         // (Length FIX.5.0SP2)
dictionary[tag::EncodedFirmAllocText]              = "EncodedFirmAllocText";                            // (data FIX.5.0SP2)
dictionary[tag::AllocationRollupInstruction]       = "AllocationRollupInstruction";                     // (int FIX.5.0SP2)
dictionary[tag::AllocGroupQuantity]                = "AllocGroupQuantity";                              // (Qty FIX.5.0SP2)
dictionary[tag::AllocGroupRemainingQuantity]       = "AllocGroupRemainingQuantity";                     // (Qty FIX.5.0SP2)
dictionary[tag::AllocReversalStatus]               = "AllocReversalStatus";                             // (int FIX.5.0SP2)
dictionary[tag::ObligationType]                    = "ObligationType";                                  // (String FIX.5.0SP2)
dictionary[tag::TradePriceNegotiationMethod]       = "TradePriceNegotiationMethod";                     // (int FIX.5.0SP2)
dictionary[tag::UpfrontPriceType]                  = "UpfrontPriceType";                                // (int FIX.5.0SP2)
dictionary[tag::UpfrontPrice]                      = "UpfrontPrice";                                    // (Price FIX.5.0SP2)
dictionary[tag::LastUpfrontPrice]                  = "LastUpfrontPrice";                                // (Price FIX.5.0SP2)
dictionary[tag::ApplLevelRecoveryIndicator]        = "ApplLevelRecoveryIndicator";                      // (int FIX.5.0SP2)
dictionary[tag::BidMDEntryID]                      = "BidMDEntryID";                                    // (String FIX.5.0SP2)
dictionary[tag::OfferMDEntryID]                    = "OfferMDEntryID";                                  // (String FIX.5.0SP2)
dictionary[tag::BidQuoteID]                        = "BidQuoteID";                                      // (String FIX.5.0SP2)
dictionary[tag::OfferQuoteID]                      = "OfferQuoteID";                                    // (String FIX.5.0SP2)
dictionary[tag::TotalBidSize]                      = "TotalBidSize";                                    // (Qty FIX.5.0SP2)
dictionary[tag::TotalOfferSize]                    = "TotalOfferSize";                                  // (Qty FIX.5.0SP2)
dictionary[tag::SecondaryQuoteID]                  = "SecondaryQuoteID";                                // (String FIX.5.0SP2)
dictionary[tag::CustodialLotID]                    = "CustodialLotID";                                  // (String FIX.5.0SP2)
dictionary[tag::VersusPurchaseDate]                = "VersusPurchaseDate";                              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::VersusPurchasePrice]               = "VersusPurchasePrice";                             // (Price FIX.5.0SP2)
dictionary[tag::CurrentCostBasis]                  = "CurrentCostBasis";                                // (Amt FIX.5.0SP2)
dictionary[tag::LegCustodialLotID]                 = "LegCustodialLotID";                               // (String FIX.5.0SP2)
dictionary[tag::LegVersusPurchaseDate]             = "LegVersusPurchaseDate";                           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegVersusPurchasePrice]            = "LegVersusPurchasePrice";                          // (Price FIX.5.0SP2)
dictionary[tag::LegCurrentCostBasis]               = "LegCurrentCostBasis";                             // (Amt FIX.5.0SP2)
dictionary[tag::RiskLimitRequestType]              = "RiskLimitRequestType";                            // (int FIX.5.0SP2)
dictionary[tag::RiskLimitRequestResult]            = "RiskLimitRequestResult";                          // (int FIX.5.0SP2)
dictionary[tag::RiskLimitRequestStatus]            = "RiskLimitRequestStatus";                          // (int FIX.5.0SP2)
dictionary[tag::RiskLimitStatus]                   = "RiskLimitStatus";                                 // (int FIX.5.0SP2)
dictionary[tag::RiskLimitResult]                   = "RiskLimitResult";                                 // (int FIX.5.0SP2)
dictionary[tag::RiskLimitUtilizationPercent]       = "RiskLimitUtilizationPercent";                     // (Percentage FIX.5.0SP2)
dictionary[tag::RiskLimitUtilizationAmount]        = "RiskLimitUtilizationAmount";                      // (Amt FIX.5.0SP2)
dictionary[tag::RiskLimitAction]                   = "RiskLimitAction";                                 // (int FIX.5.0SP2)
dictionary[tag::RiskWarningLevelAmount]            = "RiskWarningLevelAmount";                          // (int FIX.5.0SP2)
dictionary[tag::RiskWarningLevelAction]            = "RiskWarningLevelAction";                          // (int FIX.5.0SP2)
dictionary[tag::EntitlementRequestID]              = "EntitlementRequestID";                            // (String FIX.5.0SP2)
dictionary[tag::EntitlementReportID]               = "EntitlementReportID";                             // (String FIX.5.0SP2)
dictionary[tag::NoPartyEntitlements]               = "NoPartyEntitlements";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoEntitlements]                    = "NoEntitlements";                                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::EntitlementIndicator]              = "EntitlementIndicator";                            // (Boolean FIX.5.0SP2)
dictionary[tag::EntitlementType]                   = "EntitlementType";                                 // (int FIX.5.0SP2)
dictionary[tag::EntitlementID]                     = "EntitlementID";                                   // (String FIX.5.0SP2)
dictionary[tag::NoEntitlementAttrib]               = "NoEntitlementAttrib";                             // (int FIX.5.0SP2)
dictionary[tag::EntitlementAttribType]             = "EntitlementAttribType";                           // (int FIX.5.0SP2)
dictionary[tag::EntitlementAttribDatatype]         = "EntitlementAttribDatatype";                       // (int FIX.5.0SP2)
dictionary[tag::EntitlementAttribValue]            = "EntitlementAttribValue";                          // (String FIX.5.0SP2)
dictionary[tag::EntitlementAttribCurrency]         = "EntitlementAttribCurrency";                       // (Currency FIX.5.0SP2)
dictionary[tag::EntitlementStartDate]              = "EntitlementStartDate";                            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::EntitlementEndDate]                = "EntitlementEndDate";                              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::EntitlementPlatform]               = "EntitlementPlatform";                             // (String FIX.5.0SP2)
dictionary[tag::TradSesControl]                    = "TradSesControl";                                  // (int FIX.5.0SP2)
dictionary[tag::TradeVolType]                      = "TradeVolType";                                    // (int FIX.5.0SP2)
dictionary[tag::RefTickTableID]                    = "RefTickTableID";                                  // (int FIX.5.0SP2)
dictionary[tag::LegID]                             = "LegID";                                           // (String FIX.5.0SP2)
dictionary[tag::NoTargetMarketSegments]            = "NoTargetMarketSegments";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::TargetMarketSegmentID]             = "TargetMarketSegmentID";                           // (String FIX.5.0SP2)
dictionary[tag::NoAffectedMarketSegments]          = "NoAffectedMarketSegments";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::AffectedMarketSegmentID]           = "AffectedMarketSegmentID";                         // (String FIX.5.0SP2)
dictionary[tag::NoNotAffectedMarketSegments]       = "NoNotAffectedMarketSegments";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::NotAffectedMarketSegmentID]        = "NotAffectedMarketSegmentID";                      // (String FIX.5.0SP2)
dictionary[tag::NoOrderEvents]                     = "NoOrderEvents";                                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::OrderEventType]                    = "OrderEventType";                                  // (int FIX.5.0SP2)
dictionary[tag::OrderEventExecID]                  = "OrderEventExecID";                                // (String FIX.5.0SP2)
dictionary[tag::OrderEventReason]                  = "OrderEventReason";                                // (int FIX.5.0SP2)
dictionary[tag::OrderEventPx]                      = "OrderEventPx";                                    // (Price FIX.5.0SP2)
dictionary[tag::OrderEventQty]                     = "OrderEventQty";                                   // (Qty FIX.5.0SP2)
dictionary[tag::OrderEventLiquidityIndicator]      = "OrderEventLiquidityIndicator";                    // (int FIX.5.0SP2)
dictionary[tag::OrderEventText]                    = "OrderEventText";                                  // (String FIX.5.0SP2)
dictionary[tag::AuctionType]                       = "AuctionType";                                     // (int FIX.5.0SP2)
dictionary[tag::AuctionAllocationPct]              = "AuctionAllocationPct";                            // (Percentage FIX.5.0SP2)
dictionary[tag::AuctionInstruction]                = "AuctionInstruction";                              // (int FIX.5.0SP2)
dictionary[tag::RefClOrdID]                        = "RefClOrdID";                                      // (String FIX.5.0SP2)
dictionary[tag::LockType]                          = "LockType";                                        // (int FIX.5.0SP2)
dictionary[tag::LockedQty]                         = "LockedQty";                                       // (Qty FIX.5.0SP2)
dictionary[tag::SecondaryLockedQty]                = "SecondaryLockedQty";                              // (Qty FIX.5.0SP2)
dictionary[tag::ReleaseInstruction]                = "ReleaseInstruction";                              // (int FIX.5.0SP2)
dictionary[tag::ReleaseQty]                        = "ReleaseQty";                                      // (Qty FIX.5.0SP2)
dictionary[tag::NoDisclosureInstructions]          = "NoDisclosureInstructions";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::DisclosureType]                    = "DisclosureType";                                  // (int FIX.5.0SP2)
dictionary[tag::DisclosureInstruction]             = "DisclosureInstruction";                           // (int FIX.5.0SP2)
dictionary[tag::TradingCapacity]                   = "TradingCapacity";                                 // (int FIX.5.0SP2)
dictionary[tag::ClearingAccountType]               = "ClearingAccountType";                             // (int FIX.5.0SP2)
dictionary[tag::LegClearingAccountType]            = "LegClearingAccountType";                          // (int FIX.5.0SP2)
dictionary[tag::TargetPartyRoleQualifier]          = "TargetPartyRoleQualifier";                        // (int FIX.5.0SP2)
dictionary[tag::RelatedHighPrice]                  = "RelatedHighPrice";                                // (Price FIX.5.0SP2)
dictionary[tag::RelatedLowPrice]                   = "RelatedLowPrice";                                 // (Price FIX.5.0SP2)
dictionary[tag::RelatedPriceSource]                = "RelatedPriceSource";                              // (int FIX.5.0SP2)
dictionary[tag::MinQtyMethod]                      = "MinQtyMethod";                                    // (int FIX.5.0SP2)
dictionary[tag::Triggered]                         = "Triggered";                                       // (int FIX.5.0SP2)
dictionary[tag::AffectedOrigClOrdID]               = "AffectedOrigClOrdID";                             // (String FIX.5.0SP2)
dictionary[tag::NotAffSecondaryOrderID]            = "NotAffSecondaryOrderID";                          // (String FIX.5.0SP2)
dictionary[tag::EventTimePeriod]                   = "EventTimePeriod";                                 // (int FIX.5.0SP2)
dictionary[tag::EventTimeUnit]                     = "EventTimeUnit";                                   // (String FIX.5.0SP2)
dictionary[tag::LastQtyVariance]                   = "LastQtyVariance";                                 // (Qty FIX.5.0SP2)
dictionary[tag::NoCrossLegs]                       = "NoCrossLegs";                                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::SettlPriceIncrement]               = "SettlPriceIncrement";                             // (Price FIX.5.0SP2)
dictionary[tag::SettlPriceSecondaryIncrement]      = "SettlPriceSecondaryIncrement";                    // (Price FIX.5.0SP2)
dictionary[tag::ClearedIndicator]                  = "ClearedIndicator";                                // (int FIX.5.0SP2)
dictionary[tag::ContractRefPosType]                = "ContractRefPosType";                              // (int FIX.5.0SP2)
dictionary[tag::PositionCapacity]                  = "PositionCapacity";                                // (int FIX.5.0SP2)
dictionary[tag::PosQtyUnitOfMeasureCurrency]       = "PosQtyUnitOfMeasureCurrency";                     // (Currency FIX.5.0SP2)
dictionary[tag::PosQtyUnitOfMeasure]               = "PosQtyUnitOfMeasure";                             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingContractPriceRefMonth]   = "UnderlyingContractPriceRefMonth";                 // (MonthYear FIX.5.0SP2)
dictionary[tag::NoTradePriceConditions]            = "NoTradePriceConditions";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::TradePriceCondition]               = "TradePriceCondition";                             // (int FIX.5.0SP2)
dictionary[tag::TradeAllocStatus]                  = "TradeAllocStatus";                                // (int FIX.5.0SP2)
dictionary[tag::NoTradeQtys]                       = "NoTradeQtys";                                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::TradeQtyType]                      = "TradeQtyType";                                    // (int FIX.5.0SP2)
dictionary[tag::TradeQty]                          = "TradeQty";                                        // (Qty FIX.5.0SP2)
dictionary[tag::NoTradeAllocAmts]                  = "NoTradeAllocAmts";                                // (NumInGroup FIX.5.0SP2)
dictionary[tag::TradeAllocAmtType]                 = "TradeAllocAmtType";                               // (String FIX.5.0SP2)
dictionary[tag::TradeAllocAmt]                     = "TradeAllocAmt";                                   // (Amt FIX.5.0SP2)
dictionary[tag::TradeAllocCurrency]                = "TradeAllocCurrency";                              // (Currency FIX.5.0SP2)
dictionary[tag::TradeAllocGroupInstruction]        = "TradeAllocGroupInstruction";                      // (int FIX.5.0SP2)
dictionary[tag::OffsetInstruction]                 = "OffsetInstruction";                               // (int FIX.5.0SP2)
dictionary[tag::TradeAllocAmtReason]               = "TradeAllocAmtReason";                             // (int FIX.5.0SP2)
dictionary[tag::StrategyLinkID]                    = "StrategyLinkID";                                  // (String FIX.5.0SP2)
dictionary[tag::SideAvgPx]                         = "SideAvgPx";                                       // (Price FIX.5.0SP2)
dictionary[tag::SideAvgPxIndicator]                = "SideAvgPxIndicator";                              // (int FIX.5.0SP2)
dictionary[tag::SideAvgPxGroupID]                  = "SideAvgPxGroupID";                                // (String FIX.5.0SP2)
dictionary[tag::NoRelatedTrades]                   = "NoRelatedTrades";                                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelatedTradeID]                    = "RelatedTradeID";                                  // (String FIX.5.0SP2)
dictionary[tag::RelatedTradeIDSource]              = "RelatedTradeIDSource";                            // (int FIX.5.0SP2)
dictionary[tag::RelatedTradeDate]                  = "RelatedTradeDate";                                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::RelatedTradeMarketID]              = "RelatedTradeMarketID";                            // (Exchange FIX.5.0SP2)
dictionary[tag::RelatedTradeQuantity]              = "RelatedTradeQuantity";                            // (Qty FIX.5.0SP2)
dictionary[tag::NoRelatedPositions]                = "NoRelatedPositions";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelatedPositionID]                 = "RelatedPositionID";                               // (String FIX.5.0SP2)
dictionary[tag::RelatedPositionIDSource]           = "RelatedPositionIDSource";                         // (int FIX.5.0SP2)
dictionary[tag::RelatedPositionDate]               = "RelatedPositionDate";                             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::QuoteAckStatus]                    = "QuoteAckStatus";                                  // (int FIX.5.0SP2)
dictionary[tag::StrikeIndex]                       = "StrikeIndex";                                     // (String FIX.5.0SP2)
dictionary[tag::OfferID]                           = "OfferID";                                         // (String FIX.5.0SP2)
dictionary[tag::NoValueChecks]                     = "NoValueChecks";                                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::ValueCheckType]                    = "ValueCheckType";                                  // (int FIX.5.0SP2)
dictionary[tag::ValueCheckAction]                  = "ValueCheckAction";                                // (int FIX.5.0SP2)
dictionary[tag::LegSecurityXMLLen]                 = "LegSecurityXMLLen";                               // (Length FIX.5.0SP2)
dictionary[tag::LegSecurityXML]                    = "LegSecurityXML";                                  // (XMLData FIX.5.0SP2)
dictionary[tag::LegSecurityXMLSchema]              = "LegSecurityXMLSchema";                            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSecurityXMLLen]          = "UnderlyingSecurityXMLLen";                        // (Length FIX.5.0SP2)
dictionary[tag::UnderlyingSecurityXML]             = "UnderlyingSecurityXML";                           // (XMLData FIX.5.0SP2)
dictionary[tag::UnderlyingSecurityXMLSchema]       = "UnderlyingSecurityXMLSchema";                     // (String FIX.5.0SP2)
dictionary[tag::PartyDetailRequestResult]          = "PartyDetailRequestResult";                        // (int FIX.5.0SP2)
dictionary[tag::PartyDetailRequestStatus]          = "PartyDetailRequestStatus";                        // (int FIX.5.0SP2)
dictionary[tag::PartyDetailDefinitionStatus]       = "PartyDetailDefinitionStatus";                     // (int FIX.5.0SP2)
dictionary[tag::PartyDetailDefinitionResult]       = "PartyDetailDefinitionResult";                     // (int FIX.5.0SP2)
dictionary[tag::EntitlementRequestResult]          = "EntitlementRequestResult";                        // (int FIX.5.0SP2)
dictionary[tag::EntitlementRequestStatus]          = "EntitlementRequestStatus";                        // (int FIX.5.0SP2)
dictionary[tag::EntitlementStatus]                 = "EntitlementStatus";                               // (int FIX.5.0SP2)
dictionary[tag::EntitlementResult]                 = "EntitlementResult";                               // (int FIX.5.0SP2)
dictionary[tag::EntitlementRefID]                  = "EntitlementRefID";                                // (String FIX.5.0SP2)
dictionary[tag::SettlPriceUnitOfMeasure]           = "SettlPriceUnitOfMeasure";                         // (String FIX.5.0SP2)
dictionary[tag::SettlPriceUnitOfMeasureCurrency]   = "SettlPriceUnitOfMeasureCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::TradeMatchTimestamp]               = "TradeMatchTimestamp";                             // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::NoInstrmtMatchSides]               = "NoInstrmtMatchSides";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoTrdMatchSides]                   = "NoTrdMatchSides";                                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::TrdMatchSubID]                     = "TrdMatchSubID";                                   // (String FIX.5.0SP2)
dictionary[tag::NoLegExecs]                        = "NoLegExecs";                                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegExecID]                         = "LegExecID";                                       // (String FIX.5.0SP2)
dictionary[tag::LegTradeID]                        = "LegTradeID";                                      // (String FIX.5.0SP2)
dictionary[tag::LegTradeReportID]                  = "LegTradeReportID";                                // (String FIX.5.0SP2)
dictionary[tag::TradeMatchAckStatus]               = "TradeMatchAckStatus";                             // (int FIX.5.0SP2)
dictionary[tag::TradeMatchRejectReason]            = "TradeMatchRejectReason";                          // (int FIX.5.0SP2)
dictionary[tag::SideMarketSegmentID]               = "SideMarketSegmentID";                             // (String FIX.5.0SP2)
dictionary[tag::SideVenueType]                     = "SideVenueType";                                   // (char FIX.5.0SP2)
dictionary[tag::SideExecRefID]                     = "SideExecRefID";                                   // (String FIX.5.0SP2)
dictionary[tag::LegExecRefID]                      = "LegExecRefID";                                    // (String FIX.5.0SP2)
dictionary[tag::HaircutIndicator]                  = "HaircutIndicator";                                // (Boolean FIX.5.0SP2)
dictionary[tag::RegulatoryTradeID]                 = "RegulatoryTradeID";                               // (String FIX.5.0SP2)
dictionary[tag::RegulatoryTradeIDEvent]            = "RegulatoryTradeIDEvent";                          // (int FIX.5.0SP2)
dictionary[tag::RegulatoryTradeIDSource]           = "RegulatoryTradeIDSource";                         // (String FIX.5.0SP2)
dictionary[tag::RegulatoryTradeIDType]             = "RegulatoryTradeIDType";                           // (int FIX.5.0SP2)
dictionary[tag::NoRegulatoryTradeIDs]              = "NoRegulatoryTradeIDs";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoAllocRegulatoryTradeIDs]         = "NoAllocRegulatoryTradeIDs";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::AllocRegulatoryTradeID]            = "AllocRegulatoryTradeID";                          // (String FIX.5.0SP2)
dictionary[tag::AllocRegulatoryTradeIDSource]      = "AllocRegulatoryTradeIDSource";                    // (String FIX.5.0SP2)
dictionary[tag::AllocRegulatoryTradeIDEvent]       = "AllocRegulatoryTradeIDEvent";                     // (int FIX.5.0SP2)
dictionary[tag::AllocRegulatoryTradeIDType]        = "AllocRegulatoryTradeIDType";                      // (int FIX.5.0SP2)
dictionary[tag::NumOfCompetitors]                  = "NumOfCompetitors";                                // (int FIX.5.0SP2)
dictionary[tag::ResponseTime]                      = "ResponseTime";                                    // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::QuoteDisplayTime]                  = "QuoteDisplayTime";                                // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::ExposureDurationUnit]              = "ExposureDurationUnit";                            // (int FIX.5.0SP2)
dictionary[tag::CoverPrice]                        = "CoverPrice";                                      // (Price FIX.5.0SP2)
dictionary[tag::NoClearingAccountTypes]            = "NoClearingAccountTypes";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPriceMovements]                  = "NoPriceMovements";                                // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPriceMovementValues]             = "NoPriceMovementValues";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::PriceMovementValue]                = "PriceMovementValue";                              // (float FIX.5.0SP2)
dictionary[tag::PriceMovementPoint]                = "PriceMovementPoint";                              // (int FIX.5.0SP2)
dictionary[tag::PriceMovementType]                 = "PriceMovementType";                               // (int FIX.5.0SP2)
dictionary[tag::ClearingIntention]                 = "ClearingIntention";                               // (int FIX.5.0SP2)
dictionary[tag::TradeClearingInstruction]          = "TradeClearingInstruction";                        // (int FIX.5.0SP2)
dictionary[tag::BackloadedTradeIndicator]          = "BackloadedTradeIndicator";                        // (Boolean FIX.5.0SP2)
dictionary[tag::ConfirmationMethod]                = "ConfirmationMethod";                              // (int FIX.5.0SP2)
dictionary[tag::MandatoryClearingIndicator]        = "MandatoryClearingIndicator";                      // (Boolean FIX.5.0SP2)
dictionary[tag::MixedSwapIndicator]                = "MixedSwapIndicator";                              // (Boolean FIX.5.0SP2)
dictionary[tag::OffMarketPriceIndicator]           = "OffMarketPriceIndicator";                         // (Boolean FIX.5.0SP2)
dictionary[tag::VerificationMethod]                = "VerificationMethod";                              // (int FIX.5.0SP2)
dictionary[tag::ClearingRequirementException]      = "ClearingRequirementException";                    // (int FIX.5.0SP2)
dictionary[tag::IRSDirection]                      = "IRSDirection";                                    // (String FIX.5.0SP2)
dictionary[tag::RegulatoryReportType]              = "RegulatoryReportType";                            // (int FIX.5.0SP2)
dictionary[tag::VoluntaryRegulatoryReport]         = "VoluntaryRegulatoryReport";                       // (Boolean FIX.5.0SP2)
dictionary[tag::TradeCollateralization]            = "TradeCollateralization";                          // (int FIX.5.0SP2)
dictionary[tag::TradeContinuation]                 = "TradeContinuation";                               // (int FIX.5.0SP2)
dictionary[tag::AssetClass]                        = "AssetClass";                                      // (int FIX.5.0SP2)
dictionary[tag::AssetSubClass]                     = "AssetSubClass";                                   // (int FIX.5.0SP2)
dictionary[tag::AssetType]                         = "AssetType";                                       // (String FIX.5.0SP2)
dictionary[tag::SwapClass]                         = "SwapClass";                                       // (String FIX.5.0SP2)
dictionary[tag::NthToDefault]                      = "NthToDefault";                                    // (int FIX.5.0SP2)
dictionary[tag::MthToDefault]                      = "MthToDefault";                                    // (int FIX.5.0SP2)
dictionary[tag::SettledEntityMatrixSource]         = "SettledEntityMatrixSource";                       // (String FIX.5.0SP2)
dictionary[tag::SettledEntityMatrixPublicationDate] = "SettledEntityMatrixPublicationDate";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::CouponType]                        = "CouponType";                                      // (int FIX.5.0SP2)
dictionary[tag::TotalIssuedAmount]                 = "TotalIssuedAmount";                               // (Amt FIX.5.0SP2)
dictionary[tag::CouponFrequencyPeriod]             = "CouponFrequencyPeriod";                           // (int FIX.5.0SP2)
dictionary[tag::CouponFrequencyUnit]               = "CouponFrequencyUnit";                             // (String FIX.5.0SP2)
dictionary[tag::CouponDayCount]                    = "CouponDayCount";                                  // (int FIX.5.0SP2)
dictionary[tag::ConvertibleBondEquityID]           = "ConvertibleBondEquityID";                         // (String FIX.5.0SP2)
dictionary[tag::ConvertibleBondEquityIDSource]     = "ConvertibleBondEquityIDSource";                   // (String FIX.5.0SP2)
dictionary[tag::ContractPriceRefMonth]             = "ContractPriceRefMonth";                           // (MonthYear FIX.5.0SP2)
dictionary[tag::LienSeniority]                     = "LienSeniority";                                   // (int FIX.5.0SP2)
dictionary[tag::LoanFacility]                      = "LoanFacility";                                    // (int FIX.5.0SP2)
dictionary[tag::ReferenceEntityType]               = "ReferenceEntityType";                             // (int FIX.5.0SP2)
dictionary[tag::IndexSeries]                       = "IndexSeries";                                     // (int FIX.5.0SP2)
dictionary[tag::IndexAnnexVersion]                 = "IndexAnnexVersion";                               // (int FIX.5.0SP2)
dictionary[tag::IndexAnnexDate]                    = "IndexAnnexDate";                                  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::IndexAnnexSource]                  = "IndexAnnexSource";                                // (String FIX.5.0SP2)
dictionary[tag::AgreementVersion]                  = "AgreementVersion";                                // (String FIX.5.0SP2)
dictionary[tag::MasterConfirmationDesc]            = "MasterConfirmationDesc";                          // (String FIX.5.0SP2)
dictionary[tag::MasterConfirmationDate]            = "MasterConfirmationDate";                          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::MasterConfirmationAnnexDesc]       = "MasterConfirmationAnnexDesc";                     // (String FIX.5.0SP2)
dictionary[tag::MasterConfirmationAnnexDate]       = "MasterConfirmationAnnexDate";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::BrokerConfirmationDesc]            = "BrokerConfirmationDesc";                          // (String FIX.5.0SP2)
dictionary[tag::CreditSupportAgreementDesc]        = "CreditSupportAgreementDesc";                      // (String FIX.5.0SP2)
dictionary[tag::CreditSupportAgreementDate]        = "CreditSupportAgreementDate";                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::CreditSupportAgreementID]          = "CreditSupportAgreementID";                        // (String FIX.5.0SP2)
dictionary[tag::GoverningLaw]                      = "GoverningLaw";                                    // (String FIX.5.0SP2)
dictionary[tag::NoSideRegulatoryTradeIDs]          = "NoSideRegulatoryTradeIDs";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::SideRegulatoryTradeID]             = "SideRegulatoryTradeID";                           // (String FIX.5.0SP2)
dictionary[tag::SideRegulatoryTradeIDSource]       = "SideRegulatoryTradeIDSource";                     // (String FIX.5.0SP2)
dictionary[tag::SideRegulatoryTradeIDEvent]        = "SideRegulatoryTradeIDEvent";                      // (int FIX.5.0SP2)
dictionary[tag::SideRegulatoryTradeIDType]         = "SideRegulatoryTradeIDType";                       // (int FIX.5.0SP2)
dictionary[tag::NoSecondaryAssetClasses]           = "NoSecondaryAssetClasses";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::SecondaryAssetClass]               = "SecondaryAssetClass";                             // (int FIX.5.0SP2)
dictionary[tag::SecondaryAssetSubClass]            = "SecondaryAssetSubClass";                          // (int FIX.5.0SP2)
dictionary[tag::SecondaryAssetType]                = "SecondaryAssetType";                              // (String FIX.5.0SP2)
dictionary[tag::BlockTrdAllocIndicator]            = "BlockTrdAllocIndicator";                          // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingEvents]                = "NoUnderlyingEvents";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingEventType]               = "UnderlyingEventType";                             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingEventDate]               = "UnderlyingEventDate";                             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingEventTime]               = "UnderlyingEventTime";                             // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::UnderlyingEventTimeUnit]           = "UnderlyingEventTimeUnit";                         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingEventTimePeriod]         = "UnderlyingEventTimePeriod";                       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingEventPx]                 = "UnderlyingEventPx";                               // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingConstituentWeight]       = "UnderlyingConstituentWeight";                     // (float FIX.5.0SP2)
dictionary[tag::UnderlyingCouponType]              = "UnderlyingCouponType";                            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingTotalIssuedAmount]       = "UnderlyingTotalIssuedAmount";                     // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingCouponFrequencyPeriod]   = "UnderlyingCouponFrequencyPeriod";                 // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCouponFrequencyUnit]     = "UnderlyingCouponFrequencyUnit";                   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingCouponDayCount]          = "UnderlyingCouponDayCount";                        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingObligationID]            = "UnderlyingObligationID";                          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingObligationIDSource]      = "UnderlyingObligationIDSource";                    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingEquityID]                = "UnderlyingEquityID";                              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingEquityIDSource]          = "UnderlyingEquityIDSource";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingLienSeniority]           = "UnderlyingLienSeniority";                         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingLoanFacility]            = "UnderlyingLoanFacility";                          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReferenceEntityType]     = "UnderlyingReferenceEntityType";                   // (int FIX.5.0SP2)
dictionary[tag::StrikeIndexSpread]                 = "StrikeIndexSpread";                               // (PriceOffset FIX.5.0SP2)
dictionary[tag::ValuationSource]                   = "ValuationSource";                                 // (String FIX.5.0SP2)
dictionary[tag::UnderlyingIndexSeries]             = "UnderlyingIndexSeries";                           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingIndexAnnexVersion]       = "UnderlyingIndexAnnexVersion";                     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingIndexAnnexDate]          = "UnderlyingIndexAnnexDate";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingIndexAnnexSource]        = "UnderlyingIndexAnnexSource";                      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProductComplex]          = "UnderlyingProductComplex";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSecurityGroup]           = "UnderlyingSecurityGroup";                         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSettleOnOpenFlag]        = "UnderlyingSettleOnOpenFlag";                      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingAssignmentMethod]        = "UnderlyingAssignmentMethod";                      // (char FIX.5.0SP2)
dictionary[tag::UnderlyingSecurityStatus]          = "UnderlyingSecurityStatus";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingObligationType]          = "UnderlyingObligationType";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingAssetClass]              = "UnderlyingAssetClass";                            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingAssetSubClass]           = "UnderlyingAssetSubClass";                         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingAssetType]               = "UnderlyingAssetType";                             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSwapClass]               = "UnderlyingSwapClass";                             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingNthToDefault]            = "UnderlyingNthToDefault";                          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingMthToDefault]            = "UnderlyingMthToDefault";                          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSettledEntityMatrixSource] = "UnderlyingSettledEntityMatrixSource";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSettledEntityMatrixPublicationDate] = "UnderlyingSettledEntityMatrixPublicationDate";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStrikeMultiplier]        = "UnderlyingStrikeMultiplier";                      // (float FIX.5.0SP2)
dictionary[tag::UnderlyingStrikeValue]             = "UnderlyingStrikeValue";                           // (float FIX.5.0SP2)
dictionary[tag::UnderlyingStrikePriceDeterminationMethod] = "UnderlyingStrikePriceDeterminationMethod";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStrikePriceBoundaryMethod] = "UnderlyingStrikePriceBoundaryMethod";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStrikePriceBoundaryPrecision] = "UnderlyingStrikePriceBoundaryPrecision";          // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingMinPriceIncrement]       = "UnderlyingMinPriceIncrement";                     // (float FIX.5.0SP2)
dictionary[tag::UnderlyingMinPriceIncrementAmount] = "UnderlyingMinPriceIncrementAmount";               // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingOptPayoutType]           = "UnderlyingOptPayoutType";                         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptPayoutAmount]         = "UnderlyingOptPayoutAmount";                       // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPriceQuoteMethod]        = "UnderlyingPriceQuoteMethod";                      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingValuationMethod]         = "UnderlyingValuationMethod";                       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingListMethod]              = "UnderlyingListMethod";                            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCapPrice]                = "UnderlyingCapPrice";                              // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingFloorPrice]              = "UnderlyingFloorPrice";                            // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingFlexibleIndicator]       = "UnderlyingFlexibleIndicator";                     // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingFlexProductEligibilityIndicator] = "UnderlyingFlexProductEligibilityIndicator";       // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPositionLimit]           = "UnderlyingPositionLimit";                         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingNTPositionLimit]         = "UnderlyingNTPositionLimit";                       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPool]                    = "UnderlyingPool";                                  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingContractSettlMonth]      = "UnderlyingContractSettlMonth";                    // (MonthYear FIX.5.0SP2)
dictionary[tag::UnderlyingDatedDate]               = "UnderlyingDatedDate";                             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingInterestAccrualDate]     = "UnderlyingInterestAccrualDate";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingShortSaleRestriction]    = "UnderlyingShortSaleRestriction";                  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingRefTickTableID]          = "UnderlyingRefTickTableID";                        // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEvents]         = "NoUnderlyingComplexEvents";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventType]        = "UnderlyingComplexEventType";                      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexOptPayoutAmount]  = "UnderlyingComplexOptPayoutAmount";                // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPrice]       = "UnderlyingComplexEventPrice";                     // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPriceBoundaryMethod] = "UnderlyingComplexEventPriceBoundaryMethod";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPriceBoundaryPrecision] = "UnderlyingComplexEventPriceBoundaryPrecision";    // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPriceTimeType] = "UnderlyingComplexEventPriceTimeType";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCondition]   = "UnderlyingComplexEventCondition";                 // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventDates]     = "NoUnderlyingComplexEventDates";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventStartDate]   = "UnderlyingComplexEventStartDate";                 // (UTCDateOnly FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventEndDate]     = "UnderlyingComplexEventEndDate";                   // (UTCDateOnly FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventTimes]     = "NoUnderlyingComplexEventTimes";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventStartTime]   = "UnderlyingComplexEventStartTime";                 // (UTCTimeOnly FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventEndTime]     = "UnderlyingComplexEventEndTime";                   // (UTCTimeOnly FIX.5.0SP2)
dictionary[tag::NoLegEvents]                       = "NoLegEvents";                                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegEventType]                      = "LegEventType";                                    // (int FIX.5.0SP2)
dictionary[tag::LegEventDate]                      = "LegEventDate";                                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegEventTime]                      = "LegEventTime";                                    // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::LegEventTimeUnit]                  = "LegEventTimeUnit";                                // (String FIX.5.0SP2)
dictionary[tag::LegEventTimePeriod]                = "LegEventTimePeriod";                              // (int FIX.5.0SP2)
dictionary[tag::LegEventPx]                        = "LegEventPx";                                      // (Price FIX.5.0SP2)
dictionary[tag::LegEventText]                      = "LegEventText";                                    // (String FIX.5.0SP2)
dictionary[tag::LegAssetClass]                     = "LegAssetClass";                                   // (int FIX.5.0SP2)
dictionary[tag::LegAssetSubClass]                  = "LegAssetSubClass";                                // (int FIX.5.0SP2)
dictionary[tag::LegAssetType]                      = "LegAssetType";                                    // (String FIX.5.0SP2)
dictionary[tag::LegSwapClass]                      = "LegSwapClass";                                    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingEventText]               = "UnderlyingEventText";                             // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingEventTextLen]     = "EncodedUnderlyingEventTextLen";                   // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingEventText]        = "EncodedUnderlyingEventText";                      // (data FIX.5.0SP2)
dictionary[tag::EncodedLegEventTextLen]            = "EncodedLegEventTextLen";                          // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegEventText]               = "EncodedLegEventText";                             // (data FIX.5.0SP2)
dictionary[tag::NoLegSecondaryAssetClasses]        = "NoLegSecondaryAssetClasses";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegSecondaryAssetClass]            = "LegSecondaryAssetClass";                          // (int FIX.5.0SP2)
dictionary[tag::LegSecondaryAssetSubClass]         = "LegSecondaryAssetSubClass";                       // (int FIX.5.0SP2)
dictionary[tag::LegSecondaryAssetType]             = "LegSecondaryAssetType";                           // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingSecondaryAssetClasses] = "NoUnderlyingSecondaryAssetClasses";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingSecondaryAssetClass]     = "UnderlyingSecondaryAssetClass";                   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSecondaryAssetSubClass]  = "UnderlyingSecondaryAssetSubClass";                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSecondaryAssetType]      = "UnderlyingSecondaryAssetType";                    // (String FIX.5.0SP2)
dictionary[tag::PreviousClearingBusinessDate]      = "PreviousClearingBusinessDate";                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ValuationDate]                     = "ValuationDate";                                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ValuationTime]                     = "ValuationTime";                                   // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ValuationBusinessCenter]           = "ValuationBusinessCenter";                         // (String FIX.5.0SP2)
dictionary[tag::MarginAmtFXRate]                   = "MarginAmtFXRate";                                 // (float FIX.5.0SP2)
dictionary[tag::MarginAmtFXRateCalc]               = "MarginAmtFXRateCalc";                             // (char FIX.5.0SP2)
dictionary[tag::CollateralFXRate]                  = "CollateralFXRate";                                // (float FIX.5.0SP2)
dictionary[tag::CollateralFXRateCalc]              = "CollateralFXRateCalc";                            // (char FIX.5.0SP2)
dictionary[tag::CollateralAmountMarketSegmentID]   = "CollateralAmountMarketSegmentID";                 // (String FIX.5.0SP2)
dictionary[tag::CollateralAmountMarketID]          = "CollateralAmountMarketID";                        // (String FIX.5.0SP2)
dictionary[tag::PayCollectFXRate]                  = "PayCollectFXRate";                                // (float FIX.5.0SP2)
dictionary[tag::PayCollectFXRateCalc]              = "PayCollectFXRateCalc";                            // (char FIX.5.0SP2)
dictionary[tag::PosAmtStreamDesc]                  = "PosAmtStreamDesc";                                // (String FIX.5.0SP2)
dictionary[tag::PositionFXRate]                    = "PositionFXRate";                                  // (float FIX.5.0SP2)
dictionary[tag::PositionFXRateCalc]                = "PositionFXRateCalc";                              // (char FIX.5.0SP2)
dictionary[tag::PosAmtMarketSegmentID]             = "PosAmtMarketSegmentID";                           // (String FIX.5.0SP2)
dictionary[tag::PosAmtMarketID]                    = "PosAmtMarketID";                                  // (String FIX.5.0SP2)
dictionary[tag::TerminatedIndicator]               = "TerminatedIndicator";                             // (Boolean FIX.5.0SP2)
dictionary[tag::ShortMarkingExemptIndicator]       = "ShortMarkingExemptIndicator";                     // (Boolean FIX.5.0SP2)
dictionary[tag::RelatedRegulatoryTradeIDSource]    = "RelatedRegulatoryTradeIDSource";                  // (String FIX.5.0SP2)
dictionary[tag::NoAttachments]                     = "NoAttachments";                                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::AttachmentName]                    = "AttachmentName";                                  // (String FIX.5.0SP2)
dictionary[tag::AttachmentMediaType]               = "AttachmentMediaType";                             // (String FIX.5.0SP2)
dictionary[tag::AttachmentClassification]          = "AttachmentClassification";                        // (String FIX.5.0SP2)
dictionary[tag::AttachmentExternalURL]             = "AttachmentExternalURL";                           // (String FIX.5.0SP2)
dictionary[tag::AttachmentEncodingType]            = "AttachmentEncodingType";                          // (int FIX.5.0SP2)
dictionary[tag::UnencodedAttachmentLen]            = "UnencodedAttachmentLen";                          // (int FIX.5.0SP2)
dictionary[tag::EncodedAttachmentLen]              = "EncodedAttachmentLen";                            // (Length FIX.5.0SP2)
dictionary[tag::EncodedAttachment]                 = "EncodedAttachment";                               // (data FIX.5.0SP2)
dictionary[tag::NoAttachmentKeywords]              = "NoAttachmentKeywords";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::AttachmentKeyword]                 = "AttachmentKeyword";                               // (String FIX.5.0SP2)
dictionary[tag::NegotiationMethod]                 = "NegotiationMethod";                               // (int FIX.5.0SP2)
dictionary[tag::NextAuctionTime]                   = "NextAuctionTime";                                 // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::ComplexOptPayoutPaySide]           = "ComplexOptPayoutPaySide";                         // (int FIX.5.0SP2)
dictionary[tag::ComplexOptPayoutReceiveSide]       = "ComplexOptPayoutReceiveSide";                     // (int FIX.5.0SP2)
dictionary[tag::ComplexOptPayoutUnderlier]         = "ComplexOptPayoutUnderlier";                       // (String FIX.5.0SP2)
dictionary[tag::ComplexOptPayoutPercentage]        = "ComplexOptPayoutPercentage";                      // (Percentage FIX.5.0SP2)
dictionary[tag::ComplexOptPayoutTime]              = "ComplexOptPayoutTime";                            // (int FIX.5.0SP2)
dictionary[tag::ComplexOptPayoutCurrency]          = "ComplexOptPayoutCurrency";                        // (Currency FIX.5.0SP2)
dictionary[tag::ComplexEventPricePercentage]       = "ComplexEventPricePercentage";                     // (Percentage FIX.5.0SP2)
dictionary[tag::ComplexEventCurrencyOne]           = "ComplexEventCurrencyOne";                         // (Currency FIX.5.0SP2)
dictionary[tag::ComplexEventCurrencyTwo]           = "ComplexEventCurrencyTwo";                         // (Currency FIX.5.0SP2)
dictionary[tag::ComplexEventQuoteBasis]            = "ComplexEventQuoteBasis";                          // (int FIX.5.0SP2)
dictionary[tag::ComplexEventFixedFXRate]           = "ComplexEventFixedFXRate";                         // (float FIX.5.0SP2)
dictionary[tag::ComplexEventDeterminationMethod]   = "ComplexEventDeterminationMethod";                 // (String FIX.5.0SP2)
dictionary[tag::ComplexEventCalculationAgent]      = "ComplexEventCalculationAgent";                    // (int FIX.5.0SP2)
dictionary[tag::ComplexEventStrikePrice]           = "ComplexEventStrikePrice";                         // (Price FIX.5.0SP2)
dictionary[tag::ComplexEventStrikeFactor]          = "ComplexEventStrikeFactor";                        // (float FIX.5.0SP2)
dictionary[tag::ComplexEventStrikeNumberOfOptions] = "ComplexEventStrikeNumberOfOptions";               // (int FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventsXIDRef]    = "ComplexEventCreditEventsXIDRef";                  // (XIDREF FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventNotifyingParty] = "ComplexEventCreditEventNotifyingParty";           // (int FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventBusinessCenter] = "ComplexEventCreditEventBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventStandardSources] = "ComplexEventCreditEventStandardSources";          // (Boolean FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventMinimumSources] = "ComplexEventCreditEventMinimumSources";           // (int FIX.5.0SP2)
dictionary[tag::ComplexEventXID]                   = "ComplexEventXID";                                 // (XID FIX.5.0SP2)
dictionary[tag::ComplexEventXIDRef]                = "ComplexEventXIDRef";                              // (XIDREF FIX.5.0SP2)
dictionary[tag::ValuationReferenceModel]           = "ValuationReferenceModel";                         // (String FIX.5.0SP2)
dictionary[tag::StrategyType]                      = "StrategyType";                                    // (String FIX.5.0SP2)
dictionary[tag::CommonPricingIndicator]            = "CommonPricingIndicator";                          // (Boolean FIX.5.0SP2)
dictionary[tag::SettlDisruptionProvision]          = "SettlDisruptionProvision";                        // (int FIX.5.0SP2)
dictionary[tag::InstrumentRoundingDirection]       = "InstrumentRoundingDirection";                     // (char FIX.5.0SP2)
dictionary[tag::InstrumentRoundingPrecision]       = "InstrumentRoundingPrecision";                     // (int FIX.5.0SP2)
dictionary[tag::LegSettleOnOpenFlag]               = "LegSettleOnOpenFlag";                             // (String FIX.5.0SP2)
dictionary[tag::LegInstrmtAssignmentMethod]        = "LegInstrmtAssignmentMethod";                      // (char FIX.5.0SP2)
dictionary[tag::LegSecurityStatus]                 = "LegSecurityStatus";                               // (String FIX.5.0SP2)
dictionary[tag::LegRestructuringType]              = "LegRestructuringType";                            // (String FIX.5.0SP2)
dictionary[tag::LegSeniority]                      = "LegSeniority";                                    // (String FIX.5.0SP2)
dictionary[tag::LegNotionalPercentageOutstanding]  = "LegNotionalPercentageOutstanding";                // (Percentage FIX.5.0SP2)
dictionary[tag::LegOriginalNotionalPercentageOutstanding] = "LegOriginalNotionalPercentageOutstanding";        // (Percentage FIX.5.0SP2)
dictionary[tag::LegAttachmentPoint]                = "LegAttachmentPoint";                              // (Percentage FIX.5.0SP2)
dictionary[tag::LegDetachmentPoint]                = "LegDetachmentPoint";                              // (Percentage FIX.5.0SP2)
dictionary[tag::LegObligationType]                 = "LegObligationType";                               // (String FIX.5.0SP2)
dictionary[tag::LegSwapSubClass]                   = "LegSwapSubClass";                                 // (String FIX.5.0SP2)
dictionary[tag::LegNthToDefault]                   = "LegNthToDefault";                                 // (int FIX.5.0SP2)
dictionary[tag::LegMthToDefault]                   = "LegMthToDefault";                                 // (int FIX.5.0SP2)
dictionary[tag::LegSettledEntityMatrixSource]      = "LegSettledEntityMatrixSource";                    // (String FIX.5.0SP2)
dictionary[tag::LegSettledEntityMatrixPublicationDate] = "LegSettledEntityMatrixPublicationDate";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegCouponType]                     = "LegCouponType";                                   // (int FIX.5.0SP2)
dictionary[tag::LegTotalIssuedAmount]              = "LegTotalIssuedAmount";                            // (Amt FIX.5.0SP2)
dictionary[tag::LegCouponFrequencyPeriod]          = "LegCouponFrequencyPeriod";                        // (int FIX.5.0SP2)
dictionary[tag::LegCouponFrequencyUnit]            = "LegCouponFrequencyUnit";                          // (String FIX.5.0SP2)
dictionary[tag::LegCouponDayCount]                 = "LegCouponDayCount";                               // (int FIX.5.0SP2)
dictionary[tag::LegConvertibleBondEquityID]        = "LegConvertibleBondEquityID";                      // (String FIX.5.0SP2)
dictionary[tag::LegConvertibleBondEquityIDSource]  = "LegConvertibleBondEquityIDSource";                // (String FIX.5.0SP2)
dictionary[tag::LegContractPriceRefMonth]          = "LegContractPriceRefMonth";                        // (MonthYear FIX.5.0SP2)
dictionary[tag::LegLienSeniority]                  = "LegLienSeniority";                                // (int FIX.5.0SP2)
dictionary[tag::LegLoanFacility]                   = "LegLoanFacility";                                 // (int FIX.5.0SP2)
dictionary[tag::LegReferenceEntityType]            = "LegReferenceEntityType";                          // (int FIX.5.0SP2)
dictionary[tag::LegIndexSeries]                    = "LegIndexSeries";                                  // (int FIX.5.0SP2)
dictionary[tag::LegIndexAnnexVersion]              = "LegIndexAnnexVersion";                            // (int FIX.5.0SP2)
dictionary[tag::LegIndexAnnexDate]                 = "LegIndexAnnexDate";                               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegIndexAnnexSource]               = "LegIndexAnnexSource";                             // (String FIX.5.0SP2)
dictionary[tag::LegSettlRateIndex]                 = "LegSettlRateIndex";                               // (String FIX.5.0SP2)
dictionary[tag::LegSettlRateIndexLocation]         = "LegSettlRateIndexLocation";                       // (String FIX.5.0SP2)
dictionary[tag::LegOptionExpirationDesc]           = "LegOptionExpirationDesc";                         // (String FIX.5.0SP2)
dictionary[tag::EncodedLegOptionExpirationDescLen] = "EncodedLegOptionExpirationDescLen";               // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegOptionExpirationDesc]    = "EncodedLegOptionExpirationDesc";                  // (data FIX.5.0SP2)
dictionary[tag::LegStrikeMultiplier]               = "LegStrikeMultiplier";                             // (float FIX.5.0SP2)
dictionary[tag::LegStrikeValue]                    = "LegStrikeValue";                                  // (float FIX.5.0SP2)
dictionary[tag::LegStrikeUnitOfMeasure]            = "LegStrikeUnitOfMeasure";                          // (String FIX.5.0SP2)
dictionary[tag::LegStrikeIndex]                    = "LegStrikeIndex";                                  // (String FIX.5.0SP2)
dictionary[tag::LegStrikeIndexSpread]              = "LegStrikeIndexSpread";                            // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegStrikePriceDeterminationMethod] = "LegStrikePriceDeterminationMethod";               // (int FIX.5.0SP2)
dictionary[tag::LegStrikePriceBoundaryMethod]      = "LegStrikePriceBoundaryMethod";                    // (int FIX.5.0SP2)
dictionary[tag::LegStrikePriceBoundaryPrecision]   = "LegStrikePriceBoundaryPrecision";                 // (Percentage FIX.5.0SP2)
dictionary[tag::LegUnderlyingPriceDeterminationMethod] = "LegUnderlyingPriceDeterminationMethod";           // (int FIX.5.0SP2)
dictionary[tag::LegMinPriceIncrement]              = "LegMinPriceIncrement";                            // (float FIX.5.0SP2)
dictionary[tag::LegMinPriceIncrementAmount]        = "LegMinPriceIncrementAmount";                      // (Amt FIX.5.0SP2)
dictionary[tag::LegSettlMethod]                    = "LegSettlMethod";                                  // (String FIX.5.0SP2)
dictionary[tag::LegOptPayoutType]                  = "LegOptPayoutType";                                // (int FIX.5.0SP2)
dictionary[tag::LegOptPayoutAmount]                = "LegOptPayoutAmount";                              // (Amt FIX.5.0SP2)
dictionary[tag::LegPriceQuoteMethod]               = "LegPriceQuoteMethod";                             // (String FIX.5.0SP2)
dictionary[tag::LegValuationMethod]                = "LegValuationMethod";                              // (String FIX.5.0SP2)
dictionary[tag::LegValuationSource]                = "LegValuationSource";                              // (String FIX.5.0SP2)
dictionary[tag::LegValuationReferenceModel]        = "LegValuationReferenceModel";                      // (String FIX.5.0SP2)
dictionary[tag::LegListMethod]                     = "LegListMethod";                                   // (int FIX.5.0SP2)
dictionary[tag::LegCapPrice]                       = "LegCapPrice";                                     // (Price FIX.5.0SP2)
dictionary[tag::LegFloorPrice]                     = "LegFloorPrice";                                   // (Price FIX.5.0SP2)
dictionary[tag::LegFlexibleIndicator]              = "LegFlexibleIndicator";                            // (Boolean FIX.5.0SP2)
dictionary[tag::LegFlexProductEligibilityIndicator] = "LegFlexProductEligibilityIndicator";              // (Boolean FIX.5.0SP2)
dictionary[tag::LegComplexEventStartTime]          = "LegComplexEventStartTime";                        // (UTCTimeOnly FIX.5.0SP2)
dictionary[tag::LegPositionLimit]                  = "LegPositionLimit";                                // (int FIX.5.0SP2)
dictionary[tag::LegNTPositionLimit]                = "LegNTPositionLimit";                              // (int FIX.5.0SP2)
dictionary[tag::LegCPProgram]                      = "LegCPProgram";                                    // (int FIX.5.0SP2)
dictionary[tag::LegCPRegType]                      = "LegCPRegType";                                    // (String FIX.5.0SP2)
dictionary[tag::LegShortSaleRestriction]           = "LegShortSaleRestriction";                         // (int FIX.5.0SP2)
dictionary[tag::AssetGroup]                        = "AssetGroup";                                      // (int FIX.5.0SP2)
dictionary[tag::LegStrategyType]                   = "LegStrategyType";                                 // (String FIX.5.0SP2)
dictionary[tag::LegCommonPricingIndicator]         = "LegCommonPricingIndicator";                       // (Boolean FIX.5.0SP2)
dictionary[tag::LegSettlDisruptionProvision]       = "LegSettlDisruptionProvision";                     // (int FIX.5.0SP2)
dictionary[tag::LegInstrumentRoundingDirection]    = "LegInstrumentRoundingDirection";                  // (char FIX.5.0SP2)
dictionary[tag::LegInstrumentRoundingPrecision]    = "LegInstrumentRoundingPrecision";                  // (int FIX.5.0SP2)
dictionary[tag::MiscFeeRate]                       = "MiscFeeRate";                                     // (Percentage FIX.5.0SP2)
dictionary[tag::MiscFeeAmountDue]                  = "MiscFeeAmountDue";                                // (Amt FIX.5.0SP2)
dictionary[tag::NoLegComplexEvents]                = "NoLegComplexEvents";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventType]               = "LegComplexEventType";                             // (int FIX.5.0SP2)
dictionary[tag::LegComplexOptPayoutPaySide]        = "LegComplexOptPayoutPaySide";                      // (int FIX.5.0SP2)
dictionary[tag::LegComplexOptPayoutReceiveSide]    = "LegComplexOptPayoutReceiveSide";                  // (int FIX.5.0SP2)
dictionary[tag::LegComplexOptPayoutUnderlier]      = "LegComplexOptPayoutUnderlier";                    // (String FIX.5.0SP2)
dictionary[tag::LegComplexOptPayoutAmount]         = "LegComplexOptPayoutAmount";                       // (Amt FIX.5.0SP2)
dictionary[tag::LegComplexOptPayoutPercentage]     = "LegComplexOptPayoutPercentage";                   // (Percentage FIX.5.0SP2)
dictionary[tag::LegComplexOptPayoutTime]           = "LegComplexOptPayoutTime";                         // (int FIX.5.0SP2)
dictionary[tag::LegComplexOptPayoutCurrency]       = "LegComplexOptPayoutCurrency";                     // (Currency FIX.5.0SP2)
dictionary[tag::LegComplexEventPrice]              = "LegComplexEventPrice";                            // (Price FIX.5.0SP2)
dictionary[tag::LegComplexEventPricePercentage]    = "LegComplexEventPricePercentage";                  // (Percentage FIX.5.0SP2)
dictionary[tag::LegComplexEventPriceBoundaryMethod] = "LegComplexEventPriceBoundaryMethod";              // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventPriceBoundaryPrecision] = "LegComplexEventPriceBoundaryPrecision";           // (Percentage FIX.5.0SP2)
dictionary[tag::LegComplexEventPriceTimeType]      = "LegComplexEventPriceTimeType";                    // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventCondition]          = "LegComplexEventCondition";                        // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventCurrencyOne]        = "LegComplexEventCurrencyOne";                      // (Currency FIX.5.0SP2)
dictionary[tag::LegComplexEventCurrencyTwo]        = "LegComplexEventCurrencyTwo";                      // (Currency FIX.5.0SP2)
dictionary[tag::LegComplexEventQuoteBasis]         = "LegComplexEventQuoteBasis";                       // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventFixedFXRate]        = "LegComplexEventFixedFXRate";                      // (float FIX.5.0SP2)
dictionary[tag::LegComplexEventDeterminationMethod] = "LegComplexEventDeterminationMethod";              // (String FIX.5.0SP2)
dictionary[tag::LegComplexEventCalculationAgent]   = "LegComplexEventCalculationAgent";                 // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventStrikePrice]        = "LegComplexEventStrikePrice";                      // (Price FIX.5.0SP2)
dictionary[tag::LegComplexEventStrikeFactor]       = "LegComplexEventStrikeFactor";                     // (float FIX.5.0SP2)
dictionary[tag::LegComplexEventStrikeNumberOfOptions] = "LegComplexEventStrikeNumberOfOptions";            // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventsXIDRef] = "LegComplexEventCreditEventsXIDRef";               // (XIDREF FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventNotifyingParty] = "LegComplexEventCreditEventNotifyingParty";        // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventBusinessCenter] = "LegComplexEventCreditEventBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventStandardSources] = "LegComplexEventCreditEventStandardSources";       // (Boolean FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventMinimumSources] = "LegComplexEventCreditEventMinimumSources";        // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventEndTime]            = "LegComplexEventEndTime";                          // (UTCTimeOnly FIX.5.0SP2)
dictionary[tag::LegComplexEventXID]                = "LegComplexEventXID";                              // (XID FIX.5.0SP2)
dictionary[tag::LegComplexEventXIDRef]             = "LegComplexEventXIDRef";                           // (XIDREF FIX.5.0SP2)
dictionary[tag::NoLegComplexEventDates]            = "NoLegComplexEventDates";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventStartDate]          = "LegComplexEventStartDate";                        // (UTCDateOnly FIX.5.0SP2)
dictionary[tag::LegComplexEventEndDate]            = "LegComplexEventEndDate";                          // (UTCDateOnly FIX.5.0SP2)
dictionary[tag::NoLegComplexEventTimes]            = "NoLegComplexEventTimes";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegInstrumentParties]            = "NoLegInstrumentParties";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegInstrumentPartyID]              = "LegInstrumentPartyID";                            // (String FIX.5.0SP2)
dictionary[tag::LegInstrumentPartyIDSource]        = "LegInstrumentPartyIDSource";                      // (char FIX.5.0SP2)
dictionary[tag::LegInstrumentPartyRole]            = "LegInstrumentPartyRole";                          // (int FIX.5.0SP2)
dictionary[tag::NoLegInstrumentPartySubIDs]        = "NoLegInstrumentPartySubIDs";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegInstrumentPartySubID]           = "LegInstrumentPartySubID";                         // (String FIX.5.0SP2)
dictionary[tag::LegInstrumentPartySubIDType]       = "LegInstrumentPartySubIDType";                     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexOptPayoutPaySide] = "UnderlyingComplexOptPayoutPaySide";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexOptPayoutReceiveSide] = "UnderlyingComplexOptPayoutReceiveSide";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexOptPayoutUnderlier] = "UnderlyingComplexOptPayoutUnderlier";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexOptPayoutPercentage] = "UnderlyingComplexOptPayoutPercentage";            // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingComplexOptPayoutTime]    = "UnderlyingComplexOptPayoutTime";                  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexOptPayoutCurrency] = "UnderlyingComplexOptPayoutCurrency";              // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPricePercentage] = "UnderlyingComplexEventPricePercentage";           // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCurrencyOne] = "UnderlyingComplexEventCurrencyOne";               // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCurrencyTwo] = "UnderlyingComplexEventCurrencyTwo";               // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventQuoteBasis]  = "UnderlyingComplexEventQuoteBasis";                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventFixedFXRate] = "UnderlyingComplexEventFixedFXRate";               // (float FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDeterminationMethod] = "UnderlyingComplexEventDeterminationMethod";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCalculationAgent] = "UnderlyingComplexEventCalculationAgent";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventStrikePrice] = "UnderlyingComplexEventStrikePrice";               // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventStrikeFactor] = "UnderlyingComplexEventStrikeFactor";              // (float FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventStrikeNumberOfOptions] = "UnderlyingComplexEventStrikeNumberOfOptions";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventsXIDRef] = "UnderlyingComplexEventCreditEventsXIDRef";        // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventNotifyingParty] = "UnderlyingComplexEventCreditEventNotifyingParty"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventBusinessCenter] = "UnderlyingComplexEventCreditEventBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventStandardSources] = "UnderlyingComplexEventCreditEventStandardSources"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventMinimumSources] = "UnderlyingComplexEventCreditEventMinimumSources"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventXID]         = "UnderlyingComplexEventXID";                       // (XID FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventXIDRef]      = "UnderlyingComplexEventXIDRef";                    // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingSettlRateIndex]          = "UnderlyingSettlRateIndex";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSettlRateIndexLocation]  = "UnderlyingSettlRateIndexLocation";                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExpirationDesc]    = "UnderlyingOptionExpirationDesc";                  // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingOptionExpirationDescLen] = "EncodedUnderlyingOptionExpirationDescLen";        // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingOptionExpirationDesc] = "EncodedUnderlyingOptionExpirationDesc";           // (data FIX.5.0SP2)
dictionary[tag::UnderlyingSwapSubClass]            = "UnderlyingSwapSubClass";                          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStrikeUnitOfMeasure]     = "UnderlyingStrikeUnitOfMeasure";                   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStrikeIndex]             = "UnderlyingStrikeIndex";                           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStrikeIndexSpread]       = "UnderlyingStrikeIndexSpread";                     // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingValuationSource]         = "UnderlyingValuationSource";                       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingValuationReferenceModel] = "UnderlyingValuationReferenceModel";               // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStrategyType]            = "UnderlyingStrategyType";                          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingCommonPricingIndicator]  = "UnderlyingCommonPricingIndicator";                // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingSettlDisruptionProvision] = "UnderlyingSettlDisruptionProvision";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingInstrumentRoundingDirection] = "UnderlyingInstrumentRoundingDirection";           // (char FIX.5.0SP2)
dictionary[tag::UnderlyingInstrumentRoundingPrecision] = "UnderlyingInstrumentRoundingPrecision";           // (int FIX.5.0SP2)
dictionary[tag::AllocGrossTradeAmt]                = "AllocGrossTradeAmt";                              // (Amt FIX.5.0SP2)
dictionary[tag::LastQtyChanged]                    = "LastQtyChanged";                                  // (Qty FIX.5.0SP2)
dictionary[tag::TradeVersion]                      = "TradeVersion";                                    // (String FIX.5.0SP2)
dictionary[tag::HistoricalReportIndicator]         = "HistoricalReportIndicator";                       // (Boolean FIX.5.0SP2)
dictionary[tag::NoAssetAttributes]                 = "NoAssetAttributes";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::AssetAttributeType]                = "AssetAttributeType";                              // (String FIX.5.0SP2)
dictionary[tag::AssetAttributeValue]               = "AssetAttributeValue";                             // (String FIX.5.0SP2)
dictionary[tag::AssetAttributeLimit]               = "AssetAttributeLimit";                             // (String FIX.5.0SP2)
dictionary[tag::NoLegAssetAttributes]              = "NoLegAssetAttributes";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegAssetAttributeType]             = "LegAssetAttributeType";                           // (String FIX.5.0SP2)
dictionary[tag::LegAssetAttributeValue]            = "LegAssetAttributeValue";                          // (String FIX.5.0SP2)
dictionary[tag::LegAssetAttributeLimit]            = "LegAssetAttributeLimit";                          // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingAssetAttributes]       = "NoUnderlyingAssetAttributes";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingAssetAttributeType]      = "UnderlyingAssetAttributeType";                    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingAssetAttributeValue]     = "UnderlyingAssetAttributeValue";                   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingAssetAttributeLimit]     = "UnderlyingAssetAttributeLimit";                   // (String FIX.5.0SP2)
dictionary[tag::RiskLimitReportStatus]             = "RiskLimitReportStatus";                           // (int FIX.5.0SP2)
dictionary[tag::RiskLimitReportRejectReason]       = "RiskLimitReportRejectReason";                     // (int FIX.5.0SP2)
dictionary[tag::RiskLimitCheckRequestID]           = "RiskLimitCheckRequestID";                         // (String FIX.5.0SP2)
dictionary[tag::RiskLimitCheckID]                  = "RiskLimitCheckID";                                // (String FIX.5.0SP2)
dictionary[tag::RiskLimitCheckTransType]           = "RiskLimitCheckTransType";                         // (int FIX.5.0SP2)
dictionary[tag::RiskLimitCheckType]                = "RiskLimitCheckType";                              // (int FIX.5.0SP2)
dictionary[tag::RiskLimitCheckRequestRefID]        = "RiskLimitCheckRequestRefID";                      // (int FIX.5.0SP2)
dictionary[tag::RiskLimitCheckRequestType]         = "RiskLimitCheckRequestType";                       // (int FIX.5.0SP2)
dictionary[tag::RiskLimitCheckAmount]              = "RiskLimitCheckAmount";                            // (Amt FIX.5.0SP2)
dictionary[tag::RiskLimitCheckRequestStatus]       = "RiskLimitCheckRequestStatus";                     // (int FIX.5.0SP2)
dictionary[tag::RiskLimitCheckRequestResult]       = "RiskLimitCheckRequestResult";                     // (int FIX.5.0SP2)
dictionary[tag::RiskLimitApprovedAmount]           = "RiskLimitApprovedAmount";                         // (Amt FIX.5.0SP2)
dictionary[tag::PartyActionRequestID]              = "PartyActionRequestID";                            // (String FIX.5.0SP2)
dictionary[tag::PartyActionType]                   = "PartyActionType";                                 // (int FIX.5.0SP2)
dictionary[tag::ApplTestMessageIndicator]          = "ApplTestMessageIndicator";                        // (Boolean FIX.5.0SP2)
dictionary[tag::PartyActionReportID]               = "PartyActionReportID";                             // (String FIX.5.0SP2)
dictionary[tag::PartyActionResponse]               = "PartyActionResponse";                             // (int FIX.5.0SP2)
dictionary[tag::PartyActionRejectReason]           = "PartyActionRejectReason";                         // (int FIX.5.0SP2)
dictionary[tag::RefRiskLimitCheckID]               = "RefRiskLimitCheckID";                             // (String FIX.5.0SP2)
dictionary[tag::RefRiskLimitCheckIDType]           = "RefRiskLimitCheckIDType";                         // (int FIX.5.0SP2)
dictionary[tag::RiskLimitVelocityPeriod]           = "RiskLimitVelocityPeriod";                         // (int FIX.5.0SP2)
dictionary[tag::RiskLimitVelocityUnit]             = "RiskLimitVelocityUnit";                           // (String FIX.5.0SP2)
dictionary[tag::RequestingPartyRoleQualifier]      = "RequestingPartyRoleQualifier";                    // (int FIX.5.0SP2)
dictionary[tag::RiskLimitCheckModelType]           = "RiskLimitCheckModelType";                         // (int FIX.5.0SP2)
dictionary[tag::EventMonthYear]                    = "EventMonthYear";                                  // (MonthYear FIX.5.0SP2)
dictionary[tag::LegEventMonthYear]                 = "LegEventMonthYear";                               // (MonthYear FIX.5.0SP2)
dictionary[tag::UnderlyingEventMonthYear]          = "UnderlyingEventMonthYear";                        // (MonthYear FIX.5.0SP2)
dictionary[tag::RiskLimitCheckStatus]              = "RiskLimitCheckStatus";                            // (int FIX.5.0SP2)
dictionary[tag::SideRiskLimitCheckStatus]          = "SideRiskLimitCheckStatus";                        // (int FIX.5.0SP2)
dictionary[tag::NoEntitlementTypes]                = "NoEntitlementTypes";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegMidPx]                          = "LegMidPx";                                        // (Price FIX.5.0SP2)
dictionary[tag::RegulatoryTransactionType]         = "RegulatoryTransactionType";                       // (int FIX.5.0SP2)
dictionary[tag::LegAssetGroup]                     = "LegAssetGroup";                                   // (int FIX.5.0SP2)
dictionary[tag::PricePrecision]                    = "PricePrecision";                                  // (int FIX.5.0SP2)
dictionary[tag::CollateralPortfolioID]             = "CollateralPortfolioID";                           // (String FIX.5.0SP2)
dictionary[tag::EncodedComplianceTextLen]          = "EncodedComplianceTextLen";                        // (Length FIX.5.0SP2)
dictionary[tag::EncodedComplianceText]             = "EncodedComplianceText";                           // (data FIX.5.0SP2)
dictionary[tag::TradingUnitPeriodMultiplier]       = "TradingUnitPeriodMultiplier";                     // (int FIX.5.0SP2)
dictionary[tag::LegTradingUnitPeriodMultiplier]    = "LegTradingUnitPeriodMultiplier";                  // (int FIX.5.0SP2)
dictionary[tag::PartyRiskLimitStatus]              = "PartyRiskLimitStatus";                            // (int FIX.5.0SP2)
dictionary[tag::RemunerationIndicator]             = "RemunerationIndicator";                           // (int FIX.5.0SP2)
dictionary[tag::LegTotalTradeQty]                  = "LegTotalTradeQty";                                // (Qty FIX.5.0SP2)
dictionary[tag::LegLastMultipliedQty]              = "LegLastMultipliedQty";                            // (Qty FIX.5.0SP2)
dictionary[tag::LegTotalGrossTradeAmt]             = "LegTotalGrossTradeAmt";                           // (Amt FIX.5.0SP2)
dictionary[tag::LegTotalTradeMultipliedQty]        = "LegTotalTradeMultipliedQty";                      // (Qty FIX.5.0SP2)
dictionary[tag::CompressionGroupID]                = "CompressionGroupID";                              // (String FIX.5.0SP2)
dictionary[tag::SelfMatchPreventionID]             = "SelfMatchPreventionID";                           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingTradingUnitPeriodMultiplier] = "UnderlyingTradingUnitPeriodMultiplier";           // (int FIX.5.0SP2)
dictionary[tag::PosReportAction]                   = "PosReportAction";                                 // (int FIX.5.0SP2)
dictionary[tag::SettlForwardPoints]                = "SettlForwardPoints";                              // (PriceOffset FIX.5.0SP2)
dictionary[tag::SettlPriceFxRateCalc]              = "SettlPriceFxRateCalc";                            // (char FIX.5.0SP2)
dictionary[tag::TotalTradeQty]                     = "TotalTradeQty";                                   // (Qty FIX.5.0SP2)
dictionary[tag::LastMultipliedQty]                 = "LastMultipliedQty";                               // (Qty FIX.5.0SP2)
dictionary[tag::TotalGrossTradeAmt]                = "TotalGrossTradeAmt";                              // (Amt FIX.5.0SP2)
dictionary[tag::TotalTradeMultipliedQty]           = "TotalTradeMultipliedQty";                         // (Qty FIX.5.0SP2)
dictionary[tag::EncodedTradeContinuationText]      = "EncodedTradeContinuationText";                    // (data FIX.5.0SP2)
dictionary[tag::EncodedTradeContinuationTextLen]   = "EncodedTradeContinuationTextLen";                 // (Length FIX.5.0SP2)
dictionary[tag::IntraFirmTradeIndicator]           = "IntraFirmTradeIndicator";                         // (Boolean FIX.5.0SP2)
dictionary[tag::TradeContinuationText]             = "TradeContinuationText";                           // (String FIX.5.0SP2)
dictionary[tag::TaxonomyType]                      = "TaxonomyType";                                    // (char FIX.5.0SP2)
dictionary[tag::PartyRoleQualifier]                = "PartyRoleQualifier";                              // (int FIX.5.0SP2)
dictionary[tag::DerivativeInstrumentPartyRoleQualifier] = "DerivativeInstrumentPartyRoleQualifier";          // (int FIX.5.0SP2)
dictionary[tag::InstrumentPartyRoleQualifier]      = "InstrumentPartyRoleQualifier";                    // (int FIX.5.0SP2)
dictionary[tag::LegInstrumentPartyRoleQualifier]   = "LegInstrumentPartyRoleQualifier";                 // (int FIX.5.0SP2)
dictionary[tag::LegProvisionPartyRoleQualifier]    = "LegProvisionPartyRoleQualifier";                  // (int FIX.5.0SP2)
dictionary[tag::Nested2PartyRoleQualifier]         = "Nested2PartyRoleQualifier";                       // (int FIX.5.0SP2)
dictionary[tag::Nested3PartyRoleQualifier]         = "Nested3PartyRoleQualifier";                       // (int FIX.5.0SP2)
dictionary[tag::Nested4PartyRoleQualifier]         = "Nested4PartyRoleQualifier";                       // (int FIX.5.0SP2)
dictionary[tag::NestedPartyRoleQualifier]          = "NestedPartyRoleQualifier";                        // (int FIX.5.0SP2)
dictionary[tag::ProvisionPartyRoleQualifier]       = "ProvisionPartyRoleQualifier";                     // (int FIX.5.0SP2)
dictionary[tag::RequestedPartyRoleQualifier]       = "RequestedPartyRoleQualifier";                     // (int FIX.5.0SP2)
dictionary[tag::TradeContingency]                  = "TradeContingency";                                // (int FIX.5.0SP2)
dictionary[tag::RootPartyRoleQualifier]            = "RootPartyRoleQualifier";                          // (int FIX.5.0SP2)
dictionary[tag::SettlPartyRoleQualifier]           = "SettlPartyRoleQualifier";                         // (int FIX.5.0SP2)
dictionary[tag::TradeConfirmationReferenceID]      = "TradeConfirmationReferenceID";                    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingInstrumentPartyRoleQualifier] = "UnderlyingInstrumentPartyRoleQualifier";          // (int FIX.5.0SP2)
dictionary[tag::AllocRefRiskLimitCheckID]          = "AllocRefRiskLimitCheckID";                        // (String FIX.5.0SP2)
dictionary[tag::AllocRefRiskLimitCheckIDType]      = "AllocRefRiskLimitCheckIDType";                    // (int FIX.5.0SP2)
dictionary[tag::LimitUtilizationAmt]               = "LimitUtilizationAmt";                             // (Amt FIX.5.0SP2)
dictionary[tag::LimitAmt]                          = "LimitAmt";                                        // (Amt FIX.5.0SP2)
dictionary[tag::LimitRole]                         = "LimitRole";                                       // (int FIX.5.0SP2)
dictionary[tag::RegulatoryTradeIDScope]            = "RegulatoryTradeIDScope";                          // (int FIX.5.0SP2)
dictionary[tag::SideRegulatoryTradeIDScope]        = "SideRegulatoryTradeIDScope";                      // (int FIX.5.0SP2)
dictionary[tag::AllocRegulatoryTradeIDScope]       = "AllocRegulatoryTradeIDScope";                     // (int FIX.5.0SP2)
dictionary[tag::EffectiveBusinessDate]             = "EffectiveBusinessDate";                           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ListManualOrderIndicator]          = "ListManualOrderIndicator";                        // (Boolean FIX.5.0SP2)
dictionary[tag::EntitlementSubType]                = "EntitlementSubType";                              // (int FIX.5.0SP2)
dictionary[tag::QuoteModelType]                    = "QuoteModelType";                                  // (int FIX.5.0SP2)
dictionary[tag::ComplianceText]                    = "ComplianceText";                                  // (String FIX.5.0SP2)
dictionary[tag::ExecMethod]                        = "ExecMethod";                                      // (int FIX.5.0SP2)
dictionary[tag::AllocRegulatoryLegRefID]           = "AllocRegulatoryLegRefID";                         // (String FIX.5.0SP2)
dictionary[tag::ComplexEventSpotRate]              = "ComplexEventSpotRate";                            // (Price FIX.5.0SP2)
dictionary[tag::ComplexEventForwardPoints]         = "ComplexEventForwardPoints";                       // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegComplexEventSpotRate]           = "LegComplexEventSpotRate";                         // (Price FIX.5.0SP2)
dictionary[tag::LegComplexEventForwardPoints]      = "LegComplexEventForwardPoints";                    // (PriceOffset FIX.5.0SP2)
dictionary[tag::RegulatoryLegRefID]                = "RegulatoryLegRefID";                              // (String FIX.5.0SP2)
dictionary[tag::RateSourceReferemcePageHeading]    = "RateSourceReferemcePageHeading";                  // (String FIX.5.0SP2)
dictionary[tag::RelatedToSecurityID]               = "RelatedToSecurityID";                             // (String FIX.5.0SP2)
dictionary[tag::RelatedToSecurityIDSource]         = "RelatedToSecurityIDSource";                       // (String FIX.5.0SP2)
dictionary[tag::RelatedToStreamXIDRef]             = "RelatedToStreamXIDRef";                           // (XIDREF FIX.5.0SP2)
dictionary[tag::SideRegulatoryLegRefID]            = "SideRegulatoryLegRefID";                          // (String FIX.5.0SP2)
dictionary[tag::RelatedToDividendPeriodXIDRef]     = "RelatedToDividendPeriodXIDRef";                   // (XIDREF FIX.5.0SP2)
dictionary[tag::FirmTradeEventID]                  = "FirmTradeEventID";                                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventSpotRate]    = "UnderlyingComplexEventSpotRate";                  // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventForwardPoints] = "UnderlyingComplexEventForwardPoints";             // (PriceOffset FIX.5.0SP2)
dictionary[tag::FillRefID]                         = "FillRefID";                                       // (String FIX.5.0SP2)
dictionary[tag::OrderRequestID]                    = "OrderRequestID";                                  // (int FIX.5.0SP2)
dictionary[tag::MassOrderRequestID]                = "MassOrderRequestID";                              // (String FIX.5.0SP2)
dictionary[tag::MassOrderReportID]                 = "MassOrderReportID";                               // (String FIX.5.0SP2)
dictionary[tag::MassOrderRequestStatus]            = "MassOrderRequestStatus";                          // (int FIX.5.0SP2)
dictionary[tag::MassOrderRequestResult]            = "MassOrderRequestResult";                          // (int FIX.5.0SP2)
dictionary[tag::OrderResponseLevel]                = "OrderResponseLevel";                              // (int FIX.5.0SP2)
dictionary[tag::NoOrderEntries]                    = "NoOrderEntries";                                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::OrderEntryAction]                  = "OrderEntryAction";                                // (char FIX.5.0SP2)
dictionary[tag::OrderEntryID]                      = "OrderEntryID";                                    // (int FIX.5.0SP2)
dictionary[tag::ExecTypeReason]                    = "ExecTypeReason";                                  // (int FIX.5.0SP2)
dictionary[tag::TotNoOrderEntries]                 = "TotNoOrderEntries";                               // (int FIX.5.0SP2)
dictionary[tag::NoTargetPartySubIDs]               = "NoTargetPartySubIDs";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::TargetPartySubID]                  = "TargetPartySubID";                                // (String FIX.5.0SP2)
dictionary[tag::TargetPartySubIDType]              = "TargetPartySubIDType";                            // (int FIX.5.0SP2)
dictionary[tag::TransferInstructionID]             = "TransferInstructionID";                           // (String FIX.5.0SP2)
dictionary[tag::TransferID]                        = "TransferID";                                      // (String FIX.5.0SP2)
dictionary[tag::TransferReportID]                  = "TransferReportID";                                // (String FIX.5.0SP2)
dictionary[tag::TransferTransType]                 = "TransferTransType";                               // (int FIX.5.0SP2)
dictionary[tag::TransferType]                      = "TransferType";                                    // (int FIX.5.0SP2)
dictionary[tag::TransferScope]                     = "TransferScope";                                   // (int FIX.5.0SP2)
dictionary[tag::TransferStatus]                    = "TransferStatus";                                  // (int FIX.5.0SP2)
dictionary[tag::TransferRejectReason]              = "TransferRejectReason";                            // (int FIX.5.0SP2)
dictionary[tag::TransferReportType]                = "TransferReportType";                              // (int FIX.5.0SP2)
dictionary[tag::AggressorTime]                     = "AggressorTime";                                   // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::AggressorSide]                     = "AggressorSide";                                   // (char FIX.5.0SP2)
dictionary[tag::FastMarketIndicator]               = "FastMarketIndicator";                             // (Boolean FIX.5.0SP2)
dictionary[tag::LinkageHandlingIndicator]          = "LinkageHandlingIndicator";                        // (Boolean FIX.5.0SP2)
dictionary[tag::NumberOfBuyOrders]                 = "NumberOfBuyOrders";                               // (int FIX.5.0SP2)
dictionary[tag::NumberOfSellOrders]                = "NumberOfSellOrders";                              // (int FIX.5.0SP2)
dictionary[tag::SettlPriceDeterminationMethod]     = "SettlPriceDeterminationMethod";                   // (int FIX.5.0SP2)
dictionary[tag::MDStatisticReqID]                  = "MDStatisticReqID";                                // (String FIX.5.0SP2)
dictionary[tag::MDStatisticRptID]                  = "MDStatisticRptID";                                // (String FIX.5.0SP2)
dictionary[tag::MDStatisticName]                   = "MDStatisticName";                                 // (String FIX.5.0SP2)
dictionary[tag::MDStatisticDesc]                   = "MDStatisticDesc";                                 // (String FIX.5.0SP2)
dictionary[tag::MDStatisticType]                   = "MDStatisticType";                                 // (int FIX.5.0SP2)
dictionary[tag::MDStatisticScope]                  = "MDStatisticScope";                                // (int FIX.5.0SP2)
dictionary[tag::MDStatisticSubScope]               = "MDStatisticSubScope";                             // (int FIX.5.0SP2)
dictionary[tag::MDStatisticScopeType]              = "MDStatisticScopeType";                            // (int FIX.5.0SP2)
dictionary[tag::MDStatisticFrequencyPeriod]        = "MDStatisticFrequencyPeriod";                      // (int FIX.5.0SP2)
dictionary[tag::MDStatisticFrequencyUnit]          = "MDStatisticFrequencyUnit";                        // (int FIX.5.0SP2)
dictionary[tag::MDStatisticDelayPeriod]            = "MDStatisticDelayPeriod";                          // (int FIX.5.0SP2)
dictionary[tag::MDStatisticDelayUnit]              = "MDStatisticDelayUnit";                            // (int FIX.5.0SP2)
dictionary[tag::MDStatisticIntervalType]           = "MDStatisticIntervalType";                         // (int FIX.5.0SP2)
dictionary[tag::MDStatisticIntervalTypeUnit]       = "MDStatisticIntervalTypeUnit";                     // (String FIX.5.0SP2)
dictionary[tag::MDStatisticIntervalPeriod]         = "MDStatisticIntervalPeriod";                       // (int FIX.5.0SP2)
dictionary[tag::MDStatisticIntervalUnit]           = "MDStatisticIntervalUnit";                         // (int FIX.5.0SP2)
dictionary[tag::MDStatisticStartDate]              = "MDStatisticStartDate";                            // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::MDStatisticEndDate]                = "MDStatisticEndDate";                              // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::MDStatisticStartTime]              = "MDStatisticStartTime";                            // (UTCTimeOnly FIX.5.0SP2)
dictionary[tag::MDStatisticEndTime]                = "MDStatisticEndTime";                              // (UTCTimeOnly FIX.5.0SP2)
dictionary[tag::MDStatisticRatioType]              = "MDStatisticRatioType";                            // (int FIX.5.0SP2)
dictionary[tag::MDStatisticRequestResult]          = "MDStatisticRequestResult";                        // (int FIX.5.0SP2)
dictionary[tag::NoMDStatistics]                    = "NoMDStatistics";                                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::MDStatisticID]                     = "MDStatisticID";                                   // (String FIX.5.0SP2)
dictionary[tag::MDStatisticTime]                   = "MDStatisticTime";                                 // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::MDStatisticStatus]                 = "MDStatisticStatus";                               // (int FIX.5.0SP2)
dictionary[tag::MDStatisticValue]                  = "MDStatisticValue";                                // (float FIX.5.0SP2)
dictionary[tag::MDStatisticValueType]              = "MDStatisticValueType";                            // (int FIX.5.0SP2)
dictionary[tag::MDStatisticValueUnit]              = "MDStatisticValueUnit";                            // (int FIX.5.0SP2)
dictionary[tag::EncodedMDStatisticDescLen]         = "EncodedMDStatisticDescLen";                       // (Length FIX.5.0SP2)
dictionary[tag::EncodedMDStatisticDesc]            = "EncodedMDStatisticDesc";                          // (data FIX.5.0SP2)
dictionary[tag::AllocRiskLimitCheckStatus]         = "AllocRiskLimitCheckStatus";                       // (int FIX.5.0SP2)
dictionary[tag::FirmTransactionID]                 = "FirmTransactionID";                               // (String FIX.5.0SP2)
dictionary[tag::TransactionID]                     = "TransactionID";                                   // (String FIX.5.0SP2)
dictionary[tag::WireReference]                     = "WireReference";                                   // (String FIX.5.0SP2)
dictionary[tag::CollRptRejectReason]               = "CollRptRejectReason";                             // (int FIX.5.0SP2)
dictionary[tag::CollRptStatus]                     = "CollRptStatus";                                   // (int FIX.5.0SP2)
dictionary[tag::PackageID]                         = "PackageID";                                       // (String FIX.5.0SP2)
dictionary[tag::TradeNumber]                       = "TradeNumber";                                     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingAssetGroup]              = "UnderlyingAssetGroup";                            // (int FIX.5.0SP2)
dictionary[tag::LegDifferentialPrice]              = "LegDifferentialPrice";                            // (PriceOffset FIX.5.0SP2)
dictionary[tag::EncodedLegDocumentationText]       = "EncodedLegDocumentationText";                     // (data FIX.5.0SP2)
dictionary[tag::EncodedLegDocumentationTextLen]    = "EncodedLegDocumentationTextLen";                  // (Length FIX.5.0SP2)
dictionary[tag::LegAgreementCurrency]              = "LegAgreementCurrency";                            // (Currency FIX.5.0SP2)
dictionary[tag::LegAgreementDate]                  = "LegAgreementDate";                                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegAgreementDesc]                  = "LegAgreementDesc";                                // (String FIX.5.0SP2)
dictionary[tag::LegAgreementID]                    = "LegAgreementID";                                  // (String FIX.5.0SP2)
dictionary[tag::LegAgreementVersion]               = "LegAgreementVersion";                             // (String FIX.5.0SP2)
dictionary[tag::LegBrokerConfirmationDesc]         = "LegBrokerConfirmationDesc";                       // (String FIX.5.0SP2)
dictionary[tag::LegCreditSupportAgreementDate]     = "LegCreditSupportAgreementDate";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegCreditSupportAgreementDesc]     = "LegCreditSupportAgreementDesc";                   // (String FIX.5.0SP2)
dictionary[tag::LegCreditSupportAgreementID]       = "LegCreditSupportAgreementID";                     // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryType]                   = "LegDeliveryType";                                 // (int FIX.5.0SP2)
dictionary[tag::LegDocumentationText]              = "LegDocumentationText";                            // (String FIX.5.0SP2)
dictionary[tag::LegEndDate]                        = "LegEndDate";                                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegGoverningLaw]                   = "LegGoverningLaw";                                 // (String FIX.5.0SP2)
dictionary[tag::LegMarginRatio]                    = "LegMarginRatio";                                  // (Percentage FIX.5.0SP2)
dictionary[tag::LegMasterConfirmationAnnexDate]    = "LegMasterConfirmationAnnexDate";                  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegMasterConfirmationDate]         = "LegMasterConfirmationDate";                       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegMasterConfirmationDesc]         = "LegMasterConfirmationDesc";                       // (String FIX.5.0SP2)
dictionary[tag::LegMasterConfirmationAnnexDesc]    = "LegMasterConfirmationAnnexDesc";                  // (String FIX.5.0SP2)
dictionary[tag::LegStartDate]                      = "LegStartDate";                                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegTerminationType]                = "LegTerminationType";                              // (int FIX.5.0SP2)
dictionary[tag::AllocCalculatedCcyQty]             = "AllocCalculatedCcyQty";                           // (Qty FIX.5.0SP2)
dictionary[tag::CollateralRequestInstruction]      = "CollateralRequestInstruction";                    // (String FIX.5.0SP2)
dictionary[tag::CollateralRequestLinkID]           = "CollateralRequestLinkID";                         // (String FIX.5.0SP2)
dictionary[tag::CollateralRequestNumber]           = "CollateralRequestNumber";                         // (int FIX.5.0SP2)
dictionary[tag::TotNumCollateralRequests]          = "TotNumCollateralRequests";                        // (int FIX.5.0SP2)
dictionary[tag::WarningText]                       = "WarningText";                                     // (String FIX.5.0SP2)
dictionary[tag::EncodedWarningText]                = "EncodedWarningText";                              // (data FIX.5.0SP2)
dictionary[tag::EncodedWarningTextLen]             = "EncodedWarningTextLen";                           // (Length FIX.5.0SP2)
dictionary[tag::CrossedIndicator]                  = "CrossedIndicator";                                // (int FIX.5.0SP2)
dictionary[tag::TradeReportingIndicator]           = "TradeReportingIndicator";                         // (int FIX.5.0SP2)
dictionary[tag::AffiliatedFirmsTradeIndicator]     = "AffiliatedFirmsTradeIndicator";                   // (Boolean FIX.5.0SP2)
dictionary[tag::InternationalSwapIndicator]        = "InternationalSwapIndicator";                      // (Boolean FIX.5.0SP2)
dictionary[tag::MultiAssetSwapIndicator]           = "MultiAssetSwapIndicator";                         // (Boolean FIX.5.0SP2)
dictionary[tag::ClearingSettlPrice]                = "ClearingSettlPrice";                              // (Price FIX.5.0SP2)
dictionary[tag::NoRelativeValues]                  = "NoRelativeValues";                                // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelativeValueType]                 = "RelativeValueType";                               // (int FIX.5.0SP2)
dictionary[tag::RelativeValue]                     = "RelativeValue";                                   // (float FIX.5.0SP2)
dictionary[tag::RelativeValueSide]                 = "RelativeValueSide";                               // (int FIX.5.0SP2)
dictionary[tag::BidSpread]                         = "BidSpread";                                       // (float FIX.5.0SP2)
dictionary[tag::OfferSpread]                       = "OfferSpread";                                     // (float FIX.5.0SP2)
dictionary[tag::MDReportEvent]                     = "MDReportEvent";                                   // (int FIX.5.0SP2)
dictionary[tag::MDReportCount]                     = "MDReportCount";                                   // (int FIX.5.0SP2)
dictionary[tag::TotNoMarketSegmentReports]         = "TotNoMarketSegmentReports";                       // (int FIX.5.0SP2)
dictionary[tag::TotNoInstrumentReports]            = "TotNoInstrumentReports";                          // (int FIX.5.0SP2)
dictionary[tag::TotNoPartyDetailReports]           = "TotNoPartyDetailReports";                         // (int FIX.5.0SP2)
dictionary[tag::TotNoEntitlementReports]           = "TotNoEntitlementReports";                         // (int FIX.5.0SP2)
dictionary[tag::TotNoRiskLimitReports]             = "TotNoRiskLimitReports";                           // (int FIX.5.0SP2)
dictionary[tag::MarketSegmentStatus]               = "MarketSegmentStatus";                             // (int FIX.5.0SP2)
dictionary[tag::MarketSegmentType]                 = "MarketSegmentType";                               // (int FIX.5.0SP2)
dictionary[tag::MarketSegmentSubType]              = "MarketSegmentSubType";                            // (int FIX.5.0SP2)
dictionary[tag::NoRelatedMarketSegments]           = "NoRelatedMarketSegments";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::RelatedMarketSegmentID]            = "RelatedMarketSegmentID";                          // (String FIX.5.0SP2)
dictionary[tag::MarketSegmentRelationship]         = "MarketSegmentRelationship";                       // (int FIX.5.0SP2)
dictionary[tag::NoAuctionTypeRules]                = "NoAuctionTypeRules";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::AuctionTypeProductComplex]         = "AuctionTypeProductComplex";                       // (String FIX.5.0SP2)
dictionary[tag::NoPriceRangeRules]                 = "NoPriceRangeRules";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::StartPriceRange]                   = "StartPriceRange";                                 // (Price FIX.5.0SP2)
dictionary[tag::EndPriceRange]                     = "EndPriceRange";                                   // (Price FIX.5.0SP2)
dictionary[tag::PriceRangeValue]                   = "PriceRangeValue";                                 // (Price FIX.5.0SP2)
dictionary[tag::PriceRangePercentage]              = "PriceRangePercentage";                            // (Percentage FIX.5.0SP2)
dictionary[tag::PriceRangeProductComplex]          = "PriceRangeProductComplex";                        // (String FIX.5.0SP2)
dictionary[tag::PriceRangeRuleID]                  = "PriceRangeRuleID";                                // (String FIX.5.0SP2)
dictionary[tag::FastMarketPercentage]              = "FastMarketPercentage";                            // (Percentage FIX.5.0SP2)
dictionary[tag::NoQuoteSizeRules]                  = "NoQuoteSizeRules";                                // (NumInGroup FIX.5.0SP2)
dictionary[tag::QuoteSideIndicator]                = "QuoteSideIndicator";                              // (Boolean FIX.5.0SP2)
dictionary[tag::NoFlexProductEligibilities]        = "NoFlexProductEligibilities";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::FlexProductEligibilityComplex]     = "FlexProductEligibilityComplex";                   // (String FIX.5.0SP2)
dictionary[tag::NumOfComplexInstruments]           = "NumOfComplexInstruments";                         // (int FIX.5.0SP2)
dictionary[tag::MarketDepthTimeInterval]           = "MarketDepthTimeInterval";                         // (int FIX.5.0SP2)
dictionary[tag::MarketDepthTimeIntervalUnit]       = "MarketDepthTimeIntervalUnit";                     // (int FIX.5.0SP2)
dictionary[tag::MDRecoveryTimeInterval]            = "MDRecoveryTimeInterval";                          // (int FIX.5.0SP2)
dictionary[tag::MDRecoveryTimeIntervalUnit]        = "MDRecoveryTimeIntervalUnit";                      // (int FIX.5.0SP2)
dictionary[tag::PrimaryServiceLocationID]          = "PrimaryServiceLocationID";                        // (String FIX.5.0SP2)
dictionary[tag::SecondaryServiceLocationID]        = "SecondaryServiceLocationID";                      // (String FIX.5.0SP2)
dictionary[tag::MatchRuleProductComplex]           = "MatchRuleProductComplex";                         // (String FIX.5.0SP2)
dictionary[tag::CustomerPriority]                  = "CustomerPriority";                                // (int FIX.5.0SP2)
dictionary[tag::TickRuleProductComplex]            = "TickRuleProductComplex";                          // (String FIX.5.0SP2)
dictionary[tag::PreviousAdjustedOpenInterest]      = "PreviousAdjustedOpenInterest";                    // (Amt FIX.5.0SP2)
dictionary[tag::PreviousUnadjustedOpenInterest]    = "PreviousUnadjustedOpenInterest";                  // (Amt FIX.5.0SP2)
dictionary[tag::LowExercisePriceOptionIndicator]   = "LowExercisePriceOptionIndicator";                 // (Boolean FIX.5.0SP2)
dictionary[tag::BlockTradeEligibilityIndicator]    = "BlockTradeEligibilityIndicator";                  // (Boolean FIX.5.0SP2)
dictionary[tag::InstrumentPricePrecision]          = "InstrumentPricePrecision";                        // (int FIX.5.0SP2)
dictionary[tag::StrikePricePrecision]              = "StrikePricePrecision";                            // (int FIX.5.0SP2)
dictionary[tag::OrigStrikePrice]                   = "OrigStrikePrice";                                 // (Price FIX.5.0SP2)
dictionary[tag::SettlSubMethod]                    = "SettlSubMethod";                                  // (int FIX.5.0SP2)
dictionary[tag::NoClearingPriceParameters]         = "NoClearingPriceParameters";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::BusinessDayType]                   = "BusinessDayType";                                 // (int FIX.5.0SP2)
dictionary[tag::ClearingPriceOffset]               = "ClearingPriceOffset";                             // (PriceOffset FIX.5.0SP2)
dictionary[tag::VegaMultiplier]                    = "VegaMultiplier";                                  // (float FIX.5.0SP2)
dictionary[tag::AnnualTradingBusinessDays]         = "AnnualTradingBusinessDays";                       // (int FIX.5.0SP2)
dictionary[tag::TotalTradingBusinessDays]          = "TotalTradingBusinessDays";                        // (int FIX.5.0SP2)
dictionary[tag::TradingBusinessDays]               = "TradingBusinessDays";                             // (int FIX.5.0SP2)
dictionary[tag::RealizedVariance]                  = "RealizedVariance";                                // (float FIX.5.0SP2)
dictionary[tag::StandardVariance]                  = "StandardVariance";                                // (float FIX.5.0SP2)
dictionary[tag::RelatedClosePrice]                 = "RelatedClosePrice";                               // (Price FIX.5.0SP2)
dictionary[tag::OvernightInterestRate]             = "OvernightInterestRate";                           // (float FIX.5.0SP2)
dictionary[tag::AccumulatedReturnModifiedVariationMargin] = "AccumulatedReturnModifiedVariationMargin";        // (float FIX.5.0SP2)
dictionary[tag::CalculationMethod]                 = "CalculationMethod";                               // (int FIX.5.0SP2)
dictionary[tag::NoOrderAttributes]                 = "NoOrderAttributes";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::OrderAttributeType]                = "OrderAttributeType";                              // (int FIX.5.0SP2)
dictionary[tag::OrderAttributeValue]               = "OrderAttributeValue";                             // (String FIX.5.0SP2)
dictionary[tag::DeltaCrossed]                      = "DeltaCrossed";                                    // (Boolean FIX.5.0SP2)
dictionary[tag::ComplexEventFuturesPriceValuation] = "ComplexEventFuturesPriceValuation";               // (Boolean FIX.5.0SP2)
dictionary[tag::ComplexEventOptionsPriceValuation] = "ComplexEventOptionsPriceValuation";               // (Boolean FIX.5.0SP2)
dictionary[tag::ComplexEventPVFinalPriceElectionFallback] = "ComplexEventPVFinalPriceElectionFallback";        // (int FIX.5.0SP2)
dictionary[tag::StrikeIndexCurvePoint]             = "StrikeIndexCurvePoint";                           // (String FIX.5.0SP2)
dictionary[tag::StrikeIndexQuote]                  = "StrikeIndexQuote";                                // (int FIX.5.0SP2)
dictionary[tag::ExtraordinaryEventAdjustmentMethod] = "ExtraordinaryEventAdjustmentMethod";              // (int FIX.5.0SP2)
dictionary[tag::ExchangeLookAlike]                 = "ExchangeLookAlike";                               // (Boolean FIX.5.0SP2)
dictionary[tag::LegStrikeIndexCurvePoint]          = "LegStrikeIndexCurvePoint";                        // (String FIX.5.0SP2)
dictionary[tag::LegStrikeIndexQuote]               = "LegStrikeIndexQuote";                             // (int FIX.5.0SP2)
dictionary[tag::LegExtraordinaryEventAdjustmentMethod] = "LegExtraordinaryEventAdjustmentMethod";           // (int FIX.5.0SP2)
dictionary[tag::LegExchangeLookAlike]              = "LegExchangeLookAlike";                            // (Boolean FIX.5.0SP2)
dictionary[tag::LegComplexEventFuturesPriceValuation] = "LegComplexEventFuturesPriceValuation";            // (Boolean FIX.5.0SP2)
dictionary[tag::LegComplexEventOptionsPriceValuation] = "LegComplexEventOptionsPriceValuation";            // (Boolean FIX.5.0SP2)
dictionary[tag::LegComplexEventPVFinalPriceElectionFallback] = "LegComplexEventPVFinalPriceElectionFallback";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventFuturesPriceValuation] = "UnderlyingComplexEventFuturesPriceValuation";     // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventOptionsPriceValuation] = "UnderlyingComplexEventOptionsPriceValuation";     // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPVFinalPriceElectionFallback] = "UnderlyingComplexEventPVFinalPriceElectionFallback"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingNotional]                = "UnderlyingNotional";                              // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingNotionalCurrency]        = "UnderlyingNotionalCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingNotionalDeterminationMethod] = "UnderlyingNotionalDeterminationMethod";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingNotionalAdjustments]     = "UnderlyingNotionalAdjustments";                   // (int FIX.5.0SP2)
dictionary[tag::PositionID]                        = "PositionID";                                      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingNotionalXIDRef]          = "UnderlyingNotionalXIDRef";                        // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingFutureID]                = "UnderlyingFutureID";                              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingFutureIDSource]          = "UnderlyingFutureIDSource";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStrikeIndexCurvePoint]   = "UnderlyingStrikeIndexCurvePoint";                 // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStrikeIndexQuote]        = "UnderlyingStrikeIndexQuote";                      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingExtraordinaryEventAdjustmentMethod] = "UnderlyingExtraordinaryEventAdjustmentMethod";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingExchangeLookAlike]       = "UnderlyingExchangeLookAlike";                     // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingAverageVolumeLimitationPercentage] = "UnderlyingAverageVolumeLimitationPercentage";     // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingAverageVolumeLimitationPeriodDays] = "UnderlyingAverageVolumeLimitationPeriodDays";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDepositoryReceiptIndicator] = "UnderlyingDepositoryReceiptIndicator";            // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingOpenUnits]               = "UnderlyingOpenUnits";                             // (Qty FIX.5.0SP2)
dictionary[tag::UnderlyingBasketDivisor]           = "UnderlyingBasketDivisor";                         // (float FIX.5.0SP2)
dictionary[tag::UnderlyingInstrumentXID]           = "UnderlyingInstrumentXID";                         // (XID FIX.5.0SP2)
dictionary[tag::CollateralAmountType]              = "CollateralAmountType";                            // (int FIX.5.0SP2)
dictionary[tag::NoMiscFeeSubTypes]                 = "NoMiscFeeSubTypes";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::MiscFeeSubType]                    = "MiscFeeSubType";                                  // (String FIX.5.0SP2)
dictionary[tag::MiscFeeSubTypeAmt]                 = "MiscFeeSubTypeAmt";                               // (Amt FIX.5.0SP2)
dictionary[tag::MiscFeeSubTypeDesc]                = "MiscFeeSubTypeDesc";                              // (String FIX.5.0SP2)
dictionary[tag::EncodedMiscFeeSubTypeDescLen]      = "EncodedMiscFeeSubTypeDescLen";                    // (Length FIX.5.0SP2)
dictionary[tag::EncodedMiscFeeSubTypeDesc]         = "EncodedMiscFeeSubTypeDesc";                       // (data FIX.5.0SP2)
dictionary[tag::NoCommissions]                     = "NoCommissions";                                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::CommissionAmount]                  = "CommissionAmount";                                // (Amt FIX.5.0SP2)
dictionary[tag::CommissionAmountType]              = "CommissionAmountType";                            // (int FIX.5.0SP2)
dictionary[tag::CommissionBasis]                   = "CommissionBasis";                                 // (char FIX.5.0SP2)
dictionary[tag::CommissionCurrency]                = "CommissionCurrency";                              // (Currency FIX.5.0SP2)
dictionary[tag::CommissionUnitOfMeasure]           = "CommissionUnitOfMeasure";                         // (String FIX.5.0SP2)
dictionary[tag::CommissionUnitOfMeasureCurrency]   = "CommissionUnitOfMeasureCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::CommissionRate]                    = "CommissionRate";                                  // (float FIX.5.0SP2)
dictionary[tag::CommissionSharedIndicator]         = "CommissionSharedIndicator";                       // (Boolean FIX.5.0SP2)
dictionary[tag::CommissionAmountShared]            = "CommissionAmountShared";                          // (Amt FIX.5.0SP2)
dictionary[tag::CommissionLegRefID]                = "CommissionLegRefID";                              // (String FIX.5.0SP2)
dictionary[tag::CommissionDesc]                    = "CommissionDesc";                                  // (String FIX.5.0SP2)
dictionary[tag::EncodedCommissionDescLen]          = "EncodedCommissionDescLen";                        // (Length FIX.5.0SP2)
dictionary[tag::EncodedCommissionDesc]             = "EncodedCommissionDesc";                           // (data FIX.5.0SP2)
dictionary[tag::NoAllocCommissions]                = "NoAllocCommissions";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::AllocCommissionAmount]             = "AllocCommissionAmount";                           // (Amt FIX.5.0SP2)
dictionary[tag::AllocCommissionAmountType]         = "AllocCommissionAmountType";                       // (int FIX.5.0SP2)
dictionary[tag::AllocCommissionBasis]              = "AllocCommissionBasis";                            // (char FIX.5.0SP2)
dictionary[tag::AllocCommissionCurrency]           = "AllocCommissionCurrency";                         // (Currency FIX.5.0SP2)
dictionary[tag::AllocCommissionUnitOfMeasure]      = "AllocCommissionUnitOfMeasure";                    // (String FIX.5.0SP2)
dictionary[tag::AllocCommissionUnitOfMeasureCurrency] = "AllocCommissionUnitOfMeasureCurrency";            // (Currency FIX.5.0SP2)
dictionary[tag::AllocCommissionRate]               = "AllocCommissionRate";                             // (float FIX.5.0SP2)
dictionary[tag::AllocCommissionSharedIndicator]    = "AllocCommissionSharedIndicator";                  // (Boolean FIX.5.0SP2)
dictionary[tag::AllocCommissionAmountShared]       = "AllocCommissionAmountShared";                     // (Amt FIX.5.0SP2)
dictionary[tag::AllocCommissionLegRefID]           = "AllocCommissionLegRefID";                         // (String FIX.5.0SP2)
dictionary[tag::AllocCommissionDesc]               = "AllocCommissionDesc";                             // (String FIX.5.0SP2)
dictionary[tag::EncodedAllocCommissionDescLen]     = "EncodedAllocCommissionDescLen";                   // (Length FIX.5.0SP2)
dictionary[tag::EncodedAllocCommissionDesc]        = "EncodedAllocCommissionDesc";                      // (data FIX.5.0SP2)
dictionary[tag::AlgorithmicTradeIndicator]         = "AlgorithmicTradeIndicator";                       // (int FIX.5.0SP2)
dictionary[tag::NoTrdRegPublications]              = "NoTrdRegPublications";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::TrdRegPublicationType]             = "TrdRegPublicationType";                           // (int FIX.5.0SP2)
dictionary[tag::TrdRegPublicationReason]           = "TrdRegPublicationReason";                         // (int FIX.5.0SP2)
dictionary[tag::SideTradeReportingIndicator]       = "SideTradeReportingIndicator";                     // (int FIX.5.0SP2)
dictionary[tag::CrossRequestID]                    = "CrossRequestID";                                  // (String FIX.5.0SP2)
dictionary[tag::FillMatchID]                       = "FillMatchID";                                     // (String FIX.5.0SP2)
dictionary[tag::FillMatchSubID]                    = "FillMatchSubID";                                  // (String FIX.5.0SP2)
dictionary[tag::MassActionReason]                  = "MassActionReason";                                // (int FIX.5.0SP2)
dictionary[tag::MaximumPricePercentage]            = "MaximumPricePercentage";                          // (Percentage FIX.5.0SP2)
dictionary[tag::NotAffectedReason]                 = "NotAffectedReason";                               // (int FIX.5.0SP2)
dictionary[tag::TotalNotAffectedOrders]            = "TotalNotAffectedOrders";                          // (int FIX.5.0SP2)
dictionary[tag::OrderOwnershipIndicator]           = "OrderOwnershipIndicator";                         // (int FIX.5.0SP2)
dictionary[tag::LegAccount]                        = "LegAccount";                                      // (String FIX.5.0SP2)
dictionary[tag::InTheMoneyCondition]               = "InTheMoneyCondition";                             // (int FIX.5.0SP2)
dictionary[tag::LegInTheMoneyCondition]            = "LegInTheMoneyCondition";                          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingInTheMoneyCondition]     = "UnderlyingInTheMoneyCondition";                   // (int FIX.5.0SP2)
dictionary[tag::DerivativeInTheMoneyCondition]     = "DerivativeInTheMoneyCondition";                   // (int FIX.5.0SP2)
dictionary[tag::ContraryInstructionEligibilityIndicator] = "ContraryInstructionEligibilityIndicator";         // (Boolean FIX.5.0SP2)
dictionary[tag::LegContraryInstructionEligibilityIndicator] = "LegContraryInstructionEligibilityIndicator";      // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingContraryInstructionEligibilityIndicator] = "UnderlyingContraryInstructionEligibilityIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::DerivativeContraryInstructionEligibilityIndicator] = "DerivativeContraryInstructionEligibilityIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::CollateralMarketPrice]             = "CollateralMarketPrice";                           // (Price FIX.5.0SP2)
dictionary[tag::CollateralPercentOverage]          = "CollateralPercentOverage";                        // (Percentage FIX.5.0SP2)
dictionary[tag::NoSideCollateralAmounts]           = "NoSideCollateralAmounts";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::SideCollateralAmountMarketID]      = "SideCollateralAmountMarketID";                    // (String FIX.5.0SP2)
dictionary[tag::SideCollateralAmountMarketSegmentID] = "SideCollateralAmountMarketSegmentID";             // (String FIX.5.0SP2)
dictionary[tag::SideCollateralAmountType]          = "SideCollateralAmountType";                        // (int FIX.5.0SP2)
dictionary[tag::SideCollateralCurrency]            = "SideCollateralCurrency";                          // (Currency FIX.5.0SP2)
dictionary[tag::SideCollateralFXRate]              = "SideCollateralFXRate";                            // (float FIX.5.0SP2)
dictionary[tag::SideCollateralFXRateCalc]          = "SideCollateralFXRateCalc";                        // (char FIX.5.0SP2)
dictionary[tag::SideCollateralMarketPrice]         = "SideCollateralMarketPrice";                       // (Price FIX.5.0SP2)
dictionary[tag::SideCollateralPercentOverage]      = "SideCollateralPercentOverage";                    // (Percentage FIX.5.0SP2)
dictionary[tag::SideCollateralPortfolioID]         = "SideCollateralPortfolioID";                       // (String FIX.5.0SP2)
dictionary[tag::SideCollateralType]                = "SideCollateralType";                              // (String FIX.5.0SP2)
dictionary[tag::SideCurrentCollateralAmount]       = "SideCurrentCollateralAmount";                     // (Amt FIX.5.0SP2)
dictionary[tag::SideHaircutIndicator]              = "SideHaircutIndicator";                            // (Boolean FIX.5.0SP2)
dictionary[tag::ExDestinationType]                 = "ExDestinationType";                               // (int FIX.5.0SP2)
dictionary[tag::MarketCondition]                   = "MarketCondition";                                 // (int FIX.5.0SP2)
dictionary[tag::NoQuoteAttributes]                 = "NoQuoteAttributes";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::QuoteAttributeType]                = "QuoteAttributeType";                              // (int FIX.5.0SP2)
dictionary[tag::QuoteAttributeValue]               = "QuoteAttributeValue";                             // (String FIX.5.0SP2)
dictionary[tag::NoPriceQualifiers]                 = "NoPriceQualifiers";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::PriceQualifier]                    = "PriceQualifier";                                  // (int FIX.5.0SP2)
dictionary[tag::MDValueTier]                       = "MDValueTier";                                     // (int FIX.5.0SP2)
dictionary[tag::MiscFeeQualifier]                  = "MiscFeeQualifier";                                // (int FIX.5.0SP2)
dictionary[tag::MiscFeeDesc]                       = "MiscFeeDesc";                                     // (String FIX.5.0SP2)
dictionary[tag::FinancialInstrumentFullName]       = "FinancialInstrumentFullName";                     // (String FIX.5.0SP2)
dictionary[tag::EncodedFinancialInstrumentFullNameLen] = "EncodedFinancialInstrumentFullNameLen";           // (Length FIX.5.0SP2)
dictionary[tag::EncodedFinancialInstrumentFullName] = "EncodedFinancialInstrumentFullName";              // (data FIX.5.0SP2)
dictionary[tag::LegFinancialInstrumentFullName]    = "LegFinancialInstrumentFullName";                  // (String FIX.5.0SP2)
dictionary[tag::EncodedLegFinancialInstrumentFullNameLen] = "EncodedLegFinancialInstrumentFullNameLen";        // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegFinancialInstrumentFullName] = "EncodedLegFinancialInstrumentFullName";           // (data FIX.5.0SP2)
dictionary[tag::UnderlyingFinancialInstrumentFullName] = "UnderlyingFinancialInstrumentFullName";           // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingFinancialInstrumentFullNameLen] = "EncodedUnderlyingFinancialInstrumentFullNameLen"; // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingFinancialInstrumentFullName] = "EncodedUnderlyingFinancialInstrumentFullName";    // (data FIX.5.0SP2)
dictionary[tag::UnderlyingIndexCurveUnit]          = "UnderlyingIndexCurveUnit";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingIndexCurvePeriod]        = "UnderlyingIndexCurvePeriod";                      // (int FIX.5.0SP2)
dictionary[tag::CommissionAmountSubType]           = "CommissionAmountSubType";                         // (int FIX.5.0SP2)
dictionary[tag::AllocCommissionAmountSubType]      = "AllocCommissionAmountSubType";                    // (int FIX.5.0SP2)
dictionary[tag::AllocLegRefID]                     = "AllocLegRefID";                                   // (String FIX.5.0SP2)
dictionary[tag::FloatingRateIndexCurvePeriod]      = "FloatingRateIndexCurvePeriod";                    // (int FIX.5.0SP2)
dictionary[tag::FloatingRateIndexCurveSpread]      = "FloatingRateIndexCurveSpread";                    // (PriceOffset FIX.5.0SP2)
dictionary[tag::FloatingRateIndexCurveUnit]        = "FloatingRateIndexCurveUnit";                      // (String FIX.5.0SP2)
dictionary[tag::FloatingRateIndexID]               = "FloatingRateIndexID";                             // (String FIX.5.0SP2)
dictionary[tag::FloatingRateIndexIDSource]         = "FloatingRateIndexIDSource";                       // (String FIX.5.0SP2)
dictionary[tag::IndexRollMonth]                    = "IndexRollMonth";                                  // (String FIX.5.0SP2)
dictionary[tag::NoIndexRollMonths]                 = "NoIndexRollMonths";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::AssetSubType]                      = "AssetSubType";                                    // (String FIX.5.0SP2)
dictionary[tag::CommodityFinalPriceType]           = "CommodityFinalPriceType";                         // (int FIX.5.0SP2)
dictionary[tag::FinancialInstrumentShortName]      = "FinancialInstrumentShortName";                    // (String FIX.5.0SP2)
dictionary[tag::NextIndexRollDate]                 = "NextIndexRollDate";                               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegAssetSubType]                   = "LegAssetSubType";                                 // (String FIX.5.0SP2)
dictionary[tag::LegFinancialInstrumentShortName]   = "LegFinancialInstrumentShortName";                 // (String FIX.5.0SP2)
dictionary[tag::SecondaryAssetSubType]             = "SecondaryAssetSubType";                           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingFinancialInstrumentShortName] = "UnderlyingFinancialInstrumentShortName";          // (String FIX.5.0SP2)
dictionary[tag::LegSecondaryAssetSubType]          = "LegSecondaryAssetSubType";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingAssetSubType]            = "UnderlyingAssetSubType";                          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSecondaryAssetSubType]   = "UnderlyingSecondaryAssetSubType";                 // (String FIX.5.0SP2)
dictionary[tag::NoReferenceDataDates]              = "NoReferenceDataDates";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::ReferenceDataDate]                 = "ReferenceDataDate";                               // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::ReferenceDataDateType]             = "ReferenceDataDateType";                           // (int FIX.5.0SP2)
dictionary[tag::ExecutionTimestamp]                = "ExecutionTimestamp";                              // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::ReportingPx]                       = "ReportingPx";                                     // (Price FIX.5.0SP2)
dictionary[tag::ReportingQty]                      = "ReportingQty";                                    // (Qty FIX.5.0SP2)
dictionary[tag::DeliveryRouteOrCharter]            = "DeliveryRouteOrCharter";                          // (String FIX.5.0SP2)
dictionary[tag::ReturnTrigger]                     = "ReturnTrigger";                                   // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryRouteOrCharter]         = "LegDeliveryRouteOrCharter";                       // (String FIX.5.0SP2)
dictionary[tag::LegReturnTrigger]                  = "LegReturnTrigger";                                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryRouteOrCharter]  = "UnderlyingDeliveryRouteOrCharter";                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnTrigger]           = "UnderlyingReturnTrigger";                         // (int FIX.5.0SP2)
dictionary[tag::AllocRequestID]                    = "AllocRequestID";                                  // (String FIX.5.0SP2)
dictionary[tag::GroupAmount]                       = "GroupAmount";                                     // (Amt FIX.5.0SP2)
dictionary[tag::GroupRemainingAmount]              = "GroupRemainingAmount";                            // (Amt FIX.5.0SP2)
dictionary[tag::AllocGroupAmount]                  = "AllocGroupAmount";                                // (Amt FIX.5.0SP2)
dictionary[tag::PriceMarkup]                       = "PriceMarkup";                                     // (PriceOffset FIX.5.0SP2)
dictionary[tag::AveragePriceType]                  = "AveragePriceType";                                // (int FIX.5.0SP2)
dictionary[tag::AveragePriceStartTime]             = "AveragePriceStartTime";                           // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::AveragePriceEndTime]               = "AveragePriceEndTime";                             // (UTCTimestamp FIX.5.0SP2)
dictionary[tag::OrderPercentOfTotalVolume]         = "OrderPercentOfTotalVolume";                       // (Percentage FIX.5.0SP2)
dictionary[tag::AllocGroupStatus]                  = "AllocGroupStatus";                                // (int FIX.5.0SP2)
dictionary[tag::NoAdditionalTermBondRefs]          = "NoAdditionalTermBondRefs";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::AdditionalTermBondSecurityID]      = "AdditionalTermBondSecurityID";                    // (String FIX.5.0SP2)
dictionary[tag::AdditionalTermBondSecurityIDSource] = "AdditionalTermBondSecurityIDSource";              // (String FIX.5.0SP2)
dictionary[tag::AdditionalTermBondDesc]            = "AdditionalTermBondDesc";                          // (String FIX.5.0SP2)
dictionary[tag::EncodedAdditionalTermBondDescLen]  = "EncodedAdditionalTermBondDescLen";                // (Length FIX.5.0SP2)
dictionary[tag::EncodedAdditionalTermBondDesc]     = "EncodedAdditionalTermBondDesc";                   // (data FIX.5.0SP2)
dictionary[tag::AdditionalTermBondCurrency]        = "AdditionalTermBondCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::AdditionalTermBondIssuer]          = "AdditionalTermBondIssuer";                        // (String FIX.5.0SP2)
dictionary[tag::EncodedAdditionalTermBondIssuerLen] = "EncodedAdditionalTermBondIssuerLen";              // (Length FIX.5.0SP2)
dictionary[tag::EncodedAdditionalTermBondIssuer]   = "EncodedAdditionalTermBondIssuer";                 // (data FIX.5.0SP2)
dictionary[tag::AdditionalTermBondSeniority]       = "AdditionalTermBondSeniority";                     // (String FIX.5.0SP2)
dictionary[tag::AdditionalTermBondCouponType]      = "AdditionalTermBondCouponType";                    // (int FIX.5.0SP2)
dictionary[tag::AdditionalTermBondCouponRate]      = "AdditionalTermBondCouponRate";                    // (Percentage FIX.5.0SP2)
dictionary[tag::AdditionalTermBondMaturityDate]    = "AdditionalTermBondMaturityDate";                  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::AdditionalTermBondParValue]        = "AdditionalTermBondParValue";                      // (Amt FIX.5.0SP2)
dictionary[tag::AdditionalTermBondCurrentTotalIssuedAmount] = "AdditionalTermBondCurrentTotalIssuedAmount";      // (Amt FIX.5.0SP2)
dictionary[tag::AdditionalTermBondCouponFrequencyPeriod] = "AdditionalTermBondCouponFrequencyPeriod";         // (int FIX.5.0SP2)
dictionary[tag::AdditionalTermBondCouponFrequencyUnit] = "AdditionalTermBondCouponFrequencyUnit";           // (String FIX.5.0SP2)
dictionary[tag::AdditionalTermBondDayCount]        = "AdditionalTermBondDayCount";                      // (int FIX.5.0SP2)
dictionary[tag::NoAdditionalTerms]                 = "NoAdditionalTerms";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::AdditionalTermConditionPrecedentBondIndicator] = "AdditionalTermConditionPrecedentBondIndicator";   // (Boolean FIX.5.0SP2)
dictionary[tag::AdditionalTermDiscrepancyClauseIndicator] = "AdditionalTermDiscrepancyClauseIndicator";        // (Boolean FIX.5.0SP2)
dictionary[tag::NoCashSettlTerms]                  = "NoCashSettlTerms";                                // (NumInGroup FIX.5.0SP2)
dictionary[tag::CashSettlCurrency]                 = "CashSettlCurrency";                               // (Currency FIX.5.0SP2)
dictionary[tag::CashSettlValuationFirstBusinessDayOffset] = "CashSettlValuationFirstBusinessDayOffset";        // (int FIX.5.0SP2)
dictionary[tag::CashSettlValuationTime]            = "CashSettlValuationTime";                          // (LocalMktTime FIX.5.0SP2)
dictionary[tag::CashSettlBusinessCenter]           = "CashSettlBusinessCenter";                         // (String FIX.5.0SP2)
dictionary[tag::CashSettlQuoteMethod]              = "CashSettlQuoteMethod";                            // (int FIX.5.0SP2)
dictionary[tag::CashSettlQuoteAmount]              = "CashSettlQuoteAmount";                            // (Amt FIX.5.0SP2)
dictionary[tag::CashSettlQuoteCurrency]            = "CashSettlQuoteCurrency";                          // (Currency FIX.5.0SP2)
dictionary[tag::CashSettlMinimumQuoteAmount]       = "CashSettlMinimumQuoteAmount";                     // (Amt FIX.5.0SP2)
dictionary[tag::CashSettlMinimumQuoteCurrency]     = "CashSettlMinimumQuoteCurrency";                   // (Currency FIX.5.0SP2)
dictionary[tag::CashSettlDealer]                   = "CashSettlDealer";                                 // (String FIX.5.0SP2)
dictionary[tag::CashSettlBusinessDays]             = "CashSettlBusinessDays";                           // (int FIX.5.0SP2)
dictionary[tag::CashSettlAmount]                   = "CashSettlAmount";                                 // (Amt FIX.5.0SP2)
dictionary[tag::CashSettlRecoveryFactor]           = "CashSettlRecoveryFactor";                         // (float FIX.5.0SP2)
dictionary[tag::CashSettlFixedTermIndicator]       = "CashSettlFixedTermIndicator";                     // (Boolean FIX.5.0SP2)
dictionary[tag::CashSettlAccruedInterestIndicator] = "CashSettlAccruedInterestIndicator";               // (Boolean FIX.5.0SP2)
dictionary[tag::CashSettlValuationMethod]          = "CashSettlValuationMethod";                        // (int FIX.5.0SP2)
dictionary[tag::CashSettlTermXID]                  = "CashSettlTermXID";                                // (XID FIX.5.0SP2)
dictionary[tag::NoContractualDefinitions]          = "NoContractualDefinitions";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::ContractualDefinition]             = "ContractualDefinition";                           // (String FIX.5.0SP2)
dictionary[tag::NoContractualMatrices]             = "NoContractualMatrices";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::ContractualMatrixSource]           = "ContractualMatrixSource";                         // (String FIX.5.0SP2)
dictionary[tag::ContractualMatrixDate]             = "ContractualMatrixDate";                           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ContractualMatrixTerm]             = "ContractualMatrixTerm";                           // (String FIX.5.0SP2)
dictionary[tag::NoFinancingTermSupplements]        = "NoFinancingTermSupplements";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::FinancingTermSupplementDesc]       = "FinancingTermSupplementDesc";                     // (String FIX.5.0SP2)
dictionary[tag::FinancingTermSupplementDate]       = "FinancingTermSupplementDate";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoStreams]                         = "NoStreams";                                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamType]                        = "StreamType";                                      // (int FIX.5.0SP2)
dictionary[tag::StreamDesc]                        = "StreamDesc";                                      // (String FIX.5.0SP2)
dictionary[tag::StreamPaySide]                     = "StreamPaySide";                                   // (int FIX.5.0SP2)
dictionary[tag::StreamReceiveSide]                 = "StreamReceiveSide";                               // (int FIX.5.0SP2)
dictionary[tag::StreamNotional]                    = "StreamNotional";                                  // (Amt FIX.5.0SP2)
dictionary[tag::StreamCurrency]                    = "StreamCurrency";                                  // (Currency FIX.5.0SP2)
dictionary[tag::StreamText]                        = "StreamText";                                      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamEffectiveDateUnadjusted] = "UnderlyingStreamEffectiveDateUnadjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamEffectiveDateBusinessDayConvention] = "UnderlyingStreamEffectiveDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamEffectiveDateBusinessCenter] = "UnderlyingStreamEffectiveDateBusinessCenter";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamEffectiveDateRelativeTo] = "UnderlyingStreamEffectiveDateRelativeTo";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamEffectiveDateOffsetPeriod] = "UnderlyingStreamEffectiveDateOffsetPeriod";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamEffectiveDateOffsetUnit] = "UnderlyingStreamEffectiveDateOffsetUnit";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamEffectiveDateOffsetDayType] = "UnderlyingStreamEffectiveDateOffsetDayType";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamEffectiveDateAdjusted] = "UnderlyingStreamEffectiveDateAdjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamTerminationDateUnadjusted]   = "StreamTerminationDateUnadjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamTerminationDateBusinessDayConvention] = "StreamTerminationDateBusinessDayConvention";      // (int FIX.5.0SP2)
dictionary[tag::StreamTerminationDateBusinessCenter] = "StreamTerminationDateBusinessCenter";             // (String FIX.5.0SP2)
dictionary[tag::StreamTerminationDateRelativeTo]   = "StreamTerminationDateRelativeTo";                 // (int FIX.5.0SP2)
dictionary[tag::StreamTerminationDateOffsetPeriod] = "StreamTerminationDateOffsetPeriod";               // (int FIX.5.0SP2)
dictionary[tag::StreamTerminationDateOffsetUnit]   = "StreamTerminationDateOffsetUnit";                 // (String FIX.5.0SP2)
dictionary[tag::StreamTerminationDateOffsetDayType] = "StreamTerminationDateOffsetDayType";              // (int FIX.5.0SP2)
dictionary[tag::StreamTerminationDateAdjusted]     = "StreamTerminationDateAdjusted";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamCalculationPeriodBusinessDayConvention] = "StreamCalculationPeriodBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::StreamCalculationPeriodBusinessCenter] = "StreamCalculationPeriodBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::StreamFirstPeriodStartDateUnadjusted] = "StreamFirstPeriodStartDateUnadjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamFirstPeriodStartDateBusinessDayConvention] = "StreamFirstPeriodStartDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::StreamFirstPeriodStartDateBusinessCenter] = "StreamFirstPeriodStartDateBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::StreamFirstPeriodStartDateAdjusted] = "StreamFirstPeriodStartDateAdjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamFirstRegularPeriodStartDateUnadjusted] = "StreamFirstRegularPeriodStartDateUnadjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamFirstCompoundingPeriodEndDateUnadjusted] = "StreamFirstCompoundingPeriodEndDateUnadjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamLastRegularPeriodEndDateUnadjusted] = "StreamLastRegularPeriodEndDateUnadjusted";        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamCalculationFrequencyPeriod]  = "StreamCalculationFrequencyPeriod";                // (int FIX.5.0SP2)
dictionary[tag::StreamCalculationFrequencyUnit]    = "StreamCalculationFrequencyUnit";                  // (String FIX.5.0SP2)
dictionary[tag::StreamCalculationRollConvention]   = "StreamCalculationRollConvention";                 // (String FIX.5.0SP2)
dictionary[tag::NoSettlRateFallbacks]              = "NoSettlRateFallbacks";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::SettlRatePostponementMaximumDays]  = "SettlRatePostponementMaximumDays";                // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableSettlRateSource] = "LegPaymentStreamNonDeliverableSettlRateSource";   // (int FIX.5.0SP2)
dictionary[tag::SettlRatePostponementSurvey]       = "SettlRatePostponementSurvey";                     // (Boolean FIX.5.0SP2)
dictionary[tag::SettlRatePostponementCalculationAgent] = "SettlRatePostponementCalculationAgent";           // (int FIX.5.0SP2)
dictionary[tag::NoProvisions]                      = "NoProvisions";                                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProvisionType]                     = "ProvisionType";                                   // (int FIX.5.0SP2)
dictionary[tag::ProvisionDateUnadjusted]           = "ProvisionDateUnadjusted";                         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionDateBusinessDayConvention] = "ProvisionDateBusinessDayConvention";              // (int FIX.5.0SP2)
dictionary[tag::ProvisionDateBusinessCenter]       = "ProvisionDateBusinessCenter";                     // (String FIX.5.0SP2)
dictionary[tag::ProvisionDateAdjusted]             = "ProvisionDateAdjusted";                           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionDateTenorPeriod]          = "ProvisionDateTenorPeriod";                        // (int FIX.5.0SP2)
dictionary[tag::ProvisionDateTenorUnit]            = "ProvisionDateTenorUnit";                          // (String FIX.5.0SP2)
dictionary[tag::ProvisionCalculationAgent]         = "ProvisionCalculationAgent";                       // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionSinglePartyBuyerSide] = "ProvisionOptionSinglePartyBuyerSide";             // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionSinglePartySellerSide] = "ProvisionOptionSinglePartySellerSide";            // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseStyle]      = "ProvisionOptionExerciseStyle";                    // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseMultipleNotional] = "ProvisionOptionExerciseMultipleNotional";         // (Amt FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseMinimumNotional] = "ProvisionOptionExerciseMinimumNotional";          // (Amt FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseMaximumNotional] = "ProvisionOptionExerciseMaximumNotional";          // (Amt FIX.5.0SP2)
dictionary[tag::ProvisionOptionMinimumNumber]      = "ProvisionOptionMinimumNumber";                    // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionMaximumNumber]      = "ProvisionOptionMaximumNumber";                    // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseConfirmation] = "ProvisionOptionExerciseConfirmation";             // (Boolean FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlMethod]          = "ProvisionCashSettlMethod";                        // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlCurrency]        = "ProvisionCashSettlCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlCurrency2]       = "ProvisionCashSettlCurrency2";                     // (Currency FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlQuoteType]       = "ProvisionCashSettlQuoteType";                     // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlQuoteSource]     = "ProvisionCashSettlQuoteSource";                   // (int FIX.5.0SP2)
dictionary[tag::ProvisionText]                     = "ProvisionText";                                   // (String FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueTime]       = "ProvisionCashSettlValueTime";                     // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueTimeBusinessCenter] = "ProvisionCashSettlValueTimeBusinessCenter";       // (String FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueDateBusinessDayConvention] = "ProvisionCashSettlValueDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueDateBusinessCenter] = "ProvisionCashSettlValueDateBusinessCenter";       // (String FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueDateRelativeTo] = "ProvisionCashSettlValueDateRelativeTo";           // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueDateOffsetPeriod] = "ProvisionCashSettlValueDateOffsetPeriod";         // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueDateOffsetUnit] = "ProvisionCashSettlValueDateOffsetUnit";           // (String FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueDateOffsetDayType] = "ProvisionCashSettlValueDateOffsetDayType";        // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlValueDateAdjusted] = "ProvisionCashSettlValueDateAdjusted";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseBusinessDayConvention] = "ProvisionOptionExerciseBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseBusinessCenter] = "ProvisionOptionExerciseBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseEarliestDateOffsetPeriod] = "ProvisionOptionExerciseEarliestDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseEarliestDateOffsetUnit] = "ProvisionOptionExerciseEarliestDateOffsetUnit";   // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseFrequencyPeriod] = "ProvisionOptionExerciseFrequencyPeriod";          // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseFrequencyUnit] = "ProvisionOptionExerciseFrequencyUnit";            // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseStartDateUnadjusted] = "ProvisionOptionExerciseStartDateUnadjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseStartDateRelativeTo] = "ProvisionOptionExerciseStartDateRelativeTo";      // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseStartDateOffsetPeriod] = "ProvisionOptionExerciseStartDateOffsetPeriod";    // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseStartDateOffsetUnit] = "ProvisionOptionExerciseStartDateOffsetUnit";      // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseStartDateOffsetDayType] = "ProvisionOptionExerciseStartDateOffsetDayType";   // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseStartDateAdjusted] = "ProvisionOptionExerciseStartDateAdjusted";        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionExercisePeriodSkip] = "ProvisionOptionExercisePeriodSkip";               // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseBoundsFirstDateUnadjusted] = "ProvisionOptionExerciseBoundsFirstDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseBoundsLastDateUnadjusted] = "ProvisionOptionExerciseBoundsLastDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseEarliestTime] = "ProvisionOptionExerciseEarliestTime";             // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseEarliestTimeBusinessCenter] = "ProvisionOptionExerciseEarliestTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseLatestTime] = "ProvisionOptionExerciseLatestTime";               // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseLatestTimeBusinessCenter] = "ProvisionOptionExerciseLatestTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoProvisionOptionExerciseFixedDates] = "NoProvisionOptionExerciseFixedDates";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseFixedDate]  = "ProvisionOptionExerciseFixedDate";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionExerciseFixedDateType] = "ProvisionOptionExerciseFixedDateType";            // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationDateUnadjusted] = "ProvisionOptionExpirationDateUnadjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationDateBusinessDayConvention] = "ProvisionOptionExpirationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationDateBusinessCenter] = "ProvisionOptionExpirationDateBusinessCenter";     // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationDateRelativeTo] = "ProvisionOptionExpirationDateRelativeTo";         // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationDateOffsetPeriod] = "ProvisionOptionExpirationDateOffsetPeriod";       // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationDateOffsetUnit] = "ProvisionOptionExpirationDateOffsetUnit";         // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationDateOffsetDayType] = "ProvisionOptionExpirationDateOffsetDayType";      // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationDateAdjusted] = "ProvisionOptionExpirationDateAdjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationTime]     = "ProvisionOptionExpirationTime";                   // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ProvisionOptionExpirationTimeBusinessCenter] = "ProvisionOptionExpirationTimeBusinessCenter";     // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionRelevantUnderlyingDateUnadjusted] = "ProvisionOptionRelevantUnderlyingDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionOptionRelevantUnderlyingDateBusinessDayConvention] = "ProvisionOptionRelevantUnderlyingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionRelevantUnderlyingDateBusinessCenter] = "ProvisionOptionRelevantUnderlyingDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionRelevantUnderlyingDateRelativeTo] = "ProvisionOptionRelevantUnderlyingDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionRelevantUnderlyingDateOffsetPeriod] = "ProvisionOptionRelevantUnderlyingDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionRelevantUnderlyingDateOffsetUnit] = "ProvisionOptionRelevantUnderlyingDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::ProvisionOptionRelevantUnderlyingDateOffsetDayType] = "ProvisionOptionRelevantUnderlyingDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::ProvisionOptionRelevantUnderlyingDateAdjusted] = "ProvisionOptionRelevantUnderlyingDateAdjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateBusinessDayConvention] = "ProvisionCashSettlPaymentDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateBusinessCenter] = "ProvisionCashSettlPaymentDateBusinessCenter";     // (String FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateRelativeTo] = "ProvisionCashSettlPaymentDateRelativeTo";         // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateOffsetPeriod] = "ProvisionCashSettlPaymentDateOffsetPeriod";       // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateOffsetUnit] = "ProvisionCashSettlPaymentDateOffsetUnit";         // (String FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateOffsetDayType] = "ProvisionCashSettlPaymentDateOffsetDayType";      // (int FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateRangeFirst] = "ProvisionCashSettlPaymentDateRangeFirst";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateRangeLast] = "ProvisionCashSettlPaymentDateRangeLast";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoProvisionCashSettlPaymentDates]  = "NoProvisionCashSettlPaymentDates";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDate]     = "ProvisionCashSettlPaymentDate";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlPaymentDateType] = "ProvisionCashSettlPaymentDateType";               // (int FIX.5.0SP2)
dictionary[tag::NoProvisionPartyIDs]               = "NoProvisionPartyIDs";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProvisionPartyID]                  = "ProvisionPartyID";                                // (String FIX.5.0SP2)
dictionary[tag::ProvisionPartyIDSource]            = "ProvisionPartyIDSource";                          // (char FIX.5.0SP2)
dictionary[tag::ProvisionPartyRole]                = "ProvisionPartyRole";                              // (int FIX.5.0SP2)
dictionary[tag::NoProvisionPartySubIDs]            = "NoProvisionPartySubIDs";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProvisionPartySubID]               = "ProvisionPartySubID";                             // (String FIX.5.0SP2)
dictionary[tag::ProvisionPartySubIDType]           = "ProvisionPartySubIDType";                         // (int FIX.5.0SP2)
dictionary[tag::NoProtectionTerms]                 = "NoProtectionTerms";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProtectionTermNotional]            = "ProtectionTermNotional";                          // (Amt FIX.5.0SP2)
dictionary[tag::ProtectionTermCurrency]            = "ProtectionTermCurrency";                          // (Currency FIX.5.0SP2)
dictionary[tag::ProtectionTermSellerNotifies]      = "ProtectionTermSellerNotifies";                    // (Boolean FIX.5.0SP2)
dictionary[tag::ProtectionTermBuyerNotifies]       = "ProtectionTermBuyerNotifies";                     // (Boolean FIX.5.0SP2)
dictionary[tag::ProtectionTermEventBusinessCenter] = "ProtectionTermEventBusinessCenter";               // (String FIX.5.0SP2)
dictionary[tag::ProtectionTermStandardSources]     = "ProtectionTermStandardSources";                   // (Boolean FIX.5.0SP2)
dictionary[tag::ProtectionTermEventMinimumSources] = "ProtectionTermEventMinimumSources";               // (int FIX.5.0SP2)
dictionary[tag::ProtectionTermEventNewsSource]     = "ProtectionTermEventNewsSource";                   // (String FIX.5.0SP2)
dictionary[tag::ProtectionTermXID]                 = "ProtectionTermXID";                               // (XID FIX.5.0SP2)
dictionary[tag::NoProtectionTermEvents]            = "NoProtectionTermEvents";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProtectionTermEventType]           = "ProtectionTermEventType";                         // (String FIX.5.0SP2)
dictionary[tag::ProtectionTermEventValue]          = "ProtectionTermEventValue";                        // (String FIX.5.0SP2)
dictionary[tag::ProtectionTermEventCurrency]       = "ProtectionTermEventCurrency";                     // (Currency FIX.5.0SP2)
dictionary[tag::ProtectionTermEventPeriod]         = "ProtectionTermEventPeriod";                       // (int FIX.5.0SP2)
dictionary[tag::ProtectionTermEventUnit]           = "ProtectionTermEventUnit";                         // (String FIX.5.0SP2)
dictionary[tag::ProtectionTermEventDayType]        = "ProtectionTermEventDayType";                      // (int FIX.5.0SP2)
dictionary[tag::ProtectionTermEventRateSource]     = "ProtectionTermEventRateSource";                   // (String FIX.5.0SP2)
dictionary[tag::NoProtectionTermEventQualifiers]   = "NoProtectionTermEventQualifiers";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProtectionTermEventQualifier]      = "ProtectionTermEventQualifier";                    // (char FIX.5.0SP2)
dictionary[tag::NoProtectionTermObligations]       = "NoProtectionTermObligations";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::ProtectionTermObligationType]      = "ProtectionTermObligationType";                    // (String FIX.5.0SP2)
dictionary[tag::ProtectionTermObligationValue]     = "ProtectionTermObligationValue";                   // (String FIX.5.0SP2)
dictionary[tag::NoPhysicalSettlTerms]              = "NoPhysicalSettlTerms";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::PhysicalSettlCurrency]             = "PhysicalSettlCurrency";                           // (Currency FIX.5.0SP2)
dictionary[tag::PhysicalSettlBusinessDays]         = "PhysicalSettlBusinessDays";                       // (int FIX.5.0SP2)
dictionary[tag::PhysicalSettlMaximumBusinessDays]  = "PhysicalSettlMaximumBusinessDays";                // (int FIX.5.0SP2)
dictionary[tag::PhysicalSettlTermXID]              = "PhysicalSettlTermXID";                            // (XID FIX.5.0SP2)
dictionary[tag::NoPhysicalSettlDeliverableObligations] = "NoPhysicalSettlDeliverableObligations";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::PhysicalSettlDeliverableObligationType] = "PhysicalSettlDeliverableObligationType";          // (String FIX.5.0SP2)
dictionary[tag::PhysicalSettlDeliverableObligationValue] = "PhysicalSettlDeliverableObligationValue";         // (String FIX.5.0SP2)
dictionary[tag::NoPayments]                        = "NoPayments";                                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentType]                       = "PaymentType";                                     // (int FIX.5.0SP2)
dictionary[tag::PaymentPaySide]                    = "PaymentPaySide";                                  // (int FIX.5.0SP2)
dictionary[tag::PaymentReceiveSide]                = "PaymentReceiveSide";                              // (int FIX.5.0SP2)
dictionary[tag::PaymentCurrency]                   = "PaymentCurrency";                                 // (Currency FIX.5.0SP2)
dictionary[tag::PaymentAmount]                     = "PaymentAmount";                                   // (Amt FIX.5.0SP2)
dictionary[tag::PaymentPrice]                      = "PaymentPrice";                                    // (Price FIX.5.0SP2)
dictionary[tag::PaymentDateUnadjusted]             = "PaymentDateUnadjusted";                           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentBusinessDayConvention]      = "PaymentBusinessDayConvention";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentBusinessCenter]             = "PaymentBusinessCenter";                           // (String FIX.5.0SP2)
dictionary[tag::PaymentDateAdjusted]               = "PaymentDateAdjusted";                             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionValue]          = "LegMarketDisruptionValue";                        // (String FIX.5.0SP2)
dictionary[tag::PaymentDiscountFactor]             = "PaymentDiscountFactor";                           // (float FIX.5.0SP2)
dictionary[tag::PaymentPresentValueAmount]         = "PaymentPresentValueAmount";                       // (Amt FIX.5.0SP2)
dictionary[tag::PaymentPresentValueCurrency]       = "PaymentPresentValueCurrency";                     // (Currency FIX.5.0SP2)
dictionary[tag::PaymentSettlStyle]                 = "PaymentSettlStyle";                               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableSettlReferencePage] = "LegPaymentStreamNonDeliverableSettlReferencePage"; // (String FIX.5.0SP2)
dictionary[tag::PaymentText]                       = "PaymentText";                                     // (String FIX.5.0SP2)
dictionary[tag::NoPaymentSettls]                   = "NoPaymentSettls";                                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentSettlAmount]                = "PaymentSettlAmount";                              // (Amt FIX.5.0SP2)
dictionary[tag::PaymentSettlCurrency]              = "PaymentSettlCurrency";                            // (Currency FIX.5.0SP2)
dictionary[tag::NoPaymentSettlPartyIDs]            = "NoPaymentSettlPartyIDs";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentSettlPartyID]               = "PaymentSettlPartyID";                             // (String FIX.5.0SP2)
dictionary[tag::PaymentSettlPartyIDSource]         = "PaymentSettlPartyIDSource";                       // (char FIX.5.0SP2)
dictionary[tag::PaymentSettlPartyRole]             = "PaymentSettlPartyRole";                           // (int FIX.5.0SP2)
dictionary[tag::PaymentSettlPartyRoleQualifier]    = "PaymentSettlPartyRoleQualifier";                  // (int FIX.5.0SP2)
dictionary[tag::NoPaymentSettlPartySubIDs]         = "NoPaymentSettlPartySubIDs";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentSettlPartySubID]            = "PaymentSettlPartySubID";                          // (String FIX.5.0SP2)
dictionary[tag::PaymentSettlPartySubIDType]        = "PaymentSettlPartySubIDType";                      // (int FIX.5.0SP2)
dictionary[tag::NoLegStreams]                      = "NoLegStreams";                                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamType]                     = "LegStreamType";                                   // (int FIX.5.0SP2)
dictionary[tag::LegStreamDesc]                     = "LegStreamDesc";                                   // (String FIX.5.0SP2)
dictionary[tag::LegStreamPaySide]                  = "LegStreamPaySide";                                // (int FIX.5.0SP2)
dictionary[tag::LegStreamReceiveSide]              = "LegStreamReceiveSide";                            // (int FIX.5.0SP2)
dictionary[tag::LegStreamNotional]                 = "LegStreamNotional";                               // (Amt FIX.5.0SP2)
dictionary[tag::LegStreamCurrency]                 = "LegStreamCurrency";                               // (Currency FIX.5.0SP2)
dictionary[tag::LegStreamText]                     = "LegStreamText";                                   // (String FIX.5.0SP2)
dictionary[tag::LegStreamEffectiveDateUnadjusted]  = "LegStreamEffectiveDateUnadjusted";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamEffectiveDateBusinessDayConvention] = "LegStreamEffectiveDateBusinessDayConvention";     // (int FIX.5.0SP2)
dictionary[tag::LegStreamEffectiveDateBusinessCenter] = "LegStreamEffectiveDateBusinessCenter";            // (String FIX.5.0SP2)
dictionary[tag::LegStreamEffectiveDateRelativeTo]  = "LegStreamEffectiveDateRelativeTo";                // (int FIX.5.0SP2)
dictionary[tag::LegStreamEffectiveDateOffsetPeriod] = "LegStreamEffectiveDateOffsetPeriod";              // (int FIX.5.0SP2)
dictionary[tag::LegStreamEffectiveDateOffsetUnit]  = "LegStreamEffectiveDateOffsetUnit";                // (String FIX.5.0SP2)
dictionary[tag::LegStreamEffectiveDateOffsetDayType] = "LegStreamEffectiveDateOffsetDayType";             // (int FIX.5.0SP2)
dictionary[tag::LegStreamEffectiveDateAdjusted]    = "LegStreamEffectiveDateAdjusted";                  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamTerminationDateUnadjusted] = "LegStreamTerminationDateUnadjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamTerminationDateBusinessDayConvention] = "LegStreamTerminationDateBusinessDayConvention";   // (int FIX.5.0SP2)
dictionary[tag::LegStreamTerminationDateBusinessCenter] = "LegStreamTerminationDateBusinessCenter";          // (String FIX.5.0SP2)
dictionary[tag::LegStreamTerminationDateRelativeTo] = "LegStreamTerminationDateRelativeTo";              // (int FIX.5.0SP2)
dictionary[tag::LegStreamTerminationDateOffsetPeriod] = "LegStreamTerminationDateOffsetPeriod";            // (int FIX.5.0SP2)
dictionary[tag::LegStreamTerminationDateOffsetUnit] = "LegStreamTerminationDateOffsetUnit";              // (String FIX.5.0SP2)
dictionary[tag::LegStreamTerminationDateOffsetDayType] = "LegStreamTerminationDateOffsetDayType";           // (int FIX.5.0SP2)
dictionary[tag::LegStreamTerminationDateAdjusted]  = "LegStreamTerminationDateAdjusted";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamCalculationPeriodBusinessDayConvention] = "LegStreamCalculationPeriodBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegStreamCalculationPeriodBusinessCenter] = "LegStreamCalculationPeriodBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::LegStreamFirstPeriodStartDateUnadjusted] = "LegStreamFirstPeriodStartDateUnadjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamFirstPeriodStartDateBusinessDayConvention] = "LegStreamFirstPeriodStartDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegStreamFirstPeriodStartDateBusinessCenter] = "LegStreamFirstPeriodStartDateBusinessCenter";     // (String FIX.5.0SP2)
dictionary[tag::LegStreamFirstPeriodStartDateAdjusted] = "LegStreamFirstPeriodStartDateAdjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamFirstRegularPeriodStartDateUnadjusted] = "LegStreamFirstRegularPeriodStartDateUnadjusted";  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamFirstCompoundingPeriodEndDateUnadjusted] = "LegStreamFirstCompoundingPeriodEndDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamLastRegularPeriodEndDateUnadjusted] = "LegStreamLastRegularPeriodEndDateUnadjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamCalculationFrequencyPeriod] = "LegStreamCalculationFrequencyPeriod";             // (int FIX.5.0SP2)
dictionary[tag::LegStreamCalculationFrequencyUnit] = "LegStreamCalculationFrequencyUnit";               // (String FIX.5.0SP2)
dictionary[tag::LegStreamCalculationRollConvention] = "LegStreamCalculationRollConvention";              // (String FIX.5.0SP2)
dictionary[tag::NoCashSettlDealers]                = "NoCashSettlDealers";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoBusinessCenters]                 = "NoBusinessCenters";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamType]              = "LegPaymentStreamType";                            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamMarketRate]        = "LegPaymentStreamMarketRate";                      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamDelayIndicator]    = "LegPaymentStreamDelayIndicator";                  // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamSettlCurrency]     = "LegPaymentStreamSettlCurrency";                   // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentStreamDayCount]          = "LegPaymentStreamDayCount";                        // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamAccrualDays]       = "LegPaymentStreamAccrualDays";                     // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamDiscountType]      = "LegPaymentStreamDiscountType";                    // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamDiscountRate]      = "LegPaymentStreamDiscountRate";                    // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamDiscountRateDayCount] = "LegPaymentStreamDiscountRateDayCount";            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingMethod] = "LegPaymentStreamCompoundingMethod";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialPrincipalExchangeIndicator] = "LegPaymentStreamInitialPrincipalExchangeIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInterimPrincipalExchangeIndicator] = "LegPaymentStreamInterimPrincipalExchangeIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalPrincipalExchangeIndicator] = "LegPaymentStreamFinalPrincipalExchangeIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentDateBusinessDayConvention] = "LegPaymentStreamPaymentDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentDateBusinessCenter] = "LegPaymentStreamPaymentDateBusinessCenter";       // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentFrequencyPeriod] = "LegPaymentStreamPaymentFrequencyPeriod";          // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentFrequencyUnit] = "LegPaymentStreamPaymentFrequencyUnit";            // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentRollConvention] = "LegPaymentStreamPaymentRollConvention";           // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFirstPaymentDateUnadjusted] = "LegPaymentStreamFirstPaymentDateUnadjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLastRegularPaymentDateUnadjusted] = "LegPaymentStreamLastRegularPaymentDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentDateRelativeTo] = "LegPaymentStreamPaymentDateRelativeTo";           // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentDateOffsetPeriod] = "LegPaymentStreamPaymentDateOffsetPeriod";         // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentDateOffsetUnit] = "LegPaymentStreamPaymentDateOffsetUnit";           // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentDateOffsetDayType] = "LegPaymentStreamPaymentDateOffsetDayType";        // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamResetDateRelativeTo] = "LegPaymentStreamResetDateRelativeTo";             // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamResetDateBusinessDayConvention] = "LegPaymentStreamResetDateBusinessDayConvention";  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamResetDateBusinessCenter] = "LegPaymentStreamResetDateBusinessCenter";         // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamResetFrequencyPeriod] = "LegPaymentStreamResetFrequencyPeriod";            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamResetFrequencyUnit] = "LegPaymentStreamResetFrequencyUnit";              // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamResetWeeklyRollConvention] = "LegPaymentStreamResetWeeklyRollConvention";       // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialFixingDateRelativeTo] = "LegPaymentStreamInitialFixingDateRelativeTo";     // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialFixingDateBusinessDayConvention] = "LegPaymentStreamInitialFixingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialFixingDateBusinessCenter] = "LegPaymentStreamInitialFixingDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialFixingDateOffsetPeriod] = "LegPaymentStreamInitialFixingDateOffsetPeriod";   // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialFixingDateOffsetUnit] = "LegPaymentStreamInitialFixingDateOffsetUnit";     // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialFixingDateOffsetDayType] = "LegPaymentStreamInitialFixingDateOffsetDayType";  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialFixingDateAdjusted] = "LegPaymentStreamInitialFixingDateAdjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDateRelativeTo] = "LegPaymentStreamFixingDateRelativeTo";            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDateBusinessDayConvention] = "LegPaymentStreamFixingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDateBusinessCenter] = "LegPaymentStreamFixingDateBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDateOffsetPeriod] = "LegPaymentStreamFixingDateOffsetPeriod";          // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDateOffsetUnit] = "LegPaymentStreamFixingDateOffsetUnit";            // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDateOffsetDayType] = "LegPaymentStreamFixingDateOffsetDayType";         // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDateAdjusted] = "LegPaymentStreamFixingDateAdjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateCutoffDateOffsetPeriod] = "LegPaymentStreamRateCutoffDateOffsetPeriod";      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateCutoffDateOffsetUnit] = "LegPaymentStreamRateCutoffDateOffsetUnit";        // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateCutoffDateOffsetDayType] = "LegPaymentStreamRateCutoffDateOffsetDayType";     // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRate]              = "LegPaymentStreamRate";                            // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixedAmount]       = "LegPaymentStreamFixedAmount";                     // (Amt FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateOrAmountCurrency] = "LegPaymentStreamRateOrAmountCurrency";            // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFutureValueNotional] = "LegPaymentStreamFutureValueNotional";             // (Amt FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFutureValueDateAdjusted] = "LegPaymentStreamFutureValueDateAdjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndex]         = "LegPaymentStreamRateIndex";                       // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndexSource]   = "LegPaymentStreamRateIndexSource";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndexCurveUnit] = "LegPaymentStreamRateIndexCurveUnit";              // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndexCurvePeriod] = "LegPaymentStreamRateIndexCurvePeriod";            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateMultiplier]    = "LegPaymentStreamRateMultiplier";                  // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateSpread]        = "LegPaymentStreamRateSpread";                      // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateSpreadPositionType] = "LegPaymentStreamRateSpreadPositionType";          // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateTreatment]     = "LegPaymentStreamRateTreatment";                   // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCapRate]           = "LegPaymentStreamCapRate";                         // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCapRateBuySide]    = "LegPaymentStreamCapRateBuySide";                  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCapRateSellSide]   = "LegPaymentStreamCapRateSellSide";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFloorRate]         = "LegPaymentStreamFloorRate";                       // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFloorRateBuySide]  = "LegPaymentStreamFloorRateBuySide";                // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFloorRateSellSide] = "LegPaymentStreamFloorRateSellSide";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInitialRate]       = "LegPaymentStreamInitialRate";                     // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalRateRoundingDirection] = "LegPaymentStreamFinalRateRoundingDirection";      // (char FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalRatePrecision] = "LegPaymentStreamFinalRatePrecision";              // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamAveragingMethod]   = "LegPaymentStreamAveragingMethod";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNegativeRateTreatment] = "LegPaymentStreamNegativeRateTreatment";           // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInflationLagPeriod] = "LegPaymentStreamInflationLagPeriod";              // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInflationLagUnit]  = "LegPaymentStreamInflationLagUnit";                // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInflationLagDayType] = "LegPaymentStreamInflationLagDayType";             // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInflationInterpolationMethod] = "LegPaymentStreamInflationInterpolationMethod";    // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInflationIndexSource] = "LegPaymentStreamInflationIndexSource";            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInflationPublicationSource] = "LegPaymentStreamInflationPublicationSource";      // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInflationInitialIndexLevel] = "LegPaymentStreamInflationInitialIndexLevel";      // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInflationFallbackBondApplicable] = "LegPaymentStreamInflationFallbackBondApplicable"; // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFRADiscounting]    = "LegPaymentStreamFRADiscounting";                  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableRefCurrency] = "LegPaymentStreamNonDeliverableRefCurrency";       // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableFixingDatesBusinessDayConvention] = "LegPaymentStreamNonDeliverableFixingDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableFixingDatesBusinessCenter] = "LegPaymentStreamNonDeliverableFixingDatesBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableFixingDatesRelativeTo] = "LegPaymentStreamNonDeliverableFixingDatesRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableFixingDatesOffsetPeriod] = "LegPaymentStreamNonDeliverableFixingDatesOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableFixingDatesOffsetUnit] = "LegPaymentStreamNonDeliverableFixingDatesOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNonDeliverableFixingDatesOffsetDayType] = "LegPaymentStreamNonDeliverableFixingDatesOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::LegSettlRateFallbackRateSource]    = "LegSettlRateFallbackRateSource";                  // (int FIX.5.0SP2)
dictionary[tag::NoLegNonDeliverableFixingDates]    = "NoLegNonDeliverableFixingDates";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegNonDeliverableFixingDate]       = "LegNonDeliverableFixingDate";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegNonDeliverableFixingDateType]   = "LegNonDeliverableFixingDateType";                 // (int FIX.5.0SP2)
dictionary[tag::LegSettlRateFallbackReferencePage] = "LegSettlRateFallbackReferencePage";               // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableSettlRateSource] = "PaymentStreamNonDeliverableSettlRateSource";      // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableSettlReferencePage] = "PaymentStreamNonDeliverableSettlReferencePage";   // (String FIX.5.0SP2)
dictionary[tag::SettlRateFallbackRateSource]       = "SettlRateFallbackRateSource";                     // (int FIX.5.0SP2)
dictionary[tag::NoLegPaymentSchedules]             = "NoLegPaymentSchedules";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleType]            = "LegPaymentScheduleType";                          // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStubType]        = "LegPaymentScheduleStubType";                      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStartDateUnadjusted] = "LegPaymentScheduleStartDateUnadjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleEndDateUnadjusted] = "LegPaymentScheduleEndDateUnadjusted";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentSchedulePaySide]         = "LegPaymentSchedulePaySide";                       // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleReceiveSide]     = "LegPaymentScheduleReceiveSide";                   // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleNotional]        = "LegPaymentScheduleNotional";                      // (Amt FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleCurrency]        = "LegPaymentScheduleCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRate]            = "LegPaymentScheduleRate";                          // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateMultiplier]  = "LegPaymentScheduleRateMultiplier";                // (float FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateSpread]      = "LegPaymentScheduleRateSpread";                    // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateSpreadPositionType] = "LegPaymentScheduleRateSpreadPositionType";        // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateTreatment]   = "LegPaymentScheduleRateTreatment";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixedAmount]     = "LegPaymentScheduleFixedAmount";                   // (Amt FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixedCurrency]   = "LegPaymentScheduleFixedCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStepFrequencyPeriod] = "LegPaymentScheduleStepFrequencyPeriod";           // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStepFrequencyUnit] = "LegPaymentScheduleStepFrequencyUnit";             // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStepOffsetValue] = "LegPaymentScheduleStepOffsetValue";               // (Amt FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStepRate]        = "LegPaymentScheduleStepRate";                      // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStepOffsetRate]  = "LegPaymentScheduleStepOffsetRate";                // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStepRelativeTo]  = "LegPaymentScheduleStepRelativeTo";                // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDateUnadjusted] = "LegPaymentScheduleFixingDateUnadjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleWeight]          = "LegPaymentScheduleWeight";                        // (float FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDateRelativeTo] = "LegPaymentScheduleFixingDateRelativeTo";          // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDateBusinessDayConvention] = "LegPaymentScheduleFixingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDateBusinessCenter] = "LegPaymentScheduleFixingDateBusinessCenter";      // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDateOffsetPeriod] = "LegPaymentScheduleFixingDateOffsetPeriod";        // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDateOffsetUnit] = "LegPaymentScheduleFixingDateOffsetUnit";          // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDateOffsetDayType] = "LegPaymentScheduleFixingDateOffsetDayType";       // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDateAdjusted] = "LegPaymentScheduleFixingDateAdjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingTime]      = "LegPaymentScheduleFixingTime";                    // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingTimeBusinessCenter] = "LegPaymentScheduleFixingTimeBusinessCenter";      // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleInterimExchangePaymentDateRelativeTo] = "LegPaymentScheduleInterimExchangePaymentDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleInterimExchangeDatesBusinessDayConvention] = "LegPaymentScheduleInterimExchangeDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleInterimExchangeDatesBusinessCenter] = "LegPaymentScheduleInterimExchangeDatesBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleInterimExchangeDatesOffsetPeriod] = "LegPaymentScheduleInterimExchangeDatesOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleInterimExchangeDatesOffsetUnit] = "LegPaymentScheduleInterimExchangeDatesOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleInterimExchangeDatesOffsetDayType] = "LegPaymentScheduleInterimExchangeDatesOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleInterimExchangeDateAdjusted] = "LegPaymentScheduleInterimExchangeDateAdjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegPaymentScheduleRateSources]   = "NoLegPaymentScheduleRateSources";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateSource]      = "LegPaymentScheduleRateSource";                    // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateSourceType]  = "LegPaymentScheduleRateSourceType";                // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleReferencePage]   = "LegPaymentScheduleReferencePage";                 // (String FIX.5.0SP2)
dictionary[tag::NoLegPaymentStubs]                 = "NoLegPaymentStubs";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStubType]                = "LegPaymentStubType";                              // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubLength]              = "LegPaymentStubLength";                            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubRate]                = "LegPaymentStubRate";                              // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStubFixedAmount]         = "LegPaymentStubFixedAmount";                       // (Amt FIX.5.0SP2)
dictionary[tag::LegPaymentStubFixedCurrency]       = "LegPaymentStubFixedCurrency";                     // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex]               = "LegPaymentStubIndex";                             // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexSource]         = "LegPaymentStubIndexSource";                       // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexCurvePeriod]    = "LegPaymentStubIndexCurvePeriod";                  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexCurveUnit]      = "LegPaymentStubIndexCurveUnit";                    // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexRateMultiplier] = "LegPaymentStubIndexRateMultiplier";               // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexRateSpread]     = "LegPaymentStubIndexRateSpread";                   // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexRateSpreadPositionType] = "LegPaymentStubIndexRateSpreadPositionType";       // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexRateTreatment]  = "LegPaymentStubIndexRateTreatment";                // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexCapRate]        = "LegPaymentStubIndexCapRate";                      // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexCapRateBuySide] = "LegPaymentStubIndexCapRateBuySide";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexCapRateSellSide] = "LegPaymentStubIndexCapRateSellSide";              // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexFloorRate]      = "LegPaymentStubIndexFloorRate";                    // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexFloorRateBuySide] = "LegPaymentStubIndexFloorRateBuySide";             // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndexFloorRateSellSide] = "LegPaymentStubIndexFloorRateSellSide";            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2]              = "LegPaymentStubIndex2";                            // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2Source]        = "LegPaymentStubIndex2Source";                      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2CurvePeriod]   = "LegPaymentStubIndex2CurvePeriod";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2CurveUnit]     = "LegPaymentStubIndex2CurveUnit";                   // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2RateMultiplier] = "LegPaymentStubIndex2RateMultiplier";              // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2RateSpread]    = "LegPaymentStubIndex2RateSpread";                  // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2RateSpreadPositionType] = "LegPaymentStubIndex2RateSpreadPositionType";      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2RateTreatment] = "LegPaymentStubIndex2RateTreatment";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2CapRate]       = "LegPaymentStubIndex2CapRate";                     // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStubIndex2FloorRate]     = "LegPaymentStubIndex2FloorRate";                   // (Percentage FIX.5.0SP2)
dictionary[tag::NoLegProvisions]                   = "NoLegProvisions";                                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProvisionType]                  = "LegProvisionType";                                // (int FIX.5.0SP2)
dictionary[tag::LegProvisionDateUnadjusted]        = "LegProvisionDateUnadjusted";                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionDateBusinessDayConvention] = "LegProvisionDateBusinessDayConvention";           // (int FIX.5.0SP2)
dictionary[tag::LegProvisionDateBusinessCenter]    = "LegProvisionDateBusinessCenter";                  // (String FIX.5.0SP2)
dictionary[tag::LegProvisionDateAdjusted]          = "LegProvisionDateAdjusted";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionDateTenorPeriod]       = "LegProvisionDateTenorPeriod";                     // (int FIX.5.0SP2)
dictionary[tag::LegProvisionDateTenorUnit]         = "LegProvisionDateTenorUnit";                       // (String FIX.5.0SP2)
dictionary[tag::LegProvisionCalculationAgent]      = "LegProvisionCalculationAgent";                    // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionSinglePartyBuyerSide] = "LegProvisionOptionSinglePartyBuyerSide";          // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionSinglePartySellerSide] = "LegProvisionOptionSinglePartySellerSide";         // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseStyle]   = "LegProvisionOptionExerciseStyle";                 // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseMultipleNotional] = "LegProvisionOptionExerciseMultipleNotional";      // (Amt FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseMinimumNotional] = "LegProvisionOptionExerciseMinimumNotional";       // (Amt FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseMaximumNotional] = "LegProvisionOptionExerciseMaximumNotional";       // (Amt FIX.5.0SP2)
dictionary[tag::LegProvisionOptionMinimumNumber]   = "LegProvisionOptionMinimumNumber";                 // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionMaximumNumber]   = "LegProvisionOptionMaximumNumber";                 // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseConfirmation] = "LegProvisionOptionExerciseConfirmation";          // (Boolean FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlMethod]       = "LegProvisionCashSettlMethod";                     // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlCurrency]     = "LegProvisionCashSettlCurrency";                   // (Currency FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlCurrency2]    = "LegProvisionCashSettlCurrency2";                  // (Currency FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlQuoteType]    = "LegProvisionCashSettlQuoteType";                  // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlQuoteSource]  = "LegProvisionCashSettlQuoteSource";                // (int FIX.5.0SP2)
dictionary[tag::BusinessCenter]                    = "BusinessCenter";                                  // (String FIX.5.0SP2)
dictionary[tag::LegProvisionText]                  = "LegProvisionText";                                // (String FIX.5.0SP2)
dictionary[tag::NoLegProvisionCashSettlPaymentDates] = "NoLegProvisionCashSettlPaymentDates";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDate]  = "LegProvisionCashSettlPaymentDate";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateType] = "LegProvisionCashSettlPaymentDateType";            // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseBusinessDayConvention] = "LegProvisionOptionExerciseBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseBusinessCenter] = "LegProvisionOptionExerciseBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseEarliestDateOffsetPeriod] = "LegProvisionOptionExerciseEarliestDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseEarliestDateOffsetUnit] = "LegProvisionOptionExerciseEarliestDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseFrequencyPeriod] = "LegProvisionOptionExerciseFrequencyPeriod";       // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseFrequencyUnit] = "LegProvisionOptionExerciseFrequencyUnit";         // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseStartDateUnadjusted] = "LegProvisionOptionExerciseStartDateUnadjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseStartDateRelativeTo] = "LegProvisionOptionExerciseStartDateRelativeTo";   // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseStartDateOffsetPeriod] = "LegProvisionOptionExerciseStartDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseStartDateOffsetUnit] = "LegProvisionOptionExerciseStartDateOffsetUnit";   // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseStartDateOffsetDayType] = "LegProvisionOptionExerciseStartDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseStartDateAdjusted] = "LegProvisionOptionExerciseStartDateAdjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExercisePeriodSkip] = "LegProvisionOptionExercisePeriodSkip";            // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseBoundsFirstDateUnadjusted] = "LegProvisionOptionExerciseBoundsFirstDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseBoundsLastDateUnadjusted] = "LegProvisionOptionExerciseBoundsLastDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseEarliestTime] = "LegProvisionOptionExerciseEarliestTime";          // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseEarliestTimeBusinessCenter] = "LegProvisionOptionExerciseEarliestTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseLatestTime] = "LegProvisionOptionExerciseLatestTime";            // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseLatestTimeBusinessCenter] = "LegProvisionOptionExerciseLatestTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoLegProvisionOptionExerciseFixedDates] = "NoLegProvisionOptionExerciseFixedDates";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseFixedDate] = "LegProvisionOptionExerciseFixedDate";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExerciseFixedDateType] = "LegProvisionOptionExerciseFixedDateType";         // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationDateUnadjusted] = "LegProvisionOptionExpirationDateUnadjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationDateBusinessDayConvention] = "LegProvisionOptionExpirationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationDateBusinessCenter] = "LegProvisionOptionExpirationDateBusinessCenter";  // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationDateRelativeTo] = "LegProvisionOptionExpirationDateRelativeTo";      // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationDateOffsetPeriod] = "LegProvisionOptionExpirationDateOffsetPeriod";    // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationDateOffsetUnit] = "LegProvisionOptionExpirationDateOffsetUnit";      // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationDateOffsetDayType] = "LegProvisionOptionExpirationDateOffsetDayType";   // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationDateAdjusted] = "LegProvisionOptionExpirationDateAdjusted";        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationTime]  = "LegProvisionOptionExpirationTime";                // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegProvisionOptionExpirationTimeBusinessCenter] = "LegProvisionOptionExpirationTimeBusinessCenter";  // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionRelevantUnderlyingDateUnadjusted] = "LegProvisionOptionRelevantUnderlyingDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionOptionRelevantUnderlyingDateBusinessDayConvention] = "LegProvisionOptionRelevantUnderlyingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionRelevantUnderlyingDateBusinessCenter] = "LegProvisionOptionRelevantUnderlyingDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionRelevantUnderlyingDateRelativeTo] = "LegProvisionOptionRelevantUnderlyingDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionRelevantUnderlyingDateOffsetPeriod] = "LegProvisionOptionRelevantUnderlyingDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionRelevantUnderlyingDateOffsetUnit] = "LegProvisionOptionRelevantUnderlyingDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::LegProvisionOptionRelevantUnderlyingDateOffsetDayType] = "LegProvisionOptionRelevantUnderlyingDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionOptionRelevantUnderlyingDateAdjusted] = "LegProvisionOptionRelevantUnderlyingDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateBusinessDayConvention] = "LegProvisionCashSettlPaymentDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateBusinessCenter] = "LegProvisionCashSettlPaymentDateBusinessCenter";  // (String FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateRelativeTo] = "LegProvisionCashSettlPaymentDateRelativeTo";      // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateOffsetPeriod] = "LegProvisionCashSettlPaymentDateOffsetPeriod";    // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateOffsetUnit] = "LegProvisionCashSettlPaymentDateOffsetUnit";      // (String FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateOffsetDayType] = "LegProvisionCashSettlPaymentDateOffsetDayType";   // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateRangeFirst] = "LegProvisionCashSettlPaymentDateRangeFirst";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlPaymentDateRangeLast] = "LegProvisionCashSettlPaymentDateRangeLast";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueTime]    = "LegProvisionCashSettlValueTime";                  // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueTimeBusinessCenter] = "LegProvisionCashSettlValueTimeBusinessCenter";    // (String FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueDateBusinessDayConvention] = "LegProvisionCashSettlValueDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueDateBusinessCenter] = "LegProvisionCashSettlValueDateBusinessCenter";    // (String FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueDateRelativeTo] = "LegProvisionCashSettlValueDateRelativeTo";        // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueDateOffsetPeriod] = "LegProvisionCashSettlValueDateOffsetPeriod";      // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueDateOffsetUnit] = "LegProvisionCashSettlValueDateOffsetUnit";        // (String FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueDateOffsetDayType] = "LegProvisionCashSettlValueDateOffsetDayType";     // (int FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlValueDateAdjusted] = "LegProvisionCashSettlValueDateAdjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegProvisionPartyIDs]            = "NoLegProvisionPartyIDs";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProvisionPartyID]               = "LegProvisionPartyID";                             // (String FIX.5.0SP2)
dictionary[tag::LegProvisionPartyIDSource]         = "LegProvisionPartyIDSource";                       // (char FIX.5.0SP2)
dictionary[tag::LegProvisionPartyRole]             = "LegProvisionPartyRole";                           // (int FIX.5.0SP2)
dictionary[tag::NoLegProvisionPartySubIDs]         = "NoLegProvisionPartySubIDs";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProvisionPartySubID]            = "LegProvisionPartySubID";                          // (String FIX.5.0SP2)
dictionary[tag::LegProvisionPartySubIDType]        = "LegProvisionPartySubIDType";                      // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreams]               = "NoUnderlyingStreams";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamType]              = "UnderlyingStreamType";                            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamDesc]              = "UnderlyingStreamDesc";                            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamPaySide]           = "UnderlyingStreamPaySide";                         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamReceiveSide]       = "UnderlyingStreamReceiveSide";                     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamNotional]          = "UnderlyingStreamNotional";                        // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCurrency]          = "UnderlyingStreamCurrency";                        // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingStreamText]              = "UnderlyingStreamText";                            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTerminationDateUnadjusted] = "UnderlyingStreamTerminationDateUnadjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTerminationDateBusinessDayConvention] = "UnderlyingStreamTerminationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTerminationDateBusinessCenter] = "UnderlyingStreamTerminationDateBusinessCenter";   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTerminationDateRelativeTo] = "UnderlyingStreamTerminationDateRelativeTo";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTerminationDateOffsetPeriod] = "UnderlyingStreamTerminationDateOffsetPeriod";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTerminationDateOffsetUnit] = "UnderlyingStreamTerminationDateOffsetUnit";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTerminationDateOffsetDayType] = "UnderlyingStreamTerminationDateOffsetDayType";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTerminationDateAdjusted] = "UnderlyingStreamTerminationDateAdjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationPeriodBusinessDayConvention] = "UnderlyingStreamCalculationPeriodBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationPeriodBusinessCenter] = "UnderlyingStreamCalculationPeriodBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamFirstPeriodStartDateUnadjusted] = "UnderlyingStreamFirstPeriodStartDateUnadjusted";  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamFirstPeriodStartDateBusinessDayConvention] = "UnderlyingStreamFirstPeriodStartDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamFirstPeriodStartDateBusinessCenter] = "UnderlyingStreamFirstPeriodStartDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamFirstPeriodStartDateAdjusted] = "UnderlyingStreamFirstPeriodStartDateAdjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamFirstRegularPeriodStartDateUnadjusted] = "UnderlyingStreamFirstRegularPeriodStartDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamFirstCompoundingPeriodEndDateUnadjusted] = "UnderlyingStreamFirstCompoundingPeriodEndDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamLastRegularPeriodEndDateUnadjusted] = "UnderlyingStreamLastRegularPeriodEndDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationFrequencyPeriod] = "UnderlyingStreamCalculationFrequencyPeriod";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationFrequencyUnit] = "UnderlyingStreamCalculationFrequencyUnit";        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationRollConvention] = "UnderlyingStreamCalculationRollConvention";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamType]       = "UnderlyingPaymentStreamType";                     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamMarketRate] = "UnderlyingPaymentStreamMarketRate";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamDelayIndicator] = "UnderlyingPaymentStreamDelayIndicator";           // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamSettlCurrency] = "UnderlyingPaymentStreamSettlCurrency";            // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamDayCount]   = "UnderlyingPaymentStreamDayCount";                 // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamAccrualDays] = "UnderlyingPaymentStreamAccrualDays";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamDiscountType] = "UnderlyingPaymentStreamDiscountType";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamDiscountRate] = "UnderlyingPaymentStreamDiscountRate";             // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamDiscountRateDayCount] = "UnderlyingPaymentStreamDiscountRateDayCount";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingMethod] = "UnderlyingPaymentStreamCompoundingMethod";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialPrincipalExchangeIndicator] = "UnderlyingPaymentStreamInitialPrincipalExchangeIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInterimPrincipalExchangeIndicator] = "UnderlyingPaymentStreamInterimPrincipalExchangeIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalPrincipalExchangeIndicator] = "UnderlyingPaymentStreamFinalPrincipalExchangeIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentDateBusinessDayConvention] = "UnderlyingPaymentStreamPaymentDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentDateBusinessCenter] = "UnderlyingPaymentStreamPaymentDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentFrequencyPeriod] = "UnderlyingPaymentStreamPaymentFrequencyPeriod";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentFrequencyUnit] = "UnderlyingPaymentStreamPaymentFrequencyUnit";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentRollConvention] = "UnderlyingPaymentStreamPaymentRollConvention";    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFirstPaymentDateUnadjusted] = "UnderlyingPaymentStreamFirstPaymentDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLastRegularPaymentDateUnadjusted] = "UnderlyingPaymentStreamLastRegularPaymentDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentDateRelativeTo] = "UnderlyingPaymentStreamPaymentDateRelativeTo";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentDateOffsetPeriod] = "UnderlyingPaymentStreamPaymentDateOffsetPeriod";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentDateOffsetUnit] = "UnderlyingPaymentStreamPaymentDateOffsetUnit";    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentDateOffsetDayType] = "UnderlyingPaymentStreamPaymentDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamResetDateRelativeTo] = "UnderlyingPaymentStreamResetDateRelativeTo";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamResetDateBusinessDayConvention] = "UnderlyingPaymentStreamResetDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamResetDateBusinessCenter] = "UnderlyingPaymentStreamResetDateBusinessCenter";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamResetFrequencyPeriod] = "UnderlyingPaymentStreamResetFrequencyPeriod";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamResetFrequencyUnit] = "UnderlyingPaymentStreamResetFrequencyUnit";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamResetWeeklyRollConvention] = "UnderlyingPaymentStreamResetWeeklyRollConvention"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialFixingDateRelativeTo] = "UnderlyingPaymentStreamInitialFixingDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialFixingDateBusinessDayConvention] = "UnderlyingPaymentStreamInitialFixingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialFixingDateBusinessCenter] = "UnderlyingPaymentStreamInitialFixingDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialFixingDateOffsetPeriod] = "UnderlyingPaymentStreamInitialFixingDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialFixingDateOffsetUnit] = "UnderlyingPaymentStreamInitialFixingDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialFixingDateOffsetDayType] = "UnderlyingPaymentStreamInitialFixingDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialFixingDateAdjusted] = "UnderlyingPaymentStreamInitialFixingDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDateRelativeTo] = "UnderlyingPaymentStreamFixingDateRelativeTo";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDateBusinessDayConvention] = "UnderlyingPaymentStreamFixingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDateBusinessCenter] = "UnderlyingPaymentStreamFixingDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDateOffsetPeriod] = "UnderlyingPaymentStreamFixingDateOffsetPeriod";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDateOffsetUnit] = "UnderlyingPaymentStreamFixingDateOffsetUnit";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDateOffsetDayType] = "UnderlyingPaymentStreamFixingDateOffsetDayType";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDateAdjusted] = "UnderlyingPaymentStreamFixingDateAdjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateCutoffDateOffsetPeriod] = "UnderlyingPaymentStreamRateCutoffDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateCutoffDateOffsetUnit] = "UnderlyingPaymentStreamRateCutoffDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateCutoffDateOffsetDayType] = "UnderlyingPaymentStreamRateCutoffDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRate]       = "UnderlyingPaymentStreamRate";                     // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixedAmount] = "UnderlyingPaymentStreamFixedAmount";              // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateOrAmountCurrency] = "UnderlyingPaymentStreamRateOrAmountCurrency";     // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFutureValueNotional] = "UnderlyingPaymentStreamFutureValueNotional";      // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFutureValueDateAdjusted] = "UnderlyingPaymentStreamFutureValueDateAdjusted";  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndex]  = "UnderlyingPaymentStreamRateIndex";                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndexSource] = "UnderlyingPaymentStreamRateIndexSource";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndexCurveUnit] = "UnderlyingPaymentStreamRateIndexCurveUnit";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndexCurvePeriod] = "UnderlyingPaymentStreamRateIndexCurvePeriod";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateMultiplier] = "UnderlyingPaymentStreamRateMultiplier";           // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateSpread] = "UnderlyingPaymentStreamRateSpread";               // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateSpreadPositionType] = "UnderlyingPaymentStreamRateSpreadPositionType";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateTreatment] = "UnderlyingPaymentStreamRateTreatment";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCapRate]    = "UnderlyingPaymentStreamCapRate";                  // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCapRateBuySide] = "UnderlyingPaymentStreamCapRateBuySide";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCapRateSellSide] = "UnderlyingPaymentStreamCapRateSellSide";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFloorRate]  = "UnderlyingPaymentStreamFloorRate";                // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFloorRateBuySide] = "UnderlyingPaymentStreamFloorRateBuySide";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFloorRateSellSide] = "UnderlyingPaymentStreamFloorRateSellSide";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInitialRate] = "UnderlyingPaymentStreamInitialRate";              // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalRateRoundingDirection] = "UnderlyingPaymentStreamFinalRateRoundingDirection"; // (char FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalRatePrecision] = "UnderlyingPaymentStreamFinalRatePrecision";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamAveragingMethod] = "UnderlyingPaymentStreamAveragingMethod";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNegativeRateTreatment] = "UnderlyingPaymentStreamNegativeRateTreatment";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInflationLagPeriod] = "UnderlyingPaymentStreamInflationLagPeriod";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInflationLagUnit] = "UnderlyingPaymentStreamInflationLagUnit";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInflationLagDayType] = "UnderlyingPaymentStreamInflationLagDayType";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInflationInterpolationMethod] = "UnderlyingPaymentStreamInflationInterpolationMethod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInflationIndexSource] = "UnderlyingPaymentStreamInflationIndexSource";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInflationPublicationSource] = "UnderlyingPaymentStreamInflationPublicationSource"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInflationInitialIndexLevel] = "UnderlyingPaymentStreamInflationInitialIndexLevel"; // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInflationFallbackBondApplicable] = "UnderlyingPaymentStreamInflationFallbackBondApplicable"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFRADiscounting] = "UnderlyingPaymentStreamFRADiscounting";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableRefCurrency] = "UnderlyingPaymentStreamNonDeliverableRefCurrency"; // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableFixingDatesBusinessDayConvention] = "UnderlyingPaymentStreamNonDeliverableFixingDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableFixingDatesBusinessCenter] = "UnderlyingPaymentStreamNonDeliverableFixingDatesBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableFixingDatesRelativeTo] = "UnderlyingPaymentStreamNonDeliverableFixingDatesRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetPeriod] = "UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetUnit] = "UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetDayType] = "UnderlyingPaymentStreamNonDeliverableFixingDatesOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::SettlRateFallbackReferencePage]    = "SettlRateFallbackReferencePage";                  // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingNonDeliverableFixingDates] = "NoUnderlyingNonDeliverableFixingDates";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingNonDeliverableFixingDate] = "UnderlyingNonDeliverableFixingDate";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingNonDeliverableFixingDateType] = "UnderlyingNonDeliverableFixingDateType";          // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingSettlRateFallbacks]    = "NoUnderlyingSettlRateFallbacks";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingSettlRatePostponementMaximumDays] = "UnderlyingSettlRatePostponementMaximumDays";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableSettlRateSource] = "UnderlyingPaymentStreamNonDeliverableSettlRateSource"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSettlRatePostponementSurvey] = "UnderlyingSettlRatePostponementSurvey";           // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingSettlRatePostponementCalculationAgent] = "UnderlyingSettlRatePostponementCalculationAgent"; // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentSchedules]      = "NoUnderlyingPaymentSchedules";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleType]     = "UnderlyingPaymentScheduleType";                   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStubType] = "UnderlyingPaymentScheduleStubType";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStartDateUnadjusted] = "UnderlyingPaymentScheduleStartDateUnadjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleEndDateUnadjusted] = "UnderlyingPaymentScheduleEndDateUnadjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentSchedulePaySide]  = "UnderlyingPaymentSchedulePaySide";                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleReceiveSide] = "UnderlyingPaymentScheduleReceiveSide";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleNotional] = "UnderlyingPaymentScheduleNotional";               // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleCurrency] = "UnderlyingPaymentScheduleCurrency";               // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRate]     = "UnderlyingPaymentScheduleRate";                   // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateMultiplier] = "UnderlyingPaymentScheduleRateMultiplier";         // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateSpread] = "UnderlyingPaymentScheduleRateSpread";             // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateSpreadPositionType] = "UnderlyingPaymentScheduleRateSpreadPositionType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateTreatment] = "UnderlyingPaymentScheduleRateTreatment";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixedAmount] = "UnderlyingPaymentScheduleFixedAmount";            // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixedCurrency] = "UnderlyingPaymentScheduleFixedCurrency";          // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStepFrequencyPeriod] = "UnderlyingPaymentScheduleStepFrequencyPeriod";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStepFrequencyUnit] = "UnderlyingPaymentScheduleStepFrequencyUnit";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStepOffsetValue] = "UnderlyingPaymentScheduleStepOffsetValue";        // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStepRate] = "UnderlyingPaymentScheduleStepRate";               // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStepOffsetRate] = "UnderlyingPaymentScheduleStepOffsetRate";         // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStepRelativeTo] = "UnderlyingPaymentScheduleStepRelativeTo";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDateUnadjusted] = "UnderlyingPaymentScheduleFixingDateUnadjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleWeight]   = "UnderlyingPaymentScheduleWeight";                 // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDateRelativeTo] = "UnderlyingPaymentScheduleFixingDateRelativeTo";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDateBusinessDayCnvtn] = "UnderlyingPaymentScheduleFixingDateBusinessDayCnvtn"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDateBusinessCenter] = "UnderlyingPaymentScheduleFixingDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDateOffsetPeriod] = "UnderlyingPaymentScheduleFixingDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDateOffsetUnit] = "UnderlyingPaymentScheduleFixingDateOffsetUnit";   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDateOffsetDayType] = "UnderlyingPaymentScheduleFixingDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDateAdjusted] = "UnderlyingPaymentScheduleFixingDateAdjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingTime] = "UnderlyingPaymentScheduleFixingTime";             // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingTimeBusinessCenter] = "UnderlyingPaymentScheduleFixingTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleInterimExchangePaymentDateRelativeTo] = "UnderlyingPaymentScheduleInterimExchangePaymentDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleInterimExchangeDatesBusinessDayConvention] = "UnderlyingPaymentScheduleInterimExchangeDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleInterimExchangeDatesBusinessCenter] = "UnderlyingPaymentScheduleInterimExchangeDatesBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleInterimExchangeDatesOffsetPeriod] = "UnderlyingPaymentScheduleInterimExchangeDatesOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleInterimExchangeDatesOffsetUnit] = "UnderlyingPaymentScheduleInterimExchangeDatesOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleInterimExchangeDatesOffsetDayType] = "UnderlyingPaymentScheduleInterimExchangeDatesOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleInterimExchangeDateAdjusted] = "UnderlyingPaymentScheduleInterimExchangeDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentScheduleRateSources] = "NoUnderlyingPaymentScheduleRateSources";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateSource] = "UnderlyingPaymentScheduleRateSource";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateSourceType] = "UnderlyingPaymentScheduleRateSourceType";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleReferencePage] = "UnderlyingPaymentScheduleReferencePage";          // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStubs]          = "NoUnderlyingPaymentStubs";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubType]         = "UnderlyingPaymentStubType";                       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubLength]       = "UnderlyingPaymentStubLength";                     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubRate]         = "UnderlyingPaymentStubRate";                       // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubFixedAmount]  = "UnderlyingPaymentStubFixedAmount";                // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubFixedCurrency] = "UnderlyingPaymentStubFixedCurrency";              // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex]        = "UnderlyingPaymentStubIndex";                      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexSource]  = "UnderlyingPaymentStubIndexSource";                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexCurvePeriod] = "UnderlyingPaymentStubIndexCurvePeriod";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexCurveUnit] = "UnderlyingPaymentStubIndexCurveUnit";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexRateMultiplier] = "UnderlyingPaymentStubIndexRateMultiplier";        // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexRateSpread] = "UnderlyingPaymentStubIndexRateSpread";            // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexRateSpreadPositionType] = "UnderlyingPaymentStubIndexRateSpreadPositionType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexRateTreatment] = "UnderlyingPaymentStubIndexRateTreatment";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexCapRate] = "UnderlyingPaymentStubIndexCapRate";               // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexCapRateBuySide] = "UnderlyingPaymentStubIndexCapRateBuySide";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexCapRateSellSide] = "UnderlyingPaymentStubIndexCapRateSellSide";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexFloorRate] = "UnderlyingPaymentStubIndexFloorRate";             // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexFloorRateBuySide] = "UnderlyingPaymentStubIndexFloorRateBuySide";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndexFloorRateSellSide] = "UnderlyingPaymentStubIndexFloorRateSellSide";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2]       = "UnderlyingPaymentStubIndex2";                     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2Source] = "UnderlyingPaymentStubIndex2Source";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2CurvePeriod] = "UnderlyingPaymentStubIndex2CurvePeriod";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2CurveUnit] = "UnderlyingPaymentStubIndex2CurveUnit";            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2RateMultiplier] = "UnderlyingPaymentStubIndex2RateMultiplier";       // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2RateSpread] = "UnderlyingPaymentStubIndex2RateSpread";           // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2RateSpreadPositionType] = "UnderlyingPaymentStubIndex2RateSpreadPositionType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2RateTreatment] = "UnderlyingPaymentStubIndex2RateTreatment";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2CapRate] = "UnderlyingPaymentStubIndex2CapRate";              // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubIndex2FloorRate] = "UnderlyingPaymentStubIndex2FloorRate";            // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamType]                 = "PaymentStreamType";                               // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamMarketRate]           = "PaymentStreamMarketRate";                         // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamDelayIndicator]       = "PaymentStreamDelayIndicator";                     // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamSettlCurrency]        = "PaymentStreamSettlCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStreamDayCount]             = "PaymentStreamDayCount";                           // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamAccrualDays]          = "PaymentStreamAccrualDays";                        // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamDiscountType]         = "PaymentStreamDiscountType";                       // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamDiscountRate]         = "PaymentStreamDiscountRate";                       // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamDiscountRateDayCount] = "PaymentStreamDiscountRateDayCount";               // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingMethod]    = "PaymentStreamCompoundingMethod";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialPrincipalExchangeIndicator] = "PaymentStreamInitialPrincipalExchangeIndicator";  // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamInterimPrincipalExchangeIndicator] = "PaymentStreamInterimPrincipalExchangeIndicator";  // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalPrincipalExchangeIndicator] = "PaymentStreamFinalPrincipalExchangeIndicator";    // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentDateBusinessDayConvention] = "PaymentStreamPaymentDateBusinessDayConvention";   // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentDateBusinessCenter] = "PaymentStreamPaymentDateBusinessCenter";          // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentFrequencyPeriod] = "PaymentStreamPaymentFrequencyPeriod";             // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentFrequencyUnit] = "PaymentStreamPaymentFrequencyUnit";               // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentRollConvention] = "PaymentStreamPaymentRollConvention";              // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamFirstPaymentDateUnadjusted] = "PaymentStreamFirstPaymentDateUnadjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamLastRegularPaymentDateUnadjusted] = "PaymentStreamLastRegularPaymentDateUnadjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentDateRelativeTo] = "PaymentStreamPaymentDateRelativeTo";              // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentDateOffsetPeriod] = "PaymentStreamPaymentDateOffsetPeriod";            // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentDateOffsetUnit] = "PaymentStreamPaymentDateOffsetUnit";              // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamResetDateRelativeTo]  = "PaymentStreamResetDateRelativeTo";                // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamResetDateBusinessDayConvention] = "PaymentStreamResetDateBusinessDayConvention";     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamResetDateBusinessCenter] = "PaymentStreamResetDateBusinessCenter";            // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamResetFrequencyPeriod] = "PaymentStreamResetFrequencyPeriod";               // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamResetFrequencyUnit]   = "PaymentStreamResetFrequencyUnit";                 // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamResetWeeklyRollConvention] = "PaymentStreamResetWeeklyRollConvention";          // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialFixingDateRelativeTo] = "PaymentStreamInitialFixingDateRelativeTo";        // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialFixingDateBusinessDayConvention] = "PaymentStreamInitialFixingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialFixingDateBusinessCenter] = "PaymentStreamInitialFixingDateBusinessCenter";    // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialFixingDateOffsetPeriod] = "PaymentStreamInitialFixingDateOffsetPeriod";      // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialFixingDateOffsetUnit] = "PaymentStreamInitialFixingDateOffsetUnit";        // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialFixingDateOffsetDayType] = "PaymentStreamInitialFixingDateOffsetDayType";     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialFixingDateAdjusted] = "PaymentStreamInitialFixingDateAdjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDateRelativeTo] = "PaymentStreamFixingDateRelativeTo";               // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDateBusinessDayConvention] = "PaymentStreamFixingDateBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDateBusinessCenter] = "PaymentStreamFixingDateBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDateOffsetPeriod] = "PaymentStreamFixingDateOffsetPeriod";             // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDateOffsetUnit] = "PaymentStreamFixingDateOffsetUnit";               // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDateOffsetDayType] = "PaymentStreamFixingDateOffsetDayType";            // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDateAdjusted]   = "PaymentStreamFixingDateAdjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamRateCutoffDateOffsetPeriod] = "PaymentStreamRateCutoffDateOffsetPeriod";         // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamRateCutoffDateOffsetUnit] = "PaymentStreamRateCutoffDateOffsetUnit";           // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateCutoffDateOffsetDayType] = "PaymentStreamRateCutoffDateOffsetDayType";        // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamRate]                 = "PaymentStreamRate";                               // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamFixedAmount]          = "PaymentStreamFixedAmount";                        // (Amt FIX.5.0SP2)
dictionary[tag::PaymentStreamRateOrAmountCurrency] = "PaymentStreamRateOrAmountCurrency";               // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStreamFutureValueNotional]  = "PaymentStreamFutureValueNotional";                // (Amt FIX.5.0SP2)
dictionary[tag::PaymentStreamFutureValueDateAdjusted] = "PaymentStreamFutureValueDateAdjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndex]            = "PaymentStreamRateIndex";                          // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndexSource]      = "PaymentStreamRateIndexSource";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndexCurveUnit]   = "PaymentStreamRateIndexCurveUnit";                 // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndexCurvePeriod] = "PaymentStreamRateIndexCurvePeriod";               // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamRateMultiplier]       = "PaymentStreamRateMultiplier";                     // (float FIX.5.0SP2)
dictionary[tag::PaymentStreamRateSpread]           = "PaymentStreamRateSpread";                         // (PriceOffset FIX.5.0SP2)
dictionary[tag::PaymentStreamRateSpreadPositionType] = "PaymentStreamRateSpreadPositionType";             // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamRateTreatment]        = "PaymentStreamRateTreatment";                      // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCapRate]              = "PaymentStreamCapRate";                            // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamCapRateBuySide]       = "PaymentStreamCapRateBuySide";                     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCapRateSellSide]      = "PaymentStreamCapRateSellSide";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFloorRate]            = "PaymentStreamFloorRate";                          // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamFloorRateBuySide]     = "PaymentStreamFloorRateBuySide";                   // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFloorRateSellSide]    = "PaymentStreamFloorRateSellSide";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInitialRate]          = "PaymentStreamInitialRate";                        // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalRateRoundingDirection] = "PaymentStreamFinalRateRoundingDirection";         // (char FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalRatePrecision]   = "PaymentStreamFinalRatePrecision";                 // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamAveragingMethod]      = "PaymentStreamAveragingMethod";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamNegativeRateTreatment] = "PaymentStreamNegativeRateTreatment";              // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInflationLagPeriod]   = "PaymentStreamInflationLagPeriod";                 // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInflationLagUnit]     = "PaymentStreamInflationLagUnit";                   // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamInflationLagDayType]  = "PaymentStreamInflationLagDayType";                // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInflationInterpolationMethod] = "PaymentStreamInflationInterpolationMethod";       // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInflationIndexSource] = "PaymentStreamInflationIndexSource";               // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInflationPublicationSource] = "PaymentStreamInflationPublicationSource";         // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamInflationInitialIndexLevel] = "PaymentStreamInflationInitialIndexLevel";         // (float FIX.5.0SP2)
dictionary[tag::PaymentStreamInflationFallbackBondApplicable] = "PaymentStreamInflationFallbackBondApplicable";    // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamFRADiscounting]       = "PaymentStreamFRADiscounting";                     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableRefCurrency] = "PaymentStreamNonDeliverableRefCurrency";          // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableFixingDatesBusinessDayConvention] = "PaymentStreamNonDeliverableFixingDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableFixingDatesBusinessCenter] = "PaymentStreamNonDeliverableFixingDatesBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableFixingDatesRelativeTo] = "PaymentStreamNonDeliverableFixingDatesRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableFixingDatesOffsetPeriod] = "PaymentStreamNonDeliverableFixingDatesOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableFixingDatesOffsetUnit] = "PaymentStreamNonDeliverableFixingDatesOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamNonDeliverableFixingDatesOffsetDayType] = "PaymentStreamNonDeliverableFixingDatesOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNonDeliverableSettlReferencePage] = "UnderlyingPaymentStreamNonDeliverableSettlReferencePage"; // (String FIX.5.0SP2)
dictionary[tag::NoNonDeliverableFixingDates]       = "NoNonDeliverableFixingDates";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::NonDeliverableFixingDate]          = "NonDeliverableFixingDate";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NonDeliverableFixingDateType]      = "NonDeliverableFixingDateType";                    // (int FIX.5.0SP2)
dictionary[tag::NoPaymentSchedules]                = "NoPaymentSchedules";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentScheduleType]               = "PaymentScheduleType";                             // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleStubType]           = "PaymentScheduleStubType";                         // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleStartDateUnadjusted] = "PaymentScheduleStartDateUnadjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentScheduleEndDateUnadjusted]  = "PaymentScheduleEndDateUnadjusted";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentSchedulePaySide]            = "PaymentSchedulePaySide";                          // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleReceiveSide]        = "PaymentScheduleReceiveSide";                      // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleNotional]           = "PaymentScheduleNotional";                         // (Amt FIX.5.0SP2)
dictionary[tag::PaymentScheduleCurrency]           = "PaymentScheduleCurrency";                         // (Currency FIX.5.0SP2)
dictionary[tag::PaymentScheduleRate]               = "PaymentScheduleRate";                             // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateMultiplier]     = "PaymentScheduleRateMultiplier";                   // (float FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateSpread]         = "PaymentScheduleRateSpread";                       // (PriceOffset FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateSpreadPositionType] = "PaymentScheduleRateSpreadPositionType";           // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateTreatment]      = "PaymentScheduleRateTreatment";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixedAmount]        = "PaymentScheduleFixedAmount";                      // (Amt FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixedCurrency]      = "PaymentScheduleFixedCurrency";                    // (Currency FIX.5.0SP2)
dictionary[tag::PaymentScheduleStepFrequencyPeriod] = "PaymentScheduleStepFrequencyPeriod";              // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleStepFrequencyUnit]  = "PaymentScheduleStepFrequencyUnit";                // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleStepOffsetValue]    = "PaymentScheduleStepOffsetValue";                  // (Amt FIX.5.0SP2)
dictionary[tag::PaymentScheduleStepRate]           = "PaymentScheduleStepRate";                         // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentScheduleStepOffsetRate]     = "PaymentScheduleStepOffsetRate";                   // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentScheduleStepRelativeTo]     = "PaymentScheduleStepRelativeTo";                   // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDateUnadjusted] = "PaymentScheduleFixingDateUnadjusted";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentScheduleWeight]             = "PaymentScheduleWeight";                           // (float FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDateRelativeTo] = "PaymentScheduleFixingDateRelativeTo";             // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDateBusinessDayConvention] = "PaymentScheduleFixingDateBusinessDayConvention";  // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDateBusinessCenter] = "PaymentScheduleFixingDateBusinessCenter";         // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDateOffsetPeriod] = "PaymentScheduleFixingDateOffsetPeriod";           // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDateOffsetUnit] = "PaymentScheduleFixingDateOffsetUnit";             // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDateOffsetDayType] = "PaymentScheduleFixingDateOffsetDayType";          // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDateAdjusted] = "PaymentScheduleFixingDateAdjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingTime]         = "PaymentScheduleFixingTime";                       // (LocalMktTime FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingTimeBusinessCenter] = "PaymentScheduleFixingTimeBusinessCenter";         // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleInterimExchangePaymentDateRelativeTo] = "PaymentScheduleInterimExchangePaymentDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleInterimExchangeDatesBusinessDayConvention] = "PaymentScheduleInterimExchangeDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleInterimExchangeDatesBusinessCenter] = "PaymentScheduleInterimExchangeDatesBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleInterimExchangeDatesOffsetPeriod] = "PaymentScheduleInterimExchangeDatesOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleInterimExchangeDatesOffsetUnit] = "PaymentScheduleInterimExchangeDatesOffsetUnit";   // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleInterimExchangeDatesOffsetDayType] = "PaymentScheduleInterimExchangeDatesOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleInterimExchangeDateAdjusted] = "PaymentScheduleInterimExchangeDateAdjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoPaymentScheduleRateSources]      = "NoPaymentScheduleRateSources";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateSource]         = "PaymentScheduleRateSource";                       // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateSourceType]     = "PaymentScheduleRateSourceType";                   // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleReferencePage]      = "PaymentScheduleReferencePage";                    // (String FIX.5.0SP2)
dictionary[tag::NoPaymentStubs]                    = "NoPaymentStubs";                                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStubType]                   = "PaymentStubType";                                 // (int FIX.5.0SP2)
dictionary[tag::PaymentStubLength]                 = "PaymentStubLength";                               // (int FIX.5.0SP2)
dictionary[tag::PaymentStubRate]                   = "PaymentStubRate";                                 // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStubFixedAmount]            = "PaymentStubFixedAmount";                          // (Amt FIX.5.0SP2)
dictionary[tag::PaymentStubFixedCurrency]          = "PaymentStubFixedCurrency";                        // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStubIndex]                  = "PaymentStubIndex";                                // (String FIX.5.0SP2)
dictionary[tag::PaymentStubIndexSource]            = "PaymentStubIndexSource";                          // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndexCurvePeriod]       = "PaymentStubIndexCurvePeriod";                     // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndexCurveUnit]         = "PaymentStubIndexCurveUnit";                       // (String FIX.5.0SP2)
dictionary[tag::PaymentStubIndexRateMultiplier]    = "PaymentStubIndexRateMultiplier";                  // (float FIX.5.0SP2)
dictionary[tag::PaymentStubIndexRateSpread]        = "PaymentStubIndexRateSpread";                      // (PriceOffset FIX.5.0SP2)
dictionary[tag::PaymentStubIndexRateSpreadPositionType] = "PaymentStubIndexRateSpreadPositionType";          // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndexRateTreatment]     = "PaymentStubIndexRateTreatment";                   // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndexCapRate]           = "PaymentStubIndexCapRate";                         // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStubIndexCapRateBuySide]    = "PaymentStubIndexCapRateBuySide";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndexCapRateSellSide]   = "PaymentStubIndexCapRateSellSide";                 // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndexFloorRate]         = "PaymentStubIndexFloorRate";                       // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStubIndexFloorRateBuySide]  = "PaymentStubIndexFloorRateBuySide";                // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndexFloorRateSellSide] = "PaymentStubIndexFloorRateSellSide";               // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2]                 = "PaymentStubIndex2";                               // (String FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2Source]           = "PaymentStubIndex2Source";                         // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2CurvePeriod]      = "PaymentStubIndex2CurvePeriod";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2CurveUnit]        = "PaymentStubIndex2CurveUnit";                      // (String FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2RateMultiplier]   = "PaymentStubIndex2RateMultiplier";                 // (float FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2RateSpread]       = "PaymentStubIndex2RateSpread";                     // (PriceOffset FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2RateSpreadPositionType] = "PaymentStubIndex2RateSpreadPositionType";         // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2RateTreatment]    = "PaymentStubIndex2RateTreatment";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2CapRate]          = "PaymentStubIndex2CapRate";                        // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStubIndex2FloorRate]        = "PaymentStubIndex2FloorRate";                      // (Percentage FIX.5.0SP2)
dictionary[tag::NoLegSettlRateFallbacks]           = "NoLegSettlRateFallbacks";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegSettlRatePostponementMaximumDays] = "LegSettlRatePostponementMaximumDays";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSettlRateFallbackRateSource] = "UnderlyingSettlRateFallbackRateSource";           // (int FIX.5.0SP2)
dictionary[tag::LegSettlRatePostponementSurvey]    = "LegSettlRatePostponementSurvey";                  // (Boolean FIX.5.0SP2)
dictionary[tag::LegSettlRatePostponementCalculationAgent] = "LegSettlRatePostponementCalculationAgent";        // (int FIX.5.0SP2)
dictionary[tag::StreamEffectiveDateUnadjusted]     = "StreamEffectiveDateUnadjusted";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamEffectiveDateBusinessDayConvention] = "StreamEffectiveDateBusinessDayConvention";        // (int FIX.5.0SP2)
dictionary[tag::StreamEffectiveDateBusinessCenter] = "StreamEffectiveDateBusinessCenter";               // (String FIX.5.0SP2)
dictionary[tag::StreamEffectiveDateRelativeTo]     = "StreamEffectiveDateRelativeTo";                   // (int FIX.5.0SP2)
dictionary[tag::StreamEffectiveDateOffsetPeriod]   = "StreamEffectiveDateOffsetPeriod";                 // (int FIX.5.0SP2)
dictionary[tag::StreamEffectiveDateOffsetUnit]     = "StreamEffectiveDateOffsetUnit";                   // (String FIX.5.0SP2)
dictionary[tag::StreamEffectiveDateOffsetDayType]  = "StreamEffectiveDateOffsetDayType";                // (int FIX.5.0SP2)
dictionary[tag::StreamEffectiveDateAdjusted]       = "StreamEffectiveDateAdjusted";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingSettlRateFallbackReferencePage] = "UnderlyingSettlRateFallbackReferencePage";        // (String FIX.5.0SP2)
dictionary[tag::CashSettlValuationSubsequentBusinessDaysOffset] = "CashSettlValuationSubsequentBusinessDaysOffset";  // (int FIX.5.0SP2)
dictionary[tag::CashSettlNumOfValuationDates]      = "CashSettlNumOfValuationDates";                    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionPartyRoleQualifier] = "UnderlyingProvisionPartyRoleQualifier";           // (int FIX.5.0SP2)
dictionary[tag::PaymentPriceType]                  = "PaymentPriceType";                                // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentDateOffsetDayType] = "PaymentStreamPaymentDateOffsetDayType";           // (int FIX.5.0SP2)
dictionary[tag::BusinessDayConvention]             = "BusinessDayConvention";                           // (int FIX.5.0SP2)
dictionary[tag::DateRollConvention]                = "DateRollConvention";                              // (String FIX.5.0SP2)
dictionary[tag::NoLegBusinessCenters]              = "NoLegBusinessCenters";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegBusinessCenter]                 = "LegBusinessCenter";                               // (String FIX.5.0SP2)
dictionary[tag::LegBusinessDayConvention]          = "LegBusinessDayConvention";                        // (int FIX.5.0SP2)
dictionary[tag::LegDateRollConvention]             = "LegDateRollConvention";                           // (String FIX.5.0SP2)
dictionary[tag::NoLegPaymentScheduleFixingDateBusinessCenters] = "NoLegPaymentScheduleFixingDateBusinessCenters";   // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegPaymentScheduleInterimExchangeDateBusinessCenters] = "NoLegPaymentScheduleInterimExchangeDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamNonDeliverableFixingDateBusinessCenters] = "NoLegPaymentStreamNonDeliverableFixingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamPaymentDateBusinessCenters] = "NoLegPaymentStreamPaymentDateBusinessCenters";    // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamResetDateBusinessCenters] = "NoLegPaymentStreamResetDateBusinessCenters";      // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamInitialFixingDateBusinessCenters] = "NoLegPaymentStreamInitialFixingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamFixingDateBusinessCenters] = "NoLegPaymentStreamFixingDateBusinessCenters";     // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegProvisionCashSettlPaymentDateBusinessCenters] = "NoLegProvisionCashSettlPaymentDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegProvisionCashSettlValueDateBusinessCenters] = "NoLegProvisionCashSettlValueDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegProvisionOptionExerciseBusinessCenters] = "NoLegProvisionOptionExerciseBusinessCenters";     // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegProvisionOptionExpirationDateBusinessCenters] = "NoLegProvisionOptionExpirationDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegProvisionOptionRelevantUnderlyingDateBusinessCenters] = "NoLegProvisionOptionRelevantUnderlyingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegProvisionDateBusinessCenters] = "NoLegProvisionDateBusinessCenters";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegStreamCalculationPeriodBusinessCenters] = "NoLegStreamCalculationPeriodBusinessCenters";     // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegStreamFirstPeriodStartDateBusinessCenters] = "NoLegStreamFirstPeriodStartDateBusinessCenters";  // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegStreamEffectiveDateBusinessCenters] = "NoLegStreamEffectiveDateBusinessCenters";         // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoLegStreamTerminationDateBusinessCenters] = "NoLegStreamTerminationDateBusinessCenters";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPaymentBusinessCenters]          = "NoPaymentBusinessCenters";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPaymentScheduleInterimExchangeDateBusinessCenters] = "NoPaymentScheduleInterimExchangeDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPaymentStreamNonDeliverableFixingDatesBusinessCenters] = "NoPaymentStreamNonDeliverableFixingDatesBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPaymentStreamPaymentDateBusinessCenters] = "NoPaymentStreamPaymentDateBusinessCenters";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPaymentStreamResetDateBusinessCenters] = "NoPaymentStreamResetDateBusinessCenters";         // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPaymentStreamInitialFixingDateBusinessCenters] = "NoPaymentStreamInitialFixingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPaymentStreamFixingDateBusinessCenters] = "NoPaymentStreamFixingDateBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoProtectionTermEventNewsSources]  = "NoProtectionTermEventNewsSources";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoProvisionCashSettlPaymentDateBusinessCenters] = "NoProvisionCashSettlPaymentDateBusinessCenters";  // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoProvisionCashSettlValueDateBusinessCenters] = "NoProvisionCashSettlValueDateBusinessCenters";    // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoProvisionOptionExerciseBusinessCenters] = "NoProvisionOptionExerciseBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoProvisionOptionExpirationDateBusinessCenters] = "NoProvisionOptionExpirationDateBusinessCenters";  // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoProvisionOptionRelevantUnderlyingDateBusinessCenters] = "NoProvisionOptionRelevantUnderlyingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoProvisionDateBusinessCenters]    = "NoProvisionDateBusinessCenters";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoStreamCalculationPeriodBusinessCenters] = "NoStreamCalculationPeriodBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoStreamFirstPeriodStartDateBusinessCenters] = "NoStreamFirstPeriodStartDateBusinessCenters";     // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoStreamEffectiveBusinessCenters]  = "NoStreamEffectiveBusinessCenters";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoStreamTerminationDateBusinessCenters] = "NoStreamTerminationDateBusinessCenters";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingBusinessCenters]       = "NoUnderlyingBusinessCenters";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingBusinessCenter]          = "UnderlyingBusinessCenter";                        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingBusinessDayConvention]   = "UnderlyingBusinessDayConvention";                 // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDateRollConvention]      = "UnderlyingDateRollConvention";                    // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentScheduleFixingDateBusinessCenters] = "NoUnderlyingPaymentScheduleFixingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentScheduleInterimExchangeDateBusinessCenters] = "NoUnderlyingPaymentScheduleInterimExchangeDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamNonDeliverableFixingDatesBusinessCenters] = "NoUnderlyingPaymentStreamNonDeliverableFixingDatesBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamPaymentDateBusinessCenters] = "NoUnderlyingPaymentStreamPaymentDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamResetDateBusinessCenters] = "NoUnderlyingPaymentStreamResetDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamInitialFixingDateBusinessCenters] = "NoUnderlyingPaymentStreamInitialFixingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamFixingDateBusinessCenters] = "NoUnderlyingPaymentStreamFixingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamCalculationPeriodBusinessCenters] = "NoUnderlyingStreamCalculationPeriodBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamFirstPeriodStartDateBusinessCenters] = "NoUnderlyingStreamFirstPeriodStartDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamEffectiveDateBusinessCenters] = "NoUnderlyingStreamEffectiveDateBusinessCenters";  // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamTerminationDateBusinessCenters] = "NoUnderlyingStreamTerminationDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::NoPaymentScheduleFixingDateBusinessCenters] = "NoPaymentScheduleFixingDateBusinessCenters";      // (NumInGroup FIX.5.0SP2)
dictionary[tag::EncodedLegStreamTextLen]           = "EncodedLegStreamTextLen";                         // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegStreamText]              = "EncodedLegStreamText";                            // (data FIX.5.0SP2)
dictionary[tag::EncodedLegProvisionTextLen]        = "EncodedLegProvisionTextLen";                      // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegProvisionText]           = "EncodedLegProvisionText";                         // (data FIX.5.0SP2)
dictionary[tag::EncodedStreamTextLen]              = "EncodedStreamTextLen";                            // (Length FIX.5.0SP2)
dictionary[tag::EncodedStreamText]                 = "EncodedStreamText";                               // (data FIX.5.0SP2)
dictionary[tag::EncodedPaymentTextLen]             = "EncodedPaymentTextLen";                           // (Length FIX.5.0SP2)
dictionary[tag::EncodedPaymentText]                = "EncodedPaymentText";                              // (data FIX.5.0SP2)
dictionary[tag::EncodedProvisionTextLen]           = "EncodedProvisionTextLen";                         // (Length FIX.5.0SP2)
dictionary[tag::EncodedProvisionText]              = "EncodedProvisionText";                            // (data FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingStreamTextLen]    = "EncodedUnderlyingStreamTextLen";                  // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingStreamText]       = "EncodedUnderlyingStreamText";                     // (data FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackValue]  = "LegMarketDisruptionFallbackValue";                // (String FIX.5.0SP2)
dictionary[tag::MarketDisruptionValue]             = "MarketDisruptionValue";                           // (String FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackValue]     = "MarketDisruptionFallbackValue";                   // (String FIX.5.0SP2)
dictionary[tag::PaymentSubType]                    = "PaymentSubType";                                  // (int FIX.5.0SP2)
dictionary[tag::NoComplexEventAveragingObservations] = "NoComplexEventAveragingObservations";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventAveragingObservationNumber] = "ComplexEventAveragingObservationNumber";          // (int FIX.5.0SP2)
dictionary[tag::ComplexEventAveragingWeight]       = "ComplexEventAveragingWeight";                     // (float FIX.5.0SP2)
dictionary[tag::NoComplexEventCreditEvents]        = "NoComplexEventCreditEvents";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventType]       = "ComplexEventCreditEventType";                     // (String FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventValue]      = "ComplexEventCreditEventValue";                    // (String FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventCurrency]   = "ComplexEventCreditEventCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventPeriod]     = "ComplexEventCreditEventPeriod";                   // (int FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventUnit]       = "ComplexEventCreditEventUnit";                     // (String FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventDayType]    = "ComplexEventCreditEventDayType";                  // (int FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventRateSource] = "ComplexEventCreditEventRateSource";               // (int FIX.5.0SP2)
dictionary[tag::NoComplexEventCreditEventQualifiers] = "NoComplexEventCreditEventQualifiers";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventQualifier]  = "ComplexEventCreditEventQualifier";                // (char FIX.5.0SP2)
dictionary[tag::NoComplexEventPeriodDateTimes]     = "NoComplexEventPeriodDateTimes";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventPeriodDate]            = "ComplexEventPeriodDate";                          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ComplexEventPeriodTime]            = "ComplexEventPeriodTime";                          // (LocalMktTime FIX.5.0SP2)
dictionary[tag::NoComplexEventPeriods]             = "NoComplexEventPeriods";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventPeriodType]            = "ComplexEventPeriodType";                          // (int FIX.5.0SP2)
dictionary[tag::ComplexEventBusinessCenter]        = "ComplexEventBusinessCenter";                      // (String FIX.5.0SP2)
dictionary[tag::NoComplexEventRateSources]         = "NoComplexEventRateSources";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventRateSource]            = "ComplexEventRateSource";                          // (int FIX.5.0SP2)
dictionary[tag::ComplexEventRateSourceType]        = "ComplexEventRateSourceType";                      // (int FIX.5.0SP2)
dictionary[tag::ComplexEventReferencePage]         = "ComplexEventReferencePage";                       // (String FIX.5.0SP2)
dictionary[tag::ComplexEventReferencePageHeading]  = "ComplexEventReferencePageHeading";                // (String FIX.5.0SP2)
dictionary[tag::NoComplexEventDateBusinessCenters] = "NoComplexEventDateBusinessCenters";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventDateBusinessCenter]    = "ComplexEventDateBusinessCenter";                  // (String FIX.5.0SP2)
dictionary[tag::ComplexEventDateUnadjusted]        = "ComplexEventDateUnadjusted";                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ComplexEventDateRelativeTo]        = "ComplexEventDateRelativeTo";                      // (int FIX.5.0SP2)
dictionary[tag::ComplexEventDateOffsetPeriod]      = "ComplexEventDateOffsetPeriod";                    // (int FIX.5.0SP2)
dictionary[tag::ComplexEventDateOffsetUnit]        = "ComplexEventDateOffsetUnit";                      // (String FIX.5.0SP2)
dictionary[tag::ComplexEventDateOffsetDayType]     = "ComplexEventDateOffsetDayType";                   // (int FIX.5.0SP2)
dictionary[tag::ComplexEventDateBusinessDayConvention] = "ComplexEventDateBusinessDayConvention";           // (int FIX.5.0SP2)
dictionary[tag::ComplexEventDateAdjusted]          = "ComplexEventDateAdjusted";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ComplexEventFixingTime]            = "ComplexEventFixingTime";                          // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ComplexEventFixingTimeBusinessCenter] = "ComplexEventFixingTimeBusinessCenter";            // (String FIX.5.0SP2)
dictionary[tag::NoComplexEventCreditEventSources]  = "NoComplexEventCreditEventSources";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventCreditEventSource]     = "ComplexEventCreditEventSource";                   // (String FIX.5.0SP2)
dictionary[tag::NoComplexEventSchedules]           = "NoComplexEventSchedules";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::ComplexEventScheduleStartDate]     = "ComplexEventScheduleStartDate";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ComplexEventScheduleEndDate]       = "ComplexEventScheduleEndDate";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ComplexEventScheduleFrequencyPeriod] = "ComplexEventScheduleFrequencyPeriod";             // (int FIX.5.0SP2)
dictionary[tag::ComplexEventScheduleFrequencyUnit] = "ComplexEventScheduleFrequencyUnit";               // (String FIX.5.0SP2)
dictionary[tag::ComplexEventScheduleRollConvention] = "ComplexEventScheduleRollConvention";              // (String FIX.5.0SP2)
dictionary[tag::NoDeliverySchedules]               = "NoDeliverySchedules";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::DeliveryScheduleType]              = "DeliveryScheduleType";                            // (int FIX.5.0SP2)
dictionary[tag::DeliveryScheduleXID]               = "DeliveryScheduleXID";                             // (XID FIX.5.0SP2)
dictionary[tag::DeliveryScheduleNotional]          = "DeliveryScheduleNotional";                        // (Qty FIX.5.0SP2)
dictionary[tag::DeliveryScheduleNotionalUnitOfMeasure] = "DeliveryScheduleNotionalUnitOfMeasure";           // (String FIX.5.0SP2)
dictionary[tag::DeliveryScheduleNotionalCommodityFrequency] = "DeliveryScheduleNotionalCommodityFrequency";      // (int FIX.5.0SP2)
dictionary[tag::DeliveryScheduleNegativeTolerance] = "DeliveryScheduleNegativeTolerance";               // (float FIX.5.0SP2)
dictionary[tag::DeliverySchedulePositiveTolerance] = "DeliverySchedulePositiveTolerance";               // (float FIX.5.0SP2)
dictionary[tag::DeliveryScheduleToleranceUnitOfMeasure] = "DeliveryScheduleToleranceUnitOfMeasure";          // (String FIX.5.0SP2)
dictionary[tag::DeliveryScheduleToleranceType]     = "DeliveryScheduleToleranceType";                   // (int FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlCountry]      = "DeliveryScheduleSettlCountry";                    // (Country FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlTimeZone]     = "DeliveryScheduleSettlTimeZone";                   // (String FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlFlowType]     = "DeliveryScheduleSettlFlowType";                   // (int FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlHolidaysProcessingInstruction] = "DeliveryScheduleSettlHolidaysProcessingInstruction"; // (int FIX.5.0SP2)
dictionary[tag::NoDeliveryScheduleSettlDays]       = "NoDeliveryScheduleSettlDays";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlDay]          = "DeliveryScheduleSettlDay";                        // (int FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlTotalHours]   = "DeliveryScheduleSettlTotalHours";                 // (int FIX.5.0SP2)
dictionary[tag::NoDeliveryScheduleSettlTimes]      = "NoDeliveryScheduleSettlTimes";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlStart]        = "DeliveryScheduleSettlStart";                      // (String FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlEnd]          = "DeliveryScheduleSettlEnd";                        // (String FIX.5.0SP2)
dictionary[tag::DeliveryScheduleSettlTimeType]     = "DeliveryScheduleSettlTimeType";                   // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamType]                = "DeliveryStreamType";                              // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamPipeline]            = "DeliveryStreamPipeline";                          // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamEntryPoint]          = "DeliveryStreamEntryPoint";                        // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamWithdrawalPoint]     = "DeliveryStreamWithdrawalPoint";                   // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamDeliveryPoint]       = "DeliveryStreamDeliveryPoint";                     // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamDeliveryRestriction] = "DeliveryStreamDeliveryRestriction";               // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamDeliveryContingency] = "DeliveryStreamDeliveryContingency";               // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamDeliveryContingentPartySide] = "DeliveryStreamDeliveryContingentPartySide";       // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamDeliverAtSourceIndicator] = "DeliveryStreamDeliverAtSourceIndicator";          // (Boolean FIX.5.0SP2)
dictionary[tag::DeliveryStreamRiskApportionment]   = "DeliveryStreamRiskApportionment";                 // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamTitleTransferLocation] = "DeliveryStreamTitleTransferLocation";             // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamTitleTransferCondition] = "DeliveryStreamTitleTransferCondition";            // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamImporterOfRecord]    = "DeliveryStreamImporterOfRecord";                  // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamNegativeTolerance]   = "DeliveryStreamNegativeTolerance";                 // (float FIX.5.0SP2)
dictionary[tag::DeliveryStreamPositiveTolerance]   = "DeliveryStreamPositiveTolerance";                 // (float FIX.5.0SP2)
dictionary[tag::DeliveryStreamToleranceUnitOfMeasure] = "DeliveryStreamToleranceUnitOfMeasure";            // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamToleranceType]       = "DeliveryStreamToleranceType";                     // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamToleranceOptionSide] = "DeliveryStreamToleranceOptionSide";               // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamTotalPositiveTolerance] = "DeliveryStreamTotalPositiveTolerance";            // (Percentage FIX.5.0SP2)
dictionary[tag::DeliveryStreamTotalNegativeTolerance] = "DeliveryStreamTotalNegativeTolerance";            // (Percentage FIX.5.0SP2)
dictionary[tag::DeliveryStreamNotionalConversionFactor] = "DeliveryStreamNotionalConversionFactor";          // (float FIX.5.0SP2)
dictionary[tag::DeliveryStreamTransportEquipment]  = "DeliveryStreamTransportEquipment";                // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamElectingPartySide]   = "DeliveryStreamElectingPartySide";                 // (int FIX.5.0SP2)
dictionary[tag::NoDeliveryStreamCycles]            = "NoDeliveryStreamCycles";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::DeliveryStreamCycleDesc]           = "DeliveryStreamCycleDesc";                         // (String FIX.5.0SP2)
dictionary[tag::EncodedDeliveryStreamCycleDescLen] = "EncodedDeliveryStreamCycleDescLen";               // (Length FIX.5.0SP2)
dictionary[tag::EncodedDeliveryStreamCycleDesc]    = "EncodedDeliveryStreamCycleDesc";                  // (data FIX.5.0SP2)
dictionary[tag::NoDeliveryStreamCommoditySources]  = "NoDeliveryStreamCommoditySources";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::DeliveryStreamCommoditySource]     = "DeliveryStreamCommoditySource";                   // (String FIX.5.0SP2)
dictionary[tag::MarketDisruptionProvision]         = "MarketDisruptionProvision";                       // (int FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackProvision] = "MarketDisruptionFallbackProvision";               // (int FIX.5.0SP2)
dictionary[tag::MarketDisruptionMaximumDays]       = "MarketDisruptionMaximumDays";                     // (int FIX.5.0SP2)
dictionary[tag::MarketDisruptionMaterialityPercentage] = "MarketDisruptionMaterialityPercentage";           // (Percentage FIX.5.0SP2)
dictionary[tag::MarketDisruptionMinimumFuturesContracts] = "MarketDisruptionMinimumFuturesContracts";         // (int FIX.5.0SP2)
dictionary[tag::NoMarketDisruptionEvents]          = "NoMarketDisruptionEvents";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::MarketDisruptionEvent]             = "MarketDisruptionEvent";                           // (String FIX.5.0SP2)
dictionary[tag::NoMarketDisruptionFallbacks]       = "NoMarketDisruptionFallbacks";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackType]      = "MarketDisruptionFallbackType";                    // (String FIX.5.0SP2)
dictionary[tag::NoMarketDisruptionFallbackReferencePrices] = "NoMarketDisruptionFallbackReferencePrices";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackUnderlierType] = "MarketDisruptionFallbackUnderlierType";           // (int FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackUnderlierSecurityID] = "MarketDisruptionFallbackUnderlierSecurityID";     // (String FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackUnderlierSecurityIDSource] = "MarketDisruptionFallbackUnderlierSecurityIDSource"; // (String FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackUnderlierSecurityDesc] = "MarketDisruptionFallbackUnderlierSecurityDesc";   // (String FIX.5.0SP2)
dictionary[tag::EncodedMarketDisruptionFallbackUnderlierSecurityDescLen] = "EncodedMarketDisruptionFallbackUnderlierSecurityDescLen"; // (Length FIX.5.0SP2)
dictionary[tag::EncodedMarketDisruptionFallbackUnderlierSecurityDesc] = "EncodedMarketDisruptionFallbackUnderlierSecurityDesc"; // (data FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackOpenUnits] = "MarketDisruptionFallbackOpenUnits";               // (Qty FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackBasketCurrency] = "MarketDisruptionFallbackBasketCurrency";          // (Currency FIX.5.0SP2)
dictionary[tag::MarketDisruptionFallbackBasketDivisor] = "MarketDisruptionFallbackBasketDivisor";           // (float FIX.5.0SP2)
dictionary[tag::ExerciseDesc]                      = "ExerciseDesc";                                    // (String FIX.5.0SP2)
dictionary[tag::EncodedExerciseDescLen]            = "EncodedExerciseDescLen";                          // (Length FIX.5.0SP2)
dictionary[tag::EncodedExerciseDesc]               = "EncodedExerciseDesc";                             // (data FIX.5.0SP2)
dictionary[tag::AutomaticExerciseIndicator]        = "AutomaticExerciseIndicator";                      // (Boolean FIX.5.0SP2)
dictionary[tag::AutomaticExerciseThresholdRate]    = "AutomaticExerciseThresholdRate";                  // (float FIX.5.0SP2)
dictionary[tag::ExerciseConfirmationMethod]        = "ExerciseConfirmationMethod";                      // (int FIX.5.0SP2)
dictionary[tag::ManualNoticeBusinessCenter]        = "ManualNoticeBusinessCenter";                      // (String FIX.5.0SP2)
dictionary[tag::FallbackExerciseIndicator]         = "FallbackExerciseIndicator";                       // (Boolean FIX.5.0SP2)
dictionary[tag::LimitedRightToConfirmIndicator]    = "LimitedRightToConfirmIndicator";                  // (Boolean FIX.5.0SP2)
dictionary[tag::ExerciseSplitTicketIndicator]      = "ExerciseSplitTicketIndicator";                    // (Boolean FIX.5.0SP2)
dictionary[tag::NoOptionExerciseBusinessCenters]   = "NoOptionExerciseBusinessCenters";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::OptionExerciseBusinessCenter]      = "OptionExerciseBusinessCenter";                    // (String FIX.5.0SP2)
dictionary[tag::OptionExerciseBusinessDayConvention] = "OptionExerciseBusinessDayConvention";             // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseEarliestDateOffsetDayType] = "OptionExerciseEarliestDateOffsetDayType";         // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseEarliestDateOffsetPeriod] = "OptionExerciseEarliestDateOffsetPeriod";          // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseEarliestDateOffsetUnit] = "OptionExerciseEarliestDateOffsetUnit";            // (String FIX.5.0SP2)
dictionary[tag::OptionExerciseFrequencyPeriod]     = "OptionExerciseFrequencyPeriod";                   // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseFrequencyUnit]       = "OptionExerciseFrequencyUnit";                     // (String FIX.5.0SP2)
dictionary[tag::OptionExerciseStartDateUnadjusted] = "OptionExerciseStartDateUnadjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::OptionExerciseStartDateRelativeTo] = "OptionExerciseStartDateRelativeTo";               // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseStartDateOffsetPeriod] = "OptionExerciseStartDateOffsetPeriod";             // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseStartDateOffsetUnit] = "OptionExerciseStartDateOffsetUnit";               // (String FIX.5.0SP2)
dictionary[tag::OptionExerciseStartDateOffsetDayType] = "OptionExerciseStartDateOffsetDayType";            // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseStartDateAdjusted]   = "OptionExerciseStartDateAdjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::OptionExerciseSkip]                = "OptionExerciseSkip";                              // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseNominationDeadline]  = "OptionExerciseNominationDeadline";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::OptionExerciseFirstDateUnadjusted] = "OptionExerciseFirstDateUnadjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::OptionExerciseLastDateUnadjusted]  = "OptionExerciseLastDateUnadjusted";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::OptionExerciseEarliestTime]        = "OptionExerciseEarliestTime";                      // (LocalMktTime FIX.5.0SP2)
dictionary[tag::OptionExerciseLatestTime]          = "OptionExerciseLatestTime";                        // (LocalMktTime FIX.5.0SP2)
dictionary[tag::OptionExerciseTimeBusinessCenter]  = "OptionExerciseTimeBusinessCenter";                // (String FIX.5.0SP2)
dictionary[tag::NoOptionExerciseDates]             = "NoOptionExerciseDates";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::OptionExerciseDate]                = "OptionExerciseDate";                              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::OptionExerciseDateType]            = "OptionExerciseDateType";                          // (int FIX.5.0SP2)
dictionary[tag::NoOptionExerciseExpirationDateBusinessCenters] = "NoOptionExerciseExpirationDateBusinessCenters";   // (NumInGroup FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationDateBusinessCenter] = "OptionExerciseExpirationDateBusinessCenter";      // (String FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationDateBusinessDayConvention] = "OptionExerciseExpirationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationDateRelativeTo] = "OptionExerciseExpirationDateRelativeTo";          // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationDateOffsetPeriod] = "OptionExerciseExpirationDateOffsetPeriod";        // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationDateOffsetUnit] = "OptionExerciseExpirationDateOffsetUnit";          // (String FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationFrequencyPeriod] = "OptionExerciseExpirationFrequencyPeriod";         // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationFrequencyUnit] = "OptionExerciseExpirationFrequencyUnit";           // (String FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationRollConvention] = "OptionExerciseExpirationRollConvention";          // (String FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationDateOffsetDayType] = "OptionExerciseExpirationDateOffsetDayType";       // (int FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationTime]      = "OptionExerciseExpirationTime";                    // (LocalMktTime FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationTimeBusinessCenter] = "OptionExerciseExpirationTimeBusinessCenter";      // (String FIX.5.0SP2)
dictionary[tag::NoOptionExerciseExpirationDates]   = "NoOptionExerciseExpirationDates";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationDate]      = "OptionExerciseExpirationDate";                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::OptionExerciseExpirationDateType]  = "OptionExerciseExpirationDateType";                // (int FIX.5.0SP2)
dictionary[tag::PaymentUnitOfMeasure]              = "PaymentUnitOfMeasure";                            // (String FIX.5.0SP2)
dictionary[tag::PaymentDateRelativeTo]             = "PaymentDateRelativeTo";                           // (int FIX.5.0SP2)
dictionary[tag::PaymentDateOffsetPeriod]           = "PaymentDateOffsetPeriod";                         // (int FIX.5.0SP2)
dictionary[tag::PaymentDateOffsetUnit]             = "PaymentDateOffsetUnit";                           // (String FIX.5.0SP2)
dictionary[tag::PaymentDateOffsetDayType]          = "PaymentDateOffsetDayType";                        // (int FIX.5.0SP2)
dictionary[tag::PaymentForwardStartType]           = "PaymentForwardStartType";                         // (int FIX.5.0SP2)
dictionary[tag::NoPaymentScheduleFixingDays]       = "NoPaymentScheduleFixingDays";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDayOfWeek]    = "PaymentScheduleFixingDayOfWeek";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDayNumber]    = "PaymentScheduleFixingDayNumber";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleXID]                = "PaymentScheduleXID";                              // (XID FIX.5.0SP2)
dictionary[tag::PaymentScheduleXIDRef]             = "PaymentScheduleXIDRef";                           // (XIDREF FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateCurrency]       = "PaymentScheduleRateCurrency";                     // (Currency FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateUnitOfMeasure]  = "PaymentScheduleRateUnitOfMeasure";                // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateConversionFactor] = "PaymentScheduleRateConversionFactor";             // (float FIX.5.0SP2)
dictionary[tag::PaymentScheduleRateSpreadType]     = "PaymentScheduleRateSpreadType";                   // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleSettlPeriodPrice]   = "PaymentScheduleSettlPeriodPrice";                 // (Price FIX.5.0SP2)
dictionary[tag::PaymentScheduleSettlPeriodPriceCurrency] = "PaymentScheduleSettlPeriodPriceCurrency";         // (Currency FIX.5.0SP2)
dictionary[tag::PaymentScheduleSettlPeriodPriceUnitOfMeasure] = "PaymentScheduleSettlPeriodPriceUnitOfMeasure";    // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleStepUnitOfMeasure]  = "PaymentScheduleStepUnitOfMeasure";                // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDayDistribution] = "PaymentScheduleFixingDayDistribution";            // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingDayCount]     = "PaymentScheduleFixingDayCount";                   // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingLagPeriod]    = "PaymentScheduleFixingLagPeriod";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingLagUnit]      = "PaymentScheduleFixingLagUnit";                    // (String FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingFirstObservationDateOffsetPeriod] = "PaymentScheduleFixingFirstObservationDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::PaymentScheduleFixingFirstObservationDateOffsetUnit] = "PaymentScheduleFixingFirstObservationDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamFlatRateIndicator]    = "PaymentStreamFlatRateIndicator";                  // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamFlatRateAmount]       = "PaymentStreamFlatRateAmount";                     // (Amt FIX.5.0SP2)
dictionary[tag::PaymentStreamFlatRateCurrency]     = "PaymentStreamFlatRateCurrency";                   // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStreamMaximumPaymentAmount] = "PaymentStreamMaximumPaymentAmount";               // (Amt FIX.5.0SP2)
dictionary[tag::PaymentStreamMaximumPaymentCurrency] = "PaymentStreamMaximumPaymentCurrency";             // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStreamMaximumTransactionAmount] = "PaymentStreamMaximumTransactionAmount";           // (Amt FIX.5.0SP2)
dictionary[tag::PaymentStreamMaximumTransactionCurrency] = "PaymentStreamMaximumTransactionCurrency";         // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStreamFixedAmountUnitOfMeasure] = "PaymentStreamFixedAmountUnitOfMeasure";           // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamTotalFixedAmount]     = "PaymentStreamTotalFixedAmount";                   // (Amt FIX.5.0SP2)
dictionary[tag::PaymentStreamWorldScaleRate]       = "PaymentStreamWorldScaleRate";                     // (float FIX.5.0SP2)
dictionary[tag::PaymentStreamContractPrice]        = "PaymentStreamContractPrice";                      // (Price FIX.5.0SP2)
dictionary[tag::PaymentStreamContractPriceCurrency] = "PaymentStreamContractPriceCurrency";              // (Currency FIX.5.0SP2)
dictionary[tag::NoPaymentStreamPricingBusinessCenters] = "NoPaymentStreamPricingBusinessCenters";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingBusinessCenter] = "PaymentStreamPricingBusinessCenter";              // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndex2CurvePeriod] = "PaymentStreamRateIndex2CurvePeriod";              // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndex2CurveUnit]  = "PaymentStreamRateIndex2CurveUnit";                // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndexLocation]    = "PaymentStreamRateIndexLocation";                  // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndexLevel]       = "PaymentStreamRateIndexLevel";                     // (Qty FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndexUnitOfMeasure] = "PaymentStreamRateIndexUnitOfMeasure";             // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamSettlLevel]           = "PaymentStreamSettlLevel";                         // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamReferenceLevel]       = "PaymentStreamReferenceLevel";                     // (Qty FIX.5.0SP2)
dictionary[tag::PaymentStreamReferenceLevelUnitOfMeasure] = "PaymentStreamReferenceLevelUnitOfMeasure";        // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamReferenceLevelEqualsZeroIndicator] = "PaymentStreamReferenceLevelEqualsZeroIndicator";  // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamRateSpreadCurrency]   = "PaymentStreamRateSpreadCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStreamRateSpreadUnitOfMeasure] = "PaymentStreamRateSpreadUnitOfMeasure";            // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateConversionFactor] = "PaymentStreamRateConversionFactor";               // (float FIX.5.0SP2)
dictionary[tag::PaymentStreamRateSpreadType]       = "PaymentStreamRateSpreadType";                     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamLastResetRate]        = "PaymentStreamLastResetRate";                      // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalRate]            = "PaymentStreamFinalRate";                          // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamCalculationLagPeriod] = "PaymentStreamCalculationLagPeriod";               // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCalculationLagUnit]   = "PaymentStreamCalculationLagUnit";                 // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamFirstObservationDateOffsetPeriod] = "PaymentStreamFirstObservationDateOffsetPeriod";   // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFirstObservationDateOffsetUnit] = "PaymentStreamFirstObservationDateOffsetUnit";     // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingDayType]       = "PaymentStreamPricingDayType";                     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingDayDistribution] = "PaymentStreamPricingDayDistribution";             // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingDayCount]      = "PaymentStreamPricingDayCount";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingBusinessCalendar] = "PaymentStreamPricingBusinessCalendar";            // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingBusinessDayConvention] = "PaymentStreamPricingBusinessDayConvention";       // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamRiskApportionmentSource] = "DeliveryStreamRiskApportionmentSource";           // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamRiskApportionmentSource] = "LegDeliveryStreamRiskApportionmentSource";        // (String FIX.5.0SP2)
dictionary[tag::NoPaymentStreamPaymentDates]       = "NoPaymentStreamPaymentDates";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentDate]          = "PaymentStreamPaymentDate";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamPaymentDateType]      = "PaymentStreamPaymentDateType";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamMasterAgreementPaymentDatesIndicator] = "PaymentStreamMasterAgreementPaymentDatesIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::NoPaymentStreamPricingDates]       = "NoPaymentStreamPricingDates";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingDate]          = "PaymentStreamPricingDate";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingDateType]      = "PaymentStreamPricingDateType";                    // (int FIX.5.0SP2)
dictionary[tag::NoPaymentStreamPricingDays]        = "NoPaymentStreamPricingDays";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingDayOfWeek]     = "PaymentStreamPricingDayOfWeek";                   // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamPricingDayNumber]     = "PaymentStreamPricingDayNumber";                   // (int FIX.5.0SP2)
dictionary[tag::NoPricingDateBusinessCenters]      = "NoPricingDateBusinessCenters";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::PricingDateBusinessCenter]         = "PricingDateBusinessCenter";                       // (String FIX.5.0SP2)
dictionary[tag::PricingDateUnadjusted]             = "PricingDateUnadjusted";                           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PricingDateBusinessDayConvention]  = "PricingDateBusinessDayConvention";                // (int FIX.5.0SP2)
dictionary[tag::PricingDateAdjusted]               = "PricingDateAdjusted";                             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PricingTime]                       = "PricingTime";                                     // (LocalMktTime FIX.5.0SP2)
dictionary[tag::PricingTimeBusinessCenter]         = "PricingTimeBusinessCenter";                       // (String FIX.5.0SP2)
dictionary[tag::NoStreamAssetAttributes]           = "NoStreamAssetAttributes";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamAssetAttributeType]          = "StreamAssetAttributeType";                        // (String FIX.5.0SP2)
dictionary[tag::StreamAssetAttributeValue]         = "StreamAssetAttributeValue";                       // (String FIX.5.0SP2)
dictionary[tag::StreamAssetAttributeLimit]         = "StreamAssetAttributeLimit";                       // (String FIX.5.0SP2)
dictionary[tag::NoStreamCalculationPeriodDates]    = "NoStreamCalculationPeriodDates";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamCalculationPeriodDate]       = "StreamCalculationPeriodDate";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamCalculationPeriodDateType]   = "StreamCalculationPeriodDateType";                 // (int FIX.5.0SP2)
dictionary[tag::StreamCalculationPeriodDatesXID]   = "StreamCalculationPeriodDatesXID";                 // (XID FIX.5.0SP2)
dictionary[tag::StreamCalculationPeriodDatesXIDRef] = "StreamCalculationPeriodDatesXIDRef";              // (XIDREF FIX.5.0SP2)
dictionary[tag::StreamCalculationBalanceOfFirstPeriod] = "StreamCalculationBalanceOfFirstPeriod";           // (Boolean FIX.5.0SP2)
dictionary[tag::StreamCalculationCorrectionPeriod] = "StreamCalculationCorrectionPeriod";               // (int FIX.5.0SP2)
dictionary[tag::StreamCalculationCorrectionUnit]   = "StreamCalculationCorrectionUnit";                 // (String FIX.5.0SP2)
dictionary[tag::NoStreamCommoditySettlBusinessCenters] = "NoStreamCommoditySettlBusinessCenters";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlBusinessCenter] = "StreamCommoditySettlBusinessCenter";              // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityBase]               = "StreamCommodityBase";                             // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityType]               = "StreamCommodityType";                             // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySecurityID]         = "StreamCommoditySecurityID";                       // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySecurityIDSource]   = "StreamCommoditySecurityIDSource";                 // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityDesc]               = "StreamCommodityDesc";                             // (String FIX.5.0SP2)
dictionary[tag::EncodedStreamCommodityDescLen]     = "EncodedStreamCommodityDescLen";                   // (Length FIX.5.0SP2)
dictionary[tag::EncodedStreamCommodityDesc]        = "EncodedStreamCommodityDesc";                      // (data FIX.5.0SP2)
dictionary[tag::StreamCommodityUnitOfMeasure]      = "StreamCommodityUnitOfMeasure";                    // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityCurrency]           = "StreamCommodityCurrency";                         // (Currency FIX.5.0SP2)
dictionary[tag::StreamCommodityExchange]           = "StreamCommodityExchange";                         // (Exchange FIX.5.0SP2)
dictionary[tag::StreamCommodityRateSource]         = "StreamCommodityRateSource";                       // (int FIX.5.0SP2)
dictionary[tag::StreamCommodityRateReferencePage]  = "StreamCommodityRateReferencePage";                // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityRateReferencePageHeading] = "StreamCommodityRateReferencePageHeading";         // (String FIX.5.0SP2)
dictionary[tag::StreamDataProvider]                = "StreamDataProvider";                              // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityPricingType]        = "StreamCommodityPricingType";                      // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityNearbySettlDayPeriod] = "StreamCommodityNearbySettlDayPeriod";             // (int FIX.5.0SP2)
dictionary[tag::StreamCommodityNearbySettlDayUnit] = "StreamCommodityNearbySettlDayUnit";               // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlDateUnadjusted] = "StreamCommoditySettlDateUnadjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlDateBusinessDayConvention] = "StreamCommoditySettlDateBusinessDayConvention";   // (int FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlDateAdjusted]  = "StreamCommoditySettlDateAdjusted";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlMonth]         = "StreamCommoditySettlMonth";                       // (int FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlDateRollPeriod] = "StreamCommoditySettlDateRollPeriod";              // (int FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlDateRollUnit]  = "StreamCommoditySettlDateRollUnit";                // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlDayType]       = "StreamCommoditySettlDayType";                     // (int FIX.5.0SP2)
dictionary[tag::StreamCommodityXID]                = "StreamCommodityXID";                              // (XID FIX.5.0SP2)
dictionary[tag::StreamCommodityXIDRef]             = "StreamCommodityXIDRef";                           // (XIDREF FIX.5.0SP2)
dictionary[tag::NoStreamCommodityAltIDs]           = "NoStreamCommodityAltIDs";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamCommodityAltID]              = "StreamCommodityAltID";                            // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityAltIDSource]        = "StreamCommodityAltIDSource";                      // (String FIX.5.0SP2)
dictionary[tag::NoStreamCommodityDataSources]      = "NoStreamCommodityDataSources";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamCommodityDataSourceID]       = "StreamCommodityDataSourceID";                     // (String FIX.5.0SP2)
dictionary[tag::StreamCommodityDataSourceIDType]   = "StreamCommodityDataSourceIDType";                 // (int FIX.5.0SP2)
dictionary[tag::NoStreamCommoditySettlDays]        = "NoStreamCommoditySettlDays";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlDay]           = "StreamCommoditySettlDay";                         // (int FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlTotalHours]    = "StreamCommoditySettlTotalHours";                  // (int FIX.5.0SP2)
dictionary[tag::NoStreamCommoditySettlTimes]       = "NoStreamCommoditySettlTimes";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlStart]         = "StreamCommoditySettlStart";                       // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlEnd]           = "StreamCommoditySettlEnd";                         // (String FIX.5.0SP2)
dictionary[tag::NoStreamCommoditySettlPeriods]     = "NoStreamCommoditySettlPeriods";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlCountry]       = "StreamCommoditySettlCountry";                     // (Country FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlTimeZone]      = "StreamCommoditySettlTimeZone";                    // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlFlowType]      = "StreamCommoditySettlFlowType";                    // (int FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodNotional] = "StreamCommoditySettlPeriodNotional";              // (Qty FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodNotionalUnitOfMeasure] = "StreamCommoditySettlPeriodNotionalUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodFrequencyPeriod] = "StreamCommoditySettlPeriodFrequencyPeriod";       // (int FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodFrequencyUnit] = "StreamCommoditySettlPeriodFrequencyUnit";         // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodPrice]   = "StreamCommoditySettlPeriodPrice";                 // (Price FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodPriceUnitOfMeasure] = "StreamCommoditySettlPeriodPriceUnitOfMeasure";    // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodPriceCurrency] = "StreamCommoditySettlPeriodPriceCurrency";         // (Currency FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlHolidaysProcessingInstruction] = "StreamCommoditySettlHolidaysProcessingInstruction"; // (int FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodXID]     = "StreamCommoditySettlPeriodXID";                   // (XID FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlPeriodXIDRef]  = "StreamCommoditySettlPeriodXIDRef";                // (XIDREF FIX.5.0SP2)
dictionary[tag::StreamXID]                         = "StreamXID";                                       // (XID FIX.5.0SP2)
dictionary[tag::PaymentLegRefID]                   = "PaymentLegRefID";                                 // (String FIX.5.0SP2)
dictionary[tag::StreamNotionalXIDRef]              = "StreamNotionalXIDRef";                            // (XIDREF FIX.5.0SP2)
dictionary[tag::StreamNotionalFrequencyPeriod]     = "StreamNotionalFrequencyPeriod";                   // (int FIX.5.0SP2)
dictionary[tag::StreamNotionalFrequencyUnit]       = "StreamNotionalFrequencyUnit";                     // (String FIX.5.0SP2)
dictionary[tag::StreamNotionalCommodityFrequency]  = "StreamNotionalCommodityFrequency";                // (int FIX.5.0SP2)
dictionary[tag::StreamNotionalUnitOfMeasure]       = "StreamNotionalUnitOfMeasure";                     // (String FIX.5.0SP2)
dictionary[tag::StreamTotalNotional]               = "StreamTotalNotional";                             // (Qty FIX.5.0SP2)
dictionary[tag::StreamTotalNotionalUnitOfMeasure]  = "StreamTotalNotionalUnitOfMeasure";                // (String FIX.5.0SP2)
dictionary[tag::NoMandatoryClearingJurisdictions]  = "NoMandatoryClearingJurisdictions";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::MandatoryClearingJurisdiction]     = "MandatoryClearingJurisdiction";                   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermXIDRef]    = "UnderlyingProtectionTermXIDRef";                  // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingSettlTermXIDRef]         = "UnderlyingSettlTermXIDRef";                       // (XIDREF FIX.5.0SP2)
dictionary[tag::NoLegAdditionalTermBondRefs]       = "NoLegAdditionalTermBondRefs";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondSecurityID]   = "LegAdditionalTermBondSecurityID";                 // (String FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondSecurityIDSource] = "LegAdditionalTermBondSecurityIDSource";           // (String FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondDesc]         = "LegAdditionalTermBondDesc";                       // (String FIX.5.0SP2)
dictionary[tag::EncodedLegAdditionalTermBondDescLen] = "EncodedLegAdditionalTermBondDescLen";             // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegAdditionalTermBondDesc]  = "EncodedLegAdditionalTermBondDesc";                // (data FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondCurrency]     = "LegAdditionalTermBondCurrency";                   // (Currency FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondIssuer]       = "LegAdditionalTermBondIssuer";                     // (String FIX.5.0SP2)
dictionary[tag::EncodedLegAdditionalTermBondIssuerLen] = "EncodedLegAdditionalTermBondIssuerLen";           // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegAdditionalTermBondIssuer] = "EncodedLegAdditionalTermBondIssuer";              // (data FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondSeniority]    = "LegAdditionalTermBondSeniority";                  // (String FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondCouponType]   = "LegAdditionalTermBondCouponType";                 // (int FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondCouponRate]   = "LegAdditionalTermBondCouponRate";                 // (Percentage FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondMaturityDate] = "LegAdditionalTermBondMaturityDate";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondParValue]     = "LegAdditionalTermBondParValue";                   // (Amt FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondCurrentTotalIssuedAmount] = "LegAdditionalTermBondCurrentTotalIssuedAmount";   // (Amt FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondCouponFrequencyPeriod] = "LegAdditionalTermBondCouponFrequencyPeriod";      // (int FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondCouponFrequencyUnit] = "LegAdditionalTermBondCouponFrequencyUnit";        // (String FIX.5.0SP2)
dictionary[tag::LegAdditionalTermBondDayCount]     = "LegAdditionalTermBondDayCount";                   // (int FIX.5.0SP2)
dictionary[tag::NoLegAdditionalTerms]              = "NoLegAdditionalTerms";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegAdditionalTermConditionPrecedentBondIndicator] = "LegAdditionalTermConditionPrecedentBondIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::LegAdditionalTermDiscrepancyClauseIndicator] = "LegAdditionalTermDiscrepancyClauseIndicator";     // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionValue]   = "UnderlyingMarketDisruptionValue";                 // (String FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackValue] = "UnderlyingMarketDisruptionFallbackValue";         // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingAdditionalTermBondRefs] = "NoUnderlyingAdditionalTermBondRefs";              // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondSecurityID] = "UnderlyingAdditionalTermBondSecurityID";          // (String FIX.5.0SP2)
dictionary[tag::NoLegCashSettlDealers]             = "NoLegCashSettlDealers";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegCashSettlDealer]                = "LegCashSettlDealer";                              // (String FIX.5.0SP2)
dictionary[tag::NoLegCashSettlTerms]               = "NoLegCashSettlTerms";                             // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegCashSettlCurrency]              = "LegCashSettlCurrency";                            // (Currency FIX.5.0SP2)
dictionary[tag::LegCasSettlValuationFirstBusinessDayOffset] = "LegCasSettlValuationFirstBusinessDayOffset";      // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlValuationSubsequentBusinessDaysOffset] = "LegCashSettlValuationSubsequentBusinessDaysOffset"; // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlNumOfValuationDates]   = "LegCashSettlNumOfValuationDates";                 // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlValuationTime]         = "LegCashSettlValuationTime";                       // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegCashSettlBusinessCenter]        = "LegCashSettlBusinessCenter";                      // (String FIX.5.0SP2)
dictionary[tag::LegCashSettlQuoteMethod]           = "LegCashSettlQuoteMethod";                         // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlQuoteAmount]           = "LegCashSettlQuoteAmount";                         // (Amt FIX.5.0SP2)
dictionary[tag::LegCashSettlQuoteCurrency]         = "LegCashSettlQuoteCurrency";                       // (Currency FIX.5.0SP2)
dictionary[tag::LegCashSettlMinimumQuoteAmount]    = "LegCashSettlMinimumQuoteAmount";                  // (Amt FIX.5.0SP2)
dictionary[tag::LegCashSettlMinimumQuoteCurrency]  = "LegCashSettlMinimumQuoteCurrency";                // (Currency FIX.5.0SP2)
dictionary[tag::LegCashSettlBusinessDays]          = "LegCashSettlBusinessDays";                        // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlAmount]                = "LegCashSettlAmount";                              // (Amt FIX.5.0SP2)
dictionary[tag::LegCashSettlRecoveryFactor]        = "LegCashSettlRecoveryFactor";                      // (float FIX.5.0SP2)
dictionary[tag::LegCashSettlFixedTermIndicator]    = "LegCashSettlFixedTermIndicator";                  // (Boolean FIX.5.0SP2)
dictionary[tag::LegCashSettlAccruedInterestIndicator] = "LegCashSettlAccruedInterestIndicator";            // (Boolean FIX.5.0SP2)
dictionary[tag::LegCashSettlValuationMethod]       = "LegCashSettlValuationMethod";                     // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlTermXID]               = "LegCashSettlTermXID";                             // (XID FIX.5.0SP2)
dictionary[tag::NoLegComplexEventAveragingObservations] = "NoLegComplexEventAveragingObservations";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventAveragingObservationNumber] = "LegComplexEventAveragingObservationNumber";       // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventAveragingWeight]    = "LegComplexEventAveragingWeight";                  // (float FIX.5.0SP2)
dictionary[tag::NoLegComplexEventCreditEvents]     = "NoLegComplexEventCreditEvents";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventType]    = "LegComplexEventCreditEventType";                  // (String FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventValue]   = "LegComplexEventCreditEventValue";                 // (String FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventCurrency] = "LegComplexEventCreditEventCurrency";              // (Currency FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventPeriod]  = "LegComplexEventCreditEventPeriod";                // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventUnit]    = "LegComplexEventCreditEventUnit";                  // (String FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventDayType] = "LegComplexEventCreditEventDayType";               // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventRateSource] = "LegComplexEventCreditEventRateSource";            // (int FIX.5.0SP2)
dictionary[tag::NoLegComplexEventCreditEventQualifiers] = "NoLegComplexEventCreditEventQualifiers";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventQualifier] = "LegComplexEventCreditEventQualifier";             // (char FIX.5.0SP2)
dictionary[tag::NoLegComplexEventPeriodDateTimes]  = "NoLegComplexEventPeriodDateTimes";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventPeriodDate]         = "LegComplexEventPeriodDate";                       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegComplexEventPeriodTime]         = "LegComplexEventPeriodTime";                       // (LocalMktTime FIX.5.0SP2)
dictionary[tag::NoLegComplexEventPeriods]          = "NoLegComplexEventPeriods";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventPeriodType]         = "LegComplexEventPeriodType";                       // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventBusinessCenter]     = "LegComplexEventBusinessCenter";                   // (String FIX.5.0SP2)
dictionary[tag::NoLegComplexEventRateSources]      = "NoLegComplexEventRateSources";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventRateSource]         = "LegComplexEventRateSource";                       // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventRateSourceType]     = "LegComplexEventRateSourceType";                   // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventReferencePage]      = "LegComplexEventReferencePage";                    // (String FIX.5.0SP2)
dictionary[tag::LegComplexEvenReferencePageHeading] = "LegComplexEvenReferencePageHeading";              // (String FIX.5.0SP2)
dictionary[tag::NoLegComplexEventDateBusinessCenters] = "NoLegComplexEventDateBusinessCenters";            // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventDateBusinessCenter] = "LegComplexEventDateBusinessCenter";               // (String FIX.5.0SP2)
dictionary[tag::LegComplexEventDateUnadjusted]     = "LegComplexEventDateUnadjusted";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegComplexEventDateRelativeTo]     = "LegComplexEventDateRelativeTo";                   // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventDateOffsetPeriod]   = "LegComplexEventDateOffsetPeriod";                 // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventDateOffsetUnit]     = "LegComplexEventDateOffsetUnit";                   // (String FIX.5.0SP2)
dictionary[tag::LegComplexEventDateOffsetDayType]  = "LegComplexEventDateOffsetDayType";                // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventDateBusinessDayConvention] = "LegComplexEventDateBusinessDayConvention";        // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventDateAdjusted]       = "LegComplexEventDateAdjusted";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegComplexEventFixingTime]         = "LegComplexEventFixingTime";                       // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegComplexEventFixingTimeBusinessCenter] = "LegComplexEventFixingTimeBusinessCenter";         // (String FIX.5.0SP2)
dictionary[tag::NoLegComplexEventCreditEventSources] = "NoLegComplexEventCreditEventSources";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventCreditEventSource]  = "LegComplexEventCreditEventSource";                // (String FIX.5.0SP2)
dictionary[tag::NoLegComplexEventSchedules]        = "NoLegComplexEventSchedules";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegComplexEventScheduleStartDate]  = "LegComplexEventScheduleStartDate";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegComplexEventScheduleEndDate]    = "LegComplexEventScheduleEndDate";                  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegComplexEventScheduleFrequencyPeriod] = "LegComplexEventScheduleFrequencyPeriod";          // (int FIX.5.0SP2)
dictionary[tag::LegComplexEventScheduleFrequencyUnit] = "LegComplexEventScheduleFrequencyUnit";            // (String FIX.5.0SP2)
dictionary[tag::LegComplexEventScheduleRollConvention] = "LegComplexEventScheduleRollConvention";           // (String FIX.5.0SP2)
dictionary[tag::ProvisionCashSettlQuoteReferencePage] = "ProvisionCashSettlQuoteReferencePage";            // (String FIX.5.0SP2)
dictionary[tag::LegProvisionCashSettlQuoteReferencePage] = "LegProvisionCashSettlQuoteReferencePage";         // (String FIX.5.0SP2)
dictionary[tag::NoLegDeliverySchedules]            = "NoLegDeliverySchedules";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleType]           = "LegDeliveryScheduleType";                         // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleXID]            = "LegDeliveryScheduleXID";                          // (XID FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleNotional]       = "LegDeliveryScheduleNotional";                     // (Qty FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleNotionalUnitOfMeasure] = "LegDeliveryScheduleNotionalUnitOfMeasure";        // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleNotionalCommodityFrequency] = "LegDeliveryScheduleNotionalCommodityFrequency";   // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleNegativeTolerance] = "LegDeliveryScheduleNegativeTolerance";            // (float FIX.5.0SP2)
dictionary[tag::LegDeliverySchedulePositiveTolerance] = "LegDeliverySchedulePositiveTolerance";            // (float FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleToleranceUnitOfMeasure] = "LegDeliveryScheduleToleranceUnitOfMeasure";       // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleToleranceType]  = "LegDeliveryScheduleToleranceType";                // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlCountry]   = "LegDeliveryScheduleSettlCountry";                 // (Country FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlTimeZone]  = "LegDeliveryScheduleSettlTimeZone";                // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlFlowType]  = "LegDeliveryScheduleSettlFlowType";                // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlHolidaysProcessingInstruction] = "LegDeliveryScheduleSettlHolidaysProcessingInstruction"; // (int FIX.5.0SP2)
dictionary[tag::NoLegDeliveryScheduleSettlDays]    = "NoLegDeliveryScheduleSettlDays";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlDay]       = "LegDeliveryScheduleSettlDay";                     // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlTotalHours] = "LegDeliveryScheduleSettlTotalHours";              // (int FIX.5.0SP2)
dictionary[tag::NoLegDeliveryScheduleSettlTimes]   = "NoLegDeliveryScheduleSettlTimes";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlStart]     = "LegDeliveryScheduleSettlStart";                   // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlEnd]       = "LegDeliveryScheduleSettlEnd";                     // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryScheduleSettlTimeType]  = "LegDeliveryScheduleSettlTimeType";                // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamType]             = "LegDeliveryStreamType";                           // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamPipeline]         = "LegDeliveryStreamPipeline";                       // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamEntryPoint]       = "LegDeliveryStreamEntryPoint";                     // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamWithdrawalPoint]  = "LegDeliveryStreamWithdrawalPoint";                // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamDeliveryPoint]    = "LegDeliveryStreamDeliveryPoint";                  // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamDeliveryRestriction] = "LegDeliveryStreamDeliveryRestriction";            // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamDeliveryContingency] = "LegDeliveryStreamDeliveryContingency";            // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamDeliveryContingentPartySide] = "LegDeliveryStreamDeliveryContingentPartySide";    // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamDeliverAtSourceIndicator] = "LegDeliveryStreamDeliverAtSourceIndicator";       // (Boolean FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamRiskApportionment] = "LegDeliveryStreamRiskApportionment";              // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamTitleTransferLocation] = "LegDeliveryStreamTitleTransferLocation";          // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamTitleTransferCondition] = "LegDeliveryStreamTitleTransferCondition";         // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamImporterOfRecord] = "LegDeliveryStreamImporterOfRecord";               // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamNegativeTolerance] = "LegDeliveryStreamNegativeTolerance";              // (float FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamPositiveTolerance] = "LegDeliveryStreamPositiveTolerance";              // (float FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamToleranceUnitOfMeasure] = "LegDeliveryStreamToleranceUnitOfMeasure";         // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamToleranceType]    = "LegDeliveryStreamToleranceType";                  // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamToleranceOptionSide] = "LegDeliveryStreamToleranceOptionSide";            // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamTotalPositiveTolerance] = "LegDeliveryStreamTotalPositiveTolerance";         // (Percentage FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamTotalNegativeTolerance] = "LegDeliveryStreamTotalNegativeTolerance";         // (Percentage FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamNotionalConversionFactor] = "LegDeliveryStreamNotionalConversionFactor";       // (float FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamTransportEquipment] = "LegDeliveryStreamTransportEquipment";             // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamElectingPartySide] = "LegDeliveryStreamElectingPartySide";              // (int FIX.5.0SP2)
dictionary[tag::NoLegStreamAssetAttributes]        = "NoLegStreamAssetAttributes";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamAssetAttributeType]       = "LegStreamAssetAttributeType";                     // (String FIX.5.0SP2)
dictionary[tag::LegStreamAssetAttributeValue]      = "LegStreamAssetAttributeValue";                    // (String FIX.5.0SP2)
dictionary[tag::LegStreamAssetAttributeLimit]      = "LegStreamAssetAttributeLimit";                    // (String FIX.5.0SP2)
dictionary[tag::NoLegDeliveryStreamCycles]         = "NoLegDeliveryStreamCycles";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamCycleDesc]        = "LegDeliveryStreamCycleDesc";                      // (String FIX.5.0SP2)
dictionary[tag::EncodedLegDeliveryStreamCycleDescLen] = "EncodedLegDeliveryStreamCycleDescLen";            // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegDeliveryStreamCycleDesc] = "EncodedLegDeliveryStreamCycleDesc";               // (data FIX.5.0SP2)
dictionary[tag::NoLegDeliveryStreamCommoditySources] = "NoLegDeliveryStreamCommoditySources";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamCommoditySource]  = "LegDeliveryStreamCommoditySource";                // (String FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionProvision]      = "LegMarketDisruptionProvision";                    // (int FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackProvision] = "LegMarketDisruptionFallbackProvision";            // (int FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionMaximumDays]    = "LegMarketDisruptionMaximumDays";                  // (int FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionMaterialityPercentage] = "LegMarketDisruptionMaterialityPercentage";        // (Percentage FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionMinimumFuturesContracts] = "LegMarketDisruptionMinimumFuturesContracts";      // (int FIX.5.0SP2)
dictionary[tag::NoLegMarketDisruptionEvents]       = "NoLegMarketDisruptionEvents";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionEvent]          = "LegMarketDisruptionEvent";                        // (String FIX.5.0SP2)
dictionary[tag::NoLegMarketDisruptionFallbacks]    = "NoLegMarketDisruptionFallbacks";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackType]   = "LegMarketDisruptionFallbackType";                 // (String FIX.5.0SP2)
dictionary[tag::NoLegMarketDisruptionFallbackReferencePrices] = "NoLegMarketDisruptionFallbackReferencePrices";    // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackUnderlierType] = "LegMarketDisruptionFallbackUnderlierType";        // (int FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackUnderlierSecurityID] = "LegMarketDisruptionFallbackUnderlierSecurityID";  // (String FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackUnderlierSecurityIDSource] = "LegMarketDisruptionFallbackUnderlierSecurityIDSource"; // (String FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackUnderlierSecurityDesc] = "LegMarketDisruptionFallbackUnderlierSecurityDesc"; // (String FIX.5.0SP2)
dictionary[tag::EncodedLegMarketDisruptionFallbackUnderlierSecurityDescLen] = "EncodedLegMarketDisruptionFallbackUnderlierSecurityDescLen"; // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegMarketDisruptionFallbackUnderlierSecurityDesc] = "EncodedLegMarketDisruptionFallbackUnderlierSecurityDesc"; // (data FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackOpenUnits] = "LegMarketDisruptionFallbackOpenUnits";            // (Qty FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackBasketCurrency] = "LegMarketDisruptionFallbackBasketCurrency";       // (Currency FIX.5.0SP2)
dictionary[tag::LegMarketDisruptionFallbackBasketDivisor] = "LegMarketDisruptionFallbackBasketDivisor";        // (float FIX.5.0SP2)
dictionary[tag::LegExerciseDesc]                   = "LegExerciseDesc";                                 // (String FIX.5.0SP2)
dictionary[tag::EncodedLegExerciseDescLen]         = "EncodedLegExerciseDescLen";                       // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegExerciseDesc]            = "EncodedLegExerciseDesc";                          // (data FIX.5.0SP2)
dictionary[tag::LegAutomaticExerciseIndicator]     = "LegAutomaticExerciseIndicator";                   // (Boolean FIX.5.0SP2)
dictionary[tag::LegAutomaticExerciseThresholdRate] = "LegAutomaticExerciseThresholdRate";               // (float FIX.5.0SP2)
dictionary[tag::LegExerciseConfirmationMethod]     = "LegExerciseConfirmationMethod";                   // (int FIX.5.0SP2)
dictionary[tag::LegManualNoticeBusinessCenter]     = "LegManualNoticeBusinessCenter";                   // (String FIX.5.0SP2)
dictionary[tag::LegFallbackExerciseIndicator]      = "LegFallbackExerciseIndicator";                    // (Boolean FIX.5.0SP2)
dictionary[tag::LegLimitRightToConfirmIndicator]   = "LegLimitRightToConfirmIndicator";                 // (Boolean FIX.5.0SP2)
dictionary[tag::LegExerciseSplitTicketIndicator]   = "LegExerciseSplitTicketIndicator";                 // (Boolean FIX.5.0SP2)
dictionary[tag::NoLegOptionExerciseBusinessCenters] = "NoLegOptionExerciseBusinessCenters";              // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegOptionExerciseBusinessCenter]   = "LegOptionExerciseBusinessCenter";                 // (String FIX.5.0SP2)
dictionary[tag::LegOptionExerciseBusinessDayConvention] = "LegOptionExerciseBusinessDayConvention";          // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseEarliestDateOffsetDayType] = "LegOptionExerciseEarliestDateOffsetDayType";      // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseEarliestDateOffsetPeriod] = "LegOptionExerciseEarliestDateOffsetPeriod";       // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseEarliestDateOffsetUnit] = "LegOptionExerciseEarliestDateOffsetUnit";         // (String FIX.5.0SP2)
dictionary[tag::LegOptionExerciseFrequencyPeriod]  = "LegOptionExerciseFrequencyPeriod";                // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseFrequencyUnit]    = "LegOptionExerciseFrequencyUnit";                  // (String FIX.5.0SP2)
dictionary[tag::LegOptionExerciseStartDateUnadjusted] = "LegOptionExerciseStartDateUnadjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegOptionExerciseStartDateRelativeTo] = "LegOptionExerciseStartDateRelativeTo";            // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseStartDateOffsetPeriod] = "LegOptionExerciseStartDateOffsetPeriod";          // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseStartDateOffsetUnit] = "LegOptionExerciseStartDateOffsetUnit";            // (String FIX.5.0SP2)
dictionary[tag::LegOptionExerciseStartDateOffsetDayType] = "LegOptionExerciseStartDateOffsetDayType";         // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseStartDateAdjusted] = "LegOptionExerciseStartDateAdjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegOptionExerciseSkip]             = "LegOptionExerciseSkip";                           // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseNominationDeadline] = "LegOptionExerciseNominationDeadline";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegOptionExerciseFirstDateUnadjusted] = "LegOptionExerciseFirstDateUnadjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegOptionExerciseLastDateUnadjusted] = "LegOptionExerciseLastDateUnadjusted";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegOptionExerciseEarliestTime]     = "LegOptionExerciseEarliestTime";                   // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegOptionExerciseLatestTime]       = "LegOptionExerciseLatestTime";                     // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegOptionExerciseTimeBusinessCenter] = "LegOptionExerciseTimeBusinessCenter";             // (String FIX.5.0SP2)
dictionary[tag::NoLegOptionExerciseDates]          = "NoLegOptionExerciseDates";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegOptionExerciseDate]             = "LegOptionExerciseDate";                           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegOptionExerciseDateType]         = "LegOptionExerciseDateType";                       // (int FIX.5.0SP2)
dictionary[tag::NoLegOptionExerciseExpirationDateBusinessCenters] = "NoLegOptionExerciseExpirationDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationDateBusinessCenter] = "LegOptionExerciseExpirationDateBusinessCenter";   // (String FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationDateBusinessDayConvention] = "LegOptionExerciseExpirationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationDateRelativeTo] = "LegOptionExerciseExpirationDateRelativeTo";       // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationDateOffsetPeriod] = "LegOptionExerciseExpirationDateOffsetPeriod";     // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationDateOffsetUnit] = "LegOptionExerciseExpirationDateOffsetUnit";       // (String FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationFrequencyPeriod] = "LegOptionExerciseExpirationFrequencyPeriod";      // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationFrequencyUnit] = "LegOptionExerciseExpirationFrequencyUnit";        // (String FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationRollConvention] = "LegOptionExerciseExpirationRollConvention";       // (String FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationDateOffsetDayType] = "LegOptionExerciseExpirationDateOffsetDayType";    // (int FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationTime]   = "LegOptionExerciseExpirationTime";                 // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationTimeBusinessCenter] = "LegOptionExerciseExpirationTimeBusinessCenter";   // (String FIX.5.0SP2)
dictionary[tag::NoLegOptionExerciseExpirationDates] = "NoLegOptionExerciseExpirationDates";              // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationDate]   = "LegOptionExerciseExpirationDate";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegOptionExerciseExpirationDateType] = "LegOptionExerciseExpirationDateType";             // (int FIX.5.0SP2)
dictionary[tag::NoLegPaymentScheduleFixingDays]    = "NoLegPaymentScheduleFixingDays";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDayOfWeek] = "LegPaymentScheduleFixingDayOfWeek";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDayNumber] = "LegPaymentScheduleFixingDayNumber";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleXID]             = "LegPaymentScheduleXID";                           // (XID FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleXIDRef]          = "LegPaymentScheduleXIDRef";                        // (XIDREF FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateCurrency]    = "LegPaymentScheduleRateCurrency";                  // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateUnitOfMeasure] = "LegPaymentScheduleRateUnitOfMeasure";             // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateConversionFactor] = "LegPaymentScheduleRateConversionFactor";          // (float FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleRateSpreadType]  = "LegPaymentScheduleRateSpreadType";                // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleSettlPeriodPrice] = "LegPaymentScheduleSettlPeriodPrice";              // (Price FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleSettlPeriodPriceCurrency] = "LegPaymentScheduleSettlPeriodPriceCurrency";      // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleSettlPeriodPriceUnitOfMeasure] = "LegPaymentScheduleSettlPeriodPriceUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleStepUnitOfMeasure] = "LegPaymentScheduleStepUnitOfMeasure";             // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDayDistribution] = "LegPaymentScheduleFixingDayDistribution";         // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingDayCount]  = "LegPaymentScheduleFixingDayCount";                // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingLagPeriod] = "LegPaymentScheduleFixingLagPeriod";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingLagUnit]   = "LegPaymentScheduleFixingLagUnit";                 // (String FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingFirstObservationDateOffsetPeriod] = "LegPaymentScheduleFixingFirstObservationDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentScheduleFixingFirstObservationDateOffsetUnit] = "LegPaymentScheduleFixingFirstObservationDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFlatRateIndicator] = "LegPaymentStreamFlatRateIndicator";               // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFlatRateAmount]    = "LegPaymentStreamFlatRateAmount";                  // (Amt FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFlatRateCurrency]  = "LegPaymentStreamFlatRateCurrency";                // (Currency FIX.5.0SP2)
dictionary[tag::LegStreamMaximumPaymentAmount]     = "LegStreamMaximumPaymentAmount";                   // (Amt FIX.5.0SP2)
dictionary[tag::LegStreamMaximumPaymentCurrency]   = "LegStreamMaximumPaymentCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::LegStreamMaximumTransactionAmount] = "LegStreamMaximumTransactionAmount";               // (Amt FIX.5.0SP2)
dictionary[tag::LegStreamMaximumTransactionCurrency] = "LegStreamMaximumTransactionCurrency";             // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixedAmountUnitOfMeasure] = "LegPaymentStreamFixedAmountUnitOfMeasure";        // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamTotalFixedAmount]  = "LegPaymentStreamTotalFixedAmount";                // (Amt FIX.5.0SP2)
dictionary[tag::LegPaymentStreamWorldScaleRate]    = "LegPaymentStreamWorldScaleRate";                  // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamContractPrice]     = "LegPaymentStreamContractPrice";                   // (Price FIX.5.0SP2)
dictionary[tag::LegPaymentStreamContractPriceCurrency] = "LegPaymentStreamContractPriceCurrency";           // (Currency FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamPricingBusinessCenters] = "NoLegPaymentStreamPricingBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingBusinessCenter] = "LegPaymentStreamPricingBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndex2CurveUnit] = "LegPaymentStreamRateIndex2CurveUnit";             // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndex2CurvePeriod] = "LegPaymentStreamRateIndex2CurvePeriod";           // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndexLocation] = "LegPaymentStreamRateIndexLocation";               // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndexLevel]    = "LegPaymentStreamRateIndexLevel";                  // (Qty FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndexUnitOfMeasure] = "LegPaymentStreamRateIndexUnitOfMeasure";          // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamSettlLevel]        = "LegPaymentStreamSettlLevel";                      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamReferenceLevel]    = "LegPaymentStreamReferenceLevel";                  // (Qty FIX.5.0SP2)
dictionary[tag::LegPaymentStreamReferenceLevelUnitOfMeasure] = "LegPaymentStreamReferenceLevelUnitOfMeasure";     // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamReferenceLevelEqualsZeroIndicator] = "LegPaymentStreamReferenceLevelEqualsZeroIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateSpreadCurrency] = "LegPaymentStreamRateSpreadCurrency";              // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateSpreadUnitOfMeasure] = "LegPaymentStreamRateSpreadUnitOfMeasure";         // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateConversionFactor] = "LegPaymentStreamRateConversionFactor";            // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateSpreadType]    = "LegPaymentStreamRateSpreadType";                  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLastResetRate]     = "LegPaymentStreamLastResetRate";                   // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalRate]         = "LegPaymentStreamFinalRate";                       // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCalculationLagPeriod] = "LegPaymentStreamCalculationLagPeriod";            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCalculationLagUnit] = "LegPaymentStreamCalculationLagUnit";              // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFirstObservationDateOffsetPeriod] = "LegPaymentStreamFirstObservationDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFirstObservationDateOffsetUnit] = "LegPaymentStreamFirstObservationDateOffsetUnit";  // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingDayType]    = "LegPaymentStreamPricingDayType";                  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingDayDistribution] = "LegPaymentStreamPricingDayDistribution";          // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingDayCount]   = "LegPaymentStreamPricingDayCount";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingBusinessCalendar] = "LegPaymentStreamPricingBusinessCalendar";         // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingBusinessDayConvention] = "LegPaymentStreamPricingBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamRiskApportionmentSource] = "UnderlyingDeliveryStreamRiskApportionmentSource"; // (String FIX.5.0SP2)
dictionary[tag::StreamCommoditySettlTimeType]      = "StreamCommoditySettlTimeType";                    // (int FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamPaymentDates]    = "NoLegPaymentStreamPaymentDates";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentDate]       = "LegPaymentStreamPaymentDate";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPaymentDateType]   = "LegPaymentStreamPaymentDateType";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamMasterAgreementPaymentDatesIndicator] = "LegPaymentStreamMasterAgreementPaymentDatesIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamPricingDates]    = "NoLegPaymentStreamPricingDates";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingDate]       = "LegPaymentStreamPricingDate";                     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingDateType]   = "LegPaymentStreamPricingDateType";                 // (int FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamPricingDays]     = "NoLegPaymentStreamPricingDays";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingDayOfWeek]  = "LegPaymentStreamPricingDayOfWeek";                // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamPricingDayNumber]  = "LegPaymentStreamPricingDayNumber";                // (int FIX.5.0SP2)
dictionary[tag::NoLegPhysicalSettlTerms]           = "NoLegPhysicalSettlTerms";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPhysicalSettlTermXID]           = "LegPhysicalSettlTermXID";                         // (XID FIX.5.0SP2)
dictionary[tag::LegPhysicalSettlCurency]           = "LegPhysicalSettlCurency";                         // (Currency FIX.5.0SP2)
dictionary[tag::LegPhysicalSettlBusinessDays]      = "LegPhysicalSettlBusinessDays";                    // (int FIX.5.0SP2)
dictionary[tag::LegPhysicalSettlMaximumBusinessDays] = "LegPhysicalSettlMaximumBusinessDays";             // (int FIX.5.0SP2)
dictionary[tag::NoLegPhysicalSettlDeliverableObligations] = "NoLegPhysicalSettlDeliverableObligations";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPhysicalSettlDeliverableObligationType] = "LegPhysicalSettlDeliverableObligationType";       // (String FIX.5.0SP2)
dictionary[tag::LegPhysicalSettlDeliverableObligationValue] = "LegPhysicalSettlDeliverableObligationValue";      // (String FIX.5.0SP2)
dictionary[tag::NoLegPricingDateBusinessCenters]   = "NoLegPricingDateBusinessCenters";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPricingDateBusinessCenter]      = "LegPricingDateBusinessCenter";                    // (String FIX.5.0SP2)
dictionary[tag::LegPricingDateUnadjusted]          = "LegPricingDateUnadjusted";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPricingDateBusinessDayConvention] = "LegPricingDateBusinessDayConvention";             // (int FIX.5.0SP2)
dictionary[tag::LegPricingDateAdjusted]            = "LegPricingDateAdjusted";                          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPricingTime]                    = "LegPricingTime";                                  // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegPricingTimeBusinessCenter]      = "LegPricingTimeBusinessCenter";                    // (String FIX.5.0SP2)
dictionary[tag::NoLegProtectionTermEventNewsSources] = "NoLegProtectionTermEventNewsSources";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventNewsSource]  = "LegProtectionTermEventNewsSource";                // (String FIX.5.0SP2)
dictionary[tag::NoLegProtectionTerms]              = "NoLegProtectionTerms";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProtectionTermXID]              = "LegProtectionTermXID";                            // (XID FIX.5.0SP2)
dictionary[tag::LegProtectionTermNotional]         = "LegProtectionTermNotional";                       // (Amt FIX.5.0SP2)
dictionary[tag::LegProtectionTermCurrency]         = "LegProtectionTermCurrency";                       // (Currency FIX.5.0SP2)
dictionary[tag::LegProtectionTermSellerNotifies]   = "LegProtectionTermSellerNotifies";                 // (Boolean FIX.5.0SP2)
dictionary[tag::LegProtectionTermBuyerNotifies]    = "LegProtectionTermBuyerNotifies";                  // (Boolean FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventBusinessCenter] = "LegProtectionTermEventBusinessCenter";            // (String FIX.5.0SP2)
dictionary[tag::LegProtectionTermStandardSources]  = "LegProtectionTermStandardSources";                // (Boolean FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventMinimumSources] = "LegProtectionTermEventMinimumSources";            // (int FIX.5.0SP2)
dictionary[tag::NoLegProtectionTermEvents]         = "NoLegProtectionTermEvents";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventType]        = "LegProtectionTermEventType";                      // (String FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventValue]       = "LegProtectionTermEventValue";                     // (String FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventCurrency]    = "LegProtectionTermEventCurrency";                  // (Currency FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventPeriod]      = "LegProtectionTermEventPeriod";                    // (int FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventUnit]        = "LegProtectionTermEventUnit";                      // (String FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventDayType]     = "LegProtectionTermEventDayType";                   // (int FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventRateSource]  = "LegProtectionTermEventRateSource";                // (String FIX.5.0SP2)
dictionary[tag::NoLegProtectionTermEventQualifiers] = "NoLegProtectionTermEventQualifiers";              // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProtectionTermEventQualifier]   = "LegProtectionTermEventQualifier";                 // (char FIX.5.0SP2)
dictionary[tag::NoLegProtectionTermObligations]    = "NoLegProtectionTermObligations";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegProtectionTermObligationType]   = "LegProtectionTermObligationType";                 // (String FIX.5.0SP2)
dictionary[tag::LegProtectionTermObligationValue]  = "LegProtectionTermObligationValue";                // (String FIX.5.0SP2)
dictionary[tag::NoLegStreamCalculationPeriodDates] = "NoLegStreamCalculationPeriodDates";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamCalculationPeriodDate]    = "LegStreamCalculationPeriodDate";                  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamCalculationPeriodDateType] = "LegStreamCalculationPeriodDateType";              // (int FIX.5.0SP2)
dictionary[tag::LegStreamCalculationPeriodDatesXID] = "LegStreamCalculationPeriodDatesXID";              // (XID FIX.5.0SP2)
dictionary[tag::LegStreamCalculationPeriodDatesXIDRef] = "LegStreamCalculationPeriodDatesXIDRef";           // (XIDREF FIX.5.0SP2)
dictionary[tag::LegStreamCalculationBalanceOfFirstPeriod] = "LegStreamCalculationBalanceOfFirstPeriod";        // (Boolean FIX.5.0SP2)
dictionary[tag::LegStreamCalculationCorrectionPeriod] = "LegStreamCalculationCorrectionPeriod";            // (int FIX.5.0SP2)
dictionary[tag::LegStreamCalculationCorrectionUnit] = "LegStreamCalculationCorrectionUnit";              // (String FIX.5.0SP2)
dictionary[tag::NoLegStreamCommoditySettlBusinessCenters] = "NoLegStreamCommoditySettlBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlBusinessCenter] = "LegStreamCommoditySettlBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityBase]            = "LegStreamCommodityBase";                          // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityType]            = "LegStreamCommodityType";                          // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySecurityID]      = "LegStreamCommoditySecurityID";                    // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySecurityIDSource] = "LegStreamCommoditySecurityIDSource";              // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityDesc]            = "LegStreamCommodityDesc";                          // (String FIX.5.0SP2)
dictionary[tag::EncodedLegStreamCommodityDescLen]  = "EncodedLegStreamCommodityDescLen";                // (Length FIX.5.0SP2)
dictionary[tag::EncodedLegStreamCommodityDesc]     = "EncodedLegStreamCommodityDesc";                   // (data FIX.5.0SP2)
dictionary[tag::LegStreamCommodityUnitOfMeasure]   = "LegStreamCommodityUnitOfMeasure";                 // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityCurrency]        = "LegStreamCommodityCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::LegStreamCommodityExchange]        = "LegStreamCommodityExchange";                      // (Exchange FIX.5.0SP2)
dictionary[tag::LegStreamCommodityRateSource]      = "LegStreamCommodityRateSource";                    // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommodityRateReferencePage] = "LegStreamCommodityRateReferencePage";             // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityRateReferencePageHeading] = "LegStreamCommodityRateReferencePageHeading";      // (String FIX.5.0SP2)
dictionary[tag::LegStreamDataProvider]             = "LegStreamDataProvider";                           // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityPricingType]     = "LegStreamCommodityPricingType";                   // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityNearbySettlDayPeriod] = "LegStreamCommodityNearbySettlDayPeriod";          // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommodityNearbySettlDayUnit] = "LegStreamCommodityNearbySettlDayUnit";            // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlDateUnadjusted] = "LegStreamCommoditySettlDateUnadjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlDateBusinessDayConvention] = "LegStreamCommoditySettlDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlDateAdjusted] = "LegStreamCommoditySettlDateAdjusted";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlMonth]      = "LegStreamCommoditySettlMonth";                    // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlDateRollPeriod] = "LegStreamCommoditySettlDateRollPeriod";           // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlDateRollUnit] = "LegStreamCommoditySettlDateRollUnit";             // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlDayType]    = "LegStreamCommoditySettlDayType";                  // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommodityXID]             = "LegStreamCommodityXID";                           // (XID FIX.5.0SP2)
dictionary[tag::LegStreamCommodityXIDRef]          = "LegStreamCommodityXIDRef";                        // (XIDREF FIX.5.0SP2)
dictionary[tag::NoLegStreamCommodityAltIDs]        = "NoLegStreamCommodityAltIDs";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamCommodityAltID]           = "LegStreamCommodityAltID";                         // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityAltIDSource]     = "LegStreamCommodityAltIDSource";                   // (String FIX.5.0SP2)
dictionary[tag::NoLegStreamCommodityDataSources]   = "NoLegStreamCommodityDataSources";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamCommodityDataSourceID]    = "LegStreamCommodityDataSourceID";                  // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityDataSourceIDType] = "LegStreamCommodityDataSourceIDType";              // (int FIX.5.0SP2)
dictionary[tag::NoLegStreamCommoditySettlDays]     = "NoLegStreamCommoditySettlDays";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlDay]        = "LegStreamCommoditySettlDay";                      // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlTotalHours] = "LegStreamCommoditySettlTotalHours";               // (int FIX.5.0SP2)
dictionary[tag::NoLegStreamCommoditySettlTimes]    = "NoLegStreamCommoditySettlTimes";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlStart]      = "LegStreamCommoditySettlStart";                    // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlEnd]        = "LegStreamCommoditySettlEnd";                      // (String FIX.5.0SP2)
dictionary[tag::NoLegStreamCommoditySettlPeriods]  = "NoLegStreamCommoditySettlPeriods";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlCountry]    = "LegStreamCommoditySettlCountry";                  // (Country FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlTimeZone]   = "LegStreamCommoditySettlTimeZone";                 // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlFlowType]   = "LegStreamCommoditySettlFlowType";                 // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodNotional] = "LegStreamCommoditySettlPeriodNotional";           // (Qty FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodNotionalUnitOfMeasure] = "LegStreamCommoditySettlPeriodNotionalUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodFrequencyPeriod] = "LegStreamCommoditySettlPeriodFrequencyPeriod";    // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodFrequencyUnit] = "LegStreamCommoditySettlPeriodFrequencyUnit";      // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodPrice] = "LegStreamCommoditySettlPeriodPrice";              // (Price FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodPriceUnitOfMeasure] = "LegStreamCommoditySettlPeriodPriceUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodPriceCurrency] = "LegStreamCommoditySettlPeriodPriceCurrency";      // (Currency FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlHolidaysProcessingInstruction] = "LegStreamCommoditySettlHolidaysProcessingInstruction"; // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodXID]  = "LegStreamCommoditySettlPeriodXID";                // (XID FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlPeriodXIDRef] = "LegStreamCommoditySettlPeriodXIDRef";             // (XIDREF FIX.5.0SP2)
dictionary[tag::LegStreamXID]                      = "LegStreamXID";                                    // (XID FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondSecurityIDSource] = "UnderlyingAdditionalTermBondSecurityIDSource";    // (String FIX.5.0SP2)
dictionary[tag::LegStreamNotionalXIDRef]           = "LegStreamNotionalXIDRef";                         // (XIDREF FIX.5.0SP2)
dictionary[tag::LegStreamNotionalFrequencyPeriod]  = "LegStreamNotionalFrequencyPeriod";                // (int FIX.5.0SP2)
dictionary[tag::LegStreamNotionalFrequencyUnit]    = "LegStreamNotionalFrequencyUnit";                  // (String FIX.5.0SP2)
dictionary[tag::LegStreamNotionalCommodityFrequency] = "LegStreamNotionalCommodityFrequency";             // (int FIX.5.0SP2)
dictionary[tag::LegStreamNotionalUnitOfMeasure]    = "LegStreamNotionalUnitOfMeasure";                  // (String FIX.5.0SP2)
dictionary[tag::LegStreamTotalNotional]            = "LegStreamTotalNotional";                          // (Qty FIX.5.0SP2)
dictionary[tag::LegStreamTotalNotionalUnitOfMeasure] = "LegStreamTotalNotionalUnitOfMeasure";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondDesc]  = "UnderlyingAdditionalTermBondDesc";                // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingAdditionalTermBondDescLen] = "EncodedUnderlyingAdditionalTermBondDescLen";      // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingAdditionalTermBondDesc] = "EncodedUnderlyingAdditionalTermBondDesc";         // (data FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondCurrency] = "UnderlyingAdditionalTermBondCurrency";            // (Currency FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventAveragingObservations] = "NoUnderlyingComplexEventAveragingObservations";   // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventAveragingObservationNumber] = "UnderlyingComplexEventAveragingObservationNumber"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventAveragingWeight] = "UnderlyingComplexEventAveragingWeight";           // (float FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventCreditEvents] = "NoUnderlyingComplexEventCreditEvents";            // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventType] = "UnderlyingComplexEventCreditEventType";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventValue] = "UnderlyingComplexEventCreditEventValue";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventCurrency] = "UnderlyingComplexEventCreditEventCurrency";       // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventPeriod] = "UnderlyingComplexEventCreditEventPeriod";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventUnit] = "UnderlyingComplexEventCreditEventUnit";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventDayType] = "UnderlyingComplexEventCreditEventDayType";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventRateSource] = "UnderlyingComplexEventCreditEventRateSource";     // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventCreditEventQualifiers] = "NoUnderlyingComplexEventCreditEventQualifiers";   // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventQualifier] = "UnderlyingComplexEventCreditEventQualifier";      // (char FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventPeriodDateTimes] = "NoUnderlyingComplexEventPeriodDateTimes";         // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPeriodDate]  = "UnderlyingComplexEventPeriodDate";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPeriodTime]  = "UnderlyingComplexEventPeriodTime";                // (LocalMktTime FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventPeriods]   = "NoUnderlyingComplexEventPeriods";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventPeriodType]  = "UnderlyingComplexEventPeriodType";                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventBusinessCenter] = "UnderlyingComplexEventBusinessCenter";            // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventRateSources] = "NoUnderlyingComplexEventRateSources";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventRateSource]  = "UnderlyingComplexEventRateSource";                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventRateSourceType] = "UnderlyingComplexEventRateSourceType";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventReferencePage] = "UnderlyingComplexEventReferencePage";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventReferencePageHeading] = "UnderlyingComplexEventReferencePageHeading";      // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventDateBusinessCenters] = "NoUnderlyingComplexEventDateBusinessCenters";     // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDateBusinessCenter] = "UnderlyingComplexEventDateBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDateUnadjusted] = "UnderlyingComplexEventDateUnadjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDateRelativeTo] = "UnderlyingComplexEventDateRelativeTo";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDateOffsetPeriod] = "UnderlyingComplexEventDateOffsetPeriod";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDateOffsetUnit] = "UnderlyingComplexEventDateOffsetUnit";            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDateOffsetDayType] = "UnderlyingComplexEventDateOffsetDayType";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDateBusinessDayConvention] = "UnderlyingComplexEventDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventDateAdjusted] = "UnderlyingComplexEventDateAdjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventFixingTime]  = "UnderlyingComplexEventFixingTime";                // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventFixingTimeBusinessCenter] = "UnderlyingComplexEventFixingTimeBusinessCenter";  // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventCreditEventSources] = "NoUnderlyingComplexEventCreditEventSources";      // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventCreditEventSource] = "UnderlyingComplexEventCreditEventSource";         // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingComplexEventSchedules] = "NoUnderlyingComplexEventSchedules";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventScheduleStartDate] = "UnderlyingComplexEventScheduleStartDate";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventScheduleEndDate] = "UnderlyingComplexEventScheduleEndDate";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventScheduleFrequencyPeriod] = "UnderlyingComplexEventScheduleFrequencyPeriod";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventScheduleFrequencyUnit] = "UnderlyingComplexEventScheduleFrequencyUnit";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingComplexEventScheduleRollConvention] = "UnderlyingComplexEventScheduleRollConvention";    // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingDeliverySchedules]     = "NoUnderlyingDeliverySchedules";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleType]    = "UnderlyingDeliveryScheduleType";                  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleXID]     = "UnderlyingDeliveryScheduleXID";                   // (XID FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleNotional] = "UnderlyingDeliveryScheduleNotional";              // (Qty FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleNotionalUnitOfMeasure] = "UnderlyingDeliveryScheduleNotionalUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleNotionalCommodityFrequency] = "UnderlyingDeliveryScheduleNotionalCommodityFrequency"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleNegativeTolerance] = "UnderlyingDeliveryScheduleNegativeTolerance";     // (float FIX.5.0SP2)
dictionary[tag::UnderlyingDeliverySchedulePositiveTolerance] = "UnderlyingDeliverySchedulePositiveTolerance";     // (float FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleToleranceUnitOfMeasure] = "UnderlyingDeliveryScheduleToleranceUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleToleranceType] = "UnderlyingDeliveryScheduleToleranceType";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlCountry] = "UnderlyingDeliveryScheduleSettlCountry";          // (Country FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlTimeZone] = "UnderlyingDeliveryScheduleSettlTimeZone";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlFlowType] = "UnderlyingDeliveryScheduleSettlFlowType";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlHolidaysProcessingInstruction] = "UnderlyingDeliveryScheduleSettlHolidaysProcessingInstruction"; // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingDeliveryScheduleSettlDays] = "NoUnderlyingDeliveryScheduleSettlDays";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlDay] = "UnderlyingDeliveryScheduleSettlDay";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlTotalHours] = "UnderlyingDeliveryScheduleSettlTotalHours";       // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingDeliveryScheduleSettlTimes] = "NoUnderlyingDeliveryScheduleSettlTimes";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlStart] = "UnderlyingDeliveryScheduleSettlStart";            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlEnd] = "UnderlyingDeliveryScheduleSettlEnd";              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryScheduleSettlTimeType] = "UnderlyingDeliveryScheduleSettlTimeType";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamType]      = "UnderlyingDeliveryStreamType";                    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamPipeline]  = "UnderlyingDeliveryStreamPipeline";                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamEntryPoint] = "UnderlyingDeliveryStreamEntryPoint";              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamWithdrawalPoint] = "UnderlyingDeliveryStreamWithdrawalPoint";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamDeliveryPoint] = "UnderlyingDeliveryStreamDeliveryPoint";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamDeliveryRestriction] = "UnderlyingDeliveryStreamDeliveryRestriction";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamDeliveryContingency] = "UnderlyingDeliveryStreamDeliveryContingency";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamDeliveryContingentPartySide] = "UnderlyingDeliveryStreamDeliveryContingentPartySide"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamDeliverAtSourceIndicator] = "UnderlyingDeliveryStreamDeliverAtSourceIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamRiskApportionment] = "UnderlyingDeliveryStreamRiskApportionment";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamTitleTransferLocation] = "UnderlyingDeliveryStreamTitleTransferLocation";   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamTitleTransferCondition] = "UnderlyingDeliveryStreamTitleTransferCondition";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamImporterOfRecord] = "UnderlyingDeliveryStreamImporterOfRecord";        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamNegativeTolerance] = "UnderlyingDeliveryStreamNegativeTolerance";       // (float FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamPositiveTolerance] = "UnderlyingDeliveryStreamPositiveTolerance";       // (float FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamToleranceUnitOfMeasure] = "UnderlyingDeliveryStreamToleranceUnitOfMeasure";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamToleranceType] = "UnderlyingDeliveryStreamToleranceType";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamToleranceOptionSide] = "UnderlyingDeliveryStreamToleranceOptionSide";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamTotalPositiveTolerance] = "UnderlyingDeliveryStreamTotalPositiveTolerance";  // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamTotalNegativeTolerance] = "UnderlyingDeliveryStreamTotalNegativeTolerance";  // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamNotionalConversionFactor] = "UnderlyingDeliveryStreamNotionalConversionFactor"; // (float FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamTransportEquipment] = "UnderlyingDeliveryStreamTransportEquipment";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamElectingPartySide] = "UnderlyingDeliveryStreamElectingPartySide";       // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamAssetAttributes] = "NoUnderlyingStreamAssetAttributes";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamAssetAttributeType] = "UnderlyingStreamAssetAttributeType";              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamAssetAttributeValue] = "UnderlyingStreamAssetAttributeValue";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamAssetAttributeLimit] = "UnderlyingStreamAssetAttributeLimit";             // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingDeliveryStreamCycles]  = "NoUnderlyingDeliveryStreamCycles";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamCycleDesc] = "UnderlyingDeliveryStreamCycleDesc";               // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingDeliveryStreamCycleDescLen] = "EncodedUnderlyingDeliveryStreamCycleDescLen";     // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingDeliveryStreamCycleDesc] = "EncodedUnderlyingDeliveryStreamCycleDesc";        // (data FIX.5.0SP2)
dictionary[tag::NoUnderlyingDeliveryStreamCommoditySources] = "NoUnderlyingDeliveryStreamCommoditySources";      // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamCommoditySource] = "UnderlyingDeliveryStreamCommoditySource";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingExerciseDesc]            = "UnderlyingExerciseDesc";                          // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingExerciseDescLen]  = "EncodedUnderlyingExerciseDescLen";                // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingExerciseDesc]     = "EncodedUnderlyingExerciseDesc";                   // (data FIX.5.0SP2)
dictionary[tag::UnderlyingAutomaticExerciseIndicator] = "UnderlyingAutomaticExerciseIndicator";            // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingAutomaticExerciseThresholdRate] = "UnderlyingAutomaticExerciseThresholdRate";        // (float FIX.5.0SP2)
dictionary[tag::UnderlyingExerciseConfirmationMethod] = "UnderlyingExerciseConfirmationMethod";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingManualNoticeBusinessCenter] = "UnderlyingManualNoticeBusinessCenter";            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingFallbackExerciseIndicator] = "UnderlyingFallbackExerciseIndicator";             // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingLimitedRightToConfirmIndicator] = "UnderlyingLimitedRightToConfirmIndicator";        // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingExerciseSplitTicketIndicator] = "UnderlyingExerciseSplitTicketIndicator";          // (Boolean FIX.5.0SP2)
dictionary[tag::NoUnderlyingOptionExerciseBusinessCenters] = "NoUnderlyingOptionExerciseBusinessCenters";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseBusinessCenter] = "UnderlyingOptionExerciseBusinessCenter";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseBusinessDayConvention] = "UnderlyingOptionExerciseBusinessDayConvention";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseEarliestDateOffsetDayType] = "UnderlyingOptionExerciseEarliestDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseEarliestDateOffsetPeriod] = "UnderlyingOptionExerciseEarliestDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseEarliestDateOffsetUnit] = "UnderlyingOptionExerciseEarliestDateOffsetUnit";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseFrequencyPeriod] = "UnderlyingOptionExerciseFrequencyPeriod";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseFrequencyUnit] = "UnderlyingOptionExerciseFrequencyUnit";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseStartDateUnadjusted] = "UnderlyingOptionExerciseStartDateUnadjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseStartDateRelativeTo] = "UnderlyingOptionExerciseStartDateRelativeTo";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseStartDateOffsetPeriod] = "UnderlyingOptionExerciseStartDateOffsetPeriod";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseStartDateOffsetUnit] = "UnderlyingOptionExerciseStartDateOffsetUnit";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseStartDateOffsetDayType] = "UnderlyingOptionExerciseStartDateOffsetDayType";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseStartDateAdjusted] = "UnderlyingOptionExerciseStartDateAdjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseSkip]      = "UnderlyingOptionExerciseSkip";                    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseNominationDeadline] = "UnderlyingOptionExerciseNominationDeadline";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseFirstDateUnadjusted] = "UnderlyingOptionExerciseFirstDateUnadjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseLastDateUnadjusted] = "UnderlyingOptionExerciseLastDateUnadjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseEarliestTime] = "UnderlyingOptionExerciseEarliestTime";            // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseLatestTime] = "UnderlyingOptionExerciseLatestTime";              // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseTimeBusinessCenter] = "UnderlyingOptionExerciseTimeBusinessCenter";      // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingOptionExerciseDates]   = "NoUnderlyingOptionExerciseDates";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseDate]      = "UnderlyingOptionExerciseDate";                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseDateType]  = "UnderlyingOptionExerciseDateType";                // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingOptionExerciseExpirationDateBusinessCenters] = "NoUnderlyingOptionExerciseExpirationDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationDateBusinessCenter] = "UnderlyingOptionExerciseExpirationDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationDateBusinessDayConvention] = "UnderlyingOptionExerciseExpirationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationDateRelativeTo] = "UnderlyingOptionExerciseExpirationDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationDateOffsetPeriod] = "UnderlyingOptionExerciseExpirationDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationDateOffsetUnit] = "UnderlyingOptionExerciseExpirationDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationFrequencyPeriod] = "UnderlyingOptionExerciseExpirationFrequencyPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationFrequencyUnit] = "UnderlyingOptionExerciseExpirationFrequencyUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationRollConvention] = "UnderlyingOptionExerciseExpirationRollConvention"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationDateOffsetDayType] = "UnderlyingOptionExerciseExpirationDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationTime] = "UnderlyingOptionExerciseExpirationTime";          // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationTimeBusinessCenter] = "UnderlyingOptionExerciseExpirationTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingOptionExerciseExpirationDates] = "NoUnderlyingOptionExerciseExpirationDates";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationDate] = "UnderlyingOptionExerciseExpirationDate";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingOptionExerciseExpirationDateType] = "UnderlyingOptionExerciseExpirationDateType";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionProvision] = "UnderlyingMarketDisruptionProvision";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackProvision] = "UnderlyingMarketDisruptionFallbackProvision";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionMaximumDays] = "UnderlyingMarketDisruptionMaximumDays";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionMaterialityPercentage] = "UnderlyingMarketDisruptionMaterialityPercentage"; // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionMinimumFuturesContracts] = "UnderlyingMarketDisruptionMinimumFuturesContracts"; // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingMarketDisruptionEvents] = "NoUnderlyingMarketDisruptionEvents";              // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionEvent]   = "UnderlyingMarketDisruptionEvent";                 // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingMarketDisruptionFallbacks] = "NoUnderlyingMarketDisruptionFallbacks";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackType] = "UnderlyingMarketDisruptionFallbackType";          // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingMarketDisruptionFallbackReferencePrices] = "NoUnderlyingMarketDisruptionFallbackReferencePrices"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackUnderlierType] = "UnderlyingMarketDisruptionFallbackUnderlierType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackUnderlierSecurityID] = "UnderlyingMarketDisruptionFallbackUnderlierSecurityID"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackUnderlierSecurityIDSource] = "UnderlyingMarketDisruptionFallbackUnderlierSecurityIDSource"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackUnderlierSecurityDesc] = "UnderlyingMarketDisruptionFallbackUnderlierSecurityDesc"; // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingMarketDisruptionFallbackUnderlierSecurityDescLen] = "EncodedUnderlyingMarketDisruptionFallbackUnderlierSecurityDescLen"; // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingMarketDisruptionFallbackUnderlierSecurityDesc] = "EncodedUnderlyingMarketDisruptionFallbackUnderlierSecurityDesc"; // (data FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackOpenUnits] = "UnderlyingMarketDisruptionFallbackOpenUnits";     // (Qty FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackBasketCurrency] = "UnderlyingMarketDisruptionFallbackBasketCurrency"; // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingMarketDisruptionFallbackBasketDivisor] = "UnderlyingMarketDisruptionFallbackBasketDivisor"; // (float FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentScheduleFixingDays] = "NoUnderlyingPaymentScheduleFixingDays";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDayOfWeek] = "UnderlyingPaymentScheduleFixingDayOfWeek";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDayNumber] = "UnderlyingPaymentScheduleFixingDayNumber";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleXID]      = "UnderlyingPaymentScheduleXID";                    // (XID FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleXIDRef]   = "UnderlyingPaymentScheduleXIDRef";                 // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateCurrency] = "UnderlyingPaymentScheduleRateCurrency";           // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateUnitOfMeasure] = "UnderlyingPaymentScheduleRateUnitOfMeasure";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateConversionFactor] = "UnderlyingPaymentScheduleRateConversionFactor";   // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleRateSpreadType] = "UnderlyingPaymentScheduleRateSpreadType";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleSettlPeriodPrice] = "UnderlyingPaymentScheduleSettlPeriodPrice";       // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleSettlPeriodPriceCurrency] = "UnderlyingPaymentScheduleSettlPeriodPriceCurrency"; // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleSettlPeriodPriceUnitOfMeasure] = "UnderlyingPaymentScheduleSettlPeriodPriceUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleStepUnitOfMeasure] = "UnderlyingPaymentScheduleStepUnitOfMeasure";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDayDistribution] = "UnderlyingPaymentScheduleFixingDayDistribution";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingDayCount] = "UnderlyingPaymentScheduleFixingDayCount";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingLagPeriod] = "UnderlyingPaymentScheduleFixingLagPeriod";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingLagUnit] = "UnderlyingPaymentScheduleFixingLagUnit";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingFirstObservationDateOffsetPeriod] = "UnderlyingPaymentScheduleFixingFirstObservationDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentScheduleFixingFirstObservationDateOffsetUnit] = "UnderlyingPaymentScheduleFixingFirstObservationDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFlatRateIndicator] = "UnderlyingPaymentStreamFlatRateIndicator";        // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFlatRateAmount] = "UnderlyingPaymentStreamFlatRateAmount";           // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFlatRateCurrency] = "UnderlyingPaymentStreamFlatRateCurrency";         // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamMaximumPaymentAmount] = "UnderlyingPaymentStreamMaximumPaymentAmount";     // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamMaximumPaymentCurrency] = "UnderlyingPaymentStreamMaximumPaymentCurrency";   // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamMaximumTransactionAmount] = "UnderlyingPaymentStreamMaximumTransactionAmount"; // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamMaximumTransactionCurrency] = "UnderlyingPaymentStreamMaximumTransactionCurrency"; // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixedAmountUnitOfMeasure] = "UnderlyingPaymentStreamFixedAmountUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamTotalFixedAmount] = "UnderlyingPaymentStreamTotalFixedAmount";         // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamWorldScaleRate] = "UnderlyingPaymentStreamWorldScaleRate";           // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamContractPrice] = "UnderlyingPaymentStreamContractPrice";            // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamContractPriceCurrency] = "UnderlyingPaymentStreamContractPriceCurrency";    // (Currency FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamPricingBusinessCenters] = "NoUnderlyingPaymentStreamPricingBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingBusinessCenter] = "UnderlyingPaymentStreamPricingBusinessCenter";    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndex2CurveUnit] = "UnderlyingPaymentStreamRateIndex2CurveUnit";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndex2CurvePeriod] = "UnderlyingPaymentStreamRateIndex2CurvePeriod";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndexLocation] = "UnderlyingPaymentStreamRateIndexLocation";        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndexLevel] = "UnderlyingPaymentStreamRateIndexLevel";           // (Qty FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndexUnitOfMeasure] = "UnderlyingPaymentStreamRateIndexUnitOfMeasure";   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamSettlLevel] = "UnderlyingPaymentStreamSettlLevel";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamReferenceLevel] = "UnderlyingPaymentStreamReferenceLevel";           // (Qty FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamReferenceLevelUnitOfMeasure] = "UnderlyingPaymentStreamReferenceLevelUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamReferenceLevelEqualsZeroIndicator] = "UnderlyingPaymentStreamReferenceLevelEqualsZeroIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateSpreadCurrency] = "UnderlyingPaymentStreamRateSpreadCurrency";       // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateSpreadUnitOfMeasure] = "UnderlyingPaymentStreamRateSpreadUnitOfMeasure";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateConversionFactor] = "UnderlyingPaymentStreamRateConversionFactor";     // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateSpreadType] = "UnderlyingPaymentStreamRateSpreadType";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLastResetRate] = "UnderlyingPaymentStreamLastResetRate";            // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalRate]  = "UnderlyingPaymentStreamFinalRate";                // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCalculationLagPeriod] = "UnderlyingPaymentStreamCalculationLagPeriod";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCalculationLagUnit] = "UnderlyingPaymentStreamCalculationLagUnit";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFirstObservationDateOffsetPeriod] = "UnderlyingPaymentStreamFirstObservationDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFirstObservationDateOffsetUnit] = "UnderlyingPaymentStreamFirstObservationDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingDayType] = "UnderlyingPaymentStreamPricingDayType";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingDayDistribution] = "UnderlyingPaymentStreamPricingDayDistribution";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingDayCount] = "UnderlyingPaymentStreamPricingDayCount";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingBusinessCalendar] = "UnderlyingPaymentStreamPricingBusinessCalendar";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingBusinessDayConvention] = "UnderlyingPaymentStreamPricingBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegStreamCommoditySettlTimeType]   = "LegStreamCommoditySettlTimeType";                 // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlTimeType] = "UnderlyingStreamCommoditySettlTimeType";          // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamPaymentDates] = "NoUnderlyingPaymentStreamPaymentDates";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentDate] = "UnderlyingPaymentStreamPaymentDate";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPaymentDateType] = "UnderlyingPaymentStreamPaymentDateType";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamMasterAgreementPaymentDatesIndicator] = "UnderlyingPaymentStreamMasterAgreementPaymentDatesIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamPricingDates] = "NoUnderlyingPaymentStreamPricingDates";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingDate] = "UnderlyingPaymentStreamPricingDate";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingDateType] = "UnderlyingPaymentStreamPricingDateType";          // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamPricingDays] = "NoUnderlyingPaymentStreamPricingDays";            // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingDayOfWeek] = "UnderlyingPaymentStreamPricingDayOfWeek";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamPricingDayNumber] = "UnderlyingPaymentStreamPricingDayNumber";         // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingPricingDateBusinessCenters] = "NoUnderlyingPricingDateBusinessCenters";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPricingDateBusinessCenter] = "UnderlyingPricingDateBusinessCenter";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPricingDateUnadjusted]   = "UnderlyingPricingDateUnadjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPricingDateBusinessDayConvention] = "UnderlyingPricingDateBusinessDayConvention";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPricingDateAdjusted]     = "UnderlyingPricingDateAdjusted";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPricingTime]             = "UnderlyingPricingTime";                           // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingPricingTimeBusinessCenter] = "UnderlyingPricingTimeBusinessCenter";             // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamCalculationPeriodDates] = "NoUnderlyingStreamCalculationPeriodDates";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationPeriodDate] = "UnderlyingStreamCalculationPeriodDate";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationPeriodDateType] = "UnderlyingStreamCalculationPeriodDateType";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationPeriodDatesXID] = "UnderlyingStreamCalculationPeriodDatesXID";       // (XID FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationPeriodDatesXIDRef] = "UnderlyingStreamCalculationPeriodDatesXIDRef";    // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationBalanceOfFirstPeriod] = "UnderlyingStreamCalculationBalanceOfFirstPeriod"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationCorrectionPeriod] = "UnderlyingStreamCalculationCorrectionPeriod";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCalculationCorrectionUnit] = "UnderlyingStreamCalculationCorrectionUnit";       // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamCommoditySettlBusinessCenters] = "NoUnderlyingStreamCommoditySettlBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlBusinessCenter] = "UnderlyingStreamCommoditySettlBusinessCenter";    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityBase]     = "UnderlyingStreamCommodityBase";                   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityType]     = "UnderlyingStreamCommodityType";                   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySecurityID] = "UnderlyingStreamCommoditySecurityID";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySecurityIDSource] = "UnderlyingStreamCommoditySecurityIDSource";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityDesc]     = "UnderlyingStreamCommodityDesc";                   // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingStreamCommodityDescLen] = "EncodedUnderlyingStreamCommodityDescLen";         // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingStreamCommodityDesc] = "EncodedUnderlyingStreamCommodityDesc";            // (data FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityUnitOfMeasure] = "UnderlyingStreamCommodityUnitOfMeasure";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityCurrency] = "UnderlyingStreamCommodityCurrency";               // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityExchange] = "UnderlyingStreamCommodityExchange";               // (Exchange FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityRateSource] = "UnderlyingStreamCommodityRateSource";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityRateReferencePage] = "UnderlyingStreamCommodityRateReferencePage";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityRateReferencePageHeading] = "UnderlyingStreamCommodityRateReferencePageHeading"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamDataProvider]      = "UnderlyingStreamDataProvider";                    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityPricingType] = "UnderlyingStreamCommodityPricingType";            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityNearbySettlDayPeriod] = "UnderlyingStreamCommodityNearbySettlDayPeriod";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityNearbySettlDayUnit] = "UnderlyingStreamCommodityNearbySettlDayUnit";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlDateUnadjusted] = "UnderlyingStreamCommoditySettlDateUnadjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlDateBusinessDayConvention] = "UnderlyingStreamCommoditySettlDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlDateAdjusted] = "UnderlyingStreamCommoditySettlDateAdjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlMonth] = "UnderlyingStreamCommoditySettlMonth";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlDateRollPeriod] = "UnderlyingStreamCommoditySettlDateRollPeriod";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlDateRollUnit] = "UnderlyingStreamCommoditySettlDateRollUnit";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlDayType] = "UnderlyingStreamCommoditySettlDayType";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityXID]      = "UnderlyingStreamCommodityXID";                    // (XID FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityXIDRef]   = "UnderlyingStreamCommodityXIDRef";                 // (XIDREF FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamCommodityAltIDs] = "NoUnderlyingStreamCommodityAltIDs";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityAltID]    = "UnderlyingStreamCommodityAltID";                  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityAltIDSource] = "UnderlyingStreamCommodityAltIDSource";            // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamCommodityDataSources] = "NoUnderlyingStreamCommodityDataSources";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityDataSourceID] = "UnderlyingStreamCommodityDataSourceID";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityDataSourceIDType] = "UnderlyingStreamCommodityDataSourceIDType";       // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamCommoditySettlDays] = "NoUnderlyingStreamCommoditySettlDays";            // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlDay] = "UnderlyingStreamCommoditySettlDay";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlTotalHours] = "UnderlyingStreamCommoditySettlTotalHours";        // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamCommoditySettlTimes] = "NoUnderlyingStreamCommoditySettlTimes";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlStart] = "UnderlyingStreamCommoditySettlStart";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlEnd] = "UnderlyingStreamCommoditySettlEnd";               // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingStreamCommoditySettlPeriods] = "NoUnderlyingStreamCommoditySettlPeriods";         // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlCountry] = "UnderlyingStreamCommoditySettlCountry";           // (Country FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlTimeZone] = "UnderlyingStreamCommoditySettlTimeZone";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlFlowType] = "UnderlyingStreamCommoditySettlFlowType";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodNotional] = "UnderlyingStreamCommoditySettlPeriodNotional";    // (Qty FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodNotionalUnitOfMeasure] = "UnderlyingStreamCommoditySettlPeriodNotionalUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodFrequencyPeriod] = "UnderlyingStreamCommoditySettlPeriodFrequencyPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodFrequencyUnit] = "UnderlyingStreamCommoditySettlPeriodFrequencyUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodPrice] = "UnderlyingStreamCommoditySettlPeriodPrice";       // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodPriceUnitOfMeasure] = "UnderlyingStreamCommoditySettlPeriodPriceUnitOfMeasure"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodPriceCurrency] = "UnderlyingStreamCommoditySettlPeriodPriceCurrency"; // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlHolidaysProcessingInstruction] = "UnderlyingStreamCommoditySettlHolidaysProcessingInstruction"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodXID] = "UnderlyingStreamCommoditySettlPeriodXID";         // (XID FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommoditySettlPeriodXIDRef] = "UnderlyingStreamCommoditySettlPeriodXIDRef";      // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingStreamXID]               = "UnderlyingStreamXID";                             // (XID FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondIssuer] = "UnderlyingAdditionalTermBondIssuer";              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamNotionalXIDRef]    = "UnderlyingStreamNotionalXIDRef";                  // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingStreamNotionalFrequencyPeriod] = "UnderlyingStreamNotionalFrequencyPeriod";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamNotionalFrequencyUnit] = "UnderlyingStreamNotionalFrequencyUnit";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamNotionalCommodityFrequency] = "UnderlyingStreamNotionalCommodityFrequency";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingStreamNotionalUnitOfMeasure] = "UnderlyingStreamNotionalUnitOfMeasure";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTotalNotional]     = "UnderlyingStreamTotalNotional";                   // (Qty FIX.5.0SP2)
dictionary[tag::UnderlyingStreamTotalNotionalUnitOfMeasure] = "UnderlyingStreamTotalNotionalUnitOfMeasure";      // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingAdditionalTermBondIssuerLen] = "EncodedUnderlyingAdditionalTermBondIssuerLen";    // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingAdditionalTermBondIssuer] = "EncodedUnderlyingAdditionalTermBondIssuer";       // (data FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondSeniority] = "UnderlyingAdditionalTermBondSeniority";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondCouponType] = "UnderlyingAdditionalTermBondCouponType";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondCouponRate] = "UnderlyingAdditionalTermBondCouponRate";          // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondMaturityDate] = "UnderlyingAdditionalTermBondMaturityDate";        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondParValue] = "UnderlyingAdditionalTermBondParValue";            // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondCurrentTotalIssuedAmount] = "UnderlyingAdditionalTermBondCurrentTotalIssuedAmount"; // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondCouponFrequencyPeriod] = "UnderlyingAdditionalTermBondCouponFrequencyPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondCouponFrequencyUnit] = "UnderlyingAdditionalTermBondCouponFrequencyUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermBondDayCount] = "UnderlyingAdditionalTermBondDayCount";            // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingAdditionalTerms]       = "NoUnderlyingAdditionalTerms";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermConditionPrecedentBondIndicator] = "UnderlyingAdditionalTermConditionPrecedentBondIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalTermDiscrepancyClauseIndicator] = "UnderlyingAdditionalTermDiscrepancyClauseIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::NoUnderlyingCashSettlDealers]      = "NoUnderlyingCashSettlDealers";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDealer]         = "UnderlyingCashSettlDealer";                       // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingCashSettlTerms]        = "NoUnderlyingCashSettlTerms";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlCurrency]       = "UnderlyingCashSettlCurrency";                     // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlValuationFirstBusinessDayOffset] = "UnderlyingCashSettlValuationFirstBusinessDayOffset"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlValuationSubsequentBusinessDaysOffset] = "UnderlyingCashSettlValuationSubsequentBusinessDaysOffset"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlNumOfValuationDates] = "UnderlyingCashSettlNumOfValuationDates";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlValuationTime]  = "UnderlyingCashSettlValuationTime";                // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlBusinessCenter] = "UnderlyingCashSettlBusinessCenter";               // (String FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlQuoteMethod]    = "UnderlyingCashSettlQuoteMethod";                  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlQuoteAmount]    = "UnderlyingCashSettlQuoteAmount";                  // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlQuoteCurrency]  = "UnderlyingCashSettlQuoteCurrency";                // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlMinimumQuoteAmount] = "UnderlyingCashSettlMinimumQuoteAmount";           // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlMinimumQuoteCurrency] = "UnderlyingCashSettlMinimumQuoteCurrency";         // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlBusinessDays]   = "UnderlyingCashSettlBusinessDays";                 // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlAmount]         = "UnderlyingCashSettlAmount";                       // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlRecoveryFactor] = "UnderlyingCashSettlRecoveryFactor";               // (float FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlFixedTermIndicator] = "UnderlyingCashSettlFixedTermIndicator";           // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlAccruedInterestIndicator] = "UnderlyingCashSettlAccruedInterestIndicator";     // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlValuationMethod] = "UnderlyingCashSettlValuationMethod";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlTermXID]        = "UnderlyingCashSettlTermXID";                      // (XID FIX.5.0SP2)
dictionary[tag::NoUnderlyingPhysicalSettlTerms]    = "NoUnderlyingPhysicalSettlTerms";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPhysicalSettlCurrency]   = "UnderlyingPhysicalSettlCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPhysicalSettlBusinessDays] = "UnderlyingPhysicalSettlBusinessDays";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPhysicalSettlMaximumBusinessDays] = "UnderlyingPhysicalSettlMaximumBusinessDays";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPhysicalSettlTermXID]    = "UnderlyingPhysicalSettlTermXID";                  // (XID FIX.5.0SP2)
dictionary[tag::NoUnderlyingPhysicalSettlDeliverableObligations] = "NoUnderlyingPhysicalSettlDeliverableObligations"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPhysicalSettlDeliverableObligationType] = "UnderlyingPhysicalSettlDeliverableObligationType"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPhysicalSettlDeliverableObligationValue] = "UnderlyingPhysicalSettlDeliverableObligationValue"; // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingProtectionTerms]       = "NoUnderlyingProtectionTerms";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermNotional]  = "UnderlyingProtectionTermNotional";                // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermCurrency]  = "UnderlyingProtectionTermCurrency";                // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermSellerNotifies] = "UnderlyingProtectionTermSellerNotifies";          // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermBuyerNotifies] = "UnderlyingProtectionTermBuyerNotifies";           // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventBusinessCenter] = "UnderlyingProtectionTermEventBusinessCenter";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermStandardSources] = "UnderlyingProtectionTermStandardSources";         // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventMinimumSources] = "UnderlyingProtectionTermEventMinimumSources";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermXID]       = "UnderlyingProtectionTermXID";                     // (XID FIX.5.0SP2)
dictionary[tag::NoUnderlyingProtectionTermEvents]  = "NoUnderlyingProtectionTermEvents";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventType] = "UnderlyingProtectionTermEventType";               // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventValue] = "UnderlyingProtectionTermEventValue";              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventCurrency] = "UnderlyingProtectionTermEventCurrency";           // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventPeriod] = "UnderlyingProtectionTermEventPeriod";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventUnit] = "UnderlyingProtectionTermEventUnit";               // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventDayType] = "UnderlyingProtectionTermEventDayType";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventRateSource] = "UnderlyingProtectionTermEventRateSource";         // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingProtectionTermEventQualifiers] = "NoUnderlyingProtectionTermEventQualifiers";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventQualifier] = "UnderlyingProtectionTermEventQualifier";          // (char FIX.5.0SP2)
dictionary[tag::NoUnderlyingProtectionTermObligations] = "NoUnderlyingProtectionTermObligations";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermObligationType] = "UnderlyingProtectionTermObligationType";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermObligationValue] = "UnderlyingProtectionTermObligationValue";         // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingProtectionTermEventNewsSources] = "NoUnderlyingProtectionTermEventNewsSources";      // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProtectionTermEventNewsSource] = "UnderlyingProtectionTermEventNewsSource";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateBusinessDayConvention] = "UnderlyingProvisionCashSettlPaymentDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateRelativeTo] = "UnderlyingProvisionCashSettlPaymentDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateOffsetPeriod] = "UnderlyingProvisionCashSettlPaymentDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateOffsetUnit] = "UnderlyingProvisionCashSettlPaymentDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateOffsetDayType] = "UnderlyingProvisionCashSettlPaymentDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateRangeFirst] = "UnderlyingProvisionCashSettlPaymentDateRangeFirst"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateRangeLast] = "UnderlyingProvisionCashSettlPaymentDateRangeLast"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionCashSettlPaymentDates] = "NoUnderlyingProvisionCashSettlPaymentDates";      // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDate] = "UnderlyingProvisionCashSettlPaymentDate";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateType] = "UnderlyingProvisionCashSettlPaymentDateType";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlQuoteSource] = "UnderlyingProvisionCashSettlQuoteSource";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlQuoteReferencePage] = "UnderlyingProvisionCashSettlQuoteReferencePage";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueTime] = "UnderlyingProvisionCashSettlValueTime";           // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueTimeBusinessCenter] = "UnderlyingProvisionCashSettlValueTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueDateBusinessDayConvention] = "UnderlyingProvisionCashSettlValueDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueDateRelativeTo] = "UnderlyingProvisionCashSettlValueDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueDateOffsetPeriod] = "UnderlyingProvisionCashSettlValueDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueDateOffsetUnit] = "UnderlyingProvisionCashSettlValueDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueDateOffsetDayType] = "UnderlyingProvisionCashSettlValueDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueDateAdjusted] = "UnderlyingProvisionCashSettlValueDateAdjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionOptionExerciseFixedDates] = "NoUnderlyingProvisionOptionExerciseFixedDates";   // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseFixedDate] = "UnderlyingProvisionOptionExerciseFixedDate";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseFixedDateType] = "UnderlyingProvisionOptionExerciseFixedDateType";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseBusinessDayConvention] = "UnderlyingProvisionOptionExerciseBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseEarliestDateOffsetPeriod] = "UnderlyingProvisionOptionExerciseEarliestDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseEarliestDateOffsetUnit] = "UnderlyingProvisionOptionExerciseEarliestDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseFrequencyPeriod] = "UnderlyingProvisionOptionExerciseFrequencyPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseFrequencyUnit] = "UnderlyingProvisionOptionExerciseFrequencyUnit";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseStartDateUnadjusted] = "UnderlyingProvisionOptionExerciseStartDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseStartDateRelativeTo] = "UnderlyingProvisionOptionExerciseStartDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseStartDateOffsetPeriod] = "UnderlyingProvisionOptionExerciseStartDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseStartDateOffsetUnit] = "UnderlyingProvisionOptionExerciseStartDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseStartDateOffsetDayType] = "UnderlyingProvisionOptionExerciseStartDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseStartDateAdjusted] = "UnderlyingProvisionOptionExerciseStartDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExercisePeriodSkip] = "UnderlyingProvisionOptionExercisePeriodSkip";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseBoundsFirstDateUnadjusted] = "UnderlyingProvisionOptionExerciseBoundsFirstDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseBoundsLastDateUnadjusted] = "UnderlyingProvisionOptionExerciseBoundsLastDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseEarliestTime] = "UnderlyingProvisionOptionExerciseEarliestTime";   // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseEarliestTimeBusinessCenter] = "UnderlyingProvisionOptionExerciseEarliestTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseLatestTime] = "UnderlyingProvisionOptionExerciseLatestTime";     // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseLatestTimeBusinessCenter] = "UnderlyingProvisionOptionExerciseLatestTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationDateUnadjusted] = "UnderlyingProvisionOptionExpirationDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationDateBusinessDayConvention] = "UnderlyingProvisionOptionExpirationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationDateRelativeTo] = "UnderlyingProvisionOptionExpirationDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationDateOffsetPeriod] = "UnderlyingProvisionOptionExpirationDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationDateOffsetUnit] = "UnderlyingProvisionOptionExpirationDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationDateOffsetDayType] = "UnderlyingProvisionOptionExpirationDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationDateAdjusted] = "UnderlyingProvisionOptionExpirationDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationTime] = "UnderlyingProvisionOptionExpirationTime";         // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationTimeBusinessCenter] = "UnderlyingProvisionOptionExpirationTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionRelevantUnderlyingDateUnadjusted] = "UnderlyingProvisionOptionRelevantUnderlyingDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionRelevantUnderlyingDateBusinessDayConvention] = "UnderlyingProvisionOptionRelevantUnderlyingDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionRelevantUnderlyingDateRelativeTo] = "UnderlyingProvisionOptionRelevantUnderlyingDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionRelevantUnderlyingDateOffsetPeriod] = "UnderlyingProvisionOptionRelevantUnderlyingDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionRelevantUnderlyingDateOffsetUnit] = "UnderlyingProvisionOptionRelevantUnderlyingDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionRelevantUnderlyingDateOffsetDayType] = "UnderlyingProvisionOptionRelevantUnderlyingDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionRelevantUnderlyingDateAdjusted] = "UnderlyingProvisionOptionRelevantUnderlyingDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisions]            = "NoUnderlyingProvisions";                          // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionType]           = "UnderlyingProvisionType";                         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionDateUnadjusted] = "UnderlyingProvisionDateUnadjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionDateBusinessDayConvention] = "UnderlyingProvisionDateBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionDateAdjusted]   = "UnderlyingProvisionDateAdjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionDateTenorPeriod] = "UnderlyingProvisionDateTenorPeriod";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionDateTenorUnit]  = "UnderlyingProvisionDateTenorUnit";                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCalculationAgent] = "UnderlyingProvisionCalculationAgent";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionSinglePartyBuyerSide] = "UnderlyingProvisionOptionSinglePartyBuyerSide";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionSinglePartySellerSide] = "UnderlyingProvisionOptionSinglePartySellerSide";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseStyle] = "UnderlyingProvisionOptionExerciseStyle";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseMultipleNotional] = "UnderlyingProvisionOptionExerciseMultipleNotional"; // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseMinimumNotional] = "UnderlyingProvisionOptionExerciseMinimumNotional"; // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseMaximumNotional] = "UnderlyingProvisionOptionExerciseMaximumNotional"; // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionMinimumNumber] = "UnderlyingProvisionOptionMinimumNumber";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionMaximumNumber] = "UnderlyingProvisionOptionMaximumNumber";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseConfirmation] = "UnderlyingProvisionOptionExerciseConfirmation";   // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlMethod] = "UnderlyingProvisionCashSettlMethod";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlCurrency] = "UnderlyingProvisionCashSettlCurrency";            // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlCurrency2] = "UnderlyingProvisionCashSettlCurrency2";           // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlQuoteType] = "UnderlyingProvisionCashSettlQuoteType";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionText]           = "UnderlyingProvisionText";                         // (String FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingProvisionTextLen] = "EncodedUnderlyingProvisionTextLen";               // (Length FIX.5.0SP2)
dictionary[tag::EncodedUnderlyingProvisionText]    = "EncodedUnderlyingProvisionText";                  // (data FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionPartyIDs]     = "NoUnderlyingProvisionPartyIDs";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionPartyID]        = "UnderlyingProvisionPartyID";                      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionPartyIDSource]  = "UnderlyingProvisionPartyIDSource";                // (char FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionPartyRole]      = "UnderlyingProvisionPartyRole";                    // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionPartySubIDs]  = "NoUnderlyingProvisionPartySubIDs";                // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionPartySubID]     = "UnderlyingProvisionPartySubID";                   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionPartySubIDType] = "UnderlyingProvisionPartySubIDType";               // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionCashSettlPaymentDateBusinessCenters] = "NoUnderlyingProvisionCashSettlPaymentDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlPaymentDateBusinessCenter] = "UnderlyingProvisionCashSettlPaymentDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionCashSettlValueDateBusinessCenters] = "NoUnderlyingProvisionCashSettlValueDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionCashSettlValueDateBusinessCenter] = "UnderlyingProvisionCashSettlValueDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionOptionExerciseBusinessCenters] = "NoUnderlyingProvisionOptionExerciseBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExerciseBusinessCenter] = "UnderlyingProvisionOptionExerciseBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionOptionExpirationDateBusinessCenters] = "NoUnderlyingProvisionOptionExpirationDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionExpirationDateBusinessCenter] = "UnderlyingProvisionOptionExpirationDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionOptionRelevantUnderlyingDateBusinessCenters] = "NoUnderlyingProvisionOptionRelevantUnderlyingDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionOptionRelevantUnderlyingDateBusinessCenter] = "UnderlyingProvisionOptionRelevantUnderlyingDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingProvisionDateBusinessCenters] = "NoUnderlyingProvisionDateBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionDateBusinessCenter] = "UnderlyingProvisionDateBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamDeliveryPointSource] = "DeliveryStreamDeliveryPointSource";               // (int FIX.5.0SP2)
dictionary[tag::DeliveryStreamDeliveryPointDesc]   = "DeliveryStreamDeliveryPointDesc";                 // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamDeliveryPointSource] = "LegDeliveryStreamDeliveryPointSource";            // (int FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamDeliveryPointDesc] = "LegDeliveryStreamDeliveryPointDesc";              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamDeliveryPointSource] = "UnderlyingDeliveryStreamDeliveryPointSource";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamDeliveryPointDesc] = "UnderlyingDeliveryStreamDeliveryPointDesc";       // (String FIX.5.0SP2)
dictionary[tag::NoLegContractualDefinitions]       = "NoLegContractualDefinitions";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegContractualDefinition]          = "LegContractualDefinition";                        // (String FIX.5.0SP2)
dictionary[tag::NoLegFinancingTermSupplements]     = "NoLegFinancingTermSupplements";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegFinancingTermSupplementDesc]    = "LegFinancingTermSupplementDesc";                  // (String FIX.5.0SP2)
dictionary[tag::LegFinancingTermSupplementDate]    = "LegFinancingTermSupplementDate";                  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegContractualMatrices]          = "NoLegContractualMatrices";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegContractualMatrixSource]        = "LegContractualMatrixSource";                      // (String FIX.5.0SP2)
dictionary[tag::LegContractualMatrixDate]          = "LegContractualMatrixDate";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegContractualMatrixTerm]          = "LegContractualMatrixTerm";                        // (String FIX.5.0SP2)
dictionary[tag::CashSettlDateUnadjusted]           = "CashSettlDateUnadjusted";                         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::CashSettlDateBusinessDayConvention] = "CashSettlDateBusinessDayConvention";              // (int FIX.5.0SP2)
dictionary[tag::CashSettlDateRelativeTo]           = "CashSettlDateRelativeTo";                         // (int FIX.5.0SP2)
dictionary[tag::CashSettlDateOffsetPeriod]         = "CashSettlDateOffsetPeriod";                       // (int FIX.5.0SP2)
dictionary[tag::CashSettlDateOffsetUnit]           = "CashSettlDateOffsetUnit";                         // (String FIX.5.0SP2)
dictionary[tag::CashSettlDateOffsetDayType]        = "CashSettlDateOffsetDayType";                      // (int FIX.5.0SP2)
dictionary[tag::CashSettlDateAdjusted]             = "CashSettlDateAdjusted";                           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoCashSettlDateBusinessCenters]    = "NoCashSettlDateBusinessCenters";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::CashSettlDateBusinessCenter]       = "CashSettlDateBusinessCenter";                     // (String FIX.5.0SP2)
dictionary[tag::CashSettlPriceSource]              = "CashSettlPriceSource";                            // (String FIX.5.0SP2)
dictionary[tag::CashSettlPriceDefault]             = "CashSettlPriceDefault";                           // (int FIX.5.0SP2)
dictionary[tag::DividendFloatingRateIndex]         = "DividendFloatingRateIndex";                       // (String FIX.5.0SP2)
dictionary[tag::DividendFloatingRateIndexCurvePeriod] = "DividendFloatingRateIndexCurvePeriod";            // (int FIX.5.0SP2)
dictionary[tag::DividendFloatingRateIndexCurveUnit] = "DividendFloatingRateIndexCurveUnit";              // (String FIX.5.0SP2)
dictionary[tag::DividendFloatingRateMultiplier]    = "DividendFloatingRateMultiplier";                  // (float FIX.5.0SP2)
dictionary[tag::DividendFloatingRateSpread]        = "DividendFloatingRateSpread";                      // (PriceOffset FIX.5.0SP2)
dictionary[tag::DividendFloatingRateSpreadPositionType] = "DividendFloatingRateSpreadPositionType";          // (int FIX.5.0SP2)
dictionary[tag::DividendFloatingRateTreatment]     = "DividendFloatingRateTreatment";                   // (int FIX.5.0SP2)
dictionary[tag::DividendCapRate]                   = "DividendCapRate";                                 // (Percentage FIX.5.0SP2)
dictionary[tag::DividendCapRateBuySide]            = "DividendCapRateBuySide";                          // (int FIX.5.0SP2)
dictionary[tag::DividendCapRateSellSide]           = "DividendCapRateSellSide";                         // (int FIX.5.0SP2)
dictionary[tag::DividendFloorRate]                 = "DividendFloorRate";                               // (Percentage FIX.5.0SP2)
dictionary[tag::DividendFloorRateBuySide]          = "DividendFloorRateBuySide";                        // (int FIX.5.0SP2)
dictionary[tag::DividendFloorRateSellSide]         = "DividendFloorRateSellSide";                       // (int FIX.5.0SP2)
dictionary[tag::DividendInitialRate]               = "DividendInitialRate";                             // (Percentage FIX.5.0SP2)
dictionary[tag::DividendFinalRateRoundingDirection] = "DividendFinalRateRoundingDirection";              // (char FIX.5.0SP2)
dictionary[tag::DividendFinalRatePrecision]        = "DividendFinalRatePrecision";                      // (int FIX.5.0SP2)
dictionary[tag::DividendAveragingMethod]           = "DividendAveragingMethod";                         // (int FIX.5.0SP2)
dictionary[tag::DividendNegativeRateTreatment]     = "DividendNegativeRateTreatment";                   // (int FIX.5.0SP2)
dictionary[tag::NoDividendAccrualPaymentDateBusinessCenters] = "NoDividendAccrualPaymentDateBusinessCenters";     // (NumInGroup FIX.5.0SP2)
dictionary[tag::DividendAccrualPaymentDateBusinessCenter] = "DividendAccrualPaymentDateBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::DividendAccrualPaymentDateRelativeTo] = "DividendAccrualPaymentDateRelativeTo";            // (int FIX.5.0SP2)
dictionary[tag::DividendAccrualPaymentDateOffsetPeriod] = "DividendAccrualPaymentDateOffsetPeriod";          // (int FIX.5.0SP2)
dictionary[tag::DividendAccrualPaymentDateOffsetUnit] = "DividendAccrualPaymentDateOffsetUnit";            // (String FIX.5.0SP2)
dictionary[tag::DividendAccrualPaymentDateOffsetDayType] = "DividendAccrualPaymentDateOffsetDayType";         // (int FIX.5.0SP2)
dictionary[tag::DividendAccrualPaymentDateUnadjusted] = "DividendAccrualPaymentDateUnadjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendAccrualPaymeentDateBusinessDayConvention] = "DividendAccrualPaymeentDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::DividendAccrualPaymentDateAdjusted] = "DividendAccrualPaymentDateAdjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendReinvestmentIndicator]     = "DividendReinvestmentIndicator";                   // (Boolean FIX.5.0SP2)
dictionary[tag::DividendEntitlementEvent]          = "DividendEntitlementEvent";                        // (int FIX.5.0SP2)
dictionary[tag::DividendAmountType]                = "DividendAmountType";                              // (int FIX.5.0SP2)
dictionary[tag::DividendUnderlierRefID]            = "DividendUnderlierRefID";                          // (String FIX.5.0SP2)
dictionary[tag::ExtraordinaryDividendPartySide]    = "ExtraordinaryDividendPartySide";                  // (int FIX.5.0SP2)
dictionary[tag::ExtraordinaryDividendAmountType]   = "ExtraordinaryDividendAmountType";                 // (int FIX.5.0SP2)
dictionary[tag::ExtraordinaryDividendCurrency]     = "ExtraordinaryDividendCurrency";                   // (Currency FIX.5.0SP2)
dictionary[tag::ExtraordinaryDividendDeterminationMethod] = "ExtraordinaryDividendDeterminationMethod";        // (String FIX.5.0SP2)
dictionary[tag::DividendAccrualFixedRate]          = "DividendAccrualFixedRate";                        // (Percentage FIX.5.0SP2)
dictionary[tag::DividendCompoundingMethod]         = "DividendCompoundingMethod";                       // (int FIX.5.0SP2)
dictionary[tag::DividendNumOfIndexUnits]           = "DividendNumOfIndexUnits";                         // (int FIX.5.0SP2)
dictionary[tag::DividendCashPercentage]            = "DividendCashPercentage";                          // (Percentage FIX.5.0SP2)
dictionary[tag::DividendCashEquivalentPercentage]  = "DividendCashEquivalentPercentage";                // (Percentage FIX.5.0SP2)
dictionary[tag::NonCashDividendTreatment]          = "NonCashDividendTreatment";                        // (int FIX.5.0SP2)
dictionary[tag::DividendComposition]               = "DividendComposition";                             // (int FIX.5.0SP2)
dictionary[tag::SpecialDividendsIndicator]         = "SpecialDividendsIndicator";                       // (Boolean FIX.5.0SP2)
dictionary[tag::MaterialDividendsIndicator]        = "MaterialDividendsIndicator";                      // (Boolean FIX.5.0SP2)
dictionary[tag::OptionsExchangeDividendsIndicator] = "OptionsExchangeDividendsIndicator";               // (Boolean FIX.5.0SP2)
dictionary[tag::AdditionalDividendsIndicator]      = "AdditionalDividendsIndicator";                    // (Boolean FIX.5.0SP2)
dictionary[tag::AllDividendsIndicator]             = "AllDividendsIndicator";                           // (Boolean FIX.5.0SP2)
dictionary[tag::DividendFXTriggerDateRelativeTo]   = "DividendFXTriggerDateRelativeTo";                 // (int FIX.5.0SP2)
dictionary[tag::DividendFXTriggerDateOffsetPeriod] = "DividendFXTriggerDateOffsetPeriod";               // (int FIX.5.0SP2)
dictionary[tag::DividendFXTriggerDateOffsetUnit]   = "DividendFXTriggerDateOffsetUnit";                 // (String FIX.5.0SP2)
dictionary[tag::DividendFXTriggerDateOffsetDayType] = "DividendFXTriggerDateOffsetDayType";              // (int FIX.5.0SP2)
dictionary[tag::DividendFXTriggerDateUnadjusted]   = "DividendFXTriggerDateUnadjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendFXTriggerDateBusinessDayConvention] = "DividendFXTriggerDateBusinessDayConvention";      // (int FIX.5.0SP2)
dictionary[tag::DividendFXTriggerDateAdjusted]     = "DividendFXTriggerDateAdjusted";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoDividendFXTriggerDateBusinessCenters] = "NoDividendFXTriggerDateBusinessCenters";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::DividendFXTriggerDateBusinessCenter] = "DividendFXTriggerDateBusinessCenter";             // (String FIX.5.0SP2)
dictionary[tag::NoDividendPeriods]                 = "NoDividendPeriods";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::DividendPeriodSequence]            = "DividendPeriodSequence";                          // (int FIX.5.0SP2)
dictionary[tag::DividendPeriodStartDateUnadjusted] = "DividendPeriodStartDateUnadjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendPeriodEndDateUnadjusted]   = "DividendPeriodEndDateUnadjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendPeriodUnderlierRefID]      = "DividendPeriodUnderlierRefID";                    // (String FIX.5.0SP2)
dictionary[tag::DividendPeriodStrikePrice]         = "DividendPeriodStrikePrice";                       // (Price FIX.5.0SP2)
dictionary[tag::DividendPeriodBusinessDayConvention] = "DividendPeriodBusinessDayConvention";             // (int FIX.5.0SP2)
dictionary[tag::DividendPeriodValuationDateUnadjusted] = "DividendPeriodValuationDateUnadjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendPeriodValuationDateRelativeTo] = "DividendPeriodValuationDateRelativeTo";           // (int FIX.5.0SP2)
dictionary[tag::DividendPeriodValuationDateOffsetPeriod] = "DividendPeriodValuationDateOffsetPeriod";         // (int FIX.5.0SP2)
dictionary[tag::DividendPeriodValuationDateOffsetUnit] = "DividendPeriodValuationDateOffsetUnit";           // (String FIX.5.0SP2)
dictionary[tag::DividendPeriodValuationDateOffsetDayType] = "DividendPeriodValuationDateOffsetDayType";        // (int FIX.5.0SP2)
dictionary[tag::DividendPeriodValuationDateAdjusted] = "DividendPeriodValuationDateAdjusted";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendPeriodPaymentDateUnadjusted] = "DividendPeriodPaymentDateUnadjusted";             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendPeriodPaymentDateRelativeTo] = "DividendPeriodPaymentDateRelativeTo";             // (int FIX.5.0SP2)
dictionary[tag::DividendPeriodPaymentDateOffsetPeriod] = "DividendPeriodPaymentDateOffsetPeriod";           // (int FIX.5.0SP2)
dictionary[tag::DividendPeriodPaymentDateOffsetUnit] = "DividendPeriodPaymentDateOffsetUnit";             // (String FIX.5.0SP2)
dictionary[tag::DividendPeriodPaymentDateOffsetDayType] = "DividendPeriodPaymentDateOffsetDayType";          // (int FIX.5.0SP2)
dictionary[tag::DividendPeriodPaymentDateAdjusted] = "DividendPeriodPaymentDateAdjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::DividendPeriodXID]                 = "DividendPeriodXID";                               // (XID FIX.5.0SP2)
dictionary[tag::NoDividendPeriodBusinessCenters]   = "NoDividendPeriodBusinessCenters";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::DividendPeriodBusinessCenter]      = "DividendPeriodBusinessCenter";                    // (String FIX.5.0SP2)
dictionary[tag::NoExtraordinaryEvents]             = "NoExtraordinaryEvents";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::ExtraordinaryEventType]            = "ExtraordinaryEventType";                          // (String FIX.5.0SP2)
dictionary[tag::ExtraordinaryEventValue]           = "ExtraordinaryEventValue";                         // (String FIX.5.0SP2)
dictionary[tag::LegCashSettlDateUnadjusted]        = "LegCashSettlDateUnadjusted";                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegCashSettlDateBusinessDayConvention] = "LegCashSettlDateBusinessDayConvention";           // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlDateRelativeTo]        = "LegCashSettlDateRelativeTo";                      // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlDateOffsetPeriod]      = "LegCashSettlDateOffsetPeriod";                    // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlDateOffsetUnit]        = "LegCashSettlDateOffsetUnit";                      // (String FIX.5.0SP2)
dictionary[tag::LegCashSettlDateOffsetDayType]     = "LegCashSettlDateOffsetDayType";                   // (int FIX.5.0SP2)
dictionary[tag::LegCashSettlDateAdjusted]          = "LegCashSettlDateAdjusted";                        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegCashSettlDateBusinessCenters] = "NoLegCashSettlDateBusinessCenters";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegCashSettlDateBusinessCenter]    = "LegCashSettlDateBusinessCenter";                  // (String FIX.5.0SP2)
dictionary[tag::LegCashSettlPriceSource]           = "LegCashSettlPriceSource";                         // (String FIX.5.0SP2)
dictionary[tag::LegCashSettlPriceDefault]          = "LegCashSettlPriceDefault";                        // (int FIX.5.0SP2)
dictionary[tag::NoLegDividendAccrualPaymentDateBusinessCenters] = "NoLegDividendAccrualPaymentDateBusinessCenters";  // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDividendAccrualPaymentDateBusinessCenter] = "LegDividendAccrualPaymentDateBusinessCenter";     // (String FIX.5.0SP2)
dictionary[tag::LegDividendFloatingRateIndex]      = "LegDividendFloatingRateIndex";                    // (String FIX.5.0SP2)
dictionary[tag::LegDividendFloatingRateIndexCurvePeriod] = "LegDividendFloatingRateIndexCurvePeriod";         // (int FIX.5.0SP2)
dictionary[tag::LegDividendFloatingRateIndexCurveUnit] = "LegDividendFloatingRateIndexCurveUnit";           // (String FIX.5.0SP2)
dictionary[tag::LegDividendFloatingRateMultiplier] = "LegDividendFloatingRateMultiplier";               // (float FIX.5.0SP2)
dictionary[tag::LegDividendFloatingRateSpread]     = "LegDividendFloatingRateSpread";                   // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegDividendFloatingRateSpreadPositionType] = "LegDividendFloatingRateSpreadPositionType";       // (int FIX.5.0SP2)
dictionary[tag::LegDividendFloatingRateTreatment]  = "LegDividendFloatingRateTreatment";                // (int FIX.5.0SP2)
dictionary[tag::LegDividendCapRate]                = "LegDividendCapRate";                              // (Percentage FIX.5.0SP2)
dictionary[tag::LegDividendCapRateBuySide]         = "LegDividendCapRateBuySide";                       // (int FIX.5.0SP2)
dictionary[tag::LegDividendCapRateSellSide]        = "LegDividendCapRateSellSide";                      // (int FIX.5.0SP2)
dictionary[tag::LegDividendFloorRate]              = "LegDividendFloorRate";                            // (Percentage FIX.5.0SP2)
dictionary[tag::LegDividendFloorRateBuySide]       = "LegDividendFloorRateBuySide";                     // (int FIX.5.0SP2)
dictionary[tag::LegDividendFloorRateSellSide]      = "LegDividendFloorRateSellSide";                    // (int FIX.5.0SP2)
dictionary[tag::LegDividendInitialRate]            = "LegDividendInitialRate";                          // (Percentage FIX.5.0SP2)
dictionary[tag::LegDividendFinalRateRoundingDirection] = "LegDividendFinalRateRoundingDirection";           // (char FIX.5.0SP2)
dictionary[tag::LegDividendFinalRatePrecision]     = "LegDividendFinalRatePrecision";                   // (int FIX.5.0SP2)
dictionary[tag::LegDividendAveragingMethod]        = "LegDividendAveragingMethod";                      // (int FIX.5.0SP2)
dictionary[tag::LegDividendNegativeRateTreatment]  = "LegDividendNegativeRateTreatment";                // (int FIX.5.0SP2)
dictionary[tag::LegDividendAccrualPaymentDateRelativeTo] = "LegDividendAccrualPaymentDateRelativeTo";         // (int FIX.5.0SP2)
dictionary[tag::LegDividendAccrualPaymentDateOffsetPeriod] = "LegDividendAccrualPaymentDateOffsetPeriod";       // (int FIX.5.0SP2)
dictionary[tag::LegDividendAccrualPaymentDateOffsetUnit] = "LegDividendAccrualPaymentDateOffsetUnit";         // (String FIX.5.0SP2)
dictionary[tag::LegDividendAccrualPaymentDateOffsetDayType] = "LegDividendAccrualPaymentDateOffsetDayType";      // (int FIX.5.0SP2)
dictionary[tag::LegDividendAccrualPaymentDateUnadjusted] = "LegDividendAccrualPaymentDateUnadjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendAccrualPaymentDateBusinessDayConvention] = "LegDividendAccrualPaymentDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegDividendAccrualPaymentDateAdjusted] = "LegDividendAccrualPaymentDateAdjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendReinvestmentIndicator]  = "LegDividendReinvestmentIndicator";                // (Boolean FIX.5.0SP2)
dictionary[tag::LegDividendEntitlementEvent]       = "LegDividendEntitlementEvent";                     // (int FIX.5.0SP2)
dictionary[tag::LegDividendAmountType]             = "LegDividendAmountType";                           // (int FIX.5.0SP2)
dictionary[tag::LegDividendUnderlierRefID]         = "LegDividendUnderlierRefID";                       // (String FIX.5.0SP2)
dictionary[tag::LegExtraordinaryDividendPartySide] = "LegExtraordinaryDividendPartySide";               // (int FIX.5.0SP2)
dictionary[tag::LegExtraordinaryDividendAmountType] = "LegExtraordinaryDividendAmountType";              // (int FIX.5.0SP2)
dictionary[tag::LegExtraordinaryDividendCurrency]  = "LegExtraordinaryDividendCurrency";                // (Currency FIX.5.0SP2)
dictionary[tag::LegExtraordinaryDividendDeterminationMethod] = "LegExtraordinaryDividendDeterminationMethod";     // (String FIX.5.0SP2)
dictionary[tag::LegDividendAccrualFixedRate]       = "LegDividendAccrualFixedRate";                     // (Percentage FIX.5.0SP2)
dictionary[tag::LegDividendCompoundingMethod]      = "LegDividendCompoundingMethod";                    // (int FIX.5.0SP2)
dictionary[tag::LegDividendNumOfIndexUnits]        = "LegDividendNumOfIndexUnits";                      // (int FIX.5.0SP2)
dictionary[tag::LegDividendCashPercentage]         = "LegDividendCashPercentage";                       // (Percentage FIX.5.0SP2)
dictionary[tag::LegDividendCashEquivalentPercentage] = "LegDividendCashEquivalentPercentage";             // (Percentage FIX.5.0SP2)
dictionary[tag::LegNonCashDividendTreatment]       = "LegNonCashDividendTreatment";                     // (int FIX.5.0SP2)
dictionary[tag::LegDividendComposition]            = "LegDividendComposition";                          // (int FIX.5.0SP2)
dictionary[tag::LegSpecialDividendsIndicator]      = "LegSpecialDividendsIndicator";                    // (Boolean FIX.5.0SP2)
dictionary[tag::LegMaterialDividendsIndicator]     = "LegMaterialDividendsIndicator";                   // (Boolean FIX.5.0SP2)
dictionary[tag::LegOptionsExchangeDividendsIndicator] = "LegOptionsExchangeDividendsIndicator";            // (Boolean FIX.5.0SP2)
dictionary[tag::LegAdditionalDividendsIndicator]   = "LegAdditionalDividendsIndicator";                 // (Boolean FIX.5.0SP2)
dictionary[tag::LegAllDividendsIndicator]          = "LegAllDividendsIndicator";                        // (Boolean FIX.5.0SP2)
dictionary[tag::LegDividendFXTriggerDateRelativeTo] = "LegDividendFXTriggerDateRelativeTo";              // (int FIX.5.0SP2)
dictionary[tag::LegDividendFXTriggerDateOffsetPeriod] = "LegDividendFXTriggerDateOffsetPeriod";            // (int FIX.5.0SP2)
dictionary[tag::LegDividendFXTriggerDateOffsetUnit] = "LegDividendFXTriggerDateOffsetUnit";              // (String FIX.5.0SP2)
dictionary[tag::LegDividendFXTriggerDateOffsetDayType] = "LegDividendFXTriggerDateOffsetDayType";           // (int FIX.5.0SP2)
dictionary[tag::LegDividendFXTriggerDateUnadjusted] = "LegDividendFXTriggerDateUnadjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendFXTriggerDateBusinessDayConvention] = "LegDividendFXTriggerDateBusinessDayConvention";   // (int FIX.5.0SP2)
dictionary[tag::LegDividendFXTriggerDateAdjusted]  = "LegDividendFXTriggerDateAdjusted";                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegDividendFXTriggerDateBusinessCenters] = "NoLegDividendFXTriggerDateBusinessCenters";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDividendFXTriggerDateBusinessCenter] = "LegDividendFXTriggerDateBusinessCenter";          // (String FIX.5.0SP2)
dictionary[tag::NoLegDividendPeriods]              = "NoLegDividendPeriods";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDividendPeriodSequence]         = "LegDividendPeriodSequence";                       // (int FIX.5.0SP2)
dictionary[tag::LegDividendPeriodStartDateUnadjusted] = "LegDividendPeriodStartDateUnadjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendPeriodEndDateUnadjusted] = "LegDividendPeriodEndDateUnadjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendPeriodUnderlierRefID]   = "LegDividendPeriodUnderlierRefID";                 // (String FIX.5.0SP2)
dictionary[tag::LegDividendPeriodStrikePrice]      = "LegDividendPeriodStrikePrice";                    // (Price FIX.5.0SP2)
dictionary[tag::LegDividendPeriodBusinessDayConvention] = "LegDividendPeriodBusinessDayConvention";          // (int FIX.5.0SP2)
dictionary[tag::LegDividendPeriodValuationDateUnadjusted] = "LegDividendPeriodValuationDateUnadjusted";        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendPeriodValuationDateRelativeTo] = "LegDividendPeriodValuationDateRelativeTo";        // (int FIX.5.0SP2)
dictionary[tag::LegDividendPeriodValuationDateOffsetPeriod] = "LegDividendPeriodValuationDateOffsetPeriod";      // (int FIX.5.0SP2)
dictionary[tag::LegDividendPeriodValuationDateOffsetUnit] = "LegDividendPeriodValuationDateOffsetUnit";        // (String FIX.5.0SP2)
dictionary[tag::LegDividendPeriodValuationDateOffsetDayType] = "LegDividendPeriodValuationDateOffsetDayType";     // (int FIX.5.0SP2)
dictionary[tag::LegDividendPeriodValuationDateAdjusted] = "LegDividendPeriodValuationDateAdjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendPeriodPaymentDateUnadjusted] = "LegDividendPeriodPaymentDateUnadjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendPeriodPaymentDateRelativeTo] = "LegDividendPeriodPaymentDateRelativeTo";          // (int FIX.5.0SP2)
dictionary[tag::LegDividendPeriodPaymentDateOffsetPeriod] = "LegDividendPeriodPaymentDateOffsetPeriod";        // (int FIX.5.0SP2)
dictionary[tag::LegDividendPeriodPaymentDateOffsetUnit] = "LegDividendPeriodPaymentDateOffsetUnit";          // (String FIX.5.0SP2)
dictionary[tag::LegDividendPeriodPaymentDateOffsetDayType] = "LegDividendPeriodPaymentDateOffsetDayType";       // (int FIX.5.0SP2)
dictionary[tag::LegDividendPeriodPaymentDateAdjusted] = "LegDividendPeriodPaymentDateAdjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegDividendPeriodXID]              = "LegDividendPeriodXID";                            // (XID FIX.5.0SP2)
dictionary[tag::NoLegDividendPeriodBusinessCenters] = "NoLegDividendPeriodBusinessCenters";              // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegDividendPeriodBusinessCenter]   = "LegDividendPeriodBusinessCenter";                 // (String FIX.5.0SP2)
dictionary[tag::NoLegExtraordinaryEvents]          = "NoLegExtraordinaryEvents";                        // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegExtraordinaryEventType]         = "LegExtraordinaryEventType";                       // (String FIX.5.0SP2)
dictionary[tag::LegExtraordinaryEventValue]        = "LegExtraordinaryEventValue";                      // (String FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectingPartySide]   = "LegSettlMethodElectingPartySide";                 // (int FIX.5.0SP2)
dictionary[tag::LegMakeWholeDate]                  = "LegMakeWholeDate";                                // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegMakeWholeAmount]                = "LegMakeWholeAmount";                              // (Amt FIX.5.0SP2)
dictionary[tag::LegMakeWholeBenchmarkCurveName]    = "LegMakeWholeBenchmarkCurveName";                  // (String FIX.5.0SP2)
dictionary[tag::LegMakeWholeBenchmarkCurvePoint]   = "LegMakeWholeBenchmarkCurvePoint";                 // (String FIX.5.0SP2)
dictionary[tag::LegMakeWholeRecallSpread]          = "LegMakeWholeRecallSpread";                        // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegMakeWholeBenchmarkQuote]        = "LegMakeWholeBenchmarkQuote";                      // (int FIX.5.0SP2)
dictionary[tag::LegMakeWholeInterpolationMethod]   = "LegMakeWholeInterpolationMethod";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCashSettlIndicator] = "LegPaymentStreamCashSettlIndicator";              // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingXIDRef] = "LegPaymentStreamCompoundingXIDRef";               // (XIDREF FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingSpread] = "LegPaymentStreamCompoundingSpread";               // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInterpolationMethod] = "LegPaymentStreamInterpolationMethod";             // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamInterpolationPeriod] = "LegPaymentStreamInterpolationPeriod";             // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingFixedRate] = "LegPaymentStreamCompoundingFixedRate";            // (float FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamCompoundingDates] = "NoLegPaymentStreamCompoundingDates";              // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingDate]   = "LegPaymentStreamCompoundingDate";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingDateType] = "LegPaymentStreamCompoundingDateType";             // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingDatesBusinessDayConvention] = "LegPaymentStreamCompoundingDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingDatesRelativeTo] = "LegPaymentStreamCompoundingDatesRelativeTo";      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingDatesOffsetPeriod] = "LegPaymentStreamCompoundingDatesOffsetPeriod";    // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingDatesOffsetUnit] = "LegPaymentStreamCompoundingDatesOffsetUnit";      // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingDatesOffsetDayType] = "LegPaymentStreamCompoundingDatesOffsetDayType";   // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingPeriodSkip] = "LegPaymentStreamCompoundingPeriodSkip";           // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingFrequencyPeriod] = "LegPaymentStreamCompoundingFrequencyPeriod";      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingFrequencyUnit] = "LegPaymentStreamCompoundingFrequencyUnit";        // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingRollConvention] = "LegPaymentStreamCompoundingRollConvention";       // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamBoundsFirstDateUnadjusted] = "LegPaymentStreamBoundsFirstDateUnadjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamBoundsLastDateUnadjusted] = "LegPaymentStreamBoundsLastDateUnadjusted";        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamCompoundingDatesBusinessCenters] = "NoLegPaymentStreamCompoundingDatesBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingDatesBusinessCenter] = "LegPaymentStreamCompoundingDatesBusinessCenter";  // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingEndDateUnadjusted] = "LegPaymentStreamCompoundingEndDateUnadjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingEndDateRelativeTo] = "LegPaymentStreamCompoundingEndDateRelativeTo";    // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingEndDateOffsetPeriod] = "LegPaymentStreamCompoundingEndDateOffsetPeriod";  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingEndDateOffsetUnit] = "LegPaymentStreamCompoundingEndDateOffsetUnit";    // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingEndDateOffsetDayType] = "LegPaymentStreamCompoundingEndDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingEndDateAdjusted] = "LegPaymentStreamCompoundingEndDateAdjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingRateIndex] = "LegPaymentStreamCompoundingRateIndex";            // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingRateIndexCurvePeriod] = "LegPaymentStreamCompoundingRateIndexCurvePeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingRateIndexCurveUnit] = "LegPaymentStreamCompoundingRateIndexCurveUnit";   // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingRateMultiplier] = "LegPaymentStreamCompoundingRateMultiplier";       // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingRateSpread] = "LegPaymentStreamCompoundingRateSpread";           // (PriceOffset FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingRateSpreadPositionType] = "LegPaymentStreamCompoundingRateSpreadPositionType"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingRateTreatment] = "LegPaymentStreamCompoundingRateTreatment";        // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingCapRate] = "LegPaymentStreamCompoundingCapRate";              // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingCapRateBuySide] = "LegPaymentStreamCompoundingCapRateBuySide";       // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingCapRateSellSide] = "LegPaymentStreamCompoundingCapRateSellSide";      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingFloorRate] = "LegPaymentStreamCompoundingFloorRate";            // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingFloorRateBuySide] = "LegPaymentStreamCompoundingFloorRateBuySide";     // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingFloorRateSellSide] = "LegPaymentStreamCompoundingFloorRateSellSide";    // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingInitialRate] = "LegPaymentStreamCompoundingInitialRate";          // (Percentage FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingFinalRateRoundingDirection] = "LegPaymentStreamCompoundingFinalRateRoundingDirection"; // (char FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingFinalRatePrecision] = "LegPaymentStreamCompoundingFinalRatePrecision";   // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingAveragingMethod] = "LegPaymentStreamCompoundingAveragingMethod";      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingNegativeRateTreatment] = "LegPaymentStreamCompoundingNegativeRateTreatment"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingStartDateUnadjusted] = "LegPaymentStreamCompoundingStartDateUnadjusted";  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingStartDateRelativeTo] = "LegPaymentStreamCompoundingStartDateRelativeTo";  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingStartDateOffsetPeriod] = "LegPaymentStreamCompoundingStartDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingStartDateOffsetUnit] = "LegPaymentStreamCompoundingStartDateOffsetUnit";  // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingStartDateOffsetDayType] = "LegPaymentStreamCompoundingStartDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamCompoundingStartDateAdjusted] = "LegPaymentStreamCompoundingStartDateAdjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFormulaImageLength] = "LegPaymentStreamFormulaImageLength";              // (Length FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFormulaImage]      = "LegPaymentStreamFormulaImage";                    // (data FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalPricePaymentDateUnadjusted] = "LegPaymentStreamFinalPricePaymentDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalPricePaymentDateRelativeTo] = "LegPaymentStreamFinalPricePaymentDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalPricePaymentDateOffsetPeriod] = "LegPaymentStreamFinalPricePaymentDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalPricePaymentDateOffsetUnit] = "LegPaymentStreamFinalPricePaymentDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalPricePaymentDateOffsetDayType] = "LegPaymentStreamFinalPricePaymentDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFinalPricePaymentDateAdjusted] = "LegPaymentStreamFinalPricePaymentDateAdjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamFixingDates]     = "NoLegPaymentStreamFixingDates";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDate]        = "LegPaymentStreamFixingDate";                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFixingDateType]    = "LegPaymentStreamFixingDateType";                  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFirstObservationDateUnadjusted] = "LegPaymentStreamFirstObservationDateUnadjusted";  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFirstObservationDateRelativeTo] = "LegPaymentStreamFirstObservationDateRelativeTo";  // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFirstObservationDateOffsetDayType] = "LegPaymentStreamFirstObservationDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFirstObservationDateAdjusted] = "LegPaymentStreamFirstObservationDateAdjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStreamUnderlierRefID]    = "LegPaymentStreamUnderlierRefID";                  // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateNotionalReset]        = "LegReturnRateNotionalReset";                      // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkInitialLevel]  = "LegPaymentStreamLinkInitialLevel";                // (Price FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkClosingLevelIndicator] = "LegPaymentStreamLinkClosingLevelIndicator";       // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkExpiringLevelIndicator] = "LegPaymentStreamLinkExpiringLevelIndicator";      // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkEstimatedTradingDays] = "LegPaymentStreamLinkEstimatedTradingDays";        // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkStrikePrice]   = "LegPaymentStreamLinkStrikePrice";                 // (Price FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkStrikePriceType] = "LegPaymentStreamLinkStrikePriceType";             // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkMaximumBoundary] = "LegPaymentStreamLinkMaximumBoundary";             // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkMinimumBoundary] = "LegPaymentStreamLinkMinimumBoundary";             // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamLinkNumberOfDataSeries] = "LegPaymentStreamLinkNumberOfDataSeries";          // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamVarianceUnadjustedCap] = "LegPaymentStreamVarianceUnadjustedCap";           // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRealizedVarianceMethod] = "LegPaymentStreamRealizedVarianceMethod";          // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStreamDaysAdjustmentIndicator] = "LegPaymentStreamDaysAdjustmentIndicator";         // (Boolean FIX.5.0SP2)
dictionary[tag::LegPaymentStreamNearestExchangeContractRefID] = "LegPaymentStreamNearestExchangeContractRefID";    // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamVegaNotionalAmount] = "LegPaymentStreamVegaNotionalAmount";              // (float FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFormulaCurrency]   = "LegPaymentStreamFormulaCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFormulaCurrencyDeterminationMethod] = "LegPaymentStreamFormulaCurrencyDeterminationMethod"; // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFormulaReferenceAmount] = "LegPaymentStreamFormulaReferenceAmount";          // (int FIX.5.0SP2)
dictionary[tag::NoLegPaymentStreamFormulas]        = "NoLegPaymentStreamFormulas";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFormula]           = "LegPaymentStreamFormula";                         // (XMLData FIX.5.0SP2)
dictionary[tag::LegPaymentStreamFormulaDesc]       = "LegPaymentStreamFormulaDesc";                     // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStubEndDateUnadjusted]   = "LegPaymentStubEndDateUnadjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStubEndDateBusinessDayConvention] = "LegPaymentStubEndDateBusinessDayConvention";      // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubEndDateRelativeTo]   = "LegPaymentStubEndDateRelativeTo";                 // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubEndDateOffsetPeriod] = "LegPaymentStubEndDateOffsetPeriod";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubEndDateOffsetUnit]   = "LegPaymentStubEndDateOffsetUnit";                 // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStubEndDateOffsetDayType] = "LegPaymentStubEndDateOffsetDayType";              // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubEndDateAdjusted]     = "LegPaymentStubEndDateAdjusted";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegPaymentStubEndDateBusinessCenters] = "NoLegPaymentStubEndDateBusinessCenters";          // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStubEndDateBusinessCenter] = "LegPaymentStubEndDateBusinessCenter";             // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStubStartDateUnadjusted] = "LegPaymentStubStartDateUnadjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegPaymentStubStartDateBusinessDayConvention] = "LegPaymentStubStartDateBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubStartDateRelativeTo] = "LegPaymentStubStartDateRelativeTo";               // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubStartDateOffsetPeriod] = "LegPaymentStubStartDateOffsetPeriod";             // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubStartDateOffsetUnit] = "LegPaymentStubStartDateOffsetUnit";               // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStubStartDateOffsetDayType] = "LegPaymentStubStartDateOffsetDayType";            // (int FIX.5.0SP2)
dictionary[tag::LegPaymentStubStartDateAdjusted]   = "LegPaymentStubStartDateAdjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegPaymentStubStartDateBusinessCenters] = "NoLegPaymentStubStartDateBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegPaymentStubStartDateBusinessCenter] = "LegPaymentStubStartDateBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::LegProvisionBreakFeeElection]      = "LegProvisionBreakFeeElection";                    // (int FIX.5.0SP2)
dictionary[tag::LegProvisionBreakFeeRate]          = "LegProvisionBreakFeeRate";                        // (Percentage FIX.5.0SP2)
dictionary[tag::NoLegReturnRateDates]              = "NoLegReturnRateDates";                            // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegReturnRateDateMode]             = "LegReturnRateDateMode";                           // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationDateRelativeTo] = "LegReturnRateValuationDateRelativeTo";            // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationDateOffsetPeriod] = "LegReturnRateValuationDateOffsetPeriod";          // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationDateOffsetUnit] = "LegReturnRateValuationDateOffsetUnit";            // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationDateOffsetDayType] = "LegReturnRateValuationDateOffsetDayType";         // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationStartDateUnadjusted] = "LegReturnRateValuationStartDateUnadjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationStartDateRelativeTo] = "LegReturnRateValuationStartDateRelativeTo";       // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationStartDateOffsetPeriod] = "LegReturnRateValuationStartDateOffsetPeriod";     // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationStartDateOffsetUnit] = "LegReturnRateValuationStartDateOffsetUnit";       // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationStartDateOffsetDayType] = "LegReturnRateValuationStartDateOffsetDayType";    // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationStartDateAdjusted] = "LegReturnRateValuationStartDateAdjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationEndDateUnadjusted] = "LegReturnRateValuationEndDateUnadjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationEndDateRelativeTo] = "LegReturnRateValuationEndDateRelativeTo";         // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationEndDateOffsetPeriod] = "LegReturnRateValuationEndDateOffsetPeriod";       // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationEndDateOffsetUnit] = "LegReturnRateValuationEndDateOffsetUnit";         // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationEndDateOffsetDayType] = "LegReturnRateValuationEndDateOffsetDayType";      // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationEndDateAdjusted] = "LegReturnRateValuationEndDateAdjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationFrequencyPeriod] = "LegReturnRateValuationFrequencyPeriod";           // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationFrequencyUnit] = "LegReturnRateValuationFrequencyUnit";             // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationFrequencyRollConvention] = "LegReturnRateValuationFrequencyRollConvention";   // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationDateBusinessDayConvention] = "LegReturnRateValuationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::NoLegReturnRateFXConversions]      = "NoLegReturnRateFXConversions";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegReturnRateFXCurrencySymbol]     = "LegReturnRateFXCurrencySymbol";                   // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateFXRate]               = "LegReturnRateFXRate";                             // (float FIX.5.0SP2)
dictionary[tag::LegReturnRateFXRateCalc]           = "LegReturnRateFXRateCalc";                         // (char FIX.5.0SP2)
dictionary[tag::NoLegReturnRates]                  = "NoLegReturnRates";                                // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegReturnRatePriceSequence]        = "LegReturnRatePriceSequence";                      // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateCommissionBasis]      = "LegReturnRateCommissionBasis";                    // (char FIX.5.0SP2)
dictionary[tag::LegReturnRateCommissionAmount]     = "LegReturnRateCommissionAmount";                   // (Amt FIX.5.0SP2)
dictionary[tag::LegReturnRateCommissionCurrency]   = "LegReturnRateCommissionCurrency";                 // (Currency FIX.5.0SP2)
dictionary[tag::LegReturnRateTotalCommissionPerTrade] = "LegReturnRateTotalCommissionPerTrade";            // (Amt FIX.5.0SP2)
dictionary[tag::LegReturnRateDeterminationMethod]  = "LegReturnRateDeterminationMethod";                // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateAmountRelativeTo]     = "LegReturnRateAmountRelativeTo";                   // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteMeasureType]     = "LegReturnRateQuoteMeasureType";                   // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteUnits]           = "LegReturnRateQuoteUnits";                         // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteMethod]          = "LegReturnRateQuoteMethod";                        // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteCurrency]        = "LegReturnRateQuoteCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteCurrencyType]    = "LegReturnRateQuoteCurrencyType";                  // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteTimeType]        = "LegReturnRateQuoteTimeType";                      // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteTime]            = "LegReturnRateQuoteTime";                          // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteDate]            = "LegReturnRateQuoteDate";                          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteExpirationTime]  = "LegReturnRateQuoteExpirationTime";                // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteBusinessCenter]  = "LegReturnRateQuoteBusinessCenter";                // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateQuoteExchange]        = "LegReturnRateQuoteExchange";                      // (Exchange FIX.5.0SP2)
dictionary[tag::LegReturnRateQuotePricingModel]    = "LegReturnRateQuotePricingModel";                  // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateCashFlowType]         = "LegReturnRateCashFlowType";                       // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationTimeType]    = "LegReturnRateValuationTimeType";                  // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationTime]        = "LegReturnRateValuationTime";                      // (LocalMktTime FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationTimeBusinessCenter] = "LegReturnRateValuationTimeBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationPriceOption] = "LegReturnRateValuationPriceOption";               // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateFinalPriceFallback]   = "LegReturnRateFinalPriceFallback";                 // (int FIX.5.0SP2)
dictionary[tag::NoLegReturnRateInformationSources] = "NoLegReturnRateInformationSources";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegReturnRateInformationSource]    = "LegReturnRateInformationSource";                  // (int FIX.5.0SP2)
dictionary[tag::LegReturnRateReferencePage]        = "LegReturnRateReferencePage";                      // (String FIX.5.0SP2)
dictionary[tag::LegReturnRateReferencePageHeading] = "LegReturnRateReferencePageHeading";               // (String FIX.5.0SP2)
dictionary[tag::NoLegReturnRatePrices]             = "NoLegReturnRatePrices";                           // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegReturnRatePriceBasis]           = "LegReturnRatePriceBasis";                         // (int FIX.5.0SP2)
dictionary[tag::LegReturnRatePrice]                = "LegReturnRatePrice";                              // (Price FIX.5.0SP2)
dictionary[tag::LegReturnRatePriceCurrency]        = "LegReturnRatePriceCurrency";                      // (Currency FIX.5.0SP2)
dictionary[tag::LegReturnRatePriceType]            = "LegReturnRatePriceType";                          // (int FIX.5.0SP2)
dictionary[tag::NoLegReturnRateValuationDateBusinessCenters] = "NoLegReturnRateValuationDateBusinessCenters";     // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationDateBusinessCenter] = "LegReturnRateValuationDateBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::NoLegReturnRateValuationDates]     = "NoLegReturnRateValuationDates";                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationDate]        = "LegReturnRateValuationDate";                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegReturnRateValuationDateType]    = "LegReturnRateValuationDateType";                  // (int FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectionDateUnadjusted] = "LegSettlMethodElectionDateUnadjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectionDateBusinessDayConvention] = "LegSettlMethodElectionDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectionDateRelativeTo] = "LegSettlMethodElectionDateRelativeTo";            // (int FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectionDateOffsetPeriod] = "LegSettlMethodElectionDateOffsetPeriod";          // (int FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectionDateOffsetUnit] = "LegSettlMethodElectionDateOffsetUnit";            // (String FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectionDateOffsetDayType] = "LegSettlMethodElectionDateOffsetDayType";         // (int FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectionDateAdjusted] = "LegSettlMethodElectionDateAdjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoLegSettlMethodElectionDateBusinessCenters] = "NoLegSettlMethodElectionDateBusinessCenters";     // (NumInGroup FIX.5.0SP2)
dictionary[tag::LegSettlMethodElectionDateBusinessCenter] = "LegSettlMethodElectionDateBusinessCenter";        // (String FIX.5.0SP2)
dictionary[tag::LegStreamVersion]                  = "LegStreamVersion";                                // (String FIX.5.0SP2)
dictionary[tag::LegStreamVersionEffectiveDate]     = "LegStreamVersionEffectiveDate";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::LegStreamNotionalDeterminationMethod] = "LegStreamNotionalDeterminationMethod";            // (String FIX.5.0SP2)
dictionary[tag::LegStreamNotionalAdjustments]      = "LegStreamNotionalAdjustments";                    // (int FIX.5.0SP2)
dictionary[tag::StreamCommodityDeliveryPricingRegion] = "StreamCommodityDeliveryPricingRegion";            // (String FIX.5.0SP2)
dictionary[tag::LegStreamCommodityDeliveryPricingRegion] = "LegStreamCommodityDeliveryPricingRegion";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamCommodityDeliveryPricingRegion] = "UnderlyingStreamCommodityDeliveryPricingRegion";  // (String FIX.5.0SP2)
dictionary[tag::SettlMethodElectingPartySide]      = "SettlMethodElectingPartySide";                    // (int FIX.5.0SP2)
dictionary[tag::MakeWholeDate]                     = "MakeWholeDate";                                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::MakeWholeAmount]                   = "MakeWholeAmount";                                 // (Amt FIX.5.0SP2)
dictionary[tag::MakeWholeBenchmarkCurveName]       = "MakeWholeBenchmarkCurveName";                     // (String FIX.5.0SP2)
dictionary[tag::MakeWholeBenchmarkCurvePoint]      = "MakeWholeBenchmarkCurvePoint";                    // (String FIX.5.0SP2)
dictionary[tag::MakeWholeRecallSpread]             = "MakeWholeRecallSpread";                           // (PriceOffset FIX.5.0SP2)
dictionary[tag::MakeWholeBenchmarkQuote]           = "MakeWholeBenchmarkQuote";                         // (int FIX.5.0SP2)
dictionary[tag::MakeWholeInterpolationMethod]      = "MakeWholeInterpolationMethod";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentAmountRelativeTo]           = "PaymentAmountRelativeTo";                         // (int FIX.5.0SP2)
dictionary[tag::PaymentAmountDeterminationMethod]  = "PaymentAmountDeterminationMethod";                // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamCashSettlIndicator]   = "PaymentStreamCashSettlIndicator";                 // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingXIDRef]    = "PaymentStreamCompoundingXIDRef";                  // (XIDREF FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingSpread]    = "PaymentStreamCompoundingSpread";                  // (PriceOffset FIX.5.0SP2)
dictionary[tag::PaymentStreamInterpolationMethod]  = "PaymentStreamInterpolationMethod";                // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamInterpolationPeriod]  = "PaymentStreamInterpolationPeriod";                // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingFixedRate] = "PaymentStreamCompoundingFixedRate";               // (float FIX.5.0SP2)
dictionary[tag::NoPaymentStreamCompoundingDates]   = "NoPaymentStreamCompoundingDates";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingDate]      = "PaymentStreamCompoundingDate";                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingDateType]  = "PaymentStreamCompoundingDateType";                // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingDatesBusinessDayConvention] = "PaymentStreamCompoundingDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingDatesRelativeTo] = "PaymentStreamCompoundingDatesRelativeTo";         // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingDatesOffsetPeriod] = "PaymentStreamCompoundingDatesOffsetPeriod";       // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingDatesOffsetUnit] = "PaymentStreamCompoundingDatesOffsetUnit";         // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingDatesOffsetDayType] = "PaymentStreamCompoundingDatesOffsetDayType";      // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingPeriodSkip] = "PaymentStreamCompoundingPeriodSkip";              // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingFrequencyPeriod] = "PaymentStreamCompoundingFrequencyPeriod";         // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingFrequencyUnit] = "PaymentStreamCompoundingFrequencyUnit";           // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingRollConvention] = "PaymentStreamCompoundingRollConvention";          // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamBoundsFirstDateUnadjusted] = "PaymentStreamBoundsFirstDateUnadjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamBoundsLastDateUnadjusted] = "PaymentStreamBoundsLastDateUnadjusted";           // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoPaymentStreamCompoundingDatesBusinessCenters] = "NoPaymentStreamCompoundingDatesBusinessCenters";  // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingDatesBusinessCenter] = "PaymentStreamCompoundingDatesBusinessCenter";     // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingEndDateUnadjusted] = "PaymentStreamCompoundingEndDateUnadjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingEndDateRelativeTo] = "PaymentStreamCompoundingEndDateRelativeTo";       // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingEndDateOffsetPeriod] = "PaymentStreamCompoundingEndDateOffsetPeriod";     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingEndDateOffsetUnit] = "PaymentStreamCompoundingEndDateOffsetUnit";       // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingEndDateOffsetDayType] = "PaymentStreamCompoundingEndDateOffsetDayType";    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingEndDateAdjusted] = "PaymentStreamCompoundingEndDateAdjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingRateIndex] = "PaymentStreamCompoundingRateIndex";               // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingRateIndexCurvePeriod] = "PaymentStreamCompoundingRateIndexCurvePeriod";    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingRateIndexCurveUnit] = "PaymentStreamCompoundingRateIndexCurveUnit";      // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingRateMultiplier] = "PaymentStreamCompoundingRateMultiplier";          // (float FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingRateSpread] = "PaymentStreamCompoundingRateSpread";              // (PriceOffset FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingRateSpreadPositionType] = "PaymentStreamCompoundingRateSpreadPositionType";  // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingRateTreatment] = "PaymentStreamCompoundingRateTreatment";           // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingCapRate]   = "PaymentStreamCompoundingCapRate";                 // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingCapRateBuySide] = "PaymentStreamCompoundingCapRateBuySide";          // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingCapRateSellSide] = "PaymentStreamCompoundingCapRateSellSide";         // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingFloorRate] = "PaymentStreamCompoundingFloorRate";               // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingFloorRateBuySide] = "PaymentStreamCompoundingFloorRateBuySide";        // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingFloorRateSellSide] = "PaymentStreamCompoundingFloorRateSellSide";       // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingInitialRate] = "PaymentStreamCompoundingInitialRate";             // (Percentage FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingFinalRateRoundingDirection] = "PaymentStreamCompoundingFinalRateRoundingDirection"; // (char FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingFinalRatePrecision] = "PaymentStreamCompoundingFinalRatePrecision";      // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingAveragingMethod] = "PaymentStreamCompoundingAveragingMethod";         // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingNegativeRateTreatment] = "PaymentStreamCompoundingNegativeRateTreatment";   // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingStartDateUnadjusted] = "PaymentStreamCompoundingStartDateUnadjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingStartDateRelativeTo] = "PaymentStreamCompoundingStartDateRelativeTo";     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingStartDateOffsetPeriod] = "PaymentStreamCompoundingStartDateOffsetPeriod";   // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingStartDateOffsetUnit] = "PaymentStreamCompoundingStartDateOffsetUnit";     // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingStartDateOffsetDayType] = "PaymentStreamCompoundingStartDateOffsetDayType";  // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamCompoundingStartDateAdjusted] = "PaymentStreamCompoundingStartDateAdjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamFormulaImageLength]   = "PaymentStreamFormulaImageLength";                 // (Length FIX.5.0SP2)
dictionary[tag::PaymentStreamFormulaImage]         = "PaymentStreamFormulaImage";                       // (data FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalPricePaymentDateUnadjusted] = "PaymentStreamFinalPricePaymentDateUnadjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalPricePaymentDateRelativeTo] = "PaymentStreamFinalPricePaymentDateRelativeTo";    // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalPricePaymentDateOffsetfPeriod] = "PaymentStreamFinalPricePaymentDateOffsetfPeriod"; // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalPricePaymentDateOffsetUnit] = "PaymentStreamFinalPricePaymentDateOffsetUnit";    // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalPricePaymentDateOffsetDayType] = "PaymentStreamFinalPricePaymentDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFinalPricePaymentDateAdjusted] = "PaymentStreamFinalPricePaymentDateAdjusted";      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoPaymentStreamFixingDates]        = "NoPaymentStreamFixingDates";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDate]           = "PaymentStreamFixingDate";                         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamFixingDateType]       = "PaymentStreamFixingDateType";                     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFirstObservationDateUnadjusted] = "PaymentStreamFirstObservationDateUnadjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamFirstObservationDateRelativeTo] = "PaymentStreamFirstObservationDateRelativeTo";     // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFirstObservationDateOffsetDayType] = "PaymentStreamFirstObservationDateOffsetDayType";  // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamFirstObservationDateAdjusted] = "PaymentStreamFirstObservationDateAdjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStreamUnderlierRefID]       = "PaymentStreamUnderlierRefID";                     // (String FIX.5.0SP2)
dictionary[tag::ReturnRateNotionalReset]           = "ReturnRateNotionalReset";                         // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkInitialLevel]     = "PaymentStreamLinkInitialLevel";                   // (Price FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkClosingLevelIndicator] = "PaymentStreamLinkClosingLevelIndicator";          // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkExpiringLevelIndicator] = "PaymentStreamLinkExpiringLevelIndicator";         // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkEstimatedTradingDays] = "PaymentStreamLinkEstimatedTradingDays";           // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkStrikePrice]      = "PaymentStreamLinkStrikePrice";                    // (Price FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkStrikePriceType]  = "PaymentStreamLinkStrikePriceType";                // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkMaximumBoundary]  = "PaymentStreamLinkMaximumBoundary";                // (float FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkMinimumBoundary]  = "PaymentStreamLinkMinimumBoundary";                // (float FIX.5.0SP2)
dictionary[tag::PaymentStreamLinkNumberOfDataSeries] = "PaymentStreamLinkNumberOfDataSeries";             // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamVarianceUnadjustedCap] = "PaymentStreamVarianceUnadjustedCap";              // (float FIX.5.0SP2)
dictionary[tag::PaymentStreamRealizedVarianceMethod] = "PaymentStreamRealizedVarianceMethod";             // (int FIX.5.0SP2)
dictionary[tag::PaymentStreamDaysAdjustmentIndicator] = "PaymentStreamDaysAdjustmentIndicator";            // (Boolean FIX.5.0SP2)
dictionary[tag::PaymentStreamNearestExchangeContractRefID] = "PaymentStreamNearestExchangeContractRefID";       // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamVegaNotionalAmount]   = "PaymentStreamVegaNotionalAmount";                 // (float FIX.5.0SP2)
dictionary[tag::NoPaymentStreamFormulas]           = "NoPaymentStreamFormulas";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStreamFormula]              = "PaymentStreamFormula";                            // (XMLData FIX.5.0SP2)
dictionary[tag::PaymentStreamFormulaDesc]          = "PaymentStreamFormulaDesc";                        // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamFormulaCurrency]      = "PaymentStreamFormulaCurrency";                    // (Currency FIX.5.0SP2)
dictionary[tag::PaymentStreamFormulaCurrencyDeterminationMethod] = "PaymentStreamFormulaCurrencyDeterminationMethod"; // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamFormulaReferenceAmount] = "PaymentStreamFormulaReferenceAmount";             // (int FIX.5.0SP2)
dictionary[tag::PaymentStubEndDateUnadjusted]      = "PaymentStubEndDateUnadjusted";                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStubEndDateBusinessDayConvention] = "PaymentStubEndDateBusinessDayConvention";         // (int FIX.5.0SP2)
dictionary[tag::PaymentStubEndDateRelativeTo]      = "PaymentStubEndDateRelativeTo";                    // (int FIX.5.0SP2)
dictionary[tag::PaymentStubEndDateOffsetPeriod]    = "PaymentStubEndDateOffsetPeriod";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentStubEndDateOffsetUnit]      = "PaymentStubEndDateOffsetUnit";                    // (String FIX.5.0SP2)
dictionary[tag::PaymentStubEndDateOffsetDayType]   = "PaymentStubEndDateOffsetDayType";                 // (int FIX.5.0SP2)
dictionary[tag::PaymentStubEndDateAdjusted]        = "PaymentStubEndDateAdjusted";                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoPaymentStubEndDateBusinessCenters] = "NoPaymentStubEndDateBusinessCenters";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStubEndDateBusinessCenter]  = "PaymentStubEndDateBusinessCenter";                // (String FIX.5.0SP2)
dictionary[tag::PaymentStubStartDateUnadjusted]    = "PaymentStubStartDateUnadjusted";                  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::PaymentStubStartDateBusinessDayConvention] = "PaymentStubStartDateBusinessDayConvention";       // (int FIX.5.0SP2)
dictionary[tag::PaymentStubStartDateRelativeTo]    = "PaymentStubStartDateRelativeTo";                  // (int FIX.5.0SP2)
dictionary[tag::PaymentStubStartDateOffsetPeriod]  = "PaymentStubStartDateOffsetPeriod";                // (int FIX.5.0SP2)
dictionary[tag::PaymentStubStartDateOffsetUnit]    = "PaymentStubStartDateOffsetUnit";                  // (String FIX.5.0SP2)
dictionary[tag::PaymentStubStartDateOffsetDayType] = "PaymentStubStartDateOffsetDayType";               // (int FIX.5.0SP2)
dictionary[tag::PaymentStubStartDateAdjusted]      = "PaymentStubStartDateAdjusted";                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoPaymentStubStartDateBusinessCenters] = "NoPaymentStubStartDateBusinessCenters";           // (NumInGroup FIX.5.0SP2)
dictionary[tag::PaymentStubStartDateBusinessCenter] = "PaymentStubStartDateBusinessCenter";              // (String FIX.5.0SP2)
dictionary[tag::ProvisionBreakFeeElection]         = "ProvisionBreakFeeElection";                       // (int FIX.5.0SP2)
dictionary[tag::ProvisionBreakFeeRate]             = "ProvisionBreakFeeRate";                           // (Percentage FIX.5.0SP2)
dictionary[tag::NoReturnRateDates]                 = "NoReturnRateDates";                               // (NumInGroup FIX.5.0SP2)
dictionary[tag::ReturnRateDateMode]                = "ReturnRateDateMode";                              // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationDateRelativeTo] = "ReturnRateValuationDateRelativeTo";               // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationDateOffsetPeriod] = "ReturnRateValuationDateOffsetPeriod";             // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationDateOffsetUnit] = "ReturnRateValuationDateOffsetUnit";               // (String FIX.5.0SP2)
dictionary[tag::ReturnRateValuationDateOffsetDayType] = "ReturnRateValuationDateOffsetDayType";            // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationStartDateUnadjusted] = "ReturnRateValuationStartDateUnadjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ReturnRateValuationStartDateRelativeTo] = "ReturnRateValuationStartDateRelativeTo";          // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationStartDateOffsetPeriod] = "ReturnRateValuationStartDateOffsetPeriod";        // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationStartDateOffsetUnit] = "ReturnRateValuationStartDateOffsetUnit";          // (String FIX.5.0SP2)
dictionary[tag::ReturnRateValuationStartDateOffsetDayType] = "ReturnRateValuationStartDateOffsetDayType";       // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationStartDateAdjusted] = "ReturnRateValuationStartDateAdjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ReturnRateValuationEndDateUnadjusted] = "ReturnRateValuationEndDateUnadjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ReturnRateValuationEndDateRelativeTo] = "ReturnRateValuationEndDateRelativeTo";            // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationEndDateOffsetPeriod] = "ReturnRateValuationEndDateOffsetPeriod";          // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationEndDateOffsetUnit] = "ReturnRateValuationEndDateOffsetUnit";            // (String FIX.5.0SP2)
dictionary[tag::ReturnRateValuationEndDateOffsetDayType] = "ReturnRateValuationEndDateOffsetDayType";         // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationEndDateAdjusted] = "ReturnRateValuationEndDateAdjusted";              // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ReturnRateValuationFrequencyPeriod] = "ReturnRateValuationFrequencyPeriod";              // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationFrequencyUnit]  = "ReturnRateValuationFrequencyUnit";                // (String FIX.5.0SP2)
dictionary[tag::ReturnRateValuationFrequencyRollConvention] = "ReturnRateValuationFrequencyRollConvention";      // (String FIX.5.0SP2)
dictionary[tag::ReturnRateValuationDateBusinessDayConvention] = "ReturnRateValuationDateBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::NoReturnRateFXConversions]         = "NoReturnRateFXConversions";                       // (NumInGroup FIX.5.0SP2)
dictionary[tag::ReturnRateFXCurrencySymbol]        = "ReturnRateFXCurrencySymbol";                      // (String FIX.5.0SP2)
dictionary[tag::ReturnRateFXRate]                  = "ReturnRateFXRate";                                // (float FIX.5.0SP2)
dictionary[tag::ReturnRateFXRateCalc]              = "ReturnRateFXRateCalc";                            // (char FIX.5.0SP2)
dictionary[tag::NoReturnRates]                     = "NoReturnRates";                                   // (NumInGroup FIX.5.0SP2)
dictionary[tag::ReturnRatePriceSequence]           = "ReturnRatePriceSequence";                         // (int FIX.5.0SP2)
dictionary[tag::ReturnRateCommissionBasis]         = "ReturnRateCommissionBasis";                       // (char FIX.5.0SP2)
dictionary[tag::ReturnRateCommissionAmount]        = "ReturnRateCommissionAmount";                      // (Amt FIX.5.0SP2)
dictionary[tag::ReturnRateCommissionCurrency]      = "ReturnRateCommissionCurrency";                    // (Currency FIX.5.0SP2)
dictionary[tag::ReturnRateTotalCommissionPerTrade] = "ReturnRateTotalCommissionPerTrade";               // (Amt FIX.5.0SP2)
dictionary[tag::ReturnRateDeterminationMethod]     = "ReturnRateDeterminationMethod";                   // (String FIX.5.0SP2)
dictionary[tag::ReturnRateAmountRelativeTo]        = "ReturnRateAmountRelativeTo";                      // (int FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteMeasureType]        = "ReturnRateQuoteMeasureType";                      // (String FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteUnits]              = "ReturnRateQuoteUnits";                            // (String FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteMethod]             = "ReturnRateQuoteMethod";                           // (int FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteCurrency]           = "ReturnRateQuoteCurrency";                         // (Currency FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteCurrencyType]       = "ReturnRateQuoteCurrencyType";                     // (String FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteTimeType]           = "ReturnRateQuoteTimeType";                         // (int FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteTime]               = "ReturnRateQuoteTime";                             // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteDate]               = "ReturnRateQuoteDate";                             // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteExpirationTime]     = "ReturnRateQuoteExpirationTime";                   // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteBusinessCenter]     = "ReturnRateQuoteBusinessCenter";                   // (String FIX.5.0SP2)
dictionary[tag::ReturnRateQuoteExchange]           = "ReturnRateQuoteExchange";                         // (Exchange FIX.5.0SP2)
dictionary[tag::ReturnRateQuotePricingModel]       = "ReturnRateQuotePricingModel";                     // (String FIX.5.0SP2)
dictionary[tag::ReturnRateCashFlowType]            = "ReturnRateCashFlowType";                          // (String FIX.5.0SP2)
dictionary[tag::ReturnRateValuationTimeType]       = "ReturnRateValuationTimeType";                     // (int FIX.5.0SP2)
dictionary[tag::ReturnRateValuationTime]           = "ReturnRateValuationTime";                         // (LocalMktTime FIX.5.0SP2)
dictionary[tag::ReturnRateValuationTimeBusinessCenter] = "ReturnRateValuationTimeBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::ReturnRateValuationPriceOption]    = "ReturnRateValuationPriceOption";                  // (int FIX.5.0SP2)
dictionary[tag::ReturnRateFinalPriceFallback]      = "ReturnRateFinalPriceFallback";                    // (int FIX.5.0SP2)
dictionary[tag::NoReturnRateInformationSources]    = "NoReturnRateInformationSources";                  // (NumInGroup FIX.5.0SP2)
dictionary[tag::ReturnRateInformationSource]       = "ReturnRateInformationSource";                     // (int FIX.5.0SP2)
dictionary[tag::ReturnRateReferencePage]           = "ReturnRateReferencePage";                         // (String FIX.5.0SP2)
dictionary[tag::ReturnRateReferencePageHeading]    = "ReturnRateReferencePageHeading";                  // (String FIX.5.0SP2)
dictionary[tag::NoReturnRatePrices]                = "NoReturnRatePrices";                              // (NumInGroup FIX.5.0SP2)
dictionary[tag::ReturnRatePriceBasis]              = "ReturnRatePriceBasis";                            // (int FIX.5.0SP2)
dictionary[tag::ReturnRatePrice]                   = "ReturnRatePrice";                                 // (Price FIX.5.0SP2)
dictionary[tag::ReturnRatePriceCurrency]           = "ReturnRatePriceCurrency";                         // (Currency FIX.5.0SP2)
dictionary[tag::ReturnRatePriceType]               = "ReturnRatePriceType";                             // (int FIX.5.0SP2)
dictionary[tag::NoReturnRateValuationDateBusinessCenters] = "NoReturnRateValuationDateBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::ReturnRateValuationDateBusinessCenter] = "ReturnRateValuationDateBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::NoReturnRateValuationDates]        = "NoReturnRateValuationDates";                      // (NumInGroup FIX.5.0SP2)
dictionary[tag::ReturnRateValuationDate]           = "ReturnRateValuationDate";                         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::ReturnRateValuationDateType]       = "ReturnRateValuationDateType";                     // (int FIX.5.0SP2)
dictionary[tag::NoSettlMethodElectionDateBusinessCenters] = "NoSettlMethodElectionDateBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::SettlMethodElectionDateBusinessCenter] = "SettlMethodElectionDateBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::SettlMethodElectionDateUnadjusted] = "SettlMethodElectionDateUnadjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::SettlMethodElectionDateBusinessDayConvention] = "SettlMethodElectionDateBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::SettlMethodElectionDateRelativeTo] = "SettlMethodElectionDateRelativeTo";               // (int FIX.5.0SP2)
dictionary[tag::SettlMethodElectionDateOffsetPeriod] = "SettlMethodElectionDateOffsetPeriod";             // (int FIX.5.0SP2)
dictionary[tag::SettlMethodElectionDateOffsetUnit] = "SettlMethodElectionDateOffsetUnit";               // (String FIX.5.0SP2)
dictionary[tag::SettlMethodElectionDateOffsetDayType] = "SettlMethodElectionDateOffsetDayType";            // (int FIX.5.0SP2)
dictionary[tag::SettlMethodElectionDateAdjusted]   = "SettlMethodElectionDateAdjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamVersion]                     = "StreamVersion";                                   // (String FIX.5.0SP2)
dictionary[tag::StreamVersionEffectiveDate]        = "StreamVersionEffectiveDate";                      // (LocalMktDate FIX.5.0SP2)
dictionary[tag::StreamNotionalDeterminationMethod] = "StreamNotionalDeterminationMethod";               // (String FIX.5.0SP2)
dictionary[tag::StreamNotionalAdjustments]         = "StreamNotionalAdjustments";                       // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingCashSettlDateBusinessCenters] = "NoUnderlyingCashSettlDateBusinessCenters";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDateBusinessCenter] = "UnderlyingCashSettlDateBusinessCenter";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDateUnadjusted] = "UnderlyingCashSettlDateUnadjusted";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDateBusinessDayConvention] = "UnderlyingCashSettlDateBusinessDayConvention";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDateRelativeTo] = "UnderlyingCashSettlDateRelativeTo";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDateOffsetPeriod] = "UnderlyingCashSettlDateOffsetPeriod";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDateOffsetUnit] = "UnderlyingCashSettlDateOffsetUnit";               // (String FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDateOffsetDayType] = "UnderlyingCashSettlDateOffsetDayType";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlDateAdjusted]   = "UnderlyingCashSettlDateAdjusted";                 // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlPriceSource]    = "UnderlyingCashSettlPriceSource";                  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingCashSettlPriceDefault]   = "UnderlyingCashSettlPriceDefault";                 // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingDividendAccrualPaymentDateBusinessCenters] = "NoUnderlyingDividendAccrualPaymentDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualPaymentDateBusinessCenter] = "UnderlyingDividendAccrualPaymentDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloatingRateIndex] = "UnderlyingDividendFloatingRateIndex";             // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloatingRateIndexCurvePeriod] = "UnderlyingDividendFloatingRateIndexCurvePeriod";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloatingRateIndexCurveUnit] = "UnderlyingDividendFloatingRateIndexCurveUnit";    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloatingRateMultiplier] = "UnderlyingDividendFloatingRateMultiplier";        // (float FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloatingRateSpread] = "UnderlyingDividendFloatingRateSpread";            // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloatingRateSpreadPositionType] = "UnderlyingDividendFloatingRateSpreadPositionType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloatingRateTreatment] = "UnderlyingDividendFloatingRateTreatment";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendCapRate]         = "UnderlyingDividendCapRate";                       // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingDividendCapRateBuySide]  = "UnderlyingDividendCapRateBuySide";                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendCapRateSellSide] = "UnderlyingDividendCapRateSellSide";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloorRate]       = "UnderlyingDividendFloorRate";                     // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloorRateBuySide] = "UnderlyingDividendFloorRateBuySide";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFloorRateSellSide] = "UnderlyingDividendFloorRateSellSide";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendInitialRate]     = "UnderlyingDividendInitialRate";                   // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFinalRateRoundingDirection] = "UnderlyingDividendFinalRateRoundingDirection";    // (char FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFinalRatePrecision] = "UnderlyingDividendFinalRatePrecision";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAveragingMethod] = "UnderlyingDividendAveragingMethod";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendNegativeRateTreatment] = "UnderlyingDividendNegativeRateTreatment";         // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualPaymentDateRelativeTo] = "UnderlyingDividendAccrualPaymentDateRelativeTo";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualPaymentDateOffsetPeriod] = "UnderlyingDividendAccrualPaymentDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualPaymentDateOffsetUnit] = "UnderlyingDividendAccrualPaymentDateOffsetUnit";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualPaymentDateOffsetDayType] = "UnderlyingDividendAccrualPaymentDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualPaymentDateUnadjusted] = "UnderlyingDividendAccrualPaymentDateUnadjusted";  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualPaymentDateBusinessDayConvention] = "UnderlyingDividendAccrualPaymentDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualPaymentDateAdjusted] = "UnderlyingDividendAccrualPaymentDateAdjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendReinvestmentIndicator] = "UnderlyingDividendReinvestmentIndicator";         // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingDividendEntitlementEvent] = "UnderlyingDividendEntitlementEvent";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAmountType]      = "UnderlyingDividendAmountType";                    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendUnderlierRefID]  = "UnderlyingDividendUnderlierRefID";                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingExtraordinaryDividendPartySide] = "UnderlyingExtraordinaryDividendPartySide";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingExtraordinaryDividendAmountType] = "UnderlyingExtraordinaryDividendAmountType";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingExtraordinaryDividendCurrency] = "UnderlyingExtraordinaryDividendCurrency";         // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingExtraordinaryDividendDeterminationMethod] = "UnderlyingExtraordinaryDividendDeterminationMethod"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccrualFixedRate] = "UnderlyingDividendAccrualFixedRate";              // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingDividendCompoundingMethod] = "UnderlyingDividendCompoundingMethod";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendNumOfIndexUnits] = "UnderlyingDividendNumOfIndexUnits";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendCashPercentage]  = "UnderlyingDividendCashPercentage";                // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingDividendCashEquivalentPercentage] = "UnderlyingDividendCashEquivalentPercentage";      // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingNonCashDividendTreatment] = "UnderlyingNonCashDividendTreatment";              // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendComposition]     = "UnderlyingDividendComposition";                   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSpecialDividendsIndicator] = "UnderlyingSpecialDividendsIndicator";             // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingMaterialDividendsIndicator] = "UnderlyingMaterialDividendsIndicator";            // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingOptionsExchangeDividendsIndicator] = "UnderlyingOptionsExchangeDividendsIndicator";     // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingAdditionalDividendsIndicator] = "UnderlyingAdditionalDividendsIndicator";          // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingAllDividendsIndicator]   = "UnderlyingAllDividendsIndicator";                 // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFXTriggerDateRelativeTo] = "UnderlyingDividendFXTriggerDateRelativeTo";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFXTriggerDateOffsetPeriod] = "UnderlyingDividendFXTriggerDateOffsetPeriod";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFXTriggerDateOffsetUnit] = "UnderlyingDividendFXTriggerDateOffsetUnit";       // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFXTriggerDateOffsetDayType] = "UnderlyingDividendFXTriggerDateOffsetDayType";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFXTriggerDateUnadjusted] = "UnderlyingDividendFXTriggerDateUnadjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFXTriggerDateBusinessDayConvention] = "UnderlyingDividendFXTriggerDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFXTriggerDateAdjusted] = "UnderlyingDividendFXTriggerDateAdjusted";         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingDividendFXTriggerDateBusinessCenters] = "NoUnderlyingDividendFXTriggerDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDividendFXTriggerDateBusinessCenter] = "UnderlyingDividendFXTriggerDateBusinessCenter";   // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingDividendPayments]      = "NoUnderlyingDividendPayments";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPaymentDate]     = "UnderlyingDividendPaymentDate";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPaymentAmount]   = "UnderlyingDividendPaymentAmount";                 // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPaymentCurrency] = "UnderlyingDividendPaymentCurrency";               // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingDividendAccruedInterest] = "UnderlyingDividendAccruedInterest";               // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPayoutRatio]     = "UnderlyingDividendPayoutRatio";                   // (float FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPayoutConditions] = "UnderlyingDividendPayoutConditions";              // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingDividendPeriods]       = "NoUnderlyingDividendPeriods";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodSequence]  = "UnderlyingDividendPeriodSequence";                // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodStartDateUnadjusted] = "UnderlyingDividendPeriodStartDateUnadjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodEndDateUnadjusted] = "UnderlyingDividendPeriodEndDateUnadjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodUnderlierRefID] = "UnderlyingDividendPeriodUnderlierRefID";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodStrikePrice] = "UnderlyingDividendPeriodStrikePrice";             // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodBusinessDayConvention] = "UnderlyingDividendPeriodBusinessDayConvention";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodValuationDateUnadjusted] = "UnderlyingDividendPeriodValuationDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodValuationDateRelativeTo] = "UnderlyingDividendPeriodValuationDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodValuationDateOffsetPeriod] = "UnderlyingDividendPeriodValuationDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodValuationDateOffsetUnit] = "UnderlyingDividendPeriodValuationDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodValuationDateOffsetDayType] = "UnderlyingDividendPeriodValuationDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodValuationDateAdjusted] = "UnderlyingDividendPeriodValuationDateAdjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodPaymentDateUnadjusted] = "UnderlyingDividendPeriodPaymentDateUnadjusted";   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodPaymentDateRelativeTo] = "UnderlyingDividendPeriodPaymentDateRelativeTo";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodPaymentDateOffsetPeriod] = "UnderlyingDividendPeriodPaymentDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodPaymentDateOffsetUnit] = "UnderlyingDividendPeriodPaymentDateOffsetUnit";   // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodPaymentDateOffsetDayType] = "UnderlyingDividendPeriodPaymentDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodPaymentDateAdjusted] = "UnderlyingDividendPeriodPaymentDateAdjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodXID]       = "UnderlyingDividendPeriodXID";                     // (XID FIX.5.0SP2)
dictionary[tag::NoUnderlyingDividendPeriodBusinessCenters] = "NoUnderlyingDividendPeriodBusinessCenters";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingDividendPeriodBusinessCenter] = "UnderlyingDividendPeriodBusinessCenter";          // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingExtraordinaryEvents]   = "NoUnderlyingExtraordinaryEvents";                 // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingExtraordinaryEventType]  = "UnderlyingExtraordinaryEventType";                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingExtraordinaryEventValue] = "UnderlyingExtraordinaryEventValue";               // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectingPartySide] = "UnderlyingSettlMethodElectingPartySide";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingMakeWholeDate]           = "UnderlyingMakeWholeDate";                         // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingMakeWholeAmount]         = "UnderlyingMakeWholeAmount";                       // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingMakeWholeBenchmarkCurveName] = "UnderlyingMakeWholeBenchmarkCurveName";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingMakeWholeBenchmarkCurvePoint] = "UnderlyingMakeWholeBenchmarkCurvePoint";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingMakeWholeRecallSpread]   = "UnderlyingMakeWholeRecallSpread";                 // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingMakeWholeBenchmarkQuote] = "UnderlyingMakeWholeBenchmarkQuote";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingMakeWholeInterpolationMethod] = "UnderlyingMakeWholeInterpolationMethod";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCashSettlIndicator] = "UnderlyingPaymentStreamCashSettlIndicator";       // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingXIDRef] = "UnderlyingPaymentStreamCompoundingXIDRef";        // (XIDREF FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingSpread] = "UnderlyingPaymentStreamCompoundingSpread";        // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInterpolationMethod] = "UnderlyingPaymentStreamInterpolationMethod";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamInterpolationPeriod] = "UnderlyingPaymentStreamInterpolationPeriod";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingFixedRate] = "UnderlyingPaymentStreamCompoundingFixedRate";     // (float FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamCompoundingDates] = "NoUnderlyingPaymentStreamCompoundingDates";       // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingDate] = "UnderlyingPaymentStreamCompoundingDate";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingDateType] = "UnderlyingPaymentStreamCompoundingDateType";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingDatesBusinessDayConvention] = "UnderlyingPaymentStreamCompoundingDatesBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingDatesRelativeTo] = "UnderlyingPaymentStreamCompoundingDatesRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingDatesOffsetPeriod] = "UnderlyingPaymentStreamCompoundingDatesOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingDatesOffsetUnit] = "UnderlyingPaymentStreamCompoundingDatesOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingDatesOffsetDayType] = "UnderlyingPaymentStreamCompoundingDatesOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingPeriodSkip] = "UnderlyingPaymentStreamCompoundingPeriodSkip";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingFrequencyPeriod] = "UnderlyingPaymentStreamCompoundingFrequencyPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingFrequencyUnit] = "UnderlyingPaymentStreamCompoundingFrequencyUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingRollConvention] = "UnderlyingPaymentStreamCompoundingRollConvention"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamBoundsFirstDateUnadjusted] = "UnderlyingPaymentStreamBoundsFirstDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamBoundsLastDateUnadjusted] = "UnderlyingPaymentStreamBoundsLastDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamCompoundingDatesBusinessCenters] = "NoUnderlyingPaymentStreamCompoundingDatesBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingDatesBusinessCenter] = "UnderlyingPaymentStreamCompoundingDatesBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingEndDateUnadjusted] = "UnderlyingPaymentStreamCompoundingEndDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingEndDateRelativeTo] = "UnderlyingPaymentStreamCompoundingEndDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingEndDateOffsetPeriod] = "UnderlyingPaymentStreamCompoundingEndDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingEndDateOffsetUnit] = "UnderlyingPaymentStreamCompoundingEndDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingEndDateOffsetDayType] = "UnderlyingPaymentStreamCompoundingEndDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingEndDateAdjusted] = "UnderlyingPaymentStreamCompoundingEndDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingRateIndex] = "UnderlyingPaymentStreamCompoundingRateIndex";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingRateIndexCurvePeriod] = "UnderlyingPaymentStreamCompoundingRateIndexCurvePeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingRateIndexCurveUnit] = "UnderlyingPaymentStreamCompoundingRateIndexCurveUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingRateMultiplier] = "UnderlyingPaymentStreamCompoundingRateMultiplier"; // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingRateSpread] = "UnderlyingPaymentStreamCompoundingRateSpread";    // (PriceOffset FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingRateSpreadPositionType] = "UnderlyingPaymentStreamCompoundingRateSpreadPositionType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingRateTreatment] = "UnderlyingPaymentStreamCompoundingRateTreatment"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingCapRate] = "UnderlyingPaymentStreamCompoundingCapRate";       // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingCapRateBuySide] = "UnderlyingPaymentStreamCompoundingCapRateBuySide"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingCapRateSellSide] = "UnderlyingPaymentStreamCompoundingCapRateSellSide"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingFloorRate] = "UnderlyingPaymentStreamCompoundingFloorRate";     // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingFloorRateBuySide] = "UnderlyingPaymentStreamCompoundingFloorRateBuySide"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingFloorRateSellSide] = "UnderlyingPaymentStreamCompoundingFloorRateSellSide"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingInitialRate] = "UnderlyingPaymentStreamCompoundingInitialRate";   // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingFinalRateRoundingDirection] = "UnderlyingPaymentStreamCompoundingFinalRateRoundingDirection"; // (char FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingFinalRatePrecision] = "UnderlyingPaymentStreamCompoundingFinalRatePrecision"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingAveragingMethod] = "UnderlyingPaymentStreamCompoundingAveragingMethod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingNegativeRateTreatment] = "UnderlyingPaymentStreamCompoundingNegativeRateTreatment"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingStartDateUnadjusted] = "UnderlyingPaymentStreamCompoundingStartDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingStartDateRelativeTo] = "UnderlyingPaymentStreamCompoundingStartDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingStartDateOffsetPeriod] = "UnderlyingPaymentStreamCompoundingStartDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingStartDateOffsetUnit] = "UnderlyingPaymentStreamCompoundingStartDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingStartDateOffsetDayType] = "UnderlyingPaymentStreamCompoundingStartDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamCompoundingStartDateAdjusted] = "UnderlyingPaymentStreamCompoundingStartDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFormulaImageLength] = "UnderlyingPaymentStreamFormulaImageLength";       // (Length FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFormulaImage] = "UnderlyingPaymentStreamFormulaImage";             // (data FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalPricePaymentDateUnadjusted] = "UnderlyingPaymentStreamFinalPricePaymentDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalPricePaymentDateRelativeTo] = "UnderlyingPaymentStreamFinalPricePaymentDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalPricePaymentDateOffsetPeriod] = "UnderlyingPaymentStreamFinalPricePaymentDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalPricePaymentDateOffsetUnit] = "UnderlyingPaymentStreamFinalPricePaymentDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalPricePaymentDateOffsetDayType] = "UnderlyingPaymentStreamFinalPricePaymentDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFinalPricePaymentDateAdjusted] = "UnderlyingPaymentStreamFinalPricePaymentDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamFixingDates] = "NoUnderlyingPaymentStreamFixingDates";            // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDate] = "UnderlyingPaymentStreamFixingDate";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFixingDateType] = "UnderlyingPaymentStreamFixingDateType";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFirstObservationDateUnadjusted] = "UnderlyingPaymentStreamFirstObservationDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFirstObservationDateRelativeTo] = "UnderlyingPaymentStreamFirstObservationDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFirstObservationDateOffsetDayType] = "UnderlyingPaymentStreamFirstObservationDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFirstObservationDateAdjusted] = "UnderlyingPaymentStreamFirstObservationDateAdjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamUnderlierRefID] = "UnderlyingPaymentStreamUnderlierRefID";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateNotionalReset] = "UnderlyingReturnRateNotionalReset";               // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkInitialLevel] = "UnderlyingPaymentStreamLinkInitialLevel";         // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkClosingLevelIndicator] = "UnderlyingPaymentStreamLinkClosingLevelIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkExpiringLevelIndicator] = "UnderlyingPaymentStreamLinkExpiringLevelIndicator"; // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkEstimatedTradingDays] = "UnderlyingPaymentStreamLinkEstimatedTradingDays"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkStrikePrice] = "UnderlyingPaymentStreamLinkStrikePrice";          // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkStrikePriceType] = "UnderlyingPaymentStreamLinkStrikePriceType";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkMaximumBoundary] = "UnderlyingPaymentStreamLinkMaximumBoundary";      // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkMinimumBoundary] = "UnderlyingPaymentStreamLinkMinimumBoundary";      // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamLinkNumberOfDataSeries] = "UnderlyingPaymentStreamLinkNumberOfDataSeries";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamVarianceUnadjustedCap] = "UnderlyingPaymentStreamVarianceUnadjustedCap";    // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRealizedVarianceMethod] = "UnderlyingPaymentStreamRealizedVarianceMethod";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamDaysAdjustmentIndicator] = "UnderlyingPaymentStreamDaysAdjustmentIndicator";  // (Boolean FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamNearestExchangeContractRefID] = "UnderlyingPaymentStreamNearestExchangeContractRefID"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamVegaNotionalAmount] = "UnderlyingPaymentStreamVegaNotionalAmount";       // (float FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFormulaCurrency] = "UnderlyingPaymentStreamFormulaCurrency";          // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFormulaCurrencyDeterminationMethod] = "UnderlyingPaymentStreamFormulaCurrencyDeterminationMethod"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFormulaReferenceAmount] = "UnderlyingPaymentStreamFormulaReferenceAmount";   // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStreamFormulas] = "NoUnderlyingPaymentStreamFormulas";               // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFormula]    = "UnderlyingPaymentStreamFormula";                  // (XMLData FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamFormulaDesc] = "UnderlyingPaymentStreamFormulaDesc";              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubEndDateUnadjusted] = "UnderlyingPaymentStubEndDateUnadjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubEndDateBusinessDayConvention] = "UnderlyingPaymentStubEndDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubEndDateRelativeTo] = "UnderlyingPaymentStubEndDateRelativeTo";          // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubEndDateOffsetPeriod] = "UnderlyingPaymentStubEndDateOffsetPeriod";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubEndDateOffsetUnit] = "UnderlyingPaymentStubEndDateOffsetUnit";          // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubEndDateOffsetDayType] = "UnderlyingPaymentStubEndDateOffsetDayType";       // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubEndDateAdjusted] = "UnderlyingPaymentStubEndDateAdjusted";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStubEndDateBusinessCenters] = "NoUnderlyingPaymentStubEndDateBusinessCenters";   // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubEndDateBusinessCenter] = "UnderlyingPaymentStubEndDateBusinessCenter";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubStartDateUnadjusted] = "UnderlyingPaymentStubStartDateUnadjusted";        // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubStartDateBusinessDayConvention] = "UnderlyingPaymentStubStartDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubStartDateRelativeTo] = "UnderlyingPaymentStubStartDateRelativeTo";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubStartDateOffsetPeriod] = "UnderlyingPaymentStubStartDateOffsetPeriod";      // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubStartDateOffsetUnit] = "UnderlyingPaymentStubStartDateOffsetUnit";        // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubStartDateOffsetDayType] = "UnderlyingPaymentStubStartDateOffsetDayType";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubStartDateAdjusted] = "UnderlyingPaymentStubStartDateAdjusted";          // (LocalMktDate FIX.5.0SP2)
dictionary[tag::NoUnderlyingPaymentStubStartDateBusinessCenters] = "NoUnderlyingPaymentStubStartDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStubStartDateBusinessCenter] = "UnderlyingPaymentStubStartDateBusinessCenter";    // (String FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionBreakFeeElection] = "UnderlyingProvisionBreakFeeElection";             // (int FIX.5.0SP2)
dictionary[tag::UnderlyingProvisionBreakFeeRate]   = "UnderlyingProvisionBreakFeeRate";                 // (Percentage FIX.5.0SP2)
dictionary[tag::UnderlyingRateSpreadInitialValue]  = "UnderlyingRateSpreadInitialValue";                // (float FIX.5.0SP2)
dictionary[tag::NoUnderlyingRateSpreadSteps]       = "NoUnderlyingRateSpreadSteps";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingRateSpreadStepDate]      = "UnderlyingRateSpreadStepDate";                    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingRateSpreadStepValue]     = "UnderlyingRateSpreadStepValue";                   // (float FIX.5.0SP2)
dictionary[tag::NoUnderlyingReturnRateDates]       = "NoUnderlyingReturnRateDates";                     // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateDateMode]      = "UnderlyingReturnRateDateMode";                    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationDateRelativeTo] = "UnderlyingReturnRateValuationDateRelativeTo";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationDateOffsetPeriod] = "UnderlyingReturnRateValuationDateOffsetPeriod";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationDateOffsetUnit] = "UnderlyingReturnRateValuationDateOffsetUnit";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationDateOffsetDayType] = "UnderlyingReturnRateValuationDateOffsetDayType";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationStartDateUnadjusted] = "UnderlyingReturnRateValuationStartDateUnadjusted"; // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationStartDateRelativeTo] = "UnderlyingReturnRateValuationStartDateRelativeTo"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationStartDateOffsetPeriod] = "UnderlyingReturnRateValuationStartDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationStartDateOffsetUnit] = "UnderlyingReturnRateValuationStartDateOffsetUnit"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationStartDateOffsetDayType] = "UnderlyingReturnRateValuationStartDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationStartDateAdjusted] = "UnderlyingReturnRateValuationStartDateAdjusted";  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationEndDateUnadjusted] = "UnderlyingReturnRateValuationEndDateUnadjusted";  // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationEndDateRelativeTo] = "UnderlyingReturnRateValuationEndDateRelativeTo";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationEndDateOffsetPeriod] = "UnderlyingReturnRateValuationEndDateOffsetPeriod"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationEndDateOffsetUnit] = "UnderlyingReturnRateValuationEndDateOffsetUnit";  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationEndDateOffsetDayType] = "UnderlyingReturnRateValuationEndDateOffsetDayType"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationEndDateAdjusted] = "UnderlyingReturnRateValuationEndDateAdjusted";    // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationFrequencyPeriod] = "UnderlyingReturnRateValuationFrequencyPeriod";    // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationFrequencyUnit] = "UnderlyingReturnRateValuationFrequencyUnit";      // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationFrequencyRollConvention] = "UnderlyingReturnRateValuationFrequencyRollConvention"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationDateBusinessDayConvention] = "UnderlyingReturnRateValuationDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingReturnRateFXConversions] = "NoUnderlyingReturnRateFXConversions";             // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateFXCurrencySymbol] = "UnderlyingReturnRateFXCurrencySymbol";            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateFXRate]        = "UnderlyingReturnRateFXRate";                      // (float FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateFXRateCalc]    = "UnderlyingReturnRateFXRateCalc";                  // (char FIX.5.0SP2)
dictionary[tag::NoUnderlyingReturnRates]           = "NoUnderlyingReturnRates";                         // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRatePriceSequence] = "UnderlyingReturnRatePriceSequence";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateCommissionBasis] = "UnderlyingReturnRateCommissionBasis";             // (char FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateCommissionAmount] = "UnderlyingReturnRateCommissionAmount";            // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateCommissionCurrency] = "UnderlyingReturnRateCommissionCurrency";          // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateTotalCommissionPerTrade] = "UnderlyingReturnRateTotalCommissionPerTrade";     // (Amt FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateDeterminationMethod] = "UnderlyingReturnRateDeterminationMethod";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateAmountRelativeTo] = "UnderlyingReturnRateAmountRelativeTo";            // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteMeasureType] = "UnderlyingReturnRateQuoteMeasureType";            // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteUnits]    = "UnderlyingReturnRateQuoteUnits";                  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteMethod]   = "UnderlyingReturnRateQuoteMethod";                 // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteCurrency] = "UnderlyingReturnRateQuoteCurrency";               // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteCurrencyType] = "UnderlyingReturnRateQuoteCurrencyType";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteTimeType] = "UnderlyingReturnRateQuoteTimeType";               // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteTime]     = "UnderlyingReturnRateQuoteTime";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteDate]     = "UnderlyingReturnRateQuoteDate";                   // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteExpirationTime] = "UnderlyingReturnRateQuoteExpirationTime";         // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteBusinessCenter] = "UnderlyingReturnRateQuoteBusinessCenter";         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuoteExchange] = "UnderlyingReturnRateQuoteExchange";               // (Exchange FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateQuotePricingModel] = "UnderlyingReturnRateQuotePricingModel";           // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateCashFlowType]  = "UnderlyingReturnRateCashFlowType";                // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationTimeType] = "UnderlyingReturnRateValuationTimeType";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationTime] = "UnderlyingReturnRateValuationTime";               // (LocalMktTime FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationTimeBusinessCenter] = "UnderlyingReturnRateValuationTimeBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationPriceOption] = "UnderlyingReturnRateValuationPriceOption";        // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateFinalPriceFallback] = "UnderlyingReturnRateFinalPriceFallback";          // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingReturnRateInformationSources] = "NoUnderlyingReturnRateInformationSources";        // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateInformationSource] = "UnderlyingReturnRateInformationSource";           // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateReferencePage] = "UnderlyingReturnRateReferencePage";               // (String FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateReferencePageHeading] = "UnderlyingReturnRateReferencePageHeading";        // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingReturnRatePrices]      = "NoUnderlyingReturnRatePrices";                    // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRatePriceBasis]    = "UnderlyingReturnRatePriceBasis";                  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRatePrice]         = "UnderlyingReturnRatePrice";                       // (Price FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRatePriceCurrency] = "UnderlyingReturnRatePriceCurrency";               // (Currency FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRatePriceType]     = "UnderlyingReturnRatePriceType";                   // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingReturnRateValuationDateBusinessCenters] = "NoUnderlyingReturnRateValuationDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationDateBusinessCenter] = "UnderlyingReturnRateValuationDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::NoUnderlyingReturnRateValuationDates] = "NoUnderlyingReturnRateValuationDates";            // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationDate] = "UnderlyingReturnRateValuationDate";               // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingReturnRateValuationDateType] = "UnderlyingReturnRateValuationDateType";           // (int FIX.5.0SP2)
dictionary[tag::NoUnderlyingSettlMethodElectionDateBusinessCenters] = "NoUnderlyingSettlMethodElectionDateBusinessCenters"; // (NumInGroup FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectionDateBusinessCenter] = "UnderlyingSettlMethodElectionDateBusinessCenter"; // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectionDateUnadjusted] = "UnderlyingSettlMethodElectionDateUnadjusted";     // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectionDateBusinessDayConvention] = "UnderlyingSettlMethodElectionDateBusinessDayConvention"; // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectionDateRelativeTo] = "UnderlyingSettlMethodElectionDateRelativeTo";     // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectionDateOffsetPeriod] = "UnderlyingSettlMethodElectionDateOffsetPeriod";   // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectionDateOffsetUnit] = "UnderlyingSettlMethodElectionDateOffsetUnit";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectionDateOffsetDayType] = "UnderlyingSettlMethodElectionDateOffsetDayType";  // (int FIX.5.0SP2)
dictionary[tag::UnderlyingSettlMethodElectionDateAdjusted] = "UnderlyingSettlMethodElectionDateAdjusted";       // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamVersion]           = "UnderlyingStreamVersion";                         // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamVersionEffectiveDate] = "UnderlyingStreamVersionEffectiveDate";            // (LocalMktDate FIX.5.0SP2)
dictionary[tag::UnderlyingStreamNotionalDeterminationMethod] = "UnderlyingStreamNotionalDeterminationMethod";     // (String FIX.5.0SP2)
dictionary[tag::UnderlyingStreamNotionalAdjustments] = "UnderlyingStreamNotionalAdjustments";             // (int FIX.5.0SP2)
dictionary[tag::PaymentDesc]                       = "PaymentDesc";                                     // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndexID]       = "LegPaymentStreamRateIndexID";                     // (String FIX.5.0SP2)
dictionary[tag::LegPaymentStreamRateIndexIDSource] = "LegPaymentStreamRateIndexIDSource";               // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndexID]          = "PaymentStreamRateIndexID";                        // (String FIX.5.0SP2)
dictionary[tag::PaymentStreamRateIndexIDSource]    = "PaymentStreamRateIndexIDSource";                  // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndexID] = "UnderlyingPaymentStreamRateIndexID";              // (String FIX.5.0SP2)
dictionary[tag::UnderlyingPaymentStreamRateIndexIDSource] = "UnderlyingPaymentStreamRateIndexIDSource";        // (String FIX.5.0SP2)
dictionary[tag::DeliveryStreamRouteOrCharter]      = "DeliveryStreamRouteOrCharter";                    // (String FIX.5.0SP2)
dictionary[tag::LegDeliveryStreamRouteOrCharter]   = "LegDeliveryStreamRouteOrCharter";                 // (String FIX.5.0SP2)
dictionary[tag::UnderlyingDeliveryStreamRouteOrCharter] = "UnderlyingDeliveryStreamRouteOrCharter";          // (String FIX.5.0SP2)
dictionary[tag::BatchID]                           = "BatchID";                                         // (String FIX.5.0SP2)
dictionary[tag::BatchTotalMessages]                = "BatchTotalMessages";                              // (int FIX.5.0SP2)
dictionary[tag::BatchProcessMode]                  = "BatchProcessMode";                                // (int FIX.5.0SP2)
}
/*!
 * \brief Populate an AssociativeContainer with the names of all the FIX message types.
 *
 * \param dictionary A reference to an AssociativeContainer<std::string, std::string>
 */
 template <typename AssociativeContainer> void dictionary_init_message(AssociativeContainer& dictionary) {
dictionary["0"]           = "Heartbeat";                             // (FIX.2.7)
dictionary["1"]           = "TestRequest";                           // (FIX.2.7)
dictionary["2"]           = "ResendRequest";                         // (FIX.2.7)
dictionary["3"]           = "Reject";                                // (FIX.2.7)
dictionary["4"]           = "SequenceReset";                         // (FIX.2.7)
dictionary["5"]           = "Logout";                                // (FIX.2.7)
dictionary["6"]           = "IOI";                                   // (FIX.2.7)
dictionary["7"]           = "Advertisement";                         // (FIX.2.7)
dictionary["8"]           = "ExecutionReport";                       // (FIX.2.7)
dictionary["9"]           = "OrderCancelReject";                     // (FIX.2.7)
dictionary["A"]           = "Logon";                                 // (FIX.2.7)
dictionary["B"]           = "News";                                  // (FIX.2.7)
dictionary["C"]           = "Email";                                 // (FIX.2.7)
dictionary["D"]           = "NewOrderSingle";                        // (FIX.2.7)
dictionary["E"]           = "NewOrderList";                          // (FIX.2.7)
dictionary["F"]           = "OrderCancelRequest";                    // (FIX.2.7)
dictionary["G"]           = "OrderCancelReplaceRequest";             // (FIX.2.7)
dictionary["H"]           = "OrderStatusRequest";                    // (FIX.2.7)
dictionary["J"]           = "AllocationInstruction";                 // (FIX.2.7)
dictionary["K"]           = "ListCancelRequest";                     // (FIX.2.7)
dictionary["L"]           = "ListExecute";                           // (FIX.2.7)
dictionary["M"]           = "ListStatusRequest";                     // (FIX.2.7)
dictionary["N"]           = "ListStatus";                            // (FIX.2.7)
dictionary["P"]           = "AllocationInstructionAck";              // (FIX.2.7)
dictionary["Q"]           = "DontKnowTrade";                         // (FIX.4.0)
dictionary["R"]           = "QuoteRequest";                          // (FIX.4.0)
dictionary["S"]           = "Quote";                                 // (FIX.4.0)
dictionary["T"]           = "SettlementInstructions";                // (FIX.4.1)
dictionary["V"]           = "MarketDataRequest";                     // (FIX.4.2)
dictionary["W"]           = "MarketDataSnapshotFullRefresh";         // (FIX.4.2)
dictionary["X"]           = "MarketDataIncrementalRefresh";          // (FIX.4.2)
dictionary["Y"]           = "MarketDataRequestReject";               // (FIX.4.2)
dictionary["Z"]           = "QuoteCancel";                           // (FIX.4.2)
dictionary["a"]           = "QuoteStatusRequest";                    // (FIX.4.2)
dictionary["b"]           = "MassQuoteAck";                          // (FIX.4.2)
dictionary["c"]           = "SecurityDefinitionRequest";             // (FIX.4.2)
dictionary["d"]           = "SecurityDefinition";                    // (FIX.4.2)
dictionary["e"]           = "SecurityStatusRequest";                 // (FIX.4.2)
dictionary["f"]           = "SecurityStatus";                        // (FIX.4.2)
dictionary["g"]           = "TradingSessionStatusRequest";           // (FIX.4.2)
dictionary["h"]           = "TradingSessionStatus";                  // (FIX.4.2)
dictionary["i"]           = "MassQuote";                             // (FIX.4.2)
dictionary["j"]           = "BusinessMessageReject";                 // (FIX.4.2)
dictionary["k"]           = "BidRequest";                            // (FIX.4.2)
dictionary["l"]           = "BidResponse";                           // (FIX.4.2)
dictionary["m"]           = "ListStrikePrice";                       // (FIX.4.2)
dictionary["n"]           = "XMLnonFIX";                             // (FIX.4.3)
dictionary["o"]           = "RegistrationInstructions";              // (FIX.4.3)
dictionary["p"]           = "RegistrationInstructionsResponse";      // (FIX.4.3)
dictionary["q"]           = "OrderMassCancelRequest";                // (FIX.4.3)
dictionary["r"]           = "OrderMassCancelReport";                 // (FIX.4.3)
dictionary["s"]           = "NewOrderCross";                         // (FIX.4.3)
dictionary["t"]           = "CrossOrderCancelReplaceRequest";        // (FIX.4.3)
dictionary["u"]           = "CrossOrderCancelRequest";               // (FIX.4.3)
dictionary["v"]           = "SecurityTypeRequest";                   // (FIX.4.3)
dictionary["w"]           = "SecurityTypes";                         // (FIX.4.3)
dictionary["x"]           = "SecurityListRequest";                   // (FIX.4.3)
dictionary["y"]           = "SecurityList";                          // (FIX.4.3)
dictionary["z"]           = "DerivativeSecurityListRequest";         // (FIX.4.3)
dictionary["AA"]          = "DerivativeSecurityList";                // (FIX.4.3)
dictionary["AB"]          = "NewOrderMultileg";                      // (FIX.4.3)
dictionary["AC"]          = "MultilegOrderCancelReplace";            // (FIX.4.3)
dictionary["AD"]          = "TradeCaptureReportRequest";             // (FIX.4.3)
dictionary["AE"]          = "TradeCaptureReport";                    // (FIX.4.3)
dictionary["AF"]          = "OrderMassStatusRequest";                // (FIX.4.3)
dictionary["AG"]          = "QuoteRequestReject";                    // (FIX.4.3)
dictionary["AH"]          = "RFQRequest";                            // (FIX.4.3)
dictionary["AI"]          = "QuoteStatusReport";                     // (FIX.4.3)
dictionary["AJ"]          = "QuoteResponse";                         // (FIX.4.4)
dictionary["AK"]          = "Confirmation";                          // (FIX.4.4)
dictionary["AL"]          = "PositionMaintenanceRequest";            // (FIX.4.4)
dictionary["AM"]          = "PositionMaintenanceReport";             // (FIX.4.4)
dictionary["AN"]          = "RequestForPositions";                   // (FIX.4.4)
dictionary["AO"]          = "RequestForPositionsAck";                // (FIX.4.4)
dictionary["AP"]          = "PositionReport";                        // (FIX.4.4)
dictionary["AQ"]          = "TradeCaptureReportRequestAck";          // (FIX.4.4)
dictionary["AR"]          = "TradeCaptureReportAck";                 // (FIX.4.4)
dictionary["AS"]          = "AllocationReport";                      // (FIX.4.4)
dictionary["AT"]          = "AllocationReportAck";                   // (FIX.4.4)
dictionary["AU"]          = "ConfirmationAck";                       // (FIX.4.4)
dictionary["AV"]          = "SettlementInstructionRequest";          // (FIX.4.4)
dictionary["AW"]          = "AssignmentReport";                      // (FIX.4.4)
dictionary["AX"]          = "CollateralRequest";                     // (FIX.4.4)
dictionary["AY"]          = "CollateralAssignment";                  // (FIX.4.4)
dictionary["AZ"]          = "CollateralResponse";                    // (FIX.4.4)
dictionary["BA"]          = "CollateralReport";                      // (FIX.4.4)
dictionary["BB"]          = "CollateralInquiry";                     // (FIX.4.4)
dictionary["BC"]          = "NetworkCounterpartySystemStatusRequest"; // (FIX.4.4)
dictionary["BD"]          = "NetworkCounterpartySystemStatusResponse"; // (FIX.4.4)
dictionary["BE"]          = "UserRequest";                           // (FIX.4.4)
dictionary["BF"]          = "UserResponse";                          // (FIX.4.4)
dictionary["BG"]          = "CollateralInquiryAck";                  // (FIX.4.4)
dictionary["BH"]          = "ConfirmationRequest";                   // (FIX.4.4)
dictionary["BO"]          = "ContraryIntentionReport";               // (FIX.4.4)
dictionary["BP"]          = "SecurityDefinitionUpdateReport";        // (FIX.4.4)
dictionary["BK"]          = "SecurityListUpdateReport";              // (FIX.4.4)
dictionary["BL"]          = "AdjustedPositionReport";                // (FIX.4.4)
dictionary["BM"]          = "AllocationInstructionAlert";            // (FIX.4.4)
dictionary["BN"]          = "ExecutionAck";                          // (FIX.4.4)
dictionary["BJ"]          = "TradingSessionList";                    // (FIX.4.4)
dictionary["BI"]          = "TradingSessionListRequest";             // (FIX.4.4)
dictionary["BQ"]          = "SettlementObligationReport";            // (FIX.5.0)
dictionary["BR"]          = "DerivativeSecurityListUpdateReport";    // (FIX.5.0)
dictionary["BS"]          = "TradingSessionListUpdateReport";        // (FIX.5.0)
dictionary["BT"]          = "MarketDefinitionRequest";               // (FIX.5.0)
dictionary["BU"]          = "MarketDefinition";                      // (FIX.5.0)
dictionary["BV"]          = "MarketDefinitionUpdateReport";          // (FIX.5.0)
dictionary["CB"]          = "UserNotification";                      // (FIX.5.0)
dictionary["BZ"]          = "OrderMassActionReport";                 // (FIX.5.0)
dictionary["CA"]          = "OrderMassActionRequest";                // (FIX.5.0)
dictionary["BW"]          = "ApplicationMessageRequest";             // (FIX.5.0)
dictionary["BX"]          = "ApplicationMessageRequestAck";          // (FIX.5.0)
dictionary["BY"]          = "ApplicationMessageReport";              // (FIX.5.0)
dictionary["CC"]          = "StreamAssignmentRequest";               // (FIX.5.0SP1)
dictionary["CD"]          = "StreamAssignmentReport";                // (FIX.5.0SP1)
dictionary["CE"]          = "StreamAssignmentReportACK";             // (FIX.5.0SP1)
dictionary["CH"]          = "MarginRequirementInquiry";              // (FIX.5.0SP2)
dictionary["CI"]          = "MarginRequirementInquiryAck";           // (FIX.5.0SP2)
dictionary["CJ"]          = "MarginRequirementReport";               // (FIX.5.0SP2)
dictionary["CF"]          = "PartyDetailsListRequest";               // (FIX.5.0SP2)
dictionary["CG"]          = "PartyDetailsListReport";                // (FIX.5.0SP2)
dictionary["CK"]          = "PartyDetailsListUpdateReport";          // (FIX.5.0SP2)
dictionary["CL"]          = "PartyRiskLimitsRequest";                // (FIX.5.0SP2)
dictionary["CM"]          = "PartyRiskLimitsReport";                 // (FIX.5.0SP2)
dictionary["CN"]          = "SecurityMassStatusRequest";             // (FIX.5.0SP2)
dictionary["CO"]          = "SecurityMassStatus";                    // (FIX.5.0SP2)
dictionary["CQ"]          = "AccountSummaryReport";                  // (FIX.5.0SP2)
dictionary["CR"]          = "PartyRiskLimitsUpdateReport";           // (FIX.5.0SP2)
dictionary["CS"]          = "PartyRiskLimitsDefinitionRequest";      // (FIX.5.0SP2)
dictionary["CT"]          = "PartyRiskLimitsDefinitionRequestAck";   // (FIX.5.0SP2)
dictionary["CU"]          = "PartyEntitlementsRequest";              // (FIX.5.0SP2)
dictionary["CV"]          = "PartyEntitlementsReport";               // (FIX.5.0SP2)
dictionary["CW"]          = "QuoteAck";                              // (FIX.5.0SP2)
dictionary["CX"]          = "PartyDetailsDefinitionRequest";         // (FIX.5.0SP2)
dictionary["CY"]          = "PartyDetailsDefinitionRequestAck";      // (FIX.5.0SP2)
dictionary["CZ"]          = "PartyEntitlementsUpdateReport";         // (FIX.5.0SP2)
dictionary["DA"]          = "PartyEntitlementsDefinitionRequest";    // (FIX.5.0SP2)
dictionary["DB"]          = "PartyEntitlementsDefinitionRequestAck"; // (FIX.5.0SP2)
dictionary["DC"]          = "TradeMatchReport";                      // (FIX.5.0SP2)
dictionary["DD"]          = "TradeMatchReportAck";                   // (FIX.5.0SP2)
dictionary["DE"]          = "PartyRiskLimitsReportAck";              // (FIX.5.0SP2)
dictionary["DF"]          = "PartyRiskLimitCheckRequest";            // (FIX.5.0SP2)
dictionary["DG"]          = "PartyRiskLimitCheckRequestAck";         // (FIX.5.0SP2)
dictionary["DH"]          = "PartyActionRequest";                    // (FIX.5.0SP2)
dictionary["DI"]          = "PartyActionReport";                     // (FIX.5.0SP2)
dictionary["DJ"]          = "MassOrder";                             // (FIX.5.0SP2)
dictionary["DK"]          = "MassOrderAck";                          // (FIX.5.0SP2)
dictionary["DL"]          = "PositionTransferInstruction";           // (FIX.5.0SP2)
dictionary["DM"]          = "PositionTransferInstructionAck";        // (FIX.5.0SP2)
dictionary["DN"]          = "PositionTransferReport";                // (FIX.5.0SP2)
dictionary["DO"]          = "MarketDataStatisticsRequest";           // (FIX.5.0SP2)
dictionary["DP"]          = "MarketDataStatisticsReport";            // (FIX.5.0SP2)
dictionary["DQ"]          = "CollateralReportAck";                   // (FIX.5.0SP2)
dictionary["DR"]          = "MarketDataReport";                      // (FIX.5.0SP2)
dictionary["DS"]          = "CrossRequest";                          // (FIX.5.0SP2)
dictionary["DT"]          = "CrossRequestAck";                       // (FIX.5.0SP2)
dictionary["DU"]          = "AllocationInstructionAlertRequest";     // (FIX.5.0SP2)
}
} // namespace hffix
#endif // HFFIX_FIELDS_HEADER
